// charmetrics.h                                Copyright (C) 2020 Codemist


/**************************************************************************
 * Copyright (C) 2020, Codemist.                         A C Norman       *
 *                                                                        *
 * Redistribution and use in source and binary forms, with or without     *
 * modification, are permitted provided that the following conditions are *
 * met:                                                                   *
 *                                                                        *
 *     * Redistributions of source code must retain the relevant          *
 *       copyright notice, this list of conditions and the following      *
 *       disclaimer.                                                      *
 *     * Redistributions in binary form must reproduce the above          *
 *       copyright notice, this list of conditions and the following      *
 *       disclaimer in the documentation and/or other materials provided  *
 *       with the distribution.                                           *
 *                                                                        *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS      *
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE         *
 * COPYRIGHT OWNERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,   *
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,   *
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS  *
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND *
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR  *
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF     *
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH   *
 * DAMAGE.                                                                *
 *************************************************************************/

// $Id: charmetrics.h 5432 2020-10-15 20:40:59Z arthurcnorman $


#ifndef __STDC_CONSTANT_MACROS
#define __STDC_CONSTANT_MACROS 1
#endif

#include <stdint.h>

// Character metric hash table created using the program charmetrics.cpp
// sourceforge.net/p/reduce-algebra/code/HEAD/tree/trunk/csl/cslbase/wxfontxs
// contains README files with full credits to the fonts this is used with


// The list of font codes here must be kept in step with the list
// of names in the table.

#define F_cmuntt                      0
#define F_odokai                      1
#define F_Regular                     2
#define F_Bold                        3
#define F_Italic                      4
#define F_BoldItalic                  5
#define F_Math                        6
#define F_end                         7

extern int c_width, c_llx, c_lly, c_urx, c_ury, c_kerninfo;
extern int lookupchar(int fontnum, int codepoint);
extern int32_t lookupkernandligature(int codepoint);
extern int32_t lookupkernadjustment(int codepoint);
extern int32_t lookupligature(int codepoint);
extern int accentposition(int codepoint);

extern const std::uint16_t chardepth_WIN32[31];
extern const std::uint16_t chardepth_X11[31];
extern const std::uint16_t chardepth_OSX[31];
extern const std::uint16_t *chardepth;
extern const char *fontnames[31];

const uint64_t charmetrics[10057][5] =
{   {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000888d), UINT64_C(0x1362f78f608745f1), UINT64_C(0x1362f78f608745f1),
        UINT64_C(0x0ffb1e0f6c6665f1), UINT64_C(0x156b028f6c89c5f1)
    },
    {   UINT64_C(0x00000000000188dc), UINT64_C(0x18d2fbcce09ae6d7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000888e), UINT64_C(0x156afa13108e25e7), UINT64_C(0x156afa0f8c8e25e7),
        UINT64_C(0x156afa0f8c8e25e7), UINT64_C(0x156afa0f6c8e25f1)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188df), UINT64_C(0x1b3afa8c30a50630), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008890), UINT64_C(0x08f2f6cef85da60b), UINT64_C(0x156afa11008e2590),
        UINT64_C(0x156afa107c8e25a5), UINT64_C(0x156afa107c8e25a5)
    },
    {   UINT64_C(0x0000000000080008), UINT64_C(0x106aee0fa03e83e8), UINT64_C(0x106b1fcfa0670656),
        UINT64_C(0x106b0e54c06fc655), UINT64_C(0x106af6cfa07ba64a)
    },
    {   UINT64_C(0x0000000000000009), UINT64_C(0x106afc8e5478c69e), UINT64_C(0x106af6ce547ba69e),
        UINT64_C(0x106af50f747bc656), UINT64_C(0x106b2454c064c655)
    },
    {   UINT64_C(0x000000000000000a), UINT64_C(0x106b194e5875269e), UINT64_C(0x106b040e586a869e),
        UINT64_C(0x106aff11087785f1), UINT64_C(0x106af750e47b45fa)
    },
    {   UINT64_C(0x000000000000000b), UINT64_C(0x106b1b0d7469c465), UINT64_C(0x106b03d2847504de),
        UINT64_C(0x106b1fcfa0670465), UINT64_C(0x106afc8e5478a69e)
    },
    {   UINT64_C(0x000000000000000c), UINT64_C(0x106afa8f7479c656), UINT64_C(0x106b084fa075c656),
        UINT64_C(0x106afb0fa0798656), UINT64_C(0x106af90f747a8656)
    },
    {   UINT64_C(0x000000000000000d), UINT64_C(0x106af54fa07c6657), UINT64_C(0x106afb0f7479864b),
        UINT64_C(0x106afb4f74796656), UINT64_C(0x106af90f747a865b)
    },
    {   UINT64_C(0x000000000000000e), UINT64_C(0x106af90f747a8656), UINT64_C(0x106afb4f74796656),
        UINT64_C(0x106b200fa0672597), UINT64_C(0x106b1b0d7469c596)
    },
    {   UINT64_C(0x000000000000000f), UINT64_C(0x106afc109079060f), UINT64_C(0x106af752bc7b4584),
        UINT64_C(0x106afc109079060f), UINT64_C(0x106afd8fa0784651)
    },
    {   UINT64_C(0x0000000000000010), UINT64_C(0x106af90f887a8651), UINT64_C(0x106af4cfa07ca657),
        UINT64_C(0x106af3cfa07ac64b), UINT64_C(0x106af80f747b0656)
    },
    {   UINT64_C(0x0000000000000011), UINT64_C(0x106af2cfa07b264b), UINT64_C(0x106af48fa07d464b),
        UINT64_C(0x106af50fa07bc64b), UINT64_C(0x106af78f747c8656)
    },
    {   UINT64_C(0x0000000680000012), UINT64_C(0x106af38fa07d464b), UINT64_C(0x106b01cfa076464b),
        UINT64_C(0x106affcf747a464b), UINT64_C(0x106af48fa07c664b)
    },
    {   UINT64_C(0x0000000000000013), UINT64_C(0x106af60fa07b864b), UINT64_C(0x106af24fa07de64b),
        UINT64_C(0x106af50fa07c864b), UINT64_C(0x106afc0f74790656)
    },
    {   UINT64_C(0x0000000000000014), UINT64_C(0x106af48fa07a864b), UINT64_C(0x106afc0d74790656),
        UINT64_C(0x106af38f747fc64b), UINT64_C(0x106afb0f74798656)
    },
    {   UINT64_C(0x0000000000000015), UINT64_C(0x106af48fa07cc64b), UINT64_C(0x106aef0f747f864b),
        UINT64_C(0x106af28f807dc64b), UINT64_C(0x106af0cf807ea64b)
    },
    {   UINT64_C(0x0000000000000016), UINT64_C(0x106af4cfa07c864b), UINT64_C(0x106af2cfa07da64b),
        UINT64_C(0x106afa0fa07aa64b), UINT64_C(0x106b254e547ae69e)
    },
    {   UINT64_C(0x0000000000000017), UINT64_C(0x106afc8e5478a69e), UINT64_C(0x106af80e5464469e),
        UINT64_C(0x106b01163876861d), UINT64_C(0x106afc0e247903cf)
    },
    {   UINT64_C(0x0000000000000018), UINT64_C(0x106b09178866a667), UINT64_C(0x106afbcf888005a0),
        UINT64_C(0x106af10f887b864b), UINT64_C(0x106b004f8878c5a0)
    },
    {   UINT64_C(0x0000360000000019), UINT64_C(0x106af70f887e864b), UINT64_C(0x106afbcf887885a0),
        UINT64_C(0x106af88fa0752651), UINT64_C(0x106af54c0c7e25a2)
    },
    {   UINT64_C(0x000000078000001a), UINT64_C(0x106af10fa07e864b), UINT64_C(0x106b018fa077664c),
        UINT64_C(0x106afa0c106c864c), UINT64_C(0x106af34fa07e064b)
    },
    {   UINT64_C(0x000000000000001b), UINT64_C(0x106afc8fa078e64b), UINT64_C(0x106aed0fa080859d),
        UINT64_C(0x106af10fa07e859d), UINT64_C(0x106afc4f8878e5a0)
    },
    {   UINT64_C(0x000000000000001c), UINT64_C(0x106af10c287b859d), UINT64_C(0x106af80c2881a59d),
        UINT64_C(0x106af60fa07b659d), UINT64_C(0x106b000f8877e5a0)
    },
    {   UINT64_C(0x000000000000001d), UINT64_C(0x106af44f8876a612), UINT64_C(0x106af10f887e8597),
        UINT64_C(0x106af40f907d0597), UINT64_C(0x106af20f907e0597)
    },
    {   UINT64_C(0x000000000000001e), UINT64_C(0x106af4cfa07c8597), UINT64_C(0x106af48c107d0597),
        UINT64_C(0x106af64fa079e597), UINT64_C(0x106afc0e5478e69e)
    },
    {   UINT64_C(0x000000000000001f), UINT64_C(0x106b26ce5463a69e), UINT64_C(0x106afc0e5478e69e),
        UINT64_C(0x106affd74c77064b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000619d), UINT64_C(0x1f42ff4fecb766cd), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4e7cb7e6de)
    },
    {   UINT64_C(0x000000000000889d), UINT64_C(0x156afa0c108e26c3), UINT64_C(0x156afa0c0c8e26c2),
        UINT64_C(0x156afc0c708c26a9), UINT64_C(0x156afc0c708c26a9)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000889e), UINT64_C(0x156afa0bb88e26d6), UINT64_C(0x156afa0bb88e26d6),
        UINT64_C(0x156afe0f388c25fc), UINT64_C(0x156afe0f388c25fc)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000889f), UINT64_C(0x156afe0dc08c265c), UINT64_C(0x156afe0dc48c265d),
        UINT64_C(0x156afa0d108e2680), UINT64_C(0x156afa0d108e2680)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088a0), UINT64_C(0x156afe0d308c267e), UINT64_C(0x156afe0d308c267e),
        UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0f3c8c25fb)
    },
    {   UINT64_C(0x0000000000000028), UINT64_C(0x106aee0fa03e83e8), UINT64_C(0x106b1fcbfc67056d),
        UINT64_C(0x106af8cde47c4608), UINT64_C(0x106af94f8c7d2653)
    },
    {   UINT64_C(0x0000000000000029), UINT64_C(0x106ae10f0886a637), UINT64_C(0x106af28fa07da64b),
        UINT64_C(0x106b26ce5863a69e), UINT64_C(0x106b028c1075c650)
    },
    {   UINT64_C(0x000000000000002a), UINT64_C(0x106b09979c72464b), UINT64_C(0x106adb4bfc8986a9),
        UINT64_C(0x106af693007e06a7), UINT64_C(0x106af8cfa0762597)
    },
    {   UINT64_C(0x000000000000002b), UINT64_C(0x106af8d0347aa550), UINT64_C(0x106b19d2846a24de),
        UINT64_C(0x106adb4bfc8986a9), UINT64_C(0x106b00185c76e656)
    },
    {   UINT64_C(0x000000000000002c), UINT64_C(0x106b0a147c71e64b), UINT64_C(0x106af751c47b4632),
        UINT64_C(0x106afcd46478a6a9), UINT64_C(0x106afa545079e6a9)
    },
    {   UINT64_C(0x000000000000002d), UINT64_C(0x106b209788726667), UINT64_C(0x106af0cc287ea596),
        UINT64_C(0x106af9cc287d064b), UINT64_C(0x106b2312446584f5)
    },
    {   UINT64_C(0x000000000000002e), UINT64_C(0x106b0e0c606fe421), UINT64_C(0x106b04546475c6a9),
        UINT64_C(0x106af693007ba6a6), UINT64_C(0x106b018fa07a8597)
    },
    {   UINT64_C(0x000000000000002f), UINT64_C(0x106aedcc288066a9), UINT64_C(0x106aedcc288066a9),
        UINT64_C(0x106aedcc288066a9), UINT64_C(0x106afd4c1078456c)
    },
    {   UINT64_C(0x0000000000000030), UINT64_C(0x106af4cfa07ca70f), UINT64_C(0x106af4cfa07ca70f),
        UINT64_C(0x106af4cfa07ca720), UINT64_C(0x106af4cfa07ca71b)
    },
    {   UINT64_C(0x0000000000000031), UINT64_C(0x106af4cfa07ca71b), UINT64_C(0x106af4cfa07c8723),
        UINT64_C(0x106af38fa07d064b), UINT64_C(0x106af80c607b2656)
    },
    {   UINT64_C(0x0000000000000032), UINT64_C(0x106af48fa07d470f), UINT64_C(0x106af48fa07d470f),
        UINT64_C(0x106af48fa07d4720), UINT64_C(0x106af48fa07d471b)
    },
    {   UINT64_C(0x0000000000000033), UINT64_C(0x106b01cfa076470f), UINT64_C(0x106b01cfa076470f),
        UINT64_C(0x106b01cfa0764720), UINT64_C(0x106b01cfa076471b)
    },
    {   UINT64_C(0x0000000000000034), UINT64_C(0x106af28fa07b264b), UINT64_C(0x106af50fa07c871b),
        UINT64_C(0x106afc0f7479070f), UINT64_C(0x106afc0f7479070f)
    },
    {   UINT64_C(0x0000000000000035), UINT64_C(0x106afc0f74790720), UINT64_C(0x106afc0f7479071b),
        UINT64_C(0x106afc0f7479071b), UINT64_C(0x106b0551c47465c2)
    },
    {   UINT64_C(0x0000000000000036), UINT64_C(0x106afc0e5079069f), UINT64_C(0x106aef0f747f870f),
        UINT64_C(0x106aef0f747f870f), UINT64_C(0x106aef0f747f8720)
    },
    {   UINT64_C(0x0000000000000037), UINT64_C(0x106aef0f747f871b), UINT64_C(0x106af2cfa07da70f),
        UINT64_C(0x106af58fa07a064b), UINT64_C(0x106af20f8c7ba650)
    },
    {   UINT64_C(0x0000000000000038), UINT64_C(0x106afbcf88800667), UINT64_C(0x106afbcf88800667),
        UINT64_C(0x106afbcf8880066c), UINT64_C(0x106afbcf88800671)
    },
    {   UINT64_C(0x0000000000000039), UINT64_C(0x106afbcf8880064b), UINT64_C(0x106afbcf8880066f),
        UINT64_C(0x106af60f8c7d859f), UINT64_C(0x106b000c6078a59f)
    },
    {   UINT64_C(0x000000000000003a), UINT64_C(0x106afbcf88788667), UINT64_C(0x106afbcf88788667),
        UINT64_C(0x106afbcf8878866c), UINT64_C(0x106afbcf8878864b)
    },
    {   UINT64_C(0x000000000000003b), UINT64_C(0x106b018fa0776667), UINT64_C(0x106b018fa0776667),
        UINT64_C(0x106b018fa077666c), UINT64_C(0x106b078fa074464b)
    },
    {   UINT64_C(0x000000000000003c), UINT64_C(0x106afc0f8c78e64b), UINT64_C(0x106af10fa07e8671),
        UINT64_C(0x106afc4f8878e667), UINT64_C(0x106afc4f8878e667)
    },
    {   UINT64_C(0x000000000000003d), UINT64_C(0x106afc4f8878e66c), UINT64_C(0x106afc4f8878e671),
        UINT64_C(0x106afc4f8878e64b), UINT64_C(0x106af750d47b45fe)
    },
    {   UINT64_C(0x000000000000003e), UINT64_C(0x106af88d707aa623), UINT64_C(0x106af10f887e8667),
        UINT64_C(0x106af10f887e8667), UINT64_C(0x106af10f887e866c)
    },
    {   UINT64_C(0x000000000000003f), UINT64_C(0x106af10f887e864b), UINT64_C(0x106af48c107d0667),
        UINT64_C(0x106af0cc287b864b), UINT64_C(0x106af48c107d064b)
    },
    {   UINT64_C(0x0000000000000040), UINT64_C(0x106af4cfa07ca6cf), UINT64_C(0x106afbcf88800639),
        UINT64_C(0x106af4cfa07ca71b), UINT64_C(0x106afbcf88800667)
    },
    {   UINT64_C(0x0000000000000041), UINT64_C(0x106af4cbfc7fc657), UINT64_C(0x106afbcbfc80059f),
        UINT64_C(0x106af80f747b070f), UINT64_C(0x106b004f8878c667)
    },
    {   UINT64_C(0x0000000000000042), UINT64_C(0x106af80f747b0724), UINT64_C(0x106b004f8878c66d),
        UINT64_C(0x106af80f747b06ec), UINT64_C(0x106b004f8878c635)
    },
    {   UINT64_C(0x0000000000000043), UINT64_C(0x106af80f747b070c), UINT64_C(0x106b004f8878c667),
        UINT64_C(0x106af2cfa07b270c), UINT64_C(0x106af48f8c7f466d)
    },
    {   UINT64_C(0x0000000000000044), UINT64_C(0x106af28fa07b264b), UINT64_C(0x106af6cf8c7e864b),
        UINT64_C(0x106af48fa07d46ce), UINT64_C(0x106afbcf88788638)
    },
    {   UINT64_C(0x0000000000000045), UINT64_C(0x106af48fa07d4715), UINT64_C(0x106afbcf8878865d),
        UINT64_C(0x106af48fa07d46ec), UINT64_C(0x106afbcf88788635)
    },
    {   UINT64_C(0x0000000000000046), UINT64_C(0x106af44bfc7d664b), UINT64_C(0x106afbcbfc78a59f),
        UINT64_C(0x106af48fa07d470c), UINT64_C(0x106afbcf88788667)
    },
    {   UINT64_C(0x0000000000000047), UINT64_C(0x106af78f747d4724), UINT64_C(0x106af54c0c7e2670),
        UINT64_C(0x106af74f747c871b), UINT64_C(0x106af54c0c7e2667)
    },
    {   UINT64_C(0x0000000000000048), UINT64_C(0x106af78f747c86ec), UINT64_C(0x106af54c0c7e2638),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000049), UINT64_C(0x106af38fa07d4724), UINT64_C(0x106af10fa07e8719),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af1cfa07ee64b)
    },
    {   UINT64_C(0x000000000000004a), UINT64_C(0x106b01cfa0764729), UINT64_C(0x106b018fa0776674),
        UINT64_C(0x106b000fa076e6c7), UINT64_C(0x106b014fa0778638)
    },
    {   UINT64_C(0x000000000000004b), UINT64_C(0x106b01cfa0764715), UINT64_C(0x106b018fa077665d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000004c), UINT64_C(0x106b01cfa0764706), UINT64_C(0x106b018fa0776597),
        UINT64_C(0x106af08f747c864b), UINT64_C(0x106af74c1478e64b)
    },
    {   UINT64_C(0x000000000000004d), UINT64_C(0x106affcf747e2724), UINT64_C(0x106afa0c1072066d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000004e), UINT64_C(0x0000000000000000), UINT64_C(0x106af60fa07b870f),
        UINT64_C(0x106afc8fa078e70f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000004f), UINT64_C(0x0000000000000000), UINT64_C(0x106af60fa07b86ac),
        UINT64_C(0x106afc8fa07ea66d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000050), UINT64_C(0x0000000000000000), UINT64_C(0x106af58fa07b864b),
        UINT64_C(0x106afc4fa078e64b), UINT64_C(0x106af50fa07c870f)
    },
    {   UINT64_C(0x0000000000000051), UINT64_C(0x106af10fa07e8667), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af50fa07c870c)
    },
    {   UINT64_C(0x0000000000000052), UINT64_C(0x106af10fa07e8667), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af20f747b465d), UINT64_C(0x106af0cc1474059c)
    },
    {   UINT64_C(0x0000000000000053), UINT64_C(0x106afc0f747906ce), UINT64_C(0x106afc4f8878e638),
        UINT64_C(0x106afc0f74790715), UINT64_C(0x106afc4f8878e65d)
    },
    {   UINT64_C(0x0000000000000054), UINT64_C(0x106afc0f747ee70f), UINT64_C(0x106afc4f887fc667),
        UINT64_C(0x106af54f747d6656), UINT64_C(0x106af28f8c7da59f)
    },
    {   UINT64_C(0x0000000000000055), UINT64_C(0x106af38f747fc70f), UINT64_C(0x106af60fa07b6667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000056), UINT64_C(0x106af38f747fc70c), UINT64_C(0x106af60fa07b6667),
        UINT64_C(0x106afb0f7479870f), UINT64_C(0x106b000f8c77e667)
    },
    {   UINT64_C(0x0000000000000057), UINT64_C(0x106afb0f74798724), UINT64_C(0x106b000f8877e66d),
        UINT64_C(0x106afb0c60798656), UINT64_C(0x106b000c6077e59f)
    },
    {   UINT64_C(0x0000000000000058), UINT64_C(0x106afb0f7479870c), UINT64_C(0x106b000f8c77e667),
        UINT64_C(0x106af48c607ce64b), UINT64_C(0x106af44c5c76c611)
    },
    {   UINT64_C(0x0000000000000059), UINT64_C(0x106af48fa07cc70c), UINT64_C(0x106af44f887a0672),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000005a), UINT64_C(0x106aef0f747f8729), UINT64_C(0x106af10f887e8674),
        UINT64_C(0x106aef0f747f86ce), UINT64_C(0x106af10f887e8638)
    },
    {   UINT64_C(0x000000000000005b), UINT64_C(0x106aef0f747f8715), UINT64_C(0x106af10f887e865d),
        UINT64_C(0x106aef0f747f8723), UINT64_C(0x106af10f887e866f)
    },
    {   UINT64_C(0x000000000000005c), UINT64_C(0x106aef0f7480270f), UINT64_C(0x106af10f887fe667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000005d), UINT64_C(0x106af0cf807ea724), UINT64_C(0x106af20f907e066d),
        UINT64_C(0x106af2cfa07da724), UINT64_C(0x106af48c107d066d)
    },
    {   UINT64_C(0x000000000000005e), UINT64_C(0x106af2cfa07da71b), UINT64_C(0x106afa0fa07aa70f),
        UINT64_C(0x106af64fa079e667), UINT64_C(0x106afa0fa07aa707)
    },
    {   UINT64_C(0x01f000000000005f), UINT64_C(0x106af64fa079e64b), UINT64_C(0x106afa0fa07aa70c),
        UINT64_C(0x106af64fa079e667), UINT64_C(0x106af88fa0752651)
    },
    {   UINT64_C(0x0000000000000060), UINT64_C(0x106aee4f8c7b864b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000061), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af80f747b0656), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000001000062), UINT64_C(0x106afd8f8c84e632), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000063), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af44fa07d664b), UINT64_C(0x106af0cf747d2656)
    },
    {   UINT64_C(0x0000000000000064), UINT64_C(0x106afa4f7480a656), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aef4c1080c650), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000065), UINT64_C(0x0000000000000000), UINT64_C(0x106aedcf8c7ce64b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000066), UINT64_C(0x0000000000000000), UINT64_C(0x106af30fa07e0651),
        UINT64_C(0x106b014fa076664b), UINT64_C(0x106aee8f8c79864b)
    },
    {   UINT64_C(0x0000000000000067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af14c2881c59c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000068), UINT64_C(0x106afc0f747c66af), UINT64_C(0x106afc0f8c78e5fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000069), UINT64_C(0x0000000000000000), UINT64_C(0x106aea8c287b2651),
        UINT64_C(0x0000000000000000), UINT64_C(0x106afb0f74798656)
    },
    {   UINT64_C(0x000000000000006a), UINT64_C(0x106b000f8c77e59f), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b020c1078a651), UINT64_C(0x106af3cc10730612)
    },
    {   UINT64_C(0x000000000000006b), UINT64_C(0x0000000000000000), UINT64_C(0x106af3cf8c774651),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aeecf748026ae)
    },
    {   UINT64_C(0x000000000000006c), UINT64_C(0x106af0cf8c7ea5fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000006d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af78f747b264b)
    },
    {   UINT64_C(0x000000000000006e), UINT64_C(0x0000000000000000), UINT64_C(0x106afa4c1479e596),
        UINT64_C(0x106afa0c147a6596), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000006f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0b8c7a859a)
    },
    {   UINT64_C(0x0000000000000070), UINT64_C(0x106b268c2863c64b), UINT64_C(0x106b0c4c2870c64b),
        UINT64_C(0x106afb0c2879864b), UINT64_C(0x106b230fa0658656)
    },
    {   UINT64_C(0x0000000000000071), UINT64_C(0x106af04fa07d670c), UINT64_C(0x106af04fa07d0667),
        UINT64_C(0x106af28f8c7d0667), UINT64_C(0x106af1cf747d464b)
    },
    {   UINT64_C(0x0000000000000072), UINT64_C(0x106af1cc1076664b), UINT64_C(0x106af54c1076664b),
        UINT64_C(0x106af18f747d464b), UINT64_C(0x106af18c1076664b)
    },
    {   UINT64_C(0x0000000000000073), UINT64_C(0x106aef8c1076664b), UINT64_C(0x106af4cfa07ca724),
        UINT64_C(0x106afbcf8880066c), UINT64_C(0x106b01cfa0764723)
    },
    {   UINT64_C(0x0000000000000074), UINT64_C(0x106b018fa077666b), UINT64_C(0x106afc0f74790723),
        UINT64_C(0x106afc4f8878e66c), UINT64_C(0x106aef0f747f8723)
    },
    {   UINT64_C(0x0000000000000075), UINT64_C(0x106af10f887e866b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000077), UINT64_C(0x0000000000000000), UINT64_C(0x106af94f8c77259f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000078), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af38fa07f06ce), UINT64_C(0x106af60f8c7d8639)
    },
    {   UINT64_C(0x0000000000000079), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af78f747c8723), UINT64_C(0x106af54c0c7e266e)
    },
    {   UINT64_C(0x000000000000007a), UINT64_C(0x106af48fa07c6723), UINT64_C(0x106af34fa07e0718),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000581d), UINT64_C(0x0000000000000000), UINT64_C(0x1f4309ce88adc71a),
        UINT64_C(0x1f43144f94ac06f1), UINT64_C(0x1f43010edcb2070b)
    },
    {   UINT64_C(0x000000000000007c), UINT64_C(0x106afa0c106fa66b), UINT64_C(0x106af04fa07d664b),
        UINT64_C(0x106af04fa07d064b), UINT64_C(0x106af28f8c7d064b)
    },
    {   UINT64_C(0x000000000000007d), UINT64_C(0x106af78f747c8701), UINT64_C(0x106af54c0c7e2659),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000007e), UINT64_C(0x106af50fa07c8701), UINT64_C(0x106af10fa07e8656),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000007f), UINT64_C(0x106af38fa07d0701), UINT64_C(0x106af60f8c7d8656),
        UINT64_C(0x106afc0e5079074b), UINT64_C(0x106af88d707aa6da)
    },
    {   UINT64_C(0x0000000000000080), UINT64_C(0x106af4cfa07ca70e), UINT64_C(0x106afbcf88800655),
        UINT64_C(0x106af4cfa07ca72a), UINT64_C(0x106afbcf88800671)
    },
    {   UINT64_C(0x0000000000000081), UINT64_C(0x106af48fa07d470d), UINT64_C(0x106afbcf88788655),
        UINT64_C(0x106af48fa07d4729), UINT64_C(0x106afbcf88788671)
    },
    {   UINT64_C(0x0000000000000082), UINT64_C(0x106b01cfa076470d), UINT64_C(0x106b018fa0776655),
        UINT64_C(0x106b01cfa0764729), UINT64_C(0x106b018fa0776671)
    },
    {   UINT64_C(0x0000000000000083), UINT64_C(0x106afc0f7479070d), UINT64_C(0x106afc4f8878e655),
        UINT64_C(0x106afc0f74790729), UINT64_C(0x106afc4f8878e671)
    },
    {   UINT64_C(0x0000000000000084), UINT64_C(0x106af38f747fc70d), UINT64_C(0x106af60fa07b6655),
        UINT64_C(0x106af38f747fc729), UINT64_C(0x106af60fa07b6671)
    },
    {   UINT64_C(0x0000000000000085), UINT64_C(0x106aef0f747f870d), UINT64_C(0x106af10f887e8655),
        UINT64_C(0x106aef0f747f8729), UINT64_C(0x106af10f887e8671)
    },
    {   UINT64_C(0x0000000000000086), UINT64_C(0x106afb0bc8798656), UINT64_C(0x106b000be077e5a0),
        UINT64_C(0x106af48bf87cc64b), UINT64_C(0x106af44be076a612)
    },
    {   UINT64_C(0x0000000000000087), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af38fa07d4723), UINT64_C(0x106af10fa07e8718)
    },
    {   UINT64_C(0x0000000000000088), UINT64_C(0x0000000000000000), UINT64_C(0x106af54c947de64b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000089), UINT64_C(0x0000000000000000), UINT64_C(0x106af54c60798596),
        UINT64_C(0x106af4cfa07ca6ed), UINT64_C(0x106afbcf88800635)
    },
    {   UINT64_C(0x0000000000018964), UINT64_C(0x25e3858ae4d60776), UINT64_C(0x2582ee0b98d48744),
        UINT64_C(0x2582ee0b5cd48752), UINT64_C(0x2582ee0b5cd48752)
    },
    {   UINT64_C(0x000000000000008b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afc0f747906ec), UINT64_C(0x106afc4f8878e635)
    },
    {   UINT64_C(0x000000000000008c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af2cfa07da6ce), UINT64_C(0x106af48c107d0610)
    },
    {   UINT64_C(0x000044000000008d), UINT64_C(0x0000000000000000), UINT64_C(0x106aefcc947de59c),
        UINT64_C(0x106ae54c9478e612), UINT64_C(0x106afa0c106c8597)
    },
    {   UINT64_C(0x000000000000008e), UINT64_C(0x106af30f8c7d664b), UINT64_C(0x106af30c287d659c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018969), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x000000000001896a), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x0f22f610407705b4), UINT64_C(0x0f22f610407705b4)
    },
    {   UINT64_C(0x000000000001896b), UINT64_C(0x1fe2f78f74b945bd), UINT64_C(0x1fe2f78f74b945bd),
        UINT64_C(0x1182f80c307f86bc), UINT64_C(0x1182f80c307f86bc)
    },
    {   UINT64_C(0x000000000001896c), UINT64_C(0x23a2f50f74c985ea), UINT64_C(0x23a2f54f74c965ea),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713)
    },
    {   UINT64_C(0x000000000001896d), UINT64_C(0x14a2f4d0108d8611), UINT64_C(0x14a2f4d0108d8611),
        UINT64_C(0x209aff8b2cbc86fe), UINT64_C(0x209aff8b30bc86ff)
    },
    {   UINT64_C(0x0000000000000094), UINT64_C(0x106aee0f8079459c), UINT64_C(0x106af6cf8c7e859c),
        UINT64_C(0x106af0cf8c7b859c), UINT64_C(0x106b00cf8c7b2651)
    },
    {   UINT64_C(0x02a0480000000095), UINT64_C(0x106afccf8076e59c), UINT64_C(0x106afc0d0877a59f),
        UINT64_C(0x106af6cc1486864b), UINT64_C(0x106af6cf8c89c651)
    },
    {   UINT64_C(0x0000000000000096), UINT64_C(0x106af94f8c77659f), UINT64_C(0x106af80f8078a59c),
        UINT64_C(0x106af64f8080059c), UINT64_C(0x106afe0f8c78c59f)
    },
    {   UINT64_C(0x0000000000000097), UINT64_C(0x106af98f747a45a2), UINT64_C(0x106af68f748465a2),
        UINT64_C(0x106af54f747a85a2), UINT64_C(0x106b00cc10778596)
    },
    {   UINT64_C(0x0000000001600098), UINT64_C(0x106af6cbf889c613), UINT64_C(0x106af7cbf876a59c),
        UINT64_C(0x106af8cf807bc59f), UINT64_C(0x106af40bfc7ce596)
    },
    {   UINT64_C(0x0000000000000099), UINT64_C(0x106af08f747ec5a2), UINT64_C(0x106af1cc287e8596),
        UINT64_C(0x106af0cfa07ea651), UINT64_C(0x106af14c1476a651)
    },
    {   UINT64_C(0x000000000000009a), UINT64_C(0x106b014fa079264b), UINT64_C(0x106b00cf8c70c597),
        UINT64_C(0x106b0c4fa070e596), UINT64_C(0x106b004fa076e64b)
    },
    {   UINT64_C(0x0000000b8000009b), UINT64_C(0x106af9cfa077664b), UINT64_C(0x106aff0c107b464b),
        UINT64_C(0x106af0cc147c064b), UINT64_C(0x106af38f8c808596)
    },
    {   UINT64_C(0x034000000000009c), UINT64_C(0x106af38c28808596), UINT64_C(0x106aed4c1479c59c),
        UINT64_C(0x106ae44c147ea59c), UINT64_C(0x106af0cc1486a59c)
    },
    {   UINT64_C(0x000000000000009d), UINT64_C(0x106afa4fa079c596), UINT64_C(0x106afc0f8c78e59f),
        UINT64_C(0x106af6cf807ca59f), UINT64_C(0x106af3cf8c7d05b8)
    },
    {   UINT64_C(0x03a000000000009e), UINT64_C(0x106af70c287b664b), UINT64_C(0x106af74f8c7c2596),
        UINT64_C(0x106af74f8c7c2675), UINT64_C(0x106af74c10896596)
    },
    {   UINT64_C(0x000000000000009f), UINT64_C(0x106af5cc287b659c), UINT64_C(0x106af5cc107f259c),
        UINT64_C(0x106af5cfa07b659c), UINT64_C(0x106b01cfa074e59c)
    },
    {   UINT64_C(0x04000000000000a0), UINT64_C(0x106af88f807ee596), UINT64_C(0x106af88fa07ee59f),
        UINT64_C(0x106b000c1477e59f), UINT64_C(0x106afc8c1478c651)
    },
    {   UINT64_C(0x00000000021000a1), UINT64_C(0x106af20c107e0613), UINT64_C(0x106af98c147a459c),
        UINT64_C(0x106afb4b9478c651), UINT64_C(0x106b00cdb47ce59c)
    },
    {   UINT64_C(0x00000000000000a2), UINT64_C(0x106af4cc10800612), UINT64_C(0x106af0cf8c7ea596),
        UINT64_C(0x106afb0f74796596), UINT64_C(0x106af20f8c7a259b)
    },
    {   UINT64_C(0x00000000000000a3), UINT64_C(0x106af3cfa07d059b), UINT64_C(0x106af1cfa07e259b),
        UINT64_C(0x106af3cfa07cc67a), UINT64_C(0x106af6cfa07b8596)
    },
    {   UINT64_C(0x00000000022000a4), UINT64_C(0x106af58c108b8596), UINT64_C(0x106af54d047d2596),
        UINT64_C(0x106afa0c107ac596), UINT64_C(0x106af68b94800596)
    },
    {   UINT64_C(0x00000000000000a5), UINT64_C(0x106afc8fa078c651), UINT64_C(0x106afc8fa078c651),
        UINT64_C(0x106afc8f8c78c64b), UINT64_C(0x106aff8c1078c654)
    },
    {   UINT64_C(0x04a00000000000a6), UINT64_C(0x106afc0f70790657), UINT64_C(0x106af98fa07a2596),
        UINT64_C(0x106af8cf747c65a2), UINT64_C(0x106af88f80880631)
    },
    {   UINT64_C(0x00000000000000a7), UINT64_C(0x106af88fa07a8596), UINT64_C(0x106b02cc1480664b),
        UINT64_C(0x106af24c287da597), UINT64_C(0x106afc0fa07ac596)
    },
    {   UINT64_C(0x00000000026000a8), UINT64_C(0x106af6cc2889c613), UINT64_C(0x106afc8fa078c651),
        UINT64_C(0x106afc8fa078c651), UINT64_C(0x106af30f8c7bc64b)
    },
    {   UINT64_C(0x04e00000000000a9), UINT64_C(0x106af40c147d264b), UINT64_C(0x106af30d047d864b),
        UINT64_C(0x106af18f8c7cc64f), UINT64_C(0x106af24c147d6651)
    },
    {   UINT64_C(0x00000000000000aa), UINT64_C(0x106af1cd087a2611), UINT64_C(0x106af44c14780650),
        UINT64_C(0x106af70f8c7aa64b), UINT64_C(0x106af70fa07c064b)
    },
    {   UINT64_C(0x00000000000000ab), UINT64_C(0x106af1cf987e25cc), UINT64_C(0x106b0b0f9c7185c6),
        UINT64_C(0x106aef8c287ea59c), UINT64_C(0x106aef8c1482a59c)
    },
    {   UINT64_C(0x00000000000000ac), UINT64_C(0x106aef953c5e8680), UINT64_C(0x106aef953c5e8683),
        UINT64_C(0x106af41374558680), UINT64_C(0x106af2153c5ce629)
    },
    {   UINT64_C(0x00000000000000ad), UINT64_C(0x106af2d5305d6626), UINT64_C(0x106af2d374640626),
        UINT64_C(0x106af3553c5ea62a), UINT64_C(0x106af1d5307e2625)
    },
    {   UINT64_C(0x05000000000000ae), UINT64_C(0x106af153745dc626), UINT64_C(0x106b1fd70467064b),
        UINT64_C(0x106b0a571c75064b), UINT64_C(0x106b1b9570698690)
    },
    {   UINT64_C(0x00000000000000af), UINT64_C(0x106b1b13f86ac64b), UINT64_C(0x106b1b945c69864b),
        UINT64_C(0x106b180fa06ae4fc), UINT64_C(0x106b180fa06ae4fc)
    },
    {   UINT64_C(0x00000000000000b0), UINT64_C(0x106afe550077e6a4), UINT64_C(0x106afe550077e6a4),
        UINT64_C(0x106b0696b473c66a), UINT64_C(0x106b0696b473a66a)
    },
    {   UINT64_C(0x00000000000000b1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b07174073a66c), UINT64_C(0x106b0b9730716667)
    },
    {   UINT64_C(0x00000000000000b2), UINT64_C(0x106b27958863264b), UINT64_C(0x106b0017a876e629),
        UINT64_C(0x106b1e971871664b), UINT64_C(0x106b0b171867a64b)
    },
    {   UINT64_C(0x00000000000000b3), UINT64_C(0x106b278c286323f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b090c8c66a3a8), UINT64_C(0x106b208c8c7263a8)
    },
    {   UINT64_C(0x00000000000000b4), UINT64_C(0x106b138f8c6d459c), UINT64_C(0x106b1393d86d459c),
        UINT64_C(0x106b1811c86ae586), UINT64_C(0x106b1811c86ae586)
    },
    {   UINT64_C(0x00000000000000b5), UINT64_C(0x106b06d08073850e), UINT64_C(0x106b06d08473850f),
        UINT64_C(0x106b0acfa071a50e), UINT64_C(0x106afe118c77e51c)
    },
    {   UINT64_C(0x00000000000000b6), UINT64_C(0x106b035768754667), UINT64_C(0x106b29579c68a64b),
        UINT64_C(0x106b1917046a666f), UINT64_C(0x106b1bcca073c420)
    },
    {   UINT64_C(0x00000000000000b7), UINT64_C(0x106b04174c75064b), UINT64_C(0x106b03578880e667),
        UINT64_C(0x106ae5d29c6de59d), UINT64_C(0x106b0b16ec716640)
    },
    {   UINT64_C(0x00000000000000b8), UINT64_C(0x106af113685dc626), UINT64_C(0x106af5553c5ba680),
        UINT64_C(0x106af715305b262a), UINT64_C(0x106af1953c5d6626)
    },
    {   UINT64_C(0x00000000000000b9), UINT64_C(0x106af5553c5ba683), UINT64_C(0x106ae58f1884866e),
        UINT64_C(0x106ae58f1884866e), UINT64_C(0x106ae58f1884866e)
    },
    {   UINT64_C(0x00000000000000ba), UINT64_C(0x106ae58f1884866e), UINT64_C(0x106ae58f1484866e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000583d), UINT64_C(0x1f43000f64ac070b), UINT64_C(0x1f43034f18ae870d),
        UINT64_C(0x1f43048f18b1c72b), UINT64_C(0x1f430ccf44b30721)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088eb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x09daee131065e506)
    },
    {   UINT64_C(0x00000000000000be), UINT64_C(0x0000000000000000), UINT64_C(0x106b07155c7d6640),
        UINT64_C(0x106af3555c738640), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018999), UINT64_C(0x156af68e148fe649), UINT64_C(0x156af68dfc8fe643),
        UINT64_C(0x156af84dfc8ee649), UINT64_C(0x156af68e148fe643)
    },
    {   UINT64_C(0x00000000000000c0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee17b8800675)
    },
    {   UINT64_C(0x00000000000088ed), UINT64_C(0x1cf2fbd898ab66e6), UINT64_C(0x1cf2fbce54ab6455),
        UINT64_C(0x1cf2fbcfccab65d7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000c2), UINT64_C(0x0000000000000000), UINT64_C(0x106b1617606c06bf),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000c3), UINT64_C(0x0000000000000000), UINT64_C(0x106b26d72c63a65d),
        UINT64_C(0x106b13d72c6d065d), UINT64_C(0x106b03d71c71c64b)
    },
    {   UINT64_C(0x00000000000000c4), UINT64_C(0x0000000000000000), UINT64_C(0x106b03572c75466c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001899f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x2512ff8b64ca06ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189a0), UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685),
        UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685)
    },
    {   UINT64_C(0x00000000000189a1), UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685),
        UINT64_C(0x2043140ef8aca697), UINT64_C(0x2043140ef8aca697)
    },
    {   UINT64_C(0x00000000000000c8), UINT64_C(0x0000000000000000), UINT64_C(0x106afa4c1063440a),
        UINT64_C(0x106b278c1079c40a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b214bbc6683a7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000ca), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b0b0a6c718369), UINT64_C(0x106affca4877036b)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000cd), UINT64_C(0x0000000000000000), UINT64_C(0x106b04d2dc74a4f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x106b04d15c74a568)
    },
    {   UINT64_C(0x00000000000000ce), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b0b0a6c716368), UINT64_C(0x106b0ac9f4716369)
    },
    {   UINT64_C(0x00000000000000cf), UINT64_C(0x106af80afc7b236e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189aa), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x17d2fa8e6497862f), UINT64_C(0x17d2fa8e6497862f)
    },
    {   UINT64_C(0x00000000000189ab), UINT64_C(0x11aaf98fd87f65cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189ac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x12dafe4d8481c666), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000d3), UINT64_C(0x0000000000000000), UINT64_C(0x106aef4a047f43e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061f7), UINT64_C(0x1f42f70ee0b64704), UINT64_C(0x1f42f9ceccb826fe),
        UINT64_C(0x1f43144ef0af8703), UINT64_C(0x1f42f90ea8b80716)
    },
    {   UINT64_C(0x00000000000088f7), UINT64_C(0x1f42ee0e10bb844c), UINT64_C(0x1f42ee1870bb86e4),
        UINT64_C(0x1f42ee0dd8bb84be), UINT64_C(0x1f42ee1870bb8764)
    },
    {   UINT64_C(0x00000000000061f8), UINT64_C(0x1f42f84f10b4a711), UINT64_C(0x1f42fbcf00b5071c),
        UINT64_C(0x1f42f68e90b3e71e), UINT64_C(0x1f42f8ced0b2072f)
    },
    {   UINT64_C(0x00000000000088f8), UINT64_C(0x1f42ee0dd8bb844c), UINT64_C(0x1f42ee1870bb86f2),
        UINT64_C(0x23c2fb8eccc6c616), UINT64_C(0x1c73028cd8a60690)
    },
    {   UINT64_C(0x00000000000000d8), UINT64_C(0x0000000000000000), UINT64_C(0x106b26d68c94c667),
        UINT64_C(0x106af20a047e03e0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088f9), UINT64_C(0x2232f9d310c16506), UINT64_C(0x1fd2f3cf50bae5f7),
        UINT64_C(0x1cf2fbd0c0ab659a), UINT64_C(0x18f31e8e1c9a2646)
    },
    {   UINT64_C(0x00000000000061fa), UINT64_C(0x1f42fc4e74afe721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccf64b5871c), UINT64_C(0x1f42fe4e50b446fa)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061fb), UINT64_C(0x1f42fbce9cb7e711), UINT64_C(0x1f42f70f38b3e6f1),
        UINT64_C(0x1f42fecf58b1c726), UINT64_C(0x1f42fcceecb86731)
    },
    {   UINT64_C(0x00000000000000dd), UINT64_C(0x106b16570462464c), UINT64_C(0x106b21cd746803e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000de), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b270cc06383b2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b1b4d7867c596), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061fd), UINT64_C(0x1f43020ea4b0e71f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fcd048b246d6), UINT64_C(0x1f42facea4b1c711)
    },
    {   UINT64_C(0x00000000000000e1), UINT64_C(0x106b1fd70467064b), UINT64_C(0x106b1197046e064b),
        UINT64_C(0x106af40fa07f4673), UINT64_C(0x106b1fd464670596)
    },
    {   UINT64_C(0x00000000000000e2), UINT64_C(0x106ad90fa0800667), UINT64_C(0x106ad90fa0800667),
        UINT64_C(0x106aeb4fa07fe667), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000e3), UINT64_C(0x106ae4cf707fa672), UINT64_C(0x0000000000000000),
        UINT64_C(0x106ade4fa07de672), UINT64_C(0x106ade4fa07ea672)
    },
    {   UINT64_C(0x00000000000000e4), UINT64_C(0x106b020f8c6b464c), UINT64_C(0x106af4cfa07ca673),
        UINT64_C(0x106af38fa07ac667), UINT64_C(0x106af58fa07ba667)
    },
    {   UINT64_C(0x00000000000000e5), UINT64_C(0x106af68fa07bc673), UINT64_C(0x106af44fa07d6667),
        UINT64_C(0x106af9cfa07aa667), UINT64_C(0x106af30fa07d6667)
    },
    {   UINT64_C(0x00000000000000e6), UINT64_C(0x106afc0f70792672), UINT64_C(0x106b018fa0762667),
        UINT64_C(0x106af44fa07c6667), UINT64_C(0x106af50fa07c6673)
    },
    {   UINT64_C(0x00000000000000e7), UINT64_C(0x106af24fa07de667), UINT64_C(0x106af4cfa07c8667),
        UINT64_C(0x106af5cfa07c0667), UINT64_C(0x106afc0f70792672)
    },
    {   UINT64_C(0x00000000000000e8), UINT64_C(0x106af30fa07d6667), UINT64_C(0x106af44fa07a8667),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af80fa07b0667)
    },
    {   UINT64_C(0x00000000000000e9), UINT64_C(0x106af48fa07cc667), UINT64_C(0x106af74fa07b4672),
        UINT64_C(0x106af80fa07b0667), UINT64_C(0x106af44fa07ca667)
    },
    {   UINT64_C(0x00000000000000ea), UINT64_C(0x106af74fa07b6667), UINT64_C(0x106af5cfa07c0672),
        UINT64_C(0x106b018fa07626f6), UINT64_C(0x106af74fa07b46f6)
    },
    {   UINT64_C(0x00000000000000eb), UINT64_C(0x106af8cf8c80464c), UINT64_C(0x106af8cf747a864c),
        UINT64_C(0x106af2cbfc7ac64c), UINT64_C(0x106b13cf8c6b464c)
    },
    {   UINT64_C(0x00000000000000ec), UINT64_C(0x106af8cf8c7a8683), UINT64_C(0x106af8cf8c80459c),
        UINT64_C(0x106af8cbfc7a866c), UINT64_C(0x106af54cdc7a859c)
    },
    {   UINT64_C(0x00000000000000ed), UINT64_C(0x106af98f8c7a4662), UINT64_C(0x106af8cf747a85a2),
        UINT64_C(0x106af98d8c7a4672), UINT64_C(0x106af2cbfc7ac5a2)
    },
    {   UINT64_C(0x00000000000000ee), UINT64_C(0x106af98f8c7a466c), UINT64_C(0x106b178f8c6b459c),
        UINT64_C(0x106af90f687a65a4), UINT64_C(0x106af54f8c7c6669)
    },
    {   UINT64_C(0x00000000000000ef), UINT64_C(0x106af80c147f85a1), UINT64_C(0x106af5cf8c7a859f),
        UINT64_C(0x106af98d8c7a4672), UINT64_C(0x106af84f8c7ae59c)
    },
    {   UINT64_C(0x00000000000000f0), UINT64_C(0x106af50f8c7ba596), UINT64_C(0x106af88c287a859c),
        UINT64_C(0x106af8cde47a859c), UINT64_C(0x106af78f8c79259c)
    },
    {   UINT64_C(0x00000000000000f1), UINT64_C(0x106af5cf8c7a0596), UINT64_C(0x106af8cf8c7a85a1),
        UINT64_C(0x106af74bfc7b65a2), UINT64_C(0x106af7cc0c7b259d)
    },
    {   UINT64_C(0x00000000000000f2), UINT64_C(0x106af8cbfc7a8672), UINT64_C(0x106af60f8c7c05a1),
        UINT64_C(0x106aed4f8c70e667), UINT64_C(0x106af8cf8c7a8666)
    },
    {   UINT64_C(0x00000000000000f3), UINT64_C(0x106af84f8c7ae64c), UINT64_C(0x106af8cf8c7a864c),
        UINT64_C(0x106af60f8c7c064c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000f4), UINT64_C(0x0000000000000000), UINT64_C(0x106af98f8c7a466c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189cf), UINT64_C(0x0000000000000000), UINT64_C(0x1552f6cdf08cc64c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000000f6), UINT64_C(0x106af6cfa07bc66c), UINT64_C(0x106afc4ec478c647),
        UINT64_C(0x106afa8f347d0667), UINT64_C(0x106af9cf5c7ae5a2)
    },
    {   UINT64_C(0x00000000000000f7), UINT64_C(0x106af4cfa07bc667), UINT64_C(0x106afc4c2878c597),
        UINT64_C(0x0000000000000000), UINT64_C(0x106afc4e0c78e604)
    },
    {   UINT64_C(0x00000000000000f8), UINT64_C(0x106aea8fa07cc682), UINT64_C(0x106af8cf747a8667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000585c), UINT64_C(0x1f43018f4cb10706), UINT64_C(0x1f43040ea0b06723),
        UINT64_C(0x1f43000efcae4718), UINT64_C(0x1f42fecf3cb78700)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000585d), UINT64_C(0x1f42fe0f18aee70b), UINT64_C(0x1f4302ceb8b26738),
        UINT64_C(0x1f42f98e78b04741), UINT64_C(0x1f42ffce9cb82735)
    },
    {   UINT64_C(0x000000000000620b), UINT64_C(0x1f42f90f20b44702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7ce78b3e70f), UINT64_C(0x1f42ff4e48af4736)
    },
    {   UINT64_C(0x00000000000000fd), UINT64_C(0x0000000000000000), UINT64_C(0x106afacf8c79c59c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000620c), UINT64_C(0x1f42f9ce38b50738), UINT64_C(0x1f4310ced8afc72d),
        UINT64_C(0x1f42fa4edcb1e705), UINT64_C(0x1f42f74f00b4c706)
    },
    {   UINT64_C(0x000000000000585f), UINT64_C(0x1f430d0ee0ad472e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010f40b16712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000100), UINT64_C(0x106af48fa07d470f), UINT64_C(0x106af48fa07d471b),
        UINT64_C(0x106af38f747e264b), UINT64_C(0x106af94fa07aa70f)
    },
    {   UINT64_C(0x0000000000000101), UINT64_C(0x106af80f747b0656), UINT64_C(0x106afb0f74798656),
        UINT64_C(0x106b01cfa076464b), UINT64_C(0x106b01cfa076471b)
    },
    {   UINT64_C(0x0000000000000102), UINT64_C(0x106affcf747a464b), UINT64_C(0x106af14f747ca64b),
        UINT64_C(0x106af08fa07ca64b), UINT64_C(0x106af40fa07f064b)
    },
    {   UINT64_C(0x0000000000000103), UINT64_C(0x106af48fa07c670f), UINT64_C(0x106af28fa07da70f),
        UINT64_C(0x106af44f747d472d), UINT64_C(0x106af28d087da64b)
    },
    {   UINT64_C(0x0000000000000104), UINT64_C(0x106af4cfa07ca657), UINT64_C(0x106af38fa07ac64b),
        UINT64_C(0x106af3cfa07ac64b), UINT64_C(0x106af94fa07aa64b)
    },
    {   UINT64_C(0x0000000000000105), UINT64_C(0x106af10d087ea64b), UINT64_C(0x106af48fa07d464b),
        UINT64_C(0x106aee0fa080064b), UINT64_C(0x106afc0f747a6656)
    },
    {   UINT64_C(0x0000000000000106), UINT64_C(0x106af28fa07da64b), UINT64_C(0x106af28fa07da72d),
        UINT64_C(0x106af48fa07c664b), UINT64_C(0x106af2cf747da64b)
    },
    {   UINT64_C(0x0000000000000107), UINT64_C(0x106af24fa07de64b), UINT64_C(0x106af28fa07da64b),
        UINT64_C(0x106afc0f74790656), UINT64_C(0x106af28fa07da64b)
    },
    {   UINT64_C(0x0000000000000108), UINT64_C(0x106af48fa07a864b), UINT64_C(0x106af80f747b0656),
        UINT64_C(0x106af48fa07cc64b), UINT64_C(0x106af44f747d464b)
    },
    {   UINT64_C(0x0000000000000109), UINT64_C(0x106af2cfa07da64b), UINT64_C(0x106af4cfa07c864b),
        UINT64_C(0x106af28d087e864b), UINT64_C(0x106af28fa07da64b)
    },
    {   UINT64_C(0x000000000000010a), UINT64_C(0x106af08fa07ee64b), UINT64_C(0x106af08d087f664b),
        UINT64_C(0x106af30fa07be64b), UINT64_C(0x106af0cfa07e864b)
    },
    {   UINT64_C(0x000000000000010b), UINT64_C(0x106af94fa078a64b), UINT64_C(0x106af7cf747b0656),
        UINT64_C(0x106aee0f747f2656), UINT64_C(0x106af54fa07dc64b)
    },
    {   UINT64_C(0x000000000000010c), UINT64_C(0x106afbcf888005a0), UINT64_C(0x106afc0f8c78e64b),
        UINT64_C(0x106af14fa0786597), UINT64_C(0x106af94fa0790596)
    },
    {   UINT64_C(0x000000000000010d), UINT64_C(0x106af04d787ec596), UINT64_C(0x106afbcf887885a0),
        UINT64_C(0x106aee4fa07fc596), UINT64_C(0x106afb0f8c78059f)
    },
    {   UINT64_C(0x000000000000010e), UINT64_C(0x106af24fa07dc596), UINT64_C(0x106af24fa07dc678),
        UINT64_C(0x106af24fa07e0596), UINT64_C(0x106af28f807dc596)
    },
    {   UINT64_C(0x000000000000010f), UINT64_C(0x106af08fa07ec596), UINT64_C(0x106af24fa07dc596),
        UINT64_C(0x106afc4f8878e5a0), UINT64_C(0x106af24fa07dc596)
    },
    {   UINT64_C(0x0000000000000110), UINT64_C(0x106af10c287b859d), UINT64_C(0x106b004f8878c5a0),
        UINT64_C(0x106af48fa07cc596), UINT64_C(0x106af48c107d0597)
    },
    {   UINT64_C(0x0000000000000111), UINT64_C(0x106aef0c287f864b), UINT64_C(0x106af4cfa07c6596),
        UINT64_C(0x106af24d787ee596), UINT64_C(0x106af24fa07dc596)
    },
    {   UINT64_C(0x0000000000000112), UINT64_C(0x106aeecfa07fa596), UINT64_C(0x106aeecd78806596),
        UINT64_C(0x106af2cfa079c596), UINT64_C(0x106aefcfa07f2596)
    },
    {   UINT64_C(0x0000000000000113), UINT64_C(0x106af90fa078c596), UINT64_C(0x106afc8f8c76e59f),
        UINT64_C(0x106aee8f8c7fc59f), UINT64_C(0x106af20fa07e8596)
    },
    {   UINT64_C(0x0000000000000114), UINT64_C(0x106afbcf88788667), UINT64_C(0x106afbcf8878864a),
        UINT64_C(0x106aeb8c287ca64b), UINT64_C(0x106af9cfa07ac667)
    },
    {   UINT64_C(0x0000000000000115), UINT64_C(0x106b000f8c78c59f), UINT64_C(0x106b000f8877e5a0),
        UINT64_C(0x106b018fa077664c), UINT64_C(0x106b018fa077664b)
    },
    {   UINT64_C(0x0000000000000116), UINT64_C(0x106afa0c106c864c), UINT64_C(0x106af14f807d2596),
        UINT64_C(0x106aef4fa07d2596), UINT64_C(0x106aeb8fa07ec64b)
    },
    {   UINT64_C(0x0000000000000117), UINT64_C(0x106af24fa07e0667), UINT64_C(0x106af24fa07dc667),
        UINT64_C(0x106af48c107d0678), UINT64_C(0x106af24d787dc596)
    },
    {   UINT64_C(0x0000000000000118), UINT64_C(0x106af54f747c8656), UINT64_C(0x106aef0f8c7fa59f),
        UINT64_C(0x106af38fa07b869e), UINT64_C(0x106af28fa0786683)
    },
    {   UINT64_C(0x0000000000000119), UINT64_C(0x106aee0f747cc656), UINT64_C(0x106aee8f8c7ae59f),
        UINT64_C(0x106af44fa07ce657), UINT64_C(0x106af38fa07d259d)
    },
    {   UINT64_C(0x000000000000011a), UINT64_C(0x106aee0fa07de657), UINT64_C(0x106aec8fa07e259d),
        UINT64_C(0x106af00fa07ee64b), UINT64_C(0x106af08fa07ec596)
    },
    {   UINT64_C(0x000000000000011b), UINT64_C(0x106aee0fa07f264b), UINT64_C(0x106aeb4fa07f2596),
        UINT64_C(0x106af10c147b26e0), UINT64_C(0x106af84c1477861d)
    },
    {   UINT64_C(0x000000000000011c), UINT64_C(0x106af74fa07b464b), UINT64_C(0x106aec8c2881064b),
        UINT64_C(0x106afc0f70790656), UINT64_C(0x106af9cf747a25a2)
    },
    {   UINT64_C(0x000000000000011d), UINT64_C(0x106af20f807dc656), UINT64_C(0x106af34f807de59c),
        UINT64_C(0x106aeecf807dc70f), UINT64_C(0x106aed4f807e0667)
    },
    {   UINT64_C(0x000000000000011e), UINT64_C(0x106af30f747d864b), UINT64_C(0x106b034f74756650),
        UINT64_C(0x106af98ebc7a4684), UINT64_C(0x106af8ced87aa5cb)
    },
    {   UINT64_C(0x000000000000011f), UINT64_C(0x106aeb0f747ca726), UINT64_C(0x106aef0f8c7fa672),
        UINT64_C(0x106af54f747c871c), UINT64_C(0x106aef0f8c7fa667)
    },
    {   UINT64_C(0x0000000000000120), UINT64_C(0x106af80bfc7fa656), UINT64_C(0x106b000c147b45a4),
        UINT64_C(0x106afe4e047b64b6), UINT64_C(0x106af9972c7a4672)
    },
    {   UINT64_C(0x0000000000000121), UINT64_C(0x106af0968883a672), UINT64_C(0x106b11d72c6dc656),
        UINT64_C(0x106b12572c6e0656), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000122), UINT64_C(0x106ace4b289006de), UINT64_C(0x106aba89a899e73e),
        UINT64_C(0x106af28d087e872d), UINT64_C(0x106af24d787ee678)
    },
    {   UINT64_C(0x0000000000000123), UINT64_C(0x106af3cfa07b864b), UINT64_C(0x106aef8fa0786667),
        UINT64_C(0x106af44fa07a864b), UINT64_C(0x106af0cc2880659c)
    },
    {   UINT64_C(0x0000000000000124), UINT64_C(0x106af94fa07aa6f2), UINT64_C(0x106af94fa07925da),
        UINT64_C(0x106aec0fa07ae64b), UINT64_C(0x106af94fa0790596)
    },
    {   UINT64_C(0x0000000000000125), UINT64_C(0x106af38c107ac64b), UINT64_C(0x106af94c107c4596),
        UINT64_C(0x106aee0d0880464b), UINT64_C(0x106aee4d787fe596)
    },
    {   UINT64_C(0x0000000000000126), UINT64_C(0x106afc0bf07a6656), UINT64_C(0x106afb0bf078059f),
        UINT64_C(0x106af44d087c864b), UINT64_C(0x106af24d787e0596)
    },
    {   UINT64_C(0x0000000000000127), UINT64_C(0x106af30fa07cc64b), UINT64_C(0x106af10fa07e6596),
        UINT64_C(0x106aea4fa07c664b), UINT64_C(0x106af24fa07e064b)
    },
    {   UINT64_C(0x0000000000000128), UINT64_C(0x106af30fa07d264b), UINT64_C(0x106af2cfa07e6596),
        UINT64_C(0x106af28d087e864b), UINT64_C(0x106af24d787ee596)
    },
    {   UINT64_C(0x0000000000000129), UINT64_C(0x106af0cfa07da64b), UINT64_C(0x106af08fa07dc597),
        UINT64_C(0x106aed4c107fc64b), UINT64_C(0x106aee8c10814596)
    },
    {   UINT64_C(0x000000000000012a), UINT64_C(0x106afc0dfc7da656), UINT64_C(0x106afc0dfc79659f),
        UINT64_C(0x106af80bf07b2656), UINT64_C(0x106b000bf078a59f)
    },
    {   UINT64_C(0x000000000000012b), UINT64_C(0x106af48d087ce64b), UINT64_C(0x106af44d787cc596),
        UINT64_C(0x106af2cfa07da64b), UINT64_C(0x106af1cc287e2596)
    },
    {   UINT64_C(0x000000000000012c), UINT64_C(0x106af28fa07da64b), UINT64_C(0x106af1cc287e2596),
        UINT64_C(0x106af4cd087d464b), UINT64_C(0x106af4cd787d2596)
    },
    {   UINT64_C(0x000000000000012d), UINT64_C(0x106af2cd087f464b), UINT64_C(0x106aeecd787fc596),
        UINT64_C(0x106af28d087e864b), UINT64_C(0x106af24d787ee596)
    },
    {   UINT64_C(0x000000000000012e), UINT64_C(0x106af28fa07da64b), UINT64_C(0x106af24fa07dc596),
        UINT64_C(0x106af28fa07da64b), UINT64_C(0x106af10fa07e864b)
    },
    {   UINT64_C(0x000000000000012f), UINT64_C(0x106af24f747ee656), UINT64_C(0x106af3cf8c79e59f),
        UINT64_C(0x106af24c187ee656), UINT64_C(0x106af3cc0c79e59f)
    },
    {   UINT64_C(0x0000000000000130), UINT64_C(0x106b01cfa076464b), UINT64_C(0x106aee0fa080071b),
        UINT64_C(0x106aee4fa07fc667), UINT64_C(0x106af44c107b264b)
    },
    {   UINT64_C(0x0000000000000131), UINT64_C(0x106af24c147b6596), UINT64_C(0x106af2cd087da64b),
        UINT64_C(0x106af28d787ec596), UINT64_C(0x106af28c147da64b)
    },
    {   UINT64_C(0x0000000000000132), UINT64_C(0x106af24c147dc596), UINT64_C(0x106af28d087e664b),
        UINT64_C(0x106af24d787ec596), UINT64_C(0x106af28d087da64b)
    },
    {   UINT64_C(0x0000000000000133), UINT64_C(0x106af24d787dc596), UINT64_C(0x106af24d087e864b),
        UINT64_C(0x106af08d787fa596), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000134), UINT64_C(0x106af4cfa07ca71b), UINT64_C(0x106afbcf88800667),
        UINT64_C(0x106af4cfa07ca71b), UINT64_C(0x106afbcf8880064b)
    },
    {   UINT64_C(0x0000000000000135), UINT64_C(0x106af38fa07d064b), UINT64_C(0x106af60f8c7d859f),
        UINT64_C(0x106af48fa07d471b), UINT64_C(0x106afbcf88788667)
    },
    {   UINT64_C(0x0000000000000136), UINT64_C(0x106af0cf747d2656), UINT64_C(0x106af8cf8c77a59c),
        UINT64_C(0x106af08f747d671b), UINT64_C(0x106af8cf8c77a64b)
    },
    {   UINT64_C(0x0000000000000137), UINT64_C(0x106aee0fa080071b), UINT64_C(0x106aee4fa07fc64b),
        UINT64_C(0x106afc0f747a671b), UINT64_C(0x106afb0f8c78064a)
    },
    {   UINT64_C(0x0000000000000138), UINT64_C(0x106af78f747b264b), UINT64_C(0x106afa4c1079e596),
        UINT64_C(0x106af28fa07da71b), UINT64_C(0x106af24fa07dc656)
    },
    {   UINT64_C(0x0000000000000139), UINT64_C(0x106af28fa07da71b), UINT64_C(0x106af24fa07dc64a),
        UINT64_C(0x106afc0f7479071b), UINT64_C(0x106afc4f8878e64a)
    },
    {   UINT64_C(0x000000000000013a), UINT64_C(0x106afc0f70790656), UINT64_C(0x106afc0f8c78e59f),
        UINT64_C(0x106afc0f7079071b), UINT64_C(0x106afc0f8c78e64a)
    },
    {   UINT64_C(0x000000000000013b), UINT64_C(0x106af7cf747b071b), UINT64_C(0x106afc4f8c76e64a),
        UINT64_C(0x106af44f747d471b), UINT64_C(0x106af48c107d0656)
    },
    {   UINT64_C(0x000000000000013c), UINT64_C(0x106af44f747d471b), UINT64_C(0x106af48c107d064b),
        UINT64_C(0x106af44f747f670f), UINT64_C(0x106af48c107f4667)
    },
    {   UINT64_C(0x000000000000013d), UINT64_C(0x106af28fa07da71b), UINT64_C(0x106af24fa07dc64a),
        UINT64_C(0x106af94d087aa64b), UINT64_C(0x106af94d78790596)
    },
    {   UINT64_C(0x000000000000013e), UINT64_C(0x106af0cfa07e871b), UINT64_C(0x106aefcfa07f264b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000013f), UINT64_C(0x106af4cc1478e64b), UINT64_C(0x106af4cc147aa596),
        UINT64_C(0x106af10fa07e864b), UINT64_C(0x106af4cfa07c6596)
    },
    {   UINT64_C(0x000000000000622d), UINT64_C(0x1f42f5cef0b7a718), UINT64_C(0x1f42fbcf54b4a70a),
        UINT64_C(0x1f42fa8e1cb6a721), UINT64_C(0x1f42f98f94b7a6ff)
    },
    {   UINT64_C(0x000000000000892d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000000622e), UINT64_C(0x1f42f68eacb406ff), UINT64_C(0x1f42fa8f44afa6f6),
        UINT64_C(0x1f42f88f34ac270c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000892e), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000000144), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af2cc147da64b), UINT64_C(0x106af28c147dc596)
    },
    {   UINT64_C(0x000000000000892f), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006230), UINT64_C(0x1f42ff8f6cae86fb), UINT64_C(0x1f42fd4e74b2a744),
        UINT64_C(0x1f430ece64b48725), UINT64_C(0x1f42f68ef0b76728)
    },
    {   UINT64_C(0x0000000000008930), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006231), UINT64_C(0x1f42f58f18b7a70d), UINT64_C(0x1f43040f14b2670e),
        UINT64_C(0x1f42fb8fb4ad26ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008931), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006232), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ef8b346fa),
        UINT64_C(0x1f43014e38b1074c), UINT64_C(0x1f42fa8eb8b5c702)
    },
    {   UINT64_C(0x0000000000008932), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006233), UINT64_C(0x1f42f88f64b1c6f6), UINT64_C(0x1f42f78f28b14706),
        UINT64_C(0x1f42fa8f94b1470e), UINT64_C(0x1f43068eb8b2c72d)
    },
    {   UINT64_C(0x0000000000008933), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006234), UINT64_C(0x1f42f68e7ca78702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4e88b0272c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008934), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006235), UINT64_C(0x1f43074e78b5c739), UINT64_C(0x1f42fc4f8cb90706),
        UINT64_C(0x1f42f58ec8b7a702), UINT64_C(0x1f42fa8f48b88715)
    },
    {   UINT64_C(0x0000000000008935), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006236), UINT64_C(0x1f42fd8e88b82742), UINT64_C(0x1f42f68fb8b4c70e),
        UINT64_C(0x1f430d0e64b4e731), UINT64_C(0x1f430b4e54af8736)
    },
    {   UINT64_C(0x0000000000008936), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008937), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006238), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f58b626fb),
        UINT64_C(0x1f42fa8fe0b3470d), UINT64_C(0x1f42fd8f74b5c6ff)
    },
    {   UINT64_C(0x0000000000008938), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006239), UINT64_C(0x1f42f68edcb3e717), UINT64_C(0x1f42f98eecacc6eb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f78b34714)
    },
    {   UINT64_C(0x0000000000008939), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000000623a), UINT64_C(0x1f42f3ce5cb7473e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ef8adc705)
    },
    {   UINT64_C(0x000000000000893a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000623b), UINT64_C(0x1f42fa8f74b4c716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78f64afc6ff), UINT64_C(0x1f42f98f68ae0702)
    },
    {   UINT64_C(0x000000000000588e), UINT64_C(0x1f43038ecca006f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7500cb526ee), UINT64_C(0x1f42fc8f60b2c738)
    },
    {   UINT64_C(0x000000000000623c), UINT64_C(0x1f42f88fc4b1a706), UINT64_C(0x1f42f78ed8b7a730),
        UINT64_C(0x1f42ff8f94b2c706), UINT64_C(0x1f43048e88b46742)
    },
    {   UINT64_C(0x000000000000588f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8e7cae273a),
        UINT64_C(0x1f42fc8f60b2c738), UINT64_C(0x1f42f70e64b0e73d)
    },
    {   UINT64_C(0x000000000000623d), UINT64_C(0x1f43098e78b6e744), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68eccb36706), UINT64_C(0x1f42f68ee4b7270c)
    },
    {   UINT64_C(0x0000000000005890), UINT64_C(0x1f43014e8cb28715), UINT64_C(0x1f42f20eb4a7e718),
        UINT64_C(0x1f43080ef09f4723), UINT64_C(0x1f43064ed4a5670f)
    },
    {   UINT64_C(0x000000000000623e), UINT64_C(0x1f42f98ef8b82719), UINT64_C(0x1f4303ce88b36735),
        UINT64_C(0x1f42f68f2cb6470e), UINT64_C(0x1f430eced4afa71d)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000623f), UINT64_C(0x1f42fb8ea4b3a70a), UINT64_C(0x1f43080e64b4872f),
        UINT64_C(0x1f42f88fc4b06712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005892), UINT64_C(0x1f42f88eb8a9873e), UINT64_C(0x1f42fb8e50ab273c),
        UINT64_C(0x1f43070f5cabe735), UINT64_C(0x1f43068e88b10727)
    },
    {   UINT64_C(0x0000000000018a40), UINT64_C(0x1cf2fc1084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbceecab660f), UINT64_C(0x1cf2fbceecab660f)
    },
    {   UINT64_C(0x0000000000018a41), UINT64_C(0x1cf2f30eecafc60f), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbceecab660f), UINT64_C(0x1cf2fbceecab660f)
    },
    {   UINT64_C(0x0000000000018a42), UINT64_C(0x0ffafccd3077067e), UINT64_C(0x0ffafccd3077067e),
        UINT64_C(0x1cf2ffcd30a9467e), UINT64_C(0x1cf3000d30a9667e)
    },
    {   UINT64_C(0x0000000000018a43), UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018a44), UINT64_C(0x2c22fbd084e825a9), UINT64_C(0x1cf2fbd084aba5a9),
        UINT64_C(0x0ffafccd3077067e), UINT64_C(0x0ffafccd3077067e)
    },
    {   UINT64_C(0x0000000000008942), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a46), UINT64_C(0x1cf2fa9084ac05aa), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018a47), UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65aa)
    },
    {   UINT64_C(0x0000000000018a48), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30aba67c), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x0000000000018a49), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30aba67e)
    },
    {   UINT64_C(0x0000000000006245), UINT64_C(0x1f42fd8f44b1e6ee), UINT64_C(0x1f42f4ce8cb74716),
        UINT64_C(0x1f42fc4e80b886fd), UINT64_C(0x1f42f98ee0b506f3)
    },
    {   UINT64_C(0x0000000000008945), UINT64_C(0x16233d53cc988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a4c), UINT64_C(0x1cf2fbcd38aba67e), UINT64_C(0x1cf2fb8d30ab467e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018a4d), UINT64_C(0x1cf3114fa0a2261a), UINT64_C(0x1cf3114fa0a2261a),
        UINT64_C(0x188303cd148fa5d5), UINT64_C(0x18830fcd1495a5d5)
    },
    {   UINT64_C(0x0000000000018a4e), UINT64_C(0x0ffaff8d3c726679), UINT64_C(0x0ffb060d3c75a679),
        UINT64_C(0x1cf2ff50d8aac58f), UINT64_C(0x1cf2fd10d8a9a58f)
    },
    {   UINT64_C(0x0000000000008947), UINT64_C(0x16233d4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a50), UINT64_C(0x1cf3050dd0a6c6fc), UINT64_C(0x1cf3050dd0a6c6fc),
        UINT64_C(0x1cf2fbce30ab663e), UINT64_C(0x1cf2fbce30ab663e)
    },
    {   UINT64_C(0x0000000000018a51), UINT64_C(0x1cf2fbce30ab663e), UINT64_C(0x1cf2fbce8cab65a9),
        UINT64_C(0x1cf2fbce8cab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018a52), UINT64_C(0x1cf2f79084ad85a9), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2f790acad859f), UINT64_C(0x1cf2f790acad859f)
    },
    {   UINT64_C(0x0000000000008949), UINT64_C(0x1622eb4ae46f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a54), UINT64_C(0x1cf2f790acad8506), UINT64_C(0x0ffaff4d3062867e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x0000000000018a55), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x0000000000018a56), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x000000000000894b), UINT64_C(0x1622eb4ae498653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a58), UINT64_C(0x0ffafccd3076c67e), UINT64_C(0x0ffafccd3076c67e),
        UINT64_C(0x1cf2fbcf1cab6603), UINT64_C(0x156afc4d308d267e)
    },
    {   UINT64_C(0x0000000000018a59), UINT64_C(0x1cf2fbcf1cab6603), UINT64_C(0x156afc4d308d267e),
        UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a)
    },
    {   UINT64_C(0x0000000000018a5a), UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a),
        UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a)
    },
    {   UINT64_C(0x000000000000894d), UINT64_C(0x1622eb53cc986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a5c), UINT64_C(0x1cf2fbd180ab656a), UINT64_C(0x1cf2fbd084ab661d),
        UINT64_C(0x1cf2fbd084ab65e4), UINT64_C(0x1cf2fbcf98ab65a9)
    },
    {   UINT64_C(0x0000000000018a5d), UINT64_C(0x1cf2fbcf98ab65a9), UINT64_C(0x1cf2fbcd6cab65a9),
        UINT64_C(0x156afe0b348c2647), UINT64_C(0x1cf2f94f38ab65fc)
    },
    {   UINT64_C(0x0000000000018a5e), UINT64_C(0x156afe0b388c2648), UINT64_C(0x156afe0b888c265b),
        UINT64_C(0x1cf2f94f38ab65fc), UINT64_C(0x156afdcb888c065b)
    },
    {   UINT64_C(0x000000000000894f), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006250), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98fa4b1a6d7),
        UINT64_C(0x1f42f88efcb6e707), UINT64_C(0x1f42ffce34b2a730)
    },
    {   UINT64_C(0x0000000000018a61), UINT64_C(0x12a30bcc4c7b86b7), UINT64_C(0x0e7aff8c4c6fa6b7),
        UINT64_C(0x0e7aff8c4c6fa6b7), UINT64_C(0x0ffb0acc486c66b7)
    },
    {   UINT64_C(0x0000000000006251), UINT64_C(0x1f43034e68afe738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f58ea8b1c710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a63), UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0eab1d0c4c7666b7),
        UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0eab1d0c4c7666b7)
    },
    {   UINT64_C(0x0000000000006252), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e98b086ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78eecb726ee)
    },
    {   UINT64_C(0x0000000000018a65), UINT64_C(0x156b058d648fe671), UINT64_C(0x156b038d648ee671),
        UINT64_C(0x156af88d64896671), UINT64_C(0x0f431d0c4c78c6b7)
    },
    {   UINT64_C(0x0000000000018a66), UINT64_C(0x0f42f38c4c6406b7), UINT64_C(0x069afa8d3452a67d),
        UINT64_C(0x0ffb1a4d3068467e), UINT64_C(0x156af88e8091460b)
    },
    {   UINT64_C(0x0000000000018a67), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x156afa8fa08dc630),
        UINT64_C(0x156af0cfa092e60b), UINT64_C(0x156af40fa08ee574)
    },
    {   UINT64_C(0x0000000000018a68), UINT64_C(0x156afc4f6c9045ed), UINT64_C(0x156b014fd08a6649),
        UINT64_C(0x156af88fa091460b), UINT64_C(0x156af88fa091460b)
    },
    {   UINT64_C(0x0000000000018a69), UINT64_C(0x156af3cc808ee60b), UINT64_C(0x156af88c8091460b),
        UINT64_C(0x1c22f80fa0aa060b), UINT64_C(0x1c22f80fa0aa060b)
    },
    {   UINT64_C(0x0000000000018a6a), UINT64_C(0x156af54e808fa626), UINT64_C(0x156af70e80908626),
        UINT64_C(0x156af54e908fa62a), UINT64_C(0x156af70e9090862a)
    },
    {   UINT64_C(0x00000000000058a8), UINT64_C(0x1f42ff0f88b0c718), UINT64_C(0x1f42f94f38b6c711),
        UINT64_C(0x1f42f80f24b7a6f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a6c), UINT64_C(0x17d2fa8e6497862f), UINT64_C(0x17d2fa8e649786b5),
        UINT64_C(0x17d2fa8e6497871b), UINT64_C(0x17d2fa8e64978728)
    },
    {   UINT64_C(0x0000000000008956), UINT64_C(0x16233d5258988776), UINT64_C(0x16232653cc988776),
        UINT64_C(0x1623265258986776), UINT64_C(0x1622eb52586f4776)
    },
    {   UINT64_C(0x0000000000018a6e), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000008957), UINT64_C(0x1622eb53cc7ae776), UINT64_C(0x1622eb52587ae776),
        UINT64_C(0x16233d4ae4988776), UINT64_C(0x1623264ae4988776)
    },
    {   UINT64_C(0x0000000000018a70), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x221afa8dc4c28657), UINT64_C(0x221afa8dc4c28657)
    },
    {   UINT64_C(0x0000000000018a71), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x0000000000018a72), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x20b2fe0c64b946b0),
        UINT64_C(0x23caf6cda4c947d3), UINT64_C(0x23caf6cb94c94713)
    },
    {   UINT64_C(0x0000000000018a73), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x246af1cda4ce4713),
        UINT64_C(0x1862ff8c9c9746a2), UINT64_C(0x1acafa0f3c9fa5fb)
    },
    {   UINT64_C(0x0000000000018a74), UINT64_C(0x1acb020f3ca3a5fb), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953054e6098062e), UINT64_C(0x1953054e6098062e)
    },
    {   UINT64_C(0x0000000000018a75), UINT64_C(0x1953058e6098262e), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953008e109a8642), UINT64_C(0x1953008e109a8642)
    },
    {   UINT64_C(0x0000000000018a76), UINT64_C(0x0ffb1a4dd0684654), UINT64_C(0x0ffb1a0dd0682654),
        UINT64_C(0x15831a4dd07e6654), UINT64_C(0x15831a4dd07e6654)
    },
    {   UINT64_C(0x0000000000018a77), UINT64_C(0x1cf2ff8fa0a94596), UINT64_C(0x1cf2ff8fa0a94675),
        UINT64_C(0x1cf2ff8caca9469f), UINT64_C(0x23cafa913cc7657b)
    },
    {   UINT64_C(0x0000000000018a78), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x14daf40f808b25e8),
        UINT64_C(0x18b2fe0fa0994586), UINT64_C(0x156af9cd308e267e)
    },
    {   UINT64_C(0x0000000000018a79), UINT64_C(0x156af9cd308e2732), UINT64_C(0x156afa0d308e267e),
        UINT64_C(0x18f2fd0e689ac630), UINT64_C(0x13a2fa0cac87069f)
    },
    {   UINT64_C(0x0000000000018a7a), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x17d2f60bdc99c6d0), UINT64_C(0x18b2f94b1c9ba703)
    },
    {   UINT64_C(0x0000000000018a7b), UINT64_C(0x17d2fa8bdc9786cf), UINT64_C(0x17d2fa8bdc9786cf),
        UINT64_C(0x17d3064bd491a6d3), UINT64_C(0x17d3064bd491a6d3)
    },
    {   UINT64_C(0x0000000000018a7c), UINT64_C(0x17d2f60bd499c6d3), UINT64_C(0x17d2f60bd499c6d3),
        UINT64_C(0x17d2fe4bd495a6d3), UINT64_C(0x17d2fe4bd495a6d3)
    },
    {   UINT64_C(0x0000000000018a7d), UINT64_C(0x1cf2fbcf6cab65f1), UINT64_C(0x105af98c287a46ae),
        UINT64_C(0x1043058e6073c6e5), UINT64_C(0x1043058e6073c67e)
    },
    {   UINT64_C(0x0000000000018a7e), UINT64_C(0x12d3034a8c7f269f), UINT64_C(0x12d3034a8c7f269f),
        UINT64_C(0x156afe0f3c8c25fc), UINT64_C(0x156afe0f3c8c25fc)
    },
    {   UINT64_C(0x0000000000018a7f), UINT64_C(0x0e5b014c4c6fc6b1), UINT64_C(0x0e5afe4c4c6e46b1),
        UINT64_C(0x17d3054f10922604), UINT64_C(0x17d3055298922524)
    },
    {   UINT64_C(0x0000000000018a80), UINT64_C(0x2332fb4b94c4a6e3), UINT64_C(0x2332fb4b94c4a6e3),
        UINT64_C(0x2332fb4b94c4a6e3), UINT64_C(0x1ce3058b80a646e8)
    },
    {   UINT64_C(0x0000000000018a81), UINT64_C(0x1ce3058b80a646e8), UINT64_C(0x1ce3058b94a646e3),
        UINT64_C(0x1ce3058b94a646e3), UINT64_C(0x24e302cb94c7a6e3)
    },
    {   UINT64_C(0x0000000000018a82), UINT64_C(0x24e302cb94c7a6e3), UINT64_C(0x1feafa8b94b7e6e3),
        UINT64_C(0x1c92fc8b94a986e3), UINT64_C(0x1592f64aa090e720)
    },
    {   UINT64_C(0x0000000000018a83), UINT64_C(0x2502f60aa0e90720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x0000000000018a84), UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x103af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x0000000000008962), UINT64_C(0x25eaee0ae4d62776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a86), UINT64_C(0x10daf60aa08e6720), UINT64_C(0x117af60aa08e6720),
        UINT64_C(0x117af60aa08e6720), UINT64_C(0x0e5af60aa08e679b)
    },
    {   UINT64_C(0x0000000000018a87), UINT64_C(0x0e5af608b48e6720), UINT64_C(0x27b3054bb0d1a6ea),
        UINT64_C(0x1fd2f94b98b206e4), UINT64_C(0x0fbb098b1471c61e)
    },
    {   UINT64_C(0x0000000000018a88), UINT64_C(0x24cb068da4c56661), UINT64_C(0x111b23caf079870d),
        UINT64_C(0x156afa0efc8e071b), UINT64_C(0x156afa0efc8e06ab)
    },
    {   UINT64_C(0x0000000000008964), UINT64_C(0x25e3858ae4d60776), UINT64_C(0x2582ee0b98d48744),
        UINT64_C(0x2582ee0b5cd48752), UINT64_C(0x2582ee0b5cd48752)
    },
    {   UINT64_C(0x0000000000018a8a), UINT64_C(0x156afa0efc8e060b), UINT64_C(0x156afa13108e2614),
        UINT64_C(0x156afa0f8c8e2506), UINT64_C(0x156afa0f8c8e25e7)
    },
    {   UINT64_C(0x0000000000018a8b), UINT64_C(0x156afa0f8c8e25e7), UINT64_C(0x16a2fa8dc492c657),
        UINT64_C(0x16a2fa8dc492c657), UINT64_C(0x0f52fa908c7565a7)
    },
    {   UINT64_C(0x0000000000018a8c), UINT64_C(0x1402f8cf3c892696), UINT64_C(0x1402f8cd988925f9),
        UINT64_C(0x1402f8ceec8925f9), UINT64_C(0x156afc4f208ce602)
    },
    {   UINT64_C(0x0000000000018a8d), UINT64_C(0x1512fa8dc48c8657), UINT64_C(0x1512fa8dc48c8657),
        UINT64_C(0x1a52fa8dc4a18712), UINT64_C(0x2262fa8bc0c1c6d8)
    },
    {   UINT64_C(0x0000000000018a8e), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713)
    },
    {   UINT64_C(0x0000000000018a8f), UINT64_C(0x12c2fa116c838571), UINT64_C(0x12c2fa116c838571),
        UINT64_C(0x0962fd0cf85b45d0), UINT64_C(0x15b2f58fa091867e)
    },
    {   UINT64_C(0x0000000000018a90), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860684), UINT64_C(0x1362fa0f24860684)
    },
    {   UINT64_C(0x0000000000008968), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x0000000000018a92), UINT64_C(0x1362fa094886077a), UINT64_C(0x1362fa094886077a),
        UINT64_C(0x21b2fa0f04bf45f8), UINT64_C(0x21b2fa0f00bf45f7)
    },
    {   UINT64_C(0x0000000000008969), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x0000000000018a94), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f5cf2c8826ae),
        UINT64_C(0x1362f5cf2c8826ae), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x0000000000018a95), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1862f60f2c9c0600),
        UINT64_C(0x1862f60f2c9c0600), UINT64_C(0x1613088f2c93e600)
    },
    {   UINT64_C(0x0000000000018a96), UINT64_C(0x1612f3cf2c898600), UINT64_C(0x1362f5ce6c882631),
        UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x0000000000018a97), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362f5cf2c882600),
        UINT64_C(0x1362f5cf2c882704), UINT64_C(0x1362f58d74884600)
    },
    {   UINT64_C(0x0000000000018a98), UINT64_C(0x1362f58b1c884600), UINT64_C(0x1362f94fa08665c7),
        UINT64_C(0x1362f58f2c88470e), UINT64_C(0x1362f58b1c884600)
    },
    {   UINT64_C(0x000000000000896c), UINT64_C(0x23a2f50f74c985ea), UINT64_C(0x23a2f54f74c965ea),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713)
    },
    {   UINT64_C(0x0000000000018a9a), UINT64_C(0x156afa0cac8e269f), UINT64_C(0x156afa0d308e267e),
        UINT64_C(0x156afa11f08e25f1), UINT64_C(0x156afa0f6c8e25f1)
    },
    {   UINT64_C(0x0000000000018a9b), UINT64_C(0x156afa0f088e2607), UINT64_C(0x156afa0f348e26a7),
        UINT64_C(0x156afa11808e2737), UINT64_C(0x156afa10048e26ab)
    },
    {   UINT64_C(0x0000000000018a9c), UINT64_C(0x156afa0d588e2672), UINT64_C(0x156afa0d6c8e2670),
        UINT64_C(0x156afa0d6c8e2670), UINT64_C(0x156afa0f348e25fc)
    },
    {   UINT64_C(0x0000000000018a9d), UINT64_C(0x1fbafa1104b76589), UINT64_C(0x1f2afa1180b5256a),
        UINT64_C(0x2ce2fa1180ec056a), UINT64_C(0x156afa0df88e264b)
    },
    {   UINT64_C(0x0000000000018a9e), UINT64_C(0x156af790108f66a7), UINT64_C(0x156af90f388aa5fc),
        UINT64_C(0x156b010f388ea5fc), UINT64_C(0x156af90f388aa70e)
    },
    {   UINT64_C(0x000000000000896f), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x14a2f4cee08d85c5), UINT64_C(0x14a2f4cee08d85c5)
    },
    {   UINT64_C(0x0000000000018aa0), UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afd0d7c8ca659),
        UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afd0d7c8ca6f1)
    },
    {   UINT64_C(0x0000000000008970), UINT64_C(0x209af6cb2cb826fe), UINT64_C(0x209aff8b30bc86ff),
        UINT64_C(0x14a2fbced8870613), UINT64_C(0x14a2fbced4870612)
    },
    {   UINT64_C(0x0000000000018aa2), UINT64_C(0x157afc0c408c265c), UINT64_C(0x156afa0acc8e26d2),
        UINT64_C(0x156afa0acc8e26d2), UINT64_C(0x156afc09008c278a)
    },
    {   UINT64_C(0x0000000000018aa3), UINT64_C(0x156afc09008c278a), UINT64_C(0x156afa0ce08e26d2),
        UINT64_C(0x156afa0ce08e26d2), UINT64_C(0x156afd09fc8fa74b)
    },
    {   UINT64_C(0x0000000000018aa4), UINT64_C(0x156afd09fc8fa74b), UINT64_C(0x156afbca588d4734),
        UINT64_C(0x156afbca588d4734), UINT64_C(0x156afd09fc8ca74a)
    },
    {   UINT64_C(0x0000000000008972), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x18b2f94b1c9ba703), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000018aa6), UINT64_C(0x156afc0db88c2668), UINT64_C(0x156afd0c548ca6b6),
        UINT64_C(0x156afd0c548ca6b6), UINT64_C(0x156afd0c308ca6be)
    },
    {   UINT64_C(0x0000000000008973), UINT64_C(0x1c72f54ce0aca690), UINT64_C(0x1c72f4cce0ad0690),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000018aa8), UINT64_C(0x156afa0b748e26ee), UINT64_C(0x156af90f388aa5fc),
        UINT64_C(0x156b010f388ea5fc), UINT64_C(0x1d2af44e04b00649)
    },
    {   UINT64_C(0x0000000000008974), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000006275), UINT64_C(0x1f43004fd8ad8713), UINT64_C(0x1f42fc8eb4a88723),
        UINT64_C(0x1f43004f24b4c704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018aab), UINT64_C(0x156afd0e048ca64d), UINT64_C(0x156afd0e048ca64d),
        UINT64_C(0x156afa10108e261b), UINT64_C(0x156afe0e048c2649)
    },
    {   UINT64_C(0x0000000000006276), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f2cb8271a),
        UINT64_C(0x1f42f68f54b7a708), UINT64_C(0x1f42ff4f3cb6a70f)
    },
    {   UINT64_C(0x0000000000018aad), UINT64_C(0x156afd0c548ca6b5), UINT64_C(0x156afe4b908c46d3),
        UINT64_C(0x156afe4b908c46d3), UINT64_C(0x156afa0b548e26d3)
    },
    {   UINT64_C(0x0000000000018aae), UINT64_C(0x156afa0b548e26d3), UINT64_C(0x156afa0acc8e26d3),
        UINT64_C(0x156afa0acc8e26d3), UINT64_C(0x1d2af44f38b005fc)
    },
    {   UINT64_C(0x0000000000018aaf), UINT64_C(0x1d2af44f38b005fc), UINT64_C(0x156afd0f388ca5fc),
        UINT64_C(0x156afd0f388ca5fc), UINT64_C(0x156afd0e048ca647)
    },
    {   UINT64_C(0x0000000000018ab0), UINT64_C(0x156afd0e048ca647), UINT64_C(0x156afd0e048ca647),
        UINT64_C(0x156afd0e048ca647), UINT64_C(0x156afd0e048ca6ad)
    },
    {   UINT64_C(0x0000000000008978), UINT64_C(0x1c72f4d38cad0690), UINT64_C(0x1c72f4cce0ad04e4),
        UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x0000000000018ab2), UINT64_C(0x156afd0d108ca681), UINT64_C(0x156afd0b588ca6d2),
        UINT64_C(0x156afd0b588ca6d2), UINT64_C(0x156afd4aa48be6b5)
    },
    {   UINT64_C(0x00000000000058cc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43150e70aae702), UINT64_C(0x1f42f7cf40b0e703)
    },
    {   UINT64_C(0x0000000000018ab4), UINT64_C(0x156aff8f388de5fc), UINT64_C(0x156afd0e048cc649),
        UINT64_C(0x156afd0e048ca649), UINT64_C(0x156afd0c5c8ca6b3)
    },
    {   UINT64_C(0x0000000000018ab5), UINT64_C(0x156afd0c5c8ca6b3), UINT64_C(0x156afd0c5c8ca6b3),
        UINT64_C(0x156afd0c5c8ca6b3), UINT64_C(0x2712fd0f38d345fc)
    },
    {   UINT64_C(0x0000000000018ab6), UINT64_C(0x2712fd0f38d345fc), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0da08606a1), UINT64_C(0x1362fa0e1c86069f)
    },
    {   UINT64_C(0x0000000000018ab7), UINT64_C(0x1362f0cf78860615), UINT64_C(0x1362fa0f78860615),
        UINT64_C(0x0f8afe0fa074a67e), UINT64_C(0x156afa0fa08e255b)
    },
    {   UINT64_C(0x0000000000018ab8), UINT64_C(0x156afa0fa08e255b), UINT64_C(0x156afa0fa08e267e),
        UINT64_C(0x156afd0fa08ca67e), UINT64_C(0x1ae2f98fa0a2e67e)
    },
    {   UINT64_C(0x0000000000018ab9), UINT64_C(0x156afd0fa08ca67e), UINT64_C(0x1ae2f98fa0a2e67e),
        UINT64_C(0x156afc4fa08cc67e), UINT64_C(0x156afa0fa08e2623)
    },
    {   UINT64_C(0x000000000000627d), UINT64_C(0x1f42fb4e50b46742), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4eecb4e71a), UINT64_C(0x1f430a4eacaf871c)
    },
    {   UINT64_C(0x0000000000018abb), UINT64_C(0x12c2fa0fe88385d1), UINT64_C(0x12c2fa0fe88385d1),
        UINT64_C(0x0ca2f3ccac6e269a), UINT64_C(0x0fb3074d3870a67c)
    },
    {   UINT64_C(0x0000000000018abc), UINT64_C(0x0fb3074d3870a67c), UINT64_C(0x0fb3074cb470a69d),
        UINT64_C(0x105af08cac7ea69f), UINT64_C(0x156afa0cac8e269f)
    },
    {   UINT64_C(0x0000000000018abd), UINT64_C(0x156b0eccac83e69f), UINT64_C(0x156af10cac92c69f),
        UINT64_C(0x156b33ce08714648), UINT64_C(0x2492fc8d34c5867d)
    },
    {   UINT64_C(0x000000000000627f), UINT64_C(0x1f42fa4e70b4473a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74f90b5c70a), UINT64_C(0x1f42fb0eccb1272b)
    },
    {   UINT64_C(0x0000000000018abf), UINT64_C(0x1473058b948486e3), UINT64_C(0x162af98c289166ae),
        UINT64_C(0x0cd3070cac65469a), UINT64_C(0x0ef3058b946e86e3)
    },
    {   UINT64_C(0x0000000000006280), UINT64_C(0x1f43068f4cae8720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014f9cb6471e), UINT64_C(0x1f42fe0eecb1271f)
    },
    {   UINT64_C(0x00000000000058d3), UINT64_C(0x1f42f64f14ad4727), UINT64_C(0x1f42f20e5cb54728),
        UINT64_C(0x1f42f94ef0abc70d), UINT64_C(0x1f42f74f64af6714)
    },
    {   UINT64_C(0x0000000000006281), UINT64_C(0x1f42fc8efcb426f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43138e6ca7471e)
    },
    {   UINT64_C(0x00000000000058d4), UINT64_C(0x1f42fecf2ca86727), UINT64_C(0x1f42f88f50b3c6f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018ac4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x0000000000018ac5), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0)
    },
    {   UINT64_C(0x0000000000018ac6), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x2082f80cd0bb86f4)
    },
    {   UINT64_C(0x0000000000018ac7), UINT64_C(0x2082f80cd0bb86f4), UINT64_C(0x0782fa9250564534),
        UINT64_C(0x0782fa9250564534), UINT64_C(0x1c72f70dfcabc69a)
    },
    {   UINT64_C(0x0000000000018ac8), UINT64_C(0x1c72f70dfcabc69a), UINT64_C(0x1c73028cd8a60690),
        UINT64_C(0x1c73028cd8a60690), UINT64_C(0x1cf2f40da4af4661)
    },
    {   UINT64_C(0x0000000000018ac9), UINT64_C(0x25baff8b38cca6f9), UINT64_C(0x185af94e209a462d),
        UINT64_C(0x185af94e209a462d), UINT64_C(0x1102f80dfc7d8649)
    },
    {   UINT64_C(0x0000000000018aca), UINT64_C(0x1102f80dfc7d8649), UINT64_C(0x105af48fe07ca5d0),
        UINT64_C(0x0b2af48fe067e5d0), UINT64_C(0x0b2af48fe067e5d0)
    },
    {   UINT64_C(0x0000000000018acb), UINT64_C(0x1a52fa8fb0a185dc), UINT64_C(0x1a52fa8fb0a185dc),
        UINT64_C(0x12a2fa8dc482c657), UINT64_C(0x12a2fa8dc482c657)
    },
    {   UINT64_C(0x0000000000018acc), UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x1cf2fd0c00aac6cb),
        UINT64_C(0x2582fb0eaccde621), UINT64_C(0x3132fbd088fc65a9)
    },
    {   UINT64_C(0x0000000000018acd), UINT64_C(0x1cf2fc1084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x2c22fbd084e825a9)
    },
    {   UINT64_C(0x0000000000018ace), UINT64_C(0x1cf2fbd084aba5a9), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018acf), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fa9084ac05aa),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000018ad0), UINT64_C(0x1cf2fbd084ab661d), UINT64_C(0x1cf2fbd084ab65e4),
        UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2f94f38ab65fc)
    },
    {   UINT64_C(0x0000000000018ad1), UINT64_C(0x1cf2f94f38ab65fc), UINT64_C(0x1d02fbcc94aba6a5),
        UINT64_C(0x1d02fbcc94aba6a5), UINT64_C(0x1cf2fbd084ab65e4)
    },
    {   UINT64_C(0x0000000000018ad2), UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2fbd084ab65e4),
        UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2fbcf98ab65a9)
    },
    {   UINT64_C(0x0000000000018ad3), UINT64_C(0x1cf2fbcf98ab65a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018ad4), UINT64_C(0x18d2fd0f289a4653), UINT64_C(0x18d2fd0f289a4653),
        UINT64_C(0x15e2f6cf6c91a63d), UINT64_C(0x1b0af94da8a786b0)
    },
    {   UINT64_C(0x0000000000018ad5), UINT64_C(0x1b0af94da4a786b0), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000628b), UINT64_C(0x1f42ffcf04b2a6ec), UINT64_C(0x1f42fa0fd4ba2707),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058de), UINT64_C(0x1f42fe0ee0b346f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f54eb8b7a71d), UINT64_C(0x1f42f80f40b8a721)
    },
    {   UINT64_C(0x000000000000628c), UINT64_C(0x1f42facf90b146fa), UINT64_C(0x1f43024eacb7473c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f71048b6e72e)
    },
    {   UINT64_C(0x00000000000058df), UINT64_C(0x1f42fe4f1caee6ef), UINT64_C(0x1f42fece6cae8702),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000628d), UINT64_C(0x1f42fb4e6cb4072e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4e7cb6c731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000628e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8eccb08702),
        UINT64_C(0x1f42fb4e84bb2737), UINT64_C(0x1f42ff4ff8b3e6fb)
    },
    {   UINT64_C(0x000000000000898e), UINT64_C(0x0000000000000000), UINT64_C(0x23f3188e58b906c0),
        UINT64_C(0x23f3188e58b906c0), UINT64_C(0x23f3188e58b906c0)
    },
    {   UINT64_C(0x000000000000628f), UINT64_C(0x1f42fa9054b7470b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303ce50ac8736), UINT64_C(0x1f42f2cff8b7471a)
    },
    {   UINT64_C(0x000000000000898f), UINT64_C(0x1a2af98db8a16657), UINT64_C(0x140af80e5889a6c0),
        UINT64_C(0x140af80e5889a6c0), UINT64_C(0x12dafe4c8881c6a6)
    },
    {   UINT64_C(0x0000000000006290), UINT64_C(0x1f42fd8f9caec702), UINT64_C(0x1f42f84fd8b50702),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008990), UINT64_C(0x12dafe4d8481c666), UINT64_C(0x12dafe4d8481c666),
        UINT64_C(0x14a2fb8d948c0662), UINT64_C(0x1e52fe8ce0af86c4)
    },
    {   UINT64_C(0x0000000000006291), UINT64_C(0x1f42fa8f5cb5c6fb), UINT64_C(0x1f42fc8ebcb786f6),
        UINT64_C(0x1f42fb8ff0b5a712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008991), UINT64_C(0x1e52fdcd70aac701), UINT64_C(0x0000000000000000),
        UINT64_C(0x1a42fd0de8a006e0), UINT64_C(0x13c3010de88686c2)
    },
    {   UINT64_C(0x0000000000006292), UINT64_C(0x1f43028e9cb2471d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8ea8b36711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008992), UINT64_C(0x1e22f44de8b3e6e0), UINT64_C(0x186af8cd749ac66c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006293), UINT64_C(0x1f42fa8fa8b3e6fe), UINT64_C(0x1f42f98fc8b5670a),
        UINT64_C(0x1f42f99008b20706), UINT64_C(0x1f42f98f40b0c706)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006294), UINT64_C(0x1f42ff0ee8b18726), UINT64_C(0x1f42fd8fa4b146fd),
        UINT64_C(0x1f43014f9cae86f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058e7), UINT64_C(0x1f42fdcf40b5c6dd), UINT64_C(0x1f42fa8f70ade708),
        UINT64_C(0x1f42f7cf34aee6fc), UINT64_C(0x1f42fb4fb8b206f4)
    },
    {   UINT64_C(0x0000000000006295), UINT64_C(0x1f42fb8ff8ac8726), UINT64_C(0x1f42fe4f18b6e703),
        UINT64_C(0x1f42fa8f44b68719), UINT64_C(0x1f42f88f10aea70d)
    },
    {   UINT64_C(0x00000000000058e8), UINT64_C(0x1f42f78ee0afc724), UINT64_C(0x1f42f70ef8b7a718),
        UINT64_C(0x1f42f90eecb6c720), UINT64_C(0x1f43078e18b1273c)
    },
    {   UINT64_C(0x0000000000006296), UINT64_C(0x1f42fa8fa8afe70a), UINT64_C(0x1f42f78fb8b58709),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4fd4b7c70b)
    },
    {   UINT64_C(0x00000000000058e9), UINT64_C(0x1f42ffcf78ac26f0), UINT64_C(0x1f42f98f28b8271c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80eecb3c718)
    },
    {   UINT64_C(0x0000000000006297), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f58ae4735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058ea), UINT64_C(0x1f42fc8e9cb3c732), UINT64_C(0x1f42f78f2cb9272d),
        UINT64_C(0x1f42f64ee8b4e718), UINT64_C(0x1f42f88f38b5a6e4)
    },
    {   UINT64_C(0x0000000000006298), UINT64_C(0x1f42f68f0cb5c728), UINT64_C(0x1f42f9cff8afe719),
        UINT64_C(0x1f42f58f28b08730), UINT64_C(0x1f42fc8efcaf4704)
    },
    {   UINT64_C(0x0000000000008998), UINT64_C(0x156af68e148fe649), UINT64_C(0x156af68dfc8fe643),
        UINT64_C(0x156af84dfc8ee649), UINT64_C(0x156af68e148fe643)
    },
    {   UINT64_C(0x0000000000006299), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4f40b8070d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008999), UINT64_C(0x156af68e148fe649), UINT64_C(0x156af68dfc8fe643),
        UINT64_C(0x156af84dfc8ee649), UINT64_C(0x156af68e148fe643)
    },
    {   UINT64_C(0x000000000000629a), UINT64_C(0x1f42fd8f6cb5c6ff), UINT64_C(0x1f42fe8e64b5e739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ff8b5e715)
    },
    {   UINT64_C(0x000000000000899a), UINT64_C(0x0000000000000000), UINT64_C(0x0c32f94dac69a6b2),
        UINT64_C(0x1182fa8dac7e46b2), UINT64_C(0x1a42f80dac98a732)
    },
    {   UINT64_C(0x000000000000629b), UINT64_C(0x1f42fa8f94ade716), UINT64_C(0x1f42f98f84b50734),
        UINT64_C(0x1f42f64fb8b6070d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058ee), UINT64_C(0x1f42fa0e8caf8706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74ec4b6671b), UINT64_C(0x1f42f70f24b70704)
    },
    {   UINT64_C(0x000000000000629c), UINT64_C(0x1f42fa4f98ad8719), UINT64_C(0x1f42fe8fc8b30711),
        UINT64_C(0x1f42fc8f70b1272a), UINT64_C(0x1f42f88e7cb56739)
    },
    {   UINT64_C(0x00000000000058ef), UINT64_C(0x1f42fa0f20b06709), UINT64_C(0x1f42f48ef4b146f8),
        UINT64_C(0x1f42f90eb4b7e702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000629d), UINT64_C(0x1f42fd8f10ab2719), UINT64_C(0x1f42f98fbcb6c719),
        UINT64_C(0x1f42f98e9cad2719), UINT64_C(0x1f42f98ed4ae2719)
    },
    {   UINT64_C(0x00000000000058f0), UINT64_C(0x1f42f68edcb6c73d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f00aa270b), UINT64_C(0x1f42fb0e50b2e70b)
    },
    {   UINT64_C(0x000000000000629e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e90ae0741),
        UINT64_C(0x1f42f98f0cac2719), UINT64_C(0x1f42fa8f4cb6e70d)
    },
    {   UINT64_C(0x00000000000058f1), UINT64_C(0x1f42f98efcb3a702), UINT64_C(0x1f43068e80ac8700),
        UINT64_C(0x1f42f78f80b80720), UINT64_C(0x1f42fc0f20ad070c)
    },
    {   UINT64_C(0x000000000000629f), UINT64_C(0x1f42fb8fe8b4c70a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8fe8b266f7)
    },
    {   UINT64_C(0x000000000000899f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x2512ff8b64ca06ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062a0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f24b2870a),
        UINT64_C(0x1f42f79024b7c719), UINT64_C(0x1f42f98eecb0a719)
    },
    {   UINT64_C(0x00000000000089a0), UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685),
        UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685)
    },
    {   UINT64_C(0x00000000000062a1), UINT64_C(0x1f42fd8f18b6c71e), UINT64_C(0x1f42f54e7cb6c742),
        UINT64_C(0x1f42f98f44b02721), UINT64_C(0x1f42f84eb0b7272c)
    },
    {   UINT64_C(0x00000000000089a1), UINT64_C(0x2043188f44aa4685), UINT64_C(0x2043188f44aa4685),
        UINT64_C(0x2043140ef8aca697), UINT64_C(0x2043140ef8aca697)
    },
    {   UINT64_C(0x00000000000062a2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f0cb8271d),
        UINT64_C(0x1f42ff0edcb80729), UINT64_C(0x1f42f64f64b5470b)
    },
    {   UINT64_C(0x00000000000089a2), UINT64_C(0x2043140ef8aca697), UINT64_C(0x2043140ef8aca697),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062a3), UINT64_C(0x1f42fb4f98b64733), UINT64_C(0x1f42f9900cb74711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ed8b72713)
    },
    {   UINT64_C(0x00000000000058f6), UINT64_C(0x1f42fa4f14af46fc), UINT64_C(0x1f42f84f40ac06fb),
        UINT64_C(0x1f42fa0eccb34720), UINT64_C(0x1f42f90f34b6471e)
    },
    {   UINT64_C(0x00000000000062a4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f48fd8b7c707),
        UINT64_C(0x1f42f94f7cb3c710), UINT64_C(0x1f42fe0e44b4a738)
    },
    {   UINT64_C(0x00000000000058f7), UINT64_C(0x1f42f90e4cb52720), UINT64_C(0x1f42f84ee4b586f6),
        UINT64_C(0x1f42fe8f3cb14706), UINT64_C(0x1f42fc0ee8b366ec)
    },
    {   UINT64_C(0x00000000000062a5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccfd0b66709),
        UINT64_C(0x1f42f64e5cb4071f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058f8), UINT64_C(0x1f42fa0ef4b16703), UINT64_C(0x1f42f5cf88b5a720),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f30aa6710)
    },
    {   UINT64_C(0x00000000000062a6), UINT64_C(0x1f42fb8e9cb2e717), UINT64_C(0x1f42f64fbcb266fe),
        UINT64_C(0x1f42fc8f28a8a712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058f9), UINT64_C(0x1f42f84e8cb366eb), UINT64_C(0x1f42fb0e2cb0a70f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43170eacab071d)
    },
    {   UINT64_C(0x00000000000062a7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facfb8b8271a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058fa), UINT64_C(0x1f42fe0e90b1670a), UINT64_C(0x1f42f30f0cb3a723),
        UINT64_C(0x1f42f70f50b7c6fc), UINT64_C(0x1f43104eb0ae6701)
    },
    {   UINT64_C(0x00000000000062a8), UINT64_C(0x1f42fbcf78b4a725), UINT64_C(0x1f42fb8f40b5c71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f88b54716)
    },
    {   UINT64_C(0x00000000000089a8), UINT64_C(0x2f32f44d34f827e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062a9), UINT64_C(0x1f42f54fa8b746f8), UINT64_C(0x1f42fb8f88b4a719),
        UINT64_C(0x1f42f78f40adc70e), UINT64_C(0x1f42f8cf2cb0672c)
    },
    {   UINT64_C(0x00000000000089a9), UINT64_C(0x0000000000000000), UINT64_C(0x14a2fb8b3c8c06f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062aa), UINT64_C(0x1f42f58ee8b92742), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68f94b8270b), UINT64_C(0x1f42fb4f50b7870b)
    },
    {   UINT64_C(0x00000000000089aa), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x17d2fa8e6497862f), UINT64_C(0x17d2fa8e6497862f)
    },
    {   UINT64_C(0x00000000000062ab), UINT64_C(0x1f42f58f94b7a70b), UINT64_C(0x1f42f34fe0b8e737),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089ab), UINT64_C(0x11aaf98fd87f65cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062ac), UINT64_C(0x1f42f84e70b5c725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f54e98b9672c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089ac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x12dafe4d8481c666), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062ad), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e5cb16712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005900), UINT64_C(0x1f42fa0f18b0e720), UINT64_C(0x1f42f90f64aca703),
        UINT64_C(0x1f42facef8b0e710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062ae), UINT64_C(0x1f42f7cfb8b246f6), UINT64_C(0x1f42f68ef8b2e715),
        UINT64_C(0x1f42fa8e9cb72726), UINT64_C(0x1f42fc8ef4b7a6ff)
    },
    {   UINT64_C(0x0000000000005901), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f60b2c70c),
        UINT64_C(0x1f42f6ce80b5a74a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062af), UINT64_C(0x1f42fb8ff8b84725), UINT64_C(0x1f42f88f80b76703),
        UINT64_C(0x1f43100e7caa272c), UINT64_C(0x1f43018f0caf472c)
    },
    {   UINT64_C(0x0000000000005902), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cef4af871d),
        UINT64_C(0x1f42f94e6caee706), UINT64_C(0x1f42fb4f44b64703)
    },
    {   UINT64_C(0x00000000000062b0), UINT64_C(0x1f42f7cee8b6e71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cfc0b346fe), UINT64_C(0x1f42f5cf08b7271a)
    },
    {   UINT64_C(0x0000000000005903), UINT64_C(0x1f42f64e98b74700), UINT64_C(0x1f42f80f18aa0727),
        UINT64_C(0x1f42f68ef0b2c6ef), UINT64_C(0x1f42f80e50b5c734)
    },
    {   UINT64_C(0x00000000000062b1), UINT64_C(0x1f42f4ce94b56740), UINT64_C(0x1f42f48e70b74745),
        UINT64_C(0x1f42fd8f30b4e72d), UINT64_C(0x1f43014facb7a715)
    },
    {   UINT64_C(0x0000000000019100), UINT64_C(0x1692f04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1692fa4f5494669b), UINT64_C(0x1692f18fa094c68c)
    },
    {   UINT64_C(0x00000000000062b2), UINT64_C(0x1f42fd8f8cb846d8), UINT64_C(0x1f42f98f88b5a719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4e70b5072c)
    },
    {   UINT64_C(0x0000000000005905), UINT64_C(0x1f42f88f24b7e704), UINT64_C(0x1f42f50ed8b62712),
        UINT64_C(0x1f42f8cef4b0072b), UINT64_C(0x1f42f88ed4b2c739)
    },
    {   UINT64_C(0x00000000000062b3), UINT64_C(0x1f42f68e8cb72739), UINT64_C(0x1f42f88ec4b70730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4cf48b6a715)
    },
    {   UINT64_C(0x0000000000019102), UINT64_C(0x0c2af30fa06cc68c), UINT64_C(0x0fa2eece207a468c),
        UINT64_C(0x1852f58fa09ea68c), UINT64_C(0x14daf2cfa08e468c)
    },
    {   UINT64_C(0x00000000000062b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4f3cb84726),
        UINT64_C(0x1f43018f20b4071e), UINT64_C(0x1f42fc4ea0b406fe)
    },
    {   UINT64_C(0x0000000000019103), UINT64_C(0x1d82f18fa0b1a68c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x131af20fa089868c)
    },
    {   UINT64_C(0x00000000000062b5), UINT64_C(0x1f42f79038b886f6), UINT64_C(0x1f42f7cf6cb6872d),
        UINT64_C(0x1f42fb8f38b7a711), UINT64_C(0x1f42fa8f5cb94718)
    },
    {   UINT64_C(0x0000000000019104), UINT64_C(0x1852f6cce09b669b), UINT64_C(0x1692f48fa098068c),
        UINT64_C(0x1162f6cf547ea69c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x00000000000062b6), UINT64_C(0x1f43020ee8b60719), UINT64_C(0x1f42ff8ee0b64701),
        UINT64_C(0x1f42f48fb8b8470a), UINT64_C(0x1f42f78fb8b66709)
    },
    {   UINT64_C(0x0000000000019105), UINT64_C(0x1692f20f5496268c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1f42f2cf64b9268c), UINT64_C(0x1692f20fa095e68c)
    },
    {   UINT64_C(0x00000000000062b7), UINT64_C(0x1f42f78e9cb82731), UINT64_C(0x1f42fd8f5cb6270a),
        UINT64_C(0x1f42f78f8cae0717), UINT64_C(0x1f42f68fc8b6e6f6)
    },
    {   UINT64_C(0x0000000000019106), UINT64_C(0x1692f1cfa095e68c), UINT64_C(0x14daf50fa08dc68c),
        UINT64_C(0x0fa2f44f687b85c1), UINT64_C(0x1162f24f687fa68c)
    },
    {   UINT64_C(0x00000000000062b8), UINT64_C(0x1f42fd9028afc706), UINT64_C(0x1f42fa8fd0b68708),
        UINT64_C(0x1f42fcced8b466e5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089b8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15e3058e608a46ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062b9), UINT64_C(0x1f42fd8f38b0a707), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68e6cb6e731), UINT64_C(0x1f42f88ef0b3c724)
    },
    {   UINT64_C(0x000000000000590c), UINT64_C(0x1f42f78e60b3e700), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84efcae8712)
    },
    {   UINT64_C(0x00000000000062ba), UINT64_C(0x1f42fa8f30b1a6ec), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e8cb82739)
    },
    {   UINT64_C(0x0000000000019109), UINT64_C(0x1162f38fa082668c), UINT64_C(0x08b2f1cfa05e868c),
        UINT64_C(0x1a0af1cfa0a445c1), UINT64_C(0x1162f34fa081e5c1)
    },
    {   UINT64_C(0x00000000000062bb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cea0b6c72b),
        UINT64_C(0x1f42fa8fa8b7c72f), UINT64_C(0x1f42f58edcb246f2)
    },
    {   UINT64_C(0x000000000000590e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f64b20700),
        UINT64_C(0x1f42fa0ed0abc718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062bc), UINT64_C(0x1f42f78fc0b6a6ee), UINT64_C(0x1f42fd8e2cb5c742),
        UINT64_C(0x1f42fd8f78b346e0), UINT64_C(0x1f42fc8f2cb02721)
    },
    {   UINT64_C(0x000000000000590f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78e98b88722),
        UINT64_C(0x1f42f88e6cb1e717), UINT64_C(0x1f42f48ea0b806dc)
    },
    {   UINT64_C(0x00000000000062bd), UINT64_C(0x1f42f78ef0b6c70f), UINT64_C(0x1f42fc8f48b446f5),
        UINT64_C(0x1f42fc4f7cb4c72d), UINT64_C(0x1f42f78f28b7070e)
    },
    {   UINT64_C(0x000000000001910c), UINT64_C(0x1692f3cf6896e5b5), UINT64_C(0x0fa2f10fa07b05b5),
        UINT64_C(0x0fa2f20c6c7ac5b5), UINT64_C(0x0de2f34fa07305b5)
    },
    {   UINT64_C(0x00000000000062be), UINT64_C(0x1f43004f50b86725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9ce34abe738), UINT64_C(0x1f42f98f68b42705)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062bf), UINT64_C(0x1f42fa4f6cb7e712), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014fe8b0e72c), UINT64_C(0x1f42fe4e5cb766eb)
    },
    {   UINT64_C(0x000000000001910e), UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1352f78fa098e675),
        UINT64_C(0x16f2fa8f709a4684), UINT64_C(0x1b4af78fa0b1e675)
    },
    {   UINT64_C(0x00000000000062c0), UINT64_C(0x1f42f78edcb8a738), UINT64_C(0x1f42f7ced0b00714),
        UINT64_C(0x1f42ff4eecafa70e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089c0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1e0af6d0e8afa64c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062c1), UINT64_C(0x1f42f54ef0b7e73d), UINT64_C(0x1f43028eb4ae0700),
        UINT64_C(0x1f42f98f58ab473a), UINT64_C(0x1f4302ce84b7e743)
    },
    {   UINT64_C(0x0000000000005914), UINT64_C(0x1f42f94f0cb726f7), UINT64_C(0x1f42fa4f24ad2718),
        UINT64_C(0x1f42f5cf08b3e720), UINT64_C(0x1f42f3cf14b80705)
    },
    {   UINT64_C(0x000000001f88fec0), UINT64_C(0x1312f1cfa091c69b), UINT64_C(0x1162f18fa081869b),
        UINT64_C(0x1162f1cfa081669b), UINT64_C(0x1a0af1cfa0a4269b)
    },
    {   UINT64_C(0x00000000000089c2), UINT64_C(0x0000000000000000), UINT64_C(0x1592f691c88e4613),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062c3), UINT64_C(0x1f42f74eb0b6e70e), UINT64_C(0x1f430a0e90af6726),
        UINT64_C(0x1f42f64ff8b5670c), UINT64_C(0x1f42f94f90b58721)
    },
    {   UINT64_C(0x0000000000019112), UINT64_C(0x160afe4f6c9f6675), UINT64_C(0x11fafd0f609d8675),
        UINT64_C(0x1ca2fd0f60c82675), UINT64_C(0x18b2f44fa0a3c675)
    },
    {   UINT64_C(0x00000000000062c4), UINT64_C(0x1f42fad030b0c702), UINT64_C(0x1f43008f54b3c709),
        UINT64_C(0x1f42fd8f1cb866fd), UINT64_C(0x1f43028eecb246e6)
    },
    {   UINT64_C(0x00000000000089c4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x179af68f709686ab)
    },
    {   UINT64_C(0x00000000000062c5), UINT64_C(0x1f42f88e78b7a72f), UINT64_C(0x1f42fd8f10b72700),
        UINT64_C(0x1f42f7cfc0b6c70e), UINT64_C(0x1f42f54e9cb4c740)
    },
    {   UINT64_C(0x0000000000005918), UINT64_C(0x1f42f9cee8b30713), UINT64_C(0x1f42f78f98b786f0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x1ff3f87dc0008018), UINT64_C(0x0a6af2978c5cc68e), UINT64_C(0x0de2f74f7875c5b4),
        UINT64_C(0x0fa2eecf78790693), UINT64_C(0x0de2f44f787205b4)
    },
    {   UINT64_C(0x0000000000005919), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64ec8b06724),
        UINT64_C(0x1f42f58f3cb826e8), UINT64_C(0x1f42facf08b14732)
    },
    {   UINT64_C(0x00000000000062c7), UINT64_C(0x1f42fbceacb02706), UINT64_C(0x1f42f88f44b22724),
        UINT64_C(0x1f43008f68b04700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000591a), UINT64_C(0x1f42f8ceacb14708), UINT64_C(0x1f42f88e34ae673d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f2cb646f6)
    },
    {   UINT64_C(0x00000000000062c8), UINT64_C(0x1f42f58fb4b2073c), UINT64_C(0x1f42fb4fb8b5070a),
        UINT64_C(0x1f42facef8b5c6fd), UINT64_C(0x1f42f6ce50b726ef)
    },
    {   UINT64_C(0x000000000000591b), UINT64_C(0x1f42f64ef4b4272d), UINT64_C(0x1f42f70ef4b02703),
        UINT64_C(0x1f4310ce60ace723), UINT64_C(0x1f43094e34b1a745)
    },
    {   UINT64_C(0x00000000000062c9), UINT64_C(0x1f42f78e94b0e70f), UINT64_C(0x1f42fccf34b4e71c),
        UINT64_C(0x1f42f88e70b3672d), UINT64_C(0x1f42f7cf40b22722)
    },
    {   UINT64_C(0x000000000000591c), UINT64_C(0x1f4312ce5cb1271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f50ed8b52707), UINT64_C(0x1f42f90f04acc70c)
    },
    {   UINT64_C(0x00000000000062ca), UINT64_C(0x1f42f68f50b6471d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8e28b08716), UINT64_C(0x1f42f58f00b1e725)
    },
    {   UINT64_C(0x00000000000089ca), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18aaf6cdf89a264d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062cb), UINT64_C(0x1f42f58e50b8a725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42face70b4e734), UINT64_C(0x1f42fd8ee4b2a716)
    },
    {   UINT64_C(0x00000000000029c6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aff931077669e)
    },
    {   UINT64_C(0x00000000000029c7), UINT64_C(0x106aff931077669e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000591f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0ed8ad86e5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e74b3c703)
    },
    {   UINT64_C(0x00000000000062cd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facec0b386f1),
        UINT64_C(0x1f42f78f54b4e711), UINT64_C(0x1f43014e40afc734)
    },
    {   UINT64_C(0x0000000000005920), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90efcb48707), UINT64_C(0x1f43028e60b14722)
    },
    {   UINT64_C(0x00000000000062ce), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f44b52735),
        UINT64_C(0x1f43070e60afe732), UINT64_C(0x1f42fa8f10b526f9)
    },
    {   UINT64_C(0x0000000000005921), UINT64_C(0x1f42facee4b2a6f5), UINT64_C(0x1f42f5cf34b98718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f44f98b726eb)
    },
    {   UINT64_C(0x00000000000062cf), UINT64_C(0x1f42fcce94b28722), UINT64_C(0x1f43098e70ace722),
        UINT64_C(0x1f42f58ee4b8670c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089cf), UINT64_C(0x0000000000000000), UINT64_C(0x1552f6cdf08cc64c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062d0), UINT64_C(0x1f42f84ef8b726de), UINT64_C(0x1f42f8ce6cb14738),
        UINT64_C(0x1f42fa4e30aee6f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001911f), UINT64_C(0x16eb000f78a3c685), UINT64_C(0x128afe8f64a22685),
        UINT64_C(0x1ceafe8f5ccba685), UINT64_C(0x1942f50fa0a64685)
    },
    {   UINT64_C(0x00000000000062d1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014fccb4e716),
        UINT64_C(0x1f42f9cee0b34738), UINT64_C(0x1f42f9ce94b7471b)
    },
    {   UINT64_C(0x0000000000005924), UINT64_C(0x1f42f8ced8b1c716), UINT64_C(0x1f42f8ceccb0071a),
        UINT64_C(0x1f42f64f6cb7470b), UINT64_C(0x1f42f7cef4b68731)
    },
    {   UINT64_C(0x00000000000062d2), UINT64_C(0x1f42f68e80b78705), UINT64_C(0x1f42f5cf60b7c6fe),
        UINT64_C(0x1f42f54edcb50721), UINT64_C(0x1f42fe4edcb0c6e6)
    },
    {   UINT64_C(0x0000000000019121), UINT64_C(0x0eeaf90f787805b6), UINT64_C(0x129af90f68882695),
        UINT64_C(0x0f92f90f7877e5b6), UINT64_C(0x11e2f90c688d8695)
    },
    {   UINT64_C(0x00000000000062d3), UINT64_C(0x1f42f70eb4b7071e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8eacb0871c), UINT64_C(0x1f42f94e98b4473c)
    },
    {   UINT64_C(0x0000000000005926), UINT64_C(0x1f42f84efcb8e6f9), UINT64_C(0x1f42f94e50b06744),
        UINT64_C(0x1f42f58f90b346fc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062d4), UINT64_C(0x1f42fd8e94b14733), UINT64_C(0x1f42f98ef0b6870e),
        UINT64_C(0x1f42f78ed4b62704), UINT64_C(0x1f42f54f58b58724)
    },
    {   UINT64_C(0x0000000000019123), UINT64_C(0x1222fbcf7484e696), UINT64_C(0x0ad2fa8f7c654695),
        UINT64_C(0x17c2f64f7c9965bb), UINT64_C(0x117af64f788045bb)
    },
    {   UINT64_C(0x00000000000062d5), UINT64_C(0x1f42f98fb0b06709), UINT64_C(0x1f42f54f98b726f6),
        UINT64_C(0x1f42f6cef4b5c711), UINT64_C(0x1f42fccf2cb8a6e8)
    },
    {   UINT64_C(0x0000000000019124), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x11dae5cc6c83c5bd),
        UINT64_C(0x1072f90c6c8105b6), UINT64_C(0x0dcaf64fa07385bb)
    },
    {   UINT64_C(0x00000000000062d6), UINT64_C(0x1f42f78e58b8a719), UINT64_C(0x1f42f9cfa4b0e707),
        UINT64_C(0x1f42f88e5cb36733), UINT64_C(0x1f42f58f54b7a741)
    },
    {   UINT64_C(0x0000000000019125), UINT64_C(0x0ed2fbcf7472e5b6), UINT64_C(0x0afaf90f78664638),
        UINT64_C(0x10baf64f787d45b7), UINT64_C(0x1152fb0f6881e5c1)
    },
    {   UINT64_C(0x00000000000062d7), UINT64_C(0x1f42fc8f50b3c72d), UINT64_C(0x1f42f7cf3cb82702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98ef8b6a704)
    },
    {   UINT64_C(0x000000000000592a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f28e98b5c70c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f94b5c725)
    },
    {   UINT64_C(0x00000000000062d8), UINT64_C(0x1f42fd8ec8b42721), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f78b86729)
    },
    {   UINT64_C(0x0000000000019127), UINT64_C(0x1abaf5cf64a8468a), UINT64_C(0x0000000000000000),
        UINT64_C(0x18eaf74f64a02697), UINT64_C(0x1baaf70f64a4c697)
    },
    {   UINT64_C(0x00000000000062d9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43118e54ae272d),
        UINT64_C(0x1f43058e70b14732), UINT64_C(0x1f42fb8eecb5c71d)
    },
    {   UINT64_C(0x0000000000019128), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x182b02cf649b0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062da), UINT64_C(0x1f42fbce58b76749), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e3cb0872d), UINT64_C(0x1f42f5cee0b2e72c)
    },
    {   UINT64_C(0x0000000000019129), UINT64_C(0x0000000000000000), UINT64_C(0x1912f04cdca1868a),
        UINT64_C(0x1f8af80f64bc0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001912a), UINT64_C(0x0000000000000000), UINT64_C(0x1e52f78f64b60697),
        UINT64_C(0x15a3028f64916690), UINT64_C(0x1c72f78f64ad4697)
    },
    {   UINT64_C(0x00000000000062dc), UINT64_C(0x1f42f98e48b48733), UINT64_C(0x1f4304cdc8b0074f),
        UINT64_C(0x1f42fb4e64b4e743), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001912b), UINT64_C(0x15a3028f08916690), UINT64_C(0x0000000000000000),
        UINT64_C(0x173afecf64962690), UINT64_C(0x1c82f8cf64afe697)
    },
    {   UINT64_C(0x00000000000062dd), UINT64_C(0x1f42f98ef4b6c702), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88ed0b7a73e)
    },
    {   UINT64_C(0x000000000001912c), UINT64_C(0x1a52f70f64a32697), UINT64_C(0x1d22f6cf64b1c697),
        UINT64_C(0x21b2f6cf64c44697), UINT64_C(0x1bdaf70f64aba697)
    },
    {   UINT64_C(0x00000000000062de), UINT64_C(0x1f42fbce58b44745), UINT64_C(0x1f42f9ceb4b24727),
        UINT64_C(0x1f42fb9094b82715), UINT64_C(0x1f430f4e40ab0731)
    },
    {   UINT64_C(0x000000000001912d), UINT64_C(0x1cf304cc18b10697), UINT64_C(0x1d22fccf64b08697),
        UINT64_C(0x199af58f749d45a1), UINT64_C(0x1222f9cf70846697)
    },
    {   UINT64_C(0x00000000000062df), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e60b74735),
        UINT64_C(0x1f42f98eaca94739), UINT64_C(0x1f43024f68b48716)
    },
    {   UINT64_C(0x0000000000005932), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e98b0e6fc),
        UINT64_C(0x1f43030e60b48734), UINT64_C(0x1f42f94ec4b006f7)
    },
    {   UINT64_C(0x00000000000062e0), UINT64_C(0x1f42fc8e90ae0732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42face84aea737), UINT64_C(0x1f42ff4eb0b2e733)
    },
    {   UINT64_C(0x000000000001912f), UINT64_C(0x0000000000000000), UINT64_C(0x178af78f7494c697),
        UINT64_C(0x0f8302cf747b0675), UINT64_C(0x16d2f04c34984675)
    },
    {   UINT64_C(0x00000000000062e1), UINT64_C(0x1f42fa8fc0b7870e), UINT64_C(0x1f42f78f30b4a729),
        UINT64_C(0x1f43058e90ae872f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019130), UINT64_C(0x16b2f80f7491c697), UINT64_C(0x121afa0f7485e697),
        UINT64_C(0x2072fa4f74b8c5a1), UINT64_C(0x17cafa4f749625a1)
    },
    {   UINT64_C(0x00000000000062e2), UINT64_C(0x1f42facf6cb54712), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4e8cb0671c), UINT64_C(0x1f430c0e24ace748)
    },
    {   UINT64_C(0x00000000000089e2), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000062e3), UINT64_C(0x1f42fd8fc4b16713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f54b8670e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019132), UINT64_C(0x101afd8f747205fb), UINT64_C(0x113af58f7481067a),
        UINT64_C(0x178af58f74952590), UINT64_C(0x1352f58f748745a1)
    },
    {   UINT64_C(0x00000000000062e4), UINT64_C(0x1f43070e24af0737), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f58e58b6e73a), UINT64_C(0x1f42fa8e64b48743)
    },
    {   UINT64_C(0x00000000000089e4), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000062e5), UINT64_C(0x1f42f8cf68b4a715), UINT64_C(0x1f42f78ed0b06717),
        UINT64_C(0x1f42f7ce98b68737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019134), UINT64_C(0x1ec2fa8f4cb5e6a3), UINT64_C(0x2122fbcf4cb9a6a3),
        UINT64_C(0x1c82fd0f4cac26a3), UINT64_C(0x1efafd0f4cafc6a3)
    },
    {   UINT64_C(0x00000000000062e6), UINT64_C(0x1f42f7ce6cb5472c), UINT64_C(0x1f42f6ce3cb0c72c),
        UINT64_C(0x1f42f98e94b4c741), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089e6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1cb2f6d0b8aa2599)
    },
    {   UINT64_C(0x00000000000062e7), UINT64_C(0x1f42f7cebcb34745), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7cf6cb8270e), UINT64_C(0x1f42f48f14b52739)
    },
    {   UINT64_C(0x000000000000593a), UINT64_C(0x1f42f58fa8b3e6df), UINT64_C(0x1f42fa0f38b0a700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60f88b446f3)
    },
    {   UINT64_C(0x00000000000062e8), UINT64_C(0x1f42f910487786e7), UINT64_C(0x1f42fc4f34b3e70b),
        UINT64_C(0x1f43040f80b24709), UINT64_C(0x1f42f90eecb1271f)
    },
    {   UINT64_C(0x0000000000019137), UINT64_C(0x28a2fd0f4cda26a3), UINT64_C(0x223afd0f4cc546a3),
        UINT64_C(0x194b000f4c9c26a3), UINT64_C(0x200afbcf4cbac6a3)
    },
    {   UINT64_C(0x00000000000062e9), UINT64_C(0x1f42f4cfb0b646eb), UINT64_C(0x1f42fb9044b4670e),
        UINT64_C(0x1f42fa4f18b266eb), UINT64_C(0x1f42fa106cb4a6eb)
    },
    {   UINT64_C(0x0000000000019138), UINT64_C(0x194b000ed09c26a3), UINT64_C(0x20c2fbcf4cb826a3),
        UINT64_C(0x1983024f4ca026a3), UINT64_C(0x2032fe4f4cbea6a3)
    },
    {   UINT64_C(0x00000000000062ea), UINT64_C(0x1f42fc4ee4b20705), UINT64_C(0x1f42f85010b066eb),
        UINT64_C(0x1f42fb5090aee6ea), UINT64_C(0x1f42fa4fd4b2a723)
    },
    {   UINT64_C(0x0000000000019139), UINT64_C(0x1e22fd0f4caf86a3), UINT64_C(0x2082fd0f4cbe86a3),
        UINT64_C(0x2942fd0f4ce1c6a3), UINT64_C(0x204afe0f4cbcc6a3)
    },
    {   UINT64_C(0x00000000000062eb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ef8ace705),
        UINT64_C(0x1f42fd500cb526e0), UINT64_C(0x1f42f70f68b3a70a)
    },
    {   UINT64_C(0x000000000001913a), UINT64_C(0x1eeafd0c20b6e6a3), UINT64_C(0x1f22fa8f4cb886a3),
        UINT64_C(0x1d72f6cf68aaa5b6), UINT64_C(0x1432fd0f688c86a3)
    },
    {   UINT64_C(0x00000000000062ec), UINT64_C(0x1f42fe9028b506ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8ef8b28722), UINT64_C(0x1f42ff0f08afc720)
    },
    {   UINT64_C(0x000000000000593f), UINT64_C(0x1f42f9ce48b0271d), UINT64_C(0x1f42fb4edcb56708),
        UINT64_C(0x1f42f70ea8b4072c), UINT64_C(0x1f42f4cf10b6e70e)
    },
    {   UINT64_C(0x00000000000062ed), UINT64_C(0x1f42fb8ff8b0a6ee), UINT64_C(0x1f42fc504cb1e6ee),
        UINT64_C(0x1f42f84efcaf670b), UINT64_C(0x1f42f84f68ac46f9)
    },
    {   UINT64_C(0x000000000001913c), UINT64_C(0x199af4cc209ee5b6), UINT64_C(0x1a32fbcf689d46a3),
        UINT64_C(0x1172f80f688146a2), UINT64_C(0x1a42f84c24a566a2)
    },
    {   UINT64_C(0x00000000000062ee), UINT64_C(0x1f42fd0eacb3e712), UINT64_C(0x1f42fd9078b366e9),
        UINT64_C(0x1f42fc8fa8b646ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001913d), UINT64_C(0x1952fbcf6899c6a3), UINT64_C(0x1452f8cf688d86a3),
        UINT64_C(0x238af94f68c2a5b6), UINT64_C(0x1a9af94f689ee5b6)
    },
    {   UINT64_C(0x00000000000062ef), UINT64_C(0x1f42f99068b626eb), UINT64_C(0x1f42fb4fe4b5c6ff),
        UINT64_C(0x1f42f70fbcb546ee), UINT64_C(0x1f42fe4fb0afc6f7)
    },
    {   UINT64_C(0x000000000001913e), UINT64_C(0x1a82f6cf68a005b6), UINT64_C(0x1baaf44c6c9ec5b6),
        UINT64_C(0x1c8af6cc6ca6a5b6), UINT64_C(0x152af80fa08f85b6)
    },
    {   UINT64_C(0x00000000000062f0), UINT64_C(0x1f42f78fc0b5c6e3), UINT64_C(0x1f42fa9080b0e70b),
        UINT64_C(0x1f42fb5018b006f4), UINT64_C(0x1f42fd4f5cb3e6f2)
    },
    {   UINT64_C(0x000000000001913f), UINT64_C(0x1192facf6876a615), UINT64_C(0x1352f9cf688b0685),
        UINT64_C(0x1a52f5cf689dc5aa), UINT64_C(0x16e2f80f689245b2)
    },
    {   UINT64_C(0x00000000000062f1), UINT64_C(0x1f42fb8f08b2c6ff), UINT64_C(0x1f42fc9094b2e6fe),
        UINT64_C(0x1f42fc503cad66ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005944), UINT64_C(0x1f42f7cefcb5c727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ef0b1a712), UINT64_C(0x1f42fa4e84b58734)
    },
    {   UINT64_C(0x00000000000062f2), UINT64_C(0x1f42fb4ef4af8717), UINT64_C(0x1f42fb8ee8ae0716),
        UINT64_C(0x1f42f78e84b7a71c), UINT64_C(0x1f42fa501cb546f7)
    },
    {   UINT64_C(0x00000000000089f2), UINT64_C(0x1e32fd0f3cafc5fd), UINT64_C(0x1e32fd0f3cafc5fd),
        UINT64_C(0x0000000000000000), UINT64_C(0x1c32ee0d30aa667e)
    },
    {   UINT64_C(0x0000000000008045), UINT64_C(0x131af10fa0892754), UINT64_C(0x0de2f44f78738680),
        UINT64_C(0x131af10fa089272a), UINT64_C(0x0de2f44f78738656)
    },
    {   UINT64_C(0x00000000000089f3), UINT64_C(0x0a2af30c4c67270e), UINT64_C(0x1c32ee0d30aa667e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062f4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54e9cb6c6e7),
        UINT64_C(0x1f42fb0fe8ac26f0), UINT64_C(0x1f42fa1010afa6f6)
    },
    {   UINT64_C(0x00000000000089f4), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x1362f5cf2c882600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x156afa8fa08dc630)
    },
    {   UINT64_C(0x00000000000062f5), UINT64_C(0x1f42fa0eecb6471e), UINT64_C(0x1f42fd8f50b4a708),
        UINT64_C(0x1f42fd5028b0c706), UINT64_C(0x1f42f84ef8b1072a)
    },
    {   UINT64_C(0x00000000000089f5), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x1fdaf80e60b7262e),
        UINT64_C(0x1fdafb8e60b8e62e), UINT64_C(0x2662f80e60d3062e)
    },
    {   UINT64_C(0x00000000000062f6), UINT64_C(0x1f42fa4f7cab0711), UINT64_C(0x1f42f98ff4b5470d),
        UINT64_C(0x1f42fb8f68b3a719), UINT64_C(0x1f42f54e7cb6471a)
    },
    {   UINT64_C(0x000000000000c740), UINT64_C(0x101af04fa07d65c3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c741), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x109af30fa07e65b5)
    },
    {   UINT64_C(0x00000000000089f7), UINT64_C(0x1a8afa913ca2657b), UINT64_C(0x3132fbd084fc65aa),
        UINT64_C(0x3132fbd084fc65aa), UINT64_C(0x0fb3074cb470a69d)
    },
    {   UINT64_C(0x00000000000062f8), UINT64_C(0x1f42f70ff8b5c706), UINT64_C(0x1f42fd0fa0b4272d),
        UINT64_C(0x1f42fd8ef4ae6727), UINT64_C(0x1f42fa4f00ad4703)
    },
    {   UINT64_C(0x00000000000089f8), UINT64_C(0x18b2f94b1c9ba703), UINT64_C(0x17e2f94e44986635),
        UINT64_C(0x191af94e449d4635), UINT64_C(0x191af94e449d4635)
    },
    {   UINT64_C(0x00000000000062f9), UINT64_C(0x1f42fb4eb0b44728), UINT64_C(0x1f42fccf70ae4711),
        UINT64_C(0x1f42fc4e8cb2672f), UINT64_C(0x1f42fa8ff4b14715)
    },
    {   UINT64_C(0x00000000000089f9), UINT64_C(0x24f2f94d28cca67e), UINT64_C(0x24f2f94d28cca67e),
        UINT64_C(0x0fc31d0c4c7ac6b5), UINT64_C(0x0fc2f38c4c6606b5)
    },
    {   UINT64_C(0x000000000000c747), UINT64_C(0x12c2f20f548785b5), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019149), UINT64_C(0x0f9afacc5c7825bc), UINT64_C(0x1082fa0c5c7a069f),
        UINT64_C(0x0c02f88f5868c69e), UINT64_C(0x0acaf90c5c65669f)
    },
    {   UINT64_C(0x00000000000062fb), UINT64_C(0x1f42fd10bcb4c6f7), UINT64_C(0x1f42f6cf98b1e6e7),
        UINT64_C(0x1f42fb4f38ab2712), UINT64_C(0x1f42f9502cb346f2)
    },
    {   UINT64_C(0x0000000000003bfc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106b1256086dc66f)
    },
    {   UINT64_C(0x00000000000062fc), UINT64_C(0x1f42fa4f98b40715), UINT64_C(0x1f42f90ef8b14704),
        UINT64_C(0x1f42f84f28b5a72d), UINT64_C(0x1f42f94fb4b6a6df)
    },
    {   UINT64_C(0x0000000000003bfd), UINT64_C(0x0000000000000000), UINT64_C(0x106afc979878a667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062fd), UINT64_C(0x1f42facfe0b60722), UINT64_C(0x1f42fd8f80ac66fb),
        UINT64_C(0x1f42fa8facb3c717), UINT64_C(0x1f42f48eccb1e71d)
    },
    {   UINT64_C(0x000000000001914c), UINT64_C(0x1032fa4f40792657), UINT64_C(0x0bb2f78f5868a678),
        UINT64_C(0x143afd0f5888a5c6), UINT64_C(0x101af9cf58788632)
    },
    {   UINT64_C(0x00000000000062fe), UINT64_C(0x1f42fa8f5caf270e), UINT64_C(0x1f42f7cf50b6a6d7),
        UINT64_C(0x1f42f78fecb0c710), UINT64_C(0x1f42fc8fa4ad8706)
    },
    {   UINT64_C(0x00000000000089fe), UINT64_C(0x3132fbceecfc660f), UINT64_C(0x3132fbceecfc660f),
        UINT64_C(0x3132fbcf78fc65ed), UINT64_C(0x3132fbd084fc65aa)
    },
    {   UINT64_C(0x00000000000062ff), UINT64_C(0x1f42f88f7cb7a6ea), UINT64_C(0x1f42f78ef8b34715),
        UINT64_C(0x1f42fb8f04b546e2), UINT64_C(0x1f42f6cf74aec701)
    },
    {   UINT64_C(0x00000000000089ff), UINT64_C(0x3132fbd084fc65aa), UINT64_C(0x3132fbceecfc660f),
        UINT64_C(0x3132fbceecfc660f), UINT64_C(0x3132fbd088fc65a9)
    },
    {   UINT64_C(0x0000000000006300), UINT64_C(0x1f42fb8fd8afe6f2), UINT64_C(0x1f42f78f28afa71b),
        UINT64_C(0x1f42f94f5cb44704), UINT64_C(0x1f42f90f3cac870a)
    },
    {   UINT64_C(0x0000000000005953), UINT64_C(0x1f42fc903cb42700), UINT64_C(0x1f42f58f4cb7c71c),
        UINT64_C(0x1f42f90fc4b62730), UINT64_C(0x1f42fc4f50b6a70f)
    },
    {   UINT64_C(0x0000000000006301), UINT64_C(0x1f42ff5000ae46ff), UINT64_C(0x1f42f78f28afe70f),
        UINT64_C(0x1f42f6cef4b0c72f), UINT64_C(0x1f42fa8e8caf070a)
    },
    {   UINT64_C(0x0000000000005954), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf34b786fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43029008b5e6fe)
    },
    {   UINT64_C(0x0000000000006302), UINT64_C(0x1f42fb8fc0b54710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7500cb0670a), UINT64_C(0x1f42f7cebcb4c72d)
    },
    {   UINT64_C(0x0000000000005955), UINT64_C(0x1f42fa0f78b6a700), UINT64_C(0x1f42fa8f28b8872e),
        UINT64_C(0x1f42ff8f54b7270e), UINT64_C(0x1f42f3cfbcb6a70f)
    },
    {   UINT64_C(0x0000000000006303), UINT64_C(0x1f42fa8fccaf0722), UINT64_C(0x1f42fecf08b4c728),
        UINT64_C(0x1f42fc4fa4b58711), UINT64_C(0x1f42f98eccb3472e)
    },
    {   UINT64_C(0x0000000000005956), UINT64_C(0x1f42f7cf4cb3a6f6), UINT64_C(0x1f42faceecb78736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbd02cb1c705)
    },
    {   UINT64_C(0x0000000000006304), UINT64_C(0x1f42fa0ef0b3a71a), UINT64_C(0x1f42f78f64ad4716),
        UINT64_C(0x1f42fb4f28ae66f4), UINT64_C(0x1f42fa4ee8ac6702)
    },
    {   UINT64_C(0x0000000000019153), UINT64_C(0x15d2fe4f688da67e), UINT64_C(0x11c2f10fa090267e),
        UINT64_C(0x1cc2f10fa0b5267e), UINT64_C(0x1802f6cfa09a267e)
    },
    {   UINT64_C(0x0000000000006305), UINT64_C(0x1f42fbcef8af66f6), UINT64_C(0x1f42fc8ea0b4e70e),
        UINT64_C(0x1f42fa8f4cb1a712), UINT64_C(0x1f42f88ed8b26715)
    },
    {   UINT64_C(0x0000000000005958), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8f38b5a728), UINT64_C(0x1f42face9cb70720)
    },
    {   UINT64_C(0x0000000000006306), UINT64_C(0x1f42fdcfa8b3a71c), UINT64_C(0x1f42fe8f3caf870b),
        UINT64_C(0x1f42f54ec0ae0739), UINT64_C(0x1f42fb8e8cb0e722)
    },
    {   UINT64_C(0x0000000000019155), UINT64_C(0x1042f94f7879e5b4), UINT64_C(0x11f2f94f787f6693),
        UINT64_C(0x105af94f787a45b4), UINT64_C(0x0b82f44fa0746693)
    },
    {   UINT64_C(0x0000000000006307), UINT64_C(0x1f42f40ff8b10708), UINT64_C(0x1f42fd8ff8acc712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f8cb2c71f)
    },
    {   UINT64_C(0x000000000000595a), UINT64_C(0x1f42fa0fa8b7271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ff4b806ea), UINT64_C(0x1f42ff8f4cb426fd)
    },
    {   UINT64_C(0x0000000000006308), UINT64_C(0x1f42f94f90b6a70e), UINT64_C(0x1f42fa8fa4b366dc),
        UINT64_C(0x1f42f98f14b30719), UINT64_C(0x1f42f94fa4b10700)
    },
    {   UINT64_C(0x000000000000595b), UINT64_C(0x1f43018f88b5c712), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43110f10b88701)
    },
    {   UINT64_C(0x0000000000006309), UINT64_C(0x1f42fc8ef8b24729), UINT64_C(0x1f42fccffcadc6fd),
        UINT64_C(0x1f42f78eb4b54724), UINT64_C(0x1f42fb9020b0c6d7)
    },
    {   UINT64_C(0x000000000000595c), UINT64_C(0x1f43034f70b6c709), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecfa8b62715), UINT64_C(0x1f43028fa4b88728)
    },
    {   UINT64_C(0x000000000000630a), UINT64_C(0x1f42f7cf6caf472d), UINT64_C(0x1f42fa8ef8b326fc),
        UINT64_C(0x1f42fa8f04ab8716), UINT64_C(0x1f42f84f4ca98710)
    },
    {   UINT64_C(0x000000000000595d), UINT64_C(0x1f4308cef8b2c727), UINT64_C(0x1f43098f64b68706),
        UINT64_C(0x1f43034f1cb88703), UINT64_C(0x1f42f9cec8b8a719)
    },
    {   UINT64_C(0x000000000000630b), UINT64_C(0x1f42f5ce50b82712), UINT64_C(0x1f42fa8e7cb3c72a),
        UINT64_C(0x1f42f58f50b14702), UINT64_C(0x1f42fb8fb4b08707)
    },
    {   UINT64_C(0x000000000001915a), UINT64_C(0x1522f30fa09565aa), UINT64_C(0x1182f58fa080c5aa),
        UINT64_C(0x0ea2f30c387e45aa), UINT64_C(0x103af8cfa07a05aa)
    },
    {   UINT64_C(0x000000000000630c), UINT64_C(0x1f42fbcf4caca6fa), UINT64_C(0x1f42f70f30b4c72d),
        UINT64_C(0x1f42f8cf18a80712), UINT64_C(0x1f42f60f60b7a6f2)
    },
    {   UINT64_C(0x000000000001915b), UINT64_C(0x1ac2fa8f3ca326a5), UINT64_C(0x1a8afa8f54a1c6a5),
        UINT64_C(0x182afb8f5499e6a5), UINT64_C(0x1bdafb8f54a706a5)
    },
    {   UINT64_C(0x000000000000630d), UINT64_C(0x1f42f70f40b3c70d), UINT64_C(0x1f42f88eecad6729),
        UINT64_C(0x1f42f80ef4b1270e), UINT64_C(0x1f42f94fc8b8a6e9)
    },
    {   UINT64_C(0x000000000001915c), UINT64_C(0x18a2fb8f5499e6a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1a0afb8f54a026a5), UINT64_C(0x1a5af88c6ca1e6a5)
    },
    {   UINT64_C(0x000000000000630e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f78afc712),
        UINT64_C(0x1f42f8cef4b626d6), UINT64_C(0x1f42f64f98b7c6d7)
    },
    {   UINT64_C(0x0000000000005961), UINT64_C(0x1f430a4eacac671c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f4fc8b1c71a)
    },
    {   UINT64_C(0x000000000000630f), UINT64_C(0x1f42f70f5cb386f1), UINT64_C(0x1f42f90f0cb7a6f2),
        UINT64_C(0x1f42fd4fd8b26702), UINT64_C(0x1f42fccf2cb9c720)
    },
    {   UINT64_C(0x000000000001915e), UINT64_C(0x236afa8f3cc5a6a5), UINT64_C(0x1af2fa8f3ca3c6a5),
        UINT64_C(0x1c6afb8f54a946a5), UINT64_C(0x1a92fa8c6ca1e6a5)
    },
    {   UINT64_C(0x0000000000006310), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cfe8b76712),
        UINT64_C(0x1f42f70e50b5c709), UINT64_C(0x1f42f68ef8b626d2)
    },
    {   UINT64_C(0x000000000001915f), UINT64_C(0x1d12fb8eb4af46a5), UINT64_C(0x1ba2fa8f3ca7a6a5),
        UINT64_C(0x1aa2fb8f54a2c6a5), UINT64_C(0x18cafb8f3c9b06a5)
    },
    {   UINT64_C(0x0000000000006311), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cf20b44702),
        UINT64_C(0x1f43085008b4e6f8), UINT64_C(0x1f430450c0b166d1)
    },
    {   UINT64_C(0x0000000000019160), UINT64_C(0x1ae2fb8f3ca3a6a5), UINT64_C(0x1abafa8f54a286a5),
        UINT64_C(0x230afa8f54c3c6a5), UINT64_C(0x199afa8f3c9f66a5)
    },
    {   UINT64_C(0x0000000000006312), UINT64_C(0x1f43088f10b1471b), UINT64_C(0x1f42fc8f9cb78706),
        UINT64_C(0x1f43170f30a88728), UINT64_C(0x1f43020f74b4c6f7)
    },
    {   UINT64_C(0x0000000000005965), UINT64_C(0x1f430a4fa4b6a6fd), UINT64_C(0x1f4301ceacad2701),
        UINT64_C(0x1f430e0ee4af2735), UINT64_C(0x1f43000e5cb30730)
    },
    {   UINT64_C(0x0000000000006313), UINT64_C(0x1f42fa4fc8b7c70a), UINT64_C(0x1f4301ce9cb48724),
        UINT64_C(0x1f43144ef4b1e701), UINT64_C(0x1f42fd8eacb58718)
    },
    {   UINT64_C(0x0000000000005966), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcce7cb64730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4e4cb6e718)
    },
    {   UINT64_C(0x0000000000006314), UINT64_C(0x1f43108e98a9c744), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005967), UINT64_C(0x1f42f9ceb8b6c724), UINT64_C(0x1f430d4e5caf471b),
        UINT64_C(0x1f42fece7cb36715), UINT64_C(0x1f42f90e5cb66732)
    },
    {   UINT64_C(0x0000000000006315), UINT64_C(0x1f42fe4f48b6a716), UINT64_C(0x1f430f0f24b726fd),
        UINT64_C(0x1f43084eb0b786ec), UINT64_C(0x1f42f98f98b7a6b5)
    },
    {   UINT64_C(0x0000000000005968), UINT64_C(0x1f42f98e98b60720), UINT64_C(0x1f42fb8e7cb92731),
        UINT64_C(0x1f42face6cb7070c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006316), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64f20b7a6da), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005969), UINT64_C(0x1f43084e1cabc728), UINT64_C(0x1f43054e8ca9a72e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054e6cb3a724)
    },
    {   UINT64_C(0x0000000000006317), UINT64_C(0x1f42fdcf48b4e6d5), UINT64_C(0x1f42fdcfacb306d9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf0cb40715)
    },
    {   UINT64_C(0x000000000000596a), UINT64_C(0x1f43048ec8b0c6e0), UINT64_C(0x1f43010eb8b3271e),
        UINT64_C(0x1f43020eacb3c6f8), UINT64_C(0x1f42f4ceacb12733)
    },
    {   UINT64_C(0x0000000000006318), UINT64_C(0x0000000000000000), UINT64_C(0x1f43160e88b44727),
        UINT64_C(0x1f42ffce5cb92748), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000596b), UINT64_C(0x1f42fc8e4cb38726), UINT64_C(0x1f43080ef8b38708),
        UINT64_C(0x1f42fece9cb3a70d), UINT64_C(0x1f42f7ced8b2a71d)
    },
    {   UINT64_C(0x0000000000006319), UINT64_C(0x1f43004f58b546c7), UINT64_C(0x1f43004f70ab26f3),
        UINT64_C(0x1f43044e94b14723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019168), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa082e67e),
        UINT64_C(0x14faf4cf688e268c), UINT64_C(0x15a3008fa090867e)
    },
    {   UINT64_C(0x000000000000631a), UINT64_C(0x1f42fe8e7cab271c), UINT64_C(0x1f42fe8f8cb8a6ec),
        UINT64_C(0x1f43100e48b80736), UINT64_C(0x1f42fa4e80b58732)
    },
    {   UINT64_C(0x000000000000596d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4ea4b2e6dc), UINT64_C(0x1f43024e5cb6671b)
    },
    {   UINT64_C(0x000000000000631b), UINT64_C(0x1f42facf30b2270d), UINT64_C(0x1f42f78f4cb786d1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f24b00702)
    },
    {   UINT64_C(0x000000000001916a), UINT64_C(0x0c8af94fa06b067e), UINT64_C(0x0c72f10f6864a67e),
        UINT64_C(0x13d3008fa08d467e), UINT64_C(0x117b008fa082c67e)
    },
    {   UINT64_C(0x000000000000631c), UINT64_C(0x1f43060f10b666fb), UINT64_C(0x1f42fd8f40b526c5),
        UINT64_C(0x1f43038f58b2a6c1), UINT64_C(0x1f43004fa4b146e4)
    },
    {   UINT64_C(0x000000000000596f), UINT64_C(0x1f42f68f50b92707), UINT64_C(0x1f42fb0ff0b6c70c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78eacb506fb)
    },
    {   UINT64_C(0x000000000000631d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cf48b6a6dd),
        UINT64_C(0x1f43054ec4b08710), UINT64_C(0x1f43038f50b2270e)
    },
    {   UINT64_C(0x000000000001916c), UINT64_C(0x1662f58ce494e68c), UINT64_C(0x126b008fa087267e),
        UINT64_C(0x10eaf60f687aa68c), UINT64_C(0x1302f1cfa088a67e)
    },
    {   UINT64_C(0x000000000000631e), UINT64_C(0x1f43190e949e8736), UINT64_C(0x1f43094f40ade71d),
        UINT64_C(0x1f42fe8ed4afa715), UINT64_C(0x1f4300cee8b126fc)
    },
    {   UINT64_C(0x000000000001916d), UINT64_C(0x14aaff4f6888867e), UINT64_C(0x1472f5cf748c667e),
        UINT64_C(0x1ccaf54f74ae067e), UINT64_C(0x15e2f5cfa092267e)
    },
    {   UINT64_C(0x000000000000631f), UINT64_C(0x1f42fe4f6cb346dd), UINT64_C(0x1f4302cf64ae26fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005972), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf88b5070b)
    },
    {   UINT64_C(0x0000000000006320), UINT64_C(0x1f43024f38ae6709), UINT64_C(0x1f42fd8f58b446ff),
        UINT64_C(0x1f42fccf2caa26f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001916f), UINT64_C(0x0e42f3cf787485b7), UINT64_C(0x0f72f50f74752694),
        UINT64_C(0x0de2f3cf787405b7), UINT64_C(0x0a82f30fa06ca693)
    },
    {   UINT64_C(0x0000000000006321), UINT64_C(0x1f4300cef4b16701), UINT64_C(0x1f42ff8f70b8a703),
        UINT64_C(0x1f4300cf04b1e6e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019170), UINT64_C(0x0f82f34c4078e5b7), UINT64_C(0x0f3afdcfa0738694),
        UINT64_C(0x06e2fe0fa052068f), UINT64_C(0x07f2db8c4055a68f)
    },
    {   UINT64_C(0x0000000000006322), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f24b4e711),
        UINT64_C(0x1f42fb8f34ae8710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019171), UINT64_C(0x0e2afdcfa0770694), UINT64_C(0x066afd4fa0508694),
        UINT64_C(0x17a2fe4fa094e5b8), UINT64_C(0x0f3afdcfa07385b8)
    },
    {   UINT64_C(0x0000000000006323), UINT64_C(0x1f43000f6cb72705), UINT64_C(0x1f42fe0f3cb2e6fa),
        UINT64_C(0x1f42f94f74b5e6f9), UINT64_C(0x1f43058ec8b7e720)
    },
    {   UINT64_C(0x0000000000019172), UINT64_C(0x0f9af50f787965b7), UINT64_C(0x0f92fecc407945b8),
        UINT64_C(0x0f92f50c4074e5b8), UINT64_C(0x0a82fdcfa06785b8)
    },
    {   UINT64_C(0x0000000000006324), UINT64_C(0x1f42fe8ec0b866f3), UINT64_C(0x1f43008f6cb626dd),
        UINT64_C(0x1f42fbced4b166ee), UINT64_C(0x1f42fd8f1cb166fb)
    },
    {   UINT64_C(0x0000000000005977), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e74b486fb),
        UINT64_C(0x1f42f58e98b406f0), UINT64_C(0x1f42f60ed0b78733)
    },
    {   UINT64_C(0x0000000000006325), UINT64_C(0x1f43010f80b74706), UINT64_C(0x1f4303cf50b1670c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecea8b5c720)
    },
    {   UINT64_C(0x0000000000019174), UINT64_C(0x15f2f50f6892e5ad), UINT64_C(0x0f12f58fa07705ad),
        UINT64_C(0x0f22f50c407725ad), UINT64_C(0x0dfaf44fa072a5ad)
    },
    {   UINT64_C(0x0000000000018c0c), UINT64_C(0x2c22f95114e96589), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c7a0), UINT64_C(0x1f42f2cf64b92783), UINT64_C(0x1692f3cf6896e6b1),
        UINT64_C(0x1f42f2cf64b92783), UINT64_C(0x1692f3cf6896e6b1)
    },
    {   UINT64_C(0x000000000000c7a1), UINT64_C(0x1f42f2cf64b92754), UINT64_C(0x1692f3cf6896e682),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019176), UINT64_C(0x13db020fa089268c), UINT64_C(0x1233020fa085c68c),
        UINT64_C(0x1752f74f5492669b), UINT64_C(0x165b020fa08de68c)
    },
    {   UINT64_C(0x0000000000006328), UINT64_C(0x1f4317ce7c9d0741), UINT64_C(0x1f42ff8f1cb20718),
        UINT64_C(0x1f430b4ee0b20722), UINT64_C(0x1f431cce7cad071e)
    },
    {   UINT64_C(0x0000000000019177), UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x0f0af1ce2071468c),
        UINT64_C(0x1643020fa096e68c), UINT64_C(0x12db020fa087e68c)
    },
    {   UINT64_C(0x0000000000006329), UINT64_C(0x1f42f88fdcaec6e6), UINT64_C(0x1f42fc8f98b50702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43144ed0b4673c)
    },
    {   UINT64_C(0x0000000000019178), UINT64_C(0x1c8b020fa0a6a68c), UINT64_C(0x16a3020f588f068c),
        UINT64_C(0x1852f6cf589b669c), UINT64_C(0x122b020fa085a68c)
    },
    {   UINT64_C(0x000000000000632a), UINT64_C(0x1f43034e90b32739), UINT64_C(0x1f42f68f90b766d0),
        UINT64_C(0x1f4309ce7cb6a747), UINT64_C(0x1f43028e70b3c728)
    },
    {   UINT64_C(0x0000000000019179), UINT64_C(0x185af6cce09cc69b), UINT64_C(0x14f3020fa090a68c),
        UINT64_C(0x1152f6cf547e669b), UINT64_C(0x140af18fa08ce68c)
    },
    {   UINT64_C(0x000000000000632b), UINT64_C(0x1f430e4e60b4e735), UINT64_C(0x1f43040e54b0871d),
        UINT64_C(0x1f43040e90b08729), UINT64_C(0x1f42f4cf04b7c728)
    },
    {   UINT64_C(0x000000000001917a), UINT64_C(0x15db00cf548c868c), UINT64_C(0x1592f44f5891a68c),
        UINT64_C(0x1f2af58f64b7668c), UINT64_C(0x1722f80fa096068c)
    },
    {   UINT64_C(0x000000000000632c), UINT64_C(0x1f43058e40b06749), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a4e84af073d), UINT64_C(0x1f43058e40b5e74e)
    },
    {   UINT64_C(0x000000000001917b), UINT64_C(0x15b2f30fa092c68c), UINT64_C(0x146af44fa08c668c),
        UINT64_C(0x0f4af3cf687405c1), UINT64_C(0x1002fbcf6c7b668c)
    },
    {   UINT64_C(0x000000000000632d), UINT64_C(0x1f43028e40b4e73b), UINT64_C(0x1f42ffcee0b56733),
        UINT64_C(0x1f42f3106cb76702), UINT64_C(0x1f431bcecca7a730)
    },
    {   UINT64_C(0x000000000001917c), UINT64_C(0x0e72f44f6875c5c1), UINT64_C(0x1032f44f6878668c),
        UINT64_C(0x0e22f44f6c74a5c1), UINT64_C(0x0aa2f18fa06d469b)
    },
    {   UINT64_C(0x000000000000632e), UINT64_C(0x1f43050e4cb4e741), UINT64_C(0x1f42f7cf80b34700),
        UINT64_C(0x1f42f78fa0b6c6c2), UINT64_C(0x1f431bce4cb1c74f)
    },
    {   UINT64_C(0x000000000001917d), UINT64_C(0x0fc2f50c687bc5c0), UINT64_C(0x0ff2fbcfa077668c),
        UINT64_C(0x07aafa8fa056e698), UINT64_C(0x0a22dfcc74602698)
    },
    {   UINT64_C(0x000000000000632f), UINT64_C(0x1f42fc8fccae4706), UINT64_C(0x1f42fc900cb0a709),
        UINT64_C(0x1f43024fe0b106c6), UINT64_C(0x1f43174e9cab6738)
    },
    {   UINT64_C(0x0000000000005982), UINT64_C(0x1f4309cf38ac4703), UINT64_C(0x1f43070f1cadc6db),
        UINT64_C(0x1f430a4e98b3c70c), UINT64_C(0x1f43058ec0b6e712)
    },
    {   UINT64_C(0x0000000000006330), UINT64_C(0x1f43050e4cad8741), UINT64_C(0x1f42ff8e70b46749),
        UINT64_C(0x1f42f68f6cb76719), UINT64_C(0x1f43018e4cb0874a)
    },
    {   UINT64_C(0x000000000001917f), UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1002fbcc6c7b65c1),
        UINT64_C(0x1002f44c6c77a5c1), UINT64_C(0x0cdafbcfa07145c1)
    },
    {   UINT64_C(0x0000000000006331), UINT64_C(0x1f42f5cf0cb18731), UINT64_C(0x1f43014f54b0070a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43140ebcb84735)
    },
    {   UINT64_C(0x0000000000019180), UINT64_C(0x0c0af44f6c6b25c1), UINT64_C(0x0c12efcf706ce65e),
        UINT64_C(0x1032fbcf647865b5), UINT64_C(0x0e72f1cf687665b5)
    },
    {   UINT64_C(0x0000000000006332), UINT64_C(0x1f43198e80a58732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98f1cb76714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019181), UINT64_C(0x15eaf24f689405b5), UINT64_C(0x0fd2f30fa07b45b5),
        UINT64_C(0x0ec2f28c6c7765b5), UINT64_C(0x0dcaf34fa072a5b5)
    },
    {   UINT64_C(0x0000000000006333), UINT64_C(0x1f43184ea8a3a72a), UINT64_C(0x1f42f8cfa0b62715),
        UINT64_C(0x1f42f90e9cb60737), UINT64_C(0x1f43014fa8aba6e1)
    },
    {   UINT64_C(0x0000000000019182), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa08ea67e),
        UINT64_C(0x14fb060f689ce68c), UINT64_C(0x15a3008fa09c667e)
    },
    {   UINT64_C(0x0000000000006334), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ced8b746d1),
        UINT64_C(0x1f42f4cef0ba4722), UINT64_C(0x1f42f8ce80abc739)
    },
    {   UINT64_C(0x000000000000c7bc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f1cfa095e783), UINT64_C(0x0fa2f20c6c7ac6b1)
    },
    {   UINT64_C(0x0000000000006335), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78ed8ab4708),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68e88af8725)
    },
    {   UINT64_C(0x0000000000019184), UINT64_C(0x0c8afccfa07e867e), UINT64_C(0x0c72f38f6879467e),
        UINT64_C(0x13d3008fa099a67e), UINT64_C(0x117b008fa085067e)
    },
    {   UINT64_C(0x0000000000006336), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f70a9a6e6),
        UINT64_C(0x1f43038e68b6c72c), UINT64_C(0x1f42fe0e48b3874e)
    },
    {   UINT64_C(0x0000000000019185), UINT64_C(0x1a5b00cfa0b3267e), UINT64_C(0x151b008f689e467e),
        UINT64_C(0x165306cf689fe68c), UINT64_C(0x106b008fa08e467e)
    },
    {   UINT64_C(0x0000000000006337), UINT64_C(0x1f42f68f00b4a71e), UINT64_C(0x1f42f8ceecb446ba),
        UINT64_C(0x1f43100e80b16749), UINT64_C(0x1f42fe8fc0b6271b)
    },
    {   UINT64_C(0x0000000000019186), UINT64_C(0x166306cce49fe68c), UINT64_C(0x126b008fa08e667e),
        UINT64_C(0x10eafd8f6889268c), UINT64_C(0x1303164fa09c067e)
    },
    {   UINT64_C(0x0000000000006338), UINT64_C(0x1f42f4cfb8b5e715), UINT64_C(0x1f42f68e90b34733),
        UINT64_C(0x1f4318ce6cb3671d), UINT64_C(0x1f43160e70aaa745)
    },
    {   UINT64_C(0x0000000000019187), UINT64_C(0x14ab0b4f689d267e), UINT64_C(0x14731f0f74a1067e),
        UINT64_C(0x1ccb1e8f74c2a67e), UINT64_C(0x15e2f5cfa0a3467e)
    },
    {   UINT64_C(0x0000000000006339), UINT64_C(0x1f42fc8ee0b5c70e), UINT64_C(0x1f42f98e5cb0e716),
        UINT64_C(0x1f42f9cf0cb6a720), UINT64_C(0x1f42f88f30b6c6f3)
    },
    {   UINT64_C(0x0000000000019188), UINT64_C(0x13b31c8fa09f467e), UINT64_C(0x13eaf50fa09de67e),
        UINT64_C(0x0e02fbcf7878e5b7), UINT64_C(0x0f83008f78816694)
    },
    {   UINT64_C(0x000000000000633a), UINT64_C(0x1f431a0e6ca8c73a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b0e6cace742), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000598d), UINT64_C(0x1f43148e7ca2c700), UINT64_C(0x1f43069018b206c8),
        UINT64_C(0x1f42f70f60b02710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000633b), UINT64_C(0x1f42f6cf84b786f2), UINT64_C(0x1f42fa8f50b86725),
        UINT64_C(0x1f430f0e30b16726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001918a), UINT64_C(0x0f82ec4c408665b7), UINT64_C(0x0f3afdcfa07e4694),
        UINT64_C(0x06e2ff4fa067268f), UINT64_C(0x07f2d08c406ac68f)
    },
    {   UINT64_C(0x000000000000633c), UINT64_C(0x1f42fc8f40b7a711), UINT64_C(0x1f42f88e7cb466e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf14b186f0)
    },
    {   UINT64_C(0x000000000001918b), UINT64_C(0x0e2afdcfa0840694), UINT64_C(0x066afd4fa065a694),
        UINT64_C(0x17a2fe4fa09f65b8), UINT64_C(0x0f3afdcfa07e45b8)
    },
    {   UINT64_C(0x000000000000633d), UINT64_C(0x1f42fecf4cada70e), UINT64_C(0x1f42fa8ee0a986ca),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001918c), UINT64_C(0x0f9b010f788185b7), UINT64_C(0x0f92f18c4081c5b8),
        UINT64_C(0x0f93000c408325b8), UINT64_C(0x0a82fdcfa07565b8)
    },
    {   UINT64_C(0x000000000000633e), UINT64_C(0x1f42f74eb0b84742), UINT64_C(0x1f43000f88b606de),
        UINT64_C(0x1f42facebcb54742), UINT64_C(0x1f42f98e70b3871d)
    },
    {   UINT64_C(0x000000000001918d), UINT64_C(0x0c2afd4f787485b7), UINT64_C(0x091b060f786d862c),
        UINT64_C(0x0f5b044f748185ad), UINT64_C(0x0ed311cf6883e5ad)
    },
    {   UINT64_C(0x000000000000633f), UINT64_C(0x1f42fa8ec0b5472b), UINT64_C(0x1f43078e50b7a744),
        UINT64_C(0x1f43044ec0b1e71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c40), UINT64_C(0x106af8cfa076259f), UINT64_C(0x106b018fa07a859f),
        UINT64_C(0x106b19ce546a449f), UINT64_C(0x106af68f747bc656)
    },
    {   UINT64_C(0x0000000000006340), UINT64_C(0x1f43164e3cb44733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5ce44b66731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a40), UINT64_C(0x1cf2fc1084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbceecab660f), UINT64_C(0x1cf2fbceecab660f)
    },
    {   UINT64_C(0x0000000000006341), UINT64_C(0x1f43048e30b3e73d), UINT64_C(0x1f42ff8e40afa742),
        UINT64_C(0x1f42f5cfccade6fd), UINT64_C(0x1f430b8eb0ac2731)
    },
    {   UINT64_C(0x0000000000008a41), UINT64_C(0x1cf2f30eecafc60f), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbceecab660f), UINT64_C(0x1cf2fbceecab660f)
    },
    {   UINT64_C(0x0000000000006342), UINT64_C(0x1f42fc8f3cb16737), UINT64_C(0x1f42f4ce9cb1c711),
        UINT64_C(0x1f4302ce98afe73d), UINT64_C(0x1f43098e48ada72a)
    },
    {   UINT64_C(0x0000000000003c43), UINT64_C(0x106b015710768650), UINT64_C(0x106b0a571c75064b),
        UINT64_C(0x106b03d71c71c64a), UINT64_C(0x106ae90fa07ac64b)
    },
    {   UINT64_C(0x0000000000006343), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf00b16736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ed4b4c72e)
    },
    {   UINT64_C(0x0000000000008a43), UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000006344), UINT64_C(0x1f42facf24b64720), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf7cb4470f)
    },
    {   UINT64_C(0x0000000000008a44), UINT64_C(0x2c22fbd084e825a9), UINT64_C(0x1cf2fbd084aba5a9),
        UINT64_C(0x0ffafccd3077067e), UINT64_C(0x0ffafccd3077067e)
    },
    {   UINT64_C(0x0000000000006345), UINT64_C(0x1f43084eb0a5a743), UINT64_C(0x1f42f98e80af2739),
        UINT64_C(0x1f42fb8e60af8736), UINT64_C(0x1f42fc4eb0a98737)
    },
    {   UINT64_C(0x0000000000003c46), UINT64_C(0x106af78f68798676), UINT64_C(0x106afc0f8077c66c),
        UINT64_C(0x106ae94c147dc64b), UINT64_C(0x106aebcc147de596)
    },
    {   UINT64_C(0x0000000000006346), UINT64_C(0x0000000000000000), UINT64_C(0x1f430bce40b0a72d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbce70b5e73d)
    },
    {   UINT64_C(0x0000000000003c47), UINT64_C(0x106af2cd087dc64b), UINT64_C(0x106af24d087ea64b),
        UINT64_C(0x106af44d087a864b), UINT64_C(0x106afc0d74790656)
    },
    {   UINT64_C(0x0000000000006347), UINT64_C(0x0000000000000000), UINT64_C(0x1f431e0f24a166f4),
        UINT64_C(0x1f43240ee09ee734), UINT64_C(0x1f43210f049cc72e)
    },
    {   UINT64_C(0x0000000000003c48), UINT64_C(0x106af0cf807ea64b), UINT64_C(0x106af28d787ee596),
        UINT64_C(0x106af08d787fc596), UINT64_C(0x106af0ca147b859c)
    },
    {   UINT64_C(0x0000000000006348), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098ee0af8712),
        UINT64_C(0x1f43084f0caea708), UINT64_C(0x1f430a8e90af6721)
    },
    {   UINT64_C(0x0000000000003c49), UINT64_C(0x106af80c2881a59d), UINT64_C(0x106af20f907e0597),
        UINT64_C(0x106aec0d087ae64b), UINT64_C(0x106af14f747b0656)
    },
    {   UINT64_C(0x0000000000006349), UINT64_C(0x1f43220f24b06704), UINT64_C(0x1f43064fe8b44703),
        UINT64_C(0x1f43028f3cb6a6d6), UINT64_C(0x1f43054ec0ae472a)
    },
    {   UINT64_C(0x0000000000003c4a), UINT64_C(0x106af38d787e6596), UINT64_C(0x106af68f8c78a59f),
        UINT64_C(0x106af24c147de64b), UINT64_C(0x106af44c147a864b)
    },
    {   UINT64_C(0x000000000000634a), UINT64_C(0x1f43064edc9fa714), UINT64_C(0x1f43009020b6e6e1),
        UINT64_C(0x1f43060ee4b50737), UINT64_C(0x1f43144edcb4672d)
    },
    {   UINT64_C(0x000000000000520d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f00b54720)
    },
    {   UINT64_C(0x000000000000634b), UINT64_C(0x1f42ff903cb546e1), UINT64_C(0x1f43084f60ace716),
        UINT64_C(0x1f4305502cb126db), UINT64_C(0x1f430c4edcaee707)
    },
    {   UINT64_C(0x0000000000003c4c), UINT64_C(0x106af4ccfc7ca673), UINT64_C(0x106af30cfc7d6667),
        UINT64_C(0x106af5ccfc7c0672), UINT64_C(0x106af4cfa07ca673)
    },
    {   UINT64_C(0x000000000000634c), UINT64_C(0x1f43084eacaf2727), UINT64_C(0x1f43044ea4b30723),
        UINT64_C(0x1f430f4e98b0072a), UINT64_C(0x1f43100ea8aa8723)
    },
    {   UINT64_C(0x0000000000003c4d), UINT64_C(0x106ade4fa06ec6f6), UINT64_C(0x106af8cf8c80459c),
        UINT64_C(0x106ae34f8c6fc666), UINT64_C(0x106aedcab88043e7)
    },
    {   UINT64_C(0x000000000000634d), UINT64_C(0x1f43054fd8ae4706), UINT64_C(0x1f43058e88b2872c),
        UINT64_C(0x1f43029024b7c6b3), UINT64_C(0x1f43054e6cac072d)
    },
    {   UINT64_C(0x0000000000008a4d), UINT64_C(0x1cf3114fa0a2261a), UINT64_C(0x1cf3114fa0a2261a),
        UINT64_C(0x188303cd148fa5d5), UINT64_C(0x18830fcd1495a5d5)
    },
    {   UINT64_C(0x000000000000634e), UINT64_C(0x1f431dcf04a8c728), UINT64_C(0x1f43164e88a9a72c),
        UINT64_C(0x1f430dced4a80727), UINT64_C(0x1f43055028ada6eb)
    },
    {   UINT64_C(0x0000000000008a4e), UINT64_C(0x0ffaff8d3c726679), UINT64_C(0x0ffb060d3c75a679),
        UINT64_C(0x1cf2ff50d8aac58f), UINT64_C(0x1cf2fd10d8a9a58f)
    },
    {   UINT64_C(0x000000000000634f), UINT64_C(0x1f43028f28b56703), UINT64_C(0x1f430e8e88ab671d),
        UINT64_C(0x1f431b8ec8a5671a), UINT64_C(0x1f43028f50b02724)
    },
    {   UINT64_C(0x0000000000003c50), UINT64_C(0x106af80c147f85a1), UINT64_C(0x106b1e971471664a),
        UINT64_C(0x106b0b171467a64a), UINT64_C(0x106b06cfa06b850e)
    },
    {   UINT64_C(0x0000000000006350), UINT64_C(0x1f43120eb8ac6717), UINT64_C(0x1f43064edcaba71b),
        UINT64_C(0x1f43038f6cb7c70a), UINT64_C(0x1f43038fa0b16703)
    },
    {   UINT64_C(0x00000000000059a3), UINT64_C(0x1f42ff4f7cbb2719), UINT64_C(0x1f4302ceacac4700),
        UINT64_C(0x1f42fa0f80b58716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006351), UINT64_C(0x1f43204ef8b12724), UINT64_C(0x1f43044f54ae870a),
        UINT64_C(0x1f4302902cb726cf), UINT64_C(0x1f43018fc0b4470c)
    },
    {   UINT64_C(0x0000000000008a51), UINT64_C(0x1cf2fbce30ab663e), UINT64_C(0x1cf2fbce8cab65a9),
        UINT64_C(0x1cf2fbce8cab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000006352), UINT64_C(0x1f42ff905cb686eb), UINT64_C(0x1f4305cee8b7c723),
        UINT64_C(0x1f42facf48b80706), UINT64_C(0x1f43038f0cb6a720)
    },
    {   UINT64_C(0x0000000000008a52), UINT64_C(0x1cf2f79084ad85a9), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2f790acad859f), UINT64_C(0x1cf2f790acad859f)
    },
    {   UINT64_C(0x0000000000006353), UINT64_C(0x1f43018f5cb10702), UINT64_C(0x1f430e0edcb28726),
        UINT64_C(0x1f43034f8cae6712), UINT64_C(0x1f43140ea0ab0739)
    },
    {   UINT64_C(0x0000000000008a53), UINT64_C(0x0ffaff4d3075a67e), UINT64_C(0x0ffaff4d3075a67e),
        UINT64_C(0x1cf2f79310ad859f), UINT64_C(0x0ffb258d3075a67e)
    },
    {   UINT64_C(0x0000000000006354), UINT64_C(0x1f43028f34afe702), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68eb8b4c733)
    },
    {   UINT64_C(0x000000000000038f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af38e547ac69e)
    },
    {   UINT64_C(0x00000000000080a7), UINT64_C(0x11e2f34fa08485b8), UINT64_C(0x0c1ae84bfc720693),
        UINT64_C(0x103aee4bfc7ce5aa), UINT64_C(0x0eb2f34fa075a5b8)
    },
    {   UINT64_C(0x0000000000008a55), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x0000000000006356), UINT64_C(0x1f4308ce9cb36729), UINT64_C(0x1f43064eb4b16725),
        UINT64_C(0x1f43018f30b3c73d), UINT64_C(0x1f43074eb0b7a73e)
    },
    {   UINT64_C(0x0000000000008a56), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x0000000000006357), UINT64_C(0x1f43070f1cae071a), UINT64_C(0x1f42fb8e98a42716),
        UINT64_C(0x1f430e8f1cad872d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a57), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x0000000000006358), UINT64_C(0x1f43064f50b02723), UINT64_C(0x1f43018f1cae6736),
        UINT64_C(0x1f42fa8ef4b5072e), UINT64_C(0x1f4302cee0b38727)
    },
    {   UINT64_C(0x000000000000c804), UINT64_C(0x0a6af9124c626507), UINT64_C(0x0a6af9124c626507),
        UINT64_C(0x0fa2ee124c7d0507), UINT64_C(0x0fa2ee12747d04f7)
    },
    {   UINT64_C(0x0000000000006359), UINT64_C(0x1f43084ea8b28739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccfacb78720), UINT64_C(0x1f42facfe4b88711)
    },
    {   UINT64_C(0x000000ee9d78c806), UINT64_C(0x0a6aff95305e469b), UINT64_C(0x0a6b01d5305f669b),
        UINT64_C(0x0a6b01ccd05f6483), UINT64_C(0x0a6b01d5305f669b)
    },
    {   UINT64_C(0x000000000000635a), UINT64_C(0x1f42ffcf6caf86f6), UINT64_C(0x1f42ffcf6cb466f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ef8b32734)
    },
    {   UINT64_C(0x0000000000003c5b), UINT64_C(0x106b084c10786597), UINT64_C(0x106b074c10732406),
        UINT64_C(0x106b278c1079c460), UINT64_C(0x106afa4c10634460)
    },
    {   UINT64_C(0x000000000000635b), UINT64_C(0x1f43064ee0af26eb), UINT64_C(0x1f42ff8f7cb02712),
        UINT64_C(0x1f42f6cfc8b84712), UINT64_C(0x1f42ffcf1cb366fc)
    },
    {   UINT64_C(0x0000000000003c5c), UINT64_C(0x106afe8c10676460), UINT64_C(0x106b0e96586fe646),
        UINT64_C(0x106af656c47bc70f), UINT64_C(0x106afccfa078a64b)
    },
    {   UINT64_C(0x000000000000635c), UINT64_C(0x1f43040ee8b66729), UINT64_C(0x1f42fb0e1cb52729),
        UINT64_C(0x1f42f74f1cb10711), UINT64_C(0x1f42f54f14b4e6f9)
    },
    {   UINT64_C(0x000000000000c80c), UINT64_C(0x22b2fd4f2cc1a6aa), UINT64_C(0x2e02fd4f2ceee6aa),
        UINT64_C(0x09b300d6785be6b1), UINT64_C(0x0e9b00d6786f86b1)
    },
    {   UINT64_C(0x000000000000635d), UINT64_C(0x1f42f9cf60b4e71a), UINT64_C(0x1f42f80ee8ae8715),
        UINT64_C(0x1f43000f28b1070a), UINT64_C(0x1f42f94f64b6671d)
    },
    {   UINT64_C(0x0007d0000000c80e), UINT64_C(0x124304ccf87ca45d), UINT64_C(0x0a6afad03064a587),
        UINT64_C(0x0a6af5103061c587), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000635e), UINT64_C(0x1f43000f44b1870a), UINT64_C(0x1f42f94f08b30709),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f1cafe715)
    },
    {   UINT64_C(0x0000000000003c5f), UINT64_C(0x106aec8f8c7fa64b), UINT64_C(0x106aec0bfc79859c),
        UINT64_C(0x106aec0c1079859c), UINT64_C(0x106aec0c1479a59c)
    },
    {   UINT64_C(0x000000000000c811), UINT64_C(0x05bac40f7069a698), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1d9afc4f6cae0699)
    },
    {   UINT64_C(0x0000000000003c60), UINT64_C(0x106af0cc1486a651), UINT64_C(0x106af14c1476a64b),
        UINT64_C(0x106b0bcc1476264b), UINT64_C(0x106af8cc107a8613)
    },
    {   UINT64_C(0x0000000000006360), UINT64_C(0x1f4300cf04ae470a), UINT64_C(0x1f42f7cf20b76711),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c814), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2fc0c807686aa),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c815), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x187b00d6789706b1)
    },
    {   UINT64_C(0x0000000000003c62), UINT64_C(0x106af3cf307d059c), UINT64_C(0x106aeb0c147c259c),
        UINT64_C(0x106af48f8c798612), UINT64_C(0x106afd8b8878459c)
    },
    {   UINT64_C(0x0000000000006362), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ef0ae06f2),
        UINT64_C(0x1f42f94f20b8e716), UINT64_C(0x1f42f74eecaf871a)
    },
    {   UINT64_C(0x0000000000008a62), UINT64_C(0x0ffb120c487006b7), UINT64_C(0x0ffb070c4c6a86b7),
        UINT64_C(0x0ffb15cc4c71e6b7), UINT64_C(0x0eab1d0c4c7666b7)
    },
    {   UINT64_C(0x0000000000006363), UINT64_C(0x1f42f94e94b646fe), UINT64_C(0x1f42fa4e94b6c6fe),
        UINT64_C(0x1f42fc0ea0b72702), UINT64_C(0x1f42fb8ee0b186fa)
    },
    {   UINT64_C(0x0000000000008a63), UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0eab1d0c4c7666b7),
        UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0eab1d0c4c7666b7)
    },
    {   UINT64_C(0x0000000000006364), UINT64_C(0x1f42fa0f00ada6f2), UINT64_C(0x1f42fcce78b6c706),
        UINT64_C(0x1f42fd0f00b446f6), UINT64_C(0x1f42fe4ed8b446f3)
    },
    {   UINT64_C(0x0000000000003c65), UINT64_C(0x106af0cb8c7b864b), UINT64_C(0x106af0cc287b864b),
        UINT64_C(0x106ae44c147ba64b), UINT64_C(0x106afc8fa078c651)
    },
    {   UINT64_C(0x0000000000006365), UINT64_C(0x1f42fc0eecb106f6), UINT64_C(0x1f42fb4f48b4671d),
        UINT64_C(0x1f42f74ef4af86ee), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c66), UINT64_C(0x106afc8fa078c651), UINT64_C(0x106b0ecfa077e651),
        UINT64_C(0x106af74f7c7b2596), UINT64_C(0x106af64f807ba59f)
    },
    {   UINT64_C(0x0000000000006366), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f64b66724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ee4b64711)
    },
    {   UINT64_C(0x0000000000008a66), UINT64_C(0x0f42f38c4c6406b7), UINT64_C(0x069afa8d3452a67d),
        UINT64_C(0x0ffb1a4d3068467e), UINT64_C(0x156af88e8091460b)
    },
    {   UINT64_C(0x0000000000006367), UINT64_C(0x1f42f7cef4b66711), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ceecb28701)
    },
    {   UINT64_C(0x0000000000008a67), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x156afa8fa08dc630),
        UINT64_C(0x156af0cfa092e60b), UINT64_C(0x156af40fa08ee574)
    },
    {   UINT64_C(0x0000000000006368), UINT64_C(0x1f42f94f0cb34706), UINT64_C(0x1f42f94eecb56711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74f34b10719)
    },
    {   UINT64_C(0x0000000000008a68), UINT64_C(0x156afc4f6c9045ed), UINT64_C(0x156b014fd08a6649),
        UINT64_C(0x156af88fa091460b), UINT64_C(0x156af88fa091460b)
    },
    {   UINT64_C(0x0000000000006369), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4ec8b40712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ef8b2a70d)
    },
    {   UINT64_C(0x0000000000008a69), UINT64_C(0x156af3cc808ee60b), UINT64_C(0x156af88c8091460b),
        UINT64_C(0x1c22f80fa0aa060b), UINT64_C(0x1c22f80fa0aa060b)
    },
    {   UINT64_C(0x000000000000636a), UINT64_C(0x1f42f34f60b38715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84f30afe713), UINT64_C(0x1f42fd0ed4b6c704)
    },
    {   UINT64_C(0x0000000000008a6a), UINT64_C(0x156af54e808fa626), UINT64_C(0x156af70e80908626),
        UINT64_C(0x156af54e908fa62a), UINT64_C(0x156af70e9090862a)
    },
    {   UINT64_C(0x000000000000636b), UINT64_C(0x1f42f64f18b28715), UINT64_C(0x1f42fa0ef4b166fe),
        UINT64_C(0x1f42fd0edcb5c709), UINT64_C(0x1f42f84ed0b4e706)
    },
    {   UINT64_C(0x0000000000008a6b), UINT64_C(0x1612f68eb893861a), UINT64_C(0x1612f48eb892861a),
        UINT64_C(0x1612f68eb893861a), UINT64_C(0x1622f48eb892861a)
    },
    {   UINT64_C(0x000000000000c82b), UINT64_C(0x0fa2f54f707a4688), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c6d), UINT64_C(0x106ace4b289006de), UINT64_C(0x106aba89a899e73e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000636d), UINT64_C(0x1f42f65084b7a6b6), UINT64_C(0x1f43014ed8ab06f8),
        UINT64_C(0x1f43000ebcb186b4), UINT64_C(0x1f42ffcf84b66726)
    },
    {   UINT64_C(0x0000000000008a6d), UINT64_C(0x17d2fa8e64978728), UINT64_C(0x1c72f40dc4ad4657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x000000000000636e), UINT64_C(0x1f43118ec0af2703), UINT64_C(0x1f4300cf84b3c6e1),
        UINT64_C(0x1f42f8cfc8b44713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a6e), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x000000000000636f), UINT64_C(0x1f4301cf38b066b8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd50a4b5c6f8), UINT64_C(0x1f42fe0fecb806bb)
    },
    {   UINT64_C(0x0000000000008a6f), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8cd4a1875a),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000006370), UINT64_C(0x0000000000000000), UINT64_C(0x1f43001030b5a6ba),
        UINT64_C(0x1f43014f9cb6870d), UINT64_C(0x1f43029088b726d8)
    },
    {   UINT64_C(0x000000000000c834), UINT64_C(0x0002789994404736), UINT64_C(0x0002789994404736),
        UINT64_C(0x0002a38d3022a67e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006371), UINT64_C(0x1f4300cfa0b3c6f0), UINT64_C(0x1f42fc0eacb086f3),
        UINT64_C(0x1f42ff1038ad86fa), UINT64_C(0x1f42fe1028b566ca)
    },
    {   UINT64_C(0x000000000000c836), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x00026e1804432682)
    },
    {   UINT64_C(0x000000000000c837), UINT64_C(0x00025158044f0682), UINT64_C(0x000231cba85e86e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c838), UINT64_C(0x0000000000000000), UINT64_C(0x00026d5790486736),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006373), UINT64_C(0x1f42f9cfd8b7a700), UINT64_C(0x1f42ff4fb8b36701),
        UINT64_C(0x1f42fecfc4ae46fa), UINT64_C(0x1f42fc4f70b32705)
    },
    {   UINT64_C(0x0000000000008a73), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x246af1cda4ce4713),
        UINT64_C(0x1862ff8c9c9746a2), UINT64_C(0x1acafa0f3c9fa5fb)
    },
    {   UINT64_C(0x000000000000c83b), UINT64_C(0x00027889c4404342), UINT64_C(0x00027889c4404342),
        UINT64_C(0x00027389cc3dc3c5), UINT64_C(0x00027889cc4043c5)
    },
    {   UINT64_C(0x000000000000c83c), UINT64_C(0x00028dd81c302735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006375), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f2cab46fd),
        UINT64_C(0x1f43019020ae06be), UINT64_C(0x1f42ff0f04b406f6)
    },
    {   UINT64_C(0x00000000000191c4), UINT64_C(0x0e62f58f7476259f), UINT64_C(0x14d2fa8cc48d65a1),
        UINT64_C(0x129af58c788f25a1), UINT64_C(0x14aaf58cc49765a1)
    },
    {   UINT64_C(0x0000000000006376), UINT64_C(0x1f43020f34ae8701), UINT64_C(0x1f43021090b3c6b1),
        UINT64_C(0x1f42f78f80b6c6bc), UINT64_C(0x1f42fa8f74b80715)
    },
    {   UINT64_C(0x000000000000c840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16baf94f5492869b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c841), UINT64_C(0x0000000000000000), UINT64_C(0x1b4af78f70a6e698),
        UINT64_C(0x0000000000000000), UINT64_C(0x15dafe4f5491469b)
    },
    {   UINT64_C(0x0000000000008a77), UINT64_C(0x1cf2ff8fa0a94596), UINT64_C(0x1cf2ff8fa0a94675),
        UINT64_C(0x1cf2ff8caca9469f), UINT64_C(0x23cafa913cc7657b)
    },
    {   UINT64_C(0x0000000000006378), UINT64_C(0x1f42fdcf2cb24704), UINT64_C(0x1f42fc4fa0b406fe),
        UINT64_C(0x1f42ffcf40a9a707), UINT64_C(0x1f42f6cf8cb7470e)
    },
    {   UINT64_C(0x000000000000c844), UINT64_C(0x0000000000000000), UINT64_C(0x18b2fb8f3c9a66a5),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f8cf688d86a3)
    },
    {   UINT64_C(0x0000000000006379), UINT64_C(0x1f42fecf04b746f8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fece7cb72733), UINT64_C(0x1f42fbcf70b826d6)
    },
    {   UINT64_C(0x000000000000c846), UINT64_C(0x1a92fbcc34a54605), UINT64_C(0x15e300cfa092468c),
        UINT64_C(0x18eaf94ea09be69b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c847), UINT64_C(0x1ba2fa8f3ca7a6a5), UINT64_C(0x187b00cfa09d468c),
        UINT64_C(0x1692f48e0c99468c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c848), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f413dcb8a68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c849), UINT64_C(0x184afb0fa099669b), UINT64_C(0x0e02f34c6c73868c),
        UINT64_C(0x17b2f6cfa098e69c), UINT64_C(0x17b2f6cf5898e68a)
    },
    {   UINT64_C(0x000000000000c84a), UINT64_C(0x179af90c949666a5), UINT64_C(0x09c2f04fa05d05c3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692f04fa094a780)
    },
    {   UINT64_C(0x000000000000c84b), UINT64_C(0x0000000000000000), UINT64_C(0x182afb8f5499e6a5),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c84c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1342fa0fa082c68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c84d), UINT64_C(0x0000000000000000), UINT64_C(0x17f3010f1894c69e),
        UINT64_C(0x15fafd0f1890e69e), UINT64_C(0x1192ffcf187c269e)
    },
    {   UINT64_C(0x000000000000c84e), UINT64_C(0x12baf80f1884669e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c84f), UINT64_C(0x1922fbcf749d65b5), UINT64_C(0x1432f3cc748c85ce),
        UINT64_C(0x0f8b00cfa08ee68c), UINT64_C(0x180300cfa095268c)
    },
    {   UINT64_C(0x000000000000c850), UINT64_C(0x1e82f70b6cb586ed), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000637f), UINT64_C(0x1f42f60f4cad26f7), UINT64_C(0x1f42fb8fb0b286d3),
        UINT64_C(0x1f43094e98afc717), UINT64_C(0x1f42fa8f94b2c70e)
    },
    {   UINT64_C(0x000000000000c852), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1a0afd4f5ca1069a)
    },
    {   UINT64_C(0x0000000000006380), UINT64_C(0x1f42fbcfc0b386de), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cf38b666c7), UINT64_C(0x1f43000e7cae2706)
    },
    {   UINT64_C(0x000000000000c854), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772ec4f709de698)
    },
    {   UINT64_C(0x000000000000c855), UINT64_C(0x1772f50f709de698), UINT64_C(0x1772ec4f709f6698),
        UINT64_C(0x1772f50f709f6698), UINT64_C(0x1772f3cf709f6698)
    },
    {   UINT64_C(0x000000000000c856), UINT64_C(0x1772f38f709f6698), UINT64_C(0x1772ec4f709d4698),
        UINT64_C(0x1772fa4f709d4698), UINT64_C(0x1772ec4f709f6698)
    },
    {   UINT64_C(0x0000000000006382), UINT64_C(0x1f42facee8b54703), UINT64_C(0x1f42facf14b7a732),
        UINT64_C(0x1f43000f58ab26e9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a82), UINT64_C(0x24e302cb94c7a6e3), UINT64_C(0x1feafa8b94b7e6e3),
        UINT64_C(0x1c92fc8b94a986e3), UINT64_C(0x1592f64aa090e720)
    },
    {   UINT64_C(0x0000000000006383), UINT64_C(0x1f43048ea8aee715), UINT64_C(0x1f42f90e64b08716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf10b86711)
    },
    {   UINT64_C(0x0000000000008a83), UINT64_C(0x2502f60aa0e90720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x0000000000006384), UINT64_C(0x1f42fb4eecb3873d), UINT64_C(0x1f42f70f50b46729),
        UINT64_C(0x1f42fe4f4cb166e9), UINT64_C(0x1f43000f68b086ef)
    },
    {   UINT64_C(0x0000000000008a84), UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x103af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x0000000000006385), UINT64_C(0x1f42f9ce6cb4a722), UINT64_C(0x1f42fc0f08b56713),
        UINT64_C(0x1f42f9cf00b22704), UINT64_C(0x1f42ff0f3cb6a6fa)
    },
    {   UINT64_C(0x0000000000008a85), UINT64_C(0x13a2f60aa0948720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x108af60aa08e6720), UINT64_C(0x1712f60aa0a4c720)
    },
    {   UINT64_C(0x0000000000006386), UINT64_C(0x1f43010f08ad670d), UINT64_C(0x1f42fc8f18b7a6b8),
        UINT64_C(0x1f4301cf7cb7a6ef), UINT64_C(0x1f42fccfacafc6fa)
    },
    {   UINT64_C(0x0000000000008a86), UINT64_C(0x10daf60aa08e6720), UINT64_C(0x117af60aa08e6720),
        UINT64_C(0x117af60aa08e6720), UINT64_C(0x0e5af60aa08e679b)
    },
    {   UINT64_C(0x0000000000006387), UINT64_C(0x1f43010f7cb6c71e), UINT64_C(0x1f42f50f14b726f7),
        UINT64_C(0x1f42f74fe0b366eb), UINT64_C(0x1f42f60fe0b2c6d6)
    },
    {   UINT64_C(0x0000000000008a87), UINT64_C(0x0e5af608b48e6720), UINT64_C(0x27b3054bb0d1a6ea),
        UINT64_C(0x1fd2f94b98b206e4), UINT64_C(0x0fbb098b1471c61e)
    },
    {   UINT64_C(0x0000000000006388), UINT64_C(0x1f42f88fd0b56702), UINT64_C(0x1f42face50b6a717),
        UINT64_C(0x1f42f8cea8af8706), UINT64_C(0x1f42fccf3cb08709)
    },
    {   UINT64_C(0x000000000000c864), UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1243058cf87bc68c),
        UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1243058cf87bc68c)
    },
    {   UINT64_C(0x000000000000c865), UINT64_C(0x1e8af5907cb505ab), UINT64_C(0x1243058c087bc6c8),
        UINT64_C(0x1e8aff0cf8b0668c), UINT64_C(0x1e8aff0cf8b0668c)
    },
    {   UINT64_C(0x0000000000008a89), UINT64_C(0x156afa0efc8e06a8), UINT64_C(0x156afa0bf48e060b),
        UINT64_C(0x156afa0c888e060b), UINT64_C(0x156af84c5892a60b)
    },
    {   UINT64_C(0x000000000000638a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ebcb8270f),
        UINT64_C(0x1f42ff0f34ad46fa), UINT64_C(0x1f42fc9044b3471a)
    },
    {   UINT64_C(0x0000000000008a8a), UINT64_C(0x156afa0efc8e060b), UINT64_C(0x156afa13108e2614),
        UINT64_C(0x156afa0f8c8e2506), UINT64_C(0x156afa0f8c8e25e7)
    },
    {   UINT64_C(0x000000000000638b), UINT64_C(0x1f4301cf8ca9c6fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4fa8aca702), UINT64_C(0x1f43000ec8b386e4)
    },
    {   UINT64_C(0x00000000000059de), UINT64_C(0x1f42fb8eecb5e704), UINT64_C(0x1f42fb8f18b42700),
        UINT64_C(0x1f42f98ef8b72704), UINT64_C(0x1f42fb8f08b18704)
    },
    {   UINT64_C(0x000000000000638c), UINT64_C(0x1f42fa4f00b8e72a), UINT64_C(0x1f42f40f6cb76731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcee0b8071b)
    },
    {   UINT64_C(0x000000000000c86c), UINT64_C(0x1242f94cf87d6696), UINT64_C(0x1243024cf881e696),
        UINT64_C(0x1242f94cf87d6696), UINT64_C(0x1243024cf881e696)
    },
    {   UINT64_C(0x000000000000638d), UINT64_C(0x1f42fcce94ab2733), UINT64_C(0x1f42fccfccb10710),
        UINT64_C(0x1f42fa4f84b566d8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a8d), UINT64_C(0x1512fa8dc48c8657), UINT64_C(0x1512fa8dc48c8657),
        UINT64_C(0x1a52fa8dc4a18712), UINT64_C(0x2262fa8bc0c1c6d8)
    },
    {   UINT64_C(0x000000000000638e), UINT64_C(0x1f42fc4f4cb6470a), UINT64_C(0x1f42f60ea8b3271f),
        UINT64_C(0x1f43014ff4b5270d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c870), UINT64_C(0x1e8afe92e4b045dd), UINT64_C(0x1e8afe8fb4b04511),
        UINT64_C(0x114329cd187aa69e), UINT64_C(0x1142ffcd1865a69e)
    },
    {   UINT64_C(0x000000000000c871), UINT64_C(0x1e8aff0dd8b02652), UINT64_C(0x1b02fe8d0ca2468c),
        UINT64_C(0x1e8aff0dd8b02652), UINT64_C(0x1e8aff0dd8b02652)
    },
    {   UINT64_C(0x000000000000c872), UINT64_C(0x1b02fe8d0ca2468c), UINT64_C(0x1e8aff0dd8b02652),
        UINT64_C(0x1b02fe8d0ca2468c), UINT64_C(0x1e8afe8f4cb04623)
    },
    {   UINT64_C(0x0000000000006390), UINT64_C(0x1f42fa4f28b82716), UINT64_C(0x1f42fa0fe0b1670e),
        UINT64_C(0x1f42fa8e8cb88715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c874), UINT64_C(0x1e8aff0ea0b02622), UINT64_C(0x1652f80cf892c68c),
        UINT64_C(0x1e8aff0ea0b02622), UINT64_C(0x1652f80cf892c68c)
    },
    {   UINT64_C(0x0000000000006391), UINT64_C(0x1f42fb8e8cb5072c), UINT64_C(0x1f42ff4f50b5e710),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43064e80b6873e)
    },
    {   UINT64_C(0x000000000000c876), UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x1cf2fb8d30ab867e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006392), UINT64_C(0x1f42ff0fbcb0e6eb), UINT64_C(0x1f42f74fd0b2e725),
        UINT64_C(0x1f42f78f80b0671b), UINT64_C(0x1f42f7ced0ab4702)
    },
    {   UINT64_C(0x00000000000059e5), UINT64_C(0x1f42f88f38b5c70c), UINT64_C(0x1f42f98f48b7671e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf18b7a723)
    },
    {   UINT64_C(0x0000000000006393), UINT64_C(0x1f42f9cefcb74721), UINT64_C(0x1f42fd4ec4b586ce),
        UINT64_C(0x1f42fecef8ae8700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c87a), UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e),
        UINT64_C(0x156af94c7c8ea6a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006394), UINT64_C(0x1f4300ce98b946ef), UINT64_C(0x1f42ff4fbcb7e6ef),
        UINT64_C(0x1f43014f6caf470f), UINT64_C(0x1f4301cf40afc6d9)
    },
    {   UINT64_C(0x0000000000008a94), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f5cf2c8826ae),
        UINT64_C(0x1362f5cf2c8826ae), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x000000000000c87d), UINT64_C(0x0000000000000000), UINT64_C(0x1b02fe8d0ca2468c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a95), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1862f60f2c9c0600),
        UINT64_C(0x1862f60f2c9c0600), UINT64_C(0x1613088f2c93e600)
    },
    {   UINT64_C(0x0000000000006396), UINT64_C(0x0000000000000000), UINT64_C(0x1f43080e38ada734),
        UINT64_C(0x1f43018f70b486fa), UINT64_C(0x1f43000e6cb1c6fd)
    },
    {   UINT64_C(0x000000000000c880), UINT64_C(0x12baef4fa088c68c), UINT64_C(0x10dafdcf2c7a06f9),
        UINT64_C(0x117af90f78846696), UINT64_C(0x12bb010fa07fe68c)
    },
    {   UINT64_C(0x000000000000c881), UINT64_C(0x12bb010da47fe70b), UINT64_C(0x189afa8e389aa63a),
        UINT64_C(0x154af3cfa090c68c), UINT64_C(0x154af3cfa090c68c)
    },
    {   UINT64_C(0x000000000000c882), UINT64_C(0x1773028f6c92060b), UINT64_C(0x1773028d58920690),
        UINT64_C(0x0fa2fd102c7585db), UINT64_C(0x1773028f6c92060b)
    },
    {   UINT64_C(0x0000000000006398), UINT64_C(0x1f42f78f80b3270e), UINT64_C(0x1f42fbcf58b006e7),
        UINT64_C(0x1f42f90ee8ae4715), UINT64_C(0x1f42f64f14aec705)
    },
    {   UINT64_C(0x000000000000c884), UINT64_C(0x1eb2f50b94b5c6e3), UINT64_C(0x1c92f80b94aba6e3),
        UINT64_C(0x1772fe92e4942511), UINT64_C(0x1812fe4f70942679)
    },
    {   UINT64_C(0x000000000000c885), UINT64_C(0x1772fe4ebc942701), UINT64_C(0x1243018c9c7dc6c4),
        UINT64_C(0x0e22f4511473e583), UINT64_C(0x124b0290287d65de)
    },
    {   UINT64_C(0x000000000000c886), UINT64_C(0x0c52f8111c6ac581), UINT64_C(0x0f6b03510c718586),
        UINT64_C(0x1e2b0e8b94bd879a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c887), UINT64_C(0x0000000000000000), UINT64_C(0x1823020fa09505aa),
        UINT64_C(0x1e23020fa0ad05aa), UINT64_C(0x156afa8fa08dc630)
    },
    {   UINT64_C(0x000000000000c888), UINT64_C(0x18c2fa8fa0970621), UINT64_C(0x18c2fa8e78970621),
        UINT64_C(0x15baf4cf3891e5fe), UINT64_C(0x0903070cac56069a)
    },
    {   UINT64_C(0x000000000000c889), UINT64_C(0x0cdaf3ccac6f069a), UINT64_C(0x0f3b070cac6ee69a),
        UINT64_C(0x134af3ccac88c69a), UINT64_C(0x1402fb0f30880600)
    },
    {   UINT64_C(0x000000000000c88a), UINT64_C(0x1402fb0f30880600), UINT64_C(0x1452fe8f1c878605),
        UINT64_C(0x1452fe8f1c878605), UINT64_C(0x114af60aa09a2720)
    },
    {   UINT64_C(0x000000000000c88b), UINT64_C(0x1afaf60aa0c0e720), UINT64_C(0x24b2f60aa0e7c720),
        UINT64_C(0x127af58aa099e720), UINT64_C(0x1c3af60aa0c0e720)
    },
    {   UINT64_C(0x000000000000c88c), UINT64_C(0x25f2f60aa0e7c720), UINT64_C(0x128af60aa09a2720),
        UINT64_C(0x128af60aa09a2720), UINT64_C(0x128af60aa09a2720)
    },
    {   UINT64_C(0x000000000000c88d), UINT64_C(0x1772fe8efc942627), UINT64_C(0x1772fe8efc942627),
        UINT64_C(0x11531d8efc6c0627), UINT64_C(0x1772ff0efc93e627)
    },
    {   UINT64_C(0x000000000000c88e), UINT64_C(0x1772fe92e4942607), UINT64_C(0x1772fe8f0c944607),
        UINT64_C(0x1772fe8efc942627), UINT64_C(0x1772feceb493c61d)
    },
    {   UINT64_C(0x000000000000639e), UINT64_C(0x1f42f90e2cb6073f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0ef4b7e739), UINT64_C(0x1f42ff4ef0b7670d)
    },
    {   UINT64_C(0x0000000000008a9e), UINT64_C(0x156af790108f66a7), UINT64_C(0x156af90f388aa5fc),
        UINT64_C(0x156b010f388ea5fc), UINT64_C(0x156af90f388aa70e)
    },
    {   UINT64_C(0x000000000000639f), UINT64_C(0x1f43028f60b3e730), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303ce9cacc711)
    },
    {   UINT64_C(0x000000000000c892), UINT64_C(0x1772ff100893e5e4), UINT64_C(0x1772ff0ee093e62f),
        UINT64_C(0x1772ff0e7493e620), UINT64_C(0x1772ff0dec93e64d)
    },
    {   UINT64_C(0x00000000000063a0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64edcb64708),
        UINT64_C(0x1f42fe4f04b48719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c894), UINT64_C(0x1772ff114c93c683), UINT64_C(0x1772ff0d1c93c683),
        UINT64_C(0x1772ff0d1c93c683), UINT64_C(0x1772ff0d1c93c683)
    },
    {   UINT64_C(0x00000000000063a1), UINT64_C(0x1f43054e14b10740), UINT64_C(0x1f42fcced0b006f4),
        UINT64_C(0x1f42f74e54b9073c), UINT64_C(0x1f42fd0ee0af0718)
    },
    {   UINT64_C(0x000000000000c896), UINT64_C(0x1772ff114c93c6e1), UINT64_C(0x1772ff114c93c72c),
        UINT64_C(0x1772ff114c93c72c), UINT64_C(0x1772ff114c93c731)
    },
    {   UINT64_C(0x00000000000063a2), UINT64_C(0x1f42fe0f64b6a6fc), UINT64_C(0x1f43080e58aa8741),
        UINT64_C(0x1f42fe0e8cb0872a), UINT64_C(0x1f42ff0e68b6473a)
    },
    {   UINT64_C(0x000000000000c898), UINT64_C(0x1772ff0d3093c67e), UINT64_C(0x1772ff100c93c5e3),
        UINT64_C(0x1772ff0d3093c698), UINT64_C(0x1772ff0ebc93c638)
    },
    {   UINT64_C(0x00000000000063a3), UINT64_C(0x1f4301cec8b12720), UINT64_C(0x1f42fd0e8cb14708),
        UINT64_C(0x1f42ff0f78b126f6), UINT64_C(0x1f43014f20ac671f)
    },
    {   UINT64_C(0x000000000000c89a), UINT64_C(0x1773020b089246c1), UINT64_C(0x1773020b089246c1),
        UINT64_C(0x1f42f78f40b6a5fe), UINT64_C(0x1f42f78f40b6a5fe)
    },
    {   UINT64_C(0x000000000000c89b), UINT64_C(0x0d0af98c9c6ce6c4), UINT64_C(0x1772ff0e4493e637),
        UINT64_C(0x1773020dd4924659), UINT64_C(0x1773020dd4924659)
    },
    {   UINT64_C(0x000000000000c89c), UINT64_C(0x1773020bf49246b5), UINT64_C(0x1773020bf49246b5),
        UINT64_C(0x1772feccc893c69a), UINT64_C(0x1772feccc893c69a)
    },
    {   UINT64_C(0x00000000000063a5), UINT64_C(0x1f42f94f6cb1c730), UINT64_C(0x1f42f94eecb7e71d),
        UINT64_C(0x1f42fc0e9cb06705), UINT64_C(0x1f43094e64b1871b)
    },
    {   UINT64_C(0x000000000000c89e), UINT64_C(0x1773020ab0924720), UINT64_C(0x1773020ab0924720),
        UINT64_C(0x1773020f449245fb), UINT64_C(0x1773020f449245fb)
    },
    {   UINT64_C(0x00000000000063a6), UINT64_C(0x1f42f60f20b44702), UINT64_C(0x1f43024f4cb1470c),
        UINT64_C(0x1f42f58f10b846f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003ca7), UINT64_C(0x106b16550c6be650), UINT64_C(0x106b09572c72666c),
        UINT64_C(0x106b12173067c671), UINT64_C(0x106b039730750671)
    },
    {   UINT64_C(0x000000000000c8a1), UINT64_C(0x1773028d58920690), UINT64_C(0x1773028d58920690),
        UINT64_C(0x1773028e0c92066f), UINT64_C(0x1773028e0c92066f)
    },
    {   UINT64_C(0x0000000000003ca8), UINT64_C(0x106b039730792671), UINT64_C(0x106af58f747c064b),
        UINT64_C(0x106aee0f747ea64b), UINT64_C(0x106ae00fa079064b)
    },
    {   UINT64_C(0x000000000000c8a3), UINT64_C(0x1452fe8f1c878605), UINT64_C(0x1452fe8f1c878605),
        UINT64_C(0x1452fe8f1c878605), UINT64_C(0x177303cf349165fc)
    },
    {   UINT64_C(0x0000000000008aa8), UINT64_C(0x156afa0b748e26ee), UINT64_C(0x156af90f388aa5fc),
        UINT64_C(0x156b010f388ea5fc), UINT64_C(0x1d2af44e04b00649)
    },
    {   UINT64_C(0x000000000000c8a5), UINT64_C(0x1452fe8f1c878605), UINT64_C(0x1b02fa8d98a44662),
        UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x1b02fa8d98a44662)
    },
    {   UINT64_C(0x0000000000003caa), UINT64_C(0x106af08d087ec64b), UINT64_C(0x106afb0f8c7d2597),
        UINT64_C(0x106aefcf8c7f2596), UINT64_C(0x106aeb8fa079664b)
    },
    {   UINT64_C(0x00000000000063aa), UINT64_C(0x1f42f78ea0b4272d), UINT64_C(0x1f42f78e6cb7e72c),
        UINT64_C(0x1f42fe0e58af673a), UINT64_C(0x1f430e0e6cab6737)
    },
    {   UINT64_C(0x000000000000c8a8), UINT64_C(0x1c72f94d28aaa67d), UINT64_C(0x1c72f94d28aaa67d),
        UINT64_C(0x177304cfa090e68c), UINT64_C(0x177304cfa090e68c)
    },
    {   UINT64_C(0x000000000000c8a9), UINT64_C(0x177304cfa090e68c), UINT64_C(0x177304cfa090e68c),
        UINT64_C(0x115b04cfa078868c), UINT64_C(0x115b04cfa078868c)
    },
    {   UINT64_C(0x000000000000c8aa), UINT64_C(0x177304cfa090e68c), UINT64_C(0x1e6304cfa0acc68c),
        UINT64_C(0x1d8304cfa0a9868c), UINT64_C(0x1d8304cfa0a9868c)
    },
    {   UINT64_C(0x000000000000c8ab), UINT64_C(0x1c8af34fa0a5468c), UINT64_C(0x1c82f34fa0a5468c),
        UINT64_C(0x2242f34fa0be868c), UINT64_C(0x2282f34fa0bd868c)
    },
    {   UINT64_C(0x000000000000c8ac), UINT64_C(0x1773068f08900607), UINT64_C(0x1773068f08900607),
        UINT64_C(0x1773024f409225fe), UINT64_C(0x1773024f409225fe)
    },
    {   UINT64_C(0x000000000000c8ad), UINT64_C(0x1773024ddc922655), UINT64_C(0x1773024ddc922655),
        UINT64_C(0x2602fa9120d0459c), UINT64_C(0x2602fa9120d0459c)
    },
    {   UINT64_C(0x000000000000c8ae), UINT64_C(0x1ba2fa9120a6c59c), UINT64_C(0x1772fe4ebc94261b),
        UINT64_C(0x0f93008c407385b5), UINT64_C(0x1402fb0cac880600)
    },
    {   UINT64_C(0x000000000000c8af), UINT64_C(0x1402fb0f308806a1), UINT64_C(0x1402fb0f308806a1),
        UINT64_C(0x1772fd0fa094c65e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x000000000000c8b0), UINT64_C(0x1e8afb8b94b1e6e3), UINT64_C(0x1e8afb8b94b1e6e3),
        UINT64_C(0x1e0b058b80aae6e8), UINT64_C(0x1e0b058b80aae6e8)
    },
    {   UINT64_C(0x000000000000c8b1), UINT64_C(0x1242f8cfe48225eb), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772fe8ea494263b)
    },
    {   UINT64_C(0x0000000000008aaf), UINT64_C(0x1d2af44f38b005fc), UINT64_C(0x156afd0f388ca5fc),
        UINT64_C(0x156afd0f388ca5fc), UINT64_C(0x156afd0e048ca647)
    },
    {   UINT64_C(0x00000000000063b0), UINT64_C(0x1f42facf04b6070d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8f2cb14719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c8b4), UINT64_C(0x1772fdce94946640), UINT64_C(0x1772fdce94946640),
        UINT64_C(0x1772fe4f1c942605), UINT64_C(0x1772fe4f1c942605)
    },
    {   UINT64_C(0x000000000000c8b5), UINT64_C(0x1452fe8f1c87866b), UINT64_C(0x156afa0cac8e269a),
        UINT64_C(0x1773020f409245fe), UINT64_C(0x1773020f409245fe)
    },
    {   UINT64_C(0x0000000000005a04), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cee4b04707),
        UINT64_C(0x1f42fdcea4af0708), UINT64_C(0x1f42fb8eccb8a70c)
    },
    {   UINT64_C(0x00000000000063b2), UINT64_C(0x1f42f78e88b6e72b), UINT64_C(0x1f42face70b2e735),
        UINT64_C(0x1f43084e7cb06744), UINT64_C(0x1f42f78f38b6071e)
    },
    {   UINT64_C(0x0000000000008ab2), UINT64_C(0x156afd0d108ca681), UINT64_C(0x156afd0b588ca6d2),
        UINT64_C(0x156afd0b588ca6d2), UINT64_C(0x156afd4aa48be6b5)
    },
    {   UINT64_C(0x000000000000c8b9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1772fecc8093c69a), UINT64_C(0x1772fecc8093c69a)
    },
    {   UINT64_C(0x0000000000008ab3), UINT64_C(0x156afe8aa48c86b5), UINT64_C(0x2a42fe0eccdf8616),
        UINT64_C(0x2a42fe0eccdf8616), UINT64_C(0x156afa8f388b65fc)
    },
    {   UINT64_C(0x00000000000063b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8ebcb4c71a),
        UINT64_C(0x1f42f8ce2cb6672a), UINT64_C(0x1f42fd8ebcb7471a)
    },
    {   UINT64_C(0x0000000000008ab4), UINT64_C(0x156aff8f388de5fc), UINT64_C(0x156afd0e048cc649),
        UINT64_C(0x156afd0e048ca649), UINT64_C(0x156afd0c5c8ca6b3)
    },
    {   UINT64_C(0x000000000000c8bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1773028f6c9206c7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008ab5), UINT64_C(0x156afd0c5c8ca6b3), UINT64_C(0x156afd0c5c8ca6b3),
        UINT64_C(0x156afd0c5c8ca6b3), UINT64_C(0x2712fd0f38d345fc)
    },
    {   UINT64_C(0x00000000000063b6), UINT64_C(0x1f42fdceacb3e716), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ce5cb72707)
    },
    {   UINT64_C(0x0000000000008ab6), UINT64_C(0x2712fd0f38d345fc), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0da08606a1), UINT64_C(0x1362fa0e1c86069f)
    },
    {   UINT64_C(0x00000000000063b7), UINT64_C(0x1f42fcceccae2722), UINT64_C(0x1f42fd8ebcb3671e),
        UINT64_C(0x1f42fbcef8b2a716), UINT64_C(0x1f42fb8eccb72726)
    },
    {   UINT64_C(0x000000000000c8c2), UINT64_C(0x0eab170c9c77e6c3), UINT64_C(0x0eaaf08c9c64a6c3),
        UINT64_C(0x0eab170c9c77e6c4), UINT64_C(0x0eaaf08c9c64a6c4)
    },
    {   UINT64_C(0x00000000000063b8), UINT64_C(0x1f42f68ebcb34716), UINT64_C(0x1f42fbcedcb7e712),
        UINT64_C(0x1f42f80ec8b6a71a), UINT64_C(0x1f430acec8ad4715)
    },
    {   UINT64_C(0x000000000000c8c4), UINT64_C(0x1772fe5150942577), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000810b), UINT64_C(0x1322f2cfa086c67e), UINT64_C(0x145af58f688b268c),
        UINT64_C(0x1c32f2cf68aa668c), UINT64_C(0x13eaeecfa08bc67e)
    },
    {   UINT64_C(0x0000000000008ab9), UINT64_C(0x156afd0fa08ca67e), UINT64_C(0x1ae2f98fa0a2e67e),
        UINT64_C(0x156afc4fa08cc67e), UINT64_C(0x156afa0fa08e2623)
    },
    {   UINT64_C(0x00000000000063ba), UINT64_C(0x1f42f98eacb88715), UINT64_C(0x1f42fccedcab8703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f50b5471a)
    },
    {   UINT64_C(0x000000000000c8c8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x2012f751a4ba4562), UINT64_C(0x2012f751a4ba4562)
    },
    {   UINT64_C(0x000000000000810d), UINT64_C(0x0e72f18d7c7565aa), UINT64_C(0x0e92f78f787525b4),
        UINT64_C(0x168af18fa096e5b0), UINT64_C(0x0c32f18f786b25b4)
    },
    {   UINT64_C(0x0000000000008abb), UINT64_C(0x12c2fa0fe88385d1), UINT64_C(0x12c2fa0fe88385d1),
        UINT64_C(0x0ca2f3ccac6e269a), UINT64_C(0x0fb3074d3870a67c)
    },
    {   UINT64_C(0x00000000000063bc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f28b6272a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008abc), UINT64_C(0x0fb3074d3870a67c), UINT64_C(0x0fb3074cb470a69d),
        UINT64_C(0x105af08cac7ea69f), UINT64_C(0x156afa0cac8e269f)
    },
    {   UINT64_C(0x000000000000c8cd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf3034d30a7a6d7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008abd), UINT64_C(0x156b0eccac83e69f), UINT64_C(0x156af10cac92c69f),
        UINT64_C(0x156b33ce08714648), UINT64_C(0x2492fc8d34c5867d)
    },
    {   UINT64_C(0x00000000000063be), UINT64_C(0x1f42fd8f28b54732), UINT64_C(0x1f42f7cef8afc712),
        UINT64_C(0x1f42fe8ebcaae70b), UINT64_C(0x1f42fa8e78b70707)
    },
    {   UINT64_C(0x0000000000008abe), UINT64_C(0x2493048d34c9867d), UINT64_C(0x156afd0c308ca6be),
        UINT64_C(0x156afd0c308ca6be), UINT64_C(0x1bf2f98c28a886ae)
    },
    {   UINT64_C(0x00000000000063bf), UINT64_C(0x1f42f94edcb7a713), UINT64_C(0x1f42f60f24b76725),
        UINT64_C(0x1f42f54f28b4e712), UINT64_C(0x1f42facf18af8703)
    },
    {   UINT64_C(0x0000000000008abf), UINT64_C(0x1473058b948486e3), UINT64_C(0x162af98c289166ae),
        UINT64_C(0x0cd3070cac65469a), UINT64_C(0x0ef3058b946e86e3)
    },
    {   UINT64_C(0x00000000000063c0), UINT64_C(0x1f42fa8eccb2470e), UINT64_C(0x1f42fa8f1caf26f7),
        UINT64_C(0x1f43080eacb5c722), UINT64_C(0x1f42f94f18b6c71e)
    },
    {   UINT64_C(0x0000000000005a13), UINT64_C(0x1f430c8e40b34738), UINT64_C(0x1f42f70e94b8c729),
        UINT64_C(0x1f42fccf78b42727), UINT64_C(0x1f42fdce7cb22707)
    },
    {   UINT64_C(0x00000000000063c1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f28b8270e),
        UINT64_C(0x1f42fa8f48ab6707), UINT64_C(0x1f42faceccb6471a)
    },
    {   UINT64_C(0x0000000000005a14), UINT64_C(0x1f42f8cf08af471f), UINT64_C(0x1f43094e58b48738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98eecb2c733)
    },
    {   UINT64_C(0x00000000000063c2), UINT64_C(0x1f42fbced0b3670b), UINT64_C(0x1f43080eb0b22717),
        UINT64_C(0x1f42f68f18b1c723), UINT64_C(0x1f4305cea8b5472c)
    },
    {   UINT64_C(0x0000000000005a15), UINT64_C(0x1f42f7cf68ae2726), UINT64_C(0x1f42f98f68b1c710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063c3), UINT64_C(0x1f42f74e80b6c721), UINT64_C(0x1f42f88f08b7a71e),
        UINT64_C(0x1f42fc8eecaa671a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a16), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64e9cb76727),
        UINT64_C(0x1f42fa4eb0b56726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063c4), UINT64_C(0x1f42f58f08b74722), UINT64_C(0x1f42f8cf08b58722),
        UINT64_C(0x1f42f94f08b86726), UINT64_C(0x1f42f4ceccb8e72d)
    },
    {   UINT64_C(0x0000000000008ac4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x00000000000063c5), UINT64_C(0x1f42f84edcafe730), UINT64_C(0x1f42f58eccb5c722),
        UINT64_C(0x1f42fd8f2cae070b), UINT64_C(0x1f42fbcf68b7671e)
    },
    {   UINT64_C(0x0000000000008ac5), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0)
    },
    {   UINT64_C(0x00000000000063c6), UINT64_C(0x1f42fd8f38b4a70b), UINT64_C(0x1f42f5cf2cb7a72f),
        UINT64_C(0x1f42fe4f38b18712), UINT64_C(0x1f42f78f08b4272e)
    },
    {   UINT64_C(0x0000000000008ac6), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x2082f80cd0bb86f4)
    },
    {   UINT64_C(0x00000000000063c7), UINT64_C(0x1f42f6ceacb2871e), UINT64_C(0x1f42f6ceacb52702),
        UINT64_C(0x1f42f1ce70b24726), UINT64_C(0x1f42f7ceccb92707)
    },
    {   UINT64_C(0x0000000000008ac7), UINT64_C(0x2082f80cd0bb86f4), UINT64_C(0x0782fa9250564534),
        UINT64_C(0x0782fa9250564534), UINT64_C(0x1c72f70dfcabc69a)
    },
    {   UINT64_C(0x00000000000063c8), UINT64_C(0x1f42fbceecb5c716), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf38b0470e)
    },
    {   UINT64_C(0x0000000000008ac8), UINT64_C(0x1c72f70dfcabc69a), UINT64_C(0x1c73028cd8a60690),
        UINT64_C(0x1c73028cd8a60690), UINT64_C(0x1cf2f40da4af4661)
    },
    {   UINT64_C(0x000000001f413ec0), UINT64_C(0x107ac94c6492a68e), UINT64_C(0x0fa2cacc647aa691),
        UINT64_C(0x0fa2cacc707f4692), UINT64_C(0x1742c94c64992691)
    },
    {   UINT64_C(0x0000000000013ec1), UINT64_C(0x174ac94c649de692), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063ca), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088e3cb16727),
        UINT64_C(0x1f42f7ceecb9a726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008aca), UINT64_C(0x1102f80dfc7d8649), UINT64_C(0x105af48fe07ca5d0),
        UINT64_C(0x0b2af48fe067e5d0), UINT64_C(0x0b2af48fe067e5d0)
    },
    {   UINT64_C(0x00000000000063cb), UINT64_C(0x1f42fa8ef8b1e70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cebcb9c70e), UINT64_C(0x1f42fb8e4cb6e736)
    },
    {   UINT64_C(0x0000000000008acb), UINT64_C(0x1a52fa8fb0a185dc), UINT64_C(0x1a52fa8fb0a185dc),
        UINT64_C(0x12a2fa8dc482c657), UINT64_C(0x12a2fa8dc482c657)
    },
    {   UINT64_C(0x000000000000c8eb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x09daee12e465e511)
    },
    {   UINT64_C(0x0000000000008acc), UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x1cf2fd0c00aac6cb),
        UINT64_C(0x2582fb0eaccde621), UINT64_C(0x3132fbd088fc65a9)
    },
    {   UINT64_C(0x00000000000063cd), UINT64_C(0x1f42fb8ebcb3a71a), UINT64_C(0x1f42f88ea4b1670f),
        UINT64_C(0x1f42f58f08b10713), UINT64_C(0x1f42f8cee8b3e70f)
    },
    {   UINT64_C(0x0000000000008acd), UINT64_C(0x1cf2fc1084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x2c22fbd084e825a9)
    },
    {   UINT64_C(0x00000000000063ce), UINT64_C(0x1f42f6ceacb84739), UINT64_C(0x1f42f94f08b6e719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f48f08b2472e)
    },
    {   UINT64_C(0x0000000000008ace), UINT64_C(0x1cf2fbd084aba5a9), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x00000000000063cf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f4cf04b80735), UINT64_C(0x1f42fececcb44738)
    },
    {   UINT64_C(0x0000000000008acf), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fa9084ac05aa),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x00000000000063d0), UINT64_C(0x1f42f88eacb4a702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0e6cb76741), UINT64_C(0x1f42fb8e8cb88715)
    },
    {   UINT64_C(0x0000000000005a23), UINT64_C(0x1f42fa8f18b4c713), UINT64_C(0x1f42fb0f64b50728),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f3cafe6f6)
    },
    {   UINT64_C(0x00000000000063d1), UINT64_C(0x1f42f6cef8b6e72e), UINT64_C(0x1f42f88e84b80732),
        UINT64_C(0x1f42f8cf78b7e71a), UINT64_C(0x1f42face90b7a713)
    },
    {   UINT64_C(0x0000000000008ad1), UINT64_C(0x1cf2f94f38ab65fc), UINT64_C(0x1d02fbcc94aba6a5),
        UINT64_C(0x1d02fbcc94aba6a5), UINT64_C(0x1cf2fbd084ab65e4)
    },
    {   UINT64_C(0x00000000000063d2), UINT64_C(0x1f42f5ce8cb82727), UINT64_C(0x1f42fdce44b7a738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ceccb9670b)
    },
    {   UINT64_C(0x0000000000008ad2), UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2fbd084ab65e4),
        UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2fbcf98ab65a9)
    },
    {   UINT64_C(0x00000000000063d3), UINT64_C(0x1f42f50e84b82732), UINT64_C(0x1f42f70f48b56712),
        UINT64_C(0x1f42f6cefcb7e70b), UINT64_C(0x1f42fb8ed0b5671a)
    },
    {   UINT64_C(0x0000000000008ad3), UINT64_C(0x1cf2fbcf98ab65a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063d4), UINT64_C(0x1f42fb8f5cb24728), UINT64_C(0x1f42f8cedcb6a70e),
        UINT64_C(0x1f42fd0f10b16729), UINT64_C(0x1f42f78ed4b5670e)
    },
    {   UINT64_C(0x0000000000008ad4), UINT64_C(0x18d2fd0f289a4653), UINT64_C(0x18d2fd0f289a4653),
        UINT64_C(0x15e2f6cf6c91a63d), UINT64_C(0x1b0af94da8a786b0)
    },
    {   UINT64_C(0x00000000000063d5), UINT64_C(0x1f42f4ce8cb5c733), UINT64_C(0x1f42f7ce3cae872d),
        UINT64_C(0x1f42fa8e84b0e728), UINT64_C(0x1f42f7cedcb4a715)
    },
    {   UINT64_C(0x0000000000008ad5), UINT64_C(0x1b0af94da4a786b0), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063d6), UINT64_C(0x1f42f78e74b7671e), UINT64_C(0x1f42f98e8cb2e725),
        UINT64_C(0x1f43030e7cb1a73c), UINT64_C(0x1f42fcced0b5070e)
    },
    {   UINT64_C(0x0000000000005a29), UINT64_C(0x1f42fc8f18b8c708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f8eb0b0672c), UINT64_C(0x1f42f88f38b34717)
    },
    {   UINT64_C(0x00000000000063d7), UINT64_C(0x1f42f68ec8b7471a), UINT64_C(0x1f43030e30b6e71b),
        UINT64_C(0x1f42fd8eccb2a71e), UINT64_C(0x1f42fece8cb3c73d)
    },
    {   UINT64_C(0x0000000000005a2a), UINT64_C(0x1f430c0eecb4c738), UINT64_C(0x1f42fbcf58ad46ec),
        UINT64_C(0x1f42fe8f48ad66e4), UINT64_C(0x1f42fa8f18b326e4)
    },
    {   UINT64_C(0x00000000000063d8), UINT64_C(0x1f42facf38b4070b), UINT64_C(0x1f43084ef0ad4702),
        UINT64_C(0x1f42f6cebcb6a728), UINT64_C(0x1f42f6cf1cb6a71a)
    },
    {   UINT64_C(0x0000000000005a2b), UINT64_C(0x1f42ff8ea4ade711), UINT64_C(0x1f42f7cef4b54708),
        UINT64_C(0x1f43028ed4afc6f8), UINT64_C(0x1f42f7cedcb02731)
    },
    {   UINT64_C(0x00000000000063d9), UINT64_C(0x1f42fb4f28b42711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43064eacaec71e), UINT64_C(0x1f4300cf54b42716)
    },
    {   UINT64_C(0x0000000000005a2c), UINT64_C(0x1f42f9ceb4b74738), UINT64_C(0x1f42fbcf18adc708),
        UINT64_C(0x1f42fccf48b266fc), UINT64_C(0x1f42f7ceecb6a714)
    },
    {   UINT64_C(0x00000000000063da), UINT64_C(0x1f43030f48b3a712), UINT64_C(0x1f43004eacb50716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018f34ae0716)
    },
    {   UINT64_C(0x000000000000c908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f80dc0780407)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a2e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40f08b8a72f),
        UINT64_C(0x1f430a8e54ad6719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063dc), UINT64_C(0x1f43104ee4ae8723), UINT64_C(0x1f42ff4f38b28710),
        UINT64_C(0x1f42ff4f30b30724), UINT64_C(0x1f43014f04a9c70e)
    },
    {   UINT64_C(0x0000000000005a2f), UINT64_C(0x1f42fb4e94b4e737), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063dd), UINT64_C(0x1f43020f94b1871d), UINT64_C(0x1f43010f64abc710),
        UINT64_C(0x1f43024f54b4470f), UINT64_C(0x1f43014f20b26702)
    },
    {   UINT64_C(0x0000000000005a30), UINT64_C(0x1f42fc8eacb5c704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f68b34708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063de), UINT64_C(0x1f42fe4f44b7270a), UINT64_C(0x1f43034f80af0718),
        UINT64_C(0x1f43030f44aae711), UINT64_C(0x1f42ff4f88afc715)
    },
    {   UINT64_C(0x0000000000005a31), UINT64_C(0x1f43064e50b48735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8f68b4a70c), UINT64_C(0x1f42fb8eecb326f3)
    },
    {   UINT64_C(0x00000000000063df), UINT64_C(0x1f42ff8f44b4e701), UINT64_C(0x1f430d0f28ace710),
        UINT64_C(0x1f43034f88b24721), UINT64_C(0x1f43014f38b5c71c)
    },
    {   UINT64_C(0x0000000000005a32), UINT64_C(0x1f42f88e44b1e71d), UINT64_C(0x1f42f9ce6cabc741),
        UINT64_C(0x1f42face68b52739), UINT64_C(0x1f42f7cf04b4e730)
    },
    {   UINT64_C(0x00000000000063e0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f74b54712),
        UINT64_C(0x1f43040f44b82701), UINT64_C(0x1f42fe4f48b5c714)
    },
    {   UINT64_C(0x0000000000005a33), UINT64_C(0x1f42f6ce9cb5c6ff), UINT64_C(0x1f42f6cedcb72714),
        UINT64_C(0x1f42fc8ef8b3470b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063e1), UINT64_C(0x1f42fa0f58b66704), UINT64_C(0x1f43040f3cabc716),
        UINT64_C(0x1f42fdcf08af0711), UINT64_C(0x1f430e8e7cb36727)
    },
    {   UINT64_C(0x0000000000005a34), UINT64_C(0x1f430d4e40b46728), UINT64_C(0x1f42fb4ed8b006ff),
        UINT64_C(0x1f42f6ce4cb92739), UINT64_C(0x1f42f88eacb3671e)
    },
    {   UINT64_C(0x00000000000063e2), UINT64_C(0x1f430f4e6cade720), UINT64_C(0x1f42fe8f08b1e706),
        UINT64_C(0x1f42fe4f48b3a706), UINT64_C(0x1f42fc4fa8ad4730)
    },
    {   UINT64_C(0x000000000000c918), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000063e3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f78b5470d),
        UINT64_C(0x1f43020f48b00705), UINT64_C(0x1f43014f08b6c717)
    },
    {   UINT64_C(0x000000000000c91a), UINT64_C(0x15baee0f5495669a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063e4), UINT64_C(0x1f43010f60af4724), UINT64_C(0x1f42ff8eacb3e720),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0efcb5a707)
    },
    {   UINT64_C(0x0000000000005a37), UINT64_C(0x1f42f88f58b72714), UINT64_C(0x1f42fd8eeca8a6f4),
        UINT64_C(0x1f42f98f28a846fb), UINT64_C(0x1f42f88e6cb8473a)
    },
    {   UINT64_C(0x00000000000063e5), UINT64_C(0x1f430f0f58ada713), UINT64_C(0x1f43028ec0b6671e),
        UINT64_C(0x1f42ff4f18ace714), UINT64_C(0x1f42fc0f98b4a714)
    },
    {   UINT64_C(0x0000000000005a38), UINT64_C(0x1f43124e5cb4e730), UINT64_C(0x1f42f78eccad0704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce98b54742)
    },
    {   UINT64_C(0x00000000000063e6), UINT64_C(0x1f42fb4f58b36716), UINT64_C(0x1f42fc8f70b3e701),
        UINT64_C(0x1f42fe0e8cafe71f), UINT64_C(0x1f43038e6cb46741)
    },
    {   UINT64_C(0x0000000000005a39), UINT64_C(0x1f42fc8ec0b2e728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88eccb5070b), UINT64_C(0x1f42f9ce90b5a724)
    },
    {   UINT64_C(0x00000000000063e7), UINT64_C(0x1f43068e5cad873c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8ed8aac711), UINT64_C(0x1f42f58e3cb28726)
    },
    {   UINT64_C(0x0000000000005a3a), UINT64_C(0x1f4309ce5cb06712), UINT64_C(0x1f42f88e7cac46ff),
        UINT64_C(0x1f42fb8f08b2471f), UINT64_C(0x1f42fb8f08b6c709)
    },
    {   UINT64_C(0x00000000000063e8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ebcb72735)
    },
    {   UINT64_C(0x0000000000005a3b), UINT64_C(0x1f42f8ceacb72713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8eacb7a73b), UINT64_C(0x1f42f6cedcb5a707)
    },
    {   UINT64_C(0x00000000000063e9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facec8b5873b), UINT64_C(0x1f42fa8ec8b5673b)
    },
    {   UINT64_C(0x0000000000005a3c), UINT64_C(0x1f42f6cf38b00717), UINT64_C(0x1f42f6cebcb3e72d),
        UINT64_C(0x1f42f7ceecb72741), UINT64_C(0x1f42f6ceccb866ff)
    },
    {   UINT64_C(0x00000000000063ea), UINT64_C(0x1f42fd0e6cb2e72d), UINT64_C(0x1f4301ce9cb5072c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cedcb20715)
    },
    {   UINT64_C(0x0000000000005a3d), UINT64_C(0x1f42f8d014b686f0), UINT64_C(0x1f42f8cee8b6470b),
        UINT64_C(0x1f42f78e6cb6c70e), UINT64_C(0x1f42f78f58b5a713)
    },
    {   UINT64_C(0x00000000000063eb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43040eb8b2071e),
        UINT64_C(0x1f42ffcedcb20715), UINT64_C(0x1f42f9ce9cb5072c)
    },
    {   UINT64_C(0x0000000000005a3e), UINT64_C(0x1f43094e4cb5c705), UINT64_C(0x1f42f6ce6cb1e735),
        UINT64_C(0x1f42facf68b26727), UINT64_C(0x1f430e8e9cb2673e)
    },
    {   UINT64_C(0x00000000000063ec), UINT64_C(0x1f42f98efcb62702), UINT64_C(0x1f42fc8e34b2a713),
        UINT64_C(0x1f43094e68b40740), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a3f), UINT64_C(0x1f43044ee8b4e714), UINT64_C(0x1f43054ea8b3271f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c92d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c92e), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000063ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf50b0c6f5),
        UINT64_C(0x1f42fe4f60b366ee), UINT64_C(0x1f42fdcf60b3871e)
    },
    {   UINT64_C(0x000000000000c930), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c931), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c932), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000063f0), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cf40b0a6ed),
        UINT64_C(0x1f4305cfd4b5e6e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c934), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c935), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c936), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c937), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c938), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c939), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000c93a), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063f4), UINT64_C(0x1f43080fa0b2c6f6), UINT64_C(0x1f43020f04b386f3),
        UINT64_C(0x1f43014f00b3c70c), UINT64_C(0x1f42fbcf64b42709)
    },
    {   UINT64_C(0x0000000000005a47), UINT64_C(0x1f42f4cf28b8271f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f28ac070c)
    },
    {   UINT64_C(0x00000000000063f5), UINT64_C(0x1f42fd0f1cb4c6df), UINT64_C(0x1f43044f58b106e5),
        UINT64_C(0x1f4306cf34aee6d5), UINT64_C(0x1f4301cf08b186d4)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063f6), UINT64_C(0x1f43034f60b1a6e7), UINT64_C(0x1f43064fb0b22703),
        UINT64_C(0x1f43008f50b26708), UINT64_C(0x1f43010f68b186fa)
    },
    {   UINT64_C(0x000000000000c940), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x15c33c0ae46ea776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063f7), UINT64_C(0x1f43068f74b2e6ef), UINT64_C(0x1f4303cf38ae071c),
        UINT64_C(0x1f43020f34af0720), UINT64_C(0x1f43038f74b346ea)
    },
    {   UINT64_C(0x0000000000005a4a), UINT64_C(0x1f42f64ec0b76717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7cf28b4271f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c943), UINT64_C(0x16233d8ae498853c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c944), UINT64_C(0x1622eb4ae46f453c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c945), UINT64_C(0x16233d93cc988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c946), UINT64_C(0x1622eb53cc6f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c947), UINT64_C(0x16233d8ae4988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a4d), UINT64_C(0x1f42f6cef8b6c70b), UINT64_C(0x1f42f6ce8cb786fc),
        UINT64_C(0x1f42f84f0cb66718), UINT64_C(0x1f42f60eccb6c70f)
    },
    {   UINT64_C(0x000000000000c949), UINT64_C(0x1622eb4ae46f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a4e), UINT64_C(0x1f42f78f28b5a6ff), UINT64_C(0x1f42fc8f58b0272d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70e9cb506ff)
    },
    {   UINT64_C(0x000000000000c94b), UINT64_C(0x1622eb4ae498653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005371), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005372), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005373), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005374), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005375), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005376), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005377), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005378), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x000000000000c954), UINT64_C(0x1622eb5258986599), UINT64_C(0x1623264ae47b0776),
        UINT64_C(0x16233d8ae4988599), UINT64_C(0x1623264ae498853c)
    },
    {   UINT64_C(0x000000000000c955), UINT64_C(0x1623264ae4986599), UINT64_C(0x1622eb4ae46f4599),
        UINT64_C(0x1622eb4ae47ae53c), UINT64_C(0x1622eb4ae47ae599)
    },
    {   UINT64_C(0x000000000000c956), UINT64_C(0x16233d9258988776), UINT64_C(0x16232653cc988776),
        UINT64_C(0x1623265258986776), UINT64_C(0x1622eb52586f4776)
    },
    {   UINT64_C(0x000000000000c957), UINT64_C(0x1622eb53cc7ae776), UINT64_C(0x1622eb52587ae776),
        UINT64_C(0x16233d8ae4988776), UINT64_C(0x1623264ae4988776)
    },
    {   UINT64_C(0x000000000000537d), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x000000000000537e), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x000000000000537f), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x0000000000005380), UINT64_C(0x1f430554a8b2a5c1), UINT64_C(0x1f430f4fe0af86d1),
        UINT64_C(0x1f4307cfd0aec6f6), UINT64_C(0x1f430350a0ac06e3)
    },
    {   UINT64_C(0x0000000000005381), UINT64_C(0x1f42fbd038b4870b), UINT64_C(0x1f42fe8f50b346fd),
        UINT64_C(0x1f430bd23cacc6e0), UINT64_C(0x1f4309cfccb186e3)
    },
    {   UINT64_C(0x0000000000005382), UINT64_C(0x1f430f4f08b76711), UINT64_C(0x1f42ff1120b2e6c0),
        UINT64_C(0x1f43019034b4c6f9), UINT64_C(0x1f43030f54b306eb)
    },
    {   UINT64_C(0x0000000000005383), UINT64_C(0x1f430b8f14b086c1), UINT64_C(0x1f430acf68aca6ce),
        UINT64_C(0x1f43040e64a9a6f1), UINT64_C(0x1f431b4e80a42707)
    },
    {   UINT64_C(0x0000000000005384), UINT64_C(0x1f43120e9ca846ff), UINT64_C(0x1f430790b8b3e6ca),
        UINT64_C(0x1f43070fc8af06d8), UINT64_C(0x1f430aceecac6718)
    },
    {   UINT64_C(0x0000000000005385), UINT64_C(0x1f43009038b526e5), UINT64_C(0x1f4307d090ab66da),
        UINT64_C(0x1f42f69004b746f3), UINT64_C(0x1f42ff9044b2c6f0)
    },
    {   UINT64_C(0x0000000000006407), UINT64_C(0x1f43010f08b3c6f5), UINT64_C(0x1f42fb8ef4b246e9),
        UINT64_C(0x1f43038f50b286f9), UINT64_C(0x1f42fe0eecb1c71e)
    },
    {   UINT64_C(0x0000000000005387), UINT64_C(0x1f43190ee0a82727), UINT64_C(0x1f43105038ac86dd),
        UINT64_C(0x1f43040f90b446f6), UINT64_C(0x1f43008f98b486e4)
    },
    {   UINT64_C(0x0000000000005388), UINT64_C(0x1f43014f0cb1c70e), UINT64_C(0x1f43100f7caa86c1),
        UINT64_C(0x1f43088f98b186fd), UINT64_C(0x1f43094e74ade747)
    },
    {   UINT64_C(0x0000000000005389), UINT64_C(0x1f4316cf84a946cb), UINT64_C(0x1f43008f6ca286fb),
        UINT64_C(0x1f43068fc8b2471e), UINT64_C(0x1f430b4f74b2e715)
    },
    {   UINT64_C(0x000000000000538a), UINT64_C(0x1f43568f3880e703), UINT64_C(0x1f433b4f3094c713),
        UINT64_C(0x1f43078f48b6a701), UINT64_C(0x1f43274f28a20705)
    },
    {   UINT64_C(0x000000000000538b), UINT64_C(0x1f43328f388aa6f8), UINT64_C(0x1f43188e88a7472d),
        UINT64_C(0x1f4301cea8b1e6e0), UINT64_C(0x1f4320cf34a50715)
    },
    {   UINT64_C(0x000000000000538c), UINT64_C(0x1f430d4e7caf2734), UINT64_C(0x1f430f0e7cab4720),
        UINT64_C(0x1f43224e98a46734), UINT64_C(0x1f431b4e40a74736)
    },
    {   UINT64_C(0x000000000000538d), UINT64_C(0x1f430bcf48ab6704), UINT64_C(0x1f43058eb0afc74a),
        UINT64_C(0x1f434a565c8946a5), UINT64_C(0x1f4336d61097263f)
    },
    {   UINT64_C(0x000000000000538e), UINT64_C(0x1f430bcfacb786fc), UINT64_C(0x1f4303ce88b3e6e4),
        UINT64_C(0x1f4306cf6ca54721), UINT64_C(0x1f43074fccb4c70e)
    },
    {   UINT64_C(0x000000000000538f), UINT64_C(0x1f43048e6cb2e72c), UINT64_C(0x1f43138f58aa06c8),
        UINT64_C(0x1f42fe4e8cb80707), UINT64_C(0x1f42f7d01c6566d8)
    },
    {   UINT64_C(0x0000000000005390), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4308d0c8b466e5), UINT64_C(0x1f43020f78a8a6ec)
    },
    {   UINT64_C(0x0000000000005391), UINT64_C(0x0000000000000000), UINT64_C(0x1f430acf98b426f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4fb0b546f4)
    },
    {   UINT64_C(0x0000000000005392), UINT64_C(0x1f43175074a5c6ec), UINT64_C(0x1f430e4fbcb3268a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4fe8b686fc)
    },
    {   UINT64_C(0x0000000000005393), UINT64_C(0x1f43094ec8a7c72d), UINT64_C(0x1f430acec8a5a730),
        UINT64_C(0x1f43058e50b4c72e), UINT64_C(0x1f4305cfb0b56709)
    },
    {   UINT64_C(0x000000000000c96f), UINT64_C(0x0000000000000000), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c970), UINT64_C(0x0000000000000000), UINT64_C(0x209af6cb30b826ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005396), UINT64_C(0x1f430a4e6cb74734), UINT64_C(0x1f4319d018ad26c7),
        UINT64_C(0x1f430d9048abc6cd), UINT64_C(0x1f430d58bcabe6e6)
    },
    {   UINT64_C(0x000000000000c972), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18b2f94b1c9ba703), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005398), UINT64_C(0x1f431e900c9e46e3), UINT64_C(0x1f432e8f2896a708),
        UINT64_C(0x1f42f98fd8b1c6d1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005399), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4313cec8ab6706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000539a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054fd8b1c6db),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000539c), UINT64_C(0x1f43170f88a7a6ca), UINT64_C(0x1f42fd5018b5c704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0ef8b46718)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x07b0b40002690011), UINT64_C(0x1692ec0fa0960675), UINT64_C(0x131aedcfa08dc675),
        UINT64_C(0x131af00fa08f2675), UINT64_C(0x1692fb0f5898c682)
    },
    {   UINT64_C(0x000000000000539f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ef8b50721), UINT64_C(0x1f4309cf0caf0710)
    },
    {   UINT64_C(0x000000000000c97b), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053a1), UINT64_C(0x1f43010f44b28716), UINT64_C(0x1f43534f0c88e6f4),
        UINT64_C(0x1f43224f78a026f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053a2), UINT64_C(0x1f4301ce9cb2271b), UINT64_C(0x1f43058eb8b34736),
        UINT64_C(0x1f430c0e48b06733), UINT64_C(0x1f430c4e44afc733)
    },
    {   UINT64_C(0x00000000000053a3), UINT64_C(0x1f42fe9204b4a672), UINT64_C(0x1f430a4e84af26e9),
        UINT64_C(0x1f43038f10b286fd), UINT64_C(0x1f43078f4caf06c2)
    },
    {   UINT64_C(0x00000000000053a4), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b5074b206c5),
        UINT64_C(0x1f43019034b146eb), UINT64_C(0x1f430b4ee8b386d2)
    },
    {   UINT64_C(0x00000000000053a5), UINT64_C(0x1f42f9d050b606d3), UINT64_C(0x1f43078e8cb0e734),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053a6), UINT64_C(0x1f4306505caf66ee), UINT64_C(0x1f43099060b2c6e9),
        UINT64_C(0x1f430b100caf66c1), UINT64_C(0x1f4308cf84b1471a)
    },
    {   UINT64_C(0x00000000000053a7), UINT64_C(0x1f43098fc8adc6ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cfd0b32707), UINT64_C(0x1f43088fd0b246e0)
    },
    {   UINT64_C(0x00000000000053a8), UINT64_C(0x1f43115460aba630), UINT64_C(0x1f430790b0b406fb),
        UINT64_C(0x1f43084f68b04721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053a9), UINT64_C(0x1f430b4f80b12718), UINT64_C(0x1f430dcf34afa71b),
        UINT64_C(0x1f4314cf2cae4721), UINT64_C(0x1f43040f54a1271b)
    },
    {   UINT64_C(0x00000000000053aa), UINT64_C(0x1f43104e48af673e), UINT64_C(0x1f43068f8caf8715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8e78b2673b)
    },
    {   UINT64_C(0x00000000000053ab), UINT64_C(0x1f430a4ee0b08731), UINT64_C(0x1f430d4e98b2873b),
        UINT64_C(0x1f43150ea0b26722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053ac), UINT64_C(0x1f43074f00aee731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4ef0a92716), UINT64_C(0x1f430c8ea0ac073a)
    },
    {   UINT64_C(0x00000000000053ad), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070e98b32722),
        UINT64_C(0x1f43110eb8b08728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053ae), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098ebcb06725),
        UINT64_C(0x1f42fa50a4b786cd), UINT64_C(0x1f43440f8890a6fd)
    },
    {   UINT64_C(0x00000000000053af), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f80b006fa)
    },
    {   UINT64_C(0x00000000000053b0), UINT64_C(0x1f43070ea8af0720), UINT64_C(0x1f43080facae8707),
        UINT64_C(0x1f430c4f8cac4725), UINT64_C(0x1f43088ee8ae4702)
    },
    {   UINT64_C(0x00000000000053b1), UINT64_C(0x1f42fa9010b3c6d2), UINT64_C(0x1f43008f90b22700),
        UINT64_C(0x1f430a4edcad671f), UINT64_C(0x1f42fd8fe8b3c715)
    },
    {   UINT64_C(0x00000000000053b2), UINT64_C(0x1f42f5cfacb78706), UINT64_C(0x1f42f60f78b7a6fc),
        UINT64_C(0x1f42fa0eccb7c728), UINT64_C(0x1f4300ce50b8c711)
    },
    {   UINT64_C(0x00000000000053b3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43028fa4b14704),
        UINT64_C(0x1f42f90fd8b826e0), UINT64_C(0x1f4303cfe4b0a713)
    },
    {   UINT64_C(0x00000000000053b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf40b78713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf48b68715)
    },
    {   UINT64_C(0x00000000000053b5), UINT64_C(0x1f42f9cf2cb5a702), UINT64_C(0x1f4300cf9cb22702),
        UINT64_C(0x1f42fa0fd4b58710), UINT64_C(0x1f42facfacb52711)
    },
    {   UINT64_C(0x2ad526a49419001d), UINT64_C(0x08b2f78f7463860a), UINT64_C(0x0fa2f88f7479e5a1),
        UINT64_C(0x0de2f30f5873c5a1), UINT64_C(0x14daf1cf588f85a1)
    },
    {   UINT64_C(0x0000000000005a72), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f0cb9a700),
        UINT64_C(0x1f43104eecb02714), UINT64_C(0x1f42f7ce60b6473d)
    },
    {   UINT64_C(0x000000b2d631001e), UINT64_C(0x0de2e74f747665a1), UINT64_C(0x0de2e80c6873c5a1),
        UINT64_C(0x0c2aed8e5c6e0594), UINT64_C(0x0c82faccdc716697)
    },
    {   UINT64_C(0x00000000000053b9), UINT64_C(0x1f4301cf0cb7c72e), UINT64_C(0x1f4315d118ad26c3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053ba), UINT64_C(0x1f42f78f68b6c6f6), UINT64_C(0x1f42fc0f74b4a6f8),
        UINT64_C(0x1f42fbcfb8b4c709), UINT64_C(0x1f43070fb8af0709)
    },
    {   UINT64_C(0x00000000000053bb), UINT64_C(0x1f4306cf38af2711), UINT64_C(0x1f42f58fc0b7c70c),
        UINT64_C(0x1f42f40fc8b8870f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053bc), UINT64_C(0x1f42ffce8cb2a714), UINT64_C(0x1f42fa8f18b546e2),
        UINT64_C(0x1f4303ce98b0c720), UINT64_C(0x1f42f90f9cb60702)
    },
    {   UINT64_C(0x00000000000053bd), UINT64_C(0x1f430e8f40ab4709), UINT64_C(0x1f42fa4eacb56710),
        UINT64_C(0x1f42fe8ec8b34719), UINT64_C(0x1f42fb0f24b50710)
    },
    {   UINT64_C(0x00000000000053be), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcfd4b4a710)
    },
    {   UINT64_C(0x00000000000053bf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38f2cb8c714),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f90b24710)
    },
    {   UINT64_C(0x000000000000c99b), UINT64_C(0x0000000000000000), UINT64_C(0x0dab038f8c6f26cc),
        UINT64_C(0x0f53064c586fa71a), UINT64_C(0x0f52fb0c5875471a)
    },
    {   UINT64_C(0x00000000000053c1), UINT64_C(0x1f43018ee8b1c6fd), UINT64_C(0x1f42fe0f60b386f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053c2), UINT64_C(0x1f42facf78b146f8), UINT64_C(0x1f42f8cfa0b60712),
        UINT64_C(0x1f42f70f90b6e700), UINT64_C(0x1f430b0f80b7a6fa)
    },
    {   UINT64_C(0x00000000000053c3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fad004b5c6fa),
        UINT64_C(0x1f42f80f6cb5a71e), UINT64_C(0x1f42f94f78b806fb)
    },
    {   UINT64_C(0x00000000000053c4), UINT64_C(0x1f42f90ea0b28708), UINT64_C(0x1f42fa0eccb64727),
        UINT64_C(0x1f42fece5cb026ed), UINT64_C(0x1f42fdcf88ada6e4)
    },
    {   UINT64_C(0x00000000000053c5), UINT64_C(0x1f42fa0f48b5e6ea), UINT64_C(0x1f42f80f88b5c70f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cfe0b5c6fc)
    },
    {   UINT64_C(0x00000000000053c6), UINT64_C(0x1f42fa0f48b2c6f0), UINT64_C(0x1f42fdcf58b6a6f7),
        UINT64_C(0x1f4302cf44b84709), UINT64_C(0x1f42f90f88ae4712)
    },
    {   UINT64_C(0x00000000000053c7), UINT64_C(0x1f42fdcf70b3a724), UINT64_C(0x1f42fdcf78afa6e9),
        UINT64_C(0x1f4306ce7cb7270d), UINT64_C(0x1f42fe0f08af0718)
    },
    {   UINT64_C(0x00000000000053c8), UINT64_C(0x1f42fd8f1cb00716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccea4ae86dd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053c9), UINT64_C(0x1f42facf40a546fd), UINT64_C(0x1f42fccf58b4a6eb),
        UINT64_C(0x1f42fdcf98b6c6f7), UINT64_C(0x1f42fbcf88b546f6)
    },
    {   UINT64_C(0x00000000000053ca), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecf88abe70c), UINT64_C(0x1f42fbcfa4ad4704)
    },
    {   UINT64_C(0x00000000000053cb), UINT64_C(0x1f42facf38b8a6f3), UINT64_C(0x1f42f8cf48af0706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccfc0a98701)
    },
    {   UINT64_C(0x00000000000053cc), UINT64_C(0x1f42fecf88b426ef), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccff4b0c702), UINT64_C(0x1f42f8cf78b466f2)
    },
    {   UINT64_C(0x00000000000053cd), UINT64_C(0x1f42fbce7cb2e725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ee8b7a722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053ce), UINT64_C(0x1f42fdce88ae4720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ed8a706fd), UINT64_C(0x1f42feced8b28706)
    },
    {   UINT64_C(0x00000000000053cf), UINT64_C(0x1f42f8cf48b3c702), UINT64_C(0x1f42f7cf78b24704),
        UINT64_C(0x1f42fbcf88afe706), UINT64_C(0x1f42f90f88adc6f9)
    },
    {   UINT64_C(0x00000000000053d0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf48a906ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f20f98ae870c)
    },
    {   UINT64_C(0x00000000000053d1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0ff4b7a715), UINT64_C(0x1f42f90eb8b4872d)
    },
    {   UINT64_C(0x000000000000642d), UINT64_C(0x1f43010e9caf26ce), UINT64_C(0x1f4300ce7cae06ff),
        UINT64_C(0x1f42fb0eacb2c724), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053d3), UINT64_C(0x1f43038f98b5c6de), UINT64_C(0x1f42fad014b366fb),
        UINT64_C(0x1f42fbcf58b56706), UINT64_C(0x1f42ffcf68b2c6f5)
    },
    {   UINT64_C(0x00000000000053d4), UINT64_C(0x1f42f90f0cb1c6fd), UINT64_C(0x1f42f9cfb8b386f2),
        UINT64_C(0x1f42facf88b306fd), UINT64_C(0x1f42fbceecb7271e)
    },
    {   UINT64_C(0x00000000000053d5), UINT64_C(0x1f42facf58aa66f9), UINT64_C(0x1f42f50eb0b4e719),
        UINT64_C(0x1f4301cf48b446e2), UINT64_C(0x1f42fe8f90af4709)
    },
    {   UINT64_C(0x00000000000053d6), UINT64_C(0x1f4302cef8b4e72d), UINT64_C(0x1f4304ceb0b88736),
        UINT64_C(0x1f42f90f78b92717), UINT64_C(0x1f42f90e78b38744)
    },
    {   UINT64_C(0x00000000000053d7), UINT64_C(0x1f42face78aea736), UINT64_C(0x1f42fcceb4aac722),
        UINT64_C(0x1f42fbcef4b4e6ea), UINT64_C(0x1f42f9cf08b6e71d)
    },
    {   UINT64_C(0x00000000000053d8), UINT64_C(0x1f42fbcef4ad6729), UINT64_C(0x1f42facf74b646f3),
        UINT64_C(0x1f42fbcf78b12702), UINT64_C(0x1f42facf60aa6710)
    },
    {   UINT64_C(0x00000000000053d9), UINT64_C(0x1f42facf8cb466ff), UINT64_C(0x1f4303cfbcb66703),
        UINT64_C(0x1f42fbd004ae2711), UINT64_C(0x1f42face88b08707)
    },
    {   UINT64_C(0x00000000000053da), UINT64_C(0x1f42fc8f78b46728), UINT64_C(0x1f42f40ef8b6c715),
        UINT64_C(0x1f42fccfb8ae66ff), UINT64_C(0x1f42fdcf2cb766eb)
    },
    {   UINT64_C(0x00000000000053db), UINT64_C(0x1f42f80fb4b38706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf58b726e4), UINT64_C(0x1f42fa0ec8b4c739)
    },
    {   UINT64_C(0x00000000000053dc), UINT64_C(0x1f42f5cfb8afa703), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf48b1670d)
    },
    {   UINT64_C(0x00000000000053dd), UINT64_C(0x1f42f90e7cb2270c), UINT64_C(0x1f42fd8ea8b20730),
        UINT64_C(0x1f42f70fa8aa270a), UINT64_C(0x1f42f80eccb726f5)
    },
    {   UINT64_C(0x00000000000053de), UINT64_C(0x1f42facf98ae46fe), UINT64_C(0x1f42fb0f3cb0870f),
        UINT64_C(0x1f4303cf0cb7a707), UINT64_C(0x1f42f5cfb8b6870e)
    },
    {   UINT64_C(0x00000000000053df), UINT64_C(0x1f42fa0fc4b266f5), UINT64_C(0x1f42fccf88b246f8),
        UINT64_C(0x1f42f80eccb8670d), UINT64_C(0x1f42fa0f3cb5071a)
    },
    {   UINT64_C(0x00000000000053e0), UINT64_C(0x1f42fdcefcab46eb), UINT64_C(0x1f42f70f20b5e704),
        UINT64_C(0x1f42fccfd4b2a6fb), UINT64_C(0x1f42facfa8b746ff)
    },
    {   UINT64_C(0x00000000000053e1), UINT64_C(0x1f42f80f98ad0708), UINT64_C(0x1f42f90f50a946f2),
        UINT64_C(0x1f43038e9cb5c736), UINT64_C(0x1f42fccf48af6713)
    },
    {   UINT64_C(0x00000000000053e2), UINT64_C(0x1f42fbcf18aa673d), UINT64_C(0x1f42faced4b4c70d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cef4aa4708)
    },
    {   UINT64_C(0x00000000000053e3), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301ce84b5e725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70eacba6739)
    },
    {   UINT64_C(0x00000000000053e4), UINT64_C(0x1f42f90f88b6a714), UINT64_C(0x1f42fecef4af871b),
        UINT64_C(0x1f42facf14ad06e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053e5), UINT64_C(0x1f42fb0e6caf270e), UINT64_C(0x1f42fbcfb4ac06e7),
        UINT64_C(0x1f42ffceecb6a70a), UINT64_C(0x1f42f80f0ca7a6fe)
    },
    {   UINT64_C(0x00000000000053e6), UINT64_C(0x1f42fecf9cb28706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffcec8ab26fb), UINT64_C(0x1f42f9cfa8b3e719)
    },
    {   UINT64_C(0x00000000000053e7), UINT64_C(0x1f42f70ed8b08717), UINT64_C(0x1f42fecf44b7e70d),
        UINT64_C(0x1f42f80f38b5a6e0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053e8), UINT64_C(0x1f42fbcf98b24708), UINT64_C(0x1f42fe4fd0b366fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42facfd4a886fc)
    },
    {   UINT64_C(0x00000000000053e9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0fb8b10717),
        UINT64_C(0x1f42fdcee0a98719), UINT64_C(0x1f42f7cf40ab070c)
    },
    {   UINT64_C(0x00000000000053ea), UINT64_C(0x1f42f94f08b7a6f6), UINT64_C(0x1f42fbcf88b4c706),
        UINT64_C(0x1f42fb0ef8b5e707), UINT64_C(0x1f42fe8f68b32708)
    },
    {   UINT64_C(0x00000000000053eb), UINT64_C(0x1f42fccf24b70712), UINT64_C(0x1f42facff4b52710),
        UINT64_C(0x1f42facee8b5a713), UINT64_C(0x1f42fbcf80b64702)
    },
    {   UINT64_C(0x00000000000053ec), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facfb0b586f4), UINT64_C(0x1f42f90f78afe6e6)
    },
    {   UINT64_C(0x00000000000053ed), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf20b6e6f2),
        UINT64_C(0x1f42facfa4ab270c), UINT64_C(0x1f42fdcf64ad870e)
    },
    {   UINT64_C(0x00000000000053ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf64b32700),
        UINT64_C(0x1f42fbcf1cb7a70b), UINT64_C(0x1f42fccf68b686f5)
    },
    {   UINT64_C(0x00000000000053ef), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f7cb88700)
    },
    {   UINT64_C(0x00000000000053f0), UINT64_C(0x1f42f70ee0b646f3), UINT64_C(0x1f42f60f9cb346db),
        UINT64_C(0x1f42ffcf3cb0671a), UINT64_C(0x1f42facfe4b7e708)
    },
    {   UINT64_C(0x00000000000053f1), UINT64_C(0x1f42fbce78b34721), UINT64_C(0x1f42f80ea8b7a700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f90b466dd)
    },
    {   UINT64_C(0x00000000000053f2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f0cb306ea),
        UINT64_C(0x1f42fccf2cb7a71a), UINT64_C(0x1f42fd8f88af26e7)
    },
    {   UINT64_C(0x00000000000053f3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f1cb4c706),
        UINT64_C(0x1f42fb1008b8c6d4), UINT64_C(0x1f42fdcefcaca70e)
    },
    {   UINT64_C(0x39d734000001003c), UINT64_C(0x0fa2f4cf747ac693), UINT64_C(0x0fa2f18f7c7b8658),
        UINT64_C(0x0fa2f4cf74790680), UINT64_C(0x0fa2f4cf74790680)
    },
    {   UINT64_C(0x00000000000053f5), UINT64_C(0x1f42f40f68b4c700), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f2cad86e4), UINT64_C(0x1f42fccf48bb06fd)
    },
    {   UINT64_C(0x00000000000053f6), UINT64_C(0x1f42f10ed4b26734), UINT64_C(0x1f42f90e1caf871e),
        UINT64_C(0x1f42f70fc4b2c702), UINT64_C(0x1f42f90fa0b30710)
    },
    {   UINT64_C(0x0600bc0002b83d40), UINT64_C(0x106af6cfa07ba5f5), UINT64_C(0x106af40fa07c05e8),
        UINT64_C(0x106af94fa07a85f5), UINT64_C(0x106aee8fa07a85e8)
    },
    {   UINT64_C(0x00000000000053f8), UINT64_C(0x1f42fdcf78b7a70f), UINT64_C(0x1f42facf98b3071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f90b0e701)
    },
    {   UINT64_C(0x00000000000053f9), UINT64_C(0x1f430f4edcb58727), UINT64_C(0x1f43078f10b98714),
        UINT64_C(0x1f42f80ee4ace720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053fa), UINT64_C(0x1f42f80f14a886fc), UINT64_C(0x1f42fdcf98ad86f9),
        UINT64_C(0x1f42fbcfd4a8270a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053fb), UINT64_C(0x1f42f9cf74b206e3), UINT64_C(0x1f42fdcf88b5c6fe),
        UINT64_C(0x1f42fcce80b7473a), UINT64_C(0x1f42f70ebcb4070e)
    },
    {   UINT64_C(0x00000000000053fc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf48b7a6fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f48b2c716)
    },
    {   UINT64_C(0x00000000000053fd), UINT64_C(0x1f42f60e58b1c707), UINT64_C(0x1f42f20ea8b8a740),
        UINT64_C(0x1f42f40f68b9870c), UINT64_C(0x1f42f60f40b7c6d1)
    },
    {   UINT64_C(0x00000000000053fe), UINT64_C(0x1f42fbce5cb88741), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f24bac728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053ff), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffce8cb7a732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005400), UINT64_C(0x1f42fccfa4b7e71e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005401), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbceb8b16732),
        UINT64_C(0x1f42f8cf7cae870e), UINT64_C(0x1f42fccf58b4e703)
    },
    {   UINT64_C(0x0000000000005402), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8f0cb6a734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcfd8ac2727)
    },
    {   UINT64_C(0x0000000000005403), UINT64_C(0x1f42f30f28b0872a), UINT64_C(0x1f42fa0f8cb5c706),
        UINT64_C(0x1f42f60f4cb16700), UINT64_C(0x1f42f30f74b7a71f)
    },
    {   UINT64_C(0x0000000000005404), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f14ace716),
        UINT64_C(0x1f42facf10ac6705), UINT64_C(0x1f42fa0f34aee700)
    },
    {   UINT64_C(0x0000000000003d47), UINT64_C(0x106af20df47ec5e8), UINT64_C(0x106af30df47e65f3),
        UINT64_C(0x106b208df466e453), UINT64_C(0x106af78fa07b25e8)
    },
    {   UINT64_C(0x0000000000005406), UINT64_C(0x1f42facefcaee700), UINT64_C(0x1f42fbcfd4b446fc),
        UINT64_C(0x1f42fa0e80b5c741), UINT64_C(0x1f42f60ed4b4e726)
    },
    {   UINT64_C(0x0000000000005407), UINT64_C(0x1f42f60f34aa6729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ef0ad670e), UINT64_C(0x1f42f70f1cb4672a)
    },
    {   UINT64_C(0x0000000000006448), UINT64_C(0x1f4304ceb8b40714), UINT64_C(0x1f42fdceacb2c731),
        UINT64_C(0x1f42ff8ec8aec709), UINT64_C(0x1f4304ce5caf0713)
    },
    {   UINT64_C(0x0000000000005409), UINT64_C(0x1f42fecf38b7070e), UINT64_C(0x1f42f7cfb4b1070a),
        UINT64_C(0x1f42facf0cb86719), UINT64_C(0x1f42f40e64b2870b)
    },
    {   UINT64_C(0x0000000000006449), UINT64_C(0x1f43070e8cb266fb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040e7cb1871f), UINT64_C(0x1f42f90e9cb44716)
    },
    {   UINT64_C(0x000000000000540b), UINT64_C(0x1f42f80e9cb26721), UINT64_C(0x1f42f90f68b2c71b),
        UINT64_C(0x1f42f9cee4b5c6fb), UINT64_C(0x1f42f8cf1cb88706)
    },
    {   UINT64_C(0x000000000000644a), UINT64_C(0x1f43128e98b2672d), UINT64_C(0x1f430a8eb8afa6f9),
        UINT64_C(0x1f43094ed8b0e70e), UINT64_C(0x1f43068e8cb0e727)
    },
    {   UINT64_C(0x000000000000540d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ed0b46725),
        UINT64_C(0x1f42f80f68b02700), UINT64_C(0x1f42f50e2cb7a710)
    },
    {   UINT64_C(0x000000000000540e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78ef0b6c728),
        UINT64_C(0x1f42f94f08b0e731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000540f), UINT64_C(0x1f42fb0f38b4e6f7), UINT64_C(0x1f42f80f24b6a722),
        UINT64_C(0x1f42f80f7cab0715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005410), UINT64_C(0x1f42fd0f94afe701), UINT64_C(0x1f42fd0eacb16705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f30f88b3a71a)
    },
    {   UINT64_C(0x0000000000005411), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0f48af46fd),
        UINT64_C(0x1f42fdcef4b6a6f2), UINT64_C(0x1f42f9cf98b9070c)
    },
    {   UINT64_C(0x0000000000005412), UINT64_C(0x1f4300ceb4ace6fc), UINT64_C(0x1f42fece7cb32743),
        UINT64_C(0x1f43000edcaaa6ed), UINT64_C(0x1f42fd0eccae06d5)
    },
    {   UINT64_C(0x0000000000005413), UINT64_C(0x1f42f90f8cb3e737), UINT64_C(0x1f42fbcf60b406f8),
        UINT64_C(0x1f42f8cfa0b6e70c), UINT64_C(0x1f42f70ed8ae66f8)
    },
    {   UINT64_C(0x000000000001004c), UINT64_C(0x0a6aec0fa06e871a), UINT64_C(0x08b2f9cf745be5a1),
        UINT64_C(0x1772ec0f58a06675), UINT64_C(0x0fa2fa4c647d0676)
    },
    {   UINT64_C(0x0000000000005415), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf00b4e73b),
        UINT64_C(0x1f42f88f50b62725), UINT64_C(0x1f4300cee0aaa6df)
    },
    {   UINT64_C(0x0000000000005416), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cfc4b806f9), UINT64_C(0x1f42fe0f80aca6e7)
    },
    {   UINT64_C(0x0000000000005417), UINT64_C(0x1f42facea4b5e737), UINT64_C(0x1f42ffce90ab26f7),
        UINT64_C(0x1f42fe0eecae86f2), UINT64_C(0x1f4301cf84af471d)
    },
    {   UINT64_C(0x0000000000005418), UINT64_C(0x1f43000f2cb1c6ee), UINT64_C(0x1f42fecf34af86ff),
        UINT64_C(0x1f42fa0fe8b5c6d8), UINT64_C(0x1f42fd0ee0b0e709)
    },
    {   UINT64_C(0x0000000000005419), UINT64_C(0x1f4301cf8cac0712), UINT64_C(0x1f42fa0ebcb68721),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000541a), UINT64_C(0x1f4300ceecb066f5), UINT64_C(0x1f42fd0ee0a6c720),
        UINT64_C(0x1f42fbcf40ad66f1), UINT64_C(0x1f42fdcebcb06701)
    },
    {   UINT64_C(0x0000000000005aa4), UINT64_C(0x1f42f88e90ae872f), UINT64_C(0x1f42fa8f20b266fb),
        UINT64_C(0x1f430ece78afe71d), UINT64_C(0x1f42f20f28b906ec)
    },
    {   UINT64_C(0x000000000000541c), UINT64_C(0x1f43010f60af670d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecfecb08702), UINT64_C(0x1f43000f48ac670f)
    },
    {   UINT64_C(0x000000000000541d), UINT64_C(0x1f42f7cf40ab070c), UINT64_C(0x1f42fecee0a9872c),
        UINT64_C(0x1f42fbcf5cb0c702), UINT64_C(0x1f42fcce8cb86739)
    },
    {   UINT64_C(0x0000000000006453), UINT64_C(0x1f4308cf64af6705), UINT64_C(0x1f430c0ffcb506cc),
        UINT64_C(0x1f4307cf2cae26e9), UINT64_C(0x1f4302d098b3e6eb)
    },
    {   UINT64_C(0x000000000000541f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ef0b58729),
        UINT64_C(0x1f42fa0f18ae8723), UINT64_C(0x1f42fe0f90ae8719)
    },
    {   UINT64_C(0x0000000000005420), UINT64_C(0x1f42f90f68b82716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0eecb3c708), UINT64_C(0x1f42ff0ed0b0671a)
    },
    {   UINT64_C(0x0000000000005421), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0e8cb6c738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0e2cb4a729)
    },
    {   UINT64_C(0x0000000000010053), UINT64_C(0x1692fd0f5895e6dd), UINT64_C(0x0fa2f4cf747e6607),
        UINT64_C(0x1692fd0f5897274a), UINT64_C(0x0fa2f4cf74812672)
    },
    {   UINT64_C(0x0000000000005423), UINT64_C(0x1f42fb0f28ad46ef), UINT64_C(0x1f42fbce84b48727),
        UINT64_C(0x1f42ff0f00b0e701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005424), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccee8ab8702),
        UINT64_C(0x1f42fa0f14b1272b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005425), UINT64_C(0x1f42f94f58b3c736), UINT64_C(0x1f42fc0f78ad6711),
        UINT64_C(0x1f42facf3cb82729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005426), UINT64_C(0x1f43050e6cb7872d), UINT64_C(0x1f42fdcee0b26719),
        UINT64_C(0x1f42f6cfd4b62703), UINT64_C(0x1f42fa0f5cb34719)
    },
    {   UINT64_C(0x0000000000005427), UINT64_C(0x1f42facf78b0c73a), UINT64_C(0x1f43030eacace6e2),
        UINT64_C(0x1f42f70f68abc714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010056), UINT64_C(0x131aeacfa0880753), UINT64_C(0x0c2af94fa073c67f),
        UINT64_C(0x0fa2f24f587e0754), UINT64_C(0x0c2af20f6c70e680)
    },
    {   UINT64_C(0x0000000000005429), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f74b4e71a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cfa8ae06fe)
    },
    {   UINT64_C(0x000000000000542a), UINT64_C(0x1f42fd0f20aea6f3), UINT64_C(0x1f42fa0f84af8709),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000542b), UINT64_C(0x1f42f90e98b1471e), UINT64_C(0x1f42f80e4cb2c72c),
        UINT64_C(0x1f43020e84ae8732), UINT64_C(0x1f42fbcf58b56731)
    },
    {   UINT64_C(0x000000000000542c), UINT64_C(0x1f4301ceb0aee702), UINT64_C(0x1f42fdcf48b6e6ef),
        UINT64_C(0x1f42fb0fa4b88702), UINT64_C(0x1f42fdce2cb46740)
    },
    {   UINT64_C(0x000000000000542d), UINT64_C(0x1f42fc0fa4b126f5), UINT64_C(0x1f42f90e70b38734),
        UINT64_C(0x1f42facf04b76706), UINT64_C(0x1f42fbce60b1872e)
    },
    {   UINT64_C(0x000000000000542e), UINT64_C(0x1f42fbcf38b7a70f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0eb8b886ee), UINT64_C(0x1f42facea0b38724)
    },
    {   UINT64_C(0x000000000000542f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e00b2c72d),
        UINT64_C(0x1f42f80f6cafa721), UINT64_C(0x1f42ff0e98af66f2)
    },
    {   UINT64_C(0x0000000000005430), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf48b30707),
        UINT64_C(0x1f42f8cea4b16730), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005431), UINT64_C(0x1f42fece90af66ff), UINT64_C(0x1f42ff8f1cb2c730),
        UINT64_C(0x1f42fecf10b706f6), UINT64_C(0x1f42f7cea0b5c71e)
    },
    {   UINT64_C(0x0000000000005432), UINT64_C(0x1f42fa0f08b766ff), UINT64_C(0x1f4308cf2cb5c721),
        UINT64_C(0x1f42fb0f20b5e6fb), UINT64_C(0x1f42fece50b5e724)
    },
    {   UINT64_C(0x0000000000005433), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f68b4a6e9),
        UINT64_C(0x1f42f9cf20b36705), UINT64_C(0x1f42fece94b8672a)
    },
    {   UINT64_C(0x0000000000005434), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f18b7c72c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcce60b1c70e)
    },
    {   UINT64_C(0x0000000000005435), UINT64_C(0x1f42fa0e34b36735), UINT64_C(0x1f42fbcf60b72733),
        UINT64_C(0x1f42fbcebcb1c72d), UINT64_C(0x1f42f7ce64b4a6f8)
    },
    {   UINT64_C(0x0000000000005436), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0e98b6c731), UINT64_C(0x1f42f60f30b8a6e8)
    },
    {   UINT64_C(0x0000000000005437), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60e64b5c703),
        UINT64_C(0x1f42fd0e54b3e70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005438), UINT64_C(0x1f42fa0eb4b70711), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f20b5472d)
    },
    {   UINT64_C(0x0000000000005439), UINT64_C(0x1f42f70e7cb2e70b), UINT64_C(0x1f42f80ed4b50740),
        UINT64_C(0x1f42f50fc4b8670e), UINT64_C(0x1f42faced4b06726)
    },
    {   UINT64_C(0x0000000000006461), UINT64_C(0x1f4306cec0b086f5), UINT64_C(0x1f43041024b2070e),
        UINT64_C(0x1f4305cf18b16702), UINT64_C(0x1f4305ce88b26731)
    },
    {   UINT64_C(0x000000000000543b), UINT64_C(0x1f42facf58b286f2), UINT64_C(0x1f42facefcb6e6fd),
        UINT64_C(0x1f42f9cf90b00738), UINT64_C(0x1f42f80e5cb42711)
    },
    {   UINT64_C(0x000000000000543c), UINT64_C(0x1f43040e70b14739), UINT64_C(0x1f42fcced8ae8729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ebcb5e6f9)
    },
    {   UINT64_C(0x000000000000543d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f1cafc709),
        UINT64_C(0x1f42f80f88b14702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000543e), UINT64_C(0x1f42f50eb8b4c715), UINT64_C(0x1f42f9cf18ac470b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42face8cb32721)
    },
    {   UINT64_C(0x000000000000543f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f30b3c6e8),
        UINT64_C(0x1f42f80eccb1c714), UINT64_C(0x1f42f90f74b5c719)
    },
    {   UINT64_C(0x0000000000005440), UINT64_C(0x1f42ffceacb1e74c), UINT64_C(0x1f43000e74b2872c),
        UINT64_C(0x1f42fe8e64b6a73c), UINT64_C(0x1f42f70f60b1271d)
    },
    {   UINT64_C(0x0000000000005441), UINT64_C(0x1f42ff8f98b06738), UINT64_C(0x1f42f8cea4b28715),
        UINT64_C(0x1f42f51008b786eb), UINT64_C(0x1f42f60f00b32702)
    },
    {   UINT64_C(0x0000000000005442), UINT64_C(0x1f42f60ec4b8a732), UINT64_C(0x1f42f24f88b86718),
        UINT64_C(0x1f42f8cebcb3e701), UINT64_C(0x1f42facf0cad0749)
    },
    {   UINT64_C(0x0000000000005443), UINT64_C(0x1f42f9cf44b626eb), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005444), UINT64_C(0x1f42f60ee0afa734), UINT64_C(0x1f42fccf1cab46d8),
        UINT64_C(0x1f42facf18b02726), UINT64_C(0x1f42f80f50ad8731)
    },
    {   UINT64_C(0x0000000000005445), UINT64_C(0x1f42face50b24735), UINT64_C(0x1f42fbce38b54725),
        UINT64_C(0x1f42fdcfa4b3a6ef), UINT64_C(0x1f42f41004b806fa)
    },
    {   UINT64_C(0x0000000000005446), UINT64_C(0x1f42fccf0cb5672c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60eacb1c745), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005447), UINT64_C(0x1f42fbce7cb1c722), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90eb4b1e73d)
    },
    {   UINT64_C(0x0000000000005448), UINT64_C(0x1f42fa0e9cb62724), UINT64_C(0x1f42f9cf88b22703),
        UINT64_C(0x1f42fbcef0b20710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005449), UINT64_C(0x1f42fb0e90b7a6f9), UINT64_C(0x1f42ffcea4acc729),
        UINT64_C(0x1f42fbcf18b14722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000544a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ea0b4c6d4),
        UINT64_C(0x1f42fdcea0b3e72a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000544b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40f18b7c6f3),
        UINT64_C(0x1f42f40f28b7a6ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000544c), UINT64_C(0x1f42fd0f1caf871e), UINT64_C(0x1f42f40f10b5e6f6),
        UINT64_C(0x1f42f70eecadc722), UINT64_C(0x1f42fbce5cb3073e)
    },
    {   UINT64_C(0x000000000000544d), UINT64_C(0x1f42f30ea0b38721), UINT64_C(0x1f42f90eacb2672c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0efcb4670e)
    },
    {   UINT64_C(0x0000000000010069), UINT64_C(0x0000000000000000), UINT64_C(0x0fc2db4c6c798685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000544f), UINT64_C(0x1f42f50ebcb68711), UINT64_C(0x1f42fc0ea8ad8725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058fbcb646cb)
    },
    {   UINT64_C(0x000000000001006a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0aa2f5cbfc666695), UINT64_C(0x08b2e08c3863860a)
    },
    {   UINT64_C(0x0000000000005451), UINT64_C(0x1f430a5024b4c6da), UINT64_C(0x1f43088f74b56726),
        UINT64_C(0x1f430d4f9cb4a701), UINT64_C(0x1f4306cf40b56720)
    },
    {   UINT64_C(0x000000000001006b), UINT64_C(0x0000000000000000), UINT64_C(0x09b2f78f74770693),
        UINT64_C(0x0000000000000000), UINT64_C(0x1793078f58aca6e5)
    },
    {   UINT64_C(0x0000000000005453), UINT64_C(0x1f43044f7cb4872d), UINT64_C(0x1f4308cf18b4e72e),
        UINT64_C(0x1f43058ee4afc726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005454), UINT64_C(0x1f430d0fb0ac06e8), UINT64_C(0x1f43088f88b2070f),
        UINT64_C(0x1f430c4f20b1e716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005455), UINT64_C(0x1f430d8ec4b30734), UINT64_C(0x1f42ff8f80b4e6e9),
        UINT64_C(0x1f43180fb4b0c726), UINT64_C(0x1f430a8e84b7c73b)
    },
    {   UINT64_C(0x0000000000005456), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf54b04705),
        UINT64_C(0x1f43110e64b20727), UINT64_C(0x1f42fd8f3cb20719)
    },
    {   UINT64_C(0x0000000000005457), UINT64_C(0x1f4307cf10b48728), UINT64_C(0x1f42fd8f3cb6a71a),
        UINT64_C(0x1f42fd8f40b5671a), UINT64_C(0x1f42fb8f78b706f2)
    },
    {   UINT64_C(0x0000000000005458), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf3cb2071a),
        UINT64_C(0x1f42f7cf94b56725), UINT64_C(0x1f42fd8f40b3a717)
    },
    {   UINT64_C(0x0000000000005459), UINT64_C(0x0000000000000000), UINT64_C(0x1f43085094b3c684),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431d4ee8a3471f)
    },
    {   UINT64_C(0x000000000000545a), UINT64_C(0x1f43028f74b7270c), UINT64_C(0x1f430c0f58ac86de),
        UINT64_C(0x1f42fb4f44b4e72c), UINT64_C(0x1f42fd11c4b926b4)
    },
    {   UINT64_C(0x000000000000545b), UINT64_C(0x1f42f71034b74705), UINT64_C(0x1f430990d0b066d9),
        UINT64_C(0x1f42fd0edcb9a734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x09412023c0014012), UINT64_C(0x1852e80fa0a26685), UINT64_C(0x0c2ae60fa0714685),
        UINT64_C(0x0fa2e28e14800685), UINT64_C(0x14dae8cfa0964685)
    },
    {   UINT64_C(0x000000000000545d), UINT64_C(0x1f43021000b266d3), UINT64_C(0x1f4304cfb0afe711),
        UINT64_C(0x1f43038f30b28744), UINT64_C(0x1f43070f54b3a700)
    },
    {   UINT64_C(0x1171f40007214014), UINT64_C(0x131ae70fa08b2685), UINT64_C(0x1692f4cc6094e695),
        UINT64_C(0x14dae70fa08c6685), UINT64_C(0x1162ee8f58804695)
    },
    {   UINT64_C(0x000000000000545f), UINT64_C(0x1f4307ce9cb70743), UINT64_C(0x1f43060f20b1a73e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005460), UINT64_C(0x1f43104ebcaf472c), UINT64_C(0x1f43018fccb806dc),
        UINT64_C(0x1f4316cf94a466e0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005461), UINT64_C(0x0000000000000000), UINT64_C(0x1f431d8ef8a2871c),
        UINT64_C(0x1f43238f54a0c6e4), UINT64_C(0x1f4303ce44ac872d)
    },
    {   UINT64_C(0x1ff3f87dc0014018), UINT64_C(0x0a6b0357b063a6a1), UINT64_C(0x0fa2e8cf687785b6),
        UINT64_C(0x0fa2ea8f6c7606a3), UINT64_C(0x0de2eccf6c6f85b6)
    },
    {   UINT64_C(0x22240e8040014019), UINT64_C(0x0fa2e8cf6c7f26a3), UINT64_C(0x0de2ef4f6c7045b6),
        UINT64_C(0x0a6ac3cc6c7646a2), UINT64_C(0x0fa2e10c747a25b6)
    },
    {   UINT64_C(0x2330008c5179401a), UINT64_C(0x1162eacf7c7cc6a3), UINT64_C(0x08b2ee8f7c5f4694),
        UINT64_C(0x08b2becc64616695), UINT64_C(0x0fa2e84f807ae6a3)
    },
    {   UINT64_C(0x24e490911211401b), UINT64_C(0x08b2ee8f7c62c6a3), UINT64_C(0x1852ea8f7c98e5b6),
        UINT64_C(0x1162ec8f7c7c45b6), UINT64_C(0x0fa2ed4f6c75a5b6)
    },
    {   UINT64_C(0x2874b295d2a9401c), UINT64_C(0x0fa2d00c6c7645b6), UINT64_C(0x0fa2ee4c6c7965b6),
        UINT64_C(0x0c2ae8cfa06f25b6), UINT64_C(0x0c2ae94f6c6825b6)
    },
    {   UINT64_C(0x2af52ea59449401d), UINT64_C(0x08b2eb4f7c61a63a), UINT64_C(0x1162f1cf7c7c25b6),
        UINT64_C(0x0de2f1cf6c70a5b6), UINT64_C(0x14daf1cf6c8b45b6)
    },
    {   UINT64_C(0x000000b3d649401e), UINT64_C(0x0fa2e2cf6c7925b6), UINT64_C(0x0de2d68c6c6f85b6),
        UINT64_C(0x0c2ae34e686c85a9), UINT64_C(0x0ae2ef0cb4750696)
    },
    {   UINT64_C(0x0000000000005469), UINT64_C(0x1f430c0f10b306fc), UINT64_C(0x1f430f0f6caf06fe),
        UINT64_C(0x1f42ff8f74b2a6f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000546a), UINT64_C(0x1f430f0f48ab0703), UINT64_C(0x1f4312cee0a9070c),
        UINT64_C(0x1f43050e5cb36729), UINT64_C(0x1f434d509887c694)
    },
    {   UINT64_C(0x000000000000546b), UINT64_C(0x1f43020ec4b7c735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4f18aa26d4)
    },
    {   UINT64_C(0x000000000000546c), UINT64_C(0x1f43004f04b72721), UINT64_C(0x1f4308cfdcb4e6c6),
        UINT64_C(0x1f43084ee8ace70d), UINT64_C(0x1f430b0f98b2c700)
    },
    {   UINT64_C(0x000000000000546d), UINT64_C(0x1f430b0ec4b3c701), UINT64_C(0x1f4308d048b566cd),
        UINT64_C(0x1f430c0f88a96704), UINT64_C(0x1f4300ceccb8871d)
    },
    {   UINT64_C(0x000000000000546e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0fbcb426ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43084f24acc716)
    },
    {   UINT64_C(0x000000000000546f), UINT64_C(0x1f42fdcf54b5871a), UINT64_C(0x1f4307ced8a90701),
        UINT64_C(0x1f42fc4f1cb70701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005470), UINT64_C(0x1f43038f00b1c726), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005471), UINT64_C(0x1f42fd8ea0b54745), UINT64_C(0x1f43024f8cad46da),
        UINT64_C(0x1f430a8f48b34732), UINT64_C(0x1f42ff4f48b5a6ff)
    },
    {   UINT64_C(0x0000000000005472), UINT64_C(0x1f43014eb4b28744), UINT64_C(0x1f430acf00aca70f),
        UINT64_C(0x1f43034ebcb0872d), UINT64_C(0x1f43050f80a5c6f1)
    },
    {   UINT64_C(0x0000000000014029), UINT64_C(0x0fa2e78f78804606), UINT64_C(0x0fa2f64fa08d0685),
        UINT64_C(0x06e2fe8f5851c695), UINT64_C(0x0fa2f70d6477e695)
    },
    {   UINT64_C(0x000000000001007c), UINT64_C(0x08b2cf0c6470267d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001402b), UINT64_C(0x12f2fad15083e577), UINT64_C(0x0a6aee9238606502),
        UINT64_C(0x175af58f58984695), UINT64_C(0x0a6afad8446fa657)
    },
    {   UINT64_C(0x000000000001402c), UINT64_C(0x0c8302d5e86ca698), UINT64_C(0x11d2f64fa081a620),
        UINT64_C(0x0962ee93e865a693), UINT64_C(0x0962f253c466a693)
    },
    {   UINT64_C(0x0000000000005477), UINT64_C(0x1f4302ce7cb52737), UINT64_C(0x1f43064f98b946dd),
        UINT64_C(0x1f430b0f94ad06b0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x2e9000000001402e), UINT64_C(0x0a6ada0c385203ed), UINT64_C(0x0962f593e8642693),
        UINT64_C(0x0962fc15e069e695), UINT64_C(0x0fa2f11020790587)
    },
    {   UINT64_C(0x000000000001402f), UINT64_C(0x1772efcf6898a693), UINT64_C(0x1772ebcf6898e693),
        UINT64_C(0x1772efcf68994693), UINT64_C(0x0fa2f58c6c7325d4)
    },
    {   UINT64_C(0x325634bf57914030), UINT64_C(0x14dadd0fa088a79b), UINT64_C(0x14dadd0fa088a79b),
        UINT64_C(0x14dadd0fa088a794), UINT64_C(0x14dadd0fa08b0771)
    },
    {   UINT64_C(0x000000000000547b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43060f44b2071c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d8fb0b5a6fc)
    },
    {   UINT64_C(0x0000000000014032), UINT64_C(0x14dae74fa090279b), UINT64_C(0x14dae74fa090279b),
        UINT64_C(0x14dae74fa0902794), UINT64_C(0x14dae74fa0902771)
    },
    {   UINT64_C(0x000000000000547d), UINT64_C(0x0000000000000000), UINT64_C(0x1f43139098aca657),
        UINT64_C(0x1f4312d018af46c6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x36d6d40000014034), UINT64_C(0x1692e64fa0960685), UINT64_C(0x1692e74f649c0771),
        UINT64_C(0x1692f4cf5894e79b), UINT64_C(0x1692f4cf5894e79b)
    },
    {   UINT64_C(0x000000000000547f), UINT64_C(0x1f43124ff4ab46c3), UINT64_C(0x1f43140f8cb146ef),
        UINT64_C(0x1f430b4f8cace6f2), UINT64_C(0x1f43130fa8aa8716)
    },
    {   UINT64_C(0x0000000000005480), UINT64_C(0x1f4308d078a646a6), UINT64_C(0x1f43158feca3c6d9),
        UINT64_C(0x1f43478f0c8d46f4), UINT64_C(0x1f4311d084a666b7)
    },
    {   UINT64_C(0x000000001c714037), UINT64_C(0x1692fecf589b8771), UINT64_C(0x131affcfa090e79b),
        UINT64_C(0x131ae74fa0862685), UINT64_C(0x0fa2bc0c8079a6a9)
    },
    {   UINT64_C(0x000000e65cb14038), UINT64_C(0x0fa2e8cf687786a1), UINT64_C(0x0fa2e8cf687786a1),
        UINT64_C(0x0fa2e8cf6879e69a), UINT64_C(0x0fa2e8cf687ca677)
    },
    {   UINT64_C(0x000744e7dce14039), UINT64_C(0x0fa2e8cf687b2677), UINT64_C(0x0fa2e8cf687786dc),
        UINT64_C(0x1692eccf6c92a5b6), UINT64_C(0x0de2e80c386f85b6)
    },
    {   UINT64_C(0x000750e94001403a), UINT64_C(0x0de2ef4f6c7046a1), UINT64_C(0x0de2ef4f6c72e6a1),
        UINT64_C(0x0de2ef4f6c78469a), UINT64_C(0x0de2ef4f6c794677)
    },
    {   UINT64_C(0x000000000001403b), UINT64_C(0x08b2ee8f7c6346a1), UINT64_C(0x08b2ee8f7c6546a1),
        UINT64_C(0x08b2ee8f7c6aa69a), UINT64_C(0x08b2ee8f7c6bc677)
    },
    {   UINT64_C(0x0000000000005486), UINT64_C(0x1f43010f20a6670a), UINT64_C(0x1f430c4f2ca7a6f0),
        UINT64_C(0x1f43104ec4a8a6e9), UINT64_C(0x1f42fd0ef0a72704)
    },
    {   UINT64_C(0x000764001d89403d), UINT64_C(0x0fa2ed4f6c78469a), UINT64_C(0x0fa2ed4f6c7b2677),
        UINT64_C(0x0fa2ed4f6c794677), UINT64_C(0x11d2f64f2c81a5ff)
    },
    {   UINT64_C(0x000000000001403e), UINT64_C(0x0fa2ed4dc475a618), UINT64_C(0x1162f1cf7c7c26a1),
        UINT64_C(0x1162f1cf7c7c26a1), UINT64_C(0x1162f1cf7c7c269a)
    },
    {   UINT64_C(0x000000000001403f), UINT64_C(0x1162f1cf7c7c2677), UINT64_C(0x0de2d68c6c70a6a1),
        UINT64_C(0x0fa2d00c6c7646a3), UINT64_C(0x0de2d68c6c780677)
    },
    {   UINT64_C(0x0000000000014040), UINT64_C(0x14dadd0fa088a701), UINT64_C(0x0fa2e8cf687b4632),
        UINT64_C(0x14dadd0fa088a75d), UINT64_C(0x0fa2e8cf687ae68e)
    },
    {   UINT64_C(0x0000000000014041), UINT64_C(0x14dadd0cec8e8693), UINT64_C(0x0fa2e8ccec7de5b6),
        UINT64_C(0x14daf60f58932770), UINT64_C(0x0de2eccf6c6f86a1)
    },
    {   UINT64_C(0x0000000000014042), UINT64_C(0x14daf60f58932769), UINT64_C(0x0de2eccf6c72669a),
        UINT64_C(0x14daf60f58932746), UINT64_C(0x0de2eccf6c6f8677)
    },
    {   UINT64_C(0x0000000000014043), UINT64_C(0x14daf60f58932769), UINT64_C(0x0de2eccf6c75269a),
        UINT64_C(0x1692e28fa0942769), UINT64_C(0x1492e8cf6c9946ae)
    },
    {   UINT64_C(0x0000000000014044), UINT64_C(0x1692e64fa0960685), UINT64_C(0x0fa2e8cf6c8226a3),
        UINT64_C(0x14dae74fa0902701), UINT64_C(0x0de2ef4f6c746632)
    },
    {   UINT64_C(0x0000000000014045), UINT64_C(0x14dae74fa090275d), UINT64_C(0x0de2ef4f6c7a468e),
        UINT64_C(0x14dae74fa0902746), UINT64_C(0x0de2ef4f6c704677)
    },
    {   UINT64_C(0x0000000000014046), UINT64_C(0x14dae74cc8902685), UINT64_C(0x0de2ef4cc87045b6),
        UINT64_C(0x14dae74fa0902769), UINT64_C(0x0de2ef4f6c7b469a)
    },
    {   UINT64_C(0x0000000000014047), UINT64_C(0x1692f34f5896a769), UINT64_C(0x0fa2e10c747a269a),
        UINT64_C(0x1692f34f5896a75d), UINT64_C(0x0fa2e10c747a268e)
    },
    {   UINT64_C(0x0000000000014048), UINT64_C(0x1692f34f5896a746), UINT64_C(0x0fa2e10c747a2677),
        UINT64_C(0x1692f34a0496a695), UINT64_C(0x0fa2e10c747a2728)
    },
    {   UINT64_C(0x0000000000014049), UINT64_C(0x1852e80fa0a26769), UINT64_C(0x1162eacf7c7cc769),
        UINT64_C(0x1852e80fa0a28685), UINT64_C(0x1162eacf7c7cc6a3)
    },
    {   UINT64_C(0x000000000001404a), UINT64_C(0x0c2ae60fa0794746), UINT64_C(0x08b2ebcf7c6a4677),
        UINT64_C(0x0c2ae60fa076e701), UINT64_C(0x08b2eb4f7c67e632)
    },
    {   UINT64_C(0x000000000001404b), UINT64_C(0x0c2ae60fa077c75d), UINT64_C(0x08b2ee8f7c67868e),
        UINT64_C(0x0c2ae60cec714685), UINT64_C(0x08b2ee8cec5f4694)
    },
    {   UINT64_C(0x000000000000ca71), UINT64_C(0x1c72f94d28aaa67d), UINT64_C(0x1c72f94d28aaa67d),
        UINT64_C(0x1c72f94d28aaa67d), UINT64_C(0x1c72f94d28aaa67d)
    },
    {   UINT64_C(0x000000000001404d), UINT64_C(0x0fa2e28e1483c769), UINT64_C(0x08b2becc6465c69a),
        UINT64_C(0x14dae8ca04964685), UINT64_C(0x0fa2e84a047ae6a3)
    },
    {   UINT64_C(0x000000000001404e), UINT64_C(0x12c2ef8fa094a5be), UINT64_C(0x131ae88fa0884770),
        UINT64_C(0x08b2ee8f7c698770), UINT64_C(0x131ae88a04884685)
    },
    {   UINT64_C(0x000000000001404f), UINT64_C(0x08b2de8a0462c6a3), UINT64_C(0x131ae88fa091e695),
        UINT64_C(0x0e1aee8f7c7ce6ae), UINT64_C(0x131ae88fa0884685)
    },
    {   UINT64_C(0x0000000000014050), UINT64_C(0x0bbaee8f7c6e46a3), UINT64_C(0x131ae88fa0884685),
        UINT64_C(0x08b2eacf7c6426a3), UINT64_C(0x1692e74f649c0770)
    },
    {   UINT64_C(0x0000000000014051), UINT64_C(0x1162ec8f7c7c46a1), UINT64_C(0x1692e74a049c0685),
        UINT64_C(0x1162ec8a047c45b6), UINT64_C(0x1692e74f649c0769)
    },
    {   UINT64_C(0x0000000000014052), UINT64_C(0x1162ec8f7c7dc69a), UINT64_C(0x15e2f88f7c90a6ae),
        UINT64_C(0x1692e28c74942685), UINT64_C(0x10faec8c6479c5b6)
    },
    {   UINT64_C(0x0000000000014053), UINT64_C(0x1692f4cf5894e701), UINT64_C(0x0fa2ed4f6c782632),
        UINT64_C(0x1692f4cf5894e75d), UINT64_C(0x0fa2ed4f6c7b868e)
    },
    {   UINT64_C(0x0000000000006493), UINT64_C(0x1f42fb0f08b726f1), UINT64_C(0x1f42f91018b5e6ef),
        UINT64_C(0x1f42ffd01cb2c6f4), UINT64_C(0x1f42ffd05cb2c70e)
    },
    {   UINT64_C(0x000000000000549f), UINT64_C(0x1f42fd8eacad0721), UINT64_C(0x1f4301cee0ab070c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000eaca98724)
    },
    {   UINT64_C(0x0000000000014056), UINT64_C(0x14dae70fa08c6769), UINT64_C(0x0c2ae8cfa071e69a),
        UINT64_C(0x1162ee8f58804770), UINT64_C(0x0c2ae94f6c6de6a1)
    },
    {   UINT64_C(0x0000000000014057), UINT64_C(0x1162ee8f58804769), UINT64_C(0x0c2ae94f6c6c669a),
        UINT64_C(0x1162ee8c38804695), UINT64_C(0x0c2ae94c386825b6)
    },
    {   UINT64_C(0x0000000000014058), UINT64_C(0x1162ee8f58804769), UINT64_C(0x0c2ae94f6c71e69a),
        UINT64_C(0x131afa4c388fc685), UINT64_C(0x08b2db4c3862a63a)
    },
    {   UINT64_C(0x0000000000014059), UINT64_C(0x131afa4fa08fc769), UINT64_C(0x0cdaeb4f7c7ce6ae),
        UINT64_C(0x131afa4fa08fc685), UINT64_C(0x08b2e68f7c61a63a)
    },
    {   UINT64_C(0x000000000001405a), UINT64_C(0x1692fecf589b8731), UINT64_C(0x1162f1cf7c7c2677),
        UINT64_C(0x1692fecf589b8701), UINT64_C(0x1162f1cf7c7c2632)
    },
    {   UINT64_C(0x000000000001405b), UINT64_C(0x1692fecf589b875d), UINT64_C(0x1162f1cf7c7c268e),
        UINT64_C(0x1692fecf589b8781), UINT64_C(0x1162f1cf7c7c26c1)
    },
    {   UINT64_C(0x000000000001405c), UINT64_C(0x1692fecf589b8761), UINT64_C(0x1162f1cf7c8066a1),
        UINT64_C(0x1692feccec9b8685), UINT64_C(0x1162f1ccec80e5b6)
    },
    {   UINT64_C(0x000000000001405d), UINT64_C(0x1bcafe0f58b40769), UINT64_C(0x14daf1cf6c8b469a),
        UINT64_C(0x131affcfa090e769), UINT64_C(0x0de2d68c6c6fa69a)
    },
    {   UINT64_C(0x000000000001405e), UINT64_C(0x131affcfa090e746), UINT64_C(0x131aeb0fa0882770),
        UINT64_C(0x0c2ae34e686de6a1), UINT64_C(0x131aeb0fa0882746)
    },
    {   UINT64_C(0x000000000001405f), UINT64_C(0x0c2ae34e686c8677), UINT64_C(0x131aeb0fa0882769),
        UINT64_C(0x0c2ae34e6871e69a), UINT64_C(0x0a6af18fa081869b)
    },
    {   UINT64_C(0x0000000000014060), UINT64_C(0x0fa2ea8f6c7606a3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054ab), UINT64_C(0x1f43034facad8704), UINT64_C(0x1f4300cf10b226f6),
        UINT64_C(0x1f43004ed8b066f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010098), UINT64_C(0x1652f00c50a26693), UINT64_C(0x129af00c5086e5ca),
        UINT64_C(0x1192fb0f7484c5a1), UINT64_C(0x0de2f1cbf873c5a1)
    },
    {   UINT64_C(0x000000000000ca88), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1772fe4ebc942766), UINT64_C(0x1772fe4ebc9426c8)
    },
    {   UINT64_C(0x000000000000ca89), UINT64_C(0x1772fe4ebc9426d2), UINT64_C(0x1772fe4b2494261b),
        UINT64_C(0x1772fe4be094261b), UINT64_C(0x1862fe4bc49fc61b)
    },
    {   UINT64_C(0x000000000000ca8a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1772fe8f0c942511), UINT64_C(0x1772fe8f0c942607)
    },
    {   UINT64_C(0x000000000000ca8b), UINT64_C(0x1772fe8f0c942607), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000ca8c), UINT64_C(0x15f2fe8f1c8e06d1), UINT64_C(0x15f2fe8ca48e0602),
        UINT64_C(0x15f2fe8eb48e0602), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014068), UINT64_C(0x1692f4cf58a346b4), UINT64_C(0x10caf40f6c88e5e3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014069), UINT64_C(0x0000000000000000), UINT64_C(0x1042d50c6c78c689),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001009c), UINT64_C(0x1692f10bfc9685a1), UINT64_C(0x1592f10bfc9285a1),
        UINT64_C(0x12f2d28bfc8705a1), UINT64_C(0x0f92f18bfc7b65a1)
    },
    {   UINT64_C(0x000000000001406b), UINT64_C(0x0000000000000000), UINT64_C(0x0b42ed4f7c76c69b),
        UINT64_C(0x0000000000000000), UINT64_C(0x183afecf58ae270b)
    },
    {   UINT64_C(0x000000000001406c), UINT64_C(0x1162f1cf7c90862f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005af2), UINT64_C(0x1f42fa9020b766d3), UINT64_C(0x1f431a4eb0ad0700),
        UINT64_C(0x1f4300ce08b72741), UINT64_C(0x1f42fccf10b5c6e6)
    },
    {   UINT64_C(0x000000000001009e), UINT64_C(0x14a2f58bfc8d4693), UINT64_C(0x0c92e2cfa066c5a1),
        UINT64_C(0x0bfae2cfa06e8693), UINT64_C(0x0b0ae2cbfc6945a1)
    },
    {   UINT64_C(0x000000000001406f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f74f78786605), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014070), UINT64_C(0x0682f18fa06146cc), UINT64_C(0x0acaf18fa07266cc),
        UINT64_C(0x0b82f18fa07546cc), UINT64_C(0x0962f94f6c6ae694)
    },
    {   UINT64_C(0x00000000000054bb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078f54b10719)
    },
    {   UINT64_C(0x000000000000ca97), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1402fb0f3088075f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000ca98), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1402fb09b4880600)
    },
    {   UINT64_C(0x000000000000ca99), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1772ff0d1c93c577), UINT64_C(0x1772ff100c93c6ef)
    },
    {   UINT64_C(0x000000000000ca9a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1772fed1b093c61d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054c0), UINT64_C(0x1f431f4f2ca2871b), UINT64_C(0x1f43020f6caf6728),
        UINT64_C(0x1f431a0f5ca56707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054c1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43270f00b28734),
        UINT64_C(0x1f43198f24aae72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100a3), UINT64_C(0x0de2f30f5873c5a1), UINT64_C(0x14daf1cf588f85a1),
        UINT64_C(0x0de2f08fa078066f), UINT64_C(0x13cafd8fa089e5b8)
    },
    {   UINT64_C(0x00000000000054c3), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4e70a7e743),
        UINT64_C(0x1f430b8e5caac735), UINT64_C(0x1f42fb0ee0b60722)
    },
    {   UINT64_C(0x000000000000ca9f), UINT64_C(0x0000000000000000), UINT64_C(0x1773020d70924670),
        UINT64_C(0x1773020d70924670), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054c5), UINT64_C(0x0000000000000000), UINT64_C(0x1f4318903cae26c9),
        UINT64_C(0x1f42fbd004b206f6), UINT64_C(0x1f43079028b1a6f5)
    },
    {   UINT64_C(0x000000000000caa1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1773020c4c92466e)
    },
    {   UINT64_C(0x00000000000054c7), UINT64_C(0x1f4314cf34aac6f9), UINT64_C(0x1f43150fc4abe6ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43140ee4ab46e6)
    },
    {   UINT64_C(0x00000000000100a6), UINT64_C(0x169af38f5c96868f), UINT64_C(0x0e62f2cfa07da5b8),
        UINT64_C(0x0efaf30f687945c3), UINT64_C(0x11d2f54f748fc5eb)
    },
    {   UINT64_C(0x00000000000054c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43160f54ab26df), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054ca), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43180f88ada704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054cb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43110f20b0c706),
        UINT64_C(0x1f43188f0caaa6fe), UINT64_C(0x1f430f4f54aec711)
    },
    {   UINT64_C(0x000000000000caa7), UINT64_C(0x0000000000000000), UINT64_C(0x1772feccc493c699),
        UINT64_C(0x1772feccc493c699), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054cd), UINT64_C(0x1f430e0ef0aac70c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43160eb4aa4705)
    },
    {   UINT64_C(0x00000000000054ce), UINT64_C(0x1f430d907cabe69c), UINT64_C(0x1f431450a0ac46db),
        UINT64_C(0x1f431a8f58a7c6da), UINT64_C(0x1f43154f78a7e6ee)
    },
    {   UINT64_C(0x00000000000054cf), UINT64_C(0x1f43190ee4ade6e3), UINT64_C(0x1f43178f5cae06f8),
        UINT64_C(0x1f43128f64b0070f), UINT64_C(0x1f43180f58ab26fb)
    },
    {   UINT64_C(0x00000000000054d0), UINT64_C(0x1f43148f3cac6708), UINT64_C(0x1f4303cef8b1872d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f18b5472a)
    },
    {   UINT64_C(0x00000000000054d1), UINT64_C(0x1f430d8f38af46dc), UINT64_C(0x1f43010f08b5471b),
        UINT64_C(0x1f4308cf20ae270f), UINT64_C(0x1f42ff8e98b4072c)
    },
    {   UINT64_C(0x00000000000054d2), UINT64_C(0x1f4306cee8b36717), UINT64_C(0x1f42ff8e2cb64734),
        UINT64_C(0x1f4303ce7cb2673c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054d3), UINT64_C(0x1f42ff4eacb506fb), UINT64_C(0x1f43120fccab46d9),
        UINT64_C(0x1f430d8ec8ac670b), UINT64_C(0x1f43000f88b026fb)
    },
    {   UINT64_C(0x00000000000054d4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018e1cb36748),
        UINT64_C(0x1f4301ce5cb36734), UINT64_C(0x1f43038e4cb46733)
    },
    {   UINT64_C(0x0000000000003daf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afa0c106c8597), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054d6), UINT64_C(0x1f43064e8cae8727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0e34b4e73c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054d7), UINT64_C(0x1f434f0e7ca5e72e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4305cee8b22722), UINT64_C(0x1f430c4edcada70e)
    },
    {   UINT64_C(0x00000000000054d8), UINT64_C(0x1f4321cecca42718), UINT64_C(0x1f43018e78b26737),
        UINT64_C(0x1f43078f689c870e), UINT64_C(0x1f43210ed0a06729)
    },
    {   UINT64_C(0x00000000000054d9), UINT64_C(0x1f43284f2ca0e711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028f08b22730), UINT64_C(0x1f430dce8cb06716)
    },
    {   UINT64_C(0x00000000000054da), UINT64_C(0x0000000000000000), UINT64_C(0x1f43468f0894a6ae),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f906cabe69f)
    },
    {   UINT64_C(0x00000000000054db), UINT64_C(0x1f43138e5caae6f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f14ac4708), UINT64_C(0x1f43100eacaa271d)
    },
    {   UINT64_C(0x00000000000054dc), UINT64_C(0x1f4310ce6cad271e), UINT64_C(0x1f42f98f1cb0a73d),
        UINT64_C(0x1f42fe8ed8ad06fa), UINT64_C(0x1f430f0e8cad86e6)
    },
    {   UINT64_C(0x00000000000054dd), UINT64_C(0x1f43094e8cad0719), UINT64_C(0x1f43130e9cada727),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054ec4b80739)
    },
    {   UINT64_C(0x00000000000100b1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0a6b04d7506ea67d), UINT64_C(0x0a6b0c575073c67d)
    },
    {   UINT64_C(0x0000000000003db4), UINT64_C(0x106afc181479064b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054e0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8f44a8a6e0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054e1), UINT64_C(0x1f42ff4fb4b246e4), UINT64_C(0x1f42fc4f98a766ec),
        UINT64_C(0x1f42fecf98a5e6de), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054e2), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070fb8af26e9),
        UINT64_C(0x1f42ff4f0ca806de), UINT64_C(0x1f42fd5020ab06e1)
    },
    {   UINT64_C(0x00000000000054e3), UINT64_C(0x1f42fc8fd4b2c6e4), UINT64_C(0x1f42fb8f08ac8706),
        UINT64_C(0x1f42fe4fb8afe6f0), UINT64_C(0x1f42f88ea8a146f5)
    },
    {   UINT64_C(0x00000000000054e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074facaec6d9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b09), UINT64_C(0x1f4318cef8b78734), UINT64_C(0x1f43124e8cafc71a),
        UINT64_C(0x1f430c8ec0b6a72a), UINT64_C(0x1f43064e4cb26738)
    },
    {   UINT64_C(0x00000000000054e6), UINT64_C(0x1f42fc8f98aec701), UINT64_C(0x1f42f88e4caf8726),
        UINT64_C(0x1f42fbceb0b4a722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001409d), UINT64_C(0x1262e74f708b45b6), UINT64_C(0x0fa2ed4f6c75a5b6),
        UINT64_C(0x176af3cf8c9c65b6), UINT64_C(0x156aed4f988cc5c5)
    },
    {   UINT64_C(0x000000000001409e), UINT64_C(0x159aed4c048d8695), UINT64_C(0x0d5aee0fa071c5b6),
        UINT64_C(0x0f6aee0fa07a06a3), UINT64_C(0x0da2ee0bfc72a5b6)
    },
    {   UINT64_C(0x000000000001409f), UINT64_C(0x0c2ad84bfc6f25b6), UINT64_C(0x0c2ae24bfc6f25b6),
        UINT64_C(0x0b42e8cfa072a5cc), UINT64_C(0x0a92f08fa06305cc)
    },
    {   UINT64_C(0x00000000000140a0), UINT64_C(0x0f92f00fa07ee5b8), UINT64_C(0x0f92f00fa08925b8),
        UINT64_C(0x0c2ae60c386825b6), UINT64_C(0x0d42d40bfc878693)
    },
    {   UINT64_C(0x00000000000054eb), UINT64_C(0x1f42f98efcb5870c), UINT64_C(0x1f42f84f5cb6c704),
        UINT64_C(0x1f42f98ef8aec6f3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140a2), UINT64_C(0x09badf0bfc61a63a), UINT64_C(0x1162ea0f7c7ec5b6),
        UINT64_C(0x0fa2f1cf808385ac), UINT64_C(0x10b2f28f787c05b6)
    },
    {   UINT64_C(0x00000000000140a3), UINT64_C(0x0de2f1cf6c70a5b6), UINT64_C(0x14daf1cf6c8b45b6),
        UINT64_C(0x0de2f20fa07d4683), UINT64_C(0x13cafe4fa08a45b8)
    },
    {   UINT64_C(0x00000000000140a4), UINT64_C(0x0dc2e80c387125a9), UINT64_C(0x0cdae80e1c6d85a9),
        UINT64_C(0x0f9aeb8bf08445aa), UINT64_C(0x0f9aeb8ad48085aa)
    },
    {   UINT64_C(0x00000000000140a5), UINT64_C(0x1092f44fa07f8695), UINT64_C(0x1092fe4fa07e2695),
        UINT64_C(0x0f3af44f68772685), UINT64_C(0x0efaf30bec8285b6)
    },
    {   UINT64_C(0x00000000000140a6), UINT64_C(0x169af14f5c9a4690), UINT64_C(0x0f6aeb8fa07b45b8),
        UINT64_C(0x0e8af20f687d85c3), UINT64_C(0x1222f54f7494c602)
    },
    {   UINT64_C(0x00000000000140a7), UINT64_C(0x1232f34fa09305b8), UINT64_C(0x0edae18bfc786695),
        UINT64_C(0x0fa2f38bb88085b1), UINT64_C(0x0f2af08fa07905b8)
    },
    {   UINT64_C(0x00000000000054f2), UINT64_C(0x1f43039010b826cc), UINT64_C(0x1f4313d034b586a5),
        UINT64_C(0x1f4301d090b7c6e9), UINT64_C(0x1f42fecf38b5a723)
    },
    {   UINT64_C(0x00000000000140a9), UINT64_C(0x19a2e8cbf0a426a3), UINT64_C(0x198ae8ce1c9b66a3),
        UINT64_C(0x1182ed4f6c80063a), UINT64_C(0x0e2ae68bfc924693)
    },
    {   UINT64_C(0x00000000000140aa), UINT64_C(0x12c2ed4f588bc63a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054f5), UINT64_C(0x1f42f88f78b6a705), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ef4b826ef), UINT64_C(0x1f430d4ef4b1272e)
    },
    {   UINT64_C(0x00000000000140ac), UINT64_C(0x0be2efd4c06c273c), UINT64_C(0x0be2efd4c46c2731),
        UINT64_C(0x0af2f412606e8746), UINT64_C(0x0c2af355006e869a)
    },
    {   UINT64_C(0x00000000000140ad), UINT64_C(0x0c2aee95006c269a), UINT64_C(0x0c2aee924c6ce69a),
        UINT64_C(0x0c32ef550478c694), UINT64_C(0x0e12f1d4cc78e69a)
    },
    {   UINT64_C(0x00000000000140ae), UINT64_C(0x0af2f0d2606ec69a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6b0e1564680695)
    },
    {   UINT64_C(0x00000000000054f9), UINT64_C(0x1f43098fb8b1671a), UINT64_C(0x1f431b4ed0a9072a),
        UINT64_C(0x1f4314cf5ca2e6ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140b0), UINT64_C(0x0abaed536066e69a), UINT64_C(0x0a32f313606c069a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140b1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0a6af817b06c669a), UINT64_C(0x0a6b01d7b071e69a)
    },
    {   UINT64_C(0x00000000000064c2), UINT64_C(0x1f42f90f28b7a718), UINT64_C(0x1f42f90f00b306fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f78b186fe)
    },
    {   UINT64_C(0x00000000000054fd), UINT64_C(0x0000000000000000), UINT64_C(0x1f43178fbca786d6),
        UINT64_C(0x1f4304ce7cb22715), UINT64_C(0x1f43088f3cad06fb)
    },
    {   UINT64_C(0x00000000000054fe), UINT64_C(0x1f43204f18a5e714), UINT64_C(0x1f43011058b7868a),
        UINT64_C(0x1f42ff1094b30684), UINT64_C(0x1f4300cf64acc6f0)
    },
    {   UINT64_C(0x00000000000054ff), UINT64_C(0x1f4308cfacb0469f), UINT64_C(0x1f43044fdcb766e7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140b6), UINT64_C(0x0a6affd7b06ee68e), UINT64_C(0x0a6b16d7d4632677),
        UINT64_C(0x0a6b0dd8146906da), UINT64_C(0x0a6ae40cec562414)
    },
    {   UINT64_C(0x0000000000005501), UINT64_C(0x1f43000f2cb76724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300d010ae26ec), UINT64_C(0x1f4304ceecb386c7)
    },
    {   UINT64_C(0x0000000000005502), UINT64_C(0x1f42fdcf28b84718), UINT64_C(0x1f43144edcad0730),
        UINT64_C(0x1f43204e9ca4e71d), UINT64_C(0x1f43008e7cb38712)
    },
    {   UINT64_C(0x0000000000005503), UINT64_C(0x1f431c4f24a46707), UINT64_C(0x1f42f18f98a6471f),
        UINT64_C(0x1f42fc0fa8aac707), UINT64_C(0x1f43134eb0b4a736)
    },
    {   UINT64_C(0x0000000000005504), UINT64_C(0x1f430210c4b626e1), UINT64_C(0x1f43130f0caea711),
        UINT64_C(0x1f4303cfdcb646e5), UINT64_C(0x1f43034f7cafe6d0)
    },
    {   UINT64_C(0x0000000000005505), UINT64_C(0x0000000000000000), UINT64_C(0x1f43250fd8a226c4),
        UINT64_C(0x1f43084f48a766df), UINT64_C(0x1f43084ef4aca6f4)
    },
    {   UINT64_C(0x0000000000005506), UINT64_C(0x1f43000e6cb50707), UINT64_C(0x1f42ff0fb8b806f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f68b2e6de)
    },
    {   UINT64_C(0x0000000000005507), UINT64_C(0x1f42fe1050b8469b), UINT64_C(0x1f4306cefcb76739),
        UINT64_C(0x1f42f60efcb506d6), UINT64_C(0x1f42f80e90b7a70c)
    },
    {   UINT64_C(0x00000000000064c8), UINT64_C(0x1f42f84ffcb9c70e), UINT64_C(0x1f42f90ee0b4671d),
        UINT64_C(0x1f42fbce94b48719), UINT64_C(0x1f42f75020b846ec)
    },
    {   UINT64_C(0x0000000000005509), UINT64_C(0x1f43000eb8b726f4), UINT64_C(0x1f43014f08b0469c),
        UINT64_C(0x1f43088f28aec6f2), UINT64_C(0x1f42ff106cb926a1)
    },
    {   UINT64_C(0x000000000000550a), UINT64_C(0x1f42fc0f58b6470a), UINT64_C(0x1f42fd0f74b926fd),
        UINT64_C(0x1f42fe0f64b4e6b7), UINT64_C(0x1f4300ceecb82725)
    },
    {   UINT64_C(0x000000000000550b), UINT64_C(0x1f43014e7cae86ea), UINT64_C(0x1f42f58f84b4c705),
        UINT64_C(0x1f42fd8f68b72710), UINT64_C(0x1f43124fa8a96722)
    },
    {   UINT64_C(0x000000000000550c), UINT64_C(0x1f4302cff8ad070b), UINT64_C(0x1f4305cf48b5a72c),
        UINT64_C(0x1f4303cef8a6e6df), UINT64_C(0x1f43048f70b30704)
    },
    {   UINT64_C(0x000000000000550d), UINT64_C(0x1f43124f70b006fe), UINT64_C(0x1f4300ce88b4c71b),
        UINT64_C(0x1f42fa8eb4a8e6cd), UINT64_C(0x1f42fd0fc0b546f1)
    },
    {   UINT64_C(0x000000000000550e), UINT64_C(0x1f4301d004b766ef), UINT64_C(0x1f4304cf84b74717),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f34b0e717)
    },
    {   UINT64_C(0x000000000000550f), UINT64_C(0x1f42f98ff4b7a6e5), UINT64_C(0x1f42ff0ee8b3271d),
        UINT64_C(0x1f43068efcb3e6fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005510), UINT64_C(0x1f430a8ec4b3c701), UINT64_C(0x1f42ff0f50ac4709),
        UINT64_C(0x1f43280fc8a5c6e3), UINT64_C(0x1f42fd0fc0b646f0)
    },
    {   UINT64_C(0x0000000000005511), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd1010b5e6f7),
        UINT64_C(0x1f430e4ef4b6c701), UINT64_C(0x1f430e8ebcad86e3)
    },
    {   UINT64_C(0x0000000000005512), UINT64_C(0x1f43118fa4b0c6ee), UINT64_C(0x1f4301cf74b1a704),
        UINT64_C(0x1f43008f58b36721), UINT64_C(0x1f43035040b7a70c)
    },
    {   UINT64_C(0x0000000000003dce), UINT64_C(0x106af90f747a8656), UINT64_C(0x106afb0bfc7985a2),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005514), UINT64_C(0x1f43004f14aac700), UINT64_C(0x1f43024f00b7a70f),
        UINT64_C(0x1f4305503cb5a6d3), UINT64_C(0x1f4302cfa8b5e707)
    },
    {   UINT64_C(0x0000000000005515), UINT64_C(0x1f42fcd020b646ed), UINT64_C(0x1f43000f54adc6d7),
        UINT64_C(0x1f4304cfb8a746be), UINT64_C(0x1f4308cf88ab26db)
    },
    {   UINT64_C(0x0000000000005516), UINT64_C(0x1f4325ce989ca711), UINT64_C(0x1f431fcf50a506f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f78b6a6fd)
    },
    {   UINT64_C(0x0000000000005517), UINT64_C(0x1f4304cf10ac2731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300106cb1c713)
    },
    {   UINT64_C(0x0000000000005518), UINT64_C(0x1f42fe0ed8b626f7), UINT64_C(0x1f42fe0f3cb3a6c6),
        UINT64_C(0x1f43064f9cb466df), UINT64_C(0x1f43000f04b546c7)
    },
    {   UINT64_C(0x0000000000005519), UINT64_C(0x1f42ff0f54b6a6e6), UINT64_C(0x1f4302cf48b4e6fe),
        UINT64_C(0x1f42fc8fccaf26e4), UINT64_C(0x1f42ff0fc0b246e9)
    },
    {   UINT64_C(0x000000000000551a), UINT64_C(0x1f4307cf3ca4a70c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa1068b566d3), UINT64_C(0x1f4302cf1ca5e6f2)
    },
    {   UINT64_C(0x000000000000551b), UINT64_C(0x1f4302118cb5c653), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43010eacb486dd)
    },
    {   UINT64_C(0x000000000000551c), UINT64_C(0x1f4308cf38af06ee), UINT64_C(0x1f42fc1034b92708),
        UINT64_C(0x1f42f91050b826c8), UINT64_C(0x1f42fc0e9cbaa727)
    },
    {   UINT64_C(0x000000000000551d), UINT64_C(0x1f4302ceecadc6fb), UINT64_C(0x1f42fe0f8cb3e6ca),
        UINT64_C(0x1f42fb9058b726f4), UINT64_C(0x1f42ff8e3caf86dc)
    },
    {   UINT64_C(0x000000000000551e), UINT64_C(0x1f42fc8fa4b066bd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000f18afa6e1), UINT64_C(0x1f43064ec8ae6738)
    },
    {   UINT64_C(0x000000000000551f), UINT64_C(0x1f4302cf64b5671c), UINT64_C(0x1f42f80e5cb7a747),
        UINT64_C(0x1f4302d0acb166a1), UINT64_C(0x1f4303cfe8af4706)
    },
    {   UINT64_C(0x0000000000005520), UINT64_C(0x1f430250d8b6a6db), UINT64_C(0x1f43010fa8b1a6fc),
        UINT64_C(0x1f43054f10af66c8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005521), UINT64_C(0x1f43024fa4ab26fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8f3cb52739), UINT64_C(0x1f4303cfccb3a6d3)
    },
    {   UINT64_C(0x0000000000005522), UINT64_C(0x1f4302cf18af6709), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8e68aba728)
    },
    {   UINT64_C(0x0000000000005523), UINT64_C(0x1f42fccef8afc712), UINT64_C(0x1f4302cf7cabe6e8),
        UINT64_C(0x1f43000f1cb84715), UINT64_C(0x1f42fd0f2cb72712)
    },
    {   UINT64_C(0x0000000000005524), UINT64_C(0x1f42f90ea4b186fd), UINT64_C(0x1f42fb0efcb526e9),
        UINT64_C(0x1f43098f8cb4a6f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005525), UINT64_C(0x1f43054f04b0a70d), UINT64_C(0x1f43024f98b2e713),
        UINT64_C(0x1f42fb8fc0b1a6f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005526), UINT64_C(0x1f42fe0e9ca8871a), UINT64_C(0x1f43000fb4b40711),
        UINT64_C(0x1f42fd8ee4b8a738), UINT64_C(0x1f4305ced4b04703)
    },
    {   UINT64_C(0x0000000000005527), UINT64_C(0x0000000000000000), UINT64_C(0x1f4300d090ae26e6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005528), UINT64_C(0x1f430b4e2cb08720), UINT64_C(0x1f42fb0e2cb4e6f8),
        UINT64_C(0x1f430b4eb0ada72f), UINT64_C(0x1f42fb9028b5a6ff)
    },
    {   UINT64_C(0x0000000000005529), UINT64_C(0x1f43034fecb2a70b), UINT64_C(0x1f4303d018ad06c7),
        UINT64_C(0x1f42fd0ed8b6472f), UINT64_C(0x1f42fd8ee4aac6f6)
    },
    {   UINT64_C(0x000000000000552a), UINT64_C(0x1f430c4f00b06721), UINT64_C(0x1f43008e3cb5e736),
        UINT64_C(0x1f42fe8ebcba273a), UINT64_C(0x1f42f9d004b386d7)
    },
    {   UINT64_C(0x00000000000140e1), UINT64_C(0x09631117b0666690), UINT64_C(0x0be2f4d7b0758690),
        UINT64_C(0x14dadd0fa088a69d), UINT64_C(0x0a6b0b147c5f85b3)
    },
    {   UINT64_C(0x000000000000552c), UINT64_C(0x1f43034ec8aba703), UINT64_C(0x1f4301cf6ca7470f),
        UINT64_C(0x1f43004f9cb2670b), UINT64_C(0x1f43054f48afc703)
    },
    {   UINT64_C(0x00000000000140e3), UINT64_C(0x1692f0cf5894e69d), UINT64_C(0x0000000000000000),
        UINT64_C(0x15e2f00fa0a9669d), UINT64_C(0x1942f44fa09f469d)
    },
    {   UINT64_C(0x00000000000140e4), UINT64_C(0x08b2ef8f7c72e690), UINT64_C(0x14dadd0fa088a693),
        UINT64_C(0x14dae7cfa08c8685), UINT64_C(0x124aeacfa0924685)
    },
    {   UINT64_C(0x000000000000552f), UINT64_C(0x1f43104e7cac66f6), UINT64_C(0x1f43000f8cad26d1),
        UINT64_C(0x1f42fc0f44b2a712), UINT64_C(0x1f42fa8e8cb4e702)
    },
    {   UINT64_C(0x0000000000005530), UINT64_C(0x1f42f98eb8b72734), UINT64_C(0x1f4309d024af86e6),
        UINT64_C(0x1f42fc901cb286c0), UINT64_C(0x1f43000f50b006cc)
    },
    {   UINT64_C(0x0000000000005531), UINT64_C(0x1f4301cfbcb4c70f), UINT64_C(0x1f43020ee4b3c722),
        UINT64_C(0x1f42fe0e9caaa732), UINT64_C(0x1f4300cf94b1c711)
    },
    {   UINT64_C(0x00000000000140e8), UINT64_C(0x1852e80fa0a26685), UINT64_C(0x131ae70fa08b2685),
        UINT64_C(0x0000000000000000), UINT64_C(0x13caeb4fa08be685)
    },
    {   UINT64_C(0x0000000000005533), UINT64_C(0x1f4301cf90b626f7), UINT64_C(0x1f43010f7cab46f1),
        UINT64_C(0x1f43024fa8b4470b), UINT64_C(0x1f43010f74b3c6ce)
    },
    {   UINT64_C(0x0000000000005534), UINT64_C(0x1f43024fbcb026d0), UINT64_C(0x1f42fe904cb246bd),
        UINT64_C(0x1f42fb8f94b3c6ed), UINT64_C(0x1f43018f48b32721)
    },
    {   UINT64_C(0x0000000000005535), UINT64_C(0x1f43004ee8b0471a), UINT64_C(0x1f43064f20a6e70b),
        UINT64_C(0x1f43000e9cb6071b), UINT64_C(0x1f43044ec8b16713)
    },
    {   UINT64_C(0x0000000000005536), UINT64_C(0x1f42fa4f3cb54710), UINT64_C(0x1f42ff8ff4b726f9),
        UINT64_C(0x1f43004f38b4c6f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005537), UINT64_C(0x1f43014ef8b50712), UINT64_C(0x1f43014f44b6270f),
        UINT64_C(0x1f4301ce5cb2872e), UINT64_C(0x1f42f90f68b046f0)
    },
    {   UINT64_C(0x0000000000005538), UINT64_C(0x1f43000eecb5e70d), UINT64_C(0x1f43228e9ca06738),
        UINT64_C(0x1f42fe0e40b18700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140ef), UINT64_C(0x105ad98c6c7ae5a9), UINT64_C(0x0eaae84f6c75a5b6),
        UINT64_C(0x0cfaeccc6c73c6a2), UINT64_C(0x0fa2ed4f6c75a5b6)
    },
    {   UINT64_C(0x000000000000553a), UINT64_C(0x1f4307cee4ae0725), UINT64_C(0x1f4300907cb626c7),
        UINT64_C(0x1f42fb8ee8b346e8), UINT64_C(0x1f4303cf1cb706e9)
    },
    {   UINT64_C(0x000000000000553b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43068fa4b1e6f3),
        UINT64_C(0x1f4300ce78b2e739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000553c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0e80b3072f),
        UINT64_C(0x1f43030efcb0670e), UINT64_C(0x1f42fd0ee8b4a6f5)
    },
    {   UINT64_C(0x00000000000140f3), UINT64_C(0x0fa2ed4f6c75a690), UINT64_C(0x10c2ec4f6c7a2690),
        UINT64_C(0x16faed4f6c930690), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000553e), UINT64_C(0x1f43000eb4b7470c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90eb4aea71d), UINT64_C(0x1f43020e9cb08702)
    },
    {   UINT64_C(0x000000000000553f), UINT64_C(0x1f42ff0e6caba72e), UINT64_C(0x1f42fb8f48b426ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a8e3cac073a)
    },
    {   UINT64_C(0x0000000000005540), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f48b0671c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e6cac2731)
    },
    {   UINT64_C(0x0000000000005541), UINT64_C(0x1f4305cf58abe6f2), UINT64_C(0x1f42f70e70b7071c),
        UINT64_C(0x1f4302cee4b76724), UINT64_C(0x1f42fccefcb80719)
    },
    {   UINT64_C(0x00000000000064e5), UINT64_C(0x1f42fa0fd8b6c711), UINT64_C(0x1f42f94f4cafe71d),
        UINT64_C(0x1f42fecef0a98716), UINT64_C(0x1f42f64f30b8c70a)
    },
    {   UINT64_C(0x0000000000005543), UINT64_C(0x1f42fc8f9cb766d5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90eacad0708), UINT64_C(0x1f42fd8e6cad273f)
    },
    {   UINT64_C(0x0000000000005544), UINT64_C(0x1f43014eccb0c72f), UINT64_C(0x1f43010f94b2c6f6),
        UINT64_C(0x1f43010f3cae06d1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005545), UINT64_C(0x1f42ff0fa8b626f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ff0b446d2), UINT64_C(0x1f43010f30b68725)
    },
    {   UINT64_C(0x0000000000005546), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbced0b6e6fe), UINT64_C(0x1f43034f20b28706)
    },
    {   UINT64_C(0x00000000000140fd), UINT64_C(0x1692f4cf5894e695), UINT64_C(0x0e92ed4f6c7425b7),
        UINT64_C(0x0f32eccf6073e5b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005548), UINT64_C(0x1f43054fbcb00713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4304cf34a7871e), UINT64_C(0x1f4305cfacb1e714)
    },
    {   UINT64_C(0x0000000000005549), UINT64_C(0x1f43000fa8b1c717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0eccb746fa), UINT64_C(0x1f42fc0e9cb2a6c2)
    },
    {   UINT64_C(0x0000000000014100), UINT64_C(0x0000000000000000), UINT64_C(0x14dae54fa08f2771),
        UINT64_C(0x18ab020c6c9aa685), UINT64_C(0x12e2e60fa090c79b)
    },
    {   UINT64_C(0x000000000000554b), UINT64_C(0x1f4305cf20b76729), UINT64_C(0x1f43084f04b0670a),
        UINT64_C(0x1f4306ceaca72735), UINT64_C(0x1f43020edcb56731)
    },
    {   UINT64_C(0x00000000000064ea), UINT64_C(0x1f42fd0e3cb4c6ff), UINT64_C(0x1f42fd0fa4b4e6e0),
        UINT64_C(0x1f42fa0f10b646fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000554d), UINT64_C(0x1f4301cf5cb32726), UINT64_C(0x1f42fc8f0cb5471b),
        UINT64_C(0x1f43044eb0b00714), UINT64_C(0x1f42fe0e98af0729)
    },
    {   UINT64_C(0x000000000000554e), UINT64_C(0x1f42f90f9cb6c723), UINT64_C(0x1f42f78facb6c6e5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034e9cb4c717)
    },
    {   UINT64_C(0x0000000000014105), UINT64_C(0x15c2d14cc0984685), UINT64_C(0x14dae54fa08f2685),
        UINT64_C(0x1cfade4fa0b7a68e), UINT64_C(0x118aeb8f58832696)
    },
    {   UINT64_C(0x0000000000005550), UINT64_C(0x1f43054f54b1c6d8), UINT64_C(0x1f42fecec8a98702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4305ce44b48718)
    },
    {   UINT64_C(0x0000000000014107), UINT64_C(0x1bd2e58f70b08685), UINT64_C(0x180ae60fa0a16685),
        UINT64_C(0x1692fb4f58982695), UINT64_C(0x17fae54fa0a12685)
    },
    {   UINT64_C(0x0000000000014108), UINT64_C(0x1272e68fa08ae685), UINT64_C(0x14dafe4f58974695),
        UINT64_C(0x131b020fa093a685), UINT64_C(0x14d306cf58978685)
    },
    {   UINT64_C(0x0000000000014109), UINT64_C(0x1a0b000fa0a48685), UINT64_C(0x12fadecfa090a685),
        UINT64_C(0x1812e60cc0a18685), UINT64_C(0x17b30c0fa0a00685)
    },
    {   UINT64_C(0x0000000000005554), UINT64_C(0x1f43000e6caf6716), UINT64_C(0x1f43000eecb5e70c),
        UINT64_C(0x1f43000fc8ad86d5), UINT64_C(0x1f43014f28b8e72a)
    },
    {   UINT64_C(0x0000000000005555), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cee8af273a),
        UINT64_C(0x1f43054fa8b44710), UINT64_C(0x1f42fd0ef4af070a)
    },
    {   UINT64_C(0x00000000000100ed), UINT64_C(0x0e62f40f74780684), UINT64_C(0x0de2f58f7473a5a1),
        UINT64_C(0x0e32f58cbc79e693), UINT64_C(0x0ed2f18c6c75c5a1)
    },
    {   UINT64_C(0x000000000001410d), UINT64_C(0x0f82f38f6c78c69f), UINT64_C(0x0d7af38f6c7265b6),
        UINT64_C(0x1c12ed0f6cadc5b6), UINT64_C(0x0c82eb4f6c6dc5b6)
    },
    {   UINT64_C(0x0000000000005558), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8eccb1471a),
        UINT64_C(0x1f42ff0ff4aca6cc), UINT64_C(0x1f43000eb8b34710)
    },
    {   UINT64_C(0x0000000000005559), UINT64_C(0x1f43000e6cb5a739), UINT64_C(0x1f43014f1cb726f9),
        UINT64_C(0x1f42fd8f48b4e709), UINT64_C(0x1f4306ce98b24729)
    },
    {   UINT64_C(0x000000000000555a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cf40ab66fc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014111), UINT64_C(0x1772f50c6c9826a3), UINT64_C(0x0e42e48f6c7e65b6),
        UINT64_C(0x10f2f68cd47e85b6), UINT64_C(0x109afa0f7c7d25b6)
    },
    {   UINT64_C(0x0000000000014112), UINT64_C(0x1902f70f7c9ec5b6), UINT64_C(0x1902f70cd49ec5b6),
        UINT64_C(0x1292f54f6c8405b6), UINT64_C(0x1792f8cf6c9905b6)
    },
    {   UINT64_C(0x0000000000014113), UINT64_C(0x0f5af8cf6c7725b6), UINT64_C(0x0de2eb0f6c7125b6),
        UINT64_C(0x1722f10f6c9745b6), UINT64_C(0x10d2ee8f747e05a9)
    },
    {   UINT64_C(0x0000000000014114), UINT64_C(0x0000000000000000), UINT64_C(0x0d92f38f6c7b6677),
        UINT64_C(0x105af10c6c7bc6a3), UINT64_C(0x0b82ec0f687786a1)
    },
    {   UINT64_C(0x0000000000014115), UINT64_C(0x0cfaf38f6c76c5b6), UINT64_C(0x0c2aee0f6c6a85b6),
        UINT64_C(0x08b2f30f7c618694), UINT64_C(0x08b2f38f7c6e4677)
    },
    {   UINT64_C(0x0000000000014116), UINT64_C(0x08b2c5cc6464e695), UINT64_C(0x15b2e20f6c9085b6),
        UINT64_C(0x16eaf10f6c9565b6), UINT64_C(0x1162f10f7c7ee6a3)
    },
    {   UINT64_C(0x0000000000005561), UINT64_C(0x1f42fd4ebcb70743), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8fb8b2c71e), UINT64_C(0x1f42fd0ef8aa870f)
    },
    {   UINT64_C(0x0000000000005562), UINT64_C(0x1f42fd0ec0b22709), UINT64_C(0x1f42fdd000b446fd),
        UINT64_C(0x1f42fc0e70b4e727), UINT64_C(0x1f42fe8e6cb6871d)
    },
    {   UINT64_C(0x0000000000005563), UINT64_C(0x1f430e8e4ca7a71a), UINT64_C(0x1f42fe0e6cb92706),
        UINT64_C(0x1f4301cf40af66c1), UINT64_C(0x1f43000f3cb0e719)
    },
    {   UINT64_C(0x0000000000005564), UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cea0b2671d),
        UINT64_C(0x1f42fe8ee0b5e70b), UINT64_C(0x1f4305cefcb646ff)
    },
    {   UINT64_C(0x0000000000005565), UINT64_C(0x1f42fe0f40b3c6e2), UINT64_C(0x1f42ff102cb366db),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005566), UINT64_C(0x1f4300cef0afe713), UINT64_C(0x1f43000eb4b8e73e),
        UINT64_C(0x1f4301cf94b62729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005567), UINT64_C(0x1f43004e50b50740), UINT64_C(0x1f4303cef4af6709),
        UINT64_C(0x1f42fe0e9cb3a6f6), UINT64_C(0x1f43000ea4aa6715)
    },
    {   UINT64_C(0x00000000000064f8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f24b56721),
        UINT64_C(0x1f42fecefcb28703), UINT64_C(0x1f42fb0ff4b2470e)
    },
    {   UINT64_C(0x0000000000005569), UINT64_C(0x1f43010f80b22726), UINT64_C(0x1f42fd0ee0b3671a),
        UINT64_C(0x1f430e0f4caca709), UINT64_C(0x1f4300cfb4b2c713)
    },
    {   UINT64_C(0x000000000000556a), UINT64_C(0x1f43000f3cb3871a), UINT64_C(0x1f42f90f80b60739),
        UINT64_C(0x1f43150e4cb6e738), UINT64_C(0x1f42fd8f5cb3c6f6)
    },
    {   UINT64_C(0x000000000000556b), UINT64_C(0x1f430bce8cb9672f), UINT64_C(0x1f43000e9ca9271d),
        UINT64_C(0x1f43038e5cb1e73a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000556c), UINT64_C(0x1f42f88e8cb6471d), UINT64_C(0x1f42fd0fc0b346d9),
        UINT64_C(0x1f42ff8ee0b3070e), UINT64_C(0x1f42fd8f6cb36700)
    },
    {   UINT64_C(0x000000000000556d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f40b2470e),
        UINT64_C(0x1f43104f58aa66fc), UINT64_C(0x1f43020ed0af8731)
    },
    {   UINT64_C(0x0000000000014124), UINT64_C(0x10dae60fa08ea72a), UINT64_C(0x0b42f5cf7c77a636),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000556f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0efcafc714),
        UINT64_C(0x1f43148edcabc6fd), UINT64_C(0x1f43140ef8b14718)
    },
    {   UINT64_C(0x0000000000005570), UINT64_C(0x1f43024f44b646ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010ef0b2e725), UINT64_C(0x1f42fb0e98abe721)
    },
    {   UINT64_C(0x0000000000005571), UINT64_C(0x1f43014f0cb3670c), UINT64_C(0x1f4301cec8b44724),
        UINT64_C(0x1f42fb8ec0b8871e), UINT64_C(0x1f43018e54b2073b)
    },
    {   UINT64_C(0x0000000000005572), UINT64_C(0x1f43010ebcb4c726), UINT64_C(0x1f42fd8e20b1e728),
        UINT64_C(0x1f42fe0ebcb166f1), UINT64_C(0x1f43054ed4ad8725)
    },
    {   UINT64_C(0x0000000000005573), UINT64_C(0x1f42fd0f54b5a70b), UINT64_C(0x1f42ff8efcb0e6f3),
        UINT64_C(0x1f4304cec0b28724), UINT64_C(0x1f42ff8e90b7271c)
    },
    {   UINT64_C(0x00000000000064fe), UINT64_C(0x1f42fb0f98b5e72f), UINT64_C(0x1f42fdcfe4b16703),
        UINT64_C(0x1f42fc0eecb68706), UINT64_C(0x1f42fb0ebcb6e710)
    },
    {   UINT64_C(0x0000000000005575), UINT64_C(0x1f4300cefcafe72a), UINT64_C(0x1f42fb0ee4b12709),
        UINT64_C(0x1f43004e6cb56719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064ff), UINT64_C(0x1f42fc0eccb4c711), UINT64_C(0x1f42f90eb4b0e725),
        UINT64_C(0x1f42fc0fe8b5a711), UINT64_C(0x1f42fb0ebcb46725)
    },
    {   UINT64_C(0x0000000000005577), UINT64_C(0x1f4303ce74b4673d), UINT64_C(0x1f43000fb0b0e6f0),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cfbcb44739)
    },
    {   UINT64_C(0x0000000000005578), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0e64b6c721),
        UINT64_C(0x1f42fe1094b4a706), UINT64_C(0x1f43030f08acc6ee)
    },
    {   UINT64_C(0x0000000000005579), UINT64_C(0x1f42ff0efcb4070e), UINT64_C(0x1f43034e5cb7473b),
        UINT64_C(0x1f42fd8e60b1c723), UINT64_C(0x1f43054ec8b52715)
    },
    {   UINT64_C(0x000000000000557a), UINT64_C(0x1f42f98f04b5c70d), UINT64_C(0x1f4304cf2ca7a722),
        UINT64_C(0x1f42fe0eecb5e725), UINT64_C(0x1f4302cfd8b346f6)
    },
    {   UINT64_C(0x0000000000005b54), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0f08b346fb),
        UINT64_C(0x1f43078eb8b36702), UINT64_C(0x1f43088f24b2271a)
    },
    {   UINT64_C(0x0000000000010100), UINT64_C(0x0000000000000000), UINT64_C(0x131aee4fa08d6740),
        UINT64_C(0x169aff8c60916675), UINT64_C(0x11cae50fa089e77a)
    },
    {   UINT64_C(0x000000000000557d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f98b346f1),
        UINT64_C(0x1f42f88e6cb5a71b), UINT64_C(0x1f42fc8f74b7870e)
    },
    {   UINT64_C(0x000000000000557e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f18b5a705),
        UINT64_C(0x1f43020eb8b0672f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000557f), UINT64_C(0x1f43024f9cb2e723), UINT64_C(0x1f42f64f48b5c6e8),
        UINT64_C(0x1f42fb8ff0b5c6f9), UINT64_C(0x1f42fc0e60b82747)
    },
    {   UINT64_C(0x0000000000005580), UINT64_C(0x1f430b8eb8aca715), UINT64_C(0x1f43014f3cb426fa),
        UINT64_C(0x1f4307cf28afa70e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005581), UINT64_C(0x1f42ff0e6cb3c731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cf10b5673d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005582), UINT64_C(0x1f42fa0ec4ac473e), UINT64_C(0x1f43058e9cb1c748),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005583), UINT64_C(0x1f42fe8ef0aca6ff), UINT64_C(0x1f42ff8ea8b6c733),
        UINT64_C(0x1f43014e58ad871f), UINT64_C(0x1f42fe5050b606e2)
    },
    {   UINT64_C(0x0000000000005584), UINT64_C(0x1f42ff0ea0b6a731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014f8cb3e709), UINT64_C(0x1f42fc8f78ae46be)
    },
    {   UINT64_C(0x0000000000005585), UINT64_C(0x1f43034f70b486e3), UINT64_C(0x1f43024eecae66f3),
        UINT64_C(0x1f4303ce34b38738), UINT64_C(0x1f43134e60b14736)
    },
    {   UINT64_C(0x0000000000005586), UINT64_C(0x1f42fe8f54aa6722), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f0cb68726)
    },
    {   UINT64_C(0x0000000000005587), UINT64_C(0x1f42fb0e4cb6c729), UINT64_C(0x1f42fb0e9cb906ef),
        UINT64_C(0x1f42f88f88b18722), UINT64_C(0x1f42f84e9cb1e6e7)
    },
    {   UINT64_C(0x0000000000005588), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f0cab271b)
    },
    {   UINT64_C(0x0000000000005589), UINT64_C(0x1f42fe0f48b44709), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f78aca70a)
    },
    {   UINT64_C(0x0000000000010107), UINT64_C(0x1972e5cfa0a96675), UINT64_C(0x1622e78fa09c2675),
        UINT64_C(0x1642fd0f5895e683), UINT64_C(0x1602e6cfa09ba675)
    },
    {   UINT64_C(0x000000000000558b), UINT64_C(0x1f43014f08b666f5), UINT64_C(0x1f42fc8f30ae670e),
        UINT64_C(0x1f42fece8cb7e708), UINT64_C(0x1f4301ceccb3a734)
    },
    {   UINT64_C(0x0000000000010108), UINT64_C(0x11c2e80fa086c675), UINT64_C(0x14dafecf5894c682),
        UINT64_C(0x1162ff8fa08f0675), UINT64_C(0x1483098f68980675)
    },
    {   UINT64_C(0x000000000000558d), UINT64_C(0x1f42ff8e70b54726), UINT64_C(0x1f42fe0f00b3e741),
        UINT64_C(0x1f42f54e6cb3a723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010109), UINT64_C(0x1823004fa09d4675), UINT64_C(0x11fadd4fa08ba675),
        UINT64_C(0x1612e7ccd49be675), UINT64_C(0x1372fb8fa0916675)
    },
    {   UINT64_C(0x000000000000558f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024e5cb0670e),
        UINT64_C(0x1f43024e8cb0671a), UINT64_C(0x1f42fa8f6cb04701)
    },
    {   UINT64_C(0x000000000001010a), UINT64_C(0x1d42ea8fa0b8a675), UINT64_C(0x1d42ea8cd4b8a675),
        UINT64_C(0x15bafdcfa0900675), UINT64_C(0x1aa2e70fa0ae2675)
    },
    {   UINT64_C(0x0000000000008c0c), UINT64_C(0x2c22f95114e96589), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000650d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70e58b6473b),
        UINT64_C(0x1f42f80f1cb56705), UINT64_C(0x1f42fa0eccb3c727)
    },
    {   UINT64_C(0x0000000000005593), UINT64_C(0x1f42ff0f2cad2732), UINT64_C(0x1f42ff0fb8b0670c),
        UINT64_C(0x1f42fb0f1cb08739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001010c), UINT64_C(0x1012f3cf747ac5a1), UINT64_C(0x0f92f70f74816693),
        UINT64_C(0x0dd2f5cf747365a1), UINT64_C(0x0c32ee4f746e85a1)
    },
    {   UINT64_C(0x0000000000005595), UINT64_C(0x1f42fc0fa4b506f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004eccb0e73f)
    },
    {   UINT64_C(0x0000000000005596), UINT64_C(0x1f42fa4fccb826c1), UINT64_C(0x1f42fd8eb4b92724),
        UINT64_C(0x1f43040eb0b0670d), UINT64_C(0x1f42ff0f88b0670e)
    },
    {   UINT64_C(0x0000000000005597), UINT64_C(0x1f42fc0ef0b10741), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0fccb72718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005598), UINT64_C(0x1f42f84f40b3e71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014fa0b2c734), UINT64_C(0x1f43004ed8ace6e8)
    },
    {   UINT64_C(0x0000000000005599), UINT64_C(0x1f42fd0e78b6c730), UINT64_C(0x1f43010eb4b6073b),
        UINT64_C(0x1f43004edcb20706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000559a), UINT64_C(0x1f4314cf78aaa715), UINT64_C(0x1f43028fa4b16712),
        UINT64_C(0x1f42fd8eccb78715), UINT64_C(0x1f42fe8ee4b4e73a)
    },
    {   UINT64_C(0x000000000000559b), UINT64_C(0x1f42fd8fd4b4a71a), UINT64_C(0x1f42f88f28b42702),
        UINT64_C(0x1f42fe0e68b6c70c), UINT64_C(0x1f42fa8e70b16724)
    },
    {   UINT64_C(0x000000000000559c), UINT64_C(0x1f42f98f18b6a72d), UINT64_C(0x1f42f98e68b5c730),
        UINT64_C(0x1f42fa8ee4b5a730), UINT64_C(0x1f43004f14b04724)
    },
    {   UINT64_C(0x000000000000559d), UINT64_C(0x1f43000ea0b26748), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4302ce88b0273f), UINT64_C(0x1f42fe8fecb5c703)
    },
    {   UINT64_C(0x000000000000559e), UINT64_C(0x1f42fa8f70b3e70b), UINT64_C(0x1f42fd8ed4b3c72c),
        UINT64_C(0x1f42fd0ea0b40708), UINT64_C(0x1f42fe0f8cb5c70c)
    },
    {   UINT64_C(0x000000000000559f), UINT64_C(0x1f42fc0ed8b1670e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8ea4b0a725), UINT64_C(0x1f42fe8e8cb64715)
    },
    {   UINT64_C(0x00000000000055a0), UINT64_C(0x1f42fd8e48afe736), UINT64_C(0x1f42fb8e98ab6701),
        UINT64_C(0x1f42fb8f8cb246dd), UINT64_C(0x1f42f3cf60b646bd)
    },
    {   UINT64_C(0x00000000000055a1), UINT64_C(0x1f42fb8ea4b3072b), UINT64_C(0x1f42f98e84aee6f9),
        UINT64_C(0x1f42fb0ea0ae8739), UINT64_C(0x1f42f84e88b62725)
    },
    {   UINT64_C(0x00000000000055a2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4e8cb4e743), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055a3), UINT64_C(0x1f42ff8eacb76721), UINT64_C(0x1f42ff8f44b3e72a),
        UINT64_C(0x1f42fa4e60b3e738), UINT64_C(0x1f42f98e70b4a722)
    },
    {   UINT64_C(0x00000000000055a4), UINT64_C(0x1f42fb8e34b36745), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ee0b50731)
    },
    {   UINT64_C(0x00000000000055a5), UINT64_C(0x1f42f98e80b22722), UINT64_C(0x1f42fa8e60abc741),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0edcb9071b)
    },
    {   UINT64_C(0x0000000000010115), UINT64_C(0x0d62f48f7475a5a1), UINT64_C(0x0c2aebcf6c6925a2),
        UINT64_C(0x08b2f8cf745ec676), UINT64_C(0x08b2f8cf746b2646)
    },
    {   UINT64_C(0x00000000000055a7), UINT64_C(0x1f42fd0ee8b306f5), UINT64_C(0x1f42fe8e70af6735),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010116), UINT64_C(0x08b2c30c645b6674), UINT64_C(0x153ae30f708d65a1),
        UINT64_C(0x15caf34f748fa5a1), UINT64_C(0x0ffaf30f7c7a6693)
    },
    {   UINT64_C(0x00000000000055a9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43040fa8b1c720),
        UINT64_C(0x1f43014f70b566d2), UINT64_C(0x1f43000f1cb1072c)
    },
    {   UINT64_C(0x00000000000055aa), UINT64_C(0x1f42fa4f20b3e71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f00ae66fa), UINT64_C(0x1f42f60f9cb76706)
    },
    {   UINT64_C(0x00000000000055ab), UINT64_C(0x1f42fc0fdcae870c), UINT64_C(0x1f42f50f5cb0a722),
        UINT64_C(0x1f4313ce48ac072e), UINT64_C(0x1f42fe0ec8ada703)
    },
    {   UINT64_C(0x0000000000010118), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x154af2cfa08c2675), UINT64_C(0x10f2f14f747d8693)
    },
    {   UINT64_C(0x00000000000055ad), UINT64_C(0x1f42f4ce8cb6e732), UINT64_C(0x1f42ff8e8cb1a733),
        UINT64_C(0x1f42ff0f48b4471f), UINT64_C(0x1f42fe0e38b4672b)
    },
    {   UINT64_C(0x000000000000651b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cedcb426f2),
        UINT64_C(0x1f42f80e90b5472b), UINT64_C(0x1f42f80e5cb7a704)
    },
    {   UINT64_C(0x00000000000055af), UINT64_C(0x1f4304cf0cb2e73b), UINT64_C(0x1f43010e80b5873d),
        UINT64_C(0x1f43034ea8ad6735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055b0), UINT64_C(0x1f42f88ebcb7e72c), UINT64_C(0x1f42fe0e5cb4e72a),
        UINT64_C(0x1f43058ec0ae272e), UINT64_C(0x1f42fb0ee8b6c6fe)
    },
    {   UINT64_C(0x00000000000055b1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f28b78702),
        UINT64_C(0x1f43010e78b1274f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055b2), UINT64_C(0x1f42fd0ec8b66740), UINT64_C(0x1f42fd0ef8b46710),
        UINT64_C(0x1f43064e34b38744), UINT64_C(0x1f43020e50b50731)
    },
    {   UINT64_C(0x00000000000055b3), UINT64_C(0x1f42f84ebcb7c736), UINT64_C(0x1f43000ee0b60721),
        UINT64_C(0x1f42f98f2cb5c733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e80aca730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0eb8af8738)
    },
    {   UINT64_C(0x00000000000055b5), UINT64_C(0x1f42ff8e70b1a731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43188f58a8e6ed)
    },
    {   UINT64_C(0x00000000000055b6), UINT64_C(0x1f43174f24a6e708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431bcfe8a5a6f5), UINT64_C(0x1f430b5164ad6677)
    },
    {   UINT64_C(0x00000000000055b7), UINT64_C(0x0000000000000000), UINT64_C(0x1f43178f8ca5e703),
        UINT64_C(0x1f430e50b8af66b2), UINT64_C(0x1f43244ec0a9c72a)
    },
    {   UINT64_C(0x00000000000055b8), UINT64_C(0x1f431acf7ca686f9), UINT64_C(0x1f43180f88a6e6f9),
        UINT64_C(0x1f431acf38a606e7), UINT64_C(0x1f431acf38a606e7)
    },
    {   UINT64_C(0x00000000000055b9), UINT64_C(0x1f43190f68a606ff), UINT64_C(0x1f431b4edca8c6e3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431a4f18a766f7)
    },
    {   UINT64_C(0x00000000000055ba), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4323cec0a6272d), UINT64_C(0x1f43178eaca686e7)
    },
    {   UINT64_C(0x00000000000055bb), UINT64_C(0x0000000000000000), UINT64_C(0x1f431a4f38a546e7),
        UINT64_C(0x1f43174f48a5e6fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055bc), UINT64_C(0x1f43178f98a56701), UINT64_C(0x1f4325cf2ca1c711),
        UINT64_C(0x1f43178f88a6a6f9), UINT64_C(0x1f43178f88a6a6f9)
    },
    {   UINT64_C(0x00000000000055bd), UINT64_C(0x1f431a0f68a786f0), UINT64_C(0x1f43190f38a606e7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43174f68a6c6fc)
    },
    {   UINT64_C(0x00000000000055be), UINT64_C(0x0000000000000000), UINT64_C(0x1f43194f68a766fd),
        UINT64_C(0x1f431acf38a6e6f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055bf), UINT64_C(0x0000000000000000), UINT64_C(0x1f431a8f58a806e7),
        UINT64_C(0x1f431a4f38a506ea), UINT64_C(0x1f43170f28a8e708)
    },
    {   UINT64_C(0x00000000000055c0), UINT64_C(0x1f43180f18a686f8), UINT64_C(0x1f43198f28a706ed),
        UINT64_C(0x1f43194f28a766ee), UINT64_C(0x1f43188f48a766ff)
    },
    {   UINT64_C(0x00000000000055c1), UINT64_C(0x1f43184fa4a4a711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43194f18a6e6fc), UINT64_C(0x1f43134ebca7e710)
    },
    {   UINT64_C(0x00000000000055c2), UINT64_C(0x1f43188f28a8a706), UINT64_C(0x1f43160f58a74718),
        UINT64_C(0x1f43150eeca7471b), UINT64_C(0x1f43190f38a8c714)
    },
    {   UINT64_C(0x00000000000055c3), UINT64_C(0x1f43170f18a5a700), UINT64_C(0x1f43128ecca5870e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43160f28a76706)
    },
    {   UINT64_C(0x00000000000055c4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43174f08a9270c), UINT64_C(0x1f43120ecca6870a)
    },
    {   UINT64_C(0x00000000000055c5), UINT64_C(0x1f43174f18a8e6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43160eeca74711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055c6), UINT64_C(0x1f4315cecca6e721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431a8f38a70706), UINT64_C(0x1f43190efca64707)
    },
    {   UINT64_C(0x00000000000055c7), UINT64_C(0x1f43190f1ca6c703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43148ebcaac71e), UINT64_C(0x1f430a506cb366e9)
    },
    {   UINT64_C(0x00000000000055c8), UINT64_C(0x1f42fb4fa0b326f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0ee0b466ee), UINT64_C(0x1f43154f90b2c6f5)
    },
    {   UINT64_C(0x00000000000055c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f250c0b966e2), UINT64_C(0x1f43058fa0afc6dc)
    },
    {   UINT64_C(0x0000000000006529), UINT64_C(0x1f42fb0f7cb626ff), UINT64_C(0x1f42ff0f44a6e70e),
        UINT64_C(0x1f42fa5000b4c704), UINT64_C(0x1f42fa0f18ad8717)
    },
    {   UINT64_C(0x00000000000055cb), UINT64_C(0x1f42fcced4b20709), UINT64_C(0x1f43114f70ace715),
        UINT64_C(0x1f42ff8facb4c6e6), UINT64_C(0x1f42fad0acb586cb)
    },
    {   UINT64_C(0x00000000000055cc), UINT64_C(0x1f4300cffcb9e725), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ceeca86722)
    },
    {   UINT64_C(0x00000000000055cd), UINT64_C(0x1f4303cf90ad4710), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f7cb266ee)
    },
    {   UINT64_C(0x00000000000055ce), UINT64_C(0x1f4302111cb1a6dd), UINT64_C(0x1f42fe8f94aca70b),
        UINT64_C(0x1f4304cf78b0e6de), UINT64_C(0x1f42fa8e7cb1e705)
    },
    {   UINT64_C(0x00000000000055cf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88fbcb746de), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055d0), UINT64_C(0x1f42fe9090b526e1), UINT64_C(0x1f42fc4f50b6e6e5),
        UINT64_C(0x1f42fe9050b606e2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055d1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010f5cb806ed),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028ef4ad472a)
    },
    {   UINT64_C(0x0000000000000740), UINT64_C(0x106af74fa07b259f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055d3), UINT64_C(0x1f4310cf70b486ff), UINT64_C(0x1f42f98f1cb74711),
        UINT64_C(0x1f43004f64b4a71a), UINT64_C(0x1f42fc4f4cb0c6f6)
    },
    {   UINT64_C(0x00000000000055d4), UINT64_C(0x1f43108fb0b04701), UINT64_C(0x1f42fecfccb5e713),
        UINT64_C(0x1f431a0fa0adc704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000743), UINT64_C(0x0000000000000000), UINT64_C(0x106af70fa07b8596),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055d6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f4fc0b446df), UINT64_C(0x1f42fe509cb086fe)
    },
    {   UINT64_C(0x00000000000055d7), UINT64_C(0x1f42fd8fc0ab66d5), UINT64_C(0x1f42fecf78a906fe),
        UINT64_C(0x1f42fdcefcaea72e), UINT64_C(0x1f43019028b326f3)
    },
    {   UINT64_C(0x0000000000000746), UINT64_C(0x106afa4fa079e597), UINT64_C(0x106af00f807aa596),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000747), UINT64_C(0x106af50f807c8596), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055da), UINT64_C(0x1f42fd0fc8b146eb), UINT64_C(0x1f42fecf6cb226e9),
        UINT64_C(0x1f42ff4eacb56729), UINT64_C(0x1f42fe4f2ca6e6e1)
    },
    {   UINT64_C(0x00000000000055db), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff1000b386d2),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f95020ace6db)
    },
    {   UINT64_C(0x00000000000055dc), UINT64_C(0x1f42ff8f54af46d2), UINT64_C(0x1f43020fb4b4c6e8),
        UINT64_C(0x1f42ff0f18af66f2), UINT64_C(0x1f42ff0f70afe6d9)
    },
    {   UINT64_C(0x00000000000055dd), UINT64_C(0x1f43134f94aca70a), UINT64_C(0x1f42fdd064af06c6),
        UINT64_C(0x1f42fd4f44b5e6ce), UINT64_C(0x1f42f64f34b50701)
    },
    {   UINT64_C(0x00000000000055de), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f94b2a6e0)
    },
    {   UINT64_C(0x00000000000055df), UINT64_C(0x1f42fcce9cb0a710), UINT64_C(0x1f42f84f44b8a716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74ef0b6e6e5)
    },
    {   UINT64_C(0x00000000000055e0), UINT64_C(0x1f42fd8f00b206e2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cf88b4c6ff), UINT64_C(0x1f42fd103cb1c6e3)
    },
    {   UINT64_C(0x00000000000055e1), UINT64_C(0x1f43114fb4afc70d), UINT64_C(0x1f42fe8ff4b18701),
        UINT64_C(0x1f43024ef8a8a70a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055e2), UINT64_C(0x1f43028ee0b1470a), UINT64_C(0x1f43000f74b2870a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030f70ab071f)
    },
    {   UINT64_C(0x00000000000055e3), UINT64_C(0x1f42fd0f18ab66d7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcce88b46725)
    },
    {   UINT64_C(0x00000000000055e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b8f94af4732), UINT64_C(0x1f42fb0f50adc6f9)
    },
    {   UINT64_C(0x00000000000055e5), UINT64_C(0x1f430d8f50aee6fc), UINT64_C(0x1f4310ceecade72e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4cfd8b886e0)
    },
    {   UINT64_C(0x00000000000055e6), UINT64_C(0x1f42f44fc4b906de), UINT64_C(0x1f42f7cf14b466fc),
        UINT64_C(0x1f43020efcb06732), UINT64_C(0x1f42facf18b6c6ff)
    },
    {   UINT64_C(0x00000000000055e7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f7cb4471b),
        UINT64_C(0x1f42fd0f44b2c6ef), UINT64_C(0x1f4300ce3cb70733)
    },
    {   UINT64_C(0x00000000000055e8), UINT64_C(0x1f42fd0f34b726ed), UINT64_C(0x1f43040f68aa66f6),
        UINT64_C(0x1f42fb0fa8ad26f3), UINT64_C(0x1f42fa5064b5e6ee)
    },
    {   UINT64_C(0x00000000000055e9), UINT64_C(0x1f42f5cfa0b3e6d6), UINT64_C(0x1f42f74f2cb8e6f1),
        UINT64_C(0x1f43134f9cad4706), UINT64_C(0x1f42fccfdcaa26e3)
    },
    {   UINT64_C(0x00000000000055ea), UINT64_C(0x0000000000000000), UINT64_C(0x1f4311cf9cab26f0),
        UINT64_C(0x1f42fa0eacb5671b), UINT64_C(0x1f430f4f90aec71b)
    },
    {   UINT64_C(0x00000000000055eb), UINT64_C(0x0000000000000000), UINT64_C(0x1f4304d0ccb346e2),
        UINT64_C(0x1f42f90e98b90739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055ec), UINT64_C(0x1f42fd0e9cb40705), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43020f88b0870f), UINT64_C(0x1f42f70eccb70715)
    },
    {   UINT64_C(0x00000000000055ed), UINT64_C(0x1f42fe10a0ad86e0), UINT64_C(0x1f43000f4caf86f2),
        UINT64_C(0x1f42fa4e5cb1872d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055ee), UINT64_C(0x1f42f78f68b3e6ef), UINT64_C(0x1f42fecf08b186de),
        UINT64_C(0x1f42fd8ec4b0070c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055ef), UINT64_C(0x1f430c4f30aac6f2), UINT64_C(0x1f43154f30ae2724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98ef4b2a721)
    },
    {   UINT64_C(0x00000000000055f0), UINT64_C(0x1f43054e3cafe72b), UINT64_C(0x1f4301cf2cb7471a),
        UINT64_C(0x1f42f9cf10b5e6d9), UINT64_C(0x1f42fd4f78b5c72e)
    },
    {   UINT64_C(0x00000000000055f1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8edca94727), UINT64_C(0x1f42f68ed8aaa712)
    },
    {   UINT64_C(0x00000000000055f2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc9070b6a6ee)
    },
    {   UINT64_C(0x00000000000055f3), UINT64_C(0x1f42fa4eb8b7270e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84f2cb4273f), UINT64_C(0x1f42ffcfd0b746dc)
    },
    {   UINT64_C(0x00000000000055f4), UINT64_C(0x1f42f80f10b3c704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78edcb46739), UINT64_C(0x1f42fa8eccb56724)
    },
    {   UINT64_C(0x00000000000055f5), UINT64_C(0x1f42fa8f00ac0719), UINT64_C(0x1f42ff9028b066e4),
        UINT64_C(0x1f42fa4fb4b54702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000764), UINT64_C(0x106af94c148b259f), UINT64_C(0x106af48c147e8651),
        UINT64_C(0x106afbcc148c859f), UINT64_C(0x106af84c148725a2)
    },
    {   UINT64_C(0x0000000000000765), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106afc8c1476e59c)
    },
    {   UINT64_C(0x00000000000055f8), UINT64_C(0x1f43018e4cb4c751), UINT64_C(0x1f42fe8fc8b146d7),
        UINT64_C(0x1f42fa4f14b5a707), UINT64_C(0x1f43040e6caee71d)
    },
    {   UINT64_C(0x00000000000055f9), UINT64_C(0x1f42fd8eb8b3e741), UINT64_C(0x1f43030e58ae8721),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f50aee6ee)
    },
    {   UINT64_C(0x0000000000006541), UINT64_C(0x1f42fa0f8cb1270d), UINT64_C(0x1f42f90fe4af0706),
        UINT64_C(0x1f42fa0fc8b26706), UINT64_C(0x1f42fa0f60af2705)
    },
    {   UINT64_C(0x00000000000055fb), UINT64_C(0x1f43020e8cb54735), UINT64_C(0x1f42ff4f00b2c70d),
        UINT64_C(0x1f43030ef8b1c6f8), UINT64_C(0x1f43014f94b72704)
    },
    {   UINT64_C(0x00000000000055fc), UINT64_C(0x1f42fd4f48b4c726), UINT64_C(0x1f42fb5020b326f6),
        UINT64_C(0x1f43008e4cb58726), UINT64_C(0x1f4304cfc0ae470b)
    },
    {   UINT64_C(0x00000000000055fd), UINT64_C(0x1f43028f48b02707), UINT64_C(0x1f42f68f84b44709),
        UINT64_C(0x1f43078efcb3a71a), UINT64_C(0x1f42fd8edcb4e72d)
    },
    {   UINT64_C(0x00000000000055fe), UINT64_C(0x1f43014f54a9a6eb), UINT64_C(0x1f42fbcf6cb6471d),
        UINT64_C(0x1f42fd4f3cb8e745), UINT64_C(0x1f43008e6cb26729)
    },
    {   UINT64_C(0x00000000000055ff), UINT64_C(0x1f42fd4e74b1c71e), UINT64_C(0x1f42fece7cb2a6f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005600), UINT64_C(0x1f43004f74aa06cc), UINT64_C(0x1f4301cec8b4c6ca),
        UINT64_C(0x1f430e0f6cb48731), UINT64_C(0x1f43078f58aec71d)
    },
    {   UINT64_C(0x0000000000005601), UINT64_C(0x1f43034fccb10700), UINT64_C(0x1f4312cf84b78703),
        UINT64_C(0x1f42f9cefcb3e72d), UINT64_C(0x1f43180f24ac4734)
    },
    {   UINT64_C(0x0000000000005602), UINT64_C(0x1f42fd8f58aa06d6), UINT64_C(0x1f42fe8e2cb1e716),
        UINT64_C(0x1f43114f08ab870e), UINT64_C(0x1f43008f74ae06f2)
    },
    {   UINT64_C(0x0000000000005603), UINT64_C(0x1f42fdcf5cac66e1), UINT64_C(0x1f43008f88b02721),
        UINT64_C(0x1f42fe8ed8b5a71e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005604), UINT64_C(0x1f42fd0fd0afc6e6), UINT64_C(0x1f4308cf6caae715),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005605), UINT64_C(0x1f42fe8f18b346d8), UINT64_C(0x1f431acf50a7671b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005606), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307d008ae66e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f50aee6e2)
    },
    {   UINT64_C(0x0000000000005607), UINT64_C(0x1f42fb4e4cb7a739), UINT64_C(0x1f42facf6cb106e2),
        UINT64_C(0x1f4300cebcb54721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005608), UINT64_C(0x1f42f74fa4b646fb), UINT64_C(0x1f4303cf58b2e718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf94ae8709)
    },
    {   UINT64_C(0x0000000000005b9b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43104ff4b146f6), UINT64_C(0x1f43120e90b4e71e)
    },
    {   UINT64_C(0x000000000000560a), UINT64_C(0x1f42ff0eb0ade6fc), UINT64_C(0x1f42fa4edcb826ef),
        UINT64_C(0x1f42fc4f34b46725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b9c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098f2cab870b),
        UINT64_C(0x1f43078ee4b38721), UINT64_C(0x1f4304cf08aa0723)
    },
    {   UINT64_C(0x000000000000560c), UINT64_C(0x1f42f88f2cb146f5), UINT64_C(0x1f42f94f58b6e72a),
        UINT64_C(0x1f431bcf7caa86d4), UINT64_C(0x1f4304cf24ac66f2)
    },
    {   UINT64_C(0x0000000000005b9d), UINT64_C(0x1f4309cf94aee6f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040f0cb0670b), UINT64_C(0x1f43034e94b64717)
    },
    {   UINT64_C(0x000000000000654b), UINT64_C(0x1f42fa0f80b34712), UINT64_C(0x1f42fb1010b68712),
        UINT64_C(0x1f42f70fa4b6e711), UINT64_C(0x1f42f80fa8b1470f)
    },
    {   UINT64_C(0x000000000000560f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf7cb266f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f68b086fa)
    },
    {   UINT64_C(0x000000000000654c), UINT64_C(0x1f42fa102cb426fa), UINT64_C(0x1f42fa0fbcaea706),
        UINT64_C(0x1f42fa0ff4b2c706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b9f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c4e68ab8729), UINT64_C(0x1f430ad020ae46db)
    },
    {   UINT64_C(0x0000000000000780), UINT64_C(0x106af4cb247ca657), UINT64_C(0x106afbcc0c8005a0),
        UINT64_C(0x106af3cfa07ac6ec), UINT64_C(0x106af10f887b86e1)
    },
    {   UINT64_C(0x0000000000000781), UINT64_C(0x106af3cd487ac64b), UINT64_C(0x106af10d347b864b),
        UINT64_C(0x106af3cdc07ac64b), UINT64_C(0x106af10dc87b864b)
    },
    {   UINT64_C(0x0000000000000782), UINT64_C(0x106af80c607b2701), UINT64_C(0x106b000c6078a656),
        UINT64_C(0x106af2cfa07b26ec), UINT64_C(0x106af70f887e86e1)
    },
    {   UINT64_C(0x0000000000000783), UINT64_C(0x106af2cd487b264b), UINT64_C(0x106af70d347e864b),
        UINT64_C(0x106af2cdc07b264b), UINT64_C(0x106af70dc47e864b)
    },
    {   UINT64_C(0x0000000000000784), UINT64_C(0x106af2cba07b264b), UINT64_C(0x106af70b8c7e864b),
        UINT64_C(0x106af2cc687b264b), UINT64_C(0x106af70c547e864b)
    },
    {   UINT64_C(0x0000000000005ba2), UINT64_C(0x1f43044ef0b06729), UINT64_C(0x1f43028ff0b5e6c7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000786), UINT64_C(0x106af48c687d464b), UINT64_C(0x106afbcc5479c5a0),
        UINT64_C(0x106af48cd47d464b), UINT64_C(0x106afbcce47b25a0)
    },
    {   UINT64_C(0x0000000000000787), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af50fa07bc6ec), UINT64_C(0x106af88fa07526e6)
    },
    {   UINT64_C(0x0000000000000788), UINT64_C(0x106af78f747c86ce), UINT64_C(0x106af54c0c7e2613),
        UINT64_C(0x106af38fa07d46ec), UINT64_C(0x106af10fa07e86e1)
    },
    {   UINT64_C(0x0000000000000789), UINT64_C(0x106af38d487d464b), UINT64_C(0x106af10d487e864b),
        UINT64_C(0x106af38fa07d46ec), UINT64_C(0x106af10fa07e86e1)
    },
    {   UINT64_C(0x000000000000078a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af38ca47d464b), UINT64_C(0x106af10ca47e864b)
    },
    {   UINT64_C(0x000000000000078b), UINT64_C(0x106afc8cd478a64b), UINT64_C(0x106b018cf877664c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000078c), UINT64_C(0x106af48fa07c6701), UINT64_C(0x106af34fa07e0702),
        UINT64_C(0x106af48d487c664b), UINT64_C(0x106af34d487e064b)
    },
    {   UINT64_C(0x000000000000078d), UINT64_C(0x106af48dc07c664b), UINT64_C(0x106af34ddc7e064b),
        UINT64_C(0x106af60d487b864b), UINT64_C(0x106afc8d4878e64b)
    },
    {   UINT64_C(0x000000000000078e), UINT64_C(0x106af60d487b86ce), UINT64_C(0x106aeb0d4878e6bd),
        UINT64_C(0x106af60dc07b864b), UINT64_C(0x106afc8ddc78e64b)
    },
    {   UINT64_C(0x000000000000078f), UINT64_C(0x106af60c687b864b), UINT64_C(0x106afc8c6878e64b),
        UINT64_C(0x106af24fa07de701), UINT64_C(0x106aed0fa0808656)
    },
    {   UINT64_C(0x0000000000000790), UINT64_C(0x106af24fa07de6ec), UINT64_C(0x106aed0fa0808634),
        UINT64_C(0x106af24d487de64b), UINT64_C(0x106aed0d4880859d)
    },
    {   UINT64_C(0x0000000000000791), UINT64_C(0x106af50fa07c86ec), UINT64_C(0x106af10fa07e8634),
        UINT64_C(0x106af50d487c864b), UINT64_C(0x106af10d487e859d)
    },
    {   UINT64_C(0x0000000000000792), UINT64_C(0x106af50dc07c864b), UINT64_C(0x106af10ddc7e859d),
        UINT64_C(0x106af50c687c864b), UINT64_C(0x106af10c687e859d)
    },
    {   UINT64_C(0x0000000000005625), UINT64_C(0x1f4301ce98b1670f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8e80afc72c)
    },
    {   UINT64_C(0x0000000000006557), UINT64_C(0x1f42f84f20b5a725), UINT64_C(0x1f42fa0ebcad4716),
        UINT64_C(0x1f42f70ffcb5c702), UINT64_C(0x1f42f90fb8b646fe)
    },
    {   UINT64_C(0x0000000000000795), UINT64_C(0x106af48fa07a8701), UINT64_C(0x106af10c287b8656),
        UINT64_C(0x106af48fa07a86ec), UINT64_C(0x106af10c287b8634)
    },
    {   UINT64_C(0x0000000000005628), UINT64_C(0x1f4303cea8aee703), UINT64_C(0x1f42f94edcb38732),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf2cb20716)
    },
    {   UINT64_C(0x0000000000005629), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f18b3c72a),
        UINT64_C(0x1f42fecef0b3270e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000562a), UINT64_C(0x1f4307cf04aea715), UINT64_C(0x1f42fe0eecb8870e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcec0b3673a)
    },
    {   UINT64_C(0x000000000000562b), UINT64_C(0x1f431ccf44b24729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108ea4aa6739), UINT64_C(0x1f430c8f44a68722)
    },
    {   UINT64_C(0x000000000000079a), UINT64_C(0x106afb0d1c7986ec), UINT64_C(0x106b000d3477e635),
        UINT64_C(0x106af48fa07cc6ec), UINT64_C(0x106af44f8876a6a7)
    },
    {   UINT64_C(0x000000000000079b), UINT64_C(0x106af48d487cc64b), UINT64_C(0x106af44d3476a612),
        UINT64_C(0x106af48dc07cc64b), UINT64_C(0x106af44dc4774612)
    },
    {   UINT64_C(0x000000000000562e), UINT64_C(0x1f43004eecb2670b), UINT64_C(0x1f43030ebcb0e6fc),
        UINT64_C(0x1f42ff4f60b2e70d), UINT64_C(0x1f43018f20b00729)
    },
    {   UINT64_C(0x000000000000562f), UINT64_C(0x1f430e8f20b3c70d), UINT64_C(0x1f42f68f24b726ff),
        UINT64_C(0x1f430a8f00b4e727), UINT64_C(0x1f43010ec8a90707)
    },
    {   UINT64_C(0x0000000000005630), UINT64_C(0x0000000000000000), UINT64_C(0x1f43064f10ad0725),
        UINT64_C(0x1f430a0f54b3e6ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005631), UINT64_C(0x0000000000000000), UINT64_C(0x1f43164ee0ae4739),
        UINT64_C(0x1f430ccf60b006ea), UINT64_C(0x1f42f84f38af472e)
    },
    {   UINT64_C(0x0000000000005632), UINT64_C(0x1f4301cebcb3272e), UINT64_C(0x1f42fc8eacb5c711),
        UINT64_C(0x1f42f70f34b6272f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005633), UINT64_C(0x1f42f74e94b6c72b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccedcb2e720), UINT64_C(0x1f42fa4f88b526ee)
    },
    {   UINT64_C(0x0000000000005634), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68ed4b78733),
        UINT64_C(0x1f42f98f3cb4272f), UINT64_C(0x1f42f94eacb4471d)
    },
    {   UINT64_C(0x0000000000005635), UINT64_C(0x1f42ff4e48b5074c), UINT64_C(0x1f42fc0e64b86739),
        UINT64_C(0x1f42fbcf24b4c6f1), UINT64_C(0x1f42fbcf4cb4a72d)
    },
    {   UINT64_C(0x00000000000007a4), UINT64_C(0x106afa0fa07aa724), UINT64_C(0x106af64fa079e66d),
        UINT64_C(0x106afa0d487aa64b), UINT64_C(0x106af64d4879e597)
    },
    {   UINT64_C(0x0000000000005637), UINT64_C(0x1f42f94e9cb5e71d), UINT64_C(0x1f42fd4f14b6070c),
        UINT64_C(0x1f42f70eb4b7073e), UINT64_C(0x1f4305ceb0b2072b)
    },
    {   UINT64_C(0x00000000000007a6), UINT64_C(0x106af20f907e067e), UINT64_C(0x106af48c107d067e),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af88fa07526e7)
    },
    {   UINT64_C(0x0000000000005639), UINT64_C(0x1f42fc4e38b5e72b), UINT64_C(0x1f42f88f90b6473a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e10b72712)
    },
    {   UINT64_C(0x00000000000007a8), UINT64_C(0x106af4cd3c7c8657), UINT64_C(0x106afbcd3c80059f),
        UINT64_C(0x106af4cfa07ca773), UINT64_C(0x106afbcf888006bf)
    },
    {   UINT64_C(0x00000000000007a9), UINT64_C(0x106af4cfa07ca7c4), UINT64_C(0x106afbcf8880070f),
        UINT64_C(0x106af4cfa07ca7c4), UINT64_C(0x106afbcf8880070f)
    },
    {   UINT64_C(0x00000000000007aa), UINT64_C(0x106af4cfa07ca806), UINT64_C(0x106afbcf88800751),
        UINT64_C(0x106af4cfa07ca7aa), UINT64_C(0x106afbcf888006f5)
    },
    {   UINT64_C(0x000000000000563d), UINT64_C(0x1f43134ed8afc72f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43114f80ac0716), UINT64_C(0x1f43068f1caf2728)
    },
    {   UINT64_C(0x000000000000563e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4eb0b2473c),
        UINT64_C(0x1f430d4ed8b26728), UINT64_C(0x1f42f5ce60b7a704)
    },
    {   UINT64_C(0x000000000000563f), UINT64_C(0x1f430d4eb8ae272e), UINT64_C(0x1f43024e68b28739),
        UINT64_C(0x1f430b8fa8af470f), UINT64_C(0x1f42fe8e90b26739)
    },
    {   UINT64_C(0x0000000000005640), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431094a4afe726), UINT64_C(0x1f42ffcebcae26f2)
    },
    {   UINT64_C(0x00000000000007af), UINT64_C(0x106af48fa07d4709), UINT64_C(0x106afbcf88788671),
        UINT64_C(0x106af48fa07d47c4), UINT64_C(0x106afbcf8878870f)
    },
    {   UINT64_C(0x00000000000007b0), UINT64_C(0x106af48fa07d47c4), UINT64_C(0x106afbcf8878870f),
        UINT64_C(0x106af48fa07d4806), UINT64_C(0x106afbcf88788751)
    },
    {   UINT64_C(0x00000000000007b1), UINT64_C(0x106af48fa07d47aa), UINT64_C(0x106afbcf887886f5),
        UINT64_C(0x106af48d3c7d470b), UINT64_C(0x106afbcd4078a657)
    },
    {   UINT64_C(0x00000000000007b2), UINT64_C(0x106b01cfa0764773), UINT64_C(0x106b018fa07766bf),
        UINT64_C(0x106b01cd3c76464b), UINT64_C(0x106b018d3c77664b)
    },
    {   UINT64_C(0x00000000000007b3), UINT64_C(0x106afc0d3c790656), UINT64_C(0x106afc4d4078e5a0),
        UINT64_C(0x106afc0f74790773), UINT64_C(0x106afc4f8878e6bf)
    },
    {   UINT64_C(0x00000000000007b4), UINT64_C(0x106afc0f747907c4), UINT64_C(0x106afc4f8878e70f),
        UINT64_C(0x106afc0f747907c4), UINT64_C(0x106afc4f8878e70f)
    },
    {   UINT64_C(0x00000000000007b5), UINT64_C(0x106afc0f74790806), UINT64_C(0x106afc4f8878e751),
        UINT64_C(0x106afc0f747907aa), UINT64_C(0x106afc4f8878e6f5)
    },
    {   UINT64_C(0x00000000000007b6), UINT64_C(0x106afc0d3c79070b), UINT64_C(0x106afc4d4078e657),
        UINT64_C(0x106afc0f747c6732), UINT64_C(0x106afc0f8c78e67d)
    },
    {   UINT64_C(0x0000000000005649), UINT64_C(0x1f43094e60b02751), UINT64_C(0x1f42fd0ea0afa71f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4ff4b3a711)
    },
    {   UINT64_C(0x00000000000007b8), UINT64_C(0x106afc0f747c6729), UINT64_C(0x106afc0f8c78e674),
        UINT64_C(0x106afc0d407c66af), UINT64_C(0x106afc0d3c78e5fa)
    },
    {   UINT64_C(0x000000000000564b), UINT64_C(0x1f430e0fa0ad670c), UINT64_C(0x1f430a0fdcb4a6f2),
        UINT64_C(0x1f430c4f90b146f4), UINT64_C(0x1f43050e9cb64733)
    },
    {   UINT64_C(0x000000000000564c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43110facb6c72e),
        UINT64_C(0x1f42fd0e94b40739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000564d), UINT64_C(0x1f43094fa4ae06e9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e94b3e733)
    },
    {   UINT64_C(0x000000000000564e), UINT64_C(0x1f42fe4e3cb6472d), UINT64_C(0x1f430a8f70aee70c),
        UINT64_C(0x1f43030ef0b6e739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007bd), UINT64_C(0x106af28d3c7da64b), UINT64_C(0x106af48c107d0597),
        UINT64_C(0x106af2cfa07da773), UINT64_C(0x106af48c107d06bf)
    },
    {   UINT64_C(0x0000000000005650), UINT64_C(0x1f431a0f68ab86df), UINT64_C(0x1f43048f10b7271a),
        UINT64_C(0x1f430b8f60afe720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005651), UINT64_C(0x1f42fc4ed8b44737), UINT64_C(0x1f43074e6cb4c727),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43074e3cb3e73c)
    },
    {   UINT64_C(0x00000000000007c0), UINT64_C(0x106af8cf8c804667), UINT64_C(0x106af8cf8c804667),
        UINT64_C(0x106af8cf8c804667), UINT64_C(0x106af8cf8c804667)
    },
    {   UINT64_C(0x0000000000005653), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060f10b8e72a), UINT64_C(0x1f42ff8e88b6e737)
    },
    {   UINT64_C(0x0000000000005654), UINT64_C(0x1f430d4f1cb1472e), UINT64_C(0x1f43124f40b4e728),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008e48b8273b)
    },
    {   UINT64_C(0x0000000000005655), UINT64_C(0x1f430c4e64b8273e), UINT64_C(0x1f43104ef0b1e72d),
        UINT64_C(0x1f43134f84afe714), UINT64_C(0x1f430a0ed0b7a740)
    },
    {   UINT64_C(0x0000000000005656), UINT64_C(0x1f430f8ec4b0e720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098f0caf072a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005657), UINT64_C(0x1f43138f10a586fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006570), UINT64_C(0x1f43178ec4a76708), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4311ce90a62702)
    },
    {   UINT64_C(0x0000000000005659), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024ec8b30729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024ec8b30744)
    },
    {   UINT64_C(0x00000000000007c8), UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667),
        UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667)
    },
    {   UINT64_C(0x00000000000007c9), UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667),
        UINT64_C(0x106af2cbfc7ac69e), UINT64_C(0x106af2cbfc7ac69e)
    },
    {   UINT64_C(0x000000000000565c), UINT64_C(0x1f43138f30aa070e), UINT64_C(0x1f43110ee4abe737),
        UINT64_C(0x1f42f90e9cb24711), UINT64_C(0x1f42fb4f78b7c710)
    },
    {   UINT64_C(0x000000000000565d), UINT64_C(0x1f42fe5078b4c70d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f94af8700), UINT64_C(0x1f42f64ee8b386ed)
    },
    {   UINT64_C(0x00000000000007cc), UINT64_C(0x106b0f0f8c6b2667), UINT64_C(0x106b13cf8c6b4667),
        UINT64_C(0x106b034f8c6b4667), UINT64_C(0x106b010f8c6b4667)
    },
    {   UINT64_C(0x00000000000007cd), UINT64_C(0x106afdcf8c6b4667), UINT64_C(0x106afd4f8c6b4667),
        UINT64_C(0x106af40f8c6d469e), UINT64_C(0x106af40f8c6d469e)
    },
    {   UINT64_C(0x0000000000005660), UINT64_C(0x1f43120fd4b3e6dc), UINT64_C(0x1f42f84ef4aee71d),
        UINT64_C(0x1f42fa5088afc700), UINT64_C(0x1f42f950a0b586fd)
    },
    {   UINT64_C(0x0000000000005661), UINT64_C(0x1f43024f04b4872c), UINT64_C(0x1f42f44fecb3e6f5),
        UINT64_C(0x1f42ff0fb8ad66e0), UINT64_C(0x1f42f91018ac870e)
    },
    {   UINT64_C(0x0000000000005662), UINT64_C(0x1f42f94f4cad06f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f3901cb066f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005663), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f48ed8b1e71a),
        UINT64_C(0x1f42f78e6cb706e9), UINT64_C(0x1f42f85014b466eb)
    },
    {   UINT64_C(0x0000000000007dad), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000007dae), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x00000000000007d4), UINT64_C(0x106af8cf8c7a8667), UINT64_C(0x106af8cf8c7a8667),
        UINT64_C(0x106af8cf8c7a8667), UINT64_C(0x106af8cf8c7a8667)
    },
    {   UINT64_C(0x0000000000005667), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcfe4b346f3),
        UINT64_C(0x1f42f650a4b3c705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005668), UINT64_C(0x1f42f78f78af46ef), UINT64_C(0x1f42f98e6cb026ed),
        UINT64_C(0x1f42fc0f68b626dc), UINT64_C(0x1f42f50ff8b606f3)
    },
    {   UINT64_C(0x0000000000005669), UINT64_C(0x1f42f68e7cb42715), UINT64_C(0x1f42fccf1cb1a712),
        UINT64_C(0x1f42f80ed4af2721), UINT64_C(0x1f42f64fe8b446f5)
    },
    {   UINT64_C(0x0000000000007db3), UINT64_C(0x1f42f54f7cb7e6ca), UINT64_C(0x1f4306cfd8af26ca),
        UINT64_C(0x1f4324cfd8a025d4), UINT64_C(0x1f434a565c8946a5)
    },
    {   UINT64_C(0x00000000000007d9), UINT64_C(0x106af60f8c7c0667), UINT64_C(0x106af60f8c7c0667),
        UINT64_C(0x106af60f8c7c069e), UINT64_C(0x106af60f8c7c069e)
    },
    {   UINT64_C(0x0000000000007db5), UINT64_C(0x1f4313d744aa66c1), UINT64_C(0x1f434d509887c694),
        UINT64_C(0x1f431a4f5ca58707), UINT64_C(0x1f430d907cabe69c)
    },
    {   UINT64_C(0x000000000000566d), UINT64_C(0x1f43098e6cafe6e4), UINT64_C(0x1f42f70fe0b4c6ee),
        UINT64_C(0x1f42f9100cb186f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007dc), UINT64_C(0x106af8cf8c80464c), UINT64_C(0x106af8cf8c80464c),
        UINT64_C(0x106af8cf747a864c), UINT64_C(0x106af8cf747a864c)
    },
    {   UINT64_C(0x00000000000007dd), UINT64_C(0x106af2cbfc7ac64c), UINT64_C(0x106af2cbfc7ac64c),
        UINT64_C(0x106b0c8f8c6b464c), UINT64_C(0x106b13cf8c6b464c)
    },
    {   UINT64_C(0x00000000000007de), UINT64_C(0x106af84f8c7ae64c), UINT64_C(0x106af84f8c7ae64c),
        UINT64_C(0x106af8cf8c7a864c), UINT64_C(0x106af8cf8c7a864c)
    },
    {   UINT64_C(0x00000000000007df), UINT64_C(0x106af60f8c7c064c), UINT64_C(0x106af60f8c7c064c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007e0), UINT64_C(0x106af8ccc0804667), UINT64_C(0x106af8ccc0804667),
        UINT64_C(0x106af8ccc0804667), UINT64_C(0x106af8ccc0804667)
    },
    {   UINT64_C(0x00000000000007e1), UINT64_C(0x106af8ccc0804667), UINT64_C(0x106af8ccc0804667),
        UINT64_C(0x106af8ccc080469e), UINT64_C(0x106af8ccc080469e)
    },
    {   UINT64_C(0x0000000000007dbd), UINT64_C(0x1f43820e2ca4c73c), UINT64_C(0x1f431b8e2c71c73c),
        UINT64_C(0x1f432dd1189bc696), UINT64_C(0x1f43218e509ea665)
    },
    {   UINT64_C(0x0000000000007dbe), UINT64_C(0x1f43118e58a80716), UINT64_C(0x1f431fd178a1465e),
        UINT64_C(0x1f430a5118a8c6b5), UINT64_C(0x1f43308ee893e647)
    },
    {   UINT64_C(0x00000000000007e4), UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667),
        UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667)
    },
    {   UINT64_C(0x0000000000005677), UINT64_C(0x1f42ffce6cb2c749), UINT64_C(0x1f42f74eacb5c716),
        UINT64_C(0x1f42f74f4cad06f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dc1), UINT64_C(0x1f43141038a68708), UINT64_C(0x1f43294f008ee703),
        UINT64_C(0x1f431c0f00a34703), UINT64_C(0x1f43150e90a466d6)
    },
    {   UINT64_C(0x0000000000005679), UINT64_C(0x1f42fb4fc4adc6f9), UINT64_C(0x1f42f60f70b186fb),
        UINT64_C(0x1f42ff4edcb2c73c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007e8), UINT64_C(0x106af60cc07c0667), UINT64_C(0x106af60cc07c0667),
        UINT64_C(0x106af60cc07c0667), UINT64_C(0x106af60cc07c0667)
    },
    {   UINT64_C(0x000000000000567b), UINT64_C(0x1f42f70ffcb306d8), UINT64_C(0x1f43078e80aee721),
        UINT64_C(0x1f42fb9000b546f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006582), UINT64_C(0x1f43144ee0a646f5), UINT64_C(0x1f43130f08a846eb),
        UINT64_C(0x1f43144ef4a686f1), UINT64_C(0x1f43130ee0a666eb)
    },
    {   UINT64_C(0x000000000000567d), UINT64_C(0x1f430e8f20b0c717), UINT64_C(0x1f42f78f58b746ee),
        UINT64_C(0x1f42f54f3cb906f3), UINT64_C(0x1f42f68ec4b0671a)
    },
    {   UINT64_C(0x00000000000007ec), UINT64_C(0x106af38f8c82c670), UINT64_C(0x106af38f8c82c60f),
        UINT64_C(0x106af8ccc080464c), UINT64_C(0x106af8ccc080459c)
    },
    {   UINT64_C(0x000000000000567f), UINT64_C(0x1f42feced4ae4736), UINT64_C(0x1f42f64fc0b2871f),
        UINT64_C(0x1f42f60f70b5c6d6), UINT64_C(0x1f42fcce8cb54730)
    },
    {   UINT64_C(0x0000000000005680), UINT64_C(0x1f42f80eb0b50700), UINT64_C(0x1f43044e60b4872a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0fd8b1c6fe)
    },
    {   UINT64_C(0x0000000000005681), UINT64_C(0x1f430a8eb4ae671c), UINT64_C(0x1f42f650a8af86f6),
        UINT64_C(0x1f42f94fa0b44705), UINT64_C(0x1f42f70f38b5c6fc)
    },
    {   UINT64_C(0x0000000000005682), UINT64_C(0x1f430a4ee8b0070f), UINT64_C(0x1f42f74e70b4c73c),
        UINT64_C(0x1f42fa4f28b826e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005683), UINT64_C(0x1f42f61010b366f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f00b8c705)
    },
    {   UINT64_C(0x0000000000005684), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0e54b42719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54f38b76706)
    },
    {   UINT64_C(0x00000014400007f3), UINT64_C(0x106af0cd847f0667), UINT64_C(0x106b13170470c667),
        UINT64_C(0x106b0d97046ce667), UINT64_C(0x106b03d70875069e)
    },
    {   UINT64_C(0x0000000000005686), UINT64_C(0x1f42f5ce88b76730), UINT64_C(0x1f42fccf6cb7c6f9),
        UINT64_C(0x1f43024f3cb166e2), UINT64_C(0x1f42fa0f94b6c6f3)
    },
    {   UINT64_C(0x00000000000007f5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af40f8c6d4667), UINT64_C(0x106af40f8c6d469e)
    },
    {   UINT64_C(0x00000000000007f6), UINT64_C(0x106aedcfa07fe728), UINT64_C(0x106aedcfa07fe6c7),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aeb4fa07fe667)
    },
    {   UINT64_C(0x00000015000007f7), UINT64_C(0x0000000000000000), UINT64_C(0x106b10d70470a667),
        UINT64_C(0x106b0d17046d0667), UINT64_C(0x106b03d70875069e)
    },
    {   UINT64_C(0x00000000000007f8), UINT64_C(0x106aedcf8c7fe673), UINT64_C(0x106aedcf8c7fe612),
        UINT64_C(0x106af8cf8c7a8683), UINT64_C(0x106af8cf8c7a8683)
    },
    {   UINT64_C(0x000000000000568b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facef4b6470e),
        UINT64_C(0x1f42fccf2cb506fb), UINT64_C(0x1f42f8cf94b626f3)
    },
    {   UINT64_C(0x000000000000568c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64fdcb4c6ee),
        UINT64_C(0x1f42f48f40ac86fa), UINT64_C(0x1f42f54f28aec6e3)
    },
    {   UINT64_C(0x000000000000568d), UINT64_C(0x1f42f80fa0ad4703), UINT64_C(0x1f42fa4f48b5a6e4),
        UINT64_C(0x1f42fa0f0cb7471a), UINT64_C(0x1f42f58fd0b4c6eb)
    },
    {   UINT64_C(0x00000000000007fc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af60cc07c064c), UINT64_C(0x106af60cc07c05a1)
    },
    {   UINT64_C(0x000000000000568f), UINT64_C(0x1f42f58fccafe6fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8fa0b2c6f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007fe), UINT64_C(0x0000000000000000), UINT64_C(0x106ae4cf707fa672),
        UINT64_C(0x0000000000000000), UINT64_C(0x106ade4fa07ea672)
    },
    {   UINT64_C(0x00000000000007ff), UINT64_C(0x106af34d847de672), UINT64_C(0x106b1fd70467064b),
        UINT64_C(0x106b1c573463e667), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000800), UINT64_C(0x0000000000000000), UINT64_C(0x106aee0fa03e83e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0fa03e83e8)
    },
    {   UINT64_C(0x0000000000000801), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0fa03e83e8)
    },
    {   UINT64_C(0x0000000000000802), UINT64_C(0x106aee0fa03e83e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dde), UINT64_C(0x1f430751acace647), UINT64_C(0x1f430751acb0e6fb),
        UINT64_C(0x1f42fd11acaf86ed), UINT64_C(0x1f432a5090986692)
    },
    {   UINT64_C(0x0000000000000804), UINT64_C(0x106b03d2847504de), UINT64_C(0x106b03d2847504de),
        UINT64_C(0x106b03d2847504de), UINT64_C(0x106b03d2847504de)
    },
    {   UINT64_C(0x0000000000000805), UINT64_C(0x106b03d2847504de), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b150e546c869e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000806), UINT64_C(0x106b18950c698690), UINT64_C(0x106b1b13f86ac64b),
        UINT64_C(0x106b1b0d7469c465), UINT64_C(0x106b1b13f86ac64b)
    },
    {   UINT64_C(0x0000000000000807), UINT64_C(0x106af0950c7d664a), UINT64_C(0x106af3550c7ec64b),
        UINT64_C(0x106af34d787ec465), UINT64_C(0x106af3550c7ec64b)
    },
    {   UINT64_C(0x0000000000000808), UINT64_C(0x106afe4bfc77c650), UINT64_C(0x106afe4c1477c650),
        UINT64_C(0x106afc913878c5e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000809), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afa4fa079c465), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000080a), UINT64_C(0x0000000000000000), UINT64_C(0x106afd4c287c264b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007de6), UINT64_C(0x1f431f10109e0673), UINT64_C(0x1f43108f84a7c6e2),
        UINT64_C(0x1f431051b0a7e685), UINT64_C(0x1f4327902c9d2675)
    },
    {   UINT64_C(0x000000000000080c), UINT64_C(0x106af44e547d069e), UINT64_C(0x106af2ce547d869e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007de8), UINT64_C(0x1f432b50c89d0698), UINT64_C(0x1f432b90c894268e),
        UINT64_C(0x1f432c90c49c468e), UINT64_C(0x1f432c90c89c468e)
    },
    {   UINT64_C(0x000000000000080e), UINT64_C(0x0000000000000000), UINT64_C(0x106afe4fa06f859f),
        UINT64_C(0x106b0ecfa077c59f), UINT64_C(0x106af60fa07c05d2)
    },
    {   UINT64_C(0x000000000000080f), UINT64_C(0x0000000000000000), UINT64_C(0x106afd4fa0784656),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0bfc8043be)
    },
    {   UINT64_C(0x0000000000000810), UINT64_C(0x106b04968c74c667), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000811), UINT64_C(0x106afc8e5478c69e), UINT64_C(0x106b134e4070a69e),
        UINT64_C(0x106b0cce546d669e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007ded), UINT64_C(0x1f432c90a09c468e), UINT64_C(0x1f432c90c89c468e),
        UINT64_C(0x1f432c50c89d668e), UINT64_C(0x1f43270f689e468e)
    },
    {   UINT64_C(0x0000000000007dee), UINT64_C(0x1f4328d0c897c68e), UINT64_C(0x1f431490c4a2e68e),
        UINT64_C(0x1f43144f64a5468e), UINT64_C(0x1f431d50c4a0868e)
    },
    {   UINT64_C(0x0000000000000814), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afc8fa078c69e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000815), UINT64_C(0x106aee4bf88043bd), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056a8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98ee0b0e6e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007df2), UINT64_C(0x1f433450b09905d3), UINT64_C(0x1f433cd09c8e25cc),
        UINT64_C(0x1f432f90c89445cc), UINT64_C(0x1f432f90c894462f)
    },
    {   UINT64_C(0x0000000000007df3), UINT64_C(0x1f4330d0b09145d3), UINT64_C(0x1f432f90ac9545cc),
        UINT64_C(0x1f432d10c89ca5cc), UINT64_C(0x1f433750c896e5cc)
    },
    {   UINT64_C(0x0000000000007df4), UINT64_C(0x1f4337d09c96c5cc), UINT64_C(0x1f4333d0c89825cc),
        UINT64_C(0x1f432c8eb09c45cc), UINT64_C(0x1f4337d0a096a5ca)
    },
    {   UINT64_C(0x0000000000007df5), UINT64_C(0x1f4337d0c896a5cc), UINT64_C(0x1f432f8eb09ac5cc),
        UINT64_C(0x1f432cceb09d268e), UINT64_C(0x1f433910c896e5cc)
    },
    {   UINT64_C(0x0000000000007df6), UINT64_C(0x1f432f4fdc93e5cc), UINT64_C(0x1f433510c892c5cc),
        UINT64_C(0x1f432790c49aa5cc), UINT64_C(0x1f43274fd89c45cc)
    },
    {   UINT64_C(0x00000000000056ae), UINT64_C(0x1f42f60f20b70712), UINT64_C(0x1f42f64f30ada6d1),
        UINT64_C(0x1f42f50fa0b8a6e8), UINT64_C(0x1f430b4eecafc72f)
    },
    {   UINT64_C(0x0000000000007df8), UINT64_C(0x1f4324d0b09985d3), UINT64_C(0x1f4335d0b09405cc),
        UINT64_C(0x1f432c90589c4698), UINT64_C(0x1f430210f4b186c3)
    },
    {   UINT64_C(0x0000000000007df9), UINT64_C(0x1f43028fa4b14719), UINT64_C(0x1f432290c0b026a6),
        UINT64_C(0x1f43388f4ca306e1), UINT64_C(0x1f43408fa0a1c6dd)
    },
    {   UINT64_C(0x0000000000007dfa), UINT64_C(0x1f4323cfe89a86f5), UINT64_C(0x1f43478f0c8d46f4),
        UINT64_C(0x1f430f16f0aec725), UINT64_C(0x1f43090f20b325ca)
    },
    {   UINT64_C(0x0000000000000820), UINT64_C(0x106b19ce546a449f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dfc), UINT64_C(0x1f431c50b4a126a7), UINT64_C(0x1f431bcf4899c700),
        UINT64_C(0x1f431bd0b4ad86b1), UINT64_C(0x1f431bcf449846f3)
    },
    {   UINT64_C(0x000000000000659e), UINT64_C(0x1f430c4f74b1e724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8efcb44705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dfe), UINT64_C(0x1f432010b8ad86a7), UINT64_C(0x1f432050b4ad86b1),
        UINT64_C(0x1f432210b4ad86b1), UINT64_C(0x1f43200ef899a6f3)
    },
    {   UINT64_C(0x0000000000007dff), UINT64_C(0x1f431d4f1099a6f3), UINT64_C(0x1f43220f149846f3),
        UINT64_C(0x1f432310b49cc6b1), UINT64_C(0x1f431f0f149846f3)
    },
    {   UINT64_C(0x00000000000056b7), UINT64_C(0x1f42f80e4cb36725), UINT64_C(0x1f42f40f08b54721),
        UINT64_C(0x1f42f34ee4b58729), UINT64_C(0x1f42f68f54b5a704)
    },
    {   UINT64_C(0x00000000000056b8), UINT64_C(0x1f43018eb0ae271e), UINT64_C(0x1f42f50ee8a98725),
        UINT64_C(0x1f430c4f08acc704), UINT64_C(0x1f42f70ee4b126fd)
    },
    {   UINT64_C(0x0000000000019101), UINT64_C(0x14daf20fa08ea68c), UINT64_C(0x131af20fa087668c),
        UINT64_C(0x1852f74f549ce69b), UINT64_C(0x1852f34fa09d668c)
    },
    {   UINT64_C(0x0000000000000828), UINT64_C(0x0000000000000000), UINT64_C(0x106af80e547b069e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000829), UINT64_C(0x106af94f8c7d2653), UINT64_C(0x0000000000000000),
        UINT64_C(0x106ae8cfa082c64b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000082a), UINT64_C(0x0000000000000000), UINT64_C(0x106af04f807ec64b),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af6ce887f469e)
    },
    {   UINT64_C(0x000000000000082b), UINT64_C(0x106aee0fa07ca650), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000082c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af74e547c869e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056bf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60e80b72738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f5cafe709)
    },
    {   UINT64_C(0x00000000000056c0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60fb0b6870c),
        UINT64_C(0x1f42f44f28b46721), UINT64_C(0x1f430f8ef8af4716)
    },
    {   UINT64_C(0x00000000000056c1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50e6cb24732),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ec4b4e72c)
    },
    {   UINT64_C(0x000000000001910a), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x1162f2cc6c8005c1),
        UINT64_C(0x1162f68c6c8185c1), UINT64_C(0x0de2f50fa074c5c1)
    },
    {   UINT64_C(0x000000000001910b), UINT64_C(0x0c2af44f686ba5c1), UINT64_C(0x0a6af2cf7068065e),
        UINT64_C(0x1162f20f6881c5b5), UINT64_C(0x0fa2f34f687b25b5)
    },
    {   UINT64_C(0x00000000000056c4), UINT64_C(0x1f42f94fd4b5c704), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f74b5a700)
    },
    {   UINT64_C(0x000000000001910d), UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675),
        UINT64_C(0x14fafa8f7097667b), UINT64_C(0x18b2f78fa09e2675)
    },
    {   UINT64_C(0x00000000000065a7), UINT64_C(0x1f430d8e3cb2e73f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43050f48b906ec), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001910f), UINT64_C(0x0f02f78fa080c675), UINT64_C(0x10e2fd0f708c0675),
        UINT64_C(0x17d2f78fa0a2c675), UINT64_C(0x1622f78fa0920675)
    },
    {   UINT64_C(0x0000000000019110), UINT64_C(0x1f6af78fa0c26675), UINT64_C(0x1a9af78fa0af2675),
        UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1292f78fa0968675)
    },
    {   UINT64_C(0x0000000000005bfb), UINT64_C(0x1f43058e54b6c722), UINT64_C(0x1f43048f4cb04732),
        UINT64_C(0x1f42ffcf38b26711), UINT64_C(0x1f430b4e98b4471d)
    },
    {   UINT64_C(0x00000000000056ca), UINT64_C(0x1f42f80f2caf6719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60e60b2a735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bfc), UINT64_C(0x1f43018e8cb626e1), UINT64_C(0x1f43058e5cae6742),
        UINT64_C(0x1f43048fe8b6a6fb), UINT64_C(0x1f43048ec0afe71f)
    },
    {   UINT64_C(0x0000000000019114), UINT64_C(0x0cfaf80f747085a1), UINT64_C(0x10a2f80f70806684),
        UINT64_C(0x0deaf80f7471c5a1), UINT64_C(0x115af80cb48b6684)
    },
    {   UINT64_C(0x0000000000019115), UINT64_C(0x0f62f30cb47c05a1), UINT64_C(0x0000000000000000),
        UINT64_C(0x09bafa8f745ea650), UINT64_C(0x0c2aea0cb46d0650)
    },
    {   UINT64_C(0x0000000000019116), UINT64_C(0x10f2f94f74806684), UINT64_C(0x09f2f94f78614684),
        UINT64_C(0x1632f58f809385a1), UINT64_C(0x0f8af58f8078e5a1)
    },
    {   UINT64_C(0x0000000000005bfe), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cf64b0e706), UINT64_C(0x1f42fb0f20b446f6)
    },
    {   UINT64_C(0x0000000000019118), UINT64_C(0x0dc2fa8f746f45a1), UINT64_C(0x09caf80f7c61e61f),
        UINT64_C(0x0ed2f58f7c7605a1), UINT64_C(0x0fd3000f7c7a65b2)
    },
    {   UINT64_C(0x00000000000056d1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88ec4b4e712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4ee8b72722)
    },
    {   UINT64_C(0x0000000000000840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0f747c6656)
    },
    {   UINT64_C(0x00000000000056d3), UINT64_C(0x1f4302ce78b1a738), UINT64_C(0x1f42f90e10b72712),
        UINT64_C(0x1f42fe0ea4b1e716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065ae), UINT64_C(0x1f42fdce80b6a709), UINT64_C(0x1f42f8cefcad6729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430fce70b3e729)
    },
    {   UINT64_C(0x00000000000056d5), UINT64_C(0x1f42fd4fd0b466d2), UINT64_C(0x1f4301ce88b06720),
        UINT64_C(0x1f42f90f34b406d7), UINT64_C(0x1f430a4edcb0e732)
    },
    {   UINT64_C(0x000000000001911e), UINT64_C(0x1952fbcd389d0695), UINT64_C(0x190af88fa0a08685),
        UINT64_C(0x14fafbcf78968695), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x0000000000000845), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106ae34f8c802650), UINT64_C(0x106adb4bfc8986a9)
    },
    {   UINT64_C(0x00000000000056d8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0f74b1270a), UINT64_C(0x1f43068e90b0873d)
    },
    {   UINT64_C(0x0000000000000847), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af30f7480664b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000848), UINT64_C(0x106af014e07f06a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aee14e07f26a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000849), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af5cfa07c0656), UINT64_C(0x106af5cf747c264b)
    },
    {   UINT64_C(0x00000000000056dc), UINT64_C(0x1f42ff8f24b58715), UINT64_C(0x1f42f7ce74b2c737),
        UINT64_C(0x1f43038facb206b4), UINT64_C(0x1f43074e60ae873e)
    },
    {   UINT64_C(0x000000000000084b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af58f8c7ae653), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019126), UINT64_C(0x1972fb0f68a265c1), UINT64_C(0x125af64f808265b6),
        UINT64_C(0x103af6cc6c7fc5b6), UINT64_C(0x109af6cf547ce5b5)
    },
    {   UINT64_C(0x00000000000056df), UINT64_C(0x0000000000000000), UINT64_C(0x1f43078e4cb24747),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e58b2673c)
    },
    {   UINT64_C(0x00000000000065b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f80b52718),
        UINT64_C(0x1f42fd4f74b14710), UINT64_C(0x1f42facf28b26706)
    },
    {   UINT64_C(0x00000000000056e1), UINT64_C(0x1f430d4f68b48726), UINT64_C(0x1f4311cf38b1a735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43138e9cb0e729)
    },
    {   UINT64_C(0x00000000000065b5), UINT64_C(0x1f42fecf78b266ff), UINT64_C(0x1f42f74f88b44719),
        UINT64_C(0x1f42fd0f18b5c712), UINT64_C(0x1f42ff4ed8b44707)
    },
    {   UINT64_C(0x0000000000005c08), UINT64_C(0x1f43058edca9a708), UINT64_C(0x1f42feceecb086eb),
        UINT64_C(0x1f43044ee8b0c709), UINT64_C(0x1f43048f60b18711)
    },
    {   UINT64_C(0x00000000000065b6), UINT64_C(0x1f42fb4f00b4c735), UINT64_C(0x1f43030f0cb12737),
        UINT64_C(0x1f43088e98a86735), UINT64_C(0x1f4309cf48b6c722)
    },
    {   UINT64_C(0x00000000000056e5), UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cf48afe722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43174ec0afa71a)
    },
    {   UINT64_C(0x000000000001912e), UINT64_C(0x14b2f58f748825a1), UINT64_C(0x1a6af58f74a5e697),
        UINT64_C(0x0000000000000000), UINT64_C(0x156af4cc5c92a697)
    },
    {   UINT64_C(0x00000000000056e7), UINT64_C(0x1f43144f4cb0c72d), UINT64_C(0x1f4313cfa0ad6713),
        UINT64_C(0x1f43160f24a9a71d), UINT64_C(0x1f43060f78af8731)
    },
    {   UINT64_C(0x00000000000065b8), UINT64_C(0x1f42fa4f88b16703), UINT64_C(0x1f42fe8ea0b14702),
        UINT64_C(0x1f430a0efcb64719), UINT64_C(0x1f42fb8f14b7a725)
    },
    {   UINT64_C(0x0000000000019131), UINT64_C(0x0000000000000000), UINT64_C(0x182af3cc5c9545a1),
        UINT64_C(0x1862f58c5c9b65a1), UINT64_C(0x1222fa0fa08605a4)
    },
    {   UINT64_C(0x00000000000056ea), UINT64_C(0x1f4311cf7cafe716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43128f24ae471a), UINT64_C(0x1f4314cfa8aac726)
    },
    {   UINT64_C(0x0000000000019133), UINT64_C(0x1bc2f58f74a905a1), UINT64_C(0x1782fe4f7492e5a1),
        UINT64_C(0x1492f58c348ba590), UINT64_C(0x159afb0f748ba5c6)
    },
    {   UINT64_C(0x00000000000065ba), UINT64_C(0x1f43124f8caea6db), UINT64_C(0x1f430d4e60af472c),
        UINT64_C(0x1f430e8f64b2c6fd), UINT64_C(0x1f43058e20afa71f)
    },
    {   UINT64_C(0x0000000000019135), UINT64_C(0x19d305cf4ca166a3), UINT64_C(0x2092fe4f4cbea6a3),
        UINT64_C(0x1a13028f4ca266a3), UINT64_C(0x249afe4f4ccec6a3)
    },
    {   UINT64_C(0x0000000000019136), UINT64_C(0x1f2af9cf4cb8a6a3), UINT64_C(0x1c52f2cc20ad46a3),
        UINT64_C(0x2412f94f4ccbc6a3), UINT64_C(0x2062f80f4cbd66a3)
    },
    {   UINT64_C(0x00000000000056ef), UINT64_C(0x0000000000000000), UINT64_C(0x1f43148eecad272d),
        UINT64_C(0x1f43124f14ac6714), UINT64_C(0x1f43010f08ace725)
    },
    {   UINT64_C(0x00000000000056f0), UINT64_C(0x1f43170e7cad671e), UINT64_C(0x1f4309ceb0ac271e),
        UINT64_C(0x1f42fecefcb84721), UINT64_C(0x1f43040ef8b0a721)
    },
    {   UINT64_C(0x00000000000056f1), UINT64_C(0x1f43048e20b14745), UINT64_C(0x1f43150ec0aec731),
        UINT64_C(0x1f430ccea8ab0735), UINT64_C(0x1f4302cf38b46733)
    },
    {   UINT64_C(0x00000000000056f2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4e64b40745),
        UINT64_C(0x1f43180ed0a7271e), UINT64_C(0x1f43064e80b6c735)
    },
    {   UINT64_C(0x000000000001913b), UINT64_C(0x17e2f6cf6893e5b6), UINT64_C(0x1daaf50f68b086a3),
        UINT64_C(0x16b2f6cf688f85b6), UINT64_C(0x1802f44c6c9c26a3)
    },
    {   UINT64_C(0x00000000000065be), UINT64_C(0x1f43104e50ae6721), UINT64_C(0x1f430a8eccacc72c),
        UINT64_C(0x1f4314ce54ace72f), UINT64_C(0x1f43128f4cb22722)
    },
    {   UINT64_C(0x00000000000056f5), UINT64_C(0x1f43148eb4ab2719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124ed8b2a736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000864), UINT64_C(0x106af551ac7c65c8), UINT64_C(0x106afccfa078a64a),
        UINT64_C(0x106af551ac7c65c8), UINT64_C(0x106afccfa078a64a)
    },
    {   UINT64_C(0x0000000000000865), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af74fa0848653)
    },
    {   UINT64_C(0x0000000000000866), UINT64_C(0x106af74f8084864b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056f9), UINT64_C(0x1f43024f1cad6735), UINT64_C(0x1f430e4e60b8c748),
        UINT64_C(0x1f43038e50b2a741), UINT64_C(0x1f43098e38afc736)
    },
    {   UINT64_C(0x00000000000056fa), UINT64_C(0x1f4304ce68b62739), UINT64_C(0x1f43048e44b78739),
        UINT64_C(0x1f43134eaca70723), UINT64_C(0x1f43140e28a9a73e)
    },
    {   UINT64_C(0x00000000000056fb), UINT64_C(0x1f430f8e74b5e73c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028e38b54735), UINT64_C(0x1f431bced0a7871f)
    },
    {   UINT64_C(0x0000000000019144), UINT64_C(0x21dafa8f48bf669f), UINT64_C(0x19dafa8f489f669f),
        UINT64_C(0x1a2af74f40a0c69f), UINT64_C(0x19baf80c709f269f)
    },
    {   UINT64_C(0x0000000000019145), UINT64_C(0x1b0af74ea0a4469f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ac2fbcf40a2a69f), UINT64_C(0x17f2f9cf4098c69f)
    },
    {   UINT64_C(0x0000000000019146), UINT64_C(0x189afa8f489b86a0), UINT64_C(0x19fafa0f40a0269f),
        UINT64_C(0x219afa0f40bea69f), UINT64_C(0x17daf98f249a669f)
    },
    {   UINT64_C(0x0000000000019147), UINT64_C(0x17f2f9cc7097c69f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1092facf587a65bc), UINT64_C(0x100af98f5878469f)
    },
    {   UINT64_C(0x0000000000019148), UINT64_C(0x0c0afc4f586985bc), UINT64_C(0x0fd2f94f5877669f),
        UINT64_C(0x0d22f9cf586de5bc), UINT64_C(0x0a3af4cc5c66069e)
    },
    {   UINT64_C(0x0000000000005701), UINT64_C(0x1f42f94eb4b2071e), UINT64_C(0x1f42f94f4cb5c729),
        UINT64_C(0x1f42fd4eccb0071a), UINT64_C(0x1f42fc0eccb3e724)
    },
    {   UINT64_C(0x000000000001914a), UINT64_C(0x0d22fa0f586c869f), UINT64_C(0x0c72f98f586a469f),
        UINT64_C(0x1c72fccf3ca985bc), UINT64_C(0x13e2fd0f3c8745bc)
    },
    {   UINT64_C(0x0000000000005703), UINT64_C(0x1f43078e48afa70a), UINT64_C(0x1f42fe8e74b5271c),
        UINT64_C(0x1f4313ce38aec745), UINT64_C(0x1f43110f7cada6ea)
    },
    {   UINT64_C(0x0000000000005704), UINT64_C(0x1f43180ec4b486fb), UINT64_C(0x1f43170f0cb0e711),
        UINT64_C(0x1f4302cea4b74703), UINT64_C(0x1f4317cf30a6a727)
    },
    {   UINT64_C(0x000000000001914d), UINT64_C(0x17baf84f3c970632), UINT64_C(0x0e42f94cac7145bc),
        UINT64_C(0x1022fa0c5c788632), UINT64_C(0x0e4af8cc5c7165bc)
    },
    {   UINT64_C(0x0000000000005706), UINT64_C(0x1f43144f98ab66f8), UINT64_C(0x1f4312cf04ad2721),
        UINT64_C(0x1f4312cf04ad2721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005707), UINT64_C(0x1f4303cedcb6c71f), UINT64_C(0x1f43114f70ada716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcec8b36705)
    },
    {   UINT64_C(0x0000000000005708), UINT64_C(0x1f42f68eb4b7472a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4f98b446fd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019151), UINT64_C(0x1abaff8fa0a0a67e), UINT64_C(0x0000000000000000),
        UINT64_C(0x17c2f94f6897e68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000570a), UINT64_C(0x1f42fc4f38b086f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cfe4b2a726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000570b), UINT64_C(0x1f42f78f9cb7a723), UINT64_C(0x1f43040eacb0871d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000570c), UINT64_C(0x1f42f68f18b4270e), UINT64_C(0x1f42ffcfe8b56709),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f0cb48701)
    },
    {   UINT64_C(0x000000000000570d), UINT64_C(0x1f42f78f80b3a711), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f6cb50717)
    },
    {   UINT64_C(0x000000000000570e), UINT64_C(0x1f43008fc4a326ef), UINT64_C(0x1f42fe4f0cb326e2),
        UINT64_C(0x1f4304d018b866d3), UINT64_C(0x1f42f88fc8b2a6ee)
    },
    {   UINT64_C(0x000000000000570f), UINT64_C(0x1f42f40fd0b006ff), UINT64_C(0x1f42fccf90b786e2),
        UINT64_C(0x1f42fa8f58b4870d), UINT64_C(0x1f42f7cf08b82727)
    },
    {   UINT64_C(0x0000000000005710), UINT64_C(0x1f42fa8f20ae4712), UINT64_C(0x1f42fa8fa0b3a6fc),
        UINT64_C(0x1f42ff8fa0a7a6e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005711), UINT64_C(0x1f42ff4ec4b566fd), UINT64_C(0x1f42fb8f38b08710),
        UINT64_C(0x1f42f98f90ab4703), UINT64_C(0x1f42fe4f90a5c6eb)
    },
    {   UINT64_C(0x0000000000005712), UINT64_C(0x1f42fc8f50a92707), UINT64_C(0x1f42fe8fccb186f6),
        UINT64_C(0x1f42f98f80a5c708), UINT64_C(0x1f42fa8f7cb1a70e)
    },
    {   UINT64_C(0x0000000000000881), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af68fa07bc673), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005714), UINT64_C(0x1f42f6cf64b76723), UINT64_C(0x1f42fa8ed0a36728),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43120eb4a986fc)
    },
    {   UINT64_C(0x000000000001915d), UINT64_C(0x18b2fb8f3c9a66a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1b02f88f3ca446a5), UINT64_C(0x15dafacf3c8f26a5)
    },
    {   UINT64_C(0x0000000000000884), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af753ec7b4539), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000885), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aff11087785f1)
    },
    {   UINT64_C(0x0000000000000886), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aed0f8488a690), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019161), UINT64_C(0x1a2afa8c6ca046a5), UINT64_C(0x179af90c949666a5),
        UINT64_C(0x12c2fbcf4082a5c3), UINT64_C(0x117af94f407d869f)
    },
    {   UINT64_C(0x0000000000019162), UINT64_C(0x0e82fbcf407205c3), UINT64_C(0x116afa0f3c7d469e),
        UINT64_C(0x0ee2fbcf4073e5c3), UINT64_C(0x0b92f64c486a86a4)
    },
    {   UINT64_C(0x0000000000019163), UINT64_C(0x11b2fbcc347dc5c3), UINT64_C(0x1202f94c347f069f),
        UINT64_C(0x0d6af6cf406de6a1), UINT64_C(0x0c2af80c3468a6a1)
    },
    {   UINT64_C(0x0000000000019164), UINT64_C(0x0e42fa0f4070c69f), UINT64_C(0x0d8af94f406de69f),
        UINT64_C(0x1ec2f80f40b305c3), UINT64_C(0x15c2f80f408f05c3)
    },
    {   UINT64_C(0x0000000000019165), UINT64_C(0x1152f94f407ce5c3), UINT64_C(0x1402f70c3487a639),
        UINT64_C(0x11f2fbcc347fc5c3), UINT64_C(0x106af80f407c25c3)
    },
    {   UINT64_C(0x0000000000019166), UINT64_C(0x116afb0f247da66b), UINT64_C(0x0db2f94f446dc678),
        UINT64_C(0x154af6cf408d25c3), UINT64_C(0x11eafbcf40804639)
    },
    {   UINT64_C(0x0000000000019167), UINT64_C(0x1a92fbcf40a1e639), UINT64_C(0x104afa8c5c7ba5c3),
        UINT64_C(0x12a2fbcc34818639), UINT64_C(0x0f22f70c347525c3)
    },
    {   UINT64_C(0x00000000000065d4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054ee4afc730),
        UINT64_C(0x1f430a0e90b22730), UINT64_C(0x1f42fccef8b78709)
    },
    {   UINT64_C(0x0000000000019169), UINT64_C(0x123b008fa082067e), UINT64_C(0x10bb008fa07fe67e),
        UINT64_C(0x15baf4cf688ce68c), UINT64_C(0x1493008fa087867e)
    },
    {   UINT64_C(0x0000000000005722), UINT64_C(0x1f42ff4f08b026f3), UINT64_C(0x1f42fccf6cb086ec),
        UINT64_C(0x1f43240ee0a74706), UINT64_C(0x1f42fe902cb4a6f7)
    },
    {   UINT64_C(0x000000000001916b), UINT64_C(0x1a5b00cfa09e867e), UINT64_C(0x151b008f6889a67e),
        UINT64_C(0x1652f58f6894068c), UINT64_C(0x106b008fa07e867e)
    },
    {   UINT64_C(0x0000000000005724), UINT64_C(0x1f42ffcf84b56716), UINT64_C(0x1f42fe8edcb9a731),
        UINT64_C(0x1f42f88f5cb4a6f5), UINT64_C(0x1f42fd8e9caec6e3)
    },
    {   UINT64_C(0x0000000000005725), UINT64_C(0x1f43050e9cb72706), UINT64_C(0x1f43040ed8b4072e),
        UINT64_C(0x1f42fcd004b1a6cc), UINT64_C(0x1f4326ceb0a2272b)
    },
    {   UINT64_C(0x0000000000005726), UINT64_C(0x1f42fd8fc0b726f9), UINT64_C(0x1f43064f88b62714),
        UINT64_C(0x1f4308cec4af2726), UINT64_C(0x1f431f4ea4a9c727)
    },
    {   UINT64_C(0x0000000000005727), UINT64_C(0x1f432a4f98aca727), UINT64_C(0x1f43168ef49ec72a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cfbcb6a6e5)
    },
    {   UINT64_C(0x0000000000005728), UINT64_C(0x1f42f9cf4caf66ca), UINT64_C(0x1f43180f48a426f7),
        UINT64_C(0x1f43120eacaf2717), UINT64_C(0x1f42fccf34abe6fa)
    },
    {   UINT64_C(0x0000000000005729), UINT64_C(0x1f42f5502cb726f2), UINT64_C(0x1f43004f7cb7870d),
        UINT64_C(0x1f430ccfc0ad4707), UINT64_C(0x1f43144e909fa71b)
    },
    {   UINT64_C(0x00000000000065d9), UINT64_C(0x1f42f85024b686f6), UINT64_C(0x1f42f78e74ae86f8),
        UINT64_C(0x1f42ff0f60b6e6eb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019173), UINT64_C(0x0c2afa4f786a45b7), UINT64_C(0x091aee4f7862662c),
        UINT64_C(0x0f5afdcf747445ad), UINT64_C(0x0ed2f5cf6875e5ad)
    },
    {   UINT64_C(0x000000000000572c), UINT64_C(0x1f42f8cfdcb046ef), UINT64_C(0x1f42fd4ef8b4a70e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ea8b3e724)
    },
    {   UINT64_C(0x0000000000019175), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c),
        UINT64_C(0x169afa4f5494869b), UINT64_C(0x162b020fa092c68c)
    },
    {   UINT64_C(0x00000000000065db), UINT64_C(0x1f42fd0e9cb2e729), UINT64_C(0x1f43024ee4b1473d),
        UINT64_C(0x1f42fd8f28adc727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000572f), UINT64_C(0x1f42f90eacb606fe), UINT64_C(0x1f4318cef8a4471b),
        UINT64_C(0x1f43074fc8aee6fa), UINT64_C(0x1f43208eccaa271b)
    },
    {   UINT64_C(0x00000000000065dc), UINT64_C(0x1f42fd8f18b72727), UINT64_C(0x1f42f6cee4b7473d),
        UINT64_C(0x1f42f8cf08b6a732), UINT64_C(0x1f42fc8e8cb12726)
    },
    {   UINT64_C(0x0000000000005731), UINT64_C(0x1f42fdcef8b70706), UINT64_C(0x1f42fa0ed0b58714),
        UINT64_C(0x1f42fa8f4cb726e7), UINT64_C(0x1f43030eb0b5471b)
    },
    {   UINT64_C(0x0000000000005732), UINT64_C(0x1f42f80f2cb7a6eb), UINT64_C(0x1f43118ee0b26717),
        UINT64_C(0x1f431c4eb0a1273c), UINT64_C(0x1f43004ef8ae070e)
    },
    {   UINT64_C(0x0000000000005733), UINT64_C(0x1f42fad038b2e6df), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43270eb4b1073b), UINT64_C(0x1f42fcd00cb466bb)
    },
    {   UINT64_C(0x0000000000005734), UINT64_C(0x1f42fecfa0abc70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028f28a986eb), UINT64_C(0x1f42fd8f18b1c70f)
    },
    {   UINT64_C(0x0000000000005735), UINT64_C(0x1f42fb8f78aec6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f88b166df), UINT64_C(0x1f42ff4fb0b5271c)
    },
    {   UINT64_C(0x000000000001917e), UINT64_C(0x103afbcfa07dc68c), UINT64_C(0x075afbcfa055068c),
        UINT64_C(0x1842fbcfa098a5c1), UINT64_C(0x0ff2fbcfa07765c1)
    },
    {   UINT64_C(0x0000000000005737), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c0f40a66713), UINT64_C(0x1f430a4e40a6e734)
    },
    {   UINT64_C(0x00000000000065e0), UINT64_C(0x1f42fbceccae2722), UINT64_C(0x1f42fbcedcb90712),
        UINT64_C(0x1f42f88f04b6a70b), UINT64_C(0x1f42fa8eecb4a716)
    },
    {   UINT64_C(0x0000000000005739), UINT64_C(0x1f42f78f08b7a70f), UINT64_C(0x1f42fdcef4b3471d),
        UINT64_C(0x1f4316cf50a96716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065e1), UINT64_C(0x1f42f9cf08b5671c), UINT64_C(0x1f42fbcf18b6c718),
        UINT64_C(0x1f42f9ceb4b5a729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000573b), UINT64_C(0x1f42fecf2ca94701), UINT64_C(0x1f43048ed4aac702),
        UINT64_C(0x1f42ffcf68b506bf), UINT64_C(0x1f43070e78af072b)
    },
    {   UINT64_C(0x000000000000573c), UINT64_C(0x1f42f8ce0cba2744), UINT64_C(0x1f42ff0f54aea6ca),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000573d), UINT64_C(0x1f42fd8fb8b726f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431f0e58ad8745), UINT64_C(0x1f43120eb8a8c723)
    },
    {   UINT64_C(0x000000000000573e), UINT64_C(0x1f42fccf50b1c6ef), UINT64_C(0x1f430ccf10b2a70f),
        UINT64_C(0x1f42f6cf10b726d9), UINT64_C(0x1f42fc8f60b76731)
    },
    {   UINT64_C(0x000000000000573f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f60b9a716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf0cb166e9)
    },
    {   UINT64_C(0x0000000000005740), UINT64_C(0x1f432d0ed8b0a720), UINT64_C(0x1f430a4f58b4472a),
        UINT64_C(0x1f42fd0f98b0a717), UINT64_C(0x1f42fecef8b6471a)
    },
    {   UINT64_C(0x0000000000005741), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8fb4b0c6ff), UINT64_C(0x1f43128e74b1c73b)
    },
    {   UINT64_C(0x0000000000005742), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058e3cb24729)
    },
    {   UINT64_C(0x00000000000008b1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aee50847fc613), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005744), UINT64_C(0x0000000000000000), UINT64_C(0x1f431e4ec8b0e741),
        UINT64_C(0x1f430c8e3cb1472f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005745), UINT64_C(0x1f42f84e6cac6726), UINT64_C(0x1f42ffcfd0b2a6e3),
        UINT64_C(0x1f42f30ee4ab4727), UINT64_C(0x1f431dce24a54739)
    },
    {   UINT64_C(0x0000000000005746), UINT64_C(0x1f43010f90b72711), UINT64_C(0x1f43060eb8b70718),
        UINT64_C(0x1f43010f70b3e728), UINT64_C(0x1f42fd8f04ac26ec)
    },
    {   UINT64_C(0x0000000000005747), UINT64_C(0x0000000000000000), UINT64_C(0x1f4308ce94aee731),
        UINT64_C(0x1f43030e9cb06720), UINT64_C(0x1f43080f40b2a710)
    },
    {   UINT64_C(0x0000000000019190), UINT64_C(0x13db020fa099868c), UINT64_C(0x1233020fa099268c),
        UINT64_C(0x175308cf54a0a69b), UINT64_C(0x165b020fa0a2e68c)
    },
    {   UINT64_C(0x0000000000005749), UINT64_C(0x1f42ff8e5cb36722), UINT64_C(0x1f43050e4cb0e71d),
        UINT64_C(0x1f42fccf18b1a710), UINT64_C(0x1f42fecec0b6071b)
    },
    {   UINT64_C(0x000000000000574a), UINT64_C(0x1f43010fa8b5e723), UINT64_C(0x1f43020e98a6c727),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000574b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018e7caea73b),
        UINT64_C(0x1f43254e80a14727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019194), UINT64_C(0x15db0ccf54a1868c), UINT64_C(0x15931e4f58a6a68c),
        UINT64_C(0x1f2b1f8f64cc668c), UINT64_C(0x1722f80fa0a9268c)
    },
    {   UINT64_C(0x0000000000019195), UINT64_C(0x15b31d0fa0a7c68c), UINT64_C(0x146af44fa09ea68c),
        UINT64_C(0x0f4afa0f687de5c1), UINT64_C(0x1002facf6c84468c)
    },
    {   UINT64_C(0x0000000000019196), UINT64_C(0x0e72ffcf688005c1), UINT64_C(0x1032ff4f688ca68c),
        UINT64_C(0x0e22ffcf6c7c05c1), UINT64_C(0x0aa3000fa081269c)
    },
    {   UINT64_C(0x0000000000019197), UINT64_C(0x0fc2ee8c688965c0), UINT64_C(0x0ff2fbcfa082468c),
        UINT64_C(0x07aafccfa06c4698), UINT64_C(0x0a22d78c78758698)
    },
    {   UINT64_C(0x0000000000019198), UINT64_C(0x103afbcfa089668c), UINT64_C(0x075afbcfa06a068c),
        UINT64_C(0x1842fbcfa0a3a5c1), UINT64_C(0x0ff2fbcfa08245c1)
    },
    {   UINT64_C(0x0000000000019199), UINT64_C(0x0fab000f688245c1), UINT64_C(0x1002eecc6c8465c1),
        UINT64_C(0x1002ff4c6c8645c1), UINT64_C(0x0cdafbcfa07f65c1)
    },
    {   UINT64_C(0x000000000001919a), UINT64_C(0x0c0af74f6c75c5c1), UINT64_C(0x0c13068f7076665f),
        UINT64_C(0x1033024f6485a5b6), UINT64_C(0x0e730e4f6884a5b6)
    },
    {   UINT64_C(0x000000000001919b), UINT64_C(0x15eb0ecf68a245b6), UINT64_C(0x0fd2f30fa08745b6),
        UINT64_C(0x0ec2e74c7085a5b6), UINT64_C(0x0dcaf34fa080c5b6)
    },
    {   UINT64_C(0x000000000001919c), UINT64_C(0x106af48fa07c8689), UINT64_C(0x106af54fa07a867e),
        UINT64_C(0x106af80f747ac688), UINT64_C(0x106af44fa07ae67e)
    },
    {   UINT64_C(0x000000000001919d), UINT64_C(0x106af5cfa07d067e), UINT64_C(0x106af68fa07b867e),
        UINT64_C(0x106af74f747c6688), UINT64_C(0x106af48fa07c867e)
    },
    {   UINT64_C(0x0000000000005756), UINT64_C(0x1f42f90f48b5c719), UINT64_C(0x1f42f70e3cb10731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4ec0b7a722)
    },
    {   UINT64_C(0x000000000001919f), UINT64_C(0x106af34fa07d267e), UINT64_C(0x106af5cfa07be67e),
        UINT64_C(0x106afc0f7478c688), UINT64_C(0x106af5cfa07a667e)
    },
    {   UINT64_C(0x00000000000191a0), UINT64_C(0x106afc0d7478c688), UINT64_C(0x106af48f747f867e),
        UINT64_C(0x106afb0f74794688), UINT64_C(0x106af48fa07c867e)
    },
    {   UINT64_C(0x00000000000191a1), UINT64_C(0x106af04f747ec67e), UINT64_C(0x106af24f807dc67e),
        UINT64_C(0x106af0cf807e867e), UINT64_C(0x106af40fa07ca67e)
    },
    {   UINT64_C(0x00000000000008c8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af752ac7b4586), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000575b), UINT64_C(0x1f430e8edcb1e71c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040fa8b3c72b)
    },
    {   UINT64_C(0x00000000000008ca), UINT64_C(0x0000000000000000), UINT64_C(0x106b134e5876a69e),
        UINT64_C(0x106b00ce586d669e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191a5), UINT64_C(0x106af40fa07da649), UINT64_C(0x106afdcfa077e649),
        UINT64_C(0x106aee8fa07f85b0), UINT64_C(0x106af24fa07da5b0)
    },
    {   UINT64_C(0x00000000000191a6), UINT64_C(0x106afd8f887805b3), UINT64_C(0x106af24c2c7aa5b0),
        UINT64_C(0x106af94c2c80c5b0), UINT64_C(0x106af74fa07b25b0)
    },
    {   UINT64_C(0x000000000000575f), UINT64_C(0x1f42f80f64b3471b), UINT64_C(0x1f42fbcec8af2704),
        UINT64_C(0x1f43084ebcb1873a), UINT64_C(0x1f430b0ea0a5c72a)
    },
    {   UINT64_C(0x0000000000005760), UINT64_C(0x1f42fdce94aee70b), UINT64_C(0x1f42f50ea8b2c725),
        UINT64_C(0x1f42fbce9cb0e738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005761), UINT64_C(0x1f42f8ce3cb3c748), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0e64b1c72f), UINT64_C(0x1f42fa8f7cb3c6ef)
    },
    {   UINT64_C(0x0000000000005762), UINT64_C(0x1f430a4ef0afa72a), UINT64_C(0x1f430c8e5cb16728),
        UINT64_C(0x1f43008e5caf673f), UINT64_C(0x1f42fe4ef0b3672a)
    },
    {   UINT64_C(0x00000000000191ab), UINT64_C(0x153af20fa08ea68c), UINT64_C(0x15aaf50fa08dc68c),
        UINT64_C(0x1952f34fa09d668c), UINT64_C(0x1852f6cf589b669c)
    },
    {   UINT64_C(0x0000000000005764), UINT64_C(0x1f430bcf04acc73f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf1caf6723), UINT64_C(0x1f42f80ea4b70710)
    },
    {   UINT64_C(0x00000000000191ad), UINT64_C(0x1692f20f5896268c), UINT64_C(0x137af50fa088e68c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x1862f34fa09d668c)
    },
    {   UINT64_C(0x00000000000191ae), UINT64_C(0x131af20fa089868c), UINT64_C(0x1852f6cf589b669c),
        UINT64_C(0x14caf18fa08ce68c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x0000000000005767), UINT64_C(0x1f430e8e74b04739), UINT64_C(0x1f42f54f64b786f7),
        UINT64_C(0x1f431e0f0ca4e71b), UINT64_C(0x1f42fc0e90b26739)
    },
    {   UINT64_C(0x0000000000005768), UINT64_C(0x1f42f54ea8b6e719), UINT64_C(0x1f42fd8f58aee733),
        UINT64_C(0x1f42fd0ef8b50712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191b1), UINT64_C(0x1032f10c6c7d25c1), UINT64_C(0x0fb2f48f687a269c),
        UINT64_C(0x0de2f50f687425c1), UINT64_C(0x0e5af3cc6c75269c)
    },
    {   UINT64_C(0x00000000000191b2), UINT64_C(0x1222f10c6c82a5c1), UINT64_C(0x0faaf44f687a069c),
        UINT64_C(0x0a32f1cf686485b5), UINT64_C(0x122af34fa08465c1)
    },
    {   UINT64_C(0x00000000000191b3), UINT64_C(0x1112f2cf5880669c), UINT64_C(0x1312f94c6c8805b5),
        UINT64_C(0x1032f1cf687c65c1), UINT64_C(0x0e8af3cc6c75669c)
    },
    {   UINT64_C(0x00000000000191b4), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x13baf30f588aa5b5),
        UINT64_C(0x111af94c6c7ee5c1), UINT64_C(0x0e82f3cc747605c1)
    },
    {   UINT64_C(0x00000000000191b5), UINT64_C(0x11c2f44f6880a5b5), UINT64_C(0x0f62f28f6877a5b5),
        UINT64_C(0x1202f10f688365c1), UINT64_C(0x146af40c6c8d25c1)
    },
    {   UINT64_C(0x00000000000191b6), UINT64_C(0x1322f34c6c87c5c1), UINT64_C(0x17daf10c6c9c65c1),
        UINT64_C(0x16f2f48f689705c1), UINT64_C(0x101af44f687be6ab)
    },
    {   UINT64_C(0x00000000000191b7), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x143af10f688c069c),
        UINT64_C(0x119af10f5482c5c1), UINT64_C(0x146af40c6c8d268c)
    },
    {   UINT64_C(0x0000000000005770), UINT64_C(0x1f42f90e48ae8727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43170efca9073b), UINT64_C(0x1f43020f1cb2e724)
    },
    {   UINT64_C(0x0000000000005771), UINT64_C(0x0000000000000000), UINT64_C(0x1f430aceecac6740),
        UINT64_C(0x1f42fa8f1cb326fc), UINT64_C(0x1f42fa8eccb5c71a)
    },
    {   UINT64_C(0x00000000000191ba), UINT64_C(0x1b4af78fa0b1e675), UINT64_C(0x170afa8f74978685),
        UINT64_C(0x0f02f78fa080c675), UINT64_C(0x17d2f78fa0a2c675)
    },
    {   UINT64_C(0x0000000000005c50), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f50afa71e),
        UINT64_C(0x1f42fa8e3cb5a72a), UINT64_C(0x1f42fd8f70b5c6f7)
    },
    {   UINT64_C(0x00000000000191bc), UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1b4af78fa0b1e675),
        UINT64_C(0x1292f78fa0968675), UINT64_C(0x170afa8f74978685)
    },
    {   UINT64_C(0x0000000000005775), UINT64_C(0x1f430a0ee4ad8740), UINT64_C(0x1f42fdce5cae072e),
        UINT64_C(0x1f43054e8cb1872a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191be), UINT64_C(0x18b2f44fa0a3c675), UINT64_C(0x14f2f50fa09b6683),
        UINT64_C(0x1902f60fa09fa682), UINT64_C(0x139af88f64898675)
    },
    {   UINT64_C(0x00000000000191bf), UINT64_C(0x1062f80f7880a5a1), UINT64_C(0x0f6af44cc47f4684),
        UINT64_C(0x0d62f6ccb477c5a1), UINT64_C(0x0e7af80f7476e684)
    },
    {   UINT64_C(0x00000000000191c0), UINT64_C(0x0f22f44f747605a1), UINT64_C(0x0d9af80cc47a8684),
        UINT64_C(0x0e62f58cc47765a1), UINT64_C(0x0f22f80f7479c684)
    },
    {   UINT64_C(0x0000000000005779), UINT64_C(0x0000000000000000), UINT64_C(0x1f43021158b4c6a1),
        UINT64_C(0x1f42fdcfc0ad6729), UINT64_C(0x1f43018fa0b166db)
    },
    {   UINT64_C(0x000000000000d152), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15f2f94f5490a69c), UINT64_C(0x1162f44fa08f268c)
    },
    {   UINT64_C(0x000000000000577b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014fc4acc73e), UINT64_C(0x1f42fc8f84b486fb)
    },
    {   UINT64_C(0x000000000000577c), UINT64_C(0x1f43044f78b826f3), UINT64_C(0x1f431e9040ada6e8),
        UINT64_C(0x1f43219054b086ed), UINT64_C(0x1f4328909cb166ec)
    },
    {   UINT64_C(0x000000000000577d), UINT64_C(0x1f431e5038b126cc), UINT64_C(0x1f43014ee0b1c707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcef4b92738)
    },
    {   UINT64_C(0x000000000000577e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43138ee8af070b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4eb0b3e739)
    },
    {   UINT64_C(0x0000000000019452), UINT64_C(0x0ffafccd3076e67e), UINT64_C(0x1cf2fb9084ab85a9),
        UINT64_C(0x1cf2fb9084ab85a9), UINT64_C(0x1cf2fb8d34ab867e)
    },
    {   UINT64_C(0x00000000000191c8), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x1bdaf88fa0b4c685), UINT64_C(0x187afbcf609ce695)
    },
    {   UINT64_C(0x00000000000191c9), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x18daf88fa0a76685),
        UINT64_C(0x17baf7cfa0990695), UINT64_C(0x1fc2f88fa0c46685)
    },
    {   UINT64_C(0x0000000000005782), UINT64_C(0x1f43104ee8a9a6c3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038eccb4e70b), UINT64_C(0x1f43124e74a96732)
    },
    {   UINT64_C(0x00000000000191cb), UINT64_C(0x1322f88fa09a2685), UINT64_C(0x187afbcf609ce695),
        UINT64_C(0x17bafe0fa0a0e685), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x00000000000191cc), UINT64_C(0x140af5cfa0a08695), UINT64_C(0x19daf50fa0a26685),
        UINT64_C(0x1942f50fa0a64685), UINT64_C(0x15b2f58fa0a02695)
    },
    {   UINT64_C(0x00000000000191cd), UINT64_C(0x19d2fc4fa0a46695), UINT64_C(0x13c2f8cf60898685),
        UINT64_C(0x1382f90f708d45b5), UINT64_C(0x115af50c6c876695)
    },
    {   UINT64_C(0x00000000000191ce), UINT64_C(0x0f52f90c787d65b6), UINT64_C(0x10d2f90f8081c695),
        UINT64_C(0x0f7af50f7876e5b6), UINT64_C(0x0ec2f90c747fc695)
    },
    {   UINT64_C(0x0000000000005787), UINT64_C(0x1f43098f68ab6712), UINT64_C(0x1f430e0e5cb28736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431c0e4ca5c72e)
    },
    {   UINT64_C(0x0000000000005788), UINT64_C(0x1f432ace9ca1070b), UINT64_C(0x1f43068ebcb30719),
        UINT64_C(0x1f4300cf08b7870e), UINT64_C(0x1f43098e7cb6e725)
    },
    {   UINT64_C(0x00000000000191d1), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x121af7cf6c8845aa),
        UINT64_C(0x129af64c6c84c5b6), UINT64_C(0x0f02f7cc747e05b6)
    },
    {   UINT64_C(0x00000000000191d2), UINT64_C(0x1282f90f7889e5aa), UINT64_C(0x0eaaf64f847d45aa),
        UINT64_C(0x1142f64f788165b6), UINT64_C(0x1612fbcc6c91e5b6)
    },
    {   UINT64_C(0x000000000000578b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e4cae0745),
        UINT64_C(0x1f430b4e7ca36723), UINT64_C(0x1f430c0e0cb14730)
    },
    {   UINT64_C(0x00000000000191d4), UINT64_C(0x0f0af90f787aa5b5), UINT64_C(0x12faf64f6c8786a2),
        UINT64_C(0x12faeb0f648d45b6), UINT64_C(0x155af90c6c906695)
    },
    {   UINT64_C(0x000000000000578d), UINT64_C(0x1f430a4f5cb04703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c0e0cb14728), UINT64_C(0x1f43008f68b7472d)
    },
    {   UINT64_C(0x00000000000191d6), UINT64_C(0x127b020fa085a68c), UINT64_C(0x1682f80fa093869a),
        UINT64_C(0x13db020fa089268c), UINT64_C(0x146af44fa08c668c)
    },
    {   UINT64_C(0x00000000000191d7), UINT64_C(0x165b020fa08de68c), UINT64_C(0x1852f6cf549b669b),
        UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x1643020fa096e68c)
    },
    {   UINT64_C(0x00000000000191d8), UINT64_C(0x1612f80fa091c69a), UINT64_C(0x1c8b020fa0a6a68c),
        UINT64_C(0x16a3020f588f068c), UINT64_C(0x1402f6cfa08a268c)
    },
    {   UINT64_C(0x0000000000005791), UINT64_C(0x1f43050f98b046ff), UINT64_C(0x1f4309cf78b246ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43084e5cae6730)
    },
    {   UINT64_C(0x00000000000191da), UINT64_C(0x1512f50fa08d868c), UINT64_C(0x140af18fa08ce68c),
        UINT64_C(0x1762f30fa099869b), UINT64_C(0x1902f44fa09f668c)
    },
    {   UINT64_C(0x00000000000191db), UINT64_C(0x1722f80fa096068c), UINT64_C(0x1932f1cfa0a1669b),
        UINT64_C(0x1782f60fa098869b), UINT64_C(0x16faf94f28942680)
    },
    {   UINT64_C(0x00000000000191dc), UINT64_C(0x14b2f80f688d25c1), UINT64_C(0x1002f7cc6c7b669c),
        UINT64_C(0x0fb2f08c6c7a25c1), UINT64_C(0x0fb2f44f687a269c)
    },
    {   UINT64_C(0x0000000000005795), UINT64_C(0x1f43020f1cb6c706), UINT64_C(0x1f4302ce6cb4c73a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf58b0e71f)
    },
    {   UINT64_C(0x00000000000191de), UINT64_C(0x09fafb4f686385b5), UINT64_C(0x107afbcfa07e85c1),
        UINT64_C(0x1152f28f687f869c), UINT64_C(0x1132f7cc6c7f25b5)
    },
    {   UINT64_C(0x0000000000005797), UINT64_C(0x1f43054ed0ad870b), UINT64_C(0x1f43054e5cae070b),
        UINT64_C(0x1f4303cf70b2e71c), UINT64_C(0x1f43088ea8b4671b)
    },
    {   UINT64_C(0x0000000000005798), UINT64_C(0x1f43018eecae870b), UINT64_C(0x1f4300cec4b7c703),
        UINT64_C(0x1f43044f70b4e729), UINT64_C(0x1f43138e5cb4c72e)
    },
    {   UINT64_C(0x0000000000005799), UINT64_C(0x1f43078e5caea72e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f0eaca3271a), UINT64_C(0x1f4304cedcb3c707)
    },
    {   UINT64_C(0x0000000000000908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af0cf8c7b8665), UINT64_C(0x106af8cd8c7a84d9)
    },
    {   UINT64_C(0x00000000000191e3), UINT64_C(0x11d2f18f6883c5c1), UINT64_C(0x13d2f44c6c8aa69c),
        UINT64_C(0x0f72f44c6c7925c1), UINT64_C(0x1a82ef8f68a765b5)
    },
    {   UINT64_C(0x00000000000191e4), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x1612fd0fa092668c),
        UINT64_C(0x12d2fd0fa096a68c), UINT64_C(0x1682f80fa093869a)
    },
    {   UINT64_C(0x00000000000191e5), UINT64_C(0x155afd0fa097068c), UINT64_C(0x161af44fa09ea68c),
        UINT64_C(0x1762fd0fa0a0668c), UINT64_C(0x1a7b048f54a5469b)
    },
    {   UINT64_C(0x00000000000191e6), UINT64_C(0x0d9afa8fa07da68c), UINT64_C(0x1642fd0fa0a2068c),
        UINT64_C(0x1572f30fa08f469a), UINT64_C(0x1d2afd0fa0b9268c)
    },
    {   UINT64_C(0x000000000000579f), UINT64_C(0x1f430d0f90af66ea), UINT64_C(0x1f430acfa4b0a6f1),
        UINT64_C(0x1f430a4e64ada731), UINT64_C(0x1f43070f54a1a704)
    },
    {   UINT64_C(0x0000000000006614), UINT64_C(0x1f42facf10b086e0), UINT64_C(0x1f42f68ec0b5c6f4),
        UINT64_C(0x1f43068f40af26d6), UINT64_C(0x1f43078f8cae4706)
    },
    {   UINT64_C(0x00000000000057a1), UINT64_C(0x1f43064f64ae871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004fd0b4672e), UINT64_C(0x1f42f98f24b5e729)
    },
    {   UINT64_C(0x0000000000006615), UINT64_C(0x1f43014eb4b2072d), UINT64_C(0x1f43058f10afe6d6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f2cafe704)
    },
    {   UINT64_C(0x00000000000057a3), UINT64_C(0x1f43034ebca98721), UINT64_C(0x1f42ff4e7ca4071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e84b0e740)
    },
    {   UINT64_C(0x0000000000006616), UINT64_C(0x1f42f88fdcb286ee), UINT64_C(0x1f42fa0f18b2e703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024ef0b2070b)
    },
    {   UINT64_C(0x00000000000191ed), UINT64_C(0x11f2f28f6882069c), UINT64_C(0x1292eb0c6c85a5b6),
        UINT64_C(0x106af84f688525c1), UINT64_C(0x0f0af8cc6c80269c)
    },
    {   UINT64_C(0x00000000000191ee), UINT64_C(0x10faf94f687ee5c1), UINT64_C(0x13c2f94f689085b6),
        UINT64_C(0x1182e5cc6c8185c1), UINT64_C(0x102afb0c6c83c5c1)
    },
    {   UINT64_C(0x0000000000019466), UINT64_C(0x0df2f34f7873a693), UINT64_C(0x0872fc4f786025b8),
        UINT64_C(0x0ec3028fa07985b8), UINT64_C(0x0f4af00f747a4693)
    },
    {   UINT64_C(0x00000000000191f0), UINT64_C(0x11fada0c6c8cc5c1), UINT64_C(0x15fb00cc6c9c65c1),
        UINT64_C(0x17a2fe0f689a05b5), UINT64_C(0x1122f94f6881e69b)
    },
    {   UINT64_C(0x0000000000019467), UINT64_C(0x0f3af90c447ac5ad), UINT64_C(0x0e62f58f6873e5b8),
        UINT64_C(0x0d12f64c4471c693), UINT64_C(0x0f9af50f787965b7)
    },
    {   UINT64_C(0x00000000000191f2), UINT64_C(0x1102f78c6c7f85c1), UINT64_C(0x1bcaf80f68b085b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057ab), UINT64_C(0x1f42fe4f3cb10712), UINT64_C(0x1f42f88f14b56727),
        UINT64_C(0x1f43034f2ca6e71e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000661a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43020ec4aea6ff),
        UINT64_C(0x1f42f74ec4b10719), UINT64_C(0x1f43058f20ada726)
    },
    {   UINT64_C(0x00000000000057ad), UINT64_C(0x1f42f98e60a9072e), UINT64_C(0x1f42fa8e9cb7a736),
        UINT64_C(0x1f42f78f34adc737), UINT64_C(0x1f42fd4e94b30739)
    },
    {   UINT64_C(0x000000000000661b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306cf10ade709)
    },
    {   UINT64_C(0x00000000000057af), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68f40b70736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000661c), UINT64_C(0x1f43058e50ae872b), UINT64_C(0x1f42f48ec0b0473b),
        UINT64_C(0x1f42fd4efcaf8700), UINT64_C(0x1f42fa0f78b0c714)
    },
    {   UINT64_C(0x00000000000191f9), UINT64_C(0x0fa2f6cfa079268c), UINT64_C(0x0fa2f5cf6875a68c),
        UINT64_C(0x0fa2f0cfa07ba68c), UINT64_C(0x0fa2f70f6877c68c)
    },
    {   UINT64_C(0x00000000000191fa), UINT64_C(0x0fa2f60f68794694), UINT64_C(0x0fa2f78f8076e67e),
        UINT64_C(0x0fa2fa4f6876668c), UINT64_C(0x0fa2f50f4c78c68c)
    },
    {   UINT64_C(0x00000000000191fb), UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa3028fa0684698),
        UINT64_C(0x0fa2f30fa079c698), UINT64_C(0x0fa2f28f6c7a6698)
    },
    {   UINT64_C(0x00000000000057b4), UINT64_C(0x1f42fb0f60b3472e), UINT64_C(0x1f42fb8e84b1a73d),
        UINT64_C(0x1f42f78ed0b80738), UINT64_C(0x1f42f28e4cac673b)
    },
    {   UINT64_C(0x00000000000057b5), UINT64_C(0x1f42f58e54ae6738), UINT64_C(0x1f42fb8f68ade72b),
        UINT64_C(0x1f42fb8e60b5c73c), UINT64_C(0x1f42fa8e6caa6739)
    },
    {   UINT64_C(0x000000000000661f), UINT64_C(0x1f43030f58ab86fe), UINT64_C(0x1f42fa4f5caae708),
        UINT64_C(0x1f42f48f5caac6e6), UINT64_C(0x1f43064f48ab2719)
    },
    {   UINT64_C(0x00000000000191ff), UINT64_C(0x106afc8f74788691), UINT64_C(0x106af8cf747a6696),
        UINT64_C(0x106af8cf747a6691), UINT64_C(0x106afc8f74788691)
    },
    {   UINT64_C(0x00000000000057b8), UINT64_C(0x1f42f78f54b5c739), UINT64_C(0x1f42fc4f58a9a730),
        UINT64_C(0x1f42f68ec4b7c740), UINT64_C(0x1f42f68e84ae073d)
    },
    {   UINT64_C(0x00000000000057b9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e64b30724),
        UINT64_C(0x1f42fa8eccaac72e), UINT64_C(0x1f42fe4e84ae672f)
    },
    {   UINT64_C(0x0000000000006621), UINT64_C(0x1f42fd4fa8ac86fb), UINT64_C(0x1f42f84f4cac26ef),
        UINT64_C(0x1f42f58f68ada718), UINT64_C(0x1f42fb4f7caa0721)
    },
    {   UINT64_C(0x00000000000057bb), UINT64_C(0x1f42f98e94b0a73d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4e44af4739), UINT64_C(0x1f42f78eacb2272e)
    },
    {   UINT64_C(0x0000000000006622), UINT64_C(0x1f43060fc4a926d8), UINT64_C(0x1f4302cec0ab2703),
        UINT64_C(0x1f42fd8f8caac6e5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c75), UINT64_C(0x1f42f88f48b38712), UINT64_C(0x1f430b0f28b04731),
        UINT64_C(0x1f42fc8ee8b20701), UINT64_C(0x1f42ff0f90b326ed)
    },
    {   UINT64_C(0x00000000000057be), UINT64_C(0x1f42fdcf68b3a701), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8e98b5a739), UINT64_C(0x1f42fdcf68b3a6c8)
    },
    {   UINT64_C(0x00000000000057bf), UINT64_C(0x1f42fdcf68b3a6cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004f24b366fa), UINT64_C(0x1f42fd1094b486eb)
    },
    {   UINT64_C(0x0000000000006624), UINT64_C(0x1f43084f6caa86fc), UINT64_C(0x1f42ff8f5cae6719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf20ab870a)
    },
    {   UINT64_C(0x00000000000057c1), UINT64_C(0x1f42fecec8b62719), UINT64_C(0x1f430e4eecb6470c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4308ce5cb6a736)
    },
    {   UINT64_C(0x000000000000d176), UINT64_C(0x13db020fa089268c), UINT64_C(0x1233020fa085c68c),
        UINT64_C(0x1752f74f5492669b), UINT64_C(0x165b020fa08de68c)
    },
    {   UINT64_C(0x00000000000057c3), UINT64_C(0x1f430b8e80b28736), UINT64_C(0x1f430d4e80b28736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e80b28736)
    },
    {   UINT64_C(0x00000000000057c4), UINT64_C(0x1f42ff8e80b2c736), UINT64_C(0x1f42ff8f64b4a711),
        UINT64_C(0x1f43008f64b4a711), UINT64_C(0x1f432d4edca0a70e)
    },
    {   UINT64_C(0x00000000000057c5), UINT64_C(0x1f431c8e98ad4715), UINT64_C(0x1f43140f08a0a71f),
        UINT64_C(0x1f43028ec8b1470c), UINT64_C(0x1f43140e6caec734)
    },
    {   UINT64_C(0x00000000000057c6), UINT64_C(0x1f430c8f2caf66f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ace6cb62717), UINT64_C(0x1f43074facb38716)
    },
    {   UINT64_C(0x00000000000057c7), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0facb306bb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8e68ade732)
    },
    {   UINT64_C(0x00000000000057c8), UINT64_C(0x1f430e8f5cb3a6de), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4305cf88b3671a), UINT64_C(0x1f430a0f38b2c704)
    },
    {   UINT64_C(0x00000000000057c9), UINT64_C(0x1f43028f50b0c6da), UINT64_C(0x1f430b4f14afa70a),
        UINT64_C(0x1f43094f90b0e706), UINT64_C(0x1f43008f90b9070e)
    },
    {   UINT64_C(0x00000000000057ca), UINT64_C(0x1f43098f4ca7e6b6), UINT64_C(0x1f43028e70b64735),
        UINT64_C(0x1f4307cf84ad66d0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057cb), UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ce7cb5c70b),
        UINT64_C(0x1f43088e60b6c71d), UINT64_C(0x1f4319ce64a7a72e)
    },
    {   UINT64_C(0x00000000000057cc), UINT64_C(0x1f430c0ed4ab26e7), UINT64_C(0x1f43034f44a6a702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0f60b0e6ce)
    },
    {   UINT64_C(0x00000000000057cd), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0f18b88717),
        UINT64_C(0x1f430c8f28af0703), UINT64_C(0x1f430d0f30aae6ff)
    },
    {   UINT64_C(0x000000000000d17c), UINT64_C(0x0e72f44f6875c5c1), UINT64_C(0x1032f44f6878668c),
        UINT64_C(0x0e22f44f6c74a5c1), UINT64_C(0x0aa2f18fa06d469b)
    },
    {   UINT64_C(0x00000000000057cf), UINT64_C(0x1f43060fc0aa46f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4306ce6caee703), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057d0), UINT64_C(0x1f43078f14b3a71e), UINT64_C(0x1f42fe8f14b346dc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ced8b00709)
    },
    {   UINT64_C(0x00000000000057d1), UINT64_C(0x1f4306cf64b326cd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4315ce38b28733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000940), UINT64_C(0x106aee136080251e), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b26ca1463a72d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057d3), UINT64_C(0x1f43074ef4ab0711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dce44acc738), UINT64_C(0x1f43040e54b5871f)
    },
    {   UINT64_C(0x00000000000057d4), UINT64_C(0x1f432750e0a4c677), UINT64_C(0x1f43021030b1a6be),
        UINT64_C(0x1f431b8f18ae86d6), UINT64_C(0x1f4326904ca326f7)
    },
    {   UINT64_C(0x0000000000000943), UINT64_C(0x106b26ca1480251e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000944), UINT64_C(0x106aee0a1463a51e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000945), UINT64_C(0x106b26d36080272d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000946), UINT64_C(0x106aee136063a72d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000947), UINT64_C(0x106b26ca1480272d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057da), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ced4ab871a),
        UINT64_C(0x1f42fa4ef4b5072b), UINT64_C(0x1f42fa4eb4b00709)
    },
    {   UINT64_C(0x0000000000000949), UINT64_C(0x106aee0a1463a72d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057dc), UINT64_C(0x1f42ffce6cac8729), UINT64_C(0x1f42fc0e48ac0715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43254e7c8e672b)
    },
    {   UINT64_C(0x000000000000094b), UINT64_C(0x106aee0a1480251e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006633), UINT64_C(0x1f42f0cf44b8e733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ccf48b366c6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000094d), UINT64_C(0x106aee136080272d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057e0), UINT64_C(0x1f42f88fb4b2c714), UINT64_C(0x1f42f74f78b5e715),
        UINT64_C(0x1f42f98fa4b5c714), UINT64_C(0x1f42facfb4b5271c)
    },
    {   UINT64_C(0x000000000000094f), UINT64_C(0x106aee0a1480272d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057e2), UINT64_C(0x1f42f84f2cb8670e), UINT64_C(0x1f42f54e7cb54741),
        UINT64_C(0x1f42f28fd4b0c6fd), UINT64_C(0x1f42fe4e88b62738)
    },
    {   UINT64_C(0x00000000000057e3), UINT64_C(0x1f42fb4f58b34732), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057e4), UINT64_C(0x1f42f88eccb8a71d), UINT64_C(0x1f42f74fa4b4a6f3),
        UINT64_C(0x1f42f84f28b7870d), UINT64_C(0x1f42f70f28b706fc)
    },
    {   UINT64_C(0x00000000000057e5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4eccb5c70e),
        UINT64_C(0x1f43008ef8ab46ee), UINT64_C(0x1f42f68e64b6472f)
    },
    {   UINT64_C(0x0000000000000954), UINT64_C(0x106aee122080256e), UINT64_C(0x106b12ca146da72d),
        UINT64_C(0x106b26ca1480256e), UINT64_C(0x106b12ca1480251e)
    },
    {   UINT64_C(0x0000000000000955), UINT64_C(0x106b12ca1480256e), UINT64_C(0x106aee0a1463a56e),
        UINT64_C(0x106aee0a146da51e), UINT64_C(0x106aee0a146da56e)
    },
    {   UINT64_C(0x0000000000000956), UINT64_C(0x106b26d22080272d), UINT64_C(0x106b12d36080272d),
        UINT64_C(0x106b12d22080272d), UINT64_C(0x106aee122063a72d)
    },
    {   UINT64_C(0x0000000000000957), UINT64_C(0x106aee13606da72d), UINT64_C(0x106aee12206da72d),
        UINT64_C(0x106b26ca1480272d), UINT64_C(0x106b12ca1480272d)
    },
    {   UINT64_C(0x0000000000000958), UINT64_C(0x106b12ca1480272d), UINT64_C(0x106aee0a1463a72d),
        UINT64_C(0x106aee0a146da72d), UINT64_C(0x106aee0a146da72d)
    },
    {   UINT64_C(0x0000000000000959), UINT64_C(0x106aee0a1480256e), UINT64_C(0x106aee0a1480251e),
        UINT64_C(0x106aee0a1480256e), UINT64_C(0x106aee122080272d)
    },
    {   UINT64_C(0x000000000000095a), UINT64_C(0x106aee136080272d), UINT64_C(0x106aee122080272d),
        UINT64_C(0x106aee0a1480272d), UINT64_C(0x106aee0a1480272d)
    },
    {   UINT64_C(0x00000000000057ed), UINT64_C(0x1f42f6cf84b7070a), UINT64_C(0x1f42f74f84b7470a),
        UINT64_C(0x1f42f78f84b726e9), UINT64_C(0x1f42f58f44b56741)
    },
    {   UINT64_C(0x00000000000057ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f74b82705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4e84b6c729)
    },
    {   UINT64_C(0x00000000000057ef), UINT64_C(0x1f42f94f50b3a6f7), UINT64_C(0x1f42f94f40b72708),
        UINT64_C(0x1f42fc4f18b626ee), UINT64_C(0x1f42f88fd0b166ed)
    },
    {   UINT64_C(0x00000000000057f0), UINT64_C(0x1f42fc8e80b3a72a), UINT64_C(0x1f42f94eacb6a70f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43071188b1666c)
    },
    {   UINT64_C(0x00000000000057f1), UINT64_C(0x1f433bcef88f66f8), UINT64_C(0x1f430a5050b406de),
        UINT64_C(0x1f4300cf08b5a704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057f2), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034ef8ac46f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057f3), UINT64_C(0x1f430f0f54b36705), UINT64_C(0x1f430bcf08b526f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004ed8b1a6f3)
    },
    {   UINT64_C(0x00000000000057f4), UINT64_C(0x1f430e4f20ad270c), UINT64_C(0x1f4310cf7cae26f6),
        UINT64_C(0x1f42ffce88b42725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057f5), UINT64_C(0x1f4300cedcb2e710), UINT64_C(0x1f43020eccb54700),
        UINT64_C(0x1f43010ed8af8707), UINT64_C(0x1f43094f60b16728)
    },
    {   UINT64_C(0x000000000000663f), UINT64_C(0x1f42f90f0cacc736), UINT64_C(0x1f42f90f84b50729),
        UINT64_C(0x1f42fece3cae0728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057f7), UINT64_C(0x1f42fe8f64b34732), UINT64_C(0x1f43034e8cb46723),
        UINT64_C(0x1f43120f34b04736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057f8), UINT64_C(0x1f430d8f3cb36729), UINT64_C(0x1f42ffcea8ac0715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccef8b646f8)
    },
    {   UINT64_C(0x00000000000057f9), UINT64_C(0x1f42ffced8b2270c), UINT64_C(0x1f4306cec0b9c736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cef8b6e708)
    },
    {   UINT64_C(0x00000000000057fa), UINT64_C(0x1f43020f28b4e6f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f08b7c6ff), UINT64_C(0x1f43070f18b4c6fc)
    },
    {   UINT64_C(0x00000000000057fb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0f44ae4717),
        UINT64_C(0x1f42fbcf48b6271b), UINT64_C(0x1f42f88ef8b1e6f8)
    },
    {   UINT64_C(0x00000000000057fc), UINT64_C(0x1f42fecf58b32724), UINT64_C(0x1f42fe8ef8b5270c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f48b3a6f3)
    },
    {   UINT64_C(0x00000000000057fd), UINT64_C(0x1f42fc8f38b6c6f7), UINT64_C(0x1f4302cefcb92732),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f78b1a700)
    },
    {   UINT64_C(0x00000000000057fe), UINT64_C(0x1f43004f38b3e714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8f08b246fc), UINT64_C(0x1f42ffcf18aee714)
    },
    {   UINT64_C(0x00000000000057ff), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8f00b12724),
        UINT64_C(0x1f43018ef8b6e6f8), UINT64_C(0x1f42fe0ef4b3070b)
    },
    {   UINT64_C(0x0000000000005800), UINT64_C(0x1f43014efcb126f3), UINT64_C(0x1f43120f70ac0711),
        UINT64_C(0x1f43108f78b2c6e7), UINT64_C(0x1f43038f38b40708)
    },
    {   UINT64_C(0x0000000000005801), UINT64_C(0x1f4301cf78af4704), UINT64_C(0x1f43038ef8b526f8),
        UINT64_C(0x1f43038ef8b5c6f8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005802), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f28af66ff),
        UINT64_C(0x1f43048f58a866ff), UINT64_C(0x1f42ff0ef8b406f8)
    },
    {   UINT64_C(0x0000000000005803), UINT64_C(0x1f4301cf58aba707), UINT64_C(0x1f42fdcef4a9a6ef),
        UINT64_C(0x1f430d4f18b2473a), UINT64_C(0x1f42fb0f28b2c713)
    },
    {   UINT64_C(0x0000000000000972), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106adb4bfc8986a9)
    },
    {   UINT64_C(0x0000000000005805), UINT64_C(0x1f42fecf18b28704), UINT64_C(0x1f43068f48aae70e),
        UINT64_C(0x1f43028e5cae0733), UINT64_C(0x1f42ff0f88a706ff)
    },
    {   UINT64_C(0x0000000000005806), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f48b1a71a),
        UINT64_C(0x1f4300cf38b4c6f7), UINT64_C(0x1f42facf58b066ff)
    },
    {   UINT64_C(0x0000000000005807), UINT64_C(0x1f42fc8f58b546f8), UINT64_C(0x1f430ccfe8b446f6),
        UINT64_C(0x1f42fe0f58af46ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005808), UINT64_C(0x1f430f4efcb2c73b), UINT64_C(0x1f43014f58ad0707),
        UINT64_C(0x1f42fb0ef8b64700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005809), UINT64_C(0x1f43084ef4ae8711), UINT64_C(0x1f43040ee4aee740),
        UINT64_C(0x1f42f7ce7cb48712), UINT64_C(0x1f42fa0fa4b3e71c)
    },
    {   UINT64_C(0x000000000000580a), UINT64_C(0x1f43108f20afe710), UINT64_C(0x1f42f90f28b326ff),
        UINT64_C(0x1f43040f58b76710), UINT64_C(0x1f42ff8e4cb32747)
    },
    {   UINT64_C(0x0000000000000979), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afc913878c5e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000580c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f00afc724),
        UINT64_C(0x1f42fd8edcb346f8), UINT64_C(0x1f43008f74b506ff)
    },
    {   UINT64_C(0x000000000000580d), UINT64_C(0x1f43030f74b046eb), UINT64_C(0x1f42fe8eacb4272a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43160facab070a)
    },
    {   UINT64_C(0x000000000000580e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430dcf40ab06fb),
        UINT64_C(0x1f42f9cf0cb5a728), UINT64_C(0x1f430f0f88af6720)
    },
    {   UINT64_C(0x000000000000580f), UINT64_C(0x1f43148f98b1070a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024e9cb566fb)
    },
    {   UINT64_C(0x0000000000005810), UINT64_C(0x1f42ffcec8ac4727), UINT64_C(0x1f43024f4cb0e721),
        UINT64_C(0x1f42fc0ef8ae071b), UINT64_C(0x1f43028eb8b66740)
    },
    {   UINT64_C(0x0000000000005811), UINT64_C(0x1f42fdcf38a98711), UINT64_C(0x1f42fb0f38aba6f6),
        UINT64_C(0x1f42fc8f94b6870c), UINT64_C(0x1f43008f58aec6fb)
    },
    {   UINT64_C(0x0000000000005812), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f08aba6f3),
        UINT64_C(0x1f42fe8f94b32726), UINT64_C(0x1f430f0f70ae8723)
    },
    {   UINT64_C(0x0000000000005813), UINT64_C(0x1f42f7cedcb726f8), UINT64_C(0x1f42fd8fd4b7070e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005814), UINT64_C(0x1f430e4fc0b26706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008f68af66f3), UINT64_C(0x1f42fe0f18b14703)
    },
    {   UINT64_C(0x0000000000005815), UINT64_C(0x1f43020f58b2a6e4), UINT64_C(0x1f43014f48b42743),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005816), UINT64_C(0x1f42f88f48b526f7), UINT64_C(0x1f43064f48af0748),
        UINT64_C(0x1f43130f28ad670a), UINT64_C(0x1f42f8cf18b3e707)
    },
    {   UINT64_C(0x0000000000005817), UINT64_C(0x0000000000000000), UINT64_C(0x1f43178f84ae66f5),
        UINT64_C(0x1f42fc8f48afe70f), UINT64_C(0x1f43014ee8b0e718)
    },
    {   UINT64_C(0x0000000000005818), UINT64_C(0x1f42fa0eb0b5871f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044f88b7a71d), UINT64_C(0x1f4314cf50b2673c)
    },
    {   UINT64_C(0x0000000000005819), UINT64_C(0x1f43004f28b6c708), UINT64_C(0x1f42fe8f68b7e6da),
        UINT64_C(0x1f42fd8f74ada6ff), UINT64_C(0x1f43140fb4ae06e6)
    },
    {   UINT64_C(0x000000000000581a), UINT64_C(0x1f42ffcedcb66716), UINT64_C(0x1f430b0f74b3c723),
        UINT64_C(0x1f4301cfa4b92715), UINT64_C(0x1f42ff0f40aae70c)
    },
    {   UINT64_C(0x000000000000581b), UINT64_C(0x1f43048f88b38717), UINT64_C(0x1f42fccec0b70741),
        UINT64_C(0x1f43004f68b926e8), UINT64_C(0x1f43108f34b2e74a)
    },
    {   UINT64_C(0x000000000000581c), UINT64_C(0x1f4301cf68b9a704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4f08ae4704), UINT64_C(0x1f43114f6cac66fd)
    },
    {   UINT64_C(0x000000000000098b), UINT64_C(0x0000000000000000), UINT64_C(0x106af28f9c7c8639),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000581e), UINT64_C(0x1f42ff8f38afa6f3), UINT64_C(0x1f43020f28b4c6f4),
        UINT64_C(0x1f43030f08ae46ff), UINT64_C(0x1f43020f48aaa6fb)
    },
    {   UINT64_C(0x000000000000581f), UINT64_C(0x1f43030f38ac0722), UINT64_C(0x1f42ff8e9caf06f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4ee0b12726)
    },
    {   UINT64_C(0x0000000000005820), UINT64_C(0x1f42fd0e98ab0704), UINT64_C(0x1f43028efca5070c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f68ace71b)
    },
    {   UINT64_C(0x0000000000005821), UINT64_C(0x1f43030ef0a9e715), UINT64_C(0x1f4302cfa4b726f7),
        UINT64_C(0x1f4309ceb0b28728), UINT64_C(0x1f42f90ea4b8a704)
    },
    {   UINT64_C(0x0000000000000990), UINT64_C(0x106aff0bfc7785a2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005823), UINT64_C(0x1f43020ebcb3273a), UINT64_C(0x1f42fcce5cb5472a),
        UINT64_C(0x1f42fbceacb5e706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005824), UINT64_C(0x1f43094f50aa86f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facf38b56710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005825), UINT64_C(0x1f43044f08b8a732), UINT64_C(0x1f42fe4e2cae072e),
        UINT64_C(0x1f43004eb8b1873e), UINT64_C(0x1f42fb4f0cb3e6fb)
    },
    {   UINT64_C(0x0000000000005826), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004eccb9a745), UINT64_C(0x1f42fc0f68b6a731)
    },
    {   UINT64_C(0x0000000000005827), UINT64_C(0x1f42fe0f18b1c704), UINT64_C(0x1f42fd8f68b0670b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028f94b6270c)
    },
    {   UINT64_C(0x0000000000005828), UINT64_C(0x1f4305cf4cb0c723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8ee8b82708), UINT64_C(0x1f43130ef8b0e740)
    },
    {   UINT64_C(0x0000000000005829), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4305cf68b6e729), UINT64_C(0x1f43030f04b1070a)
    },
    {   UINT64_C(0x000000000000582a), UINT64_C(0x1f4305cf30af8726), UINT64_C(0x1f43080f38aea722),
        UINT64_C(0x1f43090ee8ae072d), UINT64_C(0x1f43130f20aa2722)
    },
    {   UINT64_C(0x000000000000582b), UINT64_C(0x1f430f4f28ae8707), UINT64_C(0x1f43020f94b706f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f2caae703)
    },
    {   UINT64_C(0x000000000000099a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af60f007b664a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194a9), UINT64_C(0x21a30a07a8c94ded), UINT64_C(0x21a30a07a8c94ded),
        UINT64_C(0x21a30a07a8c94ded), UINT64_C(0x11ab264978834bfa)
    },
    {   UINT64_C(0x000000000000d1ac), UINT64_C(0x0d2af30fa06cc68c), UINT64_C(0x19a2f58fa09ea68c),
        UINT64_C(0x161af04fa092c69a), UINT64_C(0x1e62f18fa0b1a68c)
    },
    {   UINT64_C(0x000000000000582f), UINT64_C(0x1f4303ce6cb72738), UINT64_C(0x1f43020f40b64730),
        UINT64_C(0x1f4300cf68ad66ec), UINT64_C(0x1f42fe4f14b506f0)
    },
    {   UINT64_C(0x000000000000665c), UINT64_C(0x1f42fece8ca96720), UINT64_C(0x1f42fe8f88b50725),
        UINT64_C(0x1f42fb0f60ac8718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194ab), UINT64_C(0x431b018e55412473), UINT64_C(0x3af2ee0d1d2a442e),
        UINT64_C(0x3af2ee18912a470b), UINT64_C(0x3af2ee0e492a4485)
    },
    {   UINT64_C(0x000000000000d1ae), UINT64_C(0x131af20fa089868c), UINT64_C(0x1852f6cf589b669c),
        UINT64_C(0x14caf18fa08ce68c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x0000000000005833), UINT64_C(0x1f430d0f04af272e), UINT64_C(0x1f42ff4f28b6a6e0),
        UINT64_C(0x1f430e0f24afa733), UINT64_C(0x1f42fa4f08b5a6f3)
    },
    {   UINT64_C(0x000000000000665e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8fd0b3c70f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194ad), UINT64_C(0x1a52fb097c86abfa), UINT64_C(0x143b264978892bfa),
        UINT64_C(0x143afa8978734bfa), UINT64_C(0x17eb060978924bfa)
    },
    {   UINT64_C(0x000000000000665f), UINT64_C(0x1f42face9cb20715), UINT64_C(0x1f42fb8f14b2671b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cef0b3071c)
    },
    {   UINT64_C(0x00000000000194ae), UINT64_C(0x17eb05c978922bfa), UINT64_C(0x221b080978b5ebfb),
        UINT64_C(0x221b100978b9ebfb), UINT64_C(0x203b11c978aaebfa)
    },
    {   UINT64_C(0x0000000000005838), UINT64_C(0x1f4316cf1cb46744), UINT64_C(0x1f430c4f64afa724),
        UINT64_C(0x1f42fdcfd4b686e6), UINT64_C(0x1f42ff4ed8b2e71e)
    },
    {   UINT64_C(0x00000000000194af), UINT64_C(0x203b170978ad8bfa), UINT64_C(0x202b1b097cb0cbfa),
        UINT64_C(0x202b0ac97ca8abfa), UINT64_C(0x19430d07ac9a0dee)
    },
    {   UINT64_C(0x000000000000583a), UINT64_C(0x1f4303cebcb6a71a), UINT64_C(0x1f430b4f80ac06fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43128f10b18726)
    },
    {   UINT64_C(0x000000000000583b), UINT64_C(0x1f43014f08abc6fc), UINT64_C(0x1f42fcceacafa6fe),
        UINT64_C(0x1f43038e8caf8708), UINT64_C(0x1f43090f2cb0c70b)
    },
    {   UINT64_C(0x0000000000006662), UINT64_C(0x1f42f8cf0cb2471d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf50b5870c), UINT64_C(0x1f42f5ced8b26715)
    },
    {   UINT64_C(0x00000000000009ab), UINT64_C(0x0000000000000000), UINT64_C(0x106ad751088b85f1),
        UINT64_C(0x106aba4e989a268d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000009ac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aff0bfc7785a2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000583f), UINT64_C(0x1f4300cf68ae8703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018ef8b62722), UINT64_C(0x1f43024f08b406fb)
    },
    {   UINT64_C(0x0000000000006664), UINT64_C(0x1f42f8cff8af471f), UINT64_C(0x1f42f9cf58b0a71c),
        UINT64_C(0x1f42fd0ef8b88713), UINT64_C(0x1f42f5cfa4b7c6f1)
    },
    {   UINT64_C(0x0000000000005841), UINT64_C(0x1f43008f08b2e6f7), UINT64_C(0x1f43028e7cb16733),
        UINT64_C(0x1f43014f08afa711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006665), UINT64_C(0x1f42f9cf70b0c6f1), UINT64_C(0x1f42f8ceb0b4e73b),
        UINT64_C(0x1f43078f2cb1c726), UINT64_C(0x1f42f88f7cb54714)
    },
    {   UINT64_C(0x0000000000005843), UINT64_C(0x0000000000000000), UINT64_C(0x1f43104f30ade733),
        UINT64_C(0x1f42fd4f04b5c732), UINT64_C(0x1f43094eccb18731)
    },
    {   UINT64_C(0x000000000000d1b7), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x143af10f688c069c),
        UINT64_C(0x119af10f5482c5c1), UINT64_C(0x146af40c6c8d268c)
    },
    {   UINT64_C(0x0000000000005845), UINT64_C(0x1f43044f00ac6703), UINT64_C(0x1f42fe4ec0b0670f),
        UINT64_C(0x1f42fe8edcae0719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006667), UINT64_C(0x1f42fa0f84b626f6), UINT64_C(0x1f42facfc8b426f4),
        UINT64_C(0x1f4303cf08b3c704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005847), UINT64_C(0x1f43054f5cad8704), UINT64_C(0x1f43008ef4b186fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004eacb1a741)
    },
    {   UINT64_C(0x0000000000006668), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf58b3c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf78b6a6f4)
    },
    {   UINT64_C(0x0000000000005849), UINT64_C(0x1f43064eecac0727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014eacb1672c), UINT64_C(0x1f4302cf08b94712)
    },
    {   UINT64_C(0x0000000000006669), UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f30b66725),
        UINT64_C(0x1f42f80e5cb4c71d), UINT64_C(0x1f42fc100cb32703)
    },
    {   UINT64_C(0x000000000000584b), UINT64_C(0x1f430f0f08b44734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8ebcac0700), UINT64_C(0x1f42ff8e80b4e709)
    },
    {   UINT64_C(0x000000000000584c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008ec8b446ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000584d), UINT64_C(0x1f42fe8ed0b5070b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8e68b4071f), UINT64_C(0x1f4300cfa4b3c716)
    },
    {   UINT64_C(0x000000000000584e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43098f1caf6730)
    },
    {   UINT64_C(0x000000000000584f), UINT64_C(0x1f42ffce74b2a72f), UINT64_C(0x1f42fc8e50b44734),
        UINT64_C(0x1f43024ed8b64727), UINT64_C(0x1f42fe4f58abc725)
    },
    {   UINT64_C(0x0000000000005850), UINT64_C(0x1f43018eacaee705), UINT64_C(0x1f430c4f08b1272a),
        UINT64_C(0x1f43068ee0af4743), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005851), UINT64_C(0x1f43018eecb9471d), UINT64_C(0x1f42fd8ebcb8e6fb),
        UINT64_C(0x1f43014f18aca6ff), UINT64_C(0x1f43050f40aee72a)
    },
    {   UINT64_C(0x0000000000005852), UINT64_C(0x1f43110e88b0073c), UINT64_C(0x1f43010eccab871a),
        UINT64_C(0x1f42ff0eccb2a72e), UINT64_C(0x1f4312cf08ae2724)
    },
    {   UINT64_C(0x0000000000005853), UINT64_C(0x1f42fdcf44b58729), UINT64_C(0x1f42fd0f28b2c6f7),
        UINT64_C(0x1f43010e74b20718), UINT64_C(0x1f42ffceccb34704)
    },
    {   UINT64_C(0x0000000000005854), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f94b7670b),
        UINT64_C(0x1f43004ea4adc73a), UINT64_C(0x1f42fd0ee8a9270e)
    },
    {   UINT64_C(0x0000000000005855), UINT64_C(0x1f42fd0eecb2472a), UINT64_C(0x1f42fe0e74b74747),
        UINT64_C(0x1f42fd4edcada6f8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005856), UINT64_C(0x1f4300ce38b8874c), UINT64_C(0x1f42ff4ebcb2e725),
        UINT64_C(0x1f42fece8cb5c71a), UINT64_C(0x1f4300ceb8afe72f)
    },
    {   UINT64_C(0x0000000000005857), UINT64_C(0x0000000000000000), UINT64_C(0x1f4310cf30b00717),
        UINT64_C(0x1f43060e1cae0734), UINT64_C(0x1f42fd8f58b1c6f3)
    },
    {   UINT64_C(0x0000000000005858), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ef8b3a70f),
        UINT64_C(0x1f43004ee0b2e70d), UINT64_C(0x1f43090e70b3471d)
    },
    {   UINT64_C(0x00000000000009c7), UINT64_C(0x0000000000000000), UINT64_C(0x106af58f587c660c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006671), UINT64_C(0x1f42f3cfb8b8a6ea), UINT64_C(0x1f42f84fb8b666de),
        UINT64_C(0x1f42f50f58b206de), UINT64_C(0x1f42f98fa8b246f0)
    },
    {   UINT64_C(0x000000000000585b), UINT64_C(0x1f42fd0eecaee738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4ea4b26741), UINT64_C(0x1f43024ed4b1471e)
    },
    {   UINT64_C(0x0000000000006672), UINT64_C(0x1f42fa0fb8b586e9), UINT64_C(0x1f42fa0f64b1e6cb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f58ae46cb)
    },
    {   UINT64_C(0x000000000000f8c3), UINT64_C(0x0000000000000000), UINT64_C(0x0c2ae50bf47506ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1852f58bf49ea6ef)
    },
    {   UINT64_C(0x000000000000585e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a8f6cac670b),
        UINT64_C(0x1f42fc0ec4b14734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f8c4), UINT64_C(0x0000000000000000), UINT64_C(0x161af04c6492c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1d82f18bf4b1a6ef)
    },
    {   UINT64_C(0x0000000000005860), UINT64_C(0x1f42fa4eb0b2a722), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e8e44b3e738), UINT64_C(0x1f430ccf1cb06730)
    },
    {   UINT64_C(0x0000000000005861), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f10b2471a)
    },
    {   UINT64_C(0x0000000000005862), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0ef4ad4700),
        UINT64_C(0x1f43000f00b44738), UINT64_C(0x1f430fcf5caf0703)
    },
    {   UINT64_C(0x0000000000005863), UINT64_C(0x1f43070f34abe723), UINT64_C(0x1f42ff4f0cb6872d),
        UINT64_C(0x1f43020f0cb00733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005864), UINT64_C(0x1f43028e1cb4e73b), UINT64_C(0x1f430ece80b2671c),
        UINT64_C(0x1f42fbcee8b08731), UINT64_C(0x1f42ffcf08b58728)
    },
    {   UINT64_C(0x0000000000005865), UINT64_C(0x1f42ff4f78b32708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43088f3cae671d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005866), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018e94b1c729),
        UINT64_C(0x1f43078e3cb3671d), UINT64_C(0x1f42ffce6cb26713)
    },
    {   UINT64_C(0x0000000000005867), UINT64_C(0x0000000000000000), UINT64_C(0x1f43144f2cb1c706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8eecb64711)
    },
    {   UINT64_C(0x0000000000005868), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014eacaee70a),
        UINT64_C(0x1f4301cedcb3073a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005869), UINT64_C(0x1f43048ea0b36743), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4f18af8719)
    },
    {   UINT64_C(0x000000000000586a), UINT64_C(0x1f430e4f6cabc709), UINT64_C(0x1f4306cf10b26728),
        UINT64_C(0x1f42fd4ef4ac6703), UINT64_C(0x1f42fd4ef0ace6ff)
    },
    {   UINT64_C(0x000000000000586b), UINT64_C(0x1f42fbcec8b00715), UINT64_C(0x1f42f8ce90b7c73e),
        UINT64_C(0x1f43008f58b5c707), UINT64_C(0x1f42f90f08b48704)
    },
    {   UINT64_C(0x000000000000586c), UINT64_C(0x1f43000e90aa6704), UINT64_C(0x1f42f94f68b446eb),
        UINT64_C(0x1f43168e98b10736), UINT64_C(0x1f43058e78b5070f)
    },
    {   UINT64_C(0x000000000000586d), UINT64_C(0x1f42fecef8b4a70b), UINT64_C(0x1f430a0f38ad070a),
        UINT64_C(0x1f42fe0f00b66736), UINT64_C(0x1f43030f28b40719)
    },
    {   UINT64_C(0x000000000000586e), UINT64_C(0x1f42fecf38b84703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038edcad672a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000586f), UINT64_C(0x1f430ecf38af671e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cf48afa714), UINT64_C(0x1f42fb8f18b806f7)
    },
    {   UINT64_C(0x0000000000005870), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecee4b446ff),
        UINT64_C(0x1f43044ef8af873d), UINT64_C(0x1f43148eb0b44742)
    },
    {   UINT64_C(0x0000000000005871), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0eacb72716),
        UINT64_C(0x1f42ff4eecb2c6fb), UINT64_C(0x1f4309cebcb24712)
    },
    {   UINT64_C(0x0000000000005872), UINT64_C(0x1f42fcced8b3e722), UINT64_C(0x1f42fa8ef8b10732),
        UINT64_C(0x1f43014f58b6472d), UINT64_C(0x1f43020f28b1e6f2)
    },
    {   UINT64_C(0x0000000000005873), UINT64_C(0x1f43004e8cac0720), UINT64_C(0x1f42fb4e7cb72735),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005874), UINT64_C(0x1f42f98f00b5c73a), UINT64_C(0x1f43114f40b0871b),
        UINT64_C(0x1f43000fa0ab6714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005875), UINT64_C(0x1f4304cef4ad871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43080f2cab8718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000667f), UINT64_C(0x1f42f84f44b6a6f6), UINT64_C(0x1f42fc0f14a8e6ec),
        UINT64_C(0x1f42f88ff0b6a6fb), UINT64_C(0x1f42fa0facb3a71f)
    },
    {   UINT64_C(0x0000000000005877), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014e54b9c73b), UINT64_C(0x1f430a4f48b24742)
    },
    {   UINT64_C(0x0000000000006680), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ce84b36734),
        UINT64_C(0x1f42f88e8cb0e71d), UINT64_C(0x1f42fa4facb3c6ea)
    },
    {   UINT64_C(0x0000000000005879), UINT64_C(0x1f42ff0e80ae0729), UINT64_C(0x1f42f6cee0ac0733),
        UINT64_C(0x1f42fe4eacaf66f8), UINT64_C(0x1f43000e90b14721)
    },
    {   UINT64_C(0x000000000000587a), UINT64_C(0x1f42fc0f58af06f0), UINT64_C(0x1f42fe4e60b8874f),
        UINT64_C(0x1f42f80e80b8e709), UINT64_C(0x1f42fe4e7cafe700)
    },
    {   UINT64_C(0x000000000000587b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e84b0672c),
        UINT64_C(0x1f42ff8e90b3670d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000587c), UINT64_C(0x1f42fe0f10ac2703), UINT64_C(0x1f42fe8e98b2a726),
        UINT64_C(0x1f42f8cf5cb1272d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000587d), UINT64_C(0x1f42fd0ed4b40716), UINT64_C(0x1f42fe0ec4b24736),
        UINT64_C(0x1f42fc0f8caf671b), UINT64_C(0x1f42fe0f00b8073a)
    },
    {   UINT64_C(0x000000000000587e), UINT64_C(0x1f43110f14b08721), UINT64_C(0x1f43008ea0b1871f),
        UINT64_C(0x1f42fcced4b3e716), UINT64_C(0x1f42ff0f10ac4708)
    },
    {   UINT64_C(0x000000000000587f), UINT64_C(0x1f43020e6cb2c729), UINT64_C(0x1f43010e7cb2e73d),
        UINT64_C(0x1f43010e7cb5c72c), UINT64_C(0x1f42ffcf74b12722)
    },
    {   UINT64_C(0x0000000000005880), UINT64_C(0x1f430d4f7cacc735), UINT64_C(0x1f43050eb0b0e726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cebcb34745)
    },
    {   UINT64_C(0x0000000000005881), UINT64_C(0x1f4300ce68b4071f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c0f14b1e72e), UINT64_C(0x1f43050ea0b06722)
    },
    {   UINT64_C(0x0000000000005882), UINT64_C(0x1f43178eacad66fb), UINT64_C(0x1f43178e30aec71b),
        UINT64_C(0x1f4304ce88b2a730), UINT64_C(0x1f4311ceecabe700)
    },
    {   UINT64_C(0x0000000000005883), UINT64_C(0x1f4300ce84b00731), UINT64_C(0x1f4304ce70b2271d),
        UINT64_C(0x1f43018ebcb08734), UINT64_C(0x1f43008ef4b1e71a)
    },
    {   UINT64_C(0x0000000000005884), UINT64_C(0x1f43038eccb4c72b), UINT64_C(0x1f43028e88b2a72b),
        UINT64_C(0x1f4302ceb4b52731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005885), UINT64_C(0x1f43224e98b18732), UINT64_C(0x1f42fd8efcb08715),
        UINT64_C(0x1f43040ea8b1872d), UINT64_C(0x1f42fccf58b32708)
    },
    {   UINT64_C(0x0000000000005886), UINT64_C(0x1f43094f1cb266fc), UINT64_C(0x1f42fe8eb4b3271a),
        UINT64_C(0x1f42f90eb0b50729), UINT64_C(0x1f431a8e1caf672a)
    },
    {   UINT64_C(0x0000000000005887), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f58b5e708),
        UINT64_C(0x1f4305ce1caf8738), UINT64_C(0x1f42fa8f28b18725)
    },
    {   UINT64_C(0x0000000000005888), UINT64_C(0x1f43014ed8afa713), UINT64_C(0x1f42f80e94b5e711),
        UINT64_C(0x1f43028eacb0a734), UINT64_C(0x1f4306cef4af06ea)
    },
    {   UINT64_C(0x0000000000005889), UINT64_C(0x1f43048f14afc6f3), UINT64_C(0x1f4308ceb4b46715),
        UINT64_C(0x1f42fe4ed0b36716), UINT64_C(0x1f42f7ce50b3070c)
    },
    {   UINT64_C(0x000000000000588a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010ea8b56716),
        UINT64_C(0x1f42fc0ef0b0272a), UINT64_C(0x1f42f7cf50b3e701)
    },
    {   UINT64_C(0x000000000000588b), UINT64_C(0x1f4303ced4b20706), UINT64_C(0x1f43034f18ae26fe),
        UINT64_C(0x1f42f98f04b2e719), UINT64_C(0x1f42fbce68b4a705)
    },
    {   UINT64_C(0x000000000000588c), UINT64_C(0x1f42fb8ed0b22716), UINT64_C(0x1f42fa8e78b5470c),
        UINT64_C(0x1f42f74e78b4272c), UINT64_C(0x1f42f88ee8b50723)
    },
    {   UINT64_C(0x000000000000588d), UINT64_C(0x1f43090ea4b18719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4312cf00a926e7), UINT64_C(0x1f43038ecca0071f)
    },
    {   UINT64_C(0x000000000000668b), UINT64_C(0x1f42facefcb4c704), UINT64_C(0x1f43178e64a6e72c),
        UINT64_C(0x1f42facfacaf26d9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f8dc), UINT64_C(0x0000000000000000), UINT64_C(0x16f2f4cbf49746ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000668c), UINT64_C(0x1f42f68e68b6e722), UINT64_C(0x1f42facf8caee718),
        UINT64_C(0x1f42face5cb3471d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005891), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4ea8b64739), UINT64_C(0x1f42fd8ea0a3872d)
    },
    {   UINT64_C(0x000000000000668d), UINT64_C(0x1f42fc4f88b166ed), UINT64_C(0x1f42fb0fb8b026d9),
        UINT64_C(0x1f42fa8efcabc70c), UINT64_C(0x1f42f74f64b7e6f3)
    },
    {   UINT64_C(0x0000000000005893), UINT64_C(0x1f43314f648e0700), UINT64_C(0x1f430b0e9cab2726),
        UINT64_C(0x1f42f78f00b266fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005894), UINT64_C(0x1f42fe8f0cada6f5), UINT64_C(0x1f42fa0eacb04700),
        UINT64_C(0x1f42f54ee0b8e6e9), UINT64_C(0x1f42f90ec4b0270c)
    },
    {   UINT64_C(0x0000000000005895), UINT64_C(0x1f42f3cf38aca710), UINT64_C(0x1f42fd0f38b2c710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005896), UINT64_C(0x1f42facf1cb6e709), UINT64_C(0x1f42facf00b5c6ed),
        UINT64_C(0x1f42ff0e98aea6e9), UINT64_C(0x1f42f90ed0b36718)
    },
    {   UINT64_C(0x0000000000005897), UINT64_C(0x1f42f98ec8b28704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0e9cb44700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005898), UINT64_C(0x1f42f94f20b686e8), UINT64_C(0x1f42fb4f00b6c6e9),
        UINT64_C(0x1f42faceecb2e6fc), UINT64_C(0x1f42f94ee0ac471c)
    },
    {   UINT64_C(0x0000000000005899), UINT64_C(0x1f42f5cf10b846f5), UINT64_C(0x1f42fd0f00b326e4),
        UINT64_C(0x1f42fa4eacb4a6f0), UINT64_C(0x1f42fc8f40b34703)
    },
    {   UINT64_C(0x000000000000589a), UINT64_C(0x1f42f6cf20b72710), UINT64_C(0x1f42facf44ad26f5),
        UINT64_C(0x1f42fb4f48b02708), UINT64_C(0x1f42f94f40a8e6fc)
    },
    {   UINT64_C(0x000000000000589b), UINT64_C(0x1f42f90f40b10710), UINT64_C(0x1f42f90f04b2e711),
        UINT64_C(0x1f42f80f38b92708), UINT64_C(0x1f42f70f2cb56710)
    },
    {   UINT64_C(0x000000000000589c), UINT64_C(0x1f42f8cf30b486d9), UINT64_C(0x1f42fa0f84b166f9),
        UINT64_C(0x1f42fd4f1cb6c6e5), UINT64_C(0x1f42f70ed0b62701)
    },
    {   UINT64_C(0x000000000000589d), UINT64_C(0x1f42fe4e5cb886f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf48b72718), UINT64_C(0x1f42fa4f04b2c6e9)
    },
    {   UINT64_C(0x000000000000589e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf14b6e701),
        UINT64_C(0x1f42fc4f24b006e1), UINT64_C(0x1f42fc4ef8aee6f1)
    },
    {   UINT64_C(0x000000000000589f), UINT64_C(0x1f42f4cf14b52710), UINT64_C(0x1f42fa0f04b686e5),
        UINT64_C(0x1f42fe4f28b3071b), UINT64_C(0x1f42ff0ee4b72704)
    },
    {   UINT64_C(0x00000000000058a0), UINT64_C(0x1f42f94ee8b7871f), UINT64_C(0x1f42fb0f24b7e6e5),
        UINT64_C(0x1f42f80ee8b74714), UINT64_C(0x1f43000f24aec6fc)
    },
    {   UINT64_C(0x00000000000058a1), UINT64_C(0x1f42f74eccb62727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf88b54708), UINT64_C(0x1f42fd4f04ac46e9)
    },
    {   UINT64_C(0x00000000000058a2), UINT64_C(0x1f42fd0eeca446f1), UINT64_C(0x1f42f54f04b5a710),
        UINT64_C(0x1f42f88f3cb66711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058a3), UINT64_C(0x1f42fc0f30b3e6e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f04af66f0), UINT64_C(0x1f42fccf04b5e6f1)
    },
    {   UINT64_C(0x00000000000058a4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0e5caee724),
        UINT64_C(0x1f42f58e78b4272b), UINT64_C(0x1f42f38f38ba0705)
    },
    {   UINT64_C(0x00000000000058a5), UINT64_C(0x1f42fc0f18aca6f5), UINT64_C(0x1f42f88f34b7c705),
        UINT64_C(0x1f42f6ce4cb6071d), UINT64_C(0x1f42f68f44b6e714)
    },
    {   UINT64_C(0x00000000000058a6), UINT64_C(0x1f42f8ce6cb44720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facf44b0c6f5), UINT64_C(0x1f42f88ef4b3e700)
    },
    {   UINT64_C(0x00000000000058a7), UINT64_C(0x1f42f3cf58b8a725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f28f84b946f8), UINT64_C(0x1f42fccea0b1271a)
    },
    {   UINT64_C(0x0000000000006698), UINT64_C(0x1f42f80fa4b446fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cf90b52729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058a9), UINT64_C(0x1f42f80fb0a96720), UINT64_C(0x1f4300cf44b5671c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058aa), UINT64_C(0x1f42f50e7cb4e718), UINT64_C(0x1f42f54eb4b686f9),
        UINT64_C(0x1f42f74e8ca886ed), UINT64_C(0x1f42fa4f48b92719)
    },
    {   UINT64_C(0x00000000000058ab), UINT64_C(0x1f42f80f4cafe700), UINT64_C(0x1f42f74f44b6c6f9),
        UINT64_C(0x1f42f54ee0b6a6f9), UINT64_C(0x1f42f7cf44b4e6fc)
    },
    {   UINT64_C(0x00000000000058ac), UINT64_C(0x1f42f54f44b346f9), UINT64_C(0x1f42f7cf14b40728),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf54b566fc)
    },
    {   UINT64_C(0x00000000000058ad), UINT64_C(0x1f42f54f44b586f9), UINT64_C(0x1f42f58f98b7a724),
        UINT64_C(0x1f42f90f24b826f9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058ae), UINT64_C(0x1f42f54f44b546f9), UINT64_C(0x1f42f64f08b5c722),
        UINT64_C(0x1f42f2ce80b92723), UINT64_C(0x1f42f7cea8ad46fd)
    },
    {   UINT64_C(0x00000000000058af), UINT64_C(0x1f42f80e7caee72b), UINT64_C(0x1f42f94f34b0c708),
        UINT64_C(0x1f42f80f34ae46f9), UINT64_C(0x1f42fa0f50b24700)
    },
    {   UINT64_C(0x00000000000058b0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5ce6cb2a73f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058b1), UINT64_C(0x1f42efcf4cb32711), UINT64_C(0x1f42f84f24b006f9),
        UINT64_C(0x1f42f84e7cafc718), UINT64_C(0x1f42f7cf30b44708)
    },
    {   UINT64_C(0x00000000000058b2), UINT64_C(0x1f42f5cf2caac714), UINT64_C(0x1f42f5cf18b74720),
        UINT64_C(0x1f42f78ef8b146f1), UINT64_C(0x1f42f84f20b3a700)
    },
    {   UINT64_C(0x00000000000058b3), UINT64_C(0x1f42f44e4cb3071c), UINT64_C(0x1f42f9cf5cac0718),
        UINT64_C(0x1f42fb4f20b54700), UINT64_C(0x1f43078ea0b12729)
    },
    {   UINT64_C(0x000000000000669e), UINT64_C(0x1f42f68f34b466d5), UINT64_C(0x1f42fc0f44b0471b),
        UINT64_C(0x1f42f68f9cac26e4), UINT64_C(0x1f42f84f94b166ea)
    },
    {   UINT64_C(0x00000000000058b5), UINT64_C(0x1f42f74f14b6a714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f14b6e714), UINT64_C(0x1f42f20f20af26fc)
    },
    {   UINT64_C(0x000000000000669f), UINT64_C(0x1f42f9cfa4b1c6fa), UINT64_C(0x1f43094e84b1072d),
        UINT64_C(0x1f4312cee8a4c71f), UINT64_C(0x1f42f60ee8b786f3)
    },
    {   UINT64_C(0x00000000000058b7), UINT64_C(0x1f42fc8eb8b6670c), UINT64_C(0x1f42f90eb8b60718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f00b0c70c)
    },
    {   UINT64_C(0x00000000000058b8), UINT64_C(0x1f42f7cf20b6c701), UINT64_C(0x1f4301cf80b1c708),
        UINT64_C(0x1f42fc0f78b48701), UINT64_C(0x1f42fa0efcadc723)
    },
    {   UINT64_C(0x00000000000058b9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4f2cac0700),
        UINT64_C(0x1f42f5cf14b06711), UINT64_C(0x1f42fa8f18ae870a)
    },
    {   UINT64_C(0x00000000000058ba), UINT64_C(0x1f42fa0f24b4470b), UINT64_C(0x1f42fb0eb8b5e6f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ef4b4c6f1)
    },
    {   UINT64_C(0x00000000000058bb), UINT64_C(0x1f42f7cef4b5e710), UINT64_C(0x1f42f50fb0b56713),
        UINT64_C(0x1f42f98efcae0729), UINT64_C(0x1f42f7cf04b8a710)
    },
    {   UINT64_C(0x00000000000058bc), UINT64_C(0x1f42f6cf08b3a6f5), UINT64_C(0x1f42f74f38b28704),
        UINT64_C(0x1f42fc0e3cb9873c), UINT64_C(0x1f42f80e70b7272d)
    },
    {   UINT64_C(0x00000000000058bd), UINT64_C(0x1f42f88edcb726f9), UINT64_C(0x1f42fd8eecb286f5),
        UINT64_C(0x1f42f9cec4a746ff), UINT64_C(0x1f42fa8ea4b3a729)
    },
    {   UINT64_C(0x00000000000058be), UINT64_C(0x1f42fa0e8cad46fc), UINT64_C(0x1f42fa4f04b2a6e9),
        UINT64_C(0x1f42fe8eccb5c706), UINT64_C(0x1f42fb0f34b34700)
    },
    {   UINT64_C(0x00000000000058bf), UINT64_C(0x1f42f24e9cb3670a), UINT64_C(0x1f42f60e24b1e728),
        UINT64_C(0x1f42f80f84ba2718), UINT64_C(0x1f43094e14b90741)
    },
    {   UINT64_C(0x00000000000058c0), UINT64_C(0x1f42f70f24b606f5), UINT64_C(0x1f42f80e34b5072f),
        UINT64_C(0x1f42f80ebcaf8715), UINT64_C(0x1f42f98f04b106f5)
    },
    {   UINT64_C(0x00000000000058c1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f2cae8714)
    },
    {   UINT64_C(0x00000000000058c2), UINT64_C(0x1f4308ce58aac739), UINT64_C(0x1f42f80f44b5270c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f04ae46ed)
    },
    {   UINT64_C(0x000000000000800c), UINT64_C(0x0fa2f40f687a068c), UINT64_C(0x0fa309cfa06fc68c),
        UINT64_C(0x0fa2f54fa079c68c), UINT64_C(0x0fa2f84f6874668c)
    },
    {   UINT64_C(0x000000000000800d), UINT64_C(0x0fa2f10fa079a68c), UINT64_C(0x0fa2f5cf68754698),
        UINT64_C(0x0fa2f68f68790694), UINT64_C(0x0fa2f30f8076a67e)
    },
    {   UINT64_C(0x000000000000800e), UINT64_C(0x0fa2fc0f6876268c), UINT64_C(0x0fa2f58f4877e68c),
        UINT64_C(0x08b3024f745685b3), UINT64_C(0x08b3020d6c59e5b3)
    },
    {   UINT64_C(0x000000000000800f), UINT64_C(0x156afc0f408c25fe), UINT64_C(0x156afa11808e256a),
        UINT64_C(0x156afc0f408c25fe), UINT64_C(0x0de2ff0f8072468c)
    },
    {   UINT64_C(0x00000000000058c7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f3cb3a6f4),
        UINT64_C(0x1f42fa4f90b46717), UINT64_C(0x1f42fa8f40b246f9)
    },
    {   UINT64_C(0x00000000000058c8), UINT64_C(0x1f42f84f40b5e704), UINT64_C(0x1f42f90f64a9a6f9),
        UINT64_C(0x1f42fc0f08b5c700), UINT64_C(0x1f42f7ceccb3e714)
    },
    {   UINT64_C(0x09211c2340008012), UINT64_C(0x1692f28fa096667e), UINT64_C(0x0a6af28fa065e67e),
        UINT64_C(0x0baaec8f686ac67e), UINT64_C(0x1692f64fa098e67e)
    },
    {   UINT64_C(0x00000000000058ca), UINT64_C(0x1f42f94f94b80720), UINT64_C(0x1f42f4ceccb9c6f5),
        UINT64_C(0x1f42f44e5cb22737), UINT64_C(0x1f42fe8f34b44718)
    },
    {   UINT64_C(0x00000000000058cb), UINT64_C(0x1f42f80e54b3a72a), UINT64_C(0x1f42fa4ef0b686ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f28b70718)
    },
    {   UINT64_C(0x1952c855c9088015), UINT64_C(0x131af24fa088a67e), UINT64_C(0x1692f18f6896a67e),
        UINT64_C(0x1692f20f7495a67e), UINT64_C(0x1d82ef4f74b3067e)
    },
    {   UINT64_C(0x0003e8740e308016), UINT64_C(0x1692f08fa096867e), UINT64_C(0x1692f38fa096667e),
        UINT64_C(0x1322f08fa089467e), UINT64_C(0x0a6b040d3063e67e)
    },
    {   UINT64_C(0x0000000000008017), UINT64_C(0x08b2ebcf6862668c), UINT64_C(0x0a6af68d305d267e),
        UINT64_C(0x0eaaf4144476467e), UINT64_C(0x0fa2ee0dac7d039d)
    },
    {   UINT64_C(0x00000000000058cf), UINT64_C(0x1f42f7cf20b46703), UINT64_C(0x1f42f90f54b7e713),
        UINT64_C(0x1f42f6cf34b7a70b), UINT64_C(0x1f42fa0eccb20720)
    },
    {   UINT64_C(0x22240e8040008019), UINT64_C(0x0fa2f4cf787be693), UINT64_C(0x0de2f44f787385b4),
        UINT64_C(0x0a6af30fa06e6693), UINT64_C(0x0fa2f50c387945b4)
    },
    {   UINT64_C(0x00000000000058d1), UINT64_C(0x1f42f80e98b6a70c), UINT64_C(0x1f42f80e98ab46eb),
        UINT64_C(0x1f42f60f34ac0720), UINT64_C(0x1f42f80ec0b32712)
    },
    {   UINT64_C(0x00000000000058d2), UINT64_C(0x1f42f7cee4b82705), UINT64_C(0x1f42f88f18b84720),
        UINT64_C(0x1f42f7ce84b5670e), UINT64_C(0x1f42f7ce6cb38712)
    },
    {   UINT64_C(0x28b4ba96d2c8801c), UINT64_C(0x0fa2ef4c3c7945b4), UINT64_C(0x0fa2f40c3c7b85b4),
        UINT64_C(0x0a6aef4fa06865b4), UINT64_C(0x0c2afacf786a05b3)
    },
    {   UINT64_C(0x2b5536a69468801d), UINT64_C(0x08b2f14f7861662b), UINT64_C(0x0fa2f04f787a85aa),
        UINT64_C(0x0fa2f2cf687a25aa), UINT64_C(0x1692f34f689545aa)
    },
    {   UINT64_C(0x000000b55678801e), UINT64_C(0x0fa2f24fa07a65aa), UINT64_C(0x0fa2f18c3879e5aa),
        UINT64_C(0x0de2f4cfa072c5aa), UINT64_C(0x0f03070ccc6a4690)
    },
    {   UINT64_C(0x00000000000058d6), UINT64_C(0x1f42f7ced8b5e712), UINT64_C(0x1f42f7ce6cb4c725),
        UINT64_C(0x1f42f80f00b0c6e9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058d7), UINT64_C(0x1f42f84f50b6672d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf34aec708), UINT64_C(0x1f42fa4f1caa0704)
    },
    {   UINT64_C(0x00000000000058d8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f14b5c714),
        UINT64_C(0x1f42f8cf54b1c6fc), UINT64_C(0x1f42f8cef0b0c71d)
    },
    {   UINT64_C(0x00000000000058d9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f3cb346f0),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e3cb7a73e)
    },
    {   UINT64_C(0x00000000000058da), UINT64_C(0x1f42f78f64b94720), UINT64_C(0x1f42f90f30b5c712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84ed4aea70e)
    },
    {   UINT64_C(0x00000000000058db), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0e8cac8701),
        UINT64_C(0x1f42f88f90ae86f8), UINT64_C(0x1f42fa0eecab46ff)
    },
    {   UINT64_C(0x00000000000058dc), UINT64_C(0x1f42ffceacb5c6d5), UINT64_C(0x1f42f60f34b3c714),
        UINT64_C(0x1f42f6ced4b7e722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058dd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cfacb0c6e8),
        UINT64_C(0x1f42f28f14b3c714), UINT64_C(0x1f42f7ced0b56729)
    },
    {   UINT64_C(0x00000000000066b3), UINT64_C(0x1f42fbcf34b346f9), UINT64_C(0x1f42f90f00b84703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42faced0b32720)
    },
    {   UINT64_C(0x0000000000008028), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a53060c3857c5bc),
        UINT64_C(0x0fa2fb4d7876862b), UINT64_C(0x0fa2f10f807bc68c)
    },
    {   UINT64_C(0x00000000000058e0), UINT64_C(0x1f42f50ef4b5c6f9), UINT64_C(0x1f42facf14b486f8),
        UINT64_C(0x1f42fd0f80a946f3), UINT64_C(0x1f42f8ce5cb5c733)
    },
    {   UINT64_C(0x000000000000802a), UINT64_C(0x0a6af297cc660656), UINT64_C(0x17c2f78f6898c68c),
        UINT64_C(0x08a2ef15c860468c), UINT64_C(0x0fa2f89024778588)
    },
    {   UINT64_C(0x00000000000066b5), UINT64_C(0x1f42fe8f18ac4710), UINT64_C(0x1f42fa4ee4b50725),
        UINT64_C(0x1f42f68e7cb62725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000802c), UINT64_C(0x0c82fc55b869668c), UINT64_C(0x156afa0e448e25de),
        UINT64_C(0x0962ee53d863868c), UINT64_C(0x0962f153b862e68c)
    },
    {   UINT64_C(0x000000000000802d), UINT64_C(0x0a6b05578c66268e), UINT64_C(0x0fa2f70c387e85aa),
        UINT64_C(0x1282fd0d3881067e), UINT64_C(0x07d2ff92bc55251e)
    },
    {   UINT64_C(0x2ef000000000802e), UINT64_C(0x0a6afb0c445f23e8), UINT64_C(0x0962fc53d85d868c),
        UINT64_C(0x09b2ef95c864868c), UINT64_C(0x0fa2f8d02477c588)
    },
    {   UINT64_C(0x000000000000802f), UINT64_C(0x1772f88f6897a68c), UINT64_C(0x1772f70f689b268c),
        UINT64_C(0x1772f14f6898468c), UINT64_C(0x0de2f58c386d85bb)
    },
    {   UINT64_C(0x32b640c0d7c08030), UINT64_C(0x1692f1cfa096e788), UINT64_C(0x1692f1cfa096e788),
        UINT64_C(0x1692f1cfa096e784), UINT64_C(0x1692f1cfa096e760)
    },
    {   UINT64_C(0x36f000d499b08031), UINT64_C(0x1692f1cfa096e750), UINT64_C(0x1692f1cfa096e7a9),
        UINT64_C(0x1bcaee0fa0aa667e), UINT64_C(0x14daf50c448da68c)
    },
    {   UINT64_C(0x0000000000008032), UINT64_C(0x131af10fa0892788), UINT64_C(0x131af10fa0892788),
        UINT64_C(0x131af10fa0892784), UINT64_C(0x131af10fa0892750)
    },
    {   UINT64_C(0x0000000000008033), UINT64_C(0x0a6af28fa065e788), UINT64_C(0x0a6af28fa065e788),
        UINT64_C(0x0a6af08fa066a784), UINT64_C(0x0a6af24fa065e750)
    },
    {   UINT64_C(0x3736e00000008034), UINT64_C(0x1692f20fa094267e), UINT64_C(0x1692f10f7496e760),
        UINT64_C(0x1692f68f68948788), UINT64_C(0x1692f68f68948788)
    },
    {   UINT64_C(0x0006fcdedbc08035), UINT64_C(0x1692f68f68948784), UINT64_C(0x1692f68f68948760),
        UINT64_C(0x1692f68f68948750), UINT64_C(0x1402f8cf3c8925f9)
    },
    {   UINT64_C(0x393718e15c208036), UINT64_C(0x1692f68e609486c6), UINT64_C(0x1692f18f6896a788),
        UINT64_C(0x1692f18f6896a788), UINT64_C(0x1692f18f6896a784)
    },
    {   UINT64_C(0x000000001ca08037), UINT64_C(0x1692f18f6896a750), UINT64_C(0x1692f38fa0966788),
        UINT64_C(0x1162f20fa082467e), UINT64_C(0x0fa2f10f7c790693)
    },
    {   UINT64_C(0x000000e7dce08038), UINT64_C(0x0de2f74f7875c68e), UINT64_C(0x0de2f74f7875c68e),
        UINT64_C(0x0de2f74f7875c68a), UINT64_C(0x0de2f74f7875c666)
    },
    {   UINT64_C(0x000750e95d108039), UINT64_C(0x0de2f74f7875c656), UINT64_C(0x0de2f74f7875c6b1),
        UINT64_C(0x14daf78f848d85b4), UINT64_C(0x0de2f44c447205b4)
    },
    {   UINT64_C(0x00075ceac000803a), UINT64_C(0x0de2f44f7873868e), UINT64_C(0x0de2f44f7873868e),
        UINT64_C(0x0de2f44f7873868a), UINT64_C(0x0de2f44f78738656)
    },
    {   UINT64_C(0x000000000000803b), UINT64_C(0x08b2ef8fa05ce68e), UINT64_C(0x08b2f20fa060a68e),
        UINT64_C(0x08b2e9cfa063468a), UINT64_C(0x08b2eb8fa0628656)
    },
    {   UINT64_C(0x3b4762000000803c), UINT64_C(0x0fa2f54f78796696), UINT64_C(0x0fa2f20fa07b2666),
        UINT64_C(0x0fa2f54f7879468e), UINT64_C(0x0fa2f54f7879468e)
    },
    {   UINT64_C(0x00000000000058f4), UINT64_C(0x1f42f7cf14b66728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f44b1e720), UINT64_C(0x1f42fbcf6cb346f5)
    },
    {   UINT64_C(0x000000000000803e), UINT64_C(0x0fa2f54de079460f), UINT64_C(0x0fa2f04f787a868e),
        UINT64_C(0x0fa2f04f787a868e), UINT64_C(0x0fa2f04f787a868a)
    },
    {   UINT64_C(0x00000000000066bf), UINT64_C(0x1f430ace8cb466ff), UINT64_C(0x1f430b8f38aa46ef),
        UINT64_C(0x1f43088e7ca4c70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008040), UINT64_C(0x1692f1cfa096e6ed), UINT64_C(0x0de2f74f7875c619),
        UINT64_C(0x1692f1cfa096e754), UINT64_C(0x0de2f74f7875c680)
    },
    {   UINT64_C(0x00000000000066c0), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8ee0a5c707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430cce7cae6730)
    },
    {   UINT64_C(0x0000000000008042), UINT64_C(0x14daf50f688da75e), UINT64_C(0x0de2f44f7872068a),
        UINT64_C(0x14daf50f688da72a), UINT64_C(0x0de2f44f78720656)
    },
    {   UINT64_C(0x00000000000066c1), UINT64_C(0x1f430ece68a5e721), UINT64_C(0x1f4300ce70a98727),
        UINT64_C(0x1f4301cec0b4a728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008044), UINT64_C(0x1692f20fa094267e), UINT64_C(0x0fa2f4cf787de693),
        UINT64_C(0x131af10fa08926ed), UINT64_C(0x0de2f44f78738619)
    },
    {   UINT64_C(0x00000000000058fc), UINT64_C(0x1f42fd0ea4aee6e8), UINT64_C(0x1f43114e60ae0717),
        UINT64_C(0x1f42fa0f28ae870d), UINT64_C(0x1f42f78f3cb0e70e)
    },
    {   UINT64_C(0x00000000000058fd), UINT64_C(0x1f42f78edcb78711), UINT64_C(0x1f42fa0f4cb226f0),
        UINT64_C(0x1f42f98e9cb0471b), UINT64_C(0x1f42fa0ea0b1e720)
    },
    {   UINT64_C(0x00000000000058fe), UINT64_C(0x1f42fc8f58b42710), UINT64_C(0x1f42f80eb0b36715),
        UINT64_C(0x1f43008f14b246fc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058ff), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf60b2c70e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f5cb5c6fb)
    },
    {   UINT64_C(0x0000000000008049), UINT64_C(0x1692f28fa096675e), UINT64_C(0x0fa2f04fa07b675e),
        UINT64_C(0x169af24fa096467e), UINT64_C(0x0fa2f00fa07b6693)
    },
    {   UINT64_C(0x000000000000804a), UINT64_C(0x0a6aee4fa067e73a), UINT64_C(0x08b2e7cfa064a666),
        UINT64_C(0x0a6af0cfa066c6ed), UINT64_C(0x08b2e8cfa062c619)
    },
    {   UINT64_C(0x00000000000066c5), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070e64a9673d),
        UINT64_C(0x1f43040e0cb5473d), UINT64_C(0x1f430b0ebca78731)
    },
    {   UINT64_C(0x000000000000804c), UINT64_C(0x0a6af28fa065e72a), UINT64_C(0x08b2f20fa05e25b4),
        UINT64_C(0x175af28f6899867e), UINT64_C(0x10d2f20c38774693)
    },
    {   UINT64_C(0x0000000000005904), UINT64_C(0x1f42f80ee4a9270e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4ed4b3e70a), UINT64_C(0x1f42f7cf00b2c72a)
    },
    {   UINT64_C(0x000000000000804e), UINT64_C(0x10f2ef4fa08105b3), UINT64_C(0x131af10fa0894762),
        UINT64_C(0x08b2f2cfa05ea762), UINT64_C(0x131af10b4089467e)
    },
    {   UINT64_C(0x0000000000005906), UINT64_C(0x1f42fb0e50b3470a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0e50b4e72b), UINT64_C(0x1f42fb0e9cb54735)
    },
    {   UINT64_C(0x0000000000008050), UINT64_C(0x0b92f2cfa06ac693), UINT64_C(0x131af08fa089267e),
        UINT64_C(0x08b2f2cfa05ee693), UINT64_C(0x1692f10f7496e762)
    },
    {   UINT64_C(0x0000000000005908), UINT64_C(0x1f42f8cf44ae46f7), UINT64_C(0x1f42f70e9cb7a70c),
        UINT64_C(0x1f42facf14b30704), UINT64_C(0x1f42f60e84b72702)
    },
    {   UINT64_C(0x0000000000005909), UINT64_C(0x1f42f80ef4b2a711), UINT64_C(0x1f42f5cf28b806f8),
        UINT64_C(0x1f42f38ed8abc703), UINT64_C(0x1f42f84e80af6729)
    },
    {   UINT64_C(0x000000000000590a), UINT64_C(0x1f42fb4ebcac06f8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5cf0cb5c715), UINT64_C(0x1f43080eb8ae8720)
    },
    {   UINT64_C(0x000000000000590b), UINT64_C(0x1f42f70f7cb786fc), UINT64_C(0x1f42fa8ef4b7e70d),
        UINT64_C(0x1f42f64e7cb7c6fc), UINT64_C(0x1f42f74f38af46f8)
    },
    {   UINT64_C(0x0000000000008055), UINT64_C(0x14daf24fa0910762), UINT64_C(0x0a6aef4fa068668e),
        UINT64_C(0x14daf24b4091067e), UINT64_C(0x0a6aef4b406865b4)
    },
    {   UINT64_C(0x000000000000590d), UINT64_C(0x1f42ff4e84b60730), UINT64_C(0x1f42f80f20b52708),
        UINT64_C(0x1f42f98eecb92727), UINT64_C(0x1f42fa4eccb5672f)
    },
    {   UINT64_C(0x0000000000008057), UINT64_C(0x1162f8cf687be75e), UINT64_C(0x0c2af80f786a668a),
        UINT64_C(0x1162f8cc447be68c), UINT64_C(0x0c2afacc446a05b3)
    },
    {   UINT64_C(0x0000000000008058), UINT64_C(0x1162f8cf687be784), UINT64_C(0x0c2af78f786a268a),
        UINT64_C(0x131af24c4488a67e), UINT64_C(0x08b2f14c4461662b)
    },
    {   UINT64_C(0x0000000000008059), UINT64_C(0x131af24fa088a75e), UINT64_C(0x09daf14f786826a5),
        UINT64_C(0x132af24fa088a67e), UINT64_C(0x08baf0cf8c618630)
    },
    {   UINT64_C(0x000000000000805a), UINT64_C(0x1692f18f6896a739), UINT64_C(0x0fa2f04f787a8666),
        UINT64_C(0x1692f18f6896a6ed), UINT64_C(0x0fa2f04f787a8619)
    },
    {   UINT64_C(0x0000000000005912), UINT64_C(0x1f42f78f94af470c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68f98b046fc), UINT64_C(0x1f42f38f14b4a6ff)
    },
    {   UINT64_C(0x0000000000005913), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf20a8e6e7),
        UINT64_C(0x1f42f90ebcb82705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000805d), UINT64_C(0x1d82ef4f74b3075e), UINT64_C(0x1692f34f6895468a),
        UINT64_C(0x1692f38fa096675e), UINT64_C(0x0fa2f18c3879e68a)
    },
    {   UINT64_C(0x0000000000005915), UINT64_C(0x1f42f6ce6cb3672d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005916), UINT64_C(0x1f42f58eeca9a727), UINT64_C(0x1f42f84f04b006fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ee8b1a725)
    },
    {   UINT64_C(0x0000000000005917), UINT64_C(0x1f42f54edcaee708), UINT64_C(0x1f42f74eecb66702),
        UINT64_C(0x1f42f74ec8ac26fc), UINT64_C(0x1f42fa8ee8b24728)
    },
    {   UINT64_C(0x00000000000066d0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f78b7071b),
        UINT64_C(0x1f42fb4f78ba2713), UINT64_C(0x1f42facfccb4e712)
    },
    {   UINT64_C(0x0000000000008062), UINT64_C(0x0fa2f44f787e6617), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066d1), UINT64_C(0x1f430c4f84b94721), UINT64_C(0x1f42f98f04b36727),
        UINT64_C(0x1f42f98f70b546fe), UINT64_C(0x1f42f78ee8b16703)
    },
    {   UINT64_C(0x0000000000008064), UINT64_C(0x1222f64f8884c694), UINT64_C(0x0000000000000000),
        UINT64_C(0x0d92ef8d2473c6aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008065), UINT64_C(0x0000000000000000), UINT64_C(0x16faf04f78974693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000591d), UINT64_C(0x1f42f74e84b44718), UINT64_C(0x1f42f8cf0cb52702),
        UINT64_C(0x1f42f90eb0b26719), UINT64_C(0x1f42f88e6cb8e72b)
    },
    {   UINT64_C(0x000000000000591e), UINT64_C(0x1f42fc4f64b5472c), UINT64_C(0x1f42fece88b56743),
        UINT64_C(0x1f42f6cf3cb14718), UINT64_C(0x1f42f74e70b50743)
    },
    {   UINT64_C(0x0000000000008068), UINT64_C(0x1692f68f689486da), UINT64_C(0x110af54f7880e5c2),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008069), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2ef4c3c794685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000806a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0d82f30bfc720694), UINT64_C(0x0912f14c3861662b)
    },
    {   UINT64_C(0x0000000000005922), UINT64_C(0x1f42f60ef0b40710), UINT64_C(0x1f4311ce44b28733),
        UINT64_C(0x1f42f8cee4aec6f7), UINT64_C(0x1f42f8cef4ae66f7)
    },
    {   UINT64_C(0x000000000000806c), UINT64_C(0x0fa2f04f7881e619), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066d6), UINT64_C(0x1f42f58e5cb0071e), UINT64_C(0x1f43148e24ae2734),
        UINT64_C(0x1f43088f4cb02739), UINT64_C(0x1f42f88f90b12700)
    },
    {   UINT64_C(0x000000000000806e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0beaef0bf86b85aa), UINT64_C(0x0fa2f38fa07ac68c)
    },
    {   UINT64_C(0x000000000000806f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa3004f7073e603), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008070), UINT64_C(0x0502fb8fa04ba6c8), UINT64_C(0x08c2fb8fa05aa6c8),
        UINT64_C(0x0d9af68fa07086c8), UINT64_C(0x0a6b0e8f7c5c068c)
    },
    {   UINT64_C(0x00000000000066d8), UINT64_C(0x1f4300cfa8b5e703), UINT64_C(0x1f43060f54b0e6fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005929), UINT64_C(0x1f42f90f20b9270c), UINT64_C(0x1f42f7cf28b7870f),
        UINT64_C(0x1f42f90ed0b4e6fc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066d9), UINT64_C(0x1f43008fc4b7a70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010ec8b426f2), UINT64_C(0x1f42ff4f40b6c6fa)
    },
    {   UINT64_C(0x000000000000592b), UINT64_C(0x1f42f4cf50b5c717), UINT64_C(0x1f42f6cee4ba0720),
        UINT64_C(0x1f42f4cef8b46710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066da), UINT64_C(0x1f4301cfb8b726ed), UINT64_C(0x1f42fccef8b426e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000592d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74eccb2c717),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf8cae870b)
    },
    {   UINT64_C(0x00000000000066db), UINT64_C(0x1f42fe0f9cb806e1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43100e48ab4744)
    },
    {   UINT64_C(0x000000000000592f), UINT64_C(0x1f42f5cf54aca70c), UINT64_C(0x1f42f60f2cb5a6dc),
        UINT64_C(0x1f42fa8f50b5a6ec), UINT64_C(0x1f42f74f20b6a710)
    },
    {   UINT64_C(0x00000000000066dc), UINT64_C(0x1f43030facb566ed), UINT64_C(0x1f4301cfccb3e6f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d2f), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8f38afa72e),
        UINT64_C(0x1f42f84e5cb8072a), UINT64_C(0x1f42fd4ec4b86729)
    },
    {   UINT64_C(0x00000000000066dd), UINT64_C(0x1f43035004b3072f), UINT64_C(0x1f43020f04b88707),
        UINT64_C(0x1f43010fa0b166fb), UINT64_C(0x1f42ffd008b70708)
    },
    {   UINT64_C(0x000000000000807c), UINT64_C(0x08b2dc8c3863468a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005934), UINT64_C(0x1f42f4ceb0b7e717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98eb0b70721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005935), UINT64_C(0x1f42f98f1cb0673e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7ce28abc732), UINT64_C(0x1f42f4ce8cb506fc)
    },
    {   UINT64_C(0x0000000000005936), UINT64_C(0x1f42ff0e64ab8740), UINT64_C(0x1f42f58f04b60720),
        UINT64_C(0x1f42f64e88b7c734), UINT64_C(0x1f42f90e9cb4a724)
    },
    {   UINT64_C(0x0000000000005937), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78f88b666ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005938), UINT64_C(0x1f42f4ce6cb80729), UINT64_C(0x1f42fecf20b3473c),
        UINT64_C(0x1f42f94e98b3470c), UINT64_C(0x1f42f68e44b18735)
    },
    {   UINT64_C(0x0000000000005939), UINT64_C(0x1f42f98ebcb1273c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78ed4b46735), UINT64_C(0x1f42f80e6cb66744)
    },
    {   UINT64_C(0x00000000000066e1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facffcb54719),
        UINT64_C(0x1f43118ebca84706), UINT64_C(0x1f42facf38b82719)
    },
    {   UINT64_C(0x000000000000593b), UINT64_C(0x1f42f34fa8b9470e), UINT64_C(0x1f42f8ce5cb54738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ed4b04731)
    },
    {   UINT64_C(0x000000000000593c), UINT64_C(0x1f42f5ce84b1c725), UINT64_C(0x1f42f58f0cafe703),
        UINT64_C(0x1f42f6ce6cb3e741), UINT64_C(0x1f42f6ce7cb28708)
    },
    {   UINT64_C(0x000000000000593d), UINT64_C(0x1f42f7cf40b34738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e84b4c708), UINT64_C(0x1f42f9cf8cb2670a)
    },
    {   UINT64_C(0x000000000000593e), UINT64_C(0x1f42fc0f00b6a723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f48f40b5a71b), UINT64_C(0x1f42f64f98b72703)
    },
    {   UINT64_C(0x0000000000008088), UINT64_C(0x0000000000000000), UINT64_C(0x14faf4cd48900693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005940), UINT64_C(0x1f43010e6cb58730), UINT64_C(0x1f42f70e64b86738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98ef0b60720)
    },
    {   UINT64_C(0x0000000000005941), UINT64_C(0x1f42f54f50b14715), UINT64_C(0x1f4306cf10af2717),
        UINT64_C(0x1f42f80e60b04717), UINT64_C(0x1f42fa8f20b4a70b)
    },
    {   UINT64_C(0x0000000000005942), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54e78afa6e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005943), UINT64_C(0x1f42f78ec0ad66df), UINT64_C(0x1f42f6cf10b80716),
        UINT64_C(0x1f42f7cee4b50724), UINT64_C(0x1f42f94f48b4e70c)
    },
    {   UINT64_C(0x000000000000808d), UINT64_C(0x0d6af2cd4871c693), UINT64_C(0x1502f20d489025b4),
        UINT64_C(0x0c8af14d486e462c), UINT64_C(0x08b2dc8c3856a5b4)
    },
    {   UINT64_C(0x0000000000005945), UINT64_C(0x1f42f58ea4ace6f4), UINT64_C(0x1f42f8cec4b4e712),
        UINT64_C(0x1f42f9cee4b5a6f5), UINT64_C(0x1f42fbcee8b0e721)
    },
    {   UINT64_C(0x00000000000066e7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecffcb22719), UINT64_C(0x1f42f9cfdcb58720)
    },
    {   UINT64_C(0x0000000000005947), UINT64_C(0x1f42fb0ea8aa6720), UINT64_C(0x1f42f84e8cb46723),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005948), UINT64_C(0x1f42f8ce88b5472e), UINT64_C(0x1f42f84f58b66703),
        UINT64_C(0x1f42f58eb4b1c719), UINT64_C(0x1f43058e50ada735)
    },
    {   UINT64_C(0x0000000000005949), UINT64_C(0x1f42f74e8cb6c712), UINT64_C(0x1f42f90e98b4c726),
        UINT64_C(0x1f42f78ec0b3070b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066e9), UINT64_C(0x1f43190eb8a76703), UINT64_C(0x1f42fecf0cb7e6e5),
        UINT64_C(0x1f42fecf24b1a6f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008094), UINT64_C(0x0de2f00f787225b4), UINT64_C(0x0fa2f4cf787be5b4),
        UINT64_C(0x0fa2f4cf787be5b4), UINT64_C(0x0fa2ff4f78790693)
    },
    {   UINT64_C(0x0000000000008095), UINT64_C(0x0de2f08f747025b3), UINT64_C(0x0de2f44d2072a5b4),
        UINT64_C(0x114af4cbfc896693), UINT64_C(0x125af4cf7889c693)
    },
    {   UINT64_C(0x0000000000008096), UINT64_C(0x0de2f30f7872e5b4), UINT64_C(0x0de2f18f787225b4),
        UINT64_C(0x148af70f6c8fe5b4), UINT64_C(0x0db2f30f686f25c3)
    },
    {   UINT64_C(0x0000000000008097), UINT64_C(0x0db2f30f686f25c3), UINT64_C(0x137af30f6889e5c3),
        UINT64_C(0x0efaf30f687445c3), UINT64_C(0x09dae1cc386385b4)
    },
    {   UINT64_C(0x000000000000594f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbd000b5a6dd),
        UINT64_C(0x1f42fc4fc8b7e718), UINT64_C(0x1f42fbcfe0b8a71e)
    },
    {   UINT64_C(0x0000000000008099), UINT64_C(0x0fa2f14f787b45aa), UINT64_C(0x0fa2f14bfc7be5aa),
        UINT64_C(0x0fa2f04fa07b6693), UINT64_C(0x0f0af04bfc73e693)
    },
    {   UINT64_C(0x000000000000809a), UINT64_C(0x08b2f20fa05e2693), UINT64_C(0x0a6af24f786565ae),
        UINT64_C(0x0812f34fa05b65aa), UINT64_C(0x0af2f08fa0690693)
    },
    {   UINT64_C(0x0000000000005952), UINT64_C(0x1f43058fecb786fd), UINT64_C(0x1f42fa0edcb7870c),
        UINT64_C(0x1f42f74f3cb6e71c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000809c), UINT64_C(0x191af0cbfca0a5aa), UINT64_C(0x1852f20bfc96c5b4),
        UINT64_C(0x108adc8bfc7ec5b4), UINT64_C(0x10aaf20bfc89e5b4)
    },
    {   UINT64_C(0x000000000000809d), UINT64_C(0x12d2f54f8084a5aa), UINT64_C(0x0fa2f54f787945b4),
        UINT64_C(0x1682f3cf8895a5aa), UINT64_C(0x14daf74f908d25c3)
    },
    {   UINT64_C(0x000000000000809e), UINT64_C(0x14daf80bfc8cc693), UINT64_C(0x0b92f58f786b85aa),
        UINT64_C(0x0b92f58f786c0693), UINT64_C(0x0d12f58bfc7905aa)
    },
    {   UINT64_C(0x000000000000809f), UINT64_C(0x0a6aef4bfc6865b4), UINT64_C(0x0b92efcbfc68e5b4),
        UINT64_C(0x09daf08fa068a5be), UINT64_C(0x0af2ef4fa06805be)
    },
    {   UINT64_C(0x00000000000080a0), UINT64_C(0x0edaf34fa07945b8), UINT64_C(0x0edaf34fa07945b8),
        UINT64_C(0x0c2afa8c386a05b2), UINT64_C(0x0a12dc8bfc6d0693)
    },
    {   UINT64_C(0x00000000000080a1), UINT64_C(0x0982dc8c386d0693), UINT64_C(0x0c82f1cbfc77a5be),
        UINT64_C(0x0daae84bd4734693), UINT64_C(0x08b2f20d9c61c5b4)
    },
    {   UINT64_C(0x0000000000005959), UINT64_C(0x1f42f9cf5cb746f6), UINT64_C(0x1f43030fa0b4a6f6),
        UINT64_C(0x1f43024f1cb8872a), UINT64_C(0x1f42facf54b4a703)
    },
    {   UINT64_C(0x00000000000080a3), UINT64_C(0x0fa2ed0fa07745b8), UINT64_C(0x1692f34fa09545b8),
        UINT64_C(0x0de2ed8fa077e684), UINT64_C(0x125af3cfa08505b8)
    },
    {   UINT64_C(0x00000000000080a4), UINT64_C(0x1082f4cc3885a5aa), UINT64_C(0x0fdaf4cd487b65aa),
        UINT64_C(0x0ceaf10bfc6f85aa), UINT64_C(0x0d7af10adc71c5aa)
    },
    {   UINT64_C(0x00000000000066f2), UINT64_C(0x1f42f90ed8b66729), UINT64_C(0x1f43070ffcb5270e),
        UINT64_C(0x1f4317ce7ca90727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080a6), UINT64_C(0x169af64f5c94c68f), UINT64_C(0x0e62f1cfa07605b8),
        UINT64_C(0x0efaf30f687445c3), UINT64_C(0x12c2f54f748765f3)
    },
    {   UINT64_C(0x000000000000595e), UINT64_C(0x1f42ff8efcb6a72c), UINT64_C(0x1f42ff0f38b72722),
        UINT64_C(0x1f43068fc8b526f2), UINT64_C(0x1f43110e78acc745)
    },
    {   UINT64_C(0x00000000000080a8), UINT64_C(0x12c2f40c3c88462e), UINT64_C(0x0e12fa0fa070a693),
        UINT64_C(0x0e12fa0fa070a693), UINT64_C(0x1912f4cf789f6693)
    },
    {   UINT64_C(0x0000000000005960), UINT64_C(0x1f4304ced4b6a6da), UINT64_C(0x1f42fc4ef8b4a6fc),
        UINT64_C(0x1f4302cfb0b8071b), UINT64_C(0x1f43170ec0b42713)
    },
    {   UINT64_C(0x00000000000080aa), UINT64_C(0x1452f14f788ea62b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005962), UINT64_C(0x1f430a0e8cad872d), UINT64_C(0x1f430acef8ad272d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4315cf14b2a735)
    },
    {   UINT64_C(0x00000000000080ac), UINT64_C(0x0bd2efd4e06c2738), UINT64_C(0x0bd2efd4e06c2738),
        UINT64_C(0x0962f912445d073c), UINT64_C(0x07e2ef54e05e0691)
    },
    {   UINT64_C(0x0000000000005964), UINT64_C(0x1f43160e7cb10735), UINT64_C(0x1f430250c0b646eb),
        UINT64_C(0x1f4301cf24b3e729), UINT64_C(0x1f43024f78ab06e2)
    },
    {   UINT64_C(0x00000000000080ae), UINT64_C(0x0b92f192606ba690), UINT64_C(0x06830496345ea694),
        UINT64_C(0x098af2d634670694), UINT64_C(0x0a6b01d68c59c696)
    },
    {   UINT64_C(0x00000000000080af), UINT64_C(0x0a6b01d68c59c696), UINT64_C(0x0a6b01d68c59c696),
        UINT64_C(0x0632f6d73452e690), UINT64_C(0x0632f6d73452e690)
    },
    {   UINT64_C(0x00000000000080b0), UINT64_C(0x0a32f3d43c64669a), UINT64_C(0x0a32f3d43c64669a),
        UINT64_C(0x09eaf6562c6226db), UINT64_C(0x09eaf6562c6226db)
    },
    {   UINT64_C(0x00000000000080b1), UINT64_C(0x09eaebd6d46766b1), UINT64_C(0x09eaebd6d46766b1),
        UINT64_C(0x0a6af0d78c66c68a), UINT64_C(0x0a6af0d78c66c68a)
    },
    {   UINT64_C(0x00000000000080b2), UINT64_C(0x08b30bd6a05266b1), UINT64_C(0x0a72f0d82c66c641),
        UINT64_C(0x0a6b05579466868f), UINT64_C(0x0a6af397945da68f)
    },
    {   UINT64_C(0x00000000000080b3), UINT64_C(0x08b30bcc9452642e), UINT64_C(0x0a72f0cd2466e380),
        UINT64_C(0x0a6af44ca05da3d3), UINT64_C(0x0a6b030ca06503d3)
    },
    {   UINT64_C(0x00000000000080b4), UINT64_C(0x0a6b044fec5d05b4), UINT64_C(0x0a6b04544c5d05b4),
        UINT64_C(0x0a6b0010cc5f4555), UINT64_C(0x0a6affd0cc5f2555)
    },
    {   UINT64_C(0x00000000000080b5), UINT64_C(0x0a6afacfe861a4b5), UINT64_C(0x0a6afacfe861a4b5),
        UINT64_C(0x0a6affd0085f64c2), UINT64_C(0x0a6affd1305f6478)
    },
    {   UINT64_C(0x00000000000080b6), UINT64_C(0x0a7af4d78c650680), UINT64_C(0x0a6b0b97cc59a656),
        UINT64_C(0x0a6afed7a05fc6af), UINT64_C(0x0a6afe0d0c5da3e8)
    },
    {   UINT64_C(0x00000000000080b7), UINT64_C(0x0a6aee57f067e666), UINT64_C(0x0a6aed578c6d868e),
        UINT64_C(0x0952ee12885f65a3), UINT64_C(0x0a6afa164462067e)
    },
    {   UINT64_C(0x00000000000080b8), UINT64_C(0x0bd2f4130c686694), UINT64_C(0x06baf2d4e0572738),
        UINT64_C(0x091af714cc5f2691), UINT64_C(0x0be2ef54e06d0690)
    },
    {   UINT64_C(0x00000000000080b9), UINT64_C(0x0aaaf954e066673a), UINT64_C(0x0ceafa0fa06d267e),
        UINT64_C(0x0caaf80fa06c267e), UINT64_C(0x0caaf80fa06c267e)
    },
    {   UINT64_C(0x00000000000080ba), UINT64_C(0x0caaf80fa06c267e), UINT64_C(0x0caaf80fa06c267e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080bb), UINT64_C(0x0a6af34d5465642e), UINT64_C(0x0caaf0978c6fe681),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005973), UINT64_C(0x1f42f60fa4b64717), UINT64_C(0x1f43008ea4b4e707),
        UINT64_C(0x1f42f70ee4b3071d), UINT64_C(0x1f42facf80b6470b)
    },
    {   UINT64_C(0x00000000000080bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6aee4c3467e377)
    },
    {   UINT64_C(0x0000000000005975), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80e68b3670d), UINT64_C(0x1f42f98f10b2c71d)
    },
    {   UINT64_C(0x00000000000066ff), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014eacb4071c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080c0), UINT64_C(0x000291578c2c268e), UINT64_C(0x000291578c2c268e),
        UINT64_C(0x00028d978c35268a), UINT64_C(0x00028b57f0366666)
    },
    {   UINT64_C(0x0000000000005978), UINT64_C(0x1f430dcfa0b006c5), UINT64_C(0x1f430e4fb4b466e7),
        UINT64_C(0x1f430f8fd0b7a6d2), UINT64_C(0x1f43010f9cb226f7)
    },
    {   UINT64_C(0x00000000000080c2), UINT64_C(0x00028f57cc346656), UINT64_C(0x0002a157502fc6d7),
        UINT64_C(0x00029bd7a02e46af), UINT64_C(0x000289d78c3bc68e)
    },
    {   UINT64_C(0x00000000000080c3), UINT64_C(0x00028dd78c35468a), UINT64_C(0x0002af97702626a4),
        UINT64_C(0x00029c97702de6a4), UINT64_C(0x000289d78c3bc68e)
    },
    {   UINT64_C(0x00000000000080c4), UINT64_C(0x000290d78c3306e7), UINT64_C(0x000290d78c330680),
        UINT64_C(0x0002a357782a86d1), UINT64_C(0x0002a357782a86d1)
    },
    {   UINT64_C(0x00000000000080c5), UINT64_C(0x0002a357782a86d1), UINT64_C(0x0002d8d7784546d1),
        UINT64_C(0x0002964c202ea3b3), UINT64_C(0x0002914c202c23b3)
    },
    {   UINT64_C(0x00000000000080c6), UINT64_C(0x00028acb342443b3), UINT64_C(0x0002ab4b343483b3),
        UINT64_C(0x00028f17ec3486c7), UINT64_C(0x0002e3150444e5c2)
    },
    {   UINT64_C(0x000000000000597e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430fd078b246aa), UINT64_C(0x1f43180f0cac66ed)
    },
    {   UINT64_C(0x00000000000080c8), UINT64_C(0x00029c8d002dc36c), UINT64_C(0x0002b34b243ea433),
        UINT64_C(0x0002e08b24554433), UINT64_C(0x0002a80c3c27e372)
    },
    {   UINT64_C(0x00000000000080c9), UINT64_C(0x00028f4c38346371), UINT64_C(0x00029bcb702e43a3),
        UINT64_C(0x0002a34a1c2a837a), UINT64_C(0x00029a8c442ee3e8)
    },
    {   UINT64_C(0x00000000000080ca), UINT64_C(0x00029d8d0c2d63e8), UINT64_C(0x0002af8bf8244382),
        UINT64_C(0x00028dcbf4356386), UINT64_C(0x00028f0c1435237a)
    },
    {   UINT64_C(0x00000000000080cb), UINT64_C(0x00028dcbe035439f), UINT64_C(0x00028dcbe035439f),
        UINT64_C(0x0002918c1c3363a4), UINT64_C(0x0002918c403363ad)
    },
    {   UINT64_C(0x00000000000080cc), UINT64_C(0x00028b4c34366377), UINT64_C(0x00028dcc9435435b),
        UINT64_C(0x0002760ca441035b), UINT64_C(0x0002760af041035b)
    },
    {   UINT64_C(0x00000000000080cd), UINT64_C(0x000289d2f835a528), UINT64_C(0x00028e133834c4fa),
        UINT64_C(0x00027613384104fa), UINT64_C(0x00028f0e7839662c)
    },
    {   UINT64_C(0x00000000000080ce), UINT64_C(0x00028f0d3042667e), UINT64_C(0x0002a80b782b83a1),
        UINT64_C(0x00028dcca83563b3), UINT64_C(0x00029fcc142c23b3)
    },
    {   UINT64_C(0x00000000000080cf), UINT64_C(0x00028f0cac34a3a7), UINT64_C(0x00029c97d42da6b3),
        UINT64_C(0x0002a7576c286725), UINT64_C(0x0002761ba8410788)
    },
    {   UINT64_C(0x0000000000005987), UINT64_C(0x1f43140ec0ab26ef), UINT64_C(0x1f430e4e64b226f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43074f24b226fb)
    },
    {   UINT64_C(0x00000000000080d1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0002969808360691), UINT64_C(0x000af0cb1066e35c)
    },
    {   UINT64_C(0x0000000000005989), UINT64_C(0x1f43078ed0b726fd), UINT64_C(0x1f42ff8ed0b5c72e),
        UINT64_C(0x1f430c8ef8b7a6f7), UINT64_C(0x1f4309ce8cb84726)
    },
    {   UINT64_C(0x00000000000080d3), UINT64_C(0x00028d97f03786f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000598b), UINT64_C(0x0000000000000000), UINT64_C(0x1f430ed024ab46ca),
        UINT64_C(0x1f430acf34a8e706), UINT64_C(0x1f4303cf50b56725)
    },
    {   UINT64_C(0x000000000000670a), UINT64_C(0x1f43014f34aa46ff), UINT64_C(0x1f43008fa0af8726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f2cb7a72c)
    },
    {   UINT64_C(0x00000000000080d6), UINT64_C(0x0000000000000000), UINT64_C(0x000294c9e433a3a7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080d7), UINT64_C(0x000290cbfc63639c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080d8), UINT64_C(0x00028b57b46c2661), UINT64_C(0x000290d78c636680),
        UINT64_C(0x00028b4b686ae3a7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005990), UINT64_C(0x0000000000000000), UINT64_C(0x1f43090ef0b2c70e),
        UINT64_C(0x1f430e4ec0b4873d), UINT64_C(0x1f4300cf18b4a700)
    },
    {   UINT64_C(0x0000000000005991), UINT64_C(0x1f43041008b0a6fe), UINT64_C(0x1f430ad030afe6b9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdce68b7c730)
    },
    {   UINT64_C(0x0000000000005992), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038efcb16724),
        UINT64_C(0x1f4305cfb4b2e6db), UINT64_C(0x1f43038efcb16708)
    },
    {   UINT64_C(0x000000000000400e), UINT64_C(0x0fa2f810a0780698), UINT64_C(0x0fa2f8d0a0770698),
        UINT64_C(0x0fa31e11ac684608), UINT64_C(0x0fa3179038686608)
    },
    {   UINT64_C(0x000000000000670e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff5000b8072c),
        UINT64_C(0x1f42f5ce90b7871e), UINT64_C(0x1f42ffcf20b7e70c)
    },
    {   UINT64_C(0x0000000000005995), UINT64_C(0x1f430c8f08ae06ef), UINT64_C(0x1f43154e5caa871b),
        UINT64_C(0x1f430c8f48ae06f1), UINT64_C(0x1f430bcf58b82702)
    },
    {   UINT64_C(0x0000000000005996), UINT64_C(0x0000000000000000), UINT64_C(0x1f43084ef0b40714),
        UINT64_C(0x1f43074f44b7672c), UINT64_C(0x1f430fcf74b0a6df)
    },
    {   UINT64_C(0x0000000000005997), UINT64_C(0x1f43154e6ca9e725), UINT64_C(0x1f430e4f00b0c734),
        UINT64_C(0x1f43010e2cb16731), UINT64_C(0x1f42fe4ea8ae0737)
    },
    {   UINT64_C(0x0000000000006710), UINT64_C(0x1f431c8eccac0701), UINT64_C(0x1f42fecf78b72706),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005999), UINT64_C(0x1f43084fa8b4c6f6), UINT64_C(0x1f430bcfc0b1870a),
        UINT64_C(0x1f430bcf08b8872a), UINT64_C(0x1f42fc8eecb4470d)
    },
    {   UINT64_C(0x00000000000080e3), UINT64_C(0x1692f00f68948693), UINT64_C(0x0000000000000000),
        UINT64_C(0x1a42f00fa0a4c693), UINT64_C(0x1742f00fa097e693)
    },
    {   UINT64_C(0x000000000000599b), UINT64_C(0x1f430a4e7cade711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fcce34b4e745), UINT64_C(0x1f43078e88b0e71e)
    },
    {   UINT64_C(0x000000000000599c), UINT64_C(0x1f43030e6cb48733), UINT64_C(0x1f43084f10b16723),
        UINT64_C(0x1f43064fb8b7c718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000599d), UINT64_C(0x1f43074e6cb46738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f0f54aa6718), UINT64_C(0x1f42fe0e80b7e71f)
    },
    {   UINT64_C(0x000000000000599e), UINT64_C(0x1f43070eb0b666f9), UINT64_C(0x1f430a4ef8ace6eb),
        UINT64_C(0x1f42fd8ea0aca734), UINT64_C(0x1f43074f2cb3c720)
    },
    {   UINT64_C(0x00000000000080e8), UINT64_C(0x1692f28fa096667e), UINT64_C(0x116af20fa082467e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1382f58fa089867e)
    },
    {   UINT64_C(0x00000000000059a0), UINT64_C(0x1f43074fe0b066e1), UINT64_C(0x1f42fccf2cb42708),
        UINT64_C(0x1f430b8ed0aca72c), UINT64_C(0x1f4313ce8ca8a70e)
    },
    {   UINT64_C(0x00000000000080ea), UINT64_C(0x1752f38fa099069a), UINT64_C(0x1742f54fa097e68c),
        UINT64_C(0x0a6af28fa0660751), UINT64_C(0x1692f54fa0966751)
    },
    {   UINT64_C(0x00000000000080eb), UINT64_C(0x10faf54f7880a67e), UINT64_C(0x0dbaf44f7871667e),
        UINT64_C(0x1002f08c3c77067e), UINT64_C(0x089af30f785fe67e)
    },
    {   UINT64_C(0x00000000000080ec), UINT64_C(0x1062f20f787c467e), UINT64_C(0x10faf54f7880a5b4),
        UINT64_C(0x0f82fbcc3c78c693), UINT64_C(0x0ed2f08c387605b1)
    },
    {   UINT64_C(0x00000000000059a4), UINT64_C(0x1f43090ed8ad8725), UINT64_C(0x1f43024e80b40726),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059a5), UINT64_C(0x1f43014f00b306fa), UINT64_C(0x1f43020e5cb8271e),
        UINT64_C(0x1f430a4f28b66726), UINT64_C(0x1f4308cee0b54725)
    },
    {   UINT64_C(0x00000000000059a6), UINT64_C(0x1f43064e78b1a6e5), UINT64_C(0x1f43030f40b72719),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080f0), UINT64_C(0x0faaf04f687ac5aa), UINT64_C(0x0f82fbcc3c78c5b4),
        UINT64_C(0x0dcaf6cc387485b4), UINT64_C(0x1122f54f787f45aa)
    },
    {   UINT64_C(0x00000000000059a8), UINT64_C(0x1f431f0e8ca88725), UINT64_C(0x1f4303cec0b46738),
        UINT64_C(0x1f43020f60b4671e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059a9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0e50b30715), UINT64_C(0x1f43064ef8b44717)
    },
    {   UINT64_C(0x00000000000059aa), UINT64_C(0x1f43194f0cb5070b), UINT64_C(0x1f4302ced0b846ea),
        UINT64_C(0x1f43004f18af871f), UINT64_C(0x1f42fcce84b1e745)
    },
    {   UINT64_C(0x00000000000080f4), UINT64_C(0x0e12fb8f7871e69d), UINT64_C(0x1152ee0f78828693),
        UINT64_C(0x1692f54fa095c68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059ac), UINT64_C(0x1f43014f84b806dd), UINT64_C(0x1f43060f08b146f5),
        UINT64_C(0x1f4304ce20ae8735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080f6), UINT64_C(0x1692f68c3c94868c), UINT64_C(0x0fa2f54c3c7945b4),
        UINT64_C(0x14daf50c388c468c), UINT64_C(0x0e6af6cc387505d2)
    },
    {   UINT64_C(0x0000000000000b1c), UINT64_C(0x106af54c107c6658), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080f8), UINT64_C(0x190af0cc389e668c), UINT64_C(0x1082ec8c407b6625),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059b0), UINT64_C(0x1f42ff0edcb2271e), UINT64_C(0x1f4306cf0caf4730),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059b1), UINT64_C(0x1f43010e2cb06729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78e50b24715), UINT64_C(0x1f4311ce84ac2717)
    },
    {   UINT64_C(0x000000000001511a), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x15eafbcf709ba695), UINT64_C(0x1992f88fa0a14685)
    },
    {   UINT64_C(0x000000000000401e), UINT64_C(0x0fa2f510c87985cc), UINT64_C(0x0fa2f90ebc7785cc),
        UINT64_C(0x0fa300d0c873a5cc), UINT64_C(0x0fa305ce94710721)
    },
    {   UINT64_C(0x000000000001511b), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x13daf90fa09c4685),
        UINT64_C(0x1802fbcf709e8695), UINT64_C(0x1bdaf88fa0b4c685)
    },
    {   UINT64_C(0x00000000000059b5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4ee4b2e714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000671f), UINT64_C(0x1f43028fe4af8732), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43059020b62700)
    },
    {   UINT64_C(0x00000000000059b7), UINT64_C(0x1f43084ec8b0c720), UINT64_C(0x1f4301ce64b7c731),
        UINT64_C(0x1f43008ec8ae26e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008101), UINT64_C(0x145af78f688c268c), UINT64_C(0x1162fd8f687e468c),
        UINT64_C(0x0a6af28fa065e67e), UINT64_C(0x0a6af44fa066e750)
    },
    {   UINT64_C(0x0000000000008102), UINT64_C(0x0baaec8f686ac67e), UINT64_C(0x1ee2f08f68b5c67e),
        UINT64_C(0x1fcaf2cfa0b9667e), UINT64_C(0x191af28fa0a0c67e)
    },
    {   UINT64_C(0x0000000000019402), UINT64_C(0x1b4af8d010a6261d), UINT64_C(0x0aa2f9501063861d),
        UINT64_C(0x12eaf9101084a61d), UINT64_C(0x1b4af8d010a6261d)
    },
    {   UINT64_C(0x0000000000019403), UINT64_C(0x239af8d010c7861d), UINT64_C(0x121af9cf7882e693),
        UINT64_C(0x1672fa8fa092067e), UINT64_C(0x0cfafbd0847265a9)
    },
    {   UINT64_C(0x0000000000006722), UINT64_C(0x1f43030f34ab6717), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f94b3271b)
    },
    {   UINT64_C(0x0000000000008106), UINT64_C(0x169af2cfa096867e), UINT64_C(0x169af2cfa096877b),
        UINT64_C(0x1592f2cfa094468c), UINT64_C(0x155af04f6891867e)
    },
    {   UINT64_C(0x0000000000008107), UINT64_C(0x1beaf2cfa0ab667e), UINT64_C(0x16b2f2cfa096867e),
        UINT64_C(0x16caf70f6894c68c), UINT64_C(0x16a2f2cfa096a67e)
    },
    {   UINT64_C(0x0000000000008108), UINT64_C(0x11daf2cfa081667e), UINT64_C(0x152af70f688ea68c),
        UINT64_C(0x1352f58fa088867e), UINT64_C(0x163af1cf6495467e)
    },
    {   UINT64_C(0x0000000000006724), UINT64_C(0x1f43048fd8ad071b), UINT64_C(0x1f43075018b2671b),
        UINT64_C(0x1f43074fd8ae871b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000810a), UINT64_C(0x1f12f54fa0b7267e), UINT64_C(0x1f12f54d3cb7267e),
        UINT64_C(0x170af14fa096667e), UINT64_C(0x1ba2f2cfa0aaa67e)
    },
    {   UINT64_C(0x000000000001940a), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f08f248ac642),
        UINT64_C(0x156afa11f08e2552), UINT64_C(0x21f2fbd1a8bf6540)
    },
    {   UINT64_C(0x000000000001940b), UINT64_C(0x1042ee104c87c5b5), UINT64_C(0x156afa10608e25b6),
        UINT64_C(0x156afa0f148e25f9), UINT64_C(0x156afa0dcc8e2646)
    },
    {   UINT64_C(0x000000000001940c), UINT64_C(0x156afa10048e25c3), UINT64_C(0x156afa0edc8e260d),
        UINT64_C(0x156afa0e588e2632), UINT64_C(0x156afa0d308e267e)
    },
    {   UINT64_C(0x000000000001940d), UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afc0b548c26b6),
        UINT64_C(0x156afc0b548c26b6), UINT64_C(0x156afa0e988e2624)
    },
    {   UINT64_C(0x000000000001940e), UINT64_C(0x156af9cd308b067e), UINT64_C(0x156b004d308e467e),
        UINT64_C(0x156afc0cac8c269f), UINT64_C(0x156afe0cac8d269f)
    },
    {   UINT64_C(0x000000000001940f), UINT64_C(0x156af08e048d8689), UINT64_C(0x156afc8d608c8689),
        UINT64_C(0x156afa0c148e26c2), UINT64_C(0x156afa0c148fc6c2)
    },
    {   UINT64_C(0x00000000000059c8), UINT64_C(0x1f42fa0f28b82715), UINT64_C(0x1f42f94f00b3e702),
        UINT64_C(0x1f43144f0caee704), UINT64_C(0x1f42f80efcb6c715)
    },
    {   UINT64_C(0x00000000000059c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ed0b6c748), UINT64_C(0x1f42f90f58b86715)
    },
    {   UINT64_C(0x0000000000019412), UINT64_C(0x156afacc0c8b8647), UINT64_C(0x156aff4c0c8dc647),
        UINT64_C(0x1362f08f248ac600), UINT64_C(0x1362f08f248ac600)
    },
    {   UINT64_C(0x000000000000402a), UINT64_C(0x0fa309572c6f6630), UINT64_C(0x0fa2f093587bc6b7),
        UINT64_C(0x0fa30653a470e671), UINT64_C(0x0fa2f190787b45de)
    },
    {   UINT64_C(0x0000000000019414), UINT64_C(0x156afd0b888ca6e8), UINT64_C(0x156afd0c508ca70b),
        UINT64_C(0x156afd0c508ca70b), UINT64_C(0x156afb8cac8ae69f)
    },
    {   UINT64_C(0x0000000000019415), UINT64_C(0x156b008cac8d669f), UINT64_C(0x156afa11808e256a),
        UINT64_C(0x156afa11808e256a), UINT64_C(0x0fa2fa0dc47723fe)
    },
    {   UINT64_C(0x0000000000019416), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x0e7afacd9c71e661),
        UINT64_C(0x0e7afacd9c71e661), UINT64_C(0x156afa4d088ce66f)
    },
    {   UINT64_C(0x00000000000059cf), UINT64_C(0x1f42ffcedcb44708), UINT64_C(0x1f42f8cf2caca723),
        UINT64_C(0x1f42fd8eb4b32710), UINT64_C(0x1f43074ef8b5a72e)
    },
    {   UINT64_C(0x0000000000019418), UINT64_C(0x156b008a9c8e66b6), UINT64_C(0x0ca2f3cf546e25ab),
        UINT64_C(0x2503bd4f3cae664a), UINT64_C(0x2503bd5044d5e683)
    },
    {   UINT64_C(0x00000000000059d1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ee4b1a6fb),
        UINT64_C(0x1f42fe8e7cb32716), UINT64_C(0x1f42fd8ee8b0c712)
    },
    {   UINT64_C(0x00000000000059d2), UINT64_C(0x1f42f7cf08b50714), UINT64_C(0x1f42fcce84aac72b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcef8b9e710)
    },
    {   UINT64_C(0x00000000000059d3), UINT64_C(0x1f42f50f38b7871f), UINT64_C(0x1f42fdcf0cb4e700),
        UINT64_C(0x1f43048e54b3a728), UINT64_C(0x1f43054efcb6471f)
    },
    {   UINT64_C(0x000000000001941c), UINT64_C(0x156afa0de88e4647), UINT64_C(0x156afa0cac8e269f),
        UINT64_C(0x156afa0bf08e26bc), UINT64_C(0x156af9cd308b067e)
    },
    {   UINT64_C(0x00000000000059d5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf08b4c714),
        UINT64_C(0x1f42f60f38b4470c), UINT64_C(0x1f430c8e58b18727)
    },
    {   UINT64_C(0x000000000001941e), UINT64_C(0x26c2fc0d30d4267e), UINT64_C(0x156af3cd3091467e),
        UINT64_C(0x0f32fe0e18734642), UINT64_C(0x156afa0d308e267e)
    },
    {   UINT64_C(0x00000000000059d7), UINT64_C(0x1f42f98f18b4c724), UINT64_C(0x1f42f70f18b6e71b),
        UINT64_C(0x1f42facf38b3c72f), UINT64_C(0x1f43078e8cb6a72b)
    },
    {   UINT64_C(0x0000000000019420), UINT64_C(0x156afd0d308ca67e), UINT64_C(0x0f32fe0e18734642),
        UINT64_C(0x156afa11108e2587), UINT64_C(0x0f02fe1664704777)
    },
    {   UINT64_C(0x0000000000019421), UINT64_C(0x0dcafc56446ee6db), UINT64_C(0x156afd0ac48ca71a),
        UINT64_C(0x156afd0ac48ca71a), UINT64_C(0x156afd0ac48ca71a)
    },
    {   UINT64_C(0x0000000000019422), UINT64_C(0x156afd0ac48ca71a), UINT64_C(0x156afd0bd88ca6d1),
        UINT64_C(0x156afd0a4c8ca735), UINT64_C(0x156afa0f548e25fe)
    },
    {   UINT64_C(0x00000000000059db), UINT64_C(0x1f42f98f38b666f8), UINT64_C(0x1f42f88f38b62733),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf58b20723)
    },
    {   UINT64_C(0x00000000000059dc), UINT64_C(0x1f430a4f48b12722), UINT64_C(0x1f4308ce5cb5473e),
        UINT64_C(0x1f430f4ef4b426f5), UINT64_C(0x1f430b4ea4b4c736)
    },
    {   UINT64_C(0x0000000000019425), UINT64_C(0x13c2f08f748c85b3), UINT64_C(0x1382f54f7088e5b3),
        UINT64_C(0x156afa0c208e26c2), UINT64_C(0x156afa0c208e26c2)
    },
    {   UINT64_C(0x0000000000019426), UINT64_C(0x156afa10188e2619), UINT64_C(0x156afa0f548e25fe),
        UINT64_C(0x0e22f20f787505b3), UINT64_C(0x000305d82c7146a2)
    },
    {   UINT64_C(0x0000000000019427), UINT64_C(0x000af0cb1066e35c), UINT64_C(0x0002f0c98866c35b),
        UINT64_C(0x0002f0c80866c35b), UINT64_C(0x00030597907b26c6)
    },
    {   UINT64_C(0x0000000000019428), UINT64_C(0x000305d82c73a6f1), UINT64_C(0x0002ee49d467e35b),
        UINT64_C(0x000307582c7206ea), UINT64_C(0x0002ee49d467e35b)
    },
    {   UINT64_C(0x0000000000004035), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431a907ca5a6b2)
    },
    {   UINT64_C(0x00000000000059e2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f68b3c71f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf78b2271a)
    },
    {   UINT64_C(0x000000000001942b), UINT64_C(0x156afa0dc48e265a), UINT64_C(0x156afa0bf08e26bd),
        UINT64_C(0x156afa0bb08e26de), UINT64_C(0x156afa0bb08e26de)
    },
    {   UINT64_C(0x000000000001942c), UINT64_C(0x156afc0c148c26c4), UINT64_C(0x156afc0c148c26c4),
        UINT64_C(0x156afc4ac48c471a), UINT64_C(0x156afc4ac48c471a)
    },
    {   UINT64_C(0x000000000001942d), UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afc0c0c8c26c2),
        UINT64_C(0x156afc0c7c8c26ad), UINT64_C(0x156afc0c7c8c26ad)
    },
    {   UINT64_C(0x000000000001942e), UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c271a),
        UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c071a)
    },
    {   UINT64_C(0x000000000001942f), UINT64_C(0x156afd8b008ce6b7), UINT64_C(0x156afd4b008cc6b7),
        UINT64_C(0x156afd8be08ce6b7), UINT64_C(0x156afd4be08cc6b7)
    },
    {   UINT64_C(0x0000000000019430), UINT64_C(0x156afbcac48c071a), UINT64_C(0x156afe4ac48d471a),
        UINT64_C(0x156af3ccac91469a), UINT64_C(0x1322f44cc487e61f)
    },
    {   UINT64_C(0x0000000000019431), UINT64_C(0x1622f28c4c94c6b7), UINT64_C(0x1622f28c4c94c6b7),
        UINT64_C(0x1c32ee0d30aa667e), UINT64_C(0x1c32ee0d30aa667e)
    },
    {   UINT64_C(0x0000000000015136), UINT64_C(0x1f2af9cf4cb8a6a3), UINT64_C(0x1c52f2cc20ad46a3),
        UINT64_C(0x2412f94f4ccbc6a3), UINT64_C(0x2062f80f4cbd66a3)
    },
    {   UINT64_C(0x0000000000019433), UINT64_C(0x0de2f78d607424c0), UINT64_C(0x0bc2f5978869c6a6),
        UINT64_C(0x0f830a8d306ce67e), UINT64_C(0x143afa523c8be5d9)
    },
    {   UINT64_C(0x00000000000059ec), UINT64_C(0x1f42fa8e4cb40710), UINT64_C(0x1f43000f28b28710),
        UINT64_C(0x1f430e4e5cb56737), UINT64_C(0x1f42ff4ef8b1a710)
    },
    {   UINT64_C(0x0000000000019435), UINT64_C(0x0d3afbcfa069a67e), UINT64_C(0x0d3afbcfa069a67e),
        UINT64_C(0x00030717ec7086c7), UINT64_C(0x0002ee0ba86403b6)
    },
    {   UINT64_C(0x0000000000019436), UINT64_C(0x0002ee0ba86403b6), UINT64_C(0x10eaf60c3877a693),
        UINT64_C(0x09cafbd3105ec614), UINT64_C(0x09cafbd3105ec614)
    },
    {   UINT64_C(0x0000000000019437), UINT64_C(0x0002f590844e05a9), UINT64_C(0x08caff927c58e52c),
        UINT64_C(0x2a02fecf74e14797), UINT64_C(0x2a02fecf74e14797)
    },
    {   UINT64_C(0x0000000000019438), UINT64_C(0x156afa0e0c8e2656), UINT64_C(0x156af9cefc8de60b),
        UINT64_C(0x1632f1cc3891067e), UINT64_C(0x2272f1cc38c4a6dd)
    },
    {   UINT64_C(0x00000000000059f1), UINT64_C(0x1f42f7cf38ae2723), UINT64_C(0x1f42f78f08b5e70c),
        UINT64_C(0x1f42fd4ebcb186fc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001513a), UINT64_C(0x1eeafd0c20b6e6a3), UINT64_C(0x1f22fa8f4cb886a3),
        UINT64_C(0x1d72f6cf68aaa5b6), UINT64_C(0x1432fd0f688c86a3)
    },
    {   UINT64_C(0x00000000000059f3), UINT64_C(0x1f42fc4e9cab670c), UINT64_C(0x1f42f78edcb9c708),
        UINT64_C(0x1f42f78eb4b5a700), UINT64_C(0x1f42f88f58a98717)
    },
    {   UINT64_C(0x000000000001943c), UINT64_C(0x1772fa918096056a), UINT64_C(0x1772fa90109605c6),
        UINT64_C(0x1772fa8f08960608), UINT64_C(0x13e2fa8bf087c72c)
    },
    {   UINT64_C(0x00000000000059f5), UINT64_C(0x1f43104e68b0e724), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001943e), UINT64_C(0x1cf2f40da4af4661), UINT64_C(0x1cf2f40da4af4661),
        UINT64_C(0x1f42fa9310b54506), UINT64_C(0x1772fa918096056a)
    },
    {   UINT64_C(0x00000000000059f7), UINT64_C(0x1f42fa8eecb3a708), UINT64_C(0x1f42fa8e9cb0a71b),
        UINT64_C(0x1f42f5cec8ac0726), UINT64_C(0x1f42f78e84b8e708)
    },
    {   UINT64_C(0x00000000000059f8), UINT64_C(0x1f42f88ee8b0e716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f3cafe6e9), UINT64_C(0x1f42fa4eecb54708)
    },
    {   UINT64_C(0x00000000000059f9), UINT64_C(0x1f42fa4eccb7a704), UINT64_C(0x1f430a8f20b4c72f),
        UINT64_C(0x1f42fb4eecb7a70c), UINT64_C(0x1f42f68eecb9070c)
    },
    {   UINT64_C(0x00000000000059fa), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f58b0a710),
        UINT64_C(0x1f42f78eccb6a700), UINT64_C(0x1f42f78e4cb5671d)
    },
    {   UINT64_C(0x00000000000059fb), UINT64_C(0x1f430c4e6cb54733), UINT64_C(0x1f42f98eecb4e704),
        UINT64_C(0x1f42f78f28ace71b), UINT64_C(0x1f42f98ea4b5072b)
    },
    {   UINT64_C(0x00000000000059fc), UINT64_C(0x1f430e4e38ac4714), UINT64_C(0x1f42fd8f38b4e724),
        UINT64_C(0x1f42fa8f08b44704), UINT64_C(0x1f42f7ce7cafa736)
    },
    {   UINT64_C(0x00000000000059fd), UINT64_C(0x1f4309ce7cb78743), UINT64_C(0x1f42fa8ef4b6c716),
        UINT64_C(0x1f42fb4eecb54708), UINT64_C(0x1f42f68ec0b9470c)
    },
    {   UINT64_C(0x00000000000059fe), UINT64_C(0x1f42fb4f08b20710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4e98afa70c), UINT64_C(0x1f43024e78b1672f)
    },
    {   UINT64_C(0x00000000000059ff), UINT64_C(0x1f42f6ceecb7270c), UINT64_C(0x1f42fa8f68b5671b),
        UINT64_C(0x1f42fa4f08b56710), UINT64_C(0x1f42f6ce7cad271e)
    },
    {   UINT64_C(0x0000000000005a00), UINT64_C(0x1f42fd8f68b24714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64f2cb76721), UINT64_C(0x1f43020f38b18714)
    },
    {   UINT64_C(0x0000000000005a01), UINT64_C(0x1f4307ce98aea6fc), UINT64_C(0x1f42fa8f6cb5a713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f78ad470c)
    },
    {   UINT64_C(0x0000000000005a02), UINT64_C(0x1f42fa4f28b0c703), UINT64_C(0x1f42fb8e9cac070d),
        UINT64_C(0x1f42f88f68b56718), UINT64_C(0x1f42ff8f00af0720)
    },
    {   UINT64_C(0x000000000001944b), UINT64_C(0x32cb040fa0fea690), UINT64_C(0x0a3af08b446246f5),
        UINT64_C(0x07d2f84b4454c6f5), UINT64_C(0x0c82ff518867c568)
    },
    {   UINT64_C(0x000000000001944c), UINT64_C(0x0c82ff518867c568), UINT64_C(0x15732291347a057d),
        UINT64_C(0x09daee118065e56a), UINT64_C(0x14630d1010808598)
    },
    {   UINT64_C(0x000000000001944d), UINT64_C(0x14630d1010808598), UINT64_C(0x1cf2fbcd30ab867e),
        UINT64_C(0x1cf2fbcd30ab867e), UINT64_C(0x1cf2fb8d30ab667e)
    },
    {   UINT64_C(0x000000000001944e), UINT64_C(0x1cf2fb8d30ab667e), UINT64_C(0x0ffb25d14c62a4be),
        UINT64_C(0x072afd5310538506), UINT64_C(0x08aaff91b85864f7)
    },
    {   UINT64_C(0x000000000001944f), UINT64_C(0x08aaff91b85864f7), UINT64_C(0x0ffafccd3076e67e),
        UINT64_C(0x0ffafccd3076e67e), UINT64_C(0x1cf2fb8d30ab867e)
    },
    {   UINT64_C(0x0000000000005a08), UINT64_C(0x1f43064e30b34724), UINT64_C(0x1f42fa8f58b7e717),
        UINT64_C(0x1f42fa4f58b54717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019451), UINT64_C(0x09baed530465c50a), UINT64_C(0x0b7afb918c65a566),
        UINT64_C(0x0b7afb919065a567), UINT64_C(0x0ffafccd3076e67e)
    },
    {   UINT64_C(0x0000000000005a0a), UINT64_C(0x1f42fccef8b82708), UINT64_C(0x1f42fdcf94aa671c),
        UINT64_C(0x1f42f7ceacb8a73e), UINT64_C(0x1f42f9ceccb6e723)
    },
    {   UINT64_C(0x0000000000005a0b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ceecb72708),
        UINT64_C(0x1f42fdce6cb7670c), UINT64_C(0x1f42facea4b40709)
    },
    {   UINT64_C(0x0000000000019454), UINT64_C(0x0ffb25113063257b), UINT64_C(0x0b9af1930466250a),
        UINT64_C(0x0b9afb93046b250a), UINT64_C(0x0bdaff91b865255d)
    },
    {   UINT64_C(0x0000000000019455), UINT64_C(0x0bdaff91b865255d), UINT64_C(0x0bdaff91b865255d),
        UINT64_C(0x0bdaff91b865255d), UINT64_C(0x09daee0ff065e5ce)
    },
    {   UINT64_C(0x0000000000019456), UINT64_C(0x1cf327913495857d), UINT64_C(0x09daee0f1465e605),
        UINT64_C(0x204b275134a3257d), UINT64_C(0x0f02eb98c07bc662)
    },
    {   UINT64_C(0x0000000000005a0f), UINT64_C(0x1f42f88f68b8c727), UINT64_C(0x1f43008e8cb38724),
        UINT64_C(0x1f43160edcaec71f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a10), UINT64_C(0x1f430b4eb8b40733), UINT64_C(0x1f42f7ce80b3a723),
        UINT64_C(0x1f42fdcedcb4c723), UINT64_C(0x1f42f9cedcb6c72b)
    },
    {   UINT64_C(0x0000000000005a11), UINT64_C(0x1f42f9ce94b86710), UINT64_C(0x1f42f88f38b20725),
        UINT64_C(0x1f42f60f18b12708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a12), UINT64_C(0x1f430e4ebcb02735), UINT64_C(0x1f42f9cef0b5470c),
        UINT64_C(0x1f43094ed8b7671b), UINT64_C(0x1f42fdcef8b6471b)
    },
    {   UINT64_C(0x000000000001945b), UINT64_C(0x38e2eb8b8d252474), UINT64_C(0x13caeddc708dc7a3),
        UINT64_C(0x13caedcb8c8dc36a), UINT64_C(0x1acafa0d289fa67c)
    },
    {   UINT64_C(0x000000000001945c), UINT64_C(0x1acb020d28a3a67c), UINT64_C(0x156af90d2c8aa67d),
        UINT64_C(0x156b010d2c8ea67d), UINT64_C(0x16fae7cad09bc46f)
    },
    {   UINT64_C(0x000000000001945d), UINT64_C(0x16faeb4ad09d846f), UINT64_C(0x16fae7cfa09bc5a4),
        UINT64_C(0x16faeb4fa09d85a4), UINT64_C(0x0f9af50f78796693)
    },
    {   UINT64_C(0x000000000001945e), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa082e67e),
        UINT64_C(0x10bb008fa07fe67e), UINT64_C(0x14d2f5cfa08de68a)
    },
    {   UINT64_C(0x000000000001945f), UINT64_C(0x123b008fa082067e), UINT64_C(0x13eaf50fa089e67e),
        UINT64_C(0x1493008fa087867e), UINT64_C(0x1652f58f6894068c)
    },
    {   UINT64_C(0x0000000000000b86), UINT64_C(0x106afd4bfc78456c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019461), UINT64_C(0x151b008f6889a67e), UINT64_C(0x141af50fa08b667e),
        UINT64_C(0x1652f58f6894068c), UINT64_C(0x1493008fa087867e)
    },
    {   UINT64_C(0x0000000000019462), UINT64_C(0x106b008fa07e867e), UINT64_C(0x1652f58f6894068c),
        UINT64_C(0x1382f48fa088c67e), UINT64_C(0x1302f1cfa088a67e)
    },
    {   UINT64_C(0x0000000000019463), UINT64_C(0x1592f40fa091c68c), UINT64_C(0x1662f3cfa095267e),
        UINT64_C(0x15e2f5cfa092267e), UINT64_C(0x16a2f10fa0978691)
    },
    {   UINT64_C(0x0000000000019464), UINT64_C(0x1742f54fa097e68c), UINT64_C(0x10caf50f788105b7),
        UINT64_C(0x0f92f84c44796693), UINT64_C(0x0ed2f4cc407765b7)
    },
    {   UINT64_C(0x0000000000019465), UINT64_C(0x0f9af50f78796693), UINT64_C(0x0db2f38f7872e5b7),
        UINT64_C(0x0d02f64c4c718693), UINT64_C(0x0f72f84c4475e5b7)
    },
    {   UINT64_C(0x0000000000005a1e), UINT64_C(0x1f42fecec8abe711), UINT64_C(0x1f42facf48b3a6f0),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cec8b4671a)
    },
    {   UINT64_C(0x0000000000005a1f), UINT64_C(0x1f43084f0cb1a704), UINT64_C(0x1f43164f10ae0705),
        UINT64_C(0x1f42fa8f98b246ed), UINT64_C(0x1f42f88e6cb64732)
    },
    {   UINT64_C(0x0000000000019468), UINT64_C(0x0fdaefcf787b65ad), UINT64_C(0x0f92fa0c407945b6),
        UINT64_C(0x0d02f64c507245b7), UINT64_C(0x1072f50f788245ad)
    },
    {   UINT64_C(0x0000000000005a21), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60f2cb7a72c),
        UINT64_C(0x1f42f30e7caf2726), UINT64_C(0x1f42fbce9cb2e6ec)
    },
    {   UINT64_C(0x000000000001946a), UINT64_C(0x1472f10c408ec5b5), UINT64_C(0x1382f54f7888e5ae),
        UINT64_C(0x0e42f3cf787485b7), UINT64_C(0x0f4af88f707be693)
    },
    {   UINT64_C(0x000000000001946b), UINT64_C(0x1372f50c4088c694), UINT64_C(0x0f5af50c407865b7),
        UINT64_C(0x17d2efcf789ae5ad), UINT64_C(0x105af84f787aa58e)
    },
    {   UINT64_C(0x000000000001946c), UINT64_C(0x105b0dcfa071258d), UINT64_C(0x105aff0fa077658d),
        UINT64_C(0x105af9cc88786590), UINT64_C(0x105afc8c887a858c)
    },
    {   UINT64_C(0x000000000001946d), UINT64_C(0x105afe8c8877a58d), UINT64_C(0x105af74f807b464c),
        UINT64_C(0x105af44c887bc58d), UINT64_C(0x105af9cf707a2646)
    },
    {   UINT64_C(0x000000000001946e), UINT64_C(0x105af84c807ae58d), UINT64_C(0x0002fb532c7025c2),
        UINT64_C(0x00030597307806c6), UINT64_C(0x156afa0e0c8e2656)
    },
    {   UINT64_C(0x000000000001946f), UINT64_C(0x0a2aedcfa067446f), UINT64_C(0x13caedcfa08dc46f),
        UINT64_C(0x0a2aeddc706747a3), UINT64_C(0x2b62f78fa0e7467e)
    },
    {   UINT64_C(0x0000000000019470), UINT64_C(0x382afc8b9117e6e3), UINT64_C(0x1692f1cc3896e6dc),
        UINT64_C(0x14daf24c3c88a6dc), UINT64_C(0x125af0cc3c86a6dc)
    },
    {   UINT64_C(0x0000000000019471), UINT64_C(0x1692fa0c3892e6dc), UINT64_C(0x131af10c3c8926dc),
        UINT64_C(0x1322f08c3c8946dc), UINT64_C(0x1692f28c3c9666dc)
    },
    {   UINT64_C(0x0000000000019472), UINT64_C(0x1692f68c389486dc), UINT64_C(0x0a6ae80c387546dc),
        UINT64_C(0x16daf64c3c98e6dc), UINT64_C(0x15f2f1cc389466dc)
    },
    {   UINT64_C(0x0000000000019473), UINT64_C(0x1bcaf10c3caa86dc), UINT64_C(0x1692f10c3896e6dc),
        UINT64_C(0x141af54c3c8b46dc), UINT64_C(0x1692f68c389486dc)
    },
    {   UINT64_C(0x0000000000019474), UINT64_C(0x1692f28c3c9666dc), UINT64_C(0x116af20c388526dc),
        UINT64_C(0x1382f58c3c8986dc), UINT64_C(0x131af24c3888a6dc)
    },
    {   UINT64_C(0x0000000000019475), UINT64_C(0x1692f54c389666dc), UINT64_C(0x17daf6cc3c9986dc),
        UINT64_C(0x1692f08c3c9686dc), UINT64_C(0x173af38c3c9906dc)
    },
    {   UINT64_C(0x0000000000019476), UINT64_C(0x1742f54c3c97e6dc), UINT64_C(0x150afbcbe091a6dc),
        UINT64_C(0x116af00c388f26dc), UINT64_C(0x142adc0c3892e6ed)
    },
    {   UINT64_C(0x0000000000019477), UINT64_C(0x1622efcc389206dc), UINT64_C(0x156afa0d308e267e),
        UINT64_C(0x156afa0d848e265b), UINT64_C(0x156afa0d848e265b)
    },
    {   UINT64_C(0x0000000000019478), UINT64_C(0x1c32ee0d30aa667e), UINT64_C(0x1c32ee0d30aa667e),
        UINT64_C(0x0ea310cd106e4812), UINT64_C(0x0ea3038d1067a812)
    },
    {   UINT64_C(0x0000000000019479), UINT64_C(0x121af44d10838812), UINT64_C(0x0bfb1b0d106be812),
        UINT64_C(0x0bfaf30d1057e812), UINT64_C(0x1f42ee0cdcbb8369)
    },
    {   UINT64_C(0x000000000001947a), UINT64_C(0x11fb0a8d1078c812), UINT64_C(0x11fb094d10782812),
        UINT64_C(0x121af4cd10838812), UINT64_C(0x1182ee18488486e7)
    },
    {   UINT64_C(0x000000000001947b), UINT64_C(0x1172ed98f88446d6), UINT64_C(0x1f42ee1ba8bb871c),
        UINT64_C(0x1182ee18488486e7), UINT64_C(0x1172ed8b6c844373)
    },
    {   UINT64_C(0x000000000001947c), UINT64_C(0x1182ee18488486e7), UINT64_C(0x1182ee18f88486d6),
        UINT64_C(0x1f42ee1ba8bb871c), UINT64_C(0x1182ee18488486e7)
    },
    {   UINT64_C(0x000000000001947d), UINT64_C(0x1182ee0b6c848373), UINT64_C(0x1f42ee0cdcbb8369),
        UINT64_C(0x00023a0f4c2ae5fc), UINT64_C(0x1b32ee18c0ab66d5)
    },
    {   UINT64_C(0x000000000001947e), UINT64_C(0x1b3aee18c0ab66d5), UINT64_C(0x1b3aee1728ab86c7),
        UINT64_C(0x1b3aee1728ab86c8), UINT64_C(0x1b3aee0b20ab636d)
    },
    {   UINT64_C(0x000000000001947f), UINT64_C(0x1b3aee0b20ab636d), UINT64_C(0x1b3aee0b44ab83ce),
        UINT64_C(0x1b3aee0b44ab83cf), UINT64_C(0x275afbd068d509c4)
    },
    {   UINT64_C(0x0000000000019480), UINT64_C(0x2a5afa9064e1a9c4), UINT64_C(0x2a5afa9064e1a9c4),
        UINT64_C(0x2863049064d4c9c3), UINT64_C(0x210b0a0b04c6a9f8)
    },
    {   UINT64_C(0x0000000000019481), UINT64_C(0x210b0a0b04c6a9f8), UINT64_C(0x210b0a0b04c6a9f8),
        UINT64_C(0x278afd1064d529c4), UINT64_C(0x278afd1064d529c4)
    },
    {   UINT64_C(0x0000000000019482), UINT64_C(0x278b0b9064cde9ce), UINT64_C(0x278b0b903ccde9c4),
        UINT64_C(0x0e2b1b0d1073c812), UINT64_C(0x0e2af44d1060a812)
    },
    {   UINT64_C(0x0000000000019483), UINT64_C(0x0e2b1b0d10740812), UINT64_C(0x0e2af4cd1060a812),
        UINT64_C(0x1583378e64864812), UINT64_C(0x15830aceb46fc826)
    },
    {   UINT64_C(0x0000000000019484), UINT64_C(0x0e12fa8adc7086a4), UINT64_C(0x0e12fa8adc5446a9),
        UINT64_C(0x0e12fa8af07086a9), UINT64_C(0x0e12fa8adc7086a4)
    },
    {   UINT64_C(0x0000000000019485), UINT64_C(0x0e13330adc7086a9), UINT64_C(0x0e12fa8af07086a9),
        UINT64_C(0x0e12fa8a94726692), UINT64_C(0x0e12fa8a94514697)
    },
    {   UINT64_C(0x0000000000019486), UINT64_C(0x0e12fa8aa8726697), UINT64_C(0x0e12f6ca94708692),
        UINT64_C(0x0e13390a94708697), UINT64_C(0x0e12f6caa8708697)
    },
    {   UINT64_C(0x0000000000019487), UINT64_C(0x14032f0adc8986a4), UINT64_C(0x1402f80adc6e06a9),
        UINT64_C(0x14032f0af08986a9), UINT64_C(0x14032f0adc6e06a9)
    },
    {   UINT64_C(0x0000000000019488), UINT64_C(0x1402f80adc6e06a4), UINT64_C(0x14032f0adc8986a9),
        UINT64_C(0x1402f80af06e06a9), UINT64_C(0x1583378f3c6fc64a)
    },
    {   UINT64_C(0x0000000000005a41), UINT64_C(0x1f42fa8f0cb34717), UINT64_C(0x1f42f7ce68b4e727),
        UINT64_C(0x1f42fa8eeca8c6fb), UINT64_C(0x1f42fb8eacb36700)
    },
    {   UINT64_C(0x000000000001948a), UINT64_C(0x213aff5820bac6e6), UINT64_C(0x213aff0e54baa473),
        UINT64_C(0x25030a0a3cae69ce), UINT64_C(0x168aee0b1c46ca06)
    },
    {   UINT64_C(0x000000000001948b), UINT64_C(0x168b91cb1c98aa06), UINT64_C(0x1cf2ee0d3cb24424),
        UINT64_C(0x1cf2ee1870b246f1), UINT64_C(0x1cf2ee0e3cb22470)
    },
    {   UINT64_C(0x000000000001948c), UINT64_C(0x1cf2ee1870b226fd), UINT64_C(0x2da2ee0c50f5042a),
        UINT64_C(0x2da2ee1870f50732), UINT64_C(0x11b3214d1081e812)
    },
    {   UINT64_C(0x000000000001948d), UINT64_C(0x11b2f4cd106ba812), UINT64_C(0x101b1b0d107b4812),
        UINT64_C(0x101af54d10686812), UINT64_C(0x12130b0d10784812)
    },
    {   UINT64_C(0x0000000000005a46), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf38b42717),
        UINT64_C(0x1f42fc8f1cb4470b), UINT64_C(0x1f42fccfb4b4271b)
    },
    {   UINT64_C(0x000000000001948f), UINT64_C(0x16430a8d1087e812), UINT64_C(0x13c30fcd1082c812),
        UINT64_C(0x13c3038d107ca812), UINT64_C(0x1932f44b44a02a06)
    },
    {   UINT64_C(0x0000000000019490), UINT64_C(0x1932f44b44a02a06), UINT64_C(0x309afb1064fa69c4),
        UINT64_C(0x309afb1064fa69c4), UINT64_C(0x309afb1064fa69c4)
    },
    {   UINT64_C(0x0000000000019491), UINT64_C(0x278b0b903ccde9c4), UINT64_C(0x278b0b903ccde9c4),
        UINT64_C(0x240b029064c469c4), UINT64_C(0x240b029064c469c4)
    },
    {   UINT64_C(0x0000000000019492), UINT64_C(0x2fd2fd1064f649c4), UINT64_C(0x2fd2fd1064f649c4),
        UINT64_C(0x2e52fd1064f049c4), UINT64_C(0x2863049068d4c9c4)
    },
    {   UINT64_C(0x0000000000019493), UINT64_C(0x15930f49f484274b), UINT64_C(0x0cd30709f465474b),
        UINT64_C(0x126b10cb447d6a06), UINT64_C(0x13030a8b447a4a06)
    },
    {   UINT64_C(0x0000000000019494), UINT64_C(0x1932f44b44a02a06), UINT64_C(0x0e5b1d8b44734a06),
        UINT64_C(0x0e5af74b44602a06), UINT64_C(0x2ee2ee0cdcfa0369)
    },
    {   UINT64_C(0x0000000000019495), UINT64_C(0x166b0d0b4480ea06), UINT64_C(0x166b1c8b4488aa06),
        UINT64_C(0x1932f44b44a02a06), UINT64_C(0x1e9aee1870b8e6f1)
    },
    {   UINT64_C(0x000000000000676b), UINT64_C(0x1f42fa0e88b44734), UINT64_C(0x1f4315ce20a7473e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4312ce28ae273b)
    },
    {   UINT64_C(0x0000000000005a4f), UINT64_C(0x1f42f4cef8b6c6f8), UINT64_C(0x1f42f7cec0b9e717),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ef4b9a734)
    },
    {   UINT64_C(0x000000000000676c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4dfcb4a745),
        UINT64_C(0x1f42fb8eb8b3c72d), UINT64_C(0x1f42fc0ee8b1671b)
    },
    {   UINT64_C(0x0000000000005a51), UINT64_C(0x1f42fd8f18b1c704), UINT64_C(0x1f42f7ceacb166f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001949a), UINT64_C(0x28e2ee0b20e2036d), UINT64_C(0x28e2ee0b44e203ce),
        UINT64_C(0x28e2ee0b44e203ce), UINT64_C(0x2323098950cf2bf0)
    },
    {   UINT64_C(0x0000000000005a53), UINT64_C(0x1f42f8ce80ab0726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ea8ae06fb), UINT64_C(0x1f42f98f58b8271f)
    },
    {   UINT64_C(0x0000000000005a54), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6cf38b72717),
        UINT64_C(0x1f42fc8f78b26717), UINT64_C(0x1f42fb8f94b6e716)
    },
    {   UINT64_C(0x000000000001949d), UINT64_C(0x2da2ee0d54f5042a), UINT64_C(0x2da2ee1890f506f9),
        UINT64_C(0x2da2ee0e5cf50477), UINT64_C(0x2da2ee1894f50705)
    },
    {   UINT64_C(0x0000000000005a56), UINT64_C(0x1f43088e30b4e728), UINT64_C(0x1f42f60f48b9270e),
        UINT64_C(0x1f4308ced8b24738), UINT64_C(0x1f42fccef8b10704)
    },
    {   UINT64_C(0x000000000001949f), UINT64_C(0x115b1d8b447f2a06), UINT64_C(0x115af78b446c2a06),
        UINT64_C(0x137305cb4480ea06), UINT64_C(0x137304cb44806a06)
    },
    {   UINT64_C(0x0000000000005a58), UINT64_C(0x1f42f6cf38ada71b), UINT64_C(0x1f42f5cf60b7a718),
        UINT64_C(0x1f42f30f38b08717), UINT64_C(0x1f42f7cf98a9a71f)
    },
    {   UINT64_C(0x0000000000005a59), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f48b3071b),
        UINT64_C(0x1f42fb8f08afa6f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a5a), UINT64_C(0x1f42f98f30a9272d), UINT64_C(0x1f42f98eacb7e6fd),
        UINT64_C(0x1f42f2ce6cb7a725), UINT64_C(0x1f42f9ce4cb1871a)
    },
    {   UINT64_C(0x0000000000005a5b), UINT64_C(0x1f42f70e98b1c70c), UINT64_C(0x1f430d4eb8b16736),
        UINT64_C(0x1f42fccf1cb08728), UINT64_C(0x1f42f7cf10ac4704)
    },
    {   UINT64_C(0x0000000000005a5c), UINT64_C(0x1f42fb8f38b44717), UINT64_C(0x1f42f7cef8b86711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94eacb60729)
    },
    {   UINT64_C(0x0000000000005a5d), UINT64_C(0x1f42fc8eb8b1e722), UINT64_C(0x1f42fb8f48af4714),
        UINT64_C(0x1f43124e8cb00721), UINT64_C(0x1f42f6cee4b36723)
    },
    {   UINT64_C(0x0000000000005a5e), UINT64_C(0x1f42f4cedcb946ff), UINT64_C(0x1f42f6cf38b1a70f),
        UINT64_C(0x1f42f98eacb9a70c), UINT64_C(0x1f42f60edcb34704)
    },
    {   UINT64_C(0x0000000000005a5f), UINT64_C(0x1f42fe8f18b0e71c), UINT64_C(0x1f42ff4e5cb2e72a),
        UINT64_C(0x1f42facedcb5270b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a60), UINT64_C(0x1f42fc8f58b2c71f), UINT64_C(0x1f42f84f38b6671b),
        UINT64_C(0x1f42f6cf58b7671b), UINT64_C(0x1f43108ed4afc70d)
    },
    {   UINT64_C(0x0000000000005a61), UINT64_C(0x1f42f88f38b3a70c), UINT64_C(0x1f42f60f78b7a72b),
        UINT64_C(0x1f42f6ceccb84718), UINT64_C(0x1f42fd8ef8ae6712)
    },
    {   UINT64_C(0x0000000000005a62), UINT64_C(0x1f42fdcfa8aba70f), UINT64_C(0x1f42f88e7cb7870c),
        UINT64_C(0x1f42fb4e70b4c729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004076), UINT64_C(0x1f433190a099e71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f433190a099e6e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a64), UINT64_C(0x1f42f88efcb22708), UINT64_C(0x1f42f9cea0b2a734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88edcb14703)
    },
    {   UINT64_C(0x0000000000004077), UINT64_C(0x1f433190a099e713), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a66), UINT64_C(0x1f43114eacb0270e), UINT64_C(0x1f42fb8edcb8a70e),
        UINT64_C(0x1f42f6cf58ab470d), UINT64_C(0x1f42f6ceacb74723)
    },
    {   UINT64_C(0x0000000000005a67), UINT64_C(0x1f42f9cef4b2c723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f04af86f8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006778), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffceb0b2c723),
        UINT64_C(0x1f43024de8b06736), UINT64_C(0x1f43018ed8b2c72d)
    },
    {   UINT64_C(0x00000000000194b1), UINT64_C(0x4e22ee0cdd770369), UINT64_C(0x21a31947acaccdee),
        UINT64_C(0x21a31e87acaf6dee), UINT64_C(0x28eaf207ace02dee)
    },
    {   UINT64_C(0x0000000000005a6a), UINT64_C(0x1f42fc8e9cb3272e), UINT64_C(0x1f42f88ed0afa709),
        UINT64_C(0x1f42f88ea4af870f), UINT64_C(0x1f42f1ce3cb5472b)
    },
    {   UINT64_C(0x00000000000194b3), UINT64_C(0x3af2ee0b412a4373), UINT64_C(0x3af2ee18952a4704),
        UINT64_C(0x3af2ee19212a46eb), UINT64_C(0x4e22ee1ba977071c)
    },
    {   UINT64_C(0x00000000000194b4), UINT64_C(0x3af2ee18952a4704), UINT64_C(0x3af2ee0b412a4373),
        UINT64_C(0x4e22ee0cdd770369), UINT64_C(0x0002738c102966c3)
    },
    {   UINT64_C(0x00000000000194b5), UINT64_C(0x4422ee18c14f06d5), UINT64_C(0x4422ee18c14f06d5),
        UINT64_C(0x4422ee17294f06c7), UINT64_C(0x4422ee17294f06c7)
    },
    {   UINT64_C(0x0000000000005a6e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f18b726ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f58a98727)
    },
    {   UINT64_C(0x00000000000194b7), UINT64_C(0x25030a0fa0ae6b27), UINT64_C(0x25030a0fa0ae6c44),
        UINT64_C(0x25030a0fa0ae6c3f), UINT64_C(0x155337c7ac906dee)
    },
    {   UINT64_C(0x0000000000005a70), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f68aba70e),
        UINT64_C(0x1f42fccf04b006fc), UINT64_C(0x1f43094e9cb68740)
    },
    {   UINT64_C(0x00000000000194b9), UINT64_C(0x5423030e55848473), UINT64_C(0x48c2ee0d01618434),
        UINT64_C(0x48c2ee1895618719), UINT64_C(0x48c2ee0e39618497)
    },
    {   UINT64_C(0x00000000000194ba), UINT64_C(0x48c2ee189161872d), UINT64_C(0x5592ee0c5194c42a),
        UINT64_C(0x5592ee187194c732), UINT64_C(0x203b3e07acb66dee)
    },
    {   UINT64_C(0x00000000000194bb), UINT64_C(0x203b0007ac976dee), UINT64_C(0x185337c7ac970dee),
        UINT64_C(0x1852ff87ac7aedee), UINT64_C(0x1c630a47aca20dee)
    },
    {   UINT64_C(0x00000000000194bc), UINT64_C(0x1c630a07aca1edee), UINT64_C(0x27cb0d87accc2dee),
        UINT64_C(0x27cb1107accdedee), UINT64_C(0x264b1b87acc20dee)
    },
    {   UINT64_C(0x00000000000194bd), UINT64_C(0x264b1947acc0edee), UINT64_C(0x24bb1ec7acc1cdee),
        UINT64_C(0x24bb1e87acc1adee), UINT64_C(0x5dc2ee0cddb58369)
    },
    {   UINT64_C(0x0000000000005a76), UINT64_C(0x1f42fdcec0b3a740), UINT64_C(0x1f42fb0ed0b5071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f28b2270b)
    },
    {   UINT64_C(0x00000000000194bf), UINT64_C(0x48c2ee0b41618373), UINT64_C(0x48c2ee1891618718),
        UINT64_C(0x48c2ee19456186f4), UINT64_C(0x5dc2ee1ba9b5871c)
    },
    {   UINT64_C(0x0000000000005a78), UINT64_C(0x1f42fdcf80b12713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98ef0b1e70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194c1), UINT64_C(0x5dc2ee18c1b586d5), UINT64_C(0x5dc2ee18c1b586d5),
        UINT64_C(0x5dc2ee1729b586c7), UINT64_C(0x5dc2ee1729b586c7)
    },
    {   UINT64_C(0x0000000000006781), UINT64_C(0x1f430d4e0cb3073c), UINT64_C(0x1f42fe0eb0b1c72b),
        UINT64_C(0x1f42f84ea8b40721), UINT64_C(0x1f43030ed4b32735)
    },
    {   UINT64_C(0x000000000001787e), UINT64_C(0x0fa2f30fa0872698), UINT64_C(0x0fa2f60f6c87c698),
        UINT64_C(0x0fa2fbcfa0876698), UINT64_C(0x0fa2f4cf6c8fe68c)
    },
    {   UINT64_C(0x0000000000005a7c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ef8b18704),
        UINT64_C(0x1f42f7ce6cb8871b), UINT64_C(0x1f430d8e6caee73c)
    },
    {   UINT64_C(0x000000000001787f), UINT64_C(0x0fa3020f6c8e4698), UINT64_C(0x0fa30c0fa08e668c),
        UINT64_C(0x0fa2fdcf6c88c698), UINT64_C(0x0fa2f50f6c880698)
    },
    {   UINT64_C(0x0000000000006783), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ebcb5271c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcebcb52723)
    },
    {   UINT64_C(0x00000000000194c7), UINT64_C(0x13dafc0b6cbfebb8), UINT64_C(0x1c92fc0b6cbfebb8),
        UINT64_C(0x2f5afc0b6d342bb8), UINT64_C(0x13dafc0b6cbfebb8)
    },
    {   UINT64_C(0x0000000000005a80), UINT64_C(0x1f42fd8eb0ae8712), UINT64_C(0x1f42f88f04b64706),
        UINT64_C(0x1f43068e5cb42731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194c9), UINT64_C(0x16fafc0b6cbfebb8), UINT64_C(0x13dafc0b6cbfebb8),
        UINT64_C(0x1a62fc0b6cc24bb8), UINT64_C(0x13dafc0b6cbfebb8)
    },
    {   UINT64_C(0x0000000000005a82), UINT64_C(0x1f42f2cec0b26714), UINT64_C(0x1f42f2cf08b64710),
        UINT64_C(0x1f430a8f14b1c72c), UINT64_C(0x1f42f24f58b96703)
    },
    {   UINT64_C(0x0000000000017882), UINT64_C(0x17f2f28fa09e4685), UINT64_C(0x132af28fa0858685),
        UINT64_C(0x1c82f48fa0b46685), UINT64_C(0x176af6cf689a4695)
    },
    {   UINT64_C(0x0000000000005a84), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ceecb8c723),
        UINT64_C(0x1f42facf50b52721), UINT64_C(0x1f42f60ec8b44740)
    },
    {   UINT64_C(0x00000000000194cd), UINT64_C(0x1fe2f84cacbd069a), UINT64_C(0x1182f84cac83869a),
        UINT64_C(0x1182f84cac83869a), UINT64_C(0x1182f84cac83869a)
    },
    {   UINT64_C(0x0000000000006787), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8efcb4e71a),
        UINT64_C(0x1f4318ce90a74735), UINT64_C(0x1f43074edcac073d)
    },
    {   UINT64_C(0x00000000000194cf), UINT64_C(0x1182f84ca883869e), UINT64_C(0x0f82f84ca883869e),
        UINT64_C(0x1182f84cac83869f), UINT64_C(0x100af84ca483869e)
    },
    {   UINT64_C(0x00000000000194d0), UINT64_C(0x1002f84ca883869e), UINT64_C(0x13daf84ca889269e),
        UINT64_C(0x1002f8cca883869e), UINT64_C(0x1002f84cac83869f)
    },
    {   UINT64_C(0x00000000000194d1), UINT64_C(0x132af14ca887c69e), UINT64_C(0x1002f84cac83869f),
        UINT64_C(0x1002f80ca883669e), UINT64_C(0x1002f80ca883669e)
    },
    {   UINT64_C(0x0000000000005a8a), UINT64_C(0x1f42f98e3cb54717), UINT64_C(0x1f42ff8f1cb2e725),
        UINT64_C(0x1f42fbcee8b2670f), UINT64_C(0x1f42f30f88b90717)
    },
    {   UINT64_C(0x00000000000194d3), UINT64_C(0x19d2fccaa0ab0720), UINT64_C(0x0c6af6caa07b4720),
        UINT64_C(0x1122f6caa08e2720), UINT64_C(0x1b62fb8aa0b80720)
    },
    {   UINT64_C(0x00000000000194d4), UINT64_C(0x0ef2fb8aa082e720), UINT64_C(0x0dcaf6caa080c720),
        UINT64_C(0x0edaf6caa0850720), UINT64_C(0x1612f8ca70912714)
    },
    {   UINT64_C(0x00000000000194d5), UINT64_C(0x222afcca70cc6714), UINT64_C(0x0e9afcca707e2714),
        UINT64_C(0x0e9afcca707e2714), UINT64_C(0x108afcca7085e714)
    },
    {   UINT64_C(0x0000000000005a8e), UINT64_C(0x1f42facf58b2472b), UINT64_C(0x1f42f7ce5cb80730),
        UINT64_C(0x1f42f6ce6cb3673d), UINT64_C(0x1f42fa8e80b38716)
    },
    {   UINT64_C(0x00000000000194d7), UINT64_C(0x0feafb8a7086c714), UINT64_C(0x0c62f6ca707b2714),
        UINT64_C(0x0ce2f5ca707aa714), UINT64_C(0x181af94a70a66714)
    },
    {   UINT64_C(0x000000000000678c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f48ad2712),
        UINT64_C(0x1f4307cf10ac8711), UINT64_C(0x1f43154eb0ab2710)
    },
    {   UINT64_C(0x00000000000194d9), UINT64_C(0x0bc2fcc8847e2714), UINT64_C(0x0e9afc8b6c7e8bb8),
        UINT64_C(0x189afc8b6ca68bb8), UINT64_C(0x229afc8b6cce8bb8)
    },
    {   UINT64_C(0x000000000000678d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f90b1c72c),
        UINT64_C(0x1f430e8ea8a72731), UINT64_C(0x1f43124ea8a6c713)
    },
    {   UINT64_C(0x000000000001788a), UINT64_C(0x0eeaf60fa08145a9), UINT64_C(0x16eafbcfa09de5a9),
        UINT64_C(0x1192eb0fa08825a9), UINT64_C(0x1242ebcc6c8ee5a9)
    },
    {   UINT64_C(0x000000000000678e), UINT64_C(0x1f42fb4f24ad2711), UINT64_C(0x1f42fb0f28ae270a),
        UINT64_C(0x1f42f50f3caca711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001788b), UINT64_C(0x135af6cfa088c5a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000678f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f14b14723),
        UINT64_C(0x1f43168ee8a5472e), UINT64_C(0x1f4309cf24aea71e)
    },
    {   UINT64_C(0x00000000000194df), UINT64_C(0x16fafe4b6ca2abb8), UINT64_C(0x16fafe4b6ca2abb8),
        UINT64_C(0x15eaee0b6c9b2c55), UINT64_C(0x0e9afc88f8a26bb8)
    },
    {   UINT64_C(0x0000000000005a98), UINT64_C(0x1f42fdce94b4a719), UINT64_C(0x1f42f80edcb8273f),
        UINT64_C(0x1f42f6ce8cae8717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194e1), UINT64_C(0x1552fb0cac8ec69a), UINT64_C(0x1c6afb0cacab269a),
        UINT64_C(0x0f02fb0cac76669a), UINT64_C(0x0f02fb0cac76869a)
    },
    {   UINT64_C(0x0000000000005a9a), UINT64_C(0x1f42fc8e8cb3a727), UINT64_C(0x1f42fb8f1cb206f0),
        UINT64_C(0x1f42f88edcb6a6fb), UINT64_C(0x1f42f5cee8b46738)
    },
    {   UINT64_C(0x00000000000194e3), UINT64_C(0x0d22ff0ca86f669e), UINT64_C(0x1042f7cca87ac69e),
        UINT64_C(0x0a22f84ca86e069e), UINT64_C(0x0f02fb0ca876669e)
    },
    {   UINT64_C(0x00000000000194e4), UINT64_C(0x0e12ff0ca871c69e), UINT64_C(0x0e12ff0cac72069a),
        UINT64_C(0x1132ff0cac7e869a), UINT64_C(0x0e12fa8cac71c69a)
    },
    {   UINT64_C(0x00000000000194e5), UINT64_C(0x0e12fa8ca471c69e), UINT64_C(0x131af10ca887a69e),
        UINT64_C(0x0e12fa0ca872069e), UINT64_C(0x0e12fccca870e69e)
    },
    {   UINT64_C(0x0000000000005a9e), UINT64_C(0x1f42fd0f18b4070e), UINT64_C(0x1f42fb8ee4b3c72f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a9f), UINT64_C(0x1f42f8cf4cb8270c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98eacb86713), UINT64_C(0x1f42f88f48ae670e)
    },
    {   UINT64_C(0x0000000000005aa0), UINT64_C(0x1f42f8cf28afc721), UINT64_C(0x1f42f4cebcb7c72d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43150f18ac671d)
    },
    {   UINT64_C(0x0000000000005aa1), UINT64_C(0x1f42f7cf80b786fb), UINT64_C(0x1f42f60e5cb5e71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60ec0af870e)
    },
    {   UINT64_C(0x0000000000005aa2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf58b3671e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005aa3), UINT64_C(0x1f42f70ebcb166eb), UINT64_C(0x1f42f60ef8b32711),
        UINT64_C(0x1f42f40e60b7c722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000015193), UINT64_C(0x185b084ce0a7669b), UINT64_C(0x14f3020fa095c68c),
        UINT64_C(0x1152fe8f548e269b), UINT64_C(0x140b154fa0a0a68c)
    },
    {   UINT64_C(0x0000000000017893), UINT64_C(0x142af68fa09ac6a0), UINT64_C(0x133af8cd608b66a8),
        UINT64_C(0x1a8af38f40a9c6a0), UINT64_C(0x136af70fa08c66a0)
    },
    {   UINT64_C(0x0000000000015194), UINT64_C(0x15db0ccf54a1868c), UINT64_C(0x15931e4f58a6a68c),
        UINT64_C(0x1f2b1f8f64cc668c), UINT64_C(0x1722f80fa0a9268c)
    },
    {   UINT64_C(0x0000000000005aa7), UINT64_C(0x1f42f6ce54b80727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f50f38b20723), UINT64_C(0x1f42fa8eb0b006f2)
    },
    {   UINT64_C(0x0000000000015195), UINT64_C(0x15b31d0fa0a7c68c), UINT64_C(0x146af44fa09ea68c),
        UINT64_C(0x0f4afa0f687de5c1), UINT64_C(0x1002facf6c84468c)
    },
    {   UINT64_C(0x0000000000005aa9), UINT64_C(0x1f42f60f18b9c709), UINT64_C(0x1f42fb8ea4b76740),
        UINT64_C(0x1f42f7ceccab46ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005aaa), UINT64_C(0x1f42fc8f28aaa720), UINT64_C(0x1f42fb8ec4abe719),
        UINT64_C(0x1f42f6cedcb72721), UINT64_C(0x1f42f98ef8b62721)
    },
    {   UINT64_C(0x0000000000005aab), UINT64_C(0x1f42f6cea0b6a738), UINT64_C(0x1f42f8cf58b706ed),
        UINT64_C(0x1f42f9cef0b1873d), UINT64_C(0x1f42f8cf20b0c73c)
    },
    {   UINT64_C(0x000000000000679a), UINT64_C(0x1f43028f5cb4e706), UINT64_C(0x1f42ffcf30b106fd),
        UINT64_C(0x1f42fe0f1cb0e709), UINT64_C(0x1f43110eaca8c720)
    },
    {   UINT64_C(0x0000000000017897), UINT64_C(0x1462f70f2496c6af), UINT64_C(0x1de2f70f18bcc6af),
        UINT64_C(0x1682f70f68a026ae), UINT64_C(0x1682f70d609f26af)
    },
    {   UINT64_C(0x000000000000679b), UINT64_C(0x1f43020f3cb1e6fd), UINT64_C(0x1f43164eb4ada720),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000017898), UINT64_C(0x1852f70e189b86a6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000679c), UINT64_C(0x1f42f98ea4a88739), UINT64_C(0x1f42f6cf34b2a72f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4ee0b22714)
    },
    {   UINT64_C(0x0000000000017899), UINT64_C(0x12c2f9cf7883c5c1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001519a), UINT64_C(0x0c0af74f6c75c5c1), UINT64_C(0x0c13068f7076665f),
        UINT64_C(0x1033024f6485a5b6), UINT64_C(0x0e730e4f6884a5b6)
    },
    {   UINT64_C(0x000000000001789a), UINT64_C(0x12c305cfa076c5c1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ab4), UINT64_C(0x1f43140eecafc704), UINT64_C(0x1f42f5cf58b6671f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ced4aea742)
    },
    {   UINT64_C(0x000000000001789b), UINT64_C(0x12c2fb8fa080e5c1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000679f), UINT64_C(0x1f4305cf90b80704), UINT64_C(0x1f430c4f24ae4729),
        UINT64_C(0x1f43054f4cb22726), UINT64_C(0x1f42fc4f10b5872a)
    },
    {   UINT64_C(0x000000000001789c), UINT64_C(0x12c2f5cc3c82e5b7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a0), UINT64_C(0x1f42fa8ed4ace738), UINT64_C(0x1f42fb0e70b5072d),
        UINT64_C(0x1f42fb8e70b00725), UINT64_C(0x1f42fa8f38ad0736)
    },
    {   UINT64_C(0x000000000001789d), UINT64_C(0x12c2f58c3c8505aa), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4f48a96739), UINT64_C(0x1f42fd4ebcabc739)
    },
    {   UINT64_C(0x000000000001789e), UINT64_C(0x12c2f44c3884a5aa), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005abc), UINT64_C(0x1f42f8ced8b46726), UINT64_C(0x1f430c8edcb06721),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf28b4a71b)
    },
    {   UINT64_C(0x000000000001789f), UINT64_C(0x12c2fbcf7882a686), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005abe), UINT64_C(0x1f42f6ce50b56724), UINT64_C(0x1f42fcceecae672a),
        UINT64_C(0x1f42f78f28b0c708), UINT64_C(0x1f42fb8f2cb2c714)
    },
    {   UINT64_C(0x00000000000178a0), UINT64_C(0x12c2f40c3c8745aa), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e78b0872d),
        UINT64_C(0x1f42f38f6cb5c70a), UINT64_C(0x1f43034ed0afc738)
    },
    {   UINT64_C(0x00000000000178a1), UINT64_C(0x12c2f84f78848686), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ac2), UINT64_C(0x1f42f94e44af2735), UINT64_C(0x1f42f88e20b68738),
        UINT64_C(0x1f42f88e98af673a), UINT64_C(0x1f42f78e6cb22707)
    },
    {   UINT64_C(0x00000000000178a2), UINT64_C(0x12c2fa4c3c81e5b7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4e94afc736),
        UINT64_C(0x1f42fa8e50afa742), UINT64_C(0x1f42fa8e30ae4742)
    },
    {   UINT64_C(0x0000000000005ac5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9ce90b60728), UINT64_C(0x1f42f88e8cafe73c)
    },
    {   UINT64_C(0x0000000000005ac6), UINT64_C(0x1f42f3ce54b10710), UINT64_C(0x1f42fa8eacaf8725),
        UINT64_C(0x1f42f78eecb22727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ac7), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4e44ace739),
        UINT64_C(0x1f42f2cedcb0e716), UINT64_C(0x1f42f58ee8b7e721)
    },
    {   UINT64_C(0x0000000000005ac8), UINT64_C(0x1f43138f70ac4709), UINT64_C(0x1f43080fecb1c70b),
        UINT64_C(0x1f42ff8ffcb326f9), UINT64_C(0x1f43020fd4b4c700)
    },
    {   UINT64_C(0x00000000000178a5), UINT64_C(0x0000000000000000), UINT64_C(0x1842f80bf49ae6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x17d2f90bf49be6ef)
    },
    {   UINT64_C(0x0000000000005aca), UINT64_C(0x1f43048f70b546f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178a6), UINT64_C(0x0000000000000000), UINT64_C(0x163afc4bf49ca6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1b32f90bf4a786ef)
    },
    {   UINT64_C(0x0000000000005acc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f4cb7c701),
        UINT64_C(0x1f42f69020b646f6), UINT64_C(0x1f43039028b1c6fa)
    },
    {   UINT64_C(0x00000000000178a7), UINT64_C(0x0000000000000000), UINT64_C(0x17baf90bf49fe6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1492f90bf49ee6ef)
    },
    {   UINT64_C(0x0000000000005ace), UINT64_C(0x1f4302d024afe70b), UINT64_C(0x1f42fdcf24b16717),
        UINT64_C(0x1f42fdcf90b546fb), UINT64_C(0x1f43028f20b1c704)
    },
    {   UINT64_C(0x00000000000178a8), UINT64_C(0x0000000000000000), UINT64_C(0x18aafc4bf4a0e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1c9af90bf4b406ef)
    },
    {   UINT64_C(0x00000000000067ac), UINT64_C(0x1f42fa4ef0b62738), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178a9), UINT64_C(0x0000000000000000), UINT64_C(0x0fb2f98bf48026ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1442ff0bf49486ef)
    },
    {   UINT64_C(0x0000000000005ad2), UINT64_C(0x1f42facec0b7a700), UINT64_C(0x1f42f94ebcb7c715),
        UINT64_C(0x1f43068fa0b4c712), UINT64_C(0x1f42f94fb8b5c70f)
    },
    {   UINT64_C(0x0000000000005ad3), UINT64_C(0x1f42fa0ec4b7a709), UINT64_C(0x1f42fe8f70b2c716),
        UINT64_C(0x1f43008eb8b7272a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067ae), UINT64_C(0x1f43038f34b5c6fe), UINT64_C(0x1f42fdcf1cb3c708),
        UINT64_C(0x1f42f8cf34b626fe), UINT64_C(0x1f42f98eccb6e73b)
    },
    {   UINT64_C(0x0000000000005ad5), UINT64_C(0x1f42fb4f3cb826fc), UINT64_C(0x1f42ffcefcb64716),
        UINT64_C(0x1f42f74eb0b8a71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ad6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f28b24722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cfa8b8a6fb)
    },
    {   UINT64_C(0x00000000000178ac), UINT64_C(0x0000000000000000), UINT64_C(0x1912fc4bf49fa6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1392f2cc64a146ef)
    },
    {   UINT64_C(0x00000000000067b0), UINT64_C(0x1f42f78e6cb66742), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88e64ab6738), UINT64_C(0x1f42fd8e9cb2673f)
    },
    {   UINT64_C(0x00000000000040b1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4334531894e651)
    },
    {   UINT64_C(0x00000000000067b1), UINT64_C(0x1f43058eb8afe719), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005adb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70e90b5e734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e0f4cab6702)
    },
    {   UINT64_C(0x000000000000f800), UINT64_C(0x2503b80f3cb0864a), UINT64_C(0x2573b810ccd786a8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005add), UINT64_C(0x1f43008f78b246fc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43160ea0ac871e), UINT64_C(0x1f42fc4f78b646fc)
    },
    {   UINT64_C(0x0000000000005ade), UINT64_C(0x1f42fdce6cb30737), UINT64_C(0x1f43178f0caa26ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303d034b3e6d2)
    },
    {   UINT64_C(0x000000000000f803), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1773020a5492471b), UINT64_C(0x1773020bf49246ce)
    },
    {   UINT64_C(0x0000000000005ae0), UINT64_C(0x1f43031094b426e7), UINT64_C(0x1f43010facb4c6db),
        UINT64_C(0x1f42fad064b786cb), UINT64_C(0x1f43009000b5c704)
    },
    {   UINT64_C(0x0000000000014804), UINT64_C(0x0a6aed123860a502), UINT64_C(0x0a6aed123860a502),
        UINT64_C(0x0fa2e412387a2502), UINT64_C(0x0fa2e412687a24f5)
    },
    {   UINT64_C(0x0000000000005ae2), UINT64_C(0x1f43008fecba0694), UINT64_C(0x1f43028eccaaa717),
        UINT64_C(0x1f42fbce6cb7272e), UINT64_C(0x1f43040e98aee70b)
    },
    {   UINT64_C(0x0000000000005ae3), UINT64_C(0x1f43029020aec6dc), UINT64_C(0x1f4300ced4b14718),
        UINT64_C(0x1f42ff8fd0b24708), UINT64_C(0x1f42fcced8b546f8)
    },
    {   UINT64_C(0x000000000000f808), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772ff0bf493e6ce)
    },
    {   UINT64_C(0x0000000000005ae5), UINT64_C(0x1f42faced8b4c715), UINT64_C(0x1f4300cf4cb00701),
        UINT64_C(0x1f43048fa8b60729), UINT64_C(0x1f42faceecb62718)
    },
    {   UINT64_C(0x000000000000f80a), UINT64_C(0x1f42f78d08b6a688), UINT64_C(0x1f42f78d08b6a688),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178b4), UINT64_C(0x0000000000000000), UINT64_C(0x1002f90bf47f06ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1472e6cbf49dc6ef)
    },
    {   UINT64_C(0x0000000000005ae8), UINT64_C(0x1f42fe0f24b06712), UINT64_C(0x1f4301cf90b0c703),
        UINT64_C(0x1f4302ceb8b20724), UINT64_C(0x1f42fd8f3cb2c722)
    },
    {   UINT64_C(0x000000000000f80d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772fecd0893c688)
    },
    {   UINT64_C(0x0000000000005aea), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8f8cb0a72b), UINT64_C(0x1f42fbce9cb1e715)
    },
    {   UINT64_C(0x00000000000178b6), UINT64_C(0x0000000000000000), UINT64_C(0x0baaf68c647b86ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x12c2e6cbf49de6ef)
    },
    {   UINT64_C(0x0000000000005aec), UINT64_C(0x1f43038ec4ae8739), UINT64_C(0x1f42f74ebcb6e726),
        UINT64_C(0x1f42f8cebcb7a726), UINT64_C(0x1f4317cf34af271d)
    },
    {   UINT64_C(0x00000000000178b7), UINT64_C(0x0000000000000000), UINT64_C(0x1372f6cbf49106ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0beaf58c647b06ef)
    },
    {   UINT64_C(0x0000000000005aee), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018fccb68706), UINT64_C(0x1f42fd4f48b40726)
    },
    {   UINT64_C(0x00000000000178b8), UINT64_C(0x0000000000000000), UINT64_C(0x1b4af6cbf4a746ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x131af6cbf48726ef)
    },
    {   UINT64_C(0x000000000000f814), UINT64_C(0x177303cd08916688), UINT64_C(0x177303cd08916688),
        UINT64_C(0x1772ff0e8c93e626), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178b9), UINT64_C(0x0000000000000000), UINT64_C(0x11daf98bf48306ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x13e2e7cbf48fa6ef)
    },
    {   UINT64_C(0x000000000000f816), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772ff0fe093e626)
    },
    {   UINT64_C(0x0000000000005af3), UINT64_C(0x1f42ff10c8b446d5), UINT64_C(0x1f42fccee4b6a6ed),
        UINT64_C(0x1f42f7ce6cb6c748), UINT64_C(0x1f42fe8e7cb6c74a)
    },
    {   UINT64_C(0x0000000000005af4), UINT64_C(0x1f43038eb8b2c729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbce7cb5a741), UINT64_C(0x1f42f88fc0b98715)
    },
    {   UINT64_C(0x0000000000005af5), UINT64_C(0x1f4315107cb5e6f2), UINT64_C(0x1f4308ceb8ae8713),
        UINT64_C(0x1f43060e3cace718), UINT64_C(0x1f42f99014b586f2)
    },
    {   UINT64_C(0x0000000000005af6), UINT64_C(0x1f431e0f68b18711), UINT64_C(0x1f43140f18b0471e),
        UINT64_C(0x1f43140e50b2673f), UINT64_C(0x1f43008ff0b7c711)
    },
    {   UINT64_C(0x0000000000005af7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4311cebcace6fe), UINT64_C(0x1f430b0e64ad272e)
    },
    {   UINT64_C(0x0000000000005af8), UINT64_C(0x1f430f4e80b00719), UINT64_C(0x1f42f74fa8b5e6ee),
        UINT64_C(0x1f42f70f38b6a6ef), UINT64_C(0x1f430cce90af0721)
    },
    {   UINT64_C(0x0000000000005af9), UINT64_C(0x1f42f8cf18b8e6d8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f08b766c3), UINT64_C(0x1f42fc0f3cb346cd)
    },
    {   UINT64_C(0x0000000000005afa), UINT64_C(0x1f42fc0f40b586d6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4f98b226fd), UINT64_C(0x1f430ecebcade736)
    },
    {   UINT64_C(0x0000000000005afb), UINT64_C(0x1f42f68f54b30710), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54f74b5a70c)
    },
    {   UINT64_C(0x000000000000f820), UINT64_C(0x1773020a5c92471b), UINT64_C(0x177304ca4893a71c),
        UINT64_C(0x1773020bf49246ce), UINT64_C(0x1773020bf89246ce)
    },
    {   UINT64_C(0x0000000000005e15), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf7caac706),
        UINT64_C(0x1f42f61030b64708), UINT64_C(0x1f42f5cf98b44719)
    },
    {   UINT64_C(0x0000000000005afe), UINT64_C(0x1f42f44f28b6e6d5), UINT64_C(0x1f42fd1020b50713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbced8b7c70d)
    },
    {   UINT64_C(0x0000000000005aff), UINT64_C(0x1f42ff4f20b846e5), UINT64_C(0x1f42f58ebcb4a702),
        UINT64_C(0x1f42f70f88b866f1), UINT64_C(0x1f42f70e60b5672a)
    },
    {   UINT64_C(0x0000000000005b00), UINT64_C(0x1f43008f1cb5e70d), UINT64_C(0x1f42f50f6cb5a70a),
        UINT64_C(0x1f43084f08ae8722), UINT64_C(0x1f42fd8f50b74711)
    },
    {   UINT64_C(0x0000000000005b01), UINT64_C(0x1f43080f64b7e709), UINT64_C(0x1f430e0e88b0a72a),
        UINT64_C(0x1f42f80f4cb4e705), UINT64_C(0x1f42fa0f60b1070b)
    },
    {   UINT64_C(0x0000000000005b02), UINT64_C(0x1f4300cf78b9472e), UINT64_C(0x1f4301cf68b526e7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43020ed4b7c732)
    },
    {   UINT64_C(0x0000000000005b03), UINT64_C(0x1f42fa4edcb226e1), UINT64_C(0x1f42f58ef8b7a6dd),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e0e9cb2a717)
    },
    {   UINT64_C(0x0000000000005b04), UINT64_C(0x1f4318d00cb54731), UINT64_C(0x1f43148f0cb52703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f74b6e6da)
    },
    {   UINT64_C(0x0000000000005b05), UINT64_C(0x1f430a8f1cb4871b), UINT64_C(0x1f43098e7cb6870d),
        UINT64_C(0x1f42fe8eecb7072e), UINT64_C(0x1f43004f08b28738)
    },
    {   UINT64_C(0x0000000000005b06), UINT64_C(0x1f430fce78b5c729), UINT64_C(0x1f43158eb4b5c71b),
        UINT64_C(0x1f43120eecb64738), UINT64_C(0x1f430b0ee0b42742)
    },
    {   UINT64_C(0x0000000000005b07), UINT64_C(0x0000000000000000), UINT64_C(0x1f43158ed8b4c729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e54b28741)
    },
    {   UINT64_C(0x000000000000f82c), UINT64_C(0x00030717ec72a6d8), UINT64_C(0x0003070b6472a3b6),
        UINT64_C(0x000306cb647283b6), UINT64_C(0x1162f18c747b669b)
    },
    {   UINT64_C(0x000000000000f82d), UINT64_C(0x08d2f892e45c6613), UINT64_C(0x08d2f8d2e45c8613),
        UINT64_C(0x0002f1d0805045c6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b0a), UINT64_C(0x1f42fe0e98b3e732), UINT64_C(0x1f430a4f58b38721),
        UINT64_C(0x1f43174f20b54721), UINT64_C(0x1f430d8ed8b94731)
    },
    {   UINT64_C(0x0000000000005e1c), UINT64_C(0x1f4308cf2ca8a6fe), UINT64_C(0x1f42f50fe8ad46c8),
        UINT64_C(0x1f42f4cf8cb806c3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b0c), UINT64_C(0x1f4303ce70b46743), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d4e70b4871d), UINT64_C(0x1f430d4e38b5c736)
    },
    {   UINT64_C(0x0000000000005b0d), UINT64_C(0x1f43088ee8baa716), UINT64_C(0x1f43384fb48806d7),
        UINT64_C(0x1f42ff0eccb726eb), UINT64_C(0x1f42fb4ee8b48716)
    },
    {   UINT64_C(0x0000000000005b0e), UINT64_C(0x1f43000efcb5272a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430bcf24ad6700)
    },
    {   UINT64_C(0x0000000000005b0f), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cef4b606fc),
        UINT64_C(0x1f43088ffcb1a6c3), UINT64_C(0x1f43028fc8b606e3)
    },
    {   UINT64_C(0x0000000000005b10), UINT64_C(0x1f430b4f4cb086e2), UINT64_C(0x1f430c4ef4afa71c),
        UINT64_C(0x1f430d4e6cb82735), UINT64_C(0x1f43000ff4b5a6ce)
    },
    {   UINT64_C(0x0000000000005b11), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118e98aea6eb), UINT64_C(0x1f430f1034ae86db)
    },
    {   UINT64_C(0x000000000000f836), UINT64_C(0x0c82fc4f6c696698), UINT64_C(0x09d2fb8fa05f067f),
        UINT64_C(0x0d4afb8fa06ce67f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f837), UINT64_C(0x0000000000000000), UINT64_C(0x114b010fa07ae78a),
        UINT64_C(0x112afecfa07ac786), UINT64_C(0x182afecfa096c6e5)
    },
    {   UINT64_C(0x0000000000005b14), UINT64_C(0x1f430b0ee0a98710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43054fc4afe6c8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b15), UINT64_C(0x1f43080f58b2670b), UINT64_C(0x1f430a5000b166e6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4f28b2a6f7)
    },
    {   UINT64_C(0x0000000000005b16), UINT64_C(0x0000000000000000), UINT64_C(0x1f430acec0b286e2),
        UINT64_C(0x1f43044ed0b046de), UINT64_C(0x1f430b4f8cb5e6cf)
    },
    {   UINT64_C(0x0000000000005e22), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40f58ab86f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b18), UINT64_C(0x1f430a4fc4b80709), UINT64_C(0x1f430d8ed4b226de),
        UINT64_C(0x1f43030fe4b0e6e1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178cd), UINT64_C(0x0000000000000000), UINT64_C(0x0fe2f90c647ee6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fcae08c648d26ef)
    },
    {   UINT64_C(0x00000000000151ce), UINT64_C(0x0f52f90c747d65b6), UINT64_C(0x10d2f90f8081c695),
        UINT64_C(0x0f7af50f7876e5b6), UINT64_C(0x0ec2f90c747fc695)
    },
    {   UINT64_C(0x0000000000005b1b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098f04aca6e5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078fa4ab06f8)
    },
    {   UINT64_C(0x0000000000005b1c), UINT64_C(0x1f43058ef0b7a702), UINT64_C(0x1f4308cfccb6a6e9),
        UINT64_C(0x1f43048fe8b606d7), UINT64_C(0x1f43098f98b706de)
    },
    {   UINT64_C(0x0000000000005b1d), UINT64_C(0x1f43058f24b186fb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028f70b3e6f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b1e), UINT64_C(0x1f430a4ee8b526e0), UINT64_C(0x1f430b5000ad26e7),
        UINT64_C(0x1f430e8f70b4270e), UINT64_C(0x1f43058eb8b26707)
    },
    {   UINT64_C(0x0000000000005b1f), UINT64_C(0x0000000000000000), UINT64_C(0x1f430d8f18b9073a),
        UINT64_C(0x1f42ff8f20b646de), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b20), UINT64_C(0x1f42f50e80b786e5), UINT64_C(0x1f43089028b266cf),
        UINT64_C(0x1f43064e7caf26f9), UINT64_C(0x1f430a4ff0b5e6db)
    },
    {   UINT64_C(0x0000000000005b21), UINT64_C(0x1f43024fd4b326de), UINT64_C(0x1f43009010b666c2),
        UINT64_C(0x1f43078f1cb5c6f7), UINT64_C(0x1f43068f94b5c70b)
    },
    {   UINT64_C(0x0000000000005b22), UINT64_C(0x1f43094fb0b52706), UINT64_C(0x1f43044f0cb746bf),
        UINT64_C(0x1f42f88ef4b5a6f9), UINT64_C(0x1f4307cf34b4a6eb)
    },
    {   UINT64_C(0x0000000000005b23), UINT64_C(0x1f42fbcf60b2a70c), UINT64_C(0x1f43089000b126db),
        UINT64_C(0x1f43054f9cb666e2), UINT64_C(0x1f430a8fb4b3c6e1)
    },
    {   UINT64_C(0x0000000000005b24), UINT64_C(0x1f43044e9cb8672f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4fa4b5c70e), UINT64_C(0x1f430d8e94b0271f)
    },
    {   UINT64_C(0x000000000000d100), UINT64_C(0x1692f04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1692fa4f5494669b), UINT64_C(0x1692f18fa094c68c)
    },
    {   UINT64_C(0x0000000000005b26), UINT64_C(0x1f430d5038b3e6da), UINT64_C(0x1f430c4e54b48723),
        UINT64_C(0x1f43094f9cb346cf), UINT64_C(0x1f430f4e6cb2871a)
    },
    {   UINT64_C(0x000000000000d102), UINT64_C(0x0c2af30fa06cc68c), UINT64_C(0x0fa2eece207a468c),
        UINT64_C(0x1852f58fa09ea68c), UINT64_C(0x14daf2cfa08e468c)
    },
    {   UINT64_C(0x0000000000005b28), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4fa4b5c6eb),
        UINT64_C(0x1f43004fe4b286de), UINT64_C(0x1f43088ed8ae871d)
    },
    {   UINT64_C(0x000000000000d104), UINT64_C(0x1852f6cce09b669b), UINT64_C(0x1692f48fa098068c),
        UINT64_C(0x1162f6cf547ea69c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x0000000000005b2a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088f98a90701),
        UINT64_C(0x1f430b8fe4a78703), UINT64_C(0x1f43064e7cb7871b)
    },
    {   UINT64_C(0x000000000000d106), UINT64_C(0x1692f1cfa095e68c), UINT64_C(0x14daf50fa08dc68c),
        UINT64_C(0x0fa2f44f687b85c1), UINT64_C(0x1162f24f687fa68c)
    },
    {   UINT64_C(0x0000000000005b2c), UINT64_C(0x1f430ccfc4ad26cf), UINT64_C(0x1f43078fb4abc705),
        UINT64_C(0x1f43068fc4b24705), UINT64_C(0x1f430f4eecb466e3)
    },
    {   UINT64_C(0x0000000000005b2d), UINT64_C(0x1f4303ce74b78715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078f94aae6bf), UINT64_C(0x1f430b8f6cb0e700)
    },
    {   UINT64_C(0x0000000000005b2e), UINT64_C(0x1f43078e8cb00733), UINT64_C(0x1f430a4f8cae6702),
        UINT64_C(0x1f43038fc0ad66d3), UINT64_C(0x1f430a4f98ac06fd)
    },
    {   UINT64_C(0x000000000000d10a), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x1162f2cc6c8005c1),
        UINT64_C(0x1162f68c6c8185c1), UINT64_C(0x0de2f50fa074c5c1)
    },
    {   UINT64_C(0x000000000000d10b), UINT64_C(0x0c2af44f686ba5c1), UINT64_C(0x0a6af2cf7068065e),
        UINT64_C(0x1162f20f6881c5b5), UINT64_C(0x0fa2f34f687b25b5)
    },
    {   UINT64_C(0x000000000000d10c), UINT64_C(0x1692f3cf6896e5b5), UINT64_C(0x0fa2f10fa07b05b5),
        UINT64_C(0x0fa2f20c6c7ac5b5), UINT64_C(0x0de2f34fa07305b5)
    },
    {   UINT64_C(0x0000000000005b32), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307ce7cb64737),
        UINT64_C(0x1f430f502ca98700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b33), UINT64_C(0x1f430b9038b3c6d1), UINT64_C(0x1f43044e7cb88703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4fa0a8a6e9)
    },
    {   UINT64_C(0x0000000000005b34), UINT64_C(0x1f43010ff0adc6d0), UINT64_C(0x1f43058f90ae66d8),
        UINT64_C(0x1f43084fe0b626dc), UINT64_C(0x1f43038ee8ae66cc)
    },
    {   UINT64_C(0x0000000000005b35), UINT64_C(0x1f43044f38af26f9), UINT64_C(0x1f430e0ff0b40725),
        UINT64_C(0x1f43048e4cada6f0), UINT64_C(0x1f43059018aec6d4)
    },
    {   UINT64_C(0x0000000000005b36), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4eb8b1e6e6),
        UINT64_C(0x1f43058fdcb466c0), UINT64_C(0x1f4307900cb646f8)
    },
    {   UINT64_C(0x0000000000005b37), UINT64_C(0x1f430d4fc0b1a6df), UINT64_C(0x1f43060e6caf4709),
        UINT64_C(0x1f430a4f1cb48706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b38), UINT64_C(0x1f43038ed8b7a6f7), UINT64_C(0x1f43094f6cb32716),
        UINT64_C(0x1f4309cf2cb68719), UINT64_C(0x1f430c4fb0b2c6db)
    },
    {   UINT64_C(0x00000000000178dd), UINT64_C(0x0000000000000000), UINT64_C(0x11cae18bf48886ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x11daf98c6482e6ef)
    },
    {   UINT64_C(0x00000000000067e1), UINT64_C(0x1f42fcceecb646fe), UINT64_C(0x1f42fcd000b466f6),
        UINT64_C(0x1f42fe9014ad06ee), UINT64_C(0x1f42fa8ff4b646d2)
    },
    {   UINT64_C(0x0000000000005b3b), UINT64_C(0x1f43048fd8b4c6ea), UINT64_C(0x1f43098ee8b0a6e5),
        UINT64_C(0x1f43068e98aca6fd), UINT64_C(0x1f43034f44b546d9)
    },
    {   UINT64_C(0x0000000000005b3c), UINT64_C(0x1f42ff8e58b7e749), UINT64_C(0x1f43038f8cb24704),
        UINT64_C(0x1f430b4eccace6f7), UINT64_C(0x1f43040ef4b9271e)
    },
    {   UINT64_C(0x00000000000178df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18c2e40bf49fa6ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b3e), UINT64_C(0x1f430a4f88a6a71e), UINT64_C(0x1f430a4fc0b6a6ce),
        UINT64_C(0x1f43094f48ae26d7), UINT64_C(0x1f43088f58aca6da)
    },
    {   UINT64_C(0x0000000000005e36), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6cf34b7471f),
        UINT64_C(0x1f42f40e5cb2c733), UINT64_C(0x1f430b8e60ad472c)
    },
    {   UINT64_C(0x0000000000005b40), UINT64_C(0x1f43050ef0b066ef), UINT64_C(0x1f43078f58aa8706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cf84b066ee)
    },
    {   UINT64_C(0x0000000000005b41), UINT64_C(0x1f43040ffcad06d4), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f98a786c5)
    },
    {   UINT64_C(0x0000000000005b42), UINT64_C(0x1f43060f90acc703), UINT64_C(0x1f4302d014aa66ed),
        UINT64_C(0x1f43070ef8b0e6f4), UINT64_C(0x1f43114e4cb2c72e)
    },
    {   UINT64_C(0x0000000000005b43), UINT64_C(0x1f43038ef4a8e6f1), UINT64_C(0x1f430a4f90abe6d6),
        UINT64_C(0x1f43078fb8a2a704), UINT64_C(0x1f43078fb8af46c4)
    },
    {   UINT64_C(0x0000000000005b44), UINT64_C(0x1f43040f20b226d5), UINT64_C(0x1f43008fc8b826df),
        UINT64_C(0x1f43065000ae06e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000040e7), UINT64_C(0x1f42fad080b266f1), UINT64_C(0x1f430ed080ace6f1),
        UINT64_C(0x1f430b5088a966ed), UINT64_C(0x1f43151050a6a6f1)
    },
    {   UINT64_C(0x0000000000005b46), UINT64_C(0x1f43078ed8a9e6fe), UINT64_C(0x1f43004e7cb6a724),
        UINT64_C(0x1f4305ce3cb84712), UINT64_C(0x1f43090f68b7a71c)
    },
    {   UINT64_C(0x00000000000040e8), UINT64_C(0x1f4316d080a4c6f1), UINT64_C(0x1f431a5080a2e6f1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a5080ab06f1)
    },
    {   UINT64_C(0x0000000000005b48), UINT64_C(0x1f43068e4caf2704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058f64b6e6cb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000040e9), UINT64_C(0x1f430e5080abc6f1), UINT64_C(0x1f43141080a886f1),
        UINT64_C(0x1f43139080a8c6f1), UINT64_C(0x1f430bd080ac66f1)
    },
    {   UINT64_C(0x0000000000005b4a), UINT64_C(0x1f43058fa0b7c703), UINT64_C(0x1f43104e50b1c710),
        UINT64_C(0x1f430ccf9cb44702), UINT64_C(0x1f43019000b626df)
    },
    {   UINT64_C(0x00000000000040ea), UINT64_C(0x1f42f85080b666f1), UINT64_C(0x1f430f1080a906f1),
        UINT64_C(0x1f43474ffc8ee6ee), UINT64_C(0x1f43198fcca5c70b)
    },
    {   UINT64_C(0x0000000000005b4c), UINT64_C(0x1f430c0fa4b0c6c7), UINT64_C(0x1f4308ce5cb3e6c1),
        UINT64_C(0x1f43094f28aa06ff), UINT64_C(0x1f43040fccafe6eb)
    },
    {   UINT64_C(0x00000000000040eb), UINT64_C(0x1f43230fa0a106b7), UINT64_C(0x1f433e4fa093669c),
        UINT64_C(0x1f432d0da49c06d3), UINT64_C(0x1f4349cfa08da6cf)
    },
    {   UINT64_C(0x0000000000005b4e), UINT64_C(0x1f43038f58b58703), UINT64_C(0x1f430b0fccb146b8),
        UINT64_C(0x1f43074f3ca7c710), UINT64_C(0x1f430d5050b3e70f)
    },
    {   UINT64_C(0x00000000000040ec), UINT64_C(0x1f432c0fa09c8748), UINT64_C(0x1f431a5050a7c5f4),
        UINT64_C(0x1f43104e5ca5e6e6), UINT64_C(0x1f43360e2c9dc5fa)
    },
    {   UINT64_C(0x00000000000151e9), UINT64_C(0x14fb04cfa0a2669b), UINT64_C(0x1a1b000fa0a6e68c),
        UINT64_C(0x1722f30fa0a6a68c), UINT64_C(0x18bb0d4fa0af269b)
    },
    {   UINT64_C(0x00000000000040ed), UINT64_C(0x1f432390509f26e6), UINT64_C(0x1f4339105097c5f1),
        UINT64_C(0x1f432a8e449bc721), UINT64_C(0x1f4322ce449ce5f5)
    },
    {   UINT64_C(0x000000000000a9e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16f2f28f6896068c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000040ee), UINT64_C(0x1f432c90509b26e6), UINT64_C(0x1f4346d05090461c),
        UINT64_C(0x1f4324d044a125f9), UINT64_C(0x1f4324d0509b86e5)
    },
    {   UINT64_C(0x00000000000151eb), UINT64_C(0x106b030c7085e5c1), UINT64_C(0x0fdaf58f6882e69c),
        UINT64_C(0x0fc2f94f687e05c1), UINT64_C(0x0f02fa4c6c81e69c)
    },
    {   UINT64_C(0x0000000000005b55), UINT64_C(0x1f4307cf0cad86e3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b56), UINT64_C(0x1f43074f88b066d7), UINT64_C(0x1f4308ce7cb6c73b),
        UINT64_C(0x1f430b4ef8b6e71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b57), UINT64_C(0x1f43058fb4b36703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a4f7cb206c3), UINT64_C(0x1f43068fdcb1c6cf)
    },
    {   UINT64_C(0x0000000000005b58), UINT64_C(0x1f43088e4caa470b), UINT64_C(0x1f43060e88b1e72e),
        UINT64_C(0x1f430c4f9cae86d7), UINT64_C(0x1f43094f7cb2071a)
    },
    {   UINT64_C(0x0000000000014840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x164af6cf68968695), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b5a), UINT64_C(0x1f43068f1cb346eb), UINT64_C(0x1f430a4f58b5870a),
        UINT64_C(0x1f430a8ed8b6e736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014841), UINT64_C(0x0000000000000000), UINT64_C(0x1a7afb0f68a1e693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b5c), UINT64_C(0x1f43038e68ad46e5), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014842), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x199af4cc209ee5b6), UINT64_C(0x249afe4f4ccec6a3)
    },
    {   UINT64_C(0x0000000000005b5e), UINT64_C(0x1f43098f24b52711), UINT64_C(0x1f430c4f94b72713),
        UINT64_C(0x1f4302ced4b1a6e6), UINT64_C(0x1f4307cf50a9e6bc)
    },
    {   UINT64_C(0x0000000000005b5f), UINT64_C(0x1f43068f30b30718), UINT64_C(0x1f430b0f20b4c711),
        UINT64_C(0x1f43098f1cabe6fa), UINT64_C(0x1f430c4f58a606d4)
    },
    {   UINT64_C(0x00000000000151f1), UINT64_C(0x0ea2f94f687945c1), UINT64_C(0x121afb8f6886e69c),
        UINT64_C(0x1432eb8f7891a5c1), UINT64_C(0x1532fa0c6c90469c)
    },
    {   UINT64_C(0x0000000000005b61), UINT64_C(0x1f430a4e30b3e728), UINT64_C(0x1f430b4f5cb166cc),
        UINT64_C(0x1f43038eacb526cd), UINT64_C(0x1f430d0f84b146d8)
    },
    {   UINT64_C(0x0000000000005b62), UINT64_C(0x1f430e4ee0a86712), UINT64_C(0x1f430f4e60ae8724),
        UINT64_C(0x1f4307cfd0b166d4), UINT64_C(0x1f43088fecae66ef)
    },
    {   UINT64_C(0x0000000000005b63), UINT64_C(0x1f43074eb0a586f6), UINT64_C(0x1f43094edcae26fb),
        UINT64_C(0x1f43059008b746f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b64), UINT64_C(0x1f43028f1cabe6eb), UINT64_C(0x1f43094eacb82716),
        UINT64_C(0x1f43088f5cb266df), UINT64_C(0x1f430d4f0ca4e6fc)
    },
    {   UINT64_C(0x0000000000005b65), UINT64_C(0x1f42fbce80b6072e), UINT64_C(0x1f43094e88b0473e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cf80b026ff)
    },
    {   UINT64_C(0x0000000000005b66), UINT64_C(0x1f43068fa4ac06ec), UINT64_C(0x1f4303cfc0b0c6f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040ec4aea720)
    },
    {   UINT64_C(0x0000000000005b67), UINT64_C(0x1f43070fe0af0737), UINT64_C(0x1f430a4f38af871e),
        UINT64_C(0x1f43068ef8b44712), UINT64_C(0x1f43018f74b24708)
    },
    {   UINT64_C(0x0000000000005b68), UINT64_C(0x1f43064f3cab86d7), UINT64_C(0x1f430d4fccaaa6f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4fe0b2a70a)
    },
    {   UINT64_C(0x0000000000005b69), UINT64_C(0x1f43098edcb36719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078f2ca9e6d0), UINT64_C(0x1f43084f4ca8e6d3)
    },
    {   UINT64_C(0x0000000000005b6a), UINT64_C(0x1f43038ff0b626ea), UINT64_C(0x1f43084fc4ade6d7),
        UINT64_C(0x1f43044f88b06712), UINT64_C(0x1f43124edcb006ee)
    },
    {   UINT64_C(0x0000000000005b6b), UINT64_C(0x1f430f4eb8ae672a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060edcab86fc), UINT64_C(0x1f430b0f24b2c6f8)
    },
    {   UINT64_C(0x0000000000005b6c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4f74b8670b), UINT64_C(0x1f430a4fa4ade6fe)
    },
    {   UINT64_C(0x0000000000005b6d), UINT64_C(0x1f430d0fa0aea6f2), UINT64_C(0x1f4306cfe8ae06d6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068ef8b2470e)
    },
    {   UINT64_C(0x0000000000005b6e), UINT64_C(0x1f430b4fe8aae6da), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098fe0aa26d8), UINT64_C(0x1f43088f28b4e70f)
    },
    {   UINT64_C(0x000000000001484b), UINT64_C(0x2122fbcf4cb9a6a3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x16b2f6cf688f85b6)
    },
    {   UINT64_C(0x0000000000005b70), UINT64_C(0x1f430b4f60b2070d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e0f14b166f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001484c), UINT64_C(0x19d305cf4ca166a3), UINT64_C(0x2092fe4f4cbea6a3),
        UINT64_C(0x0000000000000000), UINT64_C(0x28a2fd0f4cda26a3)
    },
    {   UINT64_C(0x0000000000005b72), UINT64_C(0x1f43038fccade6e2), UINT64_C(0x1f43094f68b166e0),
        UINT64_C(0x1f43124ff4aae70a), UINT64_C(0x1f42ff8ef8b8a711)
    },
    {   UINT64_C(0x000000000000d14e), UINT64_C(0x189afa8fa09aa68c), UINT64_C(0x16cb00cfa092268c),
        UINT64_C(0x0000000000000000), UINT64_C(0x176300cfa096668c)
    },
    {   UINT64_C(0x000000000000f898), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af48f787c858e)
    },
    {   UINT64_C(0x000000000000f899), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105b09cfa0730591)
    },
    {   UINT64_C(0x000000000000d151), UINT64_C(0x1c9300cfa0a7668c), UINT64_C(0x0000000000000000),
        UINT64_C(0x189af94f549b469b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f89b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af5cc887a4590)
    },
    {   UINT64_C(0x000000000000d153), UINT64_C(0x1712ff8f5492068c), UINT64_C(0x139af24fa096868c),
        UINT64_C(0x1f22f24fa0bd668c), UINT64_C(0x18d2f30fa09ea68c)
    },
    {   UINT64_C(0x000000000000f89d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105afa4c8879c58d)
    },
    {   UINT64_C(0x000000000000f89e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af34f807d464e)
    },
    {   UINT64_C(0x000000000000f89f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af00c887de58d)
    },
    {   UINT64_C(0x000000000000f8a0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af5cf707c2646)
    },
    {   UINT64_C(0x000000000000f8a1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af44c787ce58d)
    },
    {   UINT64_C(0x0000000000005b7e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4eb8b7a73c),
        UINT64_C(0x1f430c4ee8b2072a), UINT64_C(0x1f43058f6cba8715)
    },
    {   UINT64_C(0x000000000000f8a3), UINT64_C(0x0000000000000000), UINT64_C(0x00030517307cc6c6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a2aedcfa0674497)
    },
    {   UINT64_C(0x000000000000d15b), UINT64_C(0x1ac2fa8f3ca326a5), UINT64_C(0x1a8afa8f54a1c6a5),
        UINT64_C(0x182afb8f5499e6a5), UINT64_C(0x1bdafb8f54a706a5)
    },
    {   UINT64_C(0x0000000000005b81), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4e70b54748),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43088ed8ae471e)
    },
    {   UINT64_C(0x000000000000d15d), UINT64_C(0x18b2fb8f3c9a66a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1b02f88f3ca446a5), UINT64_C(0x15dafacf3c8f26a5)
    },
    {   UINT64_C(0x000000000000d15e), UINT64_C(0x236afa8f3cc5a6a5), UINT64_C(0x1af2fa8f3ca3c6a5),
        UINT64_C(0x1c6afb8f54a946a5), UINT64_C(0x1a92fa8c6ca1e6a5)
    },
    {   UINT64_C(0x0000000000005b84), UINT64_C(0x1f4308ced8b12709), UINT64_C(0x1f43078f84b066e4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cf88b0272a)
    },
    {   UINT64_C(0x000000000000d160), UINT64_C(0x1ae2fb8f3ca3a6a5), UINT64_C(0x1abafa8f54a286a5),
        UINT64_C(0x230afa8f54c3c6a5), UINT64_C(0x199afa8f3c9f66a5)
    },
    {   UINT64_C(0x000000000000d161), UINT64_C(0x1a2afa8c6ca046a5), UINT64_C(0x179af90c949666a5),
        UINT64_C(0x12c2fbcf4082a5c3), UINT64_C(0x117af94f407d869f)
    },
    {   UINT64_C(0x000000000000d162), UINT64_C(0x0e82fbcf407205c3), UINT64_C(0x116afa0f3c7d469e),
        UINT64_C(0x0ee2fbcf4073e5c3), UINT64_C(0x0b92f64c486a86a4)
    },
    {   UINT64_C(0x000000000000d163), UINT64_C(0x11b2fbcc347dc5c3), UINT64_C(0x1202f94c347f069f),
        UINT64_C(0x0d6af6cf406de6a1), UINT64_C(0x0c2af80c3468a6a1)
    },
    {   UINT64_C(0x0000000000004109), UINT64_C(0x1f432c90c89c468e), UINT64_C(0x1f432c50c89d668e),
        UINT64_C(0x1f43270f689e468e), UINT64_C(0x1f4328d0c897c68e)
    },
    {   UINT64_C(0x000000000000d165), UINT64_C(0x1152f94f407ce5c3), UINT64_C(0x1402f70c3487a639),
        UINT64_C(0x11f2fbcc347fc5c3), UINT64_C(0x106af80f407c25c3)
    },
    {   UINT64_C(0x000000000000d166), UINT64_C(0x116afb0f247da66b), UINT64_C(0x0db2f94f446dc678),
        UINT64_C(0x154af6cf408d25c3), UINT64_C(0x11eafbcf40804639)
    },
    {   UINT64_C(0x000000000000d167), UINT64_C(0x1a92fbcf40a1e639), UINT64_C(0x104afa8c5c7ba5c3),
        UINT64_C(0x12a2fbcc34818639), UINT64_C(0x0f22f70c347525c3)
    },
    {   UINT64_C(0x000000000000410b), UINT64_C(0x1f432c90c499068e), UINT64_C(0x1f4330109c94e698),
        UINT64_C(0x1f4311109ca18698), UINT64_C(0x1f432690a09a868e)
    },
    {   UINT64_C(0x0000000000005b8e), UINT64_C(0x1f4306cf10b4c71f), UINT64_C(0x1f430b8f14ad671c),
        UINT64_C(0x1f43048efcb7c711), UINT64_C(0x1f43058eb0ad6706)
    },
    {   UINT64_C(0x000000000000410c), UINT64_C(0x1f4337d09c96a5cc), UINT64_C(0x1f4337d09c96c68e),
        UINT64_C(0x1f433810c49685cc), UINT64_C(0x1f433810c89685cd)
    },
    {   UINT64_C(0x0000000000005b90), UINT64_C(0x1f43060f64b786f6), UINT64_C(0x1f42fd0f2cb446ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbce8cb406ec)
    },
    {   UINT64_C(0x0000000000005b91), UINT64_C(0x1f43030f5ca38702), UINT64_C(0x1f43070e6cb4c725),
        UINT64_C(0x1f430c8ea0b0071a), UINT64_C(0x1f430c8e44afe717)
    },
    {   UINT64_C(0x0000000000005b92), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058efcafa6f3),
        UINT64_C(0x1f43078f08b8e738), UINT64_C(0x1f430b4e2cb1a738)
    },
    {   UINT64_C(0x0000000000005b93), UINT64_C(0x0000000000000000), UINT64_C(0x1f43084eb8ace70b),
        UINT64_C(0x1f43008ffcace6c2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b94), UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ceb4b16710),
        UINT64_C(0x1f43098e1cb36728), UINT64_C(0x1f43068fb8b7a70d)
    },
    {   UINT64_C(0x0000000000005b95), UINT64_C(0x1f430c4ea4b00703), UINT64_C(0x1f43064f24af8702),
        UINT64_C(0x1f43084f30af072f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b96), UINT64_C(0x1f43028f0cac6708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecf48b746fe), UINT64_C(0x1f430d4f10b4673d)
    },
    {   UINT64_C(0x0000000000005b97), UINT64_C(0x1f43094f70b306ec), UINT64_C(0x1f43094ef4ab471d),
        UINT64_C(0x1f430c8f10a3670e), UINT64_C(0x1f43094f44b2a71c)
    },
    {   UINT64_C(0x0000000000005b98), UINT64_C(0x1f430a4fa4b24711), UINT64_C(0x1f43098f98aca6d7),
        UINT64_C(0x1f430f0f50af46e4), UINT64_C(0x1f43058ee0af66f1)
    },
    {   UINT64_C(0x0000000000005b99), UINT64_C(0x1f43029028b0c6f4), UINT64_C(0x1f430acfb8ade715),
        UINT64_C(0x1f43078fa4ae66f3), UINT64_C(0x1f430c4e4caaa735)
    },
    {   UINT64_C(0x0000000000005b9a), UINT64_C(0x1f430b4fa0ae46ef), UINT64_C(0x1f430d0f90ab8704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034fa8b6c6dd)
    },
    {   UINT64_C(0x000000000000f8bf), UINT64_C(0x0000000000000000), UINT64_C(0x1692f04bf494a6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x14daf20bf48be6ef)
    },
    {   UINT64_C(0x000000000000d177), UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x0f0af1ce2071468c),
        UINT64_C(0x1643020fa096e68c), UINT64_C(0x12db020fa087e68c)
    },
    {   UINT64_C(0x000000000000f8c1), UINT64_C(0x0000000000000000), UINT64_C(0x14daf20bf48ea6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x14daf50bf48dc6ef)
    },
    {   UINT64_C(0x0000000000005b9e), UINT64_C(0x1f43084f70b2a722), UINT64_C(0x1f43088f38b0c6d7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d17a), UINT64_C(0x15db00cf548c868c), UINT64_C(0x1592f44f5891a68c),
        UINT64_C(0x1f2af58f64b7668c), UINT64_C(0x1722f80fa096068c)
    },
    {   UINT64_C(0x0000000000005ba0), UINT64_C(0x1f4303cef0b0a715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43094ef8ade70f), UINT64_C(0x1f43098eccb00724)
    },
    {   UINT64_C(0x000000000000f8c5), UINT64_C(0x0000000000000000), UINT64_C(0x1692f20bf49626ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x143af80c648a66ef)
    },
    {   UINT64_C(0x000000000000f8c6), UINT64_C(0x0000000000000000), UINT64_C(0x1852f6cbf49b66ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1852f34c649d66ef)
    },
    {   UINT64_C(0x0000000000005ba3), UINT64_C(0x1f42fb4f04b4e71d), UINT64_C(0x1f43080ed0ae06e9),
        UINT64_C(0x1f42ffcf68b2e6e3), UINT64_C(0x1f4303cefcb1e719)
    },
    {   UINT64_C(0x0000000000005ba4), UINT64_C(0x1f43098f0caee6f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058ee0b48712), UINT64_C(0x1f43094e7cb6872e)
    },
    {   UINT64_C(0x000000000000d180), UINT64_C(0x0c0af44f6c6b25c1), UINT64_C(0x0c12efcf706ce65e),
        UINT64_C(0x1032fbcf647865b5), UINT64_C(0x0e72f1cf687665b5)
    },
    {   UINT64_C(0x0000000000005ba6), UINT64_C(0x1f43098f3cb0471e), UINT64_C(0x1f43048e98b6870a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068f04b62706)
    },
    {   UINT64_C(0x0000000000005ba7), UINT64_C(0x1f43048f40abe708), UINT64_C(0x1f43084ea0b5a747),
        UINT64_C(0x1f43088f44b6e6e5), UINT64_C(0x1f43024f04ae86ef)
    },
    {   UINT64_C(0x0000000000005ba8), UINT64_C(0x1f43078f1cb2c71d), UINT64_C(0x1f43068ee4b106f9),
        UINT64_C(0x1f43079044ae6702), UINT64_C(0x1f4303cf7cb786d4)
    },
    {   UINT64_C(0x0000000000005ba9), UINT64_C(0x1f43098f04aea6e8), UINT64_C(0x1f430a4e80b38734),
        UINT64_C(0x1f4302cf88b6c6fb), UINT64_C(0x1f43034f24b5e716)
    },
    {   UINT64_C(0x0000000000005baa), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43080f64b36734), UINT64_C(0x1f43059044b226e6)
    },
    {   UINT64_C(0x0000000000005bab), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078ec8adc6f3), UINT64_C(0x1f43048f40aca709)
    },
    {   UINT64_C(0x0000000000005bac), UINT64_C(0x1f43098f4cac6700), UINT64_C(0x1f43020eccb34730),
        UINT64_C(0x1f43028edcb66729), UINT64_C(0x1f430c8f2ca4c6f9)
    },
    {   UINT64_C(0x0000000000005bad), UINT64_C(0x1f43088ee8b66739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078f1cb8c719), UINT64_C(0x1f43008f0ca866d8)
    },
    {   UINT64_C(0x0000000000005bae), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098e60b1c723),
        UINT64_C(0x1f43090f58aba6f5), UINT64_C(0x1f43088f68a6c707)
    },
    {   UINT64_C(0x0000000000005baf), UINT64_C(0x1f430a4f80af4701), UINT64_C(0x1f43028ee0aee708),
        UINT64_C(0x1f43078f20b5a70d), UINT64_C(0x1f4302cea8b306fe)
    },
    {   UINT64_C(0x0000000000005bb0), UINT64_C(0x1f43084ee4aaa712), UINT64_C(0x1f43058ef4b546c6),
        UINT64_C(0x1f43028e94afc738), UINT64_C(0x1f43078e94b1e705)
    },
    {   UINT64_C(0x0000000000005bb1), UINT64_C(0x1f43044ea0b7a732), UINT64_C(0x1f430a4f08b6c728),
        UINT64_C(0x1f43078f54ada6df), UINT64_C(0x1f4305ceb4afc725)
    },
    {   UINT64_C(0x0000000000005bb2), UINT64_C(0x1f43028f3caa270b), UINT64_C(0x1f4307cf10b506fc),
        UINT64_C(0x1f43054e74b26712), UINT64_C(0x1f43071020a9a6ff)
    },
    {   UINT64_C(0x0000000000005bb3), UINT64_C(0x1f43038f20b7c71b), UINT64_C(0x1f43088f64ad26fa),
        UINT64_C(0x1f43064e70ad6701), UINT64_C(0x1f43058f38b306dd)
    },
    {   UINT64_C(0x0000000000005bb4), UINT64_C(0x1f43074ee0ab06e9), UINT64_C(0x1f430a4e9cb44725),
        UINT64_C(0x1f43078f14afe6dc), UINT64_C(0x1f43084e5caf0735)
    },
    {   UINT64_C(0x0000000000005bb5), UINT64_C(0x1f43064fa0ad2719), UINT64_C(0x1f42f64e90b82731),
        UINT64_C(0x1f43054ef4b1a71a), UINT64_C(0x1f430b4fe0b2070f)
    },
    {   UINT64_C(0x000000000000f8da), UINT64_C(0x0000000000000000), UINT64_C(0x14daeacbf49246ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bb7), UINT64_C(0x1f43084ebcb6471c), UINT64_C(0x1f43094eb4ade713),
        UINT64_C(0x1f43038eacae470f), UINT64_C(0x1f4306cf48b4a6f3)
    },
    {   UINT64_C(0x0000000000005bb8), UINT64_C(0x1f43068ff0b566f8), UINT64_C(0x1f43098f24ad0703),
        UINT64_C(0x1f430a4fe0ac8713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bb9), UINT64_C(0x1f430b4fe0b08702), UINT64_C(0x1f43079018aec6fb),
        UINT64_C(0x1f430a4f08b24713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bba), UINT64_C(0x1f43048fa0ade71c), UINT64_C(0x1f4307cf74b206f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcce68b6e710)
    },
    {   UINT64_C(0x0000000000005bbb), UINT64_C(0x1f43090e8cb6872a), UINT64_C(0x1f43058ed8af66f0),
        UINT64_C(0x1f43010f88b3e706), UINT64_C(0x1f43018e9cb3073c)
    },
    {   UINT64_C(0x0000000000005bbc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f4cb20706),
        UINT64_C(0x1f430bce90b7074e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bbd), UINT64_C(0x1f430c0eeca7e722), UINT64_C(0x1f43058f2cac06fb),
        UINT64_C(0x1f42fb4f48b8070c), UINT64_C(0x1f43058e90a86709)
    },
    {   UINT64_C(0x0000000000005bbe), UINT64_C(0x1f4301ce6cb86715), UINT64_C(0x1f4305cee8ab0725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040f60ac6715)
    },
    {   UINT64_C(0x0000000000005bbf), UINT64_C(0x1f43014f04b3872c), UINT64_C(0x1f42ffced4ae273d),
        UINT64_C(0x1f430a8f34b7e739), UINT64_C(0x1f43038ea8ade72a)
    },
    {   UINT64_C(0x0000000000005bc0), UINT64_C(0x1f4301ce5cae8712), UINT64_C(0x1f430b8f68b34725),
        UINT64_C(0x1f430b0ed0ae8723), UINT64_C(0x1f43014f48b5270a)
    },
    {   UINT64_C(0x0000000000005bc1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038e6cb36728),
        UINT64_C(0x1f43030e80b84724), UINT64_C(0x1f42fccf34b5c6f5)
    },
    {   UINT64_C(0x0000000000005bc2), UINT64_C(0x1f43008f24b58700), UINT64_C(0x1f43018f54b1e71b),
        UINT64_C(0x1f43050ee4b1c711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bc3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048fc4a9c6d0),
        UINT64_C(0x1f42ff8f64b546e6), UINT64_C(0x1f430bcf6cb04700)
    },
    {   UINT64_C(0x0000000000005bc4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74ff0b706e1),
        UINT64_C(0x1f42ff8f7cae46ec), UINT64_C(0x1f43068e68aa4729)
    },
    {   UINT64_C(0x0000000000005bc5), UINT64_C(0x1f430d4f40b26731), UINT64_C(0x1f43078eb0ab473e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bc6), UINT64_C(0x1f4301cec0b3670b), UINT64_C(0x1f43070ee4b2c728),
        UINT64_C(0x1f43030efcabe6cc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bc7), UINT64_C(0x1f43000f44b5e723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43068f28b34724), UINT64_C(0x1f43058fa8ad870b)
    },
    {   UINT64_C(0x0000000000005bc8), UINT64_C(0x1f43088f48b1c734), UINT64_C(0x1f43048ed4aac71e),
        UINT64_C(0x1f43088f88b2e736), UINT64_C(0x1f43048ee8b70715)
    },
    {   UINT64_C(0x000000000000f8ed), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af80f747ac691)
    },
    {   UINT64_C(0x0000000000005bca), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f48b64703),
        UINT64_C(0x1f43020e9cb2c72b), UINT64_C(0x1f43094f08b6c721)
    },
    {   UINT64_C(0x000000000000f8ef), UINT64_C(0x106afb0f74794686), UINT64_C(0x106af8cf747a6691),
        UINT64_C(0x106af8cf747a6696), UINT64_C(0x106af8cf747a6691)
    },
    {   UINT64_C(0x000000000000f8f0), UINT64_C(0x106af8cf747a6691), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772ff0bd493e6d3)
    },
    {   UINT64_C(0x000000000000f8f1), UINT64_C(0x1772ff0bd493e6d3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bce), UINT64_C(0x1f43068e7cb74739), UINT64_C(0x1f43030ea4aba70c),
        UINT64_C(0x1f43040f28b44717), UINT64_C(0x1f43014eb0b6270a)
    },
    {   UINT64_C(0x0000000000005bcf), UINT64_C(0x1f43048eb8ad6723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060e8cb70739), UINT64_C(0x1f430e8e5cb2c740)
    },
    {   UINT64_C(0x0000000000005bd0), UINT64_C(0x1f43048efcab26f6), UINT64_C(0x1f42fd0fb4af6700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42faceccb3a714)
    },
    {   UINT64_C(0x0000000000005bd1), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8f60ac4719),
        UINT64_C(0x1f43074ee0ac6716), UINT64_C(0x1f43048e7cad6721)
    },
    {   UINT64_C(0x0000000000005bd2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4305901cb6a6e6)
    },
    {   UINT64_C(0x0000000000005bd3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43030e9cb366ed),
        UINT64_C(0x1f42ff8fb8b726df), UINT64_C(0x1f43088ea8a9a713)
    },
    {   UINT64_C(0x000000000000d1af), UINT64_C(0x1692eecfa095e69c), UINT64_C(0x1a22f28fa0a4c68c),
        UINT64_C(0x175af20fa095e68c), UINT64_C(0x1902eecfa0a0a69c)
    },
    {   UINT64_C(0x000000000000d1b0), UINT64_C(0x1852f6cfa098e69c), UINT64_C(0x159af20f6890868c),
        UINT64_C(0x1422f44f688bc5c1), UINT64_C(0x1162f94c6c80069c)
    },
    {   UINT64_C(0x000000000000d1b1), UINT64_C(0x1032f10c6c7d25c1), UINT64_C(0x0fb2f48f687a269c),
        UINT64_C(0x0de2f50f687425c1), UINT64_C(0x0e5af3cc6c75269c)
    },
    {   UINT64_C(0x000000000000d1b2), UINT64_C(0x1222f10c6c82a5c1), UINT64_C(0x0faaf44f687a069c),
        UINT64_C(0x0a32f1cf686485b5), UINT64_C(0x122af34fa08465c1)
    },
    {   UINT64_C(0x000000000000d1b3), UINT64_C(0x1112f2cf5880669c), UINT64_C(0x1312f94c6c8805b5),
        UINT64_C(0x1032f1cf687c65c1), UINT64_C(0x0e8af3cc6c75669c)
    },
    {   UINT64_C(0x0000000000005bd9), UINT64_C(0x1f43078f4caa66f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4e70b8e73f), UINT64_C(0x1f43088ed4ad672d)
    },
    {   UINT64_C(0x000000000000d1b5), UINT64_C(0x11c2f44f6880a5b5), UINT64_C(0x0f62f28f6877a5b5),
        UINT64_C(0x1202f10f688365c1), UINT64_C(0x146af40c6c8d25c1)
    },
    {   UINT64_C(0x0000000000005bdb), UINT64_C(0x1f43034e6cb2e728), UINT64_C(0x1f430c0e4cb2c72e),
        UINT64_C(0x1f43038f04acc708), UINT64_C(0x1f43094e28b2e723)
    },
    {   UINT64_C(0x0000000000005bdc), UINT64_C(0x1f430a8e74b3673c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffcf6cb206ee), UINT64_C(0x1f43038ea4b02702)
    },
    {   UINT64_C(0x000000000000d1b8), UINT64_C(0x0ffaf44c6c7b45c1), UINT64_C(0x1b02f04f68a8e5b5),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bde), UINT64_C(0x1f43088e70b06718), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014ea4b3c717), UINT64_C(0x1f43018edcb6c70e)
    },
    {   UINT64_C(0x0000000000005bdf), UINT64_C(0x1f42fe8f54af0729), UINT64_C(0x1f4303cec4ae072b),
        UINT64_C(0x1f430a4edcaf8715), UINT64_C(0x1f43048f5cac86f6)
    },
    {   UINT64_C(0x0000000000005be0), UINT64_C(0x1f43038f50b246ce), UINT64_C(0x1f43020f50b186eb),
        UINT64_C(0x1f4300cfb0b5e6f3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005be1), UINT64_C(0x1f43044fc4b7c712), UINT64_C(0x1f43068f3cb0c72d),
        UINT64_C(0x1f4300ce88b40731), UINT64_C(0x1f42fe4ebcb0c740)
    },
    {   UINT64_C(0x0000000000005be2), UINT64_C(0x1f43040f80b78710), UINT64_C(0x1f43038f58b786ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306cee4a9c6fb)
    },
    {   UINT64_C(0x0000000000005be3), UINT64_C(0x1f42fbce6cb20713), UINT64_C(0x1f4300cf04b02719),
        UINT64_C(0x1f430a4e48aea73c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005be4), UINT64_C(0x1f42ffcf4caac6e9), UINT64_C(0x1f43094f3cade6d3),
        UINT64_C(0x1f43058f6cabe6d3), UINT64_C(0x1f42ff0f3cb5a70c)
    },
    {   UINT64_C(0x0000000000005be5), UINT64_C(0x1f43008f5cb286fa), UINT64_C(0x1f43048ed8ad671e),
        UINT64_C(0x1f43044ee0aca6f7), UINT64_C(0x1f43054edca926f5)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005be7), UINT64_C(0x1f430a4f40ac26ca), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cf08ab671e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005be8), UINT64_C(0x1f4306cf50b146d3), UINT64_C(0x1f43040e4cb486f6),
        UINT64_C(0x1f43094f00ad2715), UINT64_C(0x1f42fcce6cb7a720)
    },
    {   UINT64_C(0x0000000000005be9), UINT64_C(0x1f43060e5cafe71a), UINT64_C(0x1f4301ceb8b46700),
        UINT64_C(0x1f43084f70ad86df), UINT64_C(0x1f43078f78ab4717)
    },
    {   UINT64_C(0x00000000000147a0), UINT64_C(0x1bcafe0f58b40770), UINT64_C(0x14daf1cf6c8b46a1),
        UINT64_C(0x1bcafe0f58b40770), UINT64_C(0x14daf1cf6c8b46a1)
    },
    {   UINT64_C(0x00000000000147a1), UINT64_C(0x1bcafe0f58b40746), UINT64_C(0x14daf1cf6c8b4677),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bec), UINT64_C(0x1f430a4fc0b5c706), UINT64_C(0x1f42fc8f90b5a6d9),
        UINT64_C(0x1f43084f08b26721), UINT64_C(0x1f43098fa0b2e734)
    },
    {   UINT64_C(0x0000000000005bed), UINT64_C(0x1f43088ef0b286fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4f18afe72d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bee), UINT64_C(0x1f43050f40afe708), UINT64_C(0x1f4307ceb0ab0732),
        UINT64_C(0x1f43058eecb02719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bef), UINT64_C(0x1f43048f08b3c70d), UINT64_C(0x1f43064e7cb6e713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43064ed4b74705)
    },
    {   UINT64_C(0x0000000000005bf0), UINT64_C(0x1f4302cf78b62707), UINT64_C(0x1f430b0eecaa270b),
        UINT64_C(0x1f42fdce40b72732), UINT64_C(0x1f43068f40b3a74b)
    },
    {   UINT64_C(0x0000000000005bf1), UINT64_C(0x1f43050f60b7c6c9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43024e78b1673a), UINT64_C(0x1f43068ea4b16712)
    },
    {   UINT64_C(0x0000000000005bf2), UINT64_C(0x1f43038f38b3a6ea), UINT64_C(0x1f43060f68b266ea),
        UINT64_C(0x1f43068edcb1070e), UINT64_C(0x1f43044ee8b56713)
    },
    {   UINT64_C(0x0000000000005bf3), UINT64_C(0x1f43048f30b6a6e2), UINT64_C(0x1f43078f68b2471d),
        UINT64_C(0x1f4303cf50ab2707), UINT64_C(0x1f43048f0cafc6e0)
    },
    {   UINT64_C(0x0000000000005bf4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f98ad46ff),
        UINT64_C(0x1f43060f84af8709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bf5), UINT64_C(0x1f43074ef8ad26e3), UINT64_C(0x1f43048f68af26f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005bf6), UINT64_C(0x1f430b0e34b14735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a8e58b7473a)
    },
    {   UINT64_C(0x0000000000005bf7), UINT64_C(0x1f4305cf84b1271d), UINT64_C(0x1f43044ea4b4c73b),
        UINT64_C(0x1f430a4ebcb3c73c), UINT64_C(0x1f43008e6cb4a740)
    },
    {   UINT64_C(0x0000000000005bf8), UINT64_C(0x1f43038e54b52739), UINT64_C(0x1f43028ea0a9a6f3),
        UINT64_C(0x1f43070e60b1c709), UINT64_C(0x1f43028f28b9070a)
    },
    {   UINT64_C(0x0000000000005bf9), UINT64_C(0x1f430d0e70b26739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028f44ade712), UINT64_C(0x1f43008f30b22700)
    },
    {   UINT64_C(0x0000000000005bfa), UINT64_C(0x1f43038f48aea732), UINT64_C(0x1f43088e58b4c730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068ffcb3c712)
    },
    {   UINT64_C(0x000000000000d1d6), UINT64_C(0x127b020fa085a68c), UINT64_C(0x1682f80fa093869a),
        UINT64_C(0x13db020fa089268c), UINT64_C(0x146af44fa08c668c)
    },
    {   UINT64_C(0x000000000000d1d7), UINT64_C(0x165b020fa08de68c), UINT64_C(0x1852f6cf549b669b),
        UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x1643020fa096e68c)
    },
    {   UINT64_C(0x000000000000d1d8), UINT64_C(0x1612f80fa091c69a), UINT64_C(0x1c8b020fa0a6a68c),
        UINT64_C(0x16a3020f588f068c), UINT64_C(0x1402f6cfa08a268c)
    },
    {   UINT64_C(0x000000000000d1d9), UINT64_C(0x1852f6cf589b669c), UINT64_C(0x165b020fa08de68c),
        UINT64_C(0x122b020fa085a68c), UINT64_C(0x1852f6cf549b669b)
    },
    {   UINT64_C(0x0000000000005bff), UINT64_C(0x1f43040e8cb56703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43068ef8b0a731), UINT64_C(0x1f43018eb0af86f3)
    },
    {   UINT64_C(0x0000000000005c00), UINT64_C(0x1f43004e80b0870b), UINT64_C(0x1f43014e88b7a73f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d1dc), UINT64_C(0x14b2f80f688d25c1), UINT64_C(0x1002f7cc6c7b669c),
        UINT64_C(0x0fb2f08c6c7a25c1), UINT64_C(0x0fb2f44f687a269c)
    },
    {   UINT64_C(0x0000000000005c02), UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cec8aea739),
        UINT64_C(0x1f43068ee8b3c6fb), UINT64_C(0x1f43048eb4b4e731)
    },
    {   UINT64_C(0x000000000000d1de), UINT64_C(0x09fafb4f686385b5), UINT64_C(0x107afbcfa07e85c1),
        UINT64_C(0x1152f28f687f869c), UINT64_C(0x1132f7cc6c7f25b5)
    },
    {   UINT64_C(0x000000000000d1df), UINT64_C(0x0f62fbcf687985c1), UINT64_C(0x0faaf50c6c7ba69c),
        UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1292f08f688605b5)
    },
    {   UINT64_C(0x000000000000d1e0), UINT64_C(0x0ffaf78c6c7b45c1), UINT64_C(0x0f92f48c6c7b45c1),
        UINT64_C(0x1042f44f688385b5), UINT64_C(0x0efaef8f687985b5)
    },
    {   UINT64_C(0x00000000000147bc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x131affcfa090e770), UINT64_C(0x0de2d68c6c6f86a1)
    },
    {   UINT64_C(0x0000000000005c07), UINT64_C(0x1f42f9cfc0b406d6), UINT64_C(0x1f43058f54b706d3),
        UINT64_C(0x1f42f40f0cb8a71f), UINT64_C(0x1f43058e48b40745)
    },
    {   UINT64_C(0x000000000000d1e3), UINT64_C(0x11d2f18f6883c5c1), UINT64_C(0x13d2f44c6c8aa69c),
        UINT64_C(0x0f72f44c6c7925c1), UINT64_C(0x1a82ef8f68a765b5)
    },
    {   UINT64_C(0x0000000000005c09), UINT64_C(0x1f430a4e84b4472d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004f70ae66ea), UINT64_C(0x1f43008f4cb22704)
    },
    {   UINT64_C(0x0000000000005c0a), UINT64_C(0x1f42fecf48aea6ff), UINT64_C(0x1f43058f80aec6bb),
        UINT64_C(0x1f43070ee4ad2726), UINT64_C(0x1f4300cf10b6870b)
    },
    {   UINT64_C(0x0000000000005c0b), UINT64_C(0x1f43024f44b146ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cef0afe711)
    },
    {   UINT64_C(0x0000000000005c0c), UINT64_C(0x1f43044ec8b106c0), UINT64_C(0x1f430a4f44a9c6fd),
        UINT64_C(0x1f4303cfe0b72700), UINT64_C(0x1f43068efcadc723)
    },
    {   UINT64_C(0x0000000000005c0d), UINT64_C(0x1f430a4eb8b126fe), UINT64_C(0x1f43038ed0af6724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068f00acc705)
    },
    {   UINT64_C(0x0000000000005c0e), UINT64_C(0x1f43090ec4b3e712), UINT64_C(0x1f43048edcb5e6f7),
        UINT64_C(0x1f43058e8cb1a733), UINT64_C(0x1f43038ef4b406ea)
    },
    {   UINT64_C(0x0000000000005c0f), UINT64_C(0x1f4307cec0b1a731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43088f18ada6f5), UINT64_C(0x1f43038ee8b3e70b)
    },
    {   UINT64_C(0x0000000000005c10), UINT64_C(0x1f43054ec4b126f5), UINT64_C(0x1f42ffceb4ace6f7),
        UINT64_C(0x1f43094f24b1e72a), UINT64_C(0x1f430c4ec0b0472c)
    },
    {   UINT64_C(0x0000000000005c11), UINT64_C(0x1f43038e40b26718), UINT64_C(0x1f430b4f48afc717),
        UINT64_C(0x1f43094eecb1a72c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c12), UINT64_C(0x1f4308ce5cb1c729), UINT64_C(0x1f4302cebcb48709),
        UINT64_C(0x1f43028f44af8709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c13), UINT64_C(0x1f43090e6cac872e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f8cad86cd)
    },
    {   UINT64_C(0x0000000000005c14), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034ed4b30728),
        UINT64_C(0x1f430c8e80b3072d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c15), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044f84b66701),
        UINT64_C(0x1f4304ce8cb34738), UINT64_C(0x1f43010f08b4a719)
    },
    {   UINT64_C(0x0000000000005c16), UINT64_C(0x1f4300cf98b5671b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038e84b40728), UINT64_C(0x1f42fe8e54ae66fb)
    },
    {   UINT64_C(0x0000000000005c17), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f70ac46df),
        UINT64_C(0x1f43068eb8ac870e), UINT64_C(0x1f43038e88ada71f)
    },
    {   UINT64_C(0x000000000000d1f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa2fe4fa075a698)
    },
    {   UINT64_C(0x000000000000d1f4), UINT64_C(0x0fa2f24fa07a4698), UINT64_C(0x0fa2f20f68790698),
        UINT64_C(0x0fa2f2cfa07a0698), UINT64_C(0x0fa2f38f8079468c)
    },
    {   UINT64_C(0x0000000000005c1a), UINT64_C(0x1f43018ec4b4472e), UINT64_C(0x1f42fc8f38b62709),
        UINT64_C(0x1f43008e84b68739), UINT64_C(0x1f43124fdcb64717)
    },
    {   UINT64_C(0x0000000000005c1b), UINT64_C(0x1f43121380aac5ab), UINT64_C(0x1f43154fb0af66b8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43064f98b226f7)
    },
    {   UINT64_C(0x0000000000005c1c), UINT64_C(0x1f42f70f6cae6713), UINT64_C(0x1f43018f6cad26f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c1d), UINT64_C(0x1f42ff4ff8b626eb), UINT64_C(0x1f43114f4cafe6ef),
        UINT64_C(0x1f42fa8f8cb086dd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c1e), UINT64_C(0x1f430e4edcb22732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4f94b666e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c1f), UINT64_C(0x1f43018f5cab071a), UINT64_C(0x1f43160f38b4a71d),
        UINT64_C(0x1f43120f68af6717), UINT64_C(0x1f42fe8fc8ae66e0)
    },
    {   UINT64_C(0x0000000000005c20), UINT64_C(0x1f43074f00b086e4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb9040b5c6e7), UINT64_C(0x1f4311ce74b486fa)
    },
    {   UINT64_C(0x000000000000d1fc), UINT64_C(0x0fa2f2cfa07b0698), UINT64_C(0x0fa2f14f6c7ae68c),
        UINT64_C(0x0fa2f48f6c79e698), UINT64_C(0x0fa2f6cfa079668c)
    },
    {   UINT64_C(0x0000000000005c22), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034ff4aaa6ef),
        UINT64_C(0x1f43034fecb44709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c23), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c4ec8b42727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c24), UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ebcb5c70b),
        UINT64_C(0x1f42fb8e90b3e713), UINT64_C(0x1f43034ea8b32709)
    },
    {   UINT64_C(0x0000000000005c25), UINT64_C(0x1f42fa8fc0b2c6e1), UINT64_C(0x1f42f6cf5cb4070a),
        UINT64_C(0x1f42fa8f78b606fd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c26), UINT64_C(0x1f43054eacaea6f2), UINT64_C(0x1f430d8ef8b3872b),
        UINT64_C(0x1f4304cfe0b246be), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c27), UINT64_C(0x1f43004ed8b06710), UINT64_C(0x1f43044fccb6a6ea),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018fc8b3c6e0)
    },
    {   UINT64_C(0x0000000000005c28), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018fe8b186cc),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c29), UINT64_C(0x1f43070f68ada6f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c2a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f28b646f0),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4f60ae46e3)
    },
    {   UINT64_C(0x0000000000005c2b), UINT64_C(0x1f43040fe4b226ff), UINT64_C(0x1f42fc8f24b5e72f),
        UINT64_C(0x1f43054f7cb40715), UINT64_C(0x1f42fc8f54aac6f7)
    },
    {   UINT64_C(0x0000000000005c2c), UINT64_C(0x1f43158f68a72710), UINT64_C(0x1f431d8f10ae0722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f38ae4714)
    },
    {   UINT64_C(0x0000000000005c2d), UINT64_C(0x1f42fc8fa0b1c6d8), UINT64_C(0x1f42fa8f7cb5e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8fe8ae86e4)
    },
    {   UINT64_C(0x0000000000005c2e), UINT64_C(0x1f43028ec8af2739), UINT64_C(0x1f43150fd4abe70d),
        UINT64_C(0x1f43058ef8afc727), UINT64_C(0x1f42fe9034a966d4)
    },
    {   UINT64_C(0x0000000000005c2f), UINT64_C(0x1f42fe8ef8af6707), UINT64_C(0x1f42fe8fc8ac86e0),
        UINT64_C(0x1f42fe8f74b766dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c30), UINT64_C(0x1f43018f2cb206e6), UINT64_C(0x1f42fc8f40ac66e0),
        UINT64_C(0x1f42ff9038ad86f3), UINT64_C(0x1f43008ff8afc6f3)
    },
    {   UINT64_C(0x0000000000005c31), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88fc8b286e0),
        UINT64_C(0x1f42fb8ea0b406db), UINT64_C(0x1f42fd8f78b4c6ee)
    },
    {   UINT64_C(0x0000000000005c32), UINT64_C(0x1f43048f98b3c733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98e1cae871d), UINT64_C(0x1f4307cf18b166f0)
    },
    {   UINT64_C(0x0000000000005c33), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f54b626ef),
        UINT64_C(0x1f4313cf14ada6fa), UINT64_C(0x1f430c0ec0ac0746)
    },
    {   UINT64_C(0x0000000000005c34), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f84b786f7),
        UINT64_C(0x1f42fd8ebcb446fd), UINT64_C(0x1f42f98f8cb2c6fe)
    },
    {   UINT64_C(0x0000000000005c35), UINT64_C(0x1f42fd8f48ac26dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f48ae470a)
    },
    {   UINT64_C(0x0000000000005c36), UINT64_C(0x1f42f94f9cb18700), UINT64_C(0x1f42fc8f4cb8a708),
        UINT64_C(0x1f42f98f0cb8e6f1), UINT64_C(0x1f430190a0aa8705)
    },
    {   UINT64_C(0x0000000000005c37), UINT64_C(0x1f42f98ff8b2e6ee), UINT64_C(0x1f43120f88afa6e5),
        UINT64_C(0x1f42fb8e9cb686eb), UINT64_C(0x1f42fa8fa8ac66e7)
    },
    {   UINT64_C(0x0000000000005c38), UINT64_C(0x1f42fd8e98b10717), UINT64_C(0x1f43180eb0af06f2),
        UINT64_C(0x1f42fb8f64af66f3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c39), UINT64_C(0x1f42f78e70b28722), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8f2cad0703), UINT64_C(0x1f43008f98b5e708)
    },
    {   UINT64_C(0x0000000000005c3a), UINT64_C(0x1f43018f18aec6e8), UINT64_C(0x1f42ff9000b6a6f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431bcf30aea708)
    },
    {   UINT64_C(0x0000000000005c3b), UINT64_C(0x1f42f88f4cb886f0), UINT64_C(0x1f430d0fc4afa707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4e5cac073f)
    },
    {   UINT64_C(0x0000000000005c3c), UINT64_C(0x1f42ff8f08b2e702), UINT64_C(0x1f42fecf50b326af),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8ee4ad26fa)
    },
    {   UINT64_C(0x0000000000005c3d), UINT64_C(0x1f42fc9008afc6db), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8fecaec6e8), UINT64_C(0x1f42fd8fb8ae26fa)
    },
    {   UINT64_C(0x0000000000005c3e), UINT64_C(0x1f43034f30b346ff), UINT64_C(0x1f43130f18ac472a),
        UINT64_C(0x1f43008f38b5a70e), UINT64_C(0x1f43034fe0b346dc)
    },
    {   UINT64_C(0x0000000000005c3f), UINT64_C(0x1f42fd8ef0b7a6d3), UINT64_C(0x1f42fb4e7cb6a733),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f10ae66eb)
    },
    {   UINT64_C(0x0000000000005c40), UINT64_C(0x1f43018f8cb446cc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074ef8abe707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c41), UINT64_C(0x1f43090f78aa86d5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074efca426e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c42), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4eb4a8672c),
        UINT64_C(0x1f43024e7cb2e6f9), UINT64_C(0x1f43110f14a78706)
    },
    {   UINT64_C(0x0000000000005c43), UINT64_C(0x1f42fd8f7cb2c6ff), UINT64_C(0x1f43054edcadc71a),
        UINT64_C(0x1f43094f8ca9a70a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c44), UINT64_C(0x1f43008fd8b1c6e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f4cb3c6df)
    },
    {   UINT64_C(0x0000000000005c45), UINT64_C(0x1f43020f98b1a6f1), UINT64_C(0x1f43018fbcb1670f),
        UINT64_C(0x1f42fa8f4caa46dc), UINT64_C(0x1f43008f40ad66e8)
    },
    {   UINT64_C(0x0000000000005c46), UINT64_C(0x1f43194f0ca9a717), UINT64_C(0x1f42fa4f8cb2670a),
        UINT64_C(0x1f430a4ee0b42743), UINT64_C(0x1f43134e88a76709)
    },
    {   UINT64_C(0x0000000000005c47), UINT64_C(0x1f43070ffcb8e6da), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8ec0af8729), UINT64_C(0x1f43018ea0b06705)
    },
    {   UINT64_C(0x0000000000005c48), UINT64_C(0x1f43008eacad8722), UINT64_C(0x1f43058f48b52734),
        UINT64_C(0x1f42fe8fdcad66eb), UINT64_C(0x1f430a4f98b106c9)
    },
    {   UINT64_C(0x0000000000005c49), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f8cb4e702),
        UINT64_C(0x1f430ccf1cafc725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c4a), UINT64_C(0x1f43018edca766f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c4b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff9004b246eb), UINT64_C(0x1f42ff8eacb06712)
    },
    {   UINT64_C(0x0000000000005c4c), UINT64_C(0x1f43004f90af06f1), UINT64_C(0x1f43104f68b0e6fb),
        UINT64_C(0x1f42fb8f74b5870e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c4d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43094f58b06711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c4e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8e6cb0e6ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014805), UINT64_C(0x1f42e41268b8a4f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000f05db14806), UINT64_C(0x0a6b0e1564680695), UINT64_C(0x0a6b069564644695),
        UINT64_C(0x0a6aecccc857646e), UINT64_C(0x0a6b0e1564644695)
    },
    {   UINT64_C(0x0007c4f65e914807), UINT64_C(0x0fa2fb55647ea695), UINT64_C(0x0fa2fb55647ea695),
        UINT64_C(0x0fa2dfccc870e46e), UINT64_C(0x0fa30515647ea695)
    },
    {   UINT64_C(0x0000000000014808), UINT64_C(0x0fa304cd5c7c4695), UINT64_C(0x0fa2f08d747c2695),
        UINT64_C(0x1182ff90487bc5b6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014809), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f6ca9046f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c54), UINT64_C(0x1f42fe8ff0b166fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8e60ac071a), UINT64_C(0x1f42fa8ed4afe704)
    },
    {   UINT64_C(0x0000000000005c55), UINT64_C(0x1f42fccf64aea70a), UINT64_C(0x1f430a8f90ad0703),
        UINT64_C(0x1f43004ea8b64719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c56), UINT64_C(0x1f42fb8f34b086fc), UINT64_C(0x1f42ff4fa0b226fe),
        UINT64_C(0x1f43144e78af4717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c57), UINT64_C(0x1f42fb4facb146ee), UINT64_C(0x1f42fa8f6caa86e8),
        UINT64_C(0x1f430acf68b3270b), UINT64_C(0x1f42fe8ea0ade700)
    },
    {   UINT64_C(0x0007da000001480e), UINT64_C(0x0000000000000000), UINT64_C(0x0a6af61020646587),
        UINT64_C(0x0a6af0902061a587), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c59), UINT64_C(0x1f42fa4eacb54729), UINT64_C(0x1f43018fb0b7c710),
        UINT64_C(0x1f430dcfa8b0872c), UINT64_C(0x1f43094fd4afc700)
    },
    {   UINT64_C(0x0000000000005c5a), UINT64_C(0x1f42f84ef8b70712), UINT64_C(0x1f42ff8f0cae870a),
        UINT64_C(0x1f42fc4f1cb40704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014811), UINT64_C(0x05bac40f7069a698), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c5c), UINT64_C(0x1f42f98fd8b2e6eb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4307ce74b06716), UINT64_C(0x1f43008fcca9e704)
    },
    {   UINT64_C(0x0000000000005c5d), UINT64_C(0x1f42fe0fd8b3a6d8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c5e), UINT64_C(0x1f43008f64af270e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008ff8b3a6eb), UINT64_C(0x1f42fa4efcb5c726)
    },
    {   UINT64_C(0x0000000000005c5f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f18a90735),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c60), UINT64_C(0x1f42fa4f60b5e6fb), UINT64_C(0x1f43030e84b2071e),
        UINT64_C(0x1f42f48ed8b6a709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c61), UINT64_C(0x1f42fc8f20b7471e), UINT64_C(0x1f42fa9024b626f3),
        UINT64_C(0x1f42fd8f4cae6706), UINT64_C(0x1f42fc8f84ab670b)
    },
    {   UINT64_C(0x0000000000005c62), UINT64_C(0x1f43038f90ad06c3), UINT64_C(0x1f43014ec0a8e701),
        UINT64_C(0x1f430f0eccae4728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c63), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43098e50ac873a)
    },
    {   UINT64_C(0x0000000000005c64), UINT64_C(0x1f42ff4fa0ac86ec), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074f14af071c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c65), UINT64_C(0x1f42f78f0cb5a711), UINT64_C(0x1f43040f00b086f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ec0b9870a)
    },
    {   UINT64_C(0x0000000000005c66), UINT64_C(0x1f42fd8f80aae6e8), UINT64_C(0x1f430bcf88ae06f1),
        UINT64_C(0x1f42fd8ed8b066fd), UINT64_C(0x1f42fc8ef4af06fb)
    },
    {   UINT64_C(0x0000000000005c67), UINT64_C(0x1f42ff8f7cb02705), UINT64_C(0x1f42fe8ef4b4e70b),
        UINT64_C(0x1f42ff8fe4b4a6d3), UINT64_C(0x1f4309cf28b58735)
    },
    {   UINT64_C(0x0000000000005c68), UINT64_C(0x1f43050f2cb006e3), UINT64_C(0x1f42ff8f04b0c711),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c69), UINT64_C(0x1f42fc8ff8af46d8), UINT64_C(0x1f42fb8f58b466df),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43114f3caf8721)
    },
    {   UINT64_C(0x0000000000005c6a), UINT64_C(0x1f42fc8e60b16717), UINT64_C(0x1f43008f48afe71e),
        UINT64_C(0x1f42fe8f50b1a6f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c6b), UINT64_C(0x1f430b8f88ad271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f40b0e722)
    },
    {   UINT64_C(0x0000000000005c6c), UINT64_C(0x1f42fc8f8cb126db), UINT64_C(0x1f43018f08b60735),
        UINT64_C(0x1f4307cf5caaa6e7), UINT64_C(0x1f42fd8f30b4470c)
    },
    {   UINT64_C(0x0000000000005c6d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f20ab4712),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c6e), UINT64_C(0x1f42fb8ee4b466c6), UINT64_C(0x1f42fe0e8cb02736),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c6f), UINT64_C(0x1f42fd8f38aee6d3), UINT64_C(0x1f42f88f1caf6711),
        UINT64_C(0x1f42f88f08b5470a), UINT64_C(0x1f43034edcb0670c)
    },
    {   UINT64_C(0x0000000000005c70), UINT64_C(0x1f42fc8eb8b2e6fe), UINT64_C(0x1f42fe8e9cafe72c),
        UINT64_C(0x1f42fb8e8cb00709), UINT64_C(0x1f42ff4fa4b6270b)
    },
    {   UINT64_C(0x0000000000005c71), UINT64_C(0x1f43010f44b646f0), UINT64_C(0x1f43074fc8b34706),
        UINT64_C(0x1f42fa8e9cb9470f), UINT64_C(0x1f42f98ed0b1e72f)
    },
    {   UINT64_C(0x0000000000014828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x14e2eacfa0912685)
    },
    {   UINT64_C(0x0000000000014829), UINT64_C(0x0fa2e60f707e4693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x266ae70f6cd12685)
    },
    {   UINT64_C(0x0000000000005c74), UINT64_C(0x1f42fa8e2cb3a727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98e84b7072d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001482b), UINT64_C(0x1192f68f5c82c691), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c76), UINT64_C(0x1f42f98edcade6cf), UINT64_C(0x1f430d8e64ae672e),
        UINT64_C(0x1f430d4f34b346f9), UINT64_C(0x1f4311cf44ac26f8)
    },
    {   UINT64_C(0x0000000000005c77), UINT64_C(0x1f42fa8ed4af06db), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0ebcb0a727)
    },
    {   UINT64_C(0x0000000000005c78), UINT64_C(0x1f42fc4f64b3671d), UINT64_C(0x1f42fa8e4cad0700),
        UINT64_C(0x1f43090f20b0a6fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c79), UINT64_C(0x1f42fb8f00ac671d), UINT64_C(0x1f42fb4eacb86713),
        UINT64_C(0x1f42ff8e8cb7a72c), UINT64_C(0x1f42fc8ee4b22719)
    },
    {   UINT64_C(0x0000000000005c7a), UINT64_C(0x1f42f98edcaf8723), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c7b), UINT64_C(0x1f42fe8ffcb826c8), UINT64_C(0x1f43008f04ade70e),
        UINT64_C(0x1f430c4e80b30736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c7c), UINT64_C(0x1f42fe8ef4b24700), UINT64_C(0x1f42fe8f38b10708),
        UINT64_C(0x1f43028e7cada721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c7d), UINT64_C(0x1f42fa8e48b82735), UINT64_C(0x1f42fb8ee8b4c715),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c7e), UINT64_C(0x1f42fe8ef8b146fe), UINT64_C(0x1f430d8f24ae672d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8eccb0071b)
    },
    {   UINT64_C(0x0000000000005c7f), UINT64_C(0x1f42fa8f4cb3672d), UINT64_C(0x1f42fc8e94b5c73d),
        UINT64_C(0x1f430b4e8caca739), UINT64_C(0x1f43030e98b006f7)
    },
    {   UINT64_C(0x0000000000005c80), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8ff8b2c6ee),
        UINT64_C(0x1f4309cf14afa6df), UINT64_C(0x1f42f88eacb4c70c)
    },
    {   UINT64_C(0x0000000000014837), UINT64_C(0x0000000000000000), UINT64_C(0x000231cba85e86e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c82), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8f58b1672c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c83), UINT64_C(0x1f42f98f0cb2472d), UINT64_C(0x1f42f88e6cb6c72c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c84), UINT64_C(0x1f42fe8f00b42718), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88edcafe6f1)
    },
    {   UINT64_C(0x0000000000005c85), UINT64_C(0x1f42ff4ee0ae2746), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c86), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e70b5c71e),
        UINT64_C(0x1f42fd8edcb9271a), UINT64_C(0x1f42fc8f0cac8706)
    },
    {   UINT64_C(0x0000000000005c87), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8e3cb4e729),
        UINT64_C(0x1f42f88f74b3e730), UINT64_C(0x1f42f98e5cb18724)
    },
    {   UINT64_C(0x0000000000005c88), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43080ebcabc729), UINT64_C(0x1f42fa8e7cb50735)
    },
    {   UINT64_C(0x0000000000005c89), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88e9cb56727), UINT64_C(0x1f42fe8f60b12716)
    },
    {   UINT64_C(0x0000000000005c8a), UINT64_C(0x1f4309ceccaf0708), UINT64_C(0x1f42fb8e94b8873d),
        UINT64_C(0x1f43010ee8b6a6f3), UINT64_C(0x1f431a9830a5471c)
    },
    {   UINT64_C(0x0000000000005c8b), UINT64_C(0x1f42facf88b2a6f6), UINT64_C(0x1f43028e58b4e733),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c8c), UINT64_C(0x1f430c8e70b6874a), UINT64_C(0x1f430ecf20ace722),
        UINT64_C(0x1f43068e28af4731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c8d), UINT64_C(0x0000000000000000), UINT64_C(0x1f43138e30b00740),
        UINT64_C(0x1f43098facb2e6e7), UINT64_C(0x1f43080e6cb5070c)
    },
    {   UINT64_C(0x0000000000005c8e), UINT64_C(0x1f42f98ed0b7870a), UINT64_C(0x1f42fe8e84b6c733),
        UINT64_C(0x1f42ff4e4cb76738), UINT64_C(0x1f4307cf18b4a71b)
    },
    {   UINT64_C(0x0000000000014845), UINT64_C(0x0000000000000000), UINT64_C(0x17c2f4cfa0a06685),
        UINT64_C(0x20faf40f64bf668b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c90), UINT64_C(0x1f42f60ee0b78715), UINT64_C(0x1f43028f18b106e0),
        UINT64_C(0x1f43014ed8afc71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014847), UINT64_C(0x0000000000000000), UINT64_C(0x16baf28fa0984685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c92), UINT64_C(0x1f42f84facb706fd), UINT64_C(0x1f42fb0ef8b186fe),
        UINT64_C(0x1f42fc4f4cab06eb), UINT64_C(0x1f42fc0ec0afc6fd)
    },
    {   UINT64_C(0x0000000000005c93), UINT64_C(0x1f43000e7cb5c739), UINT64_C(0x1f43004f48ae2706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ed0b7670f)
    },
    {   UINT64_C(0x0000000000005c94), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0eecb68724), UINT64_C(0x1f42fb4f24b14734)
    },
    {   UINT64_C(0x0000000000005c95), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94f30adc732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c96), UINT64_C(0x1f42fb0e70b00742), UINT64_C(0x1f43144ef8af6704),
        UINT64_C(0x1f43188eb0ac871e), UINT64_C(0x1f43098e6cb20730)
    },
    {   UINT64_C(0x0000000000005c97), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0eecb326fb),
        UINT64_C(0x1f42ff0eccad86fb), UINT64_C(0x1f43068e5cb08738)
    },
    {   UINT64_C(0x0000000000005c98), UINT64_C(0x1f42f98f94b1a708), UINT64_C(0x1f43004f68b48712),
        UINT64_C(0x1f43104e6cae8739), UINT64_C(0x1f43000eacb266fc)
    },
    {   UINT64_C(0x0000000000005c99), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4f38b5070e), UINT64_C(0x1f42facf38b786fe)
    },
    {   UINT64_C(0x0000000000005c9a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f58b1271f),
        UINT64_C(0x1f43048ef8b126f3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c9b), UINT64_C(0x1f43014ee8b06702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4e7ca986fe), UINT64_C(0x1f42f70f58b1870f)
    },
    {   UINT64_C(0x0000000000005c9c), UINT64_C(0x1f42fe0f28b426f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4f08b4470c), UINT64_C(0x1f42fecf38b246f0)
    },
    {   UINT64_C(0x0000000000005c9d), UINT64_C(0x1f43010f68b7270f), UINT64_C(0x1f43124e7ca9a71f),
        UINT64_C(0x1f4304ce6cb88735), UINT64_C(0x1f42f90f38b826ff)
    },
    {   UINT64_C(0x0000000000005c9e), UINT64_C(0x1f42ff0ed4b086f6), UINT64_C(0x1f42ff8e70b40734),
        UINT64_C(0x1f42ff8f28b22713), UINT64_C(0x1f42fdcf8cb306f3)
    },
    {   UINT64_C(0x0000000000005c9f), UINT64_C(0x1f43008f18af46f7), UINT64_C(0x1f43120e8caf273e),
        UINT64_C(0x1f43020f5cb066f7), UINT64_C(0x1f43030f0cb0070a)
    },
    {   UINT64_C(0x0000000000005ca0), UINT64_C(0x1f42f98e5cac6725), UINT64_C(0x1f430d8e7cab8721),
        UINT64_C(0x1f430d8e7cab8721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ca1), UINT64_C(0x1f42f90e94b20732), UINT64_C(0x1f42fb4f00ac06ec),
        UINT64_C(0x1f43000f38af4700), UINT64_C(0x1f42fd0eacb42715)
    },
    {   UINT64_C(0x0000000000005ca2), UINT64_C(0x1f42f9cf1cb4c6e1), UINT64_C(0x1f42fd0ed8aea715),
        UINT64_C(0x1f42ff8f38ade70b), UINT64_C(0x1f4303cf20b066d8)
    },
    {   UINT64_C(0x0000000000005ca3), UINT64_C(0x1f42fb0edcb806f7), UINT64_C(0x1f42fdcefcb64702),
        UINT64_C(0x1f43114e8cab8726), UINT64_C(0x1f42fd4f58ad4708)
    },
    {   UINT64_C(0x0000000000005ca4), UINT64_C(0x1f42fc0ea8b8e700), UINT64_C(0x1f42fe0edcb726fc),
        UINT64_C(0x1f42fb8f5cace717), UINT64_C(0x1f42fb4ea8aaa717)
    },
    {   UINT64_C(0x0000000000005ca5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94e68af871d),
        UINT64_C(0x1f430b4e6cafa747), UINT64_C(0x1f42fa4eccb1e70e)
    },
    {   UINT64_C(0x0000000000005ca6), UINT64_C(0x1f42fb8e9cae672f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d0e7caaa72e), UINT64_C(0x1f43048e8cb1c72f)
    },
    {   UINT64_C(0x0000000000005ca7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f28b00702),
        UINT64_C(0x1f42fd0ec8b5470b), UINT64_C(0x1f4317ceb8ab071a)
    },
    {   UINT64_C(0x0000000000005ca8), UINT64_C(0x1f42fa4e48b58733), UINT64_C(0x1f42fccf24af66fb),
        UINT64_C(0x1f43000e54ae2732), UINT64_C(0x1f42fc8ef8b76716)
    },
    {   UINT64_C(0x0000000000005ca9), UINT64_C(0x1f42f8cf88b3a6f8), UINT64_C(0x1f42fc8f58b10719),
        UINT64_C(0x1f42f9cec0b64700), UINT64_C(0x1f42f84edcb0c740)
    },
    {   UINT64_C(0x0000000000005caa), UINT64_C(0x1f43034e6cab871d), UINT64_C(0x1f43028e7cb06744),
        UINT64_C(0x1f42f68ea0b4272d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cab), UINT64_C(0x1f430e8fa8b5a70a), UINT64_C(0x1f432fcf6489e6e6),
        UINT64_C(0x1f43068f1cb1e70f), UINT64_C(0x1f42fd8fbcb6e6e6)
    },
    {   UINT64_C(0x0000000000005cac), UINT64_C(0x1f42f90f50b6a6ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300ced4b20715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cad), UINT64_C(0x1f42fc0ee8b2a6fd), UINT64_C(0x1f42fdcf20b266fc),
        UINT64_C(0x1f42ff0f88b64700), UINT64_C(0x1f42fccf4cac26f4)
    },
    {   UINT64_C(0x0000000000005cae), UINT64_C(0x1f42fccf10ada6e4), UINT64_C(0x1f42fccf60b6c6df),
        UINT64_C(0x1f42fcced8b666d8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005caf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffce74b3e6e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0e88b186e5)
    },
    {   UINT64_C(0x0000000000005cb0), UINT64_C(0x1f42f9cf3cb64718), UINT64_C(0x1f42f64f1cb746e8),
        UINT64_C(0x1f42f98fc0b3670d), UINT64_C(0x1f42fa0f80b0e6ed)
    },
    {   UINT64_C(0x0000000000005cb1), UINT64_C(0x1f42fa8f60b5c6fb), UINT64_C(0x1f42fa4f54afe6db),
        UINT64_C(0x1f4300cec8ae2718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cb2), UINT64_C(0x1f42fc4f8ca8a6fb), UINT64_C(0x1f42fdcf30ae86cf),
        UINT64_C(0x1f43124f2caba70b), UINT64_C(0x1f42fc0f78ad46c8)
    },
    {   UINT64_C(0x0000000000005cb3), UINT64_C(0x1f42fb4f38b6c6e9), UINT64_C(0x1f42fccf3cafe71c),
        UINT64_C(0x1f42fe4e5caf070b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cb4), UINT64_C(0x1f42f48fa0b986ef), UINT64_C(0x1f42fb4f34b7a703),
        UINT64_C(0x1f42fecf28af46ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cb5), UINT64_C(0x1f42fc0f34b766dd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf50b6a6e2), UINT64_C(0x1f42fb8f50ad2709)
    },
    {   UINT64_C(0x0000000000005cb6), UINT64_C(0x1f42fa0f48b2c6e1), UINT64_C(0x1f42fa8f30b5a701),
        UINT64_C(0x1f42fa0f34b7a6d9), UINT64_C(0x1f43050f34b026d9)
    },
    {   UINT64_C(0x0000000000005cb7), UINT64_C(0x1f42fa0f1cb226ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0ee8afe712), UINT64_C(0x1f42fccf60b1e6df)
    },
    {   UINT64_C(0x0000000000005cb8), UINT64_C(0x1f42f90f48b866ff), UINT64_C(0x1f42f9cf64b826f2),
        UINT64_C(0x1f42f98ee8b5c6f5), UINT64_C(0x1f42f64f80b4e6d3)
    },
    {   UINT64_C(0x0000000000005ef3), UINT64_C(0x1f43040ef8ae6715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43174e88ae0733), UINT64_C(0x1f42ff8e84b2a714)
    },
    {   UINT64_C(0x0000000000005cba), UINT64_C(0x1f42fa4f04b0c6d7), UINT64_C(0x1f42f90ea4b1e72d),
        UINT64_C(0x1f42fccf14abe6d0), UINT64_C(0x1f42fccf70b026ea)
    },
    {   UINT64_C(0x0000000000005cbb), UINT64_C(0x1f42fecf68a966e1), UINT64_C(0x1f42fccf4cb3c6fb),
        UINT64_C(0x1f42faced8adc701), UINT64_C(0x1f42fccf58b806e4)
    },
    {   UINT64_C(0x0000000000005cbc), UINT64_C(0x1f42ff8ef0b36718), UINT64_C(0x1f42fccf84b886d8),
        UINT64_C(0x1f42fa0f44b4a6e8), UINT64_C(0x1f42f84ea0b7270a)
    },
    {   UINT64_C(0x0000000000005cbd), UINT64_C(0x1f42f34fa0ada6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64df0b4672a), UINT64_C(0x1f42f98f30a4e701)
    },
    {   UINT64_C(0x0000000000005cbe), UINT64_C(0x1f42f98fc8b34700), UINT64_C(0x1f42fccf54b72707),
        UINT64_C(0x1f42f64f8cad4703), UINT64_C(0x1f42f84f00b7271b)
    },
    {   UINT64_C(0x0000000000005cbf), UINT64_C(0x1f43008f14b86722), UINT64_C(0x1f42fc4f5cab86fb),
        UINT64_C(0x1f42fccea8b106ca), UINT64_C(0x1f42fd1004b0e6dd)
    },
    {   UINT64_C(0x0000000000005cc0), UINT64_C(0x1f42fdcee8b4e6e6), UINT64_C(0x1f42fb4ee8adc6f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf54b5c6ef)
    },
    {   UINT64_C(0x0000000000005cc1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f44b3c712)
    },
    {   UINT64_C(0x0000000000005cc2), UINT64_C(0x1f42ff0eacafc72c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f48f6cb6a6e4), UINT64_C(0x1f43064fa8b06709)
    },
    {   UINT64_C(0x0000000000005cc3), UINT64_C(0x1f43010f18b4c6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccf94ae0715), UINT64_C(0x1f42fd0ee8a4a70e)
    },
    {   UINT64_C(0x0000000000005cc4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f10b3a6c7),
        UINT64_C(0x1f430fcfacb446df), UINT64_C(0x1f42fc0ee8b4c6ea)
    },
    {   UINT64_C(0x0000000000005cc5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f50b1a6e2),
        UINT64_C(0x1f42fccfdcae06de), UINT64_C(0x1f43000e98ae86f9)
    },
    {   UINT64_C(0x0000000000005cc6), UINT64_C(0x1f42fe0ec8aae6fc), UINT64_C(0x1f42fcceb0b6a733),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccfb8b0a6ef)
    },
    {   UINT64_C(0x0000000000005cc7), UINT64_C(0x1f42fa0ea0b1e738), UINT64_C(0x1f42fd4e9cb2c729),
        UINT64_C(0x1f42fccf98b446f0), UINT64_C(0x1f42fa4edcb566f7)
    },
    {   UINT64_C(0x0000000000005cc8), UINT64_C(0x0000000000000000), UINT64_C(0x1f43000ef0ade6e3),
        UINT64_C(0x1f42fd0f4caba6f9), UINT64_C(0x1f42ff0f10b28704)
    },
    {   UINT64_C(0x0000000000005cc9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80ef8b686fe),
        UINT64_C(0x1f42f98f1cb3e6c1), UINT64_C(0x1f42fdcf40af66df)
    },
    {   UINT64_C(0x0000000000005cca), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f9caf46ed),
        UINT64_C(0x1f42fccf3cae46f6), UINT64_C(0x1f42fccf78b086ff)
    },
    {   UINT64_C(0x0000000000005ccb), UINT64_C(0x1f42fd0ec0a966ee), UINT64_C(0x1f42fb4edcb6a705),
        UINT64_C(0x1f43065000b4c6ea), UINT64_C(0x1f42fc4ebcb4871d)
    },
    {   UINT64_C(0x0000000000005ccc), UINT64_C(0x1f4300cf4cafe6ce), UINT64_C(0x1f42fd0f18b246f4),
        UINT64_C(0x1f43010eacace6f7), UINT64_C(0x1f42fa0fe0b5c6d1)
    },
    {   UINT64_C(0x0000000000005ccd), UINT64_C(0x1f42fb4fd4b7a6fd), UINT64_C(0x1f42ffcf9cade70b),
        UINT64_C(0x1f42fd8f44b2e71d), UINT64_C(0x1f43030fecb5c712)
    },
    {   UINT64_C(0x0000000000005cce), UINT64_C(0x1f42fccf7ca8c6db), UINT64_C(0x1f42f90f54b38704),
        UINT64_C(0x1f42fc8fbcafa721), UINT64_C(0x1f42f90f40b146e8)
    },
    {   UINT64_C(0x0000000000005ccf), UINT64_C(0x1f42fc8e54b3671d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70eb0b5670f), UINT64_C(0x1f42f9ced8b74729)
    },
    {   UINT64_C(0x0000000000005cd0), UINT64_C(0x1f43010ed8b68705), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ec0b3c6e4), UINT64_C(0x1f42f8cf68b8270c)
    },
    {   UINT64_C(0x0000000000005cd1), UINT64_C(0x1f42f5cfd0b826ff), UINT64_C(0x1f42f9ce9cb2271d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cd2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ce6cb9a723),
        UINT64_C(0x1f43000eb0b9471d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cd3), UINT64_C(0x1f42fa0f24b4e703), UINT64_C(0x1f42f60f14ad872d),
        UINT64_C(0x1f43118ea4b18733), UINT64_C(0x1f42fccf44b4270b)
    },
    {   UINT64_C(0x0000000000005cd4), UINT64_C(0x1f42f8ce5cad6732), UINT64_C(0x1f42f70e4cb52713),
        UINT64_C(0x1f430c8efcb1c712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cd5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cfd8b826e5)
    },
    {   UINT64_C(0x0000000000005cd6), UINT64_C(0x1f4315cf00b5072e), UINT64_C(0x1f42f9cf7cb886cb),
        UINT64_C(0x1f42ffcf18b06718), UINT64_C(0x1f42ffcf48afe705)
    },
    {   UINT64_C(0x0000000000005cd7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdceb8ac4704),
        UINT64_C(0x1f42fd8f28b0c702), UINT64_C(0x1f42ffcf4cb4e722)
    },
    {   UINT64_C(0x0000000000005cd8), UINT64_C(0x1f42ffcf14b7870d), UINT64_C(0x1f42ffceccaf06f0),
        UINT64_C(0x1f42fa8f10b8e70d), UINT64_C(0x1f42f8cf88b626f9)
    },
    {   UINT64_C(0x0000000000005cd9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ee8b886eb),
        UINT64_C(0x1f42f90ebcab672c), UINT64_C(0x1f42fbcf10b6070d)
    },
    {   UINT64_C(0x0000000000005cda), UINT64_C(0x1f42fc0f58ac071a), UINT64_C(0x1f43008ef4b16705),
        UINT64_C(0x1f42f3cea4b74718), UINT64_C(0x1f42fa8f68b80706)
    },
    {   UINT64_C(0x0000000000005cdb), UINT64_C(0x1f42f7cf18b50703), UINT64_C(0x1f42fb0f80aec6f6),
        UINT64_C(0x1f42fe8ec4ab26f2), UINT64_C(0x1f42fd0f0cadc703)
    },
    {   UINT64_C(0x0000000000005cdc), UINT64_C(0x1f42f6ce60b14735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fcceecb66740), UINT64_C(0x1f42feced0b306f6)
    },
    {   UINT64_C(0x0000000000005cdd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f70b74733),
        UINT64_C(0x1f42f3ceccb1c719), UINT64_C(0x1f42f90f1cb00735)
    },
    {   UINT64_C(0x0000000000005cde), UINT64_C(0x1f42f90f88b646f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f80aec6ef), UINT64_C(0x1f42f68f3cb6a71a)
    },
    {   UINT64_C(0x0000000000005cdf), UINT64_C(0x1f42f9cf28adc6d1), UINT64_C(0x1f42fb0ea0b50735),
        UINT64_C(0x1f42fe4eb8b06739), UINT64_C(0x1f42fb4ee4b326fc)
    },
    {   UINT64_C(0x0000000000005ce0), UINT64_C(0x1f42f80e8cb366f0), UINT64_C(0x1f42f9ceccb60701),
        UINT64_C(0x1f42f9ced8afe717), UINT64_C(0x1f42f44e50b5072f)
    },
    {   UINT64_C(0x0000000000005ce1), UINT64_C(0x1f430a4fd0b186fe), UINT64_C(0x1f4306ceacb226ef),
        UINT64_C(0x1f43079000aac6ee), UINT64_C(0x1f430b8e7cb3c73c)
    },
    {   UINT64_C(0x0000000000005ce2), UINT64_C(0x1f43080fc4b32700), UINT64_C(0x1f43081098b2e6b8),
        UINT64_C(0x1f4307d004b2c6a6), UINT64_C(0x1f43085084b1e6b8)
    },
    {   UINT64_C(0x0000000000005ce3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4302cf28b4269b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ce4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe5014b566ce),
        UINT64_C(0x1f43021110b74685), UINT64_C(0x1f43020f14af46da)
    },
    {   UINT64_C(0x0000000000005ce5), UINT64_C(0x1f42fe4eaca94707), UINT64_C(0x1f43008f54b0e6fb),
        UINT64_C(0x1f42fbcf7cb6a6e8), UINT64_C(0x1f43018eecb486c2)
    },
    {   UINT64_C(0x0000000000005ce6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054f60ada6ec)
    },
    {   UINT64_C(0x0000000000005ce7), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cec8b326fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cfc4b706fb)
    },
    {   UINT64_C(0x0000000000005ce8), UINT64_C(0x1f42f88eacb7a706), UINT64_C(0x1f43028ed4b3a6cf),
        UINT64_C(0x1f42fc4f08b826da), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ce9), UINT64_C(0x1f42fe0e98b4a724), UINT64_C(0x1f43018eb0a526df),
        UINT64_C(0x1f4303d000b5c700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cea), UINT64_C(0x1f42fe10fcb4c6d2), UINT64_C(0x1f43031074b926dd),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb1000b54711)
    },
    {   UINT64_C(0x0000000000005ceb), UINT64_C(0x1f43024f04b666b7), UINT64_C(0x1f42fe4ffcb8e6b7),
        UINT64_C(0x1f43058eb8aec711), UINT64_C(0x1f42fdcf58b4a6d9)
    },
    {   UINT64_C(0x0000000000005cec), UINT64_C(0x1f42fc4f78b546ec), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4301cee8b9a71d), UINT64_C(0x1f42fa4f10b1872c)
    },
    {   UINT64_C(0x0000000000005ced), UINT64_C(0x1f43014fe4b3a6aa), UINT64_C(0x1f43014fc8b406ec),
        UINT64_C(0x1f42fdceb8b566d6), UINT64_C(0x1f42fe4fc8ac6701)
    },
    {   UINT64_C(0x0000000000005cee), UINT64_C(0x1f43020f68b186f1), UINT64_C(0x1f43031010ade6eb),
        UINT64_C(0x1f43194f70ab6709), UINT64_C(0x1f42fe0f9cb8c71d)
    },
    {   UINT64_C(0x0000000000005cef), UINT64_C(0x1f42f6507cb7a6cb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fece6cae66af), UINT64_C(0x1f4302cf4cab06ad)
    },
    {   UINT64_C(0x0000000000005cf0), UINT64_C(0x1f42ff50d8a906fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000facb566d9), UINT64_C(0x1f4301ceecb6e6fd)
    },
    {   UINT64_C(0x0000000000005cf1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e6caf8715),
        UINT64_C(0x1f43004f68aea6e2), UINT64_C(0x1f43029030b6a6a9)
    },
    {   UINT64_C(0x0000000000005cf2), UINT64_C(0x1f42fb909cb406ea), UINT64_C(0x1f42ff9044b306e2),
        UINT64_C(0x1f42fc500cb8a6b1), UINT64_C(0x1f42fe4f18b026ce)
    },
    {   UINT64_C(0x0000000000005cf3), UINT64_C(0x1f4301d058b2a6b6), UINT64_C(0x1f42fc4ea0baa72f),
        UINT64_C(0x1f42fd0f4cb4070e), UINT64_C(0x1f430150fcb366c5)
    },
    {   UINT64_C(0x0000000000005cf4), UINT64_C(0x1f4301100cb00703), UINT64_C(0x1f43045010b506f5),
        UINT64_C(0x1f42fe8e68b58731), UINT64_C(0x1f42f98f54b6a70b)
    },
    {   UINT64_C(0x0000000000005cf5), UINT64_C(0x1f42fd0ea0b586fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cfa8b7a705), UINT64_C(0x1f42ff0f64b746fe)
    },
    {   UINT64_C(0x0000000000005cf6), UINT64_C(0x1f42f90ee0b44719), UINT64_C(0x1f42fdcff8b08711),
        UINT64_C(0x1f42fe8f9cafa6ce), UINT64_C(0x1f42fccee8af4718)
    },
    {   UINT64_C(0x0000000000005cf7), UINT64_C(0x1f42fb4e9cb2a731), UINT64_C(0x1f42fe4ef0aae69d),
        UINT64_C(0x1f42fc0f8cb90706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cf8), UINT64_C(0x1f42fece6cb72736), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8ee4b0a737)
    },
    {   UINT64_C(0x0000000000005cf9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e8cb806ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4fd4b586d4)
    },
    {   UINT64_C(0x0000000000005cfa), UINT64_C(0x1f42fd0efcb966f1), UINT64_C(0x1f42f84f3cb42710),
        UINT64_C(0x1f42fe9038b14719), UINT64_C(0x1f42ffcf50b72706)
    },
    {   UINT64_C(0x0000000000005cfb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074fa4b4c6f6),
        UINT64_C(0x1f42f9cfb8b7c706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cfc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccef0b426df),
        UINT64_C(0x1f4300cf04af86db), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cfd), UINT64_C(0x1f43004f50b0a6fc), UINT64_C(0x1f43048fe0b126d3),
        UINT64_C(0x1f43024f08ae471c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cfe), UINT64_C(0x1f4303cf7cb266d7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4301cfd8af86b6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cff), UINT64_C(0x1f43044faca926dd), UINT64_C(0x1f42ff1020aee6bd),
        UINT64_C(0x1f42fb8f70b4c6e5), UINT64_C(0x1f4302d040b4e6e0)
    },
    {   UINT64_C(0x0000000000005d00), UINT64_C(0x1f43020f3cb6a6fd), UINT64_C(0x1f42fc4f78b5871e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4ebcb74720)
    },
    {   UINT64_C(0x0000000000005d01), UINT64_C(0x1f43008ec8a5a6ed), UINT64_C(0x1f42fc4ec8b6473d),
        UINT64_C(0x1f42ff9080b3c6dc), UINT64_C(0x1f42fe4f18b44705)
    },
    {   UINT64_C(0x0000000000005d02), UINT64_C(0x1f43034ef8b02706), UINT64_C(0x1f4301cfe8b8071a),
        UINT64_C(0x1f42f84ed8af66f2), UINT64_C(0x1f43030ee8ada71d)
    },
    {   UINT64_C(0x0000000000005d03), UINT64_C(0x1f43010f2cb6e716), UINT64_C(0x1f4300cf04aec6e5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf9cb1c706)
    },
    {   UINT64_C(0x0000000000005d04), UINT64_C(0x1f4301ceccaa8728), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d05), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040ee0adc6fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d06), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0fd8afe6f6), UINT64_C(0x1f42f80f84b64705)
    },
    {   UINT64_C(0x0000000000005d07), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf80b52703),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d08), UINT64_C(0x1f42ff8f50b6471d), UINT64_C(0x1f42fc8fbcb806ee),
        UINT64_C(0x1f42fa0fc8b9a6f9), UINT64_C(0x1f43008f7cb26715)
    },
    {   UINT64_C(0x0000000000005d09), UINT64_C(0x1f42fe8ea0b4e71a), UINT64_C(0x1f43000f34b5e728),
        UINT64_C(0x1f42fb4e8cb5c73e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d0a), UINT64_C(0x1f42fa9030b486e1), UINT64_C(0x1f42fe0f40afa6da),
        UINT64_C(0x1f42fbcf30b6472e), UINT64_C(0x1f42fd8e7cb64713)
    },
    {   UINT64_C(0x0000000000005d0b), UINT64_C(0x1f42fd1010b4e715), UINT64_C(0x1f42fa8e9cb3c724),
        UINT64_C(0x1f42fc8eecaa26eb), UINT64_C(0x1f42fb4f18b0272c)
    },
    {   UINT64_C(0x0000000000005d0c), UINT64_C(0x1f42fe8f74b0e6fa), UINT64_C(0x1f43004f1cac26d4),
        UINT64_C(0x1f43000f18b36729), UINT64_C(0x1f42f94f08b9271a)
    },
    {   UINT64_C(0x0000000000005d0d), UINT64_C(0x1f42fe8ebcb7e72e), UINT64_C(0x1f43128f48b18716),
        UINT64_C(0x1f43038eecb02723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d0e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffd040b56721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d0f), UINT64_C(0x1f42fe8f24ace716), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ec8b26719)
    },
    {   UINT64_C(0x0000000000005d10), UINT64_C(0x1f42fe8f5cb1e6fe), UINT64_C(0x1f42fb8f6cacc6fa),
        UINT64_C(0x1f42fd4f2cabc6e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d11), UINT64_C(0x1f42fc0f78b5470e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4facb3e6e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d12), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f1cb54713), UINT64_C(0x1f4302ce6cb0074a)
    },
    {   UINT64_C(0x0000000000005d13), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f5cb64700),
        UINT64_C(0x1f42fc4ea4b46708), UINT64_C(0x1f42fc4e9cb2a721)
    },
    {   UINT64_C(0x0000000000005d14), UINT64_C(0x1f42fc0ea0ae4720), UINT64_C(0x1f42ff8e94b98718),
        UINT64_C(0x1f43008f04b3870a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d15), UINT64_C(0x1f42fccf0cb2a70b), UINT64_C(0x1f42ff8f98b806db),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f68b64737)
    },
    {   UINT64_C(0x0000000000005d16), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0f54ae6700),
        UINT64_C(0x1f42fb8f44aec716), UINT64_C(0x1f42facf80b70716)
    },
    {   UINT64_C(0x0000000000005d17), UINT64_C(0x1f42fc4eb0baa730), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8ec4b1a71f), UINT64_C(0x1f43118f30b0e710)
    },
    {   UINT64_C(0x0000000000005d18), UINT64_C(0x1f42fd0ee4b426dd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ee0bb270e), UINT64_C(0x1f42ffcea4acc72e)
    },
    {   UINT64_C(0x0000000000005d19), UINT64_C(0x1f42fd0f14af870e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58ebcb5c727)
    },
    {   UINT64_C(0x0000000000005d1a), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4f2cb08737),
        UINT64_C(0x1f43010f08b26713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d1b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf3cb5c71a),
        UINT64_C(0x1f42f6ce9cb4c6fc), UINT64_C(0x1f42f98e9cb48735)
    },
    {   UINT64_C(0x0000000000005d1c), UINT64_C(0x1f42fb4fb8b8072d), UINT64_C(0x1f42f98e8cb5c719),
        UINT64_C(0x1f42facf78b72724), UINT64_C(0x1f4300cfd8b34738)
    },
    {   UINT64_C(0x0000000000005d1d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf30b6e6f5),
        UINT64_C(0x1f42fd0f64af86fc), UINT64_C(0x1f42fe8f54b4271c)
    },
    {   UINT64_C(0x0000000000005d1e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf3cb74719),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d1f), UINT64_C(0x1f43020e84b38732), UINT64_C(0x1f43054fc4b666de),
        UINT64_C(0x1f42ff0fb4b3472d), UINT64_C(0x1f43144f28aea70a)
    },
    {   UINT64_C(0x0000000000005d20), UINT64_C(0x1f42fc8e7cb0c732), UINT64_C(0x1f42fb0f44b4472e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f1caf4725)
    },
    {   UINT64_C(0x0000000000005d21), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302ceccb92722),
        UINT64_C(0x1f42fe8ed0b7470a), UINT64_C(0x1f42fad010b64719)
    },
    {   UINT64_C(0x0000000000005d22), UINT64_C(0x1f42fa0f58b5a6ee), UINT64_C(0x1f42f74f54b40721),
        UINT64_C(0x1f42ffcf30b10719), UINT64_C(0x1f42face6caf8740)
    },
    {   UINT64_C(0x0000000000005d23), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f0cb1c6f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d24), UINT64_C(0x1f42fccfe8b706ec), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4f6cb626e4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d25), UINT64_C(0x1f43018eb0ae8726), UINT64_C(0x1f42fe4ed8b1e70d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43158f34a586fc)
    },
    {   UINT64_C(0x0000000000005d26), UINT64_C(0x1f42fdce50b4472f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300ceccac2708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d27), UINT64_C(0x1f42fe4ec4b2e736), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4ef0b2c733), UINT64_C(0x1f42fe8ea4ac2723)
    },
    {   UINT64_C(0x0000000000005d28), UINT64_C(0x1f43030f60b6a70c), UINT64_C(0x1f42fd8f6cb1e6de),
        UINT64_C(0x1f43004f38b286df), UINT64_C(0x1f42fe0e80b36722)
    },
    {   UINT64_C(0x0000000000005d29), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f98b726f7),
        UINT64_C(0x1f42fccf50b7672c), UINT64_C(0x1f43028f24ab8739)
    },
    {   UINT64_C(0x0000000000005d2a), UINT64_C(0x1f42f84e9cb60734), UINT64_C(0x1f42fc0ec4b72735),
        UINT64_C(0x1f42fd0f08b2c6f8), UINT64_C(0x1f43000f00b12706)
    },
    {   UINT64_C(0x0000000000005d2b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f44b466ce),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ec4b6a727)
    },
    {   UINT64_C(0x0000000000005d2c), UINT64_C(0x1f42f84e78b68720), UINT64_C(0x1f42fd8ef8b466c7),
        UINT64_C(0x1f42fd0f5cb54705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d2d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f48b54706),
        UINT64_C(0x1f42f98f54b2c71d), UINT64_C(0x1f42faceecb5a6f8)
    },
    {   UINT64_C(0x0000000000005d2e), UINT64_C(0x1f42ff4ed0af071a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078f4cb226f3), UINT64_C(0x1f42fd4e6cb1e711)
    },
    {   UINT64_C(0x000000001ff9bec0), UINT64_C(0x12eaf30fa0906693), UINT64_C(0x1172f60fa07fe693),
        UINT64_C(0x1162f5cfa07fc693), UINT64_C(0x1a02f30fa0a22693)
    },
    {   UINT64_C(0x0000000000005d30), UINT64_C(0x1f42fc8ea8af26eb), UINT64_C(0x1f42fb4e54b28738),
        UINT64_C(0x1f42fb8ef4b1c72d), UINT64_C(0x1f430210e0b306d5)
    },
    {   UINT64_C(0x0000000000005d31), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0eecb6a71e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d32), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0e74b1a737), UINT64_C(0x1f43028f30b3c6fd)
    },
    {   UINT64_C(0x0000000000005d33), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cf28b24702)
    },
    {   UINT64_C(0x0000000000005d34), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4eecaf2719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d35), UINT64_C(0x1f42fb0f3caf6708), UINT64_C(0x1f43128e88ac6730),
        UINT64_C(0x1f42fc0eb0b64738), UINT64_C(0x1f42fece24ada735)
    },
    {   UINT64_C(0x0000000000005d36), UINT64_C(0x1f42fcce7caf8728), UINT64_C(0x1f42fbce98b20745),
        UINT64_C(0x1f42fc0e54b2e731), UINT64_C(0x1f42f8cf1cb84728)
    },
    {   UINT64_C(0x0000000000005d37), UINT64_C(0x1f42fbcf34b5a6ff), UINT64_C(0x1f42facf78b5a6e3),
        UINT64_C(0x1f42f54ff0b4271a), UINT64_C(0x1f42fa4f78b606ed)
    },
    {   UINT64_C(0x0000000000005d38), UINT64_C(0x1f42f74f38b6a72e), UINT64_C(0x1f42faceccb98715),
        UINT64_C(0x1f42f70f54b886ee), UINT64_C(0x1f4301cf38b0071c)
    },
    {   UINT64_C(0x0000000000005d39), UINT64_C(0x1f42f90efcb62736), UINT64_C(0x1f43054e78b86712),
        UINT64_C(0x1f431d8f8cb4a6f9), UINT64_C(0x1f42ff0f50b30700)
    },
    {   UINT64_C(0x0000000000005d3a), UINT64_C(0x1f42fb4f48b6c6ae), UINT64_C(0x1f43028f50b4a6c0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d3b), UINT64_C(0x1f42f50f40b5e6d4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4312cf38adc6f4), UINT64_C(0x1f43038ffcb726cf)
    },
    {   UINT64_C(0x0000000000005d3c), UINT64_C(0x1f42f98f50b5a700), UINT64_C(0x1f43028f50b4a6e2),
        UINT64_C(0x1f42ff0f50b30708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d3d), UINT64_C(0x1f42f9cf40ba26fe), UINT64_C(0x1f42fa4f3cb826da),
        UINT64_C(0x1f42f74fc0b6c70e), UINT64_C(0x1f430c4ec8b20724)
    },
    {   UINT64_C(0x0000000000005d3e), UINT64_C(0x1f42ff0f50b306c2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce9cb70717)
    },
    {   UINT64_C(0x0000000000005d3f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43148e60af872f),
        UINT64_C(0x1f42fa0f34b7c6f6), UINT64_C(0x1f42f7cf38b6e6ea)
    },
    {   UINT64_C(0x0000000000005d40), UINT64_C(0x1f42fdcf64b486ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f4cf30b7670f), UINT64_C(0x1f430d0ec4afe71e)
    },
    {   UINT64_C(0x0000000000005d41), UINT64_C(0x1f43020ff8b526e9), UINT64_C(0x1f42ff0f50b306ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0ea4abc725)
    },
    {   UINT64_C(0x0000000000005d42), UINT64_C(0x1f42fc0f18b8070c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cf20b80706)
    },
    {   UINT64_C(0x0000000000005d43), UINT64_C(0x1f43049048b846b6), UINT64_C(0x1f43110ebcb46741),
        UINT64_C(0x1f42f80ebcb66731), UINT64_C(0x1f4317ceccad2717)
    },
    {   UINT64_C(0x0000000000005d44), UINT64_C(0x1f42f9cf38b486fe), UINT64_C(0x1f43018f60b4870d),
        UINT64_C(0x1f42fe0f68b666df), UINT64_C(0x1f43064ed0b10734)
    },
    {   UINT64_C(0x0000000000005d45), UINT64_C(0x1f42f58ea4b62718), UINT64_C(0x1f43138e5cb36742),
        UINT64_C(0x1f4312ceb0b0c711), UINT64_C(0x1f42f84f64b7a71d)
    },
    {   UINT64_C(0x0000000000005d46), UINT64_C(0x1f42fecf88b20706), UINT64_C(0x1f4302ce9cb14725),
        UINT64_C(0x1f430b4f1cb46739), UINT64_C(0x1f42f80fb8b6a6da)
    },
    {   UINT64_C(0x0000000000005d47), UINT64_C(0x1f42f94fd8b6c6e5), UINT64_C(0x1f42f74e9cb2a709),
        UINT64_C(0x1f42f9ce30b5870e), UINT64_C(0x1f430d8f98b0e71c)
    },
    {   UINT64_C(0x0000000000005d48), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cfe0b7c704),
        UINT64_C(0x1f42fa8ee4afc72f), UINT64_C(0x1f43000ee4b2872f)
    },
    {   UINT64_C(0x0000000000005d49), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f3cad6709),
        UINT64_C(0x1f42ffcf58b6472c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d4a), UINT64_C(0x1f430ccf68a346f9), UINT64_C(0x1f430e8f9cb526de),
        UINT64_C(0x1f43078e54a2a72e), UINT64_C(0x1f430a8e58b0072c)
    },
    {   UINT64_C(0x0000000000005d4b), UINT64_C(0x1f4320cef89ea722), UINT64_C(0x1f43134e94a5471f),
        UINT64_C(0x1f43288ec4a0e70f), UINT64_C(0x1f43154edca9a724)
    },
    {   UINT64_C(0x0000000000005d4c), UINT64_C(0x1f431310d8ac06a1), UINT64_C(0x1f43158fcca9a708),
        UINT64_C(0x1f431b8ed8a6c6f3), UINT64_C(0x1f431cce78a4272c)
    },
    {   UINT64_C(0x0000000000005d4d), UINT64_C(0x0000000000000000), UINT64_C(0x1f431e0f70b1e6f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430ecee8a446fb)
    },
    {   UINT64_C(0x0000000000005d4e), UINT64_C(0x1f4310ce90a62738), UINT64_C(0x1f43114ec0ab8719),
        UINT64_C(0x1f4300500cb406bc), UINT64_C(0x1f4310cfdcaa26d3)
    },
    {   UINT64_C(0x0000000000005d4f), UINT64_C(0x1f430b8e6caf070f), UINT64_C(0x1f43084eeca7470e),
        UINT64_C(0x1f4321ce74a306fc), UINT64_C(0x1f42fd1094ae06c8)
    },
    {   UINT64_C(0x0000000000005d50), UINT64_C(0x1f43094e98ae06ee), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d51), UINT64_C(0x1f43100fa4a906fe), UINT64_C(0x1f43020f74b14712),
        UINT64_C(0x1f42ffd0a4b2c6d8), UINT64_C(0x1f42ff0f04ad26f6)
    },
    {   UINT64_C(0x0000000000014908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f80dc0780407)
    },
    {   UINT64_C(0x0000000000005d53), UINT64_C(0x1f42fc4e88b7c71e), UINT64_C(0x1f42f50e84b80708),
        UINT64_C(0x1f42fb0fd4b5c6f8), UINT64_C(0x1f4304cf28b50712)
    },
    {   UINT64_C(0x0000000000005d54), UINT64_C(0x0000000000000000), UINT64_C(0x1f43004fd8b266d8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d55), UINT64_C(0x1f42fb0ec8b7c73b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d56), UINT64_C(0x0000000000000000), UINT64_C(0x1f43188f20aa4724),
        UINT64_C(0x1f43028e94b62731), UINT64_C(0x1f42fb0eb4b9a72e)
    },
    {   UINT64_C(0x0000000000005d57), UINT64_C(0x1f4304cec4b0e737), UINT64_C(0x1f42fc1020b6c6ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43204e7cb3a70d)
    },
    {   UINT64_C(0x0000000000005d58), UINT64_C(0x1f43244f20a04736), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038e6cb16724), UINT64_C(0x1f4308ce94b52714)
    },
    {   UINT64_C(0x0000000000005d59), UINT64_C(0x1f42fb4ec0b1470a), UINT64_C(0x1f42fe0f28b9a71c),
        UINT64_C(0x1f43010f94b64719), UINT64_C(0x1f4306ce88af6725)
    },
    {   UINT64_C(0x0000000000005d5a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f48b7470e),
        UINT64_C(0x1f4302ce84b7873c), UINT64_C(0x1f430b4ee0af6748)
    },
    {   UINT64_C(0x0000000000005d5b), UINT64_C(0x1f4307cf2cb3872e), UINT64_C(0x1f42f40f08b8872c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f70b7872b)
    },
    {   UINT64_C(0x0000000000005d5c), UINT64_C(0x1f43044f14b1671c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060ec4b44737), UINT64_C(0x1f430b0ea4ad0713)
    },
    {   UINT64_C(0x0000000000005d5d), UINT64_C(0x1f42ff0e9caf0729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4e74afc730), UINT64_C(0x1f42f8d000b706df)
    },
    {   UINT64_C(0x0000000000005d5e), UINT64_C(0x1f42fa0e80b66737), UINT64_C(0x1f42f84f9cb56719),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d5f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cf64b6a70e),
        UINT64_C(0x1f4307cfb8af2709), UINT64_C(0x1f43068e28b1273e)
    },
    {   UINT64_C(0x0000000000005d60), UINT64_C(0x1f42ff4eecb4e6f8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43070eb8af0718), UINT64_C(0x1f42f90f6cb06725)
    },
    {   UINT64_C(0x0000000000005d61), UINT64_C(0x1f42feceacb4670c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018f90b14708), UINT64_C(0x1f43018e64b56748)
    },
    {   UINT64_C(0x0000000000005d62), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307ceb8aea72d),
        UINT64_C(0x1f430dce78b0072d), UINT64_C(0x1f4302d034b726ea)
    },
    {   UINT64_C(0x0000000000014919), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000001491a), UINT64_C(0x15baee0f5495669a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d65), UINT64_C(0x1f42f84f00a96726), UINT64_C(0x1f42fb8f0cb1872a),
        UINT64_C(0x1f42f84f28a70726), UINT64_C(0x1f42fe0ed8a3c71e)
    },
    {   UINT64_C(0x0000000000005d66), UINT64_C(0x1f42f60f64ad870a), UINT64_C(0x1f42f90ed0b06716),
        UINT64_C(0x1f42ff0f80b5c722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d67), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f24a78726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f34ab072a)
    },
    {   UINT64_C(0x0000000000005d68), UINT64_C(0x1f42fb0f3ca56726), UINT64_C(0x1f42f84f28aac726),
        UINT64_C(0x1f42fc8f88afe729), UINT64_C(0x1f42fa4f90af471a)
    },
    {   UINT64_C(0x0000000000005d69), UINT64_C(0x1f42f90f74b1871e), UINT64_C(0x1f42f6ce8cb4472e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f90ac6716)
    },
    {   UINT64_C(0x0000000000005d6a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0fd4b3a722), UINT64_C(0x1f43050f24b8071e)
    },
    {   UINT64_C(0x0000000000005d6b), UINT64_C(0x1f42fc0f88a52716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84f78b3471a), UINT64_C(0x1f42fb4f1cb0e726)
    },
    {   UINT64_C(0x0000000000005d6c), UINT64_C(0x1f42fa8ea8b26703), UINT64_C(0x1f42fa4f80ae071e),
        UINT64_C(0x1f42fb0f4cb20726), UINT64_C(0x1f42facf54b0c72e)
    },
    {   UINT64_C(0x0000000000005d6d), UINT64_C(0x1f42f94ef4a9c71e), UINT64_C(0x1f42f6cf38b38723),
        UINT64_C(0x1f42f88f24b0272e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d6e), UINT64_C(0x1f42f74ef4aee71e), UINT64_C(0x1f42f84e60b58737),
        UINT64_C(0x1f42f94f00b1a726), UINT64_C(0x1f42fe8ee8adc723)
    },
    {   UINT64_C(0x0000000000005d6f), UINT64_C(0x1f42f84ea4b84739), UINT64_C(0x1f42fb0fb0b0e722),
        UINT64_C(0x1f42fb0f84b4471e), UINT64_C(0x1f42fd8e4cb1872e)
    },
    {   UINT64_C(0x0000000000005d70), UINT64_C(0x1f42fa8ee0aa071b), UINT64_C(0x1f42ff8eaca6c713),
        UINT64_C(0x1f42f80f64aec71e), UINT64_C(0x1f42fa0f74a60713)
    },
    {   UINT64_C(0x0000000000005d71), UINT64_C(0x1f42face78a6271d), UINT64_C(0x1f42fb0f94aac71f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f94b0e722)
    },
    {   UINT64_C(0x0000000000005d72), UINT64_C(0x1f42fb0f4ca44721), UINT64_C(0x1f42fa0fb4abe722),
        UINT64_C(0x1f42f80fa0b52727), UINT64_C(0x1f42fbcf24a9271e)
    },
    {   UINT64_C(0x0000000000005d73), UINT64_C(0x1f42fc8e9caba715), UINT64_C(0x1f42fbceb4b5073c),
        UINT64_C(0x1f42facf44a74716), UINT64_C(0x1f42fbcef8a1e721)
    },
    {   UINT64_C(0x0000000000005d74), UINT64_C(0x1f42f90f08b02726), UINT64_C(0x1f42fd0e90ab6722),
        UINT64_C(0x1f42f84e6caa8726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d75), UINT64_C(0x1f42f74e5cb14748), UINT64_C(0x1f42facec0b7a743),
        UINT64_C(0x1f42f9cf94ade70e), UINT64_C(0x1f42f8ce68b0c72d)
    },
    {   UINT64_C(0x0000000000005d76), UINT64_C(0x1f42fa8f34ade72a), UINT64_C(0x1f42fe0f88b2471b),
        UINT64_C(0x1f42f78ebcaa0731), UINT64_C(0x1f42fd8f18a62731)
    },
    {   UINT64_C(0x0000000000005d77), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84eb0b22731),
        UINT64_C(0x1f42f98f3cad2725), UINT64_C(0x1f43034e34a88726)
    },
    {   UINT64_C(0x0000000000005d78), UINT64_C(0x1f42f70e88b0e729), UINT64_C(0x1f42fa8e80b02721),
        UINT64_C(0x1f42f84eeca68731), UINT64_C(0x1f42f5cf54b04739)
    },
    {   UINT64_C(0x0000000000005d79), UINT64_C(0x1f42fa4eb8afe72a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68e90af4723), UINT64_C(0x1f42fccf38ad2733)
    },
    {   UINT64_C(0x0000000000014930), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014931), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014932), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014933), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014934), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014935), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000005d80), UINT64_C(0x1f42f8cf60b4c729), UINT64_C(0x1f42f80e6cb06735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60e68b9c72a)
    },
    {   UINT64_C(0x0000000000014937), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014938), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x0000000000014939), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000001493a), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d85), UINT64_C(0x1f42f6ce7cac2725), UINT64_C(0x1f42f84f28b60729),
        UINT64_C(0x1f42fc8e94af0735), UINT64_C(0x1f42fa0f6cb2a715)
    },
    {   UINT64_C(0x0000000000005d86), UINT64_C(0x1f42f80ef4aa672d), UINT64_C(0x1f42f68ef4b48732),
        UINT64_C(0x1f42f6cf04b4272d), UINT64_C(0x1f42f74f04ae2739)
    },
    {   UINT64_C(0x0000000000005d87), UINT64_C(0x1f42fc4ef8b2c738), UINT64_C(0x1f42f98f00aaa72e),
        UINT64_C(0x1f42f84f70af873e), UINT64_C(0x1f43000e8cb42726)
    },
    {   UINT64_C(0x0000000000005d88), UINT64_C(0x1f42f54e50b5e73d), UINT64_C(0x1f42f98ee0b5873d),
        UINT64_C(0x1f42f44f60b56732), UINT64_C(0x1f42f68f38b1e731)
    },
    {   UINT64_C(0x0000000000005d89), UINT64_C(0x1f42f80eb4a86739), UINT64_C(0x1f42fbcf14ab671d),
        UINT64_C(0x1f42f90e98b26741), UINT64_C(0x1f42f84f00aee741)
    },
    {   UINT64_C(0x0000000000005d8a), UINT64_C(0x1f43000e8cb2c725), UINT64_C(0x1f42f54e90b58740),
        UINT64_C(0x1f42f8cf04b3e71e), UINT64_C(0x1f42f84f94ae8741)
    },
    {   UINT64_C(0x0000000000005d8b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e9cab071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f84b44712)
    },
    {   UINT64_C(0x0000000000005d8c), UINT64_C(0x1f42fb0ebca7a723), UINT64_C(0x1f42f9cf04b08720),
        UINT64_C(0x1f42f90f14b6872e), UINT64_C(0x1f42fa4ebcb4e735)
    },
    {   UINT64_C(0x0000000000014943), UINT64_C(0x16233d8ae498853c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014944), UINT64_C(0x1622eb4ae46f453c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014945), UINT64_C(0x16233d93cc988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014946), UINT64_C(0x1622eb53cc6f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014947), UINT64_C(0x16233d8ae4988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d92), UINT64_C(0x1f42f8cef0b60735), UINT64_C(0x1f42fa4eccafc73d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014949), UINT64_C(0x1622eb4ae46f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d94), UINT64_C(0x1f42f9ce84b2a721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84ec0b24729), UINT64_C(0x1f42fd4f60b6a715)
    },
    {   UINT64_C(0x000000000001494b), UINT64_C(0x1622eb4ae498653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d96), UINT64_C(0x1f42f98e30aee740), UINT64_C(0x1f42fb0e6cb68726),
        UINT64_C(0x1f42f98ed0a68732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001494d), UINT64_C(0x1622eb53cc986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d98), UINT64_C(0x1f42f70e9cb4a732), UINT64_C(0x1f42f84f44b6a72d),
        UINT64_C(0x1f42fa4e60b7273a), UINT64_C(0x1f42f8ce7cb1c71f)
    },
    {   UINT64_C(0x000000000001494f), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003651), UINT64_C(0x106af6cfa07bc667), UINT64_C(0x106af6cfa07bc667),
        UINT64_C(0x106af6cfa07bc667), UINT64_C(0x106af6cfa07bc667)
    },
    {   UINT64_C(0x0000000000005d9b), UINT64_C(0x1f42f8ce6cb4671f), UINT64_C(0x1f42fa4e64b0c72a),
        UINT64_C(0x1f42f5ce90b1a73a), UINT64_C(0x1f42f54f04a5e72e)
    },
    {   UINT64_C(0x0000000000005d9c), UINT64_C(0x1f42f44ea0b46739), UINT64_C(0x1f42f7cf4cb3e731),
        UINT64_C(0x1f42f74ed4ad4731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d9d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50e8cac2742),
        UINT64_C(0x1f42fcd390b4072e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d9e), UINT64_C(0x1f43030fc0b7072e), UINT64_C(0x1f4302cee0b3a715),
        UINT64_C(0x1f42fc4f40b4472e), UINT64_C(0x1f42ff0f84b78737)
    },
    {   UINT64_C(0x0000000000014955), UINT64_C(0x1623264ae4986599), UINT64_C(0x1622eb4ae46f4599),
        UINT64_C(0x1622eb4ae47ae53c), UINT64_C(0x1622eb4ae47ae599)
    },
    {   UINT64_C(0x0000000000005da0), UINT64_C(0x1f43188edca64732), UINT64_C(0x1f43114e2cb16715),
        UINT64_C(0x1f430cceccac4732), UINT64_C(0x1f43018f24b1c715)
    },
    {   UINT64_C(0x0000000000014957), UINT64_C(0x1622eb53cc7ae776), UINT64_C(0x1622eb52587ae776),
        UINT64_C(0x16233d8ae4988776), UINT64_C(0x1623264ae4988776)
    },
    {   UINT64_C(0x0000000000014958), UINT64_C(0x1623264ae4988776), UINT64_C(0x1622eb4ae46f4776),
        UINT64_C(0x1622eb4ae47ae776), UINT64_C(0x1622eb4ae47ae776)
    },
    {   UINT64_C(0x0000000000014959), UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb4ae498653c),
        UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb5258986776)
    },
    {   UINT64_C(0x000000000001495a), UINT64_C(0x1622eb53cc986776), UINT64_C(0x1622eb5258986776),
        UINT64_C(0x1622eb4ae4986776), UINT64_C(0x1622eb4ae4986776)
    },
    {   UINT64_C(0x000000000001495b), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005da6), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a8ed4abc706),
        UINT64_C(0x1f43094f7cb3c726), UINT64_C(0x1f43108f00b04718)
    },
    {   UINT64_C(0x0000000000005da7), UINT64_C(0x1f43028ee0ade726), UINT64_C(0x1f43058f28b5a6ee),
        UINT64_C(0x1f43084eacb78724), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005da9), UINT64_C(0x1f42fe8ed4b4470f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43020f10b526f3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005daa), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43034e6cb4471b), UINT64_C(0x1f43074f38b0e729)
    },
    {   UINT64_C(0x0000000000005dab), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024e84b3872f),
        UINT64_C(0x1f43028f18b68719), UINT64_C(0x1f42f84e5cb68701)
    },
    {   UINT64_C(0x0000000000005dac), UINT64_C(0x1f42fb4f84b7270f), UINT64_C(0x1f4309cf80b5270a),
        UINT64_C(0x1f42ff8fa4b4a6fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dad), UINT64_C(0x1f43018fb4b68710), UINT64_C(0x1f42ff8f70b6270e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f54b6870a)
    },
    {   UINT64_C(0x0000000000005dae), UINT64_C(0x1f42fc8ef8b5270a), UINT64_C(0x1f42fa8f24b526fb),
        UINT64_C(0x1f43024f9cb5472e), UINT64_C(0x1f42f58f50b526f5)
    },
    {   UINT64_C(0x0000000000005daf), UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cf2cb72706),
        UINT64_C(0x1f43094ee4b6270e), UINT64_C(0x1f42fc91ccb4c63e)
    },
    {   UINT64_C(0x0000000000005db0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008ff0b2870e), UINT64_C(0x1f43048f74b2c705)
    },
    {   UINT64_C(0x0000000000005db1), UINT64_C(0x1f43078f8cafc6f4), UINT64_C(0x1f42fecf54b3c715),
        UINT64_C(0x1f43038f48b86709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005db2), UINT64_C(0x1f43068f78b2c6fc), UINT64_C(0x1f43138f50b1a712),
        UINT64_C(0x1f43000fd4b3a72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005db3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43064ef4b2872d),
        UINT64_C(0x1f42ffcef8b12720), UINT64_C(0x1f430e8f7cab6714)
    },
    {   UINT64_C(0x0000000000005db4), UINT64_C(0x1f430f8f60acc718), UINT64_C(0x1f4304cfccb046f4),
        UINT64_C(0x1f4302cf40b8e728), UINT64_C(0x1f430f8f54acc6e4)
    },
    {   UINT64_C(0x0000000000005db5), UINT64_C(0x1f43068f44b1a724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a8facb04733), UINT64_C(0x1f430f8f6cacc71c)
    },
    {   UINT64_C(0x0000000000005db6), UINT64_C(0x1f4311501cab672e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4ee0ace717), UINT64_C(0x1f43088f04afc72c)
    },
    {   UINT64_C(0x0000000000005db7), UINT64_C(0x1f4301cf5cb1471b), UINT64_C(0x1f43154f10ac472d),
        UINT64_C(0x1f430bcf30b1a727), UINT64_C(0x1f43058f8cb12707)
    },
    {   UINT64_C(0x0000000000005db8), UINT64_C(0x0000000000000000), UINT64_C(0x1f43100f10b0e734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8fecb2271a)
    },
    {   UINT64_C(0x0000000000005db9), UINT64_C(0x1f42fc8f58b76743), UINT64_C(0x1f4306cfb4b48721),
        UINT64_C(0x1f43030ef8b42721), UINT64_C(0x1f42fa0f08aec72f)
    },
    {   UINT64_C(0x0000000000005dba), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8f74b4670e),
        UINT64_C(0x1f43124f38adc6ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dbb), UINT64_C(0x1f430d4f28ac46fb), UINT64_C(0x1f4311cf38b2872d),
        UINT64_C(0x1f432f10089966e2), UINT64_C(0x1f43098f3cb5e6a1)
    },
    {   UINT64_C(0x0000000000005dbc), UINT64_C(0x1f43000f38b226d9), UINT64_C(0x1f43020f4cb226d9),
        UINT64_C(0x1f43058e80af0736), UINT64_C(0x1f43049068b586c7)
    },
    {   UINT64_C(0x0000000000005dbd), UINT64_C(0x1f43058f18af8712), UINT64_C(0x1f43034f18b1a70b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a0f48ac6714)
    },
    {   UINT64_C(0x0000000000005dbe), UINT64_C(0x1f42fbcf28a86724), UINT64_C(0x1f43034f8cb226f8),
        UINT64_C(0x1f430a0e6caf26d7), UINT64_C(0x1f43060f0cb726ed)
    },
    {   UINT64_C(0x0000000000005dbf), UINT64_C(0x1f43050f38b7a6fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044eb0af871d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dc0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cea0a9472b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43074ffcadc683)
    },
    {   UINT64_C(0x0000000000005dc1), UINT64_C(0x1f430a0efcaa66d1), UINT64_C(0x1f430a8f68b4e695),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43080e80b2070e)
    },
    {   UINT64_C(0x0000000000005dc2), UINT64_C(0x1f43074ffcb626e3), UINT64_C(0x1f42fc8f0ca10719),
        UINT64_C(0x1f43059030b0e6e1), UINT64_C(0x1f42ff0ea0ada72a)
    },
    {   UINT64_C(0x0000000000005dc3), UINT64_C(0x1f43140e50a88733), UINT64_C(0x1f430b0ffcb0c6cc),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dc4), UINT64_C(0x1f43065004b64694), UINT64_C(0x1f4307cf84b3c6d9),
        UINT64_C(0x1f43084e5cad0715), UINT64_C(0x1f4307cf10b3a6e4)
    },
    {   UINT64_C(0x0000000000005dc5), UINT64_C(0x0000000000000000), UINT64_C(0x1f43064e74b9a720),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dc6), UINT64_C(0x0000000000000000), UINT64_C(0x1f43064f78ade6ff),
        UINT64_C(0x1f430f4e80b00737), UINT64_C(0x1f43064ebcb6c702)
    },
    {   UINT64_C(0x0000000000005dc7), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074eacb6470a),
        UINT64_C(0x1f430d4e8cac0738), UINT64_C(0x1f43050ee8b00729)
    },
    {   UINT64_C(0x0000000000005dc8), UINT64_C(0x1f43085014b206f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048ea0ad872c), UINT64_C(0x1f43054f60b8c6f9)
    },
    {   UINT64_C(0x0000000000005dc9), UINT64_C(0x1f42fa4f94b566e2), UINT64_C(0x1f43090e70aae71d),
        UINT64_C(0x1f4309d0a0b3c6e9), UINT64_C(0x1f430b8fe8a706d4)
    },
    {   UINT64_C(0x0000000000005dca), UINT64_C(0x1f43010fa0b806fb), UINT64_C(0x1f430a106caf4713),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dcb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43091024afc715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cf38b546ff)
    },
    {   UINT64_C(0x0000000000005dcc), UINT64_C(0x0000000000000000), UINT64_C(0x1f430bcf10b2c70f),
        UINT64_C(0x1f430c0e8cb2e6f8), UINT64_C(0x1f430b0fc8ab2709)
    },
    {   UINT64_C(0x0000000000005dcd), UINT64_C(0x1f430bcf1cab271e), UINT64_C(0x1f430fcef4ab472c),
        UINT64_C(0x1f43084f80b266da), UINT64_C(0x1f42fd8e94b72731)
    },
    {   UINT64_C(0x0000000000005dce), UINT64_C(0x1f43080e9cb46727), UINT64_C(0x1f43075044b3c71e),
        UINT64_C(0x1f43055028b40707), UINT64_C(0x1f430c4e7cb2c72f)
    },
    {   UINT64_C(0x0000000000005dcf), UINT64_C(0x1f430b4f54b82701), UINT64_C(0x1f43090f10b5c710),
        UINT64_C(0x1f43060f78a7a701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dd0), UINT64_C(0x1f43004ed0aa8735), UINT64_C(0x1f43090eb4b1472e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a0f9cb4a71f)
    },
    {   UINT64_C(0x0000000000005dd1), UINT64_C(0x1f430f4ef0ace724), UINT64_C(0x1f43064e9cade703),
        UINT64_C(0x1f43028fc8b18712), UINT64_C(0x1f43064e68b16730)
    },
    {   UINT64_C(0x0000000000005dd2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f4e70a766e9), UINT64_C(0x1f43028f4cae471f)
    },
    {   UINT64_C(0x0000000000005dd3), UINT64_C(0x1f43038ea8af872b), UINT64_C(0x1f43058f0cb42703),
        UINT64_C(0x1f43088e2caee722), UINT64_C(0x1f430fcff8aca694)
    },
    {   UINT64_C(0x0000000000005dd4), UINT64_C(0x1f43090ee8b5c712), UINT64_C(0x1f43070ffcb246e0),
        UINT64_C(0x1f43040eecb5c70d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dd5), UINT64_C(0x1f43050eecb8e6fe), UINT64_C(0x1f43034f8cb4e6f2),
        UINT64_C(0x1f43034efcb7671e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dd6), UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f2cb8a715),
        UINT64_C(0x1f43028ff8b2e6d8), UINT64_C(0x1f430c4e80b5673c)
    },
    {   UINT64_C(0x0000000000005dd7), UINT64_C(0x1f43090eb4b4673a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048eacb7c726), UINT64_C(0x1f4302ce6cb36726)
    },
    {   UINT64_C(0x0000000000005dd8), UINT64_C(0x1f42ff8f04b94718), UINT64_C(0x1f4308cff4b246ff),
        UINT64_C(0x1f43094efcb20738), UINT64_C(0x1f4300ce50b6472c)
    },
    {   UINT64_C(0x0000000000005dd9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43080e6cb4c731),
        UINT64_C(0x1f430e4f70afa702), UINT64_C(0x1f43058ed0b326fa)
    },
    {   UINT64_C(0x0000000000005dda), UINT64_C(0x1f43048f48b766f9), UINT64_C(0x1f43028ebcb6471b),
        UINT64_C(0x1f43094eacad6724), UINT64_C(0x1f430a4f18b76736)
    },
    {   UINT64_C(0x0000000000005ddb), UINT64_C(0x1f43084ef8b6a716), UINT64_C(0x1f43058f08aec6e1),
        UINT64_C(0x1f43020e7cb8670c), UINT64_C(0x1f43114e84b3e727)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f85), UINT64_C(0x1f43030edcb50702), UINT64_C(0x1f4306d04cb2c6ee),
        UINT64_C(0x1f43098f4cad6707), UINT64_C(0x1f43094f5cb60724)
    },
    {   UINT64_C(0x0000000000005dde), UINT64_C(0x0000000000000000), UINT64_C(0x1f430bced0b38722),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ddf), UINT64_C(0x1f43030f10b7e6dd), UINT64_C(0x1f43044f78b786ff),
        UINT64_C(0x1f43170e4caa0732), UINT64_C(0x1f43038e48b7a729)
    },
    {   UINT64_C(0x0000000000005de0), UINT64_C(0x1f4309ce80b36726), UINT64_C(0x1f43040f1cb0e6f3),
        UINT64_C(0x1f42f68f80b4a6f3), UINT64_C(0x1f4306cf00acc702)
    },
    {   UINT64_C(0x0000000000005de1), UINT64_C(0x1f43050f80ad06f5), UINT64_C(0x1f42fb4f6cb8a6d8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f98b386e8)
    },
    {   UINT64_C(0x0000000000005de2), UINT64_C(0x1f4305cef0b5072c), UINT64_C(0x1f43000f50b6071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430bcf00b3072f)
    },
    {   UINT64_C(0x0000000000005de3), UINT64_C(0x1f43064f58b60721), UINT64_C(0x1f430bcf84b42715),
        UINT64_C(0x1f43094f38b24725), UINT64_C(0x1f4312ceb0b0070b)
    },
    {   UINT64_C(0x0000000000005de4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044facade6f0),
        UINT64_C(0x1f43098ed0ae4702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005de5), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070e9cac8735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43074ea0ade71d)
    },
    {   UINT64_C(0x0000000000005de6), UINT64_C(0x0000000000000000), UINT64_C(0x1f43080f44b0e70b),
        UINT64_C(0x1f43090efcb0e70a), UINT64_C(0x1f43038fb4b826e4)
    },
    {   UINT64_C(0x0000000000005de7), UINT64_C(0x1f4306cef4afc728), UINT64_C(0x1f4307cef0ae6715),
        UINT64_C(0x1f43020f68ae071f), UINT64_C(0x1f43054ec0aee6fa)
    },
    {   UINT64_C(0x0000000000005de8), UINT64_C(0x1f4305cfacb08716), UINT64_C(0x1f43014f74b866d3),
        UINT64_C(0x1f43084e70aea737), UINT64_C(0x1f43080f98b006f9)
    },
    {   UINT64_C(0x0000000000005de9), UINT64_C(0x0000000000000000), UINT64_C(0x1f4310ce80b46737),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040f88b1470a)
    },
    {   UINT64_C(0x0000000000005dea), UINT64_C(0x1f43060f1cb1a735), UINT64_C(0x1f43064f24ad06e2),
        UINT64_C(0x1f43028f88b6a718), UINT64_C(0x1f43040e9caf670e)
    },
    {   UINT64_C(0x0000000000005deb), UINT64_C(0x1f43054ef8b0271f), UINT64_C(0x1f42fe8e68b86742),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005dec), UINT64_C(0x1f43000f58b72728), UINT64_C(0x1f43068eb8af8733),
        UINT64_C(0x1f43080ea0acc6f6), UINT64_C(0x1f4305cf20b0072e)
    },
    {   UINT64_C(0x0000000000005ded), UINT64_C(0x1f43080fc4ac86ef), UINT64_C(0x1f43060edcafc705),
        UINT64_C(0x1f43054ee0b06715), UINT64_C(0x1f43054f54aaa6e4)
    },
    {   UINT64_C(0x0000000000005dee), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8ee4b3a714),
        UINT64_C(0x1f43028e94b6c723), UINT64_C(0x1f43098f04b2673a)
    },
    {   UINT64_C(0x0000000000005def), UINT64_C(0x1f43010f58b64718), UINT64_C(0x1f43048e44b36741),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4e7caa073b)
    },
    {   UINT64_C(0x0000000000005df0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048eecb2c6f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005df1), UINT64_C(0x1f4308cf38adc70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028ee0b6e742)
    },
    {   UINT64_C(0x0000000000005df2), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4ed0b42731),
        UINT64_C(0x1f4303cef8ac0717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005df3), UINT64_C(0x1f43060efcb0e731), UINT64_C(0x1f43008e14afa72b),
        UINT64_C(0x1f4307ce64b5c74c), UINT64_C(0x1f4305cedcada737)
    },
    {   UINT64_C(0x0000000000005df4), UINT64_C(0x1f4305ce6caee6f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030fa0b62705)
    },
    {   UINT64_C(0x0000000000005df5), UINT64_C(0x1f43010e6cb08742), UINT64_C(0x1f430e0e50aac73e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040e7cb2273f)
    },
    {   UINT64_C(0x0000000000005df6), UINT64_C(0x1f4306cebcb26739), UINT64_C(0x1f4301cea8b18700),
        UINT64_C(0x1f4303ce5cb10737), UINT64_C(0x1f4311cee0b16711)
    },
    {   UINT64_C(0x0000000000005df7), UINT64_C(0x1f42f44f0cb9a707), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43128e7ca8671a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005df8), UINT64_C(0x1f42f68ec4b50722), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43140fa8b4e713), UINT64_C(0x1f430d4ed8b36731)
    },
    {   UINT64_C(0x0000000000005df9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb900cb10705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4fa4a886eb)
    },
    {   UINT64_C(0x0000000000005dfa), UINT64_C(0x1f43061034b546e6), UINT64_C(0x1f42fbd00cb206e9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f28b5c6f6)
    },
    {   UINT64_C(0x0000000000005dfb), UINT64_C(0x1f4301cff8b246d6), UINT64_C(0x1f42f9905cb5c6eb),
        UINT64_C(0x1f42fb8f84b2c6f4), UINT64_C(0x1f42f6cebcba4725)
    },
    {   UINT64_C(0x0000000000005dfc), UINT64_C(0x1f42fe4ed4b50727), UINT64_C(0x1f42ff8ed4b6c735),
        UINT64_C(0x1f42fbcf90b426fc), UINT64_C(0x1f42fe5024a526c3)
    },
    {   UINT64_C(0x0000000000005dfd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9d0fcb5a6a2), UINT64_C(0x1f42f8cf28b486b2)
    },
    {   UINT64_C(0x0000000000005dfe), UINT64_C(0x1f42f6ceacb526c1), UINT64_C(0x1f42f8d03cb7667e),
        UINT64_C(0x1f42f2cfc4b946cc), UINT64_C(0x1f42f6cfbcb4a717)
    },
    {   UINT64_C(0x0000000000005dff), UINT64_C(0x1f42f5118cb4669d), UINT64_C(0x1f42f30f4cad8708),
        UINT64_C(0x1f42f9cf6cb70699), UINT64_C(0x1f42f4cf88ad270b)
    },
    {   UINT64_C(0x0000000000005e00), UINT64_C(0x1f42f8cf5cb026c3), UINT64_C(0x1f42f4cf1caba6e0),
        UINT64_C(0x1f42fb0f1cb826fe), UINT64_C(0x1f42f8cf2cb486c8)
    },
    {   UINT64_C(0x0000000000005e01), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf8cb786dd),
        UINT64_C(0x1f42f5cfd0b5c6e1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e02), UINT64_C(0x0000000000000000), UINT64_C(0x1f430acf3ca14700),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e03), UINT64_C(0x1f42f8cf40b4e719), UINT64_C(0x1f42f7d050b5c702),
        UINT64_C(0x1f42f5cf18b806f3), UINT64_C(0x1f42fa8f94b7c6de)
    },
    {   UINT64_C(0x0000000000005e04), UINT64_C(0x1f42f5cff0b366ad), UINT64_C(0x1f42f8cf08b006cd),
        UINT64_C(0x1f42f7d0bcb7e6df), UINT64_C(0x1f42f6cfa8b506db)
    },
    {   UINT64_C(0x0000000000005e05), UINT64_C(0x1f42f4ce9cb4e6b5), UINT64_C(0x1f42f54f70b7e724),
        UINT64_C(0x1f42f40f1caf0709), UINT64_C(0x1f42f90ebcb4671d)
    },
    {   UINT64_C(0x0000000000005e06), UINT64_C(0x1f42f7cfd4b62714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5cfa4b5c6db), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e07), UINT64_C(0x1f42f4cf5cb846d9), UINT64_C(0x1f42f2cf90b6e6f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ed8ab0714)
    },
    {   UINT64_C(0x0000000000005e08), UINT64_C(0x1f42f4cfd4b5a6a7), UINT64_C(0x1f42fb90a8af8699),
        UINT64_C(0x1f42f9cf28b186ba), UINT64_C(0x1f42fbcfe4b1a6fa)
    },
    {   UINT64_C(0x0000000000005e09), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cff0b726ee),
        UINT64_C(0x1f43040e90af872f), UINT64_C(0x1f42f5cfe8ace705)
    },
    {   UINT64_C(0x00000000000149c0), UINT64_C(0x0000000000000000), UINT64_C(0x1e72f6d0c0b3e655),
        UINT64_C(0x1e0af6d0e4b2664b), UINT64_C(0x1e72f6d0c0b3e655)
    },
    {   UINT64_C(0x0000000000005e0b), UINT64_C(0x1f42fad088b166f3), UINT64_C(0x1f42f20fe4b6a6e7),
        UINT64_C(0x1f430a0f1cb44731), UINT64_C(0x1f42f7cefcb6c6ed)
    },
    {   UINT64_C(0x0000000000005e0c), UINT64_C(0x1f42f4ce8cb3e6c3), UINT64_C(0x1f42f6cea8b54703),
        UINT64_C(0x1f42f8cf88b58719), UINT64_C(0x1f42f91018ad0674)
    },
    {   UINT64_C(0x00000000000149c3), UINT64_C(0x112af54f747f06a9), UINT64_C(0x1abaf6908ca50660),
        UINT64_C(0x1c7af6d068ac066a), UINT64_C(0x1d2af6d1ccaee60e)
    },
    {   UINT64_C(0x00000000000149c4), UINT64_C(0x1c7af6d068ac066a), UINT64_C(0x1d8af6d1ccb02611),
        UINT64_C(0x1e72f6d140b3c633), UINT64_C(0x179af68f6c98a6a9)
    },
    {   UINT64_C(0x00000000000149c5), UINT64_C(0x1a72f70f68a3e6a9), UINT64_C(0x17d2f6cfa099669c),
        UINT64_C(0x17caf6cfa099669c), UINT64_C(0x11daedce9085e67d)
    },
    {   UINT64_C(0x00000000000149c6), UINT64_C(0x152af70f6c8ec6a9), UINT64_C(0x17daf6cfa099869c),
        UINT64_C(0x17c2f6cfa099469c), UINT64_C(0x17baf6cfa099269c)
    },
    {   UINT64_C(0x00000000000149c7), UINT64_C(0x1792f6cfa098869c), UINT64_C(0x0f72f6cfa078069c),
        UINT64_C(0x1142f6cfa07f269c), UINT64_C(0x10caf6cfa07d669c)
    },
    {   UINT64_C(0x00000000000149c8), UINT64_C(0x120af6d12082463c), UINT64_C(0x15a2f6cf6890a6a9),
        UINT64_C(0x1892f6cf689c66a9), UINT64_C(0x18a2f6cf689c86a9)
    },
    {   UINT64_C(0x00000000000149c9), UINT64_C(0x18a2f6cf689ca6a9), UINT64_C(0x18b2f6cf689d06a9),
        UINT64_C(0x18caf6cf6c9d66a9), UINT64_C(0x18d2f6cf6c9d66a9)
    },
    {   UINT64_C(0x00000000000149ca), UINT64_C(0x0000000000000000), UINT64_C(0x19baf6cf68a0e6a9),
        UINT64_C(0x18aaf6cf689cc6a9), UINT64_C(0x1a4af6cf68a366a9)
    },
    {   UINT64_C(0x00000000000149cb), UINT64_C(0x19baf6cf68a126a9), UINT64_C(0x1a0af6cf68a246a9),
        UINT64_C(0x1982f6cf6ca046a9), UINT64_C(0x19faf6cf68a206a9)
    },
    {   UINT64_C(0x00000000000149cc), UINT64_C(0x1cdaf6cf68ad86a9), UINT64_C(0x1742f6cfa097469c),
        UINT64_C(0x169af6cfa094869c), UINT64_C(0x176af6cfa097c69c)
    },
    {   UINT64_C(0x00000000000149cd), UINT64_C(0x18b2f68f689d06a9), UINT64_C(0x18c2f6cf689d46a9),
        UINT64_C(0x15baf6cf689126aa), UINT64_C(0x1842f6cf889b26a3)
    },
    {   UINT64_C(0x0000000000005e18), UINT64_C(0x1f42f8cec8b78717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6cee8b70727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149cf), UINT64_C(0x1622f6cf689286a9), UINT64_C(0x1552f6cf688f66a9),
        UINT64_C(0x15eaf6cf6891c6a9), UINT64_C(0x19d2f6cf68a166a9)
    },
    {   UINT64_C(0x00000000000149d0), UINT64_C(0x197af6cf68a006a9), UINT64_C(0x18aaf6cf689cc6a9),
        UINT64_C(0x18aaf6cf689cc6a9), UINT64_C(0x161af68f6892a6a9)
    },
    {   UINT64_C(0x0000000000005e1b), UINT64_C(0x1f42f7cfc4b726f8), UINT64_C(0x1f42f5d064aee708),
        UINT64_C(0x1f42f9ce70aea72c), UINT64_C(0x1f42f0100cb7e6df)
    },
    {   UINT64_C(0x00000000000149d2), UINT64_C(0x189af6cf689c86a9), UINT64_C(0x164af6cf689346a9),
        UINT64_C(0x18baf6cf689d06a9), UINT64_C(0x188af70f689c66a9)
    },
    {   UINT64_C(0x00000000000149d3), UINT64_C(0x0000000000000000), UINT64_C(0x1b4af6cf68a746a9),
        UINT64_C(0x0000000000000000), UINT64_C(0x17d2f6cfa099669c)
    },
    {   UINT64_C(0x0000000000005e1e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50f0cb826ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4310cebcb006e9)
    },
    {   UINT64_C(0x00000000000149d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1882f6cf689c26a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149d6), UINT64_C(0x0452f6cfa04b869c), UINT64_C(0x08aaf6cfa05cc69c),
        UINT64_C(0x0cfaf6cfa06e069c), UINT64_C(0x0c42f6d3bc6b26a9)
    },
    {   UINT64_C(0x00000000000149d7), UINT64_C(0x0c42f693bc6b26a9), UINT64_C(0x14e2f6d3bc8da6a9),
        UINT64_C(0x14e2f713bc8dc6a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149d8), UINT64_C(0x0000000000000000), UINT64_C(0x16e2f6cd6495a70e),
        UINT64_C(0x1102fc0f687b86aa), UINT64_C(0x1102f68f687e06a9)
    },
    {   UINT64_C(0x00000000000149d9), UINT64_C(0x1c72f6d040abe673), UINT64_C(0x14daf6cf688da6a9),
        UINT64_C(0x17c2f6cf689946a9), UINT64_C(0x17c2ee11849d4621)
    },
    {   UINT64_C(0x0000000000005e24), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f2ce3cb6472f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4ceecb36727)
    },
    {   UINT64_C(0x000000000000d400), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x1162f18c747b669b),
        UINT64_C(0x0aa30d8cac59469b), UINT64_C(0x10c2fd0fa0804682)
    },
    {   UINT64_C(0x000000000000d401), UINT64_C(0x0c52f90fd06a4632), UINT64_C(0x164af90fd0922632),
        UINT64_C(0x2042f90fd0ba0632), UINT64_C(0x0c52f90fd06a4632)
    },
    {   UINT64_C(0x00000000000149dd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000149de), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000149df), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000149e0), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000149e1), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000149e2), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x000000000000d408), UINT64_C(0x1773020b0c9246c0), UINT64_C(0x1773020d08924688),
        UINT64_C(0x1773020d08924688), UINT64_C(0x1773020bf09246ce)
    },
    {   UINT64_C(0x000000000000d409), UINT64_C(0x1773020bf09246ce), UINT64_C(0x0d4afccaa078e720),
        UINT64_C(0x165afccaa09d2720), UINT64_C(0x1f6afccaa0c16720)
    },
    {   UINT64_C(0x000000000000d40a), UINT64_C(0x0c52f6cac87ae72a), UINT64_C(0x1452f6caa09ae720),
        UINT64_C(0x1dbafb8aa0c16720), UINT64_C(0x0f22fb8aa083a720)
    },
    {   UINT64_C(0x000000000000d40b), UINT64_C(0x0deaf6caa0814720), UINT64_C(0x0e42f6caa082a720),
        UINT64_C(0x287afccaa0e5a720), UINT64_C(0x0ffafccaa083a720)
    },
    {   UINT64_C(0x000000000000d40c), UINT64_C(0x0ffafccaa083a720), UINT64_C(0x1282fccaa08dc720),
        UINT64_C(0x0c0af6caa079c720), UINT64_C(0x0f22fb8aa083a720)
    },
    {   UINT64_C(0x000000000000d40d), UINT64_C(0x0d0af6caa07b4720), UINT64_C(0x0d42fb8aa07c2720),
        UINT64_C(0x10bafb8aa08a0720), UINT64_C(0x0d02f6caa07da720)
    },
    {   UINT64_C(0x000000000000d40e), UINT64_C(0x0e5af6caa0808720), UINT64_C(0x19c2f94aa0ad0720),
        UINT64_C(0x107af94aa087e720), UINT64_C(0x11baf94aa08d8720)
    },
    {   UINT64_C(0x000000000000d40f), UINT64_C(0x11baf94aa08d8720), UINT64_C(0x0efaf94aa07fa7a7),
        UINT64_C(0x0cdaf6c8847e6720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149eb), UINT64_C(0x1b1af6d134a68637), UINT64_C(0x15c2f6d050912670),
        UINT64_C(0x15c2f6d050912670), UINT64_C(0x1b52f6d0d4a78653)
    },
    {   UINT64_C(0x00000000000149ec), UINT64_C(0x0000000000000000), UINT64_C(0x1b52f6d0c4a7864f),
        UINT64_C(0x17c2f6cfa099269c), UINT64_C(0x1d92f6d220b065fd)
    },
    {   UINT64_C(0x00000000000149ed), UINT64_C(0x181af690349a8677), UINT64_C(0x1b0af6d2dca645c9),
        UINT64_C(0x181af690349a8677), UINT64_C(0x1bc2f68f54a926b0)
    },
    {   UINT64_C(0x0000000000005e38), UINT64_C(0x1f42f50f5cace70a), UINT64_C(0x1f42f6cfccb4c6f7),
        UINT64_C(0x1f42f6cf58b0e70b), UINT64_C(0x1f42fc0e70b7271e)
    },
    {   UINT64_C(0x00000000000149ef), UINT64_C(0x1cfaf6d1f8ade606), UINT64_C(0x1e52f6d0d0b2e650),
        UINT64_C(0x1cb2f6912cad0639), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e3a), UINT64_C(0x1f42fe4ef0aee73e), UINT64_C(0x1f42f18eecb726f2),
        UINT64_C(0x1f42fbcee8b64746), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e3b), UINT64_C(0x1f42f9cee0b36716), UINT64_C(0x1f42f4ce98b6c713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce50b0a725)
    },
    {   UINT64_C(0x0000000000005e3c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f50edcb6e707), UINT64_C(0x1f42f74ea0b3a72e)
    },
    {   UINT64_C(0x0000000000005fb5), UINT64_C(0x1f43070eb8b54721), UINT64_C(0x1f4304cf88b5a719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43038fb0b3a6eb)
    },
    {   UINT64_C(0x0000000000005e3e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50e9cb3e725),
        UINT64_C(0x1f42f5ceccb5472e), UINT64_C(0x1f42f60f5cba0717)
    },
    {   UINT64_C(0x0000000000005e3f), UINT64_C(0x1f42f70e8cb8271d), UINT64_C(0x1f42f5cf4cb7e749),
        UINT64_C(0x1f42f60e6cb4c709), UINT64_C(0x1f42fb8f3cac66ff)
    },
    {   UINT64_C(0x0000000000005e40), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf78b4c705),
        UINT64_C(0x1f42f60edcb0e72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e41), UINT64_C(0x1f42f5cf24b7c70b), UINT64_C(0x1f42f4cef0b72712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f84b6e72d)
    },
    {   UINT64_C(0x0000000000005e42), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40f58b6a6fd),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e43), UINT64_C(0x1f42f40f40b346f0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5cfd0b6e734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e44), UINT64_C(0x1f430c8eecb2c6f3), UINT64_C(0x1f42f40f40b1a71a),
        UINT64_C(0x1f42f30ef4b2c727), UINT64_C(0x1f42f30f60b0c6ea)
    },
    {   UINT64_C(0x0000000000005e45), UINT64_C(0x1f42f30ee8b006d3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cea4b3a721)
    },
    {   UINT64_C(0x0000000000005e46), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4d06cb866ed),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cfc0b506cb)
    },
    {   UINT64_C(0x0000000000005e47), UINT64_C(0x1f43024f6cb60730), UINT64_C(0x1f42f6ced0aee6de),
        UINT64_C(0x1f42f6cea8b7c72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e48), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0ee8b5072b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf28b20720)
    },
    {   UINT64_C(0x0000000000005e49), UINT64_C(0x1f42f40eb0b68710), UINT64_C(0x1f42f5cec4b606e7),
        UINT64_C(0x1f42fa0ec0b1c72d), UINT64_C(0x1f42fb1084b7c6e0)
    },
    {   UINT64_C(0x0000000000005e4a), UINT64_C(0x1f430ecea0a8070a), UINT64_C(0x1f42f5cea0af6715),
        UINT64_C(0x1f42f90f10b306ff), UINT64_C(0x1f42f74e7cb7c72c)
    },
    {   UINT64_C(0x0000000000005e4b), UINT64_C(0x1f43050f0cb24714), UINT64_C(0x1f42f8cee8af8702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ce50b6071e)
    },
    {   UINT64_C(0x0000000000005e4c), UINT64_C(0x0000000000000000), UINT64_C(0x1f4308ceb0b24743),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e4d), UINT64_C(0x1f42f30e80b06713), UINT64_C(0x1f42f6cec0b58701),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e4e), UINT64_C(0x1f42f7ce50b50731), UINT64_C(0x1f42f8cf38b6a716),
        UINT64_C(0x1f430d4f68ae86ce), UINT64_C(0x1f43284ef890e706)
    },
    {   UINT64_C(0x0000000000005e4f), UINT64_C(0x1f43034f64b566ef), UINT64_C(0x1f42f78f70af6702),
        UINT64_C(0x1f42f98ef8b746fa), UINT64_C(0x1f42ff4f04ab66ef)
    },
    {   UINT64_C(0x0000000000005e50), UINT64_C(0x1f42fa8ef8b58706), UINT64_C(0x1f42fd4ec8ac8721),
        UINT64_C(0x1f42fa8f74b566f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e51), UINT64_C(0x1f42f98f28b786e6), UINT64_C(0x1f42fc8f94b0e6e7),
        UINT64_C(0x1f42fd4f44b3c6e7), UINT64_C(0x1f42f58fc4b62712)
    },
    {   UINT64_C(0x0000000000005e52), UINT64_C(0x1f43004e8cb64716), UINT64_C(0x1f42fb8f54b3e6fb),
        UINT64_C(0x1f42ff4f84ace6eb), UINT64_C(0x1f42fc8f54b406df)
    },
    {   UINT64_C(0x0000000000005e53), UINT64_C(0x1f42ff4f24aca6f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f84aae6ff)
    },
    {   UINT64_C(0x0000000000005e54), UINT64_C(0x1f42fb8fb4ade700), UINT64_C(0x1f42ff4f84b4c6f6),
        UINT64_C(0x1f42fd8f6cad06f2), UINT64_C(0x1f42fc8f94b5a6f7)
    },
    {   UINT64_C(0x0000000000005e55), UINT64_C(0x1f42fb8ee0b38702), UINT64_C(0x1f42f98f54b346f9),
        UINT64_C(0x1f42fb8f64b7270a), UINT64_C(0x1f42f88f94b7270a)
    },
    {   UINT64_C(0x0000000000005e56), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8ea8b0e72a), UINT64_C(0x1f42fd9000b3670a)
    },
    {   UINT64_C(0x0000000000005e57), UINT64_C(0x1f42ff4fa4af26fb), UINT64_C(0x1f42fe4fc4b8270e),
        UINT64_C(0x1f42f98e7cb1c72c), UINT64_C(0x1f43014e44b08731)
    },
    {   UINT64_C(0x0000000000005e58), UINT64_C(0x1f42f68eecab470f), UINT64_C(0x1f43124e80a8e71a),
        UINT64_C(0x1f42f78f48ac8719), UINT64_C(0x1f42fd8f30b5070f)
    },
    {   UINT64_C(0x0000000000005e59), UINT64_C(0x1f42fd8f1caa86f6), UINT64_C(0x1f42fd8e5cb56737),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8fa4b386f7)
    },
    {   UINT64_C(0x0000000000005e5a), UINT64_C(0x1f4304cef0b2c72c), UINT64_C(0x1f42f98ef8b7270d),
        UINT64_C(0x1f42fd8f88b5871e), UINT64_C(0x1f43004f64b2c6f1)
    },
    {   UINT64_C(0x0000000000005e5b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcceccb86734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f34ace721)
    },
    {   UINT64_C(0x0000000000005e5c), UINT64_C(0x1f42fb8f34b0670a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98f74b4a6fe), UINT64_C(0x1f42fa8fc0b606ff)
    },
    {   UINT64_C(0x0000000000005e5d), UINT64_C(0x1f42fb8ec0b286fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4e9cade723)
    },
    {   UINT64_C(0x0000000000005e5e), UINT64_C(0x1f42f68fd4ac470e), UINT64_C(0x1f42fb8ee8b34706),
        UINT64_C(0x1f42fd8f7cb5c734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e5f), UINT64_C(0x1f42f78e88b6470b), UINT64_C(0x1f42f98e9cb2e71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38efcb5c72f)
    },
    {   UINT64_C(0x0000000000005e60), UINT64_C(0x1f430e8ee4ab0732), UINT64_C(0x1f430b8e54b4473f),
        UINT64_C(0x1f42f78f08af46fb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e61), UINT64_C(0x1f42f88efcb5c6fb), UINT64_C(0x1f42f38ed8aee710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e62), UINT64_C(0x1f42fa8e2cb4470e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68f78b6470a), UINT64_C(0x1f42f68f64b44715)
    },
    {   UINT64_C(0x0000000000005e63), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4fe0b2670a),
        UINT64_C(0x1f42fd8f48ace725), UINT64_C(0x1f42fa8f9cb08700)
    },
    {   UINT64_C(0x0000000000005e64), UINT64_C(0x1f42fa8f0cb3c71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8e8cb72704), UINT64_C(0x1f42fa8f34b66715)
    },
    {   UINT64_C(0x0000000000005e65), UINT64_C(0x1f42fc8f54b66719), UINT64_C(0x1f42f98e1cb08717),
        UINT64_C(0x1f42f98e9cb64726), UINT64_C(0x1f42f88f44b5c6fe)
    },
    {   UINT64_C(0x0000000000005e66), UINT64_C(0x1f42fa8e4cb4c71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8f58b0c6f7), UINT64_C(0x1f42f58e80b7e731)
    },
    {   UINT64_C(0x0000000000005e67), UINT64_C(0x1f430ece80b1c723), UINT64_C(0x1f42f5cf04b7a710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e68), UINT64_C(0x1f42fb8fbcb44701), UINT64_C(0x1f42fc8ef0b2a706),
        UINT64_C(0x1f42fa8f10ade6e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e69), UINT64_C(0x1f42f88eb4af26e2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f4e78ab8736), UINT64_C(0x1f42f98eb0b6472a)
    },
    {   UINT64_C(0x0000000000005e6a), UINT64_C(0x1f42f88e84b2a717), UINT64_C(0x1f42fb0f38b5071e),
        UINT64_C(0x1f42f2ce6cb62721), UINT64_C(0x1f42f98e9cb3e71e)
    },
    {   UINT64_C(0x0000000000005e6b), UINT64_C(0x1f42f9ce48b8a72f), UINT64_C(0x1f42f88f48b52703),
        UINT64_C(0x1f42f88f64b22734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e6c), UINT64_C(0x1f42fa8eecb2870b), UINT64_C(0x1f42f88e40b38745),
        UINT64_C(0x1f42f88ef0b086f3), UINT64_C(0x1f42f58e50b64738)
    },
    {   UINT64_C(0x0000000000005e6d), UINT64_C(0x1f42fe4eccb4e72b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98e50b50731), UINT64_C(0x1f42f98f50b0e6fe)
    },
    {   UINT64_C(0x000000000001bfff), UINT64_C(0x0000000000000000), UINT64_C(0x1b42fbcc3ca4a67e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e6f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e28b6a746),
        UINT64_C(0x1f430b0f18b7472a), UINT64_C(0x1f430e8efcb1872e)
    },
    {   UINT64_C(0x0000000000005e70), UINT64_C(0x1f4307ce8cb1a733), UINT64_C(0x1f42f90f78aec6fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8f28b1471a)
    },
    {   UINT64_C(0x0000000000005e71), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f24b666d9),
        UINT64_C(0x1f4300cf38b106f9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e72), UINT64_C(0x1f42f98f64b286df), UINT64_C(0x1f430b8e88b72742),
        UINT64_C(0x1f430a4e90ad6732), UINT64_C(0x1f42fc4f74b6a6fd)
    },
    {   UINT64_C(0x0000000000005e73), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ee8b066de),
        UINT64_C(0x1f42fb8f44b886e0), UINT64_C(0x1f42f88f54b4c6d7)
    },
    {   UINT64_C(0x0000000000005e74), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ec8b7e72c),
        UINT64_C(0x1f42fbcf0cb5870e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e75), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8fa0b6a6e7),
        UINT64_C(0x1f42fc8f64b286cf), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e76), UINT64_C(0x1f42f9cfc0b746f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e77), UINT64_C(0x1f42f8cfc0b346df), UINT64_C(0x1f42ff8ee8b4c6d3),
        UINT64_C(0x1f42ff8f24b226d4), UINT64_C(0x1f42fe8f88b806f7)
    },
    {   UINT64_C(0x0000000000005e78), UINT64_C(0x1f42fccf44aac6dc), UINT64_C(0x1f42f84f94b66707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cef8b7a726)
    },
    {   UINT64_C(0x0000000000005e79), UINT64_C(0x1f42faceb8b62700), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e7cb9473d), UINT64_C(0x1f42f58f74b646f3)
    },
    {   UINT64_C(0x0000000000005e7a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f64b7a71b),
        UINT64_C(0x1f42fe8f74b446fa), UINT64_C(0x1f42fbce8cb2a700)
    },
    {   UINT64_C(0x0000000000005e7b), UINT64_C(0x1f43018f94b026ef), UINT64_C(0x1f42f98eccb0072a),
        UINT64_C(0x1f42fccf54ab26ed), UINT64_C(0x1f42f98fb4ae66f0)
    },
    {   UINT64_C(0x0000000000005e7c), UINT64_C(0x1f42fe8f68ae66e3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e7d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108e7cab071f), UINT64_C(0x1f42fecf88b2c6de)
    },
    {   UINT64_C(0x0000000000005e7e), UINT64_C(0x1f4300cf88aea6fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facf78b106f5), UINT64_C(0x1f42fb8ed0af2737)
    },
    {   UINT64_C(0x0000000000005e7f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018ec4ab2714),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e80), UINT64_C(0x1f42fcce9cb1e731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcea8b6071e), UINT64_C(0x1f42f8cf14b2e71a)
    },
    {   UINT64_C(0x0000000000005e81), UINT64_C(0x1f42f6cf44b4c710), UINT64_C(0x1f42f64f94b746f0),
        UINT64_C(0x1f4300cf58b066f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e82), UINT64_C(0x1f42f9ce6cb44702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cedcae271c), UINT64_C(0x1f42fd8fc0b5c6f1)
    },
    {   UINT64_C(0x0000000000005e83), UINT64_C(0x1f42f8ceb8b786fb), UINT64_C(0x1f42fecf10aec707),
        UINT64_C(0x1f42fdcfb4b5a705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e84), UINT64_C(0x1f42f78ec4b766ec), UINT64_C(0x1f42fb0f24b1c70a),
        UINT64_C(0x1f42f70ef0b126df), UINT64_C(0x1f42f70eb4b48723)
    },
    {   UINT64_C(0x0000000000005e85), UINT64_C(0x1f42f70f84ba471d), UINT64_C(0x1f42f50e9cb4871d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78e9cb64736)
    },
    {   UINT64_C(0x0000000000005e86), UINT64_C(0x1f42ff8f04b1e71a), UINT64_C(0x1f42f8cf04b4c718),
        UINT64_C(0x1f42f8cf38b54715), UINT64_C(0x1f42f70f04b506df)
    },
    {   UINT64_C(0x0000000000005e87), UINT64_C(0x1f42f90f34b68731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70ed8b6a6ed), UINT64_C(0x1f43134e4cb1c743)
    },
    {   UINT64_C(0x0000000000005e88), UINT64_C(0x1f42fa8f70abc6e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cee4b566fb), UINT64_C(0x1f43024f14b6a71b)
    },
    {   UINT64_C(0x0000000000005e89), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42face9cad870a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e8a), UINT64_C(0x1f42fdcfacaee6ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f04b0c6dc)
    },
    {   UINT64_C(0x0000000000005e8b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cf74b24708), UINT64_C(0x1f42facf14b0a6ff)
    },
    {   UINT64_C(0x0000000000005e8c), UINT64_C(0x1f42fa8eb0b686e0), UINT64_C(0x1f42f70ef4b84716),
        UINT64_C(0x1f42ff0fc8b3070d), UINT64_C(0x1f42fccfb4af2714)
    },
    {   UINT64_C(0x0000000000005e8d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cfb0b5c6ee)
    },
    {   UINT64_C(0x0000000000005e8e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcec0ad8708),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60fdcb0a70d)
    },
    {   UINT64_C(0x0000000000005e8f), UINT64_C(0x1f42facea0b72715), UINT64_C(0x1f42fa8ee4b08736),
        UINT64_C(0x1f43080e70ae8713), UINT64_C(0x1f42f9cf9caf6722)
    },
    {   UINT64_C(0x0000000000005e90), UINT64_C(0x1f4301cedcb5c737), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4f44b566f7), UINT64_C(0x1f42f4cee8b84716)
    },
    {   UINT64_C(0x0000000000005e91), UINT64_C(0x1f42f50efcb88701), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5ce80b1873c), UINT64_C(0x1f42face34b6a736)
    },
    {   UINT64_C(0x0000000000005e92), UINT64_C(0x1f42ff4e9cb1e732), UINT64_C(0x1f42f50e3cb80708),
        UINT64_C(0x1f42f6cf78b686cf), UINT64_C(0x1f42f78e60b78712)
    },
    {   UINT64_C(0x0000000000005e93), UINT64_C(0x1f42f84f08b7a70b), UINT64_C(0x1f42facea0b56730),
        UINT64_C(0x1f42f88f7ca88714), UINT64_C(0x1f42fa8f24b56711)
    },
    {   UINT64_C(0x0000000000005e94), UINT64_C(0x1f42f80f28b68730), UINT64_C(0x1f42fbcf20b1671d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e95), UINT64_C(0x1f42f8ced0b2e717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8ceacb42700), UINT64_C(0x1f42f60f44b34719)
    },
    {   UINT64_C(0x0000000000005e96), UINT64_C(0x1f42f7ceccb7671e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf04b6a70b), UINT64_C(0x1f42f80f54b466cf)
    },
    {   UINT64_C(0x0000000000005e97), UINT64_C(0x1f42f9cf30b0e71f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6cf48b52703)
    },
    {   UINT64_C(0x0000000000005e98), UINT64_C(0x1f42f90e64b5873e), UINT64_C(0x1f42f88f10b46728),
        UINT64_C(0x1f42f90f70b42728), UINT64_C(0x1f42f78ea0b6c738)
    },
    {   UINT64_C(0x0000000000005e99), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88eacb4a724)
    },
    {   UINT64_C(0x0000000000005e9a), UINT64_C(0x1f43058ec4b34709), UINT64_C(0x1f42f50f24b28719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf20b2a722)
    },
    {   UINT64_C(0x0000000000005e9b), UINT64_C(0x1f42f7cefcb0e722), UINT64_C(0x1f42face48aec73a),
        UINT64_C(0x1f42f8ceccb3470e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e9c), UINT64_C(0x1f42f6cea0b66738), UINT64_C(0x1f42fd8e88b5e72d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e9d), UINT64_C(0x1f43108fd4b786f1), UINT64_C(0x1f43104e9cab471a),
        UINT64_C(0x1f430f8f08b40735), UINT64_C(0x1f43158f4caa071e)
    },
    {   UINT64_C(0x0000000000005e9e), UINT64_C(0x1f4319cf4caae735), UINT64_C(0x1f43130e78b04737),
        UINT64_C(0x1f43160fbcaee721), UINT64_C(0x1f43130ec8a92729)
    },
    {   UINT64_C(0x0000000000005e9f), UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cea4ada742),
        UINT64_C(0x1f43140f5ca9470e), UINT64_C(0x1f43164e70ade741)
    },
    {   UINT64_C(0x0000000000005ea0), UINT64_C(0x1f4314cf98aba716), UINT64_C(0x1f43160f20b3c72d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0f68a8472b)
    },
    {   UINT64_C(0x0000000000005ea1), UINT64_C(0x1f4313ce40ad2740), UINT64_C(0x1f4319cec0ab0721),
        UINT64_C(0x1f43114ed8b2872d), UINT64_C(0x1f43180eccabe731)
    },
    {   UINT64_C(0x0000000000005ea2), UINT64_C(0x1f43130ec8b08722), UINT64_C(0x1f431d4ec8ab471d),
        UINT64_C(0x1f4311ceccb5271b), UINT64_C(0x1f431a8ef8ab071a)
    },
    {   UINT64_C(0x0000000000005ea3), UINT64_C(0x1f43180e1cabe735), UINT64_C(0x1f4310cf20a8472a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4312ce74ab6729)
    },
    {   UINT64_C(0x0000000000005ea4), UINT64_C(0x1f43134ef4abc72e), UINT64_C(0x1f43130f2caa0727),
        UINT64_C(0x1f4318cedcaf0723), UINT64_C(0x1f4301ceb4b1a72c)
    },
    {   UINT64_C(0x0000000000005ea5), UINT64_C(0x1f4313ce80aac731), UINT64_C(0x1f430f0f14b36728),
        UINT64_C(0x1f43098efcb3c735), UINT64_C(0x1f43130eb0ae272e)
    },
    {   UINT64_C(0x0000000000005ea6), UINT64_C(0x1f430b0ec0b14735), UINT64_C(0x1f4305ce68abe73c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ea7), UINT64_C(0x1f4319ce5ca7e71e), UINT64_C(0x1f43164e90a9671f),
        UINT64_C(0x1f43198e84a9270b), UINT64_C(0x1f43004e64af6739)
    },
    {   UINT64_C(0x0000000000005ea8), UINT64_C(0x1f430d8e8cb0e739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431a8e90a866f3), UINT64_C(0x1f43104e6cace70e)
    },
    {   UINT64_C(0x0000000000005ea9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f28b0a71f),
        UINT64_C(0x1f43174ea8ab671b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eaa), UINT64_C(0x1f43170e64aa870e), UINT64_C(0x1f43160e88ac2738),
        UINT64_C(0x1f4312ce8cb0672d), UINT64_C(0x1f43190f14a7c716)
    },
    {   UINT64_C(0x0000000000005eab), UINT64_C(0x1f43050ec0b2c70f), UINT64_C(0x1f43114e78ab672e),
        UINT64_C(0x1f43078e6cb02749), UINT64_C(0x1f430bcea0af6739)
    },
    {   UINT64_C(0x0000000000005eac), UINT64_C(0x1f4314cea4a82726), UINT64_C(0x1f43090edcad472e),
        UINT64_C(0x1f430d4eb8ad2731), UINT64_C(0x1f42fe8f54b7871d)
    },
    {   UINT64_C(0x0000000000005ead), UINT64_C(0x1f4319cecca92736), UINT64_C(0x1f4316ce7ca84735),
        UINT64_C(0x1f43068e6cb42737), UINT64_C(0x1f430b0edcaa2736)
    },
    {   UINT64_C(0x0000000000005eae), UINT64_C(0x1f43198f34a9a726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8e98b4271f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eaf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098f10ade736), UINT64_C(0x1f43180f04b2e739)
    },
    {   UINT64_C(0x0000000000005eb0), UINT64_C(0x1f43048f00b46739), UINT64_C(0x1f430acf08b1a73a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e28ade738)
    },
    {   UINT64_C(0x0000000000005eb1), UINT64_C(0x1f43168e48b38738), UINT64_C(0x1f43020e78b50736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431b4e5ca7273f)
    },
    {   UINT64_C(0x0000000000005eb2), UINT64_C(0x1f430f0e30ab0734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0e90ae2740), UINT64_C(0x1f4306501cb166e9)
    },
    {   UINT64_C(0x0000000000005eb3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ef8b44719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e9cb626fc)
    },
    {   UINT64_C(0x0000000000005eb4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fed01cad06fe),
        UINT64_C(0x1f43024e3cb1670f), UINT64_C(0x1f42fe8fc4b346d9)
    },
    {   UINT64_C(0x0000000000005eb5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f68b406cf),
        UINT64_C(0x1f43194f70b24713), UINT64_C(0x1f42fd8f0cb3c70e)
    },
    {   UINT64_C(0x0000000000005eb6), UINT64_C(0x1f43014ee8ac070e), UINT64_C(0x1f43000ff8aca6f2),
        UINT64_C(0x1f42fa0eccb5870a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eb7), UINT64_C(0x1f43048ed4b04726), UINT64_C(0x1f42f590fcb7e702),
        UINT64_C(0x1f430c4eb8b1a730), UINT64_C(0x1f430c4e8cb20738)
    },
    {   UINT64_C(0x0000000000005eb8), UINT64_C(0x1f43064e60b0673e), UINT64_C(0x1f42fb9070b4e6da),
        UINT64_C(0x1f42f28f48b94723), UINT64_C(0x1f42f90fa4b94718)
    },
    {   UINT64_C(0x0000000000005eb9), UINT64_C(0x1f42fb0f54b1e706), UINT64_C(0x1f43058ee4b2872e),
        UINT64_C(0x1f42fd0e9cb72732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eba), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b0f2cad0701), UINT64_C(0x1f42f70e70b6a724)
    },
    {   UINT64_C(0x0000000000005ebb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43028f04b10702),
        UINT64_C(0x1f42ff0f08b0e6ed), UINT64_C(0x1f42ff0efcad66fa)
    },
    {   UINT64_C(0x0000000000005ebc), UINT64_C(0x1f42f5cfc0b7a6e3), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ebd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf28b56725), UINT64_C(0x1f42fecec8b34716)
    },
    {   UINT64_C(0x0000000000005ebe), UINT64_C(0x1f42fa0e90b58725), UINT64_C(0x1f42f8cf04b006fc),
        UINT64_C(0x1f430b90a0ad6715), UINT64_C(0x1f43184ee8a8470a)
    },
    {   UINT64_C(0x0000000000005ebf), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098e5cab873b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068e9cada73b)
    },
    {   UINT64_C(0x0000000000005ec0), UINT64_C(0x1f430a4ec0b0070a), UINT64_C(0x1f430c8f94af4700),
        UINT64_C(0x1f43024ec0b16713), UINT64_C(0x1f430b4eaca3a720)
    },
    {   UINT64_C(0x0000000000005ec1), UINT64_C(0x1f430d4e7caa0710), UINT64_C(0x1f43194eacb00711),
        UINT64_C(0x1f43088ebcaea71d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ec2), UINT64_C(0x1f43074efcb24733), UINT64_C(0x1f43178e6ca3c71b),
        UINT64_C(0x1f43024ebcae4717), UINT64_C(0x1f430e4f04b14728)
    },
    {   UINT64_C(0x0000000000005ec3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431a4f2cada713), UINT64_C(0x1f4311cec0a52721)
    },
    {   UINT64_C(0x0000000000005ec4), UINT64_C(0x1f431bcea8afa723), UINT64_C(0x1f430e0f7cb4a720),
        UINT64_C(0x1f4308ceecb3a703), UINT64_C(0x1f431b4f6cae8716)
    },
    {   UINT64_C(0x0000000000005ec5), UINT64_C(0x1f43120f78b1872a), UINT64_C(0x1f43168f04b16728),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ec6), UINT64_C(0x1f43194eeca7870e), UINT64_C(0x1f430e4fdcaee729),
        UINT64_C(0x1f43164e6ca12719), UINT64_C(0x1f4312cf00aaa715)
    },
    {   UINT64_C(0x0000000000005ec7), UINT64_C(0x0000000000000000), UINT64_C(0x1f431c4e70a44723),
        UINT64_C(0x1f430e8eacac072c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ec8), UINT64_C(0x1f43044f54b1871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431c4e44aba720), UINT64_C(0x1f43118f2ca4871f)
    },
    {   UINT64_C(0x0000000000005ec9), UINT64_C(0x1f43188ea0a62715), UINT64_C(0x1f43168e28a52735),
        UINT64_C(0x1f42ff4e88b1673d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eca), UINT64_C(0x1f43084e70b8a73a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118f44aea6f9), UINT64_C(0x1f43120ec8a7672c)
    },
    {   UINT64_C(0x0000000000005ecb), UINT64_C(0x1f430d4e78aa8749), UINT64_C(0x1f43074eb4b3871b),
        UINT64_C(0x1f43118e4ca72724), UINT64_C(0x1f42fdcf88b6270c)
    },
    {   UINT64_C(0x0000000000005ecc), UINT64_C(0x1f4319ce48ab8726), UINT64_C(0x1f43194e48a6a728),
        UINT64_C(0x1f430e4e6cab8701), UINT64_C(0x1f43050f74aa870a)
    },
    {   UINT64_C(0x0000000000005ecd), UINT64_C(0x1f43174e50a4a722), UINT64_C(0x1f430f4ec4adc71a),
        UINT64_C(0x1f43090f04ae071a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ece), UINT64_C(0x1f43164eaca5a72c), UINT64_C(0x1f43068eccaf4726),
        UINT64_C(0x1f43160ee0ace72c), UINT64_C(0x1f430bce5ca60727)
    },
    {   UINT64_C(0x0000000000005ecf), UINT64_C(0x1f4314cf84b36728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060f00acc726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ed0), UINT64_C(0x1f43180f94a80715), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ed1), UINT64_C(0x1f43180ec0b0670e), UINT64_C(0x1f43144ed4adc728),
        UINT64_C(0x1f43094e2cb54745), UINT64_C(0x1f43080eaca8a722)
    },
    {   UINT64_C(0x0000000000005ed2), UINT64_C(0x1f43078f00aa2722), UINT64_C(0x1f43078e8cb06745),
        UINT64_C(0x1f4318ce54b2e722), UINT64_C(0x1f43060f40a8a728)
    },
    {   UINT64_C(0x0000000000005ed3), UINT64_C(0x1f43050ee4b56711), UINT64_C(0x1f431a8e30a86741),
        UINT64_C(0x1f43018f8cac470e), UINT64_C(0x1f42f98ef0b3a732)
    },
    {   UINT64_C(0x0000000000005ed4), UINT64_C(0x1f4313cee0ab0733), UINT64_C(0x1f43048f14b2e712),
        UINT64_C(0x1f430dce90ad072f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ed5), UINT64_C(0x1f42fb8eecb5e728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e4e8cb6a748), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ed6), UINT64_C(0x1f43058edca7270a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ece8cad4728), UINT64_C(0x1f43178e6ca42724)
    },
    {   UINT64_C(0x0000000000005ed7), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8ea4b0e737),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ed8), UINT64_C(0x1f42ff8ea8a68734), UINT64_C(0x1f43064e68b82737),
        UINT64_C(0x1f43040f28b06717), UINT64_C(0x1f43058e1ca90736)
    },
    {   UINT64_C(0x0000000000005ed9), UINT64_C(0x1f43158e20b3c73a), UINT64_C(0x1f4313ceeca7a73b),
        UINT64_C(0x1f431c4f08af871f), UINT64_C(0x1f430ece70b5073c)
    },
    {   UINT64_C(0x0000000000005eda), UINT64_C(0x0000000000000000), UINT64_C(0x1f431b8eb0a62744),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005edb), UINT64_C(0x1f43014ea4b1e744), UINT64_C(0x1f43068eecafc72b),
        UINT64_C(0x1f43064f24b04717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005edc), UINT64_C(0x1f43034e8ca9a731), UINT64_C(0x1f42fb4f08b6a735),
        UINT64_C(0x1f430dce24a4273b), UINT64_C(0x1f43058e94b20736)
    },
    {   UINT64_C(0x0000000000005edd), UINT64_C(0x1f430c8eecb34735), UINT64_C(0x1f43018eecb1c749),
        UINT64_C(0x1f430f4e60ad273a), UINT64_C(0x1f43068e70af6722)
    },
    {   UINT64_C(0x0000000000005ede), UINT64_C(0x1f430d8e3cad873a), UINT64_C(0x1f43058ec4a52726),
        UINT64_C(0x1f43118ee0a9e733), UINT64_C(0x1f43154ea0abe726)
    },
    {   UINT64_C(0x0000000000005edf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030f94b36718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ee0), UINT64_C(0x1f43178eb8a5c728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43158ea8aee719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ee1), UINT64_C(0x1f430e8e5caee731), UINT64_C(0x1f43098e30afe734),
        UINT64_C(0x1f43104e80aa672e), UINT64_C(0x1f430ece60a62738)
    },
    {   UINT64_C(0x0000000000005ee2), UINT64_C(0x1f4312cf34a62717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4306cf30b56710), UINT64_C(0x1f43178e88adc73d)
    },
    {   UINT64_C(0x0000000000005ee3), UINT64_C(0x1f43144e60a42711), UINT64_C(0x1f43004ee8ac071b),
        UINT64_C(0x1f43050e50ad272d), UINT64_C(0x1f43048e2cb8674b)
    },
    {   UINT64_C(0x0000000000005ee4), UINT64_C(0x1f43104e8caae71c), UINT64_C(0x1f430d4e6cabc738),
        UINT64_C(0x1f43080e5cae8738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ee5), UINT64_C(0x1f43148f44a82735), UINT64_C(0x1f43058eccb26748),
        UINT64_C(0x1f42ff8eccb5471b), UINT64_C(0x1f4301ce68b2c744)
    },
    {   UINT64_C(0x0000000000005ee6), UINT64_C(0x1f43158e8ca8072b), UINT64_C(0x1f43090ef8b46727),
        UINT64_C(0x1f430fceb0aaa736), UINT64_C(0x1f43044ef0b42723)
    },
    {   UINT64_C(0x0000000000005ee7), UINT64_C(0x1f431b0f4ca86713), UINT64_C(0x1f42fd0ea4b2e73d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8ec0ab4727)
    },
    {   UINT64_C(0x0000000000005ee8), UINT64_C(0x1f43068eb8b24722), UINT64_C(0x1f430e8e6cb0474d),
        UINT64_C(0x1f43150f18abc70c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ee9), UINT64_C(0x1f430a8e7cac6734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4e68ab872f), UINT64_C(0x1f43138ebca70728)
    },
    {   UINT64_C(0x0000000000005eea), UINT64_C(0x1f4304ce9cb30724), UINT64_C(0x1f43148ea0a5272a),
        UINT64_C(0x1f43124eacade723), UINT64_C(0x1f430f0eacab073b)
    },
    {   UINT64_C(0x0000000000005eeb), UINT64_C(0x1f43130f0ca8a726), UINT64_C(0x1f43058eacae8738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0ee4b32739)
    },
    {   UINT64_C(0x0000000000005eec), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f0ca6e738),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eed), UINT64_C(0x1f42ff0ea8b16731), UINT64_C(0x1f430c8e74a6073b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8f70b36726)
    },
    {   UINT64_C(0x0000000000005eee), UINT64_C(0x1f43048ea0b2074d), UINT64_C(0x1f4315ceeca9e733),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005eef), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43168eb0a7073d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ef0), UINT64_C(0x1f43118e7ca9a73e), UINT64_C(0x1f43088f00abe740),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ef1), UINT64_C(0x1f42fe4e60b72741), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d4e68b3e734), UINT64_C(0x1f42facee0a5a732)
    },
    {   UINT64_C(0x0000000000005ef2), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4e3cb0a72e),
        UINT64_C(0x1f43130ee4afe731), UINT64_C(0x1f43168f40a9273e)
    },
    {   UINT64_C(0x0000000000006010), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048fa8b30708), UINT64_C(0x1f42fc4f6cb44714)
    },
    {   UINT64_C(0x0000000000005ef4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088e78af4731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e70aa672f)
    },
    {   UINT64_C(0x0000000000005ef5), UINT64_C(0x1f431a4e54a6e733), UINT64_C(0x1f430e4e8cb14724),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ef6), UINT64_C(0x1f43198e50a42734), UINT64_C(0x1f43144e4ca80742),
        UINT64_C(0x1f43168ee4aa271c), UINT64_C(0x1f430e0e60a5e726)
    },
    {   UINT64_C(0x0000000000005ef7), UINT64_C(0x1f430e0e90b36741), UINT64_C(0x1f43058e20b00746),
        UINT64_C(0x1f430ccec8b0c736), UINT64_C(0x1f43018e38b36737)
    },
    {   UINT64_C(0x0000000000005ef8), UINT64_C(0x1f42fb4ea0b7c738), UINT64_C(0x1f43094ed8b18733),
        UINT64_C(0x1f430c4eccac0735), UINT64_C(0x1f43134e14afa73b)
    },
    {   UINT64_C(0x0000000000005ef9), UINT64_C(0x1f43150e7caca744), UINT64_C(0x1f4306ce4cb32739),
        UINT64_C(0x1f43198e80af072e), UINT64_C(0x1f4306ce6cad2730)
    },
    {   UINT64_C(0x0000000000005efa), UINT64_C(0x1f43148e98b84731), UINT64_C(0x1f43180e6ca70735),
        UINT64_C(0x1f43034e74adc740), UINT64_C(0x1f43144e34ad873e)
    },
    {   UINT64_C(0x0000000000005efb), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302ceecb1273a),
        UINT64_C(0x1f43114f44adc711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005efc), UINT64_C(0x1f43098e5caa472d), UINT64_C(0x1f43198eb8a5e738),
        UINT64_C(0x1f43108eb0b1e72e), UINT64_C(0x1f43064e7cb24733)
    },
    {   UINT64_C(0x0000000000005efd), UINT64_C(0x1f43014e68b28739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dce20aba71c), UINT64_C(0x1f43020e88b10730)
    },
    {   UINT64_C(0x0000000000005efe), UINT64_C(0x1f430e4e70b3a72a), UINT64_C(0x1f43050e50b8873d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43198e3ca9873c)
    },
    {   UINT64_C(0x0000000000005eff), UINT64_C(0x1f43130f00ad6725), UINT64_C(0x1f43038e9cafe73b),
        UINT64_C(0x1f4302ce7cb00735), UINT64_C(0x1f430f4e50ad8734)
    },
    {   UINT64_C(0x0000000000005f00), UINT64_C(0x1f430e0e6cab8734), UINT64_C(0x1f43028eb8b84736),
        UINT64_C(0x1f43138ebca88732), UINT64_C(0x1f430a8efcab6733)
    },
    {   UINT64_C(0x0000000000005f01), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048ec4a90730),
        UINT64_C(0x1f430c8ec0b1a72d), UINT64_C(0x1f42ffcf30b1671b)
    },
    {   UINT64_C(0x0000000000005f02), UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ef8aea731),
        UINT64_C(0x1f430e0ea0b0072a), UINT64_C(0x1f43108ed0b0c731)
    },
    {   UINT64_C(0x0000000000005f03), UINT64_C(0x1f42fcce60b16743), UINT64_C(0x1f4301cec4b14742),
        UINT64_C(0x1f430a0eb4afc737), UINT64_C(0x1f42fa8ec4b2272d)
    },
    {   UINT64_C(0x0000000000005f04), UINT64_C(0x1f430c8eacb2c736), UINT64_C(0x1f430e4ea4ae072f),
        UINT64_C(0x1f43058e6cafc733), UINT64_C(0x1f4306ce94af2731)
    },
    {   UINT64_C(0x0000000000005f05), UINT64_C(0x1f430bce4cacc72f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e8e7cb08728), UINT64_C(0x1f4307ce9caea72f)
    },
    {   UINT64_C(0x0000000000005f06), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054e60abe72a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f07), UINT64_C(0x1f43114f10abe731), UINT64_C(0x1f43078e58b70745),
        UINT64_C(0x1f43098e6caf6744), UINT64_C(0x1f430ace5caf2720)
    },
    {   UINT64_C(0x0000000000005f08), UINT64_C(0x1f430e8ef4ad2719), UINT64_C(0x1f43188eb0a4471d),
        UINT64_C(0x1f431e4e94a6c720), UINT64_C(0x1f43088e2cb14747)
    },
    {   UINT64_C(0x0000000000005f09), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffce84b76730),
        UINT64_C(0x1f43098e70b68721), UINT64_C(0x1f43088e98b5c74a)
    },
    {   UINT64_C(0x0000000000005f0a), UINT64_C(0x1f43104ec8acc727), UINT64_C(0x1f43100ed0ae2723),
        UINT64_C(0x1f43140e64b12734), UINT64_C(0x1f430c0e8cae073b)
    },
    {   UINT64_C(0x0000000000005f0b), UINT64_C(0x1f43074f78b8472d), UINT64_C(0x1f43090efcb4c726),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f0c), UINT64_C(0x1f43090e7cb14728), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43070ebcb70731)
    },
    {   UINT64_C(0x0000000000005f0d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e70a5872f)
    },
    {   UINT64_C(0x0000000000005f0e), UINT64_C(0x1f42fecf14b3e72a), UINT64_C(0x1f43100e48af273a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43048e38b36738)
    },
    {   UINT64_C(0x0000000000005f0f), UINT64_C(0x1f43020e90b3a733), UINT64_C(0x1f43028efcb6271e),
        UINT64_C(0x1f43008e9cb4c730), UINT64_C(0x1f430d4e54b14730)
    },
    {   UINT64_C(0x0000000000005f10), UINT64_C(0x1f43018e84a8872a), UINT64_C(0x1f4309cea0a6672b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8eb8b0a724)
    },
    {   UINT64_C(0x0000000000005f11), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088eb0b0272e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e00ad4736)
    },
    {   UINT64_C(0x0000000000005f12), UINT64_C(0x1f4308ced8b3272a), UINT64_C(0x1f430b0e64afc736),
        UINT64_C(0x1f43078e7ca40730), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f13), UINT64_C(0x1f43058e4cafe73a), UINT64_C(0x1f42fecea4ade730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4ef8b3672f)
    },
    {   UINT64_C(0x0000000000005f14), UINT64_C(0x1f42fe4ea0b6272b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e94b3e727)
    },
    {   UINT64_C(0x0000000000005f15), UINT64_C(0x1f43030ec0b7872b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4304ce98b0073c), UINT64_C(0x1f43024e3caba73c)
    },
    {   UINT64_C(0x0000000000005f16), UINT64_C(0x1f42fb8ef0b4c72b), UINT64_C(0x1f42f80e84b7872d),
        UINT64_C(0x1f43034e60ac473c), UINT64_C(0x1f4303ce54b2c740)
    },
    {   UINT64_C(0x0000000000005f17), UINT64_C(0x1f43048e50abe730), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ceb4a5a737)
    },
    {   UINT64_C(0x0000000000005f18), UINT64_C(0x1f43000e88b38738), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431b0e50a7c738)
    },
    {   UINT64_C(0x0000000000005f19), UINT64_C(0x1f430dce98b4a73c), UINT64_C(0x1f42ff8e7cb5072d),
        UINT64_C(0x1f42f90edcaf0738), UINT64_C(0x1f4303ce28b32738)
    },
    {   UINT64_C(0x0000000000005f1a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43040e4cb04734),
        UINT64_C(0x1f43084e70b08734), UINT64_C(0x1f4308ce24b0e736)
    },
    {   UINT64_C(0x0000000000005f1b), UINT64_C(0x1f430e4e5cb2c731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124ebcade72f), UINT64_C(0x1f430fce84b1c738)
    },
    {   UINT64_C(0x0000000000005f1c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8ed4acc731), UINT64_C(0x1f43094eacb70723)
    },
    {   UINT64_C(0x0000000000005f1d), UINT64_C(0x1f430bcef8b62710), UINT64_C(0x1f42ff8ee8b12714),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f1e), UINT64_C(0x1f42fcceacb566f4), UINT64_C(0x1f42ff0e9cb1c6ed),
        UINT64_C(0x1f42f9ce9cb84700), UINT64_C(0x1f430d4f54b0c718)
    },
    {   UINT64_C(0x0000000000005f1f), UINT64_C(0x1f42fa0edcae270c), UINT64_C(0x1f42f6cf04b30708),
        UINT64_C(0x1f42ff4f28b2e708), UINT64_C(0x1f42fd8e74b346fc)
    },
    {   UINT64_C(0x0000000000005f20), UINT64_C(0x1f42f7ce9cb546fc), UINT64_C(0x1f42f9ce9cb3e710),
        UINT64_C(0x1f43058e64afc701), UINT64_C(0x1f42f58f8cb7e6f5)
    },
    {   UINT64_C(0x0000000000005f21), UINT64_C(0x1f42facf18b606e8), UINT64_C(0x1f42fdceecafa6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f22), UINT64_C(0x1f42feceecb246e9), UINT64_C(0x1f42f7cf18b7270e),
        UINT64_C(0x1f43154e7ca886f7), UINT64_C(0x1f42f6ceb8b72712)
    },
    {   UINT64_C(0x0000000000005f23), UINT64_C(0x1f42fe8f18a9a6eb), UINT64_C(0x1f42fbcef8b5e6fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f24), UINT64_C(0x1f42fe8f10b34703), UINT64_C(0x1f42fe8efcb126fc),
        UINT64_C(0x1f42fdcf38b4c717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f25), UINT64_C(0x1f42fd8f38ae0707), UINT64_C(0x1f42f9cf68a84722),
        UINT64_C(0x1f42fa8ebcb2c707), UINT64_C(0x1f42fd0ef8b7271b)
    },
    {   UINT64_C(0x0000000000005f26), UINT64_C(0x1f42fbcf68a8c6f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303ce84b0a726)
    },
    {   UINT64_C(0x0000000000005f27), UINT64_C(0x1f43114ebcade71d), UINT64_C(0x1f42fa0fa4b00710),
        UINT64_C(0x1f42fb8f78b4c70b), UINT64_C(0x1f430b8f08b5c735)
    },
    {   UINT64_C(0x0000000000005f28), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f0cacc6ff),
        UINT64_C(0x1f430e8e40af6729), UINT64_C(0x1f42f60ef4b646fb)
    },
    {   UINT64_C(0x0000000000005f29), UINT64_C(0x1f4305ce78b08737), UINT64_C(0x1f4307cf78aca712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42feceecaf66e9)
    },
    {   UINT64_C(0x0000000000005f2a), UINT64_C(0x1f42fa8f3cac870b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43068f14b04720), UINT64_C(0x1f43028f3cb1670b)
    },
    {   UINT64_C(0x0000000000005f2b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f3cb7670b),
        UINT64_C(0x1f42fbcf38b68727), UINT64_C(0x1f42f7cef8b8c6f7)
    },
    {   UINT64_C(0x0000000000005f2c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43078edcb1271d),
        UINT64_C(0x1f430f4e9cafe732), UINT64_C(0x1f42fa0f28b646f3)
    },
    {   UINT64_C(0x0000000000005f2d), UINT64_C(0x1f42f8cef8b346fb), UINT64_C(0x1f42ff8e40b3e73b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f2e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ce8cb64722),
        UINT64_C(0x1f42face4cb4e718), UINT64_C(0x1f42fbceecb68700)
    },
    {   UINT64_C(0x0000000000005f2f), UINT64_C(0x1f43008f48af4715), UINT64_C(0x1f42f68f20b0e71a),
        UINT64_C(0x1f42f98ea0b58734), UINT64_C(0x1f42f9cf18b726f3)
    },
    {   UINT64_C(0x0000000000005f30), UINT64_C(0x1f42f80fa8b68707), UINT64_C(0x1f42f9cef0b7a717),
        UINT64_C(0x1f42f84f80b6672a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f31), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ee8b5a704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f88b446ec)
    },
    {   UINT64_C(0x0000000000005f32), UINT64_C(0x1f42facec8b4e716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf3caec70f), UINT64_C(0x1f43028ef8aa46f7)
    },
    {   UINT64_C(0x0000000000005f33), UINT64_C(0x1f42f70f24b706ec), UINT64_C(0x1f42f8cf78aec6ec),
        UINT64_C(0x1f42ffcf88aec708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f34), UINT64_C(0x1f42f70e3cb30721), UINT64_C(0x1f42f7cf10b026f7),
        UINT64_C(0x1f42facf48b0272d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f35), UINT64_C(0x1f42f9cedcb506f7), UINT64_C(0x1f42f8cf48b1e714),
        UINT64_C(0x1f42fbcf38b3471e), UINT64_C(0x1f42f60f88b5a723)
    },
    {   UINT64_C(0x0000000000005f36), UINT64_C(0x1f42f60ef8b78738), UINT64_C(0x1f42f7cf18b4070e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f37), UINT64_C(0x1f42f98e5cb12740), UINT64_C(0x1f42f9ce90b78732),
        UINT64_C(0x1f43098e6caf4734), UINT64_C(0x1f42f9ce64aec727)
    },
    {   UINT64_C(0x0000000000005f38), UINT64_C(0x1f42f9cec0b64729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf6cb3472c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f39), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf38b5c727)
    },
    {   UINT64_C(0x0000000000005f3a), UINT64_C(0x1f42fdcf38b0a6d8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43094e9cb02731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f3b), UINT64_C(0x1f42fc0e84b566dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60efcb2a701), UINT64_C(0x1f42faceb0b5e71a)
    },
    {   UINT64_C(0x0000000000005f3c), UINT64_C(0x1f42fa0f1cae06f8), UINT64_C(0x1f43134edcaf4727),
        UINT64_C(0x1f42fc8f30af8707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f3d), UINT64_C(0x1f42feced8af06ef), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0edcb5c721), UINT64_C(0x1f42f7ce7caee73c)
    },
    {   UINT64_C(0x0000000000005f3e), UINT64_C(0x1f431a4e60a88735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058fc8afc6e9), UINT64_C(0x1f431ecf0ca76732)
    },
    {   UINT64_C(0x0000000000005f3f), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0ef4b1c6e9),
        UINT64_C(0x1f430a4e78a36733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f40), UINT64_C(0x1f430bcff4b5870d), UINT64_C(0x1f43050f60b926e4),
        UINT64_C(0x1f430c8ee4b3071a), UINT64_C(0x1f43050e6cade6ec)
    },
    {   UINT64_C(0x0000000000005f41), UINT64_C(0x1f43108f68b06704), UINT64_C(0x1f4305cfe4b74705),
        UINT64_C(0x1f430a8f3cb38726), UINT64_C(0x1f430b8fbcb22714)
    },
    {   UINT64_C(0x0000000000005f42), UINT64_C(0x1f43030fd0b62706), UINT64_C(0x1f430d8f88aea713),
        UINT64_C(0x1f430a4e9cb26738), UINT64_C(0x1f43070ff0b7a6fb)
    },
    {   UINT64_C(0x0000000000005f43), UINT64_C(0x1f43090f60b246fd), UINT64_C(0x1f43038fb4af26f4),
        UINT64_C(0x1f4309cfd0b2c6e8), UINT64_C(0x1f430bceb8b026f2)
    },
    {   UINT64_C(0x0000000000005f44), UINT64_C(0x1f43091040b5c708), UINT64_C(0x1f4305cf58ae26ea),
        UINT64_C(0x1f43074ef8b4e6ec), UINT64_C(0x1f430bcefcb34707)
    },
    {   UINT64_C(0x0000000000005f45), UINT64_C(0x1f4302cf8cb58705), UINT64_C(0x1f4308cff4b726ff),
        UINT64_C(0x1f430c0eeca966ef), UINT64_C(0x1f430aceecb42726)
    },
    {   UINT64_C(0x0000000000005f46), UINT64_C(0x1f430c0fbcaba716), UINT64_C(0x1f430a0fc8b4c6f9),
        UINT64_C(0x1f43030fdcb5e704), UINT64_C(0x1f43014f84ba4715)
    },
    {   UINT64_C(0x0000000000005f47), UINT64_C(0x1f43028fe8b2e6e9), UINT64_C(0x1f430e100cb186e5),
        UINT64_C(0x1f43064f3cb5c6fa), UINT64_C(0x1f4308cfb8b666e5)
    },
    {   UINT64_C(0x0000000000005f48), UINT64_C(0x1f43094e3cb3c740), UINT64_C(0x1f430c0f48b2670b),
        UINT64_C(0x1f43168e74b12740), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f49), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43210ea4b086f8)
    },
    {   UINT64_C(0x0000000000005f4a), UINT64_C(0x1f43068f34b38709), UINT64_C(0x1f43094fb8b346ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430dce58b2073b)
    },
    {   UINT64_C(0x0000000000005f4b), UINT64_C(0x1f430a8fa4aec6d9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4305ce68af8725), UINT64_C(0x1f43200ef0a6a71e)
    },
    {   UINT64_C(0x0000000000005f4c), UINT64_C(0x1f4308cfccb166f6), UINT64_C(0x1f43004fc4b526f6),
        UINT64_C(0x1f4303cfacb5e6e6), UINT64_C(0x1f4307ce6cb0e725)
    },
    {   UINT64_C(0x0000000000005f4d), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8eccb2270a),
        UINT64_C(0x1f430c9030ac06fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f4e), UINT64_C(0x1f430b9050afe702), UINT64_C(0x1f430a0fc8ad2717),
        UINT64_C(0x1f43091034af66f5), UINT64_C(0x1f430a8fc4b546fb)
    },
    {   UINT64_C(0x0000000000005f4f), UINT64_C(0x1f4303cec8b0e72c), UINT64_C(0x1f430d4facafe707),
        UINT64_C(0x1f43088f1cb806f3), UINT64_C(0x1f430f4fe0abc6eb)
    },
    {   UINT64_C(0x0000000000005f50), UINT64_C(0x1f43094f94af66f9), UINT64_C(0x1f4307cf98b0c716),
        UINT64_C(0x1f43098f54b7c71a), UINT64_C(0x1f430c501cb12700)
    },
    {   UINT64_C(0x0000000000005f51), UINT64_C(0x1f430c8fb4b72711), UINT64_C(0x1f430e8fb4a9a703),
        UINT64_C(0x1f43008ec8b16721), UINT64_C(0x1f430f0ee8ab070c)
    },
    {   UINT64_C(0x0000000000005f52), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098eacb5c70b), UINT64_C(0x1f4305cf94afc705)
    },
    {   UINT64_C(0x0000000000005f53), UINT64_C(0x1f42facfbcb506ed), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cf1cb5e6ed), UINT64_C(0x1f43064e50b0c709)
    },
    {   UINT64_C(0x0000000000005f54), UINT64_C(0x1f43070fb8b0071e), UINT64_C(0x1f43060ed8b6c71e),
        UINT64_C(0x1f43060eccac66fa), UINT64_C(0x1f43094fe8af66f2)
    },
    {   UINT64_C(0x0000000000005f55), UINT64_C(0x1f43080f7cb066de), UINT64_C(0x1f43031024b546ff),
        UINT64_C(0x1f43015018b4271d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f56), UINT64_C(0x1f4306cf68b646f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f30eb8b7c732)
    },
    {   UINT64_C(0x0000000000005f57), UINT64_C(0x1f430e4e64a9a720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430bd028b926f8), UINT64_C(0x1f43070fd0b5a703)
    },
    {   UINT64_C(0x0000000000005f58), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cff8b66720),
        UINT64_C(0x1f430acf28b08730), UINT64_C(0x1f42ff8e78b0e71f)
    },
    {   UINT64_C(0x0000000000005f59), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43025044baa701), UINT64_C(0x1f43080f9cad2705)
    },
    {   UINT64_C(0x0000000000005f5a), UINT64_C(0x1f43018f34b70719), UINT64_C(0x1f4305cfecb52701),
        UINT64_C(0x1f430a0f7caea6fe), UINT64_C(0x1f4316ce70a78742)
    },
    {   UINT64_C(0x0000000000005f5b), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cdf4b72737),
        UINT64_C(0x1f42fd8e7cab673f), UINT64_C(0x1f43058f4caf26f6)
    },
    {   UINT64_C(0x0000000000005f5c), UINT64_C(0x1f4304d044b066f6), UINT64_C(0x1f43000f8cb6070a),
        UINT64_C(0x1f43054f1cac8705), UINT64_C(0x1f4305ce4cba6733)
    },
    {   UINT64_C(0x0000000000005f5d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8507cb666fd),
        UINT64_C(0x1f42fe108cb386fd), UINT64_C(0x1f430ecec8ac871e)
    },
    {   UINT64_C(0x0000000000005f5e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43060ed4a7070b),
        UINT64_C(0x1f4303ceacad271c), UINT64_C(0x1f43070f38b2e6f6)
    },
    {   UINT64_C(0x0000000000005f5f), UINT64_C(0x1f430a4f48a826f5), UINT64_C(0x1f430ccfd8b026ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f04b7a71c)
    },
    {   UINT64_C(0x0000000000005f60), UINT64_C(0x1f43080e7cb7e722), UINT64_C(0x1f43028e8cb206fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4edcad470a)
    },
    {   UINT64_C(0x0000000000005f61), UINT64_C(0x1f43068fe0b56711), UINT64_C(0x1f4308cf0cb326f2),
        UINT64_C(0x1f43080f38b66705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f62), UINT64_C(0x1f43060ef4ada72c), UINT64_C(0x1f42feceb0b3271c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f63), UINT64_C(0x1f4305cfe8b6a6f2), UINT64_C(0x1f43050ef4aea732),
        UINT64_C(0x1f43044fc8b626e0), UINT64_C(0x1f43030f70b5c708)
    },
    {   UINT64_C(0x0000000000005f64), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f80ae46ef),
        UINT64_C(0x1f43080ee4b2e70d), UINT64_C(0x1f43014fbcb426ed)
    },
    {   UINT64_C(0x0000000000005f65), UINT64_C(0x1f43038ed4b186fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43020fc8b366e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f66), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4ff4b3e70d),
        UINT64_C(0x1f42fecf18b34732), UINT64_C(0x1f42fdcf9cb3c6f5)
    },
    {   UINT64_C(0x0000000000005f67), UINT64_C(0x1f43018f20b1a711), UINT64_C(0x1f42fe4ed8b7a70e),
        UINT64_C(0x1f42ff4f98b446ea), UINT64_C(0x1f43021050b06701)
    },
    {   UINT64_C(0x0000000000005f68), UINT64_C(0x1f42f9cf14b68725), UINT64_C(0x1f43074f64ad86ff),
        UINT64_C(0x1f43034fa4aa0700), UINT64_C(0x1f43040f70b6e706)
    },
    {   UINT64_C(0x0000000000005f69), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4ef8b3473b), UINT64_C(0x1f42ff4e6cb2e71d)
    },
    {   UINT64_C(0x0000000000005f6a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054f9cb526ee),
        UINT64_C(0x1f43044eb8afc721), UINT64_C(0x1f42fe4e8cb3672c)
    },
    {   UINT64_C(0x0000000000005f6b), UINT64_C(0x1f42ff4f68b52702), UINT64_C(0x1f43080f28b4671c),
        UINT64_C(0x1f43014ec0b50713), UINT64_C(0x1f43070f04ae2700)
    },
    {   UINT64_C(0x0000000000005f6c), UINT64_C(0x1f4303cf28b0e711), UINT64_C(0x1f43060fa4af2702),
        UINT64_C(0x1f430a8f60af470d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f6d), UINT64_C(0x1f4306d038b6a716), UINT64_C(0x1f43018edcb7a720),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e40b38712)
    },
    {   UINT64_C(0x0000000000005f6e), UINT64_C(0x1f430b0f90b72711), UINT64_C(0x1f43044f8cb5e6f7),
        UINT64_C(0x1f4308ce90b6472f), UINT64_C(0x1f4305d038b8671d)
    },
    {   UINT64_C(0x0000000000005f6f), UINT64_C(0x1f43004e3cb1670b), UINT64_C(0x1f43024e6cb4472c),
        UINT64_C(0x1f4302cf00b4e726), UINT64_C(0x1f43008e6cad6735)
    },
    {   UINT64_C(0x0000000000005f70), UINT64_C(0x1f43060f00af8715), UINT64_C(0x1f43014ed0b7471b),
        UINT64_C(0x1f43004f10b16705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f71), UINT64_C(0x1f43040fe4b5e6d8), UINT64_C(0x1f42f80edcb6a6e6),
        UINT64_C(0x1f43090f24ae66ff), UINT64_C(0x1f43080facab06f7)
    },
    {   UINT64_C(0x0000000000005f72), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf94afc6e1),
        UINT64_C(0x1f431b8e88b28711), UINT64_C(0x1f43080f48b266ff)
    },
    {   UINT64_C(0x0000000000005f73), UINT64_C(0x1f43050f5cb0e70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43050f2caf6713), UINT64_C(0x1f42ff0f6cb30700)
    },
    {   UINT64_C(0x0000000000005f74), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f28b666e6),
        UINT64_C(0x1f43070f50b40726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f75), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84eb8b6671e),
        UINT64_C(0x1f43020f50b18726), UINT64_C(0x1f43068f88afc6f3)
    },
    {   UINT64_C(0x0000000000005f76), UINT64_C(0x1f4306cf40b7c725), UINT64_C(0x1f43088e6cb2c73e),
        UINT64_C(0x1f43030ee8b7271e), UINT64_C(0x1f430a4f84afe6e6)
    },
    {   UINT64_C(0x0000000000005f77), UINT64_C(0x1f42fd4f0cb3e736), UINT64_C(0x1f4304ce00b246f2),
        UINT64_C(0x1f4304cf84b706f8), UINT64_C(0x1f4306cf5cb286f8)
    },
    {   UINT64_C(0x0000000000005f78), UINT64_C(0x1f4308ce2cb5472d), UINT64_C(0x1f43094fa4b1c704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cea4b8272b)
    },
    {   UINT64_C(0x0000000000005f79), UINT64_C(0x1f42f98eecb5c72d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43080f5cb526c3), UINT64_C(0x1f430a0fa8b02712)
    },
    {   UINT64_C(0x0000000000005f7a), UINT64_C(0x1f4302cf04ae4708), UINT64_C(0x1f43094ee0b4471b),
        UINT64_C(0x1f43050f98b646e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f7b), UINT64_C(0x1f43028fccb0c6fb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43050eecb5071b), UINT64_C(0x1f4308ce6cb2473f)
    },
    {   UINT64_C(0x0000000000005f7c), UINT64_C(0x1f4301cea0b786fd), UINT64_C(0x1f42fdcfa4b726d2),
        UINT64_C(0x1f43050edcb506f5), UINT64_C(0x1f42fc4ea8a50735)
    },
    {   UINT64_C(0x0000000000005f7d), UINT64_C(0x1f42fb4e6cb9873a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038fe0b766f7), UINT64_C(0x1f4307ced8b4a6e6)
    },
    {   UINT64_C(0x0000000000005f7e), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cfc4b58712),
        UINT64_C(0x1f4306cfe4b066fe), UINT64_C(0x1f43049054b7a70c)
    },
    {   UINT64_C(0x0000000000009103), UINT64_C(0x1d82f18fa0b1a68c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x131af20fa089868c)
    },
    {   UINT64_C(0x0000000000005f80), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4ea4b466ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054ec0b6c6fb)
    },
    {   UINT64_C(0x0000000000005f81), UINT64_C(0x1f43014f9cb1e6ee), UINT64_C(0x1f42f9cf28b5a718),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f82), UINT64_C(0x1f43130e78b26734), UINT64_C(0x1f42ffcefcb28709),
        UINT64_C(0x1f43089054b4270c), UINT64_C(0x1f43060fa8b546ff)
    },
    {   UINT64_C(0x0000000000005f83), UINT64_C(0x1f43024f58b8c6ee), UINT64_C(0x1f43040ebcb6c712),
        UINT64_C(0x1f43024f54b38702), UINT64_C(0x1f43034e7cb08736)
    },
    {   UINT64_C(0x0000000000005f84), UINT64_C(0x1f43080edcad0736), UINT64_C(0x1f430a4edcb5c72e),
        UINT64_C(0x1f43040f8cb3471e), UINT64_C(0x1f43004f04b026e6)
    },
    {   UINT64_C(0x0000000000009106), UINT64_C(0x1692f1cfa095e68c), UINT64_C(0x14daf50fa08dc68c),
        UINT64_C(0x0fa2f44f687b85c1), UINT64_C(0x1162f24f687fa68c)
    },
    {   UINT64_C(0x0000000000005f86), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4306cf90b2a6d9), UINT64_C(0x1f43004e58b42738)
    },
    {   UINT64_C(0x0000000000005f87), UINT64_C(0x1f430a0efcaaa6fb), UINT64_C(0x1f43008efcb3e727),
        UINT64_C(0x1f430a0f40ae871b), UINT64_C(0x1f4301cec8b54726)
    },
    {   UINT64_C(0x0000000000005f88), UINT64_C(0x1f43070eecb5471e), UINT64_C(0x1f4308ce9caf8725),
        UINT64_C(0x1f42fa0ed4b86720), UINT64_C(0x1f43084f04afc709)
    },
    {   UINT64_C(0x0000000000005f89), UINT64_C(0x1f43060e8cb0871a), UINT64_C(0x1f42fc4eacb92724),
        UINT64_C(0x1f43020f98b526c8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f8a), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cef4b706f4),
        UINT64_C(0x1f43058eacb14706), UINT64_C(0x1f43050f58b8a707)
    },
    {   UINT64_C(0x0000000000005f8b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f28b30713),
        UINT64_C(0x1f43064f84b4670e), UINT64_C(0x1f43070f0cb5a716)
    },
    {   UINT64_C(0x0000000000005f8c), UINT64_C(0x1f43020fd0b726e6), UINT64_C(0x1f42fdcfa4b7e6e1),
        UINT64_C(0x1f4304ce98ae2709), UINT64_C(0x1f43054e90ac871d)
    },
    {   UINT64_C(0x0000000000005f8d), UINT64_C(0x1f43014e2cafa735), UINT64_C(0x1f43050f20b5670f),
        UINT64_C(0x1f4306cea0b02736), UINT64_C(0x1f4304cf40b5a730)
    },
    {   UINT64_C(0x0000000000005f8e), UINT64_C(0x1f43054fa0b64725), UINT64_C(0x1f43060f48b06714),
        UINT64_C(0x1f43030f5cb566d7), UINT64_C(0x1f42fa8e70aae740)
    },
    {   UINT64_C(0x0000000000005f8f), UINT64_C(0x1f43008ff8b946da), UINT64_C(0x1f42ff4f5cb4e720),
        UINT64_C(0x1f4308ce90b3e738), UINT64_C(0x1f42fc8e50b88744)
    },
    {   UINT64_C(0x0000000000005f90), UINT64_C(0x1f43054ef8afe700), UINT64_C(0x1f43024e5cb0871d),
        UINT64_C(0x1f43068e8caf0715), UINT64_C(0x1f42fe0eb8ab6733)
    },
    {   UINT64_C(0x0000000000005f91), UINT64_C(0x1f431f4e90ae671b), UINT64_C(0x1f43088e8cb6473b),
        UINT64_C(0x1f43070e44b72721), UINT64_C(0x1f42fc8f64b1a716)
    },
    {   UINT64_C(0x0000000000005f92), UINT64_C(0x1f43010f70b18705), UINT64_C(0x1f43054f4cb40709),
        UINT64_C(0x1f42fa8e78b5272d), UINT64_C(0x1f43008e80b22728)
    },
    {   UINT64_C(0x0000000000005f93), UINT64_C(0x1f43024ed8b0a723), UINT64_C(0x1f42feceecb34730),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f94), UINT64_C(0x1f43044f80b4a718), UINT64_C(0x1f43048f50b7670e),
        UINT64_C(0x1f42fe4eb0b2e72d), UINT64_C(0x1f430ccf28b686ed)
    },
    {   UINT64_C(0x0000000000005f95), UINT64_C(0x1f43048f28b54721), UINT64_C(0x1f43070f58b5673b),
        UINT64_C(0x1f43038f68b806fe), UINT64_C(0x1f43090ea8b30720)
    },
    {   UINT64_C(0x0000000000005f96), UINT64_C(0x1f43030ec0ae870c), UINT64_C(0x1f43014f20b8671c),
        UINT64_C(0x1f42fb0e90b4e737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005f97), UINT64_C(0x1f4304ce68b4c73f), UINT64_C(0x1f42ff8eecb2c6f5),
        UINT64_C(0x1f43030efcb5e725), UINT64_C(0x1f43050e9cb0e70d)
    },
    {   UINT64_C(0x0000000000005f98), UINT64_C(0x1f430c4e50a9e701), UINT64_C(0x1f4303ceecb5c730),
        UINT64_C(0x1f43044f1cb06728), UINT64_C(0x1f43044f1cb4c711)
    },
    {   UINT64_C(0x0000000000005f99), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f44b346ff), UINT64_C(0x1f42f94f34b5e6e9)
    },
    {   UINT64_C(0x0000000000005f9a), UINT64_C(0x1f43060ef8b3c719), UINT64_C(0x1f43030fc0b1a6f4),
        UINT64_C(0x1f43004ea4b60738), UINT64_C(0x1f43040e80b1a734)
    },
    {   UINT64_C(0x0000000000005f9b), UINT64_C(0x0000000000000000), UINT64_C(0x1f4316ce64a6c730),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ee0b7471a)
    },
    {   UINT64_C(0x0000000000005f9c), UINT64_C(0x1f4302ceccb5471c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f58b306fe), UINT64_C(0x1f43034fb4b7270c)
    },
    {   UINT64_C(0x0000000000005f9d), UINT64_C(0x1f430b4e70a4c725), UINT64_C(0x1f43040f2cae4719),
        UINT64_C(0x1f43040f94b2871e), UINT64_C(0x1f43038e84b3a73e)
    },
    {   UINT64_C(0x0000000000005f9e), UINT64_C(0x1f43020f2cb766f5), UINT64_C(0x1f43020e80b0c716),
        UINT64_C(0x1f43094ff4adc6e9), UINT64_C(0x1f43020eb8b34713)
    },
    {   UINT64_C(0x0000000000005f9f), UINT64_C(0x1f42ff4ff0b3670f), UINT64_C(0x1f43074e80b0a729),
        UINT64_C(0x1f42fecf2cb68721), UINT64_C(0x1f42ff9000b2e6ff)
    },
    {   UINT64_C(0x0000000000005fa0), UINT64_C(0x1f43028f50b64717), UINT64_C(0x1f43010edcaf2718),
        UINT64_C(0x1f4308ce54b2e728), UINT64_C(0x1f42f34e98b8e73d)
    },
    {   UINT64_C(0x0000000000005fa1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044f34b2c6f8), UINT64_C(0x1f4309cec4b10719)
    },
    {   UINT64_C(0x0000000000005fa2), UINT64_C(0x1f42ffcee4af071a), UINT64_C(0x1f43020eb0b02733),
        UINT64_C(0x1f4300ce80b0872d), UINT64_C(0x1f43064e50b62720)
    },
    {   UINT64_C(0x0000000000005fa3), UINT64_C(0x1f4300ce28ab273d), UINT64_C(0x1f43170e50a9872f),
        UINT64_C(0x1f42fa8e78b5272d), UINT64_C(0x1f4303cf14b12739)
    },
    {   UINT64_C(0x0000000000005fa4), UINT64_C(0x1f43038ee4b0e6f2), UINT64_C(0x1f4309cf3cb42711),
        UINT64_C(0x1f42fe4f90b3673a), UINT64_C(0x1f4307cf24b4c714)
    },
    {   UINT64_C(0x0000000000005fa5), UINT64_C(0x1f43024e68b6e73d), UINT64_C(0x1f43030e28b50731),
        UINT64_C(0x1f42fb4e78b5872d), UINT64_C(0x1f43078e34af2735)
    },
    {   UINT64_C(0x0000000000005fa6), UINT64_C(0x1f43020eb0b42740), UINT64_C(0x1f42ff8eccb4c6f2),
        UINT64_C(0x1f4306cea0b72711), UINT64_C(0x1f430b4e28b3c749)
    },
    {   UINT64_C(0x0000000000005fa7), UINT64_C(0x1f43024e70b60741), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4333cfe48de6df)
    },
    {   UINT64_C(0x0000000000005fa8), UINT64_C(0x1f43098f64a46719), UINT64_C(0x1f430a8f0cb3e6f4),
        UINT64_C(0x1f43099024b546ee), UINT64_C(0x1f430a8f20b2670d)
    },
    {   UINT64_C(0x0000000000005fa9), UINT64_C(0x1f43040ee8b10707), UINT64_C(0x1f430c4fd4b4270e),
        UINT64_C(0x1f430b0f64aae706), UINT64_C(0x1f4304cfdcb5e6f8)
    },
    {   UINT64_C(0x0000000000005faa), UINT64_C(0x1f43040ff8b3c70f), UINT64_C(0x1f43074f74ac2705),
        UINT64_C(0x1f430e9068b36706), UINT64_C(0x1f43091038ad6700)
    },
    {   UINT64_C(0x0000000000005fab), UINT64_C(0x1f4304cee8af870c), UINT64_C(0x1f4309d06cb166f0),
        UINT64_C(0x1f43094fbcade716), UINT64_C(0x1f43048fd4b58705)
    },
    {   UINT64_C(0x0000000000005fac), UINT64_C(0x1f43011040b3a6e0), UINT64_C(0x1f43064f48b346f7),
        UINT64_C(0x1f4306cf70abe6fc), UINT64_C(0x1f4305cfa4ac06ff)
    },
    {   UINT64_C(0x000000000000911a), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x15eafbcf709ba695), UINT64_C(0x1992f88fa0a14685)
    },
    {   UINT64_C(0x0000000000005fae), UINT64_C(0x1f43061018b366f0), UINT64_C(0x1f430ad054b466f0),
        UINT64_C(0x1f43060f8cae0701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005faf), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307d0d0b34700),
        UINT64_C(0x1f43078edcb3c70a), UINT64_C(0x1f43094ed8b0c703)
    },
    {   UINT64_C(0x0000000000005fb0), UINT64_C(0x1f430b0f5cb366f2), UINT64_C(0x1f4307105cb2e6e4),
        UINT64_C(0x1f4304cfdcb566ff), UINT64_C(0x1f43038f0cb0270b)
    },
    {   UINT64_C(0x0000000000005fb1), UINT64_C(0x1f430a104cb526ed), UINT64_C(0x1f4308ce68aec725),
        UINT64_C(0x1f430910d8b126ef), UINT64_C(0x1f43021018ae06ec)
    },
    {   UINT64_C(0x0000000000005fb2), UINT64_C(0x1f43058f60b74717), UINT64_C(0x1f430a1054ac86e5),
        UINT64_C(0x1f43088ee8b06728), UINT64_C(0x1f430a0f28afe709)
    },
    {   UINT64_C(0x0000000000005fb3), UINT64_C(0x1f43050fecab86f5), UINT64_C(0x1f430b0fe8aa06f3),
        UINT64_C(0x1f43024eacb4a6fc), UINT64_C(0x1f4309901cb3a6f6)
    },
    {   UINT64_C(0x0000000000005fb4), UINT64_C(0x1f430ad014ad66ee), UINT64_C(0x1f4303ce8cb246fe),
        UINT64_C(0x1f4308cf24b42718), UINT64_C(0x1f430a8fc8ae8711)
    },
    {   UINT64_C(0x000000000000911e), UINT64_C(0x1952fbcd389d0695), UINT64_C(0x190af88fa0a08685),
        UINT64_C(0x14fafbcf78968695), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x0000000000005fb6), UINT64_C(0x1f4305d000b446fe), UINT64_C(0x1f43044fc8b926fb),
        UINT64_C(0x1f43080f34ae872e), UINT64_C(0x1f43034eacb8272e)
    },
    {   UINT64_C(0x0000000000005fb7), UINT64_C(0x1f43058fb8b7e720), UINT64_C(0x1f4303900cb00709),
        UINT64_C(0x1f430a9028b446fa), UINT64_C(0x1f43044f8cb5e706)
    },
    {   UINT64_C(0x0000000000005fb8), UINT64_C(0x1f43080f50b566d7), UINT64_C(0x1f43074eecab670e),
        UINT64_C(0x1f430b4ef0a6c6e9), UINT64_C(0x1f43098f1cb6e70c)
    },
    {   UINT64_C(0x0000000000005fb9), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cf3cb246fe),
        UINT64_C(0x1f4302cee8b7471b), UINT64_C(0x1f4303cfa0b126f8)
    },
    {   UINT64_C(0x0000000000005fba), UINT64_C(0x1f43030ef4b0a725), UINT64_C(0x1f43074eecb06738),
        UINT64_C(0x1f43058efcb20719), UINT64_C(0x1f43044f24b4c71e)
    },
    {   UINT64_C(0x0000000000005fbb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074f68b0c72e),
        UINT64_C(0x1f43044e84b2e726), UINT64_C(0x1f43040f38b266fa)
    },
    {   UINT64_C(0x0000000000005fbc), UINT64_C(0x1f43064e6cb2c72c), UINT64_C(0x1f4303cf98ade6fe),
        UINT64_C(0x1f4308cff4b506e0), UINT64_C(0x1f43034f9cb3e701)
    },
    {   UINT64_C(0x0000000000005fbd), UINT64_C(0x1f43068f28b4671c), UINT64_C(0x1f43058eb8a98735),
        UINT64_C(0x1f43068f68b526fd), UINT64_C(0x1f43010f98acc6ee)
    },
    {   UINT64_C(0x0000000000005fbe), UINT64_C(0x1f43010f6ca8a6fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cfa8b566ea), UINT64_C(0x1f43048f60b5c71c)
    },
    {   UINT64_C(0x0000000000005fbf), UINT64_C(0x1f4304cf60ad86fe), UINT64_C(0x1f43064ff8b4671d),
        UINT64_C(0x1f43040f18aec70a), UINT64_C(0x1f43044f28b726f7)
    },
    {   UINT64_C(0x0000000000005fc0), UINT64_C(0x1f4303d008b6c6f2), UINT64_C(0x1f4305cfacac26ed),
        UINT64_C(0x1f43060ee8afe72a), UINT64_C(0x1f43068f68aec6da)
    },
    {   UINT64_C(0x0000000000005fc1), UINT64_C(0x1f43040f2cb44721), UINT64_C(0x1f43044fccb306f7),
        UINT64_C(0x1f43064f84b2a71e), UINT64_C(0x1f43064f90b4e6f7)
    },
    {   UINT64_C(0x0000000000005fc2), UINT64_C(0x1f43020fc4b486f0), UINT64_C(0x1f43080e8cb26703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0f08b1c719)
    },
    {   UINT64_C(0x0000000000005fc3), UINT64_C(0x1f43075020b1e6d3), UINT64_C(0x1f430a8fd4b466f2),
        UINT64_C(0x1f43070f84b786f0), UINT64_C(0x1f43038fdcb6e6e0)
    },
    {   UINT64_C(0x0000000000005fc4), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303d020b726e0),
        UINT64_C(0x1f42ff4f78b446e6), UINT64_C(0x1f4305cefcb4471b)
    },
    {   UINT64_C(0x0000000000005fc5), UINT64_C(0x1f43094e6cb3472a), UINT64_C(0x1f4302cf44afc705),
        UINT64_C(0x1f43098f9cabe703), UINT64_C(0x1f43050f30af46e6)
    },
    {   UINT64_C(0x0000000000005fc6), UINT64_C(0x1f42ff8ec8b7a725), UINT64_C(0x1f43050f0cb146e6),
        UINT64_C(0x1f43048ee8b1e739), UINT64_C(0x1f42fe8ec8b0a70d)
    },
    {   UINT64_C(0x0000000000005fc7), UINT64_C(0x1f43038efcb3471c), UINT64_C(0x1f43064f58b52708),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068f60ade70e)
    },
    {   UINT64_C(0x0000000000005fc8), UINT64_C(0x1f4302cfb0b1c725), UINT64_C(0x1f43080f3caf0707),
        UINT64_C(0x1f43041058b2a700), UINT64_C(0x1f4304ced4b2c72e)
    },
    {   UINT64_C(0x0000000000009128), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x182b02cf649b0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fca), UINT64_C(0x1f43020f68ade6fc), UINT64_C(0x1f4306cf44ad26ff),
        UINT64_C(0x1f43054ea0b7c713), UINT64_C(0x1f43090edcb54737)
    },
    {   UINT64_C(0x0000000000005fcb), UINT64_C(0x1f42ffcf60ad070d), UINT64_C(0x1f43014edcb68714),
        UINT64_C(0x1f4300cf5cb3a729), UINT64_C(0x1f4306ceb0b0072d)
    },
    {   UINT64_C(0x000000000000b829), UINT64_C(0x0bd2f08bfc6b45be), UINT64_C(0x0c6b00d64868c685),
        UINT64_C(0x0de2f78d607424c0), UINT64_C(0x0bc2f5978869c6a6)
    },
    {   UINT64_C(0x0000000000005fcd), UINT64_C(0x1f4303cfb4b72705), UINT64_C(0x1f43064f18b3071a),
        UINT64_C(0x1f43068f78af8717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fce), UINT64_C(0x1f42f8d07cb6c6f3), UINT64_C(0x1f43140ef8ac271b),
        UINT64_C(0x1f42fccfb4b546f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fcf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cebcb8272a),
        UINT64_C(0x1f42face7cb4e705), UINT64_C(0x1f43034f84ae26ea)
    },
    {   UINT64_C(0x0000000000005fd0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098ec4ae0714), UINT64_C(0x1f430f4ee0aee72a)
    },
    {   UINT64_C(0x0000000000005fd1), UINT64_C(0x1f42f9cec4b38726), UINT64_C(0x1f42fbce6cb0872d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fd2), UINT64_C(0x1f42f6ce7cb36726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040e94b1a72c), UINT64_C(0x1f43178e88ab6725)
    },
    {   UINT64_C(0x0000000000005fd3), UINT64_C(0x1f43074e70ad8720), UINT64_C(0x1f430c4eb8aee737),
        UINT64_C(0x1f42f64f24b7473f), UINT64_C(0x1f42f9cf04b78739)
    },
    {   UINT64_C(0x0000000000005fd4), UINT64_C(0x1f42fb8e98b3672e), UINT64_C(0x1f4315cf48a4c6e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fd5), UINT64_C(0x1f43128f4ca56718), UINT64_C(0x1f430d4e8cb0e6ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43220f4ca506ee)
    },
    {   UINT64_C(0x0000000000005fd6), UINT64_C(0x1f4315cedca646e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004ea8a4a70c), UINT64_C(0x1f43040e5cb78701)
    },
    {   UINT64_C(0x0000000000005fd7), UINT64_C(0x1f430e4f2cae270b), UINT64_C(0x1f430f8f20b126fd),
        UINT64_C(0x1f430d4e64a8e720), UINT64_C(0x1f42fd0f1cb266f5)
    },
    {   UINT64_C(0x0000000000005fd8), UINT64_C(0x1f431d0e74acc71c), UINT64_C(0x1f43054fb0adc70a),
        UINT64_C(0x1f430a8f38aee6f5), UINT64_C(0x1f43134f08abe708)
    },
    {   UINT64_C(0x0000000000005fd9), UINT64_C(0x0000000000000000), UINT64_C(0x1f4321ce2ca66717),
        UINT64_C(0x1f43150e9cab06fb), UINT64_C(0x1f43068e9cb4c700)
    },
    {   UINT64_C(0x0000000000005fda), UINT64_C(0x1f4302cf48b2c705), UINT64_C(0x1f43014e5cb28730),
        UINT64_C(0x1f430c4e98af2703), UINT64_C(0x1f430b4ebcac4707)
    },
    {   UINT64_C(0x0000000000005fdb), UINT64_C(0x1f43104f3cac26fe), UINT64_C(0x1f43160ecca926f3),
        UINT64_C(0x1f4313ced0b04704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fdc), UINT64_C(0x1f43018eb8a44724), UINT64_C(0x1f431bce8ca566f3),
        UINT64_C(0x1f43018e44b36719), UINT64_C(0x1f43124f40aec701)
    },
    {   UINT64_C(0x0000000000005fdd), UINT64_C(0x1f43124f58a886fe), UINT64_C(0x1f430f4e70a70725),
        UINT64_C(0x1f431fce84a8a71e), UINT64_C(0x1f43120e90b6871f)
    },
    {   UINT64_C(0x0000000000005fde), UINT64_C(0x1f43010f44b20724), UINT64_C(0x1f43098e9cb4271f),
        UINT64_C(0x1f43124edca9071d), UINT64_C(0x1f43074edcb10707)
    },
    {   UINT64_C(0x0000000000005fdf), UINT64_C(0x1f43038eb8af671e), UINT64_C(0x1f42fb8e90a32720),
        UINT64_C(0x1f43204e54a74719), UINT64_C(0x1f43160eb8ad670a)
    },
    {   UINT64_C(0x0000000000005fe0), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8f18afc706),
        UINT64_C(0x1f43100e44aa8723), UINT64_C(0x1f43080e70b5c72a)
    },
    {   UINT64_C(0x0000000000005fe1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43178ebcb14723),
        UINT64_C(0x1f43098ed0aae728), UINT64_C(0x1f43028e94b18717)
    },
    {   UINT64_C(0x0000000000005fe2), UINT64_C(0x1f43068ed0ac2715), UINT64_C(0x1f430e0ef4ada6f7),
        UINT64_C(0x1f4303ce9cb36744), UINT64_C(0x1f430c8eacb0671c)
    },
    {   UINT64_C(0x0000000000005fe3), UINT64_C(0x1f4304ce94b38727), UINT64_C(0x1f430d0e7cb2c71a),
        UINT64_C(0x1f430c8ef4b2473e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fe4), UINT64_C(0x0000000000000000), UINT64_C(0x1f431a0ef0a9072d),
        UINT64_C(0x1f43008f38b98705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fe5), UINT64_C(0x1f43044f3cb1272d), UINT64_C(0x1f43138e54ad672c),
        UINT64_C(0x1f43008f48b78709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fe6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8f2cb4e6fe), UINT64_C(0x1f431d0e4ca76736)
    },
    {   UINT64_C(0x0000000000005fe7), UINT64_C(0x1f42fc8eccb1e70b), UINT64_C(0x1f43010f34b32703),
        UINT64_C(0x1f42fc0f60aec73b), UINT64_C(0x1f42ff8f34b06703)
    },
    {   UINT64_C(0x0000000000005fe8), UINT64_C(0x1f43000f18aee6fd), UINT64_C(0x1f43184e50ae6733),
        UINT64_C(0x1f43034e9cae270c), UINT64_C(0x1f4306ce48af2745)
    },
    {   UINT64_C(0x0000000000005fe9), UINT64_C(0x1f42f80eb8b46732), UINT64_C(0x1f42fc8fc8af6700),
        UINT64_C(0x1f42facf60b66707), UINT64_C(0x1f42fd4fb4b2c710)
    },
    {   UINT64_C(0x0000000000005fea), UINT64_C(0x1f43124eacaa073a), UINT64_C(0x1f43010e64aee74e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005feb), UINT64_C(0x1f43054f5cb086e2), UINT64_C(0x1f42fa0e88b906fe),
        UINT64_C(0x1f43150ec4a80727), UINT64_C(0x1f42fb8eb8af671e)
    },
    {   UINT64_C(0x0000000000005fec), UINT64_C(0x1f42facef4b1c70b), UINT64_C(0x1f43034f50b12709),
        UINT64_C(0x1f4306ce4cb0c73f), UINT64_C(0x1f42f90f2cb3e6f8)
    },
    {   UINT64_C(0x0000000000005fed), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf00b1e71c),
        UINT64_C(0x1f42fc8f3cb0e72d), UINT64_C(0x1f42fe0f6cb78713)
    },
    {   UINT64_C(0x0000000000005fee), UINT64_C(0x1f42fa8ec8b7c738), UINT64_C(0x1f43134eb0a7073d),
        UINT64_C(0x1f43018f08b3a711), UINT64_C(0x1f42f94e58b7a727)
    },
    {   UINT64_C(0x0000000000005fef), UINT64_C(0x1f42f98e80b2472f), UINT64_C(0x1f42ffcfa0a3a6e5),
        UINT64_C(0x1f42f40f3cb846a9), UINT64_C(0x1f43028e9cb2e70f)
    },
    {   UINT64_C(0x0000000000005ff0), UINT64_C(0x1f42ff4ed8b326ea), UINT64_C(0x1f430a4ee4b5e72e),
        UINT64_C(0x1f42fb4f50b7a6c3), UINT64_C(0x1f43018f20aa26e4)
    },
    {   UINT64_C(0x0000000000005ff1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf1cb4871e),
        UINT64_C(0x1f42ff8e3cb2c736), UINT64_C(0x1f43088f20aba6f2)
    },
    {   UINT64_C(0x0000000000005ff2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf28aee703),
        UINT64_C(0x1f4304cf98abe70a), UINT64_C(0x1f42fe8fccb266e9)
    },
    {   UINT64_C(0x0000000000005ff3), UINT64_C(0x1f430a0fbcb24713), UINT64_C(0x1f42fd8f14b78705),
        UINT64_C(0x1f42f9cf70abc6f9), UINT64_C(0x1f43068ec4b4e702)
    },
    {   UINT64_C(0x0000000000005ff4), UINT64_C(0x1f42fe0fa8b546ec), UINT64_C(0x1f42fd8efcab8702),
        UINT64_C(0x1f43108ed0a7e729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ff5), UINT64_C(0x1f42fd8fa0af2711), UINT64_C(0x1f4304ce00b80745),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058ee0ab46de)
    },
    {   UINT64_C(0x0000000000005ff6), UINT64_C(0x1f43034f88b1a70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42faced4af0716)
    },
    {   UINT64_C(0x0000000000008740), UINT64_C(0x0ff2f1cfa07c65bc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008741), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fc2f34fa07aa5b8)
    },
    {   UINT64_C(0x0000000000005ff9), UINT64_C(0x0000000000000000), UINT64_C(0x1f4304cea4ab0716),
        UINT64_C(0x1f430c4e58ab8733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ffa), UINT64_C(0x1f430f8e50ad8727), UINT64_C(0x1f42f6cf9cb0471d),
        UINT64_C(0x1f42fb0f48b48700), UINT64_C(0x1f4300cfecb706cd)
    },
    {   UINT64_C(0x0000000000005ffb), UINT64_C(0x1f43140e40aa8739), UINT64_C(0x1f42f78f74b726d5),
        UINT64_C(0x1f43038f54ae26d2), UINT64_C(0x1f431fced4a6c712)
    },
    {   UINT64_C(0x0000000000005ffc), UINT64_C(0x1f42f94eb0b80724), UINT64_C(0x1f42fbcef8aec73a),
        UINT64_C(0x1f42fdcf74b386e8), UINT64_C(0x1f430e8e88b58730)
    },
    {   UINT64_C(0x0000000000009142), UINT64_C(0x16cafa8f4093469f), UINT64_C(0x17fafa8c7098069f),
        UINT64_C(0x1932fa8f409ce69f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ffe), UINT64_C(0x1f4302cf40aec70c), UINT64_C(0x1f42fd0ef4b68726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf3caea713)
    },
    {   UINT64_C(0x0000000000005fff), UINT64_C(0x1f43088ea8b0c731), UINT64_C(0x1f42f98f20b1a701),
        UINT64_C(0x1f42fc4f2cafc6f1), UINT64_C(0x1f43000ebcb1a726)
    },
    {   UINT64_C(0x0000000000006000), UINT64_C(0x1f42fc8f34b34707), UINT64_C(0x1f42fd8f78b22735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028e44b3273d)
    },
    {   UINT64_C(0x0000000000006001), UINT64_C(0x1f43068eb4ad8736), UINT64_C(0x1f4300cee0b4473c),
        UINT64_C(0x1f42fdce54b1e735), UINT64_C(0x1f4308cef4a7670e)
    },
    {   UINT64_C(0x0000000000006002), UINT64_C(0x1f43068e80af4736), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4304ce84ae8736)
    },
    {   UINT64_C(0x0000000000006003), UINT64_C(0x1f430e8fecad66cc), UINT64_C(0x1f42f7cf0cb3c70a),
        UINT64_C(0x1f43124f2caa26ee), UINT64_C(0x1f43078ed4abe6d3)
    },
    {   UINT64_C(0x0000000000006004), UINT64_C(0x1f4300ceb4b666f9), UINT64_C(0x1f43108e98aa270f),
        UINT64_C(0x1f43084e6cb44727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006005), UINT64_C(0x1f42f6cea4b74712), UINT64_C(0x1f42f9cedcb5e72e),
        UINT64_C(0x1f4301cf08b2870e), UINT64_C(0x1f42f90eccb9e717)
    },
    {   UINT64_C(0x0000000000006006), UINT64_C(0x1f42f58ef8b02726), UINT64_C(0x1f42f68ebcb40723),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcedca926ff)
    },
    {   UINT64_C(0x0000000000006007), UINT64_C(0x1f42f9cef8aa472b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf58b2c70b), UINT64_C(0x1f42fc8ef8b2272a)
    },
    {   UINT64_C(0x0000000000006008), UINT64_C(0x1f42fc8eecb72727), UINT64_C(0x1f42f68f38b4a71f),
        UINT64_C(0x1f43004f08af8716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006009), UINT64_C(0x1f42f4ce94b48710), UINT64_C(0x1f43008f38ade71b),
        UINT64_C(0x1f42f90f28b1e726), UINT64_C(0x1f43018ef8b3e71c)
    },
    {   UINT64_C(0x000000000000600a), UINT64_C(0x1f42f84e5cb5672a), UINT64_C(0x1f42fa8e48b54715),
        UINT64_C(0x1f42ff0eacb30714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000600b), UINT64_C(0x1f42f78eacb5a70e), UINT64_C(0x1f42f58e88b7c72f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000600c), UINT64_C(0x1f42fb8e50b5e720), UINT64_C(0x1f42f7cf78b2c71b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8e5cb36718)
    },
    {   UINT64_C(0x000000000000600d), UINT64_C(0x1f42fb0eecb3e6ce), UINT64_C(0x1f42fa4eecb446b3),
        UINT64_C(0x1f42fecf18abc6f4), UINT64_C(0x1f43100e8cb2271b)
    },
    {   UINT64_C(0x000000000000600e), UINT64_C(0x1f430e8e4cb2c707), UINT64_C(0x1f42fc4f64b6c6eb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43010f40b3e6ce)
    },
    {   UINT64_C(0x000000000000600f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f24b426ff),
        UINT64_C(0x1f43008ed8ad670c), UINT64_C(0x1f42f7cf74b746f8)
    },
    {   UINT64_C(0x000000000000b84b), UINT64_C(0x0ffb25113463257c), UINT64_C(0x0ffb25113063257b),
        UINT64_C(0x0b9af1930466250a), UINT64_C(0x0b9afb93046b250a)
    },
    {   UINT64_C(0x0000000000006011), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64eb4b9c72c), UINT64_C(0x1f43038f58b346c0)
    },
    {   UINT64_C(0x0000000000006012), UINT64_C(0x1f42fc0f18b0e6ee), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccee8b3a715), UINT64_C(0x1f43114e4caa872b)
    },
    {   UINT64_C(0x0000000000006013), UINT64_C(0x1f42fe8f50ae46d0), UINT64_C(0x1f42fc4f60af06f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4f94ada6c8)
    },
    {   UINT64_C(0x000000000000b84d), UINT64_C(0x09daee0ff065e5ce), UINT64_C(0x1cf327913495857d),
        UINT64_C(0x09daee0f1465e605), UINT64_C(0x204b275134a3257d)
    },
    {   UINT64_C(0x0000000000006015), UINT64_C(0x1f42fd0f70b166e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f0f20afc70a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b84e), UINT64_C(0x0f02eb98c07bc662), UINT64_C(0x0f02eb8cdc7bc369),
        UINT64_C(0x16fafecf74e14797), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008760), UINT64_C(0x0000000000000000), UINT64_C(0x1082f4cb247be693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006018), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cf68b7c6f5),
        UINT64_C(0x1f42fbcf04b4c6ec), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006019), UINT64_C(0x1f42fb0ec0b6c72d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ff4b806f2)
    },
    {   UINT64_C(0x000000000000601a), UINT64_C(0x1f42f6ceb8b7472c), UINT64_C(0x1f42fb0f08b06728),
        UINT64_C(0x1f42fdcf70b306f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000601b), UINT64_C(0x1f42f8ced4b4e71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ceacae072c)
    },
    {   UINT64_C(0x000000000000601c), UINT64_C(0x1f42f84eacb38737), UINT64_C(0x1f4309ce4cb0673e),
        UINT64_C(0x1f42fece1cb18736), UINT64_C(0x1f43028e6cb5a73b)
    },
    {   UINT64_C(0x000000000000601d), UINT64_C(0x1f43018f34b1e714), UINT64_C(0x1f42f80f1cb06728),
        UINT64_C(0x1f4300ce3cb3473b), UINT64_C(0x1f42ffcedcb2e700)
    },
    {   UINT64_C(0x000000000000601e), UINT64_C(0x1f42fb4f4cade735), UINT64_C(0x1f42f9ce14b68724),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000601f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcf78b02727),
        UINT64_C(0x1f430cce3cb0073b), UINT64_C(0x1f43048e98b46738)
    },
    {   UINT64_C(0x000000000000b853), UINT64_C(0x1acafa0d289fa67c), UINT64_C(0x1acb020d28a3a67c),
        UINT64_C(0x156af90d2c8aa67d), UINT64_C(0x156b010d2c8ea67d)
    },
    {   UINT64_C(0x0000000000009154), UINT64_C(0x119af10fa094267e), UINT64_C(0x0000000000000000),
        UINT64_C(0x118af94f787dc5b4), UINT64_C(0x11aafa8f78800693)
    },
    {   UINT64_C(0x0000000000006022), UINT64_C(0x0000000000000000), UINT64_C(0x1f431bcf08a38723),
        UINT64_C(0x1f42f90f90b166c6), UINT64_C(0x1f42f84fd8ad26f9)
    },
    {   UINT64_C(0x0000000000006023), UINT64_C(0x1f42fa4fe8b7c6b6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43060e00b9a71a)
    },
    {   UINT64_C(0x0000000000006024), UINT64_C(0x1f42f90f50b566f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f74b946e4), UINT64_C(0x1f43084e58aa4732)
    },
    {   UINT64_C(0x0000000000006025), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0fb8afa6be),
        UINT64_C(0x1f43224f089ea712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006026), UINT64_C(0x1f42fa0ee4b16718), UINT64_C(0x1f43354eb89de6fd),
        UINT64_C(0x1f42fd1000b446f6), UINT64_C(0x1f42f81028b366dc)
    },
    {   UINT64_C(0x0000000000009157), UINT64_C(0x113afa8fa081e693), UINT64_C(0x0812fbcfa057e693),
        UINT64_C(0x19f2fbcfa09f65b4), UINT64_C(0x1102fbcfa07ba5b4)
    },
    {   UINT64_C(0x0000000000006028), UINT64_C(0x1f42fa4f40b106bd), UINT64_C(0x1f42ff0fa4b526ed),
        UINT64_C(0x1f42f84f54b52716), UINT64_C(0x1f42f64ef8b726ef)
    },
    {   UINT64_C(0x0000000000006029), UINT64_C(0x1f43004fe4b4c6e9), UINT64_C(0x1f42fe1028b566da),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000602a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e489d4732),
        UINT64_C(0x1f42fa0f9cb026eb), UINT64_C(0x1f42f80ff8b18713)
    },
    {   UINT64_C(0x000000000000602b), UINT64_C(0x1f42f8cf1cb60708), UINT64_C(0x1f42f80ff0ac86f8),
        UINT64_C(0x1f42f64f80b866e0), UINT64_C(0x1f43018e7cb1e738)
    },
    {   UINT64_C(0x000000000000602c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f94ad870c),
        UINT64_C(0x1f43098e50afe732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000602d), UINT64_C(0x1f43044e8cab0730), UINT64_C(0x1f42fa4ee8b026cf),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43011004b546fc)
    },
    {   UINT64_C(0x000000000000602e), UINT64_C(0x1f42f80e9cb5470e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0e6cb20718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000602f), UINT64_C(0x1f42fd0f18b3e716), UINT64_C(0x1f43001034b446e1),
        UINT64_C(0x1f4320ced0b28703), UINT64_C(0x1f42fa4f14afa6fe)
    },
    {   UINT64_C(0x0000000000006030), UINT64_C(0x1f42fe4f74b4a6e6), UINT64_C(0x1f42f74fd8b006e3),
        UINT64_C(0x1f42f90eb8ae870d), UINT64_C(0x1f43238edca44727)
    },
    {   UINT64_C(0x0000000000006031), UINT64_C(0x1f43274ed4a1c734), UINT64_C(0x1f42f64f98b546e9),
        UINT64_C(0x1f42fa4fa4b266bd), UINT64_C(0x1f42fb0eb8aee6f6)
    },
    {   UINT64_C(0x0000000000006032), UINT64_C(0x1f42f84f88b3e6e9), UINT64_C(0x1f42fb0fa0a8c6d8),
        UINT64_C(0x1f42fc0ef4af8701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000915d), UINT64_C(0x18b2fb8f3c9a66a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1b02f88f3ca446a5), UINT64_C(0x15dafacf3c8f26a5)
    },
    {   UINT64_C(0x0000000000006034), UINT64_C(0x1f42fe0fb0ab86c2), UINT64_C(0x1f42ff0f70b486c0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006035), UINT64_C(0x1f430b4e14ae4732), UINT64_C(0x1f42f34ec8b5a6e4),
        UINT64_C(0x1f42fb0ea8b44730), UINT64_C(0x1f42f80e94b6a704)
    },
    {   UINT64_C(0x0000000000006036), UINT64_C(0x1f42fb0f64b486f2), UINT64_C(0x1f42fd0e98aba6f0),
        UINT64_C(0x1f42ff1010afc6c3), UINT64_C(0x1f42f80ed8ae06e8)
    },
    {   UINT64_C(0x000000000000915f), UINT64_C(0x1d12fb8eb4af46a5), UINT64_C(0x1ba2fa8f3ca7a6a5),
        UINT64_C(0x1aa2fb8f54a2c6a5), UINT64_C(0x18cafb8f3c9b06a5)
    },
    {   UINT64_C(0x0000000000006038), UINT64_C(0x1f42fd0fbcadc6ea), UINT64_C(0x1f430b0f14a92734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f34f78b126fe)
    },
    {   UINT64_C(0x0000000000006039), UINT64_C(0x1f42f60f18b64722), UINT64_C(0x1f4304ce50b7a732),
        UINT64_C(0x1f42fa0fa8b3c6dc), UINT64_C(0x1f42fe1004b56702)
    },
    {   UINT64_C(0x000000000000603a), UINT64_C(0x1f43000f84b2a714), UINT64_C(0x1f43000ef4ae8715),
        UINT64_C(0x1f43010f84ab0712), UINT64_C(0x1f42fe0fc8b046c9)
    },
    {   UINT64_C(0x000000000000603b), UINT64_C(0x1f430a0e94b10732), UINT64_C(0x1f42f74fc8a926ca),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f44eacb8073e)
    },
    {   UINT64_C(0x000000000000603c), UINT64_C(0x1f42f54f68b2c727), UINT64_C(0x1f42f55008b386fe),
        UINT64_C(0x1f42fa0f94a966ef), UINT64_C(0x1f42f54f5cb6e722)
    },
    {   UINT64_C(0x000000000000603d), UINT64_C(0x1f42f74fb8ac26cd), UINT64_C(0x1f42f75014b246c6),
        UINT64_C(0x1f42fd5010b366fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000603e), UINT64_C(0x1f42f80f28ad2721), UINT64_C(0x1f42fb0fa4ad86bf),
        UINT64_C(0x1f42fa0f14af06d7), UINT64_C(0x1f42f74eecb286f8)
    },
    {   UINT64_C(0x000000000000603f), UINT64_C(0x1f42f64ee8b2c722), UINT64_C(0x1f430e0f30b4271e),
        UINT64_C(0x1f43054ec0b02714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006040), UINT64_C(0x1f43174e74ad66fd), UINT64_C(0x1f42f90f9cb206df),
        UINT64_C(0x1f43000f6cae86f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006041), UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e64a5c736),
        UINT64_C(0x1f42f80f78b4871b), UINT64_C(0x1f42f78f94b6c726)
    },
    {   UINT64_C(0x0000000000006042), UINT64_C(0x1f42f74f2cb9a728), UINT64_C(0x1f42f90f78b7270a),
        UINT64_C(0x1f43008e60b8a737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006043), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb1010b5c6e2),
        UINT64_C(0x1f42fc4fa0ade6f5), UINT64_C(0x1f42f94fe0b166fe)
    },
    {   UINT64_C(0x000000000000b865), UINT64_C(0x16a2f10fa0978691), UINT64_C(0x1742f54fa097e68c),
        UINT64_C(0x10caf50f788105b7), UINT64_C(0x0f92f84c44796693)
    },
    {   UINT64_C(0x0000000000006045), UINT64_C(0x1f43170eb4a6072a), UINT64_C(0x1f42f80f34b286fa),
        UINT64_C(0x1f42f74e98b0672e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006046), UINT64_C(0x1f43004f5cb446f2), UINT64_C(0x1f42f74eb8b686e8),
        UINT64_C(0x1f42fc0eb8b066f3), UINT64_C(0x1f42fe4f70b146c9)
    },
    {   UINT64_C(0x0000000000006047), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0ed8af8712),
        UINT64_C(0x1f42fe4f88af46da), UINT64_C(0x1f42fe4ea8b146f4)
    },
    {   UINT64_C(0x0000000000006048), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84fb0b4e6af),
        UINT64_C(0x1f42f68f30b3a71f), UINT64_C(0x1f4303ce78ac0717)
    },
    {   UINT64_C(0x0000000000006049), UINT64_C(0x1f42fa0f24b7e6e3), UINT64_C(0x1f42ff0f6cb2c6fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f1cb4a708)
    },
    {   UINT64_C(0x000000000000604a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90efcb72735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f78b76715)
    },
    {   UINT64_C(0x000000000000604b), UINT64_C(0x1f42fd4f08b266fd), UINT64_C(0x1f42fb0f78b2a6d7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54f70af2709)
    },
    {   UINT64_C(0x000000000000604c), UINT64_C(0x1f42fc0fd0b366c3), UINT64_C(0x1f42fa4f84b7071d),
        UINT64_C(0x1f42fd0f38b546d2), UINT64_C(0x1f42fd5018b3e6f0)
    },
    {   UINT64_C(0x000000000000604d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0fa4adc6c6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000604e), UINT64_C(0x1f42f58ffcb726e4), UINT64_C(0x1f42fb0f18b9071c),
        UINT64_C(0x1f42fa0e7cafa72d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000604f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f88b24709),
        UINT64_C(0x1f42fb0e5cb4c739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006050), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80edcb7c702)
    },
    {   UINT64_C(0x0000000000006051), UINT64_C(0x1f42ff0f7cb0e6de), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90fc4b42717), UINT64_C(0x1f42fa0f0cb186ff)
    },
    {   UINT64_C(0x0000000000006052), UINT64_C(0x1f42fd0ed4b00724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94f20b00716), UINT64_C(0x1f42f90f84b7871d)
    },
    {   UINT64_C(0x0000000000006053), UINT64_C(0x1f42fc0f3cb5c70f), UINT64_C(0x1f42f64f8cb3a708),
        UINT64_C(0x1f4315cea4b7e72b), UINT64_C(0x1f42fa104cb5a6dd)
    },
    {   UINT64_C(0x0000000000006054), UINT64_C(0x1f42fd4ec4ade73c), UINT64_C(0x1f42fc0f28b50721),
        UINT64_C(0x1f42fb4f5cb006d9), UINT64_C(0x1f42fa0ee8b36719)
    },
    {   UINT64_C(0x0000000000006055), UINT64_C(0x1f42f90fd0b246f9), UINT64_C(0x1f42f75014b76715),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006056), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f34ac26f7),
        UINT64_C(0x1f42fa4fe4b68706), UINT64_C(0x1f42f94fb4af66cf)
    },
    {   UINT64_C(0x0000000000006057), UINT64_C(0x1f42f90e8cb4c71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94ed4b7a6fc), UINT64_C(0x1f42fd0f80b42706)
    },
    {   UINT64_C(0x00000000000087a1), UINT64_C(0x1d82ef4f74b3072a), UINT64_C(0x1692f34f68954656),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006059), UINT64_C(0x1f42f94f1cae0702), UINT64_C(0x1f42fd0fc8b1c6db),
        UINT64_C(0x1f42f80ee0adc713), UINT64_C(0x1f42f54ed4b8670e)
    },
    {   UINT64_C(0x000000000000605a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ef0b0a6fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74fb4afc72c)
    },
    {   UINT64_C(0x000000000000605b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f04ac26f0),
        UINT64_C(0x1f42f74fc8b366da), UINT64_C(0x1f42f74f50b0a717)
    },
    {   UINT64_C(0x000000000000605c), UINT64_C(0x1f42f80fc8b606fd), UINT64_C(0x1f42f74f5cb48715),
        UINT64_C(0x1f42f80f60b2a6d0), UINT64_C(0x1f42f90ec8b2c6c3)
    },
    {   UINT64_C(0x000000000000605d), UINT64_C(0x1f42f74fa8b6c70a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f54f48b7c6d8), UINT64_C(0x1f42f58f30ae46da)
    },
    {   UINT64_C(0x000000000000605e), UINT64_C(0x1f42fa0f1cb24706), UINT64_C(0x1f42fb0f78b64729),
        UINT64_C(0x1f42f44ef4b7272c), UINT64_C(0x1f42f74f80b2e6fd)
    },
    {   UINT64_C(0x000000000000605f), UINT64_C(0x1f42fb1004aee6cd), UINT64_C(0x1f42f81030b1c6e5),
        UINT64_C(0x1f42f74ef4b6e71f), UINT64_C(0x1f42f74fb8b7a6e7)
    },
    {   UINT64_C(0x0000000000006060), UINT64_C(0x1f42fb0e68b56721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffce54b2c742), UINT64_C(0x1f42f81018b5a6eb)
    },
    {   UINT64_C(0x0000000000006061), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ed8ae8728), UINT64_C(0x1f42f80f88b306e8)
    },
    {   UINT64_C(0x0000000000006062), UINT64_C(0x1f42f74f90ac26f9), UINT64_C(0x1f42f74f68b2c6fa),
        UINT64_C(0x1f42fa0f18b1e734), UINT64_C(0x1f43094e44b30728)
    },
    {   UINT64_C(0x0000000000006063), UINT64_C(0x1f42f84ef4b9e704), UINT64_C(0x1f42f90f20b3e709),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43100e58b1273a)
    },
    {   UINT64_C(0x0000000000006064), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f70ae06fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0e70b48733)
    },
    {   UINT64_C(0x0000000000006065), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f75020ab66e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0f08af66ed)
    },
    {   UINT64_C(0x0000000000006066), UINT64_C(0x1f42f80e6cae8701), UINT64_C(0x1f42f80f70b1a6ce),
        UINT64_C(0x1f42fd4e5cad6747), UINT64_C(0x1f42f90f80b30722)
    },
    {   UINT64_C(0x0000000000006067), UINT64_C(0x1f42fa0f84b5472d), UINT64_C(0x1f42f44eb8b86743),
        UINT64_C(0x1f43010ec0b3e711), UINT64_C(0x1f42f80e9cb5c70e)
    },
    {   UINT64_C(0x0000000000006068), UINT64_C(0x1f42f64e80b6672e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0f04aee711), UINT64_C(0x1f42f74fa4b14706)
    },
    {   UINT64_C(0x0000000000006069), UINT64_C(0x1f42fe8fb8b3471b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0e7cb20718), UINT64_C(0x1f42fb0f0cb4c712)
    },
    {   UINT64_C(0x000000000000606a), UINT64_C(0x1f42f54e54af4728), UINT64_C(0x1f42f94ee8b34731),
        UINT64_C(0x1f42f94f6cade716), UINT64_C(0x1f42f64ea8b9c71b)
    },
    {   UINT64_C(0x000000000000606b), UINT64_C(0x1f42f74f60b62720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0ec8b76716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000606c), UINT64_C(0x1f42fc0f30b6c708), UINT64_C(0x1f42fb0e90b36722),
        UINT64_C(0x1f42fa0f74b186d3), UINT64_C(0x1f42fc0f28b7a732)
    },
    {   UINT64_C(0x000000000000917a), UINT64_C(0x15db00cf548c868c), UINT64_C(0x1592f44f5891a68c),
        UINT64_C(0x1f2af58f64b7668c), UINT64_C(0x1722f80fa096068c)
    },
    {   UINT64_C(0x000000000000606e), UINT64_C(0x1f42f3cf70b8c73b), UINT64_C(0x1f42fa0eb4af2726),
        UINT64_C(0x1f42fa8e24ac073e), UINT64_C(0x1f42f54f9cab6721)
    },
    {   UINT64_C(0x000000000000606f), UINT64_C(0x1f42f80f64b726b2), UINT64_C(0x1f42fd0f38b24742),
        UINT64_C(0x1f42f34eb4b4c733), UINT64_C(0x1f42f68ee0b5073b)
    },
    {   UINT64_C(0x0000000000006070), UINT64_C(0x1f42fc8e54b4872c), UINT64_C(0x1f42f80ed4b5c711),
        UINT64_C(0x1f42f9ce64a9273c), UINT64_C(0x1f42f74ea8b46715)
    },
    {   UINT64_C(0x000000000000917c), UINT64_C(0x0e72f44f6875c5c1), UINT64_C(0x1032f44f6878668c),
        UINT64_C(0x0e22f44f6c74a5c1), UINT64_C(0x0aa2f18fa06d469b)
    },
    {   UINT64_C(0x0000000000006072), UINT64_C(0x1f42fdce3cb3c72d), UINT64_C(0x1f42f74f88b8a70f),
        UINT64_C(0x1f42f34f48b2c704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000087bc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f38fa0966762), UINT64_C(0x0fa2f18c3879e68e)
    },
    {   UINT64_C(0x0000000000006074), UINT64_C(0x1f42f90eccb22703), UINT64_C(0x1f42f64ed0b006e4),
        UINT64_C(0x1f42f74ea4b2a72b), UINT64_C(0x1f42f78ee8b6a735)
    },
    {   UINT64_C(0x0000000000006075), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f58b3e715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ef4b1e712)
    },
    {   UINT64_C(0x0000000000006076), UINT64_C(0x1f42f90f20b64732), UINT64_C(0x1f42f91020b1a70b),
        UINT64_C(0x1f42f90f30b4872c), UINT64_C(0x1f42fb4e88b0a6fa)
    },
    {   UINT64_C(0x0000000000006077), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4eb0b70738),
        UINT64_C(0x1f42fb0eb8af6705), UINT64_C(0x1f42f74ee8b3a721)
    },
    {   UINT64_C(0x0000000000006078), UINT64_C(0x1f430fce2cadc737), UINT64_C(0x1f430a0e28abe734),
        UINT64_C(0x1f42f74e50b54729), UINT64_C(0x1f43204fc8acc6f2)
    },
    {   UINT64_C(0x0000000000009180), UINT64_C(0x0c0af44f6c6b25c1), UINT64_C(0x0c12efcf706ce65e),
        UINT64_C(0x1032fbcf647865b5), UINT64_C(0x0e72f1cf687665b5)
    },
    {   UINT64_C(0x000000000000607a), UINT64_C(0x1f430e5054b28726), UINT64_C(0x1f430ace7cb2a71a),
        UINT64_C(0x1f432c0f2c99e72e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000607b), UINT64_C(0x1f43078e5cb1c73e), UINT64_C(0x1f430c8eb0b2473d),
        UINT64_C(0x1f43090ebcb2c723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000607c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf38b44701), UINT64_C(0x1f430a9028b146f6)
    },
    {   UINT64_C(0x000000000000607d), UINT64_C(0x1f43061020b7a718), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d0eb8aea6ed)
    },
    {   UINT64_C(0x000000000000607e), UINT64_C(0x1f43061038b166b3), UINT64_C(0x1f43038f90afc6f2),
        UINT64_C(0x1f430d0ea4b1c731), UINT64_C(0x1f42f9ce8cb6473e)
    },
    {   UINT64_C(0x000000000000607f), UINT64_C(0x1f4312cfb0ace6e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43088ec8b66727), UINT64_C(0x1f43068ee8b0270a)
    },
    {   UINT64_C(0x0000000000006080), UINT64_C(0x1f43160f14a58725), UINT64_C(0x1f43048e98b0e72f),
        UINT64_C(0x1f42f9ce9cb8072e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006081), UINT64_C(0x1f431c8f2ca586f7), UINT64_C(0x1f430e4e5ca2673b),
        UINT64_C(0x1f43048ef8b2c700), UINT64_C(0x1f4301cef0b44738)
    },
    {   UINT64_C(0x0000000000006082), UINT64_C(0x1f42fe8f9cb446ea), UINT64_C(0x1f42fc4e6cb88744),
        UINT64_C(0x1f430a0e68a98748), UINT64_C(0x1f43064f20b186f3)
    },
    {   UINT64_C(0x0000000000009185), UINT64_C(0x1a5b00cfa0b3267e), UINT64_C(0x151b008f689e467e),
        UINT64_C(0x165306cf689fe68c), UINT64_C(0x106b008fa08e467e)
    },
    {   UINT64_C(0x0000000000006084), UINT64_C(0x1f42f6cfd0b586f2), UINT64_C(0x1f42ff8ee8b466fd),
        UINT64_C(0x1f42f70ebcb8872b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009186), UINT64_C(0x166306cce49fe68c), UINT64_C(0x126b008fa08e667e),
        UINT64_C(0x10eafd8f6889268c), UINT64_C(0x1303164fa09c067e)
    },
    {   UINT64_C(0x0000000000006086), UINT64_C(0x1f42faceb8b54718), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcec8b04725)
    },
    {   UINT64_C(0x0000000000006087), UINT64_C(0x1f430e4e4caf674f), UINT64_C(0x1f430a0e20b3073f),
        UINT64_C(0x1f43068e9cb1c733), UINT64_C(0x1f42fcce48b5c737)
    },
    {   UINT64_C(0x0000000000006088), UINT64_C(0x1f42fc0f80ad6712), UINT64_C(0x1f42f6cf5cb5471d),
        UINT64_C(0x1f42f84f94ae671a), UINT64_C(0x1f42f84f98b62716)
    },
    {   UINT64_C(0x0000000000006089), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf44b5e711),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000608a), UINT64_C(0x1f42f8502cb52731), UINT64_C(0x1f42f3504cb8e73a),
        UINT64_C(0x1f42f64f6cb886fd), UINT64_C(0x1f42f38f44b20718)
    },
    {   UINT64_C(0x000000000000608b), UINT64_C(0x1f42f88f7cb7e729), UINT64_C(0x1f42f84f88b80718),
        UINT64_C(0x1f42fe8f8cb3472d), UINT64_C(0x1f42fe8f08aea709)
    },
    {   UINT64_C(0x000000000000608c), UINT64_C(0x1f42fbcfbcb7471d), UINT64_C(0x1f42f94f24b44718),
        UINT64_C(0x1f42fa4f3cb5c703), UINT64_C(0x1f42fb4f2cb0a713)
    },
    {   UINT64_C(0x000000000000608d), UINT64_C(0x1f42ff4e98ad870d), UINT64_C(0x1f42f74f58b38711),
        UINT64_C(0x1f42fa4f0caf6714), UINT64_C(0x1f42fa0eecb38702)
    },
    {   UINT64_C(0x000000000000608e), UINT64_C(0x1f42fa8e58b1671b), UINT64_C(0x1f42fa4f04b52701),
        UINT64_C(0x1f42fc8e9cb0a71e), UINT64_C(0x1f42f84fb0aae722)
    },
    {   UINT64_C(0x000000000000608f), UINT64_C(0x1f42f98f70b4e71e), UINT64_C(0x1f42f64e7cb6c715),
        UINT64_C(0x1f42f84f94b3071a), UINT64_C(0x1f42f90ed4b72715)
    },
    {   UINT64_C(0x0000000000006090), UINT64_C(0x1f42f84ef4b1e713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f2ce8cb70716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006091), UINT64_C(0x1f42fc8e50b1c6ff), UINT64_C(0x1f42f48e80b8a6fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f64b5c715)
    },
    {   UINT64_C(0x0000000000006092), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0fc4b60724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54f7cb1a719)
    },
    {   UINT64_C(0x0000000000006093), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf84af66fa), UINT64_C(0x1f42ff0f88ada70b)
    },
    {   UINT64_C(0x0000000000006094), UINT64_C(0x1f42fa4f00b50704), UINT64_C(0x1f43000fecb16707),
        UINT64_C(0x1f42fc4ef0ab870d), UINT64_C(0x1f42f94e8cb3c6ff)
    },
    {   UINT64_C(0x0000000000006095), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ed0b8e712),
        UINT64_C(0x1f42f84f3cb2c710), UINT64_C(0x1f42fd0f3cb5a6f6)
    },
    {   UINT64_C(0x0000000000006096), UINT64_C(0x1f42f68f20b9a723), UINT64_C(0x1f42f68f1cb82723),
        UINT64_C(0x1f42fc0f60b28725), UINT64_C(0x1f42fb0ec4b0e705)
    },
    {   UINT64_C(0x0000000000006097), UINT64_C(0x1f42fa4e4cb2c717), UINT64_C(0x1f42f54f9cb7e730),
        UINT64_C(0x1f42f54ef0b7a6f6), UINT64_C(0x1f42ef4f44b54723)
    },
    {   UINT64_C(0x0000000000006098), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ef4b5c706),
        UINT64_C(0x1f42f94e7cb5e73d), UINT64_C(0x1f42f84f00b6a731)
    },
    {   UINT64_C(0x0000000000006099), UINT64_C(0x1f42fa4ef8b36734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f9cb4271c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000609a), UINT64_C(0x1f42f88ef8b6c71d), UINT64_C(0x1f42f68e00b72731),
        UINT64_C(0x1f42f98e28b5a732), UINT64_C(0x1f42ff0f14b78739)
    },
    {   UINT64_C(0x000000000000609b), UINT64_C(0x1f42f68e8cb60733), UINT64_C(0x1f42f78f04b6e71d),
        UINT64_C(0x1f43270ee0b5e703), UINT64_C(0x1f432b8eb0b7272f)
    },
    {   UINT64_C(0x000000000000609c), UINT64_C(0x1f42fd8f50b806ca), UINT64_C(0x1f42fa0ec8b8a737),
        UINT64_C(0x1f43210f40b1c739), UINT64_C(0x1f42fd0f28b2270b)
    },
    {   UINT64_C(0x000000000000609d), UINT64_C(0x1f43048f6cb1e703), UINT64_C(0x1f42ff4f18b466fa),
        UINT64_C(0x1f42fb8f04b4c733), UINT64_C(0x1f42ff8f04b8871d)
    },
    {   UINT64_C(0x000000000000609e), UINT64_C(0x1f430b4ea8b00711), UINT64_C(0x1f43165410a74626),
        UINT64_C(0x1f43118f44ad671c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000609f), UINT64_C(0x1f43110e68ae2730), UINT64_C(0x1f430f4f2cb4e72e),
        UINT64_C(0x1f430f4f68b5e730), UINT64_C(0x1f4313cec0a8e732)
    },
    {   UINT64_C(0x00000000000060a0), UINT64_C(0x1f4318cef0ab872a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d4ed8aa2720), UINT64_C(0x1f4311cf70b46749)
    },
    {   UINT64_C(0x00000000000060a1), UINT64_C(0x1f430f4f78b4e72e), UINT64_C(0x1f430d0e3cb8673b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060a2), UINT64_C(0x1f43134ed8aa8714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4307cdf8aea745), UINT64_C(0x1f43068e50b14733)
    },
    {   UINT64_C(0x00000000000060a3), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4e48ac6746),
        UINT64_C(0x1f4318ce80a72736), UINT64_C(0x1f430e4f58b0c72a)
    },
    {   UINT64_C(0x00000000000060a4), UINT64_C(0x1f430e8e2cad872f), UINT64_C(0x1f430d0e98ae2733),
        UINT64_C(0x1f43084ee8b0871a), UINT64_C(0x1f43124e28af6746)
    },
    {   UINT64_C(0x00000000000060a5), UINT64_C(0x1f430aced8aca70a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43188ee4a5a721)
    },
    {   UINT64_C(0x00000000000060a6), UINT64_C(0x1f430ecf98b0672e), UINT64_C(0x1f43140ef4b3c730),
        UINT64_C(0x1f430e0f38b10730), UINT64_C(0x1f43104f50b06729)
    },
    {   UINT64_C(0x00000000000060a7), UINT64_C(0x1f43100f18ada722), UINT64_C(0x1f43010eb4b80739),
        UINT64_C(0x1f430f0e58b42728), UINT64_C(0x1f43128e64b1473b)
    },
    {   UINT64_C(0x00000000000060a8), UINT64_C(0x1f4312cf2cad6722), UINT64_C(0x1f4315cf98a9872a),
        UINT64_C(0x1f42ff4e8caf0734), UINT64_C(0x1f43158ef4a7871a)
    },
    {   UINT64_C(0x00000000000060a9), UINT64_C(0x1f430a0f28b1a725), UINT64_C(0x1f42fe4e30b7a73f),
        UINT64_C(0x1f430bcf78a8071c), UINT64_C(0x1f43130e9caaa73e)
    },
    {   UINT64_C(0x00000000000060aa), UINT64_C(0x1f4309cf40adc72e), UINT64_C(0x1f43030f38b6c726),
        UINT64_C(0x1f431c4eb4abc72a), UINT64_C(0x1f4313cf40aea72a)
    },
    {   UINT64_C(0x00000000000060ab), UINT64_C(0x1f430acf0cb8673e), UINT64_C(0x1f43128ed0af4730),
        UINT64_C(0x1f43100e98a7272d), UINT64_C(0x1f430c0fc0ae6718)
    },
    {   UINT64_C(0x00000000000060ac), UINT64_C(0x1f430f0e7cb4072e), UINT64_C(0x1f42fe4f68b3673d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43084e8cb26739)
    },
    {   UINT64_C(0x00000000000060ad), UINT64_C(0x1f4315cee0a7e72d), UINT64_C(0x1f4310cf00aee734),
        UINT64_C(0x1f4313cedca6a725), UINT64_C(0x1f430b8fd8ae6729)
    },
    {   UINT64_C(0x00000000000060ae), UINT64_C(0x1f43074f34ac272e), UINT64_C(0x1f430bce6caae73e),
        UINT64_C(0x1f430d8f3cae872f), UINT64_C(0x1f43290e50a6a73d)
    },
    {   UINT64_C(0x00000000000060af), UINT64_C(0x1f430f0f34b0c735), UINT64_C(0x1f43114e40aa6735),
        UINT64_C(0x1f43064e4cb18735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060b0), UINT64_C(0x1f430e0f10b28720), UINT64_C(0x1f430a4facb34708),
        UINT64_C(0x1f43120fd4ab0717), UINT64_C(0x1f430c0f6cade72f)
    },
    {   UINT64_C(0x00000000000060b1), UINT64_C(0x1f430c8edcaea72a), UINT64_C(0x1f43108edcaa472d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43150ed8a7e73a)
    },
    {   UINT64_C(0x00000000000060b2), UINT64_C(0x1f42fd4f04a6a715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084ed4ad672a), UINT64_C(0x1f43168f1cad272f)
    },
    {   UINT64_C(0x00000000000060b3), UINT64_C(0x1f43120f04af0729), UINT64_C(0x1f4309cf48b5871a),
        UINT64_C(0x1f43190fd4aa0715), UINT64_C(0x1f4313cf08ace719)
    },
    {   UINT64_C(0x000000000000b89d), UINT64_C(0x0000000000000000), UINT64_C(0x105afe8c8877a58d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060b5), UINT64_C(0x1f430c4eccad472a), UINT64_C(0x1f430fce30a7873c),
        UINT64_C(0x1f43108ed4a88712), UINT64_C(0x1f430f0f30af471b)
    },
    {   UINT64_C(0x00000000000060b6), UINT64_C(0x1f43138f08a7872a), UINT64_C(0x1f43094f94b3a71c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303ce64b06736)
    },
    {   UINT64_C(0x00000000000060b7), UINT64_C(0x1f430fcef0aae72a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43100eb4b3673d), UINT64_C(0x1f431aceb0a9e73f)
    },
    {   UINT64_C(0x00000000000060b8), UINT64_C(0x1f43130e1cae873c), UINT64_C(0x1f43094f50ad4724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4f24ac473b)
    },
    {   UINT64_C(0x00000000000060b9), UINT64_C(0x1f4313ceacaa673c), UINT64_C(0x1f4300cf78ae671f),
        UINT64_C(0x1f43038f68b54738), UINT64_C(0x1f430a4e40ad073e)
    },
    {   UINT64_C(0x00000000000060ba), UINT64_C(0x1f430ccf2ca9672a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43168e84a9e732), UINT64_C(0x1f4312cecca7873d)
    },
    {   UINT64_C(0x00000000000060bb), UINT64_C(0x1f4314cef8a92726), UINT64_C(0x1f4313cea4ac673c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028ea8b5472a)
    },
    {   UINT64_C(0x00000000000060bc), UINT64_C(0x1f4317cedcaac73c), UINT64_C(0x1f43058f4cb36741),
        UINT64_C(0x1f4315cf04a5a726), UINT64_C(0x1f430bcf18b4873a)
    },
    {   UINT64_C(0x00000000000060bd), UINT64_C(0x1f43068f2cb04736), UINT64_C(0x1f4314ce5cb1673a),
        UINT64_C(0x1f43060e50b7c744), UINT64_C(0x1f43068f00afe72a)
    },
    {   UINT64_C(0x00000000000060be), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f4e4cb0e73b),
        UINT64_C(0x1f4301ce58b5273c), UINT64_C(0x1f42ff0eacb08740)
    },
    {   UINT64_C(0x00000000000060bf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ccf54a9e73c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060c0), UINT64_C(0x1f4317cef8a72735), UINT64_C(0x1f43104f2cadc739),
        UINT64_C(0x1f430c0e70b3a73d), UINT64_C(0x1f43170f4cb5c735)
    },
    {   UINT64_C(0x00000000000060c1), UINT64_C(0x1f4300cf0cad0723), UINT64_C(0x1f430f4e48b18749),
        UINT64_C(0x1f43178e3ca8673d), UINT64_C(0x1f43034ea8b24736)
    },
    {   UINT64_C(0x00000000000060c2), UINT64_C(0x1f430a0facb22730), UINT64_C(0x1f43088ec8b3474e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060c3), UINT64_C(0x1f42fc4e2cae8737), UINT64_C(0x1f4316ceb0a98739),
        UINT64_C(0x1f43148f5ca8871a), UINT64_C(0x1f4313cf34b0472a)
    },
    {   UINT64_C(0x00000000000060c4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43180ea8a68728),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060c5), UINT64_C(0x1f43149028ab6716), UINT64_C(0x1f4313cf10ab6726),
        UINT64_C(0x1f4306cef0b38735), UINT64_C(0x1f42f88efca96733)
    },
    {   UINT64_C(0x00000000000060c6), UINT64_C(0x1f430a0f18ada745), UINT64_C(0x1f430f0ee4a72726),
        UINT64_C(0x1f43198eaca80717), UINT64_C(0x1f43198e1cb4472e)
    },
    {   UINT64_C(0x00000000000060c7), UINT64_C(0x1f43150f60a5e71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dce38a42731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060c8), UINT64_C(0x1f43024ef8b34736), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4310cee0a6871b), UINT64_C(0x1f430a0f1cad8745)
    },
    {   UINT64_C(0x00000000000060c9), UINT64_C(0x1f4307ce60a7273f), UINT64_C(0x1f43144f88abc726),
        UINT64_C(0x1f43104e88ae6729), UINT64_C(0x1f43160ee0a8871e)
    },
    {   UINT64_C(0x00000000000060ca), UINT64_C(0x1f430e4e84b3072a), UINT64_C(0x1f42fa4f14a4a72d),
        UINT64_C(0x1f43114f2caec72a), UINT64_C(0x1f43188f64ade72d)
    },
    {   UINT64_C(0x0000000000008814), UINT64_C(0x18b2fbcf009a669b), UINT64_C(0x0faaff0cf474a68c),
        UINT64_C(0x0ebafb8c8072a6aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008815), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x163300d5e48de68e)
    },
    {   UINT64_C(0x00000000000060cd), UINT64_C(0x1f430e0f00a96733), UINT64_C(0x1f43060e34aae730),
        UINT64_C(0x1f43054e9cbac74b), UINT64_C(0x1f43028e74b12731)
    },
    {   UINT64_C(0x0000000000008817), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0fa03e83e8)
    },
    {   UINT64_C(0x00000000000060cf), UINT64_C(0x1f4315ce38a7072d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306cf7cb60721)
    },
    {   UINT64_C(0x00000000000060d0), UINT64_C(0x1f43140e14ab674d), UINT64_C(0x1f430d4f6cad671c),
        UINT64_C(0x1f4308ceb4b38733), UINT64_C(0x1f42fd4f58b6a728)
    },
    {   UINT64_C(0x00000000000060d1), UINT64_C(0x1f43168ed8a5c726), UINT64_C(0x1f43014ee0b44735),
        UINT64_C(0x1f4303ce74a9c717), UINT64_C(0x1f43044e44ab272a)
    },
    {   UINT64_C(0x00000000000060d2), UINT64_C(0x1f43078e5cad6726), UINT64_C(0x1f43038e1cb2c73e),
        UINT64_C(0x1f43088f04a72719), UINT64_C(0x1f43178f0ca70724)
    },
    {   UINT64_C(0x00000000000060d3), UINT64_C(0x1f43150eacaee735), UINT64_C(0x1f430b8f28b10721),
        UINT64_C(0x1f43144ec0ac4726), UINT64_C(0x1f42fe4f18af8738)
    },
    {   UINT64_C(0x00000000000060d4), UINT64_C(0x1f42fece38b22741), UINT64_C(0x1f4311cef4ad6733),
        UINT64_C(0x1f43098ee4b52731), UINT64_C(0x1f43150e9cae0729)
    },
    {   UINT64_C(0x00000000000060d5), UINT64_C(0x1f43050f1ca8c732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43068f08b2e732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060d6), UINT64_C(0x1f43008ef4b2273c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e8f50ab072f), UINT64_C(0x1f43148f14ae871e)
    },
    {   UINT64_C(0x00000000000060d7), UINT64_C(0x1f430b8eacaec735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b8e6cb3e731), UINT64_C(0x1f43050f50b5e715)
    },
    {   UINT64_C(0x00000000000060d8), UINT64_C(0x1f4310cee8b5472a), UINT64_C(0x1f4311cf18a78722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43134f18acc715)
    },
    {   UINT64_C(0x00000000000060d9), UINT64_C(0x1f430f8e8caae724), UINT64_C(0x1f43108eacb1c719),
        UINT64_C(0x1f43128ef8ada72a), UINT64_C(0x1f43148f44ab072a)
    },
    {   UINT64_C(0x00000000000060da), UINT64_C(0x1f43114e98ac673b), UINT64_C(0x1f43088f44b30721),
        UINT64_C(0x1f4302cf24ac8712), UINT64_C(0x1f43148ee8a80725)
    },
    {   UINT64_C(0x00000000000060db), UINT64_C(0x1f430b8ef0ac6719), UINT64_C(0x1f43148f7cae670b),
        UINT64_C(0x1f430f4eccae8719), UINT64_C(0x1f43168eeca90726)
    },
    {   UINT64_C(0x00000000000060dc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43134f7ca7e72a)
    },
    {   UINT64_C(0x00000000000060dd), UINT64_C(0x1f43028f08af671e), UINT64_C(0x1f430fcefcaa0726),
        UINT64_C(0x1f42fe8f0cb80711), UINT64_C(0x1f42fa0e54b16737)
    },
    {   UINT64_C(0x00000000000060de), UINT64_C(0x1f43120e44aaa746), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4ef4aa0727), UINT64_C(0x1f42fc8ebcb5c725)
    },
    {   UINT64_C(0x0000000000008828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1162f0cfa082c67e)
    },
    {   UINT64_C(0x0000000000008829), UINT64_C(0x0fa2f10f807bc68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x24f2f20f78cd267e)
    },
    {   UINT64_C(0x00000000000060e1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43090f54aae713),
        UINT64_C(0x1f43024e70b54746), UINT64_C(0x1f43038f7ca78726)
    },
    {   UINT64_C(0x00000000000060e2), UINT64_C(0x1f430d4ea8b1272f), UINT64_C(0x1f42fd0e44a8073e),
        UINT64_C(0x1f43064eacafa73c), UINT64_C(0x1f43014e6ca8072b)
    },
    {   UINT64_C(0x00000000000060e3), UINT64_C(0x1f431c0e84b04720), UINT64_C(0x1f43198e64b2472b),
        UINT64_C(0x1f43180e6cafa73a), UINT64_C(0x1f4302cf14ac2731)
    },
    {   UINT64_C(0x00000000000060e4), UINT64_C(0x1f430f4f20ac2721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d0f0caae735), UINT64_C(0x1f42f9ce6cb7a73f)
    },
    {   UINT64_C(0x00000000000060e5), UINT64_C(0x1f43188e80a30724), UINT64_C(0x1f4306ce44b1e716),
        UINT64_C(0x1f43064ef0b00729), UINT64_C(0x1f43114eaca9e732)
    },
    {   UINT64_C(0x00000000000060e6), UINT64_C(0x1f43074e6cb2e73a), UINT64_C(0x1f43088eecb24734),
        UINT64_C(0x1f43088f28aae72e), UINT64_C(0x1f43074f00ae6738)
    },
    {   UINT64_C(0x00000000000060e7), UINT64_C(0x1f43080f64b1071e), UINT64_C(0x1f43170f70a6871e),
        UINT64_C(0x1f43144ee8b2a738), UINT64_C(0x1f42f90e28b6072d)
    },
    {   UINT64_C(0x00000000000060e8), UINT64_C(0x1f43094ec8b2a73c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124ee4af6735), UINT64_C(0x1f43144f34ade728)
    },
    {   UINT64_C(0x00000000000060e9), UINT64_C(0x1f43150e80a30724), UINT64_C(0x1f42ff8ed0b1e726),
        UINT64_C(0x1f43150e70a2c72e), UINT64_C(0x1f430f8e6cb58749)
    },
    {   UINT64_C(0x00000000000060ea), UINT64_C(0x1f43130f00ace732), UINT64_C(0x1f43078e80aec73b),
        UINT64_C(0x1f430a8e6cac672d), UINT64_C(0x1f43038ea4afe734)
    },
    {   UINT64_C(0x0000000000008834), UINT64_C(0x00027cd96c3c66e0), UINT64_C(0x00027cd96c3c66e0),
        UINT64_C(0x0002b18d3026867e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060ec), UINT64_C(0x1f43074f38aee722), UINT64_C(0x1f430e8ea4ac072f),
        UINT64_C(0x1f43038f18abe726), UINT64_C(0x1f431d4f04ace715)
    },
    {   UINT64_C(0x00000000000060ed), UINT64_C(0x1f43188eb4a4c72a), UINT64_C(0x1f43090ea0ae0735),
        UINT64_C(0x1f4305cf74b7070e), UINT64_C(0x1f430d4ef4b4472c)
    },
    {   UINT64_C(0x00000000000060ee), UINT64_C(0x1f42fd8ef8b18719), UINT64_C(0x1f43158f70ac072a),
        UINT64_C(0x1f43118e78aa272a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060ef), UINT64_C(0x1f43094f48b4271b), UINT64_C(0x1f42fdce50b8073f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4e4ca9e73e)
    },
    {   UINT64_C(0x00000000000060f0), UINT64_C(0x1f430f4f30ae472c), UINT64_C(0x1f4302ce94afe739),
        UINT64_C(0x1f43110ec8a8671d), UINT64_C(0x1f43100f10a7a726)
    },
    {   UINT64_C(0x000000000000883a), UINT64_C(0x00027a8c3842e371), UINT64_C(0x0002761808452691),
        UINT64_C(0x00024990fc4d458b), UINT64_C(0x00027e0c3c56a6dc)
    },
    {   UINT64_C(0x00000000000060f2), UINT64_C(0x1f43030f08aee729), UINT64_C(0x1f4316cef4ae6726),
        UINT64_C(0x1f43088e48aa6743), UINT64_C(0x1f4308cef8b0e739)
    },
    {   UINT64_C(0x000000000000883c), UINT64_C(0x000294d7b433a71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060f4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43144e74a90732),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43060ef8af8740)
    },
    {   UINT64_C(0x00000000000060f5), UINT64_C(0x1f43080eb0b5e738), UINT64_C(0x1f43044eacb68727),
        UINT64_C(0x1f430fcea4a5072c), UINT64_C(0x1f43080e8cab671b)
    },
    {   UINT64_C(0x00000000000060f6), UINT64_C(0x1f42fcceacb56727), UINT64_C(0x1f43098ea0b3672d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43130e40a7872a)
    },
    {   UINT64_C(0x0000000000008840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x160af94f6891668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060f8), UINT64_C(0x1f430b0f10b3e741), UINT64_C(0x1f430e0eb8a7a728),
        UINT64_C(0x1f43060e9caf8717), UINT64_C(0x1f4311cf7cb5c721)
    },
    {   UINT64_C(0x00000000000060f9), UINT64_C(0x1f4306cedcb2e72b), UINT64_C(0x1f43028ee8a86729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43070f3ca8072e)
    },
    {   UINT64_C(0x0000000000008843), UINT64_C(0x19c2f8cc709f269f), UINT64_C(0x1672ff8fa08f867e),
        UINT64_C(0x120af9cf7882a693), UINT64_C(0x121af9cf7882e693)
    },
    {   UINT64_C(0x00000000000060fb), UINT64_C(0x1f4312cf0ca4a71e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43054e50aa873e), UINT64_C(0x1f42fdce6cb3c74c)
    },
    {   UINT64_C(0x00000000000060fc), UINT64_C(0x1f42fe0ef0b78732), UINT64_C(0x1f430dce50b54744),
        UINT64_C(0x1f43084e9caf8740), UINT64_C(0x1f43080f5cb2a712)
    },
    {   UINT64_C(0x0000000000008846), UINT64_C(0x19d2fb0c3ca2660b), UINT64_C(0x1532ff8fa08d067e),
        UINT64_C(0x17eaf94e9c97e68c), UINT64_C(0x1d82f68f64ac0697)
    },
    {   UINT64_C(0x0000000000008847), UINT64_C(0x1b52fa8f48a626a8), UINT64_C(0x175aff8fa097867e),
        UINT64_C(0x1862ff4e0c9de683), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008848), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ea2f593a0b6267e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008849), UINT64_C(0x16bafa8fa093267e), UINT64_C(0x0d42f6cc386f667e),
        UINT64_C(0x1742f54fa097e68c), UINT64_C(0x1742f54f6897e67e)
    },
    {   UINT64_C(0x000000000000884a), UINT64_C(0x16b2fa8c7093069f), UINT64_C(0x0b42f60fa06105b7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692f1cfa096e74f)
    },
    {   UINT64_C(0x0000000000006102), UINT64_C(0x0000000000000000), UINT64_C(0x1f43090f40b2c724),
        UINT64_C(0x1f4311ce8cb34741), UINT64_C(0x1f42fcce70b4a747)
    },
    {   UINT64_C(0x000000000000884c), UINT64_C(0x1773070f649a4697), UINT64_C(0x1cbaf8cfa0afe690),
        UINT64_C(0x10baf14fa078467e), UINT64_C(0x2182f78f64c2868a)
    },
    {   UINT64_C(0x0000000000006104), UINT64_C(0x1f4310ce98aaa72d), UINT64_C(0x1f43070e7ca98728),
        UINT64_C(0x1f43188e74b0e73a), UINT64_C(0x1f43100ea4ad6726)
    },
    {   UINT64_C(0x000000000000884e), UINT64_C(0x12bafb0f547c668d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000884f), UINT64_C(0x150af44f708f25aa), UINT64_C(0x10e2ee0c388045b4),
        UINT64_C(0x0eaaff8fa085667e), UINT64_C(0x1672ff8fa08f867e)
    },
    {   UINT64_C(0x0000000000006107), UINT64_C(0x1f43130ecca82721), UINT64_C(0x1f430fcea0ad4725),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008851), UINT64_C(0x13b2f34fa08aa67e), UINT64_C(0x164af24fa0966675),
        UINT64_C(0x122af80f748dc693), UINT64_C(0x101af80f747b25a1)
    },
    {   UINT64_C(0x0000000000008852), UINT64_C(0x092af4cfa069c675), UINT64_C(0x0aaad40c3c6fc675),
        UINT64_C(0x14c2f94fa089c69c), UINT64_C(0x1852f50f6c9a868c)
    },
    {   UINT64_C(0x000000000000610a), UINT64_C(0x1f4315cef4ac672a), UINT64_C(0x1f43018f08b44734),
        UINT64_C(0x1f42fd0e3cb4072d), UINT64_C(0x1f430e8e38b48738)
    },
    {   UINT64_C(0x0000000000008854), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772f70f6899268c)
    },
    {   UINT64_C(0x0000000000008855), UINT64_C(0x1772f18f6899e68c), UINT64_C(0x1772f74f6897e68c),
        UINT64_C(0x1772f18f6898868c), UINT64_C(0x1772f14f6898868c)
    },
    {   UINT64_C(0x0000000000008856), UINT64_C(0x1772f18f6898868c), UINT64_C(0x1772f74f6898268c),
        UINT64_C(0x1772f54f6498c68c), UINT64_C(0x1772f74f6898c68c)
    },
    {   UINT64_C(0x0000000000008857), UINT64_C(0x1772f14f6899668c), UINT64_C(0x1772f54f6899668c),
        UINT64_C(0x1772f50f6899668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000610f), UINT64_C(0x1f4300ce30b0e746), UINT64_C(0x1f430bcee0b4a731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430ecebcafe732)
    },
    {   UINT64_C(0x0000000000006110), UINT64_C(0x1f4301ced0b06733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43050e14b3e734), UINT64_C(0x1f43108e5cabc737)
    },
    {   UINT64_C(0x0000000000006111), UINT64_C(0x1f43070ebcabe733), UINT64_C(0x1f43000ee8b52733),
        UINT64_C(0x1f42fb0e5cb4c74e), UINT64_C(0x1f43140e40ad872c)
    },
    {   UINT64_C(0x0000000000006112), UINT64_C(0x0000000000000000), UINT64_C(0x1f43064e6cb2e743),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0eecade733)
    },
    {   UINT64_C(0x0000000000006113), UINT64_C(0x1f43004e48b2472d), UINT64_C(0x1f4312ce94a5a736),
        UINT64_C(0x1f43014e4cafe745), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006114), UINT64_C(0x1f4319cf08afe72b), UINT64_C(0x1f4305ce5cb26743),
        UINT64_C(0x1f430a4e7cb44735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006115), UINT64_C(0x1f43060efcb06732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4317cf64a8c72d), UINT64_C(0x1f4300cea0b3c741)
    },
    {   UINT64_C(0x0000000000006116), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0e9caaa72b),
        UINT64_C(0x1f43144e74aee734), UINT64_C(0x1f4317ce94a9073c)
    },
    {   UINT64_C(0x0000000000006117), UINT64_C(0x1f430f8f00ae6722), UINT64_C(0x1f430f8f08ac0731),
        UINT64_C(0x1f430cce3cb5673c), UINT64_C(0x1f43120e70b0c734)
    },
    {   UINT64_C(0x0000000000006118), UINT64_C(0x1f4303cf04afc735), UINT64_C(0x1f430fce4ca88749),
        UINT64_C(0x1f42fece50b32737), UINT64_C(0x1f43070ea0b0a74e)
    },
    {   UINT64_C(0x0000000000006119), UINT64_C(0x0000000000000000), UINT64_C(0x1f43110e34ad0731),
        UINT64_C(0x1f43088e5cb24741), UINT64_C(0x1f43148e94af872c)
    },
    {   UINT64_C(0x000000000000611a), UINT64_C(0x1f430d4e3cb1e72d), UINT64_C(0x1f430e8ef8b5c71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e68aa8735)
    },
    {   UINT64_C(0x0000000000008864), UINT64_C(0x1cf2ffd088a9a5a9), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000008865), UINT64_C(0x1cf2f79084ad85a9), UINT64_C(0x0ffafd0c2076e6c2),
        UINT64_C(0x1cf2ff8d30a9867e), UINT64_C(0x1cf2ff8d30a9867e)
    },
    {   UINT64_C(0x0000000000008866), UINT64_C(0x1cf2ff8d30a9867e), UINT64_C(0x1cf2ff8d30a9867e),
        UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x0000000000008867), UINT64_C(0x1cf2ff9138a98583), UINT64_C(0x1cf2ff9138a98583),
        UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x0ffafd0d3876e67e)
    },
    {   UINT64_C(0x0000000000008868), UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x1cf2ff9088a985a9)
    },
    {   UINT64_C(0x0000000000008869), UINT64_C(0x1cf2ff9084a9a5aa), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2ff9084a9a5aa), UINT64_C(0x0ffafccd3876e67e)
    },
    {   UINT64_C(0x000000000000886a), UINT64_C(0x0ffafccd3876e67e), UINT64_C(0x1cf2ff9084a98611),
        UINT64_C(0x1cf2ff9084a98611), UINT64_C(0x1cf2ff8fa0a98611)
    },
    {   UINT64_C(0x000000000000886b), UINT64_C(0x1cf2ff8fa0a98611), UINT64_C(0x2582fa5088ce65a9),
        UINT64_C(0x1cf2f79088ad85aa), UINT64_C(0x0ffafd0d3876e67e)
    },
    {   UINT64_C(0x000000000000886c), UINT64_C(0x0e7af58d3073867e), UINT64_C(0x0e7af7cd3074a67e),
        UINT64_C(0x0e7af44d3872e67e), UINT64_C(0x0e7af7cd3874a67e)
    },
    {   UINT64_C(0x0000000000006124), UINT64_C(0x1f430fce40b40741), UINT64_C(0x1f43030ec8b0273c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000886e), UINT64_C(0x1cf2fbcd30ab86c4), UINT64_C(0x1cf2fd0e30aac63e),
        UINT64_C(0x1e730b0dd0a9c696), UINT64_C(0x1e730b0dd0a9c696)
    },
    {   UINT64_C(0x0000000000006126), UINT64_C(0x1f430c8e20afe747), UINT64_C(0x1f43138e68b4c73a),
        UINT64_C(0x1f43120e28a9e73c), UINT64_C(0x1f42fecf34b3c738)
    },
    {   UINT64_C(0x0000000000006127), UINT64_C(0x1f42fecf10ae672f), UINT64_C(0x1f43044ecca6472b),
        UINT64_C(0x1f430a8eccb1473f), UINT64_C(0x1f43054f04a9673a)
    },
    {   UINT64_C(0x0000000000006128), UINT64_C(0x1f43110f14ac073f), UINT64_C(0x1f43100e10ae473b),
        UINT64_C(0x1f43138edcb6a739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008872), UINT64_C(0x182af84d309a067e), UINT64_C(0x1cf2ff8e30a9863f),
        UINT64_C(0x182af84d309a067e), UINT64_C(0x1cf2ff8f1ca98603)
    },
    {   UINT64_C(0x0000000000008873), UINT64_C(0x1cf2ff8f1ca98603), UINT64_C(0x1cf2fd0eecaac60f),
        UINT64_C(0x1cf2f30f78afc5ed), UINT64_C(0x1cf2fd0eecaac60f)
    },
    {   UINT64_C(0x0000000000008874), UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e),
        UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e)
    },
    {   UINT64_C(0x0000000000008875), UINT64_C(0x1cf2f30f78afc5ed), UINT64_C(0x156af94c208ea6c2),
        UINT64_C(0x1cf2fbcd30abc67e), UINT64_C(0x1cf2fbcd30abc67e)
    },
    {   UINT64_C(0x0000000000008876), UINT64_C(0x1cf2fbcd30abc67e), UINT64_C(0x1cf2fbcd30abc67e),
        UINT64_C(0x1cf2f98d74a9066c), UINT64_C(0x1cf3008d78ac866d)
    },
    {   UINT64_C(0x0000000000008877), UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x1cf2fd1088aac5a9),
        UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000008878), UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000006130), UINT64_C(0x1f42ff0eb8ad873f), UINT64_C(0x1f43070e4cb6c747),
        UINT64_C(0x1f43144eb4ad673a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000887a), UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e),
        UINT64_C(0x156af94cc08ea69a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006132), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054e50b6a745),
        UINT64_C(0x1f4307ce3cb7673f), UINT64_C(0x1f43048edcb2472d)
    },
    {   UINT64_C(0x0000000000006133), UINT64_C(0x1f4312ce44af2738), UINT64_C(0x1f4304cdf8b0674b),
        UINT64_C(0x1f43018e60ad072d), UINT64_C(0x1f42ff4f50b9073b)
    },
    {   UINT64_C(0x000000000000887d), UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x182af5cd309b467e),
        UINT64_C(0x1cf2fd0c00aac6cb), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x000000000000887e), UINT64_C(0x1cf2fbd088aa25aa), UINT64_C(0x1cf2fa1088ac45aa),
        UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x000000000000887f), UINT64_C(0x1cf2f79088ad85aa), UINT64_C(0x1cf2fd1084aac5a9),
        UINT64_C(0x1cf2fd1084aac5a9), UINT64_C(0x1cf2f31084afc5a9)
    },
    {   UINT64_C(0x0000000000008880), UINT64_C(0x1182ee8fa084467e), UINT64_C(0x0e7afccf647106e0),
        UINT64_C(0x0ebaf80f74796684), UINT64_C(0x1183004fa07b667e)
    },
    {   UINT64_C(0x0000000000008881), UINT64_C(0x1182ffcdb87b66ef), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x16dafdcfa091a68a), UINT64_C(0x16dafdcf7091e67e)
    },
    {   UINT64_C(0x0000000000008882), UINT64_C(0x156afd0f348ca5fb), UINT64_C(0x156afd0d2c8ca67e),
        UINT64_C(0x0f32fe10547345b3), UINT64_C(0x156afd0f348ca5fb)
    },
    {   UINT64_C(0x0000000000008883), UINT64_C(0x156afd0d2c8ca67e), UINT64_C(0x0f32fe10547345b3),
        UINT64_C(0x110afd0fa07b2668), UINT64_C(0x1f42fb0b94b506e3)
    },
    {   UINT64_C(0x0000000000008884), UINT64_C(0x1f42fb0b94b506e3), UINT64_C(0x1c92fc8b94a986e3),
        UINT64_C(0x156afe13108c2506), UINT64_C(0x156afa0e448e25de)
    },
    {   UINT64_C(0x0000000000008885), UINT64_C(0x156afa0efc8e06cd), UINT64_C(0x105af98c287a46ae),
        UINT64_C(0x0d62f4511470e583), UINT64_C(0x105afed02477a5bf)
    },
    {   UINT64_C(0x0000000000008886), UINT64_C(0x0af2f8117465456b), UINT64_C(0x0af2f8117465456b),
        UINT64_C(0x1d030a0b94b6e7b5), UINT64_C(0x1d030a0b94b6e7b5)
    },
    {   UINT64_C(0x0000000000008887), UINT64_C(0x1d030a0b94b6e7b5), UINT64_C(0x156af84fa08ee596),
        UINT64_C(0x1cf2ff8fa0a94596), UINT64_C(0x156afa8fa08dc630)
    },
    {   UINT64_C(0x0000000000008888), UINT64_C(0x156af3cfa08ee60b), UINT64_C(0x156af38e808ec60b),
        UINT64_C(0x156afc0f749025ef), UINT64_C(0x0853070cac53469a)
    },
    {   UINT64_C(0x0000000000006140), UINT64_C(0x1f43078eb8ab8734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43160ea8a6a71e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000888a), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x0e5af60aa08e6720)
    },
    {   UINT64_C(0x000000000000888b), UINT64_C(0x15eaf60aa0aca720), UINT64_C(0x1d7af60aa0cae720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x172af60aa0aca720)
    },
    {   UINT64_C(0x000000000000888c), UINT64_C(0x1eb2f60aa0cac720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x0000000000006144), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cea4ab4737),
        UINT64_C(0x1f43154f18ac6722), UINT64_C(0x1f4316ce5cabe72d)
    },
    {   UINT64_C(0x0000000000006145), UINT64_C(0x1f43158e78a7873a), UINT64_C(0x1f43108e6caaa73f),
        UINT64_C(0x1f43090efca8e729), UINT64_C(0x1f43034f1caf273c)
    },
    {   UINT64_C(0x000000000000888f), UINT64_C(0x156afa11f08e2552), UINT64_C(0x156afa11f08e2552),
        UINT64_C(0x156afa11088e2585), UINT64_C(0x156afa503c8e25bb)
    },
    {   UINT64_C(0x0000000000006147), UINT64_C(0x1f430ccec4b6a727), UINT64_C(0x1f4314ce7cb4472b),
        UINT64_C(0x1f430acefcb10734), UINT64_C(0x1f43094f50ad672f)
    },
    {   UINT64_C(0x0000000000008891), UINT64_C(0x156afa0f148e25ef), UINT64_C(0x156afa0f348e25fc),
        UINT64_C(0x156af9cdf48e2644), UINT64_C(0x156afa0c788e266f)
    },
    {   UINT64_C(0x0000000000008892), UINT64_C(0x156afa10048e25c3), UINT64_C(0x156afa0edc8e260d),
        UINT64_C(0x156afa0eec8e2610), UINT64_C(0x156afa0f388e45fc)
    },
    {   UINT64_C(0x0000000000008893), UINT64_C(0x156afa0f348e25fc), UINT64_C(0x156afa0fc08e25da),
        UINT64_C(0x156afa102c8e25bf), UINT64_C(0x156afa11808e25bf)
    },
    {   UINT64_C(0x0000000000008894), UINT64_C(0x156afa11808e264b), UINT64_C(0x156afa0df88e264b),
        UINT64_C(0x156afa0dfc8e264b), UINT64_C(0x156afa0df88e264b)
    },
    {   UINT64_C(0x0000000000008895), UINT64_C(0x19c2fa11049f8589), UINT64_C(0x19c2fa11049f8589),
        UINT64_C(0x156afa11088e2588), UINT64_C(0x156afa11808e26d8)
    },
    {   UINT64_C(0x0000000000008896), UINT64_C(0x156afa11808e26c1), UINT64_C(0x156afa11808e273d),
        UINT64_C(0x156afa11808e273d), UINT64_C(0x156afa11808e26dc)
    },
    {   UINT64_C(0x0000000000008897), UINT64_C(0x156afa11808e273d), UINT64_C(0x156aefd18093471f),
        UINT64_C(0x156afa11808e26a7), UINT64_C(0x156afa11808e2747)
    },
    {   UINT64_C(0x0000000000008898), UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afa10108e25c6),
        UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afa0f088e2608)
    },
    {   UINT64_C(0x0000000000008899), UINT64_C(0x156afe0e048d2649), UINT64_C(0x156afe0e048d2649),
        UINT64_C(0x156afc4c548c46b6), UINT64_C(0x156afc4c548c46b6)
    },
    {   UINT64_C(0x000000000000889a), UINT64_C(0x156afc0b908c26d2), UINT64_C(0x156afc0b908c26d2),
        UINT64_C(0x1d2af44f38b005fc), UINT64_C(0x1d2af44f38b005fc)
    },
    {   UINT64_C(0x000000000000889b), UINT64_C(0x0e93034c206e26c2), UINT64_C(0x156afa0e988e2624),
        UINT64_C(0x156afc0d308c267e), UINT64_C(0x156afc0d308c267e)
    },
    {   UINT64_C(0x000000000000889c), UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afc0c0c8c46c2),
        UINT64_C(0x156afa0d108e2680), UINT64_C(0x156afa0d108e2680)
    },
    {   UINT64_C(0x0000000000006154), UINT64_C(0x0000000000000000), UINT64_C(0x1f43180eb4a6273f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4317cf10a7c73c)
    },
    {   UINT64_C(0x0000000000006155), UINT64_C(0x1f43064f08b7a730), UINT64_C(0x1f43018f00a82730),
        UINT64_C(0x1f430e4e4cb0874a), UINT64_C(0x1f43038f1cb5e724)
    },
    {   UINT64_C(0x0000000000006156), UINT64_C(0x1f430a0e90b3a745), UINT64_C(0x1f430e0ec8b1e748),
        UINT64_C(0x1f42ff8de4b2c74a), UINT64_C(0x1f4301ceacaec730)
    },
    {   UINT64_C(0x0000000000006157), UINT64_C(0x0000000000000000), UINT64_C(0x1f43120ef4b1c730),
        UINT64_C(0x1f430ccebcaec73a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088a1), UINT64_C(0x156afe4d308c667e), UINT64_C(0x156afe4d308c667e),
        UINT64_C(0x156afe0e048c2647), UINT64_C(0x156afe0e048c2647)
    },
    {   UINT64_C(0x00000000000088a2), UINT64_C(0x156afe0c0c8c26c2), UINT64_C(0x156afe0c0c8c26c2),
        UINT64_C(0x156afe0c408c265b), UINT64_C(0x156afe0c408c265b)
    },
    {   UINT64_C(0x00000000000088a3), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x156afe0f3c8c25fb)
    },
    {   UINT64_C(0x00000000000088a4), UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0e048c2647),
        UINT64_C(0x156afe0e048c2647), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x00000000000088a5), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x00000000000088a6), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x00000000000088a7), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x00000000000088a8), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x156afe0fa08c267e), UINT64_C(0x156afe0fa08c267e)
    },
    {   UINT64_C(0x0000000000006160), UINT64_C(0x1f43114f54b3272d), UINT64_C(0x1f430b8ef4af272c),
        UINT64_C(0x1f43098ef4b3c73f), UINT64_C(0x1f430e8f1ca8872d)
    },
    {   UINT64_C(0x00000000000088aa), UINT64_C(0x156afe0fa08c267e), UINT64_C(0x1ae2fc4fa0a4467e),
        UINT64_C(0x1ae2f94fa0a4667e), UINT64_C(0x1ae2fc4fa0a4467e)
    },
    {   UINT64_C(0x00000000000088ab), UINT64_C(0x1892f04fa098e67e), UINT64_C(0x1892f04fa098e67e),
        UINT64_C(0x1e42f04fa0b1c67e), UINT64_C(0x1e42f04fa0b1c67e)
    },
    {   UINT64_C(0x00000000000088ac), UINT64_C(0x156afe0eec8c260f), UINT64_C(0x156afe0eec8c260f),
        UINT64_C(0x156af40f3c8d65fb), UINT64_C(0x156afb8f3c9125fb)
    },
    {   UINT64_C(0x00000000000088ad), UINT64_C(0x156af40e048d6647), UINT64_C(0x156afb8e04912647),
        UINT64_C(0x23cafa913cc7657b), UINT64_C(0x23cafa913cc7657b)
    },
    {   UINT64_C(0x00000000000088ae), UINT64_C(0x1a8afa913ca2657b), UINT64_C(0x156afa0efc8e060b),
        UINT64_C(0x0f03008c507145aa), UINT64_C(0x1362f60d74884600)
    },
    {   UINT64_C(0x00000000000088af), UINT64_C(0x1362f60f2c88466e), UINT64_C(0x1362f60f2c88466e),
        UINT64_C(0x156afa8fa08dc630), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x00000000000088b0), UINT64_C(0x1ce2fb8b94ab46e3), UINT64_C(0x1ce2fb8b94ab46e3),
        UINT64_C(0x1ce3058ba8a646f2), UINT64_C(0x1ce3058b80a646e8)
    },
    {   UINT64_C(0x00000000000088b1), UINT64_C(0x105af48fe07ca5d0), UINT64_C(0x08f302d2a457e521),
        UINT64_C(0x15e2f6cf6c91a63d), UINT64_C(0x156afacf088dc609)
    },
    {   UINT64_C(0x00000000000088b2), UINT64_C(0x1952fb8e609d062e), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953054e6098062e), UINT64_C(0x1953008e609a862e)
    },
    {   UINT64_C(0x00000000000088b3), UINT64_C(0x1953008e609a862e), UINT64_C(0x156afa107c8e25a5),
        UINT64_C(0x1222f5cf3c8325fc), UINT64_C(0x1222f5cf3c8325fc)
    },
    {   UINT64_C(0x00000000000088b4), UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0f3c8c25fb),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x00000000000088b5), UINT64_C(0x1362fa0f2486065f), UINT64_C(0x156afa0cac8e269a),
        UINT64_C(0x156afc0f408c25fe), UINT64_C(0x156afc0f408c25fe)
    },
    {   UINT64_C(0x00000000000088b6), UINT64_C(0x27d2f94f40d825fe), UINT64_C(0x27d2f94f40d825fe),
        UINT64_C(0x156afc0a908c2726), UINT64_C(0x156afc0a908c2726)
    },
    {   UINT64_C(0x00000000000088b7), UINT64_C(0x156afe0e048c2647), UINT64_C(0x156afe0e048c2647),
        UINT64_C(0x156afe0dbc8c265b), UINT64_C(0x156afe0dbc8c265b)
    },
    {   UINT64_C(0x00000000000088b8), UINT64_C(0x156afe0c0c8c26c2), UINT64_C(0x156afe0c0c8c26c2),
        UINT64_C(0x156afe4c0c8c46c2), UINT64_C(0x156afe4c0c8c46c2)
    },
    {   UINT64_C(0x00000000000088b9), UINT64_C(0x156afe0c408c265b), UINT64_C(0x156afe0c408c265b),
        UINT64_C(0x156afa0b448e2685), UINT64_C(0x156afa0b448e2685)
    },
    {   UINT64_C(0x00000000000088ba), UINT64_C(0x156afa0b448e2686), UINT64_C(0x156afa0b448e2686),
        UINT64_C(0x13daf40d3087267e), UINT64_C(0x13dafb8d308ae67e)
    },
    {   UINT64_C(0x00000000000088bb), UINT64_C(0x13daf40c0c8726c2), UINT64_C(0x13dafb8c0c8ae6c2),
        UINT64_C(0x0ffb1e0e00666646), UINT64_C(0x1cf3091294a4c524)
    },
    {   UINT64_C(0x00000000000088bc), UINT64_C(0x1cf31e8f589a05f0), UINT64_C(0x1cf31e8f589a05f0),
        UINT64_C(0x19bafbcf349de5fb), UINT64_C(0x156afd0f348ca5fb)
    },
    {   UINT64_C(0x00000000000088bd), UINT64_C(0x0f32fd90547305b3), UINT64_C(0x156afd0f348ca6b4),
        UINT64_C(0x156afd0f348ca695), UINT64_C(0x0f32fd905473064d)
    },
    {   UINT64_C(0x00000000000088be), UINT64_C(0x156afd0cd08ca5fc), UINT64_C(0x156afd4f348ca5fb),
        UINT64_C(0x19bafbcf349de5fb), UINT64_C(0x156afccf348c85fb)
    },
    {   UINT64_C(0x00000000000088bf), UINT64_C(0x0f32fd90547305b3), UINT64_C(0x156afd4f348cc695),
        UINT64_C(0x0f32fed05473a64d), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x00000000000088c0), UINT64_C(0x12f2f44fd88725cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fbcfa0ab66ee), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088c1), UINT64_C(0x0000000000000000), UINT64_C(0x1362fa0fa0860629),
        UINT64_C(0x1362fa0fa08606c0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088c2), UINT64_C(0x0eab1d0c4c7666b1), UINT64_C(0x0eaaf4cc4c6246b1),
        UINT64_C(0x0eab1d0c4c7666b1), UINT64_C(0x0eaaf4cc4c6246b1)
    },
    {   UINT64_C(0x00000000000088c3), UINT64_C(0x0e7afb0c4472048e), UINT64_C(0x0e7afb0c4472048e),
        UINT64_C(0x0e7afb175c720754), UINT64_C(0x0e7afb175c720754)
    },
    {   UINT64_C(0x00000000000088c4), UINT64_C(0x12c2fa116c838571), UINT64_C(0x105b00d0a476a59f),
        UINT64_C(0x17d2fa8fa0978533), UINT64_C(0x17d2fa8fa0978533)
    },
    {   UINT64_C(0x00000000000088c5), UINT64_C(0x0000000000000000), UINT64_C(0x1a7af48caca2062e),
        UINT64_C(0x2262fb4bc8c166d4), UINT64_C(0x2262fb4bccc166d5)
    },
    {   UINT64_C(0x00000000000088c6), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x12c2fa116c838571),
        UINT64_C(0x156afe0e8c8c4687), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088c7), UINT64_C(0x0e7afad40471e67e), UINT64_C(0x0e7afad40471e67e),
        UINT64_C(0x0e7afacc3c71e48c), UINT64_C(0x0e7afb0c3c72048c)
    },
    {   UINT64_C(0x00000000000088c8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1fdafb91ecb72550), UINT64_C(0x1fdafb91ecb72550)
    },
    {   UINT64_C(0x0000000000006180), UINT64_C(0x1f43038df0ab4740), UINT64_C(0x1f430e4e2caec737),
        UINT64_C(0x1f43034d94b0e739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088ca), UINT64_C(0x0000000000000000), UINT64_C(0x0c83014c4c6866b1),
        UINT64_C(0x0c82fe4c4c66e6b1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088cb), UINT64_C(0x1cf302ccb8a7e69c), UINT64_C(0x1ed2fbce40b2e638),
        UINT64_C(0x1382e98d708645aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088cc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1bcb020ec0a3a61a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088cd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf3034d30a7a6d7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006185), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43170eecac472b), UINT64_C(0x1f43120e6cb2273a)
    },
    {   UINT64_C(0x00000000000088cf), UINT64_C(0x0000000000000000), UINT64_C(0x1c73030cd4a5c693),
        UINT64_C(0x0000000000000000), UINT64_C(0x155afd0ce08c66a7)
    },
    {   UINT64_C(0x00000000000088d0), UINT64_C(0x155afd0ce08c66a7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006188), UINT64_C(0x1f430b0e98ad4737), UINT64_C(0x1f430d4e90af472f),
        UINT64_C(0x1f42ff0e5cb4073f), UINT64_C(0x1f43050e74afc72d)
    },
    {   UINT64_C(0x0000000000006189), UINT64_C(0x1f430ace58afc72f), UINT64_C(0x1f43050eccb6c724),
        UINT64_C(0x1f431a8f00aa8722), UINT64_C(0x1f4301ceb4ada73b)
    },
    {   UINT64_C(0x000000000000618a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044e24b08745),
        UINT64_C(0x1f43090e40a90741), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088d4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x18d2fbcce09ae6d7)
    },
    {   UINT64_C(0x000000000000618c), UINT64_C(0x1f43028f28b12735), UINT64_C(0x1f4307ce84ab0735),
        UINT64_C(0x1f431a8eb8aa473d), UINT64_C(0x1f42fc4e64b2a73a)
    },
    {   UINT64_C(0x000000000000618d), UINT64_C(0x1f43198e74a9e748), UINT64_C(0x1f42fa8e60b2a732),
        UINT64_C(0x1f430c4f28b5e73c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000618e), UINT64_C(0x1f4312ceacaae739), UINT64_C(0x1f43070e34b8873b),
        UINT64_C(0x1f43078ea8b2473d), UINT64_C(0x1f43080e40b20748)
    },
    {   UINT64_C(0x000000000000618f), UINT64_C(0x1f42ff4e5cacc738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040e34b8873b), UINT64_C(0x1f430d4e7caee734)
    },
    {   UINT64_C(0x0000000000006190), UINT64_C(0x1f43058e6cb3073f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8e70afc73f)
    },
    {   UINT64_C(0x0000000000006191), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dcebca2c741), UINT64_C(0x1f43100e88af473f)
    },
    {   UINT64_C(0x0000000000006192), UINT64_C(0x1f430b4ec4aec731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d0e90b2c740)
    },
    {   UINT64_C(0x0000000000006193), UINT64_C(0x1f431a0e88af872d), UINT64_C(0x1f430c0ec8afa701),
        UINT64_C(0x1f42fb0ee8b36731), UINT64_C(0x1f43014eeca5e722)
    },
    {   UINT64_C(0x0000000000006194), UINT64_C(0x1f42fb8eb8ace72b), UINT64_C(0x1f43014f94aca716),
        UINT64_C(0x1f42fa8ed0af471c), UINT64_C(0x1f43014f84b7270e)
    },
    {   UINT64_C(0x0000000000006195), UINT64_C(0x1f43004e50b30737), UINT64_C(0x1f42fb8e78b5c740),
        UINT64_C(0x1f43024e94aa0739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006196), UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f20a9a729),
        UINT64_C(0x1f43004f44a90731), UINT64_C(0x1f42ff4eb8ac672e)
    },
    {   UINT64_C(0x0000000000006197), UINT64_C(0x1f42ff4e58b24739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e48ae0744), UINT64_C(0x1f42fc4f70b66731)
    },
    {   UINT64_C(0x0000000000006198), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4ebca8673a),
        UINT64_C(0x1f42fe0f30b8470a), UINT64_C(0x1f42fe4f3cb646e7)
    },
    {   UINT64_C(0x0000000000006199), UINT64_C(0x1f42f84f40b646f9), UINT64_C(0x1f42fa4f30af26fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ee4b4a731)
    },
    {   UINT64_C(0x000000000000619a), UINT64_C(0x1f42fa8f4cafa712), UINT64_C(0x1f43048f58b6a6fa),
        UINT64_C(0x1f42f74f1cb7e710), UINT64_C(0x1f4314cf8ca466f9)
    },
    {   UINT64_C(0x00000000000088e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1c8b035180a7a56a)
    },
    {   UINT64_C(0x000000000000619c), UINT64_C(0x1f42fd4f5cb506d2), UINT64_C(0x1f430d8f44b666e5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4ea8b046ff)
    },
    {   UINT64_C(0x000000000000fec1), UINT64_C(0x1a0af1cfa0a4069b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000619e), UINT64_C(0x0000000000000000), UINT64_C(0x1f4304902cb546de),
        UINT64_C(0x1f42f4cf8cb3c719), UINT64_C(0x1f42fd9054b2a6f6)
    },
    {   UINT64_C(0x000000000000619f), UINT64_C(0x1f42fd4f5cb706fe), UINT64_C(0x1f431c0f34a7a6f7),
        UINT64_C(0x1f42fe4f60ac66de), UINT64_C(0x1f43134f24a986ef)
    },
    {   UINT64_C(0x00000000000061a0), UINT64_C(0x1f42fd9020b0e704), UINT64_C(0x1f42fc50dcb786e2),
        UINT64_C(0x1f42fd4f5cae06e1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061a1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94fccb66701),
        UINT64_C(0x1f42fd5010b6e6d5), UINT64_C(0x1f42fc4fc0b2a6e1)
    },
    {   UINT64_C(0x00000000000061a2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe502cb8c6eb), UINT64_C(0x1f42fd4fb0aac6f8)
    },
    {   UINT64_C(0x00000000000061a3), UINT64_C(0x1f42fa0e3cb56734), UINT64_C(0x1f42fb1068b246c2),
        UINT64_C(0x1f43081084ab66ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061a4), UINT64_C(0x1f42fc0ee0ab0706), UINT64_C(0x1f42fd0f9cb48724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ceccaba726)
    },
    {   UINT64_C(0x00000000000061a5), UINT64_C(0x1f43004f70b526e5), UINT64_C(0x1f42fb8ef8b4e702),
        UINT64_C(0x1f42fe4fdcb506d6), UINT64_C(0x1f42ff4f78b246d0)
    },
    {   UINT64_C(0x00000000000061a6), UINT64_C(0x1f4301cf80b026ef), UINT64_C(0x1f42fe4f54b5c711),
        UINT64_C(0x1f42f98e8cae66f9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061a7), UINT64_C(0x1f42fe4ed0b6e6f1), UINT64_C(0x1f43005078b1c6c7),
        UINT64_C(0x1f42ff0ef8b64700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061a8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f40b766e1),
        UINT64_C(0x1f42fb8f48b5c6e0), UINT64_C(0x1f42fbd05cb906f1)
    },
    {   UINT64_C(0x00000000000061a9), UINT64_C(0x1f4309cef4b7e718), UINT64_C(0x1f43030ffcb5a6d3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4e60b72712)
    },
    {   UINT64_C(0x00000000000061aa), UINT64_C(0x1f42fe4f90b446f3), UINT64_C(0x1f43078ea4afe70a),
        UINT64_C(0x1f42fd0e5cb5872c), UINT64_C(0x1f42f88f8cb62736)
    },
    {   UINT64_C(0x00000000000061ab), UINT64_C(0x1f42ffcf88b566f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa1074b6a6f2)
    },
    {   UINT64_C(0x00000000000061ac), UINT64_C(0x1f42fdcfdcaf26d9), UINT64_C(0x1f42f64e58af072c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0fbcb1e6e2)
    },
    {   UINT64_C(0x00000000000061ad), UINT64_C(0x1f42fd0eb8b14706), UINT64_C(0x1f42fa4e78b58701),
        UINT64_C(0x1f42fecf68b386e9), UINT64_C(0x1f4301508cb186d9)
    },
    {   UINT64_C(0x00000000000061ae), UINT64_C(0x1f42f9ce5caf46ee), UINT64_C(0x1f42fd4f14b186ec),
        UINT64_C(0x1f42fe0f38b3c709), UINT64_C(0x1f4307ce98af8710)
    },
    {   UINT64_C(0x00000000000061af), UINT64_C(0x1f43050f0cac6703), UINT64_C(0x1f42ff0ff8af86d1),
        UINT64_C(0x1f43004f44b5c724), UINT64_C(0x1f42fe5050ab26f2)
    },
    {   UINT64_C(0x00000000000061b0), UINT64_C(0x1f42fccfd8b4e6eb), UINT64_C(0x1f43020fe8aca6d2),
        UINT64_C(0x1f42fd0fc4b4a702), UINT64_C(0x1f42fc102cabc6f6)
    },
    {   UINT64_C(0x00000000000061b1), UINT64_C(0x1f42f8cf8cb486f4), UINT64_C(0x1f42fd0fb8ad66fd),
        UINT64_C(0x1f42fd108cb646e2), UINT64_C(0x1f42ff0f84b40704)
    },
    {   UINT64_C(0x00000000000061b2), UINT64_C(0x1f43004ff0b34704), UINT64_C(0x1f42f68ec0b72723),
        UINT64_C(0x1f430e4fa4abc705), UINT64_C(0x1f4305cec4b7a71a)
    },
    {   UINT64_C(0x00000000000061b3), UINT64_C(0x1f42fb4f84b426e5), UINT64_C(0x1f43088ef8ae471a),
        UINT64_C(0x1f42fb4fb0ab26f1), UINT64_C(0x1f42fb9030b4c6d1)
    },
    {   UINT64_C(0x00000000000061b4), UINT64_C(0x1f43011078b186e4), UINT64_C(0x1f43030ec8b0e718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43044e64b00728)
    },
    {   UINT64_C(0x00000000000061b5), UINT64_C(0x1f42fa1038afa6cb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf60b5a6ef), UINT64_C(0x1f431c0f14a3c740)
    },
    {   UINT64_C(0x00000000000061b6), UINT64_C(0x1f42fb0e7cb24734), UINT64_C(0x1f42fbcff4b5470a),
        UINT64_C(0x1f42fe0f1cad0708), UINT64_C(0x1f42f80e9cb5c722)
    },
    {   UINT64_C(0x00000000000061b7), UINT64_C(0x1f42f94f24b72709), UINT64_C(0x1f42ff4f58b6a6da),
        UINT64_C(0x1f42f50facb426e5), UINT64_C(0x1f42fccfacb54700)
    },
    {   UINT64_C(0x00000000000061b8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0ee8b2c710), UINT64_C(0x1f42f94f8cad270d)
    },
    {   UINT64_C(0x00000000000061b9), UINT64_C(0x1f42f70f8cb686e0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f20b3470c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061ba), UINT64_C(0x1f42ff5008b106c3), UINT64_C(0x1f43080ec8b3e713),
        UINT64_C(0x1f43050e88a6070d), UINT64_C(0x1f42f98f80b3271f)
    },
    {   UINT64_C(0x00000000000061bb), UINT64_C(0x1f42f74e68b5273c), UINT64_C(0x1f42f95044b266d6),
        UINT64_C(0x1f430f4eb4ad6715), UINT64_C(0x1f42fa100cb56711)
    },
    {   UINT64_C(0x00000000000061bc), UINT64_C(0x1f43080ed4aea72a), UINT64_C(0x1f42fd5038b5c703),
        UINT64_C(0x1f42fb4fd8b64704), UINT64_C(0x1f43004fa4af66f2)
    },
    {   UINT64_C(0x00000000000061bd), UINT64_C(0x1f42fb4ef8b566fb), UINT64_C(0x1f42f9cf78b3c6cb),
        UINT64_C(0x1f42fb0edcb186fe), UINT64_C(0x1f42f8cee4b72703)
    },
    {   UINT64_C(0x00000000000061be), UINT64_C(0x1f43000eecae670e), UINT64_C(0x1f42f94f54aa66ed),
        UINT64_C(0x1f42fed010b7a70f), UINT64_C(0x1f42f8cf98b86701)
    },
    {   UINT64_C(0x00000000000061bf), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98ef4b6c731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061c0), UINT64_C(0x1f430f8eb8a90733), UINT64_C(0x1f42fa0f24b686f8),
        UINT64_C(0x1f42f80e4cb90736), UINT64_C(0x1f42fb8ef0b7672a)
    },
    {   UINT64_C(0x00000000000061c1), UINT64_C(0x1f42f74ee8b7e6d3), UINT64_C(0x1f42f94f34af26fe),
        UINT64_C(0x1f42fb0f78b74700), UINT64_C(0x1f43028ea0ab8716)
    },
    {   UINT64_C(0x00000000000061c2), UINT64_C(0x1f43001024b5e6da), UINT64_C(0x1f42fa0ea0b36722),
        UINT64_C(0x1f42facef8ab8700), UINT64_C(0x1f42f74ee8b64725)
    },
    {   UINT64_C(0x00000000000061c3), UINT64_C(0x1f42f8cf60b3c6e8), UINT64_C(0x1f42f64f10b86703),
        UINT64_C(0x1f42fd0ef4a986df), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061c4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088f08b4073f),
        UINT64_C(0x1f42f64fe8b706de), UINT64_C(0x1f42f9501cb686da)
    },
    {   UINT64_C(0x00000000000061c5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf88b6e705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4fccace6de)
    },
    {   UINT64_C(0x00000000000061c6), UINT64_C(0x1f42f88fe0b286d6), UINT64_C(0x1f42f80f0cb9c6f3),
        UINT64_C(0x1f430d8ed0aae73c), UINT64_C(0x1f42f7cf68b106e5)
    },
    {   UINT64_C(0x00000000000061c7), UINT64_C(0x1f43174eb8ac4736), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6ceecb86713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061c8), UINT64_C(0x1f42f94fe8af66de), UINT64_C(0x1f42fb0ef4b4873b),
        UINT64_C(0x1f42fd4fb8b446d9), UINT64_C(0x1f42f8cf14b6c71f)
    },
    {   UINT64_C(0x00000000000061c9), UINT64_C(0x1f43030ec4ada70f), UINT64_C(0x1f42f90e8cb2a70f),
        UINT64_C(0x1f42f68f70b766fc), UINT64_C(0x1f42f80f98b6470e)
    },
    {   UINT64_C(0x00000000000061ca), UINT64_C(0x1f42facf40b7e72a), UINT64_C(0x1f42f58f2cab26fa),
        UINT64_C(0x1f42fe0f24b00717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061cb), UINT64_C(0x1f42fad058b3c6ba), UINT64_C(0x1f42fb0f50b346e6),
        UINT64_C(0x1f42fb4f00b4e713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061cc), UINT64_C(0x1f430d0eb0a9a71d), UINT64_C(0x1f42f74ea8b1e723),
        UINT64_C(0x1f42f80f4cb7270c), UINT64_C(0x1f42f74ebcb4e731)
    },
    {   UINT64_C(0x00000000000061cd), UINT64_C(0x1f42fa8ef4afc704), UINT64_C(0x1f42f74ef8b12706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74efcb84728)
    },
    {   UINT64_C(0x00000000000061ce), UINT64_C(0x1f430e4ec4b286f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f50f70b766e9), UINT64_C(0x1f42f88e60b48738)
    },
    {   UINT64_C(0x0000000000008918), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x00000000000061d0), UINT64_C(0x1f42fcce8cb6472a), UINT64_C(0x1f43134ef0ace71f),
        UINT64_C(0x1f42f55010baa6e2), UINT64_C(0x1f42fbcfdcb706e5)
    },
    {   UINT64_C(0x00000000000061d1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4ee8af070c), UINT64_C(0x1f42fd0fc4b466e6)
    },
    {   UINT64_C(0x00000000000061d2), UINT64_C(0x1f42fa8fe8aee6fd), UINT64_C(0x1f42fa0eacb46722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f60b60718)
    },
    {   UINT64_C(0x00000000000061d3), UINT64_C(0x1f42f70e6cb58720), UINT64_C(0x1f42f84efcb1e6e2),
        UINT64_C(0x1f42f90ef0aec709), UINT64_C(0x1f42f84e7cb7a731)
    },
    {   UINT64_C(0x00000000000061d4), UINT64_C(0x1f42fecee0b0e6bf), UINT64_C(0x1f42fb8e94b686f0),
        UINT64_C(0x1f42fa1058b5e701), UINT64_C(0x1f42f40ea4b84717)
    },
    {   UINT64_C(0x00000000000061d5), UINT64_C(0x1f42fd0f10b566f9), UINT64_C(0x1f42f80facb0e6fb),
        UINT64_C(0x1f42f74f8cb406f3), UINT64_C(0x1f42f70f88b24719)
    },
    {   UINT64_C(0x00000000000061d6), UINT64_C(0x1f42fd0f70b506de), UINT64_C(0x1f42f8cf08af670e),
        UINT64_C(0x1f43010f18b526f8), UINT64_C(0x1f42fa4f28b56718)
    },
    {   UINT64_C(0x00000000000061d7), UINT64_C(0x1f43014eb0ada71b), UINT64_C(0x1f43014ef8b82708),
        UINT64_C(0x1f43004f0caf46cf), UINT64_C(0x1f43020f0cb086ee)
    },
    {   UINT64_C(0x00000000000061d8), UINT64_C(0x1f42f80f80b026da), UINT64_C(0x1f42fd0f08b506ee),
        UINT64_C(0x1f4304cf1caaa6e5), UINT64_C(0x1f42f74f50b40720)
    },
    {   UINT64_C(0x00000000000061d9), UINT64_C(0x1f42fb0f7cb2e70c), UINT64_C(0x1f430f8eb0b1c70e),
        UINT64_C(0x1f42fa0fa8b786d4), UINT64_C(0x1f42fa0fe0b8a706)
    },
    {   UINT64_C(0x00000000000061da), UINT64_C(0x1f43064f34b2e711), UINT64_C(0x1f42f8cfc0b566fd),
        UINT64_C(0x1f42ff0f04b46710), UINT64_C(0x1f42f74ee4b30702)
    },
    {   UINT64_C(0x00000000000061db), UINT64_C(0x1f42f8cf08b70716), UINT64_C(0x1f42ff0f98b5c6db),
        UINT64_C(0x1f42f70ed8b4471f), UINT64_C(0x1f42fc4efcb5c706)
    },
    {   UINT64_C(0x00000000000061dc), UINT64_C(0x1f42fd8f9cb6a710), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ce68b8871f)
    },
    {   UINT64_C(0x00000000000061dd), UINT64_C(0x1f42f80f0cab2700), UINT64_C(0x1f43078ea8b166ef),
        UINT64_C(0x1f42f7cebcb7a721), UINT64_C(0x1f42fc0f04b1a719)
    },
    {   UINT64_C(0x00000000000061de), UINT64_C(0x1f42fb4f54b126c6), UINT64_C(0x1f43008fe0b8a6ca),
        UINT64_C(0x1f42f8cf3cb50700), UINT64_C(0x1f42f8cf28b48707)
    },
    {   UINT64_C(0x00000000000061df), UINT64_C(0x1f42fb4f3cb2070c), UINT64_C(0x1f42fa4ef0b8a714),
        UINT64_C(0x1f42fa4eecb44717), UINT64_C(0x1f42fa0f3cb5a6f1)
    },
    {   UINT64_C(0x00000000000061e0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f74b5a6b7),
        UINT64_C(0x1f42f80e4cb00726), UINT64_C(0x1f42f74e74b6870e)
    },
    {   UINT64_C(0x00000000000061e1), UINT64_C(0x1f42fdced8af66e8), UINT64_C(0x1f4303cf78b4e712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018f78ae671a)
    },
    {   UINT64_C(0x00000000000061e2), UINT64_C(0x1f43004f54b086bf), UINT64_C(0x1f42fc4eccb26709),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f4cb6672d)
    },
    {   UINT64_C(0x00000000000061e3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074ff8b266e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f70b726d4)
    },
    {   UINT64_C(0x00000000000061e4), UINT64_C(0x1f43010e5cb3c714), UINT64_C(0x1f43020f5cb16729),
        UINT64_C(0x1f42fa8f18b7a70c), UINT64_C(0x1f42fe0eccb646ff)
    },
    {   UINT64_C(0x00000000000061e5), UINT64_C(0x1f43010f84b7c702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004eb4b8c6e1), UINT64_C(0x1f42fe0f0cb4c6f7)
    },
    {   UINT64_C(0x00000000000061e6), UINT64_C(0x1f43010fb8b06712), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431a4eb4a9c730), UINT64_C(0x1f4302cefcb0a6fa)
    },
    {   UINT64_C(0x00000000000061e7), UINT64_C(0x1f43090eb4b6a70b), UINT64_C(0x1f43010f7cb7470d),
        UINT64_C(0x1f42fecf44b1c700), UINT64_C(0x1f42f9cf50b3c6e5)
    },
    {   UINT64_C(0x00000000000061e8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c8e84afc72c), UINT64_C(0x1f42fd0ef8b8a6eb)
    },
    {   UINT64_C(0x00000000000061e9), UINT64_C(0x1f43058fbcb5c702), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061ea), UINT64_C(0x1f42fb4f6caf470f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014f08ab26ff), UINT64_C(0x1f42fc0ec4b6a739)
    },
    {   UINT64_C(0x00000000000061eb), UINT64_C(0x1f43004f10b0871f), UINT64_C(0x1f42fc8efcb066fe),
        UINT64_C(0x1f43020eacb32709), UINT64_C(0x1f430dcf30b1a71e)
    },
    {   UINT64_C(0x00000000000061ec), UINT64_C(0x1f42fa4f20b5c6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f60ace6f6), UINT64_C(0x1f42f94fb4b42716)
    },
    {   UINT64_C(0x00000000000061ed), UINT64_C(0x1f431acf18ac6735), UINT64_C(0x1f42ff8edcb0e6e6),
        UINT64_C(0x1f43104f30ab0712), UINT64_C(0x1f42fb8ed4ace735)
    },
    {   UINT64_C(0x00000000000061ee), UINT64_C(0x1f43060f18abe716), UINT64_C(0x1f42fa0efcb4a6f7),
        UINT64_C(0x1f42fd4ef4ae2712), UINT64_C(0x1f42f94ef0b54730)
    },
    {   UINT64_C(0x00000000000061ef), UINT64_C(0x1f43008f5cb1a704), UINT64_C(0x1f430f4f24b1472a),
        UINT64_C(0x1f42fc8f2cb0e70d), UINT64_C(0x1f43144f18aa8718)
    },
    {   UINT64_C(0x00000000000061f0), UINT64_C(0x1f42fdce7cb1e732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8ef8b0471d), UINT64_C(0x1f42fb8f28b466f0)
    },
    {   UINT64_C(0x00000000000061f1), UINT64_C(0x1f430b8f20b18720), UINT64_C(0x1f42fc8f24b246f3),
        UINT64_C(0x1f42fbcfc0b3471c), UINT64_C(0x1f42ff4e58b2e73a)
    },
    {   UINT64_C(0x00000000000061f2), UINT64_C(0x1f42f9cf5cb166d5), UINT64_C(0x1f42fa8ed0b746ed),
        UINT64_C(0x1f43104f30ab070d), UINT64_C(0x1f42fecfe8b3e709)
    },
    {   UINT64_C(0x00000000000061f3), UINT64_C(0x1f42fe4f54b66711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4f14b50705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061f4), UINT64_C(0x1f42fa8df8b56746), UINT64_C(0x1f42f90e3cb2473c),
        UINT64_C(0x1f42f70ebcb4c73e), UINT64_C(0x1f42f74e80b82738)
    },
    {   UINT64_C(0x00000000000061f5), UINT64_C(0x1f42fc0f1cb446da), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f0e58ace738)
    },
    {   UINT64_C(0x00000000000061f6), UINT64_C(0x1f42f58e80b1670f), UINT64_C(0x1f42fc4ea8b20707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdce74b1c705)
    },
    {   UINT64_C(0x0000000000008940), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008941), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061f9), UINT64_C(0x1f42f60ed4b266f4), UINT64_C(0x1f42fe4ee0b2e736),
        UINT64_C(0x1f43014ec4af271e), UINT64_C(0x1f42face74b746e0)
    },
    {   UINT64_C(0x0000000000008943), UINT64_C(0x16233d4ae498853c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008944), UINT64_C(0x1622eb4ae46f453c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061fc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0f24ae2724), UINT64_C(0x1f42fa4ea4b32721)
    },
    {   UINT64_C(0x0000000000008946), UINT64_C(0x1622eb53cc6f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000061fe), UINT64_C(0x0000000000000000), UINT64_C(0x1f4311ceb8af6740),
        UINT64_C(0x1f42fc0ee0b1a711), UINT64_C(0x1f42fdceb0b1c739)
    },
    {   UINT64_C(0x00000000000061ff), UINT64_C(0x1f430d0f28b2c731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0eecb26730), UINT64_C(0x1f4307ceb4b4a72c)
    },
    {   UINT64_C(0x0000000000006200), UINT64_C(0x1f42fa4efcade6ff), UINT64_C(0x1f430d0eb4abe71e),
        UINT64_C(0x1f42fb8ebcb2c72f), UINT64_C(0x1f43004eb4b7472a)
    },
    {   UINT64_C(0x0000000000006201), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cfbcb326f1),
        UINT64_C(0x1f4319ce60aa074b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006202), UINT64_C(0x1f4302ce90aea726), UINT64_C(0x1f42facec0b566f9),
        UINT64_C(0x1f42f90e6cb6c721), UINT64_C(0x1f42f98efcae0706)
    },
    {   UINT64_C(0x0000000000006203), UINT64_C(0x1f43014e7cad66fe), UINT64_C(0x1f42f60fb0b6c6ea),
        UINT64_C(0x1f42f94ec8b60725), UINT64_C(0x1f42f68f00b7470d)
    },
    {   UINT64_C(0x0000000000006204), UINT64_C(0x1f42fa4e5cb72731), UINT64_C(0x1f42fbceacb5470c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4ebcb8272e)
    },
    {   UINT64_C(0x0000000000006205), UINT64_C(0x1f42f90e88b7a731), UINT64_C(0x1f42fa0ee0afa6f5),
        UINT64_C(0x1f42fa0e88b7671e), UINT64_C(0x1f43004ea8b14703)
    },
    {   UINT64_C(0x0000000000006206), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4f0cb26740),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf30b1272f)
    },
    {   UINT64_C(0x0000000000006207), UINT64_C(0x1f43070ea8b7070a), UINT64_C(0x1f42fdd058b7a6e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4ed4b88732)
    },
    {   UINT64_C(0x0000000000006208), UINT64_C(0x1f43128f10abe702), UINT64_C(0x1f43004e74b12741),
        UINT64_C(0x1f43058ea0b56739), UINT64_C(0x1f42f40ee0b38702)
    },
    {   UINT64_C(0x0000000000006209), UINT64_C(0x1f430a4eecaba740), UINT64_C(0x1f4316ce84a72737),
        UINT64_C(0x1f42fd4ef4b32736), UINT64_C(0x1f4307ce58ae871a)
    },
    {   UINT64_C(0x000000000000620a), UINT64_C(0x1f42fe4edcaf6726), UINT64_C(0x1f43000f50b086f7),
        UINT64_C(0x1f4309ce84ae4735), UINT64_C(0x1f430f4ec4aae719)
    },
    {   UINT64_C(0x0000000000008954), UINT64_C(0x1622eb5258986599), UINT64_C(0x1623264ae47ae776),
        UINT64_C(0x16233d4ae4988599), UINT64_C(0x1623264ae498853c)
    },
    {   UINT64_C(0x0000000000008955), UINT64_C(0x1623264ae4986599), UINT64_C(0x1622eb4ae46f4599),
        UINT64_C(0x1622eb4ae47ae53c), UINT64_C(0x1622eb4ae47ae599)
    },
    {   UINT64_C(0x000000000000620d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94e54ad4735),
        UINT64_C(0x1f4307ceb4b1471f), UINT64_C(0x1f42fb0e7caf6705)
    },
    {   UINT64_C(0x000000000000620e), UINT64_C(0x1f42fb4e3cb0c72c), UINT64_C(0x1f43080e58aea734),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430eceb4a9e734)
    },
    {   UINT64_C(0x000000000000620f), UINT64_C(0x1f42f9ce6cb7e72f), UINT64_C(0x1f43000ed8afa71c),
        UINT64_C(0x1f42fbce48b3a714), UINT64_C(0x1f43128e5cade741)
    },
    {   UINT64_C(0x0000000000008959), UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb4ae498653c),
        UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb5258986776)
    },
    {   UINT64_C(0x0000000000006211), UINT64_C(0x1f42f6d0e0b5a6ef), UINT64_C(0x1f42f90ebcb2870d),
        UINT64_C(0x1f4305ce88afa721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006212), UINT64_C(0x1f42f9ce8cb64704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccf50b2c71f), UINT64_C(0x1f43150eb8ae6731)
    },
    {   UINT64_C(0x000000000000895c), UINT64_C(0x0000000000000000), UINT64_C(0x1622ea4ae498e776),
        UINT64_C(0x1622ea4ae498e776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006214), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88ec0b7e6ff), UINT64_C(0x1f42f78ef8b52703)
    },
    {   UINT64_C(0x0000000000006215), UINT64_C(0x1f42fa4f04b28711), UINT64_C(0x1f42f54f1cb2a71b),
        UINT64_C(0x1f42f74f74b48709), UINT64_C(0x1f42f88ee4b9872c)
    },
    {   UINT64_C(0x0000000000006216), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f18b7071e),
        UINT64_C(0x1f42f48f2cb4a70f), UINT64_C(0x1f42fc4f28b8672a)
    },
    {   UINT64_C(0x0000000000008960), UINT64_C(0x25eaee1460d62776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008961), UINT64_C(0x25eaee0ae4d62517), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006219), UINT64_C(0x1f432c4f74918703), UINT64_C(0x1f43054f08aec6ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78e50b7a6f7)
    },
    {   UINT64_C(0x0000000000008963), UINT64_C(0x25e2ee0ae48a4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000621b), UINT64_C(0x1f42fc4f4cb1e6f7), UINT64_C(0x1f42fc8f34b5c70a),
        UINT64_C(0x1f43050ed8b2c719), UINT64_C(0x1f42fa8f38b5670a)
    },
    {   UINT64_C(0x000000000000621c), UINT64_C(0x1f430acf44b3872b), UINT64_C(0x1f42ff4f64b52705),
        UINT64_C(0x1f42f78f54ab4709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000621d), UINT64_C(0x1f42fb8f64b206ef), UINT64_C(0x1f42fb8f94ab4715),
        UINT64_C(0x1f42fd8e98b08702), UINT64_C(0x1f43000e68b5c735)
    },
    {   UINT64_C(0x000000000000621e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f28b4e706),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000621f), UINT64_C(0x1f42ff4f7cb48706), UINT64_C(0x1f42fa8f84b26719),
        UINT64_C(0x1f4302ce58b4a733), UINT64_C(0x1f42f78f08b76704)
    },
    {   UINT64_C(0x0000000000006220), UINT64_C(0x1f42ff4ec8af670d), UINT64_C(0x1f43030e68b4c734),
        UINT64_C(0x1f42fc8f34b64708), UINT64_C(0x1f43080e58abe71c)
    },
    {   UINT64_C(0x0000000000006221), UINT64_C(0x1f42fc4f54b44703), UINT64_C(0x1f4313cec4aca730),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000896b), UINT64_C(0x1fe2f78f74b945bd), UINT64_C(0x1fe2f78f74b945bd),
        UINT64_C(0x1182f80c307f86bc), UINT64_C(0x1182f80c307f86bc)
    },
    {   UINT64_C(0x0000000000006223), UINT64_C(0x1f430b0f14b0a723), UINT64_C(0x1f42f58f90b50716),
        UINT64_C(0x1f42fa8f30aea706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006224), UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f88ade6fb),
        UINT64_C(0x1f42f98f54b5c70b), UINT64_C(0x1f42fd8f28b52709)
    },
    {   UINT64_C(0x000000000000896e), UINT64_C(0x14a3020edc8a2614), UINT64_C(0x14a3020ed88a2613),
        UINT64_C(0x1d12fe4ed8ad2613), UINT64_C(0x1d12fe4ed8ad2613)
    },
    {   UINT64_C(0x0000000000006226), UINT64_C(0x1f42fd8f44b26706), UINT64_C(0x1f43024f54aa86fe),
        UINT64_C(0x1f42f88fe0b6670c), UINT64_C(0x1f42f98fe0b2a702)
    },
    {   UINT64_C(0x0000000000006227), UINT64_C(0x1f42f68f08b6c70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43050ea8b48735), UINT64_C(0x1f42fe4fa4b82706)
    },
    {   UINT64_C(0x0000000000008971), UINT64_C(0x1d12f94ed8aaa613), UINT64_C(0x1d12f94ed8aaa613),
        UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0)
    },
    {   UINT64_C(0x0000000000006229), UINT64_C(0x1f42fc4e5cb3c735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004ef4ad06f6)
    },
    {   UINT64_C(0x000000000000622a), UINT64_C(0x1f42fa8ff0b10706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8fa4b0e711), UINT64_C(0x1f42f78fc4b7072e)
    },
    {   UINT64_C(0x000000000000622b), UINT64_C(0x1f42fd8eb8b30731), UINT64_C(0x1f43084eb4b2872a),
        UINT64_C(0x1f42ffcf38b2a709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008975), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1222fe8ce07c4690), UINT64_C(0x1223038ce07ec690)
    },
    {   UINT64_C(0x0000000000008976), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f95390aaa67e), UINT64_C(0x1c72f94d28aaa4e4)
    },
    {   UINT64_C(0x0000000000008977), UINT64_C(0x1c72f4d390776690), UINT64_C(0x1c735fd390ad0690),
        UINT64_C(0x1c735fcce0ad04e4), UINT64_C(0x1c72f48ce07764e4)
    },
    {   UINT64_C(0x000000000000622f), UINT64_C(0x1f42fb4f3cb6e713), UINT64_C(0x1f42f78fd0b306fa),
        UINT64_C(0x1f42fb8eecb5c71e), UINT64_C(0x1f42f70f54b7271c)
    },
    {   UINT64_C(0x0000000000008979), UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e),
        UINT64_C(0x105aff908c7765a4), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x000000000000897a), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x000000000000897b), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x25baff8b38cca6f9)
    },
    {   UINT64_C(0x000000000000897c), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x000000000000897d), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x000000000000897e), UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e),
        UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1752f94e7096262c)
    },
    {   UINT64_C(0x000000000000897f), UINT64_C(0x1752f94e7096262c), UINT64_C(0x12caf78f7084e5e9),
        UINT64_C(0x12caf78f7484e5ea), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x0000000000006237), UINT64_C(0x1f42ff4f1cb0c711), UINT64_C(0x1f42fe4e3cb64725),
        UINT64_C(0x1f42fd8f50b66700), UINT64_C(0x1f430bce50b9a72c)
    },
    {   UINT64_C(0x0000000000008981), UINT64_C(0x0000000000000000), UINT64_C(0x1b32fd0e98a3c677),
        UINT64_C(0x1b32fd0e98a3c677), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008982), UINT64_C(0x0000000000000000), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008983), UINT64_C(0x122afb8d9483a662), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60fa0b7668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008984), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001492d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000087a0), UINT64_C(0x1d82ef4f74b30762), UINT64_C(0x1692f34f6895468e),
        UINT64_C(0x1d82ef4f74b30762), UINT64_C(0x1692f34f6895468e)
    },
    {   UINT64_C(0x00000000000060eb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4313ce68a5272a), UINT64_C(0x1f43108eacb5c735)
    },
    {   UINT64_C(0x0000000000008988), UINT64_C(0x0000000000000000), UINT64_C(0x135aff8f1483265e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006240), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f0f14b2671f),
        UINT64_C(0x1f42f68ef8b5e711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006241), UINT64_C(0x1f42f84efcb64729), UINT64_C(0x1f43004e8cb70715),
        UINT64_C(0x1f42fc4f38b686e7), UINT64_C(0x1f42ff4ee0b7c722)
    },
    {   UINT64_C(0x0000000000006242), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f28b546f2),
        UINT64_C(0x1f42f58fa0adc729), UINT64_C(0x1f42fd4eacb78709)
    },
    {   UINT64_C(0x0000000000011111), UINT64_C(0x186afa8d4099e683), UINT64_C(0x1722f78fa0992675),
        UINT64_C(0x1452fa8f78938684), UINT64_C(0x1132f44fa0924675)
    },
    {   UINT64_C(0x0000000000006244), UINT64_C(0x1f42f98f18afc721), UINT64_C(0x1f42fa8ef8b50709),
        UINT64_C(0x1f42fc8e84aea72e), UINT64_C(0x1f42f98fe0b826ff)
    },
    {   UINT64_C(0x0000000000011112), UINT64_C(0x160afe4f6c9f6675), UINT64_C(0x11fafd0f609d8675),
        UINT64_C(0x1ca2fd0f60c82675), UINT64_C(0x18b2f44fa0a3c675)
    },
    {   UINT64_C(0x0000000000006246), UINT64_C(0x1f42fc8e8caf0741), UINT64_C(0x1f42fb8e7cb2e6f7),
        UINT64_C(0x1f42f78eb0b40715), UINT64_C(0x1f42f98efcb0e713)
    },
    {   UINT64_C(0x0000000000006247), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74e9cb6c70b),
        UINT64_C(0x1f42f98fa4b426f3), UINT64_C(0x1f42f58eeca94706)
    },
    {   UINT64_C(0x0000000000006248), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78ef8b78703),
        UINT64_C(0x1f43074e84ab471e), UINT64_C(0x1f42fb8f0cb7c6fe)
    },
    {   UINT64_C(0x0000000000006249), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68e68b5c723),
        UINT64_C(0x1f42fb8f54b666e7), UINT64_C(0x1f430d0e74af86fe)
    },
    {   UINT64_C(0x000000000000624a), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e0ea8af272a),
        UINT64_C(0x1f42fb8f34b38711), UINT64_C(0x1f42f4cfa4b80704)
    },
    {   UINT64_C(0x000000000000624b), UINT64_C(0x1f42f88edcb526e7), UINT64_C(0x1f430a4ec0ada735),
        UINT64_C(0x1f43110eb4b0670f), UINT64_C(0x1f42f68eb8aa6721)
    },
    {   UINT64_C(0x000000000000624c), UINT64_C(0x1f430b4eb4b20736), UINT64_C(0x1f430c0ed8ad272d),
        UINT64_C(0x1f42f88e6cb3c730), UINT64_C(0x1f42fa8ef4b306f2)
    },
    {   UINT64_C(0x000000000000624d), UINT64_C(0x1f42f78fe0b2a6fd), UINT64_C(0x1f42f78f4cac671d),
        UINT64_C(0x1f42f68f90b006f3), UINT64_C(0x1f42f98ff0b5c6f2)
    },
    {   UINT64_C(0x000000000000624e), UINT64_C(0x1f42f98ef0b5a727), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e88b3a73c)
    },
    {   UINT64_C(0x000000000000624f), UINT64_C(0x1f42f98f48b826f6), UINT64_C(0x1f43074e88b4a71d),
        UINT64_C(0x1f42f98efcae06fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060fd), UINT64_C(0x1f42fbceecb4a746), UINT64_C(0x1f43120e74a60734),
        UINT64_C(0x1f430c0e7cb0c735), UINT64_C(0x1f43080e5cae8735)
    },
    {   UINT64_C(0x00000000000060fe), UINT64_C(0x1f42fe8f2cb6a730), UINT64_C(0x1f43134f64b2471c),
        UINT64_C(0x1f43034e1ca4a740), UINT64_C(0x1f43058e50b6471e)
    },
    {   UINT64_C(0x000000000000899b), UINT64_C(0x0000000000000000), UINT64_C(0x0d52fc4f7869c6e8),
        UINT64_C(0x0d5300cccc6a46e8), UINT64_C(0x0d52f84ccc6ec6e8)
    },
    {   UINT64_C(0x0000000000006253), UINT64_C(0x1f42f78e7cb2e6f3), UINT64_C(0x1f42f68e68b72739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f18b786ee)
    },
    {   UINT64_C(0x0000000000006254), UINT64_C(0x1f42fc8ef0b76731), UINT64_C(0x1f42f98e28b66727),
        UINT64_C(0x1f42f48fb4b7a6e1), UINT64_C(0x1f42f68ed8b94712)
    },
    {   UINT64_C(0x0000000000006255), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8f60b68721), UINT64_C(0x1f42f58e4cb44715)
    },
    {   UINT64_C(0x0000000000006256), UINT64_C(0x1f42f98e54ba670f), UINT64_C(0x1f42f98e8cb8a707),
        UINT64_C(0x1f42fb8f04b6e713), UINT64_C(0x1f42f58e84b48746)
    },
    {   UINT64_C(0x0000000000006257), UINT64_C(0x1f42f98f14b1e730), UINT64_C(0x1f42f98f40b78703),
        UINT64_C(0x1f43054ea4b62733), UINT64_C(0x1f42facec8b80728)
    },
    {   UINT64_C(0x0000000000006258), UINT64_C(0x1f42fa8f30b2c729), UINT64_C(0x1f42f58e90b086ee),
        UINT64_C(0x1f42f98ea8b1a718), UINT64_C(0x1f42fb8edcb42744)
    },
    {   UINT64_C(0x0000000000006259), UINT64_C(0x1f42f7cf74b346f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8efcb426fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000625a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e94afa735),
        UINT64_C(0x1f42f98e54b46711), UINT64_C(0x1f42fd8fa8b3c70b)
    },
    {   UINT64_C(0x000000000000625b), UINT64_C(0x1f42f98f94b3a704), UINT64_C(0x1f42f98f28aec72e),
        UINT64_C(0x1f42fe8eccb7a6fa), UINT64_C(0x1f42f4cf7cb76702)
    },
    {   UINT64_C(0x000000000000625c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f58b446fe),
        UINT64_C(0x1f4309ce84af8730), UINT64_C(0x1f42f48ebcb56710)
    },
    {   UINT64_C(0x000000000000625d), UINT64_C(0x1f42f8ce98af0709), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f68ea4b326fb), UINT64_C(0x1f42f68ec8b74724)
    },
    {   UINT64_C(0x000000000000625e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58ea0b72711),
        UINT64_C(0x1f42fa8ee0ad6714), UINT64_C(0x1f42f88e70b4c70d)
    },
    {   UINT64_C(0x000000000000625f), UINT64_C(0x1f42f78ea8afc719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e8f88b066e5), UINT64_C(0x1f430d4fccaec6f3)
    },
    {   UINT64_C(0x0000000000006260), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf0cb6c71a),
        UINT64_C(0x1f43070ed4b3871c), UINT64_C(0x1f43088e5cae672e)
    },
    {   UINT64_C(0x0000000000006261), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f14b446df),
        UINT64_C(0x1f42fe8ea8b40736), UINT64_C(0x1f43120eb4a96708)
    },
    {   UINT64_C(0x0000000000006262), UINT64_C(0x1f430c0eb4b22722), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4309ce44ada737), UINT64_C(0x1f43114f74b4e720)
    },
    {   UINT64_C(0x0000000000006263), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4f3cb6e6fb)
    },
    {   UINT64_C(0x0000000000006264), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43108e70b58733)
    },
    {   UINT64_C(0x0000000000006265), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f40b5e6d9),
        UINT64_C(0x1f42fd8f84b906ff), UINT64_C(0x1f42ff0f6cb7c6f1)
    },
    {   UINT64_C(0x0000000000006266), UINT64_C(0x1f4306cf90b6a70d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108e6cada715), UINT64_C(0x1f42f74f2cb846e0)
    },
    {   UINT64_C(0x0000000000006267), UINT64_C(0x1f42f78ef4b666d3), UINT64_C(0x1f4300cf58b586e2),
        UINT64_C(0x1f42fdcf64b766e2), UINT64_C(0x1f42f9cf64b806e5)
    },
    {   UINT64_C(0x0000000000006268), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f48b5e6f9),
        UINT64_C(0x1f430a8f0cb646f9), UINT64_C(0x1f42fa4f84b926ed)
    },
    {   UINT64_C(0x0000000000006269), UINT64_C(0x1f42f94f3cb706f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f6cb7a728), UINT64_C(0x1f4309ce44ada738)
    },
    {   UINT64_C(0x000000000000626a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f24e74b94722),
        UINT64_C(0x1f43088f00b9672a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000626b), UINT64_C(0x1f4301cf78b7e726), UINT64_C(0x1f43030f08b786b1),
        UINT64_C(0x1f4312cedcad8734), UINT64_C(0x1f42fbcf54b7272f)
    },
    {   UINT64_C(0x000000000000626c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008f00b8473f), UINT64_C(0x1f42f84f90b66712)
    },
    {   UINT64_C(0x000000000000626d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43070f6cb78717), UINT64_C(0x1f42f58f24b8a72c)
    },
    {   UINT64_C(0x000000000000626e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f98b6a6d0),
        UINT64_C(0x1f4309ce6cb1c721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000626f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43148e44b30732),
        UINT64_C(0x1f42ff0f2cb7a712), UINT64_C(0x1f43020eb4b72736)
    },
    {   UINT64_C(0x0000000000006270), UINT64_C(0x1f42f78ed8b82739), UINT64_C(0x1f43114f58b566ed),
        UINT64_C(0x1f43028fbcb646e5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006271), UINT64_C(0x1f42ff0fdcb666f7), UINT64_C(0x1f43154f18b1c736),
        UINT64_C(0x1f42fa8f84b6e6f2), UINT64_C(0x1f4304cfacb6270a)
    },
    {   UINT64_C(0x0000000000006272), UINT64_C(0x1f4322cf34aa2726), UINT64_C(0x1f43108f34adc715),
        UINT64_C(0x1f430e0f2cb6c6fc), UINT64_C(0x1f42fd4f90b5e6ed)
    },
    {   UINT64_C(0x0000000000006273), UINT64_C(0x1f43008fbcb5c6fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4f94b8271f), UINT64_C(0x1f42fb0f80b6270d)
    },
    {   UINT64_C(0x0000000000006274), UINT64_C(0x1f4303cf84b6e70f), UINT64_C(0x1f42f94f94b6c71d),
        UINT64_C(0x1f430d8e94a1873b), UINT64_C(0x1f43030e7cabc70b)
    },
    {   UINT64_C(0x0000000000006122), UINT64_C(0x1f43168eb4ab672e), UINT64_C(0x1f430b4edcace733),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cef4afe73b)
    },
    {   UINT64_C(0x0000000000006123), UINT64_C(0x1f430c8e98ace71a), UINT64_C(0x1f430f4e80aba73d),
        UINT64_C(0x1f430b4e48b2c737), UINT64_C(0x1f4310ceccb02734)
    },
    {   UINT64_C(0x0000000000006277), UINT64_C(0x1f43110e68aa471f), UINT64_C(0x1f42fb4f88b52708),
        UINT64_C(0x1f43020f50ada70f), UINT64_C(0x1f42f94f00b3e6f2)
    },
    {   UINT64_C(0x0000000000006278), UINT64_C(0x1f430a0e44b64735), UINT64_C(0x1f42fe4f3cb82704),
        UINT64_C(0x1f430e4e78a8871f), UINT64_C(0x1f43030e7cb5673d)
    },
    {   UINT64_C(0x0000000000006279), UINT64_C(0x1f42f74ef8b38707), UINT64_C(0x1f42ff4facb6c713),
        UINT64_C(0x1f42fc4f8cb04719), UINT64_C(0x1f42f8ce5cb6273b)
    },
    {   UINT64_C(0x000000000000627a), UINT64_C(0x1f42fd4f2cb6c704), UINT64_C(0x1f42fc4ee4b16704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4eccb7a72e)
    },
    {   UINT64_C(0x000000000000627b), UINT64_C(0x1f42ff0f54b90705), UINT64_C(0x1f42fc4e90b6e705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4f08b06710)
    },
    {   UINT64_C(0x000000000000627c), UINT64_C(0x1f42fa4f20b5270d), UINT64_C(0x1f43070ea4ac4728),
        UINT64_C(0x1f42ff4e7caf6717), UINT64_C(0x1f43004f54b74716)
    },
    {   UINT64_C(0x000000000000612a), UINT64_C(0x1f42fb4e48b3c745), UINT64_C(0x1f4305ce9cb2c738),
        UINT64_C(0x1f430c8e18b36741), UINT64_C(0x1f4312ceb4ac4739)
    },
    {   UINT64_C(0x000000000000627e), UINT64_C(0x1f42fb4e9cb1670d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118e6caec711), UINT64_C(0x1f42f84ef8b54722)
    },
    {   UINT64_C(0x00000000000089c8), UINT64_C(0x17fafb4e4497c638), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000612d), UINT64_C(0x1f43118eb8a7c734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a4ee8b90732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000011130), UINT64_C(0x16b2f80f7491c697), UINT64_C(0x121afa0f7485e697),
        UINT64_C(0x2072fa4f74b8c5a1), UINT64_C(0x17cafa4f749625a1)
    },
    {   UINT64_C(0x000000000000612f), UINT64_C(0x1f43024e90b6e741), UINT64_C(0x1f43148f3cae071e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4313ce58aa0742)
    },
    {   UINT64_C(0x0000000000011131), UINT64_C(0x0000000000000000), UINT64_C(0x182af3cc5c9545a1),
        UINT64_C(0x1862f58c5c9b65a1), UINT64_C(0x1222fa0fa08605a4)
    },
    {   UINT64_C(0x00000000000089cd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15baf6cdf08ec650), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006285), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cfe8b326f8),
        UINT64_C(0x1f42fd4ff8b44728), UINT64_C(0x1f42fa9018b9e6fb)
    },
    {   UINT64_C(0x0000000000006286), UINT64_C(0x1f43039008b7e6fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88fb4b8c70d)
    },
    {   UINT64_C(0x0000000000006134), UINT64_C(0x1f42fc8e64ad6740), UINT64_C(0x1f43074e98ae073b),
        UINT64_C(0x1f4302ce4cb2673c), UINT64_C(0x1f42f74ee8b3673d)
    },
    {   UINT64_C(0x0000000000006288), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcee8b106f7), UINT64_C(0x1f42f88f8cb6c717)
    },
    {   UINT64_C(0x0000000000006289), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f20adc706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cf20b2e6f5)
    },
    {   UINT64_C(0x000000000000628a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f64b4270e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006138), UINT64_C(0x1f431c4eaca72723), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f28a84726)
    },
    {   UINT64_C(0x0000000000006139), UINT64_C(0x0000000000000000), UINT64_C(0x1f43060f58b46720),
        UINT64_C(0x1f42ff0e80b2c728), UINT64_C(0x1f43030f0caf071e)
    },
    {   UINT64_C(0x000000000000613a), UINT64_C(0x1f43030eb4b2a71e), UINT64_C(0x1f430a4eecb50738),
        UINT64_C(0x1f42ff8eecb30729), UINT64_C(0x1f43020e54b18731)
    },
    {   UINT64_C(0x000000000000613b), UINT64_C(0x1f43020ee4b1c741), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e58b24737), UINT64_C(0x1f43044f18ae2724)
    },
    {   UINT64_C(0x000000000000613c), UINT64_C(0x1f43054ef8b30728), UINT64_C(0x1f430f8e7cae872f),
        UINT64_C(0x1f43160e6ca92728), UINT64_C(0x1f43130e84ab6737)
    },
    {   UINT64_C(0x000000000000613d), UINT64_C(0x1f43120ef0ab0735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43094ed8ae6720), UINT64_C(0x1f43038f08ab6721)
    },
    {   UINT64_C(0x000000000000613e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43008eacad8712),
        UINT64_C(0x1f43038f44af672b), UINT64_C(0x1f43078eacb3071a)
    },
    {   UINT64_C(0x000000000000613f), UINT64_C(0x1f43024e60b82746), UINT64_C(0x1f430d4e50b0e73c),
        UINT64_C(0x1f4317ced0a78732), UINT64_C(0x1f4305ce84a68738)
    },
    {   UINT64_C(0x00000000000089dc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f431d0c4c78c6b7), UINT64_C(0x0f42f38c4c6406b7)
    },
    {   UINT64_C(0x0000000000006141), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e0e78b56732), UINT64_C(0x1f430b0eccb5c738)
    },
    {   UINT64_C(0x0000000000006142), UINT64_C(0x1f43140e70a88736), UINT64_C(0x1f4315ceb0b0272b),
        UINT64_C(0x1f43068e5cb0c726), UINT64_C(0x1f43098f20b4e72b)
    },
    {   UINT64_C(0x0000000000006143), UINT64_C(0x1f4303ceecb2c728), UINT64_C(0x1f43080ee4ad071b),
        UINT64_C(0x1f4306cf0cb3671f), UINT64_C(0x1f431c8e6caa0739)
    },
    {   UINT64_C(0x00000000000089e0), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000089e1), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x0000000000006146), UINT64_C(0x1f430b4e50b36736), UINT64_C(0x1f43054ee8b1a722),
        UINT64_C(0x1f42f98e60b3674a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089e3), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x0000000000006148), UINT64_C(0x1f4310cf58b5072a), UINT64_C(0x1f42fbce40b6273a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028e40b3e740)
    },
    {   UINT64_C(0x0000000000006149), UINT64_C(0x1f4312ce8ca98728), UINT64_C(0x1f4315cef0abc734),
        UINT64_C(0x1f4318cecca92732), UINT64_C(0x1f430f4f30b0e728)
    },
    {   UINT64_C(0x000000000000614a), UINT64_C(0x1f43160ebca40725), UINT64_C(0x1f4312ced8aa673d),
        UINT64_C(0x1f43038eb8b18728), UINT64_C(0x1f43150e50a8a731)
    },
    {   UINT64_C(0x000000000000614b), UINT64_C(0x1f42ff4ef0b5a728), UINT64_C(0x1f430cce5cb6e741),
        UINT64_C(0x1f43150e8ca60720), UINT64_C(0x1f43158eccb0e71c)
    },
    {   UINT64_C(0x000000000000614c), UINT64_C(0x1f42fd0ecca92726), UINT64_C(0x1f4305cf50b1c732),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000614d), UINT64_C(0x1f42facec4b5273a), UINT64_C(0x1f430b4eb8ace739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8e94ae6724)
    },
    {   UINT64_C(0x0000000000008804), UINT64_C(0x0a6af7d2a46224eb), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x0fa2ee12a47d04eb), UINT64_C(0x0fa2ee12c47d04e2)
    },
    {   UINT64_C(0x0000000000008805), UINT64_C(0x1f42ee12c4bb84e2), UINT64_C(0x3e82ee12c53884e2),
        UINT64_C(0x105b0e4cac6fc69a), UINT64_C(0x0fa2ee0af07d035b)
    },
    {   UINT64_C(0x000000f21de88806), UINT64_C(0x0a6b0ad6645e468c), UINT64_C(0x0a6b01d66459c68c),
        UINT64_C(0x0a6b01cd6c59c44e), UINT64_C(0x0a6b01d66459c68c)
    },
    {   UINT64_C(0x0007d6f89ec88807), UINT64_C(0x0de2f8d66472468c), UINT64_C(0x0de2f5966470a68c),
        UINT64_C(0x0de2f94d6c72844e), UINT64_C(0x0de2f5966470a68c)
    },
    {   UINT64_C(0x0000000000006152), UINT64_C(0x1f43058e7cafc735), UINT64_C(0x1f43100eacafc742),
        UINT64_C(0x1f43018f58b4e731), UINT64_C(0x1f43020e00b18736)
    },
    {   UINT64_C(0x0000000000006153), UINT64_C(0x0000000000000000), UINT64_C(0x1f430cce40b44725),
        UINT64_C(0x1f4308cec4b3273b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000089f0), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x15aafb0fa08ea67e), UINT64_C(0x156afd0f348ca5f9)
    },
    {   UINT64_C(0x00000000000089f1), UINT64_C(0x156afd4f348cc5f9), UINT64_C(0x0e3afbcc887086a6),
        UINT64_C(0x0e3afbcc887086a6), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x000000000000880c), UINT64_C(0x22aafd4f54c186aa), UINT64_C(0x2dfafd4f54eec6aa),
        UINT64_C(0x090b00d5e859468e), UINT64_C(0x0d5300d5e46a668e)
    },
    {   UINT64_C(0x000000000000880d), UINT64_C(0x119b00d5e47b868e), UINT64_C(0x090b00d5e859468e),
        UINT64_C(0x0d5300d5e46a668e), UINT64_C(0x119b00d5e47b868e)
    },
    {   UINT64_C(0x0000000000006158), UINT64_C(0x1f43104e6ca4c730), UINT64_C(0x1f43100e40ae8743),
        UINT64_C(0x1f430c4eccae873f), UINT64_C(0x1f43150e8cabe73e)
    },
    {   UINT64_C(0x0000000000006159), UINT64_C(0x1f43084efcaec721), UINT64_C(0x1f43054ef8afe730),
        UINT64_C(0x1f430d8ec0a5a730), UINT64_C(0x1f43034edcb0e730)
    },
    {   UINT64_C(0x000000000000615a), UINT64_C(0x1f42fa8ed4b4c732), UINT64_C(0x1f43138e34b1273f),
        UINT64_C(0x1f43068ed8b1c73c), UINT64_C(0x1f430d4e5cad072d)
    },
    {   UINT64_C(0x000000000000615b), UINT64_C(0x1f430fcf14aca729), UINT64_C(0x1f430dce9caee73f),
        UINT64_C(0x1f431c8eb8a8a730), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000615c), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088ed0a72728),
        UINT64_C(0x1f43018eb8aac70c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000615d), UINT64_C(0x1f43070facacc720), UINT64_C(0x1f43038ec8b74727),
        UINT64_C(0x1f43124e68b1873d), UINT64_C(0x1f430acee4a88735)
    },
    {   UINT64_C(0x00000000000089fa), UINT64_C(0x0c83014c4c6866b1), UINT64_C(0x0c82fe4c4c66e6b1),
        UINT64_C(0x1313004c4c82a6b7), UINT64_C(0x1312fe4c4c81a6b7)
    },
    {   UINT64_C(0x00000000000089fb), UINT64_C(0x0f431a8c4c78c6b7), UINT64_C(0x0f42f38c4c6546b7),
        UINT64_C(0x074afc0cdc58e68c), UINT64_C(0x074af38cdc54a68c)
    },
    {   UINT64_C(0x00000000000089fc), UINT64_C(0x204aff4d30b7267e), UINT64_C(0x204aff4d30b7267e),
        UINT64_C(0x1e72fb8dd0acc65a), UINT64_C(0x1e73050dd0b1865a)
    },
    {   UINT64_C(0x0000000000006161), UINT64_C(0x1f43100ea0aec739), UINT64_C(0x1f42fd4eb4b20735),
        UINT64_C(0x1f430fceb8aa2737), UINT64_C(0x1f42fc4f08b8873b)
    },
    {   UINT64_C(0x0000000000006162), UINT64_C(0x1f43050e94b7673a), UINT64_C(0x1f43074ec0acc72e),
        UINT64_C(0x1f4308ce60acc742), UINT64_C(0x1f43108e84a72742)
    },
    {   UINT64_C(0x0000000000006163), UINT64_C(0x1f4308cea8b08730), UINT64_C(0x1f43168eb4b1c730),
        UINT64_C(0x1f430d0ebcad8735), UINT64_C(0x1f43124ed0ab873d)
    },
    {   UINT64_C(0x0000000000006164), UINT64_C(0x1f4300cf10b2a730), UINT64_C(0x1f43000ea8b2673d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006165), UINT64_C(0x1f42fdce54b2c739), UINT64_C(0x1f42ff4e34b26735),
        UINT64_C(0x1f43008ea4afa72e), UINT64_C(0x1f43178eb4a6c739)
    },
    {   UINT64_C(0x0000000000006166), UINT64_C(0x1f43074ea8ad2736), UINT64_C(0x1f43004ef8ae073f),
        UINT64_C(0x1f430a8ec0ae672c), UINT64_C(0x1f43120e2cb14739)
    },
    {   UINT64_C(0x0000000000006167), UINT64_C(0x1f42ff8e2cafe72a), UINT64_C(0x1f42fc4e84a2c736),
        UINT64_C(0x1f430f4ed0b6a72c), UINT64_C(0x1f43090f2cb60736)
    },
    {   UINT64_C(0x0000000000006168), UINT64_C(0x1f43078ee0a62730), UINT64_C(0x1f43150f18a60734),
        UINT64_C(0x1f430b0ed8afe737), UINT64_C(0x1f430ccf00b5a724)
    },
    {   UINT64_C(0x0000000000006169), UINT64_C(0x1f43068f40b2c735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8e5cada73e), UINT64_C(0x1f4310cee0ae073a)
    },
    {   UINT64_C(0x000000000000616a), UINT64_C(0x1f430b0e5cb16736), UINT64_C(0x1f4311cef8ae473e),
        UINT64_C(0x1f4305ce9cb10749), UINT64_C(0x1f43100e70aa8736)
    },
    {   UINT64_C(0x000000000000616b), UINT64_C(0x1f43034e5cb0e740), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a8f08b3672b), UINT64_C(0x1f43018e50b2072f)
    },
    {   UINT64_C(0x000000000000616c), UINT64_C(0x1f43120e70ab6736), UINT64_C(0x1f4303ceb4b3e729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000e44b50741)
    },
    {   UINT64_C(0x000000000000616d), UINT64_C(0x1f43144e2ca96735), UINT64_C(0x1f43048e88b4c745),
        UINT64_C(0x1f430c8ea8b4a726), UINT64_C(0x1f43170e40ad6743)
    },
    {   UINT64_C(0x000000000000616e), UINT64_C(0x1f430bcdecad4738), UINT64_C(0x1f43108eb8ac2739),
        UINT64_C(0x1f430a8e4caee736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000062c2), UINT64_C(0x1f43034ec0b5a719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8f9cb5a71c), UINT64_C(0x1f42ff8f44afe6fe)
    },
    {   UINT64_C(0x0000000000006170), UINT64_C(0x1f430d4eb8ac272b), UINT64_C(0x1f43154eccaac733),
        UINT64_C(0x1f43118edcad4737), UINT64_C(0x1f430d4f48b2c73d)
    },
    {   UINT64_C(0x0000000000006171), UINT64_C(0x1f43064e50b5e744), UINT64_C(0x1f430b8e68a7673c),
        UINT64_C(0x1f430b0e7cb2e738), UINT64_C(0x1f43078edcb0472a)
    },
    {   UINT64_C(0x0000000000006172), UINT64_C(0x1f430c8f2cb0872a), UINT64_C(0x1f43048f20af673c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e78a6a736)
    },
    {   UINT64_C(0x00000000000062c6), UINT64_C(0x1f42fe8e7cad86f3), UINT64_C(0x1f42f88ebcb72748),
        UINT64_C(0x1f43000fe8b346fe), UINT64_C(0x1f42fe4eb0b7e742)
    },
    {   UINT64_C(0x0000000000006174), UINT64_C(0x1f4303cedcb6a748), UINT64_C(0x1f4318ce4ca4a736),
        UINT64_C(0x1f4302ce98af2724), UINT64_C(0x1f43094e8cafe735)
    },
    {   UINT64_C(0x0000000000006175), UINT64_C(0x0000000000000000), UINT64_C(0x1f43028f34aa8738),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8e64b2a729)
    },
    {   UINT64_C(0x0000000000006176), UINT64_C(0x1f43038e84acc736), UINT64_C(0x1f43098eacb30731),
        UINT64_C(0x1f431e0e60a4072d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006177), UINT64_C(0x1f43034e50b9474c), UINT64_C(0x1f4302ce64b1e742),
        UINT64_C(0x1f43070eecad674c), UINT64_C(0x1f43108eb4b14733)
    },
    {   UINT64_C(0x0000000000006178), UINT64_C(0x0000000000000000), UINT64_C(0x1f43054eecad2721),
        UINT64_C(0x1f43044f14b72737), UINT64_C(0x1f430f0e50a9a73e)
    },
    {   UINT64_C(0x00000000000062cc), UINT64_C(0x1f42faced0b946ea), UINT64_C(0x1f42f54e60b6a71e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f59024b68712)
    },
    {   UINT64_C(0x000000000000617a), UINT64_C(0x1f43060e8caa0736), UINT64_C(0x1f43070e84b2e73a),
        UINT64_C(0x1f42fc4e74b94734), UINT64_C(0x1f430b0e70aae738)
    },
    {   UINT64_C(0x000000000000617b), UINT64_C(0x1f43174e70afc737), UINT64_C(0x1f4312ce4caee73b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e0ee8aba739)
    },
    {   UINT64_C(0x000000000000617c), UINT64_C(0x1f43084ec4a46739), UINT64_C(0x1f42fbceccb08739),
        UINT64_C(0x1f4301cef0ae074c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000013857), UINT64_C(0x1302f28fa0800675), UINT64_C(0x1ad2f44fa0ae0675),
        UINT64_C(0x169af6cf5897a682), UINT64_C(0x1382f40f58922682)
    },
    {   UINT64_C(0x000000000000617e), UINT64_C(0x1f43088ef0b1e72e), UINT64_C(0x1f42fbce7cab0737),
        UINT64_C(0x1f43160e7ca5c739), UINT64_C(0x1f43098e4cb72739)
    },
    {   UINT64_C(0x0000000000008835), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x00027cd8303c66e0), UINT64_C(0x00027cd8303c66e0)
    },
    {   UINT64_C(0x0000000000008836), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x00027a97cc42e656)
    },
    {   UINT64_C(0x0000000000006181), UINT64_C(0x1f43008e6caea739), UINT64_C(0x1f42f9ce4cb6673d),
        UINT64_C(0x1f43044eacaa4735), UINT64_C(0x1f43028eacb5c736)
    },
    {   UINT64_C(0x0000000000006182), UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e38ad0744),
        UINT64_C(0x1f430c8f3cb2c721), UINT64_C(0x1f43010ea4a8271e)
    },
    {   UINT64_C(0x0000000000006183), UINT64_C(0x1f430a8de4ade741), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006184), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4ed0aa272b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f18b28727)
    },
    {   UINT64_C(0x000000000000883b), UINT64_C(0x00027ccbb03c6371), UINT64_C(0x00027ccbb03c6371),
        UINT64_C(0x00027ccbb03c63c0), UINT64_C(0x00027ccbb03c63c0)
    },
    {   UINT64_C(0x0000000000006186), UINT64_C(0x1f430b0e20ae2741), UINT64_C(0x1f4318ce40ad2740),
        UINT64_C(0x1f43048e4cae6741), UINT64_C(0x1f4308ceb8ac0737)
    },
    {   UINT64_C(0x000000000001385c), UINT64_C(0x10b2f80f747c45a1), UINT64_C(0x13a2ef4c388805a1),
        UINT64_C(0x12e2f80c388645a1), UINT64_C(0x0a62f4cfa078e5a2)
    },
    {   UINT64_C(0x00000000000062db), UINT64_C(0x1f430ace70ac0725), UINT64_C(0x1f42f5cf1cb786e6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbce4cb0e719)
    },
    {   UINT64_C(0x000000000001385d), UINT64_C(0x0fb2f68f747905a1), UINT64_C(0x0a1af30f78678661),
        UINT64_C(0x1262f80f6c840594), UINT64_C(0x0c5af94fa0784594)
    },
    {   UINT64_C(0x00000000000149ce), UINT64_C(0x1802f6cf849a46a3), UINT64_C(0x18c2f6cf689d26a9),
        UINT64_C(0x17baf6cfa099269c), UINT64_C(0x161af6cf6c9286a8)
    },
    {   UINT64_C(0x0000000000008841), UINT64_C(0x0000000000000000), UINT64_C(0x1a2afa0f68a1e68c),
        UINT64_C(0x0000000000000000), UINT64_C(0x12b2f50f6884a68c)
    },
    {   UINT64_C(0x0000000000008842), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1712f58c349345a1), UINT64_C(0x1f2afb4f64ba6697)
    },
    {   UINT64_C(0x00000000000149d1), UINT64_C(0x157af70fa08fe69c), UINT64_C(0x15c2f6cfa091269b),
        UINT64_C(0x158af6cfa090669c), UINT64_C(0x1892f68f689c66a9)
    },
    {   UINT64_C(0x0000000000008844), UINT64_C(0x1c0af48f64ad868b), UINT64_C(0x17d2f94f1897669f),
        UINT64_C(0x1d92f64f64b2e697), UINT64_C(0x121afa0f7485e697)
    },
    {   UINT64_C(0x0000000000008845), UINT64_C(0x0000000000000000), UINT64_C(0x15d2ff8fa08d067e),
        UINT64_C(0x1fa2efcf68b7468c), UINT64_C(0x17c2f78f6898c68c)
    },
    {   UINT64_C(0x00000000000149d4), UINT64_C(0x17d2f6cfa099669c), UINT64_C(0x17baf6cfa099269c),
        UINT64_C(0x17baf6cfa099269c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010008), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a6af7cf74648683),
        UINT64_C(0x0d23121634748682), UINT64_C(0x0faaee8fa082068c)
    },
    {   UINT64_C(0x0000000000010009), UINT64_C(0x0fa2f60e3c7ca6c3), UINT64_C(0x179b020f5496a6aa),
        UINT64_C(0x1853010f5898e682), UINT64_C(0x06b30f16345ca682)
    },
    {   UINT64_C(0x000000000001000a), UINT64_C(0x0a6af88ccc65e685), UINT64_C(0x0a6af20cd062a685),
        UINT64_C(0x0fa30e139c7c0682), UINT64_C(0x151b038fa08845e2)
    },
    {   UINT64_C(0x00001a010009000b), UINT64_C(0x07d2eccd9c4f644d), UINT64_C(0x0a6afa52a061c4e7),
        UINT64_C(0x07d2f4cf744fc44c), UINT64_C(0x08b2ddcf586ec682)
    },
    {   UINT64_C(0x000000000000884b), UINT64_C(0x1db2f68f64af4697), UINT64_C(0x166af9cf4092e69f),
        UINT64_C(0x1a5af6cf5ca3868c), UINT64_C(0x139af58f7483c5a1)
    },
    {   UINT64_C(0x000000000001000d), UINT64_C(0x0fa2ee4fa07a668c), UINT64_C(0x0fa2f1cf847be682),
        UINT64_C(0x0fa2f58f847fa696), UINT64_C(0x0fa300cf8081a682)
    },
    {   UINT64_C(0x000000000001000e), UINT64_C(0x0fa2f58f847c268c), UINT64_C(0x0fa2f3cf5c7c068c),
        UINT64_C(0x0a6afa8f745f25a1), UINT64_C(0x0a6af48d9c5f25a1)
    },
    {   UINT64_C(0x000000000001000f), UINT64_C(0x151b030f788885ec), UINT64_C(0x151b03918088456a),
        UINT64_C(0x151b030f788885ec), UINT64_C(0x0fa30f0f70798680)
    },
    {   UINT64_C(0x04106c03c0010010), UINT64_C(0x1cc30b8f58a34682), UINT64_C(0x131ae14fa0850684),
        UINT64_C(0x131aec0fa0880675), UINT64_C(0x14dafe8f5894a682)
    },
    {   UINT64_C(0x0000000000008850), UINT64_C(0x1cdafd4b94acc6e3), UINT64_C(0x15baff0f6892068c),
        UINT64_C(0x117af14fa07b267e), UINT64_C(0x117af14fa07b267e)
    },
    {   UINT64_C(0x000000000000619b), UINT64_C(0x1f43021078b586ff), UINT64_C(0x1f42ff5030af46ed),
        UINT64_C(0x1f4301cff4b766d2), UINT64_C(0x1f42fe4e98a5e705)
    },
    {   UINT64_C(0x0d51760005010013), UINT64_C(0x1162ec0fa0846675), UINT64_C(0x1a0ae98fa0ab8675),
        UINT64_C(0x14dae90f64996675), UINT64_C(0x1692fd0f5895e683)
    },
    {   UINT64_C(0x1121ea0006f90014), UINT64_C(0x131aee0fa08a2675), UINT64_C(0x1692fcccc895e682),
        UINT64_C(0x131aeacfa0880675), UINT64_C(0x0fa2f24f587e0683)
    },
    {   UINT64_C(0x18f2bc5448d90015), UINT64_C(0x1162fccfa08da675), UINT64_C(0x1693078f589e2675),
        UINT64_C(0x131b010f58948675), UINT64_C(0x1a0affcf58afc675)
    },
    {   UINT64_C(0x0003dc728e010016), UINT64_C(0x131ae6cfa0906675), UINT64_C(0x1163018fa08da675),
        UINT64_C(0x1162ec8fa08a4675), UINT64_C(0x0c2af34d3c6f667f)
    },
    {   UINT64_C(0x0000000000010017), UINT64_C(0x08b2e3cf58666682), UINT64_C(0x0c2af10d3c6e467f),
        UINT64_C(0x0d32ee1454734682), UINT64_C(0x0fa2ee0dac7d039d)
    },
    {   UINT64_C(0x1f73e87c00010018), UINT64_C(0x0a6b0c1750656680), UINT64_C(0x0faaf24f747a05a1),
        UINT64_C(0x0fa2f3cf7479a693), UINT64_C(0x0de2f58f7473a5a1)
    },
    {   UINT64_C(0x2193fc7e40010019), UINT64_C(0x0fa2f1cf6c806693), UINT64_C(0x0de2f5cf747205a1),
        UINT64_C(0x08b2c94c6473868e), UINT64_C(0x0fa2f00c687965a1)
    },
    {   UINT64_C(0x22b0008a1131001a), UINT64_C(0x0fa2f2cf7c7a4693), UINT64_C(0x08b2fa4f745f8676),
        UINT64_C(0x08b2cf0c64616674), UINT64_C(0x0de2f18f74782693)
    },
    {   UINT64_C(0x00000000000062f7), UINT64_C(0x1f42fa8fccb4470e), UINT64_C(0x1f42fd5008b106f6),
        UINT64_C(0x1f43008fccb6e6e6), UINT64_C(0x1f42fe0f78ada6eb)
    },
    {   UINT64_C(0x2814a6945279001c), UINT64_C(0x0fc2db4c6c7985a1), UINT64_C(0x0fa2f44c5c7b05a1),
        UINT64_C(0x0c2af94fa07205a1), UINT64_C(0x0c2af20f6c6c45a2)
    },
    {   UINT64_C(0x0000000000008a42), UINT64_C(0x0ffafccd3077067e), UINT64_C(0x0ffafccd3077067e),
        UINT64_C(0x1cf2ffcd30a9467e), UINT64_C(0x1cf3000d30a9667e)
    },
    {   UINT64_C(0x00000000000062fa), UINT64_C(0x1f42fb0ed8b266df), UINT64_C(0x1f42fd4fb0ab66fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f30b44719)
    },
    {   UINT64_C(0x000000000001001f), UINT64_C(0x089b084f5853e682), UINT64_C(0x0c82ec4cdc6a2697),
        UINT64_C(0x10eaf8127c7d452b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a45), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65aa)
    },
    {   UINT64_C(0x0000000000008a46), UINT64_C(0x1cf2fa9084ac05aa), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000008a47), UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65a9),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65aa)
    },
    {   UINT64_C(0x0000000000008a48), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30aba67c), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x0000000000008a49), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30aba67e)
    },
    {   UINT64_C(0x0000000000008a4a), UINT64_C(0x1cf2fb4d30ab667e), UINT64_C(0x1cf2fb4d30ab667e),
        UINT64_C(0x1cf2fbcd30aba67e), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x0000000000008a4b), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x0000000000008a4c), UINT64_C(0x1cf2fbcd38aba67e), UINT64_C(0x1cf2fb8d30ab467e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbd084ab65a9)
    },
    {   UINT64_C(0x0000000000010028), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0c2afccc6c66a5c2),
        UINT64_C(0x0fa3014d64798618), UINT64_C(0x0fa2f08f807f2686)
    },
    {   UINT64_C(0x0000000000010029), UINT64_C(0x0fa2e88f787fc5fe), UINT64_C(0x0fa2f50fa08a2675),
        UINT64_C(0x089b084f5853e682), UINT64_C(0x0fa2fb4d18782682)
    },
    {   UINT64_C(0x0000000000008a4f), UINT64_C(0x1cf2fcd0a0a9858f), UINT64_C(0x1cf2ff4f2caac58f),
        UINT64_C(0x1cf2ff4dd0a9861b), UINT64_C(0x1cf2ff4dd0a9861b)
    },
    {   UINT64_C(0x0000000000008a50), UINT64_C(0x1cf3050dd0a6c6fc), UINT64_C(0x1cf3050dd0a6c6fc),
        UINT64_C(0x1cf2fbce30ab663e), UINT64_C(0x1cf2fbce30ab663e)
    },
    {   UINT64_C(0x000000000001002c), UINT64_C(0x0c830755b86ee68c), UINT64_C(0x151b038fa0884620),
        UINT64_C(0x0962f653dc67068c), UINT64_C(0x0962f8d3d068e68c)
    },
    {   UINT64_C(0x000000000001002d), UINT64_C(0x0a6b1b175870e680), UINT64_C(0x0fa2e68c5c7ca594),
        UINT64_C(0x117afd0db48c2675), UINT64_C(0x07d2ff92bc55251e)
    },
    {   UINT64_C(0x2e5000000001002e), UINT64_C(0x0a6ae68c3c5543e8), UINT64_C(0x0962f8d3dc62068c),
        UINT64_C(0x09b2fed5f86bc68c), UINT64_C(0x0fa2fbd03476657b)
    },
    {   UINT64_C(0x0000000000008a54), UINT64_C(0x1cf2f790acad8506), UINT64_C(0x0ffaff4d3062867e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x000000000000886f), UINT64_C(0x1ddafb9310af25d6), UINT64_C(0x1ddafb8fd0af2506),
        UINT64_C(0x0ffb258d3075a67e), UINT64_C(0x0ffaff4d3062867e)
    },
    {   UINT64_C(0x0000000000008870), UINT64_C(0x1ddafb9310af25d6), UINT64_C(0x1ddafb8fd0af2506),
        UINT64_C(0x0ffb258d3075a67e), UINT64_C(0x0ffaff4d3062867e)
    },
    {   UINT64_C(0x0000000000008871), UINT64_C(0x1cf2ffce30a9863e), UINT64_C(0x182af5cd309b467e),
        UINT64_C(0x1cf2ffce30a9863e), UINT64_C(0x1cf2ff8e30a9863f)
    },
    {   UINT64_C(0x0000000000008a58), UINT64_C(0x0ffafccd3076c67e), UINT64_C(0x0ffafccd3076c67e),
        UINT64_C(0x1cf2fbcf1cab6603), UINT64_C(0x156afc4d308d267e)
    },
    {   UINT64_C(0x0000000000008a59), UINT64_C(0x1cf2fbcf1cab6603), UINT64_C(0x156afc4d308d267e),
        UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a)
    },
    {   UINT64_C(0x0000000000008a5a), UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a),
        UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a)
    },
    {   UINT64_C(0x0000000000013878), UINT64_C(0x0942fc0f786005b8), UINT64_C(0x0ec2f78fa07f25b8),
        UINT64_C(0x10c2f28f747d4693), UINT64_C(0x118aebcc448185ad)
    },
    {   UINT64_C(0x0000000000008a5c), UINT64_C(0x1cf2fbd180ab656a), UINT64_C(0x1cf2fbd084ab661d),
        UINT64_C(0x1cf2fbd084ab65e4), UINT64_C(0x1cf2fbcf98ab65a9)
    },
    {   UINT64_C(0x0000000000008a5d), UINT64_C(0x1cf2fbcf98ab65a9), UINT64_C(0x1cf2fbcd6cab65a9),
        UINT64_C(0x156afe0b348c2647), UINT64_C(0x1cf2f94f38ab65fc)
    },
    {   UINT64_C(0x0000000000008a5e), UINT64_C(0x156afe0b388c2648), UINT64_C(0x156afe0b888c265b),
        UINT64_C(0x1cf2f94f38ab65fc), UINT64_C(0x156afdcb888c065b)
    },
    {   UINT64_C(0x0000000000008a5f), UINT64_C(0x1cf30fcf8ca165e7), UINT64_C(0x1cf30fcf8ca165e7),
        UINT64_C(0x156b030e7489862d), UINT64_C(0x156b030e7489862d)
    },
    {   UINT64_C(0x0000000000008a60), UINT64_C(0x1293034cac7e269f), UINT64_C(0x11aaf98fd87f65cf),
        UINT64_C(0x0fbb098eb46fa61e), UINT64_C(0x12a3090c4c7a26b7)
    },
    {   UINT64_C(0x0000000000008a61), UINT64_C(0x12a30bcc4c7b86b7), UINT64_C(0x0e7aff8c4c6fa6b7),
        UINT64_C(0x0e7aff8c4c6fa6b7), UINT64_C(0x0ffb0acc486c66b7)
    },
    {   UINT64_C(0x000742001d01003d), UINT64_C(0x0fa2f4cf7479067d), UINT64_C(0x0fa2f4cf747c4658),
        UINT64_C(0x0fa2f4cf7479c646), UINT64_C(0x151b038f748845ed)
    },
    {   UINT64_C(0x000000000001003e), UINT64_C(0x0fa2f50d84792612), UINT64_C(0x0fa2f88f7479e680),
        UINT64_C(0x0fa2f88f7479e680), UINT64_C(0x0fa2f88f7479e67d)
    },
    {   UINT64_C(0x0000000000008a64), UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0c83004c4c6b26b7),
        UINT64_C(0x0c83004c4c6b26b7), UINT64_C(0x156af68d64886671)
    },
    {   UINT64_C(0x0000000000008a65), UINT64_C(0x156b058d648fe671), UINT64_C(0x156b038d648ee671),
        UINT64_C(0x156af88d64896671), UINT64_C(0x0f431d0c4c78c6b7)
    },
    {   UINT64_C(0x0000000000010041), UINT64_C(0x131ae14cfc8cc684), UINT64_C(0x0faaf24cfc80a5a1),
        UINT64_C(0x14dafe8f5894a754), UINT64_C(0x0de2f58f74746680)
    },
    {   UINT64_C(0x0000000000010042), UINT64_C(0x14dafe8f5894a753), UINT64_C(0x0de2f58f7473e67d),
        UINT64_C(0x14dafe8f5894a71a), UINT64_C(0x0de2f58f7473a646)
    },
    {   UINT64_C(0x0000000000010043), UINT64_C(0x14dafe8f5894a753), UINT64_C(0x0de2f58f7479a67d),
        UINT64_C(0x1692ec0fa0960753), UINT64_C(0x130af1cf6c95a69b)
    },
    {   UINT64_C(0x0000000000010044), UINT64_C(0x1692ec0fa0960675), UINT64_C(0x0fa2f1cf6c870693),
        UINT64_C(0x131aedcfa08dc6dd), UINT64_C(0x0de2f5cf7478c606)
    },
    {   UINT64_C(0x0000000000010045), UINT64_C(0x131aedcfa08dc74a), UINT64_C(0x0de2f5cf74796672),
        UINT64_C(0x131aedcfa08dc71a), UINT64_C(0x0de2f5cf74720646)
    },
    {   UINT64_C(0x00000000000061cf), UINT64_C(0x1f42f94f38b3e714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f04b6c6ee), UINT64_C(0x1f42f64facb7e70a)
    },
    {   UINT64_C(0x0000000000010047), UINT64_C(0x1692fb0f5898c755), UINT64_C(0x0fa2f00c6879667d),
        UINT64_C(0x1692fb0f5898c74a), UINT64_C(0x0fa2f00c687a0672)
    },
    {   UINT64_C(0x0000000000010048), UINT64_C(0x1692fb0f5898c71a), UINT64_C(0x0fa2f00c68796646),
        UINT64_C(0x1692fb0b7498c682), UINT64_C(0x0fa2f00c687966bc)
    },
    {   UINT64_C(0x0000000000010049), UINT64_C(0x1692ec0fa09ea753), UINT64_C(0x0fa2f2cf7c7a4753),
        UINT64_C(0x1692ec0fa09ea675), UINT64_C(0x0fa2f2cf7c7a4693)
    },
    {   UINT64_C(0x0000000000006326), UINT64_C(0x1f42f84f74b36721), UINT64_C(0x1f42fd8f08b1a6f1),
        UINT64_C(0x1f43010e9cb1c6f8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006327), UINT64_C(0x1f43000e6cb7e730), UINT64_C(0x1f4315cee8a30707),
        UINT64_C(0x1f431acea0a00738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a71), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x000000000001004d), UINT64_C(0x0de2ec8f58818755), UINT64_C(0x08b2cf0c646aa67d),
        UINT64_C(0x14daefcb7498c675), UINT64_C(0x0de2f18b74782693)
    },
    {   UINT64_C(0x000000000001004e), UINT64_C(0x10f2ef4fa089a5b3), UINT64_C(0x1162ec0fa0846754),
        UINT64_C(0x08b2f84f746a0754), UINT64_C(0x1162ec0b74846675)
    },
    {   UINT64_C(0x0000000000008a74), UINT64_C(0x1acb020f3ca3a5fb), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953054e6098062e), UINT64_C(0x1953054e6098062e)
    },
    {   UINT64_C(0x0000000000008a75), UINT64_C(0x1953058e6098262e), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953008e109a8642), UINT64_C(0x1953008e109a8642)
    },
    {   UINT64_C(0x0000000000008a76), UINT64_C(0x0ffb1a4dd0684654), UINT64_C(0x0ffb1a0dd0682654),
        UINT64_C(0x15831a4dd07e6654), UINT64_C(0x15831a4dd07e6654)
    },
    {   UINT64_C(0x0000000000010052), UINT64_C(0x0fa2f18f7c79e67d), UINT64_C(0x120afc8f7c82069b),
        UINT64_C(0x1692ec0f58960682), UINT64_C(0x0fa2f18c6075c5a1)
    },
    {   UINT64_C(0x0000000000008a78), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x14daf40f808b25e8),
        UINT64_C(0x18b2fe0fa0994586), UINT64_C(0x156af9cd308e267e)
    },
    {   UINT64_C(0x0000000000008a79), UINT64_C(0x156af9cd308e2732), UINT64_C(0x156afa0d308e267e),
        UINT64_C(0x18f2fd0e689ac630), UINT64_C(0x13a2fa0cac87069f)
    },
    {   UINT64_C(0x0000000000008a7a), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x17d2f60bdc99c6d0), UINT64_C(0x18b2f94b1c9ba703)
    },
    {   UINT64_C(0x0000000000008a7b), UINT64_C(0x17d2fa8bdc9786cf), UINT64_C(0x17d2fa8bdc9786cf),
        UINT64_C(0x17d3064bd491a6d3), UINT64_C(0x17d3064bd491a6d3)
    },
    {   UINT64_C(0x0000000000008a7c), UINT64_C(0x17d2f60bd499c6d3), UINT64_C(0x17d2f60bd499c6d3),
        UINT64_C(0x17d2fe4bd495a6d3), UINT64_C(0x17d2fe4bd495a6d3)
    },
    {   UINT64_C(0x0000000000008a7d), UINT64_C(0x1cf2fbcf6cab65f1), UINT64_C(0x105af98c287a46ae),
        UINT64_C(0x1043058e6073c6e5), UINT64_C(0x1043058e6073c67e)
    },
    {   UINT64_C(0x0000000000010059), UINT64_C(0x1162fccfa08da753), UINT64_C(0x08b2f78f7477269d),
        UINT64_C(0x1162fccfa08da675), UINT64_C(0x08b2f50f7463860a)
    },
    {   UINT64_C(0x0000000000008a7f), UINT64_C(0x0e5b014c4c6fc6b1), UINT64_C(0x0e5afe4c4c6e46b1),
        UINT64_C(0x17d3054f10922604), UINT64_C(0x17d3055298922524)
    },
    {   UINT64_C(0x0000000000008a80), UINT64_C(0x2332fb4b94c4a6e3), UINT64_C(0x2332fb4b94c4a6e3),
        UINT64_C(0x2332fb4b94c4a6e3), UINT64_C(0x1ce3058b80a646e8)
    },
    {   UINT64_C(0x000000000001005c), UINT64_C(0x1693078f589e2754), UINT64_C(0x0fa2f88f747e6680),
        UINT64_C(0x1693078cfc9e2675), UINT64_C(0x0fa2f88cfc81c5a1)
    },
    {   UINT64_C(0x000000000001005d), UINT64_C(0x1a0affcf58afc755), UINT64_C(0x14daf1cf588f867d),
        UINT64_C(0x1163018fa08da755), UINT64_C(0x0de2e80c6873c67d)
    },
    {   UINT64_C(0x000000000001005e), UINT64_C(0x1163018fa08da71a), UINT64_C(0x1162ec8fa08a4754),
        UINT64_C(0x0c2aed8e5c6f4680), UINT64_C(0x1162ec8fa08a471a)
    },
    {   UINT64_C(0x000000000001005f), UINT64_C(0x0c2aed8e5c6e0646), UINT64_C(0x1162ec8fa08a4753),
        UINT64_C(0x0c2aed8e5c73c67f), UINT64_C(0x0bfaf14fa07ea693)
    },
    {   UINT64_C(0x0000000000010060), UINT64_C(0x0fa2f3cf7479a693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001118e), UINT64_C(0x15f3110f68a0e5ad), UINT64_C(0x0f12f58fa08285ad),
        UINT64_C(0x0f22e94c408525ad), UINT64_C(0x0dfaf44fa07f25ad)
    },
    {   UINT64_C(0x0000000000010062), UINT64_C(0x0fa2f58f7486a60c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a88), UINT64_C(0x24cb068da4c56661), UINT64_C(0x111b23caf079870d),
        UINT64_C(0x156afa0efc8e071b), UINT64_C(0x156afa0efc8e06ab)
    },
    {   UINT64_C(0x0000000000010064), UINT64_C(0x14dafe8f88926694), UINT64_C(0x0000000000000000),
        UINT64_C(0x0ec2de8d247c46aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010065), UINT64_C(0x0000000000000000), UINT64_C(0x1502f2cf78904693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a8b), UINT64_C(0x156afa0f8c8e25e7), UINT64_C(0x16a2fa8dc492c657),
        UINT64_C(0x16a2fa8dc492c657), UINT64_C(0x0f52fa908c7565a7)
    },
    {   UINT64_C(0x0000000000010067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f18bfc75c5a1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010068), UINT64_C(0x1692fd0f58a0669b), UINT64_C(0x10b2f4cf748765bb),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a8e), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713)
    },
    {   UINT64_C(0x00000000000088a9), UINT64_C(0x156afa0fa08e267e), UINT64_C(0x156afa0fa08e267e),
        UINT64_C(0x0f8afe0fa074a67e), UINT64_C(0x0f92fe0fa074c67e)
    },
    {   UINT64_C(0x0000000000008a90), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860684), UINT64_C(0x1362fa0f24860684)
    },
    {   UINT64_C(0x0000000000008a91), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa094886077a), UINT64_C(0x1362fa094886077a)
    },
    {   UINT64_C(0x0000000000008a92), UINT64_C(0x1362fa094886077a), UINT64_C(0x1362fa094886077a),
        UINT64_C(0x21b2fa0f04bf45f8), UINT64_C(0x21b2fa0f00bf45f7)
    },
    {   UINT64_C(0x0000000000008a93), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f08e1c8ac600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x000000000001006f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f9cf70772603), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010070), UINT64_C(0x0552f1cfa05ec6c8), UINT64_C(0x0912f1cfa06de6c8),
        UINT64_C(0x0aa2f1cfa07426c8), UINT64_C(0x0a6af7cf74648683)
    },
    {   UINT64_C(0x0000000000008a96), UINT64_C(0x1612f3cf2c898600), UINT64_C(0x1362f5ce6c882631),
        UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x0000000000008a97), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362f5cf2c882600),
        UINT64_C(0x1362f5cf2c882704), UINT64_C(0x1362f58d74884600)
    },
    {   UINT64_C(0x0000000000008a98), UINT64_C(0x1362f58b1c884600), UINT64_C(0x1362f94fa08665c7),
        UINT64_C(0x1362f58f2c88470e), UINT64_C(0x1362f58b1c884600)
    },
    {   UINT64_C(0x0000000000008a99), UINT64_C(0x209aff8b30bc86ff), UINT64_C(0x209aff8b30bc86ff),
        UINT64_C(0x156afa0dfc8e256a), UINT64_C(0x156afa10108e26a7)
    },
    {   UINT64_C(0x0000000000008a9a), UINT64_C(0x156afa0cac8e269f), UINT64_C(0x156afa0d308e267e),
        UINT64_C(0x156afa11f08e25f1), UINT64_C(0x156afa0f6c8e25f1)
    },
    {   UINT64_C(0x0000000000008a9b), UINT64_C(0x156afa0f088e2607), UINT64_C(0x156afa0f348e26a7),
        UINT64_C(0x156afa11808e2737), UINT64_C(0x156afa10048e26ab)
    },
    {   UINT64_C(0x0000000000008a9c), UINT64_C(0x156afa0d588e2672), UINT64_C(0x156afa0d6c8e2670),
        UINT64_C(0x156afa0d6c8e2670), UINT64_C(0x156afa0f348e25fc)
    },
    {   UINT64_C(0x0000000000008a9d), UINT64_C(0x1fbafa1104b76589), UINT64_C(0x1f2afa1180b5256a),
        UINT64_C(0x2ce2fa1180ec056a), UINT64_C(0x156afa0df88e264b)
    },
    {   UINT64_C(0x0000000000006355), UINT64_C(0x1f42fe8fd8b3e707), UINT64_C(0x1f430a4f30b6c6f3),
        UINT64_C(0x1f43030f48aaa71b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a9f), UINT64_C(0x156b010f388ea70e), UINT64_C(0x156afc0d7c8c2659),
        UINT64_C(0x156afc0d7c8c2659), UINT64_C(0x156afd0d7c8ca659)
    },
    {   UINT64_C(0x0000000000008aa0), UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afd0d7c8ca659),
        UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afd0d7c8ca6f1)
    },
    {   UINT64_C(0x0000000000008aa1), UINT64_C(0x156afd0d7c8ca6f1), UINT64_C(0x156afa0b548e26d2),
        UINT64_C(0x156afa0b548e26d2), UINT64_C(0x156afd0c408ca65c)
    },
    {   UINT64_C(0x0000000000008aa2), UINT64_C(0x157afc0c408c265c), UINT64_C(0x156afa0acc8e26d2),
        UINT64_C(0x156afa0acc8e26d2), UINT64_C(0x156afc09008c278a)
    },
    {   UINT64_C(0x0000000000008aa3), UINT64_C(0x156afc09008c278a), UINT64_C(0x156afa0ce08e26d2),
        UINT64_C(0x156afa0ce08e26d2), UINT64_C(0x156afd09fc8fa74b)
    },
    {   UINT64_C(0x000000000001007f), UINT64_C(0x1bcae74fa0b06754), UINT64_C(0x14daf3cf748e8680),
        UINT64_C(0x1692fd0dfc95e754), UINT64_C(0x0fa2f50d84792680)
    },
    {   UINT64_C(0x0000000000008aa5), UINT64_C(0x156afd09fc8ca74a), UINT64_C(0x156afc0db88c2668),
        UINT64_C(0x156afc0db88c2668), UINT64_C(0x156afc0db88c2668)
    },
    {   UINT64_C(0x0000000000008aa6), UINT64_C(0x156afc0db88c2668), UINT64_C(0x156afd0c548ca6b6),
        UINT64_C(0x156afd0c548ca6b6), UINT64_C(0x156afd0c308ca6be)
    },
    {   UINT64_C(0x0000000000008aa7), UINT64_C(0x156afd0c308ca6be), UINT64_C(0x156afb4d108ec680),
        UINT64_C(0x156af8cd108d8680), UINT64_C(0x156afa0b748e26ee)
    },
    {   UINT64_C(0x000000000000635f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f78b3e725),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008aa9), UINT64_C(0x1872fd0f3898c5fc), UINT64_C(0x1abafd0f38a1e5fc),
        UINT64_C(0x156af6cf388ca5fc), UINT64_C(0x156afd0f388fc5fc)
    },
    {   UINT64_C(0x0000000000006361), UINT64_C(0x1f42fbcf10af4706), UINT64_C(0x1f42f74f1cb7e721),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008aab), UINT64_C(0x156afd0e048ca64d), UINT64_C(0x156afd0e048ca64d),
        UINT64_C(0x156afa10108e261b), UINT64_C(0x156afe0e048c2649)
    },
    {   UINT64_C(0x0000000000006210), UINT64_C(0x1f42fdd068b606d0), UINT64_C(0x1f430f8ec8b12716),
        UINT64_C(0x1f42fcd058b446e0), UINT64_C(0x1f42f90f18b126da)
    },
    {   UINT64_C(0x0000000000008aad), UINT64_C(0x156afd0c548ca6b5), UINT64_C(0x156afe4b908c46d3),
        UINT64_C(0x156afe4b908c46d3), UINT64_C(0x156afa0b548e26d3)
    },
    {   UINT64_C(0x0000000000008aae), UINT64_C(0x156afa0b548e26d3), UINT64_C(0x156afa0acc8e26d3),
        UINT64_C(0x156afa0acc8e26d3), UINT64_C(0x1d2af44f38b005fc)
    },
    {   UINT64_C(0x0000000000006213), UINT64_C(0x1f42fe4eccb68714), UINT64_C(0x1f42f38efcb70708),
        UINT64_C(0x1f42f68e6cb6a6f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008ab0), UINT64_C(0x156afd0e048ca647), UINT64_C(0x156afd0e048ca647),
        UINT64_C(0x156afd0e048ca647), UINT64_C(0x156afd0e048ca6ad)
    },
    {   UINT64_C(0x0000000000008ab1), UINT64_C(0x156afd0e048ca6ad), UINT64_C(0x156afe0c548c46b5),
        UINT64_C(0x156afe4c548c66b5), UINT64_C(0x156afd0d108ca681)
    },
    {   UINT64_C(0x000000000001008d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x08b2cf0c645d45a1)
    },
    {   UINT64_C(0x0000000000006217), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68ed8b7071a),
        UINT64_C(0x1f42f88f2cb6672a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006218), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84efcb6c719),
        UINT64_C(0x1f42f88eccb76727), UINT64_C(0x1f42fb8edcb5272a)
    },
    {   UINT64_C(0x000000000000636c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84ea8b1e70d),
        UINT64_C(0x1f42f7cea8b5a715), UINT64_C(0x1f4302101cb76703)
    },
    {   UINT64_C(0x000000000000621a), UINT64_C(0x1f43008e9cb6e734), UINT64_C(0x1f42f98ee8b906ef),
        UINT64_C(0x1f42f58ef8b846f3), UINT64_C(0x1f42fb8ec4aaa70f)
    },
    {   UINT64_C(0x0000000000008ab7), UINT64_C(0x1362f0cf78860615), UINT64_C(0x1362fa0f78860615),
        UINT64_C(0x0f8afe0fa074a67e), UINT64_C(0x156afa0fa08e255b)
    },
    {   UINT64_C(0x0000000000008ab8), UINT64_C(0x156afa0fa08e255b), UINT64_C(0x156afa0fa08e267e),
        UINT64_C(0x156afd0fa08ca67e), UINT64_C(0x1ae2f98fa0a2e67e)
    },
    {   UINT64_C(0x0000000000010094), UINT64_C(0x0de2f2cf787325b4), UINT64_C(0x0ffaf24f787b65b4),
        UINT64_C(0x0ffaf24f787b65b4), UINT64_C(0x0fa2f3cf747b8693)
    },
    {   UINT64_C(0x0000000000010095), UINT64_C(0x0de2f58f7473a5a1), UINT64_C(0x0de2ed4d2073a5a1),
        UINT64_C(0x0fa2f1cbfc806693), UINT64_C(0x0fa2f1cf6c9c0693)
    },
    {   UINT64_C(0x0000000000006372), UINT64_C(0x1f42fc8ec4b70715), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f91038b7070c)
    },
    {   UINT64_C(0x0000000000010097), UINT64_C(0x0f02f5cf687665c3), UINT64_C(0x14d2f5cf6891c5c3),
        UINT64_C(0x0f52f58f6877c5c3), UINT64_C(0x0b2ad50c646905a1)
    },
    {   UINT64_C(0x0000000000006374), UINT64_C(0x1f42f61010b806d3), UINT64_C(0x1f4300cf90b38722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f990a0b3470d)
    },
    {   UINT64_C(0x0000000000006222), UINT64_C(0x1f43070ebcb6c738), UINT64_C(0x1f42fe4f74b22706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309ce68b7a725)
    },
    {   UINT64_C(0x000000000001009a), UINT64_C(0x08b2f20f745f8676), UINT64_C(0x0a6afacf785fc5ae),
        UINT64_C(0x07baec0fa063c5a1), UINT64_C(0x08b2ef0f7467e693)
    },
    {   UINT64_C(0x0000000000006377), UINT64_C(0x1f42fe0fd0b306ec), UINT64_C(0x1f42ffd044b306ca),
        UINT64_C(0x1f43044fbcaf06f2), UINT64_C(0x1f42fc0f50b826f6)
    },
    {   UINT64_C(0x0000000000006225), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8fb4b5c6fd),
        UINT64_C(0x1f42fb4f54b0e6f3), UINT64_C(0x1f42f68ea0b5c716)
    },
    {   UINT64_C(0x00000000000088dc), UINT64_C(0x18d2fbcce09ae6d7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000637a), UINT64_C(0x1f42f98ea4b9a73f), UINT64_C(0x1f42fbce80b1a715),
        UINT64_C(0x1f42fecf24b46721), UINT64_C(0x1f4301cebcb1c713)
    },
    {   UINT64_C(0x0000000000006228), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f34b54709),
        UINT64_C(0x1f42fb8ea8b26721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088df), UINT64_C(0x1b3afa8c30a50630), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100a1), UINT64_C(0x0e2ad28bfc88e693), UINT64_C(0x0a9b01cbfc6ae5be),
        UINT64_C(0x0dbade8bd489c693), UINT64_C(0x0a52f78e1c6385b4)
    },
    {   UINT64_C(0x00000000000138ae), UINT64_C(0x116afb0c388686dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x10a2f2cc388f66dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000622c), UINT64_C(0x1f42f50e94b80734), UINT64_C(0x1f42f98f44b6a6f2),
        UINT64_C(0x1f430d0e80af0725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008ac9), UINT64_C(0x25baff8b38cca6f9), UINT64_C(0x185af94e209a462d),
        UINT64_C(0x185af94e209a462d), UINT64_C(0x1102f80dfc7d8649)
    },
    {   UINT64_C(0x00000000000100a5), UINT64_C(0x0fa2fbcfa07e2693), UINT64_C(0x0fa2fbcfa07c6693),
        UINT64_C(0x0c4ae7cf6872267e), UINT64_C(0x0e12f40be877e5a1)
    },
    {   UINT64_C(0x00000000000088e5), UINT64_C(0x1cf2f40da4af4661), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100a7), UINT64_C(0x11e2f44fa09265b8), UINT64_C(0x0c9ada0bfc6fc674),
        UINT64_C(0x0e7af48ba479a59f), UINT64_C(0x0eb2f44fa079a5b8)
    },
    {   UINT64_C(0x00000000000100a8), UINT64_C(0x0f02f44c5c91c62e), UINT64_C(0x0fa2fbcfa07e2693),
        UINT64_C(0x0fa2fbcfa07c6693), UINT64_C(0x173af1cf6c9b2693)
    },
    {   UINT64_C(0x00000000000100a9), UINT64_C(0x173af1cbfca00693), UINT64_C(0x1792f1cee49b2693),
        UINT64_C(0x0fa2f78f747fe60a), UINT64_C(0x102ae60bfc906693)
    },
    {   UINT64_C(0x00000000000100aa), UINT64_C(0x13c2f78f608b060a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008ad0), UINT64_C(0x1cf2fbd084ab661d), UINT64_C(0x1cf2fbd084ab65e4),
        UINT64_C(0x1cf2fbcd6cab65a9), UINT64_C(0x1cf2f94f38ab65fc)
    },
    {   UINT64_C(0x00000000000100ac), UINT64_C(0x0bd2efd4b86f672e), UINT64_C(0x0bd2efd4b872472e),
        UINT64_C(0x0962f912bc6a473b), UINT64_C(0x0a02ee950466869a)
    },
    {   UINT64_C(0x0000000000006389), UINT64_C(0x1f42fa8e8cb5e726), UINT64_C(0x1f42ff0ef0b4c706),
        UINT64_C(0x1f42fdcebcb5470c), UINT64_C(0x1f42f70fb4b806de)
    },
    {   UINT64_C(0x0000000000003c41), UINT64_C(0x106aee134c800511), UINT64_C(0x106b26d72892e637),
        UINT64_C(0x106b0e97286f8637), UINT64_C(0x106b0b171867a64a)
    },
    {   UINT64_C(0x0000000000003c42), UINT64_C(0x106ae60f48844661), UINT64_C(0x106affcfa0770463),
        UINT64_C(0x106b03d2bc750584), UINT64_C(0x106afe176877e64b)
    },
    {   UINT64_C(0x00000000000100b0), UINT64_C(0x0a32f5943c64e69a), UINT64_C(0x0a32f3d43c69669a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c44), UINT64_C(0x0000000000000000), UINT64_C(0x106adb4bfc8986a9),
        UINT64_C(0x106b1e971871664b), UINT64_C(0x106b03d2847504de)
    },
    {   UINT64_C(0x0000000000003c45), UINT64_C(0x106b19d2846a24de), UINT64_C(0x106af8cfa0762597),
        UINT64_C(0x106b018fa07a8597), UINT64_C(0x106b04d15c74a568)
    },
    {   UINT64_C(0x000000000000638f), UINT64_C(0x1f42fa4ee4b28709), UINT64_C(0x1f42fa4fa4aec6fa),
        UINT64_C(0x1f42fa4f90b386de), UINT64_C(0x1f42fa0f7cb7c6c0)
    },
    {   UINT64_C(0x00000000000088f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fa8c1ca986c3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000088f4), UINT64_C(0x0ffb25913462857d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100b6), UINT64_C(0x0a6b0b575072c672), UINT64_C(0x0a6b21d79064a646),
        UINT64_C(0x0a6b14d7906ae6ab), UINT64_C(0x0a6ae90cfc578410)
    },
    {   UINT64_C(0x00000000000100b7), UINT64_C(0x0a6b0717b473e658), UINT64_C(0x0a6b0557587b4680),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100b8), UINT64_C(0x09daf3d308686694), UINT64_C(0x06e2f854d459472d),
        UINT64_C(0x0962f214dc62c69b), UINT64_C(0x0be2ef14d46de69b)
    },
    {   UINT64_C(0x0000000000006395), UINT64_C(0x1f43000e40b00729), UINT64_C(0x1f42fbce7cb2470a),
        UINT64_C(0x1f42f78ef4b8a72b), UINT64_C(0x1f42fb4f3cb7e70a)
    },
    {   UINT64_C(0x0000000000006243), UINT64_C(0x1f42f98e8cb5a704), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044e58b60739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c4e), UINT64_C(0x106aee0c1c8043e8), UINT64_C(0x106aee1720800667),
        UINT64_C(0x106aee17fc80063e), UINT64_C(0x106afc8f7478a5a2)
    },
    {   UINT64_C(0x0000000000003c4f), UINT64_C(0x106afc8f7478a5a2), UINT64_C(0x106aee5720800667),
        UINT64_C(0x106af68fa07bc673), UINT64_C(0x106af5cfa07c0672)
    },
    {   UINT64_C(0x0000000000006399), UINT64_C(0x1f42f78fa4b88706), UINT64_C(0x1f42f78f48b406fe),
        UINT64_C(0x1f42f64e90ada71a), UINT64_C(0x1f42fece48ad6717)
    },
    {   UINT64_C(0x000000000000639a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43060e58b40733),
        UINT64_C(0x1f43000e9cb3c71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c52), UINT64_C(0x106af14c1476a59c), UINT64_C(0x106afd0f8c788651),
        UINT64_C(0x106aec4c1481059c), UINT64_C(0x106ae58f1484840b)
    },
    {   UINT64_C(0x0000000000003c53), UINT64_C(0x106ae58f188484a3), UINT64_C(0x106ae58f1884853c),
        UINT64_C(0x106ae58f188485d5), UINT64_C(0x106ae58f1884866e)
    },
    {   UINT64_C(0x0000000000003c54), UINT64_C(0x106ae58f188484a3), UINT64_C(0x106ae58f1884853c),
        UINT64_C(0x106ae58f188485d5), UINT64_C(0x106ae58f1884866e)
    },
    {   UINT64_C(0x0000000000003c55), UINT64_C(0x106ae58f1484840b), UINT64_C(0x106ae58f188484a3),
        UINT64_C(0x106ae58f1884853c), UINT64_C(0x106ae58f188485d5)
    },
    {   UINT64_C(0x0000000000003c56), UINT64_C(0x106ae58f1884866e), UINT64_C(0x106ae58f188484a3),
        UINT64_C(0x106ae58f1884853c), UINT64_C(0x106ae58f188485d5)
    },
    {   UINT64_C(0x0000000000003c57), UINT64_C(0x106ae58f1884866e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106b05969075264b)
    },
    {   UINT64_C(0x0000000000003c58), UINT64_C(0x106b03d69074464b), UINT64_C(0x106af6d7147ba64b),
        UINT64_C(0x106af6d7147ba64b), UINT64_C(0x106b03d71872a64b)
    },
    {   UINT64_C(0x0000000000003c59), UINT64_C(0x106b08d71875264b), UINT64_C(0x106af58c287ac59c),
        UINT64_C(0x106aee8f8c79864b), UINT64_C(0x106af14c107ae64b)
    },
    {   UINT64_C(0x0000000000003c5a), UINT64_C(0x106af30f807da59f), UINT64_C(0x106af3cf8c7d05b2),
        UINT64_C(0x106af74fa07b459f), UINT64_C(0x106b26cf1863a66e)
    },
    {   UINT64_C(0x00000000000063a4), UINT64_C(0x1f42f68ee8b88739), UINT64_C(0x1f42fc0e8cb5c71b),
        UINT64_C(0x1f42fc0ebcb6a726), UINT64_C(0x1f42fd0ed4b36718)
    },
    {   UINT64_C(0x0000000000008908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f80dc07803f8)
    },
    {   UINT64_C(0x0000000000003c5d), UINT64_C(0x106afccfa078a64b), UINT64_C(0x106af6cf8c7e859c),
        UINT64_C(0x106af60f8c7d859f), UINT64_C(0x106af68f747ba596)
    },
    {   UINT64_C(0x00000000000063a7), UINT64_C(0x1f42fc0ee4b1c71a), UINT64_C(0x1f42fb4ed0b06710),
        UINT64_C(0x1f42fe0e88b52715), UINT64_C(0x1f42fb0ea0b66738)
    },
    {   UINT64_C(0x00000000000063a8), UINT64_C(0x1f4310ce3cad273f), UINT64_C(0x1f42f84e6cb0e700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdce3cb1c704)
    },
    {   UINT64_C(0x00000000000063a9), UINT64_C(0x1f42f7cec0b46728), UINT64_C(0x1f42f9ce9cb52730),
        UINT64_C(0x1f42f80e58b4c70e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c61), UINT64_C(0x106aed0fa080a64b), UINT64_C(0x106af34fa07fa59c),
        UINT64_C(0x0000000000000000), UINT64_C(0x106aee0f74802656)
    },
    {   UINT64_C(0x00000000000063ab), UINT64_C(0x1f42fdced8aae720), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf40b2070d)
    },
    {   UINT64_C(0x0000000000003c63), UINT64_C(0x106afb0ca0796596), UINT64_C(0x106af8cca07a85a2),
        UINT64_C(0x106afc8e9c78c64b), UINT64_C(0x106afc8f8c78c64b)
    },
    {   UINT64_C(0x0000000000003c64), UINT64_C(0x106b0c8f7470c657), UINT64_C(0x106afb0f74798656),
        UINT64_C(0x106af5cfa07b664b), UINT64_C(0x106b060c287b864b)
    },
    {   UINT64_C(0x00000000000063ae), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f58b2e725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063af), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88eb4b3e707),
        UINT64_C(0x1f42f4cf58b80725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c67), UINT64_C(0x106af88fa07aa59f), UINT64_C(0x106afb4fa07b2596),
        UINT64_C(0x106af70fa07b6596), UINT64_C(0x106af84fa078e596)
    },
    {   UINT64_C(0x00000000000063b1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88eb8b6470f),
        UINT64_C(0x1f4303cf04b0a710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c69), UINT64_C(0x106b13d72c6d065d), UINT64_C(0x106b03d71c71c64b),
        UINT64_C(0x106b03572c75466c), UINT64_C(0x106afa4c1063440a)
    },
    {   UINT64_C(0x0000000000003c6a), UINT64_C(0x106b278c1079c40a), UINT64_C(0x106b0b0a6c718369),
        UINT64_C(0x106affca4877036b), UINT64_C(0x106b04d2dc74a4f3)
    },
    {   UINT64_C(0x0000000000003c6b), UINT64_C(0x106b04d15c74a568), UINT64_C(0x106b0b0a6c716368),
        UINT64_C(0x106b0ac9f4716369), UINT64_C(0x106af80afc7b236e)
    },
    {   UINT64_C(0x0000000000003c6c), UINT64_C(0x106aef4a047f43e0), UINT64_C(0x106af0968883a672),
        UINT64_C(0x106b11d72c6dc656), UINT64_C(0x106b12572c6e0656)
    },
    {   UINT64_C(0x0000000000008919), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000000891a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138cb), UINT64_C(0x132af88c388b06dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x129ae24c388f06dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063b9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80eb0ad070b),
        UINT64_C(0x1f42f7cebcb7a70b), UINT64_C(0x1f42facebcb3070b)
    },
    {   UINT64_C(0x000000000000e9e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x180af4cf549a469b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063bb), UINT64_C(0x1f430d8e5cb1871d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6cebcb6c722), UINT64_C(0x1f42fe8e9cb2e70b)
    },
    {   UINT64_C(0x00000000000138cd), UINT64_C(0x0ef2f54c387be6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0dc2f0cc387ac6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063bd), UINT64_C(0x1f42fcce9cb0e70e), UINT64_C(0x1f42f5ceccb3870b),
        UINT64_C(0x1f42f9cf64b4670b), UINT64_C(0x1f42f8cebcb5a70b)
    },
    {   UINT64_C(0x00000000000100e2), UINT64_C(0x13b2efcfa093668e), UINT64_C(0x1722ef0fa0a5268e),
        UINT64_C(0x0af2eecfa074268e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100e3), UINT64_C(0x1692fc8f5895e68e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1222f00fa099268e), UINT64_C(0x17d2ec8fa09ae68e)
    },
    {   UINT64_C(0x00000000000100e4), UINT64_C(0x08b2fa4f746ee671), UINT64_C(0x131ae14fa0850684),
        UINT64_C(0x131aec0fa0880675), UINT64_C(0x131af00fa08f2675)
    },
    {   UINT64_C(0x00000000000100e5), UINT64_C(0x131ae60fa0804684), UINT64_C(0x131aedcfa08dc675),
        UINT64_C(0x1162ec8fa08a4675), UINT64_C(0x1692ec0fa09ea675)
    },
    {   UINT64_C(0x00000000000100e6), UINT64_C(0x1692fd0f5895e683), UINT64_C(0x0a6aec0fa06e8675),
        UINT64_C(0x14daefcfa098c675), UINT64_C(0x131ae14fa0850684)
    },
    {   UINT64_C(0x00000000000100e7), UINT64_C(0x1a0ae98fa0ab8675), UINT64_C(0x14dae90f64996675),
        UINT64_C(0x145aec8fa0938675), UINT64_C(0x1692fd0f5895e683)
    },
    {   UINT64_C(0x00000000000100e8), UINT64_C(0x1692ec0fa09ea675), UINT64_C(0x131aee0fa08a2675),
        UINT64_C(0x0000000000000000), UINT64_C(0x1362ec8fa090e675)
    },
    {   UINT64_C(0x00000000000100e9), UINT64_C(0x1162fccfa08da675), UINT64_C(0x1163018fa08f8684),
        UINT64_C(0x172afa8fa099e675), UINT64_C(0x131ae6cfa0906675)
    },
    {   UINT64_C(0x00000000000100ea), UINT64_C(0x151b014fa09fc683), UINT64_C(0x17d2ec8fa09ae682),
        UINT64_C(0x0a6aec0fa0780740), UINT64_C(0x1163018fa08f8740)
    },
    {   UINT64_C(0x00000000000100eb), UINT64_C(0x1142f4cf74832671), UINT64_C(0x0de2f58f7473a671),
        UINT64_C(0x0ed2f18c6c75c671), UINT64_C(0x08b2fa4f74628671)
    },
    {   UINT64_C(0x00000000000100ec), UINT64_C(0x0ef2f2cf78764671), UINT64_C(0x1142f4cf748325a1),
        UINT64_C(0x0fd2e40c6c7ec68e), UINT64_C(0x0cd2f2cc6875459b)
    },
    {   UINT64_C(0x00000000000063c9), UINT64_C(0x1f42fe8ed0af270b), UINT64_C(0x1f42f9ce9cb5a70b),
        UINT64_C(0x1f43068e6cb2a735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100ee), UINT64_C(0x0f02f4cf747c468e), UINT64_C(0x08b2fa4f745be5a1),
        UINT64_C(0x0de2f18f6c78a5a1), UINT64_C(0x0e52eb0f6074668e)
    },
    {   UINT64_C(0x00000000000100ef), UINT64_C(0x1072e5cc6c7ae594), UINT64_C(0x0eb2f30f5877e5a1),
        UINT64_C(0x0e32f58cbc764693), UINT64_C(0x0fa2f4cf747905a1)
    },
    {   UINT64_C(0x00000000000063cc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78eacb52707), UINT64_C(0x1f42f6ceecb8a71a)
    },
    {   UINT64_C(0x00000000000100f1), UINT64_C(0x0cd2f10f7473c594), UINT64_C(0x0ef2f2cf787645a1),
        UINT64_C(0x1372f4cc6c8845a1), UINT64_C(0x0e4ad30c647cc5a1)
    },
    {   UINT64_C(0x00000000000100f2), UINT64_C(0x1242f1cc6c9205a1), UINT64_C(0x1572f4cf7490459f),
        UINT64_C(0x08b2fa4f746b6646), UINT64_C(0x0ef2f2cf78764646)
    },
    {   UINT64_C(0x00000000000100f3), UINT64_C(0x0fa2f4cf74790671), UINT64_C(0x0ef2f2cf78764671),
        UINT64_C(0x1572f4cf74904671), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100f4), UINT64_C(0x0e42f94f7875069e), UINT64_C(0x1162f2cf7880468e),
        UINT64_C(0x12a3018fa0952684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100f5), UINT64_C(0x0000000000000000), UINT64_C(0x139af4cc6c88e693),
        UINT64_C(0x18c2f24f74a68594), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100f6), UINT64_C(0x1692fd0c6c95e683), UINT64_C(0x0fa2f4cc6c7905a1),
        UINT64_C(0x150afbcc6491a682), UINT64_C(0x0de2f58cbc7ac5b2)
    },
    {   UINT64_C(0x00000000000100f7), UINT64_C(0x116af00fa08f2675), UINT64_C(0x0f3af60ca8798599),
        UINT64_C(0x142af2cf5892e6ed), UINT64_C(0x0e4af5cfa0762693)
    },
    {   UINT64_C(0x00000000000100f8), UINT64_C(0x1622efcc64920682), UINT64_C(0x1083054c58768610),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006282), UINT64_C(0x1f43028ef8b56725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa5000b3a70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006283), UINT64_C(0x1f42f7cfe4b3e70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43014ec8b68720), UINT64_C(0x1f43024f60b026eb)
    },
    {   UINT64_C(0x0000000000006284), UINT64_C(0x1f42faceb8b2a71e), UINT64_C(0x1f42f88ff8b6c702),
        UINT64_C(0x1f42fe5018adc717), UINT64_C(0x1f42fe8eacad871a)
    },
    {   UINT64_C(0x00000000000100fc), UINT64_C(0x10aaea0f6c8465a1), UINT64_C(0x1022f4cc6c7b05a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100fd), UINT64_C(0x1692fd0f5895e683), UINT64_C(0x0de2f58f747305a1),
        UINT64_C(0x0de2f40f747245a1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006287), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4ef4b5071a),
        UINT64_C(0x1f4300cf7cb546d0), UINT64_C(0x1f42fa8f98b986f6)
    },
    {   UINT64_C(0x00000000000063db), UINT64_C(0x1f42ff0f00af2709), UINT64_C(0x1f42fe4fc0b2c722),
        UINT64_C(0x1f43010f48b4c711), UINT64_C(0x1f42ff4f68b34709)
    },
    {   UINT64_C(0x0000000000003c93), UINT64_C(0x106af04fa07b664b), UINT64_C(0x106b09980c724667),
        UINT64_C(0x106b09979c72464b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010101), UINT64_C(0x148afecf58938682), UINT64_C(0x0fa2efcf587cc683),
        UINT64_C(0x0a6aec4fa06e4675), UINT64_C(0x0a6ae64fa074a740)
    },
    {   UINT64_C(0x0000000000010102), UINT64_C(0x0de2e58f58786675), UINT64_C(0x1e0ae54f60af2675),
        UINT64_C(0x1e32e70fa0afc675), UINT64_C(0x1892ff8fa0962675)
    },
    {   UINT64_C(0x0000000000010103), UINT64_C(0x136ae70fa090a77a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1483098f6898075f), UINT64_C(0x1692e7ccd49be675)
    },
    {   UINT64_C(0x0000000000003c97), UINT64_C(0x106af0cfa07b264b), UINT64_C(0x106af6cf8c7e8658),
        UINT64_C(0x106af4cfa07ca732), UINT64_C(0x106af4cfa07ca732)
    },
    {   UINT64_C(0x0000000000010105), UINT64_C(0x147ad44cd4958675), UINT64_C(0x131aee4fa08d6675),
        UINT64_C(0x1de2e04fa0b8067d), UINT64_C(0x11a2f04f60830684)
    },
    {   UINT64_C(0x0000000000003c99), UINT64_C(0x106af48fa07d4732), UINT64_C(0x106af48fa07d470b),
        UINT64_C(0x106b01cfa0764732), UINT64_C(0x106b01cfa0764732)
    },
    {   UINT64_C(0x0000000000003c9a), UINT64_C(0x106afbcf8880067d), UINT64_C(0x106afbcf8880067d),
        UINT64_C(0x106afbcf88800674), UINT64_C(0x106afbcf88800657)
    },
    {   UINT64_C(0x0000000000003c9b), UINT64_C(0x106afbcf88800656), UINT64_C(0x106afbcf8878867d),
        UINT64_C(0x106afbcf8878867d), UINT64_C(0x106afbcf88788657)
    },
    {   UINT64_C(0x0000000000003c9c), UINT64_C(0x106b018fa077667d), UINT64_C(0x106b018fa077667d),
        UINT64_C(0x106afc0f74790732), UINT64_C(0x106afc0f74790732)
    },
    {   UINT64_C(0x0000000000003c9d), UINT64_C(0x106afc0f74790729), UINT64_C(0x106afc0f7479070b),
        UINT64_C(0x106aef0f747f8732), UINT64_C(0x106aef0f747f8732)
    },
    {   UINT64_C(0x0000000000003c9e), UINT64_C(0x106af2cfa07da732), UINT64_C(0x106af2cfa07da732),
        UINT64_C(0x106afc4f8878e67d), UINT64_C(0x106afc4f8878e67d)
    },
    {   UINT64_C(0x0000000000003c9f), UINT64_C(0x106afc4f8878e674), UINT64_C(0x106afc4f8878e657),
        UINT64_C(0x106af10f887e867d), UINT64_C(0x106af10f887e867d)
    },
    {   UINT64_C(0x0000000000003ca0), UINT64_C(0x106af48c107d067d), UINT64_C(0x106af48c107d067d),
        UINT64_C(0x106b244fa064a440), UINT64_C(0x106b1617606c06bf)
    },
    {   UINT64_C(0x000000000001010e), UINT64_C(0x107af54f747c65a1), UINT64_C(0x107af54f747c6683),
        UINT64_C(0x0f5af28f747b25a1), UINT64_C(0x0ed2e30f7075c5a1)
    },
    {   UINT64_C(0x000000000001010f), UINT64_C(0x13cae2cf7089a598), UINT64_C(0x0fc2f30f7c7985a1),
        UINT64_C(0x0f4af54f747945a1), UINT64_C(0x0ffaf2cf7c7a65a1)
    },
    {   UINT64_C(0x0000000000010110), UINT64_C(0x0f1adacc6c7885a1), UINT64_C(0x0dcaf4cf747345a1),
        UINT64_C(0x172af24f7c9725a1), UINT64_C(0x0d2adecc686f25a1)
    },
    {   UINT64_C(0x00000000000063ed), UINT64_C(0x1f42fccf78b586e7), UINT64_C(0x1f43084e8cae8725),
        UINT64_C(0x1f43024fa4b146eb), UINT64_C(0x1f43028f48b146e7)
    },
    {   UINT64_C(0x0000000000010112), UINT64_C(0x188af5cf749ca5a1), UINT64_C(0x188af5ccc89ca5a1),
        UINT64_C(0x11baf10f748085a1), UINT64_C(0x158afa8f7490a5a1)
    },
    {   UINT64_C(0x00000000000063ef), UINT64_C(0x1f4300cf4cb246e9), UINT64_C(0x1f43050f50b026e7),
        UINT64_C(0x1f43040f54b326f7), UINT64_C(0x1f4301cf8cb0a6ee)
    },
    {   UINT64_C(0x0000000000010114), UINT64_C(0x0000000000000000), UINT64_C(0x0dc2f68f7479e646),
        UINT64_C(0x0efaf30c60768693), UINT64_C(0x0c32ee4f74776680)
    },
    {   UINT64_C(0x00000000000063f1), UINT64_C(0x1f43028eecb2a71d), UINT64_C(0x1f4305cf08b3c6db),
        UINT64_C(0x1f4303cf00b3e70d), UINT64_C(0x1f43080f7cb1a6f8)
    },
    {   UINT64_C(0x0000000000003ca9), UINT64_C(0x106af28fa07da64b), UINT64_C(0x106af9cddc7ac64b),
        UINT64_C(0x106b018fa076264b), UINT64_C(0x106af28fa07da64b)
    },
    {   UINT64_C(0x00000000000063f3), UINT64_C(0x1f43024fa4b146ed), UINT64_C(0x1f43000f1caea6fa),
        UINT64_C(0x1f42fe0f10b3a70c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003cab), UINT64_C(0x106af24fa07dc596), UINT64_C(0x106af60e6479e596),
        UINT64_C(0x106af24fa07dc596), UINT64_C(0x106aef0d787f6596)
    },
    {   UINT64_C(0x0000000000008958), UINT64_C(0x1623264ae4988776), UINT64_C(0x1622eb4ae46f4776),
        UINT64_C(0x1622eb4ae47ae776), UINT64_C(0x1622eb4ae47ae776)
    },
    {   UINT64_C(0x000000000001011a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1dcae04fa0ae2675), UINT64_C(0x172aee0f74944598)
    },
    {   UINT64_C(0x000000000000895a), UINT64_C(0x1622eb53cc986776), UINT64_C(0x1622eb5258986776),
        UINT64_C(0x1622eb4ae4986776), UINT64_C(0x1622eb4ae4986776)
    },
    {   UINT64_C(0x000000000000895b), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063f9), UINT64_C(0x1f43014f2cb6e723), UINT64_C(0x1f42fe8f1cb566f6),
        UINT64_C(0x1f42fc8fc4b4e6ee), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063fa), UINT64_C(0x1f42ff0f28b3471b), UINT64_C(0x1f4300cf70ad86eb),
        UINT64_C(0x1f42fd0f2cb6e708), UINT64_C(0x1f43018f20b6e705)
    },
    {   UINT64_C(0x00000000000063fb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014f30b6e6ff),
        UINT64_C(0x1f43058f3caec6ed), UINT64_C(0x1f42f94ed0b60725)
    },
    {   UINT64_C(0x00000000000063fc), UINT64_C(0x1f43048f04b38706), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f78af06f3)
    },
    {   UINT64_C(0x00000000000063fd), UINT64_C(0x1f43008fb8b666d5), UINT64_C(0x1f4300cf28b566e5),
        UINT64_C(0x1f42ffcf48b12719), UINT64_C(0x1f43014f2cb6e6f0)
    },
    {   UINT64_C(0x00000000000063fe), UINT64_C(0x1f43010f2cb6e723), UINT64_C(0x1f43004f28b24703),
        UINT64_C(0x1f43034f78b7e6ea), UINT64_C(0x1f43070f84b20720)
    },
    {   UINT64_C(0x00000000000063ff), UINT64_C(0x1f43020f44b2a71d), UINT64_C(0x1f43024f10b76718),
        UINT64_C(0x1f43010f28b54708), UINT64_C(0x1f43034f40b18722)
    },
    {   UINT64_C(0x0000000000006400), UINT64_C(0x1f43030f30b726f6), UINT64_C(0x1f43014f2cb6e6ff),
        UINT64_C(0x1f43020fa8b24706), UINT64_C(0x1f43014f2cb6e711)
    },
    {   UINT64_C(0x0000000000006401), UINT64_C(0x1f43030f1cb38700), UINT64_C(0x1f43014f1cb4471e),
        UINT64_C(0x1f43014ed0b4670f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006402), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048f3cb22702),
        UINT64_C(0x1f42ff0f48b226df), UINT64_C(0x1f43014f08b44725)
    },
    {   UINT64_C(0x0000000000006403), UINT64_C(0x1f43010f08b3c730), UINT64_C(0x1f4300cef8b36722),
        UINT64_C(0x1f42fc4f78b446ea), UINT64_C(0x1f42fe0ea8b3a717)
    },
    {   UINT64_C(0x0000000000006404), UINT64_C(0x1f42fe4f34b2c6db), UINT64_C(0x1f42fc0ee8b50703),
        UINT64_C(0x1f43014f40afe71f), UINT64_C(0x1f43024f5cb2e73f)
    },
    {   UINT64_C(0x0000000000006405), UINT64_C(0x1f42fecf0cb5a716), UINT64_C(0x1f43024f5cb2e703),
        UINT64_C(0x1f42fc8f5cb126d8), UINT64_C(0x1f4301cf44b5a6f8)
    },
    {   UINT64_C(0x0000000000006406), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014f54b7271e),
        UINT64_C(0x1f43014f0cb58716), UINT64_C(0x1f43010f20b586e5)
    },
    {   UINT64_C(0x000000000000896a), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x0f22f610407705b4), UINT64_C(0x0f22f610407705b4)
    },
    {   UINT64_C(0x0000000000006408), UINT64_C(0x1f43014f2cb6e731), UINT64_C(0x1f43020eacb44720),
        UINT64_C(0x1f42ff8ebcb6c721), UINT64_C(0x1f43024f28b1c709)
    },
    {   UINT64_C(0x0000000000006409), UINT64_C(0x1f43018f10ace6cb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43054f70b1a6dc), UINT64_C(0x1f43008f70b22732)
    },
    {   UINT64_C(0x000000000000896d), UINT64_C(0x14a2f4d0108d8611), UINT64_C(0x14a2f4d0108d8611),
        UINT64_C(0x209aff8b2cbc86fe), UINT64_C(0x209aff8b30bc86ff)
    },
    {   UINT64_C(0x000000000000640b), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf10b0c70e),
        UINT64_C(0x1f42fd8ea0b5072e), UINT64_C(0x1f4305cf08ae6702)
    },
    {   UINT64_C(0x000000000000640c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f28b5a6ed),
        UINT64_C(0x1f43048f2cb68719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000640d), UINT64_C(0x1f4301cf00aee712), UINT64_C(0x1f42fd0ebcb5e721),
        UINT64_C(0x1f43004ee4b2c723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000640e), UINT64_C(0x1f43000ec0b4a726), UINT64_C(0x1f42fecefcb34736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f60b026ee)
    },
    {   UINT64_C(0x000000000000640f), UINT64_C(0x1f43014f14b706fd), UINT64_C(0x1f4302cf4cafa72c),
        UINT64_C(0x1f4301cedcb7672d), UINT64_C(0x1f43010ec4aec6fc)
    },
    {   UINT64_C(0x0000000000006410), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdced8b7a725),
        UINT64_C(0x1f43030f14b62723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006411), UINT64_C(0x1f4301cf60b4c71a), UINT64_C(0x1f42f90efcb5e71a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43010f40b806f8)
    },
    {   UINT64_C(0x0000000000006412), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf60b4c73d),
        UINT64_C(0x1f42fe4f6cb7c713), UINT64_C(0x1f42fb8e9cb2e6f1)
    },
    {   UINT64_C(0x0000000000006413), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0ed4b68713),
        UINT64_C(0x1f43034f24b386fd), UINT64_C(0x1f43044f2cb4e703)
    },
    {   UINT64_C(0x0000000000006414), UINT64_C(0x1f42f74f4cb666f2), UINT64_C(0x1f42fe4ec0b7c73a),
        UINT64_C(0x1f43018f3cb1672e), UINT64_C(0x1f4302cf40b7c727)
    },
    {   UINT64_C(0x0000000000006415), UINT64_C(0x1f43018efcb4c736), UINT64_C(0x1f42fecf0cb8a748),
        UINT64_C(0x1f42fecf0cb8a716), UINT64_C(0x1f4302cefcb10734)
    },
    {   UINT64_C(0x0000000000006416), UINT64_C(0x1f42fa8eb4b4a74e), UINT64_C(0x1f42fbcea0b42728),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f08b3a726)
    },
    {   UINT64_C(0x0000000000006417), UINT64_C(0x1f42fa8ee0b5a720), UINT64_C(0x1f43004f48b526f0),
        UINT64_C(0x1f42fdcf24b78733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006418), UINT64_C(0x1f4302ced4b80748), UINT64_C(0x1f42f9cec0b58720),
        UINT64_C(0x1f42fd8ea4b66705), UINT64_C(0x1f42fe4ea4b72730)
    },
    {   UINT64_C(0x0000000000006419), UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f0cafe71e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018f0cafe713)
    },
    {   UINT64_C(0x000000000000641a), UINT64_C(0x1f43024f10b38706), UINT64_C(0x1f43014ed8b2c729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f00b4a709)
    },
    {   UINT64_C(0x000000000000641b), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cf04b2e73e),
        UINT64_C(0x1f42fd4f24b2e71e), UINT64_C(0x1f42fccf60b106db)
    },
    {   UINT64_C(0x000000000000641c), UINT64_C(0x1f42ff0ed4b16720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb4f1cb40707), UINT64_C(0x1f42fececcb46722)
    },
    {   UINT64_C(0x000000000000641d), UINT64_C(0x1f43020ee4b4a72f), UINT64_C(0x1f43058ee0b24731),
        UINT64_C(0x1f42fd4ee0b6a736), UINT64_C(0x1f43008eb0b6c724)
    },
    {   UINT64_C(0x000000000000641e), UINT64_C(0x1f4300cf14b5271a), UINT64_C(0x1f42fd4ee0b6a71a),
        UINT64_C(0x1f42fe4e98b38730), UINT64_C(0x1f42fd4ee0b6a6f7)
    },
    {   UINT64_C(0x000000000000641f), UINT64_C(0x1f42fecec0b5c731), UINT64_C(0x1f42fb4ea0b4273f),
        UINT64_C(0x1f42fd4ebcb50717), UINT64_C(0x1f42f8cef0b586fe)
    },
    {   UINT64_C(0x0000000000006420), UINT64_C(0x1f42fd8f64b76724), UINT64_C(0x1f43000ef4b18735),
        UINT64_C(0x1f42fe4f4cb7c725), UINT64_C(0x1f42fb0eb4b3672d)
    },
    {   UINT64_C(0x0000000000006421), UINT64_C(0x1f43030ec8b4471b), UINT64_C(0x1f42fc0ec0b28738),
        UINT64_C(0x1f42facec0b226fc), UINT64_C(0x1f43004f18b46713)
    },
    {   UINT64_C(0x0000000000006422), UINT64_C(0x1f42fa0f00b726f3), UINT64_C(0x1f42ff8e70b2a738),
        UINT64_C(0x1f43008e70b32738), UINT64_C(0x1f42fa0e7cb2673a)
    },
    {   UINT64_C(0x0000000000006423), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4e44b3472d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f18b2e6f5)
    },
    {   UINT64_C(0x0000000000006424), UINT64_C(0x1f42ffcebcb34725), UINT64_C(0x1f43220f84b20713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43060f38aba6d0)
    },
    {   UINT64_C(0x0000000000006425), UINT64_C(0x1f43178eacaf26e2), UINT64_C(0x1f43270ec8b0c74c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4e8cace6d6)
    },
    {   UINT64_C(0x0000000000006426), UINT64_C(0x1f42fb8e4cae86de), UINT64_C(0x1f43004e6cacc71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068eacab66aa)
    },
    {   UINT64_C(0x0000000000006427), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8ed8ae46fe),
        UINT64_C(0x1f43018e9cafe727), UINT64_C(0x1f43050e5cad871d)
    },
    {   UINT64_C(0x0000000000006428), UINT64_C(0x1f42fe8e7cb186e2), UINT64_C(0x1f42ffce6cb026fb),
        UINT64_C(0x1f42facef8acc702), UINT64_C(0x1f4301cec8ae6700)
    },
    {   UINT64_C(0x0000000000006429), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018e6cafe709),
        UINT64_C(0x1f43020e4cadc736), UINT64_C(0x1f43000e8caec6c5)
    },
    {   UINT64_C(0x000000000000642a), UINT64_C(0x1f430c0e6cac871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018ed8af86e9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000642b), UINT64_C(0x1f42fe4ec8ac8718), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030ed8af0700), UINT64_C(0x1f42f6ced8ad0713)
    },
    {   UINT64_C(0x000000000000642c), UINT64_C(0x1f43000e6caba721), UINT64_C(0x1f42fd8e0caf4707),
        UINT64_C(0x1f42f8ce6cb026e8), UINT64_C(0x1f42fb0ec8ae26fa)
    },
    {   UINT64_C(0x000000000001110d), UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675),
        UINT64_C(0x14fafa8f7097667b), UINT64_C(0x18b2f78fa09e2675)
    },
    {   UINT64_C(0x000000000000642e), UINT64_C(0x1f430bce5cb3a72c), UINT64_C(0x1f43100eacad2710),
        UINT64_C(0x1f4306cf48af070b), UINT64_C(0x1f42fece9cae2719)
    },
    {   UINT64_C(0x000000000000642f), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301ceaca96734),
        UINT64_C(0x1f43004e7cae4714), UINT64_C(0x1f42fd8eb8af4726)
    },
    {   UINT64_C(0x0000000000006430), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce9caf0724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcce9caf6734)
    },
    {   UINT64_C(0x0000000000006431), UINT64_C(0x1f43018ee8ac071a), UINT64_C(0x1f42fd0ec8a7e6f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffce9ca7e727)
    },
    {   UINT64_C(0x0000000000006432), UINT64_C(0x1f42f94e8caec6e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084e9cad272d), UINT64_C(0x1f4310ce6caf272c)
    },
    {   UINT64_C(0x0000000000006433), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43120e6cac2737), UINT64_C(0x1f43018ed8adc704)
    },
    {   UINT64_C(0x0000000000006434), UINT64_C(0x1f42fd0ee8ac8725), UINT64_C(0x1f43068ee8ae0719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4308cf18ac46f5)
    },
    {   UINT64_C(0x0000000000006435), UINT64_C(0x1f43034e8cb146ea), UINT64_C(0x1f42f94e8cb10717),
        UINT64_C(0x1f43040e7cafc6d6), UINT64_C(0x1f42ff8e7cae272b)
    },
    {   UINT64_C(0x0000000000006436), UINT64_C(0x1f43084f18aae6f6), UINT64_C(0x1f43064e4caf2701),
        UINT64_C(0x1f4301ce7cafc6ee), UINT64_C(0x1f430a8eb8ada721)
    },
    {   UINT64_C(0x0000000000006437), UINT64_C(0x1f43028e7cafc716), UINT64_C(0x1f42f90e6caec725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43020e7cafc708)
    },
    {   UINT64_C(0x0000000000006438), UINT64_C(0x1f4302ce7cafc6eb), UINT64_C(0x1f42ff8eb8b0a6fc),
        UINT64_C(0x1f430b0e8caf2704), UINT64_C(0x1f43060e7cafc727)
    },
    {   UINT64_C(0x0000000000006439), UINT64_C(0x1f42f88e4caac6fc), UINT64_C(0x1f42ff8e4cae26e9),
        UINT64_C(0x1f430b4f28ac26f8), UINT64_C(0x1f430b8ee8acc6ff)
    },
    {   UINT64_C(0x000000000000643a), UINT64_C(0x1f42f90e6cacc725), UINT64_C(0x1f43000e8cb10720),
        UINT64_C(0x1f42ff0eacaf871a), UINT64_C(0x1f42ffce4caec723)
    },
    {   UINT64_C(0x000000000000643b), UINT64_C(0x1f4300ce5cb1072d), UINT64_C(0x1f43064e4cae2727),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8ec8ad271f)
    },
    {   UINT64_C(0x000000000000643c), UINT64_C(0x1f42f3ce5cb186d6), UINT64_C(0x1f43038e8cb2072f),
        UINT64_C(0x1f42fc8ee8af872f), UINT64_C(0x1f43080e2cb16719)
    },
    {   UINT64_C(0x000000000000643d), UINT64_C(0x1f42f9ceacaf6722), UINT64_C(0x1f42fcce5cb0872a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028ea0b14701)
    },
    {   UINT64_C(0x000000000000643e), UINT64_C(0x1f42fd8ed8afe6f9), UINT64_C(0x1f430b4eb8aee6e7),
        UINT64_C(0x1f42ffcef4b7070d), UINT64_C(0x1f430bce6cade6f4)
    },
    {   UINT64_C(0x000000000000643f), UINT64_C(0x1f430bceb8ae66fd), UINT64_C(0x1f42f8ce6caf673c),
        UINT64_C(0x1f43014e9caea705), UINT64_C(0x1f42fb8eacb066ee)
    },
    {   UINT64_C(0x0000000000006440), UINT64_C(0x1f42ffce9caea6fb), UINT64_C(0x1f43044ed8b006eb),
        UINT64_C(0x1f42fd8eacb0a728), UINT64_C(0x1f42fe0ee8af0710)
    },
    {   UINT64_C(0x0000000000006441), UINT64_C(0x1f43084ee8ad66eb), UINT64_C(0x1f43094ec8afc6ff),
        UINT64_C(0x1f43078ed8ae06ea), UINT64_C(0x1f42fc0eacaf46e5)
    },
    {   UINT64_C(0x0000000000006442), UINT64_C(0x1f42ff8eb8aea6c3), UINT64_C(0x1f43060e9cb2871c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43088e8caf2715)
    },
    {   UINT64_C(0x0000000000006443), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302ce6caca6e1),
        UINT64_C(0x1f42fd4e5caee735), UINT64_C(0x1f42f98eb8af4700)
    },
    {   UINT64_C(0x0000000000006444), UINT64_C(0x1f4309ce9cae0711), UINT64_C(0x1f43030eb8ad86ff),
        UINT64_C(0x1f43068eb8b0272b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006445), UINT64_C(0x1f42ff0e9caf2731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074ee8ad46fc), UINT64_C(0x1f43074e7cb0a72d)
    },
    {   UINT64_C(0x00000000000062f3), UINT64_C(0x1f42f850a8a8670a), UINT64_C(0x1f42ff0f4caea70b),
        UINT64_C(0x1f42fb500cad86fd), UINT64_C(0x1f42fb4ff8aba6f2)
    },
    {   UINT64_C(0x0000000000006447), UINT64_C(0x1f42f74e7cb48731), UINT64_C(0x1f43058ed8b2c6ec),
        UINT64_C(0x1f42fd8e2cac6748), UINT64_C(0x1f4302ce4cb0a72c)
    },
    {   UINT64_C(0x0000000000011128), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x182b02cf649b0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000011129), UINT64_C(0x0000000000000000), UINT64_C(0x1912f04cdca1868a),
        UINT64_C(0x1f8af80f64bc0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001112a), UINT64_C(0x0000000000000000), UINT64_C(0x1e52f78f64b60697),
        UINT64_C(0x15a3028f64916690), UINT64_C(0x1c72f78f64ad4697)
    },
    {   UINT64_C(0x000000000001112b), UINT64_C(0x15a3028f08916690), UINT64_C(0x0000000000000000),
        UINT64_C(0x173afecf64962690), UINT64_C(0x1c82f8cf64afe697)
    },
    {   UINT64_C(0x000000000001112c), UINT64_C(0x1a52f70f64a32697), UINT64_C(0x1d22f6cf64b1c697),
        UINT64_C(0x21b2f6cf64c44697), UINT64_C(0x1bdaf70f64aba697)
    },
    {   UINT64_C(0x000000000000644d), UINT64_C(0x1f4302ce5cae0727), UINT64_C(0x1f42fece7cb20701),
        UINT64_C(0x1f42f88e9cb38736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000644e), UINT64_C(0x1f43030e8cb4a72c), UINT64_C(0x1f43030e6caea6dd),
        UINT64_C(0x1f42f58ee8b00732), UINT64_C(0x1f4304ce9cb0c70d)
    },
    {   UINT64_C(0x000000000000644f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000e9cb1a712), UINT64_C(0x1f4303ce7cb1272f)
    },
    {   UINT64_C(0x0000000000006450), UINT64_C(0x1f42f9ce9cafe710), UINT64_C(0x1f43064eb8b2273a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ceb8b066db)
    },
    {   UINT64_C(0x0000000000006451), UINT64_C(0x1f42f84ec8b16730), UINT64_C(0x1f43074e8cb0e738),
        UINT64_C(0x1f4301ce7cb16731), UINT64_C(0x1f42fece9cb14734)
    },
    {   UINT64_C(0x0000000000006452), UINT64_C(0x1f42f6ceb8b1c720), UINT64_C(0x1f43160f7caa06fb),
        UINT64_C(0x1f43040f9cb266e5), UINT64_C(0x1f43054f4cb06739)
    },
    {   UINT64_C(0x0000000000011133), UINT64_C(0x1bc2f58f74a905a1), UINT64_C(0x1782fe4f7492e5a1),
        UINT64_C(0x1492f58c348ba590), UINT64_C(0x159afb0f748ba5c6)
    },
    {   UINT64_C(0x0000000000003c4b), UINT64_C(0x106af08c147ec596), UINT64_C(0x106af0c9fc7b859c),
        UINT64_C(0x106af24f747da656), UINT64_C(0x106af48d70798643)
    },
    {   UINT64_C(0x0000000000006455), UINT64_C(0x1f42fd8f94b3c705), UINT64_C(0x1f43041008b146d9),
        UINT64_C(0x1f4308cfc4b446ea), UINT64_C(0x1f43001088b1c6c4)
    },
    {   UINT64_C(0x0000000000006456), UINT64_C(0x1f4308cfe8b4a6db), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4302d02cb7c6fd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006457), UINT64_C(0x0000000000000000), UINT64_C(0x1f430650e8ae46d5),
        UINT64_C(0x1f42ff9074b6c6eb), UINT64_C(0x1f430acf3cad4701)
    },
    {   UINT64_C(0x0000000000006458), UINT64_C(0x1f4307cee0b386c2), UINT64_C(0x1f43000fb0b0a6f2),
        UINT64_C(0x1f4305cf08ae6723), UINT64_C(0x1f4305cfe0b646dc)
    },
    {   UINT64_C(0x0000000000006459), UINT64_C(0x1f4308d01cb3070c), UINT64_C(0x1f43054f98b72700),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003c51), UINT64_C(0x106b174fa073a50e), UINT64_C(0x106b06952873a64b),
        UINT64_C(0x106b27953879c632), UINT64_C(0x106aec8c287a2651)
    },
    {   UINT64_C(0x000000000000645b), UINT64_C(0x1f4307ceecae66fb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43068fc8adc6cf), UINT64_C(0x1f43090fccaa46fd)
    },
    {   UINT64_C(0x000000000000645c), UINT64_C(0x1f43008fc4b566f6), UINT64_C(0x1f430a8e70af4714),
        UINT64_C(0x1f430610c4b0e6d6), UINT64_C(0x1f430a4ee8a8870e)
    },
    {   UINT64_C(0x000000000000645d), UINT64_C(0x1f42fc0f04b726fa), UINT64_C(0x1f4300ced8b38738),
        UINT64_C(0x1f43080f50b4270b), UINT64_C(0x1f43035008b2e701)
    },
    {   UINT64_C(0x000000000000645e), UINT64_C(0x1f4305503cb84737), UINT64_C(0x1f4307cf8cb0670d),
        UINT64_C(0x1f4308cfdcab8714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000645f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43020f80a986d9),
        UINT64_C(0x1f4305106cad66d8), UINT64_C(0x1f4303cf28b78717)
    },
    {   UINT64_C(0x0000000000006460), UINT64_C(0x1f4304cf78b366f2), UINT64_C(0x1f4300cef0b546fd),
        UINT64_C(0x1f42fd0eacb926f3), UINT64_C(0x1f42fc0f8cb7e711)
    },
    {   UINT64_C(0x0000000000003d18), UINT64_C(0x106af4cfa07ca70f), UINT64_C(0x106af48fa07d4701),
        UINT64_C(0x106af28fa07da701), UINT64_C(0x106afc0f7479070f)
    },
    {   UINT64_C(0x0000000000003d19), UINT64_C(0x106af44f747d4701), UINT64_C(0x106af0cfa07e8701),
        UINT64_C(0x106af7cf747b0701), UINT64_C(0x106aee0f747f2701)
    },
    {   UINT64_C(0x0000000000003d1a), UINT64_C(0x106af54fa07dc701), UINT64_C(0x0000000000000000),
        UINT64_C(0x106afbcf88800667), UINT64_C(0x106afbcf88788667)
    },
    {   UINT64_C(0x0000000000003d1b), UINT64_C(0x106af24fa07dc655), UINT64_C(0x106afc4f8878e667),
        UINT64_C(0x106af48c107d0655), UINT64_C(0x106aefcfa07f2655)
    },
    {   UINT64_C(0x0000000000003d1c), UINT64_C(0x106afc8f8c76e656), UINT64_C(0x106aee8f8c7fc656),
        UINT64_C(0x106af20fa07e8655), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006466), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cf40b706f7),
        UINT64_C(0x1f42fb0f84b706be), UINT64_C(0x1f42ffd080b7470c)
    },
    {   UINT64_C(0x0000000000003c5e), UINT64_C(0x106b000c1078a59f), UINT64_C(0x106aee0c0c78c654),
        UINT64_C(0x106afb0c0c77259f), UINT64_C(0x106af30f8c80264b)
    },
    {   UINT64_C(0x0000000000006468), UINT64_C(0x1f43040facb226f0), UINT64_C(0x1f43054eacb24719),
        UINT64_C(0x1f42ff1084b3a713), UINT64_C(0x1f4300cf8cb3e70a)
    },
    {   UINT64_C(0x0000000000006469), UINT64_C(0x1f43038df0b0c746), UINT64_C(0x1f4302cf68ac86ee),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf20b406a4)
    },
    {   UINT64_C(0x000000000000646a), UINT64_C(0x1f42ffcf4cb02721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cec0b7a701), UINT64_C(0x1f43068e90b0c72b)
    },
    {   UINT64_C(0x000000000000646b), UINT64_C(0x1f43098e34b1473e), UINT64_C(0x1f4302cf54b24728),
        UINT64_C(0x1f43024fa4b1a704), UINT64_C(0x1f4301cfd8b36718)
    },
    {   UINT64_C(0x000000000000646c), UINT64_C(0x1f43024e8cb14701), UINT64_C(0x1f42fe8facb50725),
        UINT64_C(0x1f43040e64b7e73a), UINT64_C(0x1f43078e9cae66f9)
    },
    {   UINT64_C(0x000000000000646d), UINT64_C(0x1f42ff0f84b0a71f), UINT64_C(0x1f42fe0e5cb90733),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf7cb42727)
    },
    {   UINT64_C(0x000000000000646e), UINT64_C(0x1f42f7ce70b6873f), UINT64_C(0x1f43044f04b3a6f9),
        UINT64_C(0x1f42fc8f28b3e728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000646f), UINT64_C(0x1f43004f90b3e6f7), UINT64_C(0x1f42ff1094b586ee),
        UINT64_C(0x1f43004eccb54739), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006470), UINT64_C(0x1f42fe0e70b6a73f), UINT64_C(0x1f430d0eb0b4e720),
        UINT64_C(0x1f4300ce94b6c72c), UINT64_C(0x1f42ffcea0b72711)
    },
    {   UINT64_C(0x0000000000003c68), UINT64_C(0x106afd4e4078659f), UINT64_C(0x106af84fa07c859f),
        UINT64_C(0x106aee17b8800675), UINT64_C(0x106b26d72c63a65d)
    },
    {   UINT64_C(0x0000000000006472), UINT64_C(0x1f42f3cf2cb886fd), UINT64_C(0x1f42f9cf2cafa6fd),
        UINT64_C(0x1f42facef8b66703), UINT64_C(0x1f43000eacb16718)
    },
    {   UINT64_C(0x0000000000006473), UINT64_C(0x1f43079004b366e3), UINT64_C(0x1f430b8f48b1e728),
        UINT64_C(0x1f42fe8f0cb3a6fc), UINT64_C(0x1f43058f14b1e724)
    },
    {   UINT64_C(0x0000000000006474), UINT64_C(0x1f42fdced4ac472b), UINT64_C(0x1f42fa0f44b5c71e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf8cb2270b)
    },
    {   UINT64_C(0x0000000000006475), UINT64_C(0x1f42fa0fa4b80708), UINT64_C(0x1f4300ceecaa0703),
        UINT64_C(0x1f4301904cb1a703), UINT64_C(0x1f42fdcec4a6072f)
    },
    {   UINT64_C(0x0000000000006476), UINT64_C(0x1f42ff0ec8b54714), UINT64_C(0x1f42f50eecaee713),
        UINT64_C(0x1f42fecf44b2e703), UINT64_C(0x1f43038f70b0e713)
    },
    {   UINT64_C(0x0000000000006477), UINT64_C(0x1f43030fa0b6c70d), UINT64_C(0x1f4300cef8b5e724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f45074b866f1)
    },
    {   UINT64_C(0x0000000000006478), UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cfa0b2470d),
        UINT64_C(0x1f42fecf54aca708), UINT64_C(0x1f42f90f04ae6732)
    },
    {   UINT64_C(0x0000000000006479), UINT64_C(0x1f4302cf4ca8a712), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0f84af26f6), UINT64_C(0x1f42f90eccaca716)
    },
    {   UINT64_C(0x000000000000647a), UINT64_C(0x1f42fd0eccb7c6ef), UINT64_C(0x1f42f90f7cb58709),
        UINT64_C(0x1f42fc0f24b3e708), UINT64_C(0x1f42fdcf0cadc70b)
    },
    {   UINT64_C(0x000000000000647b), UINT64_C(0x1f42fc0e7cb4c708), UINT64_C(0x1f42fccfecb4470e),
        UINT64_C(0x1f42ff0ec0b1670c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000647c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0ff0b92718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f98b446fa)
    },
    {   UINT64_C(0x000000000000647d), UINT64_C(0x1f42fc0f40b08703), UINT64_C(0x1f42f9cfe8b886f6),
        UINT64_C(0x1f42f0cffcba2720), UINT64_C(0x1f42fb104cb62718)
    },
    {   UINT64_C(0x000000000000647e), UINT64_C(0x1f42fecf7cb0a70d), UINT64_C(0x1f42fd0fa8afe700),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000647f), UINT64_C(0x1f42f44ff0b846ed), UINT64_C(0x1f42f90eccb7a6ec),
        UINT64_C(0x1f43000f08b286f8), UINT64_C(0x1f42fb0e5cb186f8)
    },
    {   UINT64_C(0x0000000000006480), UINT64_C(0x1f42fb0fc4b446e4), UINT64_C(0x1f42fb0f60ad2710),
        UINT64_C(0x1f42fd0f68b5a6fb), UINT64_C(0x1f42f94f54b3c6fc)
    },
    {   UINT64_C(0x0000000000006481), UINT64_C(0x1f42fd0e7cb286fa), UINT64_C(0x1f42fb0f18a6a700),
        UINT64_C(0x1f42fc0fd4b806f3), UINT64_C(0x1f42fa0f84b0c6ff)
    },
    {   UINT64_C(0x0000000000006482), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf48abc712),
        UINT64_C(0x1f42fb0fdcb546ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006483), UINT64_C(0x1f42fa0fb8b446e3), UINT64_C(0x1f4300cf9cb4c714),
        UINT64_C(0x1f42fd0f60b3e729), UINT64_C(0x1f4301cee8ab66f7)
    },
    {   UINT64_C(0x0000000000006484), UINT64_C(0x1f42f80f8cb90710), UINT64_C(0x1f42f60fa4b5670e),
        UINT64_C(0x1f42f84fb4b6a6f8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006485), UINT64_C(0x1f42fc4e7cb46720), UINT64_C(0x1f42fcd024b3a710),
        UINT64_C(0x1f42fb0f48b726ec), UINT64_C(0x1f42fd0f94b6a6e8)
    },
    {   UINT64_C(0x0000000000006486), UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cf9cb6a6f6),
        UINT64_C(0x1f42fd0fccb686f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006487), UINT64_C(0x1f42fd0fe0ac86f8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f08af2724), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006488), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ebcaf8707)
    },
    {   UINT64_C(0x0000000000006489), UINT64_C(0x1f42fd1004a866ec), UINT64_C(0x1f42fd0fd8b2c6f7),
        UINT64_C(0x1f42fc102cb266fc), UINT64_C(0x1f42fdcfb4b706e8)
    },
    {   UINT64_C(0x000000000000648a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0e84b30712),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000ca1900003d42), UINT64_C(0x106b0e4fa06fc5e8), UINT64_C(0x106af40fa07aa5e8),
        UINT64_C(0x106af6cfa07b85f5), UINT64_C(0x106af40fa07d05e8)
    },
    {   UINT64_C(0x000000000000648c), UINT64_C(0x1f42fdce4cb306fe), UINT64_C(0x1f42fb0fb8b7c6ea),
        UINT64_C(0x1f42fdd02cb886f8), UINT64_C(0x1f42fb0ec4b2a6fb)
    },
    {   UINT64_C(0x000000000000648d), UINT64_C(0x1f42f50f8cb7270a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4304d004ae46f5), UINT64_C(0x1f42fdcfd8b406ef)
    },
    {   UINT64_C(0x0710001c03683d45), UINT64_C(0x106af54fa07c65f3), UINT64_C(0x106af54fa07c65e8),
        UINT64_C(0x106af58fa07c05e8), UINT64_C(0x106af54fa07c65e8)
    },
    {   UINT64_C(0x0000e40000003d46), UINT64_C(0x106af4cfa07c85f3), UINT64_C(0x106afd0fa078869e),
        UINT64_C(0x106af54fa07c669e), UINT64_C(0x106af30df47c45f5)
    },
    {   UINT64_C(0x0000000000006490), UINT64_C(0x1f42f90eb8b526e4), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006491), UINT64_C(0x1f42f95034b5e6fe), UINT64_C(0x1f42fdd01cb22717),
        UINT64_C(0x1f42ffd018adc701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006492), UINT64_C(0x1f42fc0fd8af0705), UINT64_C(0x1f42f7cfa0af070a),
        UINT64_C(0x1f42ffcfe8ac6701), UINT64_C(0x1f42facfd8b76711)
    },
    {   UINT64_C(0x00000000000089f6), UINT64_C(0x1b12fa8c4ca486b6), UINT64_C(0x1b12fa8c4ca486b6),
        UINT64_C(0x2b02fe0fa0e2867e), UINT64_C(0x2b02fe0fa0e2867e)
    },
    {   UINT64_C(0x0000000000006494), UINT64_C(0x1f42fc0ff8adc706), UINT64_C(0x1f42fa1070a84716),
        UINT64_C(0x1f42fd0fccb266ee), UINT64_C(0x1f42fc0e8ca866f7)
    },
    {   UINT64_C(0x0000000000006495), UINT64_C(0x1f42fb0f74afa703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0f4cb76700), UINT64_C(0x1f42f80fe4b5e706)
    },
    {   UINT64_C(0x0000000000006496), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0e9cb00710), UINT64_C(0x1f42f80f8cb4e6f1)
    },
    {   UINT64_C(0x0000000000006497), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0f44b3070f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006498), UINT64_C(0x1f42fd1014b646fe), UINT64_C(0x1f42fdcea8b4e703),
        UINT64_C(0x1f42f5ce54b7871f), UINT64_C(0x1f42fc1028af06fe)
    },
    {   UINT64_C(0x0000000000006499), UINT64_C(0x1f42fcced8b0c712), UINT64_C(0x1f42ffce8cb786f3),
        UINT64_C(0x1f42ffcfd4b2e6fd), UINT64_C(0x1f42fa0f38b746f1)
    },
    {   UINT64_C(0x00000000000089fd), UINT64_C(0x2582fb0eaccde621), UINT64_C(0x3132fbd088fc65a9),
        UINT64_C(0x3132fbd084fc65a9), UINT64_C(0x3132fbd084fc65a9)
    },
    {   UINT64_C(0x000000000000649b), UINT64_C(0x1f42fcd008a946fd), UINT64_C(0x1f42ffd054b04701),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdcfacb4e6e9)
    },
    {   UINT64_C(0x000000000000649c), UINT64_C(0x1f42fecef8ad470a), UINT64_C(0x1f42f88f8cb646fc),
        UINT64_C(0x1f4301cebcb066f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000649d), UINT64_C(0x1f43040f54b5872e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f50adc700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000649e), UINT64_C(0x1f42fd0facb6270c), UINT64_C(0x1f42fb0e84af2704),
        UINT64_C(0x1f42fc0e6cb2671b), UINT64_C(0x1f42f70f5cb78708)
    },
    {   UINT64_C(0x000000000000649f), UINT64_C(0x1f42fa8e5cb74701), UINT64_C(0x1f42f80f44b2a706),
        UINT64_C(0x1f42fc0ec8b0070d), UINT64_C(0x1f42f80f6cb72706)
    },
    {   UINT64_C(0x00000000000064a0), UINT64_C(0x1f42fa0f30b4e71e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f08aec706), UINT64_C(0x1f42ffcf8cb64719)
    },
    {   UINT64_C(0x0000000000003c98), UINT64_C(0x106af4cfa07ca729), UINT64_C(0x106af4cfa07ca70b),
        UINT64_C(0x106af4cfa07ca70b), UINT64_C(0x106af48fa07d4732)
    },
    {   UINT64_C(0x00000000000064a2), UINT64_C(0x1f42f8103cb4c706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70f00b6a702), UINT64_C(0x1f42f71010afa6f1)
    },
    {   UINT64_C(0x00000000000064a3), UINT64_C(0x1f42fb0f58b48700), UINT64_C(0x1f42f90fccb446f7),
        UINT64_C(0x1f430a4ef0b56712), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064a4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f8cb78705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbd04cb7070b)
    },
    {   UINT64_C(0x00000000000064a5), UINT64_C(0x1f42ff0edcb266fc), UINT64_C(0x1f42f70facb2471f),
        UINT64_C(0x1f42f90eacb6a72a), UINT64_C(0x1f42f80fa4aee706)
    },
    {   UINT64_C(0x00000000000064a6), UINT64_C(0x1f42f9cf88aea724), UINT64_C(0x1f42fc0ef4b5c705),
        UINT64_C(0x1f42fdd02cb9670c), UINT64_C(0x1f42fa0fa8b4070f)
    },
    {   UINT64_C(0x0000000000011187), UINT64_C(0x14ab0b4f689d267e), UINT64_C(0x14731f0f74a1067e),
        UINT64_C(0x1ccb1e8f74c2a67e), UINT64_C(0x15e2f5cfa0a3467e)
    },
    {   UINT64_C(0x00000000000064a8), UINT64_C(0x1f42ff0f7cb1471b), UINT64_C(0x1f42fccf78b04705),
        UINT64_C(0x1f42f80f94b7a706), UINT64_C(0x1f42f61088b366f5)
    },
    {   UINT64_C(0x00000000000064a9), UINT64_C(0x1f42fd0f28b72709), UINT64_C(0x1f42fa0f70b68706),
        UINT64_C(0x1f42fb0fa8b1470a), UINT64_C(0x1f42f80f7cb50706)
    },
    {   UINT64_C(0x000000000001118a), UINT64_C(0x0f82ec4c408665b7), UINT64_C(0x0f3afdcfa07e4694),
        UINT64_C(0x06e2ff4fa067268f), UINT64_C(0x07f2d08c406ac68f)
    },
    {   UINT64_C(0x00000000000064ab), UINT64_C(0x1f42f80e80b5872e), UINT64_C(0x1f42fe4f30b36704),
        UINT64_C(0x1f43054f10b30724), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064ac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60e3cb5472f), UINT64_C(0x1f42f8cfb4b60717)
    },
    {   UINT64_C(0x00000000000064ad), UINT64_C(0x1f430e0efcb2e721), UINT64_C(0x1f42faceb4b1a711),
        UINT64_C(0x1f42fb0ef4b646e3), UINT64_C(0x1f42facf1cb1272c)
    },
    {   UINT64_C(0x00000000000064ae), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f38b566f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80e7cb46734)
    },
    {   UINT64_C(0x00000000000064af), UINT64_C(0x1f42f8100cb346ed), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064b0), UINT64_C(0x1f42fed000b1e700), UINT64_C(0x1f42fb1058ae270f),
        UINT64_C(0x1f42fa0f20b2a70d), UINT64_C(0x1f42ffcedcb6a706)
    },
    {   UINT64_C(0x00000000000064b1), UINT64_C(0x1f42fecf48b266f7), UINT64_C(0x1f42f90e4cb2e719),
        UINT64_C(0x1f43074f2cb5a71f), UINT64_C(0x1f42f9cf3cb026f6)
    },
    {   UINT64_C(0x00000000000064b2), UINT64_C(0x1f43080f3cb3e70f), UINT64_C(0x1f42fd0e7cb64706),
        UINT64_C(0x1f42fb0f4cb726fb), UINT64_C(0x1f42f80f58b846fe)
    },
    {   UINT64_C(0x00000000000064b3), UINT64_C(0x1f42f90fc0b466fa), UINT64_C(0x1f42fb0ec0b16709),
        UINT64_C(0x1f42fc0f88b5e6fb), UINT64_C(0x1f42fd0fecb2c6f8)
    },
    {   UINT64_C(0x00000000000064b4), UINT64_C(0x1f42fecf9cafc6f7), UINT64_C(0x1f42fd0f9cb40701),
        UINT64_C(0x1f42face5cbac734), UINT64_C(0x1f42fdcf50ac26f6)
    },
    {   UINT64_C(0x00000000000064b5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecfbcafc702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ef4a7e6fe)
    },
    {   UINT64_C(0x00000000000064b6), UINT64_C(0x1f42fdd028b166f6), UINT64_C(0x1f42fecf6cb2e6ff),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064b7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f0cb26702),
        UINT64_C(0x1f42fc0fb8afc705), UINT64_C(0x1f42fc0f40b066f7)
    },
    {   UINT64_C(0x00000000000064b8), UINT64_C(0x1f42fc0fa4b786fe), UINT64_C(0x1f42f90f78b98711),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064b9), UINT64_C(0x1f42f70fdcb2a715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ee4af46fe), UINT64_C(0x1f42ffcf44b726fe)
    },
    {   UINT64_C(0x00000000000064ba), UINT64_C(0x1f42fc0f80b0a702), UINT64_C(0x1f42f90f9caf06fa),
        UINT64_C(0x1f42f70f40af870a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064bb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cfb4b60717),
        UINT64_C(0x1f42fc0f28b3a706), UINT64_C(0x1f42fecfb8b28700)
    },
    {   UINT64_C(0x00000000000064bc), UINT64_C(0x1f42f91088b3c6fd), UINT64_C(0x1f42f90f7cb126fe),
        UINT64_C(0x1f42fa0f7cb58708), UINT64_C(0x1f42f54e8cb7e703)
    },
    {   UINT64_C(0x00000000000064bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0edcb826e1)
    },
    {   UINT64_C(0x00000000000064be), UINT64_C(0x1f42f80f34b16706), UINT64_C(0x1f42f54f18b766f0),
        UINT64_C(0x1f42f90fccb626fa), UINT64_C(0x1f43080ee8b366ea)
    },
    {   UINT64_C(0x00000000000064bf), UINT64_C(0x1f42f84f50b0c726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0ee4adc702), UINT64_C(0x1f42fb1008b226f1)
    },
    {   UINT64_C(0x00000000000064c0), UINT64_C(0x1f42fd0fccb7a702), UINT64_C(0x1f42f70e68b82716),
        UINT64_C(0x1f42f64facb54719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064c1), UINT64_C(0x1f42f50ed8b6a734), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70ee0b3a729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000013858), UINT64_C(0x0e7af58fa093c675), UINT64_C(0x1442f64f68980675),
        UINT64_C(0x0f6300cfa0934675), UINT64_C(0x1953070fa0b6e675)
    },
    {   UINT64_C(0x0000000000013859), UINT64_C(0x1452e80fa09ec675), UINT64_C(0x0e52f88fa090c675),
        UINT64_C(0x11b2f80f747fa5a1), UINT64_C(0x12b2f4cf74844693)
    },
    {   UINT64_C(0x00000000000064c4), UINT64_C(0x1f42f9cf28b3e715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60f00b986e0), UINT64_C(0x1f42f64efcb7070b)
    },
    {   UINT64_C(0x00000000000064c5), UINT64_C(0x1f42f90ee4b786f9), UINT64_C(0x1f42f9101cb7e71c),
        UINT64_C(0x1f430e8eb8b5871c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064c6), UINT64_C(0x1f42fa1004b6270c), UINT64_C(0x1f42f74fbcae8707),
        UINT64_C(0x1f42fa0e94b4c729), UINT64_C(0x1f42fecf18b726ff)
    },
    {   UINT64_C(0x00000000000064c7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64ef4b2a6fc),
        UINT64_C(0x1f42f90eb8b4672c), UINT64_C(0x1f42f94f34b546fd)
    },
    {   UINT64_C(0x000000000001385e), UINT64_C(0x13fafc0fa094c594), UINT64_C(0x116af18fa083c594),
        UINT64_C(0x0ebaef4c3881c594), UINT64_C(0x10e2f34fa080c594)
    },
    {   UINT64_C(0x00000000000064c9), UINT64_C(0x1f42f64f30b026fc), UINT64_C(0x1f42f64ed0aba6fc),
        UINT64_C(0x1f42fbce7cad472d), UINT64_C(0x1f42f84edcb5e6ee)
    },
    {   UINT64_C(0x00000000000064ca), UINT64_C(0x1f42f4cf90b16702), UINT64_C(0x1f42f60f90afa70c),
        UINT64_C(0x1f42f80f80b44718), UINT64_C(0x1f42fb0ef8b0e6f6)
    },
    {   UINT64_C(0x00000000000064cb), UINT64_C(0x1f42f84ef8b6672e), UINT64_C(0x1f42f94f60aee703),
        UINT64_C(0x1f42f81028ae6711), UINT64_C(0x1f42fd0f70b4672a)
    },
    {   UINT64_C(0x00000000000064cc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f14f28ba06f2), UINT64_C(0x1f42f7502cb426fb)
    },
    {   UINT64_C(0x00000000000064cd), UINT64_C(0x1f42f94fecb00703), UINT64_C(0x1f42f64fb4b7e702),
        UINT64_C(0x1f42f90f34b7c738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000637b), UINT64_C(0x1f42fc8fdcb446f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70fdcb3e6ba), UINT64_C(0x1f42f98f3cb86718)
    },
    {   UINT64_C(0x000000000000637c), UINT64_C(0x1f42fecf28ae26fc), UINT64_C(0x1f42f70e8cb50725),
        UINT64_C(0x1f42facf1cb9a6d2), UINT64_C(0x1f42fed068b6c6f7)
    },
    {   UINT64_C(0x000000000000637d), UINT64_C(0x1f42fb4fecb626b8), UINT64_C(0x1f4301cefcb20717),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8fd4b56706)
    },
    {   UINT64_C(0x000000000000637e), UINT64_C(0x1f4304ceb8b2a6ff), UINT64_C(0x1f42ff4fb8b546cf),
        UINT64_C(0x1f42fececcb726e7), UINT64_C(0x1f43018ef8b566fb)
    },
    {   UINT64_C(0x00000000000064d2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f78b406f3),
        UINT64_C(0x1f42f90ef8b6a72e), UINT64_C(0x1f42fd0efcb206f9)
    },
    {   UINT64_C(0x00000000000064d3), UINT64_C(0x1f430f4ec0b00726), UINT64_C(0x1f42f90f54b58722),
        UINT64_C(0x1f42fd0f10b106db), UINT64_C(0x1f42fd0e2cb26710)
    },
    {   UINT64_C(0x0000000000006381), UINT64_C(0x1f42f54e70af6718), UINT64_C(0x1f430dcea0a6a70d),
        UINT64_C(0x1f42fc0f1cb8071a), UINT64_C(0x1f42fbcf00b4671f)
    },
    {   UINT64_C(0x00000000000064d5), UINT64_C(0x1f42feceecb42707), UINT64_C(0x1f42fa0e3cb486df),
        UINT64_C(0x1f42f70f14b4e6fb), UINT64_C(0x1f42fd0e40b64706)
    },
    {   UINT64_C(0x00000000000064d6), UINT64_C(0x1f42fa0f8cac66fa), UINT64_C(0x1f43114ea0b2873a),
        UINT64_C(0x1f42f70ea4b26706), UINT64_C(0x1f42f50f48b9c721)
    },
    {   UINT64_C(0x00000000000064d7), UINT64_C(0x1f42f70f64b86702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84f10a946fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064d8), UINT64_C(0x1f42fa0fa4b2c704), UINT64_C(0x1f42fd0f64b606db),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f28b3471e)
    },
    {   UINT64_C(0x00000000000064d9), UINT64_C(0x1f42fa0fd0b366f5), UINT64_C(0x1f42fb0f78b346fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ed0afe6f9)
    },
    {   UINT64_C(0x0000000000013870), UINT64_C(0x14b2fd0fa08ce67e), UINT64_C(0x1192fd0fa091a67e),
        UINT64_C(0x14a2f50fa08d868a), UINT64_C(0x13fafd0fa091867e)
    },
    {   UINT64_C(0x00000000000064db), UINT64_C(0x1f42f50efcb646f7), UINT64_C(0x1f42f60f84b4c6e1),
        UINT64_C(0x1f42f40ea0b8a72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064dc), UINT64_C(0x1f42fb0f48b66717), UINT64_C(0x1f42fd0eacb7670a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064dd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cfa4b6e711),
        UINT64_C(0x1f42fd0f84b506db), UINT64_C(0x1f42fd0f9cb92703)
    },
    {   UINT64_C(0x00000000000064de), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f80b4e6df),
        UINT64_C(0x1f42f90f1cadc6fe), UINT64_C(0x1f42fd0ea8b166e1)
    },
    {   UINT64_C(0x00000000000064df), UINT64_C(0x1f42f80f20b6271f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc1010b36705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000013876), UINT64_C(0x1252f9cf788b85b7), UINT64_C(0x10baeb0c44846693),
        UINT64_C(0x0fbb030c408065b7), UINT64_C(0x0f8af58f7881a693)
    },
    {   UINT64_C(0x0000000000013877), UINT64_C(0x0f72f6cf787b05b7), UINT64_C(0x0d6af60c4c774693),
        UINT64_C(0x0f6af78c447b45b7), UINT64_C(0x1032fe4f787e6693)
    },
    {   UINT64_C(0x00000000000064e2), UINT64_C(0x1f42fa0f94b1c711), UINT64_C(0x1f42f94ec8adc6fa),
        UINT64_C(0x1f42fb0f00b36725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064e3), UINT64_C(0x1f42f80f48b92739), UINT64_C(0x1f42f94f1cb2271d),
        UINT64_C(0x1f42f90f8cb68726), UINT64_C(0x1f43034ed8b66731)
    },
    {   UINT64_C(0x00000000000064e4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0fc0b446ff),
        UINT64_C(0x1f42f90ed0b0a704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001387b), UINT64_C(0x1012fd4f787bc5b7), UINT64_C(0x14cafbcc408ea5b8),
        UINT64_C(0x1012dc0c448385b7), UINT64_C(0x147300cc4096a5b5)
    },
    {   UINT64_C(0x00000000000064e6), UINT64_C(0x1f42fb1028aec702), UINT64_C(0x1f42f70eb8b1e6ee),
        UINT64_C(0x1f42fa0f98b386fe), UINT64_C(0x1f42fa0ed8b3072b)
    },
    {   UINT64_C(0x00000000000064e7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f78b82708),
        UINT64_C(0x1f42fc0f54b5c714), UINT64_C(0x1f42f90eccb64700)
    },
    {   UINT64_C(0x00000000000064e8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0fe4b1c6ea),
        UINT64_C(0x1f42fecf3cb00722), UINT64_C(0x1f430b4f10b14722)
    },
    {   UINT64_C(0x00000000000064e9), UINT64_C(0x1f42fd0f28b566ff), UINT64_C(0x1f43098eccb42713),
        UINT64_C(0x1f42f90f00ae26f9), UINT64_C(0x1f42fc0fa0b0470e)
    },
    {   UINT64_C(0x0000000000006397), UINT64_C(0x1f42ffcf10ada6ee), UINT64_C(0x1f42facf74b6a6e6),
        UINT64_C(0x1f42fecf40b166e4), UINT64_C(0x1f42fdce80b40735)
    },
    {   UINT64_C(0x00000000000064eb), UINT64_C(0x1f42fb0f30af8721), UINT64_C(0x1f42fb0eecb3a71b),
        UINT64_C(0x1f42f68f20b74731), UINT64_C(0x1f42fa0ed8b60702)
    },
    {   UINT64_C(0x00000000000064ec), UINT64_C(0x1f42fd1000b2a702), UINT64_C(0x1f42fd0ec0b90702),
        UINT64_C(0x1f42fd0f84b72703), UINT64_C(0x1f42fdcf08b2c72d)
    },
    {   UINT64_C(0x00000000000064ed), UINT64_C(0x1f42fd0fb8b2c715), UINT64_C(0x1f42f90f00b8a700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f24aec709)
    },
    {   UINT64_C(0x000000000000639b), UINT64_C(0x1f42f58fdcb8470a), UINT64_C(0x1f42f8cea4b1471a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64ea4b86717)
    },
    {   UINT64_C(0x000000000000639c), UINT64_C(0x1f42fc4f5cb4e6df), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbce94b46740), UINT64_C(0x1f42fb0f28b4e6f6)
    },
    {   UINT64_C(0x000000000000639d), UINT64_C(0x1f42f44f94b466ee), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7cfb0b7a6ec), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064f1), UINT64_C(0x1f42f70e90b2e71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0fd8b5c6f6)
    },
    {   UINT64_C(0x00000000000064f2), UINT64_C(0x1f42f70f68b6c721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084f30b5072a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064f3), UINT64_C(0x1f42fb0f9cb44703), UINT64_C(0x1f4300cf1cb0c6fa),
        UINT64_C(0x1f42fc0ec8b346e9), UINT64_C(0x1f42f90fe8b4a70c)
    },
    {   UINT64_C(0x00000000000064f4), UINT64_C(0x1f42f84ec0b6c710), UINT64_C(0x1f42f90eb0ab4716),
        UINT64_C(0x1f42fc0eccb88720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064f5), UINT64_C(0x1f42f90f40b3a711), UINT64_C(0x1f42fb0fe8b4e719),
        UINT64_C(0x1f42fa8eacb5e73d), UINT64_C(0x1f42fc1020b8e71d)
    },
    {   UINT64_C(0x00000000000064f6), UINT64_C(0x1f42fb0fc8b466f8), UINT64_C(0x1f42fb0f18b06713),
        UINT64_C(0x1f42fdcfb0b1c703), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064f7), UINT64_C(0x1f42fb0fa4b486fe), UINT64_C(0x1f42fb0f4cb4a6fe),
        UINT64_C(0x1f42f80ec4b1672e), UINT64_C(0x1f42fb0f94b22715)
    },
    {   UINT64_C(0x0000000000008a5b), UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a),
        UINT64_C(0x156afc4d308d267e), UINT64_C(0x156afc4d308d267e)
    },
    {   UINT64_C(0x00000000000064f9), UINT64_C(0x1f42fb0efcb4c72f), UINT64_C(0x1f42f88f84b6670e),
        UINT64_C(0x1f42fe0ff0b2a6ce), UINT64_C(0x1f43028f0cb2e716)
    },
    {   UINT64_C(0x00000000000064fa), UINT64_C(0x1f430d4ee8b4e721), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003db2), UINT64_C(0x0000000000000000), UINT64_C(0x106b1d578871265b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003db3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b0bd78868665b), UINT64_C(0x106b0017887fa65b)
    },
    {   UINT64_C(0x00000000000064fd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0eb8b24733),
        UINT64_C(0x1f42fc0e6cb6a725), UINT64_C(0x1f42fc0f60b0e71c)
    },
    {   UINT64_C(0x0000000000003db5), UINT64_C(0x106b099754724678), UINT64_C(0x106aee0fa03e83e8),
        UINT64_C(0x106aec5788756667), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063ac), UINT64_C(0x1f42f38ef0b8c72e), UINT64_C(0x1f42f88e9cb64715),
        UINT64_C(0x1f42f88e9cb64715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003db7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106aee134c800511), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006501), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074edcb52728), UINT64_C(0x1f4300cf5cb646ee)
    },
    {   UINT64_C(0x0000000000006502), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0f50b24705),
        UINT64_C(0x1f4300ceecb6e709), UINT64_C(0x1f42f90ef8b0e6f1)
    },
    {   UINT64_C(0x0000000000006503), UINT64_C(0x1f42f84ef4b7272e), UINT64_C(0x1f42fc0e68b22719),
        UINT64_C(0x1f42fecf48b48704), UINT64_C(0x1f42fa0eb0b2273a)
    },
    {   UINT64_C(0x0000000000006504), UINT64_C(0x1f42fa0ea0b5c737), UINT64_C(0x1f42f91018af26fc),
        UINT64_C(0x1f42fb0f00b44705), UINT64_C(0x1f42f90f30b62703)
    },
    {   UINT64_C(0x0000000000006505), UINT64_C(0x1f42fb0ebcaf871d), UINT64_C(0x1f42fc0ec4b386fa),
        UINT64_C(0x1f42f80ed4b42720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063b3), UINT64_C(0x1f42fa8f68b8c71e), UINT64_C(0x1f430a8e8cb2672c),
        UINT64_C(0x1f430d8f0cac6721), UINT64_C(0x1f42fb8efcb40712)
    },
    {   UINT64_C(0x000000000001389d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f3cc388185aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063b5), UINT64_C(0x1f42f8ceecb0072a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdceacb4a71e)
    },
    {   UINT64_C(0x0000000000008a6c), UINT64_C(0x17d2fa8e6497862f), UINT64_C(0x17d2fa8e649786b5),
        UINT64_C(0x17d2fa8e6497871b), UINT64_C(0x17d2fa8e64978728)
    },
    {   UINT64_C(0x000000000000650a), UINT64_C(0x1f42f94f0cb04725), UINT64_C(0x1f42f80f1cb5470e),
        UINT64_C(0x1f42fb0eb4b1472c), UINT64_C(0x1f42fa0ef0b2c72d)
    },
    {   UINT64_C(0x000000000000650b), UINT64_C(0x1f42f90f3cb2271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70e8cb76732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000650c), UINT64_C(0x1f42fa0eecb4c6fa), UINT64_C(0x1f42fc0fa0b4c706),
        UINT64_C(0x1f42fb0ee0b1c702), UINT64_C(0x1f42f70f50b586fa)
    },
    {   UINT64_C(0x0000000000008a70), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x221afa8dc4c28657), UINT64_C(0x221afa8dc4c28657)
    },
    {   UINT64_C(0x000000000000650e), UINT64_C(0x1f42f90e70b2e735), UINT64_C(0x1f42fccfbcb646ec),
        UINT64_C(0x1f42fa0f00b2a719), UINT64_C(0x1f42fa0eb0b6a739)
    },
    {   UINT64_C(0x0000000000008a72), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x20b2fe0c64b946b0),
        UINT64_C(0x23caf6cda4c947d3), UINT64_C(0x23caf6cb94c94713)
    },
    {   UINT64_C(0x0000000000006510), UINT64_C(0x1f42fb0f6cb4a725), UINT64_C(0x1f42f44ee0b866ea),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006511), UINT64_C(0x1f42f90edcaf872e), UINT64_C(0x1f42f8ce9cb54731),
        UINT64_C(0x1f42faced8b50700), UINT64_C(0x1f42f90eccb78716)
    },
    {   UINT64_C(0x0000000000003dc9), UINT64_C(0x106af9ce547a269e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006513), UINT64_C(0x1f42fecee0b3e735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90e68b14715)
    },
    {   UINT64_C(0x00000000000138aa), UINT64_C(0x17f2f88c38a306dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x15c2f88c389046dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000003dcc), UINT64_C(0x106afa4f7479c5a2), UINT64_C(0x106b080fa075c59f),
        UINT64_C(0x106af98fa07985a2), UINT64_C(0x106af8cbfc7a85a2)
    },
    {   UINT64_C(0x0000000000003dcd), UINT64_C(0x106af54c287c65a3), UINT64_C(0x106afb0bfc798596),
        UINT64_C(0x106afb4f74796656), UINT64_C(0x106af8cbfc7a85a6)
    },
    {   UINT64_C(0x0000000000006517), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ef8b6a71f),
        UINT64_C(0x1f42fdce70b7672c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006518), UINT64_C(0x1f42fa0edcb7471a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f1cb58728), UINT64_C(0x1f42f84f9cb1671e)
    },
    {   UINT64_C(0x0000000000006519), UINT64_C(0x1f42f80ef8b7c716), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000651a), UINT64_C(0x1f42facf6cb4c702), UINT64_C(0x1f4301cf70b3c6df),
        UINT64_C(0x1f42f60f90b32728), UINT64_C(0x1f42face94b92752)
    },
    {   UINT64_C(0x0000000000008a7e), UINT64_C(0x12d3034a8c7f269f), UINT64_C(0x12d3034a8c7f269f),
        UINT64_C(0x156afe0f3c8c25fc), UINT64_C(0x156afe0f3c8c25fc)
    },
    {   UINT64_C(0x000000000000651c), UINT64_C(0x1f42f70edcb9271a), UINT64_C(0x1f42f80e4cb6272a),
        UINT64_C(0x1f42fb0e80b60738), UINT64_C(0x1f42f34e8cb74734)
    },
    {   UINT64_C(0x000000000000651d), UINT64_C(0x1f42f80eb0afc736), UINT64_C(0x1f42f60e3cb0c72c),
        UINT64_C(0x1f42f60f2cb6a726), UINT64_C(0x1f42f70e8cb54727)
    },
    {   UINT64_C(0x0000000000008a81), UINT64_C(0x1ce3058b80a646e8), UINT64_C(0x1ce3058b94a646e3),
        UINT64_C(0x1ce3058b94a646e3), UINT64_C(0x24e302cb94c7a6e3)
    },
    {   UINT64_C(0x00000000000138b5), UINT64_C(0x101af38be07ea6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x120af9cbe082a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006520), UINT64_C(0x1f42f60f6cb74701), UINT64_C(0x1f42fbce6cb7e72d),
        UINT64_C(0x1f42f60e6cb6c726), UINT64_C(0x1f42fdce54b2e708)
    },
    {   UINT64_C(0x0000000000006521), UINT64_C(0x0000000000000000), UINT64_C(0x1f43250f649046dc),
        UINT64_C(0x1f42fa0ff4b38708), UINT64_C(0x1f42f95008b4270f)
    },
    {   UINT64_C(0x0000000000006522), UINT64_C(0x1f42ff0ed8b20723), UINT64_C(0x1f42ff0f24b54714),
        UINT64_C(0x1f42fd0ec4a60720), UINT64_C(0x1f43000eecaee713)
    },
    {   UINT64_C(0x0000000000006523), UINT64_C(0x1f4301cf78ac6719), UINT64_C(0x1f42f85058b5270f),
        UINT64_C(0x1f42fa0f08b28707), UINT64_C(0x1f42fd0f08ab4705)
    },
    {   UINT64_C(0x0000000000006524), UINT64_C(0x1f42fd0f30a9e70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90f78b5e709), UINT64_C(0x1f42f90f04ae6732)
    },
    {   UINT64_C(0x0000000000006525), UINT64_C(0x1f42fd0fb0ae0705), UINT64_C(0x1f42fc0fb0b1c6f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd1010b5a705)
    },
    {   UINT64_C(0x0000000000006526), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ebcb04707),
        UINT64_C(0x1f42fd0f98b02715), UINT64_C(0x1f42fd102cb4c706)
    },
    {   UINT64_C(0x0000000000006527), UINT64_C(0x1f42fd101cb22711), UINT64_C(0x1f42fe8f94b64708),
        UINT64_C(0x1f42f94edcb3071e), UINT64_C(0x1f42f94f34b0a713)
    },
    {   UINT64_C(0x0000000000017827), UINT64_C(0x1842f80bf49e26ef), UINT64_C(0x17baf90bf49fe6ef),
        UINT64_C(0x1492f90bf49ee6ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008a8c), UINT64_C(0x1402f8cf3c892696), UINT64_C(0x1402f8cd988925f9),
        UINT64_C(0x1402f8ceec8925f9), UINT64_C(0x156afc4f208ce602)
    },
    {   UINT64_C(0x000000000000652a), UINT64_C(0x1f42f84efcb20727), UINT64_C(0x1f42fb0f68ada715),
        UINT64_C(0x1f42fd0fdcb6a711), UINT64_C(0x1f42fa0fe4af2707)
    },
    {   UINT64_C(0x000000000000652b), UINT64_C(0x1f42fd1010b3c703), UINT64_C(0x1f42fd0ec8b10711),
        UINT64_C(0x1f42fa1004b54718), UINT64_C(0x1f42fd1018b4270a)
    },
    {   UINT64_C(0x0000000000008a8f), UINT64_C(0x12c2fa116c838571), UINT64_C(0x12c2fa116c838571),
        UINT64_C(0x0962fd0cf85b45d0), UINT64_C(0x15b2f58fa091867e)
    },
    {   UINT64_C(0x000000000001782c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1162bf0c6c7f26a7)
    },
    {   UINT64_C(0x000000000000652e), UINT64_C(0x1f42fb0e6cada705), UINT64_C(0x1f42fb0f88b546f5),
        UINT64_C(0x1f42fd0f44b3070f), UINT64_C(0x1f42fc1008ab4707)
    },
    {   UINT64_C(0x000000000000652f), UINT64_C(0x1f42fd1030aae721), UINT64_C(0x1f4300d02cb4a71a),
        UINT64_C(0x1f42facec8b2c70b), UINT64_C(0x1f42fd104cb18705)
    },
    {   UINT64_C(0x0000000000006530), UINT64_C(0x1f42fb102cb1871e), UINT64_C(0x1f42f81010b4c707),
        UINT64_C(0x1f42fa0fbcad270e), UINT64_C(0x1f42f9cef0b64704)
    },
    {   UINT64_C(0x0000000000006531), UINT64_C(0x1f42fb0fdcae0707), UINT64_C(0x1f42fc0fa8b2c703),
        UINT64_C(0x1f42f70e9cb00702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006532), UINT64_C(0x1f42ffce6cb18707), UINT64_C(0x1f42fd1030ae0705),
        UINT64_C(0x1f42fd0fd8aee705), UINT64_C(0x1f42fd1040b62705)
    },
    {   UINT64_C(0x0000000000006533), UINT64_C(0x1f42fc0fc0b40716), UINT64_C(0x1f42fd0f64b4a711),
        UINT64_C(0x1f42fc0f08b56719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006534), UINT64_C(0x1f42fd0ef4af2725), UINT64_C(0x1f42fdcf9cb126f7),
        UINT64_C(0x1f42fd0eb8b26721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006535), UINT64_C(0x0000000000000000), UINT64_C(0x1f4300cf20ae2713),
        UINT64_C(0x1f42fd0f28b3a705), UINT64_C(0x1f42fd0ff8b3c714)
    },
    {   UINT64_C(0x00000000000138cc), UINT64_C(0x1012df8c388dc6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x10c2f80c387fc6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006537), UINT64_C(0x1f42f74f80aec724), UINT64_C(0x1f42fdcf84ae870f),
        UINT64_C(0x1f42ffcf28aaa712), UINT64_C(0x1f42fb0fd8af670a)
    },
    {   UINT64_C(0x00000000000138ce), UINT64_C(0x1002f60c388186dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x108af30c387f66dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138cf), UINT64_C(0x0a32eb8c3c7726dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1112fc4c388446dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000653a), UINT64_C(0x1f42fc102cb70706), UINT64_C(0x1f42f74f50b16708),
        UINT64_C(0x1f42f90f6cb7270b), UINT64_C(0x1f42f7100cb7e70e)
    },
    {   UINT64_C(0x00000000000138d1), UINT64_C(0x0e92f60c387c66dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0e32f04c387906dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138d2), UINT64_C(0x10aaf4cbe07cc6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x127af18c3c9746dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000653d), UINT64_C(0x1f42fa0f80b18706), UINT64_C(0x1f42f9102cb3470e),
        UINT64_C(0x1f42f90f84b4e713), UINT64_C(0x1f42f91008b4c6f4)
    },
    {   UINT64_C(0x00000000000138d4), UINT64_C(0x10b2f88c388486dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0e02f60c3881a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138d5), UINT64_C(0x1012f60c3882a6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x14baed8c3894c6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138d6), UINT64_C(0x13c2ef0c389606dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x14e2f60c389a86dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008aa4), UINT64_C(0x156afd09fc8fa74b), UINT64_C(0x156afbca588d4734),
        UINT64_C(0x156afbca588d4734), UINT64_C(0x156afd09fc8ca74a)
    },
    {   UINT64_C(0x0000000000006542), UINT64_C(0x1f42f94f1cb66713), UINT64_C(0x1f42fa0ffcb06706),
        UINT64_C(0x1f42fa0eecb06706), UINT64_C(0x1f42face9cb5a72b)
    },
    {   UINT64_C(0x0000000000006543), UINT64_C(0x1f42fa0e8cb1e721), UINT64_C(0x1f42fa1008b7e707),
        UINT64_C(0x1f42f90f2caf070a), UINT64_C(0x1f42fa0f10ada706)
    },
    {   UINT64_C(0x00000000000138da), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1082dfcbe08f86dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000063f2), UINT64_C(0x1f4303cf28b066df), UINT64_C(0x1f43024f20ac66d3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054f48b306d4)
    },
    {   UINT64_C(0x0000000000006546), UINT64_C(0x1f42fa0fe8af0706), UINT64_C(0x1f42fd0f40b1e706),
        UINT64_C(0x1f42f70f70af070b), UINT64_C(0x1f42fa0f28b6a71a)
    },
    {   UINT64_C(0x0000000000008aaa), UINT64_C(0x156afa8d7c8e8659), UINT64_C(0x156af94d7c8de65a),
        UINT64_C(0x156af94f248aa601), UINT64_C(0x156b010f248e8601)
    },
    {   UINT64_C(0x0000000000006548), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84efcaf870b),
        UINT64_C(0x1f42fa1008af2706), UINT64_C(0x1f42fdcf40b06715)
    },
    {   UINT64_C(0x0000000000008aac), UINT64_C(0x156afe0e048c2649), UINT64_C(0x156afd0c408ca65c),
        UINT64_C(0x156afd0c408ca65c), UINT64_C(0x156afd0c548ca6b5)
    },
    {   UINT64_C(0x0630000000003d41), UINT64_C(0x106af24fa07c65e8), UINT64_C(0x106af54fa07c65e8),
        UINT64_C(0x106af54fa07c65e8), UINT64_C(0x106b03cf74752672)
    },
    {   UINT64_C(0x00000000000063f8), UINT64_C(0x1f43054f24afa700), UINT64_C(0x1f43028f18b246e6),
        UINT64_C(0x1f43000f58b666f2), UINT64_C(0x1f43034f30adc6db)
    },
    {   UINT64_C(0x0000d00003383d43), UINT64_C(0x106af40fa07d05e8), UINT64_C(0x106af38fa07d45e8),
        UINT64_C(0x106b03cf707525f3), UINT64_C(0x106af24fa07dc5e8)
    },
    {   UINT64_C(0x06c0d60003503d44), UINT64_C(0x106af24fa07c45e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af54fa07925e8), UINT64_C(0x106af40fa07b25e8)
    },
    {   UINT64_C(0x000000000000654e), UINT64_C(0x1f42fa1000b186fb), UINT64_C(0x1f42fa0f4cb74708),
        UINT64_C(0x1f42f91018b08705), UINT64_C(0x1f42fa0f68b6c708)
    },
    {   UINT64_C(0x000000000000654f), UINT64_C(0x1f42fa0f68b6a711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90fbcb30717), UINT64_C(0x1f42f90fb0b58725)
    },
    {   UINT64_C(0x0000000000006550), UINT64_C(0x1f42fc0f40b54721), UINT64_C(0x1f42f90fbcb3c71a),
        UINT64_C(0x1f42fa0f10b2c71c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006551), UINT64_C(0x1f42fa0ee8b0a721), UINT64_C(0x1f4300cf5cac4710),
        UINT64_C(0x1f42fa0f68b1c716), UINT64_C(0x1f42fa0f20b44712)
    },
    {   UINT64_C(0x0000000000006552), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa1018ace706),
        UINT64_C(0x1f42f8100cb7070c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006553), UINT64_C(0x1f42fdced4ab2719), UINT64_C(0x1f42f60f18b2c71c),
        UINT64_C(0x1f42fa0e84b6271c), UINT64_C(0x1f42fa0f60ace706)
    },
    {   UINT64_C(0x0000000000006554), UINT64_C(0x1f42fc0f54af270a), UINT64_C(0x1f42ffcf2cb36725),
        UINT64_C(0x1f42fa1000b2a706), UINT64_C(0x1f42fa0f20b64706)
    },
    {   UINT64_C(0x0000000000006555), UINT64_C(0x1f42fa0fa8b06706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94eecac870b), UINT64_C(0x1f42fa0fc0b1870a)
    },
    {   UINT64_C(0x0000000000006556), UINT64_C(0x1f42fa0f4cb4a706), UINT64_C(0x1f42fa0efcabc706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ee4b3671d)
    },
    {   UINT64_C(0x0000000000008aba), UINT64_C(0x156afa0fa08e2623), UINT64_C(0x156afa0cbc8e269b),
        UINT64_C(0x156afa0fa08e267e), UINT64_C(0x156afa0fa08e267e)
    },
    {   UINT64_C(0x0000000000006558), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0eccb2670d),
        UINT64_C(0x1f42f9100cb5c6f6), UINT64_C(0x1f42fdcf2cb72714)
    },
    {   UINT64_C(0x0000000000006559), UINT64_C(0x1f42fa0f60b0e71c), UINT64_C(0x1f42fa0eb0b0872b),
        UINT64_C(0x1f42f60f30b62709), UINT64_C(0x1f42f80f3caf06e8)
    },
    {   UINT64_C(0x000000000000655a), UINT64_C(0x1f42fa0f1cb24714), UINT64_C(0x1f42fa0eb8ada715),
        UINT64_C(0x1f42f81014b1270f), UINT64_C(0x1f42fb0fe0b68713)
    },
    {   UINT64_C(0x000000000000655b), UINT64_C(0x1f42fa0ea4b3e72d), UINT64_C(0x1f42f90f60b20704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ef0ade706)
    },
    {   UINT64_C(0x000000000000655c), UINT64_C(0x1f42fd0ee4b5471f), UINT64_C(0x1f42f90f84b12712),
        UINT64_C(0x1f42fa0eccb76724), UINT64_C(0x1f42f60fa8b20721)
    },
    {   UINT64_C(0x000000000000655d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ea0b60730), UINT64_C(0x1f430c4e94b60728)
    },
    {   UINT64_C(0x000000000000655e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcff8b4c6d8), UINT64_C(0x1f42ff9014b6c6e1)
    },
    {   UINT64_C(0x000000000000655f), UINT64_C(0x1f43039020b0a6d4), UINT64_C(0x1f42f80f28b4c716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4f2cb386dd)
    },
    {   UINT64_C(0x0000000000006560), UINT64_C(0x1f4319cf2ca9e702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124ed8a4a70c), UINT64_C(0x1f431a4ec8a94705)
    },
    {   UINT64_C(0x0000000000006561), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43170ec8a7a704), UINT64_C(0x1f4312cedca9270e)
    },
    {   UINT64_C(0x0000000000006562), UINT64_C(0x1f430f4edca7470e), UINT64_C(0x1f43178ed4aa4717),
        UINT64_C(0x1f4312cedca9270e), UINT64_C(0x1f4316cf14aae717)
    },
    {   UINT64_C(0x0000000000006563), UINT64_C(0x1f43130ed0a8a717), UINT64_C(0x1f43164ed4a7a717),
        UINT64_C(0x1f43104ec0a8c72b), UINT64_C(0x1f430bced4a6670d)
    },
    {   UINT64_C(0x0000000000006564), UINT64_C(0x1f43158ee0a8071a), UINT64_C(0x1f43188f88aaa710),
        UINT64_C(0x1f43090e90a78731), UINT64_C(0x1f4314cf2ca58700)
    },
    {   UINT64_C(0x0000000000006565), UINT64_C(0x1f43110f48a70717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43120ecca9a72a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001511c), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x1172fe8f708f4685),
        UINT64_C(0x18daf88fa0a76685), UINT64_C(0x1742f88fa0960685)
    },
    {   UINT64_C(0x0000000000006567), UINT64_C(0x1f43114ed0a68711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43114ed0a68711), UINT64_C(0x1f43104ed8a68712)
    },
    {   UINT64_C(0x0000000000006568), UINT64_C(0x1f43100eccaa8720), UINT64_C(0x1f43120ed4a90725),
        UINT64_C(0x1f43108ed0aa4738), UINT64_C(0x1f43158efcab2722)
    },
    {   UINT64_C(0x000000000001511f), UINT64_C(0x16eb000f78a3c685), UINT64_C(0x128afe8f64a22685),
        UINT64_C(0x1ceafe8f5ccba685), UINT64_C(0x1942f50fa0a64685)
    },
    {   UINT64_C(0x0000000000015120), UINT64_C(0x112af7cfa0992685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x122af90f788305b6), UINT64_C(0x0feafa8f807b6695)
    },
    {   UINT64_C(0x0000000000015121), UINT64_C(0x0eeaf90f787805b6), UINT64_C(0x129af90f68882695),
        UINT64_C(0x0f92f90f7877e5b6), UINT64_C(0x11e2f90c648d8695)
    },
    {   UINT64_C(0x0000000000015122), UINT64_C(0x107af38c748065b6), UINT64_C(0x1202fa8f78826695),
        UINT64_C(0x0b2afbcf7c640654), UINT64_C(0x0d7ae98c64724654)
    },
    {   UINT64_C(0x0000000000015123), UINT64_C(0x1222fbcf7484e695), UINT64_C(0x0ad2fa8f7c654695),
        UINT64_C(0x17c2f64f7c9965bb), UINT64_C(0x117af64f788045bb)
    },
    {   UINT64_C(0x000000000000656e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43144e9ca7c70d),
        UINT64_C(0x1f43150ed0a82711), UINT64_C(0x1f43138e8caac701)
    },
    {   UINT64_C(0x0000000000015125), UINT64_C(0x0ed2fbcf7472e5b6), UINT64_C(0x0afaf90f78664638),
        UINT64_C(0x10baf64f787d45b7), UINT64_C(0x1152fb0f6881e5c1)
    },
    {   UINT64_C(0x0000000000015126), UINT64_C(0x1972fb0f68a265c1), UINT64_C(0x125af64f808265b6),
        UINT64_C(0x103af6cc6c7fc5b6), UINT64_C(0x109af6cf547ce5b6)
    },
    {   UINT64_C(0x0000000000006571), UINT64_C(0x0000000000000000), UINT64_C(0x1f43178f00a786fa),
        UINT64_C(0x1f43100ec0a66707), UINT64_C(0x1f43178ec4a76708)
    },
    {   UINT64_C(0x0000000000006572), UINT64_C(0x1f430fceeca7e727), UINT64_C(0x1f43114e78a6a70d),
        UINT64_C(0x1f43150ed8a9070a), UINT64_C(0x1f4311ce88a76707)
    },
    {   UINT64_C(0x0000000000006573), UINT64_C(0x1f4310ce7ca9e70e), UINT64_C(0x1f4310ceb4a5e70f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006574), UINT64_C(0x1f43148ea8a70710), UINT64_C(0x1f43124ee8a90713),
        UINT64_C(0x1f43170ee0a8a70d), UINT64_C(0x1f43150e90a906f8)
    },
    {   UINT64_C(0x0000000000006575), UINT64_C(0x1f430d8e48a5871c), UINT64_C(0x1f43108e98a78711),
        UINT64_C(0x1f43164e8ca8c725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006576), UINT64_C(0x1f43148ec0a826ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430fcec0a7870c), UINT64_C(0x1f43128ea4a9470f)
    },
    {   UINT64_C(0x0000000000006577), UINT64_C(0x1f43120ee8a6271d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124ec0a706ff), UINT64_C(0x1f43104f10a7071c)
    },
    {   UINT64_C(0x0000000000006578), UINT64_C(0x1f43104f10a7071c), UINT64_C(0x1f430f0ef0a6e720),
        UINT64_C(0x1f43144f08a8c727), UINT64_C(0x1f43134e98a7e705)
    },
    {   UINT64_C(0x0000000000006579), UINT64_C(0x1f43134e48a76705), UINT64_C(0x1f43134ec8a76705),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000657a), UINT64_C(0x1f43134f04a566ed), UINT64_C(0x1f43138eb8a726f5),
        UINT64_C(0x1f431a8f14a746f2), UINT64_C(0x1f43144f00a6a6f3)
    },
    {   UINT64_C(0x000000000000657b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43180efca8a6d7),
        UINT64_C(0x1f4318cf3ca9a6ef), UINT64_C(0x1f4313cec8a6e6e9)
    },
    {   UINT64_C(0x000000000000657c), UINT64_C(0x1f43144ec8a6e6ee), UINT64_C(0x1f43144ee0a6e6f1),
        UINT64_C(0x1f4316cef8a666e6), UINT64_C(0x1f43144ee4a706f2)
    },
    {   UINT64_C(0x000000000000657d), UINT64_C(0x1f43154f24a546f2), UINT64_C(0x1f43144f20a6e6f1),
        UINT64_C(0x1f43144ef0a6a6f1), UINT64_C(0x1f43144ef0a726f1)
    },
    {   UINT64_C(0x0000000000015134), UINT64_C(0x1ec2fa8f4cb5e6a3), UINT64_C(0x2122fbcf4cb9a6a3),
        UINT64_C(0x1c82fd0f4cac26a3), UINT64_C(0x1efafd0f4cafc6a3)
    },
    {   UINT64_C(0x000000000000657f), UINT64_C(0x1f43144ef4a6e6f3), UINT64_C(0x1f43144ed4a6a6e8),
        UINT64_C(0x1f43144ef0a646eb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006580), UINT64_C(0x1f43158ec0a8a6e5), UINT64_C(0x1f43108f28a6c700),
        UINT64_C(0x1f4313ced8a6e6ed), UINT64_C(0x1f43120eeca84704)
    },
    {   UINT64_C(0x0000000000006581), UINT64_C(0x1f43144ef0a646ee), UINT64_C(0x1f431a0f18a886ee),
        UINT64_C(0x1f43144ef4a666ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000015138), UINT64_C(0x194b000ed09c26a3), UINT64_C(0x20c2fbcf4cb826a3),
        UINT64_C(0x1983024f4ca026a3), UINT64_C(0x2032fe4f4cbea6a3)
    },
    {   UINT64_C(0x0000000000006583), UINT64_C(0x1f43144ef0a646eb), UINT64_C(0x1f43144ef0a646f1),
        UINT64_C(0x1f43130ee8a846e3), UINT64_C(0x1f43144ef4a686f1)
    },
    {   UINT64_C(0x0000000000006584), UINT64_C(0x1f43154ed4a6a6e8), UINT64_C(0x1f43154ef8a6e6e7),
        UINT64_C(0x1f43144ef0a686f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001513b), UINT64_C(0x17e2f6cf6893e5b6), UINT64_C(0x1daaf50f68b086a3),
        UINT64_C(0x16b2f6cf688f85b6), UINT64_C(0x1802f44c6c9c26a3)
    },
    {   UINT64_C(0x000000000001513c), UINT64_C(0x199af4cc209ee5b6), UINT64_C(0x1a32fbcf689d46a3),
        UINT64_C(0x1172f80f688146a2), UINT64_C(0x1a42f84c20a566a2)
    },
    {   UINT64_C(0x000000000001513d), UINT64_C(0x1952fbcf6899c6a3), UINT64_C(0x1452f8cf688d86a3),
        UINT64_C(0x238af94f68c2a5b6), UINT64_C(0x1a9af94f689ee5b6)
    },
    {   UINT64_C(0x000000000001513e), UINT64_C(0x1a82f6cf68a005b6), UINT64_C(0x1baaf44c6c9ec5b6),
        UINT64_C(0x1c8af6cc6ca6a5b6), UINT64_C(0x152af80fa08f85b6)
    },
    {   UINT64_C(0x000000000001513f), UINT64_C(0x1192facf6876a615), UINT64_C(0x1352f9cf688b0685),
        UINT64_C(0x1a52f5cf689dc5a9), UINT64_C(0x16e2f80f689245b2)
    },
    {   UINT64_C(0x0000000000015140), UINT64_C(0x1fa2f80f68b545b2), UINT64_C(0x19a2fdcf689b05b6),
        UINT64_C(0x1882f80c209765a9), UINT64_C(0x1872fd4f689645d5)
    },
    {   UINT64_C(0x000000000000658b), UINT64_C(0x1f430acf54b606ff), UINT64_C(0x1f43090f48b746f3),
        UINT64_C(0x1f43129020b6a6c9), UINT64_C(0x1f430a0fa8afe6db)
    },
    {   UINT64_C(0x000000000000658c), UINT64_C(0x1f43098f68b7a6d4), UINT64_C(0x1f430acedcafe6f4),
        UINT64_C(0x1f430f0fa4b5e6fe), UINT64_C(0x1f43114f94a926d5)
    },
    {   UINT64_C(0x000000000000658d), UINT64_C(0x1f43120f18a4c6c2), UINT64_C(0x1f430f4ebcb1c723),
        UINT64_C(0x1f4311cf28b7a6ef), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000658e), UINT64_C(0x0000000000000000), UINT64_C(0x1f430ccf84aec6e8),
        UINT64_C(0x1f430ccf08b346d4), UINT64_C(0x1f43134f84b646f0)
    },
    {   UINT64_C(0x000000000000658f), UINT64_C(0x1f43098ed8a9c6f7), UINT64_C(0x1f4311cf58ac06f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43134f28b546e7)
    },
    {   UINT64_C(0x0000000000006590), UINT64_C(0x1f43124fd0b0a6ee), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098eccb62702), UINT64_C(0x1f430ccf94b106d5)
    },
    {   UINT64_C(0x0000000000006591), UINT64_C(0x1f4312cef0b486e5), UINT64_C(0x1f430cd010b02701),
        UINT64_C(0x1f43114f94b206ea), UINT64_C(0x1f430d8fb4b4e6fe)
    },
    {   UINT64_C(0x0000000000006592), UINT64_C(0x1f43104f44af0700), UINT64_C(0x1f43100ff0ae46cd),
        UINT64_C(0x1f430e8ed4ae4736), UINT64_C(0x1f4310cf84b1a6e2)
    },
    {   UINT64_C(0x0000000000006593), UINT64_C(0x1f43091014abc6c0), UINT64_C(0x1f430bce2cb82737),
        UINT64_C(0x1f4305ceacb8272b), UINT64_C(0x1f43098ec4b0a715)
    },
    {   UINT64_C(0x0000000000006594), UINT64_C(0x1f430fcf64b6e6f6), UINT64_C(0x1f43058ed8b7a6d9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303ce6cb9c72d)
    },
    {   UINT64_C(0x0000000000006595), UINT64_C(0x1f4307cef8a9a707), UINT64_C(0x1f4311cff0b4c717),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006596), UINT64_C(0x1f43090ff0ae66d6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cf94afc714)
    },
    {   UINT64_C(0x0000000000017896), UINT64_C(0x1b2af70f6ca666a1), UINT64_C(0x14daf70f8495e6a9),
        UINT64_C(0x111af64fa09be6f7), UINT64_C(0x189af64f68b386a4)
    },
    {   UINT64_C(0x0000000000006598), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b0fb0b68719),
        UINT64_C(0x1f430fcfb0b5c70e), UINT64_C(0x1f430f4e6cb3e747)
    },
    {   UINT64_C(0x0000000000006446), UINT64_C(0x1f42f74eacb0c72c), UINT64_C(0x1f42fbce4cb24709),
        UINT64_C(0x1f4302ceacb20724), UINT64_C(0x1f4304ced8b0e717)
    },
    {   UINT64_C(0x000000000000659a), UINT64_C(0x1f4310cf8ca5a6e7), UINT64_C(0x1f430ccff0b6c6ef),
        UINT64_C(0x1f430dcfc4b54702), UINT64_C(0x1f430bced8b40712)
    },
    {   UINT64_C(0x000000000000659b), UINT64_C(0x1f43000f08b826d3), UINT64_C(0x1f430bce90b36725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430acf70b786f9)
    },
    {   UINT64_C(0x000000000000659c), UINT64_C(0x1f430a4f68b6a731), UINT64_C(0x1f430bcec8ace72d),
        UINT64_C(0x1f43070fb8b3c70b), UINT64_C(0x1f43070e9cb82731)
    },
    {   UINT64_C(0x000000000000659d), UINT64_C(0x1f43050e5cb3473c), UINT64_C(0x1f430f8ef0b7c732),
        UINT64_C(0x1f430fcf60ad271c), UINT64_C(0x1f430dcf40af670a)
    },
    {   UINT64_C(0x000000000000644b), UINT64_C(0x1f42ff8eb8b08706), UINT64_C(0x1f42f78e6cb4273b),
        UINT64_C(0x1f43104ec8b1c70d), UINT64_C(0x1f43030e7cb06725)
    },
    {   UINT64_C(0x000000000000644c), UINT64_C(0x1f43004e3cb30733), UINT64_C(0x1f42f90e5caec720),
        UINT64_C(0x1f42ff8f08b0070f), UINT64_C(0x1f43010e9cb226fa)
    },
    {   UINT64_C(0x00000000000065a0), UINT64_C(0x1f4310cf28b026eb), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309ce54b6a712)
    },
    {   UINT64_C(0x00000000000065a1), UINT64_C(0x1f430acfa4b546ee), UINT64_C(0x1f430ecf64b0a6e0),
        UINT64_C(0x1f430acf48b6e712), UINT64_C(0x1f43098f58b326f6)
    },
    {   UINT64_C(0x00000000000065a2), UINT64_C(0x1f43090f08b0a6d2), UINT64_C(0x1f430a0fc4adc6f2),
        UINT64_C(0x1f430ccea4b8672b), UINT64_C(0x1f430d0e90ab6729)
    },
    {   UINT64_C(0x00000000000065a3), UINT64_C(0x0000000000000000), UINT64_C(0x1f430d0fe4b3272a),
        UINT64_C(0x1f430bcefcb28713), UINT64_C(0x1f430ecee8b166f4)
    },
    {   UINT64_C(0x00000000000065a4), UINT64_C(0x1f4306cf4caf2713), UINT64_C(0x1f43098f68ad46ff),
        UINT64_C(0x1f430d4e9cb54735), UINT64_C(0x1f43098f0caac72a)
    },
    {   UINT64_C(0x00000000000065a5), UINT64_C(0x1f43124f5cb166e3), UINT64_C(0x1f43104f5ca8e704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43094f84b46728)
    },
    {   UINT64_C(0x00000000000065a6), UINT64_C(0x1f430f4f68b50704), UINT64_C(0x1f430acee4ab8715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4308ceccb7470e)
    },
    {   UINT64_C(0x0000000000006454), UINT64_C(0x1f4305cf08b006f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b0ff8a9c700), UINT64_C(0x1f43030e48b5872f)
    },
    {   UINT64_C(0x00000000000065a8), UINT64_C(0x1f4307cf4caec719), UINT64_C(0x1f43080f50ac8713),
        UINT64_C(0x1f43058ec4b5c722), UINT64_C(0x1f42ff8e1cb2e73b)
    },
    {   UINT64_C(0x00000000000065a9), UINT64_C(0x1f43030f20af2731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309ceccb64723)
    },
    {   UINT64_C(0x00000000000065aa), UINT64_C(0x1f4306cf70b3873b), UINT64_C(0x1f430d4f44b0670e),
        UINT64_C(0x1f43088f94b72730), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065ab), UINT64_C(0x1f430d0eecb406ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058f18b3a721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178ab), UINT64_C(0x0000000000000000), UINT64_C(0x20a2f90bf4c426ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1acaf90bf4acc6ef)
    },
    {   UINT64_C(0x000000000000645a), UINT64_C(0x1f43084e98ad8709), UINT64_C(0x1f43031054adc712),
        UINT64_C(0x1f4302cf88b82709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178ad), UINT64_C(0x0000000000000000), UINT64_C(0x1a12fc4bcc9fa6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x187af90bf49d26ef)
    },
    {   UINT64_C(0x00000000000065af), UINT64_C(0x1f43098e2cad0725), UINT64_C(0x1f430dce8ca1a727),
        UINT64_C(0x1f42ff8f28b4671c), UINT64_C(0x1f42ff0f48b54719)
    },
    {   UINT64_C(0x00000000000065b0), UINT64_C(0x1f42fdce74b24735), UINT64_C(0x1f42f98f84b2c70c),
        UINT64_C(0x1f42facf28b2c6ff), UINT64_C(0x1f42fccf78b42709)
    },
    {   UINT64_C(0x00000000000065b1), UINT64_C(0x1f42f8ceccb6c735), UINT64_C(0x1f42ff8eccb6c722),
        UINT64_C(0x1f43120e68b2673c), UINT64_C(0x1f42fb8e3caee72e)
    },
    {   UINT64_C(0x00000000000065b2), UINT64_C(0x1f43034e7ca80734), UINT64_C(0x1f42fb8ee8b4671d),
        UINT64_C(0x1f4303cf08b186f1), UINT64_C(0x1f4308ce4ca92731)
    },
    {   UINT64_C(0x00000000000065b3), UINT64_C(0x1f42f68eccb4e71c), UINT64_C(0x1f4311ce8cb3c741),
        UINT64_C(0x1f42fc0f58af8707), UINT64_C(0x1f430a0f78b26718)
    },
    {   UINT64_C(0x00000000000178b3), UINT64_C(0x0000000000000000), UINT64_C(0x0ffaf64bf47dc6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1432f5cbf48e46ef)
    },
    {   UINT64_C(0x0000000000006462), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038eb8b3e72d),
        UINT64_C(0x1f4303d024b7069f), UINT64_C(0x1f4300cf5cb3e725)
    },
    {   UINT64_C(0x0000000000006463), UINT64_C(0x1f4302507cb6e6da), UINT64_C(0x1f43001044b706e9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f50b406f3)
    },
    {   UINT64_C(0x0000000000006464), UINT64_C(0x1f42fe4fa0ade6e8), UINT64_C(0x1f43020e74b906fa),
        UINT64_C(0x1f4308906cb246d7), UINT64_C(0x1f42fc4fd4b2270c)
    },
    {   UINT64_C(0x0000000000006465), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fad068b52741)
    },
    {   UINT64_C(0x00000000000065b9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e78b1e738),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006467), UINT64_C(0x1f4300cfccb4a70f), UINT64_C(0x1f43024f90b06715),
        UINT64_C(0x1f4302d068b686b5), UINT64_C(0x1f430a0e54b04710)
    },
    {   UINT64_C(0x00000000000178ba), UINT64_C(0x0000000000000000), UINT64_C(0x1222f98c648586ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0daaf6cbf48566ef)
    },
    {   UINT64_C(0x00000000000178bb), UINT64_C(0x0000000000000000), UINT64_C(0x1002f88bf47ee6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0cdaf60c647b46ef)
    },
    {   UINT64_C(0x00000000000178bc), UINT64_C(0x0000000000000000), UINT64_C(0x13c2fd0bf48926ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1152fb0c648446ef)
    },
    {   UINT64_C(0x00000000000178bd), UINT64_C(0x0000000000000000), UINT64_C(0x1972f24c64a266ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x143af6cbf48c46ef)
    },
    {   UINT64_C(0x00000000000178be), UINT64_C(0x0000000000000000), UINT64_C(0x12baf30c648e86ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x109af6cbf483e6ef)
    },
    {   UINT64_C(0x00000000000065c0), UINT64_C(0x1f43154eacae2713), UINT64_C(0x1f43180e6cb40725),
        UINT64_C(0x1f43114f0caee70a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065c1), UINT64_C(0x1f43128e60b2c72a), UINT64_C(0x1f43170eb0a98706),
        UINT64_C(0x1f43098ebcb4c72e), UINT64_C(0x1f43004e88b3872e)
    },
    {   UINT64_C(0x00000000000065c2), UINT64_C(0x1f43124e3caa0712), UINT64_C(0x1f4300ce48b32717),
        UINT64_C(0x1f430bcf44ac8714), UINT64_C(0x1f43114e90ade708)
    },
    {   UINT64_C(0x00000000000065c3), UINT64_C(0x0000000000000000), UINT64_C(0x1f43028e4cae0731),
        UINT64_C(0x1f43078e60b1c728), UINT64_C(0x1f430f8e8caee724)
    },
    {   UINT64_C(0x0000000000006471), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0edcb6a71e),
        UINT64_C(0x1f43038ef8b36731), UINT64_C(0x1f430a8eb8b96740)
    },
    {   UINT64_C(0x00000000000065c5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4305ce60b2c720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065c6), UINT64_C(0x1f4316cef8a9871e), UINT64_C(0x1f43144f2ca6270a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065c7), UINT64_C(0x1f43068f00b22726), UINT64_C(0x1f43114f6cab26db),
        UINT64_C(0x1f43180ea8b56734), UINT64_C(0x1f4315cea8ac8716)
    },
    {   UINT64_C(0x00000000000065c8), UINT64_C(0x1f43148e6cb60717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c0f24b4c719), UINT64_C(0x1f43178e68ac2722)
    },
    {   UINT64_C(0x00000000000065c9), UINT64_C(0x1f4312ce74a8671f), UINT64_C(0x1f43198ebcb02722),
        UINT64_C(0x1f4308cee0a74701), UINT64_C(0x1f43068e44b1472b)
    },
    {   UINT64_C(0x00000000000065ca), UINT64_C(0x1f42face54b0c732), UINT64_C(0x1f430c0e9caa470e),
        UINT64_C(0x1f43134ee0aaa710), UINT64_C(0x1f43140ea4aa0710)
    },
    {   UINT64_C(0x00000000000065cb), UINT64_C(0x1f43158e9cadc71e), UINT64_C(0x1f430c0e74aa0729),
        UINT64_C(0x1f43158f0cabc70f), UINT64_C(0x1f430d4eecb4270f)
    },
    {   UINT64_C(0x00000000000065cc), UINT64_C(0x1f43060e60afc725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c0e90b7672e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065cd), UINT64_C(0x0000000000000000), UINT64_C(0x1f43118e9cabe70f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065ce), UINT64_C(0x1f4308ceb4ae0734), UINT64_C(0x1f43044e8caca734),
        UINT64_C(0x1f43068f24b42714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178ce), UINT64_C(0x0000000000000000), UINT64_C(0x121af30c648766ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x133af98c6489c6ef)
    },
    {   UINT64_C(0x00000000000178cf), UINT64_C(0x0000000000000000), UINT64_C(0x0b1af54c647ae6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1292f6cc649086ef)
    },
    {   UINT64_C(0x00000000000178d0), UINT64_C(0x0000000000000000), UINT64_C(0x12b2f28c648ec6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x15cae58c649aa6ef)
    },
    {   UINT64_C(0x00000000000065d2), UINT64_C(0x1f430e4f44adc714), UINT64_C(0x1f42fdcf38b5470e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43098e68b1272b)
    },
    {   UINT64_C(0x00000000000178d2), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f60bf47dc6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1462ee4c649f06ef)
    },
    {   UINT64_C(0x00000000000178d3), UINT64_C(0x0000000000000000), UINT64_C(0x13e2f4cc649006ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fc2f3cc647ec6ef)
    },
    {   UINT64_C(0x00000000000065d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000e88b18729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065d6), UINT64_C(0x1f42fccea0b70702), UINT64_C(0x1f42ff8efcb4670e),
        UINT64_C(0x1f42fccf3cb86721), UINT64_C(0x1f42f8cefcb72719)
    },
    {   UINT64_C(0x00000000000065d7), UINT64_C(0x1f42fa0ec0b6c72e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c4eacafc72a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065d8), UINT64_C(0x1f430a4e80afe736), UINT64_C(0x1f42f78e2cac0742),
        UINT64_C(0x1f430ed058aee6e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001518f), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa094e68c),
        UINT64_C(0x169b0bcf54a2269b), UINT64_C(0x162b020fa09f068c)
    },
    {   UINT64_C(0x00000000000065da), UINT64_C(0x1f42f58e64ade726), UINT64_C(0x1f43028e6cb0c740),
        UINT64_C(0x1f43028f0cb3e71e), UINT64_C(0x1f42f40eecb86729)
    },
    {   UINT64_C(0x0000000000015191), UINT64_C(0x0dc301cfa081468c), UINT64_C(0x0f0af1ce2086468c),
        UINT64_C(0x1643020fa0a4868c), UINT64_C(0x12db020fa08b068c)
    },
    {   UINT64_C(0x0000000000015192), UINT64_C(0x1c8b020fa0bba68c), UINT64_C(0x16a3020f58a4068c),
        UINT64_C(0x1853088f58a7869c), UINT64_C(0x122b020fa095668c)
    },
    {   UINT64_C(0x00000000000178dc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1092f10bf499e6ef)
    },
    {   UINT64_C(0x000000000000648b), UINT64_C(0x1f42f1d030b9a6fc), UINT64_C(0x1f43034e68b5071a),
        UINT64_C(0x1f42fd0f44b246fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178de), UINT64_C(0x0000000000000000), UINT64_C(0x12caf98c6486e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af98c648266ef)
    },
    {   UINT64_C(0x0000000000015196), UINT64_C(0x0e72ffcf688005c1), UINT64_C(0x1032ff4f688ca68c),
        UINT64_C(0x0e22ffcf6c7c05c1), UINT64_C(0x0aa3000fa081269c)
    },
    {   UINT64_C(0x000000000000648e), UINT64_C(0x1f42f60fe4b6a71b), UINT64_C(0x1f42f50f64b68711),
        UINT64_C(0x1f42fc0fb0af2701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000648f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e6cb1c70b),
        UINT64_C(0x1f42f81028b1271c), UINT64_C(0x1f42fad040b2c70f)
    },
    {   UINT64_C(0x00000000000065e3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf68b5471d),
        UINT64_C(0x1f42fccf04b72723), UINT64_C(0x1f43088e4cb2c735)
    },
    {   UINT64_C(0x00000000000065e4), UINT64_C(0x1f42fd0eb4b40729), UINT64_C(0x1f42f8cf38b606f7),
        UINT64_C(0x1f42ff8ec8b6a70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065e5), UINT64_C(0x1f43038f28b76703), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdce5cad271b)
    },
    {   UINT64_C(0x00000000000065e6), UINT64_C(0x1f42f9cea0aba709), UINT64_C(0x1f43028ef0a806e5),
        UINT64_C(0x1f43038edcb0e70f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065e7), UINT64_C(0x1f42fd8eecb246f7), UINT64_C(0x1f42fccf28b0e700),
        UINT64_C(0x1f42fd8e6cb08727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065e8), UINT64_C(0x1f42f78f48af6722), UINT64_C(0x1f43020f44b366fe),
        UINT64_C(0x1f42fe8eacb7271e), UINT64_C(0x1f42f8ceecb6470f)
    },
    {   UINT64_C(0x00000000000065e9), UINT64_C(0x1f43168ef8a7271e), UINT64_C(0x1f42fbce20b5a70c),
        UINT64_C(0x1f42f84f28b6c707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065ea), UINT64_C(0x1f42ff8f08af66fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430cceacac2723), UINT64_C(0x1f42fccf3cac8713)
    },
    {   UINT64_C(0x00000000000065eb), UINT64_C(0x1f42ff8ef8b486e8), UINT64_C(0x1f42fb0f28b4a712),
        UINT64_C(0x1f43018f58b566fc), UINT64_C(0x1f42fccef8b2e722)
    },
    {   UINT64_C(0x00000000000065ec), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffcf18b3c710), UINT64_C(0x1f42f98f0cb7270b)
    },
    {   UINT64_C(0x00000000000065ed), UINT64_C(0x1f43008ee0af66fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303ce00ad4741), UINT64_C(0x1f43160e5ca78726)
    },
    {   UINT64_C(0x00000000000065ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cef8b28705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0ebcb14716)
    },
    {   UINT64_C(0x00000000000065ef), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94eb8b36711)
    },
    {   UINT64_C(0x00000000000065f0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f48b34710),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f44b56712)
    },
    {   UINT64_C(0x0000000000003de8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af8cd707c4623), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065f2), UINT64_C(0x1f42fa4eacb56734), UINT64_C(0x1f42face98afe718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078e4caee736)
    },
    {   UINT64_C(0x00000000000065f3), UINT64_C(0x1f42fd8f48af6739), UINT64_C(0x1f42fa0ebcb6a71d),
        UINT64_C(0x1f42f80e7cb8a71d), UINT64_C(0x1f43074e40b4873b)
    },
    {   UINT64_C(0x00000000000064a1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40fa0ae4720),
        UINT64_C(0x1f42fecfc8b1270a), UINT64_C(0x1f42fd0eacb5c70e)
    },
    {   UINT64_C(0x00000000000065f5), UINT64_C(0x1f4301cf28b7c70d), UINT64_C(0x1f430acef0b266fe),
        UINT64_C(0x1f42f80e8cb54725), UINT64_C(0x1f42f90e7cb3c705)
    },
    {   UINT64_C(0x00000000000065f6), UINT64_C(0x1f42ffce8cb7a71d), UINT64_C(0x1f42f54f5cb28739),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065f7), UINT64_C(0x1f42f90e5cae6705), UINT64_C(0x1f42fa0f04b14713),
        UINT64_C(0x1f42fa0edcb32725), UINT64_C(0x1f4308ce78b92720)
    },
    {   UINT64_C(0x00000000000065f8), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305ceecb24716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301ce8cb1271d)
    },
    {   UINT64_C(0x00000000000065f9), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010e44af2730),
        UINT64_C(0x1f43148ee8a96720), UINT64_C(0x1f43000f3cb04710)
    },
    {   UINT64_C(0x00000000000064a7), UINT64_C(0x1f42f64f24b5071b), UINT64_C(0x1f42f90ef8b746f4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065fb), UINT64_C(0x1f43004fa4aea701), UINT64_C(0x1f4307cf9cab66e3),
        UINT64_C(0x1f430b0e68ad0727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065fc), UINT64_C(0x1f430f4ed0ab671f), UINT64_C(0x1f4308ce88aea70a),
        UINT64_C(0x1f42fd8ed8b3c70f), UINT64_C(0x1f42ff8e80b3872f)
    },
    {   UINT64_C(0x00000000000064aa), UINT64_C(0x1f42fa9024b5670a), UINT64_C(0x1f42fa1050ae2706),
        UINT64_C(0x1f42ffcf38adc700), UINT64_C(0x1f42f40f6cb30708)
    },
    {   UINT64_C(0x00000000000065fe), UINT64_C(0x1f42ff8eccbb070a), UINT64_C(0x1f4303cf9cb4a6fb),
        UINT64_C(0x1f42f80f88b34706), UINT64_C(0x1f42f8cea4ada705)
    },
    {   UINT64_C(0x00000000000065ff), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf3cb00706),
        UINT64_C(0x1f4304ce38af8712), UINT64_C(0x1f430b4e14aee73f)
    },
    {   UINT64_C(0x0000000000006600), UINT64_C(0x1f42f80e60b6c739), UINT64_C(0x1f431bceacad071e),
        UINT64_C(0x1f430acf5cb02709), UINT64_C(0x1f43110f2cae26ed)
    },
    {   UINT64_C(0x0000000000006601), UINT64_C(0x1f42ffcf3ca986e4), UINT64_C(0x1f42fe0f0cad86ee),
        UINT64_C(0x1f4301cf5cb0a6f7), UINT64_C(0x1f42fa8f10ab26ee)
    },
    {   UINT64_C(0x0000000000006602), UINT64_C(0x1f43034f20ac2708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0ed0ad8702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006603), UINT64_C(0x1f42fc0f6cb206f9), UINT64_C(0x1f4301cf7cad66fd),
        UINT64_C(0x1f43014f5cae66ec), UINT64_C(0x1f42fe4f9cad6716)
    },
    {   UINT64_C(0x0000000000006604), UINT64_C(0x1f42f7cf88ac86fb), UINT64_C(0x1f42fc0f3cb10705),
        UINT64_C(0x1f4300cf3cb126f5), UINT64_C(0x1f430c8f10b2a71a)
    },
    {   UINT64_C(0x0000000000006605), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044f1cadc6ed), UINT64_C(0x1f42fa4f10ad071d)
    },
    {   UINT64_C(0x0000000000006606), UINT64_C(0x1f42f84f5cada726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060ef0af86fb), UINT64_C(0x1f42f9cf28af6710)
    },
    {   UINT64_C(0x0000000000006607), UINT64_C(0x1f42f54f7cb1e6db), UINT64_C(0x1f43018f7cad26e7),
        UINT64_C(0x1f42f5cf7cad070a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006608), UINT64_C(0x1f42f98f7cab06fc), UINT64_C(0x1f43018ec0ad0703),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006609), UINT64_C(0x1f43080f3cace6fb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8f18ac4720), UINT64_C(0x1f43124f5ca9e703)
    },
    {   UINT64_C(0x000000000000660a), UINT64_C(0x1f42fb4f50ada6de), UINT64_C(0x1f43044eecae86f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f8cb206f5)
    },
    {   UINT64_C(0x000000000000660b), UINT64_C(0x1f4301cf0cb18701), UINT64_C(0x1f43044f4caee6fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43028f6cac26f9)
    },
    {   UINT64_C(0x000000000000660c), UINT64_C(0x1f42fb4f10ace71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43070f98af66bb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000660d), UINT64_C(0x1f43050ef0b006f9), UINT64_C(0x1f42facf70ad66d9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ced4ab6710)
    },
    {   UINT64_C(0x000000000000660e), UINT64_C(0x1f42fd8ef0b046fb), UINT64_C(0x1f42f98ef0ac8719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8ef0b0471d)
    },
    {   UINT64_C(0x000000000000660f), UINT64_C(0x1f43038ef0b0c70e), UINT64_C(0x1f43010ef0b2071d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006610), UINT64_C(0x0000000000000000), UINT64_C(0x1f43020f4caf26e3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078f7cad26d7)
    },
    {   UINT64_C(0x00000000000151c7), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x13d2f88fa09c2685), UINT64_C(0x13c2f60fa0882695)
    },
    {   UINT64_C(0x0000000000006612), UINT64_C(0x1f4309cec8ad66e8), UINT64_C(0x1f42ff0f4caf2700),
        UINT64_C(0x1f4300cefcaf2712), UINT64_C(0x1f43050ef0b006d3)
    },
    {   UINT64_C(0x0000000000006613), UINT64_C(0x1f42ffcfbcae070d), UINT64_C(0x1f43030f6cac8712),
        UINT64_C(0x1f42f9cf3cb36700), UINT64_C(0x1f42f5ceb4ac472d)
    },
    {   UINT64_C(0x00000000000151ca), UINT64_C(0x1b2af88fa0b20685), UINT64_C(0x1672fc4fa09d2685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1bbaf7cfa0b44685)
    },
    {   UINT64_C(0x00000000000151cb), UINT64_C(0x1322f88fa09a2685), UINT64_C(0x187afbcf609ce695),
        UINT64_C(0x17bafe0fa0a0e685), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x00000000000064c3), UINT64_C(0x1f42fa0eccb0670b), UINT64_C(0x1f42f60e2cb486fc),
        UINT64_C(0x1f42f94f90b2a6f0), UINT64_C(0x1f42f5109cb52705)
    },
    {   UINT64_C(0x00000000000151cd), UINT64_C(0x19d2fc4fa0a46695), UINT64_C(0x13c2f8cf60898685),
        UINT64_C(0x1382f90f708d45b5), UINT64_C(0x115af50c6c876695)
    },
    {   UINT64_C(0x0000000000006618), UINT64_C(0x1f43058ef8ae86dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4301cf60ae8703), UINT64_C(0x1f42f68f70ae8700)
    },
    {   UINT64_C(0x00000000000151cf), UINT64_C(0x102af64c6c7e65b6), UINT64_C(0x11b2f90f7483e696),
        UINT64_C(0x09f2fbcf7c60c5b6), UINT64_C(0x1182fbcfa086a5b6)
    },
    {   UINT64_C(0x00000000000151d0), UINT64_C(0x11d2fbcf6081a695), UINT64_C(0x13e2f64c6c89e5a9),
        UINT64_C(0x105afbcf788145b3), UINT64_C(0x0ee2f50c747b6695)
    },
    {   UINT64_C(0x00000000000151d1), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x121af7cf6c8845a9),
        UINT64_C(0x129af64c6c84c5b6), UINT64_C(0x0f02f7cc747e05b6)
    },
    {   UINT64_C(0x00000000000151d2), UINT64_C(0x1282f90f7889e5a9), UINT64_C(0x0eaaf64f847d45a9),
        UINT64_C(0x1142f64f788165b6), UINT64_C(0x1612fbcc6c91e5b6)
    },
    {   UINT64_C(0x00000000000151d3), UINT64_C(0x136af64c709305b6), UINT64_C(0x15eaf64c6c9d05b6),
        UINT64_C(0x157af38f7891a5b6), UINT64_C(0x117af90f78846696)
    },
    {   UINT64_C(0x00000000000151d4), UINT64_C(0x0f0af90f787aa5b5), UINT64_C(0x12faf64f6c8786a2),
        UINT64_C(0x12faeb0f648d45b6), UINT64_C(0x155af90c6c906695)
    },
    {   UINT64_C(0x00000000000151d5), UINT64_C(0x124af90c6c84e5b6), UINT64_C(0x1b22f64f78ac65a9),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006620), UINT64_C(0x1f42f90fb8ab86f7), UINT64_C(0x1f42fe4f5ca926df),
        UINT64_C(0x1f42ff0f3cae86f5), UINT64_C(0x1f42fe4fb8ad6716)
    },
    {   UINT64_C(0x00000000000064ce), UINT64_C(0x1f42f90ebcb6c70e), UINT64_C(0x1f42f80ed0b4a707),
        UINT64_C(0x1f42f58ef0b7c71c), UINT64_C(0x1f42f24f08b96733)
    },
    {   UINT64_C(0x00000000000064cf), UINT64_C(0x1f42fb0f04b6a6f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a4ed0b3471c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006623), UINT64_C(0x1f42f94f7cac66e3), UINT64_C(0x1f42fd0f7cac66e4),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f7cac4707)
    },
    {   UINT64_C(0x00000000000064d1), UINT64_C(0x1f43000f28b2670f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80efcb38706), UINT64_C(0x1f42fb0f4cb2670b)
    },
    {   UINT64_C(0x0000000000006625), UINT64_C(0x1f42f90f7cac6707), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf64a88705), UINT64_C(0x1f42ffcf9caf86f2)
    },
    {   UINT64_C(0x0000000000006626), UINT64_C(0x1f42f9cf8caca6f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008f98aca6f9), UINT64_C(0x1f43078f7cac6706)
    },
    {   UINT64_C(0x00000000000064d4), UINT64_C(0x1f42fdcf00b7e704), UINT64_C(0x1f42fb0eb8b5e71b),
        UINT64_C(0x1f42fd0e8cb686e0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006628), UINT64_C(0x1f4302cf30aea707), UINT64_C(0x1f42fdcf38ad06f6),
        UINT64_C(0x1f43020f7cac66e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006629), UINT64_C(0x1f43058f58ab8726), UINT64_C(0x1f43048f7cac66f9),
        UINT64_C(0x1f43104e50ad671d), UINT64_C(0x1f42fa8f1cad0732)
    },
    {   UINT64_C(0x000000000000662a), UINT64_C(0x1f43028f94b7c6ff), UINT64_C(0x1f42fb0f94b6a6e4),
        UINT64_C(0x1f4300cee8b2471e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000662b), UINT64_C(0x1f42f74f1cb806c6), UINT64_C(0x1f42f74f44b7c6dd),
        UINT64_C(0x1f42f94f30b746bf), UINT64_C(0x1f42f84f88b7e6d0)
    },
    {   UINT64_C(0x000000000000662c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90fa4b686f7),
        UINT64_C(0x1f42f58eb0b5c6ad), UINT64_C(0x1f42f68facb786dd)
    },
    {   UINT64_C(0x00000000000064da), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010f00b6e734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000662e), UINT64_C(0x1f42f54f18b886cd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f44ec0b6a6d9), UINT64_C(0x1f42f88fecb826fc)
    },
    {   UINT64_C(0x00000000000151e5), UINT64_C(0x155afd0fa097068c), UINT64_C(0x161af44fa09ea68c),
        UINT64_C(0x1762fd0fa0a0668c), UINT64_C(0x1a7b048f54a5469b)
    },
    {   UINT64_C(0x0000000000006630), UINT64_C(0x1f42f74f00b7c6f3), UINT64_C(0x1f42fc4fccb7a6d7),
        UINT64_C(0x1f42f68f54b826e8), UINT64_C(0x1f42f68f54b74716)
    },
    {   UINT64_C(0x0000000000006631), UINT64_C(0x1f42f7cf28b826ed), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88fc0b906de), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006632), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f78b6c700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f20b6c6cc)
    },
    {   UINT64_C(0x00000000000064e0), UINT64_C(0x1f42fa0eacb866e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facf9cb54712), UINT64_C(0x1f42f70facb786e0)
    },
    {   UINT64_C(0x0000000000006634), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f04b266d4),
        UINT64_C(0x1f42f84f8cb5e6d2), UINT64_C(0x1f42f74f20b6a6de)
    },
    {   UINT64_C(0x0000000000006635), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f18b70701),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006636), UINT64_C(0x1f43028f58b706e7), UINT64_C(0x1f42fb8fd0b766dd),
        UINT64_C(0x1f42f54f2cb446ea), UINT64_C(0x1f430e8ec0b6270d)
    },
    {   UINT64_C(0x00000000000151ed), UINT64_C(0x11f2f28f6882069c), UINT64_C(0x1292eb0c6c85a5b6),
        UINT64_C(0x106af84f688525c1), UINT64_C(0x0f0af8cc6c80269c)
    },
    {   UINT64_C(0x0000000000006638), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0facb88722), UINT64_C(0x1f42face50afc709)
    },
    {   UINT64_C(0x0000000000006639), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f74b1c703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ce8cb8871d)
    },
    {   UINT64_C(0x000000000000663a), UINT64_C(0x1f43090f58b8270a), UINT64_C(0x1f42f9cf6cb58712),
        UINT64_C(0x1f42f30f9cb28724), UINT64_C(0x1f42fbcf30b4c70b)
    },
    {   UINT64_C(0x000000000000663b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f50aa6721),
        UINT64_C(0x1f42fc8f88b4270b), UINT64_C(0x1f42f70f5cb90719)
    },
    {   UINT64_C(0x000000000000663c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cfb0b2c70b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000663d), UINT64_C(0x1f42f70f4cac66f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74f54b5870b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000663e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0eacb26710),
        UINT64_C(0x1f43054e74b6671f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018741), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fc2f34fa07aa5b8)
    },
    {   UINT64_C(0x0000000000006640), UINT64_C(0x1f42f90f50b6a70d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90facb3a703), UINT64_C(0x1f42f90fa8b4470f)
    },
    {   UINT64_C(0x00000000000064ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64fd8b766f2),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006642), UINT64_C(0x1f43054e40b66730), UINT64_C(0x1f42f70f8cb5871d),
        UINT64_C(0x1f42f84e70b8274d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064f0), UINT64_C(0x1f42f70f5cb0070a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcfa4b206e1), UINT64_C(0x1f42f90fd8b5c700)
    },
    {   UINT64_C(0x0000000000006644), UINT64_C(0x1f42fa8e4cb9a72c), UINT64_C(0x1f42facf00b4071b),
        UINT64_C(0x1f42f60fc0b4c70b), UINT64_C(0x1f42facef0b4c71c)
    },
    {   UINT64_C(0x0000000000006645), UINT64_C(0x1f42fd0f38ad0716), UINT64_C(0x1f42f70fdcb7271e),
        UINT64_C(0x1f42fa0f60b5c712), UINT64_C(0x1f42f60e8cb9a72a)
    },
    {   UINT64_C(0x0000000000006646), UINT64_C(0x1f42f84f44ba4726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcefcb40736), UINT64_C(0x1f42f40f90b7871f)
    },
    {   UINT64_C(0x0000000000006647), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4cf14b8271f),
        UINT64_C(0x1f42f70f28b2c71d), UINT64_C(0x1f42fa1008b32714)
    },
    {   UINT64_C(0x0000000000006648), UINT64_C(0x1f42f60ef4b7872e), UINT64_C(0x1f42f70f10b2070f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006649), UINT64_C(0x1f42f80f8cb4c710), UINT64_C(0x1f43074ec4b86719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f80b38707)
    },
    {   UINT64_C(0x000000000000664a), UINT64_C(0x1f42f8cebcb5071a), UINT64_C(0x1f42f84f7cb72707),
        UINT64_C(0x1f42f80f7cb3c70c), UINT64_C(0x1f42f00e5cb4c702)
    },
    {   UINT64_C(0x000000000000664b), UINT64_C(0x1f42f60f78ae270b), UINT64_C(0x1f42fb0fa0b146fe),
        UINT64_C(0x1f42fdce3cb3c739), UINT64_C(0x1f42fa0f0cb64715)
    },
    {   UINT64_C(0x000000000000664c), UINT64_C(0x1f42f80f70b5a712), UINT64_C(0x1f42f81030b34712),
        UINT64_C(0x1f42fd0f18ae4715), UINT64_C(0x1f42fd0f4cae8715)
    },
    {   UINT64_C(0x000000000000664d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64f54b6e73a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000064fb), UINT64_C(0x1f42fa0ebcb3671e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70f08b82725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000664f), UINT64_C(0x1f42f70f24b22730), UINT64_C(0x1f42f90f98b5070b),
        UINT64_C(0x1f42f74f40af6726), UINT64_C(0x1f42f70ef4b54709)
    },
    {   UINT64_C(0x0000000000006650), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60f20b78703),
        UINT64_C(0x1f42f80fb0b6a725), UINT64_C(0x1f42fb0f3cb5c6f7)
    },
    {   UINT64_C(0x0000000000006651), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70ef8b88712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0fbcb1a71d)
    },
    {   UINT64_C(0x0000000000006652), UINT64_C(0x1f42f84f80b1a71d), UINT64_C(0x1f42f74f44b3a726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbceecb0e71d)
    },
    {   UINT64_C(0x0000000000006500), UINT64_C(0x1f42ffcee4b0e721), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f30b48715)
    },
    {   UINT64_C(0x0000000000006654), UINT64_C(0x1f42facf68b38738), UINT64_C(0x1f42f40e84b5c72c),
        UINT64_C(0x1f42f50eb8b6472c), UINT64_C(0x1f42fb0ebcb46725)
    },
    {   UINT64_C(0x0000000000006655), UINT64_C(0x1f43038ea4b32731), UINT64_C(0x1f43018e64b42742),
        UINT64_C(0x1f42f90f40b2271e), UINT64_C(0x1f43018e10b7a733)
    },
    {   UINT64_C(0x0000000000006656), UINT64_C(0x1f42f54f24b24735), UINT64_C(0x1f42face80b34724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60edcb6a732)
    },
    {   UINT64_C(0x0000000000006657), UINT64_C(0x1f42f94e64b2c716), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80e64b84742), UINT64_C(0x1f42f80e90b6673d)
    },
    {   UINT64_C(0x0000000000006658), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cea4b5270a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f432e4ff09a86fd)
    },
    {   UINT64_C(0x0000000000006506), UINT64_C(0x1f42fdcf4cb32714), UINT64_C(0x1f42fd0fccb60715),
        UINT64_C(0x1f42f88fb0b646fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000665a), UINT64_C(0x1f42fc8ff0b66703), UINT64_C(0x1f42facf6cb6e706),
        UINT64_C(0x1f42f90ff4b0e6fd), UINT64_C(0x1f42fd8fb0b64700)
    },
    {   UINT64_C(0x0000000000006508), UINT64_C(0x1f42f90eb0b1c722), UINT64_C(0x1f42f7ce58b6a73b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006509), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f54b5a6e4)
    },
    {   UINT64_C(0x000000000000665d), UINT64_C(0x1f42fc8fd0adc6ed), UINT64_C(0x1f42f9ce7cb44704),
        UINT64_C(0x1f4301cfccb44709), UINT64_C(0x1f42fc9000af4700)
    },
    {   UINT64_C(0x0000000000018760), UINT64_C(0x0000000000000000), UINT64_C(0x1082f4cb247be693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018761), UINT64_C(0x10f2efcb247da693), UINT64_C(0x0932f2cb245ea693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018762), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0c2afacb246a05b3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018763), UINT64_C(0x0000000000000000), UINT64_C(0x1022f24b247a65aa),
        UINT64_C(0x0e2af4cb247285aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000650f), UINT64_C(0x1f42f90ea4b24737), UINT64_C(0x1f42f80f70b28702),
        UINT64_C(0x1f430acf3cb58725), UINT64_C(0x1f4302cfb8b54725)
    },
    {   UINT64_C(0x0000000000006663), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf94b54715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0fbcae4704)
    },
    {   UINT64_C(0x0000000000018766), UINT64_C(0x0dcafc56446ee6db), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006512), UINT64_C(0x1f4300cf20b206f3), UINT64_C(0x1f42facf68b80734),
        UINT64_C(0x1f42fb0e4cb8a744), UINT64_C(0x1f43064f00b3a71d)
    },
    {   UINT64_C(0x0000000000018768), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f02fe13fc7046dd)
    },
    {   UINT64_C(0x0000000000006514), UINT64_C(0x1f42fb0f30b2a70b), UINT64_C(0x1f42fa0fc4b6a71e),
        UINT64_C(0x1f42fd8efcb4e736), UINT64_C(0x1f42f34f08b8c703)
    },
    {   UINT64_C(0x0000000000006515), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0e80b1c703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80e54b886fe)
    },
    {   UINT64_C(0x0000000000006516), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80eb4b68734), UINT64_C(0x1f42f8cec0b6272d)
    },
    {   UINT64_C(0x000000000000666a), UINT64_C(0x1f42fdce28b58735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ce38b68743)
    },
    {   UINT64_C(0x000000000000666b), UINT64_C(0x1f43120ebcae0722), UINT64_C(0x1f42fc8f78b646d7),
        UINT64_C(0x1f4307cef4afa709), UINT64_C(0x1f42ffcef8b286c5)
    },
    {   UINT64_C(0x000000000000666c), UINT64_C(0x1f42fd0f5cab870b), UINT64_C(0x1f42f7cfb0b646d3),
        UINT64_C(0x1f42fe0fc0b306d1), UINT64_C(0x1f42f7cfa8b72705)
    },
    {   UINT64_C(0x000000000000666d), UINT64_C(0x1f42fc8f28ade702), UINT64_C(0x1f430f8e30aa8734),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000666e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f84ad26e7),
        UINT64_C(0x1f42f5cf68b726dd), UINT64_C(0x1f42f50f9cb926fe)
    },
    {   UINT64_C(0x000000000000666f), UINT64_C(0x1f42f64fbcb766de), UINT64_C(0x1f4317ce98a8c72a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006670), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc500cb726fa),
        UINT64_C(0x1f42fe4f30b2e6e1), UINT64_C(0x1f42fa8f50b5a6e8)
    },
    {   UINT64_C(0x000000000000651e), UINT64_C(0x1f42f80fa0b44707), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000651f), UINT64_C(0x1f42fc0ee8b3c710), UINT64_C(0x1f42fb0ed4b1a73c),
        UINT64_C(0x1f43078ee0b7673a), UINT64_C(0x1f430a8f3cb0c71e)
    },
    {   UINT64_C(0x0000000000006673), UINT64_C(0x1f43174e30a9872d), UINT64_C(0x1f42fa8f08b486cf),
        UINT64_C(0x1f42fb0f58b506c7), UINT64_C(0x1f42fe0f58b246c7)
    },
    {   UINT64_C(0x0000000000006674), UINT64_C(0x1f42fbcfa0b546e8), UINT64_C(0x1f43044e78b20738),
        UINT64_C(0x1f42fa4f64b04712), UINT64_C(0x1f42fe4f68b386c7)
    },
    {   UINT64_C(0x0000000000006675), UINT64_C(0x1f42fc0f50b4c6cd), UINT64_C(0x1f4315ceb4aa2737),
        UINT64_C(0x1f42fa8f28b726d9), UINT64_C(0x1f42fa0ec0b8e6fd)
    },
    {   UINT64_C(0x0000000000006676), UINT64_C(0x1f42fb4f8cab86f1), UINT64_C(0x1f42facf20b546cf),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60facb586f7)
    },
    {   UINT64_C(0x0000000000006677), UINT64_C(0x1f42fc0f60b626cb), UINT64_C(0x1f42fa4f88b466fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcfb0b526c5)
    },
    {   UINT64_C(0x0000000000006678), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98ec8af8701), UINT64_C(0x1f42fb0f50b706aa)
    },
    {   UINT64_C(0x0000000000006679), UINT64_C(0x1f42f88f70b1e6aa), UINT64_C(0x1f42f78f24b2e6e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf68afc6ab)
    },
    {   UINT64_C(0x000000000000667a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f70b646da),
        UINT64_C(0x1f42f80f4cb466d8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006528), UINT64_C(0x1f42fb0f60ade71a), UINT64_C(0x1f42fd0f7ca98705),
        UINT64_C(0x1f42fb0f8caec717), UINT64_C(0x1f42fc1000b60700)
    },
    {   UINT64_C(0x000000000000667c), UINT64_C(0x1f42facf40b026b1), UINT64_C(0x1f42f58fd0b6c711),
        UINT64_C(0x1f42ff0f08b306e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000667d), UINT64_C(0x1f42f88f54b206f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f94b066ec), UINT64_C(0x1f42f88ee8b92703)
    },
    {   UINT64_C(0x000000000000667e), UINT64_C(0x1f42fa4fbcb5e6f3), UINT64_C(0x1f42fa4fc0b426e7),
        UINT64_C(0x1f42fa8f34b7a6f6), UINT64_C(0x1f42f88e9cb166f3)
    },
    {   UINT64_C(0x000000000000652c), UINT64_C(0x1f42fd0fdcb2670b), UINT64_C(0x1f42fc0f30b04709),
        UINT64_C(0x1f42fd1010b2e70e), UINT64_C(0x1f42f70fd4b50714)
    },
    {   UINT64_C(0x000000000000652d), UINT64_C(0x1f42fd0fd8b2070d), UINT64_C(0x1f42fd0f48b64712),
        UINT64_C(0x1f42fd0ef8b2e705), UINT64_C(0x1f42fb100cb0c705)
    },
    {   UINT64_C(0x0000000000006681), UINT64_C(0x1f42fa8e80b6c6ff), UINT64_C(0x1f42fa4f0cb1c715),
        UINT64_C(0x1f42fa4f18ae06d6), UINT64_C(0x1f42f88efcb926dd)
    },
    {   UINT64_C(0x0000000000006682), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f54b8e6dc),
        UINT64_C(0x1f42fa4ef8b186e7), UINT64_C(0x1f42f94e8cb72707)
    },
    {   UINT64_C(0x0000000000006683), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf28b7a6d4),
        UINT64_C(0x1f42fc8f24b5c721), UINT64_C(0x1f42f78efcb3670e)
    },
    {   UINT64_C(0x0000000000006684), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88ef8b4a703),
        UINT64_C(0x1f42f64f64b766f3), UINT64_C(0x1f42f50ff4b7e706)
    },
    {   UINT64_C(0x0000000000006685), UINT64_C(0x1f42f88ec0b006e1), UINT64_C(0x1f42f5cfa8b786fb),
        UINT64_C(0x1f430cce48af872f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006686), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facfd0b04725),
        UINT64_C(0x1f42f40f78b0c734), UINT64_C(0x1f42f68f78b466e3)
    },
    {   UINT64_C(0x000000000001000c), UINT64_C(0x0fa2f60f847ca68c), UINT64_C(0x0fa2fa8fa071a68c),
        UINT64_C(0x0fa2f10fa077068c), UINT64_C(0x0fa2f20f8478a68c)
    },
    {   UINT64_C(0x0000000000006688), UINT64_C(0x1f42f8cf60b706ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88fd8b826da), UINT64_C(0x1f42f58f10b46708)
    },
    {   UINT64_C(0x0000000000006536), UINT64_C(0x1f42fdcf74b0a714), UINT64_C(0x1f42f80fc4b6e711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0fd4ad8702)
    },
    {   UINT64_C(0x000000000000668a), UINT64_C(0x1f42face6cb506f8), UINT64_C(0x1f42facfc0b886fd),
        UINT64_C(0x1f42facf24b70719), UINT64_C(0x1f42face60b6073e)
    },
    {   UINT64_C(0x0000000000006538), UINT64_C(0x1f42fa0fa0ae4716), UINT64_C(0x1f42f94f8cac66fe),
        UINT64_C(0x1f42f74eacb64722), UINT64_C(0x1f42f90f70b6871c)
    },
    {   UINT64_C(0x0000000000006539), UINT64_C(0x1f42fa0fc0b30705), UINT64_C(0x1f42f75030b18707),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0ec8b30705)
    },
    {   UINT64_C(0x08c11021c0010012), UINT64_C(0x1692ec0fa09ea675), UINT64_C(0x0a6aec0fa06e8675),
        UINT64_C(0x0de2ec8f587be675), UINT64_C(0x14daefcfa098c675)
    },
    {   UINT64_C(0x000000000000653b), UINT64_C(0x1f42f90f5cb78708), UINT64_C(0x1f42fe4fa8ada70c),
        UINT64_C(0x1f42fa0f04b4c70d), UINT64_C(0x1f42f90f78b4670d)
    },
    {   UINT64_C(0x000000000000653c), UINT64_C(0x1f42f90facb6270b), UINT64_C(0x1f42f60f60b626f4),
        UINT64_C(0x1f42f80f90ab070a), UINT64_C(0x1f42f90facb5a712)
    },
    {   UINT64_C(0x0000000000006690), UINT64_C(0x1f43030e34b68747), UINT64_C(0x1f430bced0ae0736),
        UINT64_C(0x1f42f74e90b8071e), UINT64_C(0x1f42fb8eccaf06e6)
    },
    {   UINT64_C(0x000000000000653e), UINT64_C(0x1f43000ec0b1c71e), UINT64_C(0x1f42fc0f48ad070c),
        UINT64_C(0x1f42f64f4cada719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000653f), UINT64_C(0x1f42f40eccb3c70e), UINT64_C(0x1f42f90fa8b1e6fe),
        UINT64_C(0x1f42f70f90b226fb), UINT64_C(0x1f42fa102cb50706)
    },
    {   UINT64_C(0x0000000000006540), UINT64_C(0x1f42f90f4cac471e), UINT64_C(0x1f42f64f4ca80716),
        UINT64_C(0x1f42fa102cb24706), UINT64_C(0x1f42fa1058b5c706)
    },
    {   UINT64_C(0x0000000000006694), UINT64_C(0x1f42fc8f4cb7a714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88e6cb3c702), UINT64_C(0x1f42fc4f1cb18725)
    },
    {   UINT64_C(0x0000000000006695), UINT64_C(0x1f42fd8e8cb14701), UINT64_C(0x1f42fb0f7cb4c71b),
        UINT64_C(0x1f42f9cf48afa707), UINT64_C(0x1f42f84f94b6a705)
    },
    {   UINT64_C(0x2484868fd1e9001b), UINT64_C(0x08b2f84f74616693), UINT64_C(0x1692f10f7c9685a1),
        UINT64_C(0x0fa2f18f7c79c5a1), UINT64_C(0x0fa2f4cf747905a1)
    },
    {   UINT64_C(0x0000000000006544), UINT64_C(0x1f42fa0fc8b62715), UINT64_C(0x1f42f74ef4af6729),
        UINT64_C(0x1f42fa0efcb68706), UINT64_C(0x1f42fa0f14b26706)
    },
    {   UINT64_C(0x0000000000006545), UINT64_C(0x1f42fc0f50b1271a), UINT64_C(0x1f42fccfb4b6e6f9),
        UINT64_C(0x1f42fa0ef8afa729), UINT64_C(0x1f42f80f40adc702)
    },
    {   UINT64_C(0x0000000000006699), UINT64_C(0x1f42f68ec0b5e729), UINT64_C(0x1f42f88eb0b24747),
        UINT64_C(0x1f42f88eecb446ee), UINT64_C(0x1f42f7ceacb3070c)
    },
    {   UINT64_C(0x0000000000006547), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0eecb56706),
        UINT64_C(0x1f42fa0ee4b6c706), UINT64_C(0x1f42fa1020b54706)
    },
    {   UINT64_C(0x000000000000669b), UINT64_C(0x1f4309cee8a8c6e6), UINT64_C(0x1f42f64f90b646d5),
        UINT64_C(0x1f42fa4f84b646e1), UINT64_C(0x1f42fa8f28ac8702)
    },
    {   UINT64_C(0x0000000000006549), UINT64_C(0x1f42fa100cb24709), UINT64_C(0x1f42fa0f38b2470d),
        UINT64_C(0x1f42fc0e9caae734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000654a), UINT64_C(0x1f42f50f94b5c703), UINT64_C(0x1f42f90f40b6a717),
        UINT64_C(0x1f42f90fd4b10712), UINT64_C(0x1f42fa0fd8b16706)
    },
    {   UINT64_C(0x00000000000187a0), UINT64_C(0x1d82ef4f74b30762), UINT64_C(0x1692f34f6895468e),
        UINT64_C(0x1d82ef4f74b30762), UINT64_C(0x1692f34f6895468e)
    },
    {   UINT64_C(0x00000000000187a1), UINT64_C(0x1d82ef4f74b3072a), UINT64_C(0x1692f34f68954656),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000654d), UINT64_C(0x1f42fa0f60b0c706), UINT64_C(0x1f42f80f50af86e0),
        UINT64_C(0x1f42fa0fe4b54706), UINT64_C(0x1f42fa0eecb10711)
    },
    {   UINT64_C(0x0000000000008c04), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156af08fa092867e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066a2), UINT64_C(0x1f42f60f18b2c70c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78fd8acc6db), UINT64_C(0x1f42fece84af0719)
    },
    {   UINT64_C(0x00000000000066a3), UINT64_C(0x1f42f70f6cb64703), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54f54b58710)
    },
    {   UINT64_C(0x00000000000066a4), UINT64_C(0x1f42f6cf6cb3c71e), UINT64_C(0x1f42f84ee4b3e71e),
        UINT64_C(0x1f42f6cf14b5c6ce), UINT64_C(0x1f42f6cf0cb1c715)
    },
    {   UINT64_C(0x000000000001002a), UINT64_C(0x0a6b08d790712646), UINT64_C(0x17c2f84f58986682),
        UINT64_C(0x08a2f895f86a868c), UINT64_C(0x0fa2fb503476257b)
    },
    {   UINT64_C(0x000000000001002b), UINT64_C(0x151b03915088456a), UINT64_C(0x0a6afa52a061c4e7),
        UINT64_C(0x17c2f84f58986682), UINT64_C(0x0a6b06d7f071e62f)
    },
    {   UINT64_C(0x00000000000066a7), UINT64_C(0x1f4312ce94b0e72f), UINT64_C(0x1f42f6cf90af26d5),
        UINT64_C(0x1f430b0e84b54732), UINT64_C(0x1f42f6cf3cac6715)
    },
    {   UINT64_C(0x00000000000066a8), UINT64_C(0x1f42f98f0caf66f6), UINT64_C(0x1f42fa8ec4ad86de),
        UINT64_C(0x1f42f6cf74b46708), UINT64_C(0x1f42f5cf34b626fd)
    },
    {   UINT64_C(0x00000000000066a9), UINT64_C(0x1f42f78f28b506ef), UINT64_C(0x1f42f6ceecaf271c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6cec0b4e729)
    },
    {   UINT64_C(0x000000000001002f), UINT64_C(0x1772f64f789a868c), UINT64_C(0x1772f68f789c268c),
        UINT64_C(0x1772f3cf789a868c), UINT64_C(0x0fa2f50c6c6c65c1)
    },
    {   UINT64_C(0x31f628be57710030), UINT64_C(0x131ae14fa085077a), UINT64_C(0x131ae14fa085077a),
        UINT64_C(0x131ae14fa0850777), UINT64_C(0x131ae14fa0860752)
    },
    {   UINT64_C(0x35f000d119510031), UINT64_C(0x131ae14fa0850740), UINT64_C(0x131ae14fa08507a5),
        UINT64_C(0x1bcae74fa0b06675), UINT64_C(0x14dafe8c3c94a682)
    },
    {   UINT64_C(0x0000000000010032), UINT64_C(0x131aedcfa08dc77a), UINT64_C(0x131aedcfa08dc77a),
        UINT64_C(0x131aedcfa08dc777), UINT64_C(0x131aedcfa08dc740)
    },
    {   UINT64_C(0x0000000000010033), UINT64_C(0x0a6aec0fa070477a), UINT64_C(0x0a6aec0fa072477a),
        UINT64_C(0x0a6aec0fa076c777), UINT64_C(0x0a6aec0fa077a740)
    },
    {   UINT64_C(0x3636c00000010034), UINT64_C(0x1692ec0fa0960675), UINT64_C(0x14dae90f64996752),
        UINT64_C(0x1692fd0f5895e77a), UINT64_C(0x1692fd0f5895e77a)
    },
    {   UINT64_C(0x0006dcdadb410035), UINT64_C(0x1692fd0f5895e777), UINT64_C(0x1692fd0f5895e752),
        UINT64_C(0x1692fd0f5895e740), UINT64_C(0x151b054fc08745d9)
    },
    {   UINT64_C(0x3836f8dd5ba10036), UINT64_C(0x1692fd0dfc95e6ba), UINT64_C(0x1693078f589e277a),
        UINT64_C(0x1693078f589e277a), UINT64_C(0x1693078f589e2777)
    },
    {   UINT64_C(0x000000001c210037), UINT64_C(0x1693078f589e2740), UINT64_C(0x1163018fa08da77a),
        UINT64_C(0x131aee0fa085a675), UINT64_C(0x0fa2c40c647c268f)
    },
    {   UINT64_C(0x000000e39c610038), UINT64_C(0x0faaf24f747a0680), UINT64_C(0x0faaf24f747a0680),
        UINT64_C(0x0faaf24f747ca67d), UINT64_C(0x0faaf24f747fa658)
    },
    {   UINT64_C(0x000728e49c810039), UINT64_C(0x0faaf24f747d6646), UINT64_C(0x0faaf24f747a06ad),
        UINT64_C(0x14daf3cf748e85a1), UINT64_C(0x0de2f48c3c73a5a1)
    },
    {   UINT64_C(0x000730e58001003a), UINT64_C(0x0de2f5cf74724680), UINT64_C(0x0de2f5cf74746680),
        UINT64_C(0x0de2f5cf7478c67d), UINT64_C(0x0de2f5cf7479e646)
    },
    {   UINT64_C(0x000000000001003b), UINT64_C(0x08b2f9cf74644680), UINT64_C(0x08b2f9cf74664680),
        UINT64_C(0x08b2f9cf746a667d), UINT64_C(0x08b2f9cf746ba646)
    },
    {   UINT64_C(0x00000000000066b7), UINT64_C(0x1f42fb0e7cb5c6ea), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f5cee0b786f5), UINT64_C(0x1f43028f4cab26d7)
    },
    {   UINT64_C(0x0000000000008c1b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c2af74efca7867d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006566), UINT64_C(0x1f4317cf28a9a706), UINT64_C(0x1f43118ec8a9e72b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43114ed0a68711)
    },
    {   UINT64_C(0x000000000001003f), UINT64_C(0x0fa2f88f7479e646), UINT64_C(0x0de2e80c6873c680),
        UINT64_C(0x0fa2db4c6c792693), UINT64_C(0x0de2e80c6875c646)
    },
    {   UINT64_C(0x0000000000010040), UINT64_C(0x131ae14fa08506dd), UINT64_C(0x0faaf24f747aa607),
        UINT64_C(0x131ae14fa0850746), UINT64_C(0x0faaf24f747aa672)
    },
    {   UINT64_C(0x0000000000006569), UINT64_C(0x1f4310ce6ca6471f), UINT64_C(0x1f43180efcab0719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4312cecca92725)
    },
    {   UINT64_C(0x000000000000656a), UINT64_C(0x1f43148e78a8e70e), UINT64_C(0x1f4314cee0a64713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43178ee0a88708)
    },
    {   UINT64_C(0x000000000000656b), UINT64_C(0x1f4312ceaca5c710), UINT64_C(0x1f431b0e9ca886f1),
        UINT64_C(0x1f43110e58a826fd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000656c), UINT64_C(0x1f431a8ee0a7e709), UINT64_C(0x1f431a4ef0a80711),
        UINT64_C(0x1f43160eeca7a725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000656d), UINT64_C(0x0000000000000000), UINT64_C(0x1f43154ed0a7270b),
        UINT64_C(0x1f43124ebca76711), UINT64_C(0x1f4303ceb4b086cf)
    },
    {   UINT64_C(0x0000000000010046), UINT64_C(0x131aedcce48dc675), UINT64_C(0x0de2f5cce47205a1),
        UINT64_C(0x131aedcfa08dc753), UINT64_C(0x0de2f5cf747d467d)
    },
    {   UINT64_C(0x000000000000656f), UINT64_C(0x1f43110ebca6a70a), UINT64_C(0x1f4316cee0a8c703),
        UINT64_C(0x1f43110eb4a6e715), UINT64_C(0x1f43168e9ca74706)
    },
    {   UINT64_C(0x00000000000066c3), UINT64_C(0x1f43098e88a5871e), UINT64_C(0x1f43020e7ca72720),
        UINT64_C(0x1f43138ed8a6070f), UINT64_C(0x1f430f8f00b0e720)
    },
    {   UINT64_C(0x00000000000066c4), UINT64_C(0x1f4308ce90ae6726), UINT64_C(0x1f430c0eacaa672d),
        UINT64_C(0x1f43060e84a6e722), UINT64_C(0x1f430acebca6a721)
    },
    {   UINT64_C(0x000000000001004a), UINT64_C(0x0a6aec0fa076072c), UINT64_C(0x08b2f58f746b2658),
        UINT64_C(0x0a6aec0fa07566dd), UINT64_C(0x08b2f54f74692607)
    },
    {   UINT64_C(0x000000000001004b), UINT64_C(0x0a6aec0fa076874a), UINT64_C(0x08b2f98f7469e672),
        UINT64_C(0x0a6aec0cfc6e8675), UINT64_C(0x08b2fa4cfc646676)
    },
    {   UINT64_C(0x00000000000066c7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060e24aa473f), UINT64_C(0x1f42ffce24b02743)
    },
    {   UINT64_C(0x00000000000066c8), UINT64_C(0x1f43010de8b6e730), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f8e40aaa739), UINT64_C(0x1f43030e50b4c733)
    },
    {   UINT64_C(0x00000000000066c9), UINT64_C(0x1f430cce10ad0747), UINT64_C(0x1f430fcee4a8671c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4eaca84708)
    },
    {   UINT64_C(0x000000000001004f), UINT64_C(0x08b2efcb74616693), UINT64_C(0x1162ec0fa088e682),
        UINT64_C(0x08b2f84f7476869d), UINT64_C(0x1162ec0fa0846675)
    },
    {   UINT64_C(0x0000000000010050), UINT64_C(0x0a1af84f746ec693), UINT64_C(0x1162ec0fa0846675),
        UINT64_C(0x08b2f74f7464e693), UINT64_C(0x14dae90f64996754)
    },
    {   UINT64_C(0x0000000000010051), UINT64_C(0x0fa2f18f7c79c680), UINT64_C(0x14dae90b74996675),
        UINT64_C(0x0fa2f18b7479c5a1), UINT64_C(0x14dae90f64996753)
    },
    {   UINT64_C(0x00000000000066cd), UINT64_C(0x0000000000000000), UINT64_C(0x1f43148eb0ace71b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f04b4a707)
    },
    {   UINT64_C(0x00000000000066ce), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040e98b0072b), UINT64_C(0x1f43160ec8a70724)
    },
    {   UINT64_C(0x0000000000010054), UINT64_C(0x1692fd0f58988754), UINT64_C(0x0fa2f4cf74822680),
        UINT64_C(0x1d82fa4f80b70682), UINT64_C(0x14daf30f708f45a1)
    },
    {   UINT64_C(0x0000000000010055), UINT64_C(0x131aeacfa0880754), UINT64_C(0x0c2af94fa0720680),
        UINT64_C(0x131aeacb74880675), UINT64_C(0x0c2aed8b747205a1)
    },
    {   UINT64_C(0x000000000000657e), UINT64_C(0x1f4316cf18a886e8), UINT64_C(0x1f43144f08a666ee),
        UINT64_C(0x1f43144ed4a6a6e8), UINT64_C(0x1f43168f3caa46fb)
    },
    {   UINT64_C(0x0000000000010057), UINT64_C(0x0fa2f24f587e0755), UINT64_C(0x0c2af20f6c6ea67d),
        UINT64_C(0x0fa2f24c3c7e0683), UINT64_C(0x0c2af20c3c6c45a2)
    },
    {   UINT64_C(0x0000000000010058), UINT64_C(0x0fa2f24f58810753), UINT64_C(0x0c2af20f6c73c67f),
        UINT64_C(0x1162fccc3c8da675), UINT64_C(0x08b2e48c3c63860a)
    },
    {   UINT64_C(0x00000000000066d4), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf54b74732),
        UINT64_C(0x1f42f6ced4b4a710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001005a), UINT64_C(0x1693078f589e272c), UINT64_C(0x0fa2f88f7479e658),
        UINT64_C(0x1693078f589e26dd), UINT64_C(0x0fa2f88f7479e607)
    },
    {   UINT64_C(0x000000000001005b), UINT64_C(0x1693078f589e274a), UINT64_C(0x0fa2f88f747a8672),
        UINT64_C(0x1693078f589e2773), UINT64_C(0x0fa2f88f7479e69b)
    },
    {   UINT64_C(0x00000000000066d7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008f40b6c700)
    },
    {   UINT64_C(0x0000000000006585), UINT64_C(0x1f4315cedca6e6e9), UINT64_C(0x1f43144ef4a686ed),
        UINT64_C(0x1f43144ef4a686ec), UINT64_C(0x1f4313cef0a686f3)
    },
    {   UINT64_C(0x0000000000006586), UINT64_C(0x0000000000000000), UINT64_C(0x1f43144ef4a606e6),
        UINT64_C(0x1f43144ef0a646f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006587), UINT64_C(0x1f42ff8e4cb06745), UINT64_C(0x1f43450ff49006c5),
        UINT64_C(0x1f43128edcae86d5), UINT64_C(0x1f43141004b546d2)
    },
    {   UINT64_C(0x0000000000006588), UINT64_C(0x1f43080f68b486bf), UINT64_C(0x1f43060e9cb186fc),
        UINT64_C(0x1f43084fe4b666d1), UINT64_C(0x1f430c4f2cb26708)
    },
    {   UINT64_C(0x0000000000006589), UINT64_C(0x1f430c0f68b366e0), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000658a), UINT64_C(0x1f43058f64b246cb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430acf64b6c6cc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066de), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058f50b40727), UINT64_C(0x1f43020f08b0271d)
    },
    {   UINT64_C(0x00000000000066df), UINT64_C(0x1f43010ffcb40719), UINT64_C(0x1f42fccf7cb5e70c),
        UINT64_C(0x1f4300cffcb1e719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066e0), UINT64_C(0x1f42facffcb06718), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018fecaf0719), UINT64_C(0x1f42f80f18b66716)
    },
    {   UINT64_C(0x0000000000010066), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f18f747bc693),
        UINT64_C(0x08b2f84f74616693), UINT64_C(0x0f52f58fa07a4684)
    },
    {   UINT64_C(0x00000000000066e2), UINT64_C(0x1f43064f84b0671c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066e3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4308cfbcae06f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066e4), UINT64_C(0x1f4304cfecae4719), UINT64_C(0x1f43064fd8b8e720),
        UINT64_C(0x1f42ffcf94b32719), UINT64_C(0x1f42fecedcac2719)
    },
    {   UINT64_C(0x00000000000066e5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8fecb08720),
        UINT64_C(0x1f4301cfecb1c720), UINT64_C(0x1f42f80f5cb6a720)
    },
    {   UINT64_C(0x00000000000066e6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040fa4b5c6f1), UINT64_C(0x1f4302cf44b386e2)
    },
    {   UINT64_C(0x000000000001006c), UINT64_C(0x11eaf88f748a6607), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066e8), UINT64_C(0x1f42fbcf90b2e6fd), UINT64_C(0x1f42fecfacb5e6f1),
        UINT64_C(0x1f4301cef8b546f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001006e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f00bf87845aa), UINT64_C(0x0fa2f10fa07d068c)
    },
    {   UINT64_C(0x0000000000006597), UINT64_C(0x1f43088f84b346ec), UINT64_C(0x1f43118fe4b5c704),
        UINT64_C(0x1f43114facb28712), UINT64_C(0x1f43088ed8ad0715)
    },
    {   UINT64_C(0x00000000000066eb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010fdcb3670c),
        UINT64_C(0x1f42fe0ee8b3472b), UINT64_C(0x1f42fecf8cb726ed)
    },
    {   UINT64_C(0x0000000000006599), UINT64_C(0x1f430b4f0cb846f0), UINT64_C(0x1f430d0f78ac071d),
        UINT64_C(0x1f430a0e94ad873c), UINT64_C(0x1f430bcf94ade6c7)
    },
    {   UINT64_C(0x00000000000066ed), UINT64_C(0x1f42f40f74b8871f), UINT64_C(0x1f42ffcf04b8a701),
        UINT64_C(0x1f42facf8cb4e710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066ee), UINT64_C(0x1f42fb0f18b8271d), UINT64_C(0x1f42fecee0afe718),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066ef), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044ec0b726f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f0cb6470a)
    },
    {   UINT64_C(0x00000000000066f0), UINT64_C(0x1f43080f64af2732), UINT64_C(0x1f42fd8fc4b6471e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccfb8b6a716)
    },
    {   UINT64_C(0x00000000000066f1), UINT64_C(0x1f42fd4ed4b6c6f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0eb0b026f3), UINT64_C(0x1f42fccf04b766f3)
    },
    {   UINT64_C(0x000000000000659f), UINT64_C(0x1f430bcf70b3670e), UINT64_C(0x1f430e4f38b46722),
        UINT64_C(0x1f430acf44aee6d8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0f34b3871c)
    },
    {   UINT64_C(0x00000000000066f4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43008e78b24732),
        UINT64_C(0x1f43038f9cb0c732), UINT64_C(0x1f42ff0f2cb2e73a)
    },
    {   UINT64_C(0x00000000000066f5), UINT64_C(0x1f4303cfdcafc70d), UINT64_C(0x1f43004f60b1c712),
        UINT64_C(0x1f43014f04b3672d), UINT64_C(0x1f43024e34b82744)
    },
    {   UINT64_C(0x00000000000066f6), UINT64_C(0x0000000000000000), UINT64_C(0x1f43004ebcb4a71b),
        UINT64_C(0x1f42fb4ea8b3e717), UINT64_C(0x1f42fecf2caec717)
    },
    {   UINT64_C(0x00000000000066f7), UINT64_C(0x1f42fccef0b626ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e5cb546f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066f8), UINT64_C(0x1f42fb0e9cb82726), UINT64_C(0x1f43004ef8b3a70e),
        UINT64_C(0x1f43004fc0b68709), UINT64_C(0x1f42fd8ef8b3c6fc)
    },
    {   UINT64_C(0x000000000001007e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x131ae14fa085079e), UINT64_C(0x0faaf24f747a0744)
    },
    {   UINT64_C(0x00000000000066fa), UINT64_C(0x1f43054f30b20707), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f18b3e71a), UINT64_C(0x1f42f88ea8b366d2)
    },
    {   UINT64_C(0x00000000000066fb), UINT64_C(0x1f43088e68aa6719), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066fc), UINT64_C(0x1f42fd4fc0b7c70d), UINT64_C(0x1f42f80f60b68726),
        UINT64_C(0x1f42f34fb4b8e719), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066fd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f64edcb76716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f78b30711)
    },
    {   UINT64_C(0x00000000000066fe), UINT64_C(0x1f42fecfa4b5e70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065ac), UINT64_C(0x1f430a0f98ad66e3), UINT64_C(0x1f430ecf24b28719),
        UINT64_C(0x1f43078f54aee715), UINT64_C(0x1f43064e8cb3c732)
    },
    {   UINT64_C(0x00000000000065ad), UINT64_C(0x1f43070f6cb726f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43110eb4b06719), UINT64_C(0x1f42face80b52710)
    },
    {   UINT64_C(0x0000000000006701), UINT64_C(0x1f42fb0f3cb5271d), UINT64_C(0x1f42fecfb0ae470b),
        UINT64_C(0x1f42fb4f40b6c70e), UINT64_C(0x1f43028ebcb4c71d)
    },
    {   UINT64_C(0x0000000000006702), UINT64_C(0x1f42fe8edcb4471d), UINT64_C(0x1f43004f88b3a728),
        UINT64_C(0x1f42f20e6cb9873a), UINT64_C(0x1f42fccf3cafe6f8)
    },
    {   UINT64_C(0x0000000000018805), UINT64_C(0x1f42ee12c4bb84e2), UINT64_C(0x3e82ee12c53884e2),
        UINT64_C(0x105b0e4cac6fc69a), UINT64_C(0x0fa2ee0af07d035b)
    },
    {   UINT64_C(0x0000000000006704), UINT64_C(0x1f42fdceecb3a70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4f0cb44716), UINT64_C(0x1f42ffcf44b7c706)
    },
    {   UINT64_C(0x0000000000006705), UINT64_C(0x1f43010f58b44723), UINT64_C(0x1f42f88fa8b64700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018f0cae2708)
    },
    {   UINT64_C(0x0000000000006706), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43008fc0b22705)
    },
    {   UINT64_C(0x0000000000006707), UINT64_C(0x1f43004eccb80730), UINT64_C(0x1f43028f38ae670b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006708), UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cf50b04719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43014f60b36715)
    },
    {   UINT64_C(0x0000000000006709), UINT64_C(0x1f43010ed8b16715), UINT64_C(0x1f42fd4eecafc734),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065b7), UINT64_C(0x1f42fdceccb8270f), UINT64_C(0x1f43104f48b2e6f9),
        UINT64_C(0x1f42fc8efcb6871d), UINT64_C(0x1f43148ee0a90735)
    },
    {   UINT64_C(0x000000000000670b), UINT64_C(0x1f43014f34b56719), UINT64_C(0x1f42fc8e28b5c72d),
        UINT64_C(0x1f43008fc0b22705), UINT64_C(0x1f43034f10b0e702)
    },
    {   UINT64_C(0x000000000000670c), UINT64_C(0x1f42f90e70b5e729), UINT64_C(0x1f42fb8f04b4e6d6),
        UINT64_C(0x1f4301ce40b1a745), UINT64_C(0x1f4301cef8b48702)
    },
    {   UINT64_C(0x000000000000670d), UINT64_C(0x1f42fa4f38b7271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f3cb7870c), UINT64_C(0x1f4302cf0cb7a713)
    },
    {   UINT64_C(0x00000000000065bb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4eacb1e71d)
    },
    {   UINT64_C(0x00000000000065bc), UINT64_C(0x1f43108eb8b48703), UINT64_C(0x1f43100ed8af672d),
        UINT64_C(0x1f4311cf00b1a70c), UINT64_C(0x1f43074f34aa2726)
    },
    {   UINT64_C(0x00000000000065bd), UINT64_C(0x0000000000000000), UINT64_C(0x1f4311ceb4a90725),
        UINT64_C(0x1f43008f04b76731), UINT64_C(0x1f430ecea4b3471a)
    },
    {   UINT64_C(0x0000000000010096), UINT64_C(0x0de2f5cf747285a1), UINT64_C(0x0de2f5cf747205a1),
        UINT64_C(0x13faf5cf748e65a1), UINT64_C(0x0de2f5cf6878e5c3)
    },
    {   UINT64_C(0x00000000000065bf), UINT64_C(0x0000000000000000), UINT64_C(0x1f43188e88aba72d),
        UINT64_C(0x1f43068ec8b3871f), UINT64_C(0x1f4312ceb0ae672f)
    },
    {   UINT64_C(0x0000000000006713), UINT64_C(0x1f43008e98b92718), UINT64_C(0x1f42fe8f18b58715),
        UINT64_C(0x1f43014ef4b8c720), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010099), UINT64_C(0x0f02ef0f7879e5aa), UINT64_C(0x0fa2f2cbd87a45aa),
        UINT64_C(0x0fa2f2cf7c7c4693), UINT64_C(0x0fa2ec8bfc7c4693)
    },
    {   UINT64_C(0x0000000000018817), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0fa03e83e8)
    },
    {   UINT64_C(0x000000000001009b), UINT64_C(0x0bbaf10f746c4693), UINT64_C(0x07e2f00bfc616693),
        UINT64_C(0x11faf84bfc81a693), UINT64_C(0x1692f10f7c9685a1)
    },
    {   UINT64_C(0x00000000000065c4), UINT64_C(0x1f4318cedcac4707), UINT64_C(0x1f43168f04ada721),
        UINT64_C(0x1f43078ff4b8a6dd), UINT64_C(0x1f43090e70b40729)
    },
    {   UINT64_C(0x000000000001009d), UINT64_C(0x10dae90f808965a1), UINT64_C(0x0fa2f4cf747905a1),
        UINT64_C(0x1672fa4f889ac5a1), UINT64_C(0x14e2f58f908e45c3)
    },
    {   UINT64_C(0x0000000000006719), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8f80b42703), UINT64_C(0x1f43024f50b1a727)
    },
    {   UINT64_C(0x000000000001009f), UINT64_C(0x0a6ae90bfc7205a1), UINT64_C(0x0c32f40bfc7205a1),
        UINT64_C(0x0c8af94fa07385be), UINT64_C(0x0a92fe8fa06325be)
    },
    {   UINT64_C(0x00000000000100a0), UINT64_C(0x0edaf44fa07d25b8), UINT64_C(0x0edaf44fa08725b8),
        UINT64_C(0x0c2af04c386d85a2), UINT64_C(0x0cfad28bfc86a693)
    },
    {   UINT64_C(0x000000000000671c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f88b2a6f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84e8cb8a71a)
    },
    {   UINT64_C(0x00000000000100a2), UINT64_C(0x08b2ef8bfc65060a), UINT64_C(0x0fa2f04f747a65a1),
        UINT64_C(0x10cafa4f788385aa), UINT64_C(0x0fa2fb0f7879e5a1)
    },
    {   UINT64_C(0x000000000000671e), UINT64_C(0x1f42fdcf2cb42728), UINT64_C(0x1f43004e7cb3c71e),
        UINT64_C(0x1f42fccea0b72711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000100a4), UINT64_C(0x0caaf24c38742594), UINT64_C(0x0c4af24ee46e0594),
        UINT64_C(0x0ceaf34bfc7f25aa), UINT64_C(0x0e4aefcadc8285aa)
    },
    {   UINT64_C(0x0000000000006720), UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e94ad673e),
        UINT64_C(0x1f4303cfe4aea70c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006721), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff9008b60706),
        UINT64_C(0x1f43048ed8b0e713), UINT64_C(0x1f43074fd8abc713)
    },
    {   UINT64_C(0x00000000000065cf), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8e48b42722),
        UINT64_C(0x1f430d4ef8ade71e), UINT64_C(0x1f4305ce48b5c72c)
    },
    {   UINT64_C(0x00000000000065d0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ea0ac472e), UINT64_C(0x1f43004ef8ad071e)
    },
    {   UINT64_C(0x00000000000065d1), UINT64_C(0x1f43078e48aca72c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0f64b76701), UINT64_C(0x1f430cceb0b0672f)
    },
    {   UINT64_C(0x0000000000006725), UINT64_C(0x1f43074f1cb1871b), UINT64_C(0x1f4307505caf471b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006726), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43074efcb7271b), UINT64_C(0x1f4307500cb7471b)
    },
    {   UINT64_C(0x0000000000006727), UINT64_C(0x1f43080ee8b26733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43090f48b48721), UINT64_C(0x1f43074f24b1671b)
    },
    {   UINT64_C(0x00000000000100ad), UINT64_C(0x0a02ee150466469a), UINT64_C(0x0a02ee122c68669a),
        UINT64_C(0x0c32ef9504784694), UINT64_C(0x0fa2f1d4bc7ee69a)
    },
    {   UINT64_C(0x00000000000100ae), UINT64_C(0x0a52f212c86b269d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6b01d68c5c0696)
    },
    {   UINT64_C(0x000000000000672a), UINT64_C(0x1f43140f60aa8719), UINT64_C(0x1f43038fd8b6a71b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8eb8b2a71c)
    },
    {   UINT64_C(0x000000000000672b), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074f04b0c72c),
        UINT64_C(0x1f43054f14b5c728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000672c), UINT64_C(0x1f42fbcf50b5a6fc), UINT64_C(0x1f43048f84b2e71b),
        UINT64_C(0x1f4300100cb6e700), UINT64_C(0x1f43074fdcb1871b)
    },
    {   UINT64_C(0x000000000000672d), UINT64_C(0x1f43029038ad4709), UINT64_C(0x1f43029018b5671d),
        UINT64_C(0x1f43024fc0b6471d), UINT64_C(0x1f43028f30adc71d)
    },
    {   UINT64_C(0x000000000000672e), UINT64_C(0x1f43030f9caf4710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43025014b9071d), UINT64_C(0x1f43058fdcaf6720)
    },
    {   UINT64_C(0x000000000000672f), UINT64_C(0x1f43038edcb2672f), UINT64_C(0x1f42ff0edcb4471a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065dd), UINT64_C(0x1f42fb8f58b5270d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f08b5c713), UINT64_C(0x1f42fe8ee8ae0722)
    },
    {   UINT64_C(0x00000000000065de), UINT64_C(0x1f42fccf18b42726), UINT64_C(0x1f4302cf18b12726),
        UINT64_C(0x1f42f8ceacb6c70c), UINT64_C(0x1f42f8cf28b62723)
    },
    {   UINT64_C(0x00000000000065df), UINT64_C(0x1f42f0ceecb7a711), UINT64_C(0x1f42fcce98b38714),
        UINT64_C(0x1f42f8cedcb2070c), UINT64_C(0x1f42f68f18b48723)
    },
    {   UINT64_C(0x0000000000006733), UINT64_C(0x1f4305cec8b3a72c), UINT64_C(0x1f42fece94af2716),
        UINT64_C(0x1f4302cf74ac26f4), UINT64_C(0x1f43058f28af6719)
    },
    {   UINT64_C(0x00000000000100b9), UINT64_C(0x09f2f014d469a737), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065e2), UINT64_C(0x1f42f68eecb94714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8f18b2a722), UINT64_C(0x1f42f90ef8b34725)
    },
    {   UINT64_C(0x00000000000100bb), UINT64_C(0x0a02f1cd5464a42e), UINT64_C(0x0caaf0978c6fe681),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006737), UINT64_C(0x1f42fccff0b6c708), UINT64_C(0x1f42ffceecb34733),
        UINT64_C(0x1f4302cea8b6072c), UINT64_C(0x1f42ffceb8b1c738)
    },
    {   UINT64_C(0x00000000000100bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6ad68c305ba377)
    },
    {   UINT64_C(0x0000000000006739), UINT64_C(0x0000000000000000), UINT64_C(0x1f431ace6cade73a),
        UINT64_C(0x1f42fc4efcb4272a), UINT64_C(0x1f4314ceb4b38745)
    },
    {   UINT64_C(0x000000000000673a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43004ef0b2472a),
        UINT64_C(0x1f43144e38ad270a), UINT64_C(0x1f42fe0f64b38719)
    },
    {   UINT64_C(0x000000000000673b), UINT64_C(0x1f43078e6caec745), UINT64_C(0x1f42f6ce78b3e71e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000673c), UINT64_C(0x1f42fecedcb32720), UINT64_C(0x1f42f84edcb1c720),
        UINT64_C(0x1f42f94eccb14734), UINT64_C(0x1f42ff4ed0b78722)
    },
    {   UINT64_C(0x000000000000673d), UINT64_C(0x1f4309ce8cb40741), UINT64_C(0x1f4300ce90b48720),
        UINT64_C(0x1f43118e90ad673a), UINT64_C(0x1f43018e7cb3471d)
    },
    {   UINT64_C(0x000000000000673e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ce64b3670c),
        UINT64_C(0x1f42fece7cb3471e), UINT64_C(0x1f42fb4e70b3471d)
    },
    {   UINT64_C(0x000000000000673f), UINT64_C(0x1f43040e7cb3e71d), UINT64_C(0x1f42fc0e7cb3471d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004e78b30712)
    },
    {   UINT64_C(0x0000000000006740), UINT64_C(0x1f42f98e70b3871d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ec8b78718)
    },
    {   UINT64_C(0x0000000000006741), UINT64_C(0x1f42fb0e54b20716), UINT64_C(0x1f42fb8e70b3871d),
        UINT64_C(0x1f42fc8ebcb4e73b), UINT64_C(0x1f43020ef8b4a737)
    },
    {   UINT64_C(0x0000000000006742), UINT64_C(0x1f42f48e5cb20719), UINT64_C(0x1f42fd0ec4b4273b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006743), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010ef4b22721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000065f1), UINT64_C(0x1f42f78e60b6c739), UINT64_C(0x1f42f78f18b4e71f),
        UINT64_C(0x1f42fd8f44b18723), UINT64_C(0x1f43028e44af2730)
    },
    {   UINT64_C(0x0000000000006745), UINT64_C(0x1f42fa4ec4b4c711), UINT64_C(0x1f42fa4f34b3873d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43038e90b24724)
    },
    {   UINT64_C(0x0000000000006746), UINT64_C(0x1f42fd8ef0b3a704), UINT64_C(0x1f43030f28b5e71d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4316ce84aca72c)
    },
    {   UINT64_C(0x00000000000065f4), UINT64_C(0x1f42fa4ed8b6470f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8e8cb40742)
    },
    {   UINT64_C(0x0000000000006748), UINT64_C(0x1f42fb8ed4b4870c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8ec4b6670d), UINT64_C(0x1f43008ebcb2473f)
    },
    {   UINT64_C(0x0000000000006749), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98ed0ba271b),
        UINT64_C(0x1f430c8e54b6074a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000674a), UINT64_C(0x1f43048f0cb36728), UINT64_C(0x1f43030f1cb6071c),
        UINT64_C(0x1f42f78ee0b6a73a), UINT64_C(0x1f42ff0f08b3273d)
    },
    {   UINT64_C(0x0000000000004002), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2ee0fa03e83e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000674c), UINT64_C(0x1f42fe4edcb12714), UINT64_C(0x1f42f70ed4b30714),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccea8b2e709)
    },
    {   UINT64_C(0x00000000000065fa), UINT64_C(0x0000000000000000), UINT64_C(0x1f43074f28ae2724),
        UINT64_C(0x1f42ff0f34ad2710), UINT64_C(0x1f43004fa4b62701)
    },
    {   UINT64_C(0x000000000000674e), UINT64_C(0x1f42ffced4b30714), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4ed8b6273b)
    },
    {   UINT64_C(0x000000000000674f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43154e30a8a72c),
        UINT64_C(0x1f43050f04b3c70f), UINT64_C(0x1f42f90ebcb52737)
    },
    {   UINT64_C(0x00000000000065fd), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070f0cab0706),
        UINT64_C(0x1f42f4d028ac4702), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004008), UINT64_C(0x0fa2ee0fa03e83e8), UINT64_C(0x0fa323106069a6aa),
        UINT64_C(0x0fa316575c6e069c), UINT64_C(0x0fa2f150707b6721)
    },
    {   UINT64_C(0x0000000000006752), UINT64_C(0x1f42fe4e70b36731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030ebcb5671d), UINT64_C(0x1f42fb4e8cb3e71a)
    },
    {   UINT64_C(0x0000000000006753), UINT64_C(0x1f42f6ce7cb7a710), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0ebcb4e718)
    },
    {   UINT64_C(0x0000000000006754), UINT64_C(0x1f42fc8ed0b4472e), UINT64_C(0x1f43140f44b20735),
        UINT64_C(0x1f42f98ef8b38729), UINT64_C(0x1f42fccea0b3a712)
    },
    {   UINT64_C(0x000000000000400c), UINT64_C(0x0fa2fa10a0772698), UINT64_C(0x0fa30890c86f468e),
        UINT64_C(0x0fa2fa50c877a698), UINT64_C(0x0fa2fa90a076c698)
    },
    {   UINT64_C(0x000000000000400d), UINT64_C(0x0fa2f610c879068e), UINT64_C(0x0fa2f9d0a0774683),
        UINT64_C(0x0fa2fa10a0772698), UINT64_C(0x0fa2fa1084770684)
    },
    {   UINT64_C(0x0000000000006757), UINT64_C(0x1f42ff0f20b3a71d), UINT64_C(0x1f42f98f48b5e740),
        UINT64_C(0x1f43178ddca6c723), UINT64_C(0x1f43134e50a86733)
    },
    {   UINT64_C(0x000000000000400f), UINT64_C(0x0fa2f392487a661b), UINT64_C(0x0fa2f393807a65eb),
        UINT64_C(0x0fa2f392607a6621), UINT64_C(0x0fa2fe5078750698)
    },
    {   UINT64_C(0x0000000000004010), UINT64_C(0x0fa2f150a07b8629), UINT64_C(0x0fa2f0d0c87c068e),
        UINT64_C(0x0fa2f3d0c87a268e), UINT64_C(0x0fa2f7d0a0794698)
    },
    {   UINT64_C(0x000000000001889b), UINT64_C(0x0e93034c206e26c2), UINT64_C(0x156afa0e988e2624),
        UINT64_C(0x156afc0d308c267e), UINT64_C(0x156afc0d308c267e)
    },
    {   UINT64_C(0x000000000000675b), UINT64_C(0x1f42f80ef4b5872f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0ed0b52722)
    },
    {   UINT64_C(0x00000000000100e1), UINT64_C(0x090b16175866c671), UINT64_C(0x0a6aff97586ee671),
        UINT64_C(0x131ae14fa085068e), UINT64_C(0x0a6b1394c85f25a1)
    },
    {   UINT64_C(0x0000000000004014), UINT64_C(0x0fa2f2d0c87a668e), UINT64_C(0x0fa2f58f6c7ae698),
        UINT64_C(0x0fa2f110c87b468e), UINT64_C(0x0fa2f7109c7726ad)
    },
    {   UINT64_C(0x0000000000004015), UINT64_C(0x0fa2f490c879e68e), UINT64_C(0x0fa2f450a078a68e),
        UINT64_C(0x0fa2f1d0c87b268e), UINT64_C(0x0fa2f090c87be68e)
    },
    {   UINT64_C(0x0000000000004016), UINT64_C(0x0fa2f310c87aa68e), UINT64_C(0x0fa2ef50c87c868e),
        UINT64_C(0x0fa2f6d0c878c68e), UINT64_C(0x0fa3114e946b2721)
    },
    {   UINT64_C(0x000000000001889e), UINT64_C(0x156afa0bb88e26d6), UINT64_C(0x156afa0bb88e26d6),
        UINT64_C(0x156afe0f388c25fc), UINT64_C(0x156afe0f388c25fc)
    },
    {   UINT64_C(0x0000000000004018), UINT64_C(0x0fa30f97506a46a2), UINT64_C(0x0fa2f650a07905cc),
        UINT64_C(0x0fa2f690a078c698), UINT64_C(0x0fa2fc50a075e5cb)
    },
    {   UINT64_C(0x0000000000004019), UINT64_C(0x0fa2fb10a076a698), UINT64_C(0x0fa2fad0a476c5cc),
        UINT64_C(0x0fa2f650c878e698), UINT64_C(0x0fa2facef87705ce)
    },
    {   UINT64_C(0x0000000000006763), UINT64_C(0x1f42f38ed0b5271a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74f00b6c732)
    },
    {   UINT64_C(0x0000000000006611), UINT64_C(0x1f4302cf5caf26cc), UINT64_C(0x1f42fd4f4caf270e),
        UINT64_C(0x1f42fb8fc8b006f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000401c), UINT64_C(0x0fa2f60ebc7905cc), UINT64_C(0x0fa2f64ebc7905cc),
        UINT64_C(0x0fa2f550c87465cc), UINT64_C(0x0fa302509c74a5da)
    },
    {   UINT64_C(0x000000000000401d), UINT64_C(0x0fa2f950a077865d), UINT64_C(0x0fa2f950a07785cc),
        UINT64_C(0x0fa2fb10c87825cc), UINT64_C(0x0fa2f210c87b05cc)
    },
    {   UINT64_C(0x0000000000006767), UINT64_C(0x1f43008ec0b26730), UINT64_C(0x1f43044e88b4472f),
        UINT64_C(0x1f42f74e88b4472f), UINT64_C(0x1f42fd0e88b44720)
    },
    {   UINT64_C(0x000000000000401f), UINT64_C(0x0fa3258e94624721), UINT64_C(0x0fa3078e9471e721),
        UINT64_C(0x0fa2f858487806a5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004020), UINT64_C(0x0fa2f74f84758687), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006617), UINT64_C(0x1f42fc4f2cb16723), UINT64_C(0x1f43004f2cb166e9),
        UINT64_C(0x1f42facf44ade724), UINT64_C(0x1f42fb8f1caea6e9)
    },
    {   UINT64_C(0x00000000000100f0), UINT64_C(0x0fc2f2cf58818594), UINT64_C(0x0fc2e40c6c7965a1),
        UINT64_C(0x0e32f58cbc7725a1), UINT64_C(0x0f92f4cf7480e594)
    },
    {   UINT64_C(0x0000000000006619), UINT64_C(0x1f42fdcf1caf8722), UINT64_C(0x1f42fecf50ad4702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f60b28735)
    },
    {   UINT64_C(0x000000000000676d), UINT64_C(0x1f42fd8ebcb52721), UINT64_C(0x1f4301cefcb26733),
        UINT64_C(0x1f42f58eb4b3e72d), UINT64_C(0x1f4300ce88b4472f)
    },
    {   UINT64_C(0x000000000000676e), UINT64_C(0x1f42f88e90b4270b), UINT64_C(0x1f43020e88b44733),
        UINT64_C(0x1f42f68ee4b16730), UINT64_C(0x1f42fcce88b4472f)
    },
    {   UINT64_C(0x000000000000676f), UINT64_C(0x1f42fa0ef8b1a738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e90b6471d), UINT64_C(0x1f43144dfca8e743)
    },
    {   UINT64_C(0x000000000000661d), UINT64_C(0x1f42fa8f04ae8732), UINT64_C(0x1f431acf04a7c6ff),
        UINT64_C(0x1f43084f2cac86f8), UINT64_C(0x1f43100f2cad26ed)
    },
    {   UINT64_C(0x000000000000661e), UINT64_C(0x1f42ff8f58a9e6e4), UINT64_C(0x1f4302cf0cad86ee),
        UINT64_C(0x1f42fc4f78ac26ec), UINT64_C(0x1f43020f7ca9c6fe)
    },
    {   UINT64_C(0x0000000000004029), UINT64_C(0x0fa2f2104c7b05e2), UINT64_C(0x0fa2f30ef07a8659),
        UINT64_C(0x0fa3268ddc60c685), UINT64_C(0x0fa2ffcdac742693)
    },
    {   UINT64_C(0x0000000000006773), UINT64_C(0x1f42fecf14b1e714), UINT64_C(0x1f42ff0e88b4472f),
        UINT64_C(0x1f42fa0e88b4472f), UINT64_C(0x1f42fdcea8b3a72e)
    },
    {   UINT64_C(0x000000000000402b), UINT64_C(0x0fa2ff91e0744542), UINT64_C(0x0fa30dd38c6d2513),
        UINT64_C(0x0fa2f094407bc6f2), UINT64_C(0x1f42ee1cccbb2757)
    },
    {   UINT64_C(0x0000000000006775), UINT64_C(0x0000000000000000), UINT64_C(0x1f43124e08a7a738),
        UINT64_C(0x1f431c8de8af073c), UINT64_C(0x1f43028ec8b2e741)
    },
    {   UINT64_C(0x000000000000402d), UINT64_C(0x0fa31856d467e65f), UINT64_C(0x0fa2f28e847ac5e4),
        UINT64_C(0x0fa2f58e5c794660), UINT64_C(0x0fa31e531464e536)
    },
    {   UINT64_C(0x000000000000402e), UINT64_C(0x0fa3174cb86863f3), UINT64_C(0x0fa31394506a468b),
        UINT64_C(0x0fa30ad3a46ea672), UINT64_C(0x0fa2f190787b45de)
    },
    {   UINT64_C(0x000000000000402f), UINT64_C(0x0fa2f2ce707aa66a), UINT64_C(0x0fa2f18e707b466a),
        UINT64_C(0x0fa2f14e707b666c), UINT64_C(0x0c02f28efc6c46e9)
    },
    {   UINT64_C(0x0000000000006779), UINT64_C(0x1f42feceb0b2c723), UINT64_C(0x1f431a8e78a4a738),
        UINT64_C(0x1f42fccebcb2c735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006627), UINT64_C(0x1f42fc8efcab8719), UINT64_C(0x1f43030fa4a98711),
        UINT64_C(0x1f42facf7cac66ef), UINT64_C(0x1f43018f7cac6708)
    },
    {   UINT64_C(0x000000000000677b), UINT64_C(0x1f42ff0ea8b40727), UINT64_C(0x1f42ffcf18b3472c),
        UINT64_C(0x1f42fd0eb0b2c723), UINT64_C(0x1f42f90ed8b3a72d)
    },
    {   UINT64_C(0x000000000001887e), UINT64_C(0x1cf2fbd088aa25aa), UINT64_C(0x1cf2fa1088ac45aa),
        UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x000000000000677d), UINT64_C(0x1f42fc8ea8b40727), UINT64_C(0x1f42f74ed8b4270d),
        UINT64_C(0x1f42ff8eb0b2c725), UINT64_C(0x1f42fc4eb0b1c72b)
    },
    {   UINT64_C(0x000000000000677e), UINT64_C(0x1f43020ec8b4472f), UINT64_C(0x1f42fc4e4ca9073c),
        UINT64_C(0x1f43124e84b2e73d), UINT64_C(0x1f42fc0f0cb72721)
    },
    {   UINT64_C(0x0000000000010104), UINT64_C(0x131ae1cfa0854684), UINT64_C(0x1272e70fa089e675),
        UINT64_C(0x12aae84fa085e675), UINT64_C(0x11cae50fa089e675)
    },
    {   UINT64_C(0x000000000000662d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4f7cb866d3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010106), UINT64_C(0x1622e7cfa09c2675), UINT64_C(0x1622e7cfa09c275f),
        UINT64_C(0x136ae70fa090a67d), UINT64_C(0x15dae54f609b0675)
    },
    {   UINT64_C(0x000000000000662f), UINT64_C(0x1f42f54f80b5c701), UINT64_C(0x1f42fdcfccb7e6fc),
        UINT64_C(0x1f42f48f60b986c3), UINT64_C(0x1f42f2cfdcba46cb)
    },
    {   UINT64_C(0x000000000000403a), UINT64_C(0x1f4337d0a096a6a6), UINT64_C(0x1f4337d0a096a6a6),
        UINT64_C(0x1f4337d0a096a698), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000403b), UINT64_C(0x1f4343d0c88fc6a6), UINT64_C(0x1f434590c88fc6a6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000403c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4337d09c96c6a6), UINT64_C(0x1f4337d09c96c6a6)
    },
    {   UINT64_C(0x000000000001010b), UINT64_C(0x12aae70fa081a675), UINT64_C(0x1492f1cf588e0682),
        UINT64_C(0x1b6ae60f58a8c682), UINT64_C(0x13dae1cfa0930675)
    },
    {   UINT64_C(0x000000000000403e), UINT64_C(0x0000000000000000), UINT64_C(0x1f433190a099e6a6),
        UINT64_C(0x1f433190a099e6a6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001010d), UINT64_C(0x0f4af58f74794693), UINT64_C(0x0dc2f68f747345a1),
        UINT64_C(0x18faee0f74a165a1), UINT64_C(0x0bc2e98f746b25a1)
    },
    {   UINT64_C(0x0000000000004040), UINT64_C(0x0000000000000000), UINT64_C(0x1f4332d09c94e662),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006637), UINT64_C(0x1f42f08f78ba2718), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a0f88b366a1), UINT64_C(0x1f42fc0e5cb54735)
    },
    {   UINT64_C(0x000000000000678b), UINT64_C(0x1f42fa4f5caf0715), UINT64_C(0x1f43064f20b00721),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e80b1c730)
    },
    {   UINT64_C(0x0000000000010111), UINT64_C(0x15f2f54c6c932693), UINT64_C(0x0de2e54f747565a1),
        UINT64_C(0x107af54cc87c65a1), UINT64_C(0x0f12f88f7c76c5a1)
    },
    {   UINT64_C(0x0000000000004044), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4337d0a096a662)
    },
    {   UINT64_C(0x0000000000010113), UINT64_C(0x0ebafa8f7474a5a1), UINT64_C(0x0cc2efcf746f65a1),
        UINT64_C(0x1512f34f749065a1), UINT64_C(0x0f0ae7cf7c76a598)
    },
    {   UINT64_C(0x0000000000004046), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4337d0a096a698)
    },
    {   UINT64_C(0x0000000000006790), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010f48aec714),
        UINT64_C(0x1f42ff4f48aee70b), UINT64_C(0x1f43140f30b0a70c)
    },
    {   UINT64_C(0x0000000000006791), UINT64_C(0x1f42ff8f3caca711), UINT64_C(0x1f42fb8f40b0a705),
        UINT64_C(0x1f42fdcf34b14715), UINT64_C(0x1f43038f1cb12712)
    },
    {   UINT64_C(0x0000000000010117), UINT64_C(0x0f5af28f747b2680), UINT64_C(0x0000000000000000),
        UINT64_C(0x0d2adecc6872a683), UINT64_C(0x107af54cc87c65a1)
    },
    {   UINT64_C(0x000000000000404a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f434110c88fc662)
    },
    {   UINT64_C(0x0000000000006641), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50e5cb6672e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f40f2caf26fe)
    },
    {   UINT64_C(0x0000000000006795), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f40b406fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4301cf30b206f9)
    },
    {   UINT64_C(0x0000000000006643), UINT64_C(0x1f42f50e7cb48728), UINT64_C(0x1f42f78edcb06707),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001011c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1642fd0f5895e683), UINT64_C(0x0f4af54f747945a1)
    },
    {   UINT64_C(0x000000000001011d), UINT64_C(0x1433010f589b467e), UINT64_C(0x0e82f68f588085a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006799), UINT64_C(0x1f43008f3caf6713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0f2cb46702), UINT64_C(0x1f42f94f5cb26705)
    },
    {   UINT64_C(0x0000000000004051), UINT64_C(0x1f42ee0fa03e83e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004052), UINT64_C(0x1f42ee0fa03e83e8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004053), UINT64_C(0x0000000000000000), UINT64_C(0x1f4337d09c96c662),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000679d), UINT64_C(0x0000000000000000), UINT64_C(0x1f4320cea0a10726),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000679e), UINT64_C(0x1f42fb0f28b506fe), UINT64_C(0x1f43015028b6a723),
        UINT64_C(0x1f43054f40b44732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010124), UINT64_C(0x1062e68fa08c46f7), UINT64_C(0x0a8af88f747105e3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188be), UINT64_C(0x156afd0cd08ca5fc), UINT64_C(0x156afd4f348ca5fb),
        UINT64_C(0x19bafbcf349de5fb), UINT64_C(0x156afccf348c85fb)
    },
    {   UINT64_C(0x000000000000664e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f40e68b56725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a2), UINT64_C(0x1f42fd4f28a82739), UINT64_C(0x1f43094e5cae071d),
        UINT64_C(0x1f430a4e4cada715), UINT64_C(0x1f42f58e5cade738)
    },
    {   UINT64_C(0x00000000000067a3), UINT64_C(0x1f42f88ef8ab8735), UINT64_C(0x1f42fa8e3cb1e735),
        UINT64_C(0x1f42f88ebcaf2735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188a6), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x00000000000067a5), UINT64_C(0x1f42fa8e90b22736), UINT64_C(0x1f43028e64b16720),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43064f10b0c725)
    },
    {   UINT64_C(0x0000000000006653), UINT64_C(0x1f42f70f70ae6721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facee0b5471d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a7), UINT64_C(0x1f42fa8e40aa8742), UINT64_C(0x1f42fb8eccac0736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74e3cb3e715)
    },
    {   UINT64_C(0x00000000000067a8), UINT64_C(0x1f42fa8e44a88741), UINT64_C(0x1f42f88e7caa873b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067a9), UINT64_C(0x1f43008ee0b0e73c), UINT64_C(0x1f43060e68b7673a),
        UINT64_C(0x1f4303cf00b14729), UINT64_C(0x1f42fdce68b5472f)
    },
    {   UINT64_C(0x00000000000067aa), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e5cb3e72f),
        UINT64_C(0x1f42f84e54b66743), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067ab), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90eccb48730),
        UINT64_C(0x1f42fbceb4b3a727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006659), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9d014b7a706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf50b1a6fc)
    },
    {   UINT64_C(0x00000000000067ad), UINT64_C(0x1f42f98e7cb2671b), UINT64_C(0x1f42f8cf10b00718),
        UINT64_C(0x1f43008eb8b2e730), UINT64_C(0x1f42f8ceb0b46719)
    },
    {   UINT64_C(0x000000000000665b), UINT64_C(0x1f43018fa8ab6709), UINT64_C(0x1f42f7cfd8b6a704),
        UINT64_C(0x1f42f90fd4b2c70e), UINT64_C(0x1f42fc8edcb0c713)
    },
    {   UINT64_C(0x00000000000067af), UINT64_C(0x1f42f88f00afa739), UINT64_C(0x1f42fc4f50af672e),
        UINT64_C(0x1f42fb4ee0b3072a), UINT64_C(0x1f43014e0cb1c72e)
    },
    {   UINT64_C(0x00000000000188b2), UINT64_C(0x1952fb8e609d062e), UINT64_C(0x1953054e6098062e),
        UINT64_C(0x1953054e6098062e), UINT64_C(0x1953008e609a862e)
    },
    {   UINT64_C(0x00000000000188b3), UINT64_C(0x1953008e609a862e), UINT64_C(0x156afa107c8e25a5),
        UINT64_C(0x1222f5cf3c8325fc), UINT64_C(0x1222f5cf3c8325fc)
    },
    {   UINT64_C(0x00000000000067b2), UINT64_C(0x1f42fecf20b44711), UINT64_C(0x1f4313ced8ac2720),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006660), UINT64_C(0x1f42fd8f58b6e708), UINT64_C(0x1f42f80f7cb4c6fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006661), UINT64_C(0x1f42f9d02cb6270d), UINT64_C(0x1f42fd9000afa70b),
        UINT64_C(0x1f42f8cee8b2271a), UINT64_C(0x1f42fd8ff8b60708)
    },
    {   UINT64_C(0x00000000000067b5), UINT64_C(0x1f42f84eecb926ff), UINT64_C(0x1f43010f68b686e2),
        UINT64_C(0x1f43054f90b626be), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067b6), UINT64_C(0x1f43028fb8b626f3), UINT64_C(0x1f42ffcf40b2a720),
        UINT64_C(0x1f42fb0f68b7e6d9), UINT64_C(0x1f43064eccb32734)
    },
    {   UINT64_C(0x00000000000067b7), UINT64_C(0x1f42ff8f78af0702), UINT64_C(0x1f42fb0f74b106fb),
        UINT64_C(0x1f43084f48af66d2), UINT64_C(0x1f43034f1cb28715)
    },
    {   UINT64_C(0x00000000000188ba), UINT64_C(0x156afa0b448e2686), UINT64_C(0x156afa0b448e2686),
        UINT64_C(0x13daf40d3087267e), UINT64_C(0x13dafb8d308ae67e)
    },
    {   UINT64_C(0x0000000000006666), UINT64_C(0x1f43018f80b0a70b), UINT64_C(0x1f43068ec0b68731),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067ba), UINT64_C(0x1f4310ce8cade72b), UINT64_C(0x1f42ff4f58b1a6fe),
        UINT64_C(0x1f43024ef0b64700), UINT64_C(0x1f43064f68b246b8)
    },
    {   UINT64_C(0x00000000000067bb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058e40b7e70d),
        UINT64_C(0x1f43000f54b4670a), UINT64_C(0x1f43024f20b5470b)
    },
    {   UINT64_C(0x0000000000004073), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4332d09c94e698), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004074), UINT64_C(0x1f433f50c88fc698), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4337d09c96c698), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004075), UINT64_C(0x1f433190a099e698), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f433190a099e6b0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067bf), UINT64_C(0x1f43058f78aca709), UINT64_C(0x1f43194f38b1c701),
        UINT64_C(0x1f43220fc8b0e6e6), UINT64_C(0x1f43178eccb1872a)
    },
    {   UINT64_C(0x00000000000067c0), UINT64_C(0x1f43114e8cac6731), UINT64_C(0x1f43114e60b00731),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067c1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8f70b4a6d5), UINT64_C(0x1f430e8eb0b20731)
    },
    {   UINT64_C(0x00000000000067c2), UINT64_C(0x1f4306cea8af0742), UINT64_C(0x1f42ffce90b04739),
        UINT64_C(0x1f430e0eacaf0738), UINT64_C(0x1f4311cee0ada721)
    },
    {   UINT64_C(0x00000000000067c3), UINT64_C(0x0000000000000000), UINT64_C(0x1f431f0ea8ac6725),
        UINT64_C(0x1f430a4ef0a9e71f), UINT64_C(0x1f430ece94a886ef)
    },
    {   UINT64_C(0x00000000000067c4), UINT64_C(0x1f430ccea0a46726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dcec0a6e731), UINT64_C(0x1f43054f50b5a703)
    },
    {   UINT64_C(0x00000000000067c5), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4e64b3a739),
        UINT64_C(0x1f430b8eb0b22729), UINT64_C(0x1f43104e94af270f)
    },
    {   UINT64_C(0x00000000000067c6), UINT64_C(0x1f430ece40aba745), UINT64_C(0x1f430b0e6cad8735),
        UINT64_C(0x1f43114e38b4273d), UINT64_C(0x1f430c4e60b4a731)
    },
    {   UINT64_C(0x00000000000067c7), UINT64_C(0x1f430d0e10b42741), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108e74ac8745), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067c8), UINT64_C(0x1f43158f04b5a72a), UINT64_C(0x1f43030e4cb10729),
        UINT64_C(0x1f42fc8f5cb78703), UINT64_C(0x1f4300cf2cb726f6)
    },
    {   UINT64_C(0x00000000000067c9), UINT64_C(0x1f43050f2cb746f6), UINT64_C(0x1f43068f8cb4e701),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067ca), UINT64_C(0x1f43068f8cb4e700), UINT64_C(0x1f43068f8cb4e700),
        UINT64_C(0x1f43040f8cb4e700), UINT64_C(0x1f43068f8cb4e700)
    },
    {   UINT64_C(0x00000000000067cb), UINT64_C(0x1f42fecf68b5670c), UINT64_C(0x1f43030f7cb56715),
        UINT64_C(0x1f42fb8ee8b48718), UINT64_C(0x1f43004f28b706fc)
    },
    {   UINT64_C(0x00000000000067cc), UINT64_C(0x1f42ff4f5cb3a708), UINT64_C(0x1f42ff4f5cb3a71d),
        UINT64_C(0x1f42ff4f5cb3a708), UINT64_C(0x1f42ff4f5cb3a708)
    },
    {   UINT64_C(0x00000000000067cd), UINT64_C(0x1f42fb8f04b64704), UINT64_C(0x1f42ff4f5cb3a720),
        UINT64_C(0x1f42ff4f5cb3a70a), UINT64_C(0x1f43050f6cb96722)
    },
    {   UINT64_C(0x000000000000667b), UINT64_C(0x1f42f8cf18b606bc), UINT64_C(0x1f42fc4f7caf4701),
        UINT64_C(0x1f42f78f90b2a6eb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067cf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f18b6a737),
        UINT64_C(0x1f42fc0e8cb48744), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067d0), UINT64_C(0x1f42fc4e6cb7c73f), UINT64_C(0x1f42fc4e6cad473f),
        UINT64_C(0x1f42fc4ee0b5c740), UINT64_C(0x1f42fc0ee8b1e740)
    },
    {   UINT64_C(0x00000000000067d1), UINT64_C(0x1f42fe0ee8b44720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4edcb1073d), UINT64_C(0x1f42f80f18b42731)
    },
    {   UINT64_C(0x00000000000067d2), UINT64_C(0x1f42fdce90b7e739), UINT64_C(0x1f42fcce8cb12722),
        UINT64_C(0x1f43044e58b80751), UINT64_C(0x1f430b8e54b74741)
    },
    {   UINT64_C(0x00000000000067d3), UINT64_C(0x1f4305ce28b3c731), UINT64_C(0x1f430f4e24b0e733),
        UINT64_C(0x1f430e4e38b22731), UINT64_C(0x1f43088e68b1c739)
    },
    {   UINT64_C(0x00000000000067d4), UINT64_C(0x1f43104eacb4e71d), UINT64_C(0x1f431a8f48a9a732),
        UINT64_C(0x1f430c4eb0aee732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067d5), UINT64_C(0x1f42fbcf7cb4a6ed), UINT64_C(0x1f42f4cfacb646f2),
        UINT64_C(0x1f42f7cee4b7a702), UINT64_C(0x1f42face8cb4870d)
    },
    {   UINT64_C(0x00000000000067d6), UINT64_C(0x1f42f3ceb8b72701), UINT64_C(0x1f42fccfd0b46704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcfc8b4e6fc)
    },
    {   UINT64_C(0x00000000000067d7), UINT64_C(0x1f42f88f98b8270d), UINT64_C(0x1f42fbcfecad8704),
        UINT64_C(0x1f43008fe0ada704), UINT64_C(0x1f42fccfe0b5a702)
    },
    {   UINT64_C(0x00000000000067d8), UINT64_C(0x1f42fccfecace707), UINT64_C(0x1f42fa0eecb7a70a),
        UINT64_C(0x1f42f50eecb7e71d), UINT64_C(0x1f42fb8efcb06708)
    },
    {   UINT64_C(0x00000000000067d9), UINT64_C(0x1f4310ce54b5c725), UINT64_C(0x1f42fbce78b4070e),
        UINT64_C(0x1f42f6cea0b7870c), UINT64_C(0x1f42ff4ec0af870b)
    },
    {   UINT64_C(0x0000000000006687), UINT64_C(0x1f42f8cfccb6a713), UINT64_C(0x1f42f34fb4b7a702),
        UINT64_C(0x1f42fb0f78afe700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067db), UINT64_C(0x1f42facf8cb56711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffce5cb506f5), UINT64_C(0x1f42ffcfd8b746f4)
    },
    {   UINT64_C(0x0000000000006689), UINT64_C(0x1f42f45048b52715), UINT64_C(0x1f42f5cedcb5470e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cfbcb2e6e4)
    },
    {   UINT64_C(0x00000000000067dd), UINT64_C(0x1f42fe8e50af4711), UINT64_C(0x1f42fe9054b24715),
        UINT64_C(0x1f43008eb4b20715), UINT64_C(0x1f42f90fb0b3c702)
    },
    {   UINT64_C(0x00000000000067de), UINT64_C(0x1f42fbcf70b68709), UINT64_C(0x1f43088e60ade747),
        UINT64_C(0x1f42fbce84b1a700), UINT64_C(0x1f42fbcf10b4e70a)
    },
    {   UINT64_C(0x00000000000067df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8e34ade73d), UINT64_C(0x1f430d4f4cab0702)
    },
    {   UINT64_C(0x00000000000067e0), UINT64_C(0x1f42ff8fe4b526e1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4300d054b4c6df)
    },
    {   UINT64_C(0x000000000000668e), UINT64_C(0x1f42f98f6cad4713), UINT64_C(0x1f42fdcf74b3a725),
        UINT64_C(0x1f4310cea8a9a716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000668f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f30af86c5),
        UINT64_C(0x1f42f88e94b086ef), UINT64_C(0x1f42fcce30b0671f)
    },
    {   UINT64_C(0x00000000000067e3), UINT64_C(0x1f43038fb0b3c6e8), UINT64_C(0x1f42fbcf38b7072d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006691), UINT64_C(0x1f42fb0f6cb5e71c), UINT64_C(0x1f42fa4fa4b506e2),
        UINT64_C(0x1f42fb0ef8b346de), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006692), UINT64_C(0x1f42fc8ee8b0c702), UINT64_C(0x1f42ff4f20b4e71a),
        UINT64_C(0x1f42fbce5cb54727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006693), UINT64_C(0x1f42fc8f50b5c73c), UINT64_C(0x1f42fe0efcb68716),
        UINT64_C(0x1f42fc8e8cb54738), UINT64_C(0x1f42fc8efcb686fa)
    },
    {   UINT64_C(0x00000000000067e7), UINT64_C(0x1f43020ebcb3073f), UINT64_C(0x1f42f80ee4b6672c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431f0f4cb1472e)
    },
    {   UINT64_C(0x00000000000067e8), UINT64_C(0x1f43010ef4b72733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64eacb6471d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006696), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f30b4a71d),
        UINT64_C(0x1f43110e70b42733), UINT64_C(0x1f42fcce80aee702)
    },
    {   UINT64_C(0x0000000000006697), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8ee4b38725), UINT64_C(0x1f42f94eacb6e6f4)
    },
    {   UINT64_C(0x00000000000188ed), UINT64_C(0x1cf2fbd898ab66e6), UINT64_C(0x1cf2fbce54ab6455),
        UINT64_C(0x1cf2fbcfccab65d7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1c8b035180a7a56a)
    },
    {   UINT64_C(0x000000000000669a), UINT64_C(0x1f42f80ebcafa736), UINT64_C(0x1f42f80e7cb3c722),
        UINT64_C(0x1f42f5cefcb3c706), UINT64_C(0x1f4300ce6cb48731)
    },
    {   UINT64_C(0x00000000000188e5), UINT64_C(0x1cf2f40da4af4661), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000669c), UINT64_C(0x1f42fd4fa8b5a6d7), UINT64_C(0x1f42f80f6caea6c7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fcd008b726fa)
    },
    {   UINT64_C(0x000000000000669d), UINT64_C(0x1f42f7cf54a9c6eb), UINT64_C(0x1f42fa4f9cb546d5),
        UINT64_C(0x1f42fbcfa4b206e0), UINT64_C(0x1f42f64f94b0e6d5)
    },
    {   UINT64_C(0x00000000000188f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fa8c1ca986c3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188f4), UINT64_C(0x0ffb25913462857d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066a0), UINT64_C(0x1f42fc0fb4afe6dc), UINT64_C(0x1f42f70fb8b3c717),
        UINT64_C(0x1f430e4ea4a92715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066a1), UINT64_C(0x1f42f7cedcb646f6), UINT64_C(0x1f42fa0ee8b4e704),
        UINT64_C(0x1f42f8cfd4b6c704), UINT64_C(0x1f42ff8fe8ae4701)
    },
    {   UINT64_C(0x00000000000188f7), UINT64_C(0x1162eb8d4085441f), UINT64_C(0x1162eb98708546eb),
        UINT64_C(0x1162eb8e4085445d), UINT64_C(0x1162eb98708546e9)
    },
    {   UINT64_C(0x00000000000188f8), UINT64_C(0x1cf2ed4c50b2a42a), UINT64_C(0x1cf2ed5870b2a732),
        UINT64_C(0x23c2fb8eccc6c616), UINT64_C(0x1c73028cd8a60690)
    },
    {   UINT64_C(0x00000000000188f9), UINT64_C(0x2232f9d310c16506), UINT64_C(0x1fd2f3cf50bae5f7),
        UINT64_C(0x1cf2fbd0c0ab659a), UINT64_C(0x18f31e8e1c9a2646)
    },
    {   UINT64_C(0x00000000000066a5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6cedcb92716), UINT64_C(0x1f42fc5028b04707)
    },
    {   UINT64_C(0x00000000000066a6), UINT64_C(0x1f43198e88a98721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9cf64b606e8), UINT64_C(0x1f430fce94af871f)
    },
    {   UINT64_C(0x00000000000188eb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x09daee131065e506)
    },
    {   UINT64_C(0x00000000000040b2), UINT64_C(0x0000000000000000), UINT64_C(0x1f43331938992679),
        UINT64_C(0x1f433dd388934660), UINT64_C(0x1f4332938094064c)
    },
    {   UINT64_C(0x00000000000040b3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0dc8bb23b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066aa), UINT64_C(0x1f430b0e44b3671e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f64b7e6fb)
    },
    {   UINT64_C(0x00000000000066ab), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90ee8b4c6fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f08b5a6ff)
    },
    {   UINT64_C(0x00000000000066ac), UINT64_C(0x1f42ff0f48b6a6e4), UINT64_C(0x1f42f88e8cb906f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f6cb706eb)
    },
    {   UINT64_C(0x00000000000066ad), UINT64_C(0x1f42f98f90b646f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f70b666dc), UINT64_C(0x1f42fccf58afa6e0)
    },
    {   UINT64_C(0x00000000000066ae), UINT64_C(0x1f4303cf68b30706), UINT64_C(0x1f43004f2cb646d7),
        UINT64_C(0x1f42ff0f50ad26d9), UINT64_C(0x1f43014f14b8a6ed)
    },
    {   UINT64_C(0x00000000000066af), UINT64_C(0x1f42fa4f94b6070b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf50b6e6e4), UINT64_C(0x1f42fc8ee8b286fa)
    },
    {   UINT64_C(0x00000000000066b0), UINT64_C(0x1f42fc8e5cb62723), UINT64_C(0x1f42fb4ef8b726df),
        UINT64_C(0x1f42fe8f6cb686ec), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066b1), UINT64_C(0x1f42f84f08b686fa), UINT64_C(0x1f42fb0f40b5c708),
        UINT64_C(0x1f42fe4ee0b1e70b), UINT64_C(0x1f43000f20af06f6)
    },
    {   UINT64_C(0x00000000000066b2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f20b24716), UINT64_C(0x1f42fc4f20b4c71c)
    },
    {   UINT64_C(0x0000000000018908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f80dc07803f8)
    },
    {   UINT64_C(0x00000000000066b4), UINT64_C(0x1f42fd4ed0b44716), UINT64_C(0x1f42ff0f24ac66e2),
        UINT64_C(0x1f42fcceacaf872b), UINT64_C(0x1f42fccf08b386fa)
    },
    {   UINT64_C(0x0750000000003fff), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106b03d144750524)
    },
    {   UINT64_C(0x00000000000066b6), UINT64_C(0x1f431a0e60aaa732), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000040c1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee1cccbb2757),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066b8), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048eb0adc70d),
        UINT64_C(0x1f42fecf48b32733), UINT64_C(0x1f43060e64acc733)
    },
    {   UINT64_C(0x00000000000066b9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42faceb8b3271f), UINT64_C(0x1f4300ce90ab873c)
    },
    {   UINT64_C(0x00000000000066ba), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303ce28a58716), UINT64_C(0x1f42ffce90a6073c)
    },
    {   UINT64_C(0x00000000000066bb), UINT64_C(0x1f43010eb4a70734), UINT64_C(0x1f43030ed8b06711),
        UINT64_C(0x1f4303cec8aba716), UINT64_C(0x1f4300ce80b4c73a)
    },
    {   UINT64_C(0x00000000000066bc), UINT64_C(0x0000000000000000), UINT64_C(0x1f430ccf04aae703),
        UINT64_C(0x1f42fd0ea4ac2738), UINT64_C(0x1f4300ce44ad473c)
    },
    {   UINT64_C(0x00000000000066bd), UINT64_C(0x1f43054decafe73c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430c8e4caa0714), UINT64_C(0x1f430b8ed4a5c707)
    },
    {   UINT64_C(0x00000000000066be), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8eacb06707),
        UINT64_C(0x1f430b8edcac0707), UINT64_C(0x1f42fe0e80a9c723)
    },
    {   UINT64_C(0x0000000000004009), UINT64_C(0x0fa2f4cfdc79a6cf), UINT64_C(0x0fa2f610a0792698),
        UINT64_C(0x0fa2f410a07d8698), UINT64_C(0x0fa321d75c60e69b)
    },
    {   UINT64_C(0x000000000000400a), UINT64_C(0x0fa30c0e946da721), UINT64_C(0x0fa30b0e946d2721),
        UINT64_C(0x0fa305d6c87146d1), UINT64_C(0x0fa2f391d87a6640)
    },
    {   UINT64_C(0x000000000000400b), UINT64_C(0x0fa3178ee468649d), UINT64_C(0x0fa2f3951c7a6582),
        UINT64_C(0x0fa31d905c6804a9), UINT64_C(0x0fa2f04e947c2721)
    },
    {   UINT64_C(0x00000000000066c2), UINT64_C(0x1f42ff8eb4b38733), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f8ea4b5270b), UINT64_C(0x1f4310cea4a68726)
    },
    {   UINT64_C(0x0000000000018918), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018919), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000001891a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066c6), UINT64_C(0x1f43060e30ae6735), UINT64_C(0x1f4308ce48a4c73a),
        UINT64_C(0x1f430a8e74a4c70f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004011), UINT64_C(0x0fa2f410c87a068e), UINT64_C(0x0fa2f610c879268e),
        UINT64_C(0x0fa2f610c879068e), UINT64_C(0x0fa2f050a07d0698)
    },
    {   UINT64_C(0x0000000000004012), UINT64_C(0x0fa2f210c87ae68e), UINT64_C(0x0fa30610c870268e),
        UINT64_C(0x0fa2f490a079c68e), UINT64_C(0x0fa2f410c87a268e)
    },
    {   UINT64_C(0x0000000000004013), UINT64_C(0x0fa2f390c87a668e), UINT64_C(0x0fa2f290c87bc68e),
        UINT64_C(0x0fa2f250c87bc68e), UINT64_C(0x0fa2f590a0796698)
    },
    {   UINT64_C(0x00000000000066ca), UINT64_C(0x1f43130e48abe713), UINT64_C(0x1f43140e68ad8716),
        UINT64_C(0x1f430d4e60abe722), UINT64_C(0x1f430b4e68a8a70f)
    },
    {   UINT64_C(0x00000000000066cb), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084e5caa6710), UINT64_C(0x1f43158e80a8c711)
    },
    {   UINT64_C(0x00000000000066cc), UINT64_C(0x0000000000000000), UINT64_C(0x1f4308cf0caea719),
        UINT64_C(0x1f4314cecca84712), UINT64_C(0x1f42fd4e60a7471a)
    },
    {   UINT64_C(0x0000000000004017), UINT64_C(0x0fa2f04e947c2721), UINT64_C(0x0fa311ce946b6721),
        UINT64_C(0x0fa303976472472e), UINT64_C(0x0fa2f38db87a63ae)
    },
    {   UINT64_C(0x0000000000011400), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0fa2cacc647d8691),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066cf), UINT64_C(0x1f43058f54b7673e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8f2cb32704), UINT64_C(0x1f42fa8f2cb32704)
    },
    {   UINT64_C(0x000000000000401a), UINT64_C(0x0fa2f5d0c8776698), UINT64_C(0x0fa2ffd0c8732688),
        UINT64_C(0x0fa2f7cebc75c688), UINT64_C(0x0fa2f110c87ba698)
    },
    {   UINT64_C(0x000000000000401b), UINT64_C(0x0fa2ff10c874a698), UINT64_C(0x0fa2f110c87a05cc),
        UINT64_C(0x0fa2f5d0c87945cc), UINT64_C(0x0fa2fa10a07705cc)
    },
    {   UINT64_C(0x00000000000066d2), UINT64_C(0x1f42fc8f78b36719), UINT64_C(0x1f42f9cf5cb3a70a),
        UINT64_C(0x1f42f6cf6cb38706), UINT64_C(0x1f42f5cedcb5c715)
    },
    {   UINT64_C(0x00000000000066d3), UINT64_C(0x1f42f6cf28b6070d), UINT64_C(0x1f42fa8f70b5e725),
        UINT64_C(0x1f42fa8f74b6e734), UINT64_C(0x1f42f98f7cb5a6fc)
    },
    {   UINT64_C(0x0000000000005379), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x00000000000066d5), UINT64_C(0x1f42fa4e70b42746), UINT64_C(0x1f42f6ceccb4a705),
        UINT64_C(0x1f42f88e98b48717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000537b), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x00000000000040e1), UINT64_C(0x1f436257fc8166c9), UINT64_C(0x1f4347568c8ee71b),
        UINT64_C(0x1f43124fa0a966d6), UINT64_C(0x1f435ad44c852594)
    },
    {   UINT64_C(0x00000000000040e2), UINT64_C(0x1f43134fa0a8e6c9), UINT64_C(0x1f43058fa0afc6c9),
        UINT64_C(0x1f433bcfa094a6c9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001892e), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000001892f), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018930), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x00000000000040e6), UINT64_C(0x1f4312d050aa06fc), UINT64_C(0x1f434850808de6f1),
        UINT64_C(0x1f430cd080abe6f1), UINT64_C(0x1f430a9080ada6f1)
    },
    {   UINT64_C(0x0000000000018932), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000004028), UINT64_C(0x0000000000000000), UINT64_C(0x0fa31ece7864a659),
        UINT64_C(0x0fa2fc0e78760659), UINT64_C(0x0fa2f54e94796653)
    },
    {   UINT64_C(0x0000000000018934), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018935), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018936), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000000402c), UINT64_C(0x0fa30e967c6cc68f), UINT64_C(0x0fa2f74fcc78660c),
        UINT64_C(0x0fa30c94506dc693), UINT64_C(0x0fa30c54306de693)
    },
    {   UINT64_C(0x0000000000018938), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018939), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x00000000000040ef), UINT64_C(0x1f43220e209b85d7), UINT64_C(0x1f432ed05097660b),
        UINT64_C(0x1f432d8edc96e6e4), UINT64_C(0x1f432d90509585f4)
    },
    {   UINT64_C(0x00000000000040f0), UINT64_C(0x1f43215050a525f4), UINT64_C(0x1f43248e389cc5f4),
        UINT64_C(0x1f4338ce94962612), UINT64_C(0x1f4324d050a34633)
    },
    {   UINT64_C(0x00000000000040f1), UINT64_C(0x1f4321d0509e85fe), UINT64_C(0x1f432450509e85dc),
        UINT64_C(0x1f431f4e38a30667), UINT64_C(0x1f431f4e54a025e2)
    },
    {   UINT64_C(0x00000000000040f2), UINT64_C(0x1f431bce5ca70693), UINT64_C(0x1f430bd050a8e5f1),
        UINT64_C(0x1f43444fa090667d), UINT64_C(0x1f432c0fa09c8660)
    },
    {   UINT64_C(0x00000000000040f3), UINT64_C(0x1f43390fa09606c3), UINT64_C(0x1f432c0fa09c86a9),
        UINT64_C(0x1f43178fa0a6c6ad), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066ea), UINT64_C(0x1f42facf0cb2a719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038f0cb1e715), UINT64_C(0x1f42fd8fd4b6c722)
    },
    {   UINT64_C(0x0000000000018940), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018941), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018942), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004038), UINT64_C(0x1f4332d09c94e6a6), UINT64_C(0x1f4332d09c94e6a6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018944), UINT64_C(0x1622eb4ae46f453c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005395), UINT64_C(0x1f43048edcb6a716), UINT64_C(0x1f430b4e90ace728),
        UINT64_C(0x1f430f4e7cb0072b), UINT64_C(0x1f43020e64b18748)
    },
    {   UINT64_C(0x0000000000018946), UINT64_C(0x1622eb53cc6f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018947), UINT64_C(0x16233d4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000403d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4312d0fca8069c)
    },
    {   UINT64_C(0x0000000000018949), UINT64_C(0x1622eb4ae46f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000403f), UINT64_C(0x1f433190a099e660), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001894b), UINT64_C(0x1622eb4ae498653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000015400), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x1162bf0c6c7f26a7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001894d), UINT64_C(0x1622eb53cc986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000066f9), UINT64_C(0x1f42fc9010b686f1), UINT64_C(0x1f43000f20b44705),
        UINT64_C(0x1f42fd0f84b6e6dc), UINT64_C(0x1f43004facb18705)
    },
    {   UINT64_C(0x000000000001894f), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053a0), UINT64_C(0x1f43138ea8a8c724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430e0f00b56712), UINT64_C(0x1f42faceccb4a710)
    },
    {   UINT64_C(0x0000000000004106), UINT64_C(0x1f431f10c89ae68e), UINT64_C(0x1f431f10c89ae6ff),
        UINT64_C(0x1f432090a0970697), UINT64_C(0x1f431ed09c9c268e)
    },
    {   UINT64_C(0x0000000000004107), UINT64_C(0x1f4317d0c8a7668e), UINT64_C(0x1f432910c89e268e),
        UINT64_C(0x1f432b909c9cc698), UINT64_C(0x1f432210c89fc68e)
    },
    {   UINT64_C(0x0000000000004108), UINT64_C(0x1f432c50c89c868e), UINT64_C(0x1f431cd0a0970698),
        UINT64_C(0x1f432910c89e268e), UINT64_C(0x1f432c90a09c468e)
    },
    {   UINT64_C(0x0000000000018954), UINT64_C(0x1622eb5258986599), UINT64_C(0x1623264ae47ae776),
        UINT64_C(0x16233d4ae4988599), UINT64_C(0x1623264ae498853c)
    },
    {   UINT64_C(0x000000000000410a), UINT64_C(0x1f431490c4a2e68e), UINT64_C(0x1f43144f64a5468e),
        UINT64_C(0x1f431d50c4a0868e), UINT64_C(0x1f431450c4a6268e)
    },
    {   UINT64_C(0x0000000000018956), UINT64_C(0x16233d5258988776), UINT64_C(0x16232653cc988776),
        UINT64_C(0x1623265258986776), UINT64_C(0x1622eb52586f4776)
    },
    {   UINT64_C(0x0000000000018957), UINT64_C(0x1622eb53cc7ae776), UINT64_C(0x1622eb52587ae776),
        UINT64_C(0x16233d4ae4988776), UINT64_C(0x1623264ae4988776)
    },
    {   UINT64_C(0x0000000000006703), UINT64_C(0x1f42fd0f8cb40711), UINT64_C(0x1f42fb4fe4b7a70f),
        UINT64_C(0x1f42fe0f68b54705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018959), UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb4ae498653c),
        UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb5258986776)
    },
    {   UINT64_C(0x000000000001895a), UINT64_C(0x1622eb53cc986776), UINT64_C(0x1622eb5258986776),
        UINT64_C(0x1622eb4ae4986776), UINT64_C(0x1622eb4ae4986776)
    },
    {   UINT64_C(0x000000000001895b), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001895c), UINT64_C(0x0000000000000000), UINT64_C(0x1622ea4ae498e776),
        UINT64_C(0x1622ea4ae498e776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004112), UINT64_C(0x1f432790c49aa5cc), UINT64_C(0x1f43274fd89c45cc),
        UINT64_C(0x1f432ed0c493c5cc), UINT64_C(0x1f432510c49905cc)
    },
    {   UINT64_C(0x0000000000004113), UINT64_C(0x1f433e90c490e5cc), UINT64_C(0x1f433d909c8f65cc),
        UINT64_C(0x1f4324d0b09985d3), UINT64_C(0x1f4335d0b09405cc)
    },
    {   UINT64_C(0x0000000000004114), UINT64_C(0x0000000000000000), UINT64_C(0x1f4337d0a096a65c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018960), UINT64_C(0x25eaee1460d62776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018961), UINT64_C(0x25eaee0ae4d62517), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018962), UINT64_C(0x25eaee0ae4d62776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018963), UINT64_C(0x25e2ee0ae48a4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000670f), UINT64_C(0x1f43020ef0b20705), UINT64_C(0x1f43050e64b16731),
        UINT64_C(0x1f43024f34ae0710), UINT64_C(0x1f43024e94b2e724)
    },
    {   UINT64_C(0x000000000000405a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f433190a099e662)
    },
    {   UINT64_C(0x0000000000006711), UINT64_C(0x1f43038e60b46730), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43060e3cb1c735), UINT64_C(0x1f42f8cf50b64731)
    },
    {   UINT64_C(0x0000000000006712), UINT64_C(0x1f42ff4ff4b6a720), UINT64_C(0x1f43144eb0b14719),
        UINT64_C(0x1f43008eecafc714), UINT64_C(0x1f42fb0f70b34721)
    },
    {   UINT64_C(0x0000000000018968), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x0000000000006714), UINT64_C(0x1f43008e68b5c72c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008e88b2a734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006715), UINT64_C(0x1f4302cf38b36727), UINT64_C(0x1f43044f58b6a6f9),
        UINT64_C(0x1f42f94f90b5670c), UINT64_C(0x1f43008ea8b6072c)
    },
    {   UINT64_C(0x0000000000006716), UINT64_C(0x1f4300ce84b3070d), UINT64_C(0x1f4307cee4b44716),
        UINT64_C(0x1f42fecefcb32741), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006717), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8e74b76710), UINT64_C(0x1f430dce6cb1271d)
    },
    {   UINT64_C(0x0000000000006718), UINT64_C(0x1f43000ec4b6e744), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300ceecb6471a), UINT64_C(0x1f4300cf30b2c71d)
    },
    {   UINT64_C(0x000000000001896e), UINT64_C(0x14a3020edc8a2614), UINT64_C(0x14a3020ed88a2613),
        UINT64_C(0x1d12fe4ed8ad2613), UINT64_C(0x1d12fe4ed8ad2613)
    },
    {   UINT64_C(0x000000000000671a), UINT64_C(0x1f43004e54b3e73a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000671b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e6cb62729),
        UINT64_C(0x1f42f90f78b7a71b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018971), UINT64_C(0x1d12f94ed8aaa613), UINT64_C(0x1d12f94ed8aaa613),
        UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x2142f7cbd8bea6d0)
    },
    {   UINT64_C(0x000000000000671d), UINT64_C(0x1f42fd8ea4b3a71e), UINT64_C(0x1f42fd8ee8b38700),
        UINT64_C(0x1f42f88ed8b6473d), UINT64_C(0x1f42fe4fe0b08708)
    },
    {   UINT64_C(0x0000000000018973), UINT64_C(0x1c72f54ce0aca690), UINT64_C(0x1c72f4cce0ad0690),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000018974), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000018975), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1222fe8ce07c4690), UINT64_C(0x1223038ce07ec690)
    },
    {   UINT64_C(0x0000000000018976), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f95390aaa67e), UINT64_C(0x1c72f94d28aaa4e4)
    },
    {   UINT64_C(0x0000000000018977), UINT64_C(0x1c72f4d390776690), UINT64_C(0x1c735fd390ad0690),
        UINT64_C(0x1c735fcce0ad04e4), UINT64_C(0x1c72f48ce07764e4)
    },
    {   UINT64_C(0x0000000000006723), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070fd0b76720),
        UINT64_C(0x1f430e4f60aa6723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018979), UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e),
        UINT64_C(0x105aff908c7765a4), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x000000000001897a), UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x1c72f94d2caaa67e), UINT64_C(0x1c72f94d2caaa67e)
    },
    {   UINT64_C(0x000000000001897b), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x23caf6cda4c94713), UINT64_C(0x25baff8b38cca6f9)
    },
    {   UINT64_C(0x000000000001897c), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x0000000000006728), UINT64_C(0x1f4302cfc4b64717), UINT64_C(0x1f43018fd0afe717),
        UINT64_C(0x1f42f9cfa0b5c703), UINT64_C(0x1f43019028b6671b)
    },
    {   UINT64_C(0x0000000000006729), UINT64_C(0x1f43051048b34718), UINT64_C(0x1f43028f98b486ff),
        UINT64_C(0x1f43038f2cb62717), UINT64_C(0x1f43084f60ae0723)
    },
    {   UINT64_C(0x000000000001897f), UINT64_C(0x1752f94e7096262c), UINT64_C(0x12caf78f7084e5e9),
        UINT64_C(0x12caf78f7484e5ea), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x000000000001892d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000018981), UINT64_C(0x0000000000000000), UINT64_C(0x1b32fd0e98a3c677),
        UINT64_C(0x1b32fd0e98a3c677), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018982), UINT64_C(0x0000000000000000), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018983), UINT64_C(0x122afb8d9483a662), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f60fa0b7668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018984), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006730), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306d02cb5670b)
    },
    {   UINT64_C(0x0000000000006731), UINT64_C(0x1f43028f08b0671d), UINT64_C(0x1f42fd8fe4b7a703),
        UINT64_C(0x1f4303cf0cb3e70c), UINT64_C(0x1f43068f7cb2a719)
    },
    {   UINT64_C(0x0000000000006732), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43018fecae071e), UINT64_C(0x1f4301cf64b48705)
    },
    {   UINT64_C(0x0000000000018931), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006734), UINT64_C(0x1f43034ff0af0702), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f78b40716)
    },
    {   UINT64_C(0x0000000000006735), UINT64_C(0x1f43048f34af8711), UINT64_C(0x1f43049024b3a6ee),
        UINT64_C(0x1f43144ef8ade718), UINT64_C(0x1f43020f3cb4e710)
    },
    {   UINT64_C(0x0000000000006736), UINT64_C(0x1f430a0f14b00722), UINT64_C(0x1f43000ec8b2471d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018933), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000006738), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffcf70b0a723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001898e), UINT64_C(0x0000000000000000), UINT64_C(0x23f3188e58b906c0),
        UINT64_C(0x23f3188e58b906c0), UINT64_C(0x23f3188e58b906c0)
    },
    {   UINT64_C(0x000000000001898f), UINT64_C(0x1a2af98db8a16657), UINT64_C(0x140af80e5889a6c0),
        UINT64_C(0x140af80e5889a6c0), UINT64_C(0x12dafe4c8881c6a6)
    },
    {   UINT64_C(0x0000000000018990), UINT64_C(0x12dafe4d8481c666), UINT64_C(0x12dafe4d8481c666),
        UINT64_C(0x14a2fb8d948c0662), UINT64_C(0x1e52fe8ce0af86c4)
    },
    {   UINT64_C(0x0000000000018991), UINT64_C(0x1e52fdcd70aac701), UINT64_C(0x0000000000000000),
        UINT64_C(0x1a42fd0de8a006e0), UINT64_C(0x13c3010de88686c2)
    },
    {   UINT64_C(0x0000000000018992), UINT64_C(0x1e22f44de8b3e6e0), UINT64_C(0x186af8cd749ac66c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c805), UINT64_C(0x1f42ee1274bb84f7), UINT64_C(0x3e82ee12753884f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1042f08b247e435f)
    },
    {   UINT64_C(0x0000000000018937), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0007baf51e58c807), UINT64_C(0x0fa2f615307b469b), UINT64_C(0x0fa2f1953079069b),
        UINT64_C(0x0fa2f18cd0790483), UINT64_C(0x0fa2f1953079069b)
    },
    {   UINT64_C(0x000000000000c808), UINT64_C(0x0fa2f9cd8877269b), UINT64_C(0x0fa2f94d9077869b),
        UINT64_C(0x1182ff90487bc5b6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c809), UINT64_C(0x0000000000000000), UINT64_C(0x14d3028f6c878484),
        UINT64_C(0x1f43028f6cb12484), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018998), UINT64_C(0x156af68e148fe649), UINT64_C(0x156af68dfc8fe643),
        UINT64_C(0x156af84dfc8ee649), UINT64_C(0x156af68e148fe643)
    },
    {   UINT64_C(0x0000000000006744), UINT64_C(0x1f4319cea8a74719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9ce70b2e727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001893a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001899b), UINT64_C(0x0000000000000000), UINT64_C(0x0d52fc4f7869c6e8),
        UINT64_C(0x0d5300cccc6a46e8), UINT64_C(0x0d52f84ccc6ec6e8)
    },
    {   UINT64_C(0x0000000000006747), UINT64_C(0x0000000000000000), UINT64_C(0x1f43030ea8b6a716),
        UINT64_C(0x1f43008f68b1c73c), UINT64_C(0x1f43058ee0b4e73a)
    },
    {   UINT64_C(0x000000000000c80f), UINT64_C(0x138b024f6c82869b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee1b987d072e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c810), UINT64_C(0x18f301d7909a26bd), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004094), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0fa03e83e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000674b), UINT64_C(0x1f430b8e24aca722), UINT64_C(0x1f430ace94b1a719),
        UINT64_C(0x1f43068ed4b3e715), UINT64_C(0x1f42ff4ee0b3e71c)
    },
    {   UINT64_C(0x000000000000c813), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2fc0c807684d4), UINT64_C(0x0a6afeccd05de5c0)
    },
    {   UINT64_C(0x000000000000674d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303ced4b30714), UINT64_C(0x1f431c0e30a4a72c)
    },
    {   UINT64_C(0x0000000000004098), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0fa03e83e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053f4), UINT64_C(0x1f42f30f50a8c6fd), UINT64_C(0x1f42f40f2ca9c718),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42facfa4b066e5)
    },
    {   UINT64_C(0x0000000000006750), UINT64_C(0x1f43110e30aac730), UINT64_C(0x1f43028ed4b30714),
        UINT64_C(0x1f42f7cee0b40715), UINT64_C(0x1f42fb8ec8b5271d)
    },
    {   UINT64_C(0x0000000000006751), UINT64_C(0x1f42fcce68b42731), UINT64_C(0x1f43004e88b52714),
        UINT64_C(0x1f42f88f2cb6672a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053f7), UINT64_C(0x1f42fe8e9cb0072c), UINT64_C(0x1f42ff8f24b7a706),
        UINT64_C(0x1f42fdce90b6472c), UINT64_C(0x1f42fbcfa4b64713)
    },
    {   UINT64_C(0x00000000000189a8), UINT64_C(0x2f32f44d34f827e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189a9), UINT64_C(0x0000000000000000), UINT64_C(0x14a2fb8b3c8c06f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006755), UINT64_C(0x1f42f98eb8b42714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4310ceacb4e71d), UINT64_C(0x1f42f6ce7cb36729)
    },
    {   UINT64_C(0x0000000000006756), UINT64_C(0x1f42f84eacb4e71d), UINT64_C(0x1f42ff8e78b0071d),
        UINT64_C(0x1f43058eacb4e71d), UINT64_C(0x1f43030ebcb4870e)
    },
    {   UINT64_C(0x0000000000018943), UINT64_C(0x16233d4ae498853c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006758), UINT64_C(0x1f43008f10b2e72d), UINT64_C(0x1f42fd8f0cb4e721),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006759), UINT64_C(0x1f42f94ec0b5c731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040ed4b30714)
    },
    {   UINT64_C(0x000000000000675a), UINT64_C(0x1f42fa0ed0b52724), UINT64_C(0x1f42f94edcb6a71b),
        UINT64_C(0x1f42f6cf18b2273a), UINT64_C(0x1f42f9cec8b4c720)
    },
    {   UINT64_C(0x0000000000018945), UINT64_C(0x16233d53cc988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000675c), UINT64_C(0x1f42fc0f14b4e713), UINT64_C(0x1f43048ec8b4c719),
        UINT64_C(0x1f42fccee4b16722), UINT64_C(0x1f42f58ed0b52727)
    },
    {   UINT64_C(0x000000000000675d), UINT64_C(0x1f42fd4ed4b4e711), UINT64_C(0x1f43030ed0b5271b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f2ced0b52714)
    },
    {   UINT64_C(0x000000000000675e), UINT64_C(0x1f42f84ed0b3a720), UINT64_C(0x1f430ace54aba726),
        UINT64_C(0x1f42fbcee0b4a71c), UINT64_C(0x1f42faceecb4a712)
    },
    {   UINT64_C(0x000000000000675f), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fdced0b5271d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42face9cb3c712)
    },
    {   UINT64_C(0x0000000000006760), UINT64_C(0x1f42facf0cb4a717), UINT64_C(0x1f43024ed0b52714),
        UINT64_C(0x1f42f6ced0b5271f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006761), UINT64_C(0x1f42fc0f04b4e71d), UINT64_C(0x1f43008ed0b5271c),
        UINT64_C(0x1f42f74ed0b5271b), UINT64_C(0x1f42fa0ed0b5272e)
    },
    {   UINT64_C(0x0000000000006762), UINT64_C(0x1f42fa4ec8b4c712), UINT64_C(0x1f43024ed8b30735),
        UINT64_C(0x1f42f68ed0b52714), UINT64_C(0x1f42f48e6cb5670f)
    },
    {   UINT64_C(0x00000000000189b8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15e3058e608a46ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006764), UINT64_C(0x1f42facf08b4c724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70eacb2872f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006765), UINT64_C(0x1f4303cea4b3a72f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43078e4cb2c72a), UINT64_C(0x1f42face88b04708)
    },
    {   UINT64_C(0x0000000000006766), UINT64_C(0x1f42fe0f4cb3c735), UINT64_C(0x1f42fe8e8cb34728),
        UINT64_C(0x1f43008f48b30730), UINT64_C(0x1f4308ceb0b3472f)
    },
    {   UINT64_C(0x000000000000540c), UINT64_C(0x1f42f50e78b5a729), UINT64_C(0x1f42f70f0cb4e701),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80e5cb8a720)
    },
    {   UINT64_C(0x0000000000006768), UINT64_C(0x1f43010ea8b2c717), UINT64_C(0x1f43008ea4b32726),
        UINT64_C(0x1f42f8ceb4b0672c), UINT64_C(0x1f42f78ec8b56737)
    },
    {   UINT64_C(0x0000000000006769), UINT64_C(0x1f42ffcef4b28729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43168e48a4e739), UINT64_C(0x1f42fccea4b34729)
    },
    {   UINT64_C(0x000000000000676a), UINT64_C(0x1f42fcce88b4472d), UINT64_C(0x1f4304ce1cb30741),
        UINT64_C(0x1f42ff8ec0b26727), UINT64_C(0x1f42f68e94b76741)
    },
    {   UINT64_C(0x00000000000189c0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1e0af6d0e8afa64c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000040b6), UINT64_C(0x0000000000000000), UINT64_C(0x1f4344943c8ce629),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189c2), UINT64_C(0x0000000000000000), UINT64_C(0x1592f691c88e4613),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c835), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x00027117903c8736), UINT64_C(0x0002789790404736)
    },
    {   UINT64_C(0x00000000000189c4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x179af68f709686ab)
    },
    {   UINT64_C(0x0000000000006770), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94e90b5c72d),
        UINT64_C(0x1f43030ec4b4e713), UINT64_C(0x1f4300ce88b44733)
    },
    {   UINT64_C(0x0000000000006771), UINT64_C(0x1f42f84ebcb66743), UINT64_C(0x1f42fdce90b3a72d),
        UINT64_C(0x1f43054ea8b5272e), UINT64_C(0x1f42ff8ed0b68732)
    },
    {   UINT64_C(0x0000000000006772), UINT64_C(0x1f42f80f00b6c71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040ed8b1072f), UINT64_C(0x1f42ffceb0b2a723)
    },
    {   UINT64_C(0x00000000000189c8), UINT64_C(0x17fafb4e4497c638), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006774), UINT64_C(0x1f42f98ec4b36725), UINT64_C(0x1f4301ced8b42714),
        UINT64_C(0x1f42faceacb2a723), UINT64_C(0x1f42f68ee4b44729)
    },
    {   UINT64_C(0x00000000000189ca), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18aaf6cdf89a264d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006776), UINT64_C(0x1f43088de8afe744), UINT64_C(0x1f4310cde8b1c732),
        UINT64_C(0x1f42f8ce88b20710), UINT64_C(0x1f42f48eb0b2c723)
    },
    {   UINT64_C(0x0000000000006777), UINT64_C(0x1f42f94ed8b42725), UINT64_C(0x1f42fd4eacb4270d),
        UINT64_C(0x1f42ff8eb0b2c723), UINT64_C(0x1f43068e00ab8737)
    },
    {   UINT64_C(0x00000000000189cd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15baf6cdf08ec650), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000541e), UINT64_C(0x1f42f8cf78b64739), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fdcf68b406ea), UINT64_C(0x1f42fa0eb0b3a6fd)
    },
    {   UINT64_C(0x000000000000677a), UINT64_C(0x1f42fecea8b28727), UINT64_C(0x1f43148e00b36736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4ea8b40727)
    },
    {   UINT64_C(0x0000000000018955), UINT64_C(0x1623264ae4986599), UINT64_C(0x1622eb4ae46f4599),
        UINT64_C(0x1622eb4ae47ae53c), UINT64_C(0x1622eb4ae47ae599)
    },
    {   UINT64_C(0x000000000000677c), UINT64_C(0x1f430d0de8ae0742), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118e50ace740), UINT64_C(0x1f42fc8ea8b40727)
    },
    {   UINT64_C(0x0000000000005422), UINT64_C(0x1f42fa0eecb646f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0ed0b00717)
    },
    {   UINT64_C(0x000000000000c845), UINT64_C(0x0000000000000000), UINT64_C(0x171300cfa091668c),
        UINT64_C(0x222af08f58c0c69b), UINT64_C(0x175af48f5498a69b)
    },
    {   UINT64_C(0x000000000000677f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43050df8aee738),
        UINT64_C(0x1f43014ea8b40720), UINT64_C(0x1f43034de8aa673c)
    },
    {   UINT64_C(0x0000000000006780), UINT64_C(0x1f42fe8eb0b2c723), UINT64_C(0x1f42ff4ec8b6e738),
        UINT64_C(0x1f4302cf00b26727), UINT64_C(0x1f42fc0e64b3a726)
    },
    {   UINT64_C(0x0000000000018958), UINT64_C(0x1623264ae4988776), UINT64_C(0x1622eb4ae46f4776),
        UINT64_C(0x1622eb4ae47ae776), UINT64_C(0x1622eb4ae47ae776)
    },
    {   UINT64_C(0x0000000000006782), UINT64_C(0x0000000000000000), UINT64_C(0x1f430cce28a6a73d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74ebcb52735)
    },
    {   UINT64_C(0x0000000000005428), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94ee0b80726), UINT64_C(0x1f42fa0ed4b72701)
    },
    {   UINT64_C(0x0000000000006784), UINT64_C(0x1f42f68eb8b52719), UINT64_C(0x1f4302ce50b2e731),
        UINT64_C(0x1f43028e28b74734), UINT64_C(0x1f42f7cea8b4671d)
    },
    {   UINT64_C(0x0000000000006785), UINT64_C(0x1f42f68e50b8270e), UINT64_C(0x1f42f38ea8b5a736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040e1cac6744)
    },
    {   UINT64_C(0x0000000000006786), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54ed4b42737),
        UINT64_C(0x1f42fd0ef8b4c723), UINT64_C(0x1f42fc4ec4b1c708)
    },
    {   UINT64_C(0x00000000000189dc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f431d0c4c78c6b7), UINT64_C(0x0f42f38c4c6406b7)
    },
    {   UINT64_C(0x0000000000006788), UINT64_C(0x1f43014f24ace711), UINT64_C(0x1f43028f2cae6725),
        UINT64_C(0x1f43148ec4a7c713), UINT64_C(0x1f4303ce80ade72a)
    },
    {   UINT64_C(0x0000000000006789), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307ce8cb1c705),
        UINT64_C(0x1f42facefcb00727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000678a), UINT64_C(0x1f43028f3cafa705), UINT64_C(0x1f42f68f40af8702),
        UINT64_C(0x1f43010f38ae8713), UINT64_C(0x1f4309cf4cb00718)
    },
    {   UINT64_C(0x00000000000189e0), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000189e1), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000189e2), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000189e3), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x00000000000189e4), UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9),
        UINT64_C(0x18a2f6cf689a26a9), UINT64_C(0x18a2f6cf689a26a9)
    },
    {   UINT64_C(0x000000000000c857), UINT64_C(0x1772f3cf709f6698), UINT64_C(0x1772fa4f709f6698),
        UINT64_C(0x1772f58f709f6698), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189e6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1cb2f6d0b8aa2599)
    },
    {   UINT64_C(0x0000000000006792), UINT64_C(0x1f42ff0f28b06717), UINT64_C(0x1f43014f64afa715),
        UINT64_C(0x1f42fd0efcaf6719), UINT64_C(0x1f43028f40b10701)
    },
    {   UINT64_C(0x0000000000006793), UINT64_C(0x1f42f7cf38b0e710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4301cf5cb00708), UINT64_C(0x1f42f38f0caee71c)
    },
    {   UINT64_C(0x0000000000006794), UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f5cafe715),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000543a), UINT64_C(0x1f42fbced0b10743), UINT64_C(0x1f42ffcef0ab2710),
        UINT64_C(0x1f42fa0ebcaa86fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006796), UINT64_C(0x1f43014ef0afe6f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43034f4cb0e706), UINT64_C(0x1f42f68f78b28715)
    },
    {   UINT64_C(0x0000000000006797), UINT64_C(0x1f430ace9cb28731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8f30ae66f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006798), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f30b14712)
    },
    {   UINT64_C(0x00000000000040e3), UINT64_C(0x1f43130fa0a906dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4309cfa0ada6dc), UINT64_C(0x1f430ecfa0ab26c9)
    },
    {   UINT64_C(0x00000000000040e4), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b9080ae26f1),
        UINT64_C(0x1f431e5080a366f1), UINT64_C(0x1f431dd080a2e6f1)
    },
    {   UINT64_C(0x00000000000040e5), UINT64_C(0x1f4312d080a646f1), UINT64_C(0x1f431d9080a4c6f1),
        UINT64_C(0x1f431c5080a506f1), UINT64_C(0x1f430e1080ac06f1)
    },
    {   UINT64_C(0x00000000000189f1), UINT64_C(0x156afd4f348cc5f9), UINT64_C(0x0e3afbcc887086a6),
        UINT64_C(0x0e3afbcc887086a6), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x00000000000189f2), UINT64_C(0x1e32fd0f3cafc5fd), UINT64_C(0x1e32fd0f3cafc5fd),
        UINT64_C(0x0000000000000000), UINT64_C(0x1a32ee0d30a2667e)
    },
    {   UINT64_C(0x00000000000189f3), UINT64_C(0x0a2af30c4c67270e), UINT64_C(0x1a32ee0d30a2667e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000189f4), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x1362f5cf2c882600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x156afa8fa08dc630)
    },
    {   UINT64_C(0x00000000000189f5), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x1fdaf80e60b7262e),
        UINT64_C(0x1fdafb8e60b8e62e), UINT64_C(0x2662f80e60d3062e)
    },
    {   UINT64_C(0x00000000000189f6), UINT64_C(0x1b12fa8c4ca486b6), UINT64_C(0x1b12fa8c4ca486b6),
        UINT64_C(0x2b02fe0fa0e2867e), UINT64_C(0x2b02fe0fa0e2867e)
    },
    {   UINT64_C(0x00000000000189f7), UINT64_C(0x1a8afa913ca2657b), UINT64_C(0x3132fbd084fc65aa),
        UINT64_C(0x3132fbd084fc65aa), UINT64_C(0x0fb3074cb470a69d)
    },
    {   UINT64_C(0x00000000000189f8), UINT64_C(0x18b2f94b1c9ba703), UINT64_C(0x17e2f94e44986635),
        UINT64_C(0x191af94e449d4635), UINT64_C(0x191af94e449d4635)
    },
    {   UINT64_C(0x00000000000189f9), UINT64_C(0x24f2f94d28cca67e), UINT64_C(0x24f2f94d28cca67e),
        UINT64_C(0x0fc31d0c4c7ac6b5), UINT64_C(0x0fc2f38c4c6606b5)
    },
    {   UINT64_C(0x00000000000189fa), UINT64_C(0x0c83014c4c6866b1), UINT64_C(0x0c82fe4c4c66e6b1),
        UINT64_C(0x1313004c4c82a6b7), UINT64_C(0x1312fe4c4c81a6b7)
    },
    {   UINT64_C(0x00000000000189fb), UINT64_C(0x0f431a8c4c78c6b7), UINT64_C(0x0f42f38c4c6546b7),
        UINT64_C(0x074afc0cdc58e68c), UINT64_C(0x074af38cdc54a68c)
    },
    {   UINT64_C(0x00000000000189fc), UINT64_C(0x204aff4d30b7267e), UINT64_C(0x204aff4d30b7267e),
        UINT64_C(0x1e72fb8dd0acc65a), UINT64_C(0x1e73050dd0b1865a)
    },
    {   UINT64_C(0x00000000000189fd), UINT64_C(0x2582fb0eaccde621), UINT64_C(0x3132fbd088fc65a9),
        UINT64_C(0x3132fbd084fc65a9), UINT64_C(0x3132fbd084fc65a9)
    },
    {   UINT64_C(0x00000000000189fe), UINT64_C(0x3132fbceecfc660f), UINT64_C(0x3132fbceecfc660f),
        UINT64_C(0x3132fbcf78fc65ed), UINT64_C(0x3132fbd084fc65aa)
    },
    {   UINT64_C(0x00000000000189ff), UINT64_C(0x3132fbd084fc65aa), UINT64_C(0x3132fbceecfc660f),
        UINT64_C(0x3132fbceecfc660f), UINT64_C(0x3132fbd088fc65a9)
    },
    {   UINT64_C(0x0000000000005450), UINT64_C(0x1f43085048b506bc), UINT64_C(0x1f4308cf44b3870e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0f88b3e6da)
    },
    {   UINT64_C(0x000000000000c873), UINT64_C(0x1e8afe8f4cb04623), UINT64_C(0x1e8aff0ea0b02622),
        UINT64_C(0x26c2fa8ea0d34622), UINT64_C(0x1e8aff0ea0b02622)
    },
    {   UINT64_C(0x0000000000005452), UINT64_C(0x1f43058f20b6272d), UINT64_C(0x1f43064f18b50711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43040f10b4e740)
    },
    {   UINT64_C(0x000000000000c875), UINT64_C(0x26c2fa8ea0d34622), UINT64_C(0x1652f80c0892c6c8),
        UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x1cf2fb8d30ab867e)
    },
    {   UINT64_C(0x000000000001896f), UINT64_C(0x23caf6cda4c94713), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x14a2f4cee08d85c5), UINT64_C(0x14a2f4cee08d85c5)
    },
    {   UINT64_C(0x000000000000c877), UINT64_C(0x1e8afd907cb0c5ab), UINT64_C(0x1e8afd907cb0c5ab),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018970), UINT64_C(0x209af6cb2cb826fe), UINT64_C(0x209aff8b30bc86ff),
        UINT64_C(0x14a2fbced8870613), UINT64_C(0x14a2fbced4870612)
    },
    {   UINT64_C(0x000000000000c879), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e)
    },
    {   UINT64_C(0x00000000000067b3), UINT64_C(0x1f43074e30af0735), UINT64_C(0x1f43008e38b46742),
        UINT64_C(0x1f43034e80b70723), UINT64_C(0x1f42f98f28aec721)
    },
    {   UINT64_C(0x00000000000067b4), UINT64_C(0x1f42fa8f04ae670a), UINT64_C(0x1f43080f5cad86fc),
        UINT64_C(0x1f43080f5cad86fc), UINT64_C(0x1f42fe0ef8b0a6ea)
    },
    {   UINT64_C(0x0000000000018972), UINT64_C(0x2142f7cbd8bea6d0), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x18b2f94b1c9ba703), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000004100), UINT64_C(0x0000000000000000), UINT64_C(0x1f432f50c49b06f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000545c), UINT64_C(0x1f431110acae66fe), UINT64_C(0x1f4301cf7cb4671c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43135014ad0708)
    },
    {   UINT64_C(0x00000000000067b8), UINT64_C(0x1f4304cfa8b0070d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43034f98b50713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067b9), UINT64_C(0x1f42fd4f64b6c701), UINT64_C(0x1f42f7cf34b10706),
        UINT64_C(0x1f4300d014b526fd), UINT64_C(0x1f4306ce5caae740)
    },
    {   UINT64_C(0x0000000000004104), UINT64_C(0x1f432b50c89d0698), UINT64_C(0x1f432b90c894268e),
        UINT64_C(0x1f432c90c49c468e), UINT64_C(0x1f432c90c89c468e)
    },
    {   UINT64_C(0x0000000000004105), UINT64_C(0x1f43278f689a468e), UINT64_C(0x1f432c90c49c468e),
        UINT64_C(0x1f431a50a0a68698), UINT64_C(0x1f432f90a0932698)
    },
    {   UINT64_C(0x00000000000067bc), UINT64_C(0x1f4304cf1cb346f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43034e90b72726), UINT64_C(0x1f431c8e6cb2a70f)
    },
    {   UINT64_C(0x00000000000067bd), UINT64_C(0x1f42f94f78b72703), UINT64_C(0x1f42f58ec0b36739),
        UINT64_C(0x1f42f84e40ae273e), UINT64_C(0x1f42fb0e80aea739)
    },
    {   UINT64_C(0x00000000000067be), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4e20ada713),
        UINT64_C(0x1f42fa0f70b906fd), UINT64_C(0x1f42fccf94b80711)
    },
    {   UINT64_C(0x0000000000005464), UINT64_C(0x1f431bce40a4c720), UINT64_C(0x1f43244e40a6e743),
        UINT64_C(0x1f431e0f50a5c720), UINT64_C(0x1f43110e8cac2741)
    },
    {   UINT64_C(0x0000000000005465), UINT64_C(0x1f43030e24b30724), UINT64_C(0x1f43084eb0aee714),
        UINT64_C(0x1f4313d744aa66c1), UINT64_C(0x1f4314d00cb226cc)
    },
    {   UINT64_C(0x0000000000018978), UINT64_C(0x1c72f4d38cad0690), UINT64_C(0x1c72f4cce0ad04e4),
        UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x0000000000005467), UINT64_C(0x1f42fdcf44b1a71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430f0ebcac06d5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000410d), UINT64_C(0x1f43340fdc9245cc), UINT64_C(0x1f4337d0a096a5cc),
        UINT64_C(0x1f433450b09905d3), UINT64_C(0x1f433cd09c8e25cc)
    },
    {   UINT64_C(0x000000000000410e), UINT64_C(0x1f432f90c89445cc), UINT64_C(0x1f432f90c894462f),
        UINT64_C(0x1f4330d0b09145d3), UINT64_C(0x1f432f90ac9545cc)
    },
    {   UINT64_C(0x000000000000410f), UINT64_C(0x1f432d10c89ca5cc), UINT64_C(0x1f433750c896e5cc),
        UINT64_C(0x1f4337d09c96c5cc), UINT64_C(0x1f4333d0c89825cc)
    },
    {   UINT64_C(0x0000000000004110), UINT64_C(0x1f432c8eb09c45cc), UINT64_C(0x1f4337d0a096a5ca),
        UINT64_C(0x1f4337d0c896a5cc), UINT64_C(0x1f432f8eb09ac5cc)
    },
    {   UINT64_C(0x0000000000004111), UINT64_C(0x1f432cceb09d268e), UINT64_C(0x1f433910c896e5cc),
        UINT64_C(0x1f432f4fdc93e5cc), UINT64_C(0x1f433510c892c5cc)
    },
    {   UINT64_C(0x000000000000c88f), UINT64_C(0x1772fed1b093c55e), UINT64_C(0x1772fed1b093c55e),
        UINT64_C(0x1772ff10f493e58b), UINT64_C(0x1772fe90ac9405cc)
    },
    {   UINT64_C(0x000000000000c890), UINT64_C(0x0ae2fb4f00636627), UINT64_C(0x1772fed09893c5a4),
        UINT64_C(0x1772ff105493e5b7), UINT64_C(0x1772ff105493e5b7)
    },
    {   UINT64_C(0x000000000000c891), UINT64_C(0x1772ff0ec893e5fa), UINT64_C(0x1772ff0eb093e620),
        UINT64_C(0x1772ff0d4893e620), UINT64_C(0x1772ff0d4093e66a)
    },
    {   UINT64_C(0x000000000001897d), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x000000000000c893), UINT64_C(0x1772ff0eb093e620), UINT64_C(0x1772ff0f6c93e5ee),
        UINT64_C(0x1772ff0ff893e5cc), UINT64_C(0x1772ff114c93e5cc)
    },
    {   UINT64_C(0x000000000001897e), UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1c7af94d28aaa67e),
        UINT64_C(0x1c7af94d28aaa67e), UINT64_C(0x1752f94e7096262c)
    },
    {   UINT64_C(0x00000000000067ce), UINT64_C(0x1f42ff8f5cb3a72e), UINT64_C(0x1f4301d008b44706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068e6cb3474a)
    },
    {   UINT64_C(0x0000000000005474), UINT64_C(0x1f430c0fe4ade6cd), UINT64_C(0x1f430bcf38b92710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005475), UINT64_C(0x1f42feceecb7471e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004e7cb4073b), UINT64_C(0x1f4304ce7cb1671c)
    },
    {   UINT64_C(0x0000000000005476), UINT64_C(0x1f42fd8eb8b426fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309ceecab6728)
    },
    {   UINT64_C(0x000000000000c899), UINT64_C(0x1773020dbc92465b), UINT64_C(0x1773020dc092465b),
        UINT64_C(0x1773020c289246c1), UINT64_C(0x1773020c289246c1)
    },
    {   UINT64_C(0x0000000000005478), UINT64_C(0x1f430b5024b726d8), UINT64_C(0x1f4307d03cb726ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005479), UINT64_C(0x1f430ccf54b266c1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000fdcb286d7), UINT64_C(0x1f43068e84af26c2)
    },
    {   UINT64_C(0x000000000000547a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43068eb8af26c1),
        UINT64_C(0x1f43068fa4af26d8), UINT64_C(0x1f43144f10afc721)
    },
    {   UINT64_C(0x000000000000c89d), UINT64_C(0x1772fecb3893c6f4), UINT64_C(0x1772fecb3893c6f4),
        UINT64_C(0x1773020c189246c6), UINT64_C(0x1773020c189246c6)
    },
    {   UINT64_C(0x000000000000547c), UINT64_C(0x1f42f8cf34b5c6f9), UINT64_C(0x1f4301cf6cb88723),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43070e8cb3872c)
    },
    {   UINT64_C(0x000000000000c89f), UINT64_C(0x1773020d7892466d), UINT64_C(0x1773020d7892466d),
        UINT64_C(0x1772feccfc93c68c), UINT64_C(0x1772feccfc93c68c)
    },
    {   UINT64_C(0x000000000000c8a0), UINT64_C(0x1773020dd4924659), UINT64_C(0x1773020dd4924659),
        UINT64_C(0x1773028f6c92060b), UINT64_C(0x1773028f6c92060b)
    },
    {   UINT64_C(0x00000000000067da), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f9d00cb7a702), UINT64_C(0x1f42fb5018b02702)
    },
    {   UINT64_C(0x000000000000c8a2), UINT64_C(0x1773028c7c9206d3), UINT64_C(0x1773028c7c9206d3),
        UINT64_C(0x1773028c809206c6), UINT64_C(0x1773028c809206c6)
    },
    {   UINT64_C(0x00000000000067dc), UINT64_C(0x1f42ffcf40b26706), UINT64_C(0x1f42fa8eacb2a709),
        UINT64_C(0x1f42fdcfc8b046fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c8a4), UINT64_C(0x177303cf349165fc), UINT64_C(0x177303ce2c91666c),
        UINT64_C(0x177303ce2c91666c), UINT64_C(0x1452fe8f1c878605)
    },
    {   UINT64_C(0x0000000000005483), UINT64_C(0x1f42fece84aa4705), UINT64_C(0x1f43239058a2a713),
        UINT64_C(0x1f43000f00a8c718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c8a6), UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x1882fa8e389a463a),
        UINT64_C(0x1a52f7cd98a2e662), UINT64_C(0x1b02fa8d98a44662)
    },
    {   UINT64_C(0x000000000000c8a7), UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x1b02fa8d98a44662),
        UINT64_C(0x1c72f94d28aaa67d), UINT64_C(0x1c72f94d28aaa67d)
    },
    {   UINT64_C(0x0000000000018988), UINT64_C(0x0000000000000000), UINT64_C(0x135aff8f1483265e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067e2), UINT64_C(0x1f42fbcfb8b526f9), UINT64_C(0x1f42ffd008b526f2),
        UINT64_C(0x1f42fd8ff4b5c6d5), UINT64_C(0x1f42fecfc4ae66fd)
    },
    {   UINT64_C(0x0000000000005488), UINT64_C(0x1f4302cf24a86704), UINT64_C(0x1f430bcea0ab06f1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067e4), UINT64_C(0x1f42f78f64b4e731), UINT64_C(0x1f43050ea0ab4738),
        UINT64_C(0x1f43070e1ca92745), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067e5), UINT64_C(0x1f430b0e70af072f), UINT64_C(0x1f42ff0e24b0c732),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000067e6), UINT64_C(0x1f4303cefcb24725), UINT64_C(0x1f430b8fccb2070f),
        UINT64_C(0x1f43154f04ab871a), UINT64_C(0x1f43080f08b46721)
    },
    {   UINT64_C(0x000000000000548c), UINT64_C(0x1f43040f14a6a709), UINT64_C(0x1f42fb8f84b6a6f1),
        UINT64_C(0x1f43008ed4a36705), UINT64_C(0x1f42f9cee4a70725)
    },
    {   UINT64_C(0x000000000000548d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50f80a48718),
        UINT64_C(0x1f43004f24a9071d), UINT64_C(0x1f42f48eb4aa470c)
    },
    {   UINT64_C(0x00000000000067e9), UINT64_C(0x1f42f54eb4b74708), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000548f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43020edca72703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43090f0ca6a713)
    },
    {   UINT64_C(0x000000000000c8b2), UINT64_C(0x1b32fece80a2e644), UINT64_C(0x1b32fc4e80a4a644),
        UINT64_C(0x1b32fb4e80a42644), UINT64_C(0x1b33064e809f2644)
    },
    {   UINT64_C(0x0000000000005491), UINT64_C(0x1f4304cf00aa4720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ece74a5870c), UINT64_C(0x1f42fd4f00a80720)
    },
    {   UINT64_C(0x0000000000005492), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cef4a8e724),
        UINT64_C(0x1f43154f00a7c709), UINT64_C(0x1f42fbcf4cb70729)
    },
    {   UINT64_C(0x0000000000005493), UINT64_C(0x1f42fe0ed4a7e735), UINT64_C(0x1f43058ed4b7c73f),
        UINT64_C(0x1f42fa0ec4a8c71e), UINT64_C(0x1f42fc0f14b466eb)
    },
    {   UINT64_C(0x0000000000005494), UINT64_C(0x1f430b0edca726e1), UINT64_C(0x1f42fcced0a626fd),
        UINT64_C(0x1f43088e90a8271c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a45), UINT64_C(0x1cf2fbd084ab65aa), UINT64_C(0x1cf2fbd084ab65aa),
        UINT64_C(0x1cf2fbd084ab65a9), UINT64_C(0x1cf2fbd084ab65aa)
    },
    {   UINT64_C(0x000000000000c8b8), UINT64_C(0x1773020c849246c7), UINT64_C(0x1773020c849246c7),
        UINT64_C(0x177303cbdc916700), UINT64_C(0x177303cbdc916700)
    },
    {   UINT64_C(0x0000000000005497), UINT64_C(0x1f42fbcf08a92711), UINT64_C(0x1f43060f00a8e719),
        UINT64_C(0x1f42f90ee4a7072f), UINT64_C(0x1f4301ce94ad2709)
    },
    {   UINT64_C(0x000000000000c8ba), UINT64_C(0x1772feccb493c68c), UINT64_C(0x1772feccb493c68c),
        UINT64_C(0x1773024dd4922659), UINT64_C(0x1773024dd4922659)
    },
    {   UINT64_C(0x0000000000005499), UINT64_C(0x1f43108edcaa46fe), UINT64_C(0x1f430b8f00a8e6e6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78e94a8270c)
    },
    {   UINT64_C(0x000000000000549a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018eb4ad4713),
        UINT64_C(0x1f430f4ec0aee748), UINT64_C(0x1f42fd4f0ca7270e)
    },
    {   UINT64_C(0x000000000000549b), UINT64_C(0x1f430d0eb4a8e707), UINT64_C(0x1f42fe4ed0a7e6e5),
        UINT64_C(0x1f43084efca7a700), UINT64_C(0x1f430a4ed4a8c715)
    },
    {   UINT64_C(0x000000000000549c), UINT64_C(0x1f43040f0cb08734), UINT64_C(0x1f42f64fa4b76724),
        UINT64_C(0x1f42fd0f24a76716), UINT64_C(0x1f42fcced0b42732)
    },
    {   UINT64_C(0x000000000000549d), UINT64_C(0x1f430e4ed0a8a723), UINT64_C(0x1f43010e90aa071d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98ef4a7e72e)
    },
    {   UINT64_C(0x000000000000549e), UINT64_C(0x1f43098f4ca8a725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43058ea0b14720), UINT64_C(0x1f42fa4ec0ae4710)
    },
    {   UINT64_C(0x0000000000018a4f), UINT64_C(0x1cf2fcd0a0a9858f), UINT64_C(0x1cf2ff4f2caac58f),
        UINT64_C(0x1cf2ff4dd0a9861b), UINT64_C(0x1cf2ff4dd0a9861b)
    },
    {   UINT64_C(0x00000000000054a0), UINT64_C(0x1f430e8ec4a94713), UINT64_C(0x1f43030ecca926e5),
        UINT64_C(0x1f42f68ed0aa06fc), UINT64_C(0x1f4309ceccab4721)
    },
    {   UINT64_C(0x00000000000054a1), UINT64_C(0x1f43000e90aa0708), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94eecab6727)
    },
    {   UINT64_C(0x00000000000054a2), UINT64_C(0x1f42ff0eb8b0e73c), UINT64_C(0x1f43008ee0ab0719),
        UINT64_C(0x1f42f64ee4a9e73b), UINT64_C(0x1f4304cf00aa670c)
    },
    {   UINT64_C(0x0000000000018a53), UINT64_C(0x0ffaff4d3075a67e), UINT64_C(0x0ffaff4d3075a67e),
        UINT64_C(0x1cf2f79310ad859f), UINT64_C(0x0ffb258d3075a67e)
    },
    {   UINT64_C(0x00000000000054a4), UINT64_C(0x1f43034ee0a6a72f), UINT64_C(0x1f43038ef8a6e725),
        UINT64_C(0x1f42f5cf44b7c716), UINT64_C(0x1f42fc0e74ab473d)
    },
    {   UINT64_C(0x00000000000054a5), UINT64_C(0x1f42f84f04b6871c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43064ebcaa673a), UINT64_C(0x1f43038ec0aac73c)
    },
    {   UINT64_C(0x00000000000054a6), UINT64_C(0x1f42f94ee8ade726), UINT64_C(0x1f42f8cec8aac73d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4f70a5071b)
    },
    {   UINT64_C(0x0000000000018a57), UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fbd088ab65a8), UINT64_C(0x1cf2fbd088ab65a8)
    },
    {   UINT64_C(0x000000000000c8ca), UINT64_C(0x0000000000000000), UINT64_C(0x0deaff4c9c7066c4),
        UINT64_C(0x0deaf98c9c6d86c4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054a9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8ebcaca72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001899a), UINT64_C(0x0000000000000000), UINT64_C(0x0c32f94dac69a6b2),
        UINT64_C(0x1182fa8dac7e46b2), UINT64_C(0x1a42f80dac98a732)
    },
    {   UINT64_C(0x0000000000018a5b), UINT64_C(0x1cf2fbd180ab6603), UINT64_C(0x1cf2fbcf1cab656a),
        UINT64_C(0x156afc4d308d267e), UINT64_C(0x156afc4d308d267e)
    },
    {   UINT64_C(0x00000000000054ac), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8facb266f2),
        UINT64_C(0x1f4305cf84b106f8), UINT64_C(0x1f43174f00ac272b)
    },
    {   UINT64_C(0x0000000000000741), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106afa4fa07ca596)
    },
    {   UINT64_C(0x0000000000000742), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b024f80774596), UINT64_C(0x106af9cfa07ba596)
    },
    {   UINT64_C(0x0000000000018a5f), UINT64_C(0x1cf30fcf8ca165e7), UINT64_C(0x1cf30fcf8ca165e7),
        UINT64_C(0x156b030e7489862d), UINT64_C(0x156b030e7489862d)
    },
    {   UINT64_C(0x0000000000018a60), UINT64_C(0x1293034cac7e269f), UINT64_C(0x11aaf98fd87f65cf),
        UINT64_C(0x0fbb098eb46fa61e), UINT64_C(0x12a3090c4c7a26b7)
    },
    {   UINT64_C(0x00000000000054b1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf28b2c72e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43114e94a3a72a)
    },
    {   UINT64_C(0x0000000000018a62), UINT64_C(0x0ffb120c487006b7), UINT64_C(0x0ffb070c4c6a86b7),
        UINT64_C(0x0ffb15cc4c71e6b7), UINT64_C(0x0eab1d0c4c7666b7)
    },
    {   UINT64_C(0x00000000000054b3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8f44ae0721),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a64), UINT64_C(0x0eaaf38c4c61a6b7), UINT64_C(0x0c83004c4c6b26b7),
        UINT64_C(0x0c83004c4c6b26b7), UINT64_C(0x156af68d64886671)
    },
    {   UINT64_C(0x00000000000054b5), UINT64_C(0x0000000000000000), UINT64_C(0x1f43070f14b0872c),
        UINT64_C(0x1f4308cf70abe706), UINT64_C(0x1f43004f4cab2703)
    },
    {   UINT64_C(0x00000000000054b6), UINT64_C(0x1f42f70f34b0072a), UINT64_C(0x1f42facee4b6e72f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024ec4b226ff)
    },
    {   UINT64_C(0x00000000000054b7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80e88b9073c),
        UINT64_C(0x1f43064e60b1e73f), UINT64_C(0x1f42ffce8cb7a749)
    },
    {   UINT64_C(0x00000000000054b8), UINT64_C(0x1f43008f0cb24710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffce84b4672d), UINT64_C(0x1f42facf2cb1072e)
    },
    {   UINT64_C(0x00000000000054b9), UINT64_C(0x1f43048f54aca736), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fccebcb1871a), UINT64_C(0x1f42fc8f78b4471e)
    },
    {   UINT64_C(0x00000000000189a2), UINT64_C(0x2043140ef8aca697), UINT64_C(0x2043140ef8aca697),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a6b), UINT64_C(0x1612f68eb893861a), UINT64_C(0x1612f48eb892861a),
        UINT64_C(0x1612f68eb893861a), UINT64_C(0x1622f48eb892861a)
    },
    {   UINT64_C(0x00000000000054bc), UINT64_C(0x1f42f5d00cb566f3), UINT64_C(0x1f4301cf68b2c732),
        UINT64_C(0x1f42f98eb0b5c72b), UINT64_C(0x1f42fd8f60af272b)
    },
    {   UINT64_C(0x0000000000018a6d), UINT64_C(0x17d2fa8e64978728), UINT64_C(0x1c72f40dc4ad4657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x00000000000054be), UINT64_C(0x1f42f88eb8b4272a), UINT64_C(0x1f431a4f5ca58707),
        UINT64_C(0x1f43128ee4a8472a), UINT64_C(0x1f431f4ef8a3671d)
    },
    {   UINT64_C(0x0000000000018a6f), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8cd4a1875a),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x000000000000b800), UINT64_C(0x2503bd4f3cae664a), UINT64_C(0x2503bd5044d5e683),
        UINT64_C(0x2542f58fa0cfc7e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b801), UINT64_C(0x1cf2fbd08cab65aa), UINT64_C(0x1cf2fbd084ab65fa),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b802), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afa107c8e2602), UINT64_C(0x156afa0f0c8e2607)
    },
    {   UINT64_C(0x000000000000b803), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afd0a508cc736), UINT64_C(0x156afc0c0c8c26c2)
    },
    {   UINT64_C(0x000000000000b804), UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afd4a508cc736),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b805), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afb4ac48bc71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b806), UINT64_C(0x156afecac48d871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b807), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afa0de88e4647), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b808), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x156afa0cac8e269f)
    },
    {   UINT64_C(0x000000000000b809), UINT64_C(0x0000000000000000), UINT64_C(0x156afa0bf08e26bc),
        UINT64_C(0x156af9cd308b067e), UINT64_C(0x156b004d308e467e)
    },
    {   UINT64_C(0x000000000000b80a), UINT64_C(0x1d2af44d34b0067f), UINT64_C(0x1d2af44d30b0067e),
        UINT64_C(0x26c2f8cd30d2867e), UINT64_C(0x26c2fc0d30d4267e)
    },
    {   UINT64_C(0x000000000000b80b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x156af3cd3091467e)
    },
    {   UINT64_C(0x00000000000054cc), UINT64_C(0x1f4316cf14aca6f7), UINT64_C(0x1f43124eacae4729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a8f14ad46f7)
    },
    {   UINT64_C(0x000000000000b80d), UINT64_C(0x0000000000000000), UINT64_C(0x0f32fe0e18734642),
        UINT64_C(0x0000000000000000), UINT64_C(0x156afa0d308e267e)
    },
    {   UINT64_C(0x000000000000b80e), UINT64_C(0x16e2fa8cac93c69a), UINT64_C(0x156af9cd308e067e),
        UINT64_C(0x0ebaf80d30802713), UINT64_C(0x156afd0d308ca6c8)
    },
    {   UINT64_C(0x000000000000b80f), UINT64_C(0x156afd0d308ca67e), UINT64_C(0x0f32fe0e18734642),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b810), UINT64_C(0x156afa11108e2587), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x156afd0ac48ca71a)
    },
    {   UINT64_C(0x000000000000b811), UINT64_C(0x156afd0ac48ca71a), UINT64_C(0x156afd0ac48ca71a),
        UINT64_C(0x156afd0ac48ca71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b812), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x156afd0bd88ca6d1)
    },
    {   UINT64_C(0x000000000000b813), UINT64_C(0x156afd0a4c8ca735), UINT64_C(0x156afa0f548e25fe),
        UINT64_C(0x156afd0a4c8ca735), UINT64_C(0x156afd0bd88ca6d1)
    },
    {   UINT64_C(0x000000000000b814), UINT64_C(0x156afe4d308c067e), UINT64_C(0x156afe4d308c067e),
        UINT64_C(0x156afa0ec48e2619), UINT64_C(0x13c2f48fa08a05aa)
    },
    {   UINT64_C(0x0000000000018a85), UINT64_C(0x13a2f60aa0948720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x108af60aa08e6720), UINT64_C(0x1712f60aa0a4c720)
    },
    {   UINT64_C(0x000000000000b816), UINT64_C(0x0000000000000000), UINT64_C(0x156afa0c208e26c2),
        UINT64_C(0x156afa0c208e26c2), UINT64_C(0x156afa10188e2619)
    },
    {   UINT64_C(0x000000000000b817), UINT64_C(0x156afa0f548e25fe), UINT64_C(0x0e22f20f787505b3),
        UINT64_C(0x000305d82c7146a2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b818), UINT64_C(0x0000000000000000), UINT64_C(0x0002f0c98866c35b),
        UINT64_C(0x0002f0c80866c35b), UINT64_C(0x00030597907b26c6)
    },
    {   UINT64_C(0x0000000000018a89), UINT64_C(0x156afa0efc8e06a8), UINT64_C(0x156afa0bf48e060b),
        UINT64_C(0x156afa0c888e060b), UINT64_C(0x156af84c5892a60b)
    },
    {   UINT64_C(0x000000000000b81a), UINT64_C(0x1062f04b247b6693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f32fe0e18734642)
    },
    {   UINT64_C(0x000000000000b81b), UINT64_C(0x0f32fe0e18734642), UINT64_C(0x156afd0d2c8ca6f9),
        UINT64_C(0x156afd0d2c8ca6f9), UINT64_C(0x156afd0d9c8ca6f9)
    },
    {   UINT64_C(0x000000000000b81c), UINT64_C(0x156afd0d9c8ca6f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000771), UINT64_C(0x106b05969075264b), UINT64_C(0x106b03d69074464b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054de), UINT64_C(0x1f42ff8e4cafe728), UINT64_C(0x1f42fb0e5caf6705),
        UINT64_C(0x1f43138f30b2670e), UINT64_C(0x1f42f34e7caf0712)
    },
    {   UINT64_C(0x000000000000b81f), UINT64_C(0x156afa0bb08e26de), UINT64_C(0x156afa0bb08e26de),
        UINT64_C(0x156afc0c148c26c4), UINT64_C(0x156afc0c148c26c4)
    },
    {   UINT64_C(0x000000000000b820), UINT64_C(0x156afc4ac48c471a), UINT64_C(0x156afc4ac48c471a),
        UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afc0c0c8c26c2)
    },
    {   UINT64_C(0x0000000000018a91), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa094886077a), UINT64_C(0x1362fa094886077a)
    },
    {   UINT64_C(0x000000000000b822), UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018a93), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f08e1c8ac600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x000000000000b824), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afd8b008ce6b7), UINT64_C(0x156afd4b008cc6b7)
    },
    {   UINT64_C(0x000000000000b825), UINT64_C(0x156afd8be08ce6b7), UINT64_C(0x156afd4be08cc6b7),
        UINT64_C(0x156afbcac48c071a), UINT64_C(0x156afe4ac48d471a)
    },
    {   UINT64_C(0x000000000000b826), UINT64_C(0x156af3ccac91469a), UINT64_C(0x1322f44cc487e61f),
        UINT64_C(0x1622f28c4c94c6b7), UINT64_C(0x1622f28c4c94c6b7)
    },
    {   UINT64_C(0x00000000000054e7), UINT64_C(0x1f43040f94b06708), UINT64_C(0x1f42f98efcb34711),
        UINT64_C(0x1f42f98e80b206f8), UINT64_C(0x1f42fa8f00a9671d)
    },
    {   UINT64_C(0x000000000000b828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x118af40c3881e5b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054e9), UINT64_C(0x1f42f88f28b5a6f9), UINT64_C(0x1f42f78f90b446f1),
        UINT64_C(0x1f42fa8ef4b1870b), UINT64_C(0x1f42fe4ec4aa6700)
    },
    {   UINT64_C(0x000000000000b82a), UINT64_C(0x0f830a8d306ce67e), UINT64_C(0x143afa523c8be5d9),
        UINT64_C(0x0bc2f5978869c6a6), UINT64_C(0x0f830a8d306ce67e)
    },
    {   UINT64_C(0x000000000000b82b), UINT64_C(0x0eb2f597887586a6), UINT64_C(0x13f2f6cd307ea67e),
        UINT64_C(0x0d3afbcfa069a67e), UINT64_C(0x0d3afbcfa069a67e)
    },
    {   UINT64_C(0x000000000000b82c), UINT64_C(0x00030717ec7086c7), UINT64_C(0x0002ee0ba86403b6),
        UINT64_C(0x0002ee0ba86403b6), UINT64_C(0x10eaf60c3877a693)
    },
    {   UINT64_C(0x000000000000b82d), UINT64_C(0x09cafbd3105ec614), UINT64_C(0x09cafbd3105ec614),
        UINT64_C(0x0002f590844e05a9), UINT64_C(0x08caff927c58e52c)
    },
    {   UINT64_C(0x00000000000054ee), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4ee4b40729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f70b2e71c)
    },
    {   UINT64_C(0x0000000000018a9f), UINT64_C(0x156b010f388ea70e), UINT64_C(0x156afc0d7c8c2659),
        UINT64_C(0x156afc0d7c8c2659), UINT64_C(0x156afd0d7c8ca659)
    },
    {   UINT64_C(0x000000000000b830), UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c),
        UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x1772fa918096056a)
    },
    {   UINT64_C(0x000000000000b831), UINT64_C(0x1772fa90109605c6), UINT64_C(0x1772fa90109605c6),
        UINT64_C(0x1772fa9310960506), UINT64_C(0x1772fa918096057a)
    },
    {   UINT64_C(0x000000000000b832), UINT64_C(0x1f42fa9180b5456a), UINT64_C(0x1f42fa9010b545c6),
        UINT64_C(0x1f42fa8f08b54608), UINT64_C(0x1772fa918096056a)
    },
    {   UINT64_C(0x000000000000b833), UINT64_C(0x1772fa90109605c6), UINT64_C(0x1772fa8f08960608),
        UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c)
    },
    {   UINT64_C(0x000000000000b834), UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c),
        UINT64_C(0x1cf302ccb8a7e69c), UINT64_C(0x1cf2f40da4af4661)
    },
    {   UINT64_C(0x000000000000b835), UINT64_C(0x1cf2f40da4af4661), UINT64_C(0x1f42fa9310b54506),
        UINT64_C(0x1772fa918096056a), UINT64_C(0x17d2fa8e6497862f)
    },
    {   UINT64_C(0x000000000000b836), UINT64_C(0x0c82fc4fa0696630), UINT64_C(0x07fafc0fa0576681),
        UINT64_C(0x0c22fc0fa0680681), UINT64_C(0x2e03038e00edc64a)
    },
    {   UINT64_C(0x000000000000b837), UINT64_C(0x2372fe5200c4254a), UINT64_C(0x1023004fa076278d),
        UINT64_C(0x0fa2fc4fa075678d), UINT64_C(0x1853050fa095e6da)
    },
    {   UINT64_C(0x000000000000b838), UINT64_C(0x0fa2f80fa0760780), UINT64_C(0x0c2b024fa06646dd),
        UINT64_C(0x0fa2fd0fa07426da), UINT64_C(0x1a52f6cd88a36666)
    },
    {   UINT64_C(0x000000000000b839), UINT64_C(0x0000000000000000), UINT64_C(0x105b000cac76c69a),
        UINT64_C(0x105b000cac76c69a), UINT64_C(0x1822f6cda49aa713)
    },
    {   UINT64_C(0x000000000000b83a), UINT64_C(0x21aafbcf38be45fc), UINT64_C(0x156afa0efc8e060b),
        UINT64_C(0x1c72f94d28aaa67d), UINT64_C(0x1662f94eb892661f)
    },
    {   UINT64_C(0x000000000000b83b), UINT64_C(0x156afa11808e2746), UINT64_C(0x0ffb1e0ac066671b),
        UINT64_C(0x1cf3034d30a7a6d7), UINT64_C(0x1572fa4efc8e260b)
    },
    {   UINT64_C(0x000000000000b83c), UINT64_C(0x0000000000000000), UINT64_C(0x0c32fa0fa069442a),
        UINT64_C(0x211af78d2cbfa790), UINT64_C(0x211b1f0d30aa467e)
    },
    {   UINT64_C(0x000000000000b83d), UINT64_C(0x182afbcd0098469e), UINT64_C(0x1cf2fbcd58ab8688),
        UINT64_C(0x1c6af3cdd8ad46d3), UINT64_C(0x1de2f38e08b346bf)
    },
    {   UINT64_C(0x000000000000b83e), UINT64_C(0x0fa316ce3c6885c2), UINT64_C(0x17fb040fa0936690),
        UINT64_C(0x094afd8e3c5be5c2), UINT64_C(0x36b3040fa10e4690)
    },
    {   UINT64_C(0x000000000000b83f), UINT64_C(0x32cb040fa0fea690), UINT64_C(0x32cb040fa0fea690),
        UINT64_C(0x32cb040fa0fea690), UINT64_C(0x32cb040fa0fea690)
    },
    {   UINT64_C(0x000000000000b840), UINT64_C(0x3f83040fa1318690), UINT64_C(0x32cb040fa0fea690),
        UINT64_C(0x3243040fa0fc8690), UINT64_C(0x47c3040fa1528690)
    },
    {   UINT64_C(0x000000000000b841), UINT64_C(0x0322de1394530581), UINT64_C(0x3f83040fa1318690),
        UINT64_C(0x3f83040fa1318690), UINT64_C(0x32cb040fa0fea690)
    },
    {   UINT64_C(0x000000000000b842), UINT64_C(0x32cb040fa0fea690), UINT64_C(0x0a3af08b446246f5),
        UINT64_C(0x07d2f84b4454c6f5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b843), UINT64_C(0x0c82ff518867c568), UINT64_C(0x0c82ff518867c568),
        UINT64_C(0x14631e513477e57d), UINT64_C(0x09daee118065e56a)
    },
    {   UINT64_C(0x000000000000b844), UINT64_C(0x14630d1010808598), UINT64_C(0x14630d1010808598),
        UINT64_C(0x1cf2fbcd30ab867e), UINT64_C(0x1cf2fbcd30ab867e)
    },
    {   UINT64_C(0x000000000000b845), UINT64_C(0x1cf2fb8d30ab667e), UINT64_C(0x1cf2fb8d30ab667e),
        UINT64_C(0x0ffb25d14c62a4be), UINT64_C(0x072afd5310538506)
    },
    {   UINT64_C(0x000000000000b846), UINT64_C(0x08aaff91b85864f7), UINT64_C(0x08aaff91b85864f7),
        UINT64_C(0x0ffafccd3076e67e), UINT64_C(0x0ffafccd3076e67e)
    },
    {   UINT64_C(0x000000000000b847), UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x1cf2fb8d30ab867e),
        UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x1cf2fb8d30ab867e)
    },
    {   UINT64_C(0x000000000000b848), UINT64_C(0x0ffb251118632583), UINT64_C(0x09baed530465c50a),
        UINT64_C(0x0b7afb918c65a566), UINT64_C(0x0b7afb919065a567)
    },
    {   UINT64_C(0x000000000000b849), UINT64_C(0x0ffafccd3076e67e), UINT64_C(0x0ffafccd3076e67e),
        UINT64_C(0x1cf2fb9084ab85a9), UINT64_C(0x1cf2fb9084ab85a9)
    },
    {   UINT64_C(0x000000000000b84a), UINT64_C(0x1cf2fb8d34ab867e), UINT64_C(0x1cf2fbcd30ab867e),
        UINT64_C(0x1cf2fb8d30ab667e), UINT64_C(0x1cf2fb8d30ab867d)
    },
    {   UINT64_C(0x000000000000079f), UINT64_C(0x106af28f807dc709), UINT64_C(0x106af40f907d0671),
        UINT64_C(0x106af28d287dc64b), UINT64_C(0x106af40d387d0597)
    },
    {   UINT64_C(0x000000000000b84c), UINT64_C(0x0bdaff91b865255d), UINT64_C(0x0bdaff91b865255d),
        UINT64_C(0x0bdaff91b865255d), UINT64_C(0x0bdaff91b865255d)
    },
    {   UINT64_C(0x0000000000009104), UINT64_C(0x1852f6cce09b669b), UINT64_C(0x1692f48fa098068c),
        UINT64_C(0x1162f6cf547ea69c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x0000000000009105), UINT64_C(0x1692f20f5496268c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1f42f2cf64b9268c), UINT64_C(0x1692f20fa095e68c)
    },
    {   UINT64_C(0x00000000000007a3), UINT64_C(0x106af4cfa07c86ec), UINT64_C(0x106af4cfa07c8635),
        UINT64_C(0x106af2cfa07da6ec), UINT64_C(0x106af48c107d0634)
    },
    {   UINT64_C(0x0000000000009107), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x1162f44f6881468c),
        UINT64_C(0x0de2f44f6873e5c1), UINT64_C(0x0a6af18fa06f269b)
    },
    {   UINT64_C(0x0000000000009108), UINT64_C(0x0fa2f50c687ae5c0), UINT64_C(0x1162f1cfa081468c),
        UINT64_C(0x08b2f1cfa05e869b), UINT64_C(0x0a6adfcc745f669b)
    },
    {   UINT64_C(0x0000000000009109), UINT64_C(0x1162f38fa082668c), UINT64_C(0x08b2f1cfa05e868c),
        UINT64_C(0x1a0af1cfa0a445c1), UINT64_C(0x1162f34fa081e5c1)
    },
    {   UINT64_C(0x000000000000910a), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x1162f2cc6c8005c1),
        UINT64_C(0x1162f68c6c8185c1), UINT64_C(0x0de2f50fa074c5c1)
    },
    {   UINT64_C(0x000000000000910b), UINT64_C(0x0c2af44f686ba5c1), UINT64_C(0x0a6af2cf7068065e),
        UINT64_C(0x1162f20f6881c5b5), UINT64_C(0x0fa2f34f687b25b5)
    },
    {   UINT64_C(0x000000000000910c), UINT64_C(0x1692f3cf6896e5b5), UINT64_C(0x0fa2f10fa07b05b5),
        UINT64_C(0x0fa2f20c6c7ac5b5), UINT64_C(0x0de2f34fa07305b5)
    },
    {   UINT64_C(0x000000000000910d), UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675),
        UINT64_C(0x14fafa8f7097667b), UINT64_C(0x18b2f78fa09e2675)
    },
    {   UINT64_C(0x000000000000910e), UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1352f78fa098e675),
        UINT64_C(0x16f2fa8f709a4684), UINT64_C(0x1b4af78fa0b1e675)
    },
    {   UINT64_C(0x000000000000910f), UINT64_C(0x0f02f78fa080c675), UINT64_C(0x10e2fd0f708c0675),
        UINT64_C(0x17d2f78fa0a2c675), UINT64_C(0x1622f78fa0920675)
    },
    {   UINT64_C(0x0000000000009110), UINT64_C(0x1f6af78fa0c26675), UINT64_C(0x1a9af78fa0af2675),
        UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1292f78fa0968675)
    },
    {   UINT64_C(0x00000000000007ae), UINT64_C(0x106af48d3c7d464b), UINT64_C(0x106afbcd4078a59f),
        UINT64_C(0x106af48fa07d4773), UINT64_C(0x106afbcf887886bf)
    },
    {   UINT64_C(0x0000000000009112), UINT64_C(0x160afe4f6c9f6675), UINT64_C(0x11fafd0f609d8675),
        UINT64_C(0x1ca2fd0f60c82675), UINT64_C(0x18b2f44fa0a3c675)
    },
    {   UINT64_C(0x0000000000009113), UINT64_C(0x10baf6cfa0956675), UINT64_C(0x1822fd0fa0a2c675),
        UINT64_C(0x0fb2f80f787985a1), UINT64_C(0x0eb2f94f7476c684)
    },
    {   UINT64_C(0x0000000000009114), UINT64_C(0x0cfaf80f747085a1), UINT64_C(0x10a2f80f70806684),
        UINT64_C(0x0deaf80f7471c5a1), UINT64_C(0x115af80cb48b6684)
    },
    {   UINT64_C(0x0000000000009115), UINT64_C(0x0f62f30cb47c05a1), UINT64_C(0x0000000000000000),
        UINT64_C(0x09bafa8f745ea650), UINT64_C(0x0c2aea0cb46d0650)
    },
    {   UINT64_C(0x0000000000009116), UINT64_C(0x10f2f94f74806684), UINT64_C(0x09f2f94f78614684),
        UINT64_C(0x1632f58f809385a1), UINT64_C(0x0f8af58f8078e5a1)
    },
    {   UINT64_C(0x0000000000009117), UINT64_C(0x0e52f80f747545a1), UINT64_C(0x0f4ae68cc479c5a1),
        UINT64_C(0x0e52f80cc47865a1), UINT64_C(0x0cc2f58fa06fa5a1)
    },
    {   UINT64_C(0x000000000000b861), UINT64_C(0x1652f58f6894068c), UINT64_C(0x0c8af94fa06b067e),
        UINT64_C(0x13d3008fa08d467e), UINT64_C(0x14d2f5cfa08de68a)
    },
    {   UINT64_C(0x000000000000b862), UINT64_C(0x1a5b00cfa09e867e), UINT64_C(0x151b008f6889a67e),
        UINT64_C(0x141af50fa08b667e), UINT64_C(0x1652f58f6894068c)
    },
    {   UINT64_C(0x00000000000007b7), UINT64_C(0x106afc0f747c6732), UINT64_C(0x106afc0f8c78e67d),
        UINT64_C(0x106afc0f747c6773), UINT64_C(0x106afc0f8c78e6bf)
    },
    {   UINT64_C(0x000000000000b864), UINT64_C(0x1302f1cfa088a67e), UINT64_C(0x1592f40fa091c68c),
        UINT64_C(0x1662f3cfa095267e), UINT64_C(0x15e2f5cfa092267e)
    },
    {   UINT64_C(0x00000000000007b9), UINT64_C(0x106aeecd407f864b), UINT64_C(0x106af10d407e8597),
        UINT64_C(0x106aef0f747f8773), UINT64_C(0x106af10f887e86bf)
    },
    {   UINT64_C(0x000000000000911d), UINT64_C(0x1fc2f88fa0c46685), UINT64_C(0x1b2af88fa0b20685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1322f88fa09a2685)
    },
    {   UINT64_C(0x000000000000b867), UINT64_C(0x0f72f84c4475e5b7), UINT64_C(0x0df2f34f7873a693),
        UINT64_C(0x0872fc4f786025b8), UINT64_C(0x0ec3028fa07985b8)
    },
    {   UINT64_C(0x000000000000911f), UINT64_C(0x16eb000f78a3c685), UINT64_C(0x128afe8f64a22685),
        UINT64_C(0x1ceafe8f5ccba685), UINT64_C(0x1942f50fa0a64685)
    },
    {   UINT64_C(0x000000000000b869), UINT64_C(0x0f9af50f787965b7), UINT64_C(0x0fdaefcf787b65ad),
        UINT64_C(0x0f92fa0c407945b6), UINT64_C(0x0d02f64c507245b7)
    },
    {   UINT64_C(0x00000000000007be), UINT64_C(0x106af2cfa07da729), UINT64_C(0x106af48c107d0674),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c94d), UINT64_C(0x1622eb53cc986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b86c), UINT64_C(0x0f4af88f707be693), UINT64_C(0x1372f50c4088c694),
        UINT64_C(0x0f5af50c407865b7), UINT64_C(0x17d2efcf789ae5ad)
    },
    {   UINT64_C(0x0000000000009124), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x11dae5cc6c83c5bd),
        UINT64_C(0x1072f90c6c8105b6), UINT64_C(0x0dcaf64fa07385bb)
    },
    {   UINT64_C(0x000000000000552e), UINT64_C(0x1f43034e6cb4c726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43021060b3c6b3), UINT64_C(0x1f42f68ebcb8a704)
    },
    {   UINT64_C(0x0000000000009126), UINT64_C(0x1972fb0f68a265c1), UINT64_C(0x125af64f808265b6),
        UINT64_C(0x103af6cc6c7fc5b6), UINT64_C(0x109af6cf547ce5b5)
    },
    {   UINT64_C(0x0000000000009127), UINT64_C(0x1abaf5cf64a8468a), UINT64_C(0x0000000000000000),
        UINT64_C(0x18eaf74f64a02697), UINT64_C(0x1baaf70f64a4c697)
    },
    {   UINT64_C(0x00000000000007c5), UINT64_C(0x106af8cf747a8667), UINT64_C(0x106af8cf747a8667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009129), UINT64_C(0x0000000000000000), UINT64_C(0x1912f04cdca1868a),
        UINT64_C(0x1f8af80f64bc0697), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000912a), UINT64_C(0x0000000000000000), UINT64_C(0x1e52f78f64b60697),
        UINT64_C(0x15a3028f64916690), UINT64_C(0x1c72f78f64ad4697)
    },
    {   UINT64_C(0x000000000000912b), UINT64_C(0x15a3028f08916690), UINT64_C(0x0000000000000000),
        UINT64_C(0x173afecf64962690), UINT64_C(0x1c82f8cf64afe697)
    },
    {   UINT64_C(0x000000000000912c), UINT64_C(0x1a52f70f64a32697), UINT64_C(0x1d22f6cf64b1c697),
        UINT64_C(0x21b2f6cf64c44697), UINT64_C(0x1bdaf70f64aba697)
    },
    {   UINT64_C(0x000000000000912d), UINT64_C(0x1cf304cc18b10697), UINT64_C(0x1d22fccf64b08697),
        UINT64_C(0x199af58f749d45a1), UINT64_C(0x1222f9cf70846697)
    },
    {   UINT64_C(0x000000000000912e), UINT64_C(0x14b2f58f748825a1), UINT64_C(0x1a6af58f74a5e697),
        UINT64_C(0x0000000000000000), UINT64_C(0x156af4cc5c92a697)
    },
    {   UINT64_C(0x000000000000c95a), UINT64_C(0x1622eb53cc986776), UINT64_C(0x1622eb5258986776),
        UINT64_C(0x1622eb4ae4986776), UINT64_C(0x1622eb4ae4986776)
    },
    {   UINT64_C(0x000000000000c95b), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009131), UINT64_C(0x0000000000000000), UINT64_C(0x182af3cc5c9545a1),
        UINT64_C(0x1862f58c5c9b65a1), UINT64_C(0x1222fa0fa08605a4)
    },
    {   UINT64_C(0x0000000000009132), UINT64_C(0x101afd8f747205fb), UINT64_C(0x113af58f7481067a),
        UINT64_C(0x178af58f74952590), UINT64_C(0x1352f58f748745a1)
    },
    {   UINT64_C(0x00000000000007d0), UINT64_C(0x106af84f8c7ae667), UINT64_C(0x106af84f8c7ae667),
        UINT64_C(0x106af84f8c7ae667), UINT64_C(0x106af84f8c7ae667)
    },
    {   UINT64_C(0x0000000000009134), UINT64_C(0x1ec2fa8f4cb5e6a3), UINT64_C(0x2122fbcf4cb9a6a3),
        UINT64_C(0x1c82fd0f4cac26a3), UINT64_C(0x1efafd0f4cafc6a3)
    },
    {   UINT64_C(0x0000000000009135), UINT64_C(0x19d305cf4ca166a3), UINT64_C(0x2092fe4f4cbea6a3),
        UINT64_C(0x1a13028f4ca266a3), UINT64_C(0x249afe4f4ccec6a3)
    },
    {   UINT64_C(0x0000000000009136), UINT64_C(0x1f2af9cf4cb8a6a3), UINT64_C(0x1c52f2cc20ad46a3),
        UINT64_C(0x2412f94f4ccbc6a3), UINT64_C(0x2062f80f4cbd66a3)
    },
    {   UINT64_C(0x0000000000009137), UINT64_C(0x28a2fd0f4cda26a3), UINT64_C(0x223afd0f4cc546a3),
        UINT64_C(0x194b000f4c9c26a3), UINT64_C(0x200afbcf4cbac6a3)
    },
    {   UINT64_C(0x00000000000007d5), UINT64_C(0x106af8cf8c7a8667), UINT64_C(0x106af8cf8c7a8667),
        UINT64_C(0x106af8cf8c7a869e), UINT64_C(0x106af8cf8c7a869e)
    },
    {   UINT64_C(0x0000000000005542), UINT64_C(0x1f42fc0f58b566e6), UINT64_C(0x1f43014f04b3c71f),
        UINT64_C(0x1f43000f70b72707), UINT64_C(0x1f42fe0ed8b8a70b)
    },
    {   UINT64_C(0x000000000000913a), UINT64_C(0x1eeafd0c20b6e6a3), UINT64_C(0x1f22fa8f4cb886a3),
        UINT64_C(0x1d72f6cf68aaa5b6), UINT64_C(0x1432fd0f688c86a3)
    },
    {   UINT64_C(0x000000000000913b), UINT64_C(0x17e2f6cf6893e5b6), UINT64_C(0x1daaf50f68b086a3),
        UINT64_C(0x16b2f6cf688f85b6), UINT64_C(0x1802f44c6c9c26a3)
    },
    {   UINT64_C(0x000000000000913c), UINT64_C(0x199af4cc209ee5b6), UINT64_C(0x1a32fbcf689d46a3),
        UINT64_C(0x1172f80f688146a2), UINT64_C(0x1a42f84c24a566a2)
    },
    {   UINT64_C(0x000000000000913d), UINT64_C(0x1952fbcf6899c6a3), UINT64_C(0x1452f8cf688d86a3),
        UINT64_C(0x238af94f68c2a5b6), UINT64_C(0x1a9af94f689ee5b6)
    },
    {   UINT64_C(0x000000000000913e), UINT64_C(0x1a82f6cf68a005b6), UINT64_C(0x1baaf44c6c9ec5b6),
        UINT64_C(0x1c8af6cc6ca6a5b6), UINT64_C(0x152af80fa08f85b6)
    },
    {   UINT64_C(0x000000000000913f), UINT64_C(0x1192facf6876a615), UINT64_C(0x1352f9cf688b0685),
        UINT64_C(0x1a52f5cf689dc5aa), UINT64_C(0x16e2f80f689245b2)
    },
    {   UINT64_C(0x0000000000009140), UINT64_C(0x1fa2f80f68b545b2), UINT64_C(0x19a2fdcf689b05b6),
        UINT64_C(0x1882f80c209765aa), UINT64_C(0x1872fd4f689645d5)
    },
    {   UINT64_C(0x000000000000c96c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x23caf6cda4c94713)
    },
    {   UINT64_C(0x000000000000c96d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x209aff8b30bc86ff)
    },
    {   UINT64_C(0x000000000000554c), UINT64_C(0x1f42f84f34ac46e8), UINT64_C(0x1f4303cf70ad86f3),
        UINT64_C(0x1f43044f58b086d9), UINT64_C(0x1f43004f8cb3c70e)
    },
    {   UINT64_C(0x0000000000009144), UINT64_C(0x21dafa8f48bf669f), UINT64_C(0x19dafa8f489f669f),
        UINT64_C(0x1a2af74f40a0c69f), UINT64_C(0x19baf80c709f269f)
    },
    {   UINT64_C(0x0000000000009145), UINT64_C(0x1b0af74ea0a4469f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ac2fbcf40a2a69f), UINT64_C(0x17f2f9cf4098c69f)
    },
    {   UINT64_C(0x0000000000009146), UINT64_C(0x189afa8f489b86a0), UINT64_C(0x19fafa0f40a0269f),
        UINT64_C(0x219afa0f40bea69f), UINT64_C(0x17daf98f249a669f)
    },
    {   UINT64_C(0x0000000000009147), UINT64_C(0x17f2f9cc7097c69f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1092facf587a65bc), UINT64_C(0x100af98f5878469f)
    },
    {   UINT64_C(0x00000000000007e5), UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667),
        UINT64_C(0x106af2cbfc7ac69e), UINT64_C(0x106af2cbfc7ac69e)
    },
    {   UINT64_C(0x0000000000005552), UINT64_C(0x1f43084e5cac6722), UINT64_C(0x1f42fb8f08b64703),
        UINT64_C(0x1f43014f58b8c6ed), UINT64_C(0x1f43084eecb5c6f9)
    },
    {   UINT64_C(0x0000000000005553), UINT64_C(0x1f42ffcfb4b286ff), UINT64_C(0x1f43020e84b00726),
        UINT64_C(0x1f43028f18b3c6e1), UINT64_C(0x1f4319cf1cab0720)
    },
    {   UINT64_C(0x000000000000914b), UINT64_C(0x0fbafa8f587705bc), UINT64_C(0x115af78c5c7d8632),
        UINT64_C(0x0fdafacc5c77e5bc), UINT64_C(0x0e7af78f5873c5bc)
    },
    {   UINT64_C(0x00000000000007e9), UINT64_C(0x106af60cc07c0667), UINT64_C(0x106af60cc07c0667),
        UINT64_C(0x106af60cc07c069e), UINT64_C(0x106af60cc07c069e)
    },
    {   UINT64_C(0x00000000000189f0), UINT64_C(0x156afa8fa08dc630), UINT64_C(0x23caf6cda4c94713),
        UINT64_C(0x15aafb0fa08ea67e), UINT64_C(0x156afd0f348ca5f9)
    },
    {   UINT64_C(0x0000000000005557), UINT64_C(0x1f42ff1028b746e1), UINT64_C(0x1f42f7ced4b6a715),
        UINT64_C(0x1f4305d074b446cb), UINT64_C(0x1f42fe4f18b36712)
    },
    {   UINT64_C(0x000000000000b898), UINT64_C(0x0000000000000000), UINT64_C(0x105af84f787aa58e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b899), UINT64_C(0x0000000000000000), UINT64_C(0x105b0dcfa071258d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b89a), UINT64_C(0x0000000000000000), UINT64_C(0x105aff0fa077658d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009152), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x13e2f6cf6889268c), UINT64_C(0x107af30fa08c467e)
    },
    {   UINT64_C(0x000000000000b89c), UINT64_C(0x0000000000000000), UINT64_C(0x105afc8c887a858c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000555d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68f9cb76702),
        UINT64_C(0x1f42fe0f44b306fb), UINT64_C(0x1f42fd8eacb1a702)
    },
    {   UINT64_C(0x000000000000b89e), UINT64_C(0x0000000000000000), UINT64_C(0x105af74f807b464c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b89f), UINT64_C(0x0000000000000000), UINT64_C(0x105af44c887bc58d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8a0), UINT64_C(0x0000000000000000), UINT64_C(0x105af9cf707a2646),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009158), UINT64_C(0x114af94f707e05b2), UINT64_C(0x11f2fbcc3880a5b4),
        UINT64_C(0x11f2f94c387f65b4), UINT64_C(0x096afbcfa07165b7)
    },
    {   UINT64_C(0x0000000000009159), UINT64_C(0x103af70f787ae5b4), UINT64_C(0x0a4af30f7863a661),
        UINT64_C(0x1102fbcf787ba5aa), UINT64_C(0x0ddaf30fa07a65aa)
    },
    {   UINT64_C(0x000000000000b8a3), UINT64_C(0x0002fb532c7025c2), UINT64_C(0x00030597307806c6),
        UINT64_C(0x156afa0e0c8e2656), UINT64_C(0x0a2aedcfa067446f)
    },
    {   UINT64_C(0x000000000000b8a4), UINT64_C(0x13caedcfa08dc46f), UINT64_C(0x0a2aeddc706747a3),
        UINT64_C(0x2b62f78fa0e7467e), UINT64_C(0x382afc8b9117e6e3)
    },
    {   UINT64_C(0x00000000000007f9), UINT64_C(0x106af88c287a8667), UINT64_C(0x106af88c287a8667),
        UINT64_C(0x106af8cf8c7a8667), UINT64_C(0x106af8cf8c7a869e)
    },
    {   UINT64_C(0x00000000000007fa), UINT64_C(0x106aedcfa07fe728), UINT64_C(0x106aedcfa07fe6c7),
        UINT64_C(0x0000000000000000), UINT64_C(0x106ade4fa07de672)
    },
    {   UINT64_C(0x00000000000007fb), UINT64_C(0x0000000000000000), UINT64_C(0x106b11d7046e064b),
        UINT64_C(0x106b1197046e064b), UINT64_C(0x106b20170467264c)
    },
    {   UINT64_C(0x0000000000005568), UINT64_C(0x0000000000000000), UINT64_C(0x1f43014e7caf6735),
        UINT64_C(0x1f42ff0f94af0700), UINT64_C(0x1f430c8f08a88718)
    },
    {   UINT64_C(0x0000000000009160), UINT64_C(0x1ae2fb8f3ca3a6a5), UINT64_C(0x1abafa8f54a286a5),
        UINT64_C(0x230afa8f54c3c6a5), UINT64_C(0x199afa8f3c9f66a5)
    },
    {   UINT64_C(0x0000000000009161), UINT64_C(0x1a2afa8c6ca046a5), UINT64_C(0x179af90c949666a5),
        UINT64_C(0x12c2fbcf4082a5c3), UINT64_C(0x117af94f407d869f)
    },
    {   UINT64_C(0x0000000000009162), UINT64_C(0x0e82fbcf407205c3), UINT64_C(0x116afa0f3c7d469e),
        UINT64_C(0x0ee2fbcf4073e5c3), UINT64_C(0x0b92f64c486a86a4)
    },
    {   UINT64_C(0x0000000000009163), UINT64_C(0x11b2fbcc347dc5c3), UINT64_C(0x1202f94c347f069f),
        UINT64_C(0x0d6af6cf406de6a1), UINT64_C(0x0c2af80c3468a6a1)
    },
    {   UINT64_C(0x0000000000009164), UINT64_C(0x0e42fa0f4070c69f), UINT64_C(0x0d8af94f406de69f),
        UINT64_C(0x1ec2f80f40b305c3), UINT64_C(0x15c2f80f408f05c3)
    },
    {   UINT64_C(0x0000000000009165), UINT64_C(0x1152f94f407ce5c3), UINT64_C(0x1402f70c3487a639),
        UINT64_C(0x11f2fbcc347fc5c3), UINT64_C(0x106af80f407c25c3)
    },
    {   UINT64_C(0x0000000000009166), UINT64_C(0x116afb0f247da66b), UINT64_C(0x0db2f94f446dc678),
        UINT64_C(0x154af6cf408d25c3), UINT64_C(0x11eafbcf40804639)
    },
    {   UINT64_C(0x0000000000009167), UINT64_C(0x1a92fbcf40a1e639), UINT64_C(0x104afa8c5c7ba5c3),
        UINT64_C(0x12a2fbcc34818639), UINT64_C(0x0f22f70c347525c3)
    },
    {   UINT64_C(0x0000000000009168), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa082e67e),
        UINT64_C(0x14faf4cf688e268c), UINT64_C(0x15a3008fa090867e)
    },
    {   UINT64_C(0x0000000000009169), UINT64_C(0x123b008fa082067e), UINT64_C(0x10bb008fa07fe67e),
        UINT64_C(0x15baf4cf688ce68c), UINT64_C(0x1493008fa087867e)
    },
    {   UINT64_C(0x000000000000916a), UINT64_C(0x0c8af94fa06b067e), UINT64_C(0x0c72f10f6864a67e),
        UINT64_C(0x13d3008fa08d467e), UINT64_C(0x117b008fa082c67e)
    },
    {   UINT64_C(0x000000000000916b), UINT64_C(0x1a5b00cfa09e867e), UINT64_C(0x151b008f6889a67e),
        UINT64_C(0x1652f58f6894068c), UINT64_C(0x106b008fa07e867e)
    },
    {   UINT64_C(0x000000000000916c), UINT64_C(0x1662f58ce494e68c), UINT64_C(0x126b008fa087267e),
        UINT64_C(0x10eaf60f687aa68c), UINT64_C(0x1302f1cfa088a67e)
    },
    {   UINT64_C(0x000000000000916d), UINT64_C(0x14aaff4f6888867e), UINT64_C(0x1472f5cf748c667e),
        UINT64_C(0x1ccaf54f74ae067e), UINT64_C(0x15e2f5cfa092267e)
    },
    {   UINT64_C(0x000000000000916e), UINT64_C(0x13b2f34fa08aa67e), UINT64_C(0x13eaf50fa089e67e),
        UINT64_C(0x0e02f6cf786f65b7), UINT64_C(0x0f82fdcf7878c694)
    },
    {   UINT64_C(0x000000000000916f), UINT64_C(0x0e42f3cf787485b7), UINT64_C(0x0f72f50f74752694),
        UINT64_C(0x0de2f3cf787405b7), UINT64_C(0x0a82f30fa06ca693)
    },
    {   UINT64_C(0x0000000000009170), UINT64_C(0x0f82f34c4078e5b7), UINT64_C(0x0f3afdcfa0738694),
        UINT64_C(0x06e2fe0fa052068f), UINT64_C(0x07f2db8c4055a68f)
    },
    {   UINT64_C(0x0000000000009171), UINT64_C(0x0e2afdcfa0770694), UINT64_C(0x066afd4fa0508694),
        UINT64_C(0x17a2fe4fa094e5b8), UINT64_C(0x0f3afdcfa07385b8)
    },
    {   UINT64_C(0x0000000000009172), UINT64_C(0x0f9af50f787965b7), UINT64_C(0x0f92fecc407945b8),
        UINT64_C(0x0f92f50c4074e5b8), UINT64_C(0x0a82fdcfa06785b8)
    },
    {   UINT64_C(0x0000000000009173), UINT64_C(0x0c2afa4f786a45b7), UINT64_C(0x091aee4f7862662c),
        UINT64_C(0x0f5afdcf747445ad), UINT64_C(0x0ed2f5cf6875e5ad)
    },
    {   UINT64_C(0x0000000000009174), UINT64_C(0x15f2f50f6892e5ad), UINT64_C(0x0f12f58fa07705ad),
        UINT64_C(0x0f22f50c407725ad), UINT64_C(0x0dfaf44fa072a5ad)
    },
    {   UINT64_C(0x000000000001511d), UINT64_C(0x1fc2f88fa0c46685), UINT64_C(0x1b2af88fa0b20685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1322f88fa09a2685)
    },
    {   UINT64_C(0x000000000001511e), UINT64_C(0x1952fbcd389d0695), UINT64_C(0x190af88fa0a08685),
        UINT64_C(0x14fafbcf78968695), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x0000000000009177), UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x0f0af1ce2071468c),
        UINT64_C(0x1643020fa096e68c), UINT64_C(0x12db020fa087e68c)
    },
    {   UINT64_C(0x0000000000009178), UINT64_C(0x1c8b020fa0a6a68c), UINT64_C(0x16a3020f588f068c),
        UINT64_C(0x1852f6cf589b669c), UINT64_C(0x122b020fa085a68c)
    },
    {   UINT64_C(0x0000000000009179), UINT64_C(0x185af6cce09cc69b), UINT64_C(0x14f3020fa090a68c),
        UINT64_C(0x1152f6cf547e669b), UINT64_C(0x140af18fa08ce68c)
    },
    {   UINT64_C(0x000000000000b8c3), UINT64_C(0x0a6ae80c387546dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x16daf64c3c98e6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c4), UINT64_C(0x15f2f1cc389466dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1bcaf10c3caa86dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000015124), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x11dae5cc6c83c5bd),
        UINT64_C(0x1072f90c6c8105b6), UINT64_C(0x0dcaf64fa07385bb)
    },
    {   UINT64_C(0x000000000000b8c6), UINT64_C(0x1692f68c389486dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f28c3c9666dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c7), UINT64_C(0x116af20c388526dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1382f58c3c8986dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c8), UINT64_C(0x131af24c3888a6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f54c389666dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c9), UINT64_C(0x17daf6cc3c9986dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f08c3c9686dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8ca), UINT64_C(0x173af38c3c9906dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1742f54c3c97e6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009182), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa08ea67e),
        UINT64_C(0x14fb060f689ce68c), UINT64_C(0x15a3008fa09c667e)
    },
    {   UINT64_C(0x0000000000009183), UINT64_C(0x123b008fa093467e), UINT64_C(0x10bb008fa093667e),
        UINT64_C(0x15bb064f689ce68c), UINT64_C(0x1493008fa09c267e)
    },
    {   UINT64_C(0x0000000000009184), UINT64_C(0x0c8afccfa07e867e), UINT64_C(0x0c72f38f6879467e),
        UINT64_C(0x13d3008fa099a67e), UINT64_C(0x117b008fa085067e)
    },
    {   UINT64_C(0x000000000000558e), UINT64_C(0x1f42fd8f88b46717), UINT64_C(0x1f42f80e68b8073e),
        UINT64_C(0x1f43024f0cb44716), UINT64_C(0x1f43004e70b7a735)
    },
    {   UINT64_C(0x000000000001805f), UINT64_C(0x0de2f4cfa072c656), UINT64_C(0x1322f08fa0894784),
        UINT64_C(0x0de2f4cfa072c68a), UINT64_C(0x0a72f30fa06e6693)
    },
    {   UINT64_C(0x0000000000009187), UINT64_C(0x14ab0b4f689d267e), UINT64_C(0x14731f0f74a1067e),
        UINT64_C(0x1ccb1e8f74c2a67e), UINT64_C(0x15e2f5cfa0a3467e)
    },
    {   UINT64_C(0x0000000000009188), UINT64_C(0x13b31c8fa09f467e), UINT64_C(0x13eaf50fa09de67e),
        UINT64_C(0x0e02fbcf7878e5b7), UINT64_C(0x0f83008f78816694)
    },
    {   UINT64_C(0x0000000000005592), UINT64_C(0x1f42f84fd0b566fb), UINT64_C(0x1f42f64f88b246fb),
        UINT64_C(0x1f4305cec0ac0716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000918a), UINT64_C(0x0f82ec4c408665b7), UINT64_C(0x0f3afdcfa07e4694),
        UINT64_C(0x06e2ff4fa067268f), UINT64_C(0x07f2d08c406ac68f)
    },
    {   UINT64_C(0x000000000000918b), UINT64_C(0x0e2afdcfa0840694), UINT64_C(0x066afd4fa065a694),
        UINT64_C(0x17a2fe4fa09f65b8), UINT64_C(0x0f3afdcfa07e45b8)
    },
    {   UINT64_C(0x000000000000918c), UINT64_C(0x0f9b010f788185b7), UINT64_C(0x0f92f18c4081c5b8),
        UINT64_C(0x0f93000c408325b8), UINT64_C(0x0a82fdcfa07565b8)
    },
    {   UINT64_C(0x0000000000015135), UINT64_C(0x19d305cf4ca166a3), UINT64_C(0x2092fe4f4cbea6a3),
        UINT64_C(0x1a13028f4ca266a3), UINT64_C(0x249afe4f4ccec6a3)
    },
    {   UINT64_C(0x000000000000918e), UINT64_C(0x15f3110f68a0e5ad), UINT64_C(0x0f12f58fa08285ad),
        UINT64_C(0x0f22e94c408525ad), UINT64_C(0x0dfaf44fa07f25ad)
    },
    {   UINT64_C(0x0000000000015137), UINT64_C(0x28a2fd0f4cda26a3), UINT64_C(0x223afd0f4cc546a3),
        UINT64_C(0x194b000f4c9c26a3), UINT64_C(0x200afbcf4cbac6a3)
    },
    {   UINT64_C(0x000000000000b8d9), UINT64_C(0x150afbcbe091a6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000015139), UINT64_C(0x1e22fd0f4caf86a3), UINT64_C(0x2082fd0f4cbe86a3),
        UINT64_C(0x2942fd0f4ce1c6a3), UINT64_C(0x204afe0f4cbcc6a3)
    },
    {   UINT64_C(0x000000000000b8db), UINT64_C(0x142adc0c3892e6ed), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009193), UINT64_C(0x185b084ce0a7669b), UINT64_C(0x14f3020fa095c68c),
        UINT64_C(0x1152fe8f548e269b), UINT64_C(0x140b154fa0a0a68c)
    },
    {   UINT64_C(0x0000000000009194), UINT64_C(0x15db0ccf54a1868c), UINT64_C(0x15931e4f58a6a68c),
        UINT64_C(0x1f2b1f8f64cc668c), UINT64_C(0x1722f80fa0a9268c)
    },
    {   UINT64_C(0x0000000000009195), UINT64_C(0x15b31d0fa0a7c68c), UINT64_C(0x146af44fa09ea68c),
        UINT64_C(0x0f4afa0f687de5c1), UINT64_C(0x1002facf6c84468c)
    },
    {   UINT64_C(0x0000000000009196), UINT64_C(0x0e72ffcf688005c1), UINT64_C(0x1032ff4f688ca68c),
        UINT64_C(0x0e22ffcf6c7c05c1), UINT64_C(0x0aa3000fa081269c)
    },
    {   UINT64_C(0x0000000000009197), UINT64_C(0x0fc2ee8c688965c0), UINT64_C(0x0ff2fbcfa082468c),
        UINT64_C(0x07aafccfa06c4698), UINT64_C(0x0a22d78c78758698)
    },
    {   UINT64_C(0x0000000000009198), UINT64_C(0x103afbcfa089668c), UINT64_C(0x075afbcfa06a068c),
        UINT64_C(0x1842fbcfa0a3a5c1), UINT64_C(0x0ff2fbcfa08245c1)
    },
    {   UINT64_C(0x0000000000009199), UINT64_C(0x0fab000f688245c1), UINT64_C(0x1002eecc6c8465c1),
        UINT64_C(0x1002ff4c6c8645c1), UINT64_C(0x0cdafbcfa07f65c1)
    },
    {   UINT64_C(0x000000000000919a), UINT64_C(0x0c0af74f6c75c5c1), UINT64_C(0x0c13068f7076665f),
        UINT64_C(0x1033024f6485a5b6), UINT64_C(0x0e730e4f6884a5b6)
    },
    {   UINT64_C(0x000000000000919b), UINT64_C(0x15eb0ecf68a245b6), UINT64_C(0x0fd2f30fa08745b6),
        UINT64_C(0x0ec2e74c7085a5b6), UINT64_C(0x0dcaf34fa080c5b6)
    },
    {   UINT64_C(0x000000000000919c), UINT64_C(0x106af48fa07c8689), UINT64_C(0x106af54fa07a867e),
        UINT64_C(0x106af80f747ac688), UINT64_C(0x106af44fa07ae67e)
    },
    {   UINT64_C(0x000000000000919d), UINT64_C(0x106af5cfa07d067e), UINT64_C(0x106af68fa07b867e),
        UINT64_C(0x106af74f747c6688), UINT64_C(0x106af48fa07c867e)
    },
    {   UINT64_C(0x000000000001806b), UINT64_C(0x0000000000000000), UINT64_C(0x09b2f18f78682693),
        UINT64_C(0x0000000000000000), UINT64_C(0x17f2f18f68a3c6ee)
    },
    {   UINT64_C(0x000000000000919f), UINT64_C(0x106af34fa07d267e), UINT64_C(0x106af5cfa07be67e),
        UINT64_C(0x106afc0f7478c688), UINT64_C(0x106af5cfa07a667e)
    },
    {   UINT64_C(0x00000000000091a0), UINT64_C(0x106afc0d7478c688), UINT64_C(0x106af48f747f867e),
        UINT64_C(0x106afb0f74794688), UINT64_C(0x106af48fa07c867e)
    },
    {   UINT64_C(0x00000000000091a1), UINT64_C(0x106af04f747ec67e), UINT64_C(0x106af24f807dc67e),
        UINT64_C(0x106af0cf807e867e), UINT64_C(0x106af40fa07ca67e)
    },
    {   UINT64_C(0x00000000000091a2), UINT64_C(0x106af1cfa07de67e), UINT64_C(0x106af9cfa07a667e),
        UINT64_C(0x106afc8f887f05b3), UINT64_C(0x106af24f887aa649)
    },
    {   UINT64_C(0x00000000000055ac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e9cb3270e), UINT64_C(0x1f430b8e60b16725)
    },
    {   UINT64_C(0x00000000000091a4), UINT64_C(0x106af54c107e05b5), UINT64_C(0x106af24fa07da649),
        UINT64_C(0x106b030fa076864a), UINT64_C(0x106af9cc146bc64a)
    },
    {   UINT64_C(0x00000000000091a5), UINT64_C(0x106af40fa07da649), UINT64_C(0x106afdcfa077e649),
        UINT64_C(0x106aee8fa07f85b0), UINT64_C(0x106af24fa07da5b0)
    },
    {   UINT64_C(0x000000000001806f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa3004f7073e603), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091a7), UINT64_C(0x106b000f8877a5b3), UINT64_C(0x106af44f8876862c),
        UINT64_C(0x106af24f887da5aa), UINT64_C(0x106af38f907d05aa)
    },
    {   UINT64_C(0x000000000000b8f1), UINT64_C(0x156afa0d848e265b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1c32ee0d30aa667e)
    },
    {   UINT64_C(0x00000000000091a9), UINT64_C(0x08b2f9cf745be5a1), UINT64_C(0x08b2cf0c645d45a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091aa), UINT64_C(0x16faf04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1362f20fa088a68c), UINT64_C(0x159af20fa090869a)
    },
    {   UINT64_C(0x00000000000091ab), UINT64_C(0x153af20fa08ea68c), UINT64_C(0x15aaf50fa08dc68c),
        UINT64_C(0x1952f34fa09d668c), UINT64_C(0x1852f6cf589b669c)
    },
    {   UINT64_C(0x00000000000091ac), UINT64_C(0x0d2af30fa06cc68c), UINT64_C(0x19a2f58fa09ea68c),
        UINT64_C(0x161af04fa092c69a), UINT64_C(0x1e62f18fa0b1a68c)
    },
    {   UINT64_C(0x00000000000091ad), UINT64_C(0x1692f20f5896268c), UINT64_C(0x137af50fa088e68c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x1862f34fa09d668c)
    },
    {   UINT64_C(0x00000000000091ae), UINT64_C(0x131af20fa089868c), UINT64_C(0x1852f6cf589b669c),
        UINT64_C(0x14caf18fa08ce68c), UINT64_C(0x14daf5cfa08e068c)
    },
    {   UINT64_C(0x00000000000091af), UINT64_C(0x1692eecfa095e69c), UINT64_C(0x1a22f28fa0a4c68c),
        UINT64_C(0x175af20fa095e68c), UINT64_C(0x1902eecfa0a0a69c)
    },
    {   UINT64_C(0x00000000000091b0), UINT64_C(0x1852f6cfa098e69c), UINT64_C(0x159af20f6890868c),
        UINT64_C(0x1422f44f688bc5c1), UINT64_C(0x1162f94c6c80069c)
    },
    {   UINT64_C(0x00000000000091b1), UINT64_C(0x1032f10c6c7d25c1), UINT64_C(0x0fb2f48f687a269c),
        UINT64_C(0x0de2f50f687425c1), UINT64_C(0x0e5af3cc6c75269c)
    },
    {   UINT64_C(0x00000000000091b2), UINT64_C(0x1222f10c6c82a5c1), UINT64_C(0x0faaf44f687a069c),
        UINT64_C(0x0a32f1cf686485b5), UINT64_C(0x122af34fa08465c1)
    },
    {   UINT64_C(0x00000000000091b3), UINT64_C(0x1112f2cf5880669c), UINT64_C(0x1312f94c6c8805b5),
        UINT64_C(0x1032f1cf687c65c1), UINT64_C(0x0e8af3cc6c75669c)
    },
    {   UINT64_C(0x00000000000091b4), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x13baf30f588aa5b5),
        UINT64_C(0x111af94c6c7ee5c1), UINT64_C(0x0e82f3cc747605c1)
    },
    {   UINT64_C(0x00000000000091b5), UINT64_C(0x11c2f44f6880a5b5), UINT64_C(0x0f62f28f6877a5b5),
        UINT64_C(0x1202f10f688365c1), UINT64_C(0x146af40c6c8d25c1)
    },
    {   UINT64_C(0x00000000000091b6), UINT64_C(0x1322f34c6c87c5c1), UINT64_C(0x17daf10c6c9c65c1),
        UINT64_C(0x16f2f48f689705c1), UINT64_C(0x101af44f687be6ab)
    },
    {   UINT64_C(0x00000000000091b7), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x143af10f688c069c),
        UINT64_C(0x119af10f5482c5c1), UINT64_C(0x146af40c6c8d268c)
    },
    {   UINT64_C(0x00000000000091b8), UINT64_C(0x0ffaf44c6c7b45c1), UINT64_C(0x1b02f04f68a8e5b5),
        UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675)
    },
    {   UINT64_C(0x00000000000091b9), UINT64_C(0x1342f78fa098a675), UINT64_C(0x12a2f58fa0840683),
        UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1822fd0fa0a2c675)
    },
    {   UINT64_C(0x00000000000091ba), UINT64_C(0x1b4af78fa0b1e675), UINT64_C(0x170afa8f74978685),
        UINT64_C(0x0f02f78fa080c675), UINT64_C(0x17d2f78fa0a2c675)
    },
    {   UINT64_C(0x00000000000091bb), UINT64_C(0x1672f6cfa0944683), UINT64_C(0x1f6af78fa0c26675),
        UINT64_C(0x1a9af78fa0af2675), UINT64_C(0x1612fb0fa09b2675)
    },
    {   UINT64_C(0x00000000000091bc), UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1b4af78fa0b1e675),
        UINT64_C(0x1292f78fa0968675), UINT64_C(0x170afa8f74978685)
    },
    {   UINT64_C(0x00000000000091bd), UINT64_C(0x16fafc8fa09d8675), UINT64_C(0x1132f44fa0924675),
        UINT64_C(0x132af50fa09b6684), UINT64_C(0x1822f44fa09be675)
    },
    {   UINT64_C(0x00000000000091be), UINT64_C(0x18b2f44fa0a3c675), UINT64_C(0x14f2f50fa09b6683),
        UINT64_C(0x1902f60fa09fa682), UINT64_C(0x139af88f64898675)
    },
    {   UINT64_C(0x00000000000091bf), UINT64_C(0x1062f80f7880a5a1), UINT64_C(0x0f6af44cc47f4684),
        UINT64_C(0x0d62f6ccb477c5a1), UINT64_C(0x0e7af80f7476e684)
    },
    {   UINT64_C(0x00000000000091c0), UINT64_C(0x0f22f44f747605a1), UINT64_C(0x0d9af80cc47a8684),
        UINT64_C(0x0e62f58cc47765a1), UINT64_C(0x0f22f80f7479c684)
    },
    {   UINT64_C(0x00000000000055ca), UINT64_C(0x1f42f60f08ac8728), UINT64_C(0x1f42fa4f2cb28709),
        UINT64_C(0x1f42ff8fc0b60729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091c2), UINT64_C(0x0e22fa8f7c7845a6), UINT64_C(0x0d8af44cc475e684),
        UINT64_C(0x0e52f80f747545a1), UINT64_C(0x1172f6cf6c858594)
    },
    {   UINT64_C(0x00000000000091c3), UINT64_C(0x0fb2f58cc47985a1), UINT64_C(0x0dbaf6ccc47885d2),
        UINT64_C(0x10caf80f7482e594), UINT64_C(0x0dd2f58f8c798594)
    },
    {   UINT64_C(0x00000000000091c4), UINT64_C(0x0e62f58f7476259f), UINT64_C(0x14d2fa8cc48d65a1),
        UINT64_C(0x129af58c788f25a1), UINT64_C(0x14aaf58cc49765a1)
    },
    {   UINT64_C(0x00000000000107a0), UINT64_C(0x1a0affcf58afc758), UINT64_C(0x14daf1cf588f8680),
        UINT64_C(0x1a0affcf58afc754), UINT64_C(0x14daf1cf588f8680)
    },
    {   UINT64_C(0x00000000000107a1), UINT64_C(0x1a0affcf58afc71a), UINT64_C(0x14daf1cf588f8646),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091c7), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x13d2f88fa09c2685), UINT64_C(0x13c2f60fa0882695)
    },
    {   UINT64_C(0x00000000000091c8), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x1bdaf88fa0b4c685), UINT64_C(0x187afbcf609ce695)
    },
    {   UINT64_C(0x00000000000055d2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4fa0b6470d),
        UINT64_C(0x1f42f70f94afa6fa), UINT64_C(0x1f42fa8edcb786c9)
    },
    {   UINT64_C(0x00000000000091ca), UINT64_C(0x1b2af88fa0b20685), UINT64_C(0x1672fc4fa09d2685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1bbaf7cfa0b44685)
    },
    {   UINT64_C(0x00000000000091cb), UINT64_C(0x1322f88fa09a2685), UINT64_C(0x187afbcf609ce695),
        UINT64_C(0x17bafe0fa0a0e685), UINT64_C(0x11c2f50fa0960685)
    },
    {   UINT64_C(0x00000000000055d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4f84b4c700)
    },
    {   UINT64_C(0x00000000000091cd), UINT64_C(0x19d2fc4fa0a46695), UINT64_C(0x13c2f8cf60898685),
        UINT64_C(0x1382f90f708d45b5), UINT64_C(0x115af50c6c876695)
    },
    {   UINT64_C(0x00000000000091ce), UINT64_C(0x0f52f90c787d65b6), UINT64_C(0x10d2f90f8081c695),
        UINT64_C(0x0f7af50f7876e5b6), UINT64_C(0x0ec2f90c747fc695)
    },
    {   UINT64_C(0x00000000000055d8), UINT64_C(0x1f43164fb0b2a6e2), UINT64_C(0x1f42f94f58b7a725),
        UINT64_C(0x1f4301ced8b02705), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055d9), UINT64_C(0x1f42fcce88afc730), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430010bcb466e5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091d1), UINT64_C(0x118af90f7881e5b6), UINT64_C(0x121af7cf6c8845aa),
        UINT64_C(0x129af64c6c84c5b6), UINT64_C(0x0f02f7cc747e05b6)
    },
    {   UINT64_C(0x00000000000091d2), UINT64_C(0x1282f90f7889e5aa), UINT64_C(0x0eaaf64f847d45aa),
        UINT64_C(0x1142f64f788165b6), UINT64_C(0x1612fbcc6c91e5b6)
    },
    {   UINT64_C(0x00000000000091d3), UINT64_C(0x136af64c709305b6), UINT64_C(0x15eaf64c6c9d05b6),
        UINT64_C(0x157af38f7891a5b6), UINT64_C(0x117af90f78846696)
    },
    {   UINT64_C(0x00000000000091d4), UINT64_C(0x0f0af90f787aa5b5), UINT64_C(0x12faf64f6c8786a2),
        UINT64_C(0x12faeb0f648d45b6), UINT64_C(0x155af90c6c906695)
    },
    {   UINT64_C(0x00000000000091d5), UINT64_C(0x124af90c6c84e5b6), UINT64_C(0x1b22f64f78ac65aa),
        UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c)
    },
    {   UINT64_C(0x00000000000091d6), UINT64_C(0x127b020fa085a68c), UINT64_C(0x1682f80fa093869a),
        UINT64_C(0x13db020fa089268c), UINT64_C(0x146af44fa08c668c)
    },
    {   UINT64_C(0x00000000000091d7), UINT64_C(0x165b020fa08de68c), UINT64_C(0x1852f6cf549b669b),
        UINT64_C(0x0dc2fe4fa06d668c), UINT64_C(0x1643020fa096e68c)
    },
    {   UINT64_C(0x00000000000091d8), UINT64_C(0x1612f80fa091c69a), UINT64_C(0x1c8b020fa0a6a68c),
        UINT64_C(0x16a3020f588f068c), UINT64_C(0x1402f6cfa08a268c)
    },
    {   UINT64_C(0x00000000000091d9), UINT64_C(0x1852f6cf589b669c), UINT64_C(0x165b020fa08de68c),
        UINT64_C(0x122b020fa085a68c), UINT64_C(0x1852f6cf549b669b)
    },
    {   UINT64_C(0x00000000000091da), UINT64_C(0x1512f50fa08d868c), UINT64_C(0x140af18fa08ce68c),
        UINT64_C(0x1762f30fa099869b), UINT64_C(0x1902f44fa09f668c)
    },
    {   UINT64_C(0x00000000000091db), UINT64_C(0x1722f80fa096068c), UINT64_C(0x1932f1cfa0a1669b),
        UINT64_C(0x1782f60fa098869b), UINT64_C(0x16faf94f28942680)
    },
    {   UINT64_C(0x00000000000091dc), UINT64_C(0x14b2f80f688d25c1), UINT64_C(0x1002f7cc6c7b669c),
        UINT64_C(0x0fb2f08c6c7a25c1), UINT64_C(0x0fb2f44f687a269c)
    },
    {   UINT64_C(0x00000000000091dd), UINT64_C(0x0e1af44f6874c5c1), UINT64_C(0x0fa2f50c6c7b869c),
        UINT64_C(0x0ff2f6cc6c79c5c1), UINT64_C(0x0fa2f3cf687a269c)
    },
    {   UINT64_C(0x00000000000091de), UINT64_C(0x09fafb4f686385b5), UINT64_C(0x107afbcfa07e85c1),
        UINT64_C(0x1152f28f687f869c), UINT64_C(0x1132f7cc6c7f25b5)
    },
    {   UINT64_C(0x00000000000091df), UINT64_C(0x0f62fbcf687985c1), UINT64_C(0x0faaf50c6c7ba69c),
        UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1292f08f688605b5)
    },
    {   UINT64_C(0x00000000000091e0), UINT64_C(0x0ffaf78c6c7b45c1), UINT64_C(0x0f92f48c6c7b45c1),
        UINT64_C(0x1042f44f688385b5), UINT64_C(0x0efaef8f687985b5)
    },
    {   UINT64_C(0x00000000000107bc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1163018fa08da758), UINT64_C(0x0de2e80c6873c680)
    },
    {   UINT64_C(0x00000000000091e2), UINT64_C(0x1722f88f6894a5b5), UINT64_C(0x0faaf44f687a069b),
        UINT64_C(0x0e72f44f687585c1), UINT64_C(0x10c2f6cf687fc69c)
    },
    {   UINT64_C(0x00000000000091e3), UINT64_C(0x11d2f18f6883c5c1), UINT64_C(0x13d2f44c6c8aa69c),
        UINT64_C(0x0f72f44c6c7925c1), UINT64_C(0x1a82ef8f68a765b5)
    },
    {   UINT64_C(0x00000000000091e4), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x1612fd0fa092668c),
        UINT64_C(0x12d2fd0fa096a68c), UINT64_C(0x1682f80fa093869a)
    },
    {   UINT64_C(0x00000000000091e5), UINT64_C(0x155afd0fa097068c), UINT64_C(0x161af44fa09ea68c),
        UINT64_C(0x1762fd0fa0a0668c), UINT64_C(0x1a7b048f54a5469b)
    },
    {   UINT64_C(0x00000000000091e6), UINT64_C(0x0d9afa8fa07da68c), UINT64_C(0x1642fd0fa0a2068c),
        UINT64_C(0x1572f30fa08f469a), UINT64_C(0x1d2afd0fa0b9268c)
    },
    {   UINT64_C(0x00000000000091e7), UINT64_C(0x1742fd0f58a1868c), UINT64_C(0x1592f9cfa09aa68c),
        UINT64_C(0x1a8b048f58a5869c), UINT64_C(0x174afd0fa0a0668c)
    },
    {   UINT64_C(0x0000000000015190), UINT64_C(0x13db020fa099868c), UINT64_C(0x1233020fa099268c),
        UINT64_C(0x175308cf54a0a69b), UINT64_C(0x165b020fa0a2e68c)
    },
    {   UINT64_C(0x00000000000091e9), UINT64_C(0x14fb04cfa0a2669b), UINT64_C(0x1a1b000fa0a6e68c),
        UINT64_C(0x1722f30fa0a6a68c), UINT64_C(0x18bb0d4fa0af269b)
    },
    {   UINT64_C(0x00000000000091ea), UINT64_C(0x1982f9cfa0a4869b), UINT64_C(0x18630c0f289d8680),
        UINT64_C(0x1532f9cf689665c1), UINT64_C(0x1142eb0c6c87269c)
    },
    {   UINT64_C(0x00000000000091eb), UINT64_C(0x106b030c7085e5c1), UINT64_C(0x0fdaf58f6882e69c),
        UINT64_C(0x0fc2f94f687e05c1), UINT64_C(0x0f02fa4c6c81e69c)
    },
    {   UINT64_C(0x00000000000091ec), UINT64_C(0x10a2f78c6c8025c1), UINT64_C(0x1182fe4f6883a69c),
        UINT64_C(0x0a2afc0f686445b6), UINT64_C(0x10caf78fa08745c1)
    },
    {   UINT64_C(0x00000000000055f6), UINT64_C(0x1f42facef4b126f8), UINT64_C(0x1f43044f78a5a6fa),
        UINT64_C(0x1f42ff4ffca9070a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055f7), UINT64_C(0x1f4307cf44b2871b), UINT64_C(0x1f43004ff4b5c702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf44b6673a)
    },
    {   UINT64_C(0x0000000000015197), UINT64_C(0x0fc2ee8c688965c1), UINT64_C(0x0ff2fbcfa082468c),
        UINT64_C(0x07aafccfa06c4698), UINT64_C(0x0a22d78c78758698)
    },
    {   UINT64_C(0x0000000000015198), UINT64_C(0x103afbcfa089668c), UINT64_C(0x075afbcfa06a068c),
        UINT64_C(0x1842fbcfa0a3a5c1), UINT64_C(0x0ff2fbcfa08245c1)
    },
    {   UINT64_C(0x0000000000015199), UINT64_C(0x0fab000f688245c1), UINT64_C(0x1002eecc6c8465c1),
        UINT64_C(0x1002ff4c6c8645c1), UINT64_C(0x0cdafbcfa07f65c1)
    },
    {   UINT64_C(0x00000000000091f2), UINT64_C(0x1102f78c6c7f85c1), UINT64_C(0x1bcaf80f68b085b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001519b), UINT64_C(0x15eb0ecf68a245b6), UINT64_C(0x0fd2f30fa08745b6),
        UINT64_C(0x0ec2e74c7085a5b6), UINT64_C(0x0dcaf34fa080c5b6)
    },
    {   UINT64_C(0x00000000000091f4), UINT64_C(0x0fa2f24fa07a4698), UINT64_C(0x0fa2f20f68790698),
        UINT64_C(0x0fa2f2cfa07a0698), UINT64_C(0x0fa2f38f8079468c)
    },
    {   UINT64_C(0x00000000000091f5), UINT64_C(0x0fa2f50f6c79e698), UINT64_C(0x0fa2f24fa07a268c),
        UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79a698)
    },
    {   UINT64_C(0x00000000000091f6), UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e304cfa06ae69d),
        UINT64_C(0x111afa0fa07ec68c), UINT64_C(0x10e2fa4f687a468c)
    },
    {   UINT64_C(0x00000000000091f7), UINT64_C(0x10e2f30fa080068c), UINT64_C(0x10e2f6cf687ba67e),
        UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e2f40fa07e667e)
    },
    {   UINT64_C(0x00000000000091f8), UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e2f50f707e868c),
        UINT64_C(0x0fa2f3cf687a268c), UINT64_C(0x0fa3090fa064468d)
    },
    {   UINT64_C(0x00000000000091f9), UINT64_C(0x0fa2f6cfa079268c), UINT64_C(0x0fa2f5cf6875a68c),
        UINT64_C(0x0fa2f0cfa07ba68c), UINT64_C(0x0fa2f70f6877c68c)
    },
    {   UINT64_C(0x00000000000091fa), UINT64_C(0x0fa2f60f68794694), UINT64_C(0x0fa2f78f8076e67e),
        UINT64_C(0x0fa2fa4f6876668c), UINT64_C(0x0fa2f50f4c78c68c)
    },
    {   UINT64_C(0x00000000000091fb), UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa3028fa0684698),
        UINT64_C(0x0fa2f30fa079c698), UINT64_C(0x0fa2f28f6c7a6698)
    },
    {   UINT64_C(0x00000000000091fc), UINT64_C(0x0fa2f2cfa07b0698), UINT64_C(0x0fa2f14f6c7ae68c),
        UINT64_C(0x0fa2f48f6c79e698), UINT64_C(0x0fa2f6cfa079668c)
    },
    {   UINT64_C(0x00000000000091fd), UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79e698),
        UINT64_C(0x106afbcf7478e691), UINT64_C(0x106b098fa074e691)
    },
    {   UINT64_C(0x00000000000091fe), UINT64_C(0x106afb0fa0794691), UINT64_C(0x106af8cf747a6691),
        UINT64_C(0x106af54fa07c2692), UINT64_C(0x106afb0f74794686)
    },
    {   UINT64_C(0x00000000000091ff), UINT64_C(0x106afc8f74788691), UINT64_C(0x106af8cf747a6696),
        UINT64_C(0x106af8cf747a6691), UINT64_C(0x106afc8f74788691)
    },
    {   UINT64_C(0x0000000000005609), UINT64_C(0x1f42fc8fccb7e707), UINT64_C(0x1f430d8f90b2c702),
        UINT64_C(0x1f42fe4efcb34713), UINT64_C(0x1f43034fb8b446ef)
    },
    {   UINT64_C(0x0000000000018a4a), UINT64_C(0x1cf2fb4d30ab667e), UINT64_C(0x1cf2fb4d30ab667e),
        UINT64_C(0x1cf2fbcd30aba67e), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x000000000000560b), UINT64_C(0x1f42face6cb7071c), UINT64_C(0x1f42fb8f04b1470e),
        UINT64_C(0x1f43004eacb1e706), UINT64_C(0x1f43070e88b0c74c)
    },
    {   UINT64_C(0x0000000000018a4b), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e),
        UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x1cf2fbcd30ab667e)
    },
    {   UINT64_C(0x000000000000560d), UINT64_C(0x1f42fd0ef4b4c722), UINT64_C(0x1f42fe4f60b4871d),
        UINT64_C(0x1f42fc4e50aaa736), UINT64_C(0x1f42fd8ec0ad870a)
    },
    {   UINT64_C(0x000000000000560e), UINT64_C(0x1f42f74f6cb6e6db), UINT64_C(0x1f43018f84b04710),
        UINT64_C(0x1f42f7cf44b6a72a), UINT64_C(0x1f43194ef0b24735)
    },
    {   UINT64_C(0x000000000001809f), UINT64_C(0x0a6aef4bfc6865b4), UINT64_C(0x0b92efcbfc68e5b4),
        UINT64_C(0x09daf08fa068a5be), UINT64_C(0x0af2ef4fa06805be)
    },
    {   UINT64_C(0x0000000000005610), UINT64_C(0x1f43014f08b1e6ed), UINT64_C(0x1f430ccee0ac270d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005611), UINT64_C(0x1f42ff4f28af66f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005612), UINT64_C(0x1f43144f20ae8705), UINT64_C(0x1f42f80e78b96730),
        UINT64_C(0x1f42fe4f88b6c723), UINT64_C(0x1f430f4f28b0e71a)
    },
    {   UINT64_C(0x0000000000005613), UINT64_C(0x1f42f70ec0aa8709), UINT64_C(0x1f42f34f70b8a70a),
        UINT64_C(0x1f42fccf2cb7e70a), UINT64_C(0x1f42facf74b3671e)
    },
    {   UINT64_C(0x0000000000005614), UINT64_C(0x0000000000000000), UINT64_C(0x1f4304ceccac4718),
        UINT64_C(0x1f42fa4ea8b66711), UINT64_C(0x1f42fa0f28ae66d0)
    },
    {   UINT64_C(0x0000000000005615), UINT64_C(0x1f42fa4f3cb76735), UINT64_C(0x1f42fbced0b6c724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43158efcb5c71b)
    },
    {   UINT64_C(0x0000000000005616), UINT64_C(0x1f42fb0efcb6472d), UINT64_C(0x1f42ff8e68b00711),
        UINT64_C(0x1f42f78ee4ba4717), UINT64_C(0x1f42f78ebcb7e71a)
    },
    {   UINT64_C(0x00000000000048a7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118f90a9c6b2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005618), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8e80b0c729), UINT64_C(0x1f43164ee0b1c705)
    },
    {   UINT64_C(0x0000000000005619), UINT64_C(0x1f42ff0eccab86fe), UINT64_C(0x1f42f84f18b0c6f8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000561a), UINT64_C(0x1f42f8ce54b9871e), UINT64_C(0x1f42fd1058b4071c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e94b4272f)
    },
    {   UINT64_C(0x000000000000561b), UINT64_C(0x1f42fe4fe0ac06df), UINT64_C(0x1f42ff5058b506d7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f00b10702)
    },
    {   UINT64_C(0x000000000000561c), UINT64_C(0x1f43020eecb186fc), UINT64_C(0x1f43114eecadc721),
        UINT64_C(0x1f4301ce70b1a733), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000561d), UINT64_C(0x1f42fd4eacaca70c), UINT64_C(0x1f42fa8ef8af874a),
        UINT64_C(0x1f42ff0f74b46713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000561e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd8ee8aee741),
        UINT64_C(0x1f42f98ee0b0672d), UINT64_C(0x1f42fccf20b4c72a)
    },
    {   UINT64_C(0x000000000000561f), UINT64_C(0x1f42ffcef0b06715), UINT64_C(0x1f42fe0f28b4c716),
        UINT64_C(0x1f43078ef0b22721), UINT64_C(0x1f42ff0ec4b16711)
    },
    {   UINT64_C(0x0000000000005620), UINT64_C(0x1f42fb8e8cb3670f), UINT64_C(0x1f42fecf0cb00705),
        UINT64_C(0x1f42ff4eecaf86f7), UINT64_C(0x1f43008ef8b7472f)
    },
    {   UINT64_C(0x0000000000005621), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0f50ad66e8),
        UINT64_C(0x1f42fc4e5cafc71b), UINT64_C(0x1f42fb8e60b1e73c)
    },
    {   UINT64_C(0x0000000000005622), UINT64_C(0x1f42f8cf84b3871a), UINT64_C(0x1f42fb8ee4ae2713),
        UINT64_C(0x1f42fccee8b5872d), UINT64_C(0x1f42fb8e50b7c730)
    },
    {   UINT64_C(0x0000000000005623), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8e8cb38716), UINT64_C(0x1f43160f84ac0728)
    },
    {   UINT64_C(0x0000000000005624), UINT64_C(0x1f42fb0f2cb04730), UINT64_C(0x1f43008eb8ae0716),
        UINT64_C(0x1f42fe8ee4a90719), UINT64_C(0x1f42fecf08b5e736)
    },
    {   UINT64_C(0x00000000000180aa), UINT64_C(0x1452f14f788ea62b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005626), UINT64_C(0x1f42f5cef8b766f5), UINT64_C(0x1f42fa4f30b16721),
        UINT64_C(0x1f42fe4f08b4c6ce), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005627), UINT64_C(0x1f43124ef0b5c732), UINT64_C(0x1f42fccef8b5070a),
        UINT64_C(0x1f42fc8e80afc72c), UINT64_C(0x1f42f84f40b3c729)
    },
    {   UINT64_C(0x000000000001787d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa3044f6c86c698), UINT64_C(0x0fa3210fa07da698)
    },
    {   UINT64_C(0x00000000000151c8), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x1bdaf88fa0b4c685), UINT64_C(0x187afbcf609ce695)
    },
    {   UINT64_C(0x00000000000151c9), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x18daf88fa0a76685),
        UINT64_C(0x17baf7cfa0990695), UINT64_C(0x1fc2f88fa0c46685)
    },
    {   UINT64_C(0x0000000000017880), UINT64_C(0x16eaefcfa091e685), UINT64_C(0x16caf28fa097c685),
        UINT64_C(0x0000000000000000), UINT64_C(0x1542f28fa0966685)
    },
    {   UINT64_C(0x0000000000017881), UINT64_C(0x0ed2f28fa0966685), UINT64_C(0x1672f6cf68970695),
        UINT64_C(0x0bf2f38fa071e685), UINT64_C(0x12daf2cf688f0685)
    },
    {   UINT64_C(0x00000000000151cc), UINT64_C(0x140af5cfa0a08695), UINT64_C(0x19daf50fa0a26685),
        UINT64_C(0x1942f50fa0a64685), UINT64_C(0x15b2f58fa0a02695)
    },
    {   UINT64_C(0x0000000000017883), UINT64_C(0x1572f58f68976695), UINT64_C(0x0deaf58fa0902685),
        UINT64_C(0x162af6cf689ce685), UINT64_C(0x0fc2f88fa096a685)
    },
    {   UINT64_C(0x0000000000017884), UINT64_C(0x1bdb024fa0ba6685), UINT64_C(0x17baefcfa0a68685),
        UINT64_C(0x0e72f08fa097c685), UINT64_C(0x13d2f94f6c8825b6)
    },
    {   UINT64_C(0x0000000000017885), UINT64_C(0x14aaf68f6c8be6a3), UINT64_C(0x11daf94f6c82a5b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x11faf94f6c8205b6)
    },
    {   UINT64_C(0x0000000000017886), UINT64_C(0x0db2f94fa08bc6a3), UINT64_C(0x14d2f50c6c8ec5b6),
        UINT64_C(0x14aaf68fa08b86a3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000017887), UINT64_C(0x0000000000000000), UINT64_C(0x140af68fa08b86a3),
        UINT64_C(0x0ba2f68fa07226a3), UINT64_C(0x1d72f6cfa0aea5b6)
    },
    {   UINT64_C(0x0000000000017888), UINT64_C(0x14aaf68fa08b85b6), UINT64_C(0x1252f94f6c8365b6),
        UINT64_C(0x1542eecc6c8f25b6), UINT64_C(0x14b2f94c6c8d45b6)
    },
    {   UINT64_C(0x0000000000017889), UINT64_C(0x0c9af64fa081c5b6), UINT64_C(0x10aaf64f6c7f65b6),
        UINT64_C(0x0c9af38f6873468c), UINT64_C(0x14aaf94f6c8ce5a9)
    },
    {   UINT64_C(0x0007a8f2de110807), UINT64_C(0x11631796707ec682), UINT64_C(0x116313d6707ce682),
        UINT64_C(0x1162fc4d9c71244d), UINT64_C(0x11631856707ce682)
    },
    {   UINT64_C(0x0000000000005636), UINT64_C(0x1f430e4f68acc714), UINT64_C(0x1f42fb4efcad0738),
        UINT64_C(0x1f42fc4f14b5a730), UINT64_C(0x1f42f8cef0b026f9)
    },
    {   UINT64_C(0x0000000000010809), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1bcafc4f749dc44c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005638), UINT64_C(0x1f42fa8f9cb5270c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cfa4b186ea), UINT64_C(0x1f42f84efcb1e6e0)
    },
    {   UINT64_C(0x00000000000180b4), UINT64_C(0x0a6b044fec5d05b4), UINT64_C(0x0a6b04544c5d05b4),
        UINT64_C(0x0a6b0010cc5f4555), UINT64_C(0x0a6affd0cc5f2555)
    },
    {   UINT64_C(0x000000000001080c), UINT64_C(0x22eb020f54c3e6aa), UINT64_C(0x2e3b020f54f126aa),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000563b), UINT64_C(0x1f430490a8b166ec), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ffcf68ae66ef), UINT64_C(0x1f42facf48b56707)
    },
    {   UINT64_C(0x0000000000017891), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1b3af78ee4a6c6af)
    },
    {   UINT64_C(0x0000000000017892), UINT64_C(0x179af64f789b06a7), UINT64_C(0x14daf70f709226a8),
        UINT64_C(0x1912f58fa0a386a0), UINT64_C(0x130af84f808cc6a8)
    },
    {   UINT64_C(0x000000000000ca60), UINT64_C(0x151b084c8085c6a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000017894), UINT64_C(0x142af70dd0a3e69f), UINT64_C(0x1ac2f78f68a506a7),
        UINT64_C(0x16b2f78f809486a8), UINT64_C(0x2512f78eeccdc6a9)
    },
    {   UINT64_C(0x0000000000017895), UINT64_C(0x1f2af70f04c7c72b), UINT64_C(0x1822f8cf78a046ab),
        UINT64_C(0x142af70fa099e6a0), UINT64_C(0x1852f8cd5c9aa6a8)
    },
    {   UINT64_C(0x0000000000005641), UINT64_C(0x1f42fe0fdcb186e9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430dce8cb44718), UINT64_C(0x1f430acf38b66710)
    },
    {   UINT64_C(0x0000000000005642), UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e78ade71f),
        UINT64_C(0x1f43098fd0adc6f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005643), UINT64_C(0x1f4310ce78b1a721), UINT64_C(0x1f430cce70b3672b),
        UINT64_C(0x1f43170e74b1a71d), UINT64_C(0x1f42ff4ed4ae4725)
    },
    {   UINT64_C(0x0000000000005644), UINT64_C(0x1f4303ced4b0a724), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b4ed0a9a6ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000151e4), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x1612fd0fa092668c),
        UINT64_C(0x12d2fd0fa096a68c), UINT64_C(0x1682f80fa093869a)
    },
    {   UINT64_C(0x0000000000005646), UINT64_C(0x1f43030e8cb106dc), UINT64_C(0x1f42f70eb4b526f2),
        UINT64_C(0x1f431a4ea8a1473e), UINT64_C(0x1f430f4f84aae6f8)
    },
    {   UINT64_C(0x00000000000151e6), UINT64_C(0x0d9afa8fa07da68c), UINT64_C(0x1642fd0fa0a2068c),
        UINT64_C(0x1572f30fa08f469a), UINT64_C(0x1d2afd0fa0b9268c)
    },
    {   UINT64_C(0x00000000000151e7), UINT64_C(0x1742fd0f58a1868c), UINT64_C(0x1592f9cfa09aa68c),
        UINT64_C(0x1a8b048f58a5869c), UINT64_C(0x174afd0fa0a0668c)
    },
    {   UINT64_C(0x00000000000151e8), UINT64_C(0x122afd0fa092e68c), UINT64_C(0x1a7b048f54a5469b),
        UINT64_C(0x15c2f34fa09c068c), UINT64_C(0x140b03cfa097e68c)
    },
    {   UINT64_C(0x000000000000564a), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8fd0b626dd),
        UINT64_C(0x1f430a8fc8b6871b), UINT64_C(0x1f43078f18b5a71c)
    },
    {   UINT64_C(0x00000000000151ea), UINT64_C(0x1982f9cfa0a4869b), UINT64_C(0x18630c0f289d8680),
        UINT64_C(0x1532f9cf689665c1), UINT64_C(0x1142eb0c6c87269c)
    },
    {   UINT64_C(0x000000000000ca6e), UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000151ec), UINT64_C(0x10a2f78c6c8025c1), UINT64_C(0x1182fe4f6883a69c),
        UINT64_C(0x0a2afc0f686445b6), UINT64_C(0x10caf78fa08745c1)
    },
    {   UINT64_C(0x000000000000ca70), UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x1b02fa8d98a44662),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000151ee), UINT64_C(0x10faf94f687ee5c1), UINT64_C(0x13c2f94f689085b6),
        UINT64_C(0x1182e5cc6c8185c1), UINT64_C(0x102afb0c6c83c5c1)
    },
    {   UINT64_C(0x00000000000151ef), UINT64_C(0x1332f94f688e65b6), UINT64_C(0x105af88f6882e5b6),
        UINT64_C(0x1132fd4f688045c1), UINT64_C(0x155afbcc6c90e5c1)
    },
    {   UINT64_C(0x00000000000151f0), UINT64_C(0x11fada0c6c8cc5c1), UINT64_C(0x15fb00cc6c9c65c1),
        UINT64_C(0x17a2fe0f689a05b5), UINT64_C(0x1122f94f6881e69b)
    },
    {   UINT64_C(0x0000000000005652), UINT64_C(0x1f43004ea4b64731), UINT64_C(0x1f42ff8e4cba2735),
        UINT64_C(0x1f43160e94afe735), UINT64_C(0x1f4302cef0b6c725)
    },
    {   UINT64_C(0x00000000000151f2), UINT64_C(0x1102f78c6c7f85c1), UINT64_C(0x1bcaf80f68b085b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018c04), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156af08fa092867e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178aa), UINT64_C(0x0000000000000000), UINT64_C(0x1972f90c64a746ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x17e2f90bf49846ef)
    },
    {   UINT64_C(0x0000000000010828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x131af00fa08f2675)
    },
    {   UINT64_C(0x0000000000010829), UINT64_C(0x0fa2f08f807f2686), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x23eaee0f6ccb4675)
    },
    {   UINT64_C(0x0000000000005658), UINT64_C(0x1f43078eacb2c73f), UINT64_C(0x1f43024e38b68737),
        UINT64_C(0x1f43034e70b7473d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178ae), UINT64_C(0x0000000000000000), UINT64_C(0x126afc4bf48c26ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1192f58bf49586ef)
    },
    {   UINT64_C(0x00000000000178af), UINT64_C(0x0000000000000000), UINT64_C(0x174b008bf4a426ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x12aafe8bf49f46ef)
    },
    {   UINT64_C(0x00000000000178b0), UINT64_C(0x0000000000000000), UINT64_C(0x1ea2fe8bf4cbe6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x191af68bf4a4e6ef)
    },
    {   UINT64_C(0x00000000000178b1), UINT64_C(0x0000000000000000), UINT64_C(0x11caf44bf496c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1682f88bf49626ef)
    },
    {   UINT64_C(0x00000000000178b2), UINT64_C(0x0000000000000000), UINT64_C(0x13b2f98bf488e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x124afc4bf48506ef)
    },
    {   UINT64_C(0x000000000000565e), UINT64_C(0x1f42fb4ec8b38721), UINT64_C(0x1f42f84fe8b466ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4fd8b646ec)
    },
    {   UINT64_C(0x000000000000565f), UINT64_C(0x1f42f44fe0b686f5), UINT64_C(0x1f42f88f70b5c6fa),
        UINT64_C(0x1f42fa4ffcb346ef), UINT64_C(0x1f42fb8ef8b406de)
    },
    {   UINT64_C(0x00000000000178b5), UINT64_C(0x0000000000000000), UINT64_C(0x12caf40bf48966ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x131af6cbf486a6ef)
    },
    {   UINT64_C(0x000000000000ca83), UINT64_C(0x2e62f60aa10e8720), UINT64_C(0x128af60aa09a2720),
        UINT64_C(0x128af60aa09a2720), UINT64_C(0x128af60aa09a2720)
    },
    {   UINT64_C(0x0000000000010834), UINT64_C(0x00027cd96c3c66e0), UINT64_C(0x000283996c3fc6e0),
        UINT64_C(0x0002a30d3021467e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010835), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x00027cd8303c66e0), UINT64_C(0x00027cd8303c66e0)
    },
    {   UINT64_C(0x0000000000010836), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x00027cd7cc440656)
    },
    {   UINT64_C(0x0000000000005665), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f65014b8a705), UINT64_C(0x1f42f54ed0b826f6)
    },
    {   UINT64_C(0x0000000000010838), UINT64_C(0x0000000000000000), UINT64_C(0x00027cd83041a6e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010839), UINT64_C(0x0001fb8d347bc7e7), UINT64_C(0x0002828d303b867e),
        UINT64_C(0x0002964d3033c67e), UINT64_C(0x0002594cd85a26bd)
    },
    {   UINT64_C(0x000000000001083a), UINT64_C(0x00027a8c3842e371), UINT64_C(0x0002769808456691),
        UINT64_C(0x000227d0fc51a58b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001083b), UINT64_C(0x0002f4cbb0786371), UINT64_C(0x0002f4cbb0786371),
        UINT64_C(0x00027ccbb03c63c0), UINT64_C(0x00027ccbb03c63c0)
    },
    {   UINT64_C(0x000000000000566a), UINT64_C(0x1f42f64f54b32705), UINT64_C(0x1f42fa4fe8b626fa),
        UINT64_C(0x1f42fb101cb1a6ee), UINT64_C(0x1f42f74f80ac86f9)
    },
    {   UINT64_C(0x000000000000566b), UINT64_C(0x1f42fa101cb586c7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4f70b626f5), UINT64_C(0x1f42f54fccae86ea)
    },
    {   UINT64_C(0x000000000000566c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f88aec6f3),
        UINT64_C(0x1f42fdd038b546e9), UINT64_C(0x1f42facf38b086f4)
    },
    {   UINT64_C(0x000000000000ca8f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x16f2f4cfa096e68c)
    },
    {   UINT64_C(0x0000000000010840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x15f2f6cf58964682), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010841), UINT64_C(0x0000000000000000), UINT64_C(0x1abaf9cf68a3868c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010842), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1712f58c349345a1), UINT64_C(0x1f2afb4f64ba6697)
    },
    {   UINT64_C(0x0000000000005671), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e98b20711),
        UINT64_C(0x1f42fc8f2cb606f7), UINT64_C(0x1f43048eb8ae6701)
    },
    {   UINT64_C(0x0000000000010844), UINT64_C(0x1c0af48f64ad868b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1d92f64f64b2e697), UINT64_C(0x121afa0f7485e697)
    },
    {   UINT64_C(0x0000000000005673), UINT64_C(0x1f42f40e60b8e6fb), UINT64_C(0x1f42f78ee0b546ee),
        UINT64_C(0x1f42f70f9cb7a6f7), UINT64_C(0x1f42f80f4cb6e6e5)
    },
    {   UINT64_C(0x0000000000005674), UINT64_C(0x1f42fa5054b6a6de), UINT64_C(0x1f42f68f7cb1e6f3),
        UINT64_C(0x1f42f74fe0b2e6dd), UINT64_C(0x1f42f85028b546fa)
    },
    {   UINT64_C(0x0000000000005675), UINT64_C(0x1f43038ed0b14725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70fc8b4c6f5), UINT64_C(0x1f42f64f78b6c6e3)
    },
    {   UINT64_C(0x00000000000178cb), UINT64_C(0x0000000000000000), UINT64_C(0x14faf98c6492e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x14c2ddcc6496c6ef)
    },
    {   UINT64_C(0x00000000000178cc), UINT64_C(0x0000000000000000), UINT64_C(0x1262d50c649266ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x11daf98c6482e6ef)
    },
    {   UINT64_C(0x0000000000005678), UINT64_C(0x1f42fb0f5cafc6dd), UINT64_C(0x1f42f95008b0a703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54facb666ed)
    },
    {   UINT64_C(0x000000000001084b), UINT64_C(0x1db2f68f64af4697), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x139af58f7483c5a1)
    },
    {   UINT64_C(0x000000000001084c), UINT64_C(0x1773070f649a4697), UINT64_C(0x1cbaf8cfa0afe690),
        UINT64_C(0x0000000000000000), UINT64_C(0x2182f78f64c2868a)
    },
    {   UINT64_C(0x000000000001084d), UINT64_C(0x15caf58f749385a1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178d1), UINT64_C(0x0000000000000000), UINT64_C(0x11daf6cc648786ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fc2e08c648d26ef)
    },
    {   UINT64_C(0x000000000001084f), UINT64_C(0x13daf80f708d4594), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772f58fa0a00675)
    },
    {   UINT64_C(0x000000000000567e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38fe0b3a6fd),
        UINT64_C(0x1f42f44f48b7c706), UINT64_C(0x1f42f84fd8ae66f3)
    },
    {   UINT64_C(0x00000000000178d4), UINT64_C(0x0000000000000000), UINT64_C(0x129af98c648ea6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0ed2f30c647fa6ef)
    },
    {   UINT64_C(0x00000000000178d5), UINT64_C(0x0000000000000000), UINT64_C(0x1232f30c648786ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x16b2ee4c649f06ef)
    },
    {   UINT64_C(0x00000000000178d6), UINT64_C(0x0000000000000000), UINT64_C(0x1372e3cc6499c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1682f74c64a386ef)
    },
    {   UINT64_C(0x00000000000178d7), UINT64_C(0x0000000000000000), UINT64_C(0x1872f40c64a1e6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1302f30c6493a6ef)
    },
    {   UINT64_C(0x00000000000178d8), UINT64_C(0x0000000000000000), UINT64_C(0x16baee0c649ee6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1ceaef8c64b8c6ef)
    },
    {   UINT64_C(0x00000000000178d9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0edae54bf47e26ef)
    },
    {   UINT64_C(0x00000000000178da), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x106add0bf48fe6ef)
    },
    {   UINT64_C(0x00000000000178db), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f2af20bf478c6ef)
    },
    {   UINT64_C(0x0000000000005687), UINT64_C(0x1f42f74e4cb38735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f90ef0b64705), UINT64_C(0x1f42f90f80a8e720)
    },
    {   UINT64_C(0x0000000000005688), UINT64_C(0x1f42f54fb0b70727), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0edcb26727)
    },
    {   UINT64_C(0x0000000000005689), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f54ee0b48740),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000568a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f48b7871c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178e0), UINT64_C(0x14f2f08f689146ab), UINT64_C(0x0000000000000000),
        UINT64_C(0x1372f18f688946ab), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178e1), UINT64_C(0x0cdaf28f686f465c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0b1af1cf6868c5c1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178e2), UINT64_C(0x0f6af44fa07e0682), UINT64_C(0x0f02f20fa0798682),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000568e), UINT64_C(0x1f42fe8f70af6706), UINT64_C(0x1f42fc1014b18706),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000cab1), UINT64_C(0x0000000000000000), UINT64_C(0x1773020c289246c2),
        UINT64_C(0x1773020c289246c2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005690), UINT64_C(0x1f42f74ef0b426f3), UINT64_C(0x1f42fe8e4cb44724),
        UINT64_C(0x1f43008f28aaa720), UINT64_C(0x1f42f68f84b0870d)
    },
    {   UINT64_C(0x0000000000005691), UINT64_C(0x1f43014f8cafe6fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048e9cb5c731), UINT64_C(0x1f42fd8f78b2c706)
    },
    {   UINT64_C(0x0000000000005692), UINT64_C(0x1f42fe8f40b40720), UINT64_C(0x1f42f18ff0b3a6fb),
        UINT64_C(0x1f42f90e88b7071d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005693), UINT64_C(0x1f42f74fa8b526f1), UINT64_C(0x1f42f94eacb34725),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005694), UINT64_C(0x1f42fa8f58b2c6ff), UINT64_C(0x1f43014ef0b2c6fc),
        UINT64_C(0x1f42fe0f9cb2470c), UINT64_C(0x1f43094e80af070e)
    },
    {   UINT64_C(0x0000000000005695), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4f44b30710),
        UINT64_C(0x1f42f84f40b52703), UINT64_C(0x1f42fd4fc8b186f8)
    },
    {   UINT64_C(0x0000000000005696), UINT64_C(0x1f42f58f38b3c723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74f08b5a709), UINT64_C(0x1f43004f28ae86f7)
    },
    {   UINT64_C(0x0000000000005697), UINT64_C(0x1f4307cea0b286f0), UINT64_C(0x1f42fe8fd0b5e6fe),
        UINT64_C(0x1f43008e5caf6729), UINT64_C(0x1f42fd8fb8aac6f2)
    },
    {   UINT64_C(0x0000000000005698), UINT64_C(0x1f42ff8efcaee70a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74eacb36748), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005699), UINT64_C(0x1f42fe4f08ab06fe), UINT64_C(0x1f42fc4e8caf4715),
        UINT64_C(0x1f42fa0e8cb50711), UINT64_C(0x1f42fe4e78af872c)
    },
    {   UINT64_C(0x000000000000569a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4e8cb06716),
        UINT64_C(0x1f42f9ced0b1c72b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000569b), UINT64_C(0x1f42fd5064af0703), UINT64_C(0x1f42fc507cb2a703),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000178f1), UINT64_C(0x0000000000000000), UINT64_C(0x1012f9cc6479e5b6),
        UINT64_C(0x0b2afbcf7c60c5b6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000569d), UINT64_C(0x1f43050ed0b12706), UINT64_C(0x1f42f94f08b2670b),
        UINT64_C(0x1f42f48ee8ae8719), UINT64_C(0x1f42f74ec8b4e73c)
    },
    {   UINT64_C(0x000000000000569e), UINT64_C(0x1f42fc0ee8b1c703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4301ce80b1472b), UINT64_C(0x1f42fd8fa0aec725)
    },
    {   UINT64_C(0x000000000000569f), UINT64_C(0x1f42fa8f98b2c711), UINT64_C(0x1f42fb8f28b866fb),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f48e60b5e710)
    },
    {   UINT64_C(0x00000000000056a0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4e6cb64701)
    },
    {   UINT64_C(0x00000000000056a1), UINT64_C(0x1f42fe4f84b44716), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056a2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94ed8ac0719), UINT64_C(0x1f42fb8ef4b8a72d)
    },
    {   UINT64_C(0x00000000000056a3), UINT64_C(0x1f42f99048aea6fd), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94f9cac8703), UINT64_C(0x1f42fb8ef4b06713)
    },
    {   UINT64_C(0x00000000000056a4), UINT64_C(0x1f430b8ee0b066f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74eacb7272d), UINT64_C(0x1f42ff8f38b1c708)
    },
    {   UINT64_C(0x00000000000056a5), UINT64_C(0x1f42fc8f64b146f1), UINT64_C(0x1f42f4ce7cb7c729),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4fa0b24709)
    },
    {   UINT64_C(0x00000000000056a6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f70ae870b), UINT64_C(0x1f42f68f3cb5072e)
    },
    {   UINT64_C(0x00000000000056a7), UINT64_C(0x1f43024f20aaa71e), UINT64_C(0x1f430d0edcad06f8),
        UINT64_C(0x1f42fb4f20b56707), UINT64_C(0x1f42f88ed8af0711)
    },
    {   UINT64_C(0x0000000000018a99), UINT64_C(0x209aff8b30bc86ff), UINT64_C(0x209aff8b30bc86ff),
        UINT64_C(0x156afa0dfc8e256a), UINT64_C(0x156afa10108e26a7)
    },
    {   UINT64_C(0x00000000000056a9), UINT64_C(0x1f42f61068b7870b), UINT64_C(0x1f42fd0f9caf0707),
        UINT64_C(0x1f42f94eecae06f0), UINT64_C(0x1f42f78f74af86d9)
    },
    {   UINT64_C(0x00000000000056aa), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f00ac86f2),
        UINT64_C(0x1f42f55058b506e5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056ab), UINT64_C(0x1f42fa8f08b4c707), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8e90b76721), UINT64_C(0x1f42fe0ed0afc713)
    },
    {   UINT64_C(0x00000000000056ac), UINT64_C(0x1f42faced0ad2714), UINT64_C(0x1f42f60eecb2270e),
        UINT64_C(0x1f42f60f1cb50716), UINT64_C(0x1f42f78f84b2a727)
    },
    {   UINT64_C(0x00000000000056ad), UINT64_C(0x1f42fa4ef0b7c70a), UINT64_C(0x1f42f90ef0b5271c),
        UINT64_C(0x1f42f54e7cb0e715), UINT64_C(0x1f42f50ea0b1c6f6)
    },
    {   UINT64_C(0x0000000000010880), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0ebaf80f74796684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056af), UINT64_C(0x1f42f55068b4a6df), UINT64_C(0x1f42f60ef4b2c703),
        UINT64_C(0x1f42f60eb8b7073c), UINT64_C(0x1f42f70f98b78707)
    },
    {   UINT64_C(0x00000000000056b0), UINT64_C(0x1f42f50e8cb8c71c), UINT64_C(0x1f42f38ef8b7a738),
        UINT64_C(0x1f42f48efcb5c725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056b1), UINT64_C(0x1f42f34f3cad86ee), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f54efcb9470b), UINT64_C(0x1f42f34f1cb2a6f8)
    },
    {   UINT64_C(0x00000000000056b2), UINT64_C(0x1f42fccee8b14726), UINT64_C(0x1f42f44f84b6c6f7),
        UINT64_C(0x1f42f84ed8b0e716), UINT64_C(0x1f42f50f78b186f6)
    },
    {   UINT64_C(0x00000000000056b3), UINT64_C(0x1f42f90eacb6a73d), UINT64_C(0x1f42fa0fc0b086f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056b4), UINT64_C(0x1f42f950a8b606e1), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74f3cb54707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056b5), UINT64_C(0x1f42fa4fbcae670e), UINT64_C(0x1f42f60f00b186fd),
        UINT64_C(0x1f42f48eecb0c715), UINT64_C(0x1f42f30f9cb3a6f3)
    },
    {   UINT64_C(0x00000000000056b6), UINT64_C(0x1f42f84ee4b046f9), UINT64_C(0x1f42f60f98b5e6db),
        UINT64_C(0x1f42f4cefcb8270a), UINT64_C(0x1f430f8f0caf470e)
    },
    {   UINT64_C(0x00000000000180f3), UINT64_C(0x0fcaf6cf7879a67e), UINT64_C(0x1062f20f787c467e),
        UINT64_C(0x138af54f7888e67e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018aa1), UINT64_C(0x156afd0d7c8ca6f1), UINT64_C(0x156afa0b548e26d2),
        UINT64_C(0x156afa0b548e26d2), UINT64_C(0x156afd0c408ca65c)
    },
    {   UINT64_C(0x00000000000056b9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60ee0b1671d),
        UINT64_C(0x1f42f50e3cb62741), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056ba), UINT64_C(0x1f42f70f24b4e728), UINT64_C(0x1f42f4cf38b6a716),
        UINT64_C(0x1f42f54ec0b2e702), UINT64_C(0x1f42f90f3cb84735)
    },
    {   UINT64_C(0x00000000000056bb), UINT64_C(0x1f42f74fd0b6671a), UINT64_C(0x1f42fd0edcb20719),
        UINT64_C(0x1f42f94e40b0e71b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056bc), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f20b06715),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d8eccade738)
    },
    {   UINT64_C(0x00000000000056bd), UINT64_C(0x1f42f54e2cb1473a), UINT64_C(0x1f42f40f1cb3670a),
        UINT64_C(0x1f42f54f88b28700), UINT64_C(0x1f42f44e3cae8700)
    },
    {   UINT64_C(0x00000000000056be), UINT64_C(0x1f42f60eccb3673f), UINT64_C(0x1f42f50edcb44736),
        UINT64_C(0x1f42f70f7cab46f1), UINT64_C(0x1f42f54f24b306fb)
    },
    {   UINT64_C(0x00000000000180f7), UINT64_C(0x1162f0cfa082c67e), UINT64_C(0x0eb3020ca874e5aa),
        UINT64_C(0x15faf14f68934705), UINT64_C(0x0ffafe0fa077667e)
    },
    {   UINT64_C(0x0000000000018aa5), UINT64_C(0x156afd09fc8ca74a), UINT64_C(0x156afc0db88c2668),
        UINT64_C(0x156afc0db88c2668), UINT64_C(0x156afc0db88c2668)
    },
    {   UINT64_C(0x00000000000180f8), UINT64_C(0x190af0cc389e668c), UINT64_C(0x1082ec8c407b6625),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056c2), UINT64_C(0x1f42f34e90b86728), UINT64_C(0x1f42f54efcb36741),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f3ce7cb4c6fa)
    },
    {   UINT64_C(0x00000000000056c3), UINT64_C(0x1f42f64f60b5c731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f44ed4b86708)
    },
    {   UINT64_C(0x0000000000018aa7), UINT64_C(0x156afd0c308ca6be), UINT64_C(0x156afb4d108ec680),
        UINT64_C(0x156af8cd108d8680), UINT64_C(0x156afa0b748e26ee)
    },
    {   UINT64_C(0x00000000000056c5), UINT64_C(0x1f42f38fc8b586ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43008f18b2e71a), UINT64_C(0x1f42f90f6cb04723)
    },
    {   UINT64_C(0x00000000000056c6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70f20b26719),
        UINT64_C(0x1f42f40f08b34725), UINT64_C(0x1f42f60ee4b5c702)
    },
    {   UINT64_C(0x00000000000056c7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e9cb66737),
        UINT64_C(0x1f42f94f20ae071c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018aa9), UINT64_C(0x1872fd0f3898c5fc), UINT64_C(0x1abafd0f38a1e5fc),
        UINT64_C(0x156af6cf388ca5fc), UINT64_C(0x156afd0f388fc5fc)
    },
    {   UINT64_C(0x00000000000056c9), UINT64_C(0x1f42f28ef0b88731), UINT64_C(0x1f42f90e8cb0e6fd),
        UINT64_C(0x1f42f30e80b54737), UINT64_C(0x1f42f4cf70b2c70a)
    },
    {   UINT64_C(0x0000000000018aaa), UINT64_C(0x156afa8d7c8e8659), UINT64_C(0x156af94d7c8de65a),
        UINT64_C(0x156af94f248aa601), UINT64_C(0x156b010f248e8601)
    },
    {   UINT64_C(0x00000000000180fd), UINT64_C(0x1692f68f6894868c), UINT64_C(0x0dbaf44f787165b4),
        UINT64_C(0x0de2f60f787245b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056cc), UINT64_C(0x1f42fbceb0b5271a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8fa0b2a6e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056cd), UINT64_C(0x1f43070eb0b5873a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f38e64b8a747), UINT64_C(0x1f42f74f20b44721)
    },
    {   UINT64_C(0x0000000000018aac), UINT64_C(0x156afe0e048c2649), UINT64_C(0x156afd0c408ca65c),
        UINT64_C(0x156afd0c408ca65c), UINT64_C(0x156afd0c548ca6b5)
    },
    {   UINT64_C(0x00000000000056cf), UINT64_C(0x1f42f5cef8afa6e5), UINT64_C(0x1f42fa4f68ade6ec),
        UINT64_C(0x1f42f84f78af66e7), UINT64_C(0x1f42fa0fd4b52717)
    },
    {   UINT64_C(0x00000000000056d0), UINT64_C(0x1f42f68f80b26703), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70e64b7a72f)
    },
    {   UINT64_C(0x0000000000018100), UINT64_C(0x0000000000000000), UINT64_C(0x13aaf38fa08a6750),
        UINT64_C(0x17a2f28cac96067e), UINT64_C(0x11daf2cfa0828788)
    },
    {   UINT64_C(0x00000000000056d2), UINT64_C(0x1f42fe8e54afa735), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0ea0b426fe)
    },
    {   UINT64_C(0x0000000000018101), UINT64_C(0x145af78f688c268c), UINT64_C(0x1162fd8f687e468c),
        UINT64_C(0x0a6af28fa065e67e), UINT64_C(0x0a6af44fa066e750)
    },
    {   UINT64_C(0x00000000000056d4), UINT64_C(0x1f4304cef4b466ec), UINT64_C(0x1f430f4ecc9fc6f2),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431d8f14b526fa)
    },
    {   UINT64_C(0x0000000000018102), UINT64_C(0x0baaec8f686ac67e), UINT64_C(0x1ee2f08f68b5c67e),
        UINT64_C(0x1fcaf2cfa0b9667e), UINT64_C(0x191af28fa0a0c67e)
    },
    {   UINT64_C(0x00000000000056d6), UINT64_C(0x1f42ff8eb0ae273e), UINT64_C(0x1f42fa4fb8b4e6b9),
        UINT64_C(0x1f43068e9cb0e73b), UINT64_C(0x1f430b0e78ab8729)
    },
    {   UINT64_C(0x00000000000056d7), UINT64_C(0x1f42f98f6cb54707), UINT64_C(0x1f42fbce70b16741),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf94b20702)
    },
    {   UINT64_C(0x0000000000018ab1), UINT64_C(0x156afd0e048ca6ad), UINT64_C(0x156afe0c548c46b5),
        UINT64_C(0x156afe4c548c66b5), UINT64_C(0x156afd0d108ca681)
    },
    {   UINT64_C(0x00000000000056d9), UINT64_C(0x1f42f75020b6a701), UINT64_C(0x1f43014e44b0c743),
        UINT64_C(0x1f43100f00adc70e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056da), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0f94b12709),
        UINT64_C(0x1f430b8e84af672a), UINT64_C(0x1f43024ea0af0719)
    },
    {   UINT64_C(0x00000000000056db), UINT64_C(0x1f43054eecb086e0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43020f20afc707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018ab3), UINT64_C(0x156afe8aa48c86b5), UINT64_C(0x2a42fe0eccdf8616),
        UINT64_C(0x2a42fe0eccdf8616), UINT64_C(0x156afa8f388b65fc)
    },
    {   UINT64_C(0x00000000000056dd), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010eb4b4c6fc),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034e80b20726)
    },
    {   UINT64_C(0x00000000000056de), UINT64_C(0x1f43060e44b16732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80ee0b1c6fd), UINT64_C(0x1f42f84f28b4672a)
    },
    {   UINT64_C(0x0000000000018107), UINT64_C(0x1beaf2cfa0ab667e), UINT64_C(0x16b2f2cfa096867e),
        UINT64_C(0x16caf70f6894c68c), UINT64_C(0x16a2f2cfa096a67e)
    },
    {   UINT64_C(0x00000000000056e0), UINT64_C(0x1f4313d340aba64a), UINT64_C(0x1f4312ce78b14732),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43118f90aea71e)
    },
    {   UINT64_C(0x0000000000018108), UINT64_C(0x11daf2cfa081667e), UINT64_C(0x152af70f688ea68c),
        UINT64_C(0x1352f58fa088867e), UINT64_C(0x163af1cf6495467e)
    },
    {   UINT64_C(0x00000000000056e2), UINT64_C(0x1f43120e88af2735), UINT64_C(0x1f42fe0ef4b42725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431a0eecb4e726)
    },
    {   UINT64_C(0x00000000000056e3), UINT64_C(0x1f430acf7cb2c732), UINT64_C(0x1f43070f20af26f7),
        UINT64_C(0x1f4313cf80ac66f7), UINT64_C(0x1f43004f48afc72e)
    },
    {   UINT64_C(0x00000000000056e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43110f68ae6707), UINT64_C(0x1f43108f10b0c71e)
    },
    {   UINT64_C(0x000000000001810a), UINT64_C(0x1f12f54fa0b7267e), UINT64_C(0x1f12f54d3cb7267e),
        UINT64_C(0x170af14fa096667e), UINT64_C(0x1ba2f2cfa0aaa67e)
    },
    {   UINT64_C(0x00000000000056e6), UINT64_C(0x1f43108ed8af0731), UINT64_C(0x1f430fcef0b2472c),
        UINT64_C(0x1f4309cef4b3872c), UINT64_C(0x1f430f4f88b3c721)
    },
    {   UINT64_C(0x000000000001810b), UINT64_C(0x1322f2cfa086c67e), UINT64_C(0x145af58f688b268c),
        UINT64_C(0x1c32f2cf68aa668c), UINT64_C(0x13eaeecfa08bc67e)
    },
    {   UINT64_C(0x00000000000056e8), UINT64_C(0x1f43144ffcaec72c), UINT64_C(0x1f4315ceaca8671f),
        UINT64_C(0x1f43010ed0b5a740), UINT64_C(0x1f430f4f38ae471f)
    },
    {   UINT64_C(0x00000000000056e9), UINT64_C(0x1f4313cf20b0a71e), UINT64_C(0x1f43078e74afe736),
        UINT64_C(0x1f43134f24ad0735), UINT64_C(0x1f4318cf0cae6723)
    },
    {   UINT64_C(0x0000000000018aba), UINT64_C(0x156afa0fa08e2623), UINT64_C(0x156afa0cbc8e269b),
        UINT64_C(0x156afa0fa08e267e), UINT64_C(0x156afa0fa08e267e)
    },
    {   UINT64_C(0x00000000000056eb), UINT64_C(0x1f430ccf20aca703), UINT64_C(0x1f43068ed0ae673d),
        UINT64_C(0x1f43128f38b06726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056ec), UINT64_C(0x1f430e8e60b0873f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43140eccb1c73b)
    },
    {   UINT64_C(0x00000000000056ed), UINT64_C(0x1f4304ced4b2c730), UINT64_C(0x1f43168e44b1e740),
        UINT64_C(0x1f4313ce90b6a734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056ee), UINT64_C(0x1f43094eb8b66736), UINT64_C(0x1f43010eecb60730),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009100), UINT64_C(0x1692f04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1692fa4f5494669b), UINT64_C(0x1692f18fa094c68c)
    },
    {   UINT64_C(0x0000000000009101), UINT64_C(0x14daf20fa08ea68c), UINT64_C(0x131af20fa087668c),
        UINT64_C(0x1852f74f549ce69b), UINT64_C(0x1852f34fa09d668c)
    },
    {   UINT64_C(0x0000000000009102), UINT64_C(0x0c2af30fa06cc68c), UINT64_C(0x0fa2eece207a468c),
        UINT64_C(0x1852f58fa09ea68c), UINT64_C(0x14daf2cfa08e468c)
    },
    {   UINT64_C(0x0000000000018abe), UINT64_C(0x2493048d34c9867d), UINT64_C(0x156afd0c308ca6be),
        UINT64_C(0x156afd0c308ca6be), UINT64_C(0x1bf2f98c28a886ae)
    },
    {   UINT64_C(0x00000000000056f3), UINT64_C(0x1f43154e48ae2735), UINT64_C(0x1f43180f90a9670b),
        UINT64_C(0x1f43078f38a7e717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000056f4), UINT64_C(0x1f4306cebcb5e735), UINT64_C(0x1f43020ef8aa0727),
        UINT64_C(0x1f43070e80b6a739), UINT64_C(0x1f43164e60af273d)
    },
    {   UINT64_C(0x0000000000018112), UINT64_C(0x1802f3cfa09ba5aa), UINT64_C(0x1802f3cd7c9ba5aa),
        UINT64_C(0x10daf00fa07de5aa), UINT64_C(0x14f2f3cfa08f45aa)
    },
    {   UINT64_C(0x00000000000056f6), UINT64_C(0x1f431a0e50ace741), UINT64_C(0x1f42ff8f40b4a720),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f8e74b5e73c)
    },
    {   UINT64_C(0x00000000000056f7), UINT64_C(0x0000000000000000), UINT64_C(0x1f43068edcab0735),
        UINT64_C(0x1f43124e9cae473a), UINT64_C(0x1f42fece5cb78740)
    },
    {   UINT64_C(0x00000000000056f8), UINT64_C(0x1f430c4e88ad472e), UINT64_C(0x1f43024e74b6c731),
        UINT64_C(0x1f42fcce8cb2e73d), UINT64_C(0x1f43084eccade72a)
    },
    {   UINT64_C(0x0000000000007daf), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x0000000000007db0), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x0000000000007db1), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f43520fd88986ca),
        UINT64_C(0x1f43324fd89966ca), UINT64_C(0x1f43148fd8a846ca)
    },
    {   UINT64_C(0x0000000000007db2), UINT64_C(0x1f43160f7ca786ca), UINT64_C(0x1f4329cf7c9da5d4),
        UINT64_C(0x1f430e0f7cab86ca), UINT64_C(0x1f42facf7cb526ca)
    },
    {   UINT64_C(0x0000000000018116), UINT64_C(0x08b2dacc3855e693), UINT64_C(0x15f2f00f789245aa),
        UINT64_C(0x168af3cfa094a5aa), UINT64_C(0x1002ef8fa07ce693)
    },
    {   UINT64_C(0x0000000000007db4), UINT64_C(0x1f42f7d01c6566d8), UINT64_C(0x1f43534f0c88e6f4),
        UINT64_C(0x1f43115460aba630), UINT64_C(0x1f4316cf94a466e0)
    },
    {   UINT64_C(0x00000000000056ff), UINT64_C(0x1f430ccebcb026fc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd0f6cb40704), UINT64_C(0x1f430a8e98ac8734)
    },
    {   UINT64_C(0x0000000000009111), UINT64_C(0x186afa8d4099e683), UINT64_C(0x1722f78fa0992675),
        UINT64_C(0x1452fa8f78938684), UINT64_C(0x1132f44fa0924675)
    },
    {   UINT64_C(0x0000000000007db7), UINT64_C(0x1f43155024a706d8), UINT64_C(0x1f43244faca2a6df),
        UINT64_C(0x1f43070f54a1a704), UINT64_C(0x1f42facfecb2e6a0)
    },
    {   UINT64_C(0x0000000000005702), UINT64_C(0x1f43068e40b0e738), UINT64_C(0x1f42f6ce68b58711),
        UINT64_C(0x1f43038e50b3c741), UINT64_C(0x1f43048e70afe73e)
    },
    {   UINT64_C(0x0000000000007db9), UINT64_C(0x1f42fb0f94a0c71a), UINT64_C(0x1f42fcd390b4072e),
        UINT64_C(0x1f43084e8cae8725), UINT64_C(0x1f43110eb4b06719)
    },
    {   UINT64_C(0x0000000000007dba), UINT64_C(0x0fa309572c6f6630), UINT64_C(0x1f434119b891e6ee),
        UINT64_C(0x1f4339515c95e69a), UINT64_C(0x1f431c115ca486d6)
    },
    {   UINT64_C(0x0000000000007dbb), UINT64_C(0x1f431bd048a3a679), UINT64_C(0x1f43029048a0c6d6),
        UINT64_C(0x1f433391f88ec609), UINT64_C(0x1f430b8fecb7a70e)
    },
    {   UINT64_C(0x0000000000007dbc), UINT64_C(0x1f431390b4a42698), UINT64_C(0x1f43185090a606e4),
        UINT64_C(0x1f431e90c8a32695), UINT64_C(0x1f430c5438ab6590)
    },
    {   UINT64_C(0x0000000000009118), UINT64_C(0x0dc2fa8f746f45a1), UINT64_C(0x09caf80f7c61e61f),
        UINT64_C(0x0ed2f58f7c7605a1), UINT64_C(0x0fd3000f7c7a65b2)
    },
    {   UINT64_C(0x0000000000009119), UINT64_C(0x183b000f7c9c05b4), UINT64_C(0x1132f58f7c7e45a1),
        UINT64_C(0x0f82f58cc47c85a0), UINT64_C(0x0f9af88f6878e5aa)
    },
    {   UINT64_C(0x0000000000005709), UINT64_C(0x1f430e8fecb6e70a), UINT64_C(0x1f42facf24b1c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43140f8cb2c71c)
    },
    {   UINT64_C(0x000000000000911b), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x13daf90fa09c4685),
        UINT64_C(0x1802fbcf709e8695), UINT64_C(0x1bdaf88fa0b4c685)
    },
    {   UINT64_C(0x000000000000911c), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x1172fe8f708f4685),
        UINT64_C(0x18daf88fa0a76685), UINT64_C(0x1742f88fa0960685)
    },
    {   UINT64_C(0x0000000000007dc2), UINT64_C(0x1f43150e90aca72a), UINT64_C(0x1f43250fec9d2690),
        UINT64_C(0x1f431acfecaae70a), UINT64_C(0x1f43234ec89786c2)
    },
    {   UINT64_C(0x0000000000007dc3), UINT64_C(0x1f43190ec8a7c71e), UINT64_C(0x1f431c0fe8a106ce),
        UINT64_C(0x1f431c0fe8a106ec), UINT64_C(0x1f430a4e4cada6f9)
    },
    {   UINT64_C(0x0000000000007dc4), UINT64_C(0x1f430a4e4caf8743), UINT64_C(0x1f43044fe8b066a8),
        UINT64_C(0x1f43010fe8b60700), UINT64_C(0x1f4319cf60a3c6db)
    },
    {   UINT64_C(0x0000000000009120), UINT64_C(0x112af7cfa0992685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x122af90f788305b6), UINT64_C(0x0feafa8f807b6695)
    },
    {   UINT64_C(0x0000000000009121), UINT64_C(0x0eeaf90f787805b6), UINT64_C(0x129af90f68882695),
        UINT64_C(0x0f92f90f7877e5b6), UINT64_C(0x11e2f90c688d8695)
    },
    {   UINT64_C(0x0000000000009122), UINT64_C(0x107af38c748065b6), UINT64_C(0x1202fa8f78826695),
        UINT64_C(0x0b2afbcf7c640654), UINT64_C(0x0d7ae98c64724654)
    },
    {   UINT64_C(0x0000000000009123), UINT64_C(0x1222fbcf7484e696), UINT64_C(0x0ad2fa8f7c654695),
        UINT64_C(0x17c2f64f7c9965bb), UINT64_C(0x117af64f788045bb)
    },
    {   UINT64_C(0x0000000000005713), UINT64_C(0x1f42fd4e6c9e6704), UINT64_C(0x1f42fe4f54b6e719),
        UINT64_C(0x1f42f98eccb6a71f), UINT64_C(0x1f42fa8e68b0072e)
    },
    {   UINT64_C(0x0000000000009125), UINT64_C(0x0ed2fbcf7472e5b6), UINT64_C(0x0afaf90f78664638),
        UINT64_C(0x10baf64f787d45b7), UINT64_C(0x1152fb0f6881e5c1)
    },
    {   UINT64_C(0x0000000000005715), UINT64_C(0x1f43031008b666f8), UINT64_C(0x1f42fc4ebcb8e73c),
        UINT64_C(0x1f43064eb8a9070a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005716), UINT64_C(0x1f42f68f7cb1a6d5), UINT64_C(0x1f42fd4ed4ad2702),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005717), UINT64_C(0x1f42fb8f24b3e71e), UINT64_C(0x1f42f98f1cabe71c),
        UINT64_C(0x1f43018f08aa8706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005718), UINT64_C(0x1f42f98ee4b36722), UINT64_C(0x1f42fd8eb4aae704),
        UINT64_C(0x1f42fb8e80b0e730), UINT64_C(0x1f42f6ceb8b4c721)
    },
    {   UINT64_C(0x0000000000005719), UINT64_C(0x1f42fc4e74a8a72b), UINT64_C(0x1f42f78ef8b5672d),
        UINT64_C(0x1f42f5ce94b06724), UINT64_C(0x1f42fc8eccb1c720)
    },
    {   UINT64_C(0x000000000000571a), UINT64_C(0x1f42fb8ef4b1c740), UINT64_C(0x1f43008ee0b5a716),
        UINT64_C(0x1f42f98e80b4c723), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dd1), UINT64_C(0x1f43188f009fe654), UINT64_C(0x1f43068f00a926f4),
        UINT64_C(0x1f430e4fcca766db), UINT64_C(0x1f43134fc4a526e4)
    },
    {   UINT64_C(0x0000000000007dd2), UINT64_C(0x1f431e0ec8a48703), UINT64_C(0x1f430c4fd0aa66e6),
        UINT64_C(0x1f432e903c9aa621), UINT64_C(0x1f431dd058a066a8)
    },
    {   UINT64_C(0x0000000000007dd3), UINT64_C(0x1f43264fe88ce657), UINT64_C(0x1f43124fe88d66e2),
        UINT64_C(0x1f432b4fb496262c), UINT64_C(0x1f431f0fd09ea6bd)
    },
    {   UINT64_C(0x000000000000912f), UINT64_C(0x0000000000000000), UINT64_C(0x178af78f7494c697),
        UINT64_C(0x0f8302cf747b0675), UINT64_C(0x16d2f04c34984675)
    },
    {   UINT64_C(0x0000000000009130), UINT64_C(0x16b2f80f7491c697), UINT64_C(0x121afa0f7485e697),
        UINT64_C(0x2072fa4f74b8c5a1), UINT64_C(0x17cafa4f749625a1)
    },
    {   UINT64_C(0x0000000000007dd6), UINT64_C(0x1f431ecfb899e69d), UINT64_C(0x1f43110fb8aca6eb),
        UINT64_C(0x1f4316cf34a186ac), UINT64_C(0x1f4316cf34a68708)
    },
    {   UINT64_C(0x0000000000005721), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd5030b546c9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009133), UINT64_C(0x1bc2f58f74a905a1), UINT64_C(0x1782fe4f7492e5a1),
        UINT64_C(0x1492f58c348ba590), UINT64_C(0x159afb0f748ba5c6)
    },
    {   UINT64_C(0x0000000000005723), UINT64_C(0x1f4309cee4ade733), UINT64_C(0x1f42fa8ee8b186d7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f98b7a6ef)
    },
    {   UINT64_C(0x0000000000007dda), UINT64_C(0x1f431d8fd89b868a), UINT64_C(0x1f43134fd8ad86d6),
        UINT64_C(0x1f43110fb0a886e0), UINT64_C(0x1f43110fb0af06e9)
    },
    {   UINT64_C(0x0000000000007ddb), UINT64_C(0x1f43268f6096c5c5), UINT64_C(0x1f431a4f7c9f0647),
        UINT64_C(0x1f43138f7cacc6fc), UINT64_C(0x1f4317cf94a62696)
    },
    {   UINT64_C(0x0000000000007ddc), UINT64_C(0x1f43110f94aee6e9), UINT64_C(0x1f4344d0749026c3),
        UINT64_C(0x1f4344d074a106ea), UINT64_C(0x1f43160fcca486df)
    },
    {   UINT64_C(0x0000000000009138), UINT64_C(0x194b000ed09c26a3), UINT64_C(0x20c2fbcf4cb826a3),
        UINT64_C(0x1983024f4ca026a3), UINT64_C(0x2032fe4f4cbea6a3)
    },
    {   UINT64_C(0x0000000000009139), UINT64_C(0x1e22fd0f4caf86a3), UINT64_C(0x2082fd0f4cbe86a3),
        UINT64_C(0x2942fd0f4ce1c6a3), UINT64_C(0x204afe0f4cbcc6a3)
    },
    {   UINT64_C(0x0000000000007ddf), UINT64_C(0x1f43201090a6c6ff), UINT64_C(0x1f43191090a6e6d0),
        UINT64_C(0x1f4324d07c9c8654), UINT64_C(0x1f4317507cae26ed)
    },
    {   UINT64_C(0x000000000000572a), UINT64_C(0x1f42fed098b5c6dd), UINT64_C(0x1f42fdcf48a8a72b),
        UINT64_C(0x1f43218ee8a6a71c), UINT64_C(0x1f42fa8f90b106f2)
    },
    {   UINT64_C(0x000000000000572b), UINT64_C(0x1f42fcce9caf66ed), UINT64_C(0x1f42fa0f58b726eb),
        UINT64_C(0x1f42fe8fa4b486f2), UINT64_C(0x1f42fbd034ae06bf)
    },
    {   UINT64_C(0x0000000000007de2), UINT64_C(0x1f433a103c93e6b9), UINT64_C(0x1f43225154a1c69e),
        UINT64_C(0x1f431dd0b89da685), UINT64_C(0x1f431090d8a5c67b)
    },
    {   UINT64_C(0x000000000000572d), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf5caea6f4),
        UINT64_C(0x1f42ffcfb8a786f9), UINT64_C(0x1f42fa0f60b5c6d6)
    },
    {   UINT64_C(0x0000000000007de4), UINT64_C(0x1f4336d1009325f6), UINT64_C(0x1f432c1100996663),
        UINT64_C(0x1f432010889c06d7), UINT64_C(0x1f43300fdc91e6d7)
    },
    {   UINT64_C(0x0000000000007de5), UINT64_C(0x1f430b5074af867f), UINT64_C(0x1f4324d178a1e688),
        UINT64_C(0x1f432391aca0464a), UINT64_C(0x1f43304ff098a5f2)
    },
    {   UINT64_C(0x0000000000009141), UINT64_C(0x188af9cf489b469f), UINT64_C(0x19b2fa0f409f46a8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1b22fa8f40a4a69f)
    },
    {   UINT64_C(0x0000000000007de7), UINT64_C(0x1f4324d064a16688), UINT64_C(0x1f43180fd0aa86db),
        UINT64_C(0x1f432cd07c95663d), UINT64_C(0x1f4329cf1c9d2615)
    },
    {   UINT64_C(0x0000000000009143), UINT64_C(0x0000000000000000), UINT64_C(0x1822fa8c7098a69f),
        UINT64_C(0x1a72fa8f48a2a69f), UINT64_C(0x14eaf9cf408cc69f)
    },
    {   UINT64_C(0x0000000000007de9), UINT64_C(0x1f43278f689a468e), UINT64_C(0x1f432c90c49c468e),
        UINT64_C(0x1f432f50c49b06f6), UINT64_C(0x1f431a50a0a68698)
    },
    {   UINT64_C(0x0000000000007dea), UINT64_C(0x1f432f90a0932698), UINT64_C(0x1f431f10c89ae68e),
        UINT64_C(0x1f431f10c89ae6ff), UINT64_C(0x1f432090a0970697)
    },
    {   UINT64_C(0x0000000000007deb), UINT64_C(0x1f431ed09c9c268e), UINT64_C(0x1f4317d0c8a7668e),
        UINT64_C(0x1f432910c89e268e), UINT64_C(0x1f432b909c9cc698)
    },
    {   UINT64_C(0x0000000000007dec), UINT64_C(0x1f432210c89fc68e), UINT64_C(0x1f432c50c89c868e),
        UINT64_C(0x1f431cd0a0970698), UINT64_C(0x1f432910c89e268e)
    },
    {   UINT64_C(0x0000000000009148), UINT64_C(0x0c0afc4f586985bc), UINT64_C(0x0fd2f94f5877669f),
        UINT64_C(0x0d22f9cf586de5bc), UINT64_C(0x0a3af4cc5c66069e)
    },
    {   UINT64_C(0x0000000000009149), UINT64_C(0x0f9afacc5c7825bc), UINT64_C(0x1082fa0c5c7a069f),
        UINT64_C(0x0c02f88f5868c69e), UINT64_C(0x0acaf90c5c65669f)
    },
    {   UINT64_C(0x000000000000914a), UINT64_C(0x0d22fa0f586c869f), UINT64_C(0x0c72f98f586a469f),
        UINT64_C(0x1c72fccf3ca985bc), UINT64_C(0x13e2fd0f3c8745bc)
    },
    {   UINT64_C(0x000000000000573a), UINT64_C(0x1f42f7cec0b086ff), UINT64_C(0x1f4309ceb4ada702),
        UINT64_C(0x1f42fe8fb8b986c7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000914c), UINT64_C(0x1032fa4f40792657), UINT64_C(0x0bb2f78f5868a678),
        UINT64_C(0x143afd0f5888a5c6), UINT64_C(0x101af9cf58788632)
    },
    {   UINT64_C(0x000000000000914d), UINT64_C(0x17baf84f3c970632), UINT64_C(0x0e42f94cac7145bc),
        UINT64_C(0x1022fa0c5c788632), UINT64_C(0x0e4af8cc5c7165bc)
    },
    {   UINT64_C(0x000000000000914e), UINT64_C(0x172afa8fa094e67e), UINT64_C(0x1522ff8fa08cc67e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692ff8fa093267e)
    },
    {   UINT64_C(0x000000000000914f), UINT64_C(0x1372ff8fa085667e), UINT64_C(0x0eaaff8fa085667e),
        UINT64_C(0x1612f94f6c91868c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009150), UINT64_C(0x0a13018fa05d067e), UINT64_C(0x1182f80f687c667e),
        UINT64_C(0x16faff8fa099a68a), UINT64_C(0x127aff8fa085e67e)
    },
    {   UINT64_C(0x0000000000009151), UINT64_C(0x1abaff8fa0a0a67e), UINT64_C(0x0000000000000000),
        UINT64_C(0x17c2f94f6897e68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007df7), UINT64_C(0x1f432ed0c493c5cc), UINT64_C(0x1f432510c49905cc),
        UINT64_C(0x1f433e90c490e5cc), UINT64_C(0x1f433d909c8f65cc)
    },
    {   UINT64_C(0x0000000000009153), UINT64_C(0x15d2fe4f688da67e), UINT64_C(0x11c2f10fa090267e),
        UINT64_C(0x1cc2f10fa0b5267e), UINT64_C(0x1802f6cfa09a267e)
    },
    {   UINT64_C(0x0000000000005743), UINT64_C(0x1f4306cf54ae06f2), UINT64_C(0x1f43004eecb3470b),
        UINT64_C(0x1f43000e64b6c733), UINT64_C(0x1f43040f20b02714)
    },
    {   UINT64_C(0x0000000000009155), UINT64_C(0x1042f94f7879e5b4), UINT64_C(0x11f2f94f787f6693),
        UINT64_C(0x105af94f787a45b4), UINT64_C(0x0b82f44fa0746693)
    },
    {   UINT64_C(0x0000000000009156), UINT64_C(0x11f2f94c387f65b4), UINT64_C(0x1102fbcfa07ba693),
        UINT64_C(0x0812fbcfa057e693), UINT64_C(0x098aea4c3c5dc693)
    },
    {   UINT64_C(0x0000000000010918), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010919), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000001091a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000915a), UINT64_C(0x1522f30fa09565aa), UINT64_C(0x1182f58fa080c5aa),
        UINT64_C(0x0ea2f30c387e45aa), UINT64_C(0x103af8cfa07a05aa)
    },
    {   UINT64_C(0x000000000000915b), UINT64_C(0x1ac2fa8f3ca326a5), UINT64_C(0x1a8afa8f54a1c6a5),
        UINT64_C(0x182afb8f5499e6a5), UINT64_C(0x1bdafb8f54a706a5)
    },
    {   UINT64_C(0x000000000000915c), UINT64_C(0x18a2fb8f5499e6a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1a0afb8f54a026a5), UINT64_C(0x1a5af88c6ca1e6a5)
    },
    {   UINT64_C(0x000000000000574c), UINT64_C(0x1f42fe8f9cae86fe), UINT64_C(0x1f43064e68a20712),
        UINT64_C(0x1f43064f54b4271c), UINT64_C(0x1f43010e94b6a715)
    },
    {   UINT64_C(0x000000000000915e), UINT64_C(0x236afa8f3cc5a6a5), UINT64_C(0x1af2fa8f3ca3c6a5),
        UINT64_C(0x1c6afb8f54a946a5), UINT64_C(0x1a92fa8c6ca1e6a5)
    },
    {   UINT64_C(0x000000000000b815), UINT64_C(0x1582f74f789365ec), UINT64_C(0x11daf30f6884e5c3),
        UINT64_C(0x13c2f08f748c85b3), UINT64_C(0x1382f54f7088e5b3)
    },
    {   UINT64_C(0x000000000000574f), UINT64_C(0x1f43020f20b666dd), UINT64_C(0x1f430f8ee0afc71a),
        UINT64_C(0x1f42fbcfe8b446f3), UINT64_C(0x1f4300ceacb1e706)
    },
    {   UINT64_C(0x0000000000005750), UINT64_C(0x1f42fece8cb02707), UINT64_C(0x1f42ff8ef0b4671a),
        UINT64_C(0x1f42fd8e5caee729), UINT64_C(0x1f42f80eacac0731)
    },
    {   UINT64_C(0x0000000000005751), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c0ee4a9470d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8f64b2671c)
    },
    {   UINT64_C(0x000000000000b819), UINT64_C(0x000305d82c73a6f1), UINT64_C(0x0002ee49d467e35b),
        UINT64_C(0x000307582c7206ea), UINT64_C(0x0002ee49d467e35b)
    },
    {   UINT64_C(0x0000000000005753), UINT64_C(0x1f42facf58b06723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040f74ae66f6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005754), UINT64_C(0x1f42ff4e64b6a743), UINT64_C(0x1f43074ed4ac66fc),
        UINT64_C(0x1f43120f04ae0709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005755), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018ef0b6271c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b81d), UINT64_C(0x0000000000000000), UINT64_C(0x156afa0dc48e265a),
        UINT64_C(0x156afa0dc48e265a), UINT64_C(0x156afa0bf08e26bd)
    },
    {   UINT64_C(0x0000000000005757), UINT64_C(0x1f43018f0cb1a731), UINT64_C(0x1f42fa0ef0b4a720),
        UINT64_C(0x1f430a0e6cb48735), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005758), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcedcb8a70c), UINT64_C(0x1f42fc0ee0b2671e)
    },
    {   UINT64_C(0x0000000000005759), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0f0cb066e6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd4f30ada6c9)
    },
    {   UINT64_C(0x000000000000b821), UINT64_C(0x156afc0c7c8c26ad), UINT64_C(0x156afc0c7c8c26ad),
        UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c271a)
    },
    {   UINT64_C(0x000000000001092d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000000575c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4ed0b9270e),
        UINT64_C(0x1f42fe4f18b54729), UINT64_C(0x1f43078e80aec731)
    },
    {   UINT64_C(0x000000000001092f), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010930), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010931), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010932), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010933), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010934), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000005763), UINT64_C(0x1f43170e84a6e72a), UINT64_C(0x1f42f50f1caf86e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009175), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c),
        UINT64_C(0x169afa4f5494869b), UINT64_C(0x162b020fa092c68c)
    },
    {   UINT64_C(0x0000000000009176), UINT64_C(0x13db020fa089268c), UINT64_C(0x1233020fa085c68c),
        UINT64_C(0x1752f74f5492669b), UINT64_C(0x165b020fa08de68c)
    },
    {   UINT64_C(0x0000000000005766), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0e1cb08730),
        UINT64_C(0x1f4302ce70b8a734), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b82e), UINT64_C(0x2a02fecf74e14797), UINT64_C(0x2a02fecf74e14797),
        UINT64_C(0x0000000000000000), UINT64_C(0x156afa0e0c8e2656)
    },
    {   UINT64_C(0x000000000000b82f), UINT64_C(0x156af9cefc8de60b), UINT64_C(0x1632f1cc3891067e),
        UINT64_C(0x2272f1cc38c4a6dd), UINT64_C(0x13e2fa8bf087c72c)
    },
    {   UINT64_C(0x0000000000005769), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0eacab670e)
    },
    {   UINT64_C(0x000000000000917b), UINT64_C(0x15b2f30fa092c68c), UINT64_C(0x146af44fa08c668c),
        UINT64_C(0x0f4af3cf687405c1), UINT64_C(0x1002fbcf6c7b668c)
    },
    {   UINT64_C(0x000000000000576b), UINT64_C(0x1f42fe4f0cb10738), UINT64_C(0x1f43170e9caac733),
        UINT64_C(0x1f42f78f5cb646e8), UINT64_C(0x1f430f4ef0af8723)
    },
    {   UINT64_C(0x000000000000917d), UINT64_C(0x0fc2f50c687bc5c0), UINT64_C(0x0ff2fbcfa077668c),
        UINT64_C(0x07aafa8fa056e698), UINT64_C(0x0a22dfcc74602698)
    },
    {   UINT64_C(0x000000000000917e), UINT64_C(0x103afbcfa07dc68c), UINT64_C(0x075afbcfa055068c),
        UINT64_C(0x1842fbcfa098a5c1), UINT64_C(0x0ff2fbcfa07765c1)
    },
    {   UINT64_C(0x000000000000917f), UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1002fbcc6c7b65c1),
        UINT64_C(0x1002f44c6c77a5c1), UINT64_C(0x0cdafbcfa07145c1)
    },
    {   UINT64_C(0x000000000000576f), UINT64_C(0x1f42fc0f38b7a71f), UINT64_C(0x1f42f9ced0b54733),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000009181), UINT64_C(0x15eaf24f689405b5), UINT64_C(0x0fd2f30fa07b45b5),
        UINT64_C(0x0ec2f28c6c7765b5), UINT64_C(0x0dcaf34fa072a5b5)
    },
    {   UINT64_C(0x0000000000010943), UINT64_C(0x16233d4ae498853c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005772), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce54b6e739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068ea0b1273e)
    },
    {   UINT64_C(0x0000000000010945), UINT64_C(0x16233d53cc988776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010946), UINT64_C(0x1622eb53cc6f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010947), UINT64_C(0x16233d4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005776), UINT64_C(0x1f42fb8f3cb84728), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43155024a706d8)
    },
    {   UINT64_C(0x0000000000005777), UINT64_C(0x0000000000000000), UINT64_C(0x1f43094f48a2a707),
        UINT64_C(0x1f430e0ef8a6e70f), UINT64_C(0x1f4313cf2cb7a707)
    },
    {   UINT64_C(0x0000000000009189), UINT64_C(0x0e42fecf787d65b7), UINT64_C(0x0f73000f74898694),
        UINT64_C(0x0de2ff4f787b65b7), UINT64_C(0x0a83074fa0804693)
    },
    {   UINT64_C(0x000000000001094b), UINT64_C(0x1622eb4ae498653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000577a), UINT64_C(0x1f43210fd4a8e6da), UINT64_C(0x1f4300d040b6c6c6),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4307907cb326c0)
    },
    {   UINT64_C(0x000000000001094d), UINT64_C(0x1622eb53cc986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000918d), UINT64_C(0x0c2afd4f787485b7), UINT64_C(0x091b060f786d862c),
        UINT64_C(0x0f5b044f748185ad), UINT64_C(0x0ed311cf6883e5ad)
    },
    {   UINT64_C(0x000000000001094f), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000918f), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa094e68c),
        UINT64_C(0x169b0bcf54a2269b), UINT64_C(0x162b020fa09f068c)
    },
    {   UINT64_C(0x0000000000009190), UINT64_C(0x13db020fa099868c), UINT64_C(0x1233020fa099268c),
        UINT64_C(0x175308cf54a0a69b), UINT64_C(0x165b020fa0a2e68c)
    },
    {   UINT64_C(0x0000000000009191), UINT64_C(0x0dc301cfa081468c), UINT64_C(0x0f0af1ce2086468c),
        UINT64_C(0x1643020fa0a4868c), UINT64_C(0x12db020fa08b068c)
    },
    {   UINT64_C(0x0000000000009192), UINT64_C(0x1c8b020fa0bba68c), UINT64_C(0x16a3020f58a4068c),
        UINT64_C(0x1853088f58a7869c), UINT64_C(0x122b020fa095668c)
    },
    {   UINT64_C(0x0000000000010954), UINT64_C(0x1622eb5258986599), UINT64_C(0x1623264ae47ae776),
        UINT64_C(0x16233d4ae4988599), UINT64_C(0x1623264ae498853c)
    },
    {   UINT64_C(0x0000000000010955), UINT64_C(0x1623264ae4986599), UINT64_C(0x1622eb4ae46f4599),
        UINT64_C(0x1622eb4ae47ae53c), UINT64_C(0x1622eb4ae47ae599)
    },
    {   UINT64_C(0x0000000000010956), UINT64_C(0x16233d5258988776), UINT64_C(0x16232653cc988776),
        UINT64_C(0x1623265258986776), UINT64_C(0x1622eb52586f4776)
    },
    {   UINT64_C(0x0000000000010957), UINT64_C(0x1622eb53cc7ae776), UINT64_C(0x1622eb52587ae776),
        UINT64_C(0x16233d4ae4988776), UINT64_C(0x1623264ae4988776)
    },
    {   UINT64_C(0x0000000000010958), UINT64_C(0x1623264ae4988776), UINT64_C(0x1622eb4ae46f4776),
        UINT64_C(0x1622eb4ae47ae776), UINT64_C(0x1622eb4ae47ae776)
    },
    {   UINT64_C(0x0000000000010959), UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb4ae498653c),
        UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb5258986776)
    },
    {   UINT64_C(0x000000000001095a), UINT64_C(0x1622eb53cc986776), UINT64_C(0x1622eb5258986776),
        UINT64_C(0x1622eb4ae4986776), UINT64_C(0x1622eb4ae4986776)
    },
    {   UINT64_C(0x000000000001095b), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000578a), UINT64_C(0x1f43048f58b78707), UINT64_C(0x1f43098ef4ab06f7),
        UINT64_C(0x1f42fa4efcb5670b), UINT64_C(0x1f43140ee8ada71e)
    },
    {   UINT64_C(0x000000000000c80d), UINT64_C(0x138b00d6788346b1), UINT64_C(0x09b300d6785be6b1),
        UINT64_C(0x0e9b00d6786f86b1), UINT64_C(0x138b00d6788346b1)
    },
    {   UINT64_C(0x000000000000578c), UINT64_C(0x1f4304ce8cb04711), UINT64_C(0x1f43068ec8af0727),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43010f08b7e71a)
    },
    {   UINT64_C(0x000000000000919e), UINT64_C(0x106b030fa075467e), UINT64_C(0x106b034f747a067e),
        UINT64_C(0x106af58fa07c467e), UINT64_C(0x106af74fa07b667e)
    },
    {   UINT64_C(0x000000000000578e), UINT64_C(0x1f430f8e7cb02741), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030ea0b0671e)
    },
    {   UINT64_C(0x000000000000578f), UINT64_C(0x1f43048f38aec708), UINT64_C(0x1f43094f2cace707),
        UINT64_C(0x1f43048ef0ab86e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005790), UINT64_C(0x1f430a8f2cac671d), UINT64_C(0x1f4309cf04ab6703),
        UINT64_C(0x1f43080ec8b60714), UINT64_C(0x1f43038e4caee745)
    },
    {   UINT64_C(0x0000000000004964), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f00b14706)
    },
    {   UINT64_C(0x00000000000091a3), UINT64_C(0x106b018f887885b3), UINT64_C(0x106af84f887da649),
        UINT64_C(0x106afd0f887845b3), UINT64_C(0x106af88fa075264f)
    },
    {   UINT64_C(0x0000000000005793), UINT64_C(0x1f43020f68b766f9), UINT64_C(0x1f42ff8edcb086f7),
        UINT64_C(0x1f42ff8edcadc6f7), UINT64_C(0x1f42ff8eb4b7a6fe)
    },
    {   UINT64_C(0x0000000000005794), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0e40b82733)
    },
    {   UINT64_C(0x00000000000091a6), UINT64_C(0x106afd8f887805b3), UINT64_C(0x106af24c2c7aa5b0),
        UINT64_C(0x106af94c2c80c5b0), UINT64_C(0x106af74fa07b25b0)
    },
    {   UINT64_C(0x0000000000005796), UINT64_C(0x1f43030e34b56729), UINT64_C(0x1f43020eecb9a735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303ce1cb4c73c)
    },
    {   UINT64_C(0x00000000000091a8), UINT64_C(0x106af1cf907e05aa), UINT64_C(0x106af3cfa07cc5aa),
        UINT64_C(0x106af40c147d25aa), UINT64_C(0x106af60fa079a5aa)
    },
    {   UINT64_C(0x000000000000b85f), UINT64_C(0x0f9af50f78796693), UINT64_C(0x14d2f5cfa08de68a),
        UINT64_C(0x12e3008fa082e67e), UINT64_C(0x10bb008fa07fe67e)
    },
    {   UINT64_C(0x000000000000b860), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x123b008fa082067e),
        UINT64_C(0x13eaf50fa089e67e), UINT64_C(0x1493008fa087867e)
    },
    {   UINT64_C(0x000000000000579a), UINT64_C(0x1f4305cedcad672a), UINT64_C(0x1f43054ef8b00720),
        UINT64_C(0x1f4300ce94b8e71e), UINT64_C(0x1f4309ce40aaa72b)
    },
    {   UINT64_C(0x000000000000579b), UINT64_C(0x1f43050e80b16729), UINT64_C(0x1f4303ced0b88718),
        UINT64_C(0x1f42fe8f28b22707), UINT64_C(0x1f43078eecacc71d)
    },
    {   UINT64_C(0x000000000000b863), UINT64_C(0x1493008fa087867e), UINT64_C(0x106b008fa07e867e),
        UINT64_C(0x1652f58f6894068c), UINT64_C(0x1382f48fa088c67e)
    },
    {   UINT64_C(0x000000000000579d), UINT64_C(0x1f42fece4cb2272f), UINT64_C(0x1f43088eb8b0a6c5),
        UINT64_C(0x1f430a8e8cb0e730), UINT64_C(0x1f42fe4e84b34747)
    },
    {   UINT64_C(0x000000000000579e), UINT64_C(0x1f430b8e60b26740), UINT64_C(0x1f42f8ce6cb7a72a),
        UINT64_C(0x1f43244faca2a6df), UINT64_C(0x1f43048fb8aea6f3)
    },
    {   UINT64_C(0x000000000000b866), UINT64_C(0x0ed2f4cc407765b7), UINT64_C(0x0f9af50f78796693),
        UINT64_C(0x0db2f38f7872e5b7), UINT64_C(0x0d02f64c4c718693)
    },
    {   UINT64_C(0x00000000000057a0), UINT64_C(0x1f42f80f64af472c), UINT64_C(0x1f43054f00afe72c),
        UINT64_C(0x1f42fd4f70b5c72d), UINT64_C(0x1f42ff0f88b3072c)
    },
    {   UINT64_C(0x000000000000b868), UINT64_C(0x0f4af00f747a4693), UINT64_C(0x0f3af90c447ac5ad),
        UINT64_C(0x0e62f58f6873e5b8), UINT64_C(0x0d12f64c4471c693)
    },
    {   UINT64_C(0x00000000000057a2), UINT64_C(0x1f43034f78b4a72e), UINT64_C(0x1f43024f5cb18713),
        UINT64_C(0x1f42f7ce9cb8a736), UINT64_C(0x1f43034f38b32718)
    },
    {   UINT64_C(0x000000000000b86a), UINT64_C(0x1072f50f788245ad), UINT64_C(0x0d52ee8f7871c5ad),
        UINT64_C(0x0fbaf84f787865b7), UINT64_C(0x13c2f68c408985b8)
    },
    {   UINT64_C(0x000000000000b86b), UINT64_C(0x0c7ae90c447585b7), UINT64_C(0x1472f10c408ec5b5),
        UINT64_C(0x1382f54f7888e5ae), UINT64_C(0x0e42f3cf787485b7)
    },
    {   UINT64_C(0x00000000000057a5), UINT64_C(0x1f42fc8fe0aa8712), UINT64_C(0x1f42fb8f20b2e729),
        UINT64_C(0x1f42f38f34b04740), UINT64_C(0x1f42fe4edca5c72a)
    },
    {   UINT64_C(0x00000000000057a6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f40a8872e),
        UINT64_C(0x1f42fb8ec4b5c731), UINT64_C(0x1f43045024b5072a)
    },
    {   UINT64_C(0x00000000000057a7), UINT64_C(0x1f42f78ef0b30735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f78f4cb2672a), UINT64_C(0x1f42fa8f34ace71a)
    },
    {   UINT64_C(0x00000000000057a8), UINT64_C(0x1f42f98eb8b0e73e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030fd0a9e71e), UINT64_C(0x1f43034f6cb28717)
    },
    {   UINT64_C(0x00000000000057a9), UINT64_C(0x1f42fc8ed0a8472a), UINT64_C(0x1f42fd4efcb54735),
        UINT64_C(0x1f42fb8ec8b4e736), UINT64_C(0x1f42f98f14ae6729)
    },
    {   UINT64_C(0x00000000000057aa), UINT64_C(0x1f42ff4ed0aac721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f38b20715), UINT64_C(0x1f42f98e6cae6736)
    },
    {   UINT64_C(0x0000000000004971), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64e38b5e735), UINT64_C(0x1f42f64e40b6c738)
    },
    {   UINT64_C(0x00000000000057ac), UINT64_C(0x1f42fd4e7cb06731), UINT64_C(0x1f43014ed8ae671d),
        UINT64_C(0x1f42ff4e8cb1c725), UINT64_C(0x1f43004e6caee739)
    },
    {   UINT64_C(0x0000000000004972), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42face88b52724)
    },
    {   UINT64_C(0x00000000000057ae), UINT64_C(0x1f42f98e9cb0673e), UINT64_C(0x1f43044f34a9e732),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c81f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f5af1d3ec7a46a4)
    },
    {   UINT64_C(0x00000000000091c1), UINT64_C(0x085afa8f745ae5a1), UINT64_C(0x10b2fa8fa08325a1),
        UINT64_C(0x10eafa8f607e6684), UINT64_C(0x121af58cc4832594)
    },
    {   UINT64_C(0x00000000000057b1), UINT64_C(0x1f42fa8f50b4c72e), UINT64_C(0x1f43034f2cad4721),
        UINT64_C(0x1f42ff4eecb5e72d), UINT64_C(0x1f42fe4ea4a8c72d)
    },
    {   UINT64_C(0x00000000000057b2), UINT64_C(0x1f42fb8e94b48740), UINT64_C(0x1f42f78e5cb78735),
        UINT64_C(0x1f42f78e7caf0736), UINT64_C(0x1f42f88f24b3673d)
    },
    {   UINT64_C(0x00000000000057b3), UINT64_C(0x1f42ff4ec0ab8726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff4ee4aba73c), UINT64_C(0x1f42fb0f50b5072e)
    },
    {   UINT64_C(0x00000000000091c5), UINT64_C(0x154af30f749125a1), UINT64_C(0x0ebaf80f74796684),
        UINT64_C(0x0d72f80f747445a1), UINT64_C(0x1152f30f787de68e)
    },
    {   UINT64_C(0x00000000000091c6), UINT64_C(0x118af10f6c87e5a1), UINT64_C(0x142af80cc48c0684),
        UINT64_C(0x0feaf80cb47ba5a1), UINT64_C(0x1ac2f58f74aac594)
    },
    {   UINT64_C(0x00000000000057b6), UINT64_C(0x1f42fe4ee0b2072d), UINT64_C(0x1f42fe4e70ace735),
        UINT64_C(0x1f42fb8e78b0c735), UINT64_C(0x1f42f88f04adc738)
    },
    {   UINT64_C(0x00000000000057b7), UINT64_C(0x1f42fa8e94ac873c), UINT64_C(0x1f42fa4e5cb1a73d),
        UINT64_C(0x1f42fd4f80b0e726), UINT64_C(0x1f42f68ea4aa0731)
    },
    {   UINT64_C(0x00000000000091c9), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x18daf88fa0a76685),
        UINT64_C(0x17baf7cfa0990695), UINT64_C(0x1fc2f88fa0c46685)
    },
    {   UINT64_C(0x0000000000004978), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f42ee0dc8bb2757)
    },
    {   UINT64_C(0x00000000000057ba), UINT64_C(0x1f42f98e5cb1672d), UINT64_C(0x1f42fe4ee8ac2722),
        UINT64_C(0x1f42f88eb8aa672b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091cc), UINT64_C(0x140af5cfa0a08695), UINT64_C(0x19daf50fa0a26685),
        UINT64_C(0x1942f50fa0a64685), UINT64_C(0x15b2f58fa0a02695)
    },
    {   UINT64_C(0x00000000000057bc), UINT64_C(0x1f42ff8ee8b2073c), UINT64_C(0x1f43050ec8b46739),
        UINT64_C(0x1f42fb8e18afe733), UINT64_C(0x1f42f94eb0b1073c)
    },
    {   UINT64_C(0x00000000000057bd), UINT64_C(0x1f42facfecb2e6a0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cf6cb6a6fd), UINT64_C(0x1f42ff8f68b486ef)
    },
    {   UINT64_C(0x00000000000091cf), UINT64_C(0x102af64c6c7e65b6), UINT64_C(0x11b2f90f7483e696),
        UINT64_C(0x09f2fbcf7c60c5b6), UINT64_C(0x1182fbcfa086a5b6)
    },
    {   UINT64_C(0x00000000000091d0), UINT64_C(0x11d2fbcf6081a695), UINT64_C(0x13e2f64c6c89e5aa),
        UINT64_C(0x105afbcf788145b3), UINT64_C(0x0ee2f50c747b6695)
    },
    {   UINT64_C(0x00000000000057c0), UINT64_C(0x1f4302cee8b166c9), UINT64_C(0x1f43018e5cb36733),
        UINT64_C(0x1f43038eacb36720), UINT64_C(0x1f43030ed8b22723)
    },
    {   UINT64_C(0x000000000000c828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x131af0cfa087668c)
    },
    {   UINT64_C(0x00000000000057c2), UINT64_C(0x1f42fbce2cb5e740), UINT64_C(0x1f43028e4cb26728),
        UINT64_C(0x1f43028e4cb2872d), UINT64_C(0x1f430b0e9cac66f6)
    },
    {   UINT64_C(0x000000000000c829), UINT64_C(0x0fa2f34f607a2694), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x2acaf20f68e6268c)
    },
    {   UINT64_C(0x0000000000014008), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0c2afecf6c6cc694),
        UINT64_C(0x115b1015d8818695), UINT64_C(0x0fa2e60fa08106a4)
    },
    {   UINT64_C(0x0000000000014009), UINT64_C(0x0fa2e90e107ca6c5), UINT64_C(0x17ab020f2c96e6aa),
        UINT64_C(0x1a8b010f549ee692), UINT64_C(0x08b30e15d8600695)
    },
    {   UINT64_C(0x000000000001400a), UINT64_C(0x0a6af50cd4698695), UINT64_C(0x0a6ae30cd4606695),
        UINT64_C(0x0fa30753907c0695), UINT64_C(0x11d2f64fa081a5e2)
    },
    {   UINT64_C(0x00001a010009400b), UINT64_C(0x07d2df0cc850846e), UINT64_C(0x0a6aee9238606502),
        UINT64_C(0x07d2ebcf6c4fe46f), UINT64_C(0x08b2de0f58694695)
    },
    {   UINT64_C(0x000000000001400c), UINT64_C(0x0fa2f24f687a2693), UINT64_C(0x0fa2ef4fa072e693),
        UINT64_C(0x0fa2e74fa0764693), UINT64_C(0x0fa2ea8f6c76c693)
    },
    {   UINT64_C(0x000000000001400d), UINT64_C(0x0fa2ea4fa07d6693), UINT64_C(0x0fa2eb4f6c7b4685),
        UINT64_C(0x0fa2f3cf647e268f), UINT64_C(0x0fa2fb0fa0802685)
    },
    {   UINT64_C(0x000000000001400e), UINT64_C(0x0fa2eecf6c7a0693), UINT64_C(0x0fa2eb0f7879e693),
        UINT64_C(0x0a6af3cf6c5f85b3), UINT64_C(0x0a6ae7ccc45f85b3)
    },
    {   UINT64_C(0x000000000001400f), UINT64_C(0x11d2f5cf7081e5ee), UINT64_C(0x11d2f6514c81a577),
        UINT64_C(0x11d2f5cf7081e5ee), UINT64_C(0x0fa301cf6c794694)
    },
    {   UINT64_C(0x04907003c0014010), UINT64_C(0x1d5b0b8f58a5a695), UINT64_C(0x14dadd0fa088a693),
        UINT64_C(0x14dae7cfa08c8685), UINT64_C(0x14daf60f58932695)
    },
    {   UINT64_C(0x0830c40002a94011), UINT64_C(0x1692e28fa0942685), UINT64_C(0x14dae74fa0902685),
        UINT64_C(0x14daeacfa0910685), UINT64_C(0x1692f34f5896a695)
    },
    {   UINT64_C(0x00000000000057ce), UINT64_C(0x1f43074fb8a9e6cd), UINT64_C(0x1f4306ce6caee6fa),
        UINT64_C(0x1f430c0f50ab06c9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0da1800005394013), UINT64_C(0x131ae88fa0884685), UINT64_C(0x1bcae6cf70b12685),
        UINT64_C(0x1692e74f649c0685), UINT64_C(0x1692f4cf5894e695)
    },
    {   UINT64_C(0x00000000000091e1), UINT64_C(0x1012f6cf687ba5c1), UINT64_C(0x13d2f44c6c8aa5c1),
        UINT64_C(0x11eaf04c6c83a5c1), UINT64_C(0x1542f44c6c95e5c1)
    },
    {   UINT64_C(0x1952c855c9094015), UINT64_C(0x131afa4fa08fc685), UINT64_C(0x1692fecf589b8685),
        UINT64_C(0x14dafe8f5897e685), UINT64_C(0x1bcafe0f58b40685)
    },
    {   UINT64_C(0x00000000000057d2), UINT64_C(0x1f4305ce6cae871f), UINT64_C(0x1f4307cefcb16717),
        UINT64_C(0x1f4306cf50ae6702), UINT64_C(0x1f43084ef4b446f2)
    },
    {   UINT64_C(0x0000000000014017), UINT64_C(0x08b2edcf58616695), UINT64_C(0x0a6ae00d2c69668a),
        UINT64_C(0x11d2fed4607d6685), UINT64_C(0x0fa2ee0dac7d039d)
    },
    {   UINT64_C(0x000000000000b89b), UINT64_C(0x0000000000000000), UINT64_C(0x105af9cc88786590),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057d5), UINT64_C(0x1f43124eacac671f), UINT64_C(0x1f43154edcb246e2),
        UINT64_C(0x1f431b0efcb1c72f), UINT64_C(0x1f4304cf18b3472c)
    },
    {   UINT64_C(0x00000000000057d6), UINT64_C(0x1f43155010b10717), UINT64_C(0x1f43114e5cb1e735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e14b2e737)
    },
    {   UINT64_C(0x00000000000091e8), UINT64_C(0x122afd0fa092e68c), UINT64_C(0x1a7b048f54a5469b),
        UINT64_C(0x15c2f34fa09c068c), UINT64_C(0x140b03cfa097e68c)
    },
    {   UINT64_C(0x00000000000057d8), UINT64_C(0x0000000000000000), UINT64_C(0x1f432ccef897a704),
        UINT64_C(0x1f42fccee8a7c6fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057d9), UINT64_C(0x1f42f80eb4abc70f), UINT64_C(0x1f42f90e40a4a73c),
        UINT64_C(0x1f42f90e50a14736), UINT64_C(0x1f430e0f08ad8701)
    },
    {   UINT64_C(0x000000000000b8a1), UINT64_C(0x0000000000000000), UINT64_C(0x105af84c807ae58d),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001401f), UINT64_C(0x06e2fe8f5851c695), UINT64_C(0x0ae2cdccb4646696),
        UINT64_C(0x11d2fb925c7f0533), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000091ed), UINT64_C(0x11f2f28f6882069c), UINT64_C(0x1292eb0c6c85a5b6),
        UINT64_C(0x106af84f688525c1), UINT64_C(0x0f0af8cc6c80269c)
    },
    {   UINT64_C(0x00000000000091ee), UINT64_C(0x10faf94f687ee5c1), UINT64_C(0x13c2f94f689085b6),
        UINT64_C(0x1182e5cc6c8185c1), UINT64_C(0x102afb0c6c83c5c1)
    },
    {   UINT64_C(0x00000000000091ef), UINT64_C(0x1332f94f688e65b6), UINT64_C(0x105af88f6882e5b6),
        UINT64_C(0x1132fd4f688045c1), UINT64_C(0x155afbcc6c90e5c1)
    },
    {   UINT64_C(0x00000000000091f0), UINT64_C(0x11fada0c6c8cc5c1), UINT64_C(0x15fb00cc6c9c65c1),
        UINT64_C(0x17a2fe0f689a05b5), UINT64_C(0x1122f94f6881e69b)
    },
    {   UINT64_C(0x00000000000091f1), UINT64_C(0x0ea2f94f687945c1), UINT64_C(0x121afb8f6886e69c),
        UINT64_C(0x1432eb8f7891a5c1), UINT64_C(0x1532fa0c6c90469c)
    },
    {   UINT64_C(0x00000000000057e1), UINT64_C(0x1f42f68f38b04700), UINT64_C(0x1f42f88e80b48734),
        UINT64_C(0x1f42fe4fa4b2a6f9), UINT64_C(0x1f42f78f28ada72b)
    },
    {   UINT64_C(0x00000000000091f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa2fe4fa075a698)
    },
    {   UINT64_C(0x000000000000c839), UINT64_C(0x0001f48cfc7f6807), UINT64_C(0x0002788d3440067e),
        UINT64_C(0x00028c8d3030a67e), UINT64_C(0x00024d4cf05786e0)
    },
    {   UINT64_C(0x0000000000014028), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0c2af2cc6c6685d6),
        UINT64_C(0x0fa2f88d64756628), UINT64_C(0x0fa2e60f707e4693)
    },
    {   UINT64_C(0x000000000000c83a), UINT64_C(0x00026e0be843237b), UINT64_C(0x00026e98204546b5),
        UINT64_C(0x00024210a45105a1), UINT64_C(0x00026fcbf45886ef)
    },
    {   UINT64_C(0x00000000000057e6), UINT64_C(0x1f42fc4eccb56713), UINT64_C(0x1f42fd4ee8b6a705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018e90b10703)
    },
    {   UINT64_C(0x00000000000057e7), UINT64_C(0x1f42f34f38ade708), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f34b6a708), UINT64_C(0x1f42fb8f3ca926f3)
    },
    {   UINT64_C(0x00000000000057e8), UINT64_C(0x1f42fc4ebcba2735), UINT64_C(0x1f42f34ec8b3672c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057e9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f4cb40700),
        UINT64_C(0x1f42fd4f18ba86fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057ea), UINT64_C(0x1f42f48f44b1c730), UINT64_C(0x1f42f84f44b7a734),
        UINT64_C(0x1f42fc0f34aea70d), UINT64_C(0x1f42fc4e5cafe723)
    },
    {   UINT64_C(0x00000000000057eb), UINT64_C(0x1f42f58e90b2c72c), UINT64_C(0x1f42f88f70a9c713),
        UINT64_C(0x1f42fb4f44b8a703), UINT64_C(0x1f42fa4fc4a9a717)
    },
    {   UINT64_C(0x00000000000057ec), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94e9cb3c718), UINT64_C(0x1f42f7cf74b6a741)
    },
    {   UINT64_C(0x369000d319814031), UINT64_C(0x14dadd0fa0896771), UINT64_C(0x14dadd0fa088a7d4),
        UINT64_C(0x1d82de0fa0b14685), UINT64_C(0x14daf60c38932695)
    },
    {   UINT64_C(0x00000000000109c0), UINT64_C(0x0000000000000000), UINT64_C(0x1e72f6d0c0b3e655),
        UINT64_C(0x1e0af6d0e4b2664b), UINT64_C(0x1e72f6d0c0b3e655)
    },
    {   UINT64_C(0x0000000000014033), UINT64_C(0x0c2ae60fa071479b), UINT64_C(0x0c2ae60fa075879b),
        UINT64_C(0x0c2ae60fa0792794), UINT64_C(0x0c2ae60fa07a8771)
    },
    {   UINT64_C(0x00000000000109c2), UINT64_C(0x18baf6cf6c9da6a9), UINT64_C(0x1592f6d1c8906611),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109c3), UINT64_C(0x112af54f747f06a9), UINT64_C(0x1abaf6908ca50660),
        UINT64_C(0x1c7af6d068ac066a), UINT64_C(0x1d2af6d1ccaee60e)
    },
    {   UINT64_C(0x00000000000109c4), UINT64_C(0x1c7af6d068ac066a), UINT64_C(0x1d8af6d1ccb02611),
        UINT64_C(0x1e72f6d140b3c633), UINT64_C(0x179af68f6c98a6a9)
    },
    {   UINT64_C(0x00000000000109c5), UINT64_C(0x1a72f70f68a3e6a9), UINT64_C(0x17d2f6cfa099669c),
        UINT64_C(0x17caf6cfa099669c), UINT64_C(0x11daedce9085e67d)
    },
    {   UINT64_C(0x00000000000109c6), UINT64_C(0x152af70f6c8ec6a9), UINT64_C(0x17daf6cfa099869c),
        UINT64_C(0x17c2f6cfa099469c), UINT64_C(0x17baf6cfa099269c)
    },
    {   UINT64_C(0x00000000000109c7), UINT64_C(0x1792f6cfa098869c), UINT64_C(0x0f72f6cfa078069c),
        UINT64_C(0x1142f6cfa07f269c), UINT64_C(0x10caf6cfa07d669c)
    },
    {   UINT64_C(0x00000000000057f6), UINT64_C(0x1f430fcf54b36716), UINT64_C(0x1f43048f68b28704),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c843), UINT64_C(0x1a5af88c6ca1e6a5), UINT64_C(0x180300cfa095268c),
        UINT64_C(0x1202fa8f78826695), UINT64_C(0x1202fa8f78826695)
    },
    {   UINT64_C(0x000000000000b8bf), UINT64_C(0x1692f1cc3896e6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x14daf24c3c88a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c0), UINT64_C(0x125af0cc3c86a6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692fa0c3892e6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c1), UINT64_C(0x131af10c3c8926dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1322f08c3c8946dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8c2), UINT64_C(0x1692f28c3c9666dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f68c389486dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109ce), UINT64_C(0x1802f6cf849a46a3), UINT64_C(0x18c2f6cf689d26a9),
        UINT64_C(0x17baf6cfa099269c), UINT64_C(0x161af6cf6c9286a8)
    },
    {   UINT64_C(0x00000000000109cf), UINT64_C(0x1622f6cf689286a9), UINT64_C(0x1552f6cf688f66a9),
        UINT64_C(0x15eaf6cf6891c6a9), UINT64_C(0x19d2f6cf68a166a9)
    },
    {   UINT64_C(0x000000000000b8c5), UINT64_C(0x1692f10c3896e6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x141af54c3c8b46dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109d1), UINT64_C(0x157af70fa08fe69c), UINT64_C(0x15c2f6cfa091269b),
        UINT64_C(0x158af6cfa090669c), UINT64_C(0x1892f68f689c66a9)
    },
    {   UINT64_C(0x00000000000109d2), UINT64_C(0x189af6cf689c86a9), UINT64_C(0x164af6cf689346a9),
        UINT64_C(0x18baf6cf689d06a9), UINT64_C(0x188af70f689c66a9)
    },
    {   UINT64_C(0x00000000000109d3), UINT64_C(0x0000000000000000), UINT64_C(0x1b4af6cf68a746a9),
        UINT64_C(0x0000000000000000), UINT64_C(0x17d2f6cfa099669c)
    },
    {   UINT64_C(0x00000000000109d4), UINT64_C(0x17d2f6cfa099669c), UINT64_C(0x17baf6cfa099269c),
        UINT64_C(0x17baf6cfa099269c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1882f6cf689c26a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005804), UINT64_C(0x1f42fccef0ace6f7), UINT64_C(0x1f42fbce88b00701),
        UINT64_C(0x1f42fc0f70b3a735), UINT64_C(0x1f42fbcf08b986fc)
    },
    {   UINT64_C(0x00000000000109d7), UINT64_C(0x0c42f693bc6b26a9), UINT64_C(0x14e2f6d3bc8da6a9),
        UINT64_C(0x14e2f713bc8dc6a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109d8), UINT64_C(0x0000000000000000), UINT64_C(0x16e2f6cd6495a70e),
        UINT64_C(0x1102fc0f687b86aa), UINT64_C(0x1102f68f687e06a9)
    },
    {   UINT64_C(0x00000000000109d9), UINT64_C(0x1c72f6d040abe673), UINT64_C(0x14daf6cf688da6a9),
        UINT64_C(0x17c2f6cf689946a9), UINT64_C(0x17c2ee11849d4621)
    },
    {   UINT64_C(0x000000000001404c), UINT64_C(0x0c2ae60fa0714746), UINT64_C(0x08b2ee8f7c5c45b6),
        UINT64_C(0x19bae60e14b0a685), UINT64_C(0x1142ee8c64828695)
    },
    {   UINT64_C(0x0000000000009400), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x10eaf60c3877a693),
        UINT64_C(0x101af38bd07c2438), UINT64_C(0x0a030dccac56a69a)
    },
    {   UINT64_C(0x0000000000009401), UINT64_C(0x1042fbcfa07f0684), UINT64_C(0x0e62f28f7875a5aa),
        UINT64_C(0x0aa2f9101063661d), UINT64_C(0x12eaf8d01084a61d)
    },
    {   UINT64_C(0x000000000000580b), UINT64_C(0x1f42fd8f38b2e6ef), UINT64_C(0x1f42fb8ef8b0e700),
        UINT64_C(0x1f42ff4ef8afe6ef), UINT64_C(0x1f42fdcf48b4e6f9)
    },
    {   UINT64_C(0x0000000000009403), UINT64_C(0x239af8d010c7861d), UINT64_C(0x121af9cf7882e693),
        UINT64_C(0x1672fa8fa092067e), UINT64_C(0x0cfafbd0847265a9)
    },
    {   UINT64_C(0x0000000000009404), UINT64_C(0x12b3028f647f4640), UINT64_C(0x0cfaee10846b85a9),
        UINT64_C(0x12b3020f647f0640), UINT64_C(0x18d2fdcf6499a640)
    },
    {   UINT64_C(0x0000000000009405), UINT64_C(0x18d2fe4f6499e640), UINT64_C(0x0e7aff90886fa5a9),
        UINT64_C(0x0e7aff90886fa5a9), UINT64_C(0x105af50e787d46c1)
    },
    {   UINT64_C(0x0000000000009406), UINT64_C(0x0c92fe0f6c68c5ec), UINT64_C(0x156afd0d308ca67e),
        UINT64_C(0x0c92fe0f6c68c5ec), UINT64_C(0x156afd0d308ca67e)
    },
    {   UINT64_C(0x0000000000009407), UINT64_C(0x15b2f58fa091867e), UINT64_C(0x15b2f58fa091867e),
        UINT64_C(0x15b2f78fa090867e), UINT64_C(0x105af98c287a46ae)
    },
    {   UINT64_C(0x0000000000009408), UINT64_C(0x170afecf749e6797), UINT64_C(0x170b00107c9e679a),
        UINT64_C(0x170b00107c9e6797), UINT64_C(0x156af84fa08f4594)
    },
    {   UINT64_C(0x0000000000009409), UINT64_C(0x0853070f545345ab), UINT64_C(0x0ca2f3cf546e25ab),
        UINT64_C(0x0df3048f746b05ab), UINT64_C(0x130af3cf5487c5ab)
    },
    {   UINT64_C(0x000000000000b8da), UINT64_C(0x116af00c388f26dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109e6), UINT64_C(0x1762f6d11895c63d), UINT64_C(0x1ce2f6d1d0adc610),
        UINT64_C(0x1762f6d11895c63d), UINT64_C(0x1cb2f6d238ad05f6)
    },
    {   UINT64_C(0x000000000000b8dc), UINT64_C(0x1622efcc389206dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109e8), UINT64_C(0x1b4af6d114a7463f), UINT64_C(0x19e2f6d140a16634),
        UINT64_C(0x1ce2f6d128ada63a), UINT64_C(0x1ce2f6d128ada63a)
    },
    {   UINT64_C(0x00000000000109e9), UINT64_C(0x1caaf6cfa0acc69c), UINT64_C(0x1d12f6d0f0ae8648),
        UINT64_C(0x1d1af6d0f0ae8648), UINT64_C(0x0e7af6ce147426ff)
    },
    {   UINT64_C(0x00000000000109ea), UINT64_C(0x1b9af6d0bca88657), UINT64_C(0x1a22f6d050a2c670),
        UINT64_C(0x1a22f6d050a2c670), UINT64_C(0x1b1af6d134a68637)
    },
    {   UINT64_C(0x00000000000109eb), UINT64_C(0x1b1af6d134a68637), UINT64_C(0x15c2f6d050912670),
        UINT64_C(0x15c2f6d050912670), UINT64_C(0x1b52f6d0d4a78653)
    },
    {   UINT64_C(0x00000000000109ec), UINT64_C(0x0000000000000000), UINT64_C(0x1b52f6d0c4a7864f),
        UINT64_C(0x17c2f6cfa099269c), UINT64_C(0x1d92f6d220b065fd)
    },
    {   UINT64_C(0x00000000000109ed), UINT64_C(0x181af690349a8677), UINT64_C(0x1b0af6d2dca645c9),
        UINT64_C(0x181af690349a8677), UINT64_C(0x1bc2f68f54a926b0)
    },
    {   UINT64_C(0x00000000000109ee), UINT64_C(0x1e3af6d190b30620), UINT64_C(0x1bc2f68f54a926b0),
        UINT64_C(0x19faf6d164a2062b), UINT64_C(0x1b4af71178a7462a)
    },
    {   UINT64_C(0x0000000000009414), UINT64_C(0x156afd0b888ca6e8), UINT64_C(0x156afd0c508ca70b),
        UINT64_C(0x156afd0c508ca70b), UINT64_C(0x156afb8cac8ae69f)
    },
    {   UINT64_C(0x0000000000009415), UINT64_C(0x156b008cac8d669f), UINT64_C(0x156afa11808e256a),
        UINT64_C(0x156afa11808e256a), UINT64_C(0x0fa2fa0dc47723fe)
    },
    {   UINT64_C(0x0000000000009416), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x0e7afacd9c71e661),
        UINT64_C(0x0e7afacd9c71e661), UINT64_C(0x156afa4d088ce66f)
    },
    {   UINT64_C(0x0000000000009417), UINT64_C(0x156afb8d0893064f), UINT64_C(0x156afd0d7c8ca659),
        UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afe0a9c8d26b6)
    },
    {   UINT64_C(0x0000000000009418), UINT64_C(0x156b008a9c8e66b6), UINT64_C(0x0ca2f3cf546e25ab),
        UINT64_C(0x0b72fccaa0718720), UINT64_C(0x12a2fccaa08e4720)
    },
    {   UINT64_C(0x0000000000005822), UINT64_C(0x1f42fb8e8cb486fc), UINT64_C(0x1f43148ee0b28726),
        UINT64_C(0x1f430d4f3cad86ff), UINT64_C(0x1f42f8cec0b64725)
    },
    {   UINT64_C(0x000000000000941a), UINT64_C(0x0ef2fb8aa082e720), UINT64_C(0x0dcaf6caa080c720),
        UINT64_C(0x0edaf6caa0850720), UINT64_C(0x1612f8ca70912714)
    },
    {   UINT64_C(0x000000000000941b), UINT64_C(0x222afcca70cc6714), UINT64_C(0x0e9afcca707e2714),
        UINT64_C(0x0e9afcca707e2714), UINT64_C(0x108afcca7085e714)
    },
    {   UINT64_C(0x000000000000941c), UINT64_C(0x0ad2f6ca7074e714), UINT64_C(0x0ef2fb8a7082e714),
        UINT64_C(0x0b6af6ca7074c714), UINT64_C(0x0c02fb8a70772714)
    },
    {   UINT64_C(0x000000000000941d), UINT64_C(0x0feafb8a7086c714), UINT64_C(0x0c62f6ca707b2714),
        UINT64_C(0x0ce2f5ca707aa714), UINT64_C(0x181af94a70a66714)
    },
    {   UINT64_C(0x000000000000941e), UINT64_C(0x0e3af94a707ee714), UINT64_C(0x0fc2f94a7085a714),
        UINT64_C(0x0fc2f94a7085a714), UINT64_C(0x0e2af94a707c678f)
    },
    {   UINT64_C(0x000000000000941f), UINT64_C(0x0bc2fcc8847e2714), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000b8f0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afa0d848e265b)
    },
    {   UINT64_C(0x000000000001406e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f82e10bec77c5aa), UINT64_C(0x0fa2e74fa0792693)
    },
    {   UINT64_C(0x000000000000b8f2), UINT64_C(0x1c32ee0d30aa667e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000582c), UINT64_C(0x1f42ff0f24ab26fa), UINT64_C(0x1f4306cf08aea70b),
        UINT64_C(0x1f43118f50b2072d), UINT64_C(0x1f430bce80aca72d)
    },
    {   UINT64_C(0x000000000000582d), UINT64_C(0x1f42fd8e9cb1e736), UINT64_C(0x1f43034f24b66708),
        UINT64_C(0x1f4312cf04a6e721), UINT64_C(0x1f42ffcf68b5c716)
    },
    {   UINT64_C(0x000000000000582e), UINT64_C(0x1f43028ed8b4c738), UINT64_C(0x1f43128ec4ae871a),
        UINT64_C(0x1f42ff8eecb1870a), UINT64_C(0x1f42fece3cb22731)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005830), UINT64_C(0x1f42fecf78b626e8), UINT64_C(0x1f430a8f20aea70b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43050f68b366ef)
    },
    {   UINT64_C(0x0000000000005831), UINT64_C(0x1f43028f0cb06715), UINT64_C(0x1f4301ce80b5673c),
        UINT64_C(0x1f42fe0f50a82717), UINT64_C(0x1f43030ea8b2e73d)
    },
    {   UINT64_C(0x0000000000005832), UINT64_C(0x1f43020edcb3c6e0), UINT64_C(0x1f430c0ee4abe719),
        UINT64_C(0x1f42ff8f28ab870a), UINT64_C(0x1f42fd8f38b3a6f6)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005834), UINT64_C(0x0000000000000000), UINT64_C(0x1f43100e84ae4722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4f14b4c6ec)
    },
    {   UINT64_C(0x0000000000005835), UINT64_C(0x1f43008f64af66f3), UINT64_C(0x1f43038eacaec70c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005836), UINT64_C(0x1f42fdcf74aaa714), UINT64_C(0x1f42fecf88b546e4),
        UINT64_C(0x1f42fecf58ade72b), UINT64_C(0x1f43018f40ae670a)
    },
    {   UINT64_C(0x0000000000005837), UINT64_C(0x1f43090f40b3872d), UINT64_C(0x1f43048fc8aae6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030f28b3671e)
    },
    {   UINT64_C(0x000000000001407c), UINT64_C(0x0af2d40c6479c69a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005839), UINT64_C(0x1f42fd0fb8ae66f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4f08abc726), UINT64_C(0x1f42fd8f18b26703)
    },
    {   UINT64_C(0x000000000001407e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x14dadd0fa088a7b4), UINT64_C(0x0fa2e8cf68778775)
    },
    {   UINT64_C(0x000000000001407f), UINT64_C(0x1d82de0fa0b14770), UINT64_C(0x1692eccf6c92a6a1),
        UINT64_C(0x1692f4cdac94e770), UINT64_C(0x0fa2ed4dc475a6a1)
    },
    {   UINT64_C(0x000000000000583c), UINT64_C(0x1f42fb8e68af0739), UINT64_C(0x1f43008ef8ada70e),
        UINT64_C(0x1f43040e6cb00713), UINT64_C(0x1f43098ee4b1e727)
    },
    {   UINT64_C(0x000000000000c866), UINT64_C(0x1e8aff0cf8b0668c), UINT64_C(0x1e8aff0cf8b0668c),
        UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1e8aff107cb025ab)
    },
    {   UINT64_C(0x000000000000583e), UINT64_C(0x1f43008eccb16720), UINT64_C(0x1f43108ec8ada730),
        UINT64_C(0x1f42fd4f48b26707), UINT64_C(0x1f43020ef4aae717)
    },
    {   UINT64_C(0x000000000000c867), UINT64_C(0x1de2fe90c0adc5b6), UINT64_C(0x1de2fe90c0adc5b6),
        UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x11c3038d0c7ac68c)
    },
    {   UINT64_C(0x0000000000005840), UINT64_C(0x1f42fb8f28b646fb), UINT64_C(0x1f43084ee8b32706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054f48b0e6f3)
    },
    {   UINT64_C(0x000000000000c868), UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x11c3038d0c7ac68c),
        UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1e8aff107cb025ab)
    },
    {   UINT64_C(0x0000000000005842), UINT64_C(0x1f43020e64b6472a), UINT64_C(0x1f42ff4ed0b9473b),
        UINT64_C(0x1f43034f18ae06ff), UINT64_C(0x1f43008f14b5e731)
    },
    {   UINT64_C(0x000000000000c869), UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1243058d0c7bc68c),
        UINT64_C(0x1e8aff107cb025ab), UINT64_C(0x1243058d0c7bc68c)
    },
    {   UINT64_C(0x0000000000005844), UINT64_C(0x1f43024f18b1e70c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8ef8ad671a), UINT64_C(0x1f43030ed8aee70e)
    },
    {   UINT64_C(0x000000000000c86a), UINT64_C(0x1243058c907bc6c4), UINT64_C(0x1e32fe907caf0603),
        UINT64_C(0x1e32fe907caf0603), UINT64_C(0x1e32fe8f88af0604)
    },
    {   UINT64_C(0x0000000000005846), UINT64_C(0x1f42ffcf58b5c70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430ccefcb1a713), UINT64_C(0x1f430dce68b02731)
    },
    {   UINT64_C(0x000000000000c86b), UINT64_C(0x1e32fe8f88af0604), UINT64_C(0x288afbd07cd9c5ab),
        UINT64_C(0x1e8af5907cb505ab), UINT64_C(0x1192ff0d387c4693)
    },
    {   UINT64_C(0x0000000000005848), UINT64_C(0x1f43000f48b346ef), UINT64_C(0x1f43040f58b0872b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ce94ae86e0)
    },
    {   UINT64_C(0x000000000001408d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x08b2becc645c65b6)
    },
    {   UINT64_C(0x000000000000584a), UINT64_C(0x1f430d0f10b30719), UINT64_C(0x1f4301cebcb2e700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4ed8ad8716)
    },
    {   UINT64_C(0x000000000000c86d), UINT64_C(0x1e02fe8d18ae4696), UINT64_C(0x1e02fc0cf4af8696),
        UINT64_C(0x1e5afe8fa0afa5f4), UINT64_C(0x1e5afe8fa0afa5f4)
    },
    {   UINT64_C(0x000000001ff8bec0), UINT64_C(0x12eaf30fa0906693), UINT64_C(0x1172f60fa07fe693),
        UINT64_C(0x1162f5cfa07fc693), UINT64_C(0x1a02f30fa0a22693)
    },
    {   UINT64_C(0x000000000000c86e), UINT64_C(0x1e8aff0cf8b066e8), UINT64_C(0x1e8aff0dd8b02652),
        UINT64_C(0x1e73084da4ab269d), UINT64_C(0x1e73084da4ab269d)
    },
    {   UINT64_C(0x0000000000005370), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x000000000000c86f), UINT64_C(0x1e8afe92e4b045dd), UINT64_C(0x1e8afe4fb4b02511),
        UINT64_C(0x114329cd187aa69e), UINT64_C(0x0b02ffcd1865a69e)
    },
    {   UINT64_C(0x0000000000014094), UINT64_C(0x1002f14f687c05c1), UINT64_C(0x1322f44f688885c1),
        UINT64_C(0x1322f44f688885c1), UINT64_C(0x0fa2ea8f6c76a69b)
    },
    {   UINT64_C(0x0000000000014095), UINT64_C(0x0de2eccf6c6f85b6), UINT64_C(0x0de2eccd2c7145b6),
        UINT64_C(0x0fa2e8cbfc7f26a3), UINT64_C(0x11d2e8cf6c902693)
    },
    {   UINT64_C(0x0000000000014096), UINT64_C(0x0de2ef4f6c7325b6), UINT64_C(0x0de2ef4f6c7045b6),
        UINT64_C(0x1392ef4f6c8cc5b6), UINT64_C(0x0de2ef4f687ac5c3)
    },
    {   UINT64_C(0x0000000000014097), UINT64_C(0x0f02ef4f687925c3), UINT64_C(0x158aef4f6894a5c3),
        UINT64_C(0x0f32efcf6879e5c3), UINT64_C(0x0b7ad50c646c05b6)
    },
    {   UINT64_C(0x0000000000014098), UINT64_C(0x1682e10bcc9c6693), UINT64_C(0x112ae10bcc7f85c0),
        UINT64_C(0x118af34f748285b6), UINT64_C(0x0de2f30bf87085b6)
    },
    {   UINT64_C(0x0000000000014099), UINT64_C(0x0f6af08f787b85aa), UINT64_C(0x1162eacbbc7cc5b3),
        UINT64_C(0x1162eacf7c7cc693), UINT64_C(0x10aaeacc6c79e693)
    },
    {   UINT64_C(0x000000000001409a), UINT64_C(0x08b2eb8f7c5f4694), UINT64_C(0x07eaee8f805c25b0),
        UINT64_C(0x0982e60fa066a5b6), UINT64_C(0x0a02f04f7c6c86a3)
    },
    {   UINT64_C(0x000000000001409b), UINT64_C(0x0deaf24f7c72a6a3), UINT64_C(0x091ae24bfc62c6a3),
        UINT64_C(0x137aee8bf087a6a3), UINT64_C(0x1852ea8f7c98e5b6)
    },
    {   UINT64_C(0x000000000001409c), UINT64_C(0x1852ea8bfc98e5b6), UINT64_C(0x17baea8bfc9685b6),
        UINT64_C(0x15b2d2cbfc8d85b6), UINT64_C(0x0fcaec8bfc7b45b6)
    },
    {   UINT64_C(0x0000000000005859), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f38b5672d),
        UINT64_C(0x1f430a8efcad2721), UINT64_C(0x1f430ecef4b08730)
    },
    {   UINT64_C(0x000000000000585a), UINT64_C(0x1f42fa0f48b8070f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecef8b1a704), UINT64_C(0x1f43058ee0b3672a)
    },
    {   UINT64_C(0x0000000000018008), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a6b0e8f7c5c068c),
        UINT64_C(0x0cc301565c67e68c), UINT64_C(0x0fa2ef8fa07c667e)
    },
    {   UINT64_C(0x0000000000018009), UINT64_C(0x0fa2f90e4477c6bf), UINT64_C(0x175afd4f549446aa),
        UINT64_C(0x1852f88f6c9c468c), UINT64_C(0x05a2fa165c4f268c)
    },
    {   UINT64_C(0x00000000000140a1), UINT64_C(0x0c52d78c64868693), UINT64_C(0x0cfb01cbfc6985be),
        UINT64_C(0x104ae40bd48ea693), UINT64_C(0x09b2efce3863e5e9)
    },
    {   UINT64_C(0x00001800c009800b), UINT64_C(0x07d2fbcd6c56e44e), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x07d2ff8f7455244c), UINT64_C(0x08b2ebcf6862668c)
    },
    {   UINT64_C(0x000000000001800c), UINT64_C(0x0fa2f40f687a068c), UINT64_C(0x0fa309cfa06fc68c),
        UINT64_C(0x0fa2f54fa079c68c), UINT64_C(0x0fa2f84f6874668c)
    },
    {   UINT64_C(0x000000000001800d), UINT64_C(0x0fa2f10fa079a68c), UINT64_C(0x0fa2f5cf68754698),
        UINT64_C(0x0fa2f68f68790694), UINT64_C(0x0fa2f30f8076a67e)
    },
    {   UINT64_C(0x000000000001800e), UINT64_C(0x0fa2fc0f6876268c), UINT64_C(0x0fa2f58f4877e68c),
        UINT64_C(0x08b3024f745685b3), UINT64_C(0x08b3020d6c59e5b3)
    },
    {   UINT64_C(0x000000000000537a), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x04806e0380018010), UINT64_C(0x1ccb0b0f68a3a68c), UINT64_C(0x1692f1cfa096e68a),
        UINT64_C(0x14daf24fa088a67e), UINT64_C(0x14daf50f688da68c)
    },
    {   UINT64_C(0x00000000000140a8), UINT64_C(0x0f42ee4c6c92c62e), UINT64_C(0x1092f44fa07f8695),
        UINT64_C(0x1092fe4fa07de695), UINT64_C(0x1772e8cf6c9a66a3)
    },
    {   UINT64_C(0x09211c2340018012), UINT64_C(0x1692f28fa096667e), UINT64_C(0x0a6af28fa065e67e),
        UINT64_C(0x0baaec8f686ac67e), UINT64_C(0x1692f64fa098e67e)
    },
    {   UINT64_C(0x000000000000537c), UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721),
        UINT64_C(0x1f42f4cdb8b82721), UINT64_C(0x1f42f4cdb8b82721)
    },
    {   UINT64_C(0x1171f40007218014), UINT64_C(0x116af20fa082467e), UINT64_C(0x1692f68cdc96268c),
        UINT64_C(0x14daf24fa091067e), UINT64_C(0x1162f8cf687be68c)
    },
    {   UINT64_C(0x1952c855c9098015), UINT64_C(0x131af24fa088a67e), UINT64_C(0x1692f18f6896a67e),
        UINT64_C(0x1692f20f7495a67e), UINT64_C(0x1d82ef4f74b3067e)
    },
    {   UINT64_C(0x0003e8740e318016), UINT64_C(0x1692f08fa096867e), UINT64_C(0x1692f38fa096667e),
        UINT64_C(0x1322f08fa089467e), UINT64_C(0x0a6b040d3063e67e)
    },
    {   UINT64_C(0x0000000000018017), UINT64_C(0x08b2ebcf6862668c), UINT64_C(0x0a6af68d305d267e),
        UINT64_C(0x0eaaf4144476467e), UINT64_C(0x0fa2ee0dac7d039d)
    },
    {   UINT64_C(0x1ff3f87dc0018018), UINT64_C(0x0a6af2978c5cc68e), UINT64_C(0x0de2f74f7875c5b4),
        UINT64_C(0x0fa2eecf78790693), UINT64_C(0x0de2f44f787205b4)
    },
    {   UINT64_C(0x22240e8040018019), UINT64_C(0x0fa2f4cf787be693), UINT64_C(0x0de2f44f787385b4),
        UINT64_C(0x0a6af30fa06e6693), UINT64_C(0x0fa2f50c387945b4)
    },
    {   UINT64_C(0x2340008c5179801a), UINT64_C(0x0fa2f04fa07b6693), UINT64_C(0x08b2f20fa05e2693),
        UINT64_C(0x08b2dc8c3856c693), UINT64_C(0x0fa2efcfa07da693)
    },
    {   UINT64_C(0x252498921231801b), UINT64_C(0x08b2f2cfa05ea693), UINT64_C(0x1852f20fa09f65b4),
        UINT64_C(0x0fa2f20fa07b25b4), UINT64_C(0x0fa2f54f787945b4)
    },
    {   UINT64_C(0x28b4ba96d2c9801c), UINT64_C(0x0fa2ef4c3c7945b4), UINT64_C(0x0fa2f40c3c7b85b4),
        UINT64_C(0x0a6aef4fa06865b4), UINT64_C(0x0c2afacf786a05b3)
    },
    {   UINT64_C(0x2b5536a69469801d), UINT64_C(0x08b2f14f7861662b), UINT64_C(0x0fa2f04f787a85aa),
        UINT64_C(0x0fa2f2cf687a25aa), UINT64_C(0x1692f34f689545aa)
    },
    {   UINT64_C(0x000000b55679801e), UINT64_C(0x0fa2f24fa07a65aa), UINT64_C(0x0fa2f18c3879e5aa),
        UINT64_C(0x0de2f4cfa072c5aa), UINT64_C(0x0f03070ccc6a4690)
    },
    {   UINT64_C(0x000000000001801f), UINT64_C(0x0642fecf684f268c), UINT64_C(0x0f030e8ccc6e0690),
        UINT64_C(0x10eaf8127c7d452d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140b7), UINT64_C(0x0a6afa1800716677), UINT64_C(0x0a6aff57b07cc6a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140b8), UINT64_C(0x0bdaf19298736694), UINT64_C(0x06f2ee94c459a741),
        UINT64_C(0x08c2f014cc60c69a), UINT64_C(0x0c2aeed4dc6ee69a)
    },
    {   UINT64_C(0x00000000000140b9), UINT64_C(0x0a42f054c46c0739), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005876), UINT64_C(0x1f430f4f5cad8740), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c883), UINT64_C(0x1773028d58920690), UINT64_C(0x0fa2fd102c7585db),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74b94b6e6e3)
    },
    {   UINT64_C(0x0000000000005878), UINT64_C(0x1f43018e8cb70721), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43150f20af8733)
    },
    {   UINT64_C(0x00000000000140bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6adb8c1462237c)
    },
    {   UINT64_C(0x0000000000005386), UINT64_C(0x1f43049040b346e2), UINT64_C(0x1f43144f20ad26d6),
        UINT64_C(0x1f430a8fdcb006cb), UINT64_C(0x1f42fb8fe0b446fe)
    },
    {   UINT64_C(0x0000000000018028), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a53060c3857c5bc),
        UINT64_C(0x0fa2fb4d7876862b), UINT64_C(0x0fa2f10f807bc68c)
    },
    {   UINT64_C(0x0000000000018029), UINT64_C(0x0fa2e88f787fc5fe), UINT64_C(0x0fa2e0cfa07e867e),
        UINT64_C(0x0642fecf684f268c), UINT64_C(0x0fa2ff8d5073c68c)
    },
    {   UINT64_C(0x000000000001802a), UINT64_C(0x0a6af297cc660656), UINT64_C(0x17c2f78f6898c68c),
        UINT64_C(0x08a2ef15c860468c), UINT64_C(0x0fa2f89024778588)
    },
    {   UINT64_C(0x000000000001802b), UINT64_C(0x12c2fa116c838571), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x17c2f78f6898c68c), UINT64_C(0x0a6af0d82c66c641)
    },
    {   UINT64_C(0x000000000001802c), UINT64_C(0x0c82fc55b869668c), UINT64_C(0x156afa0e448e25de),
        UINT64_C(0x0962ee53d863868c), UINT64_C(0x0962f153b862e68c)
    },
    {   UINT64_C(0x000000000001802d), UINT64_C(0x0a6b05578c66268e), UINT64_C(0x0fa2f70c387e85aa),
        UINT64_C(0x1282fd0d3881067e), UINT64_C(0x07d2ff92bc55251e)
    },
    {   UINT64_C(0x2ef000000001802e), UINT64_C(0x0a6afb0c445f23e8), UINT64_C(0x0962fc53d85d868c),
        UINT64_C(0x09b2ef95c864868c), UINT64_C(0x0fa2f8d02477c588)
    },
    {   UINT64_C(0x000000000001802f), UINT64_C(0x1772f88f6897a68c), UINT64_C(0x1772f70f689b268c),
        UINT64_C(0x1772f14f6898468c), UINT64_C(0x0de2f58c386d85bb)
    },
    {   UINT64_C(0x32b640c0d7c18030), UINT64_C(0x1692f1cfa096e788), UINT64_C(0x1692f1cfa096e788),
        UINT64_C(0x1692f1cfa096e784), UINT64_C(0x1692f1cfa096e760)
    },
    {   UINT64_C(0x36f000d499b18031), UINT64_C(0x1692f1cfa096e750), UINT64_C(0x1692f1cfa096e7a9),
        UINT64_C(0x1bcaee0fa0aa667e), UINT64_C(0x14daf50c448da68c)
    },
    {   UINT64_C(0x0000000000018032), UINT64_C(0x131af10fa0892788), UINT64_C(0x131af10fa0892788),
        UINT64_C(0x131af10fa0892784), UINT64_C(0x131af10fa0892750)
    },
    {   UINT64_C(0x0000000000018033), UINT64_C(0x0a6af28fa065e788), UINT64_C(0x0a6af28fa065e788),
        UINT64_C(0x0a6af08fa066a784), UINT64_C(0x0a6af24fa065e750)
    },
    {   UINT64_C(0x3736e00000018034), UINT64_C(0x1692f20fa094267e), UINT64_C(0x1692f10f7496e760),
        UINT64_C(0x1692f68f68948788), UINT64_C(0x1692f68f68948788)
    },
    {   UINT64_C(0x0006fcdedbc18035), UINT64_C(0x1692f68f68948784), UINT64_C(0x1692f68f68948760),
        UINT64_C(0x1692f68f68948750), UINT64_C(0x1402f8cf3c8925f9)
    },
    {   UINT64_C(0x393718e15c218036), UINT64_C(0x1692f68e609486c6), UINT64_C(0x1692f18f6896a788),
        UINT64_C(0x1692f18f6896a788), UINT64_C(0x1692f18f6896a784)
    },
    {   UINT64_C(0x000000001ca18037), UINT64_C(0x1692f18f6896a750), UINT64_C(0x1692f38fa0966788),
        UINT64_C(0x1162f20fa082467e), UINT64_C(0x0fa2f10f7c790693)
    },
    {   UINT64_C(0x000000e7dce18038), UINT64_C(0x0de2f74f7875c68e), UINT64_C(0x0de2f74f7875c68e),
        UINT64_C(0x0de2f74f7875c68a), UINT64_C(0x0de2f74f7875c666)
    },
    {   UINT64_C(0x000750e95d118039), UINT64_C(0x0de2f74f7875c656), UINT64_C(0x0de2f74f7875c6b1),
        UINT64_C(0x14daf78f848d85b4), UINT64_C(0x0de2f44c447205b4)
    },
    {   UINT64_C(0x00075ceac001803a), UINT64_C(0x0de2f44f7873868e), UINT64_C(0x0de2f44f7873868e),
        UINT64_C(0x0de2f44f7873868a), UINT64_C(0x0de2f44f78738656)
    },
    {   UINT64_C(0x000000000001803b), UINT64_C(0x08b2ef8fa05ce68e), UINT64_C(0x08b2f20fa060a68e),
        UINT64_C(0x08b2e9cfa063468a), UINT64_C(0x08b2eb8fa0628656)
    },
    {   UINT64_C(0x3b4762000001803c), UINT64_C(0x0fa2f54f78796696), UINT64_C(0x0fa2f20fa07b2666),
        UINT64_C(0x0fa2f54f7879468e), UINT64_C(0x0fa2f54f7879468e)
    },
    {   UINT64_C(0x000770001db9803d), UINT64_C(0x0fa2f54f7879468a), UINT64_C(0x0fa2f54f78794666),
        UINT64_C(0x0fa2f54f78794656), UINT64_C(0x11a2f58f788145ec)
    },
    {   UINT64_C(0x000000000001803e), UINT64_C(0x0fa2f54de079460f), UINT64_C(0x0fa2f04f787a868e),
        UINT64_C(0x0fa2f04f787a868e), UINT64_C(0x0fa2f04f787a868a)
    },
    {   UINT64_C(0x000000000001803f), UINT64_C(0x0fa2f04f787a8656), UINT64_C(0x0fa2f18c3879e68e),
        UINT64_C(0x0fa2ef4c3c794693), UINT64_C(0x0fa2f18c3879e656)
    },
    {   UINT64_C(0x0000000000018040), UINT64_C(0x1692f1cfa096e6ed), UINT64_C(0x0de2f74f7875c619),
        UINT64_C(0x1692f1cfa096e754), UINT64_C(0x0de2f74f7875c680)
    },
    {   UINT64_C(0x0000000000018041), UINT64_C(0x1692f1cd0c96e68a), UINT64_C(0x0de2f74d0c7985b4),
        UINT64_C(0x14daf50f688da762), UINT64_C(0x0de2f44f7872068e)
    },
    {   UINT64_C(0x0000000000018042), UINT64_C(0x14daf50f688da75e), UINT64_C(0x0de2f44f7872068a),
        UINT64_C(0x14daf50f688da72a), UINT64_C(0x0de2f44f78720656)
    },
    {   UINT64_C(0x0000000000005394), UINT64_C(0x1f43180f2caa66fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43028efcb2071d), UINT64_C(0x1f43024f34afc70d)
    },
    {   UINT64_C(0x0000000000018044), UINT64_C(0x1692f20fa094267e), UINT64_C(0x0fa2f4cf787de693),
        UINT64_C(0x131af10fa08926ed), UINT64_C(0x0de2f44f78738619)
    },
    {   UINT64_C(0x0000000000018045), UINT64_C(0x131af10fa0892754), UINT64_C(0x0de2f44f78738680),
        UINT64_C(0x131af10fa089272a), UINT64_C(0x0de2f44f78738656)
    },
    {   UINT64_C(0x0000000000018046), UINT64_C(0x131af10d0c89267e), UINT64_C(0x0de2f44d0c7385b4),
        UINT64_C(0x131af10fa089275e), UINT64_C(0x0de2f44f7873868a)
    },
    {   UINT64_C(0x0000000000018047), UINT64_C(0x1692f60f6897275e), UINT64_C(0x0fa2f50c3879468a),
        UINT64_C(0x1692f60f68972754), UINT64_C(0x0fa2f50c38794680)
    },
    {   UINT64_C(0x000000000000c895), UINT64_C(0x1d22ff1068aa85cb), UINT64_C(0x1d22ff1068aa85cb),
        UINT64_C(0x1772ff109c93c5bf), UINT64_C(0x1772ff114c93c711)
    },
    {   UINT64_C(0x0000000000005397), UINT64_C(0x1f42f71024b366b3), UINT64_C(0x1f42fe0fc0b76710),
        UINT64_C(0x1f43138f28b14729), UINT64_C(0x1f42fa9008b48714)
    },
    {   UINT64_C(0x000000000001804a), UINT64_C(0x0a6aee4fa067e73a), UINT64_C(0x08b2e7cfa064a666),
        UINT64_C(0x0a6af0cfa066c6ed), UINT64_C(0x08b2e8cfa062c619)
    },
    {   UINT64_C(0x00000000000140e2), UINT64_C(0x15e2f08fa09c069d), UINT64_C(0x1a92f04fa0ada69d),
        UINT64_C(0x0e12f04fa07d669d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c897), UINT64_C(0x1772ff114c93c734), UINT64_C(0x1772fbd14c9a672e),
        UINT64_C(0x1772ff114c93c6b9), UINT64_C(0x1772ff114c93c758)
    },
    {   UINT64_C(0x000000000001804d), UINT64_C(0x0baaec8f686c675e), UINT64_C(0x08b2dc8c3863668a),
        UINT64_C(0x1692f64b4098e67e), UINT64_C(0x0fa2efcb407da693)
    },
    {   UINT64_C(0x00000000000140e5), UINT64_C(0x14daddcfa0832693), UINT64_C(0x14dae74fa0902685),
        UINT64_C(0x131aeb0fa0882685), UINT64_C(0x1852e80fa0a26685)
    },
    {   UINT64_C(0x00000000000140e6), UINT64_C(0x1672f4cf5894e695), UINT64_C(0x0c2ae60fa0714685),
        UINT64_C(0x14dae8cfa0964685), UINT64_C(0x147add0fa0872693)
    },
    {   UINT64_C(0x00000000000140e7), UINT64_C(0x1bcae6cf70b12685), UINT64_C(0x1692e74f649c0685),
        UINT64_C(0x1752f44fa09b0685), UINT64_C(0x1692f4cf5894e695)
    },
    {   UINT64_C(0x0000000000018051), UINT64_C(0x0fa2f20fa07b268e), UINT64_C(0x1692f10b4096e67e),
        UINT64_C(0x0fa2f20b407b25b4), UINT64_C(0x1692f10f7496e75e)
    },
    {   UINT64_C(0x00000000000140e9), UINT64_C(0x131afa4fa08fc685), UINT64_C(0x131af34fa095a695),
        UINT64_C(0x181af48fa09de685), UINT64_C(0x14dae80fa0954685)
    },
    {   UINT64_C(0x0000000000018053), UINT64_C(0x1692f68f689486ed), UINT64_C(0x0fa2f54f78794619),
        UINT64_C(0x1692f68f68948754), UINT64_C(0x0fa2f54f78794680)
    },
    {   UINT64_C(0x00000000000140eb), UINT64_C(0x1202ed4f6c864690), UINT64_C(0x0e32eccf6c718690),
        UINT64_C(0x0f42ec4c6c79c690), UINT64_C(0x08b2ee8f7c624690)
    },
    {   UINT64_C(0x00000000000140ec), UINT64_C(0x10c2ec4f6c7d0690), UINT64_C(0x1202ed4f6c8645b6),
        UINT64_C(0x0fa2da4c6c7a86a2), UINT64_C(0x0db2eecc707825b6)
    },
    {   UINT64_C(0x0000000000018056), UINT64_C(0x14daf24fa091075e), UINT64_C(0x0a6aef4fa068668a),
        UINT64_C(0x1162f8cf687be762), UINT64_C(0x0c2afacf786a068e)
    },
    {   UINT64_C(0x00000000000140ee), UINT64_C(0x0faaed4f6c7b86a2), UINT64_C(0x08b2ee8f7c5c45b6),
        UINT64_C(0x0fa2e84f707d85b6), UINT64_C(0x0f22e58f5877e6a2)
    },
    {   UINT64_C(0x0000000000018058), UINT64_C(0x1162f8cf687be784), UINT64_C(0x0c2af78f786a268a),
        UINT64_C(0x131af24c4488a67e), UINT64_C(0x08b2f14c4461662b)
    },
    {   UINT64_C(0x00000000000140f0), UINT64_C(0x1172ec8f6485c5a9), UINT64_C(0x0f7ad9cc6c7665b6),
        UINT64_C(0x0cfaeccc6c7665b6), UINT64_C(0x0f9aed4f6c8185a9)
    },
    {   UINT64_C(0x00000000000140f1), UINT64_C(0x0cfaef0f7c7765a9), UINT64_C(0x10c2ec4f6c7a25b6),
        UINT64_C(0x1532ed4c6c8be5b6), UINT64_C(0x0ca2cc0c6c7ee5b6)
    },
    {   UINT64_C(0x00000000000140f2), UINT64_C(0x1462eccc6c97e5b6), UINT64_C(0x16faed4f6c9305b6),
        UINT64_C(0x08b2ee8f7c6a6677), UINT64_C(0x10c2ec4f6c7a2677)
    },
    {   UINT64_C(0x000000000001805c), UINT64_C(0x1692f18f6896a762), UINT64_C(0x0fa2f04f787a868e),
        UINT64_C(0x1692f18d0c96a67e), UINT64_C(0x0fa2f04d307a85aa)
    },
    {   UINT64_C(0x00000000000140f4), UINT64_C(0x0fa2f88f707a66a0), UINT64_C(0x1232f00f6c8826a2),
        UINT64_C(0x131af34fa0958695), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140f5), UINT64_C(0x0000000000000000), UINT64_C(0x1532ed4c6c8be6a3),
        UINT64_C(0x19e2ed8f6ca805a9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000140f6), UINT64_C(0x1692f4cc8094e695), UINT64_C(0x0fa2ed4c6c75a5b6),
        UINT64_C(0x14eaf60c6c91a695), UINT64_C(0x0edaeccc6c7e25d4)
    },
    {   UINT64_C(0x00000000000140f7), UINT64_C(0x14daeacfa0924685), UINT64_C(0x106af60ca87de5aa),
        UINT64_C(0x17aaec4f589d4701), UINT64_C(0x0f2af20fa078c6a2)
    },
    {   UINT64_C(0x00000000000058b4), UINT64_C(0x1f42f6cf30ad66fc), UINT64_C(0x1f42f8cf44b326fc),
        UINT64_C(0x1f42f8cf2cb06705), UINT64_C(0x1f42f74ec0ae2701)
    },
    {   UINT64_C(0x0000000000018062), UINT64_C(0x0fa2f44f787e6617), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058b6), UINT64_C(0x1f42f70eb4af671e), UINT64_C(0x1f42f60ec0ae4728),
        UINT64_C(0x1f42f50eb8b5e723), UINT64_C(0x1f42f9cf58aac715)
    },
    {   UINT64_C(0x0000000000018064), UINT64_C(0x1222f64f8884c694), UINT64_C(0x0000000000000000),
        UINT64_C(0x0d92ef8d2473c6aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018065), UINT64_C(0x0000000000000000), UINT64_C(0x16faf04f78974693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018066), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2efcfa07da693),
        UINT64_C(0x08b2f2cfa05ea693), UINT64_C(0x1042fbcfa07f0684)
    },
    {   UINT64_C(0x0000000000018067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f20bfc7b25b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018068), UINT64_C(0x1692f68f689486da), UINT64_C(0x110af54f7880e5c2),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018069), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2ef4c3c794685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014101), UINT64_C(0x148af90f5894a695), UINT64_C(0x1162e88f587d4695),
        UINT64_C(0x0c2ae60fa0714685), UINT64_C(0x0c2ae54fa07a2771)
    },
    {   UINT64_C(0x0000000000014102), UINT64_C(0x0fa2ec0e1484c685), UINT64_C(0x1dd2df4f58ae8685),
        UINT64_C(0x1eb2e60fa0b20685), UINT64_C(0x19f2ffcfa09d2685)
    },
    {   UINT64_C(0x000000000001806c), UINT64_C(0x0fa2f04f7881e619), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014104), UINT64_C(0x14dadfcfa08a0693), UINT64_C(0x13dae98fa08d2685),
        UINT64_C(0x1472e7cfa08c8685), UINT64_C(0x12e2e60fa090c685)
    },
    {   UINT64_C(0x000000000001806e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0beaef0bf86b85aa), UINT64_C(0x0fa2f38fa07ac68c)
    },
    {   UINT64_C(0x0000000000014106), UINT64_C(0x1802e5cfa0a14685), UINT64_C(0x1802e5cfa0a1479c),
        UINT64_C(0x1532e54fa095a68e), UINT64_C(0x1732df4f589e0685)
    },
    {   UINT64_C(0x00000000000058c3), UINT64_C(0x1f42f50f2cb76704), UINT64_C(0x1f42f94f34b4a6ee),
        UINT64_C(0x1f42fd8eb8b726f1), UINT64_C(0x1f42f7cf14aca6f5)
    },
    {   UINT64_C(0x00000000000058c4), UINT64_C(0x1f43028e18a8272f), UINT64_C(0x1f42f80f38b7c70d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f34b106ff)
    },
    {   UINT64_C(0x00000000000058c5), UINT64_C(0x1f42fa4f14b4a6f5), UINT64_C(0x1f42f98e5cb226f9),
        UINT64_C(0x1f42f74f50b24727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058c6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78ed8b6e714),
        UINT64_C(0x1f4306ceb0ae8719), UINT64_C(0x1f430f4eb0ae6722)
    },
    {   UINT64_C(0x000000000001410b), UINT64_C(0x13e2e94fa086c685), UINT64_C(0x14faef0f588f8695),
        UINT64_C(0x1c4ae48f58ab6695), UINT64_C(0x1632ddcfa09a0685)
    },
    {   UINT64_C(0x000000000001410c), UINT64_C(0x107af30f687ca5b6), UINT64_C(0x0f9af60f6c85c695),
        UINT64_C(0x0f12f44f6c77c5b6), UINT64_C(0x0b82ec0f686ce5b5)
    },
    {   UINT64_C(0x00000000000058c9), UINT64_C(0x1f42f90f18b4a70b), UINT64_C(0x1f42fe0eecb0e70c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43044f98b0670e)
    },
    {   UINT64_C(0x000000000001410e), UINT64_C(0x10f2f68f7c7e85b6), UINT64_C(0x10f2f68f7c7ec6a1),
        UINT64_C(0x1052f10f808065b5), UINT64_C(0x0fdae20f747a25b6)
    },
    {   UINT64_C(0x000000000001410f), UINT64_C(0x14dae20f748e25a9), UINT64_C(0x10faf14f7c7ea5b6),
        UINT64_C(0x0fa2f40f6c7905b6), UINT64_C(0x10faf14f7c7ea5b6)
    },
    {   UINT64_C(0x0000000000014110), UINT64_C(0x0f8ad60c6c7945b6), UINT64_C(0x0d9af48f6c7365b6),
        UINT64_C(0x184af08f7c9be5b6), UINT64_C(0x0dfad68c6c7345b6)
    },
    {   UINT64_C(0x00000000000058cd), UINT64_C(0x1f42f7cef8b62712), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cef8b14704), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058ce), UINT64_C(0x1f42fc8ee4b72700), UINT64_C(0x1f42f7cf40b56703),
        UINT64_C(0x1f42f38ef4b7a720), UINT64_C(0x1f42f88f90b1e6f0)
    },
    {   UINT64_C(0x000000000001807c), UINT64_C(0x08b2dc8c3863468a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058d0), UINT64_C(0x1f42f7ce2cb5871e), UINT64_C(0x1f42f90ec8b386f5),
        UINT64_C(0x1f42f70eccb6271c), UINT64_C(0x1f42f90f1cb7c6f8)
    },
    {   UINT64_C(0x000000000001807e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f1cfa096e792), UINT64_C(0x0de2f74f7875c762)
    },
    {   UINT64_C(0x000000000001807f), UINT64_C(0x1bcaee0fa0aa6762), UINT64_C(0x14daf78f848d868e),
        UINT64_C(0x1692f68e60948762), UINT64_C(0x0fa2f54de079468e)
    },
    {   UINT64_C(0x0000000000014117), UINT64_C(0x1052f10f808066a1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0dfad68c6c7506a1), UINT64_C(0x10d2f58cd47e05b6)
    },
    {   UINT64_C(0x0000000000014118), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x17cafd8fa096e685), UINT64_C(0x11caf30f6c80e6a3)
    },
    {   UINT64_C(0x00000000000058d5), UINT64_C(0x1f42f7cea0ab472b), UINT64_C(0x1f42f94f24b2872c),
        UINT64_C(0x1f42f80f00b706f6), UINT64_C(0x1f42f7ce5cb28715)
    },
    {   UINT64_C(0x000000000001411a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1e92e88fa0b14685), UINT64_C(0x1a62ed0f6ca5a5a9)
    },
    {   UINT64_C(0x000000000000c8b3), UINT64_C(0x1b33064e809f2644), UINT64_C(0x1772ff105493e5b7),
        UINT64_C(0x1402f84f30896600), UINT64_C(0x1402f84f30896600)
    },
    {   UINT64_C(0x000000000001411c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692fb4f58982695), UINT64_C(0x0fa2f40f6c7905b6)
    },
    {   UINT64_C(0x000000000001411d), UINT64_C(0x14dafe8f589c468e), UINT64_C(0x0f3af1cf6c8145b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053b6), UINT64_C(0x1f42fe0ebcb38716), UINT64_C(0x1f43020f90b18700),
        UINT64_C(0x1f4301cef0b7a706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018088), UINT64_C(0x0000000000000000), UINT64_C(0x14faf4cd48900693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000053b7), UINT64_C(0x1f42facf90b52700), UINT64_C(0x1f430b8fecb7a70e),
        UINT64_C(0x1f43034fb8b0e709), UINT64_C(0x1f42f98eacb5c72b)
    },
    {   UINT64_C(0x000000000000c8b6), UINT64_C(0x29c2f80f40e085fe), UINT64_C(0x29c2f80f40e085fe),
        UINT64_C(0x177302094092477c), UINT64_C(0x177302094092477c)
    },
    {   UINT64_C(0x00000000000053b8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8f18b4c6f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024f4cb18721)
    },
    {   UINT64_C(0x000000000000c8b7), UINT64_C(0x1773020dc092465b), UINT64_C(0x1773020dc092465b),
        UINT64_C(0x1773020d7892466d), UINT64_C(0x1773020d7892466d)
    },
    {   UINT64_C(0x000000000001808d), UINT64_C(0x0d6af2cd4871c693), UINT64_C(0x1502f20d489025b4),
        UINT64_C(0x0c8af14d486e462c), UINT64_C(0x08b2dc8c3856a5b4)
    },
    {   UINT64_C(0x00000000000058e1), UINT64_C(0x1f42f80f30b6470c), UINT64_C(0x1f42fa4e80b1e721),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f6cb546e8)
    },
    {   UINT64_C(0x00000000000058e2), UINT64_C(0x1f42facf04b64711), UINT64_C(0x1f42f7ce7cb5a740),
        UINT64_C(0x1f42fc4f7cad86e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058e3), UINT64_C(0x1f430e8e44b3473d), UINT64_C(0x1f42fb8f90b28704),
        UINT64_C(0x1f42fa4e70ad86ff), UINT64_C(0x1f42f40eb8ac8711)
    },
    {   UINT64_C(0x00000000000058e4), UINT64_C(0x1f43008f8cadc6f5), UINT64_C(0x1f42fecf20b0c6fb),
        UINT64_C(0x1f42f64e9cb5271f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058e5), UINT64_C(0x1f43190e84b14701), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f74f34b8072e), UINT64_C(0x1f42fccf50ace6e5)
    },
    {   UINT64_C(0x00000000000058e6), UINT64_C(0x1f42f98f54ae8717), UINT64_C(0x1f42f78e88b9472a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8ef8b5e70d)
    },
    {   UINT64_C(0x000000000000c8bb), UINT64_C(0x1773024c109226af), UINT64_C(0x1773024c109226af),
        UINT64_C(0x1243214ce86d668e), UINT64_C(0x1e8afd9274b0c547)
    },
    {   UINT64_C(0x0000000000018095), UINT64_C(0x0de2f08f747025b3), UINT64_C(0x0de2f44d2072a5b4),
        UINT64_C(0x114af4cbfc896693), UINT64_C(0x125af4cf7889c693)
    },
    {   UINT64_C(0x000000000000c8bc), UINT64_C(0x1e8b168e74a4662b), UINT64_C(0x1e8b168e74a4662b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018097), UINT64_C(0x0db2f30f686f25c3), UINT64_C(0x137af30f6889e5c3),
        UINT64_C(0x0efaf30f687445c3), UINT64_C(0x09dae1cc386385b4)
    },
    {   UINT64_C(0x00000000000058eb), UINT64_C(0x1f42f60ee4aa4727), UINT64_C(0x1f42f88ed4b446e9),
        UINT64_C(0x1f42ffcebca4a6f7), UINT64_C(0x1f42fe4f2caac6fc)
    },
    {   UINT64_C(0x00000000000058ec), UINT64_C(0x1f42f98edcb666f1), UINT64_C(0x1f43068ee0b0671d),
        UINT64_C(0x1f43020e88b18727), UINT64_C(0x1f42fc4f2cad671a)
    },
    {   UINT64_C(0x00000000000058ed), UINT64_C(0x1f42fb4f90aac6fc), UINT64_C(0x1f42fa4ec4b54710),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84f08b326fc)
    },
    {   UINT64_C(0x00000000000053c0), UINT64_C(0x1f42f8cf68b626f6), UINT64_C(0x1f43058f88b82701),
        UINT64_C(0x1f43054e9cafe703), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c8bf), UINT64_C(0x0000000000000000), UINT64_C(0x1773028f6c9206c7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001809d), UINT64_C(0x12d2f54f8084a5aa), UINT64_C(0x0fa2f54f787945b4),
        UINT64_C(0x1682f3cf8895a5aa), UINT64_C(0x14daf74f908d25c3)
    },
    {   UINT64_C(0x000000000000c8c0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fbcfa0ab66ee), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000058f2), UINT64_C(0x1f42fa4eb0ac270a), UINT64_C(0x1f42f9ceacb5a704),
        UINT64_C(0x1f42fc4ef4af06f1), UINT64_C(0x1f42f6cf34b2e6ed)
    },
    {   UINT64_C(0x00000000000058f3), UINT64_C(0x1f42fb0f20b466f0), UINT64_C(0x1f42f84eacb6e713),
        UINT64_C(0x1f42fb0f00b0a703), UINT64_C(0x1f42facf80b1e708)
    },
    {   UINT64_C(0x00000000000180a1), UINT64_C(0x0982dc8c386d0693), UINT64_C(0x0c82f1cbfc77a5be),
        UINT64_C(0x0daae84bd4734693), UINT64_C(0x08b2f20d9c61c5b4)
    },
    {   UINT64_C(0x00000000000058f5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0f04ac0719),
        UINT64_C(0x1f42f8ceb8b3c723), UINT64_C(0x1f42fc0efcaa46d5)
    },
    {   UINT64_C(0x00000000000180a3), UINT64_C(0x0fa2ed0fa07745b8), UINT64_C(0x1692f34fa09545b8),
        UINT64_C(0x0de2ed8fa077e684), UINT64_C(0x125af3cfa08505b8)
    },
    {   UINT64_C(0x00000000000180a4), UINT64_C(0x1082f4cc3885a5aa), UINT64_C(0x0fdaf4cd487b65aa),
        UINT64_C(0x0ceaf10bfc6f85aa), UINT64_C(0x0d7af10adc71c5aa)
    },
    {   UINT64_C(0x00000000000180a5), UINT64_C(0x0e12f9cfa0708693), UINT64_C(0x0e12fa0fa070a693),
        UINT64_C(0x0e12f9cf6870867e), UINT64_C(0x0e13020c0871c5b4)
    },
    {   UINT64_C(0x00000000000180a6), UINT64_C(0x169af64f5c94c68f), UINT64_C(0x0e62f1cfa07605b8),
        UINT64_C(0x0efaf30f687445c3), UINT64_C(0x12c2f54f748765f3)
    },
    {   UINT64_C(0x00000000000180a7), UINT64_C(0x11e2f34fa08485b8), UINT64_C(0x0c1ae84bfc720693),
        UINT64_C(0x103aee4bfc7ce5aa), UINT64_C(0x0eb2f34fa075a5b8)
    },
    {   UINT64_C(0x00000000000058fb), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9ce88ad6727),
        UINT64_C(0x1f42f98e7cb4c72b), UINT64_C(0x1f42f74e9cb3c710)
    },
    {   UINT64_C(0x00000000000180a9), UINT64_C(0x173af4cbfc98c693), UINT64_C(0x1b02f4cd20a80693),
        UINT64_C(0x10c2f14f787c662b), UINT64_C(0x0f1af14bfc820693)
    },
    {   UINT64_C(0x000000000000c8c6), UINT64_C(0x0000000000000000), UINT64_C(0x1772fe5150942577),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180ab), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x135af1cc008b05bd), UINT64_C(0x153af1cbfc99a5bd)
    },
    {   UINT64_C(0x00000000000180ac), UINT64_C(0x0bd2efd4e06c2738), UINT64_C(0x0bd2efd4e06c2738),
        UINT64_C(0x0962f912445d073c), UINT64_C(0x07e2ef54e05e0691)
    },
    {   UINT64_C(0x00000000000180ad), UINT64_C(0x08aaf094dc5ea690), UINT64_C(0x0a2af0924068c690),
        UINT64_C(0x0c32ef94dc6de690), UINT64_C(0x1042ef94cc7e8690)
    },
    {   UINT64_C(0x00000000000180ae), UINT64_C(0x0b92f192606ba690), UINT64_C(0x06830496345ea694),
        UINT64_C(0x098af2d634670694), UINT64_C(0x0a6b01d68c59c696)
    },
    {   UINT64_C(0x00000000000180af), UINT64_C(0x0a6b01d68c59c696), UINT64_C(0x0a6b01d68c59c696),
        UINT64_C(0x0632f6d73452e690), UINT64_C(0x0632f6d73452e690)
    },
    {   UINT64_C(0x00000000000180b0), UINT64_C(0x0a32f3d43c64669a), UINT64_C(0x0a32f3d43c64669a),
        UINT64_C(0x09eaf6562c6226db), UINT64_C(0x09eaf6562c6226db)
    },
    {   UINT64_C(0x00000000000180b1), UINT64_C(0x09eaebd6d46766b1), UINT64_C(0x09eaebd6d46766b1),
        UINT64_C(0x0a6af0d78c66c68a), UINT64_C(0x0a6af0d78c66c68a)
    },
    {   UINT64_C(0x00000000000180b2), UINT64_C(0x08b30bd6a05266b1), UINT64_C(0x0a72f0d82c66c641),
        UINT64_C(0x0a6b05579466868f), UINT64_C(0x0a6af397945da68f)
    },
    {   UINT64_C(0x00000000000180b3), UINT64_C(0x08b30bcc9452642e), UINT64_C(0x0a72f0cd2466e380),
        UINT64_C(0x0a6af44ca05da3d3), UINT64_C(0x0a6b030ca06503d3)
    },
    {   UINT64_C(0x0000000000005907), UINT64_C(0x1f42f84f4cb6672d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88e94af0715), UINT64_C(0x1f42f98ee8b68708)
    },
    {   UINT64_C(0x00000000000180b5), UINT64_C(0x0a6afacfe861a4b5), UINT64_C(0x0a6afacfe861a4b5),
        UINT64_C(0x0a6affd0085f64c2), UINT64_C(0x0a6affd1305f6478)
    },
    {   UINT64_C(0x00000000000180b6), UINT64_C(0x0a7af4d78c650680), UINT64_C(0x0a6b0b97cc59a656),
        UINT64_C(0x0a6afed7a05fc6af), UINT64_C(0x0a6afe0d0c5da3e8)
    },
    {   UINT64_C(0x00000000000180b7), UINT64_C(0x0a6aee57f067e666), UINT64_C(0x0a6aed578c6d868e),
        UINT64_C(0x0952ee12885f65a3), UINT64_C(0x0a6afa164462067e)
    },
    {   UINT64_C(0x00000000000180b8), UINT64_C(0x0bd2f4130c686694), UINT64_C(0x06baf2d4e0572738),
        UINT64_C(0x091af714cc5f2691), UINT64_C(0x0be2ef54e06d0690)
    },
    {   UINT64_C(0x00000000000180b9), UINT64_C(0x0aaaf954e066673a), UINT64_C(0x0ceafa0fa06d267e),
        UINT64_C(0x0caaf80fa06c267e), UINT64_C(0x0caaf80fa06c267e)
    },
    {   UINT64_C(0x00000000000180ba), UINT64_C(0x0caaf80fa06c267e), UINT64_C(0x0caaf80fa06c267e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180bb), UINT64_C(0x0a6af34d5465642e), UINT64_C(0x0caaf0978c6fe681),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c8cf), UINT64_C(0x0000000000000000), UINT64_C(0x1ce3020ca8a8069e),
        UINT64_C(0x0000000000000000), UINT64_C(0x16c2fbcc8092a6c4)
    },
    {   UINT64_C(0x0000000000005910), UINT64_C(0x1f42f84e90b3a715), UINT64_C(0x1f42f64f14b5a6e1),
        UINT64_C(0x1f42f90f60a8a700), UINT64_C(0x1f42f8cee0b36700)
    },
    {   UINT64_C(0x0000000000005911), UINT64_C(0x1f42f90f70b68700), UINT64_C(0x1f42f9cfccb00712),
        UINT64_C(0x1f42fa8f60b24700), UINT64_C(0x1f42f8cf14ae86fc)
    },
    {   UINT64_C(0x00000000000053d2), UINT64_C(0x1f42f60e1cae2720), UINT64_C(0x1f42fa0f38ad26df),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180c0), UINT64_C(0x000291578c2c268e), UINT64_C(0x000291578c2c268e),
        UINT64_C(0x00028d978c35268a), UINT64_C(0x00028b57f0366666)
    },
    {   UINT64_C(0x00000000000180c1), UINT64_C(0x00028dd82c354641), UINT64_C(0x0002761ba841071c),
        UINT64_C(0x000290d78c330680), UINT64_C(0x0002a817cc27e656)
    },
    {   UINT64_C(0x00000000000180c2), UINT64_C(0x00028f57cc346656), UINT64_C(0x0002a157502fc6d7),
        UINT64_C(0x00029bd7a02e46af), UINT64_C(0x000289d78c3bc68e)
    },
    {   UINT64_C(0x00000000000180c3), UINT64_C(0x00028dd78c35468a), UINT64_C(0x0002af97702626a4),
        UINT64_C(0x00029c97702de6a4), UINT64_C(0x000289d78c3bc68e)
    },
    {   UINT64_C(0x00000000000180c4), UINT64_C(0x000290d78c3306e7), UINT64_C(0x000290d78c330680),
        UINT64_C(0x0002a357782a86d1), UINT64_C(0x0002a357782a86d1)
    },
    {   UINT64_C(0x00000000000180c5), UINT64_C(0x0002a357782a86d1), UINT64_C(0x0002d8d7784546d1),
        UINT64_C(0x0002964c202ea3b3), UINT64_C(0x0002914c202c23b3)
    },
    {   UINT64_C(0x00000000000180c6), UINT64_C(0x00028acb342443b3), UINT64_C(0x0002ab4b343483b3),
        UINT64_C(0x00028f17ec3486c7), UINT64_C(0x0002e3150444e5c2)
    },
    {   UINT64_C(0x00000000000180c7), UINT64_C(0x0002940b782183a1), UINT64_C(0x000297cbe03023b3),
        UINT64_C(0x000297cbe03023b3), UINT64_C(0x00029c8bb82dc3b3)
    },
    {   UINT64_C(0x00000000000180c8), UINT64_C(0x00029c8d002dc36c), UINT64_C(0x0002b34b243ea433),
        UINT64_C(0x0002e08b24554433), UINT64_C(0x0002a80c3c27e372)
    },
    {   UINT64_C(0x00000000000180c9), UINT64_C(0x00028f4c38346371), UINT64_C(0x00029bcb702e43a3),
        UINT64_C(0x0002a34a1c2a837a), UINT64_C(0x00029a8c442ee3e8)
    },
    {   UINT64_C(0x00000000000180ca), UINT64_C(0x00029d8d0c2d63e8), UINT64_C(0x0002af8bf8244382),
        UINT64_C(0x00028dcbf4356386), UINT64_C(0x00028f0c1435237a)
    },
    {   UINT64_C(0x00000000000180cb), UINT64_C(0x00028dcbe035439f), UINT64_C(0x00028dcbe035439f),
        UINT64_C(0x0002918c1c3363a4), UINT64_C(0x0002918c403363ad)
    },
    {   UINT64_C(0x00000000000180cc), UINT64_C(0x00028b4c34366377), UINT64_C(0x00028dcc9435435b),
        UINT64_C(0x0002760ca441035b), UINT64_C(0x0002760af041035b)
    },
    {   UINT64_C(0x00000000000180cd), UINT64_C(0x000289d2f835a528), UINT64_C(0x00028e133834c4fa),
        UINT64_C(0x00027613384104fa), UINT64_C(0x00028f0e7839662c)
    },
    {   UINT64_C(0x00000000000180ce), UINT64_C(0x00028f0d3042667e), UINT64_C(0x0002a80b782b83a1),
        UINT64_C(0x00028dcca83563b3), UINT64_C(0x00029fcc142c23b3)
    },
    {   UINT64_C(0x00000000000180cf), UINT64_C(0x00028f0cac34a3a7), UINT64_C(0x00029c97d42da6b3),
        UINT64_C(0x0002a7576c286725), UINT64_C(0x0002761ba8410788)
    },
    {   UINT64_C(0x0000000000005923), UINT64_C(0x1f42fe0ec4b2671d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ceb0ae86fa)
    },
    {   UINT64_C(0x00000000000180d1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0002969808360691), UINT64_C(0x000af0cb1066e35c)
    },
    {   UINT64_C(0x0000000000005925), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78e8cb68713),
        UINT64_C(0x1f42f78f78b86707), UINT64_C(0x1f42f8cf28ac8722)
    },
    {   UINT64_C(0x00000000000180d3), UINT64_C(0x00028d97f03786f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005927), UINT64_C(0x1f42fb4f5cb526ff), UINT64_C(0x1f42fbcec0b0470c),
        UINT64_C(0x1f42f28f04b4473d), UINT64_C(0x1f42f8ce90b96721)
    },
    {   UINT64_C(0x0000000000005928), UINT64_C(0x1f42f80f48af8709), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f70e6caf6715), UINT64_C(0x1f42fa0e5cb4e721)
    },
    {   UINT64_C(0x00000000000180d6), UINT64_C(0x0000000000000000), UINT64_C(0x000294c9e433a3a7),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180d7), UINT64_C(0x000290cbfc63639c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180d8), UINT64_C(0x00028b57b46c2661), UINT64_C(0x000290d78c636680),
        UINT64_C(0x00028b4b686ae3a7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000592c), UINT64_C(0x1f42f8cf30b0c6f8), UINT64_C(0x1f42f7cea4b06701),
        UINT64_C(0x1f42f84f14b6472b), UINT64_C(0x1f42f08f98b5c6f7)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000592e), UINT64_C(0x1f42fbcec4b3872a), UINT64_C(0x1f42f90e64b60719),
        UINT64_C(0x1f42fb0eacaf4719), UINT64_C(0x1f42f40f4cb70728)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005930), UINT64_C(0x1f42f88f48b92701), UINT64_C(0x1f42f70f08b3e713),
        UINT64_C(0x1f42f60ef0b46713), UINT64_C(0x1f42fc4f10b3a731)
    },
    {   UINT64_C(0x0000000000005931), UINT64_C(0x1f42f60e58b1474c), UINT64_C(0x1f42f44f10b04726),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70e5cb2e720)
    },
    {   UINT64_C(0x00000000000180df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x08b3020d6c59e5b3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005933), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70eccb7471b),
        UINT64_C(0x1f430fce44b06731), UINT64_C(0x1f42fb4f6cb726df)
    },
    {   UINT64_C(0x00000000000180e1), UINT64_C(0x08ab0a578c5c867e), UINT64_C(0x0a6af2978c66067e),
        UINT64_C(0x1692f1cfa096e693), UINT64_C(0x08b30255105685b3)
    },
    {   UINT64_C(0x00000000000180e2), UINT64_C(0x1772f00fa09aa693), UINT64_C(0x1a92f00fa0a70693),
        UINT64_C(0x0eb2f00fa076a693), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180e3), UINT64_C(0x1692f00f68948693), UINT64_C(0x0000000000000000),
        UINT64_C(0x1a42f00fa0a4c693), UINT64_C(0x1742f00fa097e693)
    },
    {   UINT64_C(0x00000000000180e4), UINT64_C(0x0aa2f28f7866067e), UINT64_C(0x1692f1cfa096e68a),
        UINT64_C(0x14daf24fa088a67e), UINT64_C(0x125af0cfa086a67e)
    },
    {   UINT64_C(0x00000000000180e5), UINT64_C(0x1692fa0fa092e68a), UINT64_C(0x131af10fa089267e),
        UINT64_C(0x1322f08fa089467e), UINT64_C(0x1692f28fa096667e)
    },
    {   UINT64_C(0x00000000000180e6), UINT64_C(0x1692f68f6894868c), UINT64_C(0x0a6af28fa065e67e),
        UINT64_C(0x16daf64fa098e67e), UINT64_C(0x15f2f1cfa094668a)
    },
    {   UINT64_C(0x00000000000180e7), UINT64_C(0x1bcaf10fa0aa867e), UINT64_C(0x1692f10f7496e67e),
        UINT64_C(0x141af54fa08b467e), UINT64_C(0x1692f68f6894868c)
    },
    {   UINT64_C(0x00000000000180e8), UINT64_C(0x1692f28fa096667e), UINT64_C(0x116af20fa082467e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1382f58fa089867e)
    },
    {   UINT64_C(0x00000000000180e9), UINT64_C(0x131af24fa088a67e), UINT64_C(0x1692f54fa096668a),
        UINT64_C(0x17daf6cfa099867e), UINT64_C(0x1692f08fa096867e)
    },
    {   UINT64_C(0x00000000000180ea), UINT64_C(0x1752f38fa099069a), UINT64_C(0x1742f54fa097e68c),
        UINT64_C(0x0a6af28fa0660751), UINT64_C(0x1692f54fa0966751)
    },
    {   UINT64_C(0x00000000000180eb), UINT64_C(0x10faf54f7880a67e), UINT64_C(0x0dbaf44f7871667e),
        UINT64_C(0x1002f08c3c77067e), UINT64_C(0x089af30f785fe67e)
    },
    {   UINT64_C(0x00000000000180ec), UINT64_C(0x1062f20f787c467e), UINT64_C(0x10faf54f7880a5b4),
        UINT64_C(0x0f82fbcc3c78c693), UINT64_C(0x0ed2f08c387605b1)
    },
    {   UINT64_C(0x00000000000180ed), UINT64_C(0x0fa2f54f78794693), UINT64_C(0x0dbaf44f787165b4),
        UINT64_C(0x0dcaf6cc38716693), UINT64_C(0x1002f08c3c7705b4)
    },
    {   UINT64_C(0x00000000000180ee), UINT64_C(0x0f82f4cf78790693), UINT64_C(0x089af30f785fe5b4),
        UINT64_C(0x0fa2efcfa07d65b4), UINT64_C(0x0f8af10f747c0693)
    },
    {   UINT64_C(0x00000000000180ef), UINT64_C(0x1082fbcc3c7f05aa), UINT64_C(0x0e3af30f6875e5b4),
        UINT64_C(0x0dcaf6cc38716693), UINT64_C(0x0fcaf6cf7879a5b4)
    },
    {   UINT64_C(0x00000000000180f0), UINT64_C(0x0faaf04f687ac5aa), UINT64_C(0x0f82fbcc3c78c5b4),
        UINT64_C(0x0dcaf6cc387485b4), UINT64_C(0x1122f54f787f45aa)
    },
    {   UINT64_C(0x00000000000180f1), UINT64_C(0x0eeaeecf7875c5aa), UINT64_C(0x1062f20f787c45b4),
        UINT64_C(0x137af54c3c88a5b4), UINT64_C(0x0fa2f0cc307b45b4)
    },
    {   UINT64_C(0x00000000000180f2), UINT64_C(0x15b2f30c3c9405b4), UINT64_C(0x138af54f7888e5b4),
        UINT64_C(0x0aa2f28f78660656), UINT64_C(0x1062f20f787c4656)
    },
    {   UINT64_C(0x0000000000005946), UINT64_C(0x1f42f84e54b58737), UINT64_C(0x1f42f7ce24b6a736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f70edcb4671e)
    },
    {   UINT64_C(0x00000000000180f4), UINT64_C(0x0e12fb8f7871e69d), UINT64_C(0x1152ee0f78828693),
        UINT64_C(0x1692f54fa095c68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180f5), UINT64_C(0x0000000000000000), UINT64_C(0x137af54c3c88a693),
        UINT64_C(0x17d2ef8f789945aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007fff), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f4faca886f8)
    },
    {   UINT64_C(0x000000000000594a), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68e7cb50735),
        UINT64_C(0x1f42f80e64b56722), UINT64_C(0x1f42f50e80b1c730)
    },
    {   UINT64_C(0x000000000000594b), UINT64_C(0x1f42f74ea4b7672c), UINT64_C(0x1f42ff4f2cb0c736),
        UINT64_C(0x1f42fece70b4272d), UINT64_C(0x1f43124f34b5e731)
    },
    {   UINT64_C(0x000000000000594c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42facf64b4a70a), UINT64_C(0x1f42f60e64b46707)
    },
    {   UINT64_C(0x000000000000594d), UINT64_C(0x1f430f4f94b3a70d), UINT64_C(0x1f431b4ffca7c6e4),
        UINT64_C(0x1f43088f58b7e70e), UINT64_C(0x1f43108ff0b226ee)
    },
    {   UINT64_C(0x000000000000594e), UINT64_C(0x1f42f6cf38b7471b), UINT64_C(0x1f430ed02cb86714),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff9004b5c712)
    },
    {   UINT64_C(0x00000000000180fc), UINT64_C(0x113af88f787ee5b4), UINT64_C(0x0fa2f54c447945b4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005950), UINT64_C(0x0000000000000000), UINT64_C(0x1f43084f6cb6e711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cfc4b266e5)
    },
    {   UINT64_C(0x0000000000005951), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018fecb8470b),
        UINT64_C(0x1f42f50f28b78714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008008), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a6b0e8f7c5c068c),
        UINT64_C(0x0cc301565c67e68c), UINT64_C(0x0fa2ef8fa07c667e)
    },
    {   UINT64_C(0x0000000000008009), UINT64_C(0x0fa2f90e4477c6bf), UINT64_C(0x175afd4f549446aa),
        UINT64_C(0x1852f88f6c9c468c), UINT64_C(0x05a2fa165c4f268c)
    },
    {   UINT64_C(0x000000000000800a), UINT64_C(0x0a6afa0cdc64868c), UINT64_C(0x0a6af54cdc62268c),
        UINT64_C(0x0fa2ff13c474a68c), UINT64_C(0x156afa0efc8e060b)
    },
    {   UINT64_C(0x00001800c008800b), UINT64_C(0x07d2fbcd6c56e44e), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x07d2ff8f7455244c), UINT64_C(0x08b2ebcf6862668c)
    },
    {   UINT64_C(0x0000000000018103), UINT64_C(0x1592f2cfa0944788), UINT64_C(0x0000000000000000),
        UINT64_C(0x163af1cf6495477b), UINT64_C(0x165af2cd3c95867e)
    },
    {   UINT64_C(0x0000000000005957), UINT64_C(0x1f42fbcf78b62709), UINT64_C(0x1f4305cfacb466e0),
        UINT64_C(0x1f43061014b806fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018105), UINT64_C(0x14caf18d3c8f467e), UINT64_C(0x13aaf38fa08a667e),
        UINT64_C(0x1feaf00fa0bd268c), UINT64_C(0x1202f50f6882a68c)
    },
    {   UINT64_C(0x0000000000018106), UINT64_C(0x169af2cfa096867e), UINT64_C(0x169af2cfa096877b),
        UINT64_C(0x1592f2cfa094468c), UINT64_C(0x155af04f6891867e)
    },
    {   UINT64_C(0x04806e0380008010), UINT64_C(0x1ccb0b0f68a3a68c), UINT64_C(0x1692f1cfa096e68a),
        UINT64_C(0x14daf24fa088a67e), UINT64_C(0x14daf50f688da68c)
    },
    {   UINT64_C(0x0810c00002988011), UINT64_C(0x1692f20fa094267e), UINT64_C(0x131af10fa089267e),
        UINT64_C(0x1162f0cfa082c67e), UINT64_C(0x1692f60f6897268c)
    },
    {   UINT64_C(0x0000000000018109), UINT64_C(0x180af78fa099e67e), UINT64_C(0x1662f04fa096667e),
        UINT64_C(0x165af2cd3c95867e), UINT64_C(0x148aeecfa08e667e)
    },
    {   UINT64_C(0x0da1800005308013), UINT64_C(0x131af10fa089467e), UINT64_C(0x1bcaf10fa0aa867e),
        UINT64_C(0x1692f10f7496e67e), UINT64_C(0x1692f68f6894868c)
    },
    {   UINT64_C(0x1171f40007208014), UINT64_C(0x116af20fa082467e), UINT64_C(0x1692f68cdc96268c),
        UINT64_C(0x14daf24fa091067e), UINT64_C(0x1162f8cf687be68c)
    },
    {   UINT64_C(0x000000000000595f), UINT64_C(0x1f42fd0f20b6a71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cf98b2470e)
    },
    {   UINT64_C(0x000000000001810d), UINT64_C(0x0e72f18d7c7565aa), UINT64_C(0x0e92f78f787525b4),
        UINT64_C(0x168af18fa096e5b0), UINT64_C(0x0c32f18f786b25b4)
    },
    {   UINT64_C(0x000000000001810e), UINT64_C(0x106af3cfa07d45aa), UINT64_C(0x106af3cfa07d46a8),
        UINT64_C(0x0fbaf3cfa07c65b0), UINT64_C(0x0f9af00f787a05aa)
    },
    {   UINT64_C(0x000000000001810f), UINT64_C(0x134af3cfa088c5aa), UINT64_C(0x106af3cfa07d45aa),
        UINT64_C(0x1002f6cf787a05b4), UINT64_C(0x106af3cfa07d45aa)
    },
    {   UINT64_C(0x0000000000005963), UINT64_C(0x1f42f78ec0b4a720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a4e40ad672d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x2340008c5178801a), UINT64_C(0x0fa2f04fa07b6693), UINT64_C(0x08b2f20fa05e2693),
        UINT64_C(0x08b2dc8c3856c693), UINT64_C(0x0fa2efcfa07da693)
    },
    {   UINT64_C(0x252498921230801b), UINT64_C(0x08b2f2cfa05ea693), UINT64_C(0x1852f20fa09f65b4),
        UINT64_C(0x0fa2f20fa07b25b4), UINT64_C(0x0fa2f54f787945b4)
    },
    {   UINT64_C(0x0000000000018113), UINT64_C(0x0e4af3cfa073a5aa), UINT64_C(0x0de2f18f7871c5b4),
        UINT64_C(0x1712f3cf789665b4), UINT64_C(0x0ebaef0fa07685aa)
    },
    {   UINT64_C(0x0000000000018114), UINT64_C(0x0000000000000000), UINT64_C(0x0e92f78f78752656),
        UINT64_C(0x1002ef8c38756693), UINT64_C(0x0c52f24fa06ee68f)
    },
    {   UINT64_C(0x0000000000018115), UINT64_C(0x0de2f68f787445b4), UINT64_C(0x0c2afa4f7869c5b3),
        UINT64_C(0x08b2f54fa05fc693), UINT64_C(0x08b2ee4fa063e656)
    },
    {   UINT64_C(0x000000000000801f), UINT64_C(0x0642fecf684f268c), UINT64_C(0x0f030e8ccc6e0690),
        UINT64_C(0x10eaf8127c7d452d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018117), UINT64_C(0x0fbaf3cfa07c668f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f5af00c387ae6a8), UINT64_C(0x1032f3cd7c7c65aa)
    },
    {   UINT64_C(0x0000000000018118), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1752f48fa097a67e), UINT64_C(0x10daf00fa07de693)
    },
    {   UINT64_C(0x000000000000596c), UINT64_C(0x1f42fdce7cb2271e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43040eacb406b6), UINT64_C(0x1f4308ceccb18722)
    },
    {   UINT64_C(0x000000000001811a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f32ef8fa0ba867e), UINT64_C(0x1692f18fa09705aa)
    },
    {   UINT64_C(0x000000000000596e), UINT64_C(0x1f42f78ec8b6870e), UINT64_C(0x1f430acf88b12716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf20b666ea)
    },
    {   UINT64_C(0x000000000001811c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16caf70f6894c68c), UINT64_C(0x1002f6cf787a05b4)
    },
    {   UINT64_C(0x0000000000005970), UINT64_C(0x0000000000000000), UINT64_C(0x1f430bce8cb22736),
        UINT64_C(0x1f42fbce9caca724), UINT64_C(0x1f42f70e7cb766f9)
    },
    {   UINT64_C(0x0000000000005971), UINT64_C(0x1f42f80f50b66706), UINT64_C(0x1f42facf10b86715),
        UINT64_C(0x1f42fcce3cb0872c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000c06), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x106b120e547c669e), UINT64_C(0x106af54e546de69e)
    },
    {   UINT64_C(0x0000000000008029), UINT64_C(0x0fa2e88f787fc5fe), UINT64_C(0x0fa2e0cfa07e867e),
        UINT64_C(0x0642fecf684f268c), UINT64_C(0x0fa2ff8d5073c68c)
    },
    {   UINT64_C(0x0000000000005974), UINT64_C(0x1f42f60f48b6a702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f80f40b6a716), UINT64_C(0x1f42f80e94ae4715)
    },
    {   UINT64_C(0x000000000000802b), UINT64_C(0x12c2fa116c838571), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x17c2f78f6898c68c), UINT64_C(0x0a6af0d82c66c641)
    },
    {   UINT64_C(0x0000000000005976), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38f64b8c70c),
        UINT64_C(0x1f42f60e78b006f9), UINT64_C(0x1f42f70f0cb72719)
    },
    {   UINT64_C(0x0000000000018124), UINT64_C(0x11daf2cfa082870b), UINT64_C(0x0c52f24fa06ee616),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005405), UINT64_C(0x1f42f81014abc702), UINT64_C(0x1f42fc0fb8b066ed),
        UINT64_C(0x1f42fece4cb74739), UINT64_C(0x1f42f7cee4af26fb)
    },
    {   UINT64_C(0x0000000000005979), UINT64_C(0x0000000000000000), UINT64_C(0x1f43308fd89986bb),
        UINT64_C(0x1f43005064b586b6), UINT64_C(0x1f4316919ca4066b)
    },
    {   UINT64_C(0x000000000000597a), UINT64_C(0x1f432a8ea0a64720), UINT64_C(0x1f42ff0e9cb3e710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000597b), UINT64_C(0x1f431f8e78a50739), UINT64_C(0x1f42f9cf98b5070b),
        UINT64_C(0x1f43144e80b5072a), UINT64_C(0x1f430e4eecb5c70a)
    },
    {   UINT64_C(0x000000000000597c), UINT64_C(0x1f43010e7cb2a6db), UINT64_C(0x1f43068e6cb1c70d),
        UINT64_C(0x1f43114fccad86e5), UINT64_C(0x1f430c4f0cab46ed)
    },
    {   UINT64_C(0x000000000000597d), UINT64_C(0x1f4305cf08b526ce), UINT64_C(0x1f431f0f30a626ee),
        UINT64_C(0x1f430dcf08af670c), UINT64_C(0x1f430b4f88aa2707)
    },
    {   UINT64_C(0x0000000000005408), UINT64_C(0x1f42f40eecb106fe), UINT64_C(0x1f42f80fa8acc6fc),
        UINT64_C(0x1f42fbcf38b4e72b), UINT64_C(0x1f42f5cfe4b88704)
    },
    {   UINT64_C(0x000000000000597f), UINT64_C(0x1f43098fecb206eb), UINT64_C(0x1f43038fd4b586f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005980), UINT64_C(0x1f430a8eb8ace70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4319ce3cabc72c), UINT64_C(0x1f43020f74af26e9)
    },
    {   UINT64_C(0x0000000000005981), UINT64_C(0x1f43028fc0b406dc), UINT64_C(0x1f43054fccb3a6bf),
        UINT64_C(0x1f43144f7cb28706), UINT64_C(0x1f43098ed8ae071e)
    },
    {   UINT64_C(0x000000000000540a), UINT64_C(0x1f42f40fc4b08713), UINT64_C(0x1f42fa0e44b26724),
        UINT64_C(0x1f42f7cfc4b8670a), UINT64_C(0x1f42f90f1cb7c70d)
    },
    {   UINT64_C(0x0000000000005983), UINT64_C(0x1f431d4f20a706fe), UINT64_C(0x1f430ed054ac8696),
        UINT64_C(0x1f43100f48a9a70a), UINT64_C(0x1f431d4e70b1c745)
    },
    {   UINT64_C(0x0000000000005984), UINT64_C(0x1f43054f54b7a6d6), UINT64_C(0x1f43038efcb7a6e8),
        UINT64_C(0x1f4309cf0cb006d4), UINT64_C(0x1f43130e40a9e739)
    },
    {   UINT64_C(0x0000000000005985), UINT64_C(0x1f43068ec0b2e73d), UINT64_C(0x1f4302ce5cb2a71b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005986), UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f74b0e6f6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000770001db8803d), UINT64_C(0x0fa2f54f7879468a), UINT64_C(0x0fa2f54f78794666),
        UINT64_C(0x0fa2f54f78794656), UINT64_C(0x11a2f58f788145ec)
    },
    {   UINT64_C(0x0000000000005988), UINT64_C(0x1f43054f58b4c717), UINT64_C(0x1f43081024aac6fb),
        UINT64_C(0x1f43060f74ae26e6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000803f), UINT64_C(0x0fa2f04f787a8656), UINT64_C(0x0fa2f18c3879e68e),
        UINT64_C(0x0fa2ef4c3c794693), UINT64_C(0x0fa2f18c3879e656)
    },
    {   UINT64_C(0x000000000000598a), UINT64_C(0x1f430b4e98ac273a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a0f14ac6712)
    },
    {   UINT64_C(0x0000000000008041), UINT64_C(0x1692f1cd0c96e68a), UINT64_C(0x0de2f74d0c7985b4),
        UINT64_C(0x14daf50f688da762), UINT64_C(0x0de2f44f7872068e)
    },
    {   UINT64_C(0x000000000000598c), UINT64_C(0x1f43030f80b246fb), UINT64_C(0x1f42fd8f24b426fb),
        UINT64_C(0x1f43048ec8af86fa), UINT64_C(0x1f4306cfd0b346e5)
    },
    {   UINT64_C(0x0000000000008043), UINT64_C(0x14daf50f688da75e), UINT64_C(0x0de2f44f7872068a),
        UINT64_C(0x1692f20fa094275e), UINT64_C(0x1252f4cf788a06a5)
    },
    {   UINT64_C(0x000000000000598e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43040ee8b72701),
        UINT64_C(0x1f431dcee4aa26f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000598f), UINT64_C(0x1f43070f8cb4a6f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43120f7cb086e6), UINT64_C(0x1f43058f94afa700)
    },
    {   UINT64_C(0x0000000000008046), UINT64_C(0x131af10d0c89267e), UINT64_C(0x0de2f44d0c7385b4),
        UINT64_C(0x131af10fa089275e), UINT64_C(0x0de2f44f7873868a)
    },
    {   UINT64_C(0x0000000000008047), UINT64_C(0x1692f60f6897275e), UINT64_C(0x0fa2f50c3879468a),
        UINT64_C(0x1692f60f68972754), UINT64_C(0x0fa2f50c38794680)
    },
    {   UINT64_C(0x0000000000008048), UINT64_C(0x1692f60f6897272a), UINT64_C(0x0fa2f50c38794656),
        UINT64_C(0x1692f60b4097268c), UINT64_C(0x0fa2f50c387946e6)
    },
    {   UINT64_C(0x0000000000005993), UINT64_C(0x1f43070f6cade6f1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43048ef0b5672a)
    },
    {   UINT64_C(0x0000000000005994), UINT64_C(0x0000000000000000), UINT64_C(0x1f431bce40a7e710),
        UINT64_C(0x1f4304101cb0e6af), UINT64_C(0x1f430b8f60b4e714)
    },
    {   UINT64_C(0x000000000000804b), UINT64_C(0x0a6af28fa065e754), UINT64_C(0x08b2edcfa0618680),
        UINT64_C(0x0a6af28d0c65e67e), UINT64_C(0x08b2f20d0c612693)
    },
    {   UINT64_C(0x0000000000005414), UINT64_C(0x1f42ff4ec0b2c739), UINT64_C(0x1f42fc0f78b626ed),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f90ade6d3)
    },
    {   UINT64_C(0x000000000000804d), UINT64_C(0x0baaec8f686c675e), UINT64_C(0x08b2dc8c3863668a),
        UINT64_C(0x1692f64b4098e67e), UINT64_C(0x0fa2efcb407da693)
    },
    {   UINT64_C(0x0000000000005998), UINT64_C(0x0000000000000000), UINT64_C(0x1f430acefcab86fe),
        UINT64_C(0x1f43044ed0a9671d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000804f), UINT64_C(0x08b2f2cb405ea693), UINT64_C(0x131af10fa0894693),
        UINT64_C(0x0beaf2cfa06bc6a6), UINT64_C(0x1362f54fa08b667e)
    },
    {   UINT64_C(0x000000000000599a), UINT64_C(0x1f42fd8ea8b2472f), UINT64_C(0x1f42ff8f60b2a726),
        UINT64_C(0x1f43074f14b3a71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008051), UINT64_C(0x0fa2f20fa07b268e), UINT64_C(0x1692f10b4096e67e),
        UINT64_C(0x0fa2f20b407b25b4), UINT64_C(0x1692f10f7496e75e)
    },
    {   UINT64_C(0x0000000000008052), UINT64_C(0x0fa2f20fa07b268a), UINT64_C(0x1272f30fa08546a6),
        UINT64_C(0x1632f20f5892a68e), UINT64_C(0x0fc2f20c387385b4)
    },
    {   UINT64_C(0x0000000000008053), UINT64_C(0x1692f68f689486ed), UINT64_C(0x0fa2f54f78794619),
        UINT64_C(0x1692f68f68948754), UINT64_C(0x0fa2f54f78794680)
    },
    {   UINT64_C(0x0000000000008054), UINT64_C(0x1692f68f68948762), UINT64_C(0x0fa2f54f7879468e),
        UINT64_C(0x1bcaf58f88ad2684), UINT64_C(0x1692f58f7894c5b4)
    },
    {   UINT64_C(0x000000000000599f), UINT64_C(0x1f43094fb8b0870e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43064eb0b02706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008056), UINT64_C(0x14daf24fa091075e), UINT64_C(0x0a6aef4fa068668a),
        UINT64_C(0x1162f8cf687be762), UINT64_C(0x0c2afacf786a068e)
    },
    {   UINT64_C(0x00000000000059a1), UINT64_C(0x1f43050f84aee70f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0f6cb5a6dc), UINT64_C(0x1f43070f88b806db)
    },
    {   UINT64_C(0x00000000000059a2), UINT64_C(0x1f430e8e4cae4748), UINT64_C(0x1f430ace3cb4c6e8),
        UINT64_C(0x1f430a0eecaa26e7), UINT64_C(0x1f43124e70b2a72d)
    },
    {   UINT64_C(0x000000000000c919), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000541b), UINT64_C(0x1f42f84fd4b06703), UINT64_C(0x1f42fdcf88aee6e8),
        UINT64_C(0x1f42f94eacae66fa), UINT64_C(0x1f42fc0fb4b32716)
    },
    {   UINT64_C(0x000000000000805b), UINT64_C(0x1692f18f6896a754), UINT64_C(0x0fa2f04f787a8680),
        UINT64_C(0x1692f18f6896a76a), UINT64_C(0x0fa2f04f787a86af)
    },
    {   UINT64_C(0x000000000000805c), UINT64_C(0x1692f18f6896a762), UINT64_C(0x0fa2f04f787a868e),
        UINT64_C(0x1692f18d0c96a67e), UINT64_C(0x0fa2f04d307a85aa)
    },
    {   UINT64_C(0x00000000000059a7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8efcb046ed),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43054f24b44715)
    },
    {   UINT64_C(0x000000000000805e), UINT64_C(0x1692f38fa0966750), UINT64_C(0x1322f08fa0894762),
        UINT64_C(0x0de2f4cfa072c68e), UINT64_C(0x1322f08fa089472a)
    },
    {   UINT64_C(0x000000000000805f), UINT64_C(0x0de2f4cfa072c656), UINT64_C(0x1322f08fa0894784),
        UINT64_C(0x0de2f4cfa072c68a), UINT64_C(0x0a72f30fa06e6693)
    },
    {   UINT64_C(0x0000000000008060), UINT64_C(0x0fa2e94f78798693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059ab), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43070e84b62739), UINT64_C(0x1f4301cf20b4c72a)
    },
    {   UINT64_C(0x0000000000018c1b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1c2af74efca7867d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059ad), UINT64_C(0x1f42fd8e84b92729), UINT64_C(0x1f4305cf14b10724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ce48b2672c)
    },
    {   UINT64_C(0x00000000000059ae), UINT64_C(0x1f42ff0e70b84735), UINT64_C(0x1f42fd0ea8b3c721),
        UINT64_C(0x1f43040e5cb54704), UINT64_C(0x1f43060e7cb2070b)
    },
    {   UINT64_C(0x00000000000059af), UINT64_C(0x1f43068e88af4737), UINT64_C(0x1f43050e5cb3e71b),
        UINT64_C(0x1f42fb0ee8b72706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008066), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2efcfa07da693),
        UINT64_C(0x08b2f2cfa05ea693), UINT64_C(0x1042fbcfa07f0684)
    },
    {   UINT64_C(0x0000000000008067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f20bfc7b25b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059b2), UINT64_C(0x1f4305cf1caf8719), UINT64_C(0x1f4305cee4b60734),
        UINT64_C(0x1f4306ce98af871d), UINT64_C(0x1f4305ce50b1e71f)
    },
    {   UINT64_C(0x00000000000059b3), UINT64_C(0x1f43168f04a7c71a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8ea8b12725)
    },
    {   UINT64_C(0x00000000000059b4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43020f2cb8870c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000806b), UINT64_C(0x0000000000000000), UINT64_C(0x09b2f18f78682693),
        UINT64_C(0x0000000000000000), UINT64_C(0x17f2f18f68a3c6ee)
    },
    {   UINT64_C(0x00000000000059b6), UINT64_C(0x1f4304cf38ae86e2), UINT64_C(0x1f43084ef0b0670f),
        UINT64_C(0x1f42ff0dd8b6c735), UINT64_C(0x1f42ff0eccade71c)
    },
    {   UINT64_C(0x000000000000806d), UINT64_C(0x0000000000000000), UINT64_C(0x1322f08fa089467e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059b8), UINT64_C(0x1f4308ce90b0873c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43080fe8b4471d)
    },
    {   UINT64_C(0x00000000000059b9), UINT64_C(0x1f4306ce8cb3a6f7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4e30b5e746), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059ba), UINT64_C(0x1f4306cf98b4a705), UINT64_C(0x1f430c4e50ae6735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f0e34ab472f)
    },
    {   UINT64_C(0x00000000000059bb), UINT64_C(0x1f4305ceecb6271e), UINT64_C(0x1f43084ebcb26739),
        UINT64_C(0x1f4303cf38b42715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059bc), UINT64_C(0x1f431a1124a606a0), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43151030ace71e), UINT64_C(0x1f43130edcabe732)
    },
    {   UINT64_C(0x00000000000059bd), UINT64_C(0x1f43088ef8b28706), UINT64_C(0x1f43104edcaa873b),
        UINT64_C(0x1f43248e80a406ff), UINT64_C(0x1f430f4e70a78729)
    },
    {   UINT64_C(0x00000000000059be), UINT64_C(0x1f43028eb0b36720), UINT64_C(0x1f4319ce60a7873e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059bf), UINT64_C(0x1f43144e8cb5e731), UINT64_C(0x1f431b4ee0a4e73f),
        UINT64_C(0x1f43190ee0aaa73f), UINT64_C(0x1f43028e60b3473b)
    },
    {   UINT64_C(0x00000000000059c0), UINT64_C(0x1f43094ebcb7e734), UINT64_C(0x1f42fece80ad471b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43020e80b62738)
    },
    {   UINT64_C(0x00000000000059c1), UINT64_C(0x1f42fb0e4cb7a726), UINT64_C(0x1f42fc4f30b34718),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059c2), UINT64_C(0x1f430b4f48990715), UINT64_C(0x1f43000e90b08743),
        UINT64_C(0x1f42f80fe0b2c6bb), UINT64_C(0x1f43050fb8a8e6fa)
    },
    {   UINT64_C(0x00000000000059c3), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb5034b4e6f3),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f80f88ada6da)
    },
    {   UINT64_C(0x00000000000059c4), UINT64_C(0x1f42f80f04ad66fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64ef8af66be), UINT64_C(0x1f42f44fc4b7a6dc)
    },
    {   UINT64_C(0x00000000000059c5), UINT64_C(0x1f43038f20a88724), UINT64_C(0x1f42f90fb4b246fa),
        UINT64_C(0x1f42f64e68b76706), UINT64_C(0x1f430d8eeca8071f)
    },
    {   UINT64_C(0x00000000000059c6), UINT64_C(0x1f42fa4f60b5870c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058f1caf4724)
    },
    {   UINT64_C(0x00000000000059c7), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6cf64aa272b),
        UINT64_C(0x1f43008e90b24738), UINT64_C(0x1f42fccef4abe731)
    },
    {   UINT64_C(0x000000000000807e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f1cfa096e792), UINT64_C(0x0de2f74f7875c762)
    },
    {   UINT64_C(0x000000000000807f), UINT64_C(0x1bcaee0fa0aa6762), UINT64_C(0x14daf78f848d868e),
        UINT64_C(0x1692f68e60948762), UINT64_C(0x0fa2f54de079468e)
    },
    {   UINT64_C(0x00000000000059ca), UINT64_C(0x1f43044eecb84723), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43098e5cb8c72a), UINT64_C(0x1f430a0e7cb4e723)
    },
    {   UINT64_C(0x00000000000059cb), UINT64_C(0x1f43060e8cb8c723), UINT64_C(0x1f42fd8f28b2870c),
        UINT64_C(0x1f430ecf48b7072f), UINT64_C(0x1f43080f28b5c713)
    },
    {   UINT64_C(0x00000000000059cc), UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e5cb7c72b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf1cb9c6fb)
    },
    {   UINT64_C(0x00000000000059cd), UINT64_C(0x1f42fe8f38ae8727), UINT64_C(0x1f430c4e98b4c710),
        UINT64_C(0x1f43058ed8afc72c), UINT64_C(0x1f43028f68b14704)
    },
    {   UINT64_C(0x00000000000059ce), UINT64_C(0x1f42fc8f68ac8704), UINT64_C(0x1f42f8cf08b78710),
        UINT64_C(0x1f42facf18b6e704), UINT64_C(0x1f42fdce98a4c710)
    },
    {   UINT64_C(0x000000000000c92f), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000059d0), UINT64_C(0x1f430e8edcac6705), UINT64_C(0x1f42f5cf68b7a704),
        UINT64_C(0x1f43154ef4ad871d), UINT64_C(0x1f42fb8f08b62713)
    },
    {   UINT64_C(0x0000000000010804), UINT64_C(0x0a6afa529c61c4e9), UINT64_C(0x0a6afa529c61c4e9),
        UINT64_C(0x0fa2ec12a07e04ea), UINT64_C(0x0fa2ec92b47da4db)
    },
    {   UINT64_C(0x0000000000010805), UINT64_C(0x1bcaec92b4ae44db), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000ec5d310806), UINT64_C(0x0a6b18d670654682), UINT64_C(0x0a6b13d67062c682),
        UINT64_C(0x0a6af90d9c55644d), UINT64_C(0x0a6b18567062c682)
    },
    {   UINT64_C(0x00000000000059d4), UINT64_C(0x1f42fb8ee0b02723), UINT64_C(0x1f42f98ea4b56727),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f4cf2ca9e717)
    },
    {   UINT64_C(0x0000000000010808), UINT64_C(0x0fa3074d247b8682), UINT64_C(0x0fa2f38d647be682),
        UINT64_C(0x105aff908c7765a4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059d6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42face48b1a700),
        UINT64_C(0x1f42fbceccb8c708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c933), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000059d8), UINT64_C(0x1f42f98ef8b1e733), UINT64_C(0x1f43168ee8b2271f),
        UINT64_C(0x1f4305cef0afc722), UINT64_C(0x1f42fbcef8b4a733)
    },
    {   UINT64_C(0x00000000000059d9), UINT64_C(0x1f42f84f48b6471a), UINT64_C(0x1f430d4ebcb3471a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059da), UINT64_C(0x1f42fb8f40b1071c), UINT64_C(0x1f42fc8f3cb00714),
        UINT64_C(0x1f42f7ce8cb40737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0007be000001080e), UINT64_C(0x0000000000000000), UINT64_C(0x0a6afad03461a57b),
        UINT64_C(0x0a6afb103461c57b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001080f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee1ba87d071c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059dd), UINT64_C(0x1f42fc4f28ae46f8), UINT64_C(0x1f42f30e5cb1e723),
        UINT64_C(0x1f42fa4f18b5e6f7), UINT64_C(0x1f42fa8f48b3a710)
    },
    {   UINT64_C(0x0000000000010811), UINT64_C(0x053ac3cf7868a68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059df), UINT64_C(0x1f42f88e7cb2c71b), UINT64_C(0x1f42f98ee0b32703),
        UINT64_C(0x1f42fb8f94b7a714), UINT64_C(0x1f42fc8f18b56714)
    },
    {   UINT64_C(0x00000000000059e0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f28b5a704),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98eccafe704)
    },
    {   UINT64_C(0x00000000000059e1), UINT64_C(0x1f42f88f08aca70c), UINT64_C(0x1f42fb4f18b026fc),
        UINT64_C(0x1f42fc4f08b4070c), UINT64_C(0x1f42f6cf28b4a70c)
    },
    {   UINT64_C(0x0000000000008098), UINT64_C(0x1292f60c508dc693), UINT64_C(0x10caf60c507765ca),
        UINT64_C(0x11d2f58f7481e5aa), UINT64_C(0x0fa2f2cbf87a85aa)
    },
    {   UINT64_C(0x00000000000059e3), UINT64_C(0x1f42f98f58b8c708), UINT64_C(0x1f42f78ee8b8c704),
        UINT64_C(0x1f42fa4f08b346f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059e4), UINT64_C(0x1f42f88e8cb3072f), UINT64_C(0x1f42fc8f18b0e710),
        UINT64_C(0x1f42fb8eb0b1a70c), UINT64_C(0x1f42fc8e7cb48717)
    },
    {   UINT64_C(0x000000000000809b), UINT64_C(0x0bbaf10fa06bc693), UINT64_C(0x0972f08bfc6a8693),
        UINT64_C(0x112af2cbfc81c693), UINT64_C(0x1852f0cf789ec5aa)
    },
    {   UINT64_C(0x00000000000059e6), UINT64_C(0x1f42f6cf30a866f8), UINT64_C(0x1f42f58f68b20708),
        UINT64_C(0x1f42fa8f18b5a710), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000059e7), UINT64_C(0x1f42ffcee8b44707), UINT64_C(0x1f42ff8ef8b6c71a),
        UINT64_C(0x1f42f7cf48b7a718), UINT64_C(0x1f42fd4eecafc704)
    },
    {   UINT64_C(0x00000000000059e8), UINT64_C(0x1f42f90f70b5e731), UINT64_C(0x1f42f50ef0b8272f),
        UINT64_C(0x1f42fb8f58afa71b), UINT64_C(0x1f43038f44b5272b)
    },
    {   UINT64_C(0x00000000000059e9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f68ab870d),
        UINT64_C(0x1f42fdcf38b3a70d), UINT64_C(0x1f42fb8f68b6a710)
    },
    {   UINT64_C(0x00000000000059ea), UINT64_C(0x1f42fc8f28b62701), UINT64_C(0x1f43018e78b1c733),
        UINT64_C(0x1f42fb8f1cb7a6fb), UINT64_C(0x1f42fa8f68b2c71e)
    },
    {   UINT64_C(0x00000000000059eb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43104ebcb00730),
        UINT64_C(0x1f42fd4eecae8708), UINT64_C(0x1f42fbcf58af071b)
    },
    {   UINT64_C(0x00000000000080a2), UINT64_C(0x0872f14bfc61e62b), UINT64_C(0x0fa2f04f787a85aa),
        UINT64_C(0x10caf98f787bc5aa), UINT64_C(0x0fa2f60f787a05b4)
    },
    {   UINT64_C(0x00000000000059ed), UINT64_C(0x1f43008f04ab8717), UINT64_C(0x1f42fb4ee0b5071b),
        UINT64_C(0x1f430ccef4b2c72d), UINT64_C(0x1f42f48f08b4470c)
    },
    {   UINT64_C(0x00000000000059ee), UINT64_C(0x1f42f7cf08a9e71a), UINT64_C(0x1f42fa4edcb32704),
        UINT64_C(0x1f42f88e7cb92708), UINT64_C(0x1f42f58f08b76714)
    },
    {   UINT64_C(0x00000000000080a5), UINT64_C(0x0e12f9cfa0708693), UINT64_C(0x0e12fa0fa070a693),
        UINT64_C(0x0e12f9cf6870867e), UINT64_C(0x0e13020c0871c5b4)
    },
    {   UINT64_C(0x00000000000059f0), UINT64_C(0x1f42f98eecb8670d), UINT64_C(0x1f42f78f48ad270c),
        UINT64_C(0x1f42f98eccb80704), UINT64_C(0x1f42f68ee8ba2710)
    },
    {   UINT64_C(0x0000000000009402), UINT64_C(0x1b4af8d010a6261d), UINT64_C(0x0aa2f9501063861d),
        UINT64_C(0x12eaf9101084a61d), UINT64_C(0x1b4af8d010a6261d)
    },
    {   UINT64_C(0x00000000000059f2), UINT64_C(0x1f42f98e4cb3e705), UINT64_C(0x1f42f88eecb9e708),
        UINT64_C(0x1f42f68f20bac71f), UINT64_C(0x1f43088e30b4870f)
    },
    {   UINT64_C(0x00000000000080a9), UINT64_C(0x173af4cbfc98c693), UINT64_C(0x1b02f4cd20a80693),
        UINT64_C(0x10c2f14f787c662b), UINT64_C(0x0f1af14bfc820693)
    },
    {   UINT64_C(0x00000000000059f4), UINT64_C(0x1f430c8eb8b38741), UINT64_C(0x1f42fa8f18b5670c),
        UINT64_C(0x1f430e4e88b1e72a), UINT64_C(0x1f430c4ea8b42721)
    },
    {   UINT64_C(0x00000000000080ab), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x135af1cc008b05bd), UINT64_C(0x153af1cbfc99a5bd)
    },
    {   UINT64_C(0x00000000000059f6), UINT64_C(0x1f42fc4f48ab0723), UINT64_C(0x1f42fa8e7cb10714),
        UINT64_C(0x1f42f3cedcad4710), UINT64_C(0x1f42f88e5cb1c705)
    },
    {   UINT64_C(0x00000000000080ad), UINT64_C(0x08aaf094dc5ea690), UINT64_C(0x0a2af0924068c690),
        UINT64_C(0x0c32ef94dc6de690), UINT64_C(0x1042ef94cc7e8690)
    },
    {   UINT64_C(0x000000000001082b), UINT64_C(0x0fa2f20f7081c680), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000940a), UINT64_C(0x1362f08f248ac642), UINT64_C(0x1362f08f248ac642),
        UINT64_C(0x156afa11f08e2552), UINT64_C(0x21f2fbd1a8bf6540)
    },
    {   UINT64_C(0x000000000000940b), UINT64_C(0x1042ee104c87c5b5), UINT64_C(0x156afa10608e25b6),
        UINT64_C(0x156afa0f148e25f9), UINT64_C(0x156afa0dcc8e2646)
    },
    {   UINT64_C(0x000000000000940c), UINT64_C(0x156afa10048e25c3), UINT64_C(0x156afa0edc8e260d),
        UINT64_C(0x156afa0e588e2632), UINT64_C(0x156afa0d308e267e)
    },
    {   UINT64_C(0x000000000000940d), UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afc0b548c26b6),
        UINT64_C(0x156afc0b548c26b6), UINT64_C(0x156afa0e988e2624)
    },
    {   UINT64_C(0x000000000000940e), UINT64_C(0x156af9cd308b067e), UINT64_C(0x156b004d308e467e),
        UINT64_C(0x156afc0cac8c269f), UINT64_C(0x156afe0cac8d269f)
    },
    {   UINT64_C(0x000000000000940f), UINT64_C(0x156af08e048d8689), UINT64_C(0x156afc8d608c8689),
        UINT64_C(0x156afa0c148e26c2), UINT64_C(0x156afa0c148fc6c2)
    },
    {   UINT64_C(0x0000000000009410), UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c071a),
        UINT64_C(0x156afe0d308c267e), UINT64_C(0x156afe0d308c267e)
    },
    {   UINT64_C(0x0000000000009411), UINT64_C(0x156afbcd308c067e), UINT64_C(0x156afe4d308d467e),
        UINT64_C(0x15bafe4c748d46ab), UINT64_C(0x15bafe4c748d46ab)
    },
    {   UINT64_C(0x0000000000009412), UINT64_C(0x156afacc0c8b8647), UINT64_C(0x156aff4c0c8dc647),
        UINT64_C(0x1362f08f248ac600), UINT64_C(0x1362f08f248ac600)
    },
    {   UINT64_C(0x0000000000009413), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x156afd0b888ca6e8)
    },
    {   UINT64_C(0x0000000000005a03), UINT64_C(0x1f42fb8f64a7a715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8f28ae471b), UINT64_C(0x1f42fb8ef8aee717)
    },
    {   UINT64_C(0x0000000000010837), UINT64_C(0x00025c97cc4cc656), UINT64_C(0x0002394c2c5a66bd),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a05), UINT64_C(0x1f430f4e20b4c71e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f40f18b1c710), UINT64_C(0x1f430c8eacb6a728)
    },
    {   UINT64_C(0x0000000000005a06), UINT64_C(0x1f42ff8e98aca715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf28b7c717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a07), UINT64_C(0x1f42faceccb44703), UINT64_C(0x1f42fdcef8b72708),
        UINT64_C(0x1f43028e94b14712), UINT64_C(0x1f42fdce78b30710)
    },
    {   UINT64_C(0x0000000000009419), UINT64_C(0x19d2fccaa0ab0720), UINT64_C(0x0c6af6caa07b4720),
        UINT64_C(0x1122f6caa08e2720), UINT64_C(0x1b62fb8aa0b80720)
    },
    {   UINT64_C(0x0000000000005a09), UINT64_C(0x0000000000000000), UINT64_C(0x1f43104e24b48721),
        UINT64_C(0x1f42fdcedcb18708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000544e), UINT64_C(0x1f42fb0edcb1c700), UINT64_C(0x1f42fc0e60b08734),
        UINT64_C(0x1f42f70f5cb6c72c), UINT64_C(0x1f42fcce9cae6735)
    },
    {   UINT64_C(0x00000000000080c1), UINT64_C(0x00028dd82c354641), UINT64_C(0x0002761ba841071c),
        UINT64_C(0x000290d78c330680), UINT64_C(0x0002a817cc27e656)
    },
    {   UINT64_C(0x0000000000005a0c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42faceccb74700),
        UINT64_C(0x1f42f7cef8aea71b), UINT64_C(0x1f42f8ceccafc700)
    },
    {   UINT64_C(0x0000000000005a0d), UINT64_C(0x1f42fdcefcb5471e), UINT64_C(0x1f42f7ce74b02710),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e8cb0871b)
    },
    {   UINT64_C(0x0000000000005a0e), UINT64_C(0x1f42fc8f58aec710), UINT64_C(0x1f42fccef8b7e71b),
        UINT64_C(0x1f42fece98af6704), UINT64_C(0x1f42fdcef0b5c70d)
    },
    {   UINT64_C(0x000000000000c94f), UINT64_C(0x1622eb4ae4986776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010843), UINT64_C(0x0000000000000000), UINT64_C(0x16e2f24fa09e6675),
        UINT64_C(0x100af94f747ae684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080c7), UINT64_C(0x0002940b782183a1), UINT64_C(0x000297cbe03023b3),
        UINT64_C(0x000297cbe03023b3), UINT64_C(0x00029c8bb82dc3b3)
    },
    {   UINT64_C(0x0000000000010845), UINT64_C(0x0000000000000000), UINT64_C(0x16baf44fa09ce675),
        UINT64_C(0x20b2f2cf64bf6684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010846), UINT64_C(0x0000000000000000), UINT64_C(0x157af24fa0944675),
        UINT64_C(0x169af6ce8497a682), UINT64_C(0x1d82f68f64ac0697)
    },
    {   UINT64_C(0x0000000000010847), UINT64_C(0x0000000000000000), UINT64_C(0x157af24fa0944675),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010848), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ea2f5937cb62675), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010849), UINT64_C(0x1792efcfa09c4675), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a17), UINT64_C(0x1f42fb8f60b4c714), UINT64_C(0x1f42f64ec0b76739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbce9cb4c737)
    },
    {   UINT64_C(0x0000000000005a18), UINT64_C(0x1f42fa8fd4b3471f), UINT64_C(0x1f42fb8f58b56715),
        UINT64_C(0x1f42fa8f4ca9272d), UINT64_C(0x1f42fc8f78a9a723)
    },
    {   UINT64_C(0x0000000000005a19), UINT64_C(0x1f42fc8f68b0a71b), UINT64_C(0x1f42fa8f08b7271a),
        UINT64_C(0x1f42f88e9cb30708), UINT64_C(0x1f42f68f38b7a6fc)
    },
    {   UINT64_C(0x0000000000005a1a), UINT64_C(0x1f430f4eecaee72a), UINT64_C(0x1f42f88f3cafe723),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8cf88aea6f3)
    },
    {   UINT64_C(0x0000000000005a1b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f34b326ec),
        UINT64_C(0x1f42f98f1cab06e8), UINT64_C(0x1f42fbcf38b1270b)
    },
    {   UINT64_C(0x0000000000005a1c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f1cafa6c8),
        UINT64_C(0x1f42fd8f04b36729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a1d), UINT64_C(0x1f42f8cf14b08719), UINT64_C(0x1f42fccf44afe6f7),
        UINT64_C(0x1f42f5cf28a9a71a), UINT64_C(0x1f42fd8f20a746ff)
    },
    {   UINT64_C(0x0000000000010851), UINT64_C(0x0000000000000000), UINT64_C(0x164af24fa0966675),
        UINT64_C(0x122af80f748dc693), UINT64_C(0x101af80f747b25a1)
    },
    {   UINT64_C(0x0000000000010852), UINT64_C(0x092af4cfa069c675), UINT64_C(0x0aaad40c3c6fc675),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a20), UINT64_C(0x1f42fccee8b5c703), UINT64_C(0x1f430d4ef8b3071d),
        UINT64_C(0x1f42f7ce9cb9271c), UINT64_C(0x1f42f10ef8b3e713)
    },
    {   UINT64_C(0x000000000000c958), UINT64_C(0x1623264ae4988776), UINT64_C(0x1622eb4ae46f4776),
        UINT64_C(0x1622eb4ae47ae776), UINT64_C(0x1622eb4ae47ae776)
    },
    {   UINT64_C(0x0000000000005a22), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f48b166e4),
        UINT64_C(0x1f430f4e7cafe70a), UINT64_C(0x1f42ff8f08a4a6e0)
    },
    {   UINT64_C(0x000000000000c959), UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb4ae498653c),
        UINT64_C(0x1622eb4ae4986599), UINT64_C(0x1622eb5258986776)
    },
    {   UINT64_C(0x0000000000005a24), UINT64_C(0x1f42fd8f28ade6e8), UINT64_C(0x1f42f9cf68b346f7),
        UINT64_C(0x1f42f98ed0b386e7), UINT64_C(0x1f42f98e6cb3e739)
    },
    {   UINT64_C(0x0000000000005a25), UINT64_C(0x1f42f7cf68b3271b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fecf2ca926e8), UINT64_C(0x1f42f7cf5cb36727)
    },
    {   UINT64_C(0x0000000000005a26), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e6cb006ff)
    },
    {   UINT64_C(0x0000000000005a27), UINT64_C(0x1f42fb8f48b16700), UINT64_C(0x1f42f4ce98b54721),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078e8caee748)
    },
    {   UINT64_C(0x0000000000005a28), UINT64_C(0x1f4300cf58a9e6e7), UINT64_C(0x1f42fc8f0cb3c6ea),
        UINT64_C(0x1f42f7ced0af4727), UINT64_C(0x1f42f9cf0cb3a70b)
    },
    {   UINT64_C(0x00000000000080df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x08b3020d6c59e5b3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000545e), UINT64_C(0x1f43018f40b3872b), UINT64_C(0x1f43171070a9272b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43014eacb5c72a)
    },
    {   UINT64_C(0x00000000000080e1), UINT64_C(0x08ab0a578c5c867e), UINT64_C(0x0a6af2978c66067e),
        UINT64_C(0x1692f1cfa096e693), UINT64_C(0x08b30255105685b3)
    },
    {   UINT64_C(0x00000000000080e2), UINT64_C(0x1772f00fa09aa693), UINT64_C(0x1a92f00fa0a70693),
        UINT64_C(0x0eb2f00fa076a693), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a2d), UINT64_C(0x1f42f6ceccb786ff), UINT64_C(0x1f430acedcb0e72b),
        UINT64_C(0x1f42f6cee8b74727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080e4), UINT64_C(0x0aa2f28f7866067e), UINT64_C(0x1692f1cfa096e68a),
        UINT64_C(0x14daf24fa088a67e), UINT64_C(0x125af0cfa086a67e)
    },
    {   UINT64_C(0x00000000000080e5), UINT64_C(0x1692fa0fa092e68a), UINT64_C(0x131af10fa089267e),
        UINT64_C(0x1322f08fa089467e), UINT64_C(0x1692f28fa096667e)
    },
    {   UINT64_C(0x00000000000080e6), UINT64_C(0x1692f68f6894868c), UINT64_C(0x0a6af28fa065e67e),
        UINT64_C(0x16daf64fa098e67e), UINT64_C(0x15f2f1cfa094668a)
    },
    {   UINT64_C(0x00000000000080e7), UINT64_C(0x1bcaf10fa0aa867e), UINT64_C(0x1692f10f7496e67e),
        UINT64_C(0x141af54fa08b467e), UINT64_C(0x1692f68f6894868c)
    },
    {   UINT64_C(0x0000000000005462), UINT64_C(0x1f431b0f48a2c6d5), UINT64_C(0x1f42ff0ec8b34724),
        UINT64_C(0x1f42ff4f3cb686e8), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080e9), UINT64_C(0x131af24fa088a67e), UINT64_C(0x1692f54fa096668a),
        UINT64_C(0x17daf6cfa099867e), UINT64_C(0x1692f08fa096867e)
    },
    {   UINT64_C(0x0000000000005463), UINT64_C(0x1f43024f70b506de), UINT64_C(0x1f42ff4f0cb5e70a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43190f10a3a6ec)
    },
    {   UINT64_C(0x0000000000005a35), UINT64_C(0x1f42fa8ec4b38716), UINT64_C(0x1f42f98f00b02717),
        UINT64_C(0x1f42f98eacb76703), UINT64_C(0x1f43154e10b3473c)
    },
    {   UINT64_C(0x0000000000005a36), UINT64_C(0x1f42face9cb72732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f8cf48aee727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080ed), UINT64_C(0x0fa2f54f78794693), UINT64_C(0x0dbaf44f787165b4),
        UINT64_C(0x0dcaf6cc38716693), UINT64_C(0x1002f08c3c7705b4)
    },
    {   UINT64_C(0x00000000000080ee), UINT64_C(0x0f82f4cf78790693), UINT64_C(0x089af30f785fe5b4),
        UINT64_C(0x0fa2efcfa07d65b4), UINT64_C(0x0f8af10f747c0693)
    },
    {   UINT64_C(0x00000000000080ef), UINT64_C(0x1082fbcc3c7f05aa), UINT64_C(0x0e3af30f6875e5b4),
        UINT64_C(0x0dcaf6cc38716693), UINT64_C(0x0fcaf6cf7879a5b4)
    },
    {   UINT64_C(0x0000000000005466), UINT64_C(0x1f4304cf20b2c704), UINT64_C(0x1f43148f00a6c6db),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43198edca9a6fb)
    },
    {   UINT64_C(0x00000000000080f1), UINT64_C(0x0eeaeecf7875c5aa), UINT64_C(0x1062f20f787c45b4),
        UINT64_C(0x137af54c3c88a5b4), UINT64_C(0x0fa2f0cc307b45b4)
    },
    {   UINT64_C(0x00000000000080f2), UINT64_C(0x15b2f30c3c9405b4), UINT64_C(0x138af54f7888e5b4),
        UINT64_C(0x0aa2f28f78660656), UINT64_C(0x1062f20f787c4656)
    },
    {   UINT64_C(0x00000000000080f3), UINT64_C(0x0fcaf6cf7879a67e), UINT64_C(0x1062f20f787c467e),
        UINT64_C(0x138af54f7888e67e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005468), UINT64_C(0x1f42fe8f74b226f6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43120ea8b6e6ff), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080f5), UINT64_C(0x0000000000000000), UINT64_C(0x137af54c3c88a693),
        UINT64_C(0x17d2ef8f789945aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a40), UINT64_C(0x1f42fa0f38b56717), UINT64_C(0x1f42fd8ed0b0e71c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080f7), UINT64_C(0x1162f0cfa082c67e), UINT64_C(0x0eb3020ca874e5aa),
        UINT64_C(0x15faf14f68934705), UINT64_C(0x0ffafe0fa077667e)
    },
    {   UINT64_C(0x0000000000005a42), UINT64_C(0x1f42f98f4caa870b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8e4cb2471e), UINT64_C(0x1f42f6ced0ac071b)
    },
    {   UINT64_C(0x0000000000005a43), UINT64_C(0x1f42fd8f28ae06f4), UINT64_C(0x1f42f78f18b6c710),
        UINT64_C(0x1f42f7ce9cb3c732), UINT64_C(0x1f42f7cf48b2671b)
    },
    {   UINT64_C(0x0000000000005a44), UINT64_C(0x1f42ff8ef8b0270c), UINT64_C(0x1f42f98e0cb1e708),
        UINT64_C(0x1f42f6cf48b88716), UINT64_C(0x1f42f5ce80b8e6ff)
    },
    {   UINT64_C(0x0000000000005a45), UINT64_C(0x1f42fe8ef8ad2703), UINT64_C(0x1f42f78f28b84706),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8f88b2070b)
    },
    {   UINT64_C(0x00000000000080fc), UINT64_C(0x113af88f787ee5b4), UINT64_C(0x0fa2f54c447945b4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000080fd), UINT64_C(0x1692f68f6894868c), UINT64_C(0x0dbaf44f787165b4),
        UINT64_C(0x0de2f60f787245b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a48), UINT64_C(0x1f430f4ea8b0c725), UINT64_C(0x1f42fd4f10b3c714),
        UINT64_C(0x1f43010f78b20717), UINT64_C(0x1f42f8cf50b6071d)
    },
    {   UINT64_C(0x0000000000005a49), UINT64_C(0x1f42fc8f30ae8717), UINT64_C(0x1f42ff4f08b226ec),
        UINT64_C(0x1f42f34f50b8e70e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008100), UINT64_C(0x0000000000000000), UINT64_C(0x13aaf38fa08a6750),
        UINT64_C(0x17a2f28cac96067e), UINT64_C(0x11daf2cfa0828788)
    },
    {   UINT64_C(0x0000000000005a4b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78f38ad4713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60eccb28705)
    },
    {   UINT64_C(0x0000000000005a4c), UINT64_C(0x1f42f60e1cb18732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8e1cb16729), UINT64_C(0x1f42fd8eccb48708)
    },
    {   UINT64_C(0x0000000000008103), UINT64_C(0x1592f2cfa0944788), UINT64_C(0x0000000000000000),
        UINT64_C(0x163af1cf6495477b), UINT64_C(0x165af2cd3c95867e)
    },
    {   UINT64_C(0x0000000000008104), UINT64_C(0x164af04fa096268a), UINT64_C(0x131af2cfa086a67e),
        UINT64_C(0x145af2cfa088e67e), UINT64_C(0x11daf2cfa082867e)
    },
    {   UINT64_C(0x0000000000008105), UINT64_C(0x14caf18d3c8f467e), UINT64_C(0x13aaf38fa08a667e),
        UINT64_C(0x1feaf00fa0bd268c), UINT64_C(0x1202f50f6882a68c)
    },
    {   UINT64_C(0x0000000000005a50), UINT64_C(0x1f42f94ebcb14700), UINT64_C(0x1f42f88e70b7270a),
        UINT64_C(0x1f42f3cf38b28714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010884), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x151b039310884506), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a52), UINT64_C(0x1f42fc8ee4b226f3), UINT64_C(0x1f42f6ceccb56716),
        UINT64_C(0x1f42fb8f3cae871f), UINT64_C(0x1f42fa8e90afe700)
    },
    {   UINT64_C(0x0000000000008109), UINT64_C(0x180af78fa099e67e), UINT64_C(0x1662f04fa096667e),
        UINT64_C(0x165af2cd3c95867e), UINT64_C(0x148aeecfa08e667e)
    },
    {   UINT64_C(0x0000000000005473), UINT64_C(0x1f43070ef8b54738), UINT64_C(0x1f43080eacb72741),
        UINT64_C(0x1f42fbcf6cb786f1), UINT64_C(0x1f43094f08b3e713)
    },
    {   UINT64_C(0x0000000000005a55), UINT64_C(0x1f42facf78b4e713), UINT64_C(0x1f42fb8e64b4c738),
        UINT64_C(0x1f42fa8e7cafe6ec), UINT64_C(0x1f42f5cefcb5270b)
    },
    {   UINT64_C(0x000000000000810c), UINT64_C(0x0e12f74f787645b4), UINT64_C(0x0fdaf7cf787a4695),
        UINT64_C(0x0ed2f40fa07545aa), UINT64_C(0x0c52f24fa06ee5aa)
    },
    {   UINT64_C(0x0000000000005a57), UINT64_C(0x1f42f4ced0adc704), UINT64_C(0x1f42f9ce4cb6c712),
        UINT64_C(0x1f42f88e98b0e70b), UINT64_C(0x1f42fc8eacb56719)
    },
    {   UINT64_C(0x000000000000810e), UINT64_C(0x106af3cfa07d45aa), UINT64_C(0x106af3cfa07d46a8),
        UINT64_C(0x0fbaf3cfa07c65b0), UINT64_C(0x0f9af00f787a05aa)
    },
    {   UINT64_C(0x000000000000810f), UINT64_C(0x134af3cfa088c5aa), UINT64_C(0x106af3cfa07d45aa),
        UINT64_C(0x1002f6cf787a05b4), UINT64_C(0x106af3cfa07d45aa)
    },
    {   UINT64_C(0x0000000000008110), UINT64_C(0x0f9aed8c3c7865b4), UINT64_C(0x0e42f84f787405b4),
        UINT64_C(0x0d92f00fa073c5aa), UINT64_C(0x0f5af00c387ae5aa)
    },
    {   UINT64_C(0x0000000000008111), UINT64_C(0x1532f8cc3c8de67e), UINT64_C(0x0f4af18fa07a05aa),
        UINT64_C(0x106af3cd7c7d45aa), UINT64_C(0x1002f28fa07ba5aa)
    },
    {   UINT64_C(0x0000000000008112), UINT64_C(0x1802f3cfa09ba5aa), UINT64_C(0x1802f3cd7c9ba5aa),
        UINT64_C(0x10daf00fa07de5aa), UINT64_C(0x14f2f3cfa08f45aa)
    },
    {   UINT64_C(0x0000000000008113), UINT64_C(0x0e4af3cfa073a5aa), UINT64_C(0x0de2f18f7871c5b4),
        UINT64_C(0x1712f3cf789665b4), UINT64_C(0x0ebaef0fa07685aa)
    },
    {   UINT64_C(0x0000000000008114), UINT64_C(0x0000000000000000), UINT64_C(0x0e92f78f78752656),
        UINT64_C(0x1002ef8c38756693), UINT64_C(0x0c52f24fa06ee68f)
    },
    {   UINT64_C(0x0000000000008115), UINT64_C(0x0de2f68f787445b4), UINT64_C(0x0c2afa4f7869c5b3),
        UINT64_C(0x08b2f54fa05fc693), UINT64_C(0x08b2ee4fa063e656)
    },
    {   UINT64_C(0x0000000000008116), UINT64_C(0x08b2dacc3855e693), UINT64_C(0x15f2f00f789245aa),
        UINT64_C(0x168af3cfa094a5aa), UINT64_C(0x1002ef8fa07ce693)
    },
    {   UINT64_C(0x0000000000008117), UINT64_C(0x0fbaf3cfa07c668f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f5af00c387ae6a8), UINT64_C(0x1032f3cd7c7c65aa)
    },
    {   UINT64_C(0x0000000000008118), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1752f48fa097a67e), UINT64_C(0x10daf00fa07de693)
    },
    {   UINT64_C(0x0000000000005a63), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cef4a8a705),
        UINT64_C(0x1f42fccedcb5c708), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000811a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f32ef8fa0ba867e), UINT64_C(0x1692f18fa09705aa)
    },
    {   UINT64_C(0x0000000000005a65), UINT64_C(0x1f42f60e4cafc722), UINT64_C(0x1f42f98eecb80723),
        UINT64_C(0x1f43044ef8b6472d), UINT64_C(0x1f42f88ea8b306f3)
    },
    {   UINT64_C(0x000000000000811c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16caf70f6894c68c), UINT64_C(0x1002f6cf787a05b4)
    },
    {   UINT64_C(0x000000000000811d), UINT64_C(0x17f2f20f749d868c), UINT64_C(0x10daf2cf688105b0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a68), UINT64_C(0x1f42f8ceccb50700), UINT64_C(0x1f42f7ced4a9a71a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f58af0711)
    },
    {   UINT64_C(0x0000000000005a69), UINT64_C(0x1f42f78f00b0e6fb), UINT64_C(0x1f42fb8eccb6e6f3),
        UINT64_C(0x1f42f90e70b8e70f), UINT64_C(0x1f42fa8eecadc70c)
    },
    {   UINT64_C(0x000000000000547e), UINT64_C(0x1f43114fd8abe6f9), UINT64_C(0x1f430f9058ade6a2),
        UINT64_C(0x1f431f4f68a62720), UINT64_C(0x1f43078fa4afe70c)
    },
    {   UINT64_C(0x0000000000005a6b), UINT64_C(0x1f42f98f28b4870a), UINT64_C(0x1f42f5cee4b40725),
        UINT64_C(0x1f430c0e98b0c732), UINT64_C(0x1f42f6cf40b28708)
    },
    {   UINT64_C(0x0000000000005a6c), UINT64_C(0x1f42fbcea4aea719), UINT64_C(0x1f42f50ef0b72721),
        UINT64_C(0x1f42f60f78b7a72b), UINT64_C(0x1f42f98f58b64704)
    },
    {   UINT64_C(0x0000000000005a6d), UINT64_C(0x1f42f60ee4ae46fb), UINT64_C(0x1f42fb8e4cb1671f),
        UINT64_C(0x1f42f98efcb2e70e), UINT64_C(0x1f42f7cf08afe71c)
    },
    {   UINT64_C(0x0000000000008124), UINT64_C(0x11daf2cfa082870b), UINT64_C(0x0c52f24fa06ee616),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a6f), UINT64_C(0x1f42f6cefcafc717), UINT64_C(0x1f42f5ceecae670f),
        UINT64_C(0x1f42f88e84b346f3), UINT64_C(0x1f42f98f48ac8717)
    },
    {   UINT64_C(0x0000000000005481), UINT64_C(0x1f43004fdcb286c8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cf3cb74724), UINT64_C(0x1f42fbd03cb026f6)
    },
    {   UINT64_C(0x0000000000005a71), UINT64_C(0x1f42f6cebcafe703), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98eecaba711), UINT64_C(0x1f42f6ced0b0c71a)
    },
    {   UINT64_C(0x0000000000005482), UINT64_C(0x1f43000ef4a6a715), UINT64_C(0x1f430b8ed4aa6713),
        UINT64_C(0x1f42fb4eeca7671a), UINT64_C(0x1f4306cee0af2717)
    },
    {   UINT64_C(0x0000000000005a73), UINT64_C(0x1f42f5cf78b7c714), UINT64_C(0x1f42f8cf68ba4731),
        UINT64_C(0x1f42fdcf2cb2672c), UINT64_C(0x1f42f98e9cb5472a)
    },
    {   UINT64_C(0x0000000000005a74), UINT64_C(0x1f42f60f68b7071b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf00b0070c)
    },
    {   UINT64_C(0x0000000000005a75), UINT64_C(0x1f42fa8e60b5c713), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005484), UINT64_C(0x1f42f8cea4a76704), UINT64_C(0x1f42facec8a98709),
        UINT64_C(0x1f42fe8e74a96718), UINT64_C(0x1f43034ed8acc6f0)
    },
    {   UINT64_C(0x0000000000005a77), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbced4b4c739),
        UINT64_C(0x1f42f9cf30b5a71a), UINT64_C(0x1f42f88f38b0670c)
    },
    {   UINT64_C(0x0000000000005485), UINT64_C(0x1f4302ce68b12711), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43000eb8a1c720), UINT64_C(0x1f4304ce68a90711)
    },
    {   UINT64_C(0x0000000000005a79), UINT64_C(0x1f42f6ce94b68704), UINT64_C(0x1f42f7ce9cb5671d),
        UINT64_C(0x1f42f7ce84b58729), UINT64_C(0x1f430a8e5cb4271d)
    },
    {   UINT64_C(0x0000000000005a7a), UINT64_C(0x1f42f6ce6cb04722), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8eb0b46722)
    },
    {   UINT64_C(0x0000000000005a7b), UINT64_C(0x1f42f6cf98b6c6f0), UINT64_C(0x1f42f8cf24b36715),
        UINT64_C(0x1f42f5ce70b6c721), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005487), UINT64_C(0x1f430d0ec0aa86f1), UINT64_C(0x1f42fd4fb4ae6719),
        UINT64_C(0x1f4301cec4a94709), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a7d), UINT64_C(0x1f42f5cefcb1e719), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f40eccad2700), UINT64_C(0x1f430e8eacb1c716)
    },
    {   UINT64_C(0x0000000000005a7e), UINT64_C(0x1f42f6cef8b82704), UINT64_C(0x1f42fa8e60a70713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f20f98b72713)
    },
    {   UINT64_C(0x0000000000005a7f), UINT64_C(0x1f430b0e40b0c738), UINT64_C(0x1f42f78e94b0a733),
        UINT64_C(0x1f42f6cf00b64707), UINT64_C(0x1f42fb8f78b1472f)
    },
    {   UINT64_C(0x0000000000005489), UINT64_C(0x1f42fc4e94a84718), UINT64_C(0x1f42fc4e74a92713),
        UINT64_C(0x1f4306cef0af0704), UINT64_C(0x1f42fc505cb486ff)
    },
    {   UINT64_C(0x0000000000005a81), UINT64_C(0x1f42f5cee8b64703), UINT64_C(0x1f42f7cf08b6a6ff),
        UINT64_C(0x1f42fa8ee8b1a71f), UINT64_C(0x1f42f60ee4b66703)
    },
    {   UINT64_C(0x000000000000548a), UINT64_C(0x1f430f8ee8aa0719), UINT64_C(0x1f42fb8edca7c712),
        UINT64_C(0x1f42fbcf3caa8708), UINT64_C(0x1f4308cec0a766fc)
    },
    {   UINT64_C(0x0000000000005a83), UINT64_C(0x1f42ff8edcb2c714), UINT64_C(0x1f42f7cea0b56731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98eecac8710)
    },
    {   UINT64_C(0x000000000000548b), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cf0ca7a6f5),
        UINT64_C(0x1f42faceb8a98711), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a85), UINT64_C(0x1f42f7ce6cb8e72b), UINT64_C(0x1f42f6cf2cb72704),
        UINT64_C(0x1f43080e4cb6e73e), UINT64_C(0x1f42f60ef0af0716)
    },
    {   UINT64_C(0x0000000000005a86), UINT64_C(0x1f42f98f18b2c71d), UINT64_C(0x1f42f7cea4b18717),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce48b7a712)
    },
    {   UINT64_C(0x0000000000005a87), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5cf34b0470f),
        UINT64_C(0x1f42fa8f28afc714), UINT64_C(0x1f42f88e4cb36740)
    },
    {   UINT64_C(0x0000000000005a88), UINT64_C(0x1f42f78edcb22726), UINT64_C(0x1f42fbcf88b54727),
        UINT64_C(0x1f4300ce38b24716), UINT64_C(0x1f42fa8eccb5c740)
    },
    {   UINT64_C(0x0000000000005a89), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ceccb42718),
        UINT64_C(0x1f42f60ebcb4c71e), UINT64_C(0x1f42f6cefcb906f7)
    },
    {   UINT64_C(0x000000000000548e), UINT64_C(0x1f43054ec8b80739), UINT64_C(0x1f42fdcef8a66710),
        UINT64_C(0x1f42fc8ed8a8071b), UINT64_C(0x1f42fb8eaca8a716)
    },
    {   UINT64_C(0x0000000000005a8b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0f94b58727), UINT64_C(0x1f42fc8f10b0e715)
    },
    {   UINT64_C(0x0000000000005a8c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf00b1470c),
        UINT64_C(0x1f42fccf10b86716), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a8d), UINT64_C(0x1f42f6cea8b226fb), UINT64_C(0x1f42f60f2cb34718),
        UINT64_C(0x1f42f88ef8b64715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005490), UINT64_C(0x1f43184f18a70713), UINT64_C(0x1f42fe8e94a866ed),
        UINT64_C(0x1f43084edca626fe), UINT64_C(0x1f42fb4f00a8c72f)
    },
    {   UINT64_C(0x0000000000005a8f), UINT64_C(0x1f42fa8ebcb36707), UINT64_C(0x1f42f7ce44b6274f),
        UINT64_C(0x1f42f6cf68b4871a), UINT64_C(0x1f42f88e4cb36710)
    },
    {   UINT64_C(0x0000000000005a90), UINT64_C(0x1f42f2ceecb2c708), UINT64_C(0x1f42f88e7cb10708),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a91), UINT64_C(0x1f42f8cf58b786ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084e9cb44730), UINT64_C(0x1f42f50f0cb4e720)
    },
    {   UINT64_C(0x0000000000005a92), UINT64_C(0x1f42f3cee8b58735), UINT64_C(0x1f42f6ce2cb26708),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf08b8072f)
    },
    {   UINT64_C(0x0000000000005a93), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ce48b34708),
        UINT64_C(0x1f42f8cf18b7a704), UINT64_C(0x1f42fd8eecb4e726)
    },
    {   UINT64_C(0x0000000000005a94), UINT64_C(0x1f430d8e40b28740), UINT64_C(0x1f42f98e78b80710),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a95), UINT64_C(0x1f42f98f08b726f3), UINT64_C(0x1f42f98e64b6a738),
        UINT64_C(0x1f42f98e8cb4c713), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a96), UINT64_C(0x1f42f98e48ac0717), UINT64_C(0x1f42f6ceecb786fc),
        UINT64_C(0x1f42f6ce78b3c729), UINT64_C(0x1f42f24f68b5a714)
    },
    {   UINT64_C(0x0000000000005a97), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98e4cb1e71f),
        UINT64_C(0x1f42fe8eecb2a6fa), UINT64_C(0x1f42fece88b4c72f)
    },
    {   UINT64_C(0x0000000000005495), UINT64_C(0x1f43080f00a7e6f1), UINT64_C(0x1f42fd4ea4ada701),
        UINT64_C(0x1f43000ef4aa6701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a99), UINT64_C(0x1f430f8e88b30720), UINT64_C(0x1f430e4eacaf8722),
        UINT64_C(0x1f42f88eccb46719), UINT64_C(0x1f42f60eb0ae272e)
    },
    {   UINT64_C(0x0000000000005496), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84eacae473f), UINT64_C(0x1f430a4efcaa8704)
    },
    {   UINT64_C(0x0000000000005a9b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e5cad8724),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa8eecb2a6ef)
    },
    {   UINT64_C(0x0000000000005a9c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facefcb186f0),
        UINT64_C(0x1f42f98eb0b5a72a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005a9d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8ee0b526f4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005498), UINT64_C(0x0000000000000000), UINT64_C(0x1f430e4f40a506f5),
        UINT64_C(0x1f43018ec4ae4710), UINT64_C(0x1f43038efcb0c716)
    },
    {   UINT64_C(0x0000000000004804), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffd50cb2c582)
    },
    {   UINT64_C(0x0000000000004805), UINT64_C(0x1f42ff954cb2a572), UINT64_C(0x1f4305d4ccafa563),
        UINT64_C(0x1f433ecf24934718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004806), UINT64_C(0x1f4396d968a2273c), UINT64_C(0x1f43195940642737),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004807), UINT64_C(0x1f437f1900b0a720), UINT64_C(0x1f4303593872e735),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004808), UINT64_C(0x1f43344e289866fe), UINT64_C(0x1f43354e6497e6fe),
        UINT64_C(0x1f4357d45486c5b0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004809), UINT64_C(0x0000000000000000), UINT64_C(0x1f431e5494a445a2),
        UINT64_C(0x1f43089494af65a2), UINT64_C(0x1f4357d45486c5b0)
    },
    {   UINT64_C(0x0000000000005aa5), UINT64_C(0x1f42f9ceb4b28722), UINT64_C(0x1f43050ebcb04724),
        UINT64_C(0x1f42f4cf2cb86704), UINT64_C(0x1f43024e6cb26728)
    },
    {   UINT64_C(0x0000000000005aa6), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98ec8b46705), UINT64_C(0x1f42f98f5cb546f8)
    },
    {   UINT64_C(0x000000000000480c), UINT64_C(0x1f4311d0a0a9c698), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43019ab8722745), UINT64_C(0x1f42fd18806de6fb)
    },
    {   UINT64_C(0x0000000000005aa8), UINT64_C(0x1f43100f08ad8716), UINT64_C(0x1f42f60e1cb06701),
        UINT64_C(0x1f42f40f28b7a71f), UINT64_C(0x1f42fe8ec4b6a740)
    },
    {   UINT64_C(0x000000000000480e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43151010a706be)
    },
    {   UINT64_C(0x000000000000480f), UINT64_C(0x1f433c0fa0948698), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ee1c50bb2757), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004810), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8dacb2c715), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005aac), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ce8cb02736)
    },
    {   UINT64_C(0x0000000000005aad), UINT64_C(0x1f42face74b6a71d), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6ce74b10710), UINT64_C(0x1f42f70ef4b10700)
    },
    {   UINT64_C(0x0000000000005aae), UINT64_C(0x1f42f60e78b24732), UINT64_C(0x1f42f40eecb6a6f8),
        UINT64_C(0x1f42f60e48b8272c), UINT64_C(0x1f42f6cf28b6070b)
    },
    {   UINT64_C(0x0000000000005aaf), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f28b0a714),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ab0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facf34b52721),
        UINT64_C(0x1f42fb8ed8b2e72b), UINT64_C(0x1f42f8cf38b28717)
    },
    {   UINT64_C(0x0000000000005ab1), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60e8cb70719),
        UINT64_C(0x1f42f1cedcb846fc), UINT64_C(0x1f42f7cef8b206ff)
    },
    {   UINT64_C(0x0000000000005ab2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8ceecad272f)
    },
    {   UINT64_C(0x0000000000005ab3), UINT64_C(0x1f42f8cec8b006f3), UINT64_C(0x1f42f88eb0ae06f7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60ed0b486fb)
    },
    {   UINT64_C(0x00000000000054a3), UINT64_C(0x1f42ff4ea4ad471a), UINT64_C(0x1f42f9cef4a76716),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ab6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccec0ad86fd),
        UINT64_C(0x1f42f8cf18ad6723), UINT64_C(0x1f42f6ce9cae072b)
    },
    {   UINT64_C(0x0000000000005ab7), UINT64_C(0x1f430ece8cb5e735), UINT64_C(0x1f42faceb0ab073d),
        UINT64_C(0x1f42f60f4cb8270c), UINT64_C(0x1f42f60edcb6c720)
    },
    {   UINT64_C(0x0000000000005ab8), UINT64_C(0x1f42f6cf54b4a706), UINT64_C(0x1f42f50eccb08714),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ab9), UINT64_C(0x0000000000000000), UINT64_C(0x1f42facee4b16712),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cf2cafc70f)
    },
    {   UINT64_C(0x0000000000005aba), UINT64_C(0x1f42facf60b02721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f58b6e715), UINT64_C(0x1f43154edcb0470e)
    },
    {   UINT64_C(0x0000000000005abb), UINT64_C(0x1f42f98fa8b6870f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98edcaee703), UINT64_C(0x1f42f78eccb7c70c)
    },
    {   UINT64_C(0x00000000000054a7), UINT64_C(0x0000000000000000), UINT64_C(0x1f43048fe0ac0711),
        UINT64_C(0x1f430bcf98af86ec), UINT64_C(0x1f43010f70ad6713)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054a8), UINT64_C(0x1f42f90fc4b2a6e8), UINT64_C(0x1f43148ee4b40728),
        UINT64_C(0x1f42f85000ace704), UINT64_C(0x1f43034ee0abe72e)
    },
    {   UINT64_C(0x0000000000005abf), UINT64_C(0x1f42f88ee0b7670b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ac0), UINT64_C(0x1f42fcceb0b5e729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e60b1e723), UINT64_C(0x1f42f78e54b6e73e)
    },
    {   UINT64_C(0x0000000000005ac1), UINT64_C(0x1f42f98eccac071b), UINT64_C(0x1f42f88ec8b66724),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054aa), UINT64_C(0x1f43049010adc6f7), UINT64_C(0x1f42ff0fb8b00703),
        UINT64_C(0x1f42fdceb8b4072e), UINT64_C(0x1f42f6cf84adc6ff)
    },
    {   UINT64_C(0x0000000000005ac3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f6ceb8b54704)
    },
    {   UINT64_C(0x0000000000005ac4), UINT64_C(0x1f42f78ec4b62732), UINT64_C(0x1f42f88e64b54734),
        UINT64_C(0x1f43068e4cb3c735), UINT64_C(0x1f42f9ce7cb50735)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000482b), UINT64_C(0x1f43234f849ac687), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054ad), UINT64_C(0x1f43174f00ac26fc), UINT64_C(0x1f42fd4f1cb3e739),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ac9), UINT64_C(0x1f42fe9000b0e701), UINT64_C(0x1f43088fc4b046ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43074facb3c6f9)
    },
    {   UINT64_C(0x00000000000054ae), UINT64_C(0x0000000000000000), UINT64_C(0x1f4306cf50af2710),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cf50ab2710)
    },
    {   UINT64_C(0x0000000000005acb), UINT64_C(0x1f4302cf98b7a6f7), UINT64_C(0x1f43020f7cb726ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43064f28b726f5)
    },
    {   UINT64_C(0x00000000000054af), UINT64_C(0x1f42fb8facb08715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa4fecaec70a), UINT64_C(0x1f43088eecad8712)
    },
    {   UINT64_C(0x0000000000005acd), UINT64_C(0x1f43080f64b7a70e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cedcb5c707), UINT64_C(0x1f42fc8edcb24703)
    },
    {   UINT64_C(0x00000000000054b0), UINT64_C(0x1f42fe8f74ad470e), UINT64_C(0x1f43074f70b1e6f3),
        UINT64_C(0x1f43064f84b0471d), UINT64_C(0x1f43004ee4af6738)
    },
    {   UINT64_C(0x0000000000005acf), UINT64_C(0x1f42fc4fd4b34719), UINT64_C(0x1f42fad018b7a711),
        UINT64_C(0x1f42ffcf94b5c71e), UINT64_C(0x1f4305cfd0b4c700)
    },
    {   UINT64_C(0x0000000000005ad0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcf4cb82704),
        UINT64_C(0x1f43098fd8b2c6f8), UINT64_C(0x1f42fa0fdcb72705)
    },
    {   UINT64_C(0x0000000000005ad1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43080fd0b5c6e7),
        UINT64_C(0x1f43058fecb6c70e), UINT64_C(0x1f43050f50b6c712)
    },
    {   UINT64_C(0x00000000000054b2), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffcf0cb74736),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d8f54b0c6f5)
    },
    {   UINT64_C(0x000000000001a9c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f0af04bfc73e693)
    },
    {   UINT64_C(0x0000000000005ad4), UINT64_C(0x1f43038f98b3c6f6), UINT64_C(0x1f42fe8ee4b8471a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88f78b6471e)
    },
    {   UINT64_C(0x0000000000010908), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2f80dc07803f8)
    },
    {   UINT64_C(0x00000000000054b4), UINT64_C(0x1f43000facabc706), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8f18ae872c), UINT64_C(0x1f43094ea4b0070c)
    },
    {   UINT64_C(0x0000000000005ad7), UINT64_C(0x1f42fd0f38b546f9), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030f18b1c6f6), UINT64_C(0x1f42f80f60b3c6f2)
    },
    {   UINT64_C(0x0000000000005ad8), UINT64_C(0x1f4307cf60b626f0), UINT64_C(0x1f42f8ce6cb6a71e),
        UINT64_C(0x1f4301d030b546e2), UINT64_C(0x1f42fed080b246d1)
    },
    {   UINT64_C(0x0000000000005ad9), UINT64_C(0x1f4303d070b68704), UINT64_C(0x1f430a4e50b12740),
        UINT64_C(0x1f430dce3cb5673c), UINT64_C(0x1f42f94f24b8a719)
    },
    {   UINT64_C(0x0000000000005ada), UINT64_C(0x0000000000000000), UINT64_C(0x1f43068e4caf2740),
        UINT64_C(0x1f43030f80b246f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd5050aa473c)
    },
    {   UINT64_C(0x0000000000005adc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd8e50b34734), UINT64_C(0x1f42fe0e50b38734)
    },
    {   UINT64_C(0x0000000000004842), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd5080b2e73c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c009), UINT64_C(0x0fa2f54e147986d6), UINT64_C(0x176afd4f2c9486aa),
        UINT64_C(0x1a0afd8f60a1269b), UINT64_C(0x08b300d5f058069b)
    },
    {   UINT64_C(0x0000000000005adf), UINT64_C(0x1f43094e8cb4e6d8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4303cf08ae66de), UINT64_C(0x1f4307cf94b9870d)
    },
    {   UINT64_C(0x0000000000004845), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430790c8ae868e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ae1), UINT64_C(0x1f43000e74b846fa), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4300cfacae86fa), UINT64_C(0x1f43030f04aca708)
    },
    {   UINT64_C(0x00000000000054ba), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ffceb4b3071a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068f44b2e70a)
    },
    {   UINT64_C(0x0000000000004848), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f78fecb6c6bb),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005ae4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43018f14b3e705),
        UINT64_C(0x1f43050f7ca72702), UINT64_C(0x1f43038fecb726e8)
    },
    {   UINT64_C(0x000000000000484a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430e8fecab4725)
    },
    {   UINT64_C(0x0000000000005ae6), UINT64_C(0x1f43078ef4b5672e), UINT64_C(0x1f4300ced4aea709),
        UINT64_C(0x1f43000ed8b3670d), UINT64_C(0x1f43019000b246d7)
    },
    {   UINT64_C(0x0000000000005ae7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43030f30a7c716), UINT64_C(0x1f43001040b2a6d2)
    },
    {   UINT64_C(0x00000000000054bd), UINT64_C(0x1f42f74ea8b4e739), UINT64_C(0x1f42fa4f3cb48703),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0ef4ada72c)
    },
    {   UINT64_C(0x0000000000005ae9), UINT64_C(0x1f42fbce70af8733), UINT64_C(0x1f42fa0f08b44711),
        UINT64_C(0x1f43018e90b2871c), UINT64_C(0x1f42facfd0b6e6c7)
    },
    {   UINT64_C(0x000000000000c00f), UINT64_C(0x1773020f409245fe), UINT64_C(0x1772ff114c93c577),
        UINT64_C(0x1773020f409245fe), UINT64_C(0x0fa2fc4f6c762699)
    },
    {   UINT64_C(0x0000000000005aeb), UINT64_C(0x0000000000000000), UINT64_C(0x1f43020f60b066f2),
        UINT64_C(0x1f42fa4f48b7c71f), UINT64_C(0x1f42f9ceecaf6731)
    },
    {   UINT64_C(0x00000000000054bf), UINT64_C(0x1f430a4e50a7c727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431bce78a72726), UINT64_C(0x1f43138f1ca9671a)
    },
    {   UINT64_C(0x0000000000005aed), UINT64_C(0x1f4307cfa0b266d4), UINT64_C(0x1f42f74f94b7a718),
        UINT64_C(0x1f43008f4cb546e3), UINT64_C(0x1f43004fbcb62711)
    },
    {   UINT64_C(0x000000000001a9e4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x16f2f28f6896068c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005aef), UINT64_C(0x1f43038f68b70726), UINT64_C(0x1f42f60fbcb7e70b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f98b626e3)
    },
    {   UINT64_C(0x0000000000005af0), UINT64_C(0x1f43028f10b626ea), UINT64_C(0x1f43070facb62701),
        UINT64_C(0x1f4301cf38b7a716), UINT64_C(0x1f43068f7cb58712)
    },
    {   UINT64_C(0x0000000000005af1), UINT64_C(0x1f42fe8f54b38714), UINT64_C(0x1f43004ed4b6a715),
        UINT64_C(0x1f43028fb0b4c6db), UINT64_C(0x1f4304cebcb5c6f3)
    },
    {   UINT64_C(0x00000000000054c2), UINT64_C(0x1f43194ea8aea735), UINT64_C(0x1f43124e9caa271a),
        UINT64_C(0x1f43170e54aa472b), UINT64_C(0x1f43180eb4a9a704)
    },
    {   UINT64_C(0x0000000000004858), UINT64_C(0x1f434590148ea6c8), UINT64_C(0x1f431cd0149f66c8),
        UINT64_C(0x1f430e5014a6e6c8), UINT64_C(0x1f42ff1014b2e6c8)
    },
    {   UINT64_C(0x0000000000004859), UINT64_C(0x1f430b101cabc6c8), UINT64_C(0x1f42ff1014b306c8),
        UINT64_C(0x1f42f8d014b546c8), UINT64_C(0x1f42f69014b786c8)
    },
    {   UINT64_C(0x000000000000485a), UINT64_C(0x1f42fd9014b2a6c8), UINT64_C(0x1f43179014aa46c8),
        UINT64_C(0x1f431110c8a5a68e), UINT64_C(0x1f430750c8aac68e)
    },
    {   UINT64_C(0x00000000000054c4), UINT64_C(0x1f43188e7ca9272e), UINT64_C(0x1f42fece3ca9c72a),
        UINT64_C(0x1f430b4e58aac729), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000485c), UINT64_C(0x1f43520fd88986ca), UINT64_C(0x1f43324fd89966ca),
        UINT64_C(0x1f43148fd8a846ca), UINT64_C(0x1f43160f7ca786ca)
    },
    {   UINT64_C(0x000000000000485d), UINT64_C(0x1f4329cf7c9da5d4), UINT64_C(0x1f430e0f7cab86ca),
        UINT64_C(0x1f42facf7cb526ca), UINT64_C(0x1f42f54f7cb7e6ca)
    },
    {   UINT64_C(0x000000000000485e), UINT64_C(0x1f4306cfd8af26ca), UINT64_C(0x1f4324cfd8a025d4),
        UINT64_C(0x1f4305cfdcafa6cb), UINT64_C(0x1f42f98fd8b5c6ca)
    },
    {   UINT64_C(0x00000000000054c6), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fecf1cb56706),
        UINT64_C(0x1f4318cf50ad06cb), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001092e), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000005afc), UINT64_C(0x1f42fd8f2cb24702), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4f9cb3c703), UINT64_C(0x1f42f9ceb8b3e740)
    },
    {   UINT64_C(0x0000000000005afd), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe8f68b42713),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d0f6cb5a6c7)
    },
    {   UINT64_C(0x00000000000054c8), UINT64_C(0x1f431a0f74ac46ec), UINT64_C(0x1f43190fe4adc6eb),
        UINT64_C(0x1f43148f54af86f8), UINT64_C(0x1f43188f70aca6fd)
    },
    {   UINT64_C(0x0000000000004864), UINT64_C(0x1f42ef1324bb85fa), UINT64_C(0x1f43464db8908754),
        UINT64_C(0x1f42ee1320bae5fb), UINT64_C(0x1f43468dc490a757)
    },
    {   UINT64_C(0x0000000000004865), UINT64_C(0x1f42fc92b4b44622), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757)
    },
    {   UINT64_C(0x0000000000004866), UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010935), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010936), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010937), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010938), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x0000000000010939), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c)
    },
    {   UINT64_C(0x000000000001093a), UINT64_C(0x1562ee0f6894068c), UINT64_C(0x1562ee0f6894068c),
        UINT64_C(0x1562ee0f6894068c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b08), UINT64_C(0x1f4313ce5cb7472c), UINT64_C(0x1f4313ce7cb5873a),
        UINT64_C(0x1f430f4eccb5e723), UINT64_C(0x1f430c8f08b9e738)
    },
    {   UINT64_C(0x000000000000486e), UINT64_C(0x1f430210f4b186c3), UINT64_C(0x1f43028fa4b14719),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c01f), UINT64_C(0x06e2fe8f5451c69b), UINT64_C(0x0c52fb8ce46d06a2),
        UINT64_C(0x1042f552547be535), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b0b), UINT64_C(0x1f43098eacb3e732), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108e70b2a736), UINT64_C(0x1f430a0e94b88739)
    },
    {   UINT64_C(0x0000000000004871), UINT64_C(0x1f42f5d058b7a659), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010940), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x16233d4ae46f4776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004873), UINT64_C(0x1f430950e0ade65a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004874), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f550d0b7e664), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004875), UINT64_C(0x1f42f4d0d0b82664), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010944), UINT64_C(0x1622eb4ae46f453c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b12), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4ff4b4a6df),
        UINT64_C(0x1f43088fb8b646cb), UINT64_C(0x1f43068f40aa46f4)
    },
    {   UINT64_C(0x0000000000005b13), UINT64_C(0x1f430a4e6caa8713), UINT64_C(0x1f4304cf40b706ff),
        UINT64_C(0x1f430c8fdcb7c6d3), UINT64_C(0x1f43088f98b4a70a)
    },
    {   UINT64_C(0x0000000000004879), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431ad114a5263a), UINT64_C(0x1f432c90589c4698)
    },
    {   UINT64_C(0x000000000000487a), UINT64_C(0x1f431ad114a5263a), UINT64_C(0x1f432c8fa09c4669),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000010949), UINT64_C(0x1622eb4ae46f4776), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b17), UINT64_C(0x1f43084f9cb126c3), UINT64_C(0x1f43068f4cb46717),
        UINT64_C(0x1f43020f00b6c71b), UINT64_C(0x1f430f9054b5c6de)
    },
    {   UINT64_C(0x00000000000054d5), UINT64_C(0x1f43020f68b0a726), UINT64_C(0x1f43068e8cae871b),
        UINT64_C(0x1f430d8f28af671f), UINT64_C(0x1f43114f08ad0732)
    },
    {   UINT64_C(0x0000000000005b19), UINT64_C(0x1f43144f40af06db), UINT64_C(0x1f43078f28b3070b),
        UINT64_C(0x1f43058f98b266ed), UINT64_C(0x1f43090e5cb366d4)
    },
    {   UINT64_C(0x0000000000005b1a), UINT64_C(0x1f430f4faca586ae), UINT64_C(0x1f430d4fd4a4c6db),
        UINT64_C(0x1f430e507cb446df), UINT64_C(0x1f43058eb8b3a715)
    },
    {   UINT64_C(0x0000000000004880), UINT64_C(0x1f42fd4e38b3e6f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43304e649a66fe), UINT64_C(0x1f431dcf60a3a6c0)
    },
    {   UINT64_C(0x0000000000004881), UINT64_C(0x0000000000000000), UINT64_C(0x1f432410b0a08663),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e5cb2c6df)
    },
    {   UINT64_C(0x0000000000004882), UINT64_C(0x1f433150509126f2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0f64b386c0)
    },
    {   UINT64_C(0x0000000000004883), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43131080a6c6f1)
    },
    {   UINT64_C(0x0000000000004884), UINT64_C(0x0000000000000000), UINT64_C(0x1f43169080a7e6f1),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4334511498660b)
    },
    {   UINT64_C(0x0000000000004885), UINT64_C(0x0000000000000000), UINT64_C(0x1f432cce389c0739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f432c51249c66a7)
    },
    {   UINT64_C(0x0000000000004886), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f1ce34bb8757), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004887), UINT64_C(0x0000000000000000), UINT64_C(0x1f43035244ada656),
        UINT64_C(0x1f42fc1220b2e63e), UINT64_C(0x1f43111080aa26f1)
    },
    {   UINT64_C(0x0000000000004888), UINT64_C(0x1f430ed044aa66cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43654dc8800757)
    },
    {   UINT64_C(0x0000000000004889), UINT64_C(0x0000000000000000), UINT64_C(0x1f434b4dc88ce757),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f431010c8ada68e)
    },
    {   UINT64_C(0x0000000000005b25), UINT64_C(0x1f43028f00ab66e6), UINT64_C(0x1f43058ed8b0c6d5),
        UINT64_C(0x1f43094ea8abe728), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000488b), UINT64_C(0x1f4323ce14a0a74b), UINT64_C(0x1f43038e08b0c748),
        UINT64_C(0x1f43394f00982714), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b27), UINT64_C(0x1f43074e94a606ee), UINT64_C(0x1f42f60eecb7a6ec),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4f6ca9e70b)
    },
    {   UINT64_C(0x000000000000488d), UINT64_C(0x1f43115004ab66a9), UINT64_C(0x1f43115004ab66a9),
        UINT64_C(0x1f435e9194836666), UINT64_C(0x1f432010d8a2a68f)
    },
    {   UINT64_C(0x0000000000005b29), UINT64_C(0x1f43044f38ac26da), UINT64_C(0x1f430a4fa0ab66cf),
        UINT64_C(0x1f430a8fa4b72703), UINT64_C(0x1f4308cfacb64702)
    },
    {   UINT64_C(0x000000000000488f), UINT64_C(0x1f42f953a0b5e5d1), UINT64_C(0x1f42fa1298b4c633),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b2b), UINT64_C(0x1f43048e9cb8e723), UINT64_C(0x1f43008eccb0c704),
        UINT64_C(0x1f43094fb0b146d0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054df), UINT64_C(0x1f43024fc8b0c70b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43094e5cb32731)
    },
    {   UINT64_C(0x0000000000004892), UINT64_C(0x1f42f95124b5e69b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004893), UINT64_C(0x1f431b10c8a48698), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b2f), UINT64_C(0x1f43104fe8a9e6df), UINT64_C(0x1f430dcfd0b08702),
        UINT64_C(0x1f43100f2caa0708), UINT64_C(0x1f430a4f68afe6d5)
    },
    {   UINT64_C(0x0000000000005b30), UINT64_C(0x1f430c8ea0a9c6d0), UINT64_C(0x1f430e5038b866d3),
        UINT64_C(0x1f43058ee4ab26c3), UINT64_C(0x1f430b4f34a86705)
    },
    {   UINT64_C(0x0000000000005b31), UINT64_C(0x1f43075038b5c6d0), UINT64_C(0x1f43038fe8a7c6af),
        UINT64_C(0x1f43094f68b546fc), UINT64_C(0x1f43029008af86da)
    },
    {   UINT64_C(0x000000000000c033), UINT64_C(0x0c2af30fa06cc7ab), UINT64_C(0x0c2af30fa06cc7ab),
        UINT64_C(0x0c2af30fa06cc7a2), UINT64_C(0x0c2af30fa06cc77c)
    },
    {   UINT64_C(0x0000000000004898), UINT64_C(0x1f431fd100a3c686), UINT64_C(0x1f430fd278a88628),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004899), UINT64_C(0x1f432c90d09c468e), UINT64_C(0x1f432c90d09c468e),
        UINT64_C(0x1f4313d01caac6ed), UINT64_C(0x1f430fd01ca886ed)
    },
    {   UINT64_C(0x000000000000489a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc4e3cb46707), UINT64_C(0x1f42fc4e3cb46707)
    },
    {   UINT64_C(0x000000000000489b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431dd0d49d4695), UINT64_C(0x1f4329d0d4a34695)
    },
    {   UINT64_C(0x000000000000489c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4308d068ae2702), UINT64_C(0x1f43089074ae6702)
    },
    {   UINT64_C(0x00000000000054e5), UINT64_C(0x1f43080fa0ae66dd), UINT64_C(0x1f430bcf30ac86e4),
        UINT64_C(0x1f42fb8f94b6e704), UINT64_C(0x1f43034f28a926f8)
    },
    {   UINT64_C(0x0000000000005b39), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4f94b4e6e2),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43094ec4b186fd)
    },
    {   UINT64_C(0x0000000000005b3a), UINT64_C(0x1f430f5018b4c6fc), UINT64_C(0x1f430acfa4b626f4),
        UINT64_C(0x1f430a4fd4a326d3), UINT64_C(0x1f43088fe4aba6df)
    },
    {   UINT64_C(0x00000000000048a0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0fe0b386a1), UINT64_C(0x1f42fe0fe0b386a1)
    },
    {   UINT64_C(0x00000000000048a1), UINT64_C(0x1f43248fa0a04698), UINT64_C(0x1f43248fa0a04698),
        UINT64_C(0x1f42fe0f60b386e1), UINT64_C(0x1f42fd8f60b3c6e1)
    },
    {   UINT64_C(0x0000000000005b3d), UINT64_C(0x0000000000000000), UINT64_C(0x1f43078eecb4c70c),
        UINT64_C(0x1f43140f40b4a713), UINT64_C(0x1f43088fbcb5e707)
    },
    {   UINT64_C(0x00000000000054e8), UINT64_C(0x1f430bcf30ac86e4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f38a726e8), UINT64_C(0x1f42f4ce5cad8704)
    },
    {   UINT64_C(0x0000000000005b3f), UINT64_C(0x1f43090fd8b2670f), UINT64_C(0x1f43064e6cb546eb),
        UINT64_C(0x1f43094facad66ca), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000048a5), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f8fd8aaa6d1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000048a6), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f8fd8aaa6d1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054ea), UINT64_C(0x1f42fb8f18afa6dd), UINT64_C(0x1f42fc4fd0b586e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000048a8), UINT64_C(0x1f4315cfd0a7a6a2), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000048a9), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c90b0a966dc),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b45), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d4f78b0e6d6), UINT64_C(0x1f43094f74b74715)
    },
    {   UINT64_C(0x00000000000054ec), UINT64_C(0x1f42f88f74b646f4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8e68ac8713), UINT64_C(0x1f42f60e8cb76720)
    },
    {   UINT64_C(0x0000000000005b47), UINT64_C(0x0000000000000000), UINT64_C(0x1f430b4f58b2e702),
        UINT64_C(0x1f43094facab66cf), UINT64_C(0x1f43068f20b3c726)
    },
    {   UINT64_C(0x00000000000054ed), UINT64_C(0x1f42fb8e40a6871a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4314d0f4a6068f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b49), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4e4cb46735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43058eb0ad2722)
    },
    {   UINT64_C(0x00000000000048af), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43111080aa26f1)
    },
    {   UINT64_C(0x0000000000005b4b), UINT64_C(0x1f42fdcf50b7c6cb), UINT64_C(0x1f4306cf78b166c4),
        UINT64_C(0x1f43028fc0b226e0), UINT64_C(0x1f43160eecb0e6ed)
    },
    {   UINT64_C(0x00000000000054ef), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43068f68b0e6e7)
    },
    {   UINT64_C(0x0000000000005b4d), UINT64_C(0x1f43048e78b20716), UINT64_C(0x1f43088f20a96737),
        UINT64_C(0x1f43094f44ace721), UINT64_C(0x1f43088f98ad86b8)
    },
    {   UINT64_C(0x00000000000054f0), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038f98b8e71d),
        UINT64_C(0x1f43078eecb46727), UINT64_C(0x1f43064e50b5a746)
    },
    {   UINT64_C(0x0000000000005b4f), UINT64_C(0x1f430b0f80ae86eb), UINT64_C(0x1f43070f3cb7a6ea),
        UINT64_C(0x1f430c4f7cb9a704), UINT64_C(0x1f43004e78b5a728)
    },
    {   UINT64_C(0x0000000000005b50), UINT64_C(0x1f43058f84ae46dc), UINT64_C(0x1f430d4fe8b72716),
        UINT64_C(0x1f43090fbcb46700), UINT64_C(0x1f43048fd4b34710)
    },
    {   UINT64_C(0x0000000000005b51), UINT64_C(0x1f43010f00b20726), UINT64_C(0x1f430c8f1caae706),
        UINT64_C(0x1f430e8eb4b2c713), UINT64_C(0x1f430a8f98b36704)
    },
    {   UINT64_C(0x0000000000005b52), UINT64_C(0x1f430c8f2ca3670e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43099028a6c6f5), UINT64_C(0x1f43094f40ab070c)
    },
    {   UINT64_C(0x0000000000005b53), UINT64_C(0x0000000000000000), UINT64_C(0x1f43078fc0b70700),
        UINT64_C(0x1f430f4edcb4670b), UINT64_C(0x1f43068ef0a7a6dd)
    },
    {   UINT64_C(0x00000000000054f3), UINT64_C(0x1f43019070b70670), UINT64_C(0x1f42fccfe0b6272c),
        UINT64_C(0x1f42ffcf58b2a6e7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c9f2), UINT64_C(0x200afd8f6cb4660b), UINT64_C(0x200afd8f6cb4660b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054f4), UINT64_C(0x0000000000000000), UINT64_C(0x1f43038fa0b22722),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019107), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x1162f44f6881468c),
        UINT64_C(0x0de2f44f6873e5c1), UINT64_C(0x0a6af18fa06f269b)
    },
    {   UINT64_C(0x0000000000019108), UINT64_C(0x0fa2f50c687ae5c0), UINT64_C(0x1162f1cfa081468c),
        UINT64_C(0x08b2f1cfa05e869b), UINT64_C(0x0a6adfcc745f669b)
    },
    {   UINT64_C(0x0000000000005b59), UINT64_C(0x1f43050f08b1070b), UINT64_C(0x1f430b4f78af26e6),
        UINT64_C(0x1f430c4eb4ae0719), UINT64_C(0x1f430b8fd8aea6e5)
    },
    {   UINT64_C(0x00000000000054f6), UINT64_C(0x1f431d4ef8ad871f), UINT64_C(0x1f42fe0f80b6a722),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa4f40b5a700)
    },
    {   UINT64_C(0x0000000000005b5b), UINT64_C(0x1f430dd00cb3a6df), UINT64_C(0x1f430a8f40b7a6f6),
        UINT64_C(0x1f430b4e9cb36734), UINT64_C(0x1f430b0f3cb006d0)
    },
    {   UINT64_C(0x00000000000054f7), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43164eb8b4c729)
    },
    {   UINT64_C(0x0000000000005b5d), UINT64_C(0x1f43048fb8b5a6e4), UINT64_C(0x1f4302cf30ae66e4),
        UINT64_C(0x1f43068ec8ad2712), UINT64_C(0x1f43098eb8b80726)
    },
    {   UINT64_C(0x00000000000054f8), UINT64_C(0x1f4312cf70aa671b), UINT64_C(0x1f42f58f7cb80726),
        UINT64_C(0x1f43080e5caec74c), UINT64_C(0x1f4323d1f8a7066c)
    },
    {   UINT64_C(0x00000000000048c4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a52f8ad6628), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b60), UINT64_C(0x1f43080f58b226e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084ed8b4e6e4), UINT64_C(0x1f43094f8caae6d0)
    },
    {   UINT64_C(0x0000000000019111), UINT64_C(0x186afa8d4099e683), UINT64_C(0x1722f78fa0992675),
        UINT64_C(0x1452fa8f78938684), UINT64_C(0x1132f44fa0924675)
    },
    {   UINT64_C(0x00000000000054fa), UINT64_C(0x1f4309d08caec6a1), UINT64_C(0x1f430e8f28ada6e1),
        UINT64_C(0x1f4307d078a706db), UINT64_C(0x1f43118ef4a60711)
    },
    {   UINT64_C(0x0000000000019113), UINT64_C(0x10baf6cfa0956675), UINT64_C(0x1822fd0fa0a2c675),
        UINT64_C(0x0fb2f80f787985a1), UINT64_C(0x0eb2f94f7476c684)
    },
    {   UINT64_C(0x00000000000054fb), UINT64_C(0x1f43120f38a206e6), UINT64_C(0x1f4300518cb846b7),
        UINT64_C(0x1f42ff8f24b3e69f), UINT64_C(0x1f43098f5cb206e8)
    },
    {   UINT64_C(0x000000000000c9fa), UINT64_C(0x0deaff4c9c7066c4), UINT64_C(0x0deaf98c9c6d86c4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000054fc), UINT64_C(0x1f431b0f68a5a732), UINT64_C(0x1f43085054b646d9),
        UINT64_C(0x1f43104ef0b4671a), UINT64_C(0x1f42fdcf58b10703)
    },
    {   UINT64_C(0x0000000000019117), UINT64_C(0x0e52f80f747545a1), UINT64_C(0x0f4ae68cc479c5a1),
        UINT64_C(0x0e52f80cc47865a1), UINT64_C(0x0cc2f58fa06fa5a1)
    },
    {   UINT64_C(0x000000000000480d), UINT64_C(0x0000000000000000), UINT64_C(0x1f437e1aacafc744),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019119), UINT64_C(0x183b000f7c9c05b4), UINT64_C(0x1132f58f7c7e45a1),
        UINT64_C(0x0f82f58cc47c85a0), UINT64_C(0x0f9af88f6878e5aa)
    },
    {   UINT64_C(0x000000000001911a), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x15eafbcf709ba695), UINT64_C(0x1992f88fa0a14685)
    },
    {   UINT64_C(0x000000000001911b), UINT64_C(0x16e2f88fa09cc685), UINT64_C(0x13daf90fa09c4685),
        UINT64_C(0x1802fbcf709e8695), UINT64_C(0x1bdaf88fa0b4c685)
    },
    {   UINT64_C(0x000000000001911c), UINT64_C(0x0fb2f88fa0842685), UINT64_C(0x1172fe8f708f4685),
        UINT64_C(0x18daf88fa0a76685), UINT64_C(0x1742f88fa0960685)
    },
    {   UINT64_C(0x000000000001911d), UINT64_C(0x1fc2f88fa0c46685), UINT64_C(0x1b2af88fa0b20685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1322f88fa09a2685)
    },
    {   UINT64_C(0x0000000000005500), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf8cb586fd),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302cf40b52723)
    },
    {   UINT64_C(0x0000000000005b6f), UINT64_C(0x1f430bcf00ae270f), UINT64_C(0x1f430bcf40ae6708),
        UINT64_C(0x1f430e4f0cb406e6), UINT64_C(0x1f43004f5cb8a6ec)
    },
    {   UINT64_C(0x0000000000019120), UINT64_C(0x112af7cfa0992685), UINT64_C(0x18eafe8fa0a64685),
        UINT64_C(0x122af90f788305b6), UINT64_C(0x0feafa8f807b6695)
    },
    {   UINT64_C(0x0000000000005b71), UINT64_C(0x1f430e8f90ace6f9), UINT64_C(0x1f43028e6cb1a732),
        UINT64_C(0x1f43084f1cae0739), UINT64_C(0x1f43054ebcb1671d)
    },
    {   UINT64_C(0x0000000000019122), UINT64_C(0x107af38c748065b6), UINT64_C(0x1202fa8f78826695),
        UINT64_C(0x0b2afbcf7c640654), UINT64_C(0x0d7ae98c64724654)
    },
    {   UINT64_C(0x0000000000005b73), UINT64_C(0x1f43078fa0ac46ff), UINT64_C(0x1f43084f70a5e6de),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4309cfe4af6700)
    },
    {   UINT64_C(0x0000000000005b74), UINT64_C(0x1f430f501ca986e9), UINT64_C(0x1f43048fd0b80705),
        UINT64_C(0x1f43074e8cb4c711), UINT64_C(0x1f4308cef0acc709)
    },
    {   UINT64_C(0x0000000000005b75), UINT64_C(0x1f430d8f44afc713), UINT64_C(0x1f430c9054ab4712),
        UINT64_C(0x1f430bcec8ae6711), UINT64_C(0x1f430a0f20a8e6ed)
    },
    {   UINT64_C(0x0000000000005b76), UINT64_C(0x1f43088ef4ad2738), UINT64_C(0x1f43048f08aac724),
        UINT64_C(0x1f4308cfc0b3471c), UINT64_C(0x1f43088f14aac6f8)
    },
    {   UINT64_C(0x0000000000005b77), UINT64_C(0x1f43090f90a8a6bc), UINT64_C(0x1f43051068b286c9),
        UINT64_C(0x1f43038f28b7a713), UINT64_C(0x1f43078f98aee6d5)
    },
    {   UINT64_C(0x0000000000005b78), UINT64_C(0x1f430b0ee8b146cb), UINT64_C(0x1f430dcf90b72704),
        UINT64_C(0x1f4307cf4cada6f3), UINT64_C(0x1f430a4f60b4e6fb)
    },
    {   UINT64_C(0x0000000000005b79), UINT64_C(0x1f43048fd0b8a6ee), UINT64_C(0x1f430b0f4cb2c6fd),
        UINT64_C(0x1f42fdcf90b64701), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b7a), UINT64_C(0x1f43084ed4b6c72e), UINT64_C(0x1f43080f20b36725),
        UINT64_C(0x1f4309cf24b6c708), UINT64_C(0x1f43080f94afc6ff)
    },
    {   UINT64_C(0x0000000000005b7b), UINT64_C(0x1f43018e5cafe72d), UINT64_C(0x1f43080f38b2c715),
        UINT64_C(0x1f4309cebcb1c72a), UINT64_C(0x1f43084ebcab270e)
    },
    {   UINT64_C(0x0000000000005b7c), UINT64_C(0x1f430b8fb0b046fe), UINT64_C(0x1f43094f44b986f3),
        UINT64_C(0x1f43074f60b1670a), UINT64_C(0x1f43060e64b1e736)
    },
    {   UINT64_C(0x0000000000005b7d), UINT64_C(0x1f4310cfecad86f9), UINT64_C(0x1f43084ed8ae671e),
        UINT64_C(0x1f43068e8cb1670e), UINT64_C(0x1f4308cfb0b606df)
    },
    {   UINT64_C(0x0000000000005508), UINT64_C(0x1f42fe0fb4b626f9), UINT64_C(0x1f43054fe8b6e6df),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd1214b26646)
    },
    {   UINT64_C(0x0000000000005b7f), UINT64_C(0x1f42f90ebcb4c730), UINT64_C(0x1f43070fc4af46fd),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b80), UINT64_C(0x1f4309ce7cb2a71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43088f88ace719)
    },
    {   UINT64_C(0x000000000000bec1), UINT64_C(0x19f2f30fa0a20693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005b82), UINT64_C(0x1f4305ce50afa70a), UINT64_C(0x1f430b8e60aca724),
        UINT64_C(0x1f43050ff4a8e6df), UINT64_C(0x1f430b9090acc724)
    },
    {   UINT64_C(0x0000000000005b83), UINT64_C(0x1f43030ee8b4c6f0), UINT64_C(0x1f43094eccb00731),
        UINT64_C(0x1f43084f14ad0722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c05c), UINT64_C(0x1692f20f54962783), UINT64_C(0x1162f20f6881c6b1),
        UINT64_C(0x1692f20c6c96268c), UINT64_C(0x1162f20c6c82e5b5)
    },
    {   UINT64_C(0x0000000000005b85), UINT64_C(0x1f430b903cad870f), UINT64_C(0x1f43064ef4af8723),
        UINT64_C(0x1f43088eacaba713), UINT64_C(0x1f430c0e74b6c722)
    },
    {   UINT64_C(0x0000000000005b86), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4fe0b60722),
        UINT64_C(0x1f43088f0cad2711), UINT64_C(0x1f430d4f04b6a717)
    },
    {   UINT64_C(0x0000000000005b87), UINT64_C(0x1f43094f88ac66f0), UINT64_C(0x1f43048ea4b8473d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43088e60b3c72e)
    },
    {   UINT64_C(0x0000000000005b88), UINT64_C(0x1f430c4f08b3c728), UINT64_C(0x1f43068f04b6671d),
        UINT64_C(0x1f43038fa0b206db), UINT64_C(0x1f43068f24b4e708)
    },
    {   UINT64_C(0x0000000000005b89), UINT64_C(0x1f4303cf04b2c71a), UINT64_C(0x1f430b0f90b0c6ef),
        UINT64_C(0x1f430c8f48ac86f3), UINT64_C(0x1f430c0e8cb2671d)
    },
    {   UINT64_C(0x0000000000005b8a), UINT64_C(0x1f43094f44b086c7), UINT64_C(0x1f4306d044afc6db),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43088eecb8473d)
    },
    {   UINT64_C(0x0000000000005b8b), UINT64_C(0x1f43094ef4adc710), UINT64_C(0x1f43088ec4a98710),
        UINT64_C(0x1f43048e24aee721), UINT64_C(0x1f430a4f10b6a72e)
    },
    {   UINT64_C(0x0000000000005b8c), UINT64_C(0x1f43038e5cb64730), UINT64_C(0x1f43018f94b566ed),
        UINT64_C(0x1f43038fccace70e), UINT64_C(0x1f43040ec8b366e7)
    },
    {   UINT64_C(0x0000000000005b8d), UINT64_C(0x1f4307ced8ade71f), UINT64_C(0x1f43078fe4ac0708),
        UINT64_C(0x1f43068fbcade72e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109c1), UINT64_C(0x1ea2f6cfa0b4a69c), UINT64_C(0x0000000000000000),
        UINT64_C(0x18aaf6cf689ce6a9), UINT64_C(0x18b2f6cf689ce6a9)
    },
    {   UINT64_C(0x0000000000005b8f), UINT64_C(0x1f430c0f14b06726), UINT64_C(0x1f4302cefcad070b),
        UINT64_C(0x1f43078e6cb1e708), UINT64_C(0x1f4305ce54b3c723)
    },
    {   UINT64_C(0x0000000000019140), UINT64_C(0x1fa2f80f68b545b2), UINT64_C(0x19a2fdcf689b05b6),
        UINT64_C(0x1882f80c209765aa), UINT64_C(0x1872fd4f689645d5)
    },
    {   UINT64_C(0x0000000000019141), UINT64_C(0x188af9cf489b469f), UINT64_C(0x19b2fa0f409f46a8),
        UINT64_C(0x0000000000000000), UINT64_C(0x1b22fa8f40a4a69f)
    },
    {   UINT64_C(0x0000000000019142), UINT64_C(0x16cafa8f4093469f), UINT64_C(0x17fafa8c7098069f),
        UINT64_C(0x1932fa8f409ce69f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019143), UINT64_C(0x0000000000000000), UINT64_C(0x1822fa8c7098a69f),
        UINT64_C(0x1a72fa8f48a2a69f), UINT64_C(0x14eaf9cf408cc69f)
    },
    {   UINT64_C(0x0000000000005513), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc9000b706cb), UINT64_C(0x1f42ff0ee8b306f9)
    },
    {   UINT64_C(0x00000000000109c8), UINT64_C(0x120af6d12082463c), UINT64_C(0x15a2f6cf6890a6a9),
        UINT64_C(0x1892f6cf689c66a9), UINT64_C(0x18a2f6cf689c86a9)
    },
    {   UINT64_C(0x00000000000109c9), UINT64_C(0x18a2f6cf689ca6a9), UINT64_C(0x18b2f6cf689d06a9),
        UINT64_C(0x18caf6cf6c9d66a9), UINT64_C(0x18d2f6cf6c9d66a9)
    },
    {   UINT64_C(0x00000000000109ca), UINT64_C(0x0000000000000000), UINT64_C(0x19baf6cf68a0e6a9),
        UINT64_C(0x18aaf6cf689cc6a9), UINT64_C(0x1a4af6cf68a366a9)
    },
    {   UINT64_C(0x00000000000109cb), UINT64_C(0x19baf6cf68a126a9), UINT64_C(0x1a0af6cf68a246a9),
        UINT64_C(0x1982f6cf6ca046a9), UINT64_C(0x19faf6cf68a206a9)
    },
    {   UINT64_C(0x00000000000109cc), UINT64_C(0x1cdaf6cf68ad86a9), UINT64_C(0x1742f6cfa097469c),
        UINT64_C(0x169af6cfa094869c), UINT64_C(0x176af6cfa097c69c)
    },
    {   UINT64_C(0x00000000000109cd), UINT64_C(0x18b2f68f689d06a9), UINT64_C(0x18c2f6cf689d46a9),
        UINT64_C(0x15baf6cf689126aa), UINT64_C(0x1842f6cf889b26a3)
    },
    {   UINT64_C(0x000000000001914b), UINT64_C(0x0fbafa8f587705bc), UINT64_C(0x115af78c5c7d8632),
        UINT64_C(0x0fdafacc5c77e5bc), UINT64_C(0x0e7af78f5873c5bc)
    },
    {   UINT64_C(0x0000000000004841), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8e28b2c72e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000109d0), UINT64_C(0x197af6cf68a006a9), UINT64_C(0x18aaf6cf689cc6a9),
        UINT64_C(0x18aaf6cf689cc6a9), UINT64_C(0x161af68f6892a6a9)
    },
    {   UINT64_C(0x000000000001914e), UINT64_C(0x172afa8fa094e67e), UINT64_C(0x1522ff8fa08cc67e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692ff8fa093267e)
    },
    {   UINT64_C(0x000000000001914f), UINT64_C(0x1372ff8fa085667e), UINT64_C(0x0eaaff8fa085667e),
        UINT64_C(0x1612f94f6c91868c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019150), UINT64_C(0x0a13018fa05d067e), UINT64_C(0x1182f80f687c667e),
        UINT64_C(0x16faff8fa099a68a), UINT64_C(0x127aff8fa085e67e)
    },
    {   UINT64_C(0x0000000000005ba1), UINT64_C(0x0000000000000000), UINT64_C(0x1f43084f7cb286ee),
        UINT64_C(0x1f43058fc0b726f2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019152), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x13e2f6cf6889268c), UINT64_C(0x107af30fa08c467e)
    },
    {   UINT64_C(0x00000000000109d6), UINT64_C(0x0452f6cfa04b869c), UINT64_C(0x08aaf6cfa05cc69c),
        UINT64_C(0x0cfaf6cfa06e069c), UINT64_C(0x0c42f6d3bc6b26a9)
    },
    {   UINT64_C(0x0000000000019154), UINT64_C(0x119af10fa094267e), UINT64_C(0x0000000000000000),
        UINT64_C(0x118af94f787dc5b4), UINT64_C(0x11aafa8f78800693)
    },
    {   UINT64_C(0x0000000000005ba5), UINT64_C(0x1f430a8f54b02717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43004e7cb4073b), UINT64_C(0x1f43030eb8b1c715)
    },
    {   UINT64_C(0x0000000000019156), UINT64_C(0x11f2f94c387f65b4), UINT64_C(0x1102fbcfa07ba693),
        UINT64_C(0x0812fbcfa057e693), UINT64_C(0x098aea4c3c5dc693)
    },
    {   UINT64_C(0x0000000000019157), UINT64_C(0x113afa8fa081e693), UINT64_C(0x0812fbcfa057e693),
        UINT64_C(0x19f2fbcfa09f65b4), UINT64_C(0x1102fbcfa07ba5b4)
    },
    {   UINT64_C(0x0000000000019158), UINT64_C(0x114af94f707e05b2), UINT64_C(0x11f2fbcc3880a5b4),
        UINT64_C(0x11f2f94c387f65b4), UINT64_C(0x096afbcfa07165b7)
    },
    {   UINT64_C(0x0000000000019159), UINT64_C(0x103af70f787ae5b4), UINT64_C(0x0a4af30f7863a661),
        UINT64_C(0x1102fbcf787ba5aa), UINT64_C(0x0ddaf30fa07a65aa)
    },
    {   UINT64_C(0x00000000000109dd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109de), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109df), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e0), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e1), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e2), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e3), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e4), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x00000000000109e5), UINT64_C(0x1bf2f6d088aa0662), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004918), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x00000000000109e7), UINT64_C(0x1cfaf6d020ae067c), UINT64_C(0x1d02f6d1a4ade61a),
        UINT64_C(0x1d02f6d1a0ae261b), UINT64_C(0x1a12f6d20ca26601)
    },
    {   UINT64_C(0x000000000000491a), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000005bb6), UINT64_C(0x1f43078f0cab06d3), UINT64_C(0x1f42ff8f68b2c725),
        UINT64_C(0x1f43044f14b26717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000491c), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f6ce2cb7272d)
    },
    {   UINT64_C(0x000000000000491d), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x000000000000491e), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x000000000000491f), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x0000000000004920), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x00000000000109ef), UINT64_C(0x1cfaf6d1f8ade606), UINT64_C(0x1e52f6d0d0b2e650),
        UINT64_C(0x1cb2f6912cad0639), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004922), UINT64_C(0x1f431fce5ca2a6dc), UINT64_C(0x1f431a4e5ca566dc),
        UINT64_C(0x1f4318ce5ca626dc), UINT64_C(0x1f43178e5ca6c6e0)
    },
    {   UINT64_C(0x000000000001916e), UINT64_C(0x13b2f34fa08aa67e), UINT64_C(0x13eaf50fa089e67e),
        UINT64_C(0x0e02f6cf786f65b7), UINT64_C(0x0f82fdcf7878c694)
    },
    {   UINT64_C(0x0000000000004924), UINT64_C(0x1f43180e5ca686dc), UINT64_C(0x1f43140e5ca886dc),
        UINT64_C(0x1f43140e5ca886dc), UINT64_C(0x1f43140e5ca886dc)
    },
    {   UINT64_C(0x0000000000004925), UINT64_C(0x1f43140e5ca886dc), UINT64_C(0x1f43140e5ca886dc),
        UINT64_C(0x1f43140e5ca886dc), UINT64_C(0x1f43140e5ca886dc)
    },
    {   UINT64_C(0x0000000000004926), UINT64_C(0x1f43140e5ca886dc), UINT64_C(0x1f43140e5ca886dc),
        UINT64_C(0x1f43140e5ca886dc), UINT64_C(0x1f430cce5cac26dc)
    },
    {   UINT64_C(0x0000000000004927), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x0000000000004928), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x000000000000552b), UINT64_C(0x1f42fb8f5cb8c705), UINT64_C(0x1f42fd0f3caa0702),
        UINT64_C(0x1f42fd0e6cb7270c), UINT64_C(0x1f42fe4f48b7a71b)
    },
    {   UINT64_C(0x000000000000492a), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x000000000000492b), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x000000000000492c), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x000000000000552d), UINT64_C(0x1f43004fa0b6c6ff), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff0f78af26e9), UINT64_C(0x1f42f90fd8b4e6da)
    },
    {   UINT64_C(0x0000000000005bc9), UINT64_C(0x1f43058f30b34709), UINT64_C(0x1f43094f7caf0706),
        UINT64_C(0x1f43084e78ae8722), UINT64_C(0x1f42ff8f08b7c719)
    },
    {   UINT64_C(0x000000000000c07f), UINT64_C(0x1f42ef0fa0b56783), UINT64_C(0x1692f64f689546b1),
        UINT64_C(0x1852f6ce789b6783), UINT64_C(0x0fa2f44e307a06b1)
    },
    {   UINT64_C(0x0000000000005bcb), UINT64_C(0x1f430b4e70b3e748), UINT64_C(0x1f43044f28b30726),
        UINT64_C(0x1f4304ceecb3472d), UINT64_C(0x1f43098eb4aa8707)
    },
    {   UINT64_C(0x0000000000005bcc), UINT64_C(0x1f43068ec8a8e71a), UINT64_C(0x1f42fccf38b546fb),
        UINT64_C(0x1f43008f54b76702), UINT64_C(0x1f43038e8caf073d)
    },
    {   UINT64_C(0x0000000000005bcd), UINT64_C(0x0000000000000000), UINT64_C(0x1f4302d01cb4a6e1),
        UINT64_C(0x1f43038f94ada724), UINT64_C(0x1f4300ceacb2670e)
    },
    {   UINT64_C(0x0000000000004933), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004934), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004935), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004936), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000005532), UINT64_C(0x1f42fb8f0cb8a70b), UINT64_C(0x1f42fd4ea0b14741),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019183), UINT64_C(0x123b008fa093467e), UINT64_C(0x10bb008fa093667e),
        UINT64_C(0x15bb064f689ce68c), UINT64_C(0x1493008fa09c267e)
    },
    {   UINT64_C(0x0000000000005bd4), UINT64_C(0x1f43060f84ae26f9), UINT64_C(0x1f43000f74b6c730),
        UINT64_C(0x1f43048f00b24731), UINT64_C(0x1f43024f70ae671d)
    },
    {   UINT64_C(0x0000000000005bd5), UINT64_C(0x1f430c4ec8ac072d), UINT64_C(0x1f43040f74b26700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43018eacb2c717)
    },
    {   UINT64_C(0x0000000000005bd6), UINT64_C(0x1f43084ea4b5c727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044eccb16721), UINT64_C(0x1f43048ec0b606e0)
    },
    {   UINT64_C(0x0000000000005bd7), UINT64_C(0x1f43064f1cb3470f), UINT64_C(0x1f43008e98b5a733),
        UINT64_C(0x1f42fecf58b666f3), UINT64_C(0x1f43094f08ad670d)
    },
    {   UINT64_C(0x0000000000005bd8), UINT64_C(0x1f430a8f0cafe6f6), UINT64_C(0x1f43028f10b6a6f6),
        UINT64_C(0x1f43048f24b2672f), UINT64_C(0x1f43048f04ab46f1)
    },
    {   UINT64_C(0x0000000000019189), UINT64_C(0x0e42fecf787d65b7), UINT64_C(0x0f73000f74898694),
        UINT64_C(0x0de2ff4f787b65b7), UINT64_C(0x0a83074fa0804693)
    },
    {   UINT64_C(0x0000000000005bda), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044f70ae8707),
        UINT64_C(0x1f4302cf00ad2708), UINT64_C(0x1f42fece8cb32710)
    },
    {   UINT64_C(0x0000000000004940), UINT64_C(0x1f42ee1548bb8574), UINT64_C(0x1f42ee14c4bb8594),
        UINT64_C(0x1f4366cdb87f2757), UINT64_C(0x1f435ecdb8834757)
    },
    {   UINT64_C(0x0000000000004941), UINT64_C(0x1f42fb9548b4c573), UINT64_C(0x1f42fb94c4b4c594),
        UINT64_C(0x1f43670e947f2721), UINT64_C(0x1f435ece94834721)
    },
    {   UINT64_C(0x0000000000005bdd), UINT64_C(0x1f4304cf50b2070e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43064f78b0a6e9), UINT64_C(0x1f43038e6cade726)
    },
    {   UINT64_C(0x000000000001918e), UINT64_C(0x15f3110f68a0e5ad), UINT64_C(0x0f12f58fa08285ad),
        UINT64_C(0x0f22e94c408525ad), UINT64_C(0x0dfaf44fa07f25ad)
    },
    {   UINT64_C(0x000000000001918f), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa094e68c),
        UINT64_C(0x169b0bcf54a2269b), UINT64_C(0x162b020fa09f068c)
    },
    {   UINT64_C(0x0000000000005539), UINT64_C(0x1f42fc0eecb56701), UINT64_C(0x1f42fe8eb0b7c72d),
        UINT64_C(0x1f42fd0eacb2a72d), UINT64_C(0x1f43010ec0b40712)
    },
    {   UINT64_C(0x0000000000019191), UINT64_C(0x0dc301cfa081468c), UINT64_C(0x0f0af1ce2086468c),
        UINT64_C(0x1643020fa0a4868c), UINT64_C(0x12db020fa08b068c)
    },
    {   UINT64_C(0x0000000000019192), UINT64_C(0x1c8b020fa0bba68c), UINT64_C(0x16a3020f58a4068c),
        UINT64_C(0x1853088f58a7869c), UINT64_C(0x122b020fa095668c)
    },
    {   UINT64_C(0x0000000000019193), UINT64_C(0x185b084ce0a7669b), UINT64_C(0x14f3020fa095c68c),
        UINT64_C(0x1152fe8f548e269b), UINT64_C(0x140b154fa0a0a68c)
    },
    {   UINT64_C(0x0000000000004949), UINT64_C(0x1f42ee0db87f2757), UINT64_C(0x1f42ee0db87f2757),
        UINT64_C(0x1f42ee0db8834757), UINT64_C(0x1f42ee0db8834757)
    },
    {   UINT64_C(0x000000000000488a), UINT64_C(0x1f431010c8ada68e), UINT64_C(0x1f43231080a0e6f2),
        UINT64_C(0x1f4323d050a186ed), UINT64_C(0x1f43464e348cc74a)
    },
    {   UINT64_C(0x000000000000494b), UINT64_C(0x1f42ee0db8bb8574), UINT64_C(0x1f42ee0db8bb8594),
        UINT64_C(0x1f42ee0db8bb8594), UINT64_C(0x1f42ee0db8bb8594)
    },
    {   UINT64_C(0x000000000000494c), UINT64_C(0x1f42ee0db8bb8573), UINT64_C(0x1f42ee0db8bb8594),
        UINT64_C(0x1f42ee0db8bb8594), UINT64_C(0x1f42ee0db8bb8594)
    },
    {   UINT64_C(0x000000000000553d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43000e40b4671e)
    },
    {   UINT64_C(0x000000000000494e), UINT64_C(0x1f42ee1548bb8757), UINT64_C(0x1f42ee14c4bb8757),
        UINT64_C(0x1f42ee14c4bb8757), UINT64_C(0x1f42ee14c4bb8757)
    },
    {   UINT64_C(0x0000000000005bea), UINT64_C(0x1f43070f74b1c6ea), UINT64_C(0x1f43088f14b0e71b),
        UINT64_C(0x1f4302cef8b48705), UINT64_C(0x1f43070f04ae0715)
    },
    {   UINT64_C(0x0000000000005beb), UINT64_C(0x1f43038e9cb2271e), UINT64_C(0x1f43030ee8b3c70b),
        UINT64_C(0x1f42fe8e84b56727), UINT64_C(0x1f42ff8ebcb0e712)
    },
    {   UINT64_C(0x0000000000004951), UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757),
        UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757)
    },
    {   UINT64_C(0x0000000000004952), UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757),
        UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757)
    },
    {   UINT64_C(0x000000000001919e), UINT64_C(0x106b030fa075467e), UINT64_C(0x106b034f747a067e),
        UINT64_C(0x106af58fa07c467e), UINT64_C(0x106af74fa07b667e)
    },
    {   UINT64_C(0x0000000000004894), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43111048a946d0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004955), UINT64_C(0x1f435c4dc8bb259e), UINT64_C(0x1f42ee0dc87f059e),
        UINT64_C(0x1f42ee0dc8846572), UINT64_C(0x1f42ee0dc884659e)
    },
    {   UINT64_C(0x0000000000004956), UINT64_C(0x1f436754a0bb2757), UINT64_C(0x1f435c554cbb2757),
        UINT64_C(0x1f435c54a0bb2757), UINT64_C(0x1f42ee14a07f0757)
    },
    {   UINT64_C(0x00000000000191a2), UINT64_C(0x106af1cfa07de67e), UINT64_C(0x106af9cfa07a667e),
        UINT64_C(0x106afc8f887f05b3), UINT64_C(0x106af24f887aa649)
    },
    {   UINT64_C(0x00000000000191a3), UINT64_C(0x106b018f887885b3), UINT64_C(0x106af84f887da649),
        UINT64_C(0x106afd0f887845b3), UINT64_C(0x106af88fa075264f)
    },
    {   UINT64_C(0x00000000000191a4), UINT64_C(0x106af54c107e05b5), UINT64_C(0x106af24fa07da649),
        UINT64_C(0x106b030fa076864a), UINT64_C(0x106af9cc146bc64a)
    },
    {   UINT64_C(0x000000000000495a), UINT64_C(0x1f42ee154cbb2757), UINT64_C(0x1f42ee14a0bb2757),
        UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f42ee0dc8bb2757)
    },
    {   UINT64_C(0x000000000000495b), UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f4366cdb8bb8574),
        UINT64_C(0x1f42ee0db87f2574), UINT64_C(0x1f42ee15487f2757)
    },
    {   UINT64_C(0x00000000000191a7), UINT64_C(0x106b000f8877a5b3), UINT64_C(0x106af44f8876862c),
        UINT64_C(0x106af24f887da5aa), UINT64_C(0x106af38f907d05aa)
    },
    {   UINT64_C(0x00000000000191a8), UINT64_C(0x106af1cf907e05aa), UINT64_C(0x106af3cfa07cc5aa),
        UINT64_C(0x106af40c147d25aa), UINT64_C(0x106af60fa079a5aa)
    },
    {   UINT64_C(0x00000000000191a9), UINT64_C(0x08b2f9cf745be5a1), UINT64_C(0x08b2cf0c645d45a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191aa), UINT64_C(0x16faf04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1362f20fa088a68c), UINT64_C(0x159af20fa090869a)
    },
    {   UINT64_C(0x0000000000004960), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ee0dc8bb23f0),
        UINT64_C(0x1f42ee0dc8bb2469), UINT64_C(0x1f42ee0dc8bb24e6)
    },
    {   UINT64_C(0x00000000000191ac), UINT64_C(0x0d2af30fa06cc68c), UINT64_C(0x19a2f58fa09ea68c),
        UINT64_C(0x161af04fa092c69a), UINT64_C(0x1e62f18fa0b1a68c)
    },
    {   UINT64_C(0x0000000000005bfd), UINT64_C(0x1f42ffce9cb46717), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4308ced4ae06d6), UINT64_C(0x1f430d8eb8abe710)
    },
    {   UINT64_C(0x0000000000004963), UINT64_C(0x1f42ee0dc87d0757), UINT64_C(0x1f42ee0dc86d6757),
        UINT64_C(0x1f42ee0dc85dc757), UINT64_C(0x1f42ee0dc84e2757)
    },
    {   UINT64_C(0x00000000000191af), UINT64_C(0x1692eecfa095e69c), UINT64_C(0x1a22f28fa0a4c68c),
        UINT64_C(0x175af20fa095e68c), UINT64_C(0x1902eecfa0a0a69c)
    },
    {   UINT64_C(0x00000000000191b0), UINT64_C(0x1852f6cfa098e69c), UINT64_C(0x159af20f6890868c),
        UINT64_C(0x1422f44f688bc5c1), UINT64_C(0x1162f94c6c80069c)
    },
    {   UINT64_C(0x0000000000005c01), UINT64_C(0x1f43058e7caf4715), UINT64_C(0x1f42ff8f94ace721),
        UINT64_C(0x1f430a4e40acc749), UINT64_C(0x1f43000e60ade72d)
    },
    {   UINT64_C(0x000000000000554a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe8e4cb64707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c03), UINT64_C(0x1f43038efcafc71b), UINT64_C(0x1f43030f3cad6715),
        UINT64_C(0x1f43054ed4b12731), UINT64_C(0x1f4304cf18a9c710)
    },
    {   UINT64_C(0x0000000000005c04), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cebcb6e725),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c05), UINT64_C(0x1f42ff4ef8b746ff), UINT64_C(0x1f4302cf94b24702),
        UINT64_C(0x1f43068e5cae86ee), UINT64_C(0x1f43068eecb3a71d)
    },
    {   UINT64_C(0x0000000000005c06), UINT64_C(0x1f4305cf1cb22734), UINT64_C(0x1f43030fbcb646fa),
        UINT64_C(0x1f42f90ef8b6a716), UINT64_C(0x1f4302ceecb6e731)
    },
    {   UINT64_C(0x000000000000496c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4302d048b126c9), UINT64_C(0x1f4302d048b126c9)
    },
    {   UINT64_C(0x00000000000191b8), UINT64_C(0x0ffaf44c6c7b45c1), UINT64_C(0x1b02f04f68a8e5b5),
        UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675)
    },
    {   UINT64_C(0x00000000000191b9), UINT64_C(0x1342f78fa098a675), UINT64_C(0x12a2f58fa0840683),
        UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1822fd0fa0a2c675)
    },
    {   UINT64_C(0x000000000000496f), UINT64_C(0x1f4302d048b106c9), UINT64_C(0x1f43044fecb1069e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191bb), UINT64_C(0x1672f6cfa0944683), UINT64_C(0x1f6af78fa0c26675),
        UINT64_C(0x1a9af78fa0af2675), UINT64_C(0x1612fb0fa09b2675)
    },
    {   UINT64_C(0x000000000000554f), UINT64_C(0x1f43010ee8b2e70c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe0fdcb4c726), UINT64_C(0x1f4302cf90b22702)
    },
    {   UINT64_C(0x00000000000191bd), UINT64_C(0x16fafc8fa09d8675), UINT64_C(0x1132f44fa0924675),
        UINT64_C(0x132af50fa09b6684), UINT64_C(0x1822f44fa09be675)
    },
    {   UINT64_C(0x0000000000004973), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42face88b52724), UINT64_C(0x1f42fa8e7cb4e720)
    },
    {   UINT64_C(0x0000000000004974), UINT64_C(0x1f43018e64b1c6e6), UINT64_C(0x1f43018e64b1c6e6),
        UINT64_C(0x1f43018e64b1c6e6), UINT64_C(0x1f43018e64b1c6e6)
    },
    {   UINT64_C(0x0000000000005551), UINT64_C(0x1f42ff0f10b6a702), UINT64_C(0x1f430b0e4cafe72a),
        UINT64_C(0x1f430b0e58b00737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191c1), UINT64_C(0x085afa8f745ae5a1), UINT64_C(0x10b2fa8fa08325a1),
        UINT64_C(0x10eafa8f607e6684), UINT64_C(0x121af58cc4832594)
    },
    {   UINT64_C(0x00000000000191c2), UINT64_C(0x0e22fa8f7c7845a6), UINT64_C(0x0d8af44cc475e684),
        UINT64_C(0x0e52f80f747545a1), UINT64_C(0x1172f6cf6c858594)
    },
    {   UINT64_C(0x00000000000191c3), UINT64_C(0x0fb2f58cc47985a1), UINT64_C(0x0dbaf6ccc47885d2),
        UINT64_C(0x10caf80f7482e594), UINT64_C(0x0dd2f58f8c798594)
    },
    {   UINT64_C(0x0000000000004979), UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f42ee0dc8bb2757),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191c5), UINT64_C(0x154af30f749125a1), UINT64_C(0x0ebaf80f74796684),
        UINT64_C(0x0d72f80f747445a1), UINT64_C(0x1152f30f787de68e)
    },
    {   UINT64_C(0x00000000000191c6), UINT64_C(0x118af10f6c87e5a1), UINT64_C(0x142af80cc48c0684),
        UINT64_C(0x0feaf80cb47ba5a1), UINT64_C(0x1ac2f58f74aac594)
    },
    {   UINT64_C(0x00000000000191c7), UINT64_C(0x17baf7cfa0990695), UINT64_C(0x16b2f88fa097e685),
        UINT64_C(0x13d2f88fa09c2685), UINT64_C(0x13c2f60fa0882695)
    },
    {   UINT64_C(0x0000000000005c18), UINT64_C(0x1f43078f20aee71d), UINT64_C(0x1f43094e8cafe73c),
        UINT64_C(0x1f43068e80b46743), UINT64_C(0x1f43048ea0ac8734)
    },
    {   UINT64_C(0x0000000000005c19), UINT64_C(0x1f43048e7cb68724), UINT64_C(0x1f43030ea0b54745),
        UINT64_C(0x1f42fdcf14ad46ed), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191ca), UINT64_C(0x1b2af88fa0b20685), UINT64_C(0x1672fc4fa09d2685),
        UINT64_C(0x184afbcf609ce695), UINT64_C(0x1bbaf7cfa0b44685)
    },
    {   UINT64_C(0x0000000000004980), UINT64_C(0x1f42fe8e74b34714), UINT64_C(0x1f4302cfecb1268c),
        UINT64_C(0x1f4302ce1cb12739), UINT64_C(0x1f42f94eb4b5e702)
    },
    {   UINT64_C(0x0000000000004981), UINT64_C(0x0000000000000000), UINT64_C(0x1f43040fe0b1470e),
        UINT64_C(0x1f43048f10b1a6d9), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004982), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f8fd8aaa6d1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004983), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431510d8a8064a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191cf), UINT64_C(0x102af64c6c7e65b6), UINT64_C(0x11b2f90f7483e696),
        UINT64_C(0x09f2fbcf7c60c5b6), UINT64_C(0x1182fbcfa086a5b6)
    },
    {   UINT64_C(0x00000000000191d0), UINT64_C(0x11d2fbcf6081a695), UINT64_C(0x13e2f64c6c89e5aa),
        UINT64_C(0x105afbcf788145b3), UINT64_C(0x0ee2f50c747b6695)
    },
    {   UINT64_C(0x0000000000005c21), UINT64_C(0x1f43018fd8b066e0), UINT64_C(0x1f43160f68ade708),
        UINT64_C(0x1f43008fa8b486e3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004987), UINT64_C(0x1f430110d4b2061f), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430110b4b20617), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191d3), UINT64_C(0x136af64c709305b6), UINT64_C(0x15eaf64c6c9d05b6),
        UINT64_C(0x157af38f7891a5b6), UINT64_C(0x117af90f78846696)
    },
    {   UINT64_C(0x000000000000555b), UINT64_C(0x1f43074ee4b00715), UINT64_C(0x1f42fc8f80b1270b),
        UINT64_C(0x1f42ff0f88b0670e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191d5), UINT64_C(0x124af90c6c84e5b6), UINT64_C(0x1b22f64f78ac65aa),
        UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c)
    },
    {   UINT64_C(0x000000000000498b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc0eb4b48720)
    },
    {   UINT64_C(0x000000000000498c), UINT64_C(0x1f42f4ce64b826f6), UINT64_C(0x1f42f4ce64b826f6),
        UINT64_C(0x1f42f4ce64b826f6), UINT64_C(0x1f42f4ce64b826f6)
    },
    {   UINT64_C(0x000000000000498d), UINT64_C(0x1f42f4ce64b826f6), UINT64_C(0x1f42f4ce64b826f6),
        UINT64_C(0x1f42f4ce64b826f6), UINT64_C(0x1f42f4ce64b826f6)
    },
    {   UINT64_C(0x00000000000191d9), UINT64_C(0x1852f6cf589b669c), UINT64_C(0x165b020fa08de68c),
        UINT64_C(0x122b020fa085a68c), UINT64_C(0x1852f6cf549b669b)
    },
    {   UINT64_C(0x000000000000555e), UINT64_C(0x1f43020edcad071c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430f0eacb06738)
    },
    {   UINT64_C(0x0000000000004990), UINT64_C(0x1f431d0e84a3c726), UINT64_C(0x1f430f8fd8aaa6d1),
        UINT64_C(0x1f43294e7c9da72d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000555f), UINT64_C(0x1f4305ce4cb58731), UINT64_C(0x1f42fe0e5cb58733),
        UINT64_C(0x1f42f90fc4b7a6ff), UINT64_C(0x1f4300ce9cb36729)
    },
    {   UINT64_C(0x00000000000191dd), UINT64_C(0x0e1af44f6874c5c1), UINT64_C(0x0fa2f50c6c7b869c),
        UINT64_C(0x0ff2f6cc6c79c5c1), UINT64_C(0x0fa2f3cf687a269c)
    },
    {   UINT64_C(0x0000000000005560), UINT64_C(0x1f42fd0ea8b8270f), UINT64_C(0x1f4303cf68b146f7),
        UINT64_C(0x1f42fd0f20b606fa), UINT64_C(0x1f42feced8ade71a)
    },
    {   UINT64_C(0x00000000000191df), UINT64_C(0x0f62fbcf687985c1), UINT64_C(0x0faaf50c6c7ba69c),
        UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1292f08f688605b5)
    },
    {   UINT64_C(0x00000000000191e0), UINT64_C(0x0ffaf78c6c7b45c1), UINT64_C(0x0f92f48c6c7b45c1),
        UINT64_C(0x1042f44f688385b5), UINT64_C(0x0efaef8f687985b5)
    },
    {   UINT64_C(0x00000000000191e1), UINT64_C(0x1012f6cf687ba5c1), UINT64_C(0x13d2f44c6c8aa5c1),
        UINT64_C(0x11eaf04c6c83a5c1), UINT64_C(0x1542f44c6c95e5c1)
    },
    {   UINT64_C(0x00000000000191e2), UINT64_C(0x1722f88f6894a5b5), UINT64_C(0x0faaf44f687a069b),
        UINT64_C(0x0e72f44f687585c1), UINT64_C(0x10c2f6cf687fc69c)
    },
    {   UINT64_C(0x0000000000004998), UINT64_C(0x1f431ccfa0a426a8), UINT64_C(0x1f430d4f20abe688),
        UINT64_C(0x1f431ccee4a426c7), UINT64_C(0x1f43150f60a80698)
    },
    {   UINT64_C(0x0000000000004999), UINT64_C(0x1f4318cf60a626b7), UINT64_C(0x1f43110f20aa0679),
        UINT64_C(0x1f4320cf20a226b7), UINT64_C(0x1f43110f60aa06a8)
    },
    {   UINT64_C(0x000000000000499a), UINT64_C(0x1f43105030aa66d0), UINT64_C(0x1f43464ebc8f66ff),
        UINT64_C(0x1f43228edca146ff), UINT64_C(0x1f430dce64ab86c7)
    },
    {   UINT64_C(0x000000000000499b), UINT64_C(0x1f43130ebca906ff), UINT64_C(0x1f4337ce7096a702),
        UINT64_C(0x1f434f90988ac64a), UINT64_C(0x1f43258e409fc70c)
    },
    {   UINT64_C(0x00000000000191e7), UINT64_C(0x1742fd0f58a1868c), UINT64_C(0x1592f9cfa09aa68c),
        UINT64_C(0x1a8b048f58a5869c), UINT64_C(0x174afd0fa0a0668c)
    },
    {   UINT64_C(0x00000000000191e8), UINT64_C(0x122afd0fa092e68c), UINT64_C(0x1a7b048f54a5469b),
        UINT64_C(0x15c2f34fa09c068c), UINT64_C(0x140b03cfa097e68c)
    },
    {   UINT64_C(0x00000000000191e9), UINT64_C(0x14fb04cfa0a2669b), UINT64_C(0x1a1b000fa0a6e68c),
        UINT64_C(0x1722f30fa0a6a68c), UINT64_C(0x18bb0d4fa0af269b)
    },
    {   UINT64_C(0x00000000000191ea), UINT64_C(0x1982f9cfa0a4869b), UINT64_C(0x18630c0f289d8680),
        UINT64_C(0x1532f9cf689665c1), UINT64_C(0x1142eb0c6c87269c)
    },
    {   UINT64_C(0x00000000000191eb), UINT64_C(0x106b030c7085e5c1), UINT64_C(0x0fdaf58f6882e69c),
        UINT64_C(0x0fc2f94f687e05c1), UINT64_C(0x0f02fa4c6c81e69c)
    },
    {   UINT64_C(0x00000000000191ec), UINT64_C(0x10a2f78c6c8025c1), UINT64_C(0x1182fe4f6883a69c),
        UINT64_C(0x0a2afc0f686445b6), UINT64_C(0x10caf78fa08745c1)
    },
    {   UINT64_C(0x00000000000049a2), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f4d3c4b8259e), UINT64_C(0x1f42f4d3c4b8259e)
    },
    {   UINT64_C(0x00000000000049a3), UINT64_C(0x1f42f4d114b8264a), UINT64_C(0x1f42f4d114b8264a),
        UINT64_C(0x1f42f4d114b8264a), UINT64_C(0x1f42f4d114b8264a)
    },
    {   UINT64_C(0x00000000000191ef), UINT64_C(0x1332f94f688e65b6), UINT64_C(0x105af88f6882e5b6),
        UINT64_C(0x1132fd4f688045c1), UINT64_C(0x155afbcc6c90e5c1)
    },
    {   UINT64_C(0x000000000000c0ba), UINT64_C(0x0caaf80fa06c868c), UINT64_C(0x0caaf80fa06c868c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191f1), UINT64_C(0x0ea2f94f687945c1), UINT64_C(0x121afb8f6886e69c),
        UINT64_C(0x1432eb8f7891a5c1), UINT64_C(0x1532fa0c6c90469c)
    },
    {   UINT64_C(0x000000000000c0bb), UINT64_C(0x09d2ef4d0465242e), UINT64_C(0x0c5aef57e06f46b8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000191f3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa2fe4fa075a698)
    },
    {   UINT64_C(0x00000000000191f4), UINT64_C(0x0fa2f24fa07a4698), UINT64_C(0x0fa2f20f68790698),
        UINT64_C(0x0fa2f2cfa07a0698), UINT64_C(0x0fa2f38f8079468c)
    },
    {   UINT64_C(0x00000000000191f5), UINT64_C(0x0fa2f50f6c79e698), UINT64_C(0x0fa2f24fa07a268c),
        UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79a698)
    },
    {   UINT64_C(0x00000000000191f6), UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e304cfa06ae69d),
        UINT64_C(0x111afa0fa07ec68c), UINT64_C(0x10e2fa4f687a468c)
    },
    {   UINT64_C(0x00000000000191f7), UINT64_C(0x10e2f30fa080068c), UINT64_C(0x10e2f6cf687ba67e),
        UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e2f40fa07e667e)
    },
    {   UINT64_C(0x00000000000191f8), UINT64_C(0x10e2f50f687e868c), UINT64_C(0x10e2f50f707e868c),
        UINT64_C(0x0fa2f3cf687a268c), UINT64_C(0x0fa3090fa064468d)
    },
    {   UINT64_C(0x0000000000014918), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000556e), UINT64_C(0x0000000000000000), UINT64_C(0x1f43000f7cb8a719),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4303cec0b80733)
    },
    {   UINT64_C(0x000000000000ca6d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1b02fa8d98a44662), UINT64_C(0x1b02fa8d98a44662)
    },
    {   UINT64_C(0x00000000000191fc), UINT64_C(0x0fa2f2cfa07b0698), UINT64_C(0x0fa2f14f6c7ae68c),
        UINT64_C(0x0fa2f48f6c79e698), UINT64_C(0x0fa2f6cfa079668c)
    },
    {   UINT64_C(0x00000000000191fd), UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79e698),
        UINT64_C(0x106afbcf7478e691), UINT64_C(0x106b098fa074e691)
    },
    {   UINT64_C(0x00000000000191fe), UINT64_C(0x106afb0fa0794691), UINT64_C(0x106af8cf747a6691),
        UINT64_C(0x106af54fa07c2692), UINT64_C(0x106afb0f74794686)
    },
    {   UINT64_C(0x0000000000005c4f), UINT64_C(0x1f42f84f2cb64708), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000772), UINT64_C(0x106af6d7147ba64b), UINT64_C(0x106af6d7147ba64b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c51), UINT64_C(0x1f42fe4f34b0c6f5), UINT64_C(0x1f42fe9004b1e6fb),
        UINT64_C(0x1f42f5cf88b806db), UINT64_C(0x1f42fd8eb8b006eb)
    },
    {   UINT64_C(0x0000000000005c52), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb4e8cb82735),
        UINT64_C(0x1f43030fd8b04707), UINT64_C(0x1f42fd8fe8b0c6e3)
    },
    {   UINT64_C(0x0000000000005c53), UINT64_C(0x1f42ff4facb5a70f), UINT64_C(0x1f43004f78b7a6d7),
        UINT64_C(0x1f430a8eccae673c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0c4), UINT64_C(0x000289d7e0324724), UINT64_C(0x000289d7e032469b),
        UINT64_C(0x00029897f02ac74b), UINT64_C(0x00029897f02ac74b)
    },
    {   UINT64_C(0x0000000000004bc6), UINT64_C(0x1f434f0e7ca5e72e), UINT64_C(0x1f43468f0894a6ae),
        UINT64_C(0x1f42fc8f44a8a6e0), UINT64_C(0x1f4314d0f4a6068f)
    },
    {   UINT64_C(0x0000000000005574), UINT64_C(0x1f42fd0ec0af4716), UINT64_C(0x1f42fe0f70b5a722),
        UINT64_C(0x1f43010f18b82727), UINT64_C(0x1f42fd0e4cb4e704)
    },
    {   UINT64_C(0x0000000000004bc7), UINT64_C(0x1f43039010b826cc), UINT64_C(0x1f4323d1f8a7066c),
        UINT64_C(0x1f43188f58a8e6ed), UINT64_C(0x1f430a506cb366e9)
    },
    {   UINT64_C(0x0000000000005c58), UINT64_C(0x1f43008eacb146eb), UINT64_C(0x1f42fd4f14b52702),
        UINT64_C(0x1f430dcf24aca701), UINT64_C(0x1f42fd8f08b326e3)
    },
    {   UINT64_C(0x0000000000004bc8), UINT64_C(0x1f4302d024b46700), UINT64_C(0x1f431094a4afe726),
        UINT64_C(0x1f43098fd0adc6f7), UINT64_C(0x1f43138f0ca1c71f)
    },
    {   UINT64_C(0x0000000000005576), UINT64_C(0x0000000000000000), UINT64_C(0x1f43010eacb3871b),
        UINT64_C(0x1f4305ce6cb1c724), UINT64_C(0x1f42fe8eccb64735)
    },
    {   UINT64_C(0x0000000000005c5b), UINT64_C(0x1f42fc8eccb20713), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43048eb0b0873d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0c8), UINT64_C(0x0002950c682c835c), UINT64_C(0x0002b1cb243bc433),
        UINT64_C(0x0002d68b244e2433), UINT64_C(0x00029f8be827637b)
    },
    {   UINT64_C(0x000000000000a9c9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f0af04bfc73e693)
    },
    {   UINT64_C(0x000000000000c0c9), UINT64_C(0x0002854be834837b), UINT64_C(0x0002950b442ca3a6),
        UINT64_C(0x00029889042ac390), UINT64_C(0x0002934c382d63e8)
    },
    {   UINT64_C(0x0000000000004bcb), UINT64_C(0x1f4317ce88a2c709), UINT64_C(0x1f4315900cac66d7),
        UINT64_C(0x1f43155024a706d8), UINT64_C(0x1f43021158b4c6a1)
    },
    {   UINT64_C(0x00000000000049c5), UINT64_C(0x1f430c4f68ac669a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43250e90a0066d)
    },
    {   UINT64_C(0x0000000000004bcc), UINT64_C(0x1f431e9040ada6e8), UINT64_C(0x1f43254e5ca2c72c),
        UINT64_C(0x1f43084ee8b34709), UINT64_C(0x1f43244faca2a6df)
    },
    {   UINT64_C(0x000000000000c0cb), UINT64_C(0x0002858b94346395), UINT64_C(0x0002858b8c346393),
        UINT64_C(0x000289cbdc32439a), UINT64_C(0x000289cbdc32439a)
    },
    {   UINT64_C(0x0000000000004bcd), UINT64_C(0x1f43070f54a1a704), UINT64_C(0x1f42facfecb2e6a0),
        UINT64_C(0x1f43004f24b366fa), UINT64_C(0x1f430b0e9cac66f6)
    },
    {   UINT64_C(0x000000000000557b), UINT64_C(0x1f42fc0eecb26722), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f9cb466ff)
    },
    {   UINT64_C(0x0000000000004bce), UINT64_C(0x1f432d4edca0a70e), UINT64_C(0x1f432750e0a4c677),
        UINT64_C(0x1f432ccef897a704), UINT64_C(0x1f43254e7c8e672b)
    },
    {   UINT64_C(0x000000000000557c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43010e98ba0711), UINT64_C(0x1f42fb0f14b2e723)
    },
    {   UINT64_C(0x0000000000004bcf), UINT64_C(0x1f43071188b1666c), UINT64_C(0x1f43178eacad66fb),
        UINT64_C(0x1f4312cf00a926e7), UINT64_C(0x1f43068e88b10727)
    },
    {   UINT64_C(0x000000000000c0ce), UINT64_C(0x0002878d3042667e), UINT64_C(0x0002904abc26c3c7),
        UINT64_C(0x000283cbcc32e3a1), UINT64_C(0x000295cb8029a3a2)
    },
    {   UINT64_C(0x0000000000004bd0), UINT64_C(0x1f43124f34b5e731), UINT64_C(0x1f430f4f94b3a70d),
        UINT64_C(0x1f430f4fc8b1c71a), UINT64_C(0x1f43000e5cb30730)
    },
    {   UINT64_C(0x000000000000c0cf), UINT64_C(0x0002878bf830e38f), UINT64_C(0x00029697c02bc6b7),
        UINT64_C(0x00029f97b0272759), UINT64_C(0x0002711bf03e8792)
    },
    {   UINT64_C(0x000000001fb17ec0), UINT64_C(0x132ac3cc6c9946a2), UINT64_C(0x1162bf0c6c7ec6a7),
        UINT64_C(0x1162bf8c6c83a6a8), UINT64_C(0x1ac2c3cc6ca446a7)
    },
    {   UINT64_C(0x0000000000017ec1), UINT64_C(0x1ab2c3cc6ca8e6a8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004bd2), UINT64_C(0x1f431a1124a606a0), UINT64_C(0x1f430b4f48990715),
        UINT64_C(0x1f43044eecb84723), UINT64_C(0x1f43138f70ac4709)
    },
    {   UINT64_C(0x000000000000c0d1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x00028798203126b5), UINT64_C(0x0002ee0a9867c35f)
    },
    {   UINT64_C(0x0000000000004bd3), UINT64_C(0x1f4301d030b546e2), UINT64_C(0x1f43178f0caa26ee),
        UINT64_C(0x1f4317cf34af271d), UINT64_C(0x1f42fccf10b5c6e6)
    },
    {   UINT64_C(0x00000000000049d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa8ea8b54718), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004bd4), UINT64_C(0x1f4315107cb5e6f2), UINT64_C(0x1f43008ff0b7c711),
        UINT64_C(0x1f430e0e9cb2a717), UINT64_C(0x1f430a8f1cb4871b)
    },
    {   UINT64_C(0x0000000000005c72), UINT64_C(0x1f42f88f9cb78718), UINT64_C(0x1f42f78f1cb52712),
        UINT64_C(0x1f430e8eacafa717), UINT64_C(0x1f42fa8f58b206f6)
    },
    {   UINT64_C(0x0000000000005c73), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc8e78b7c732), UINT64_C(0x1f42fe8ec8aba6fa)
    },
    {   UINT64_C(0x0000000000004919), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000001492e), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000491b), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000001492f), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x00000000000049dd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f98de8b5c705), UINT64_C(0x1f42f98de8b5c705)
    },
    {   UINT64_C(0x000000000000ca84), UINT64_C(0x128af60aa09a2720), UINT64_C(0x128af60aa09a2720),
        UINT64_C(0x132af60aa09a2720), UINT64_C(0x128af60aa09a2720)
    },
    {   UINT64_C(0x00000000000049df), UINT64_C(0x1f42f98de8b5c705), UINT64_C(0x1f42f98de8b5c705),
        UINT64_C(0x1f42f98de8b5c705), UINT64_C(0x1f42f98de8b5c705)
    },
    {   UINT64_C(0x000000000000ca85), UINT64_C(0x151af60aa09a6720), UINT64_C(0x128af60aa09a2720),
        UINT64_C(0x137af60aa09a2720), UINT64_C(0x18baf60aa0ab6720)
    },
    {   UINT64_C(0x0000000000004921), UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724),
        UINT64_C(0x1f42f4ce50b82724), UINT64_C(0x1f42f4ce50b82724)
    },
    {   UINT64_C(0x000000000000ca86), UINT64_C(0x13caf60aa09a2720), UINT64_C(0x146af60aa09a2720),
        UINT64_C(0x146af60aa09a2720), UINT64_C(0x116af60aa09aa7a7)
    },
    {   UINT64_C(0x0000000000004923), UINT64_C(0x1f431b4e5ca4e6dc), UINT64_C(0x1f43194e5ca5e6dc),
        UINT64_C(0x1f431b4e5ca4e6dc), UINT64_C(0x1f43184e5ca666dc)
    },
    {   UINT64_C(0x000000000000ca87), UINT64_C(0x116af608849aa720), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000049e5), UINT64_C(0x1f43065080af6654), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c81), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff8f78b026ef),
        UINT64_C(0x1f42f98e94b6c71c), UINT64_C(0x1f4305ceccb32730)
    },
    {   UINT64_C(0x000000000000558a), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044e5cb5a741),
        UINT64_C(0x1f43024f0caa46ea), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000049e8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4302d0d4b12639), UINT64_C(0x1f430050e8b2664b)
    },
    {   UINT64_C(0x0000000000004929), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724)
    },
    {   UINT64_C(0x0000000000014936), UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a),
        UINT64_C(0x15baee0f5495669a), UINT64_C(0x15baee0f5495669a)
    },
    {   UINT64_C(0x000000000000558c), UINT64_C(0x1f42face88aea723), UINT64_C(0x1f43024f14aba6e9),
        UINT64_C(0x1f42fb0ed8af2729), UINT64_C(0x1f43024ebcb26715)
    },
    {   UINT64_C(0x0000000000004bdf), UINT64_C(0x1f42fd8f78b22735), UINT64_C(0x1f430e8fecad66cc),
        UINT64_C(0x1f43084e6cb44727), UINT64_C(0x1f42fe8e5cb36718)
    },
    {   UINT64_C(0x000000000000492d), UINT64_C(0x1f42f48e50b80724), UINT64_C(0x1f42f48e50b80724),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000000492e), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000000492f), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004930), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004931), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004932), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000005590), UINT64_C(0x1f43024f48b0e6ef), UINT64_C(0x1f43000f60b22702),
        UINT64_C(0x1f4302ce80b2e748), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c8f), UINT64_C(0x1f42fb5004b4e6f0), UINT64_C(0x1f430a8f14b6071f),
        UINT64_C(0x1f43110e3cad0706), UINT64_C(0x1f431b4e8c982723)
    },
    {   UINT64_C(0x0000000000005591), UINT64_C(0x0000000000000000), UINT64_C(0x1f4301ce70af272e),
        UINT64_C(0x1f42fd0f00b3e6fe), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005c91), UINT64_C(0x1f42facf0cb72714), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43038ef0b1c719), UINT64_C(0x1f430e4ed09c672b)
    },
    {   UINT64_C(0x0000000000004937), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004938), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x0000000000004939), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000000493a), UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005594), UINT64_C(0x1f43060f78af6715), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004f3cb2c72a)
    },
    {   UINT64_C(0x0000000000004be7), UINT64_C(0x1f4302101cb76703), UINT64_C(0x1f430e0e6cab6737),
        UINT64_C(0x1f43084e7cb06744), UINT64_C(0x1f43038e6cb46741)
    },
    {   UINT64_C(0x0000000000000796), UINT64_C(0x106af38f747fc6ec), UINT64_C(0x106af60fa07b6634),
        UINT64_C(0x106af38d1c7fc64b), UINT64_C(0x106af60d487b659d)
    },
    {   UINT64_C(0x0000000000014940), UINT64_C(0x1622eb53cc98653c), UINT64_C(0x0000000000000000),
        UINT64_C(0x15c33c0ae46ea776), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000797), UINT64_C(0x106af38d1c7fc6ce), UINT64_C(0x106af60d487b6610),
        UINT64_C(0x106af38d947fc64b), UINT64_C(0x106af60ddc7b659d)
    },
    {   UINT64_C(0x0000000000004be9), UINT64_C(0x1f43038ef8b36731), UINT64_C(0x1f43079004b366e3),
        UINT64_C(0x1f42fa0f44b5c71e), UINT64_C(0x1f430c4e94b60728)
    },
    {   UINT64_C(0x0000000000000798), UINT64_C(0x106afb0f747986ec), UINT64_C(0x106b000f8877e635),
        UINT64_C(0x106afb0d1c798656), UINT64_C(0x106b000d3477e5a0)
    },
    {   UINT64_C(0x0000000000004942), UINT64_C(0x1f42f4d548b82573), UINT64_C(0x1f42f4d4c4b82594),
        UINT64_C(0x1f43670e287f273c), UINT64_C(0x1f435ece2883473c)
    },
    {   UINT64_C(0x0000000000004943), UINT64_C(0x1f4366cdb8bb8574), UINT64_C(0x1f43670db8bb8594),
        UINT64_C(0x1f435ecdb8bb8573), UINT64_C(0x1f435ecdb8bb8594)
    },
    {   UINT64_C(0x0000000000004944), UINT64_C(0x1f42ee0db87f2574), UINT64_C(0x1f42ee0db87f2594),
        UINT64_C(0x1f42ee0db8834573), UINT64_C(0x1f42ee0db8834594)
    },
    {   UINT64_C(0x0000000000004945), UINT64_C(0x1f4366d548bb8757), UINT64_C(0x1f436714c4bb8757),
        UINT64_C(0x1f435ed548bb8757), UINT64_C(0x1f435ed4c4bb8757)
    },
    {   UINT64_C(0x0000000000004946), UINT64_C(0x1f42ee15487f2757), UINT64_C(0x1f42ee14c47f2757),
        UINT64_C(0x1f42ee1548834757), UINT64_C(0x1f42ee14c4834757)
    },
    {   UINT64_C(0x0000000000004947), UINT64_C(0x1f4366cdb8bb8757), UINT64_C(0x1f43670db8bb8757),
        UINT64_C(0x1f435ecdb8bb8757), UINT64_C(0x1f435ecdb8bb8757)
    },
    {   UINT64_C(0x0000000000004948), UINT64_C(0x1f435ecdb8bb8757), UINT64_C(0x1f435ecdb8bb8757),
        UINT64_C(0x1f435ecdb8bb8757), UINT64_C(0x1f435ecdb8bb8757)
    },
    {   UINT64_C(0x000000000000079c), UINT64_C(0x106af48c687cc64b), UINT64_C(0x106af44c5476a612),
        UINT64_C(0x106aef0d1c7f864b), UINT64_C(0x106af10d347e8597)
    },
    {   UINT64_C(0x000000000000494a), UINT64_C(0x1f42ee0db8834757), UINT64_C(0x1f42ee0db8834757),
        UINT64_C(0x1f42ee0db8834757), UINT64_C(0x1f42ee0db8834757)
    },
    {   UINT64_C(0x000000000000079d), UINT64_C(0x106aef0ca87f864b), UINT64_C(0x106af10ce47e8597),
        UINT64_C(0x106aef0c3c7f864b), UINT64_C(0x106af10c547e8597)
    },
    {   UINT64_C(0x000000000000ca9b), UINT64_C(0x0000000000000000), UINT64_C(0x1772ff0eb093e6df),
        UINT64_C(0x1772ff114c93c75c), UINT64_C(0x1772ff100893e6d8)
    },
    {   UINT64_C(0x000000000000494d), UINT64_C(0x1f42ee1548bb8757), UINT64_C(0x1f42ee14c4bb8757),
        UINT64_C(0x1f42ee14c4bb8757), UINT64_C(0x1f42ee14c4bb8757)
    },
    {   UINT64_C(0x000000000000ca9c), UINT64_C(0x1772ff0ce093e690), UINT64_C(0x1772fe4d24942681),
        UINT64_C(0x1772fe4d24942681), UINT64_C(0x1772feceb093c620)
    },
    {   UINT64_C(0x000000000000494f), UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757),
        UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757)
    },
    {   UINT64_C(0x0000000000004950), UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757),
        UINT64_C(0x1f42ee0db8bb8757), UINT64_C(0x1f42ee0db8bb8757)
    },
    {   UINT64_C(0x00000000000007a0), UINT64_C(0x106af0cf807ea701), UINT64_C(0x106af20f907e0656),
        UINT64_C(0x106af0cf807ea701), UINT64_C(0x106af20f907e0656)
    },
    {   UINT64_C(0x0000000000004bf2), UINT64_C(0x1f42fd8e9cb2673f), UINT64_C(0x1f43008e38b46742),
        UINT64_C(0x1f43080f5cad86fc), UINT64_C(0x1f430e4e20ada713)
    },
    {   UINT64_C(0x00000000000007a1), UINT64_C(0x106af0cf807ea6ec), UINT64_C(0x106af20f907e0635),
        UINT64_C(0x106af0cf807ea6ec), UINT64_C(0x106af20f907e0634)
    },
    {   UINT64_C(0x0000000000004954), UINT64_C(0x1f42ee14a0bb259e), UINT64_C(0x1f435c4dc8846757),
        UINT64_C(0x1f43674dc8bb259e), UINT64_C(0x1f435c4dc8bb2572)
    },
    {   UINT64_C(0x00000000000007a2), UINT64_C(0x106af0cd287ea64b), UINT64_C(0x106af20d387e0597),
        UINT64_C(0x106af4cfa07c86ec), UINT64_C(0x106af4cfa07c8634)
    },
    {   UINT64_C(0x0000000000004bf4), UINT64_C(0x1f43068e6cb3474a), UINT64_C(0x1f43044e58b80751),
        UINT64_C(0x1f430c4eb0aee732), UINT64_C(0x1f42fbcf38b7072d)
    },
    {   UINT64_C(0x0000000000004957), UINT64_C(0x1f42ee154c846757), UINT64_C(0x1f42ee14a0846757),
        UINT64_C(0x1f43674dc8bb2757), UINT64_C(0x1f435c4dc8bb2757)
    },
    {   UINT64_C(0x0000000000004958), UINT64_C(0x1f435c4dc8bb2757), UINT64_C(0x1f42ee0dc87f0757),
        UINT64_C(0x1f42ee0dc8846757), UINT64_C(0x1f42ee0dc8846757)
    },
    {   UINT64_C(0x0000000000004959), UINT64_C(0x1f42ee0dc8bb259e), UINT64_C(0x1f42ee0dc8bb2572),
        UINT64_C(0x1f42ee0dc8bb259e), UINT64_C(0x1f42ee14a0bb2757)
    },
    {   UINT64_C(0x000000000000caa2), UINT64_C(0x1773020c4c92466e), UINT64_C(0x1772fecadc93c700),
        UINT64_C(0x1772fecadc93c700), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007a5), UINT64_C(0x106afa0dc07aa64b), UINT64_C(0x106af64ddc79e597),
        UINT64_C(0x106af10ddc7e864b), UINT64_C(0x106af44f8876a6a8)
    },
    {   UINT64_C(0x000000000000495c), UINT64_C(0x1f4366d548bb8757), UINT64_C(0x1f42ee0dc8bb2757),
        UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f42ee0dc8bb2757)
    },
    {   UINT64_C(0x000000000000495d), UINT64_C(0x1f42ee1c40bb872f), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005cb9), UINT64_C(0x1f4301cefcac86e6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f08b566da), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000055a6), UINT64_C(0x1f42fc8e78b48738), UINT64_C(0x1f42fc0f70b0671e),
        UINT64_C(0x1f431f4e60a64726), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000caa5), UINT64_C(0x0000000000000000), UINT64_C(0x1773020d70924670),
        UINT64_C(0x1773020d70924670), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004961), UINT64_C(0x1f42ee0dc8bb2563), UINT64_C(0x1f42ee0dc8bb25e0),
        UINT64_C(0x1f42ee0dc8bb265d), UINT64_C(0x1f42ee0dc8bb26da)
    },
    {   UINT64_C(0x0000000000004962), UINT64_C(0x1f42ee0dc8bb2757), UINT64_C(0x1f42ee0dc8abe757),
        UINT64_C(0x1f42ee0dc89c4757), UINT64_C(0x1f42ee0dc88ca757)
    },
    {   UINT64_C(0x00000000000055a8), UINT64_C(0x1f42fe8f18b34710), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43024e34b18744), UINT64_C(0x1f430f4f0ca7070f)
    },
    {   UINT64_C(0x000000000000bfff), UINT64_C(0x0000000000000000), UINT64_C(0x1b42fbcc3ca4a67e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004965), UINT64_C(0x1f42ee1b6cbb8757), UINT64_C(0x1f43c90db8bb8757),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014954), UINT64_C(0x1622eb5258986599), UINT64_C(0x1623264ae47b0776),
        UINT64_C(0x16233d8ae4988599), UINT64_C(0x1623264ae498853c)
    },
    {   UINT64_C(0x00000000000007ab), UINT64_C(0x106af4cd3c7c870b), UINT64_C(0x106afbcd3c800657),
        UINT64_C(0x106af4cfa07ca7c4), UINT64_C(0x106afbcf8880070f)
    },
    {   UINT64_C(0x0000000000004968), UINT64_C(0x1f43024f38afa706), UINT64_C(0x1f43040f1caec701),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007ac), UINT64_C(0x106af4cfa07ca7c4), UINT64_C(0x106afbcf8880070f),
        UINT64_C(0x106af4cfa07ca806), UINT64_C(0x106afbcf88800751)
    },
    {   UINT64_C(0x0000000000014956), UINT64_C(0x16233d9258988776), UINT64_C(0x16232653cc988776),
        UINT64_C(0x1623265258986776), UINT64_C(0x1622eb52586f4776)
    },
    {   UINT64_C(0x00000000000007ad), UINT64_C(0x106af4cfa07ca7aa), UINT64_C(0x106afbcf888006f5),
        UINT64_C(0x106af4cd3c7c870b), UINT64_C(0x106afbcd3c800656)
    },
    {   UINT64_C(0x000000000000caab), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1773020de4924653)
    },
    {   UINT64_C(0x000000000000c008), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a6b024f6c5de69b),
        UINT64_C(0x115b02d5f079869b), UINT64_C(0x0fa2ef4fa07c66a4)
    },
    {   UINT64_C(0x000000000000caac), UINT64_C(0x1773020de4924653), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c00a), UINT64_C(0x0a6af98d0064c69e), UINT64_C(0x0a6af4cd0062669e),
        UINT64_C(0x0fa2fc139c76869b), UINT64_C(0x1772fe4ebc94261b)
    },
    {   UINT64_C(0x00001a010008c00b), UINT64_C(0x07d2f7ccd05a6483), UINT64_C(0x0a6af9124c626507),
        UINT64_C(0x07d2f84f6c58c484), UINT64_C(0x08b2e80f5464469b)
    },
    {   UINT64_C(0x000000000000c00c), UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa2fe4fa075a698),
        UINT64_C(0x0fa2f24fa07a4698), UINT64_C(0x0fa2f20f68790698)
    },
    {   UINT64_C(0x000000000000c00d), UINT64_C(0x0fa2f2cfa07a0698), UINT64_C(0x0fa2f38f8079468c),
        UINT64_C(0x0fa2f50f6c79e698), UINT64_C(0x0fa2f24fa07a268c)
    },
    {   UINT64_C(0x000000000000c00e), UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79a698),
        UINT64_C(0x0a6b028f6c5de5c0), UINT64_C(0x0a6b028cd05fc5c0)
    },
    {   UINT64_C(0x000000000000caaf), UINT64_C(0x0000000000000000), UINT64_C(0x1773028f6c92060b),
        UINT64_C(0x1773028f6c92060b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x04706c038000c010), UINT64_C(0x1d13090f54a5469b), UINT64_C(0x1692f04fa094a69a),
        UINT64_C(0x14daf20fa08be68c), UINT64_C(0x1692fa4f5494669b)
    },
    {   UINT64_C(0x0810c0000298c011), UINT64_C(0x1692f18fa094c68c), UINT64_C(0x14daf20fa08ea68c),
        UINT64_C(0x131af20fa087668c), UINT64_C(0x1852f74f549ce69b)
    },
    {   UINT64_C(0x09211c234000c012), UINT64_C(0x1852f34fa09d668c), UINT64_C(0x0c2af30fa06cc68c),
        UINT64_C(0x0fa2eece207a468c), UINT64_C(0x1852f58fa09ea68c)
    },
    {   UINT64_C(0x0000000000004c05), UINT64_C(0x1f43820e2ca4c73c), UINT64_C(0x1f431b8e2c71c73c),
        UINT64_C(0x1f437fce94b12721), UINT64_C(0x1f4302ce9472a721)
    },
    {   UINT64_C(0x1161f2000718c014), UINT64_C(0x131af20fa089868c), UINT64_C(0x1852f6cce09b669b),
        UINT64_C(0x1692f48fa098068c), UINT64_C(0x1162f6cf547ea69c)
    },
    {   UINT64_C(0x1942c6558900c015), UINT64_C(0x14daf5cfa08e068c), UINT64_C(0x1692f20f5496268c),
        UINT64_C(0x1692f20f5896268c), UINT64_C(0x1f42f2cf64b9268c)
    },
    {   UINT64_C(0x0003e673ce28c016), UINT64_C(0x1692f20fa095e68c), UINT64_C(0x1692f1cfa095e68c),
        UINT64_C(0x14daf50fa08dc68c), UINT64_C(0x0a6afecd4c64268e)
    },
    {   UINT64_C(0x000000000000c017), UINT64_C(0x08b2e7cf5464669b), UINT64_C(0x0a6af60d4c5fc68e),
        UINT64_C(0x122b00547c7e268c), UINT64_C(0x0fa2ee0dac7d039d)
    },
    {   UINT64_C(0x1fe3f67d8000c018), UINT64_C(0x0a6af017e05d46b1), UINT64_C(0x0fa2f44f687b85c1),
        UINT64_C(0x1162f24f687fa68c), UINT64_C(0x0de2f44f687445c1)
    },
    {   UINT64_C(0x22140c800000c019), UINT64_C(0x1162f44f6881468c), UINT64_C(0x0de2f44f6873e5c1),
        UINT64_C(0x0a6af18fa06f269b), UINT64_C(0x0fa2f50c687ae5c1)
    },
    {   UINT64_C(0x2320008bd168c01a), UINT64_C(0x1162f1cfa081468c), UINT64_C(0x08b2f1cfa05e869b),
        UINT64_C(0x0a6adfcc745f669b), UINT64_C(0x1162f38fa082668c)
    },
    {   UINT64_C(0x24a4889011f0c01b), UINT64_C(0x08b2f1cfa05e868c), UINT64_C(0x1a0af1cfa0a445c1),
        UINT64_C(0x1162f34fa081e5c1), UINT64_C(0x0fa2f44f687a05c1)
    },
    {   UINT64_C(0x2824aa94d288c01c), UINT64_C(0x1162f2cc6c8005c1), UINT64_C(0x1162f68c6c8185c1),
        UINT64_C(0x0de2f50fa074c5c1), UINT64_C(0x0c2af44f686ba5c1)
    },
    {   UINT64_C(0x2ae528a4d420c01d), UINT64_C(0x0a6af2cf7068065e), UINT64_C(0x1162f20f6881c5b5),
        UINT64_C(0x0fa2f34f687b25b5), UINT64_C(0x1692f3cf6896e5b5)
    },
    {   UINT64_C(0x000000b35640c01e), UINT64_C(0x0fa2f10fa07b05b5), UINT64_C(0x0fa2f20c6c7ac5b5),
        UINT64_C(0x0de2f34fa07305b5), UINT64_C(0x0c52f38ce46906a2)
    },
    {   UINT64_C(0x000000000000d101), UINT64_C(0x14daf20fa08ea68c), UINT64_C(0x131af20fa087668c),
        UINT64_C(0x1852f74f549ce69b), UINT64_C(0x1852f34fa09d668c)
    },
    {   UINT64_C(0x00000000000007ba), UINT64_C(0x106aeecf74802732), UINT64_C(0x106af0cf8c7ea67d),
        UINT64_C(0x106aeecf74802732), UINT64_C(0x106af0cf8c7ea67d)
    },
    {   UINT64_C(0x000000000000d103), UINT64_C(0x1d82f18fa0b1a68c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x131af20fa089868c)
    },
    {   UINT64_C(0x00000000000007bb), UINT64_C(0x106aeecf74802773), UINT64_C(0x106af0cf8c7ea6bf),
        UINT64_C(0x106aeecf74802729), UINT64_C(0x106af0cf8c7ea674)
    },
    {   UINT64_C(0x000000000000d105), UINT64_C(0x1692f20f5496268c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x1f42f2cf64b9268c), UINT64_C(0x1692f20fa095e68c)
    },
    {   UINT64_C(0x00000000000007bc), UINT64_C(0x106aeecd3c8026ae), UINT64_C(0x106af0cd407ea5fa),
        UINT64_C(0x106af2cfa07da701), UINT64_C(0x106af48c107d0656)
    },
    {   UINT64_C(0x000000000000d107), UINT64_C(0x0de2f44f687445c1), UINT64_C(0x1162f44f6881468c),
        UINT64_C(0x0de2f44f6873e5c1), UINT64_C(0x0a6af18fa06f269b)
    },
    {   UINT64_C(0x000000000000d108), UINT64_C(0x0fa2f50c687ae5c1), UINT64_C(0x1162f1cfa081468c),
        UINT64_C(0x08b2f1cfa05e869b), UINT64_C(0x0a6adfcc745f669b)
    },
    {   UINT64_C(0x000000000000d109), UINT64_C(0x1162f38fa082668c), UINT64_C(0x08b2f1cfa05e868c),
        UINT64_C(0x1a0af1cfa0a445c1), UINT64_C(0x1162f34fa081e5c1)
    },
    {   UINT64_C(0x000000000000c028), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x0a6b028c745e05dd),
        UINT64_C(0x0fa2fb4d7077c634), UINT64_C(0x0fa2f34f607a2694)
    },
    {   UINT64_C(0x000000000000c029), UINT64_C(0x0fa2e78f78804606), UINT64_C(0x0fa2de0fa082e68c),
        UINT64_C(0x06e2fe8f5451c69b), UINT64_C(0x0fa2fc4d9075e69b)
    },
    {   UINT64_C(0x000000000000c02a), UINT64_C(0x0a6aed980468a682), UINT64_C(0x175af48f5498a69b),
        UINT64_C(0x0962edd5d4642698), UINT64_C(0x0fa2f3d03079a587)
    },
    {   UINT64_C(0x000000000000c02b), UINT64_C(0x1772fe5150942577), UINT64_C(0x0a6af9124c626507),
        UINT64_C(0x175af48f5498a69b), UINT64_C(0x0a6aee587467e665)
    },
    {   UINT64_C(0x000000000000c02c), UINT64_C(0x0c82fc55e8696698), UINT64_C(0x1812fe4d449425ee),
        UINT64_C(0x0962ee13ec640698), UINT64_C(0x0962eed3d063a698)
    },
    {   UINT64_C(0x000000000000c02d), UINT64_C(0x0a6b0397e06706b1), UINT64_C(0x1162f64c688185b5),
        UINT64_C(0x13fafd0cb886e68c), UINT64_C(0x07d2f8538058c589)
    },
    {   UINT64_C(0x00000000000007c1), UINT64_C(0x106af8cf8c804667), UINT64_C(0x106af8cf8c804667),
        UINT64_C(0x106af8cf8c80469e), UINT64_C(0x106af8cf8c80469e)
    },
    {   UINT64_C(0x000000000000c02f), UINT64_C(0x1772f50f709b6698), UINT64_C(0x1772ec4f709f6698),
        UINT64_C(0x1772f3cf709a2698), UINT64_C(0x0fa2fbcc7c75e5dd)
    },
    {   UINT64_C(0x32062cbe9780c030), UINT64_C(0x1692f04fa094a7ab), UINT64_C(0x1692f04fa094a7ab),
        UINT64_C(0x1692f04fa094a7a2), UINT64_C(0x1692f04fa094a784)
    },
    {   UINT64_C(0x361000d15950c031), UINT64_C(0x1692f04fa094a77c), UINT64_C(0x1692f04fa094a7d0),
        UINT64_C(0x1f42ef0fa0b5668c), UINT64_C(0x1692fa4c3894669b)
    },
    {   UINT64_C(0x000000000000c113), UINT64_C(0x0f92f34fa07ae5b5), UINT64_C(0x0e2af40f687425c1),
        UINT64_C(0x188af34f689da5c1), UINT64_C(0x1072f0cfa07e25b5)
    },
    {   UINT64_C(0x0000000000004c15), UINT64_C(0x1f431acfecaae70a), UINT64_C(0x1f43234ec89786c2),
        UINT64_C(0x1f43190ec8a7c71e), UINT64_C(0x1f431c0fe8a106ce)
    },
    {   UINT64_C(0x00000000000007c4), UINT64_C(0x106af8cf747a8667), UINT64_C(0x106af8cf747a8667),
        UINT64_C(0x106af8cf747a8667), UINT64_C(0x106af8cf747a8667)
    },
    {   UINT64_C(0x0006e0db5b50c035), UINT64_C(0x1852f6cf549b67a2), UINT64_C(0x1852f6cf549b6784),
        UINT64_C(0x1852f6cf549b677c), UINT64_C(0x15f2fe8f1c8e0602)
    },
    {   UINT64_C(0x3856fcdddbb0c036), UINT64_C(0x1852f6ce789b66c9), UINT64_C(0x1692f20f549627ab),
        UINT64_C(0x1692f20f549627ab), UINT64_C(0x1692f20f549627a2)
    },
    {   UINT64_C(0x000000001c30c037), UINT64_C(0x1692f20f5496277c), UINT64_C(0x1692f1cfa095e7ab),
        UINT64_C(0x131af20fa089868c), UINT64_C(0x1162f2cf707f269b)
    },
    {   UINT64_C(0x000000e45c70c038), UINT64_C(0x0fa2f44f687b86b1), UINT64_C(0x0fa2f44f687b86b1),
        UINT64_C(0x0fa2f44f687b86a8), UINT64_C(0x0fa2f44f687b868a)
    },
    {   UINT64_C(0x000734e5dca0c039), UINT64_C(0x0fa2f44f687b8682), UINT64_C(0x0fa2f44f687b86d8),
        UINT64_C(0x1692f64f689545c1), UINT64_C(0x0de2f44c387445c1)
    },
    {   UINT64_C(0x000740e74000c03a), UINT64_C(0x0de2f44f6873e6b1), UINT64_C(0x0de2f44f6873e6b1),
        UINT64_C(0x0de2f44f6873e6a8), UINT64_C(0x0de2f44f6873e682)
    },
    {   UINT64_C(0x000000000000c03b), UINT64_C(0x08b2f18fa05ea6b1), UINT64_C(0x08b2f1cfa05ec6b1),
        UINT64_C(0x08b2e6cfa06506a8), UINT64_C(0x08b2e6cfa0654682)
    },
    {   UINT64_C(0x3a6746000000c03c), UINT64_C(0x0fa2f44f687a069b), UINT64_C(0x1162f34fa081e68a),
        UINT64_C(0x0fa2f44f687a06b1), UINT64_C(0x0fa2f44f687a06b1)
    },
    {   UINT64_C(0x000754001d48c03d), UINT64_C(0x0fa2f44f687a06a8), UINT64_C(0x0fa2f44f687a068a),
        UINT64_C(0x0fa2f44f687a0682), UINT64_C(0x11d2f64f2481a601)
    },
    {   UINT64_C(0x000000000000c03e), UINT64_C(0x0fa2f44e307a060d), UINT64_C(0x1162f20f6881c6b1),
        UINT64_C(0x1162f20f6881c6b1), UINT64_C(0x1162f20f6881c6a8)
    },
    {   UINT64_C(0x000000000000c03f), UINT64_C(0x1162f20f6881c682), UINT64_C(0x0fa2f20c6c7ac6b1),
        UINT64_C(0x1162f2cc6c80068c), UINT64_C(0x0fa2f20c6c7ac682)
    },
    {   UINT64_C(0x000000000000c040), UINT64_C(0x1692f04fa094a712), UINT64_C(0x0fa2f44f687b8640),
        UINT64_C(0x1692f04fa094a76d), UINT64_C(0x0fa2f44f687b869b)
    },
    {   UINT64_C(0x000000000000c041), UINT64_C(0x1692f04c6c98a69a), UINT64_C(0x0fa2f44c6c85a5c1),
        UINT64_C(0x1692fa4f54946783), UINT64_C(0x0de2f44f687446b1)
    },
    {   UINT64_C(0x000000000000c042), UINT64_C(0x1692fa4f5494677a), UINT64_C(0x0de2f44f687446a8),
        UINT64_C(0x1692fa4f54946754), UINT64_C(0x0de2f44f68744682)
    },
    {   UINT64_C(0x000000000000c043), UINT64_C(0x1692fa4f5494677a), UINT64_C(0x0de2f44f687446a8),
        UINT64_C(0x1692f18fa094c77a), UINT64_C(0x1542f44f689746ad)
    },
    {   UINT64_C(0x000000000000c044), UINT64_C(0x1692ef8fa094c68c), UINT64_C(0x1162f44f6881468c),
        UINT64_C(0x14daf20fa08ea712), UINT64_C(0x0de2f44f6873e640)
    },
    {   UINT64_C(0x000000000000c045), UINT64_C(0x14daf20fa08ea76d), UINT64_C(0x0de2f44f6873e69b),
        UINT64_C(0x14daf20fa08ea754), UINT64_C(0x0de2f44f6873e682)
    },
    {   UINT64_C(0x000000000000c046), UINT64_C(0x14daf20c6c8ea68c), UINT64_C(0x0de2f44c6c74e5c1),
        UINT64_C(0x14daf20fa08ea77a), UINT64_C(0x0de2f44f6873e6a8)
    },
    {   UINT64_C(0x000000000000c047), UINT64_C(0x1852f74f549ce77a), UINT64_C(0x0fa2f50c687ae6a8),
        UINT64_C(0x1852f74f549ce76d), UINT64_C(0x0fa2f50c687ae69b)
    },
    {   UINT64_C(0x000000000000c048), UINT64_C(0x1852f74f549ce754), UINT64_C(0x0fa2f50c687ae682),
        UINT64_C(0x1852f749b89ce69b), UINT64_C(0x0fa2f50c687ae747)
    },
    {   UINT64_C(0x000000000000c049), UINT64_C(0x1852f34fa09d677a), UINT64_C(0x1162f1cfa081477a),
        UINT64_C(0x1852f34fa09d668c), UINT64_C(0x1162f1cfa081468c)
    },
    {   UINT64_C(0x000000000000c04a), UINT64_C(0x0c2af18fa06de75c), UINT64_C(0x08b2e24fa066468a),
        UINT64_C(0x0c2af30fa06cc712), UINT64_C(0x08b2e7cfa064a640)
    },
    {   UINT64_C(0x000000000000c04b), UINT64_C(0x0c2af30fa06cc76c), UINT64_C(0x08b2eb4fa063069b),
        UINT64_C(0x0c2af30c6c6f268c), UINT64_C(0x08b2f1cc6c66a69b)
    },
    {   UINT64_C(0x000000000000c04c), UINT64_C(0x0c2af30fa06cc754), UINT64_C(0x08b2f1cfa05e85b5),
        UINT64_C(0x1a32f30e20b1268c), UINT64_C(0x1142f1cc7480e69b)
    },
    {   UINT64_C(0x000000000000c04d), UINT64_C(0x0fa2eece207a677a), UINT64_C(0x0a6adfcc746866a8),
        UINT64_C(0x1852f589b89ea68c), UINT64_C(0x1162f389b882668c)
    },
    {   UINT64_C(0x00000000000007d1), UINT64_C(0x106af84f8c7ae667), UINT64_C(0x106af84f8c7ae667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c04f), UINT64_C(0x08b2f1c9b85e868c), UINT64_C(0x14daf2cfa08e469b),
        UINT64_C(0x0e4af1cfa075c6ad), UINT64_C(0x14daf2cfa08e468c)
    },
    {   UINT64_C(0x000000000000c050), UINT64_C(0x0cf2f1cfa075a68c), UINT64_C(0x14daf28fa08e468c),
        UINT64_C(0x08b2e88fa064668c), UINT64_C(0x1692f20f58962783)
    },
    {   UINT64_C(0x000000000000c051), UINT64_C(0x1162f34fa081e6b1), UINT64_C(0x1692f209b896268c),
        UINT64_C(0x1162f349b881e5c1), UINT64_C(0x1692f20f5896277a)
    },
    {   UINT64_C(0x000000000000c052), UINT64_C(0x1162f34fa081e6a8), UINT64_C(0x160af14fa09526ad),
        UINT64_C(0x16e2f18e2097868c), UINT64_C(0x1162f34c6c7bc5c1)
    },
    {   UINT64_C(0x000000000000c053), UINT64_C(0x1852f6cf549b6712), UINT64_C(0x0fa2f44f687a0640),
        UINT64_C(0x1852f6cf549b676d), UINT64_C(0x0fa2f44f687a069b)
    },
    {   UINT64_C(0x000000000000c054), UINT64_C(0x1852f6cf549b6783), UINT64_C(0x0fa2f44f687a06b1),
        UINT64_C(0x1f42f38f8cb92694), UINT64_C(0x1692f38f689585c1)
    },
    {   UINT64_C(0x000000000000c055), UINT64_C(0x1692f48fa0980783), UINT64_C(0x0de2f50fa074c6b1),
        UINT64_C(0x1692f489b898068c), UINT64_C(0x0de2f509b874c5c1)
    },
    {   UINT64_C(0x000000000000c056), UINT64_C(0x1692f48fa098077a), UINT64_C(0x0de2f50fa074c6a8),
        UINT64_C(0x1162f6cf547ea783), UINT64_C(0x0c2af44f686c06b1)
    },
    {   UINT64_C(0x000000000000c057), UINT64_C(0x1162f6cf547ea77a), UINT64_C(0x0c2af38f686ba6a8),
        UINT64_C(0x1162f6cc387ea69c), UINT64_C(0x0c2af44c386ba5c1)
    },
    {   UINT64_C(0x000000000000c058), UINT64_C(0x1162f6cf547ea77a), UINT64_C(0x0c2af38f686ba6a8),
        UINT64_C(0x14daf5cc388e068c), UINT64_C(0x0a6af2cc3868065e)
    },
    {   UINT64_C(0x000000000000c059), UINT64_C(0x14daf5cfa08e077a), UINT64_C(0x0cfaf2cf707626ad),
        UINT64_C(0x14daf5cfa08e068c), UINT64_C(0x0a6af24f7068065e)
    },
    {   UINT64_C(0x000000000000c05a), UINT64_C(0x1692f20f5496275e), UINT64_C(0x1162f20f6881c68a),
        UINT64_C(0x1692f20f54962712), UINT64_C(0x1162f20f6881c640)
    },
    {   UINT64_C(0x000000000000c05b), UINT64_C(0x1692f20f5496276d), UINT64_C(0x1162f20f6881c69b),
        UINT64_C(0x1692f20f5496278f), UINT64_C(0x1162f20f6881c6cc)
    },
    {   UINT64_C(0x00000000000007d8), UINT64_C(0x106af60f8c7c0667), UINT64_C(0x106af60f8c7c0667),
        UINT64_C(0x106af60f8c7c0667), UINT64_C(0x106af60f8c7c0667)
    },
    {   UINT64_C(0x000000000000c05d), UINT64_C(0x1f42f2cf64b9277a), UINT64_C(0x1692f3cf6896e6a8),
        UINT64_C(0x1692f1cfa095e77a), UINT64_C(0x0fa2f20c6c7ac6a8)
    },
    {   UINT64_C(0x000000000000c05e), UINT64_C(0x1692f1cfa095e754), UINT64_C(0x14daf50fa08dc783),
        UINT64_C(0x0de2f34fa07306b1), UINT64_C(0x14daf50fa08dc754)
    },
    {   UINT64_C(0x000000000000c05f), UINT64_C(0x0de2f34fa0730682), UINT64_C(0x14daf50fa08dc77a),
        UINT64_C(0x0de2f34fa07306a8), UINT64_C(0x0a6af18fa06f269b)
    },
    {   UINT64_C(0x000000000000c060), UINT64_C(0x114ae70f687f068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c2c), UINT64_C(0x1f43110fb8aca6eb), UINT64_C(0x1f4316cf34a186ac),
        UINT64_C(0x1f4316cf34a68708), UINT64_C(0x1f432091749d865b)
    },
    {   UINT64_C(0x000000000000c062), UINT64_C(0x11c2f58f68864628), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c2d), UINT64_C(0x1f43199174af0718), UINT64_C(0x1f4312cf70a866c1),
        UINT64_C(0x1f430c0f70ac870a), UINT64_C(0x1f431bd040a42696)
    },
    {   UINT64_C(0x000000000000c064), UINT64_C(0x1312f78f9087e696), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee0d347cc6aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c065), UINT64_C(0x0000000000000000), UINT64_C(0x18eaf18f789e668c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c066), UINT64_C(0x0000000000000000), UINT64_C(0x10aaf10fa081269b),
        UINT64_C(0x091af40fa05fa68c), UINT64_C(0x10c2fd0fa0804682)
    },
    {   UINT64_C(0x000000000000c067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x117af34c6c81e5c1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c068), UINT64_C(0x1852f6cf54a106c4), UINT64_C(0x1152f44f688685e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c069), UINT64_C(0x0000000000000000), UINT64_C(0x1132f08c6c7ee689),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c06a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0df2f44c10732699), UINT64_C(0x0adaf28c3867e65e)
    },
    {   UINT64_C(0x000000000000c06b), UINT64_C(0x0000000000000000), UINT64_C(0x0b9af2cf706f269b),
        UINT64_C(0x0000000000000000), UINT64_C(0x18e2f20f54a70712)
    },
    {   UINT64_C(0x000000000000f804), UINT64_C(0x1773020bf49246ce), UINT64_C(0x1773020a5492471b),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d14f), UINT64_C(0x145300cfa088e68c), UINT64_C(0x0ed300cfa088e68c),
        UINT64_C(0x177af94f5494469b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d150), UINT64_C(0x0be3020fa064068c), UINT64_C(0x1352fa8f5483068c),
        UINT64_C(0x18c300cfa09e668c), UINT64_C(0x14b2ff8fa08a668c)
    },
    {   UINT64_C(0x000000000000c06f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x107b018f7876a605), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c070), UINT64_C(0x05d2fd0fa04e46cc), UINT64_C(0x09cafd0fa05e26cc),
        UINT64_C(0x0deaf7cfa07126cc), UINT64_C(0x0a6b024f6c5de69b)
    },
    {   UINT64_C(0x000000000000f809), UINT64_C(0x0000000000000000), UINT64_C(0x1772feca3c93e73c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d154), UINT64_C(0x1462f3cfa09ae68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x137afa8f6884e5c1), UINT64_C(0x141afd0f6888a68c)
    },
    {   UINT64_C(0x000000000000d155), UINT64_C(0x11f2fa8f688005c1), UINT64_C(0x141afa8f6887668c),
        UINT64_C(0x11eafa8f687fe5c1), UINT64_C(0x0ed2f44fa081868c)
    },
    {   UINT64_C(0x000000000000d156), UINT64_C(0x141afa8c6c8765c1), UINT64_C(0x1382fd0fa085068c),
        UINT64_C(0x0a52fe4fa05fa69b), UINT64_C(0x0b9ae90c6c65669b)
    },
    {   UINT64_C(0x000000000000d157), UINT64_C(0x1432fd0fa08c268c), UINT64_C(0x0a2afd0fa05fa68c),
        UINT64_C(0x1c62fd0fa0a885c1), UINT64_C(0x1382fd0fa08505c1)
    },
    {   UINT64_C(0x000000000000d158), UINT64_C(0x12b2f94f6883a5c1), UINT64_C(0x141afd0c6c88a5c1),
        UINT64_C(0x141afa8c6c8765c1), UINT64_C(0x0a9afd0fa07625c1)
    },
    {   UINT64_C(0x000000000000d159), UINT64_C(0x112afb0f687ca5c1), UINT64_C(0x0df2f44f6871e68c),
        UINT64_C(0x135afbcf608465b5), UINT64_C(0x0f72ef8fa08285b5)
    },
    {   UINT64_C(0x000000000000d15a), UINT64_C(0x1892f38fa0a125b5), UINT64_C(0x14a2f44fa08de5b5),
        UINT64_C(0x0ebaebcc6c81a5b5), UINT64_C(0x100af80fa079a5b5)
    },
    {   UINT64_C(0x00000000000049de), UINT64_C(0x1f42f98de8b5c705), UINT64_C(0x1f42f98de8b5c705),
        UINT64_C(0x1f42f98de8b5c705), UINT64_C(0x1f42f98de8b5c705)
    },
    {   UINT64_C(0x000000000000d15c), UINT64_C(0x18a2fb8f5499e6a5), UINT64_C(0x191afb8c6c9c06a5),
        UINT64_C(0x1a0afb8f54a026a5), UINT64_C(0x1a5af88c6ca1e6a5)
    },
    {   UINT64_C(0x000000000000f813), UINT64_C(0x0000000000000000), UINT64_C(0x1772ff0ee493e611),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c07c), UINT64_C(0x0a6adfcc746866a8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d15f), UINT64_C(0x1d12fb8eb4af46a5), UINT64_C(0x1ba2fa8f3ca7a6a5),
        UINT64_C(0x1aa2fb8f54a2c6a5), UINT64_C(0x18cafb8f3c9b06a5)
    },
    {   UINT64_C(0x000000000000c07e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f04fa094a7b4), UINT64_C(0x0fa2f44f687b8783)
    },
    {   UINT64_C(0x000000000000f817), UINT64_C(0x1772ff0f2493e611), UINT64_C(0x0000000000000000),
        UINT64_C(0x000305d87473a6e2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f818), UINT64_C(0x0000000000000000), UINT64_C(0x0002ee08cc67c35f),
        UINT64_C(0x0002ee070067c35f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f819), UINT64_C(0x0a6aea18746a272d), UINT64_C(0x0002ea093c6a235f),
        UINT64_C(0x000304d874744709), UINT64_C(0x0002ea093c6a235f)
    },
    {   UINT64_C(0x000000000000d164), UINT64_C(0x0e42fa0f4070c69f), UINT64_C(0x0d8af94f406de69f),
        UINT64_C(0x1ec2f80f40b305c3), UINT64_C(0x15c2f80f408f05c3)
    },
    {   UINT64_C(0x000000000000f81b), UINT64_C(0x0000000000000000), UINT64_C(0x1773028ddc92072b),
        UINT64_C(0x1773028ddc92072b), UINT64_C(0x1773028ddc92072b)
    },
    {   UINT64_C(0x000000000000f81c), UINT64_C(0x1773028ddc92072b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c3e), UINT64_C(0x1f43004ea8b48704), UINT64_C(0x1f430010dcb64705),
        UINT64_C(0x1f43064f24b6e706), UINT64_C(0x1f4353143c8405a8)
    },
    {   UINT64_C(0x00000000000007ed), UINT64_C(0x106af8ccc080464c), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af8cf8c804667), UINT64_C(0x106af8ccc0804667)
    },
    {   UINT64_C(0x000000000000f81f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1773020c089246ca), UINT64_C(0x1773020bf89246ce)
    },
    {   UINT64_C(0x00000000000007ee), UINT64_C(0x106aee0fa0800729), UINT64_C(0x106aedcfa07fe6c8),
        UINT64_C(0x0000000000000000), UINT64_C(0x106af40fa07f4673)
    },
    {   UINT64_C(0x000000000000f821), UINT64_C(0x1773020c089246ca), UINT64_C(0x1773020bf89246ce),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007ef), UINT64_C(0x106af28d847da673), UINT64_C(0x106b26173468c667),
        UINT64_C(0x106b200d8467046f), UINT64_C(0x106b26173468c667)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000007f0), UINT64_C(0x106b03d7fc750667), UINT64_C(0x106b03d74c75069e),
        UINT64_C(0x106af2cbfc7ac64c), UINT64_C(0x106af2cbfc7ac5a2)
    },
    {   UINT64_C(0x000000000000c08d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6adfcc745f05b5)
    },
    {   UINT64_C(0x00000000000007f1), UINT64_C(0x106af2cbfc7ac64c), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af2cbfc7ac667), UINT64_C(0x106af2cbfc7ac667)
    },
    {   UINT64_C(0x0000000000004c43), UINT64_C(0x1f4311cec0ab2700), UINT64_C(0x1f431b0ee8a6a706),
        UINT64_C(0x1f43064f68b106f7), UINT64_C(0x1f4300cedcb16700)
    },
    {   UINT64_C(0x00000000000007f2), UINT64_C(0x0000000000000000), UINT64_C(0x106ad90fa0800667),
        UINT64_C(0x0000000000000000), UINT64_C(0x106ad90fa0800667)
    },
    {   UINT64_C(0x0000000000004c44), UINT64_C(0x1f43280e7c98673a), UINT64_C(0x1f4320cf089fe6ff),
        UINT64_C(0x1f43000f30b286e5), UINT64_C(0x1f42f40f5cb386cc)
    },
    {   UINT64_C(0x0000000000005d52), UINT64_C(0x1f42f9102cb5c6c0), UINT64_C(0x1f42f58fd0b7c6e1),
        UINT64_C(0x1f42f84ed8b66742), UINT64_C(0x1f42fd0ff0b68714)
    },
    {   UINT64_C(0x000000000000d175), UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c),
        UINT64_C(0x169afa4f5494869b), UINT64_C(0x162b020fa092c68c)
    },
    {   UINT64_C(0x00000000000007f4), UINT64_C(0x106ade0f8c780670), UINT64_C(0x106ade0f8c78060f),
        UINT64_C(0x106b020f8c6b464c), UINT64_C(0x106b020f8c6b464c)
    },
    {   UINT64_C(0x000000000000c095), UINT64_C(0x0ec2f1cf687305c1), UINT64_C(0x0eeaf58d1c7625c1),
        UINT64_C(0x12d2f60bfc91068c), UINT64_C(0x1452f60f68910693)
    },
    {   UINT64_C(0x000000000000d178), UINT64_C(0x1c8b020fa0a6a68c), UINT64_C(0x16a3020f588f068c),
        UINT64_C(0x1852f6cf589b669c), UINT64_C(0x122b020fa085a68c)
    },
    {   UINT64_C(0x000000000000d179), UINT64_C(0x185af6cce09cc69b), UINT64_C(0x14f3020fa090a68c),
        UINT64_C(0x1152f6cf547e669b), UINT64_C(0x140af18fa08ce68c)
    },
    {   UINT64_C(0x000000000000c098), UINT64_C(0x139af74bcc914693), UINT64_C(0x11daf74bcc7b05c1),
        UINT64_C(0x1272f6cf7483e5b5), UINT64_C(0x10a2f40bfc7de5b5)
    },
    {   UINT64_C(0x000000000000d17b), UINT64_C(0x15b2f30fa092c68c), UINT64_C(0x146af44fa08c668c),
        UINT64_C(0x0f4af3cf687405c1), UINT64_C(0x1002fbcf6c7b668c)
    },
    {   UINT64_C(0x000000000000c09a), UINT64_C(0x0922f34fa05f469b), UINT64_C(0x0b72f38f8068e5b0),
        UINT64_C(0x094af48fa05f85b5), UINT64_C(0x0c5af1cfa06e068c)
    },
    {   UINT64_C(0x000000000000d17d), UINT64_C(0x0fc2f50c687bc5c1), UINT64_C(0x0ff2fbcfa077668c),
        UINT64_C(0x07aafa8fa056e698), UINT64_C(0x0a22dfcc74602698)
    },
    {   UINT64_C(0x000000000000d17e), UINT64_C(0x103afbcfa07dc68c), UINT64_C(0x075afbcfa055068c),
        UINT64_C(0x1842fbcfa098a5c1), UINT64_C(0x0ff2fbcfa07765c1)
    },
    {   UINT64_C(0x000000000000d17f), UINT64_C(0x0faaf44f687a05c1), UINT64_C(0x1002fbcc6c7b65c1),
        UINT64_C(0x1002f44c6c77a5c1), UINT64_C(0x0cdafbcfa07145c1)
    },
    {   UINT64_C(0x000000000000c09e), UINT64_C(0x164af94c0491e695), UINT64_C(0x0e42f6cf7875a5b7),
        UINT64_C(0x0e42f6cf7875a68c), UINT64_C(0x0fd2f6cbfc8505b7)
    },
    {   UINT64_C(0x000000000000d181), UINT64_C(0x15eaf24f689405b5), UINT64_C(0x0fd2f30fa07b45b5),
        UINT64_C(0x0ec2f28c6c7765b5), UINT64_C(0x0dcaf34fa072a5b5)
    },
    {   UINT64_C(0x000000000000f838), UINT64_C(0x1142f88fa07e4780), UINT64_C(0x0bd2fbcfa066e6e5),
        UINT64_C(0x0f0afdcfa074e6da), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0a1), UINT64_C(0x0baadfcbfc744693), UINT64_C(0x0c62f00bfc7705be),
        UINT64_C(0x0c7aea0bd476a693), UINT64_C(0x0a6af2cd9c6805e9)
    },
    {   UINT64_C(0x00000000000055fa), UINT64_C(0x0000000000000000), UINT64_C(0x1f43008ec4b44711),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d63), UINT64_C(0x1f430cce78b3a720), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb0ee8b50703), UINT64_C(0x1f42fc8fa8b70712)
    },
    {   UINT64_C(0x0000000000005d64), UINT64_C(0x1f43090e2cb0473d), UINT64_C(0x1f4300d004b66703),
        UINT64_C(0x1f42fb0f94a0c71a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0a5), UINT64_C(0x0d0afbcfa073c693), UINT64_C(0x0d0afbcfa073c693),
        UINT64_C(0x0d0afbcf6873c685), UINT64_C(0x0efb000c007665c1)
    },
    {   UINT64_C(0x00000000000007fd), UINT64_C(0x106af60cc07c064c), UINT64_C(0x0000000000000000),
        UINT64_C(0x106af60f8c7c0667), UINT64_C(0x106af60cc07c0667)
    },
    {   UINT64_C(0x000000000000c0a7), UINT64_C(0x1232f34fa084a5b8), UINT64_C(0x0c52df0bfc72469b),
        UINT64_C(0x1162f38c448265b5), UINT64_C(0x0eb2f24fa07585b8)
    },
    {   UINT64_C(0x000000000000c0a8), UINT64_C(0x13e2f68c6c90e62e), UINT64_C(0x0fa2fbcfa073c693),
        UINT64_C(0x0fa2fbcfa073c693), UINT64_C(0x1b22f44f68a7e68c)
    },
    {   UINT64_C(0x000000000000c0a9), UINT64_C(0x1952f44bf0a1c68c), UINT64_C(0x1e02f44d10b3268c),
        UINT64_C(0x1392f2cf7089465e), UINT64_C(0x10e2f2cbfc8cc693)
    },
    {   UINT64_C(0x000000000000c0aa), UINT64_C(0x15e2f2cf7094c65e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000f843), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x165322d1347d657d), UINT64_C(0x09daee114c65e577)
    },
    {   UINT64_C(0x000000000000c0ac), UINT64_C(0x0bd2ef94dc6c2732), UINT64_C(0x0bd2efd4e06c2738),
        UINT64_C(0x0962f4526c60a74c), UINT64_C(0x0872f114dc5fc6a3)
    },
    {   UINT64_C(0x000000000000c0ad), UINT64_C(0x0922f094b85f869a), UINT64_C(0x09faf0922c69469a),
        UINT64_C(0x0ca2f31504702694), UINT64_C(0x1132f3d4cc808691)
    },
    {   UINT64_C(0x000000000000c0ae), UINT64_C(0x0be2f2126c6d469a), UINT64_C(0x0682f59634598694),
        UINT64_C(0x0b22f2d6346c0694), UINT64_C(0x0a6af7d5185a6695)
    },
    {   UINT64_C(0x000000000000c0af), UINT64_C(0x0a6af7d51c5a6696), UINT64_C(0x07d2f7d51c5a6696),
        UINT64_C(0x0a72fe55985dc67e), UINT64_C(0x0a72fe55985dc67e)
    },
    {   UINT64_C(0x000000000000c0b0), UINT64_C(0x0b0af5936068269a), UINT64_C(0x0b0af5936068269a),
        UINT64_C(0x09eaf5961863a6e0), UINT64_C(0x09eaf5961863a6e0)
    },
    {   UINT64_C(0x000000000000c0b1), UINT64_C(0x09eaea96b46806b8), UINT64_C(0x09eaea96b46806b8),
        UINT64_C(0x0a6aed97e06866a8), UINT64_C(0x0a6aed97e06866a8)
    },
    {   UINT64_C(0x000000000000c0b2), UINT64_C(0x08bb0a16bc5366b8), UINT64_C(0x0b92f318746a4665),
        UINT64_C(0x0852f317e05ec6b1), UINT64_C(0x0852f317e05ec6b1)
    },
    {   UINT64_C(0x000000000000c0b3), UINT64_C(0x08b30a0c9453642e), UINT64_C(0x0b92f30d206a4390),
        UINT64_C(0x0a6af1cca05e23e1), UINT64_C(0x0a6b020ca06643e1)
    },
    {   UINT64_C(0x000000000000c0b4), UINT64_C(0x0a6b01cfb05e45c2), UINT64_C(0x0a6b01d4385e45c2),
        UINT64_C(0x0a6afe5098600562), UINT64_C(0x0a6afe5098600562)
    },
    {   UINT64_C(0x000000000000c0b5), UINT64_C(0x0a6afacfb061a4b6), UINT64_C(0x0a6afacfb061a4b6),
        UINT64_C(0x0a72fd4fc460a4cb), UINT64_C(0x0a72fd50f060a47e)
    },
    {   UINT64_C(0x000000000000c0b6), UINT64_C(0x0a6af1d7e066469b), UINT64_C(0x0a6b0798045b6682),
        UINT64_C(0x0a6afd180460a6d6), UINT64_C(0x0a6b048cec666414)
    },
    {   UINT64_C(0x000000000000c0b7), UINT64_C(0x0a6aea182c6a268a), UINT64_C(0x0a6aead7e073a6b1),
        UINT64_C(0x0922ee12886445c9), UINT64_C(0x0822f097885dc6d0)
    },
    {   UINT64_C(0x000000000000c0b8), UINT64_C(0x0d22f0929871c694), UINT64_C(0x05f2ef54dc55c732),
        UINT64_C(0x0962f2d4a061469f), UINT64_C(0x0be2f114dc6d869a)
    },
    {   UINT64_C(0x000000000000c0b9), UINT64_C(0x0a42f314dc66e73f), UINT64_C(0x0caaf80fa06c868c),
        UINT64_C(0x0caaf80fa06c868c), UINT64_C(0x0caaf80fa06c868c)
    },
    {   UINT64_C(0x0000000000005d7a), UINT64_C(0x1f42fc4ec8aae717), UINT64_C(0x1f42fa0eacb58756),
        UINT64_C(0x1f42f90f54af6722), UINT64_C(0x1f42fb4f28a6c713)
    },
    {   UINT64_C(0x0000000000005d7b), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68e74ab6735),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38ea0ae2735)
    },
    {   UINT64_C(0x0000000000005d7c), UINT64_C(0x1f42f9cf14b1e732), UINT64_C(0x1f42fb8ebcadc731),
        UINT64_C(0x1f42f64eccb42738), UINT64_C(0x1f42f88edcb6c738)
    },
    {   UINT64_C(0x0000000000005d7d), UINT64_C(0x1f42f74f94b18726), UINT64_C(0x1f42fb4e4cb14731),
        UINT64_C(0x1f42f84ee4b34731), UINT64_C(0x1f42f38e50baa72d)
    },
    {   UINT64_C(0x0000000000005d7e), UINT64_C(0x1f42f98ea0abe72d), UINT64_C(0x1f42f30f08ace72a),
        UINT64_C(0x1f42f9ce3cb1c73a), UINT64_C(0x1f42f38e14afa735)
    },
    {   UINT64_C(0x0000000000005d7f), UINT64_C(0x1f42f4cea8a6272c), UINT64_C(0x1f42f38e8cb1e735),
        UINT64_C(0x1f42fb4ed0b42732), UINT64_C(0x1f42f84eb8ac6736)
    },
    {   UINT64_C(0x000000000000c0c0), UINT64_C(0x000291d7e02e26b1), UINT64_C(0x000291d7e02e26b1),
        UINT64_C(0x00028597e03466a8), UINT64_C(0x000282182c36268a)
    },
    {   UINT64_C(0x0000000000005d81), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94ec8ae672f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d82), UINT64_C(0x1f42f8cf30b24729), UINT64_C(0x1f42fa8e6cb2c739),
        UINT64_C(0x1f42f70f18b14725), UINT64_C(0x1f42fb4ee4b40726)
    },
    {   UINT64_C(0x0000000000005d83), UINT64_C(0x1f42f70ea8a92731), UINT64_C(0x1f42fb0ea4b1673e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f38e84b0e730)
    },
    {   UINT64_C(0x0000000000005d84), UINT64_C(0x1f42f38eb8b24735), UINT64_C(0x1f42f98eacaaa725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f98f94b28732)
    },
    {   UINT64_C(0x000000000000c0c5), UINT64_C(0x00029897f02ac74b), UINT64_C(0x0002d117f047074b),
        UINT64_C(0x000291cba42e23a2), UINT64_C(0x000291cba42e23a2)
    },
    {   UINT64_C(0x000000000000c0c6), UINT64_C(0x000283cb2022a3ae), UINT64_C(0x0002a60b2033c3ae),
        UINT64_C(0x00028797ec32e6d8), UINT64_C(0x0002de952046c5e1)
    },
    {   UINT64_C(0x000000000000c0c7), UINT64_C(0x0002904abc26c3c7), UINT64_C(0x000292cb602da3a2),
        UINT64_C(0x000292cb602da3a2), UINT64_C(0x0002950b242c83a2)
    },
    {   UINT64_C(0x000000000000d1aa), UINT64_C(0x16faf04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x1362f20fa088a68c), UINT64_C(0x159af20fa090869a)
    },
    {   UINT64_C(0x000000000000d1ab), UINT64_C(0x153af20fa08ea68c), UINT64_C(0x15aaf50fa08dc68c),
        UINT64_C(0x1952f34fa09d668c), UINT64_C(0x1852f6cf589b669c)
    },
    {   UINT64_C(0x000000000000c0ca), UINT64_C(0x0002930cec2da414), UINT64_C(0x0002a8cbe422c37d),
        UINT64_C(0x000283cb9032e392), UINT64_C(0x0002850bd832a380)
    },
    {   UINT64_C(0x000000000000d1ad), UINT64_C(0x1692f20f5896268c), UINT64_C(0x137af50fa088e68c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x1862f34fa09d668c)
    },
    {   UINT64_C(0x000000000000c0cc), UINT64_C(0x0002820bf436237c), UINT64_C(0x0002864c5c33e35f),
        UINT64_C(0x0002710cb43e835f), UINT64_C(0x0002710b243e835f)
    },
    {   UINT64_C(0x0000000000005d8d), UINT64_C(0x1f42f44e3cace740), UINT64_C(0x1f42f88f44b4272e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d8e), UINT64_C(0x1f42f60ea4a62731), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f94f28aae737), UINT64_C(0x1f42fc0ea8b4871e)
    },
    {   UINT64_C(0x0000000000005d8f), UINT64_C(0x1f42f9cf54b02721), UINT64_C(0x1f42fa8f20aa671a),
        UINT64_C(0x1f42f70f64b22728), UINT64_C(0x1f42fa4ec4b0c72a)
    },
    {   UINT64_C(0x0000000000005d90), UINT64_C(0x1f42f84ed0a98726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa0e90b4673a), UINT64_C(0x1f42f7cf58b4e726)
    },
    {   UINT64_C(0x0000000000005d91), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84ea4b0873c), UINT64_C(0x1f42f80ef8a7e729)
    },
    {   UINT64_C(0x000000000000d1b4), UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x13baf30f588aa5b5),
        UINT64_C(0x111af94c6c7ee5c1), UINT64_C(0x0e82f3cc747605c1)
    },
    {   UINT64_C(0x0000000000005d93), UINT64_C(0x1f42fc4ecca7672d), UINT64_C(0x1f42f90fc0b06715),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000d1b6), UINT64_C(0x1322f34c6c87c5c1), UINT64_C(0x17daf10c6c9c65c1),
        UINT64_C(0x16f2f48f689705c1), UINT64_C(0x101af44f687be6ab)
    },
    {   UINT64_C(0x0000000000005d95), UINT64_C(0x1f42fbcf3cada726), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f84e7cb0873c), UINT64_C(0x1f42f80eb4ad8730)
    },
    {   UINT64_C(0x000000000000c0d6), UINT64_C(0x0000000000000000), UINT64_C(0x00029449e03363a6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d97), UINT64_C(0x1f42f5cf50b32721), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88f1cac0735), UINT64_C(0x1f42fb4eccb3c727)
    },
    {   UINT64_C(0x000000000000c0d8), UINT64_C(0x00028217e470468a), UINT64_C(0x00028957f85fa69b),
        UINT64_C(0x0002820b0c6da3b2), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d99), UINT64_C(0x1f42f58e6caa8732), UINT64_C(0x1f42f78f04b72725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f30eb0b8e742)
    },
    {   UINT64_C(0x0000000000005d9a), UINT64_C(0x1f42fbcef0b4a742), UINT64_C(0x1f42f90f18ab6735),
        UINT64_C(0x1f42fc4eb4b2073a), UINT64_C(0x1f42f54ef4afa72e)
    },
    {   UINT64_C(0x00000000000149c1), UINT64_C(0x1ea2f6cfa0b4a69c), UINT64_C(0x0000000000000000),
        UINT64_C(0x18aaf6cf689ce6a9), UINT64_C(0x18b2f6cf689ce6a9)
    },
    {   UINT64_C(0x0000000000005617), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8eb0b4c71e),
        UINT64_C(0x1f43054eecb6a738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149c2), UINT64_C(0x18baf6cf6c9da6a9), UINT64_C(0x1592f6d1c8906611),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001bec1), UINT64_C(0x19f2f30fa0a20693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005d9f), UINT64_C(0x1f42fa0f68b78747), UINT64_C(0x1f43194ffca6c70d),
        UINT64_C(0x1f430d5008ae66e2), UINT64_C(0x1f430b0f9cb586e7)
    },
    {   UINT64_C(0x0003e8740e314016), UINT64_C(0x14dae80fa0954685), UINT64_C(0x131affcfa090e685),
        UINT64_C(0x131aeb0fa0882685), UINT64_C(0x0a6ae4cd246bc68a)
    },
    {   UINT64_C(0x0000000000005da1), UINT64_C(0x1f430ecf4cac472b), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f431a0ee4b1872c), UINT64_C(0x1f43040ee8b18728)
    },
    {   UINT64_C(0x0000000000005da2), UINT64_C(0x1f42ff8fb8b6a6de), UINT64_C(0x1f4305508cb566de),
        UINT64_C(0x1f4305cee8baa6ff), UINT64_C(0x1f42f80e40b46747)
    },
    {   UINT64_C(0x0000000000005da3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43044fc8b7c71e), UINT64_C(0x1f43094ed8b0c711)
    },
    {   UINT64_C(0x0000000000005da4), UINT64_C(0x1f430a0e2cade714), UINT64_C(0x1f43048f60b28712),
        UINT64_C(0x1f43084f88b1a6f2), UINT64_C(0x1f4306cf4cb3e710)
    },
    {   UINT64_C(0x0000000000005da5), UINT64_C(0x0000000000000000), UINT64_C(0x1f43088fb8afe6c0),
        UINT64_C(0x1f43018f88b38706), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0e6), UINT64_C(0x1852f6cf589b669c), UINT64_C(0x0c2af30fa06cc68c),
        UINT64_C(0x1852f58fa09ea68c), UINT64_C(0x161af04fa092c69a)
    },
    {   UINT64_C(0x000000000000c0e7), UINT64_C(0x1d82f18fa0b1a68c), UINT64_C(0x1692f20f5896268c),
        UINT64_C(0x143af80fa08a668c), UINT64_C(0x1852f6cf549b669b)
    },
    {   UINT64_C(0x000000000000c0e8), UINT64_C(0x1852f34fa09d668c), UINT64_C(0x131af20fa089868c),
        UINT64_C(0x0000000000000000), UINT64_C(0x14faf50fa08ea68c)
    },
    {   UINT64_C(0x000000000000c0e9), UINT64_C(0x14daf5cfa08e068c), UINT64_C(0x15faefcfa095269c),
        UINT64_C(0x1a22f28fa0a4c68c), UINT64_C(0x1692f20fa095e68c)
    },
    {   UINT64_C(0x000000000000c0ea), UINT64_C(0x1942f1cfa0a2269c), UINT64_C(0x1802f50fa09b069c),
        UINT64_C(0x0c2af30fa06cc77b), UINT64_C(0x15faefcfa095277b)
    },
    {   UINT64_C(0x000000000000c0eb), UINT64_C(0x1422f44f688bc69c), UINT64_C(0x0de2f50f6874269c),
        UINT64_C(0x124af10c6c82a69c), UINT64_C(0x0a32f1cf6864869c)
    },
    {   UINT64_C(0x000000000000c0ec), UINT64_C(0x1202f10f6883669c), UINT64_C(0x1422f44f688bc5c1),
        UINT64_C(0x1162f94c6c80069c), UINT64_C(0x1032f10c6c7d25c1)
    },
    {   UINT64_C(0x000000000000c0ed), UINT64_C(0x0fb2f48f687a269c), UINT64_C(0x0de2f50f687425c1),
        UINT64_C(0x0e5af3cc6c75269c), UINT64_C(0x124af10c6c82a5c1)
    },
    {   UINT64_C(0x000000000000c0ee), UINT64_C(0x0faaf44f687a069c), UINT64_C(0x0a32f1cf686485b5),
        UINT64_C(0x122af34fa08465c1), UINT64_C(0x111af2cf5880669c)
    },
    {   UINT64_C(0x000000000000c0ef), UINT64_C(0x1312f94c6c8805b5), UINT64_C(0x1032f1cf687c65c1),
        UINT64_C(0x0ea2f3cc6c75669c), UINT64_C(0x0fa2f44f687a05c1)
    },
    {   UINT64_C(0x000000000000c0f0), UINT64_C(0x13baf30f588aa5b5), UINT64_C(0x111af94c6c7ee5c1),
        UINT64_C(0x0e82f3cc747605c1), UINT64_C(0x11c2f44f6880a5b5)
    },
    {   UINT64_C(0x000000000000c0f1), UINT64_C(0x0f62f28f6877a5b5), UINT64_C(0x1202f10f688365c1),
        UINT64_C(0x146af40c6c8d25c1), UINT64_C(0x1322f34c6c87c5c1)
    },
    {   UINT64_C(0x000000000000c0f2), UINT64_C(0x17daf10c6c9c65c1), UINT64_C(0x16eaf48f689705c1),
        UINT64_C(0x0a7aed8f6868a682), UINT64_C(0x1202f10f68836682)
    },
    {   UINT64_C(0x000000000000d1d5), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1592f44fa091a69a), UINT64_C(0x13e3020fa088c68c)
    },
    {   UINT64_C(0x000000000000c0f4), UINT64_C(0x0fa2fb8f787846a1), UINT64_C(0x143af10f688c069c),
        UINT64_C(0x173aefcfa09a269c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001800a), UINT64_C(0x0a6afa0cdc64868c), UINT64_C(0x0a6af54cdc62268c),
        UINT64_C(0x0fa2ff13c474a68c), UINT64_C(0x156afa0efc8e060b)
    },
    {   UINT64_C(0x000000000000c0f6), UINT64_C(0x1852f6cc6c9b669b), UINT64_C(0x0fa2f44c6c7a05c1),
        UINT64_C(0x1542f94c548f269b), UINT64_C(0x0fc2f3cc747ae5df)
    },
    {   UINT64_C(0x000000000000c0f7), UINT64_C(0x1362f20fa088a68c), UINT64_C(0x0f5af94c6c77c5b5),
        UINT64_C(0x17aaf6cf6897e705), UINT64_C(0x0f2af54fa077269c)
    },
    {   UINT64_C(0x000000000000d1da), UINT64_C(0x1512f50fa08d868c), UINT64_C(0x140af18fa08ce68c),
        UINT64_C(0x1762f30fa099869b), UINT64_C(0x1902f44fa09f668c)
    },
    {   UINT64_C(0x000000000000d1db), UINT64_C(0x1722f80fa096068c), UINT64_C(0x1932f1cfa0a1669b),
        UINT64_C(0x1782f60fa098869b), UINT64_C(0x16faf94f28942680)
    },
    {   UINT64_C(0x000000000001800f), UINT64_C(0x156afc0f408c25fe), UINT64_C(0x156afa11808e256a),
        UINT64_C(0x156afc0f408c25fe), UINT64_C(0x0de2ff0f8072468c)
    },
    {   UINT64_C(0x000000000000d1dd), UINT64_C(0x0e1af44f6874c5c1), UINT64_C(0x0fa2f50c6c7b869c),
        UINT64_C(0x0ff2f6cc6c79c5c1), UINT64_C(0x0fa2f3cf687a269c)
    },
    {   UINT64_C(0x0810c00002998011), UINT64_C(0x1692f20fa094267e), UINT64_C(0x131af10fa089267e),
        UINT64_C(0x1162f0cfa082c67e), UINT64_C(0x1692f60f6897268c)
    },
    {   UINT64_C(0x000000000000c0fd), UINT64_C(0x1852f6cf549b669b), UINT64_C(0x0de2f44f687445c1),
        UINT64_C(0x0de2f18f6872e5c1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0da1800005318013), UINT64_C(0x131af10fa089467e), UINT64_C(0x1bcaf10fa0aa867e),
        UINT64_C(0x1692f10f7496e67e), UINT64_C(0x1692f68f6894868c)
    },
    {   UINT64_C(0x000000000000d1e1), UINT64_C(0x1012f6cf687ba5c1), UINT64_C(0x13d2f44c6c8aa5c1),
        UINT64_C(0x11eaf04c6c83a5c1), UINT64_C(0x1542f44c6c95e5c1)
    },
    {   UINT64_C(0x000000000000d1e2), UINT64_C(0x1722f88f6894a5b5), UINT64_C(0x0faaf44f687a069b),
        UINT64_C(0x0e72f44f687585c1), UINT64_C(0x10c2f6cf687fc69c)
    },
    {   UINT64_C(0x000000000000c101), UINT64_C(0x156af74f548e469b), UINT64_C(0x1162f6cf547ea69c),
        UINT64_C(0x0c2af30fa06cc68c), UINT64_C(0x0c2af30fa06cc77c)
    },
    {   UINT64_C(0x000000000000f89a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105afb4fa079458d)
    },
    {   UINT64_C(0x000000000000c103), UINT64_C(0x17baf34fa09b2783), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f1cf4895e786), UINT64_C(0x1812f34ce09ca68c)
    },
    {   UINT64_C(0x000000000000f89c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x105af88c887c858c)
    },
    {   UINT64_C(0x000000000000c105), UINT64_C(0x165af40ce094e68c), UINT64_C(0x14daf20fa08ea68c),
        UINT64_C(0x2352f60fa0c6e694), UINT64_C(0x11d2f38f5480e69b)
    },
    {   UINT64_C(0x000000000000562c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f94f98b6a705),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc8ec4b1073c)
    },
    {   UINT64_C(0x000000000000c107), UINT64_C(0x1d82f18fa0b1a68c), UINT64_C(0x1852f34fa09d668c),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x17d2f14fa09c668c)
    },
    {   UINT64_C(0x000000000001402a), UINT64_C(0x0a6afbd7d4702677), UINT64_C(0x175af58f58984695),
        UINT64_C(0x0852f215dc67c695), UINT64_C(0x0fa2f11020790587)
    },
    {   UINT64_C(0x000000000000c109), UINT64_C(0x1a92f44fa0a5a68c), UINT64_C(0x1692f20fa095e68c),
        UINT64_C(0x1812f34ce09ca68c), UINT64_C(0x16e2efcfa097468c)
    },
    {   UINT64_C(0x000000000000c10a), UINT64_C(0x1fe2f34fa0bba68c), UINT64_C(0x1fe2f34ce0bba68c),
        UINT64_C(0x192af84fa09d268c), UINT64_C(0x1f62f20fa0b9a68c)
    },
    {   UINT64_C(0x000000000000c10b), UINT64_C(0x1502f2cfa08c868c), UINT64_C(0x156af74f548f869b),
        UINT64_C(0x1ddaf34f54b1869b), UINT64_C(0x1702f10fa094668c)
    },
    {   UINT64_C(0x000000000000f8a4), UINT64_C(0x13caedcfa08dc497), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c10d), UINT64_C(0x10eaf2cd648005b5), UINT64_C(0x0de2f44f6873e5c1),
        UINT64_C(0x17d2f18fa09c05bb), UINT64_C(0x0df2f44f5c7305c1)
    },
    {   UINT64_C(0x000000000001402d), UINT64_C(0x0a6b10d7b06de6a1), UINT64_C(0x1202df0c647f05a9),
        UINT64_C(0x134afd0c9c936685), UINT64_C(0x07d2fad3a457657d)
    },
    {   UINT64_C(0x000000000000c10f), UINT64_C(0x148af30fa08e85b5), UINT64_C(0x1182f34fa08265b5),
        UINT64_C(0x0fa2f44f687a05c1), UINT64_C(0x1162f34fa08265b5)
    },
    {   UINT64_C(0x000000000000c110), UINT64_C(0x1162f2cc6c8005c1), UINT64_C(0x0de2f44f687445c1),
        UINT64_C(0x0feaf28fa07c25b5), UINT64_C(0x1042f20c6c7d45b5)
    },
    {   UINT64_C(0x000000000000c111), UINT64_C(0x16b2f5cc6c95268c), UINT64_C(0x0fa2f10fa07b05b5),
        UINT64_C(0x1162f34d648265b5), UINT64_C(0x117af30fa08245b5)
    },
    {   UINT64_C(0x000000000000c112), UINT64_C(0x1a4af34fa0a585b5), UINT64_C(0x1a4af34d64a585b5),
        UINT64_C(0x12faf1cfa08885b5), UINT64_C(0x17baf38fa09b25b5)
    },
    {   UINT64_C(0x000000000000d1f5), UINT64_C(0x0fa2f50f6c79e698), UINT64_C(0x0fa2f24fa07a268c),
        UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79a698)
    },
    {   UINT64_C(0x000000000000c114), UINT64_C(0x0000000000000000), UINT64_C(0x0de2f44f6873e682),
        UINT64_C(0x1162f1cc6c7b268c), UINT64_C(0x0e1af34fa074c6b1)
    },
    {   UINT64_C(0x000000000000c115), UINT64_C(0x0e2af40f687425c1), UINT64_C(0x0c2af44f686ba5c1),
        UINT64_C(0x08b2f1cfa05e869b), UINT64_C(0x08b2e68fa0652682)
    },
    {   UINT64_C(0x000000000000c116), UINT64_C(0x0a6adfcc745f669b), UINT64_C(0x17c2f0cf749ba5b5),
        UINT64_C(0x183af34fa09d85b5), UINT64_C(0x1162f1cfa081468c)
    },
    {   UINT64_C(0x000000000000c117), UINT64_C(0x1162f38fa08266b1), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2f20c6c7d469b), UINT64_C(0x1162f34d648265b5)
    },
    {   UINT64_C(0x000000000000c118), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x18caf5cfa09ba68c), UINT64_C(0x12d2f1cfa087e68c)
    },
    {   UINT64_C(0x000000000000d1fb), UINT64_C(0x0fa2f40f6c7a0698), UINT64_C(0x0fa3028fa0684698),
        UINT64_C(0x0fa2f30fa079c698), UINT64_C(0x0fa2f28f6c7a6698)
    },
    {   UINT64_C(0x000000000000c11a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x231af58fa0c6868c), UINT64_C(0x17d2f18fa09c05b5)
    },
    {   UINT64_C(0x000000000000d1fd), UINT64_C(0x0fa2f50f6c798698), UINT64_C(0x0fa2f48f6c79e698),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c11c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1852f6cf549b669b), UINT64_C(0x0fa2f44f687a05c1)
    },
    {   UINT64_C(0x000000000000c11d), UINT64_C(0x18caf20f589fc69b), UINT64_C(0x117af34f688345be),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0006f0dd5b914035), UINT64_C(0x1692f4cf5894e794), UINT64_C(0x1692f4cf5894e771),
        UINT64_C(0x1692f4cf5894e771), UINT64_C(0x11d2fa0fe07fc5d2)
    },
    {   UINT64_C(0x00000000000149e3), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x38d70cdfdbf14036), UINT64_C(0x1692f4cdac94e6e4), UINT64_C(0x1692fecf589b879b),
        UINT64_C(0x1692fecf589b879b), UINT64_C(0x1692fecf589b8794)
    },
    {   UINT64_C(0x00000000000149e4), UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9),
        UINT64_C(0x18a2f6cf689cc6a9), UINT64_C(0x18a2f6cf689cc6a9)
    },
    {   UINT64_C(0x000000000000563a), UINT64_C(0x1f430dcf00af6721), UINT64_C(0x1f42fa0eb8af870d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4302d024b46700)
    },
    {   UINT64_C(0x00000000000149e5), UINT64_C(0x1bf2f6d088aa0662), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c124), UINT64_C(0x1392f18fa088c729), UINT64_C(0x0e1af34fa074c642),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000149e6), UINT64_C(0x1762f6d11895c63d), UINT64_C(0x1ce2f6d1d0adc610),
        UINT64_C(0x1762f6d11895c63d), UINT64_C(0x1cb2f6d238ad05f6)
    },
    {   UINT64_C(0x000000000000563c), UINT64_C(0x1f43000ee49fa71e), UINT64_C(0x1f430b0fccad072e),
        UINT64_C(0x1f430d4f78abe71e), UINT64_C(0x1f43104f78ad671e)
    },
    {   UINT64_C(0x00000000000149e7), UINT64_C(0x1cfaf6d020ae067c), UINT64_C(0x1d02f6d1a4ade61a),
        UINT64_C(0x1d02f6d1a0ae261b), UINT64_C(0x1a12f6d20ca26601)
    },
    {   UINT64_C(0x000000000000f8c0), UINT64_C(0x0000000000000000), UINT64_C(0x1362f20c6488a6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692f64c6492a6ef)
    },
    {   UINT64_C(0x00000000000149e8), UINT64_C(0x1b4af6d114a7463f), UINT64_C(0x19e2f6d140a16634),
        UINT64_C(0x1ce2f6d128ada63a), UINT64_C(0x1ce2f6d128ada63a)
    },
    {   UINT64_C(0x000000000000f8c2), UINT64_C(0x0000000000000000), UINT64_C(0x1852f34bf49d66ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1852f6cc649b66ef)
    },
    {   UINT64_C(0x00000000000149e9), UINT64_C(0x1caaf6cfa0acc69c), UINT64_C(0x1d12f6d0f0ae8648),
        UINT64_C(0x1d1af6d0f0ae8648), UINT64_C(0x0e7af6ce147426ff)
    },
    {   UINT64_C(0x3ae756000001403c), UINT64_C(0x0fa2ed4f6c7746a3), UINT64_C(0x1162ec8f7c7de677),
        UINT64_C(0x0fa2ed4f6c75a6a1), UINT64_C(0x0fa2ed4f6c75a6a1)
    },
    {   UINT64_C(0x00000000000149ea), UINT64_C(0x1b9af6d0bca88657), UINT64_C(0x1a22f6d050a2c670),
        UINT64_C(0x1a22f6d050a2c670), UINT64_C(0x1b1af6d134a68637)
    },
    {   UINT64_C(0x0000000000018043), UINT64_C(0x14daf50f688da75e), UINT64_C(0x0de2f44f7872068a),
        UINT64_C(0x1692f20fa094275e), UINT64_C(0x1252f4cf788a06a5)
    },
    {   UINT64_C(0x000000000000f8c7), UINT64_C(0x0000000000000000), UINT64_C(0x131af20bf48986ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x14faf50c648ea6ef)
    },
    {   UINT64_C(0x000000000000f8c8), UINT64_C(0x0000000000000000), UINT64_C(0x14daf5cbf48e06ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x169af18c649606ef)
    },
    {   UINT64_C(0x000000000000f8c9), UINT64_C(0x0000000000000000), UINT64_C(0x1a22f28c64a4c6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692f20bf495e6ef)
    },
    {   UINT64_C(0x000000000000f8ca), UINT64_C(0x0000000000000000), UINT64_C(0x1922f0cc64a1a6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x1802f50c649b06ef)
    },
    {   UINT64_C(0x0000000000018048), UINT64_C(0x1692f60f6897272a), UINT64_C(0x0fa2f50c38794656),
        UINT64_C(0x1692f60b4097268c), UINT64_C(0x0fa2f50c387946e6)
    },
    {   UINT64_C(0x0000000000018049), UINT64_C(0x1692f28fa096675e), UINT64_C(0x0fa2f04fa07b675e),
        UINT64_C(0x169af24fa096467e), UINT64_C(0x0fa2f00fa07b6693)
    },
    {   UINT64_C(0x00000000000149ee), UINT64_C(0x1e3af6d190b30620), UINT64_C(0x1bc2f68f54a926b0),
        UINT64_C(0x19faf6d164a2062b), UINT64_C(0x1b4af71178a7462a)
    },
    {   UINT64_C(0x000000000001804b), UINT64_C(0x0a6af28fa065e754), UINT64_C(0x08b2edcfa0618680),
        UINT64_C(0x0a6af28d0c65e67e), UINT64_C(0x08b2f20d0c612693)
    },
    {   UINT64_C(0x000000000001804c), UINT64_C(0x0a6af28fa065e72a), UINT64_C(0x08b2f20fa05e25b4),
        UINT64_C(0x175af28f6899867e), UINT64_C(0x10d2f20c38774693)
    },
    {   UINT64_C(0x0000000000005645), UINT64_C(0x1f43090e90aba73d), UINT64_C(0x1f43138f0ca1c71f),
        UINT64_C(0x1f43098e9cb4272a), UINT64_C(0x1f43028fd8b7c6ce)
    },
    {   UINT64_C(0x000000000001804e), UINT64_C(0x10f2ef4fa08105b3), UINT64_C(0x131af10fa0894762),
        UINT64_C(0x08b2f2cfa05ea762), UINT64_C(0x131af10b4089467e)
    },
    {   UINT64_C(0x000000000001804f), UINT64_C(0x08b2f2cb405ea693), UINT64_C(0x131af10fa0894693),
        UINT64_C(0x0beaf2cfa06bc6a6), UINT64_C(0x1362f54fa08b667e)
    },
    {   UINT64_C(0x0000000000018050), UINT64_C(0x0b92f2cfa06ac693), UINT64_C(0x131af08fa089267e),
        UINT64_C(0x08b2f2cfa05ee693), UINT64_C(0x1692f10f7496e762)
    },
    {   UINT64_C(0x0000000000005647), UINT64_C(0x1f43010f38b58731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030f30af6711)
    },
    {   UINT64_C(0x0000000000018052), UINT64_C(0x0fa2f20fa07b268a), UINT64_C(0x1272f30fa08546a6),
        UINT64_C(0x1632f20f5892a68e), UINT64_C(0x0fc2f20c387385b4)
    },
    {   UINT64_C(0x0000000000005648), UINT64_C(0x1f42f90e80ad4730), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43094e3cb6274a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018054), UINT64_C(0x1692f68f68948762), UINT64_C(0x0fa2f54f7879468e),
        UINT64_C(0x1bcaf58f88ad2684), UINT64_C(0x1692f58f7894c5b4)
    },
    {   UINT64_C(0x0000000000018055), UINT64_C(0x14daf24fa0910762), UINT64_C(0x0a6aef4fa068668e),
        UINT64_C(0x14daf24b4091067e), UINT64_C(0x0a6aef4b406865b4)
    },
    {   UINT64_C(0x000000000000f8d9), UINT64_C(0x0000000000000000), UINT64_C(0x14eaf60bf491a6ef),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018057), UINT64_C(0x1162f8cf687be75e), UINT64_C(0x0c2af80f786a668a),
        UINT64_C(0x1162f8cc447be68c), UINT64_C(0x0c2afacc446a05b3)
    },
    {   UINT64_C(0x000000000000f8db), UINT64_C(0x0000000000000000), UINT64_C(0x17aae1cbf49d4701),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018059), UINT64_C(0x131af24fa088a75e), UINT64_C(0x09daf14f786826a5),
        UINT64_C(0x132af24fa088a67e), UINT64_C(0x08baf0cf8c618630)
    },
    {   UINT64_C(0x000000000001805a), UINT64_C(0x1692f18f6896a739), UINT64_C(0x0fa2f04f787a8666),
        UINT64_C(0x1692f18f6896a6ed), UINT64_C(0x0fa2f04f787a8619)
    },
    {   UINT64_C(0x000000000001805b), UINT64_C(0x1692f18f6896a754), UINT64_C(0x0fa2f04f787a8680),
        UINT64_C(0x1692f18f6896a76a), UINT64_C(0x0fa2f04f787a86af)
    },
    {   UINT64_C(0x000000000000f8df), UINT64_C(0x14daf20bf48ea6ef), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001805d), UINT64_C(0x1d82ef4f74b3075e), UINT64_C(0x1692f34f6895468a),
        UINT64_C(0x1692f38fa096675e), UINT64_C(0x0fa2f18c3879e68a)
    },
    {   UINT64_C(0x000000000001805e), UINT64_C(0x1692f38fa0966750), UINT64_C(0x1322f08fa0894762),
        UINT64_C(0x0de2f4cfa072c68e), UINT64_C(0x1322f08fa089472a)
    },
    {   UINT64_C(0x0000000000005e0a), UINT64_C(0x1f42f9cf3cb4470e), UINT64_C(0x1f42f90e9cb4c719),
        UINT64_C(0x1f42fececcb4469c), UINT64_C(0x1f42f9d020b1c6e6)
    },
    {   UINT64_C(0x0000000000018060), UINT64_C(0x0fa2e94f78798693), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000564f), UINT64_C(0x1f43044e5cb72729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43108f9cb5c70d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e0d), UINT64_C(0x1f42f9cfe0b70724), UINT64_C(0x1f42f80e5cb6470b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7ce98b0673c)
    },
    {   UINT64_C(0x0000000000005e0e), UINT64_C(0x1f42f3cf58b186cb), UINT64_C(0x1f42f2cfd0b6c70b),
        UINT64_C(0x1f42f7cfd0b286d9), UINT64_C(0x1f4314cf0caa271f)
    },
    {   UINT64_C(0x0000000000005e0f), UINT64_C(0x1f42f8d07cb446f8), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f6cf24b146f1), UINT64_C(0x1f42fc0f94b46700)
    },
    {   UINT64_C(0x0000000000005e10), UINT64_C(0x1f42f8d014aaa6de), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5ceacb9c72a)
    },
    {   UINT64_C(0x0000000000005e11), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fad00cb2e711),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f9cfb8ad46f8)
    },
    {   UINT64_C(0x0000000000005e12), UINT64_C(0x1f42fbcf3cafa70d), UINT64_C(0x1f42f80ef8b4e71d),
        UINT64_C(0x1f42f8cf44b44722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e13), UINT64_C(0x1f42f5cf3cb8670a), UINT64_C(0x1f42facf14b626cd),
        UINT64_C(0x1f42f8cf3cab86dd), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e14), UINT64_C(0x1f42facf80af86cb), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f7d058b2a6d0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001806a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0d82f30bfc720694), UINT64_C(0x0912f14c3861662b)
    },
    {   UINT64_C(0x000000000000f8ee), UINT64_C(0x106b048fa076c691), UINT64_C(0x106afb0fa0794691),
        UINT64_C(0x106af8cf747a6691), UINT64_C(0x106af54fa07c2692)
    },
    {   UINT64_C(0x0000000000005e17), UINT64_C(0x1f42f71028b4c6ed), UINT64_C(0x1f42f6cf60adc6fe),
        UINT64_C(0x1f42f70fb8b0e71d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001806d), UINT64_C(0x0000000000000000), UINT64_C(0x1322f08fa089467e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e19), UINT64_C(0x1f42f51000b5c704), UINT64_C(0x1f42f40f3cb3c6e1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e1a), UINT64_C(0x1f42fbcf28b0670c), UINT64_C(0x1f430ecee0a9a706),
        UINT64_C(0x1f42f9cf50b2c71f), UINT64_C(0x1f42f7cff0ba070d)
    },
    {   UINT64_C(0x0000000000018070), UINT64_C(0x0502fb8fa04ba6c8), UINT64_C(0x08c2fb8fa05aa6c8),
        UINT64_C(0x0d9af68fa07086c8), UINT64_C(0x0a6b0e8f7c5c068c)
    },
    {   UINT64_C(0x0000000000014054), UINT64_C(0x1692f4cf58960770), UINT64_C(0x0fa2ed4f6c7f66a1),
        UINT64_C(0x1d82f3cf80b4c68d), UINT64_C(0x1692ef8f6c92c5b6)
    },
    {   UINT64_C(0x0000000000005e1d), UINT64_C(0x1f42f48fb4b82702), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f3cff0b6a70a)
    },
    {   UINT64_C(0x0000000000014055), UINT64_C(0x14dae70fa08c6770), UINT64_C(0x0c2ae8cfa06f26a1),
        UINT64_C(0x14dae70a048c6685), UINT64_C(0x0c2ad48a046f25b6)
    },
    {   UINT64_C(0x0000000000005e1f), UINT64_C(0x1f42f4cff0ad86d6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f40fecaea6ef), UINT64_C(0x1f42f5cfc4b1c70b)
    },
    {   UINT64_C(0x0000000000005e20), UINT64_C(0x1f42f68e94b84728), UINT64_C(0x1f43000f1cb7e739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f58e58b28725)
    },
    {   UINT64_C(0x0000000000005e21), UINT64_C(0x1f42f68eacb8a70b), UINT64_C(0x1f42f48f5cb086e6),
        UINT64_C(0x1f43070eb0ae2726), UINT64_C(0x1f42f38fecb76717)
    },
    {   UINT64_C(0x000000000000565a), UINT64_C(0x1f430c4eccac6726), UINT64_C(0x1f42fe4e80b3c703),
        UINT64_C(0x1f4300ce34b44749), UINT64_C(0x1f430c8ed8af4727)
    },
    {   UINT64_C(0x0000000000005e23), UINT64_C(0x1f42f4cef8b5c72e), UINT64_C(0x1f42fccedcb4e701),
        UINT64_C(0x1f42f6ceacb6c735), UINT64_C(0x1f42f84ee4b0671b)
    },
    {   UINT64_C(0x000000000000565b), UINT64_C(0x1f4309cea4ada733), UINT64_C(0x1f430d4f54b1c72e),
        UINT64_C(0x1f4305ce54b5472f), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e25), UINT64_C(0x1f42f84f58b32719), UINT64_C(0x1f42f38ed8b92742),
        UINT64_C(0x1f42f58efcb9c6fa), UINT64_C(0x1f42f40fb4b6670e)
    },
    {   UINT64_C(0x0000000000005e26), UINT64_C(0x1f42f40f5cb72717), UINT64_C(0x1f42f58f40af46d5),
        UINT64_C(0x1f42f58f4caea6fb), UINT64_C(0x1f42f9cef4b34728)
    },
    {   UINT64_C(0x0000000000005e27), UINT64_C(0x1f42f78ea0b62718), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4313cecca92710), UINT64_C(0x1f42f6cef4b8a741)
    },
    {   UINT64_C(0x0000000000005e28), UINT64_C(0x1f42f6cf04b20732), UINT64_C(0x1f42f58e60b8871f),
        UINT64_C(0x1f42f7cf04b3a6dc), UINT64_C(0x1f42f6cf30af670f)
    },
    {   UINT64_C(0x0000000000005e29), UINT64_C(0x1f42f5cf44b5c712), UINT64_C(0x1f42f7cf80b0a702),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43030f4caa673c)
    },
    {   UINT64_C(0x0000000000005e2a), UINT64_C(0x1f42f40f40b586dc), UINT64_C(0x1f42f60f8cac86ff),
        UINT64_C(0x1f42f4cf9cb1672b), UINT64_C(0x1f42f8cf28b7e6e7)
    },
    {   UINT64_C(0x0000000000005e2b), UINT64_C(0x1f42f24f74b806c2), UINT64_C(0x1f42f40f38ad86f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f54b38700)
    },
    {   UINT64_C(0x0000000000005e2c), UINT64_C(0x1f42f3d034b54709), UINT64_C(0x1f42f8cf7cb50719),
        UINT64_C(0x1f42f4ce3cb66729), UINT64_C(0x1f42f40f48b24734)
    },
    {   UINT64_C(0x0000000000005e2d), UINT64_C(0x1f42f40f64b6472f), UINT64_C(0x1f43030ee0b1272c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e2e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f50fa8b4c710),
        UINT64_C(0x1f42f40f0cb4a735), UINT64_C(0x1f42fccf00b246f4)
    },
    {   UINT64_C(0x0000000000005e2f), UINT64_C(0x1f42f9cf1cb546f5), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f1cf44b926f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e30), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5d050b02701),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90f4cb72706)
    },
    {   UINT64_C(0x0000000000005e31), UINT64_C(0x1f42facf0cb46712), UINT64_C(0x1f42f3cec0b36739),
        UINT64_C(0x1f42f48ea0b84725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e32), UINT64_C(0x1f42f5cfb4b7e71b), UINT64_C(0x1f42faceacb70701),
        UINT64_C(0x1f42fc0f8ca98703), UINT64_C(0x1f42f70fecb2471e)
    },
    {   UINT64_C(0x0000000000005e33), UINT64_C(0x1f42faceecb3a71b), UINT64_C(0x1f42f9cf3cae8712),
        UINT64_C(0x1f42f9cf20b6471c), UINT64_C(0x1f42f6ce9cb8e71e)
    },
    {   UINT64_C(0x0000000000005e34), UINT64_C(0x1f43014f0cb5e72d), UINT64_C(0x1f42f7cf70b2c719),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e35), UINT64_C(0x1f42f6cf48b726ff), UINT64_C(0x1f42f40f90b8071e),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005664), UINT64_C(0x1f42fa5030b8a705), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f64e8caf6705), UINT64_C(0x1f42f54f8cb7e71a)
    },
    {   UINT64_C(0x0000000000005e37), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fa0fa0b5a6dc),
        UINT64_C(0x1f42face98b2a70c), UINT64_C(0x1f42f40e90b8871e)
    },
    {   UINT64_C(0x0000000000014062), UINT64_C(0x1182eccf6c8ce628), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e39), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7cf24ac4700),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5ce84b4e730)
    },
    {   UINT64_C(0x0000000000005666), UINT64_C(0x1f42f95038af46f1), UINT64_C(0x1f42f64ec0b6271e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078e6caee71c)
    },
    {   UINT64_C(0x0000000000004ba0), UINT64_C(0x1f433194c899c6d0), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014064), UINT64_C(0x1002fb8f90930696), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2d84d3081a6ab), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005e3d), UINT64_C(0x1f42f4cff4b80720), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f5ce40b0073b)
    },
    {   UINT64_C(0x0000000000014065), UINT64_C(0x0000000000000000), UINT64_C(0x16faeacf789506a3),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018094), UINT64_C(0x0de2f00f787225b4), UINT64_C(0x0fa2f4cf787be5b4),
        UINT64_C(0x0fa2f4cf787be5b4), UINT64_C(0x0fa2ff4f78790693)
    },
    {   UINT64_C(0x0000000000014066), UINT64_C(0x0000000000000000), UINT64_C(0x0fa2e84f807ae69b),
        UINT64_C(0x08b2ee8f7c62c6a3), UINT64_C(0x0f02f20fa0770682)
    },
    {   UINT64_C(0x0000000000018096), UINT64_C(0x0de2f30f7872e5b4), UINT64_C(0x0de2f18f787225b4),
        UINT64_C(0x148af70f6c8fe5b4), UINT64_C(0x0db2f30f686f25c3)
    },
    {   UINT64_C(0x0000000000014067), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x10c2ec8c6c79c5b6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018098), UINT64_C(0x1292f60c508dc693), UINT64_C(0x10caf60c507765ca),
        UINT64_C(0x11d2f58f7481e5aa), UINT64_C(0x0fa2f2cbf87a85aa)
    },
    {   UINT64_C(0x0000000000018099), UINT64_C(0x0fa2f14f787b45aa), UINT64_C(0x0fa2f14bfc7be5aa),
        UINT64_C(0x0fa2f04fa07b6693), UINT64_C(0x0f0af04bfc73e693)
    },
    {   UINT64_C(0x000000000001809a), UINT64_C(0x08b2f20fa05e2693), UINT64_C(0x0a6af24f786565ae),
        UINT64_C(0x0812f34fa05b65aa), UINT64_C(0x0af2f08fa0690693)
    },
    {   UINT64_C(0x000000000001809b), UINT64_C(0x0bbaf10fa06bc693), UINT64_C(0x0972f08bfc6a8693),
        UINT64_C(0x112af2cbfc81c693), UINT64_C(0x1852f0cf789ec5aa)
    },
    {   UINT64_C(0x000000000001809c), UINT64_C(0x191af0cbfca0a5aa), UINT64_C(0x1852f20bfc96c5b4),
        UINT64_C(0x108adc8bfc7ec5b4), UINT64_C(0x10aaf20bfc89e5b4)
    },
    {   UINT64_C(0x000000000001406a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0c82f98bfc6e0694), UINT64_C(0x08f2e1cc3862a63a)
    },
    {   UINT64_C(0x000000000001809e), UINT64_C(0x14daf80bfc8cc693), UINT64_C(0x0b92f58f786b85aa),
        UINT64_C(0x0b92f58f786c0693), UINT64_C(0x0d12f58bfc7905aa)
    },
    {   UINT64_C(0x000000000000566e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f68eccb6472b),
        UINT64_C(0x1f42f60eacb64712), UINT64_C(0x1f43058ec0b0c72d)
    },
    {   UINT64_C(0x00000000000180a0), UINT64_C(0x0edaf34fa07945b8), UINT64_C(0x0edaf34fa07945b8),
        UINT64_C(0x0c2afa8c386a05b2), UINT64_C(0x0a12dc8bfc6d0693)
    },
    {   UINT64_C(0x000000000000566f), UINT64_C(0x1f42f55004b5c6f1), UINT64_C(0x1f42f80e6cb02711),
        UINT64_C(0x1f42fecef0b24722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000180a2), UINT64_C(0x0872f14bfc61e62b), UINT64_C(0x0fa2f04f787a85aa),
        UINT64_C(0x10caf98f787bc5aa), UINT64_C(0x0fa2f60f787a05b4)
    },
    {   UINT64_C(0x0000000000005670), UINT64_C(0x1f42f60eb8b4c6e8), UINT64_C(0x1f42f54ec8b0670c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f81020b546f5)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019400), UINT64_C(0x07d2ee0fa03e83e8), UINT64_C(0x10eaf60c3877a693),
        UINT64_C(0x101af38bd07c2438), UINT64_C(0x0a030dccac56a69a)
    },
    {   UINT64_C(0x0000000000019401), UINT64_C(0x1042fbcfa07f0684), UINT64_C(0x0e62f28f7875a5aa),
        UINT64_C(0x0aa2f9101063661d), UINT64_C(0x12eaf8d01084a61d)
    },
    {   UINT64_C(0x0000000000005672), UINT64_C(0x1f42f80f2cb726e5), UINT64_C(0x1f42f5cefcb7a705),
        UINT64_C(0x1f42f64edcaee71b), UINT64_C(0x1f42f94fe8ac86fe)
    },
    {   UINT64_C(0x00000000000180a8), UINT64_C(0x12c2f40c3c88462e), UINT64_C(0x0e12fa0fa070a693),
        UINT64_C(0x0e12fa0fa070a693), UINT64_C(0x1912f4cf789f6693)
    },
    {   UINT64_C(0x0000000000019404), UINT64_C(0x12b3028f647f4640), UINT64_C(0x0cfaee10846b85a9),
        UINT64_C(0x12b3020f647f0640), UINT64_C(0x18d2fdcf6499a640)
    },
    {   UINT64_C(0x0000000000019405), UINT64_C(0x18d2fe4f6499e640), UINT64_C(0x0e7aff90886fa5a9),
        UINT64_C(0x0e7aff90886fa5a9), UINT64_C(0x105af50e787d46c1)
    },
    {   UINT64_C(0x0000000000019406), UINT64_C(0x0c92fe0f6c68c5ec), UINT64_C(0x156afd0d308ca67e),
        UINT64_C(0x0c92fe0f6c68c5ec), UINT64_C(0x156afd0d308ca67e)
    },
    {   UINT64_C(0x0000000000019407), UINT64_C(0x15b2f58fa091867e), UINT64_C(0x15b2f58fa091867e),
        UINT64_C(0x15b2f78fa090867e), UINT64_C(0x105af98c287a46ae)
    },
    {   UINT64_C(0x0000000000019408), UINT64_C(0x170afecf749e6797), UINT64_C(0x170b00107c9e679a),
        UINT64_C(0x170b00107c9e6797), UINT64_C(0x156af84fa08f4594)
    },
    {   UINT64_C(0x0000000000019409), UINT64_C(0x0853070f545345ab), UINT64_C(0x0ca2f3cf546e25ab),
        UINT64_C(0x0df3048f746b05ab), UINT64_C(0x130af3cf5487c5ab)
    },
    {   UINT64_C(0x0000000000005676), UINT64_C(0x1f42f50e1cb40707), UINT64_C(0x1f42f40f68b86710),
        UINT64_C(0x1f42f84fe0b846fc), UINT64_C(0x1f42f80f00b206dd)
    },
    {   UINT64_C(0x0000000000004bc0), UINT64_C(0x1f430554a8b2a5c1), UINT64_C(0x1f43568f3880e703),
        UINT64_C(0x1f434a565c8946a5), UINT64_C(0x1f42f7d01c6566d8)
    },
    {   UINT64_C(0x0000000000004bc1), UINT64_C(0x1f4319d018ad26c7), UINT64_C(0x1f43534f0c88e6f4),
        UINT64_C(0x1f42fe9204b4a672), UINT64_C(0x1f43115460aba630)
    },
    {   UINT64_C(0x0000000000004bc2), UINT64_C(0x1f42fa50a4b786cd), UINT64_C(0x1f43058fbcb646cb),
        UINT64_C(0x1f43085094b3c684), UINT64_C(0x1f42fd11c4b926b4)
    },
    {   UINT64_C(0x0000000000004bc3), UINT64_C(0x1f4316cf94a466e0), UINT64_C(0x1f4313d744aa66c1),
        UINT64_C(0x1f434d509887c694), UINT64_C(0x1f430b5024b726d8)
    },
    {   UINT64_C(0x0000000000004bc4), UINT64_C(0x1f43139098aca657), UINT64_C(0x1f4308d078a646a6),
        UINT64_C(0x1f42ff4f70a5071b), UINT64_C(0x1f431a4f5ca58707)
    },
    {   UINT64_C(0x0000000000019410), UINT64_C(0x156afc0ac48c271a), UINT64_C(0x156afbcac48c071a),
        UINT64_C(0x156afe0d308c267e), UINT64_C(0x156afe0d308c267e)
    },
    {   UINT64_C(0x0000000000019411), UINT64_C(0x156afbcd308c067e), UINT64_C(0x156afe4d308d467e),
        UINT64_C(0x15bafe4c748d46ab), UINT64_C(0x15bafe4c748d46ab)
    },
    {   UINT64_C(0x000000000000567a), UINT64_C(0x1f42f78f08b6c71b), UINT64_C(0x1f42f60e3cb6e703),
        UINT64_C(0x1f42f71058b2c6ee), UINT64_C(0x1f42fe4fe8b366d5)
    },
    {   UINT64_C(0x0000000000019413), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x156afd0b888ca6e8)
    },
    {   UINT64_C(0x0000000000004bc9), UINT64_C(0x1f430d4ff4b3a711), UINT64_C(0x1f42fb4f78b7c710),
        UINT64_C(0x1f4304cef4b466ec), UINT64_C(0x1f4313d340aba64a)
    },
    {   UINT64_C(0x0000000000004bca), UINT64_C(0x1f430d4ebcb30722), UINT64_C(0x1f43110f7cada6ea),
        UINT64_C(0x1f42fa4f98b446fd), UINT64_C(0x1f43008fc4a326ef)
    },
    {   UINT64_C(0x000000000000567c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f60e84b8a71f),
        UINT64_C(0x1f42f48f6caf86f2), UINT64_C(0x1f42f90f68ae46f3)
    },
    {   UINT64_C(0x0000000000019417), UINT64_C(0x156afb8d0893064f), UINT64_C(0x156afd0d7c8ca659),
        UINT64_C(0x156afd0d7c8ca659), UINT64_C(0x156afe0a9c8d26b6)
    },
    {   UINT64_C(0x00000000000180bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6aee4c3467e377)
    },
    {   UINT64_C(0x0000000000019419), UINT64_C(0x2542f58fa0cfc7e6), UINT64_C(0x28fafbcfa0db8992),
        UINT64_C(0x1cf2fbd08cab65aa), UINT64_C(0x1cf2fbd084ab65fa)
    },
    {   UINT64_C(0x000000000001941a), UINT64_C(0x156afa107c8e2602), UINT64_C(0x156afa0f0c8e2607),
        UINT64_C(0x156afd0a508cc736), UINT64_C(0x156afc0c0c8c26c2)
    },
    {   UINT64_C(0x000000000001941b), UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afd4a508cc736),
        UINT64_C(0x156afb4ac48bc71a), UINT64_C(0x156afecac48d871a)
    },
    {   UINT64_C(0x0000000000004bd1), UINT64_C(0x1f43084e1cabc728), UINT64_C(0x1f430acf88b12716),
        UINT64_C(0x1f430dcfa0b006c5), UINT64_C(0x1f43308fd89986bb)
    },
    {   UINT64_C(0x000000000001941d), UINT64_C(0x156b004d308e467e), UINT64_C(0x1d2af44d34b0067f),
        UINT64_C(0x1d2af44d30b0067e), UINT64_C(0x26c2f8cd30d2867e)
    },
    {   UINT64_C(0x0000000000005e6e), UINT64_C(0x1f4319ceb4a5a701), UINT64_C(0x1f43128e98a8e736),
        UINT64_C(0x1f43170eecaa26f1), UINT64_C(0x1f43210ef0a76717)
    },
    {   UINT64_C(0x000000000001941f), UINT64_C(0x16e2fa8cac93c69a), UINT64_C(0x156af9cd308e067e),
        UINT64_C(0x0ebaf80d30802713), UINT64_C(0x156afd0d308ca6c8)
    },
    {   UINT64_C(0x0000000000004bd5), UINT64_C(0x1f43088ee8baa716), UINT64_C(0x1f43124fdcb64717),
        UINT64_C(0x1f43010ee8b6a6f3), UINT64_C(0x1f43098facb2e6e7)
    },
    {   UINT64_C(0x0000000000004bd6), UINT64_C(0x1f4307cf18b4a71b), UINT64_C(0x1f431b4e8c982723),
        UINT64_C(0x1f430e4ed09c672b), UINT64_C(0x1f43144ef8af6704)
    },
    {   UINT64_C(0x0000000000004bd7), UINT64_C(0x1f43098e6cb20730), UINT64_C(0x1f430e8fa8b5a70a),
        UINT64_C(0x1f430a4fd0b186fe), UINT64_C(0x1f43081098b2e6b8)
    },
    {   UINT64_C(0x0000000000019423), UINT64_C(0x156afd0a4c8ca735), UINT64_C(0x156afd0bd88ca6d1),
        UINT64_C(0x156afe4d308c067e), UINT64_C(0x156afe4d308c067e)
    },
    {   UINT64_C(0x0000000000019424), UINT64_C(0x156afa0ec48e2619), UINT64_C(0x13c2f48fa08a05aa),
        UINT64_C(0x1582f74f789365ec), UINT64_C(0x11daf30f6884e5c3)
    },
    {   UINT64_C(0x0000000000004bda), UINT64_C(0x1f42fcd390b4072e), UINT64_C(0x1f43194ffca6c70d),
        UINT64_C(0x1f43028f18b68719), UINT64_C(0x1f42fc91ccb4c63e)
    },
    {   UINT64_C(0x0000000000004bdb), UINT64_C(0x1f432f10089966e2), UINT64_C(0x1f4311cee0b16711),
        UINT64_C(0x1f43140fa8b4e713), UINT64_C(0x1f42fe5024a526c3)
    },
    {   UINT64_C(0x0000000000004bdc), UINT64_C(0x1f430d4f68ae86ce), UINT64_C(0x1f4319ceb4a5a701),
        UINT64_C(0x1f430b0f18b7472a), UINT64_C(0x1f43108fd4b786f1)
    },
    {   UINT64_C(0x0000000000005685), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f90e8cb7271e),
        UINT64_C(0x1f42fd8ff4b7e704), UINT64_C(0x1f42f70fb0b466e4)
    },
    {   UINT64_C(0x0000000000019429), UINT64_C(0x1062f04b247b6693), UINT64_C(0x0f32fe0e18734642),
        UINT64_C(0x0f32fe0e18734642), UINT64_C(0x156afd0d2c8ca6f9)
    },
    {   UINT64_C(0x000000000001942a), UINT64_C(0x156afd0d2c8ca6f9), UINT64_C(0x156afd0d9c8ca6f9),
        UINT64_C(0x156afd0d9c8ca6f9), UINT64_C(0x156afa0dc48e265a)
    },
    {   UINT64_C(0x0000000000004be0), UINT64_C(0x1f43048e98b46738), UINT64_C(0x1f431bcf08a38723),
        UINT64_C(0x1f43204fc8acc6f2), UINT64_C(0x1f432c0f2c99e72e)
    },
    {   UINT64_C(0x0000000000004be1), UINT64_C(0x1f430a9028b146f6), UINT64_C(0x1f4312cfb0ace6e8),
        UINT64_C(0x1f430a4efcb086dd), UINT64_C(0x1f42ffcec8b04725)
    },
    {   UINT64_C(0x0000000000004be2), UINT64_C(0x1f42fcce48b5c737), UINT64_C(0x1f43270ee0b5e703),
        UINT64_C(0x1f43210f40b1c739), UINT64_C(0x1f430b4ea8b00711)
    },
    {   UINT64_C(0x0000000000004be3), UINT64_C(0x1f430c0ec8afa701), UINT64_C(0x1f4314cf8ca466f9),
        UINT64_C(0x1f42fdd068b606d0), UINT64_C(0x1f42fe4eccb68714)
    },
    {   UINT64_C(0x0000000000004be4), UINT64_C(0x1f42fb8edcb5272a), UINT64_C(0x1f430e8f88b066e5),
        UINT64_C(0x1f43114f74b4e720), UINT64_C(0x1f430d8e94a1873b)
    },
    {   UINT64_C(0x0000000000004be5), UINT64_C(0x1f43068f4cae8720), UINT64_C(0x1f42f94fc8b8a6e9),
        UINT64_C(0x1f43085008b4e6f8), UINT64_C(0x1f430f0f24b726fd)
    },
    {   UINT64_C(0x0000000000004be6), UINT64_C(0x1f43190e949e8736), UINT64_C(0x1f4315cee8a30707),
        UINT64_C(0x1f43084ea8b28739), UINT64_C(0x1f43040ee8b66729)
    },
    {   UINT64_C(0x0000000000019432), UINT64_C(0x16f2f28f6896068c), UINT64_C(0x118af40c3881e5b4),
        UINT64_C(0x0bd2f08bfc6b45be), UINT64_C(0x0c6b00d64868c685)
    },
    {   UINT64_C(0x0000000000004be8), UINT64_C(0x1f42f98efcb62702), UINT64_C(0x1f43084e8cae8725),
        UINT64_C(0x1f43220f84b20713), UINT64_C(0x1f43160f7caa06fb)
    },
    {   UINT64_C(0x0000000000019434), UINT64_C(0x0bc2f5978869c6a6), UINT64_C(0x0f830a8d306ce67e),
        UINT64_C(0x0eb2f597887586a6), UINT64_C(0x13f2f6cd307ea67e)
    },
    {   UINT64_C(0x0000000000004bea), UINT64_C(0x1f4319cf2ca9e702), UINT64_C(0x1f42ff8e4cb06745),
        UINT64_C(0x1f43110eb4b06719), UINT64_C(0x1f42f8cefcad6729)
    },
    {   UINT64_C(0x0000000000004beb), UINT64_C(0x1f43124f8caea6db), UINT64_C(0x1f43054ee4afc730),
        UINT64_C(0x1f430c4eacafc72a), UINT64_C(0x1f430ed058aee6e8)
    },
    {   UINT64_C(0x0000000000004bec), UINT64_C(0x1f43028e6cb0c740), UINT64_C(0x1f43078e4caee736),
        UINT64_C(0x1f4307cf9cab66e3), UINT64_C(0x1f42ff8e80b3872f)
    },
    {   UINT64_C(0x0000000000004bed), UINT64_C(0x1f431bceacad071e), UINT64_C(0x1f43028f94b7c6ff),
        UINT64_C(0x1f430e8ec0b6270d), UINT64_C(0x1f42fc0e5cb54735)
    },
    {   UINT64_C(0x0000000000019439), UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c),
        UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c)
    },
    {   UINT64_C(0x000000000001943a), UINT64_C(0x1772fa918096056a), UINT64_C(0x1772fa90109605c6),
        UINT64_C(0x1772fa90109605c6), UINT64_C(0x1772fa9310960506)
    },
    {   UINT64_C(0x000000000001943b), UINT64_C(0x1772fa918096057a), UINT64_C(0x1f42fa9180b5456a),
        UINT64_C(0x1f42fa9010b545c6), UINT64_C(0x1f42fa8f08b54608)
    },
    {   UINT64_C(0x0000000000004bf1), UINT64_C(0x1f4320cea0a10726), UINT64_C(0x1f42fc4f10b5872a),
        UINT64_C(0x1f43060e68b7673a), UINT64_C(0x1f42f98eccb6e73b)
    },
    {   UINT64_C(0x000000000001943d), UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x13e2fa8bf087c72c),
        UINT64_C(0x13e2fa8bf087c72c), UINT64_C(0x1cf302ccb8a7e69c)
    },
    {   UINT64_C(0x0000000000004bf3), UINT64_C(0x1f43194f38b1c701), UINT64_C(0x1f430a4ef0a9e71f),
        UINT64_C(0x1f43054f50b5a703), UINT64_C(0x1f43158f04b5a72a)
    },
    {   UINT64_C(0x000000000001943f), UINT64_C(0x17d2fa8e6497862f), UINT64_C(0x0c82fc4fa0696630),
        UINT64_C(0x07fafc0fa0576681), UINT64_C(0x0c22fc0fa0680681)
    },
    {   UINT64_C(0x0000000000019440), UINT64_C(0x2e03038e00edc64a), UINT64_C(0x2372fe5200c4254a),
        UINT64_C(0x1023004fa076278d), UINT64_C(0x0fa2fc4fa075678d)
    },
    {   UINT64_C(0x0000000000019441), UINT64_C(0x1853050fa095e6da), UINT64_C(0x0fa2f80fa0760780),
        UINT64_C(0x0c2b024fa06646dd), UINT64_C(0x0fa2fd0fa07426da)
    },
    {   UINT64_C(0x0000000000019442), UINT64_C(0x1a52f6cd88a36666), UINT64_C(0x15e3058e608a46ed),
        UINT64_C(0x105b000cac76c69a), UINT64_C(0x105b000cac76c69a)
    },
    {   UINT64_C(0x0000000000019443), UINT64_C(0x1822f6cda49aa713), UINT64_C(0x21aafbcf38be45fc),
        UINT64_C(0x156afa0efc8e060b), UINT64_C(0x1c72f94d28aaa67d)
    },
    {   UINT64_C(0x0000000000019444), UINT64_C(0x1662f94eb892661f), UINT64_C(0x156afa11808e2746),
        UINT64_C(0x0ffb1e0ac066671b), UINT64_C(0x1cf3034d30a7a6d7)
    },
    {   UINT64_C(0x0000000000019445), UINT64_C(0x1572fa4efc8e260b), UINT64_C(0x0c32fa0fa069442a),
        UINT64_C(0x211af78d2cbfa790), UINT64_C(0x211b1f0d30aa467e)
    },
    {   UINT64_C(0x0000000000019446), UINT64_C(0x182afbcd0098469e), UINT64_C(0x1cf2fbcd58ab8688),
        UINT64_C(0x1c6af3cdd8ad46d3), UINT64_C(0x1de2f38e08b346bf)
    },
    {   UINT64_C(0x0000000000019447), UINT64_C(0x0fa316ce3c6885c2), UINT64_C(0x17fb040fa0936690),
        UINT64_C(0x094afd8e3c5be5c2), UINT64_C(0x36b3040fa10e4690)
    },
    {   UINT64_C(0x0000000000019448), UINT64_C(0x32cb040fa0fea690), UINT64_C(0x32cb040fa0fea690),
        UINT64_C(0x32cb040fa0fea690), UINT64_C(0x32cb040fa0fea690)
    },
    {   UINT64_C(0x0000000000019449), UINT64_C(0x3f83040fa1318690), UINT64_C(0x32cb040fa0fea690),
        UINT64_C(0x3243040fa0fc8690), UINT64_C(0x47c3040fa1528690)
    },
    {   UINT64_C(0x000000000001944a), UINT64_C(0x0322de1394530581), UINT64_C(0x3f83040fa1318690),
        UINT64_C(0x3f83040fa1318690), UINT64_C(0x32cb040fa0fea690)
    },
    {   UINT64_C(0x0000000000004c00), UINT64_C(0x1f42ee0fa03e83e8), UINT64_C(0x1f434c13e48ca5d8),
        UINT64_C(0x1f434c53a48c45dd), UINT64_C(0x1f433391f88ec609)
    },
    {   UINT64_C(0x0000000000004c01), UINT64_C(0x0000000000000000), UINT64_C(0x1f431390b4a42698),
        UINT64_C(0x1f43185090a606e4), UINT64_C(0x1f431e90c8a32695)
    },
    {   UINT64_C(0x0000000000004c02), UINT64_C(0x1f43800e2cab473c), UINT64_C(0x1f430e4e2c72a73c),
        UINT64_C(0x1f436f0e2cb6c73c), UINT64_C(0x1f42f74e2c7b473c)
    },
    {   UINT64_C(0x0000000000004c03), UINT64_C(0x1f437fd1d0b12725), UINT64_C(0x1f43064ed472c651),
        UINT64_C(0x1f437651e0ada71a), UINT64_C(0x1f430a4ea877a64c)
    },
    {   UINT64_C(0x0000000000004c04), UINT64_C(0x1f437f8e0caa4748), UINT64_C(0x1f43110e0c72c748),
        UINT64_C(0x1f43004facb266f8), UINT64_C(0x1f43129014a8c6c5)
    },
    {   UINT64_C(0x0000000000019450), UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x1cf2fb8d30ab867e),
        UINT64_C(0x1cf2fb8d30ab867e), UINT64_C(0x0ffb251118632583)
    },
    {   UINT64_C(0x00000000000180f6), UINT64_C(0x1692f68c3c94868c), UINT64_C(0x0fa2f54c3c7945b4),
        UINT64_C(0x14daf50c388c468c), UINT64_C(0x0e6af6cc387505d2)
    },
    {   UINT64_C(0x0000000000004c07), UINT64_C(0x0000000000000000), UINT64_C(0x1f43799898b10735),
        UINT64_C(0x1f43001874752731), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019453), UINT64_C(0x1cf2fbcd30ab867e), UINT64_C(0x1cf2fb8d30ab667e),
        UINT64_C(0x1cf2fb8d30ab867d), UINT64_C(0x0ffb25113463257c)
    },
    {   UINT64_C(0x0000000000004c09), UINT64_C(0x1f431691349f46d8), UINT64_C(0x1f431e8ef8a0e752),
        UINT64_C(0x1f43089778ac26f7), UINT64_C(0x1f43129538a886ef)
    },
    {   UINT64_C(0x0000000000004c0a), UINT64_C(0x1f43125090abe6f0), UINT64_C(0x1f4318d018b186e8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000569c), UINT64_C(0x1f43004facb2c6e7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019457), UINT64_C(0x0f02eb8cdc7bc369), UINT64_C(0x16fafecf74e14797),
        UINT64_C(0x1c0ae7d7a0b007a3), UINT64_C(0x1c0aeb57a0b1c7a3)
    },
    {   UINT64_C(0x0000000000019458), UINT64_C(0x1c0ae7cb8cb0049e), UINT64_C(0x1c0aeb4b8cb1c49e),
        UINT64_C(0x0a2aedcb8c67436a), UINT64_C(0x39a2eb9c712648aa)
    },
    {   UINT64_C(0x0000000000019459), UINT64_C(0x39a2eb877126436a), UINT64_C(0x38e2e7d4f92347a3),
        UINT64_C(0x38e2eb94f92527a3), UINT64_C(0x38e2e7cb8d234548)
    },
    {   UINT64_C(0x000000000001945a), UINT64_C(0x38e2eb8b8d252548), UINT64_C(0x38e2e7d8492347a3),
        UINT64_C(0x38e2eb98492527a3), UINT64_C(0x38e2e7cb8d234474)
    },
    {   UINT64_C(0x0000000000004c10), UINT64_C(0x0000000000000000), UINT64_C(0x1f43218e509ea665),
        UINT64_C(0x1f43118e58a80716), UINT64_C(0x1f431fd178a1465e)
    },
    {   UINT64_C(0x0000000000004c11), UINT64_C(0x1f430a5118a8c6b5), UINT64_C(0x1f43308ee893e647),
        UINT64_C(0x1f43248f049a46ec), UINT64_C(0x1f43284fcc9b4650)
    },
    {   UINT64_C(0x0000000000004c12), UINT64_C(0x1f431ecfcca3a6e1), UINT64_C(0x1f43178fac9ee664),
        UINT64_C(0x1f43070fa0a9e6f6), UINT64_C(0x1f43119030aaa6cf)
    },
    {   UINT64_C(0x0000000000004c13), UINT64_C(0x1f430e5030ab86e9), UINT64_C(0x1f431e50389a66e6),
        UINT64_C(0x1f43141038a68708), UINT64_C(0x1f43294f008ee703)
    },
    {   UINT64_C(0x0000000000018104), UINT64_C(0x164af04fa096268a), UINT64_C(0x131af2cfa086a67e),
        UINT64_C(0x145af2cfa088e67e), UINT64_C(0x11daf2cfa082867e)
    },
    {   UINT64_C(0x0000000000019460), UINT64_C(0x0c8af94fa06b067e), UINT64_C(0x13d3008fa08d467e),
        UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x1a5b00cfa09e867e)
    },
    {   UINT64_C(0x0000000000004c16), UINT64_C(0x1f431c0fe8a106ec), UINT64_C(0x1f430a4e4cada6f9),
        UINT64_C(0x1f430a4e4caf8743), UINT64_C(0x1f43044fe8b066a8)
    },
    {   UINT64_C(0x0000000000004c17), UINT64_C(0x1f43010fe8b60700), UINT64_C(0x1f4319cf60a3c6db),
        UINT64_C(0x1f43130f60af66db), UINT64_C(0x1f43134fb4a586d4)
    },
    {   UINT64_C(0x0000000000004c18), UINT64_C(0x1f43134fb4ae06f2), UINT64_C(0x1f43198ec8a166e6),
        UINT64_C(0x1f43164ec8abe723), UINT64_C(0x1f4317500ca105bb)
    },
    {   UINT64_C(0x0000000000004c19), UINT64_C(0x1f4307101cabe62d), UINT64_C(0x1f4300501cb5c6f3),
        UINT64_C(0x1f430dd054a6c6bc), UINT64_C(0x1f430a9054ad46bc)
    },
    {   UINT64_C(0x0000000000004c1a), UINT64_C(0x1f43274f909e46db), UINT64_C(0x1f431d0f90aa46f3),
        UINT64_C(0x1f43164fc8a8e6ff), UINT64_C(0x1f4313100ca506c9)
    },
    {   UINT64_C(0x0000000000004c1b), UINT64_C(0x1f4310904ca906d3), UINT64_C(0x1f43064f74ab26e6),
        UINT64_C(0x1f430b10d0a8a6b7), UINT64_C(0x1f43130f44aa06a1)
    },
    {   UINT64_C(0x000000000001810c), UINT64_C(0x0e12f74f787645b4), UINT64_C(0x0fdaf7cf787a4695),
        UINT64_C(0x0ed2f40fa07545aa), UINT64_C(0x0c52f24fa06ee5aa)
    },
    {   UINT64_C(0x0000000000004c1d), UINT64_C(0x1f430b4f7cb186f2), UINT64_C(0x1f4307d054afa6df),
        UINT64_C(0x1f43011054b166df), UINT64_C(0x1f43011054b046df)
    },
    {   UINT64_C(0x0000000000019469), UINT64_C(0x0d52ee8f7871c5ad), UINT64_C(0x0fbaf84f787865b7),
        UINT64_C(0x13c2f68c408985b8), UINT64_C(0x0c7ae90c447585b7)
    },
    {   UINT64_C(0x0000000000004c1f), UINT64_C(0x1f43014fd4b706ca), UINT64_C(0x1f43014fd4b886ef),
        UINT64_C(0x1f43224fa8a166d1), UINT64_C(0x1f43110fe0ae86d9)
    },
    {   UINT64_C(0x0000000000018110), UINT64_C(0x0f9aed8c3c7865b4), UINT64_C(0x0e42f84f787405b4),
        UINT64_C(0x0d92f00fa073c5aa), UINT64_C(0x0f5af00c387ae5aa)
    },
    {   UINT64_C(0x0000000000018111), UINT64_C(0x1532f8cc3c8de67e), UINT64_C(0x0f4af18fa07a05aa),
        UINT64_C(0x106af3cd7c7d45aa), UINT64_C(0x1002f28fa07ba5aa)
    },
    {   UINT64_C(0x0000000000004c22), UINT64_C(0x1f431bcf5ca3a6f0), UINT64_C(0x1f43290f789b86fa),
        UINT64_C(0x1f432c8ed89926fe), UINT64_C(0x1f431f8fa89d26c7)
    },
    {   UINT64_C(0x0000000000004c23), UINT64_C(0x1f43040f64b0e708), UINT64_C(0x1f431dcfb8a106e4),
        UINT64_C(0x1f43188f009fe654), UINT64_C(0x1f43068f00a926f4)
    },
    {   UINT64_C(0x0000000000004c24), UINT64_C(0x1f430e4fcca766db), UINT64_C(0x1f43134fc4a526e4),
        UINT64_C(0x1f431e0ec8a48703), UINT64_C(0x1f430c4fd0aa66e6)
    },
    {   UINT64_C(0x0000000000004c25), UINT64_C(0x1f43214e88b1c709), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c26), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f435098088a46d1)
    },
    {   UINT64_C(0x0000000000004c27), UINT64_C(0x1f435258788966e8), UINT64_C(0x1f431bd048a3a679),
        UINT64_C(0x1f43029048a0c6d6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c28), UINT64_C(0x0000000000000000), UINT64_C(0x1f432e903c9aa621),
        UINT64_C(0x1f431dd058a066a8), UINT64_C(0x1f43264fe88ce657)
    },
    {   UINT64_C(0x0000000000004c29), UINT64_C(0x1f43124fe88d66e2), UINT64_C(0x1f432b4fb496262c),
        UINT64_C(0x1f431f0fd09ea6bd), UINT64_C(0x1f432151ac9f45f5)
    },
    {   UINT64_C(0x0000000000004c2a), UINT64_C(0x1f4310d1a0a6c657), UINT64_C(0x1f43258fd0a0e643),
        UINT64_C(0x1f43118fd0a646d0), UINT64_C(0x1f431a509c9a06c5)
    },
    {   UINT64_C(0x0000000000004c2b), UINT64_C(0x1f4317109caac6eb), UINT64_C(0x1f43130f5ca4e6a2),
        UINT64_C(0x1f43130f5cad46fa), UINT64_C(0x1f431ecfb899e69d)
    },
    {   UINT64_C(0x0000000000007dac), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f84e48b66729),
        UINT64_C(0x1f42f84e48b66729), UINT64_C(0x1f42f84e48b66729)
    },
    {   UINT64_C(0x000000000001811d), UINT64_C(0x17f2f20f749d868c), UINT64_C(0x10daf2cf688105b0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004d00), UINT64_C(0x0000000000000000), UINT64_C(0x1f43049064b046ea),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c2f), UINT64_C(0x1f43119054a666e7), UINT64_C(0x1f43220f6c9da650),
        UINT64_C(0x1f4317cf6cabe6f4), UINT64_C(0x1f431d8fd89b868a)
    },
    {   UINT64_C(0x0000000000004c30), UINT64_C(0x1f43134fd8ad86d6), UINT64_C(0x1f43110fb0a886e0),
        UINT64_C(0x1f43110fb0af06e9), UINT64_C(0x1f43268f6096c5c5)
    },
    {   UINT64_C(0x0000000000004c31), UINT64_C(0x1f431a4f7c9f0647), UINT64_C(0x1f43138f7cacc6fc),
        UINT64_C(0x1f4317cf94a62696), UINT64_C(0x1f43110f94aee6e9)
    },
    {   UINT64_C(0x0000000000004c32), UINT64_C(0x1f4344d0749026c3), UINT64_C(0x1f4344d074a106ea),
        UINT64_C(0x1f43160fcca486df), UINT64_C(0x1f4313d1d0a6063c)
    },
    {   UINT64_C(0x0000000000004c33), UINT64_C(0x1f431f510c99669e), UINT64_C(0x1f43134f889ca6e7),
        UINT64_C(0x1f432ad0a49d2679), UINT64_C(0x1f430751acace647)
    },
    {   UINT64_C(0x0000000000004c34), UINT64_C(0x1f430751acb0e6fb), UINT64_C(0x1f42fd11acaf86ed),
        UINT64_C(0x1f432a5090986692), UINT64_C(0x1f43201090a6c6ff)
    },
    {   UINT64_C(0x0000000000004c35), UINT64_C(0x1f43191090a6e6d0), UINT64_C(0x1f4324d07c9c8654),
        UINT64_C(0x1f4317507cae26ed), UINT64_C(0x1f4317507cab06f2)
    },
    {   UINT64_C(0x0000000000004c36), UINT64_C(0x1f4305d248acc659), UINT64_C(0x1f4305d248acc6c0),
        UINT64_C(0x1f4305d248acc6d6), UINT64_C(0x1f43174fe8a266bd)
    },
    {   UINT64_C(0x0000000000004c37), UINT64_C(0x1f43174fe8aa86e9), UINT64_C(0x1f43174fe8aca703),
        UINT64_C(0x1f43141170a3a66a), UINT64_C(0x1f433a103c93e6b9)
    },
    {   UINT64_C(0x0000000000004c38), UINT64_C(0x1f43225154a1c69e), UINT64_C(0x1f431dd0b89da685),
        UINT64_C(0x1f431090d8a5c67b), UINT64_C(0x1f432450389c4640)
    },
    {   UINT64_C(0x0000000000004c39), UINT64_C(0x1f4310d038a2a6b4), UINT64_C(0x1f4323d1cca0e5f5),
        UINT64_C(0x1f4310d1cca6e657), UINT64_C(0x1f4336d1009325f6)
    },
    {   UINT64_C(0x0000000000004c3a), UINT64_C(0x1f432c1100996663), UINT64_C(0x1f432010889c06d7),
        UINT64_C(0x1f43300fdc91e6d7), UINT64_C(0x1f430b5074af867f)
    },
    {   UINT64_C(0x0000000000004c3b), UINT64_C(0x1f4324d178a1e688), UINT64_C(0x1f432391aca0464a),
        UINT64_C(0x1f43304ff098a5f2), UINT64_C(0x1f431f10109e0673)
    },
    {   UINT64_C(0x0000000000004c3c), UINT64_C(0x1f43108f84a7c6e2), UINT64_C(0x1f431051b0a7e685),
        UINT64_C(0x1f4327902c9d2675), UINT64_C(0x1f4324d064a16688)
    },
    {   UINT64_C(0x0000000000004c3d), UINT64_C(0x1f43180fd0aa86db), UINT64_C(0x1f432cd07c95663d),
        UINT64_C(0x1f4329cf1c9d2615), UINT64_C(0x1f4307cf4cb3c705)
    },
    {   UINT64_C(0x0000000000019489), UINT64_C(0x12c2f6caec8546a4), UINT64_C(0x12c2f6caec8546a4),
        UINT64_C(0x2e53050f8ce009c4), UINT64_C(0x2e53050f8ce949c4)
    },
    {   UINT64_C(0x0000000000004c3f), UINT64_C(0x1f430c5438ab6590), UINT64_C(0x1f4339515c95e69a),
        UINT64_C(0x1f431c115ca486d6), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007db6), UINT64_C(0x1f43468f0894a6ae), UINT64_C(0x1f4314d0f4a6068f),
        UINT64_C(0x1f43098fd0adc6f7), UINT64_C(0x1f4313d340aba64a)
    },
    {   UINT64_C(0x0000000000004c41), UINT64_C(0x0000000000000000), UINT64_C(0x1f430f4edca7271c),
        UINT64_C(0x1f43010ef4ac0709), UINT64_C(0x1f430c4fc8a806c7)
    },
    {   UINT64_C(0x0000000000004c42), UINT64_C(0x1f430a5074a8268c), UINT64_C(0x1f42ff0f14abe721),
        UINT64_C(0x1f42fd0f98b26708), UINT64_C(0x1f432c0f08a946f0)
    },
    {   UINT64_C(0x000000000001948e), UINT64_C(0x12130b0d10784812), UINT64_C(0x18f30b0d10924812),
        UINT64_C(0x18f30e0d1093c812), UINT64_C(0x16430a8d1087e812)
    },
    {   UINT64_C(0x0000000000007db8), UINT64_C(0x1f432750e0a4c677), UINT64_C(0x1f432ccef897a704),
        UINT64_C(0x1f430f4f94b3a70d), UINT64_C(0x1f430dcfa0b006c5)
    },
    {   UINT64_C(0x0000000000004c45), UINT64_C(0x1f430ece5ca02734), UINT64_C(0x1f4301cee0ae871b),
        UINT64_C(0x1f4310cff8a8e6cd), UINT64_C(0x1f430c4efcab2704)
    },
    {   UINT64_C(0x0000000000004c46), UINT64_C(0x1f43010e64b0e73e), UINT64_C(0x1f4309cfacad46e3),
        UINT64_C(0x1f430c8f50a786e6), UINT64_C(0x1f430950a0b226b1)
    },
    {   UINT64_C(0x0000000000004c47), UINT64_C(0x1f42fd4eacb1470c), UINT64_C(0x1f42f00f88b5e6ba),
        UINT64_C(0x1f43024fb8a6c6f4), UINT64_C(0x1f43064fc8b0c6b9)
    },
    {   UINT64_C(0x0000000000004c48), UINT64_C(0x1f43130ef0a926fe), UINT64_C(0x1f430f4ff0aea6da),
        UINT64_C(0x1f430c0f68a926fd), UINT64_C(0x1f42fbcf20ab2702)
    },
    {   UINT64_C(0x0000000000004c49), UINT64_C(0x1f42f14ee0ad6711), UINT64_C(0x1f43039080ae26f3),
        UINT64_C(0x1f4300cf60b066ee), UINT64_C(0x1f42f8547cb745a5)
    },
    {   UINT64_C(0x0000000000004c4a), UINT64_C(0x1f42ff4f38a426f7), UINT64_C(0x1f43108f8ca3c6af),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000019496), UINT64_C(0x1e8aed9920b8a6e0), UINT64_C(0x2ee2ee1ba8fa071c),
        UINT64_C(0x1e9aee1870b8e6f1), UINT64_C(0x1e8aed8b6cb8a373)
    },
    {   UINT64_C(0x0000000000019497), UINT64_C(0x1e9aee1870b8e6f1), UINT64_C(0x1e9aee1920b8e6e0),
        UINT64_C(0x2ee2ee1ba8fa071c), UINT64_C(0x1e9aee1870b8e6f1)
    },
    {   UINT64_C(0x0000000000019498), UINT64_C(0x1e9aee0b6cb8e373), UINT64_C(0x2ee2ee0cdcfa0369),
        UINT64_C(0x00023a0fa03dc67e), UINT64_C(0x28daee18c0e206d5)
    },
    {   UINT64_C(0x0000000000019499), UINT64_C(0x28e2ee18c0e206d5), UINT64_C(0x28e2ee1728e206c7),
        UINT64_C(0x28e2ee1728e206c7), UINT64_C(0x28e2ee0b20e2036d)
    },
    {   UINT64_C(0x00000000000140bb), UINT64_C(0x09d2ef4d0465242e), UINT64_C(0x0c5aef57e06f46b8),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001949b), UINT64_C(0x2323098950cf2bf0), UINT64_C(0x2323098950cf2bf0),
        UINT64_C(0x10631d8b447a6a06), UINT64_C(0x1072f9cb44688a06)
    },
    {   UINT64_C(0x000000000001949c), UINT64_C(0x10631d8b447a6a06), UINT64_C(0x1072f9cb44688a06),
        UINT64_C(0x3233009820fe06e6), UINT64_C(0x3233008e54fe0473)
    },
    {   UINT64_C(0x0000000000007dbf), UINT64_C(0x1f43248f049a46ec), UINT64_C(0x1f43284fcc9b4650),
        UINT64_C(0x1f431ecfcca3a6e1), UINT64_C(0x1f43178fac9ee664)
    },
    {   UINT64_C(0x000000000001949e), UINT64_C(0x3af2ee0c512a442a), UINT64_C(0x3af2ee18712a4732),
        UINT64_C(0x1583278b448fea06), UINT64_C(0x1582f74b4477ca06)
    },
    {   UINT64_C(0x0000000000007dc0), UINT64_C(0x1f43070fa0a9e6f6), UINT64_C(0x1f43119030aaa6cf),
        UINT64_C(0x1f430e5030ab86e9), UINT64_C(0x1f431e50389a66e6)
    },
    {   UINT64_C(0x00000000000194a0), UINT64_C(0x1c2b054b44a1aa06), UINT64_C(0x1c2b090b44a38a06),
        UINT64_C(0x19db0c8b44950a06), UINT64_C(0x19db0fcb4096aa05)
    },
    {   UINT64_C(0x00000000000194a1), UINT64_C(0x18cb14cb38952a06), UINT64_C(0x18cb070b388e4a06),
        UINT64_C(0x1c530f4b1c9f2a1a), UINT64_C(0x17731b897891ebfa)
    },
    {   UINT64_C(0x00000000000194a2), UINT64_C(0x177302c978858bfa), UINT64_C(0x226af58978c46bfa),
        UINT64_C(0x0fe32649787bebfa), UINT64_C(0x0fe2f2497861ebfa)
    },
    {   UINT64_C(0x00000000000194a3), UINT64_C(0x3e82ee0cdd388369), UINT64_C(0x1c5311c978982bfa),
        UINT64_C(0x1c531d49789debfa), UINT64_C(0x226af58978c46bfa)
    },
    {   UINT64_C(0x00000000000194a4), UINT64_C(0x2da2ee1870f506f1), UINT64_C(0x2d92ed9920f4c6ee),
        UINT64_C(0x3e82ee1ba938871c), UINT64_C(0x2da2ee1870f506f1)
    },
    {   UINT64_C(0x00000000000194a5), UINT64_C(0x2d92ed8b34f4c373), UINT64_C(0x2da2ee1870f506f1),
        UINT64_C(0x2da2ee1920f506ee), UINT64_C(0x3e82ee1ba938871c)
    },
    {   UINT64_C(0x00000000000194a6), UINT64_C(0x2da2ee1870f506f1), UINT64_C(0x2da2ee0b34f50373),
        UINT64_C(0x3e82ee0cdd388369), UINT64_C(0x0002664d302e067e)
    },
    {   UINT64_C(0x00000000000194a7), UINT64_C(0x3682ee18c11886d5), UINT64_C(0x3682ee18c11886d5),
        UINT64_C(0x3682ee17291886c7), UINT64_C(0x3682ee17291886c7)
    },
    {   UINT64_C(0x00000000000194a8), UINT64_C(0x3682ee0b2118836d), UINT64_C(0x3682ee0b2118836d),
        UINT64_C(0x3682ee0b451883ce), UINT64_C(0x3682ee0b451883ce)
    },
    {   UINT64_C(0x0000000000007dc5), UINT64_C(0x1f43130f60af66db), UINT64_C(0x1f43134fb4a586d4),
        UINT64_C(0x1f43134fb4ae06f2), UINT64_C(0x1f43198ec8a166e6)
    },
    {   UINT64_C(0x00000000000194aa), UINT64_C(0x11aaf1c978690bfa), UINT64_C(0x11ab264978834bfa),
        UINT64_C(0x11aaf1c978690bfa), UINT64_C(0x431b0198214126e6)
    },
    {   UINT64_C(0x0000000000007dc6), UINT64_C(0x1f43164ec8abe723), UINT64_C(0x1f4317500ca105bb),
        UINT64_C(0x1f4307101cabe62d), UINT64_C(0x1f4300501cb5c6f3)
    },
    {   UINT64_C(0x00000000000194ac), UINT64_C(0x3af2ee18912a4717), UINT64_C(0x4842ee0c515f842a),
        UINT64_C(0x4842ee18715f8732), UINT64_C(0x1a5330497ca14bfa)
    },
    {   UINT64_C(0x0000000000007dc7), UINT64_C(0x1f430dd054a6c6bc), UINT64_C(0x1f430a9054ad46bc),
        UINT64_C(0x1f43274f909e46db), UINT64_C(0x1f431d0f90aa46f3)
    },
    {   UINT64_C(0x00000000000056c8), UINT64_C(0x1f42fa0edcb3c6f7), UINT64_C(0x1f42f74f38b30712),
        UINT64_C(0x1f42f68e64b7472f), UINT64_C(0x1f42f98e5cb4e71b)
    },
    {   UINT64_C(0x0000000000007dc8), UINT64_C(0x1f43164fc8a8e6ff), UINT64_C(0x1f4313100ca506c9),
        UINT64_C(0x1f4310904ca906d3), UINT64_C(0x1f43064f74ab26e6)
    },
    {   UINT64_C(0x00000000000194b0), UINT64_C(0x19430107ac940dee), UINT64_C(0x28eaf207ace02dee),
        UINT64_C(0x14ab37c7ac8dcdee), UINT64_C(0x14aaf4c7ac6c4dee)
    },
    {   UINT64_C(0x0000000000007dc9), UINT64_C(0x1f430b10d0a8a6b7), UINT64_C(0x1f43130f44aa06a1),
        UINT64_C(0x1f430f8f44ad2710), UINT64_C(0x1f4308cf44b886ef)
    },
    {   UINT64_C(0x00000000000194b2), UINT64_C(0x3af2ee18952a4704), UINT64_C(0x3af2ee19212a46eb),
        UINT64_C(0x4e22ee1ba977071c), UINT64_C(0x3af2ee18952a4704)
    },
    {   UINT64_C(0x0000000000007dca), UINT64_C(0x1f430b4f7cabe6a3), UINT64_C(0x1f430b4f7cb2e707),
        UINT64_C(0x1f430b4f7cb186f2), UINT64_C(0x1f4307d054afa6df)
    },
    {   UINT64_C(0x00000000000056cb), UINT64_C(0x1f42f60f38b0a6f5), UINT64_C(0x1f42f54f04b10707),
        UINT64_C(0x1f43034e70b0c723), UINT64_C(0x1f42f70f04b32736)
    },
    {   UINT64_C(0x0000000000007dcb), UINT64_C(0x1f43011054b166df), UINT64_C(0x1f43011054b046df),
        UINT64_C(0x1f42ff91e0b06646), UINT64_C(0x1f4302d1e0b206bf)
    },
    {   UINT64_C(0x00000000000194b6), UINT64_C(0x4422ee0b214f036d), UINT64_C(0x4422ee0b214f036d),
        UINT64_C(0x4422ee0b454f03ce), UINT64_C(0x4422ee0b454f03ce)
    },
    {   UINT64_C(0x0000000000007dcc), UINT64_C(0x1f4302d1e0b206d6), UINT64_C(0x1f430f0fd4aae6ca),
        UINT64_C(0x1f43014fd4b706ca), UINT64_C(0x1f43014fd4b886ef)
    },
    {   UINT64_C(0x00000000000194b8), UINT64_C(0x1552f4c7ac6eedee), UINT64_C(0x155337c7ac906dee),
        UINT64_C(0x1552f4c7ac6eedee), UINT64_C(0x54230318218486e6)
    },
    {   UINT64_C(0x0000000000007dcd), UINT64_C(0x1f43224fa8a166d1), UINT64_C(0x1f43110fe0ae86d9),
        UINT64_C(0x1f43088facace6ca), UINT64_C(0x1f4312cfd8a7a6cf)
    },
    {   UINT64_C(0x00000000000056ce), UINT64_C(0x1f42f38e74b64722), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dce), UINT64_C(0x1f43250f609ec6f8), UINT64_C(0x1f431bcfe8a24660),
        UINT64_C(0x1f43060fe8aa26e8), UINT64_C(0x1f43230f84a0c644)
    },
    {   UINT64_C(0x0000000000013827), UINT64_C(0x178af74c38a0e6dc), UINT64_C(0x1612f88c389a06dc),
        UINT64_C(0x1382f88c389906dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dcf), UINT64_C(0x1f430e4f98a826e2), UINT64_C(0x1f4329cf5c9b8651),
        UINT64_C(0x1f431bcf5ca3a6f0), UINT64_C(0x1f43290f789b86fa)
    },
    {   UINT64_C(0x00000000000194be), UINT64_C(0x48c2ee1891618718), UINT64_C(0x48c2ee19456186f4),
        UINT64_C(0x5dc2ee1ba9b5871c), UINT64_C(0x48c2ee1891618718)
    },
    {   UINT64_C(0x0000000000007dd0), UINT64_C(0x1f432c8ed89926fe), UINT64_C(0x1f431f8fa89d26c7),
        UINT64_C(0x1f43040f64b0e708), UINT64_C(0x1f431dcfb8a106e4)
    },
    {   UINT64_C(0x00000000000194c0), UINT64_C(0x48c2ee1891618718), UINT64_C(0x48c2ee0b41618373),
        UINT64_C(0x5dc2ee0cddb58369), UINT64_C(0x000261c8882f27a8)
    },
    {   UINT64_C(0x000000000001382c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0fa2cacc647d8691)
    },
    {   UINT64_C(0x00000000000194c2), UINT64_C(0x5dc2ee0b21b5836d), UINT64_C(0x5dc2ee0b21b5836d),
        UINT64_C(0x5dc2ee0b45b583ce), UINT64_C(0x5dc2ee0b45b583ce)
    },
    {   UINT64_C(0x00000000000194c3), UINT64_C(0x652b049821c7e6e6), UINT64_C(0x652b048e55c7e473),
        UINT64_C(0x652aee0cadd32438), UINT64_C(0x652aee1895d32732)
    },
    {   UINT64_C(0x00000000000194c4), UINT64_C(0x6532ee0e39d3449d), UINT64_C(0x6532ee1895d34734),
        UINT64_C(0x62e2ee0c51ca042a), UINT64_C(0x62e2ee1871ca0732)
    },
    {   UINT64_C(0x00000000000194c5), UINT64_C(0x124afc0b6cbfebb8), UINT64_C(0x1bfafc0b6ce6abb8),
        UINT64_C(0x25aafc0b6d0d6bb8), UINT64_C(0x13dafc0b6cbfebb8)
    },
    {   UINT64_C(0x00000000000194c6), UINT64_C(0x1d8afc0b6ce6abb8), UINT64_C(0x273afc0b6d0d6bb8),
        UINT64_C(0x13dafc0b6cbfebb8), UINT64_C(0x13dafc0b6cbfebb8)
    },
    {   UINT64_C(0x0000000000007dd4), UINT64_C(0x1f432151ac9f45f5), UINT64_C(0x1f4310d1a0a6c657),
        UINT64_C(0x1f43258fd0a0e643), UINT64_C(0x1f43118fd0a646d0)
    },
    {   UINT64_C(0x00000000000194c8), UINT64_C(0x13dafc0b6cbfebb8), UINT64_C(0x13dafc0b6cbfebb8),
        UINT64_C(0x13dafc0b6cbfebb8), UINT64_C(0x13dafc0b6cbfebb8)
    },
    {   UINT64_C(0x0000000000007dd5), UINT64_C(0x1f431a509c9a06c5), UINT64_C(0x1f4317109caac6eb),
        UINT64_C(0x1f43130f5ca4e6a2), UINT64_C(0x1f43130f5cad46fa)
    },
    {   UINT64_C(0x00000000000194ca), UINT64_C(0x16fafc0b6cbfebb8), UINT64_C(0x199af40b6cc06bb8),
        UINT64_C(0x13dafc0b6cbfebb8), UINT64_C(0x16fafc0b6cbfebb8)
    },
    {   UINT64_C(0x00000000000194cb), UINT64_C(0x16fafc0b6cbfebb8), UINT64_C(0x13e2fc0b6cc00c55),
        UINT64_C(0x124afc08f8bfebb8), UINT64_C(0x0f82f84cac83869a)
    },
    {   UINT64_C(0x00000000000194cc), UINT64_C(0x16b2f84caca0469a), UINT64_C(0x1de2f84cacbd069a),
        UINT64_C(0x1182f84cac83869a), UINT64_C(0x18b2f84caca0469a)
    },
    {   UINT64_C(0x0000000000007dd7), UINT64_C(0x1f432091749d865b), UINT64_C(0x1f43199174af0718),
        UINT64_C(0x1f4312cf70a866c1), UINT64_C(0x1f430c0f70ac870a)
    },
    {   UINT64_C(0x00000000000194ce), UINT64_C(0x128af84ca883869e), UINT64_C(0x2402f84cacd9c69f),
        UINT64_C(0x1002f84ca883869e), UINT64_C(0x1002f84ca883869d)
    },
    {   UINT64_C(0x0000000000007dd8), UINT64_C(0x1f431bd040a42696), UINT64_C(0x1f431bd040a746ea),
        UINT64_C(0x1f431390189d465e), UINT64_C(0x1f430cd018abc6de)
    },
    {   UINT64_C(0x0000000000004bc5), UINT64_C(0x1f4318903cae26c9), UINT64_C(0x1f4318cf50ad06cb),
        UINT64_C(0x1f430d907cabe69c), UINT64_C(0x1f4303cef8b1872d)
    },
    {   UINT64_C(0x0000000000007dd9), UINT64_C(0x1f431190549d0681), UINT64_C(0x1f43119054a666e7),
        UINT64_C(0x1f43220f6c9da650), UINT64_C(0x1f4317cf6cabe6f4)
    },
    {   UINT64_C(0x00000000000194d2), UINT64_C(0x0e72f84ca88386f8), UINT64_C(0x0f82f84b3083869e),
        UINT64_C(0x0b72fccaa0718720), UINT64_C(0x12a2fccaa08e4720)
    },
    {   UINT64_C(0x0000000000004c88), UINT64_C(0x1f42fb8f00b4c705), UINT64_C(0x1f42fb8f00b4c705),
        UINT64_C(0x1f42fb8f00b4c705), UINT64_C(0x1f42fb8f00b4c705)
    },
    {   UINT64_C(0x0000000000004c89), UINT64_C(0x1f42fb8f00b4c705), UINT64_C(0x1f42fb8f00b4c705),
        UINT64_C(0x1f42fb8f00b4c705), UINT64_C(0x1f42fb8f00b4c705)
    },
    {   UINT64_C(0x0000000000004c8a), UINT64_C(0x1f42fb8f00b4c705), UINT64_C(0x1f42fb8f00b4c705),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194d6), UINT64_C(0x0ad2f6ca7074e714), UINT64_C(0x0ef2fb8a7082e714),
        UINT64_C(0x0b6af6ca7074c714), UINT64_C(0x0c02fb8a70772714)
    },
    {   UINT64_C(0x0000000000004c8c), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f44e54b84724),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000194d8), UINT64_C(0x0e3af94a707ee714), UINT64_C(0x0fc2f94a7085a714),
        UINT64_C(0x0fc2f94a7085a714), UINT64_C(0x0e2af94a707c678f)
    },
    {   UINT64_C(0x0000000000007ddd), UINT64_C(0x1f4313d1d0a6063c), UINT64_C(0x1f431f510c99669e),
        UINT64_C(0x1f43134f889ca6e7), UINT64_C(0x1f432ad0a49d2679)
    },
    {   UINT64_C(0x00000000000194da), UINT64_C(0x1342fc0b6c942bb8), UINT64_C(0x1a8af7cb6ca8abb8),
        UINT64_C(0x244af70b6cd10bb8), UINT64_C(0x1302f9cb6c9a8bb8)
    },
    {   UINT64_C(0x00000000000194db), UINT64_C(0x1342fc0b6c9bcbb8), UINT64_C(0x12eafc0b6ca0abb8),
        UINT64_C(0x1c92fc8b6ca98bb8), UINT64_C(0x2baafc8b6cf2cbb8)
    },
    {   UINT64_C(0x00000000000194dc), UINT64_C(0x130af6cb6c8f6bb8), UINT64_C(0x130af6cb688f6bb7),
        UINT64_C(0x1492f44b689a4bb7), UINT64_C(0x13aafc0b6c8debb8)
    },
    {   UINT64_C(0x00000000000194dd), UINT64_C(0x1302f9cb6c9a8bb8), UINT64_C(0x11c2fc8b6c892bb8),
        UINT64_C(0x1092fc8b6c896bb8), UINT64_C(0x15bafc8b6c9f8bb8)
    },
    {   UINT64_C(0x00000000000194de), UINT64_C(0x133afc0b6c940bb8), UINT64_C(0x146afc0b6c93cbb8),
        UINT64_C(0x1d8af40b6cc06bb8), UINT64_C(0x12aafd8b6c8a8bb8)
    },
    {   UINT64_C(0x0000000000007de0), UINT64_C(0x1f4317507cab06f2), UINT64_C(0x1f4305d248acc659),
        UINT64_C(0x1f4305d248acc6c0), UINT64_C(0x1f4305d248acc6d6)
    },
    {   UINT64_C(0x00000000000194e0), UINT64_C(0x0b2afb0cac6d669a), UINT64_C(0x125afb0cac8a269a),
        UINT64_C(0x198afb0caca6e69a), UINT64_C(0x0e22fb0cac72069a)
    },
    {   UINT64_C(0x0000000000007de1), UINT64_C(0x1f43174fe8a266bd), UINT64_C(0x1f43174fe8aa86e9),
        UINT64_C(0x1f43174fe8aca703), UINT64_C(0x1f43141170a3a66a)
    },
    {   UINT64_C(0x00000000000194e2), UINT64_C(0x0f02fb0cac79469a), UINT64_C(0x1162f84ca87ee69e),
        UINT64_C(0x20a2ff0ca8c5a69e), UINT64_C(0x0d22ff0ca86f669e)
    },
    {   UINT64_C(0x0000000000004bd8), UINT64_C(0x1f42fbcf34b5a6ff), UINT64_C(0x1f431d8f8cb4a6f9),
        UINT64_C(0x1f42fecf88b20706), UINT64_C(0x1f430d8f98b0e71c)
    },
    {   UINT64_C(0x0000000000004bd9), UINT64_C(0x1f430ccf68a346f9), UINT64_C(0x1f431310d8ac06a1),
        UINT64_C(0x1f4302d034b726ea), UINT64_C(0x1f42fb0f94a0c71a)
    },
    {   UINT64_C(0x0000000000007de3), UINT64_C(0x1f432450389c4640), UINT64_C(0x1f4310d038a2a6b4),
        UINT64_C(0x1f4323d1cca0e5f5), UINT64_C(0x1f4310d1cca6e657)
    },
    {   UINT64_C(0x00000000000194e6), UINT64_C(0x0e12fccca870e69e), UINT64_C(0x0bdaff0cac7286f8),
        UINT64_C(0x0b2afb0b3470869a), UINT64_C(0x2503bd4fa0ae6663)
    },
    {   UINT64_C(0x00000000000194e7), UINT64_C(0x2503bd4fa0d5e65a), UINT64_C(0x13e30f4b1c7d6a1a),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004bdd), UINT64_C(0x1f4306501cb166e9), UINT64_C(0x1f42f8cf04b006fc),
        UINT64_C(0x1f43094eacb70723), UINT64_C(0x1f431a4e60a88735)
    },
    {   UINT64_C(0x0000000000004bde), UINT64_C(0x1f43068f78af8717), UINT64_C(0x1f4315cf48a4c6e0),
        UINT64_C(0x1f4303ce9cb36744), UINT64_C(0x1f42ffcfa0a3a6e5)
    },
    {   UINT64_C(0x0000000000013855), UINT64_C(0x14faeecfa08a4675), UINT64_C(0x1572f24fa0930675),
        UINT64_C(0x13faf24fa0918675), UINT64_C(0x0eaaf28fa0918675)
    },
    {   UINT64_C(0x0000000000013856), UINT64_C(0x15f2f6cf58964682), UINT64_C(0x0a02f34fa06a4675),
        UINT64_C(0x1192f20f5888e675), UINT64_C(0x15e2f24fa099c675)
    },
    {   UINT64_C(0x000000000001110e), UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1352f78fa098e675),
        UINT64_C(0x16f2fa8f709a4684), UINT64_C(0x1b4af78fa0b1e675)
    },
    {   UINT64_C(0x000000000001110f), UINT64_C(0x0f02f78fa080c675), UINT64_C(0x10e2fd0f708c0675),
        UINT64_C(0x17d2f78fa0a2c675), UINT64_C(0x1622f78fa0920675)
    },
    {   UINT64_C(0x0000000000011110), UINT64_C(0x1f6af78fa0c26675), UINT64_C(0x1a9af78fa0af2675),
        UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1292f78fa0968675)
    },
    {   UINT64_C(0x000000000001385a), UINT64_C(0x0fc2f80f747a85a1), UINT64_C(0x0a5af50fa07f6693),
        UINT64_C(0x12baf04c308605a1), UINT64_C(0x1262f4cfa0826693)
    },
    {   UINT64_C(0x000000000001385b), UINT64_C(0x110af44fa080c693), UINT64_C(0x0992f4cfa06b0693),
        UINT64_C(0x1acaf4cfa0a405a1), UINT64_C(0x1262f4cfa08265a1)
    },
    {   UINT64_C(0x0000000000011113), UINT64_C(0x10baf6cfa0956675), UINT64_C(0x1822fd0fa0a2c675),
        UINT64_C(0x0fb2f80f787985a1), UINT64_C(0x0eb2f94f7476c684)
    },
    {   UINT64_C(0x0000000000011114), UINT64_C(0x0cfaf80f747085a1), UINT64_C(0x10a2f80f70806684),
        UINT64_C(0x0deaf80f7471c5a1), UINT64_C(0x115af80cb48b6684)
    },
    {   UINT64_C(0x0000000000011115), UINT64_C(0x0f62f30cb47c05a1), UINT64_C(0x0000000000000000),
        UINT64_C(0x09bafa8f745ea650), UINT64_C(0x0c2aea0cb46d0650)
    },
    {   UINT64_C(0x0000000000011116), UINT64_C(0x10f2f94f74806684), UINT64_C(0x09f2f94f78614684),
        UINT64_C(0x1632f58f809385a1), UINT64_C(0x0f8af58f8078e5a1)
    },
    {   UINT64_C(0x0000000000011117), UINT64_C(0x0e52f80f747545a1), UINT64_C(0x0f4ae68cc479c5a1),
        UINT64_C(0x0e52f80cc47865a1), UINT64_C(0x0cc2f58fa06fa5a1)
    },
    {   UINT64_C(0x0000000000011118), UINT64_C(0x0dc2fa8f746f45a1), UINT64_C(0x09caf80f7c61e61f),
        UINT64_C(0x0ed2f58f7c7605a1), UINT64_C(0x0fd3000f7c7a65b2)
    },
    {   UINT64_C(0x0000000000011119), UINT64_C(0x183b000f7c9c05b4), UINT64_C(0x1132f58f7c7e45a1),
        UINT64_C(0x0f82f58cc47c85a0), UINT64_C(0x0f9af88f6878e5aa)
    },
    {   UINT64_C(0x00000000000140ea), UINT64_C(0x14aaf24fa0a00695), UINT64_C(0x1942f44fa09f4695),
        UINT64_C(0x0c2ae60fa07b4771), UINT64_C(0x131af34fa095a771)
    },
    {   UINT64_C(0x0000000000004bee), UINT64_C(0x1f43078f2cb1c726), UINT64_C(0x1f43068ec0b68731),
        UINT64_C(0x1f43120ebcae0722), UINT64_C(0x1f430b0e44b3671e)
    },
    {   UINT64_C(0x0000000000004bef), UINT64_C(0x1f431a0e60aaa732), UINT64_C(0x1f43028f4cab26d7),
        UINT64_C(0x1f430fcee4a8671c), UINT64_C(0x1f43158e80a8c711)
    },
    {   UINT64_C(0x0000000000004bf0), UINT64_C(0x1f4314cecca84712), UINT64_C(0x1f43058f54b7673e),
        UINT64_C(0x1f43088f4cb02739), UINT64_C(0x1f431ace6cade73a)
    },
    {   UINT64_C(0x0d9180000530c013), UINT64_C(0x14daf2cfa08e468d), UINT64_C(0x1d82f18fa0b1a68c),
        UINT64_C(0x1692f20f5896268c), UINT64_C(0x1852f6cf549b669b)
    },
    {   UINT64_C(0x0000000000007def), UINT64_C(0x1f431450c4a6268e), UINT64_C(0x1f432c90c499068e),
        UINT64_C(0x1f4330109c94e698), UINT64_C(0x1f4311109ca18698)
    },
    {   UINT64_C(0x00000000000140ed), UINT64_C(0x0f82ed4f6c7786a2), UINT64_C(0x0e32eccf6c7185b6),
        UINT64_C(0x0cfaeccc6c79a6a2), UINT64_C(0x0f42ec4c6c79c5b6)
    },
    {   UINT64_C(0x0000000000007df0), UINT64_C(0x1f432690a09a868e), UINT64_C(0x1f4337d09c96a5cc),
        UINT64_C(0x1f4337d09c96c68e), UINT64_C(0x1f433810c49685cc)
    },
    {   UINT64_C(0x0000000000004bf5), UINT64_C(0x1f43020ebcb3073f), UINT64_C(0x1f43010ef4b72733),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007df1), UINT64_C(0x1f433810c89685cd), UINT64_C(0x1f43340fdc9245cc),
        UINT64_C(0x1f4337d0a096a5cc), UINT64_C(0x1f4337d0a096a65c)
    },
    {   UINT64_C(0x000000000001386d), UINT64_C(0x0fa3038f6886c68c), UINT64_C(0x0fa325cfa079268d),
        UINT64_C(0x0fa2f6cfa086468c), UINT64_C(0x0fa2f90f6882868c)
    },
    {   UINT64_C(0x000000000001386e), UINT64_C(0x0fa2fb0fa082e68c), UINT64_C(0x0fa2fa4f688cc68c),
        UINT64_C(0x0fa302cf688ba694), UINT64_C(0x0fa3128f808b867e)
    },
    {   UINT64_C(0x000000000001386f), UINT64_C(0x0fa3024f6884868c), UINT64_C(0x0fa2facf4c86e68c),
        UINT64_C(0x10c2f94f78806693), UINT64_C(0x14a2f50fa08d868a)
    },
    {   UINT64_C(0x0000000000011127), UINT64_C(0x1abaf5cf64a8468a), UINT64_C(0x0000000000000000),
        UINT64_C(0x18eaf74f64a02697), UINT64_C(0x1baaf70f64a4c697)
    },
    {   UINT64_C(0x0000000000013871), UINT64_C(0x15d2f44fa09d867e), UINT64_C(0x15e2fd0fa09a667e),
        UINT64_C(0x186300cf689ce68c), UINT64_C(0x0d8afa8fa07d667e)
    },
    {   UINT64_C(0x0000000000013872), UINT64_C(0x13bafd0fa097e67e), UINT64_C(0x14c2f30fa08c868a),
        UINT64_C(0x1bd2fd0fa0b1467e), UINT64_C(0x16a2fd0f689c867e)
    },
    {   UINT64_C(0x0000000000013873), UINT64_C(0x1692f9cfa09cc67e), UINT64_C(0x186300cf689ce68c),
        UINT64_C(0x15e2fd0fa09a667e), UINT64_C(0x10d2fd0fa08c867e)
    },
    {   UINT64_C(0x0000000000013874), UINT64_C(0x186300cf689ce68c), UINT64_C(0x1472f34fa096c67e),
        UINT64_C(0x124b000fa090e67e), UINT64_C(0x128b02cfa099268c)
    },
    {   UINT64_C(0x0000000000013875), UINT64_C(0x1702fb0fa09a867e), UINT64_C(0x1692f30fa0a1e67e),
        UINT64_C(0x1643084fa0a32691), UINT64_C(0x18daf7cfa0a1e68c)
    },
    {   UINT64_C(0x000000000001112d), UINT64_C(0x1cf304cc18b10697), UINT64_C(0x1d22fccf64b08697),
        UINT64_C(0x199af58f749d45a1), UINT64_C(0x1222f9cf70846697)
    },
    {   UINT64_C(0x000000000001112e), UINT64_C(0x14b2f58f748825a1), UINT64_C(0x1a6af58f74a5e697),
        UINT64_C(0x0000000000000000), UINT64_C(0x156af4cc5c92a697)
    },
    {   UINT64_C(0x000000000001112f), UINT64_C(0x0000000000000000), UINT64_C(0x178af78f7494c697),
        UINT64_C(0x0f8302cf747b0675), UINT64_C(0x16d2f04c34984675)
    },
    {   UINT64_C(0x0000000000013879), UINT64_C(0x0bc2f84f687285b8), UINT64_C(0x0d92f8cc44788693),
        UINT64_C(0x10aaf94f787da5b7), UINT64_C(0x11aaf94f788825ad)
    },
    {   UINT64_C(0x000000000001387a), UINT64_C(0x10b2e5cc407e45b6), UINT64_C(0x0da2fb0c507d05b7),
        UINT64_C(0x12faf94f788ca5ad), UINT64_C(0x0ea2f88f787b45ad)
    },
    {   UINT64_C(0x0000000000011132), UINT64_C(0x101afd8f747205fb), UINT64_C(0x113af58f7481067a),
        UINT64_C(0x178af58f74952590), UINT64_C(0x1352f58f748745a1)
    },
    {   UINT64_C(0x000000000001387c), UINT64_C(0x13b2fa8f788e05ae), UINT64_C(0x0e72f94f7878e5b7),
        UINT64_C(0x10b2f94f70802693), UINT64_C(0x1442fa0c408d4694)
    },
    {   UINT64_C(0x000000000001387d), UINT64_C(0x10c2f78c407f45b7), UINT64_C(0x18daf80f78a3e5ad),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c08), UINT64_C(0x0000000000000000), UINT64_C(0x1f43594f28832717),
        UINT64_C(0x1f43408f7492a6f7), UINT64_C(0x1f43270f84a046eb)
    },
    {   UINT64_C(0x00000000000140f8), UINT64_C(0x16f2f4cc6c974695), UINT64_C(0x1092f9cc6c78e667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dfb), UINT64_C(0x1f431b0f48a2c6d5), UINT64_C(0x1f431bcf68a5e6f7),
        UINT64_C(0x1f42fc8f18b5462e), UINT64_C(0x1f431f10e0ad86b1)
    },
    {   UINT64_C(0x00000000000056fc), UINT64_C(0x1f430d0e60aea72f), UINT64_C(0x1f430e0e9cad6725),
        UINT64_C(0x1f43118f10b04731), UINT64_C(0x1f43108df0aa4732)
    },
    {   UINT64_C(0x2e7000000000c02e), UINT64_C(0x0a6aff0c386343e8), UINT64_C(0x0962f513ec60a698),
        UINT64_C(0x0a52f295d4658698), UINT64_C(0x0fa2f4d0307a2587)
    },
    {   UINT64_C(0x00000000000056fd), UINT64_C(0x0000000000000000), UINT64_C(0x1f4305cf0cb4e73c),
        UINT64_C(0x1f43138df0abc732), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000007dfd), UINT64_C(0x1f431d10e0ad86b1), UINT64_C(0x1f431c50b4ad86a7),
        UINT64_C(0x1f431d50b4ad86b1), UINT64_C(0x1f432050b49cc6b1)
    },
    {   UINT64_C(0x00000000000056fe), UINT64_C(0x1f430d4ebcb30722), UINT64_C(0x1f43028f6cb0e704),
        UINT64_C(0x1f430c4e80b10740), UINT64_C(0x1f430a8e8cb3c6f1)
    },
    {   UINT64_C(0x000000000000c032), UINT64_C(0x14daf20fa08ea7ab), UINT64_C(0x14daf20fa08ea7ab),
        UINT64_C(0x14daf20fa08ea7a2), UINT64_C(0x14daf20fa08ea77c)
    },
    {   UINT64_C(0x00000000000140fc), UINT64_C(0x11cae18f648885b6), UINT64_C(0x102aeb0c6877c5b6),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x3656c4000000c034), UINT64_C(0x1692ef8fa094c68c), UINT64_C(0x1692f20f58962784),
        UINT64_C(0x1852f6cf549b67ab), UINT64_C(0x1852f6cf549b67ab)
    },
    {   UINT64_C(0x0000000000005700), UINT64_C(0x0000000000000000), UINT64_C(0x1f42face98b30730),
        UINT64_C(0x1f43078ea4aec73f), UINT64_C(0x1f430b4ea4b0873e)
    },
    {   UINT64_C(0x0000000000004c14), UINT64_C(0x1f431c0f00a34703), UINT64_C(0x1f43150e90a466d6),
        UINT64_C(0x1f43150e90aca72a), UINT64_C(0x1f43250fec9d2690)
    },
    {   UINT64_C(0x000000000001388b), UINT64_C(0x0000000000000000), UINT64_C(0x1aa2f8ceeca4068d),
        UINT64_C(0x16a2f6cf94972686), UINT64_C(0x11caf8cf7487c687)
    },
    {   UINT64_C(0x000000000001388c), UINT64_C(0x190af68fa0a1067e), UINT64_C(0x114af80f90896686),
        UINT64_C(0x1462f8cfa097467e), UINT64_C(0x1222f80d94870687)
    },
    {   UINT64_C(0x000000000001388d), UINT64_C(0x19faf84f4ca82680), UINT64_C(0x11faf78fa088667e),
        UINT64_C(0x13c2f5cdc0a0a67e), UINT64_C(0x194af58f6ca06686)
    },
    {   UINT64_C(0x000000000001388e), UINT64_C(0x15aaf58f84902686), UINT64_C(0x2472f80eeccb8687),
        UINT64_C(0x1deaf80f0cc38703), UINT64_C(0x170af78f7899a685)
    },
    {   UINT64_C(0x000000000001388f), UINT64_C(0x14daf78fa097267e), UINT64_C(0x1742f8cd94968687),
        UINT64_C(0x1ab2f78f94a4867e), UINT64_C(0x13d2f78fa0926687)
    },
    {   UINT64_C(0x0000000000013890), UINT64_C(0x0feaf84fa099c6b9), UINT64_C(0x198af74f6cb54688),
        UINT64_C(0x13f2f64f1c93868d), UINT64_C(0x1de2f64f20bb4695)
    },
    {   UINT64_C(0x0000000000013891), UINT64_C(0x15a2f78f6c9ae688), UINT64_C(0x167af68d949de68b),
        UINT64_C(0x1782f78e2897c680), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c1c), UINT64_C(0x1f430f8f44ad2710), UINT64_C(0x1f4308cf44b886ef),
        UINT64_C(0x1f430b4f7cabe6a3), UINT64_C(0x1f430b4f7cb2e707)
    },
    {   UINT64_C(0x0000000000005705), UINT64_C(0x1f431b4f30a70713), UINT64_C(0x1f431c4ed8a5c6ee),
        UINT64_C(0x1f43184e98b52722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004c1e), UINT64_C(0x1f42ff91e0b06646), UINT64_C(0x1f4302d1e0b206bf),
        UINT64_C(0x1f4302d1e0b206d6), UINT64_C(0x1f430f0fd4aae6ca)
    },
    {   UINT64_C(0x0000000000014103), UINT64_C(0x1532e54fa095a79b), UINT64_C(0x0000000000000000),
        UINT64_C(0x14d306cf5897879f), UINT64_C(0x1852e5ccc0a16685)
    },
    {   UINT64_C(0x0000000000004c20), UINT64_C(0x1f43088facace6ca), UINT64_C(0x1f4312cfd8a7a6cf),
        UINT64_C(0x1f43250f609ec6f8), UINT64_C(0x1f431bcfe8a24660)
    },
    {   UINT64_C(0x0000000000004c21), UINT64_C(0x1f43060fe8aa26e8), UINT64_C(0x1f43230f84a0c644),
        UINT64_C(0x1f430e4f98a826e2), UINT64_C(0x1f4329cf5c9b8651)
    },
    {   UINT64_C(0x0000000000013898), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2fc0f747ec5b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000013899), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d3070fa07265b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001389a), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2fccfa07b65b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001389b), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f80c3c7ea5b5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001389c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f24c3c8245aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004ce7), UINT64_C(0x1f42fed080b3660e), UINT64_C(0x1f42ff5050b1660e),
        UINT64_C(0x1f43031080b1e6f0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001389e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f50f7883a684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001389f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f80c3c8265aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a0), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2fa8f787f6684), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a1), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x11d2f3cc3c8045b4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c04e), UINT64_C(0x12c2f2cfa08ce5be), UINT64_C(0x14daf2cfa08e4783),
        UINT64_C(0x08b2f1cfa05f0783), UINT64_C(0x14daf2c9b88e468c)
    },
    {   UINT64_C(0x000000000001410a), UINT64_C(0x1e02e5cfa0b94685), UINT64_C(0x1e02e5ccc0b94685),
        UINT64_C(0x186308cfa098c685), UINT64_C(0x1ecae94fa0bc6685)
    },
    {   UINT64_C(0x0000000000004c2e), UINT64_C(0x1f431bd040a746ea), UINT64_C(0x1f431390189d465e),
        UINT64_C(0x1f430cd018abc6de), UINT64_C(0x1f431190549d0681)
    },
    {   UINT64_C(0x00000000000138a5), UINT64_C(0x178af74c389cc6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x15d2f88c389446dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a6), UINT64_C(0x1532fb0c389806dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x19f2f88c38a1a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a7), UINT64_C(0x1612f88c389906dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1382f88c3c9906dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a8), UINT64_C(0x1802fb0c3c9e46dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x19caf88c38aa66dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138a9), UINT64_C(0x0d6aec4c3878e6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1092fd0c388586dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004cf4), UINT64_C(0x0000000000000000), UINT64_C(0x1f4315d080aaa6f1),
        UINT64_C(0x1f42f38e68b746f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138ab), UINT64_C(0x1e4af88c38bc66dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x18faf88c38a726dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138ac), UINT64_C(0x17e2fb0c389ae6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x122af18c3c9746dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138ad), UINT64_C(0x17e2fb0c3c9ae6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x15daf88c3893c6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018740), UINT64_C(0x0ff2f1cfa07c65bc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138af), UINT64_C(0x1612fecc389ee6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x12eafc0c3899c6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b0), UINT64_C(0x19fafd8c38b606dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x170af4cc389ce6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b1), UINT64_C(0x0f62ee4c388f66dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1572fd8c3897c6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b2), UINT64_C(0x11aaf88be08126dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1092f9cbe080c6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b3), UINT64_C(0x0eeaf64be07d26dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1252f18be08726dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b4), UINT64_C(0x0f52f00be079e6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1232ed0be09686dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018747), UINT64_C(0x123af34f688485b8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b6), UINT64_C(0x0a32ebcc3c7746dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1132e08be09026db), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b7), UINT64_C(0x1152fc4be08866dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0a7aea8c3c76a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b8), UINT64_C(0x19baf60be0a166dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x11aaf60be082a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138b9), UINT64_C(0x10aaf88be07f66dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x122ae80c3c8b26dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138ba), UINT64_C(0x104af80be07fe6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0da2f60be07de6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138bb), UINT64_C(0x0e92f48be07c46dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0b0ae88c3c75a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138bc), UINT64_C(0x10caf34be07da6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fd3000c3882a6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138bd), UINT64_C(0x183b000c3ca1a6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x11b2f60be08786dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138be), UINT64_C(0x1092f60c388666dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0f9af80be07de6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000571b), UINT64_C(0x1f42f88e40a9e739), UINT64_C(0x1f43024e84a80727),
        UINT64_C(0x1f4317ce88a2c709), UINT64_C(0x1f431acf8cb4a72c)
    },
    {   UINT64_C(0x000000000000c06c), UINT64_C(0x12c2f20f688cc63c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000571c), UINT64_C(0x0000000000000000), UINT64_C(0x1f4315900cac66d7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe4f14b646fa)
    },
    {   UINT64_C(0x000000000000c06e), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0dcaf04bec7265aa), UINT64_C(0x101af4cfa07c0698)
    },
    {   UINT64_C(0x000000000000571d), UINT64_C(0x1f43118e78a0e727), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43118e78a0e727), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060cb), UINT64_C(0x1f43044ed8a68722), UINT64_C(0x1f43124e98b1273f),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43034f64b0e736)
    },
    {   UINT64_C(0x000000000000571e), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fd0f74b5a703),
        UINT64_C(0x1f42f98fd8b5c6ac), UINT64_C(0x1f42fb8fecaea6b5)
    },
    {   UINT64_C(0x00000000000060cc), UINT64_C(0x0000000000000000), UINT64_C(0x1f43098e8cb6074e),
        UINT64_C(0x1f43140fc4ab6728), UINT64_C(0x1f43124f4cade719)
    },
    {   UINT64_C(0x000000000000571f), UINT64_C(0x1f42fc90c8b6a6e6), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43039140b386dd), UINT64_C(0x1f43014f28aea700)
    },
    {   UINT64_C(0x0000000000005fad), UINT64_C(0x0000000000000000), UINT64_C(0x1f43009008b646f2),
        UINT64_C(0x1f43090f80b6c6fc), UINT64_C(0x1f43048fd0b7c6f6)
    },
    {   UINT64_C(0x0000000000005720), UINT64_C(0x0000000000000000), UINT64_C(0x1f43160ee09f8717),
        UINT64_C(0x1f43284f64a7c717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060ce), UINT64_C(0x1f43084e6cb08743), UINT64_C(0x1f42fd8fa8af271e),
        UINT64_C(0x1f43158e74b36740), UINT64_C(0x1f43120ed8ad06f2)
    },
    {   UINT64_C(0x0000000000011182), UINT64_C(0x14d2f5cfa08de68a), UINT64_C(0x12e3008fa08ea67e),
        UINT64_C(0x14fb060f689ce68c), UINT64_C(0x15a3008fa09c667e)
    },
    {   UINT64_C(0x0000000000011183), UINT64_C(0x123b008fa093467e), UINT64_C(0x10bb008fa093667e),
        UINT64_C(0x15bb064f689ce68c), UINT64_C(0x1493008fa09c267e)
    },
    {   UINT64_C(0x0000000000011184), UINT64_C(0x0c8afccfa07e867e), UINT64_C(0x0c72f38f6879467e),
        UINT64_C(0x13d3008fa099a67e), UINT64_C(0x117b008fa085067e)
    },
    {   UINT64_C(0x0000000000011185), UINT64_C(0x1a5b00cfa0b3267e), UINT64_C(0x151b008f689e467e),
        UINT64_C(0x165306cf689fe68c), UINT64_C(0x106b008fa08e467e)
    },
    {   UINT64_C(0x0000000000011186), UINT64_C(0x166306cce49fe68c), UINT64_C(0x126b008fa08e667e),
        UINT64_C(0x10eafd8f6889268c), UINT64_C(0x1303164fa09c067e)
    },
    {   UINT64_C(0x00000000000138d0), UINT64_C(0x116afb0c388be6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x13b2ee0c3c9586dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000011188), UINT64_C(0x13b31c8fa09f467e), UINT64_C(0x13eaf50fa09de67e),
        UINT64_C(0x0e02fbcf7878e5b7), UINT64_C(0x0f83008f78816694)
    },
    {   UINT64_C(0x0000000000011189), UINT64_C(0x0e42fecf787d65b7), UINT64_C(0x0f73000f74898694),
        UINT64_C(0x0de2ff4f787b65b7), UINT64_C(0x0a83074fa0804693)
    },
    {   UINT64_C(0x00000000000138d3), UINT64_C(0x1242f60c388866dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0ea2ee4c387806dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001118b), UINT64_C(0x0e2afdcfa0840694), UINT64_C(0x066afd4fa065a694),
        UINT64_C(0x17a2fe4fa09f65b8), UINT64_C(0x0f3afdcfa07e45b8)
    },
    {   UINT64_C(0x000000000001118c), UINT64_C(0x0f9b010f788185b7), UINT64_C(0x0f92f18c4081c5b8),
        UINT64_C(0x0f93000c408325b8), UINT64_C(0x0a82fdcfa07565b8)
    },
    {   UINT64_C(0x000000000001118d), UINT64_C(0x0c2afd4f787485b7), UINT64_C(0x091b060f786d862c),
        UINT64_C(0x0f5b044f748185ad), UINT64_C(0x0ed311cf6883e5ad)
    },
    {   UINT64_C(0x00000000000138d7), UINT64_C(0x16eaf88c3c9d46dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x12d2f60c388846dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138d8), UINT64_C(0x14d2f88c389fc6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x1bcaf60c3caea6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138d9), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0de2efcbe07ac6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004d24), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8e80b4c721)
    },
    {   UINT64_C(0x00000000000138db), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0e4af5cbe07626dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138dc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1082f00be097e6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138dd), UINT64_C(0x10aaea0be08466dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x10aaf88c388026dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138de), UINT64_C(0x10aaf6cc387dc6dc), UINT64_C(0x0000000000000000),
        UINT64_C(0x0eeaf88c3881e6dc), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000138df), UINT64_C(0x0000000000000000), UINT64_C(0x1632e18c389546dc),
        UINT64_C(0x0000000000000000), UINT64_C(0x12f2f08f7889a693)
    },
    {   UINT64_C(0x00000000000138e0), UINT64_C(0x0000000000000000), UINT64_C(0x1152f7cf78820693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0b0aef8f7866e62b)
    },
    {   UINT64_C(0x00000000000138e1), UINT64_C(0x0000000000000000), UINT64_C(0x0a32f1cf786145b4),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f52f58fa07d4684)
    },
    {   UINT64_C(0x00000000000138e2), UINT64_C(0x0000000000000000), UINT64_C(0x0f52f58fa07a4684),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001480c), UINT64_C(0x22f3020f2cc406aa), UINT64_C(0x2e43020f2cf146aa),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fc9), UINT64_C(0x1f43014f0cae06f3), UINT64_C(0x1f4308cf1cae66fa),
        UINT64_C(0x1f43038f2cb4a711), UINT64_C(0x1f43050ed4ae26f8)
    },
    {   UINT64_C(0x000000000000572e), UINT64_C(0x1f4300ce6caaa73b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f7d040b686d6)
    },
    {   UINT64_C(0x000000000001480f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee1b987d072e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005fcc), UINT64_C(0x1f4302cf48b0c6f2), UINT64_C(0x1f43034f38b32711),
        UINT64_C(0x1f43040eccb5471c), UINT64_C(0x1f43018ec0b606f2)
    },
    {   UINT64_C(0x000000000000c094), UINT64_C(0x1002f14f687a05c1), UINT64_C(0x1302f60f6888c5c1),
        UINT64_C(0x1302f60f6888c5c1), UINT64_C(0x1183008f687fe69b)
    },
    {   UINT64_C(0x0000000000005730), UINT64_C(0x0000000000000000), UINT64_C(0x1f43150eaca5c72f),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c096), UINT64_C(0x0e4af44f6873e5c1), UINT64_C(0x0e4af2cf687325c1),
        UINT64_C(0x1582f84f689325c1), UINT64_C(0x0f6af44f687565c3)
    },
    {   UINT64_C(0x000000000000c097), UINT64_C(0x0f6af44f687565c3), UINT64_C(0x155af44f6890c5c3),
        UINT64_C(0x105af44f687925c3), UINT64_C(0x0a92e08c7465c5b5)
    },
    {   UINT64_C(0x00000000000060df), UINT64_C(0x1f42fe4e84b8c741), UINT64_C(0x1f43028e98ae6737),
        UINT64_C(0x1f430d0e60afe73a), UINT64_C(0x1f43040ee8a9a72a)
    },
    {   UINT64_C(0x000000000000c099), UINT64_C(0x1012f24f787ca5aa), UINT64_C(0x1132f24c188185aa),
        UINT64_C(0x1132f18fa0810693), UINT64_C(0x1162f20c6c7b2693)
    },
    {   UINT64_C(0x00000000000060e0), UINT64_C(0x0000000000000000), UINT64_C(0x1f430a0f48aee70b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430a4e6caae731)
    },
    {   UINT64_C(0x000000000000c09b), UINT64_C(0x0df2f24fa074068c), UINT64_C(0x0a32f1cbfc6e868c),
        UINT64_C(0x135af40bf089e68c), UINT64_C(0x19e2f20fa0a465c1)
    },
    {   UINT64_C(0x000000000000c09c), UINT64_C(0x1adaf20bfca705c1), UINT64_C(0x1a7af34bfc9ec5c1),
        UINT64_C(0x138adfcbfc87c5c1), UINT64_C(0x1312f34bfc9205c1)
    },
    {   UINT64_C(0x000000000000c09d), UINT64_C(0x12e2f68f708445b5), UINT64_C(0x1042f68f687b25c1),
        UINT64_C(0x172af50f8c97a5b5), UINT64_C(0x15c2f88f989025c5)
    },
    {   UINT64_C(0x00000000000111a9), UINT64_C(0x08b2f9cf745be5a1), UINT64_C(0x08b2cf0c645d45a1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c09f), UINT64_C(0x0d52f08bfc7285c1), UINT64_C(0x0e32f10bfc72c5c1),
        UINT64_C(0x0b3af1cfa06ec5cc), UINT64_C(0x0ca2f08fa06e25cc)
    },
    {   UINT64_C(0x000000000000c0a0), UINT64_C(0x1022f34fa07c65b8), UINT64_C(0x1022f34fa07c65b8),
        UINT64_C(0x0c2af44c386ba5c1), UINT64_C(0x0e52e50bfc714693)
    },
    {   UINT64_C(0x0000000000005736), UINT64_C(0x1f42fed050b366c3), UINT64_C(0x1f42fd4e80b2e72c),
        UINT64_C(0x1f430d4f78b34710), UINT64_C(0x1f43128e84a1c71b)
    },
    {   UINT64_C(0x000000000000c0a2), UINT64_C(0x0a9af2cbfc6f265e), UINT64_C(0x1162f04f6881c5b5),
        UINT64_C(0x0fa2f14f807b65ac), UINT64_C(0x10b2f3cf787e65b9)
    },
    {   UINT64_C(0x000000000000c0a3), UINT64_C(0x0fa2f34fa07b25c3), UINT64_C(0x1692f3cfa096e5c3),
        UINT64_C(0x0fa2f20fa07ac682), UINT64_C(0x13caf98fa087e5b8)
    },
    {   UINT64_C(0x000000000000c0a4), UINT64_C(0x109af34c3886a5b5), UINT64_C(0x10d2f34d487f25b5),
        UINT64_C(0x0dc2f00bf07445aa), UINT64_C(0x0dc2f00ad47445aa)
    },
    {   UINT64_C(0x0000000000005738), UINT64_C(0x1f42fd0f00b40719), UINT64_C(0x1f42fe1010b54704),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0a6), UINT64_C(0x169af14f5c970690), UINT64_C(0x0e42f1cfa075a5b8),
        UINT64_C(0x0e8af0cf687765c3), UINT64_C(0x12c2f54f7488e601)
    },
    {   UINT64_C(0x000000000000d402), UINT64_C(0x164af90fd0922632), UINT64_C(0x2042f90fd0ba0632),
        UINT64_C(0x2a3af8cfd0e1c632), UINT64_C(0x1862fbcfa099269b)
    },
    {   UINT64_C(0x000000000000d403), UINT64_C(0x0d62ff107c7405ab), UINT64_C(0x1163020f647a0690),
        UINT64_C(0x0d62ee107c6b85ab), UINT64_C(0x1163020f647a0690)
    },
    {   UINT64_C(0x000000000000d404), UINT64_C(0x1652f80f6492c640), UINT64_C(0x1652f80f6492c640),
        UINT64_C(0x1242f70e788306c1), UINT64_C(0x16f2f4cfa096e68c)
    },
    {   UINT64_C(0x000000000000d405), UINT64_C(0x16f2f4cfa096e68c), UINT64_C(0x1592f7cfa08fa68c),
        UINT64_C(0x1032f94c9c79a6c4), UINT64_C(0x19030a1010a80797)
    },
    {   UINT64_C(0x000000000000d406), UINT64_C(0x1772fc0fa0946597), UINT64_C(0x0913044f5457a5ab),
        UINT64_C(0x1772ff0eec93e607), UINT64_C(0x1772ff0d6093e670)
    },
    {   UINT64_C(0x000000000000d407), UINT64_C(0x1772ff0ea093e63e), UINT64_C(0x1772ff0cc493c697),
        UINT64_C(0x1772ff0bd493c6d3), UINT64_C(0x1773020b0c9246c0)
    },
    {   UINT64_C(0x00000000000111b8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x166af6cfa0942683), UINT64_C(0x15c2f78fa0944675)
    },
    {   UINT64_C(0x00000000000111b9), UINT64_C(0x1342f78fa098a675), UINT64_C(0x12a2f58fa0840683),
        UINT64_C(0x1652f78fa09a4675), UINT64_C(0x1822fd0fa0a2c675)
    },
    {   UINT64_C(0x00000000000111ba), UINT64_C(0x1b4af78fa0b1e675), UINT64_C(0x170afa8f74978685),
        UINT64_C(0x0f02f78fa080c675), UINT64_C(0x17d2f78fa0a2c675)
    },
    {   UINT64_C(0x00000000000111bb), UINT64_C(0x1672f6cfa0944683), UINT64_C(0x1f6af78fa0c26675),
        UINT64_C(0x1a9af78fa0af2675), UINT64_C(0x1612fb0fa09b2675)
    },
    {   UINT64_C(0x00000000000111bc), UINT64_C(0x16e2fa8f74978685), UINT64_C(0x1b4af78fa0b1e675),
        UINT64_C(0x1292f78fa0968675), UINT64_C(0x170afa8f74978685)
    },
    {   UINT64_C(0x00000000000111bd), UINT64_C(0x16fafc8fa09d8675), UINT64_C(0x1132f44fa0924675),
        UINT64_C(0x132af50fa09b6684), UINT64_C(0x1822f44fa09be675)
    },
    {   UINT64_C(0x00000000000111be), UINT64_C(0x18b2f44fa0a3c675), UINT64_C(0x14f2f50fa09b6683),
        UINT64_C(0x1902f60fa09fa682), UINT64_C(0x139af88f64898675)
    },
    {   UINT64_C(0x00000000000111bf), UINT64_C(0x1062f80f7880a5a1), UINT64_C(0x0f6af44cc47f4684),
        UINT64_C(0x0d62f6ccb477c5a1), UINT64_C(0x0e7af80f7476e684)
    },
    {   UINT64_C(0x00000000000111c0), UINT64_C(0x0f22f44f747605a1), UINT64_C(0x0d9af80cc47a8684),
        UINT64_C(0x0e62f58cc47765a1), UINT64_C(0x0f22f80f7479c684)
    },
    {   UINT64_C(0x00000000000111c1), UINT64_C(0x085afa8f745ae5a1), UINT64_C(0x10b2fa8fa08325a1),
        UINT64_C(0x10eafa8f607e6684), UINT64_C(0x121af58cc4832594)
    },
    {   UINT64_C(0x00000000000111c2), UINT64_C(0x0e22fa8f7c7845a6), UINT64_C(0x0d8af44cc475e684),
        UINT64_C(0x0e52f80f747545a1), UINT64_C(0x1172f6cf6c858594)
    },
    {   UINT64_C(0x00000000000111c3), UINT64_C(0x0fb2f58cc47985a1), UINT64_C(0x0dbaf6ccc47885d2),
        UINT64_C(0x10caf80f7482e594), UINT64_C(0x0dd2f58f8c798594)
    },
    {   UINT64_C(0x00000000000111c4), UINT64_C(0x0e62f58f7476259f), UINT64_C(0x14d2fa8cc48d65a1),
        UINT64_C(0x129af58c788f25a1), UINT64_C(0x14aaf58cc49765a1)
    },
    {   UINT64_C(0x00000000000111c5), UINT64_C(0x154af30f749125a1), UINT64_C(0x0ebaf80f74796684),
        UINT64_C(0x0d72f80f747445a1), UINT64_C(0x1152f30f787de68e)
    },
    {   UINT64_C(0x00000000000111c6), UINT64_C(0x118af10f6c87e5a1), UINT64_C(0x142af80cc48c0684),
        UINT64_C(0x0feaf80cb47ba5a1), UINT64_C(0x1ac2f58f74aac594)
    },
    {   UINT64_C(0x00000000000060f1), UINT64_C(0x1f430e8e30ad6735), UINT64_C(0x1f430e4e7caf8742),
        UINT64_C(0x1f43074e8cb70732), UINT64_C(0x1f42fdcf98aa6722)
    },
    {   UINT64_C(0x000000000000c0bd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6aea0bf46a237c)
    },
    {   UINT64_C(0x0000000000005ff7), UINT64_C(0x1f43158f0ca6a6f3), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fb8f70b406eb), UINT64_C(0x1f42fc4e7cac8727)
    },
    {   UINT64_C(0x0000000000005ff8), UINT64_C(0x1f43068e3cb34736), UINT64_C(0x1f430b8e40a86739),
        UINT64_C(0x1f42f54f84b226db), UINT64_C(0x1f430a8ed8b0c713)
    },
    {   UINT64_C(0x00000000000060f3), UINT64_C(0x1f43110ecca90735), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4313cf50ae8740), UINT64_C(0x1f430e4e90aaa734)
    },
    {   UINT64_C(0x000000000000c0c1), UINT64_C(0x000286587433e665), UINT64_C(0x0002711bf03e872e),
        UINT64_C(0x000289d7e032469b), UINT64_C(0x00029f9804276682)
    },
    {   UINT64_C(0x000000000000c0c2), UINT64_C(0x0002855804348682), UINT64_C(0x00029a174c2e26d7),
        UINT64_C(0x00029518042ca6d6), UINT64_C(0x000278d7e03aa6b1)
    },
    {   UINT64_C(0x000000000000c0c3), UINT64_C(0x00028597e03466a8), UINT64_C(0x0002a8d7e82426c2),
        UINT64_C(0x00029497e82cc6c2), UINT64_C(0x000278d7e03aa6b1)
    },
    {   UINT64_C(0x0000000000005ffd), UINT64_C(0x1f42f4cfc8b666ee), UINT64_C(0x1f42f9ced0b1e70b),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fccf38aec705)
    },
    {   UINT64_C(0x0000000000005748), UINT64_C(0x1f43040ed8b5472e), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fbcf00b7a732), UINT64_C(0x1f430c4ea8aee71b)
    },
    {   UINT64_C(0x0000000000014843), UINT64_C(0x0000000000000000), UINT64_C(0x182af34fa0a38685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014844), UINT64_C(0x1f2af9cf4cb8a6a3), UINT64_C(0x0000000000000000),
        UINT64_C(0x2062f80f4cbd66a3), UINT64_C(0x0fa2f8cf688d86a3)
    },
    {   UINT64_C(0x00000000000060f7), UINT64_C(0x1f43104e98b60739), UINT64_C(0x1f42ff4ebcb4a73a),
        UINT64_C(0x1f43084f48a8c713), UINT64_C(0x1f4314cea0b3e735)
    },
    {   UINT64_C(0x0000000000014846), UINT64_C(0x0000000000000000), UINT64_C(0x0f8af28fa097e685),
        UINT64_C(0x1792f6ce789a4695), UINT64_C(0x20c2fbcf4cb826a3)
    },
    {   UINT64_C(0x0000000000004ca8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42face8cb50724)
    },
    {   UINT64_C(0x0000000000014848), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f413dcb8a68c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014849), UINT64_C(0x193af3cfa0a72685), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0cd), UINT64_C(0x0002821294362524), UINT64_C(0x0002869320310502),
        UINT64_C(0x00026e93203d4502), UINT64_C(0x0002878e7839262c)
    },
    {   UINT64_C(0x00000000000060fa), UINT64_C(0x1f430a4ec8afc732), UINT64_C(0x1f43060e9cb3a725),
        UINT64_C(0x1f430a4efca9e72b), UINT64_C(0x1f43140f74a7872e)
    },
    {   UINT64_C(0x000000000000574d), UINT64_C(0x1f43008e8caee732), UINT64_C(0x1f43050ee8b14707),
        UINT64_C(0x1f43078ea8ad8713), UINT64_C(0x1f43058f9cad6720)
    },
    {   UINT64_C(0x000000000001484d), UINT64_C(0x1a82f6cf68a005b6), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000574e), UINT64_C(0x1f43030f20abe6f8), UINT64_C(0x1f43024e1cb4a725),
        UINT64_C(0x1f4302cea0b30707), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001484f), UINT64_C(0x16d2f60f6c97e5a9), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x18e2f6cfa0a52685)
    },
    {   UINT64_C(0x000000000000c0d3), UINT64_C(0x00027e982c34672d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000014851), UINT64_C(0x0000000000000000), UINT64_C(0x1762f28fa09a2685),
        UINT64_C(0x13caf94f6c95c6a3), UINT64_C(0x11faf94f6c8205b6)
    },
    {   UINT64_C(0x0000000000014852), UINT64_C(0x0bdaf80fa0722685), UINT64_C(0x0d2ad6cc6c776685),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000187bc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1692f38fa0966762), UINT64_C(0x0fa2f18c3879e68e)
    },
    {   UINT64_C(0x000000000000c0d7), UINT64_C(0x000289cbd8640399), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000060ff), UINT64_C(0x1f43140ee4b0673b), UINT64_C(0x1f42ffcf48b88722),
        UINT64_C(0x1f4309ce54b1872e), UINT64_C(0x1f430f0e44a7272a)
    },
    {   UINT64_C(0x0000000000005752), UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff4ea4b4c739),
        UINT64_C(0x1f42fa8ed8b2c714), UINT64_C(0x1f42ffcf38ace6ea)
    },
    {   UINT64_C(0x0000000000006100), UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4e34aae73c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43060e5caf4740)
    },
    {   UINT64_C(0x0000000000006014), UINT64_C(0x1f42f6cf78b06724), UINT64_C(0x1f42ff4e8cb2e6d8),
        UINT64_C(0x1f42f90f84b466fa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006101), UINT64_C(0x1f430bcea8a9a748), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43124e4cab6731), UINT64_C(0x1f43064e4cb0072b)
    },
    {   UINT64_C(0x0000000000006016), UINT64_C(0x1f42fe8ea0b00740), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430b0eacb8671d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006017), UINT64_C(0x1f42f70f38b366f2), UINT64_C(0x1f42f60ee0b6471b),
        UINT64_C(0x1f43144eb8aaa726), UINT64_C(0x1f42f5cdd8aa874d)
    },
    {   UINT64_C(0x000000000000c0df), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0a6b028cd05fc5c0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006103), UINT64_C(0x1f4312cea8aae746), UINT64_C(0x1f430e4e4cb5c73e),
        UINT64_C(0x1f43078e80aee73b), UINT64_C(0x1f43130e98a8072e)
    },
    {   UINT64_C(0x000000000000c0e1), UINT64_C(0x09630b97e05f669c), UINT64_C(0x0c32ed97e06f869c),
        UINT64_C(0x1692f04fa094a6a4), UINT64_C(0x0a6b02945c5de5c0)
    },
    {   UINT64_C(0x000000000000c0e2), UINT64_C(0x1902f08fa0a166a4), UINT64_C(0x1cc2f08fa0af86a4),
        UINT64_C(0x1092f08fa07f06a4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c0e3), UINT64_C(0x1852f08f549b66a4), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ae2f08fa0a846a4), UINT64_C(0x1862f08fa09da6a4)
    },
    {   UINT64_C(0x000000000000c0e4), UINT64_C(0x0c32ed8f686f869c), UINT64_C(0x1692f04fa094a69a),
        UINT64_C(0x14daf20fa08be68c), UINT64_C(0x1362f20fa088a68c)
    },
    {   UINT64_C(0x000000000000c0e5), UINT64_C(0x1692f64fa092a69a), UINT64_C(0x14daf20fa08ea68c),
        UINT64_C(0x14daf50fa08dc68c), UINT64_C(0x1852f34fa09d668c)
    },
    {   UINT64_C(0x0000000000006106), UINT64_C(0x1f43088e90b4e71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430b8e20ac6738)
    },
    {   UINT64_C(0x0000000000006020), UINT64_C(0x1f43094f34b1e725), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8e68b40729), UINT64_C(0x1f43054e84b16726)
    },
    {   UINT64_C(0x0000000000006021), UINT64_C(0x1f42fc4e88b4871d), UINT64_C(0x1f4302ce58b1e735),
        UINT64_C(0x1f42f9ce7cb92730), UINT64_C(0x1f42ff0e6cb6473c)
    },
    {   UINT64_C(0x000000000000575a), UINT64_C(0x1f4301cef4afa717), UINT64_C(0x1f431c4e54a9273e),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43005004ab870b)
    },
    {   UINT64_C(0x0000000000006108), UINT64_C(0x1f430a8e34ad4738), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430a8e94ad6729), UINT64_C(0x1f4308ceccb3873c)
    },
    {   UINT64_C(0x0000000000008808), UINT64_C(0x0fa2fccd4c75c68c), UINT64_C(0x0fa2fc8d3c75c68c),
        UINT64_C(0x105aff908c7765a4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006109), UINT64_C(0x1f430fcf44aba71e), UINT64_C(0x1f4310cf1caaa72b),
        UINT64_C(0x1f430ece98ac271f), UINT64_C(0x1f4310ceccaee73e)
    },
    {   UINT64_C(0x0000000000008809), UINT64_C(0x0000000000000000), UINT64_C(0x14db09cf7483e44c),
        UINT64_C(0x1f4309cf74ad844c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006027), UINT64_C(0x1f42fd4edcad46e4), UINT64_C(0x1f42fc0f18b4e6f5),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb0f84b206d0)
    },
    {   UINT64_C(0x000000000000575d), UINT64_C(0x1f42fc0ee8b80715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fc0f58b466f5), UINT64_C(0x1f42fb0fe0b6c6d6)
    },
    {   UINT64_C(0x000000000000610b), UINT64_C(0x1f43180e78ab873d), UINT64_C(0x1f43150e0cab472d),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43160e28a78737)
    },
    {   UINT64_C(0x000000000000575e), UINT64_C(0x0000000000000000), UINT64_C(0x1f4314ce98a9e734),
        UINT64_C(0x1f42fb0eecb7c717), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000610c), UINT64_C(0x1f43138e20b42738), UINT64_C(0x1f43120ef4af073e),
        UINT64_C(0x1f430e4e4ca9273a), UINT64_C(0x1f43140e78aa272b)
    },
    {   UINT64_C(0x000000000000c0f3), UINT64_C(0x0fa2f44f687a069c), UINT64_C(0x1202f10f6883669c),
        UINT64_C(0x16eaf48f6897069c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000610d), UINT64_C(0x1f43074e58a88738), UINT64_C(0x1f43120e64a9a73b),
        UINT64_C(0x1f43098ea4b80731), UINT64_C(0x1f43130e58a72733)
    },
    {   UINT64_C(0x000000000000c0f5), UINT64_C(0x0000000000000000), UINT64_C(0x146af40c6c8d268c),
        UINT64_C(0x1b02f04f68a8e5b5), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000610e), UINT64_C(0x1f43120f00b2c73f), UINT64_C(0x1f43094e58ad6737),
        UINT64_C(0x1f43160e34a6273a), UINT64_C(0x1f430f4e44b0071d)
    },
    {   UINT64_C(0x0007ec000000880e), UINT64_C(0x0ffafccd3077444e), UINT64_C(0x0a6afdd024622588),
        UINT64_C(0x0a6afa1024604588), UINT64_C(0x156afa0efc8de60b)
    },
    {   UINT64_C(0x000000000000c0f8), UINT64_C(0x1a3af64c6ca2a69c), UINT64_C(0x131af54c6c876667),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000880f), UINT64_C(0x112b0e8f7c77068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee1ba87d071c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006033), UINT64_C(0x1f43094e5cad0738), UINT64_C(0x1f42f54f70b806e6),
        UINT64_C(0x1f42fd0f94ad86fa), UINT64_C(0x1f42fa0e8cb16707)
    },
    {   UINT64_C(0x0000000000008810), UINT64_C(0x18f30017a09946ad), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6af7d250622534)
    },
    {   UINT64_C(0x000000000000c0fc), UINT64_C(0x119af10f5482c5c1), UINT64_C(0x0ffaf44c6c7b45c1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008811), UINT64_C(0x053ac40f6867e68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1a3aff0f80a3a68c)
    },
    {   UINT64_C(0x0000000000006037), UINT64_C(0x1f42f54f8cb7c6f5), UINT64_C(0x1f42f90ff8b566f4),
        UINT64_C(0x1f42fb4f34b58722), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005765), UINT64_C(0x1f4303cee0b1471b), UINT64_C(0x1f42fa4fb0b286fe),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fe0ef4b0c6f2)
    },
    {   UINT64_C(0x000000000000c100), UINT64_C(0x0000000000000000), UINT64_C(0x14daf20fa08ea77c),
        UINT64_C(0x1ac2f80e1ca3a68c), UINT64_C(0x13c2f30fa08987ab)
    },
    {   UINT64_C(0x0000000000008813), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ff0cf474a4d8), UINT64_C(0x08b2fd0d6c5765b3)
    },
    {   UINT64_C(0x000000000000c102), UINT64_C(0x0fa2eece207a468c), UINT64_C(0x1f6af08f58b6468c),
        UINT64_C(0x20f2f34fa0bc668c), UINT64_C(0x1b9af80fa0ab068c)
    },
    {   UINT64_C(0x0000000000014880), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x117af90f78846696), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c104), UINT64_C(0x1692f04fa094a69a), UINT64_C(0x14daf20fa08be68c),
        UINT64_C(0x14daf20fa08be68c), UINT64_C(0x13c2f30fa089868c)
    },
    {   UINT64_C(0x0000000000004ce3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fece68b4e60e), UINT64_C(0x1f43024e68b466f0)
    },
    {   UINT64_C(0x000000000000c106), UINT64_C(0x1852f34fa09d668c), UINT64_C(0x1852f34fa09d6786),
        UINT64_C(0x17baf34fa09b2694), UINT64_C(0x1712f08f5898668c)
    },
    {   UINT64_C(0x0000000000014884), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x12f2fad2e483e511), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c108), UINT64_C(0x131af20fa089868c), UINT64_C(0x162af70f5492c69b),
        UINT64_C(0x14daf5cfa08e068c), UINT64_C(0x1692f1cf4895e68c)
    },
    {   UINT64_C(0x000000000000576a), UINT64_C(0x1f43084f50b12700), UINT64_C(0x1f42fd4f50b5a73b),
        UINT64_C(0x1f43144f18aa073b), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004ce8), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f85080b5c70c),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006044), UINT64_C(0x1f42f84eecb6072a), UINT64_C(0x1f42fe5044ae86db),
        UINT64_C(0x1f42f90f28b066ef), UINT64_C(0x1f42f58f50b6a708)
    },
    {   UINT64_C(0x000000000000c10c), UINT64_C(0x102af88f687da5c1), UINT64_C(0x0fa2f44f687a069b),
        UINT64_C(0x0f62f34fa079e5b5), UINT64_C(0x0e1af34fa074c5b5)
    },
    {   UINT64_C(0x000000000000576c), UINT64_C(0x1f42f50f38b386fb), UINT64_C(0x1f43014ed0af2720),
        UINT64_C(0x1f43048ee4a5c738), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000c10e), UINT64_C(0x1162f34fa08265b5), UINT64_C(0x1162f34fa082669b),
        UINT64_C(0x1162f38fa08265bb), UINT64_C(0x1112f0cf7480a5b5)
    },
    {   UINT64_C(0x000000000000576d), UINT64_C(0x1f4309cf00b4072e), UINT64_C(0x1f42fd4f1cb106f9),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fbcec0b28726)
    },
    {   UINT64_C(0x000000000000611b), UINT64_C(0x1f43088e6cb3273f), UINT64_C(0x1f43140f24aea72c),
        UINT64_C(0x1f42fdceb8b4c743), UINT64_C(0x1f430fce4cac872d)
    },
    {   UINT64_C(0x000000000000576e), UINT64_C(0x1f42fc4eacb6672c), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88ed0a90737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000611c), UINT64_C(0x1f43014e00b38734), UINT64_C(0x1f4315cf64ae072a),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43024e30afc738)
    },
    {   UINT64_C(0x0000000000004cf1), UINT64_C(0x1f43015050b1060e), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000611d), UINT64_C(0x1f43040ebcad472c), UINT64_C(0x1f42ffced0b86745),
        UINT64_C(0x1f4313ceb4af2745), UINT64_C(0x1f430d4e6caf6741)
    },
    {   UINT64_C(0x0000000000004cf3), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fa5080b466f1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000611e), UINT64_C(0x1f430d4eccb14745), UINT64_C(0x1f43068f00ae0731),
        UINT64_C(0x1f43114e6cae873d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004cf5), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f8d080b546f1),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000611f), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c4e5cb14738),
        UINT64_C(0x1f43150ec4afc72d), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000881f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x10eafc53d87b068c)
    },
    {   UINT64_C(0x0000000000006120), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43114e4caf4741), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005773), UINT64_C(0x1f4303ce8cb0c72a), UINT64_C(0x1f42fd8ee8b7a737),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43004ed8b1c734)
    },
    {   UINT64_C(0x0000000000006121), UINT64_C(0x1f430e4f10ab6729), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f430d4e6cade729), UINT64_C(0x1f43010eecb0272d)
    },
    {   UINT64_C(0x0000000000005774), UINT64_C(0x0000000000000000), UINT64_C(0x1f42fb8e50b4c731),
        UINT64_C(0x1f430a8f18ada721), UINT64_C(0x1f430d0e78ac0741)
    },
    {   UINT64_C(0x0000000000018804), UINT64_C(0x0a6af7d2a46224eb), UINT64_C(0x0a6af7d2a86224e9),
        UINT64_C(0x0fa2ee12a47d04eb), UINT64_C(0x0fa2ee12c47d04e2)
    },
    {   UINT64_C(0x0000000000006058), UINT64_C(0x1f42f64edcb82725), UINT64_C(0x1f42f54fa0ae86bf),
        UINT64_C(0x1f42f90f28b026b7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000f21de98806), UINT64_C(0x0a6b0ad6645e468c), UINT64_C(0x0a6b01d66459c68c),
        UINT64_C(0x0a6b01cd6c59c44e), UINT64_C(0x0a6b01d66459c68c)
    },
    {   UINT64_C(0x0007d6f89ec98807), UINT64_C(0x0de2f8d66472468c), UINT64_C(0x0de2f5966470a68c),
        UINT64_C(0x0de2f94d6c72844e), UINT64_C(0x0de2f5966470a68c)
    },
    {   UINT64_C(0x0000000000018808), UINT64_C(0x0fa2fccd4c75c68c), UINT64_C(0x0fa2fc8d3c75c68c),
        UINT64_C(0x105aff908c7765a4), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018809), UINT64_C(0x0000000000000000), UINT64_C(0x14db09cf7483e44c),
        UINT64_C(0x1f4309cf74ad844c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006125), UINT64_C(0x1f43118e7cab2731), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f430d4eb4b20736)
    },
    {   UINT64_C(0x0000000000005778), UINT64_C(0x1f43104f78ac66eb), UINT64_C(0x1f43068f64b646c7),
        UINT64_C(0x1f430f8e7cb20740), UINT64_C(0x1f43084e7cae6719)
    },
    {   UINT64_C(0x000000000001880c), UINT64_C(0x22aafd4f54c186aa), UINT64_C(0x2dfafd4f54eec6aa),
        UINT64_C(0x090b00d5e859468e), UINT64_C(0x0d5300d5e46a668e)
    },
    {   UINT64_C(0x000000000001880d), UINT64_C(0x119b00d5e47b868e), UINT64_C(0x090b00d5e859468e),
        UINT64_C(0x0d5300d5e46a668e), UINT64_C(0x119b00d5e47b868e)
    },
    {   UINT64_C(0x0007ec000001880e), UINT64_C(0x0ffafccd3077444e), UINT64_C(0x0a6afdd024622588),
        UINT64_C(0x0a6afa1024604588), UINT64_C(0x156afa0efc8de60b)
    },
    {   UINT64_C(0x000000000001880f), UINT64_C(0x112b0e8f7c77068c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ee1ba87d071c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018810), UINT64_C(0x18f30017a09946ad), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0a6af7d250622534)
    },
    {   UINT64_C(0x0000000000018811), UINT64_C(0x053ac40f6867e68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1a3aff0f80a3a68c)
    },
    {   UINT64_C(0x0000000000006129), UINT64_C(0x1f430ace80b6872a), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4e84adc734)
    },
    {   UINT64_C(0x0000000000018813), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0fa2ff0cf474a4d8), UINT64_C(0x08b2fd0d6c5765b3)
    },
    {   UINT64_C(0x0000000000018814), UINT64_C(0x18b2fbcf009a669b), UINT64_C(0x0faaff0cf474a68c),
        UINT64_C(0x0ebafb8c8072a6aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018815), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x163300d5e48de68e)
    },
    {   UINT64_C(0x000000000000612b), UINT64_C(0x1f43104f08ac073d), UINT64_C(0x1f43148deca86740),
        UINT64_C(0x1f430c8e4cae4742), UINT64_C(0x1f43030ed8aa472d)
    },
    {   UINT64_C(0x000000000000882b), UINT64_C(0x0fa2f78f70784680), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000612c), UINT64_C(0x1f42fc4f04b0073c), UINT64_C(0x1f42fcced8a80731),
        UINT64_C(0x1f430f4ea8aa2748), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000577f), UINT64_C(0x0000000000000000), UINT64_C(0x1f43068ef0b24719),
        UINT64_C(0x1f43254e5ca2c72c), UINT64_C(0x1f42ff4e5cb3072c)
    },
    {   UINT64_C(0x000000000000606d), UINT64_C(0x1f42f64fa8ac86ef), UINT64_C(0x1f42f64e7caf672c),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74ef8b086f3)
    },
    {   UINT64_C(0x0000000000005780), UINT64_C(0x0000000000000000), UINT64_C(0x1f4329cef89ec70c),
        UINT64_C(0x1f430a8e7cb0e72f), UINT64_C(0x1f4304ce3cb06738)
    },
    {   UINT64_C(0x000000000000612e), UINT64_C(0x1f4313cef8b60735), UINT64_C(0x1f43124e5cb1c73e),
        UINT64_C(0x1f42fe0f24b0e73b), UINT64_C(0x1f430c0f08a72739)
    },
    {   UINT64_C(0x0000000000005781), UINT64_C(0x1f43058e9cb3e6f0), UINT64_C(0x1f430e4ee8aa66f6),
        UINT64_C(0x1f4301cf28b626f7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006071), UINT64_C(0x1f42f90ebcb7a723), UINT64_C(0x1f42f90ec0ad66d7),
        UINT64_C(0x1f42f74f98b1e731), UINT64_C(0x1f42f80e9caec719)
    },
    {   UINT64_C(0x000000000001881f), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x10eafc53d87b068c)
    },
    {   UINT64_C(0x0000000000006073), UINT64_C(0x1f42f75020b5a6fd), UINT64_C(0x1f42f34ea0b82740),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42f74ea0b0073d)
    },
    {   UINT64_C(0x0000000000005783), UINT64_C(0x1f43028e7cb3e746), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f4304cf28b5e6fb), UINT64_C(0x1f4306cee8ae8710)
    },
    {   UINT64_C(0x0000000000006131), UINT64_C(0x1f42fd8e9cb56748), UINT64_C(0x1f43068e6cb2c733),
        UINT64_C(0x1f42fc8e2ca68735), UINT64_C(0x1f4310ce80ab8739)
    },
    {   UINT64_C(0x0000000000005784), UINT64_C(0x1f43068f58b5a70b), UINT64_C(0x1f43000e4cb46740),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000004d1c), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f4306ce64af471b)
    },
    {   UINT64_C(0x0000000000005785), UINT64_C(0x1f43064f58b42702), UINT64_C(0x1f430a4f78aa6703),
        UINT64_C(0x1f4309cf58aea70b), UINT64_C(0x1f43064ebcb2e704)
    },
    {   UINT64_C(0x0000000000006079), UINT64_C(0x0000000000000000), UINT64_C(0x1f4306cf80b786e1),
        UINT64_C(0x1f43008fd0b026c8), UINT64_C(0x1f42ff8ec8b2a70c)
    },
    {   UINT64_C(0x0000000000005786), UINT64_C(0x1f43158e9cafc73e), UINT64_C(0x1f43048f38b72717),
        UINT64_C(0x1f43104e3cb44728), UINT64_C(0x1f43240e6c9ea73a)
    },
    {   UINT64_C(0x0000000000018828), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1162f0cfa082c67e)
    },
    {   UINT64_C(0x0000000000018829), UINT64_C(0x0fa2f10f807bc68c), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x24f2f20f78cd267e)
    },
    {   UINT64_C(0x0000000000006135), UINT64_C(0x1f43124eecabc72a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43120e48ad0730), UINT64_C(0x1f43008f08af872d)
    },
    {   UINT64_C(0x000000000001882b), UINT64_C(0x0fa2f78f70784680), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006136), UINT64_C(0x0000000000000000), UINT64_C(0x1f43044e40b04732),
        UINT64_C(0x1f43014e68b2073a), UINT64_C(0x1f42fbceacb2a739)
    },
    {   UINT64_C(0x0000000000005789), UINT64_C(0x1f43078e6ca7271f), UINT64_C(0x1f43138e6cac8715),
        UINT64_C(0x1f430b4e8caee71f), UINT64_C(0x1f430b8f30a9c711)
    },
    {   UINT64_C(0x0000000000006137), UINT64_C(0x0000000000000000), UINT64_C(0x1f430d8f40ad4731),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43118ea0aa872c)
    },
    {   UINT64_C(0x0000000000008837), UINT64_C(0x00025817cc4a8656), UINT64_C(0x0002394c2c5a66bd),
        UINT64_C(0x0002378cd05b46f4), UINT64_C(0x00021c0a4c698733)
    },
    {   UINT64_C(0x0000000000006083), UINT64_C(0x1f430a4efcb086dd), UINT64_C(0x1f42fc0edcb8c725),
        UINT64_C(0x1f42f7cedcb6a725), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008838), UINT64_C(0x0000000000000000), UINT64_C(0x00027cd83041a6e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006085), UINT64_C(0x1f42f58efcb726ed), UINT64_C(0x1f42f78f38b646f6),
        UINT64_C(0x1f42fdcf20ad270d), UINT64_C(0x1f43020f1cb1a721)
    },
    {   UINT64_C(0x0000000000008839), UINT64_C(0x0001fb8d347bc7e7), UINT64_C(0x0002828d3039867e),
        UINT64_C(0x00029a4d3031c67e), UINT64_C(0x00024b8cd85346bd)
    },
    {   UINT64_C(0x0000000000018834), UINT64_C(0x00027cd96c3c66e0), UINT64_C(0x00027cd96c3c66e0),
        UINT64_C(0x0002b18d3026867e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018835), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x00027cd8303c66e0), UINT64_C(0x00027cd8303c66e0)
    },
    {   UINT64_C(0x0000000000018836), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x00027a97cc42e656)
    },
    {   UINT64_C(0x0000000000018837), UINT64_C(0x00025817cc4a8656), UINT64_C(0x0002394c2c5a66bd),
        UINT64_C(0x0002378cd05b46f4), UINT64_C(0x00021c0a4c698733)
    },
    {   UINT64_C(0x0000000000018838), UINT64_C(0x0000000000000000), UINT64_C(0x00027cd83041a6e0),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018839), UINT64_C(0x0001fb8d347bc7e7), UINT64_C(0x0002828d3039867e),
        UINT64_C(0x00029a4d3031c67e), UINT64_C(0x00024b8cd85346bd)
    },
    {   UINT64_C(0x000000000001883a), UINT64_C(0x00027a8c3842e371), UINT64_C(0x0002761808452691),
        UINT64_C(0x00024990fc4d458b), UINT64_C(0x00027e0c3c56a6dc)
    },
    {   UINT64_C(0x000000000001883b), UINT64_C(0x00027ccbb03c6371), UINT64_C(0x00027ccbb03c6371),
        UINT64_C(0x00027ccbb03c63c0), UINT64_C(0x00027ccbb03c63c0)
    },
    {   UINT64_C(0x000000000001883c), UINT64_C(0x000294d7b433a71b), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008747), UINT64_C(0x123af34f688485b8), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000005792), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42ff8edcad26f7), UINT64_C(0x1f43044e5cb12726)
    },
    {   UINT64_C(0x0000000000018840), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x160af94f6891668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018841), UINT64_C(0x0000000000000000), UINT64_C(0x1a2afa0f68a1e68c),
        UINT64_C(0x0000000000000000), UINT64_C(0x12b2f50f6884a68c)
    },
    {   UINT64_C(0x0000000000018842), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1712f58c349345a1), UINT64_C(0x1f2afb4f64ba6697)
    },
    {   UINT64_C(0x0000000000018843), UINT64_C(0x19c2f8cc709f269f), UINT64_C(0x1672ff8fa08f867e),
        UINT64_C(0x120af9cf7882a693), UINT64_C(0x121af9cf7882e693)
    },
    {   UINT64_C(0x0000000000018844), UINT64_C(0x1c0af48f64ad868b), UINT64_C(0x17d2f94f1897669f),
        UINT64_C(0x1d92f64f64b2e697), UINT64_C(0x121afa0f7485e697)
    },
    {   UINT64_C(0x0000000000018845), UINT64_C(0x0000000000000000), UINT64_C(0x15d2ff8fa08d067e),
        UINT64_C(0x1fa2efcf68b7468c), UINT64_C(0x17c2f78f6898c68c)
    },
    {   UINT64_C(0x0000000000018846), UINT64_C(0x19d2fb0c3ca2660b), UINT64_C(0x1532ff8fa08d067e),
        UINT64_C(0x17eaf94e9c97e68c), UINT64_C(0x1d82f68f64ac0697)
    },
    {   UINT64_C(0x0000000000018847), UINT64_C(0x1b52fa8f48a626a8), UINT64_C(0x175aff8fa097867e),
        UINT64_C(0x1862ff4e0c9de683), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018848), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1ea2f593a0b6267e), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000018849), UINT64_C(0x16bafa8fa093267e), UINT64_C(0x0d42f6cc386f667e),
        UINT64_C(0x1742f54fa097e68c), UINT64_C(0x1742f54f6897e67e)
    },
    {   UINT64_C(0x000000000001884a), UINT64_C(0x16b2fa8c7093069f), UINT64_C(0x0b42f60fa06105b7),
        UINT64_C(0x0000000000000000), UINT64_C(0x1692f1cfa096e74f)
    },
    {   UINT64_C(0x000000000001884b), UINT64_C(0x1db2f68f64af4697), UINT64_C(0x166af9cf4092e69f),
        UINT64_C(0x1a5af6cf5ca3868c), UINT64_C(0x139af58f7483c5a1)
    },
    {   UINT64_C(0x000000000001884c), UINT64_C(0x1773070f649a4697), UINT64_C(0x1cbaf8cfa0afe690),
        UINT64_C(0x10baf14fa078467e), UINT64_C(0x2182f78f64c2868a)
    },
    {   UINT64_C(0x000000000001884d), UINT64_C(0x15caf58f749385a1), UINT64_C(0x1552f8cf6c8dc68d),
        UINT64_C(0x13fafc4f5486068d), UINT64_C(0x0fcaf80f5478668d)
    },
    {   UINT64_C(0x000000000001884e), UINT64_C(0x12bafb0f547c668d), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001884f), UINT64_C(0x150af44f708f25aa), UINT64_C(0x10e2ee0c388045b4),
        UINT64_C(0x0eaaff8fa085667e), UINT64_C(0x1672ff8fa08f867e)
    },
    {   UINT64_C(0x0000000000018850), UINT64_C(0x1cdafd4b94acc6e3), UINT64_C(0x15baff0f6892068c),
        UINT64_C(0x117af14fa07b267e), UINT64_C(0x117af14fa07b267e)
    },
    {   UINT64_C(0x0000000000018851), UINT64_C(0x13b2f34fa08aa67e), UINT64_C(0x164af24fa0966675),
        UINT64_C(0x122af80f748dc693), UINT64_C(0x101af80f747b25a1)
    },
    {   UINT64_C(0x0000000000018852), UINT64_C(0x092af4cfa069c675), UINT64_C(0x0aaad40c3c6fc675),
        UINT64_C(0x14c2f94fa089c69c), UINT64_C(0x1852f50f6c9a868c)
    },
    {   UINT64_C(0x000000000000579c), UINT64_C(0x1f43028ec4b48715), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43084ee8b34709), UINT64_C(0x1f43094ed8b1e716)
    },
    {   UINT64_C(0x0000000000018854), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x1772f70f6899268c)
    },
    {   UINT64_C(0x0000000000018855), UINT64_C(0x1772f18f6899e68c), UINT64_C(0x1772f74f6897e68c),
        UINT64_C(0x1772f18f6898868c), UINT64_C(0x1772f14f6898868c)
    },
    {   UINT64_C(0x0000000000018856), UINT64_C(0x1772f18f6898868c), UINT64_C(0x1772f74f6898268c),
        UINT64_C(0x1772f54f6498c68c), UINT64_C(0x1772f74f6898c68c)
    },
    {   UINT64_C(0x0000000000018857), UINT64_C(0x1772f14f6899668c), UINT64_C(0x1772f54f6899668c),
        UINT64_C(0x1772f50f6899668c), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008761), UINT64_C(0x10f2efcb247da693), UINT64_C(0x0932f2cb245ea693),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008762), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0c2afacb246a05b3), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008763), UINT64_C(0x0000000000000000), UINT64_C(0x1022f24b247a65aa),
        UINT64_C(0x0e2af4cb247285aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000884d), UINT64_C(0x15caf58f749385a1), UINT64_C(0x1552f8cf6c8dc68d),
        UINT64_C(0x13fafc4f5486068d), UINT64_C(0x0fcaf80f5478668d)
    },
    {   UINT64_C(0x000000000000614e), UINT64_C(0x1f43180f3cb12726), UINT64_C(0x1f4300cf34b1e729),
        UINT64_C(0x1f43148e70a90735), UINT64_C(0x1f43120f10ad6729)
    },
    {   UINT64_C(0x0000000000008766), UINT64_C(0x0dcafc56446ee6db), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000000614f), UINT64_C(0x1f43020ebca90723), UINT64_C(0x1f430d0f30b62731),
        UINT64_C(0x1f43108edcaee736), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008768), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0f02fe13fc7046dd)
    },
    {   UINT64_C(0x0000000000006150), UINT64_C(0x1f4305ceacb4473a), UINT64_C(0x1f430f8e64ae0739),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fece54b70742)
    },
    {   UINT64_C(0x00000000000060b4), UINT64_C(0x0000000000000000), UINT64_C(0x1f430c8f1cb6671a),
        UINT64_C(0x1f42ff8e64b3e740), UINT64_C(0x1f43034e6cb7a750)
    },
    {   UINT64_C(0x0000000000006151), UINT64_C(0x1f43180e98a78744), UINT64_C(0x1f43114e78af272e),
        UINT64_C(0x1f4301cec0b3a744), UINT64_C(0x1f43114e3caf673f)
    },
    {   UINT64_C(0x00000000000057a4), UINT64_C(0x1f42fa8f689fc726), UINT64_C(0x1f42fecf5cb06716),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42fc4eccad872f)
    },
    {   UINT64_C(0x0000000000018864), UINT64_C(0x1cf2ffd088a9a5a9), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000018865), UINT64_C(0x1cf2f79084ad85a9), UINT64_C(0x0ffafd0c2076e6c2),
        UINT64_C(0x1cf2ff8d30a9867e), UINT64_C(0x1cf2ff8d30a9867e)
    },
    {   UINT64_C(0x0000000000018866), UINT64_C(0x1cf2ff8d30a9867e), UINT64_C(0x1cf2ff8d30a9867e),
        UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x0000000000018867), UINT64_C(0x1cf2ff9138a98583), UINT64_C(0x1cf2ff9138a98583),
        UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x0ffafd0d3876e67e)
    },
    {   UINT64_C(0x0000000000018868), UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2ff9088a985a9), UINT64_C(0x1cf2ff9088a985a9)
    },
    {   UINT64_C(0x0000000000018869), UINT64_C(0x1cf2ff9084a9a5aa), UINT64_C(0x0ffafd0d3876e67e),
        UINT64_C(0x1cf2ff9084a9a5aa), UINT64_C(0x0ffafccd3876e67e)
    },
    {   UINT64_C(0x000000000001886a), UINT64_C(0x0ffafccd3876e67e), UINT64_C(0x1cf2ff9084a98611),
        UINT64_C(0x1cf2ff9084a98611), UINT64_C(0x1cf2ff8fa0a98611)
    },
    {   UINT64_C(0x000000000001886b), UINT64_C(0x1cf2ff8fa0a98611), UINT64_C(0x2582fa5088ce65a9),
        UINT64_C(0x1cf2f79088ad85aa), UINT64_C(0x0ffafd0d3876e67e)
    },
    {   UINT64_C(0x000000000001886c), UINT64_C(0x0e7af58d3073867e), UINT64_C(0x0e7af7cd3074a67e),
        UINT64_C(0x0e7af44d3872e67e), UINT64_C(0x0e7af7cd3874a67e)
    },
    {   UINT64_C(0x000000000001886d), UINT64_C(0x1cf2ff8d38a9867e), UINT64_C(0x1cf2ff8d30a9867e),
        UINT64_C(0x1cf2f90fa0acc5fe), UINT64_C(0x1cf2f90fa0acc5fe)
    },
    {   UINT64_C(0x000000000001886e), UINT64_C(0x1cf2fbcd30ab86c4), UINT64_C(0x1cf2fd0e30aac63e),
        UINT64_C(0x1e730b0dd0a9c696), UINT64_C(0x1e730b0dd0a9c696)
    },
    {   UINT64_C(0x000000000001886f), UINT64_C(0x1ddafb9310af25d6), UINT64_C(0x1ddafb8fd0af2506),
        UINT64_C(0x0ffb258d3075a67e), UINT64_C(0x0ffaff4d3062867e)
    },
    {   UINT64_C(0x0000000000018870), UINT64_C(0x1ddafb9310af25d6), UINT64_C(0x1ddafb8fd0af2506),
        UINT64_C(0x0ffb258d3075a67e), UINT64_C(0x0ffaff4d3062867e)
    },
    {   UINT64_C(0x0000000000018871), UINT64_C(0x1cf2ffce30a9863e), UINT64_C(0x182af5cd309b467e),
        UINT64_C(0x1cf2ffce30a9863e), UINT64_C(0x1cf2ff8e30a9863f)
    },
    {   UINT64_C(0x0000000000018872), UINT64_C(0x182af84d309a067e), UINT64_C(0x1cf2ff8e30a9863f),
        UINT64_C(0x182af84d309a067e), UINT64_C(0x1cf2ff8f1ca98603)
    },
    {   UINT64_C(0x0000000000018873), UINT64_C(0x1cf2ff8f1ca98603), UINT64_C(0x1cf2fd0eecaac60f),
        UINT64_C(0x1cf2f30f78afc5ed), UINT64_C(0x1cf2fd0eecaac60f)
    },
    {   UINT64_C(0x0000000000018874), UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e),
        UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e)
    },
    {   UINT64_C(0x0000000000018875), UINT64_C(0x1cf2f30f78afc5ed), UINT64_C(0x156af94c208ea6c2),
        UINT64_C(0x1cf2fbcd30abc67e), UINT64_C(0x1cf2fbcd30abc67e)
    },
    {   UINT64_C(0x0000000000018876), UINT64_C(0x1cf2fbcd30abc67e), UINT64_C(0x1cf2fbcd30abc67e),
        UINT64_C(0x1cf2f98d74a9066c), UINT64_C(0x1cf3008d78ac866d)
    },
    {   UINT64_C(0x0000000000018877), UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x1cf2fd1088aac5a9),
        UINT64_C(0x0ffafd0d3076e67e), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000018878), UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x0ffafd0d3076e67e),
        UINT64_C(0x1cf2fd1088aac5a9), UINT64_C(0x0ffafd0d3076e67e)
    },
    {   UINT64_C(0x0000000000018879), UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa),
        UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e)
    },
    {   UINT64_C(0x000000000001887a), UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e),
        UINT64_C(0x156af94cc08ea69a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057b0), UINT64_C(0x0000000000000000), UINT64_C(0x1f42f88e78a68736),
        UINT64_C(0x1f42fa8f0ca9272d), UINT64_C(0x1f42f70f8cb70740)
    },
    {   UINT64_C(0x000000000000615e), UINT64_C(0x1f4303ced0ade726), UINT64_C(0x1f43184ef8ad4736),
        UINT64_C(0x1f430c8eacaf4733), UINT64_C(0x1f4304cf1cace720)
    },
    {   UINT64_C(0x000000000001887d), UINT64_C(0x1cf2ff9084a985a8), UINT64_C(0x182af5cd309b467e),
        UINT64_C(0x1cf2fd0c00aac6cb), UINT64_C(0x1cf2fd1088aac5aa)
    },
    {   UINT64_C(0x000000000000615f), UINT64_C(0x1f4300ceacb08728), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f43130eb4aa2730), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x000000000001887f), UINT64_C(0x1cf2f79088ad85aa), UINT64_C(0x1cf2fd1084aac5a9),
        UINT64_C(0x1cf2fd1084aac5a9), UINT64_C(0x1cf2f31084afc5a9)
    },
    {   UINT64_C(0x0000000000018880), UINT64_C(0x1182ee8fa084467e), UINT64_C(0x0e7afccf647106e0),
        UINT64_C(0x0ebaf80f74796684), UINT64_C(0x1183004fa07b667e)
    },
    {   UINT64_C(0x0000000000018881), UINT64_C(0x1182ffcdb87b66ef), UINT64_C(0x17d2fa8e6497862f),
        UINT64_C(0x16dafdcfa091a68a), UINT64_C(0x16dafdcf7091e67e)
    },
    {   UINT64_C(0x0000000000018882), UINT64_C(0x156afd0f348ca5fb), UINT64_C(0x156afd0d2c8ca67e),
        UINT64_C(0x0f32fe10547345b3), UINT64_C(0x156afd0f348ca5fb)
    },
    {   UINT64_C(0x0000000000018883), UINT64_C(0x156afd0d2c8ca67e), UINT64_C(0x0f32fe10547345b3),
        UINT64_C(0x110afd0fa07b2668), UINT64_C(0x1f42fb0b94b506e3)
    },
    {   UINT64_C(0x0000000000018884), UINT64_C(0x1f42fb0b94b506e3), UINT64_C(0x1c92fc8b94a986e3),
        UINT64_C(0x156afe13108c2506), UINT64_C(0x156afa0e448e25de)
    },
    {   UINT64_C(0x0000000000018885), UINT64_C(0x156afa0efc8e06cd), UINT64_C(0x105af98c287a46ae),
        UINT64_C(0x0d62f4511470e583), UINT64_C(0x105afed02477a5bf)
    },
    {   UINT64_C(0x0000000000018886), UINT64_C(0x0af2f8117465456b), UINT64_C(0x0af2f8117465456b),
        UINT64_C(0x1d030a0b94b6e7b5), UINT64_C(0x1d030a0b94b6e7b5)
    },
    {   UINT64_C(0x0000000000018887), UINT64_C(0x1d030a0b94b6e7b5), UINT64_C(0x156af84fa08ee596),
        UINT64_C(0x1cf2ff8fa0a94596), UINT64_C(0x156afa8fa08dc630)
    },
    {   UINT64_C(0x0000000000018888), UINT64_C(0x156af3cfa08ee60b), UINT64_C(0x156af38e808ec60b),
        UINT64_C(0x156afc0f749025ef), UINT64_C(0x0853070cac53469a)
    },
    {   UINT64_C(0x0000000000018889), UINT64_C(0x0ca2f3ccac6e269a), UINT64_C(0x105b0e4cac6fc69a),
        UINT64_C(0x130af3ccac87c69a), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x000000000001888a), UINT64_C(0x1362f5cf2c882600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x0e5af60aa08e6720)
    },
    {   UINT64_C(0x000000000001888b), UINT64_C(0x15eaf60aa0aca720), UINT64_C(0x1d7af60aa0cae720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x172af60aa0aca720)
    },
    {   UINT64_C(0x000000000001888c), UINT64_C(0x1eb2f60aa0cac720), UINT64_C(0x0f9af60aa08e6720),
        UINT64_C(0x0f9af60aa08e6720), UINT64_C(0x0f9af60aa08e6720)
    },
    {   UINT64_C(0x000000000001888d), UINT64_C(0x1362f78f608745f1), UINT64_C(0x1362f78f608745f1),
        UINT64_C(0x0ffb1e0f6c6665f1), UINT64_C(0x156b028f6c89c5f1)
    },
    {   UINT64_C(0x000000000001888e), UINT64_C(0x156afa13108e25e7), UINT64_C(0x156afa0f8c8e25e7),
        UINT64_C(0x156afa0f8c8e25e7), UINT64_C(0x156afa0f6c8e25f1)
    },
    {   UINT64_C(0x000000000001888f), UINT64_C(0x156afa11f08e2552), UINT64_C(0x156afa11f08e2552),
        UINT64_C(0x156afa11088e2585), UINT64_C(0x156afa503c8e25bb)
    },
    {   UINT64_C(0x0000000000018890), UINT64_C(0x08f2f6cef85da60b), UINT64_C(0x156afa11008e2590),
        UINT64_C(0x156afa107c8e25a5), UINT64_C(0x156afa107c8e25a5)
    },
    {   UINT64_C(0x0000000000018891), UINT64_C(0x156afa0f148e25ef), UINT64_C(0x156afa0f348e25fc),
        UINT64_C(0x156af9cdf48e2644), UINT64_C(0x156afa0c788e266f)
    },
    {   UINT64_C(0x0000000000018892), UINT64_C(0x156afa10048e25c3), UINT64_C(0x156afa0edc8e260d),
        UINT64_C(0x156afa0eec8e2610), UINT64_C(0x156afa0f388e45fc)
    },
    {   UINT64_C(0x0000000000018893), UINT64_C(0x156afa0f348e25fc), UINT64_C(0x156afa0fc08e25da),
        UINT64_C(0x156afa102c8e25bf), UINT64_C(0x156afa11808e25bf)
    },
    {   UINT64_C(0x0000000000018894), UINT64_C(0x156afa11808e264b), UINT64_C(0x156afa0df88e264b),
        UINT64_C(0x156afa0dfc8e264b), UINT64_C(0x156afa0df88e264b)
    },
    {   UINT64_C(0x0000000000018895), UINT64_C(0x19c2fa11049f8589), UINT64_C(0x19c2fa11049f8589),
        UINT64_C(0x156afa11088e2588), UINT64_C(0x156afa11808e26d8)
    },
    {   UINT64_C(0x0000000000018896), UINT64_C(0x156afa11808e26c1), UINT64_C(0x156afa11808e273d),
        UINT64_C(0x156afa11808e273d), UINT64_C(0x156afa11808e26dc)
    },
    {   UINT64_C(0x0000000000018897), UINT64_C(0x156afa11808e273d), UINT64_C(0x156aefd18093471f),
        UINT64_C(0x156afa11808e26a7), UINT64_C(0x156afa11808e2747)
    },
    {   UINT64_C(0x0000000000018898), UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afa10108e25c6),
        UINT64_C(0x156afa0d308e267e), UINT64_C(0x156afa0f088e2608)
    },
    {   UINT64_C(0x0000000000018899), UINT64_C(0x156afe0e048d2649), UINT64_C(0x156afe0e048d2649),
        UINT64_C(0x156afc4c548c46b6), UINT64_C(0x156afc4c548c46b6)
    },
    {   UINT64_C(0x000000000001889a), UINT64_C(0x156afc0b908c26d2), UINT64_C(0x156afc0b908c26d2),
        UINT64_C(0x1d2af44f38b005fc), UINT64_C(0x1d2af44f38b005fc)
    },
    {   UINT64_C(0x000000000000886d), UINT64_C(0x1cf2ff8d38a9867e), UINT64_C(0x1cf2ff8d30a9867e),
        UINT64_C(0x1cf2f90fa0acc5fe), UINT64_C(0x1cf2f90fa0acc5fe)
    },
    {   UINT64_C(0x000000000001889c), UINT64_C(0x156afc0c0c8c26c2), UINT64_C(0x156afc0c0c8c46c2),
        UINT64_C(0x156afa0d108e2680), UINT64_C(0x156afa0d108e2680)
    },
    {   UINT64_C(0x000000000001889d), UINT64_C(0x156afa0c108e26c3), UINT64_C(0x156afa0c0c8e26c2),
        UINT64_C(0x156afc0c708c26a9), UINT64_C(0x156afc0c708c26a9)
    },
    {   UINT64_C(0x000000000000616f), UINT64_C(0x0000000000000000), UINT64_C(0x1f430dcea4b72734),
        UINT64_C(0x1f43130e78aac74d), UINT64_C(0x1f43068f28b68732)
    },
    {   UINT64_C(0x000000000001889f), UINT64_C(0x156afe0dc08c265c), UINT64_C(0x156afe0dc48c265d),
        UINT64_C(0x156afa0d108e2680), UINT64_C(0x156afa0d108e2680)
    },
    {   UINT64_C(0x00000000000188a0), UINT64_C(0x156afe0d308c267e), UINT64_C(0x156afe0d308c267e),
        UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0f3c8c25fb)
    },
    {   UINT64_C(0x00000000000188a1), UINT64_C(0x156afe4d308c667e), UINT64_C(0x156afe4d308c667e),
        UINT64_C(0x156afe0e048c2647), UINT64_C(0x156afe0e048c2647)
    },
    {   UINT64_C(0x00000000000188a2), UINT64_C(0x156afe0c0c8c26c2), UINT64_C(0x156afe0c0c8c26c2),
        UINT64_C(0x156afe0c408c265b), UINT64_C(0x156afe0c408c265b)
    },
    {   UINT64_C(0x00000000000188a3), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x156afe0f3c8c25fb)
    },
    {   UINT64_C(0x00000000000188a4), UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0e048c2647),
        UINT64_C(0x156afe0e048c2647), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x00000000000188a5), UINT64_C(0x1362fa0f24860600), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657)
    },
    {   UINT64_C(0x0000000000006173), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cf50afc745),
        UINT64_C(0x1f430e0f58ad873c), UINT64_C(0x1f4301cef0b02739)
    },
    {   UINT64_C(0x00000000000188a7), UINT64_C(0x1a52fa8dc4a18657), UINT64_C(0x1a52fa8dc4a18657),
        UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x00000000000188a8), UINT64_C(0x1c72f94d28aaa67e), UINT64_C(0x1c72f94d2caaa67e),
        UINT64_C(0x156afe0fa08c267e), UINT64_C(0x156afe0fa08c267e)
    },
    {   UINT64_C(0x00000000000188a9), UINT64_C(0x156afa0fa08e267e), UINT64_C(0x156afa0fa08e267e),
        UINT64_C(0x0f8afe0fa074a67e), UINT64_C(0x0f92fe0fa074c67e)
    },
    {   UINT64_C(0x00000000000188aa), UINT64_C(0x156afe0fa08c267e), UINT64_C(0x1ae2fc4fa0a4467e),
        UINT64_C(0x1ae2f94fa0a4667e), UINT64_C(0x1ae2fc4fa0a4467e)
    },
    {   UINT64_C(0x00000000000188ab), UINT64_C(0x1892f04fa098e67e), UINT64_C(0x1892f04fa098e67e),
        UINT64_C(0x1e42f04fa0b1c67e), UINT64_C(0x1e42f04fa0b1c67e)
    },
    {   UINT64_C(0x00000000000188ac), UINT64_C(0x156afe0eec8c260f), UINT64_C(0x156afe0eec8c260f),
        UINT64_C(0x156af40f3c8d65fb), UINT64_C(0x156afb8f3c9125fb)
    },
    {   UINT64_C(0x00000000000188ad), UINT64_C(0x156af40e048d6647), UINT64_C(0x156afb8e04912647),
        UINT64_C(0x23cafa913cc7657b), UINT64_C(0x23cafa913cc7657b)
    },
    {   UINT64_C(0x00000000000188ae), UINT64_C(0x1a8afa913ca2657b), UINT64_C(0x156afa0efc8e060b),
        UINT64_C(0x0f03008c507145aa), UINT64_C(0x1362f60d74884600)
    },
    {   UINT64_C(0x00000000000188af), UINT64_C(0x1362f60f2c88466e), UINT64_C(0x1362f60f2c88466e),
        UINT64_C(0x156afa8fa08dc630), UINT64_C(0x1c7af94d28aaa67e)
    },
    {   UINT64_C(0x00000000000188b0), UINT64_C(0x1ce2fb8b94ab46e3), UINT64_C(0x1ce2fb8b94ab46e3),
        UINT64_C(0x1ce3058ba8a646f2), UINT64_C(0x1ce3058b80a646e8)
    },
    {   UINT64_C(0x00000000000188b1), UINT64_C(0x105af48fe07ca5d0), UINT64_C(0x08f302d2a457e521),
        UINT64_C(0x15e2f6cf6c91a63d), UINT64_C(0x156afacf088dc609)
    },
    {   UINT64_C(0x0000000000006179), UINT64_C(0x1f42ff4e80b6c73f), UINT64_C(0x1f43078e5cb30740),
        UINT64_C(0x1f43024e94a9a745), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008879), UINT64_C(0x1cf2fd1088aac5aa), UINT64_C(0x1cf2fd1088aac5aa),
        UINT64_C(0x1cf2fd0eecaac60f), UINT64_C(0x156af94d308ea67e)
    },
    {   UINT64_C(0x00000000000188b4), UINT64_C(0x156afe0f3c8c25fb), UINT64_C(0x156afe0f3c8c25fb),
        UINT64_C(0x1362fa0f24860600), UINT64_C(0x1362fa0f24860600)
    },
    {   UINT64_C(0x00000000000188b5), UINT64_C(0x1362fa0f2486065f), UINT64_C(0x156afa0cac8e269a),
        UINT64_C(0x156afc0f408c25fe), UINT64_C(0x156afc0f408c25fe)
    },
    {   UINT64_C(0x00000000000188b6), UINT64_C(0x27d2f94f40d825fe), UINT64_C(0x27d2f94f40d825fe),
        UINT64_C(0x156afc0a908c2726), UINT64_C(0x156afc0a908c2726)
    },
    {   UINT64_C(0x00000000000188b7), UINT64_C(0x156afe0e048c2647), UINT64_C(0x156afe0e048c2647),
        UINT64_C(0x156afe0dbc8c265b), UINT64_C(0x156afe0dbc8c265b)
    },
    {   UINT64_C(0x00000000000188b8), UINT64_C(0x156afe0c0c8c26c2), UINT64_C(0x156afe0c0c8c26c2),
        UINT64_C(0x156afe4c0c8c46c2), UINT64_C(0x156afe4c0c8c46c2)
    },
    {   UINT64_C(0x00000000000188b9), UINT64_C(0x156afe0c408c265b), UINT64_C(0x156afe0c408c265b),
        UINT64_C(0x156afa0b448e2685), UINT64_C(0x156afa0b448e2685)
    },
    {   UINT64_C(0x000000000000617d), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42f88e98b1a739), UINT64_C(0x1f42ffcf10b54725)
    },
    {   UINT64_C(0x00000000000188bb), UINT64_C(0x13daf40c0c8726c2), UINT64_C(0x13dafb8c0c8ae6c2),
        UINT64_C(0x0ffb1e0e00666646), UINT64_C(0x1cf3091294a4c524)
    },
    {   UINT64_C(0x00000000000188bc), UINT64_C(0x1cf31e8f589a05f0), UINT64_C(0x1cf31e8f589a05f0),
        UINT64_C(0x19bafbcf349de5fb), UINT64_C(0x156afd0f348ca5fb)
    },
    {   UINT64_C(0x00000000000188bd), UINT64_C(0x0f32fd90547305b3), UINT64_C(0x156afd0f348ca6b4),
        UINT64_C(0x156afd0f348ca695), UINT64_C(0x0f32fd905473064d)
    },
    {   UINT64_C(0x000000000000617f), UINT64_C(0x0000000000000000), UINT64_C(0x1f4307cef0b1672f),
        UINT64_C(0x1f4303cf30a42739), UINT64_C(0x1f43050ea8a88735)
    },
    {   UINT64_C(0x00000000000188bf), UINT64_C(0x0f32fd90547305b3), UINT64_C(0x156afd4f348cc695),
        UINT64_C(0x0f32fed05473a64d), UINT64_C(0x1c72f94d28aaa67e)
    },
    {   UINT64_C(0x00000000000188c0), UINT64_C(0x12f2f44fd88725cf), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf2fbcfa0ab66ee), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188c1), UINT64_C(0x0000000000000000), UINT64_C(0x1362fa0fa0860629),
        UINT64_C(0x1362fa0fa08606c0), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188c2), UINT64_C(0x0eab1d0c4c7666b1), UINT64_C(0x0eaaf4cc4c6246b1),
        UINT64_C(0x0eab1d0c4c7666b1), UINT64_C(0x0eaaf4cc4c6246b1)
    },
    {   UINT64_C(0x00000000000188c3), UINT64_C(0x0e7afb0c4472048e), UINT64_C(0x0e7afb0c4472048e),
        UINT64_C(0x0e7afb175c720754), UINT64_C(0x0e7afb175c720754)
    },
    {   UINT64_C(0x00000000000188c4), UINT64_C(0x12c2fa116c838571), UINT64_C(0x105b00d0a476a59f),
        UINT64_C(0x17d2fa8fa0978533), UINT64_C(0x17d2fa8fa0978533)
    },
    {   UINT64_C(0x00000000000188c5), UINT64_C(0x0000000000000000), UINT64_C(0x1a7af48caca2062e),
        UINT64_C(0x2262fb4bc8c166d4), UINT64_C(0x2262fb4bccc166d5)
    },
    {   UINT64_C(0x00000000000188c6), UINT64_C(0x1cf2fbcd30ab667e), UINT64_C(0x12c2fa116c838571),
        UINT64_C(0x156afe0e8c8c4687), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188c7), UINT64_C(0x0e7afad40471e67e), UINT64_C(0x0e7afad40471e67e),
        UINT64_C(0x0e7afacc3c71e48c), UINT64_C(0x0e7afb0c3c72048c)
    },
    {   UINT64_C(0x00000000000188c8), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1fdafb91ecb72550), UINT64_C(0x1fdafb91ecb72550)
    },
    {   UINT64_C(0x00000000000057d7), UINT64_C(0x1f42ff8e8cb2e735), UINT64_C(0x1f43024e8cb46735),
        UINT64_C(0x1f42ff8e14b2e737), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188ca), UINT64_C(0x0000000000000000), UINT64_C(0x0c83014c4c6866b1),
        UINT64_C(0x0c82fe4c4c66e6b1), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188cb), UINT64_C(0x1cf302ccb8a7e69c), UINT64_C(0x1ed2fbce40b2e638),
        UINT64_C(0x1382e98d708645aa), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188cc), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1bcb020ec0a3a61a), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000188cd), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x1cf3034d30a7a6d7), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000006187), UINT64_C(0x1f430c8eacab673a), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fe4f7cafc72d), UINT64_C(0x1f43160e9cb18734)
    },
    {   UINT64_C(0x00000000000188cf), UINT64_C(0x0000000000000000), UINT64_C(0x1c73030cd4a5c693),
        UINT64_C(0x0000000000000000), UINT64_C(0x155afd0ce08c66a7)
    },
    {   UINT64_C(0x00000000000188d0), UINT64_C(0x155afd0ce08c66a7), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057db), UINT64_C(0x1f42f88ea4acc70f), UINT64_C(0x1f430b8e58ab8725),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f43078f3cac4705)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x0000000000008889), UINT64_C(0x0ca2f3ccac6e269a), UINT64_C(0x105b0e4cac6fc69a),
        UINT64_C(0x130af3ccac87c69a), UINT64_C(0x1362f5cf2c882600)
    },
    {   UINT64_C(0x00000000000188d4), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x18d2fbcce09ae6d7)
    },
    {   UINT64_C(0x00000000000057dd), UINT64_C(0x1f43024eb8ae46fe), UINT64_C(0x0000000000000000),
        UINT64_C(0x1f42fd4f44b1e6fa), UINT64_C(0x1f42f94fb4b1070e)
    },
    {   UINT64_C(0x000000000000618b), UINT64_C(0x1f43048e40b3873a), UINT64_C(0x1f43150e5ca64746),
        UINT64_C(0x1f43084e1ca6a736), UINT64_C(0x1f430bce54ac872f)
    },
    {   UINT64_C(0x00000000000057de), UINT64_C(0x1f42fd4f84b646fb), UINT64_C(0x1f42f74fa4b82708),
        UINT64_C(0x0000000000000000), UINT64_C(0x1f42ff0fe0ad8706)
    },
    {   UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000),
        UINT64_C(0x0000000000000000), UINT64_C(0x0000000000000000)
    },
    {   UINT64_C(0x00000000000057df), UINT64_C(0x1f42f94f38b5a715), UINT64_C(0x1f42fb4f44b5070c),
        UINT64_C(0x1f42fc4f34b72708), UINT64_C(0x1f42f94e6cb2473f)
    }
};

#define CHAR_METRICS_MODULUS 4955
#define CHAR_METRICS_OFFSET 5000

const uint32_t topcentre[153] =
{   UINT32_C(0xe10020df),
    UINT32_C(0x00000000),
    UINT32_C(0xe54020e1),
    UINT32_C(0xdfa00300),
    UINT32_C(0xdfa00301),
    UINT32_C(0xe3400302),
    UINT32_C(0xe3400303),
    UINT32_C(0xe3400304),
    UINT32_C(0xe3400305),
    UINT32_C(0xe3000306),
    UINT32_C(0xe54020e9),
    UINT32_C(0xde4020ea),
    UINT32_C(0xe5800309),
    UINT32_C(0xe340030a),
    UINT32_C(0xe580030b),
    UINT32_C(0xe340030c),
    UINT32_C(0xe2a020ef),
    UINT32_C(0xe44020f0),
    UINT32_C(0xe580030f),
    UINT32_C(0xe3000310),
    UINT32_C(0xe3000311),
    UINT32_C(0xe3400312),
    UINT32_C(0xe3400313),
    UINT32_C(0xe3400314),
    UINT32_C(0x00000000),
    UINT32_C(0xe2200316),
    UINT32_C(0xdfa00317),
    UINT32_C(0xda000318),
    UINT32_C(0xea400319),
    UINT32_C(0xe340031a),
    UINT32_C(0x00000000),
    UINT32_C(0xdb00031c),
    UINT32_C(0xe340031d),
    UINT32_C(0xe340031e),
    UINT32_C(0xe340031f),
    UINT32_C(0xe3400320),
    UINT32_C(0xf1600321),
    UINT32_C(0x00000000),
    UINT32_C(0xe3400323),
    UINT32_C(0xe3400324),
    UINT32_C(0xe3400325),
    UINT32_C(0xe3400326),
    UINT32_C(0xe3400327),
    UINT32_C(0xe3400328),
    UINT32_C(0xe3400329),
    UINT32_C(0xe360032a),
    UINT32_C(0xe380032b),
    UINT32_C(0xe340032c),
    UINT32_C(0xe340032d),
    UINT32_C(0xe340032e),
    UINT32_C(0xe340032f),
    UINT32_C(0xe3400330),
    UINT32_C(0xe3400331),
    UINT32_C(0xe3400332),
    UINT32_C(0xe3400333),
    UINT32_C(0xe2800334),
    UINT32_C(0xe3200335),
    UINT32_C(0xe3400336),
    UINT32_C(0xe5c00337),
    UINT32_C(0xea400338),
    UINT32_C(0xe5000339),
    UINT32_C(0xe360033a),
    UINT32_C(0xe340033b),
    UINT32_C(0xe340033c),
    UINT32_C(0xe340033d),
    UINT32_C(0xe340033e),
    UINT32_C(0xe340033f),
    UINT32_C(0x29400043),
    UINT32_C(0x2bc00044),
    UINT32_C(0x26000045),
    UINT32_C(0x22c00046),
    UINT32_C(0x2e400047),
    UINT32_C(0x2d000048),
    UINT32_C(0xe5e00346),
    UINT32_C(0x1a80004a),
    UINT32_C(0x2f40004b),
    UINT32_C(0x1280004c),
    UINT32_C(0x36c0004d),
    UINT32_C(0x2d00004e),
    UINT32_C(0xe460034c),
    UINT32_C(0x22e00050),
    UINT32_C(0x2e000051),
    UINT32_C(0x2a400052),
    UINT32_C(0x21600053),
    UINT32_C(0x26200054),
    UINT32_C(0x2d000055),
    UINT32_C(0x2c800056),
    UINT32_C(0x3a800057),
    UINT32_C(0x2ca00058),
    UINT32_C(0x2d400059),
    UINT32_C(0x2600005a),
    UINT32_C(0x00000063),
    UINT32_C(0xe4400359),
    UINT32_C(0x00000069),
    UINT32_C(0x2941d44e),
    UINT32_C(0x0000006b),
    UINT32_C(0x000020e8),
    UINT32_C(0x1e000061),
    UINT32_C(0x1d800062),
    UINT32_C(0x1b400063),
    UINT32_C(0x20600064),
    UINT32_C(0x1c000065),
    UINT32_C(0x19400066),
    UINT32_C(0x1f200067),
    UINT32_C(0x1f000068),
    UINT32_C(0x10000069),
    UINT32_C(0x1200006a),
    UINT32_C(0x2000006b),
    UINT32_C(0x1140006c),
    UINT32_C(0x3180006d),
    UINT32_C(0x1f40006e),
    UINT32_C(0x1f40006f),
    UINT32_C(0x1dc00070),
    UINT32_C(0x20000071),
    UINT32_C(0x15400072),
    UINT32_C(0x19000073),
    UINT32_C(0x12400074),
    UINT32_C(0x1e800075),
    UINT32_C(0x1f000076),
    UINT32_C(0x2cc00077),
    UINT32_C(0x1f000078),
    UINT32_C(0x1e800079),
    UINT32_C(0x1bc0007a),
    UINT32_C(0x000020ec),
    UINT32_C(0xe20020e4),
    UINT32_C(0xe2a020e5),
    UINT32_C(0xe4c020e6),
    UINT32_C(0xe1c020e7),
    UINT32_C(0xe54020e8),
    UINT32_C(0x2d200041),
    UINT32_C(0x26200042),
    UINT32_C(0xf00020eb),
    UINT32_C(0xe2a020ec),
    UINT32_C(0xe2a020ed),
    UINT32_C(0xe2a020ee),
    UINT32_C(0xe3400307),
    UINT32_C(0xe3400308),
    UINT32_C(0x14c00049),
    UINT32_C(0xe2a020d0),
    UINT32_C(0xe2a020d1),
    UINT32_C(0xe4e020d2),
    UINT32_C(0xe440030d),
    UINT32_C(0xe340030e),
    UINT32_C(0x2d20004f),
    UINT32_C(0xe2a020d6),
    UINT32_C(0xe2a020d7),
    UINT32_C(0x0000004a),
    UINT32_C(0x0000004b),
    UINT32_C(0x0000004e),
    UINT32_C(0xe54020db),
    UINT32_C(0xe08020dc),
    UINT32_C(0xe0c020dd),
    UINT32_C(0xe0c020de)
};

#define TOPCENTRE_MODULUS 88
#define TOPCENTRE_OFFSET 64

const uint32_t variant_table[104][6] =
{   {0x00002a0a, 0x00002a0a, 0x0010824b, 0x00000000, 0x00000000, 0x00000000},
    {0x002023dc, 0x0010822d, 0x00108274, 0x001082ad, 0x001082e5, 0x0010830e},
    {0x00002233, 0x00002233, 0x0010831c, 0x00000000, 0x00000000, 0x00000000},
    {0x002023dd, 0x0010822e, 0x00108275, 0x001082ae, 0x001082e6, 0x0010830f},
    {0x00002a0c, 0x00002a0c, 0x0010831e, 0x00000000, 0x00000000, 0x00000000},
    {0x002023de, 0x0010822f, 0x00108276, 0x001082af, 0x001082e7, 0x00108310},
    {0x00002a0d, 0x00002a0d, 0x0010831f, 0x00000000, 0x00000000, 0x00000000},
    {0x002023df, 0x00108230, 0x00108277, 0x001082b0, 0x001082e8, 0x00108311},
    {0x00002a0e, 0x00002a0e, 0x00108320, 0x00000000, 0x00000000, 0x00000000},
    {0x002023e0, 0x00108231, 0x00108278, 0x001082b1, 0x001082e9, 0x00108312},
    {0x00002772, 0x00002772, 0x00108233, 0x0010827a, 0x001082b3, 0x001082eb},
    {0x0000222b, 0x0000222b, 0x00108314, 0x00000000, 0x00000000, 0x00000000},
    {0x0000222c, 0x0000222c, 0x00108315, 0x00000000, 0x00000000, 0x00000000},
    {0x0000222d, 0x0000222d, 0x00108316, 0x00000000, 0x00000000, 0x00000000},
    {0x0000222e, 0x0000222e, 0x00108317, 0x00000000, 0x00000000, 0x00000000},
    {0x0000222f, 0x0000222f, 0x00108318, 0x00000000, 0x00000000, 0x00000000},
    {0x00002230, 0x00002230, 0x00108319, 0x00000000, 0x00000000, 0x00000000},
    {0x00002231, 0x00002231, 0x0010831a, 0x00000000, 0x00000000, 0x00000000},
    {0x00002309, 0x00002309, 0x0010820b, 0x0010826f, 0x001082a8, 0x001082e0},
    {0x00002983, 0x00002983, 0x0010823b, 0x00108282, 0x001082bb, 0x001082f3},
    {0x00002984, 0x00002984, 0x0010823c, 0x00108283, 0x001082bc, 0x001082f4},
    {0x00002985, 0x00002985, 0x0010823d, 0x00108284, 0x001082bd, 0x001082f5},
    {0x00002986, 0x00002986, 0x0010823e, 0x00108285, 0x001082be, 0x001082f6},
    {0x002002f7, 0x001081ef, 0x0010825b, 0x00108294, 0x001082cc, 0x001082fc},
    {0x00002308, 0x00002308, 0x0010820a, 0x0010826e, 0x001082a7, 0x001082df},
    {0x000027e9, 0x000027e9, 0x00108238, 0x0010827f, 0x001082b8, 0x001082f0},
    {0x0000230a, 0x0000230a, 0x0010820c, 0x00108270, 0x001082a9, 0x001082e1},
    {0x0000230b, 0x0000230b, 0x0010820d, 0x00108271, 0x001082aa, 0x001082e2},
    {0x00002a18, 0x00002a18, 0x0010832a, 0x00000000, 0x00000000, 0x00000000},
    {0x00000028, 0x00000028, 0x001081e2, 0x0010824e, 0x00108287, 0x001082bf},
    {0x0020203e, 0x001081ed, 0x00108259, 0x00108292, 0x001082ca, 0x001082fa},
    {0x00000029, 0x00000029, 0x001081e3, 0x0010824f, 0x00108288, 0x001082c0},
    {0x000029f8, 0x000029f8, 0x0010823f, 0x00000000, 0x00000000, 0x00000000},
    {0x000029f9, 0x000029f9, 0x00108240, 0x00000000, 0x00000000, 0x00000000},
    {0x00200302, 0x001081f0, 0x0010825c, 0x00108295, 0x001082cd, 0x001082fd},
    {0x00200303, 0x001081f1, 0x0010825d, 0x00108296, 0x001082ce, 0x001082fe},
    {0x00002a1c, 0x00002a1c, 0x0010832e, 0x00000000, 0x00000000, 0x00000000},
    {0x00200305, 0x001081f2, 0x0010825e, 0x00108297, 0x001082cf, 0x001082ff},
    {0x002020d0, 0x001081f7, 0x00108263, 0x0010829c, 0x001082d4, 0x00108304},
    {0x00002a04, 0x00002a04, 0x00108245, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a00, 0x00002a00, 0x00108241, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a01, 0x00002a01, 0x00108242, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a02, 0x00002a02, 0x00108243, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a03, 0x00002a03, 0x00108244, 0x00000000, 0x00000000, 0x00000000},
    {0x0020030c, 0x001081f3, 0x0010825f, 0x00108298, 0x001082d0, 0x00108300},
    {0x00002a05, 0x00002a05, 0x00108246, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a06, 0x00002a06, 0x00108247, 0x00000000, 0x00000000, 0x00000000},
    {0x0000002f, 0x0000002f, 0x001081e4, 0x00108250, 0x00108289, 0x001082c1},
    {0x00002a08, 0x00002a08, 0x00108249, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a09, 0x00002a09, 0x0010824a, 0x00000000, 0x00000000, 0x00000000},
    {0x002020d6, 0x001081f9, 0x00108265, 0x0010829e, 0x001082d6, 0x00108306},
    {0x00002a0b, 0x00002a0b, 0x0010831d, 0x00000000, 0x00000000, 0x00000000},
    {0x002020d7, 0x001081fa, 0x00108266, 0x0010829f, 0x001082d7, 0x00108307},
    {0x00002211, 0x00002211, 0x00108202, 0x00000000, 0x00000000, 0x00000000},
    {0x00002773, 0x00002773, 0x00108234, 0x0010827b, 0x001082b4, 0x001082ec},
    {0x00002a0f, 0x00002a0f, 0x00108321, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a10, 0x00002a10, 0x00108322, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a11, 0x00002a11, 0x00108323, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a12, 0x00002a12, 0x00108324, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a13, 0x00002a13, 0x00108325, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a14, 0x00002a14, 0x00108326, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a15, 0x00002a15, 0x00108327, 0x00000000, 0x00000000, 0x00000000},
    {0x0000221a, 0x0000221a, 0x00108203, 0x0010826b, 0x001082a4, 0x00000000},
    {0x00002a17, 0x00002a17, 0x00108329, 0x00000000, 0x00000000, 0x00000000},
    {0x0000221c, 0x0000221c, 0x00108205, 0x0010826d, 0x001082a6, 0x00000000},
    {0x00002a19, 0x00002a19, 0x0010832b, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a16, 0x00002a16, 0x00108328, 0x00000000, 0x00000000, 0x00000000},
    {0x00002a1b, 0x00002a1b, 0x0010832d, 0x00000000, 0x00000000, 0x00000000},
    {0x000027e7, 0x000027e7, 0x00108236, 0x0010827d, 0x001082b6, 0x001082ee},
    {0x000027e8, 0x000027e8, 0x00108237, 0x0010827e, 0x001082b7, 0x001082ef},
    {0x00002a1a, 0x00002a1a, 0x0010832c, 0x00000000, 0x00000000, 0x00000000},
    {0x000027ea, 0x000027ea, 0x00108239, 0x00108280, 0x001082b9, 0x001082f1},
    {0x000027eb, 0x000027eb, 0x0010823a, 0x00108281, 0x001082ba, 0x001082f2},
    {0x002020d1, 0x001081f8, 0x00108264, 0x0010829d, 0x001082d5, 0x00108305},
    {0x0000007b, 0x0000007b, 0x001081e8, 0x00108254, 0x0010828d, 0x001082c5},
    {0x000022c0, 0x000022c0, 0x00108206, 0x00000000, 0x00000000, 0x00000000},
    {0x000022c1, 0x000022c1, 0x00108207, 0x00000000, 0x00000000, 0x00000000},
    {0x000022c2, 0x000022c2, 0x00108208, 0x00000000, 0x00000000, 0x00000000},
    {0x000022c3, 0x000022c3, 0x00108209, 0x00000000, 0x00000000, 0x00000000},
    {0x002002c7, 0x001081ee, 0x0010825a, 0x00108293, 0x001082cb, 0x001082fb},
    {0x00200330, 0x001081f4, 0x00108260, 0x00108299, 0x001082d1, 0x00108301},
    {0x002002c6, 0x001081eb, 0x00108257, 0x00108290, 0x001082c8, 0x001082f8},
    {0x00200332, 0x001081f5, 0x00108261, 0x0010829a, 0x001082d2, 0x00108302},
    {0x00002140, 0x00002140, 0x001081ff, 0x00000000, 0x00000000, 0x00000000},
    {0x00002afc, 0x0010824c, 0x00108286, 0x00000000, 0x00000000, 0x00000000},
    {0x002023b5, 0x00108229, 0x00108273, 0x001082ac, 0x001082e4, 0x0010830d},
    {0x00002232, 0x00002232, 0x0010831b, 0x00000000, 0x00000000, 0x00000000},
    {0x0000220f, 0x0000220f, 0x00108200, 0x00000000, 0x00000000, 0x00000000},
    {0x00002210, 0x00002210, 0x00108201, 0x00000000, 0x00000000, 0x00000000},
    {0x000027e6, 0x000027e6, 0x00108235, 0x0010827c, 0x001082b5, 0x001082ed},
    {0x002023e1, 0x00108232, 0x00108279, 0x001082b2, 0x001082ea, 0x00108313},
    {0x0000005b, 0x0000005b, 0x001081e5, 0x00108251, 0x0010828a, 0x001082c2},
    {0x0000005c, 0x0000005c, 0x001081ea, 0x00108256, 0x0010828f, 0x001082c7},
    {0x0000005d, 0x0000005d, 0x001081e6, 0x00108252, 0x0010828b, 0x001082c3},
    {0x002020ec, 0x001081fb, 0x00108267, 0x001082a0, 0x001082d8, 0x00108308},
    {0x00002aff, 0x0010824d, 0x0010839d, 0x00000000, 0x00000000, 0x00000000},
    {0x002023b4, 0x00108228, 0x00108272, 0x001082ab, 0x001082e3, 0x0010830c},
    {0x0000007d, 0x0000007d, 0x001081e9, 0x00108255, 0x0010828e, 0x001082c6},
    {0x002020ee, 0x001081fd, 0x00108269, 0x001082a2, 0x001082da, 0x0010830a},
    {0x0000221b, 0x0000221b, 0x00108204, 0x0010826c, 0x001082a5, 0x00000000},
    {0x002002dc, 0x001081ec, 0x00108258, 0x00108291, 0x001082c9, 0x001082f9},
    {0x002020ed, 0x001081fc, 0x00108268, 0x001082a1, 0x001082d9, 0x00108309},
    {0x00002a07, 0x00002a07, 0x00108248, 0x00000000, 0x00000000, 0x00000000},
    {0x002020ef, 0x001081fe, 0x0010826a, 0x001082a3, 0x001082db, 0x0010830b}
};

#define VARIANT_MODULUS 51
#define VARIANT_OFFSET 53

const uint32_t extension_table[142][11] =
{   {   0x000027f0,
        0x0c90815a, 0x81300064,
        0x0c8027f0, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027f1,
        0x000027f1, 0x03320064,
        0x0c90815a, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002983,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002002f7,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020ec,
        0x1910815c, 0x81f400c8,
        0x259081fb, 0x03670000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000290b,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023b4,
        0x00108169, 0x071c0258,
        0x2590816d, 0x8064012c,
        0x4b10816a, 0x071c0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023b5,
        0x0010816b, 0x071c0258,
        0x2590816e, 0x8064012c,
        0x4b10816c, 0x071c0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020d0,
        0x001081f7, 0x0367012c,
        0x1910815b, 0x81f400c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020d1,
        0x1910815b, 0x81f400c8,
        0x259081f8, 0x03670000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021d0,
        0x000021d0, 0x032601f4,
        0x19108132, 0x81b300c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002191,
        0x258023d0, 0x8130012c,
        0x22202191, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027e9,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000022c2,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00202b45,
        0x00002b45, 0x03320032,
        0x06508159, 0x813b012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00200303,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002772,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002773,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021a4,
        0x00002190, 0x03120258,
        0x258023af, 0x8064012c,
        0x4b0027de, 0x05b80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00000338,
        0x00108222, 0x03ed0258,
        0x4b10821f, 0x83f201f4,
        0x19108221, 0x03f200c8,
        0x3e90821f, 0x83f20258,
        0x4b108220, 0x03ed0000
    },
    {   0x002021a6,
        0x000027dd, 0x05b80258,
        0x258023af, 0x8064012c,
        0x4b002192, 0x03120000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027eb,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027ee,
        0x00108212, 0x03ed0064,
        0x0c908211, 0x83f20064,
        0x0c908210, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002309,
        0x2a90821a, 0x83f20154,
        0x2a908219, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0020030c,
        0x001081f2, 0x03e803e8,
        0x7d1081f2, 0x83e80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000230b,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00202190,
        0x00002190, 0x0312012c,
        0x258023af, 0x813b012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000007d,
        0x00108222, 0x03ed0258,
        0x4b10821f, 0x83f201f4,
        0x19108221, 0x03f200c8,
        0x3e90821f, 0x83f20258,
        0x4b108220, 0x03ed0000
    },
    {   0x00002986,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0a,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00202194,
        0x00002190, 0x03120258,
        0x258023af, 0x8064012c,
        0x4b002192, 0x03120000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002140,
        0x00002016, 0x036f01b8,
        0x37002016, 0x836f0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021be,
        0x258023d0, 0x8130012c,
        0x222021be, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0c,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021bf,
        0x258023d0, 0x8130012c,
        0x222021bf, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0d,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002016,
        0x00002016, 0x036f01b8,
        0x37002016, 0x836f0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020ed,
        0x001081fc, 0x0367012c,
        0x1910815c, 0x81f400c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020ee,
        0x001081fd, 0x0368012c,
        0x1910815c, 0x81f400c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020ef,
        0x1910815c, 0x81f400c8,
        0x259081fe, 0x03680000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00000028,
        0x00108212, 0x03ed0096,
        0x3e908211, 0x83f201f4,
        0x12d08210, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000029f8,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000029f9,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021bd,
        0x000021bd, 0x034f0258,
        0x258023af, 0x813b012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00000029,
        0x00108215, 0x03ed0096,
        0x3e908214, 0x83f201f4,
        0x12d08213, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000005c,
        0x00108218, 0x03ed03b6,
        0x3e908217, 0x83f201f4,
        0x76d08216, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021c0,
        0x258023af, 0x813b012c,
        0x4b0021c0, 0x034f0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000002f,
        0x00108215, 0x03ed0096,
        0x3e908214, 0x83f201f4,
        0x12d08213, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000221b,
        0x001082dd, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000221c,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023e0,
        0x00108160, 0x03a500c8,
        0x1910816e, 0x81f400c8,
        0x4b108164, 0x07480258,
        0x1910816e, 0x81f400c8,
        0x19108161, 0x03a50000
    },
    {   0x002023e1,
        0x00108160, 0x03a500c8,
        0x1910816e, 0x81f400c8,
        0x4b108164, 0x07480258,
        0x1910816e, 0x81f400c8,
        0x19108161, 0x03a50000
    },
    {   0x00002a03,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a16,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00200302,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a17,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a07,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000220f,
        0x000021d3, 0x0332012c,
        0x0c908131, 0x812c0064,
        0x258021d1, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002210,
        0x000021d3, 0x0332012c,
        0x0c908131, 0x812c0064,
        0x258021d1, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002211,
        0x000021d3, 0x0332012c,
        0x0c908131, 0x812c0064,
        0x258021d1, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00200330,
        0x001081f2, 0x03e803e8,
        0x7d1081f2, 0x83e80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021c3,
        0x000021c3, 0x03320111,
        0x258023d0, 0x8130012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00200332,
        0x001081f5, 0x03e803e8,
        0x7d1081f5, 0x83e80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021d2,
        0x19108132, 0x81b300c8,
        0x3e8021d2, 0x03260000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a10,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021d4,
        0x000021d0, 0x032601f4,
        0x19108132, 0x806400c8,
        0x3e8021d2, 0x03260000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a12,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a13,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a14,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a15,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002232,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002233,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002aff,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00200305,
        0x001081f2, 0x03e803e8,
        0x7d1081f2, 0x83e80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002193,
        0x00002193, 0x03320111,
        0x258023d0, 0x8130012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a1b,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002195,
        0x00002193, 0x03320111,
        0x258023d0, 0x8130012c,
        0x22202191, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00108224,
        0x00108222, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c90821c, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021d5,
        0x000021d3, 0x0332012c,
        0x0c908131, 0x812c0064,
        0x258021d1, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00108225,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023dc,
        0x00108165, 0x073f0258,
        0x2590816d, 0x8258012c,
        0x4b108166, 0x073f0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023dd,
        0x00108167, 0x073f0258,
        0x2590816e, 0x8258012c,
        0x4b108168, 0x073f0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002308,
        0x2a908217, 0x83f20154,
        0x2a908216, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023df,
        0x00108160, 0x03a500c8,
        0x1910816e, 0x81f400c8,
        0x4b108164, 0x07480258,
        0x1910816e, 0x81f400c8,
        0x19108161, 0x03a50000
    },
    {   0x0000222b,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000222c,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000222d,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000222e,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000222f,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002230,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002231,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000022c0,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000005d,
        0x0010821b, 0x03ed03b6,
        0x3e90821a, 0x83f201f4,
        0x76d08219, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000005b,
        0x00108218, 0x03ed03b6,
        0x3e908217, 0x83f201f4,
        0x76d08216, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000022c3,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002002c6,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002002c7,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00202b46,
        0x25908159, 0x813b0032,
        0x06402b46, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002afc,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021da,
        0x000021da, 0x032601f4,
        0x19108157, 0x813b00c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021db,
        0x19108157, 0x813b00c8,
        0x3e8021db, 0x03260000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a00,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a01,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a02,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000022c1,
        0x001082de, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a04,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a05,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a06,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000230a,
        0x00108218, 0x03ed0154,
        0x2a908217, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a08,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a09,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002021bc,
        0x000021bc, 0x034f0258,
        0x258023af, 0x813b012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0b,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0020203e,
        0x001081f5, 0x03e803e8,
        0x7d1081f5, 0x83e80000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000221a,
        0x001082dc, 0x073f0258,
        0x26d0839b, 0x827b0136,
        0x3e90839c, 0x02720000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0e,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a0f,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002002dc,
        0x0010821e, 0x03ed0064,
        0x0c90821f, 0x83f20064,
        0x0c908220, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a11,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002984,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002985,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021c2,
        0x000021c2, 0x03320111,
        0x258023d0, 0x8130012c,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000007b,
        0x0010821e, 0x03ed0258,
        0x4b10821f, 0x83f201f4,
        0x1910821d, 0x03f200c8,
        0x3e90821f, 0x83f20258,
        0x4b10821c, 0x03ed0000
    },
    {   0x0000007c,
        0x0000007c, 0x02b20258,
        0x4b00007c, 0x82b20000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020d7,
        0x1910815b, 0x81f400c8,
        0x259081fa, 0x03680000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a18,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a19,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a1a,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00202192,
        0x258023af, 0x813b012c,
        0x25802192, 0x03120000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x00002a1c,
        0x0000290b, 0x03320064,
        0x0c908158, 0x81300064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002023de,
        0x0010815e, 0x03a500c8,
        0x1910816d, 0x81f400c8,
        0x4b108163, 0x07480258,
        0x1910816d, 0x81f400c8,
        0x1910815f, 0x03a50000
    },
    {   0x000027e6,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027e7,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027e8,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020e1,
        0x001081f9, 0x03680258,
        0x1910815b, 0x806400c8,
        0x4b1081fa, 0x03680000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027ea,
        0x0010821b, 0x03ed0154,
        0x2a90821a, 0x83f20154,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021d1,
        0x0c908131, 0x812c0064,
        0x258021d1, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x002020d6,
        0x001081f9, 0x0368012c,
        0x1910815b, 0x81f400c8,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000021d3,
        0x000021d3, 0x0332012c,
        0x0c908131, 0x812c0064,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x0000290a,
        0x0c908158, 0x81300064,
        0x0c80290a, 0x03320000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    },
    {   0x000027ef,
        0x00108215, 0x03ed0064,
        0x0c908214, 0x83f20064,
        0x0c908213, 0x03ed0000,
        0x00000000, 0x00000000,
        0x00000000, 0x00000000
    }
};

#define EXTENSION_MODULUS 101
#define EXTENSION_OFFSET 3

const std::int16_t fontkern[] =
{   -1,        // cmuntt [118 items]
    117,       // odokai
    117,       // cslSTIX-Regular [1024 items]
    1141,      // cslSTIX-Bold [1010 items]
    2151,      // cslSTIX-Italic [1001 items]
    3152,      // cslSTIX-BoldItalic
    4167       // cslSTIXMath-Regular [-4167 items]
};

const uint32_t kerntable[] =
{   0x00200309,   // [0:1] space + hookabovecomb ligature #0 (cmuntt)
    0x00a00489,   // [1:2] space + COMBINING_CYRILLIC_MILLIONS_SIGN ligature #1 (cmuntt)
    0x01200488,   // [2:3] space + COMBINING_CYRILLIC_HUNDRED_THOUSANDS_SIGN ligature #2 (cmuntt)
    0x01a00486,   // [3:4] space + COMBINING_CYRILLIC_PSILI_PNEUMATA ligature #3 (cmuntt)
    0x02200485,   // [4:5] space + COMBINING_CYRILLIC_DASIA_PNEUMATA ligature #4 (cmuntt)
    0x02a00484,   // [5:6] space + COMBINING_CYRILLIC_PALATALIZATION ligature #5 (cmuntt)
    0x0320034d,   // [6:7] space + COMBINING_LEFT_RIGHT_ARROW_BELOW ligature #6 (cmuntt)
    0x03a0033c,   // [7:8] space + COMBINING_SEAGULL_BELOW ligature #7 (cmuntt)
    0x0420033b,   // [8:9] space + COMBINING_SQUARE_BELOW ligature #8 (cmuntt)
    0x04a0033a,   // [9:10] space + COMBINING_INVERTED_BRIDGE_BELOW ligature #9 (cmuntt)
    0x05200337,   // [10:11] space + COMBINING_SHORT_SOLIDUS_OVERLAY ligature #10 (cmuntt)
    0x05a00335,   // [11:12] space + COMBINING_SHORT_STROKE_OVERLAY ligature #11 (cmuntt)
    0x0620032b,   // [12:13] space + COMBINING_INVERTED_DOUBLE_ARCH_BELOW ligature #12 (cmuntt)
    0x06a0032a,   // [13:14] space + COMBINING_BRIDGE_BELOW ligature #13 (cmuntt)
    0x07200322,   // [14:15] space + COMBINING_RETROFLEX_HOOK_BELOW ligature #14 (cmuntt)
    0x07a00321,   // [15:16] space + COMBINING_PALATALIZED_HOOK_BELOW ligature #15 (cmuntt)
    0x08200311,   // [16:17] space + COMBINING_INVERTED_BREVE ligature #16 (cmuntt)
    0x08a0030f,   // [17:18] space + COMBINING_DOUBLE_GRAVE_ACCENT ligature #17 (cmuntt)
    0x0920030e,   // [18:19] space + COMBINING_DOUBLE_VERTICAL_LINE_ABOVE ligature #18 (cmuntt)
    0x09a0030d,   // [19:20] space + COMBINING_VERTICAL_LINE_ABOVE ligature #19 (cmuntt)
    0x0a200303,   // [20:21] space + COMBINING_TILDE ligature #20 (cmuntt)
    0x0aa01dc9,   // [21:22] space + COMBINING_ACUTE_GRAVE_ACUTE ligature #21 (cmuntt)
    0x0b201dc8,   // [22:23] space + COMBINING_GRAVE_ACUTE_GRAVE ligature #22 (cmuntt)
    0x0ba01dc5,   // [23:24] space + COMBINING_GRAVE_MACRON ligature #23 (cmuntt)
    0x0c601dc4,   // [24:25] space + COMBINING_MACRON_ACUTE ligature #24 (cmuntt) ;;
    0x0ce0004a,   // [25:26] I + J ligature #25 (cmuntt) ;;
    0x0d20006c,   // [26:27] f + l ligature #26 (cmuntt)
    0x0da00069,   // [27:28] f + i ligature #27 (cmuntt)
    0x0e600066,   // [28:29] f + f ligature #28 (cmuntt) ;;
    0x0ee0006a,   // [29:30] i + j ligature #29 (cmuntt) ;;
    0x0f600074,   // [30:31] longs + t ligature #30 (cmuntt) ;;
    0x2b800295,   // [31:32] LATIN_SMALL_LETTER_C_WITH_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [32:33] LATIN_SMALL_LETTER_C_WITH_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b800295,   // [33:34] LATIN_SMALL_LETTER_T_WITH_CURL + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [34:35] LATIN_SMALL_LETTER_T_WITH_CURL + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b80f147,   // [35:36] LATIN_SMALL_LETTER_D_WITH_TAIL + beta.ipa : 87 (cmuntt)
    0x2b80029d,   // [36:37] LATIN_SMALL_LETTER_D_WITH_TAIL + LATIN_SMALL_LETTER_J_WITH_CROSSED_TAIL : 87 (cmuntt)
    0x2b800283,   // [37:38] LATIN_SMALL_LETTER_D_WITH_TAIL + LATIN_SMALL_LETTER_ESH : 87 (cmuntt)
    0x2b800272,   // [38:39] LATIN_SMALL_LETTER_D_WITH_TAIL + LATIN_SMALL_LETTER_N_WITH_LEFT_HOOK : 87 (cmuntt)
    0x0e800261,   // [39:40] LATIN_SMALL_LETTER_D_WITH_TAIL + LATIN_SMALL_LETTER_SCRIPT_G : 29 (cmuntt)
    0x2bc0025f,   // [40:41] LATIN_SMALL_LETTER_D_WITH_TAIL + LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE : 87 (cmuntt) ;;
    0x2b800295,   // [41:42] LATIN_SMALL_LETTER_D_WITH_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [42:43] LATIN_SMALL_LETTER_D_WITH_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b800295,   // [43:44] LATIN_SMALL_LETTER_G_WITH_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [44:45] LATIN_SMALL_LETTER_G_WITH_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b80f147,   // [45:46] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + beta.ipa : 87 (cmuntt)
    0x2b80029d,   // [46:47] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_J_WITH_CROSSED_TAIL : 87 (cmuntt)
    0x2b800283,   // [47:48] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_ESH : 87 (cmuntt)
    0x2b800272,   // [48:49] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_N_WITH_LEFT_HOOK : 87 (cmuntt)
    0x0e800261,   // [49:50] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_SCRIPT_G : 29 (cmuntt)
    0x2bc0025f,   // [50:51] LATIN_SMALL_LETTER_L_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE : 87 (cmuntt) ;;
    0x2b80f147,   // [51:52] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + beta.ipa : 87 (cmuntt)
    0x2b80029d,   // [52:53] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_J_WITH_CROSSED_TAIL : 87 (cmuntt)
    0x2b800283,   // [53:54] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_ESH : 87 (cmuntt)
    0x2b800272,   // [54:55] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_N_WITH_LEFT_HOOK : 87 (cmuntt)
    0x0e800261,   // [55:56] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_SCRIPT_G : 29 (cmuntt)
    0x2bc0025f,   // [56:57] LATIN_SMALL_LETTER_N_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE : 87 (cmuntt) ;;
    0x2b80f147,   // [57:58] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + beta.ipa : 87 (cmuntt)
    0x2b80029d,   // [58:59] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + LATIN_SMALL_LETTER_J_WITH_CROSSED_TAIL : 87 (cmuntt)
    0x2b800283,   // [59:60] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + LATIN_SMALL_LETTER_ESH : 87 (cmuntt)
    0x2b800272,   // [60:61] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + LATIN_SMALL_LETTER_N_WITH_LEFT_HOOK : 87 (cmuntt)
    0x0e800261,   // [61:62] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + LATIN_SMALL_LETTER_SCRIPT_G : 29 (cmuntt)
    0x2bc0025f,   // [62:63] LATIN_SMALL_LETTER_TURNED_R_WITH_HOOK + LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE : 87 (cmuntt) ;;
    0x2b800295,   // [63:64] LATIN_SMALL_LETTER_ESH + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [64:65] LATIN_SMALL_LETTER_ESH + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b800295,   // [65:66] LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE_AND_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [66:67] LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE_AND_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b80f147,   // [67:68] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + beta.ipa : 87 (cmuntt)
    0x2b80029d,   // [68:69] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_J_WITH_CROSSED_TAIL : 87 (cmuntt)
    0x2b800283,   // [69:70] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_ESH : 87 (cmuntt)
    0x2b800272,   // [70:71] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_N_WITH_LEFT_HOOK : 87 (cmuntt)
    0x0e800261,   // [71:72] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_SCRIPT_G : 29 (cmuntt)
    0x2bc0025f,   // [72:73] LATIN_SMALL_LETTER_Z_WITH_RETROFLEX_HOOK + LATIN_SMALL_LETTER_DOTLESS_J_WITH_STROKE : 87 (cmuntt) ;;
    0x2b800295,   // [73:74] LATIN_LETTER_SMALL_CAPITAL_G_WITH_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [74:75] LATIN_LETTER_SMALL_CAPITAL_G_WITH_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b800295,   // [75:76] LATIN_SMALL_LETTER_Q_WITH_HOOK + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [76:77] LATIN_SMALL_LETTER_Q_WITH_HOOK + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0x2b800295,   // [77:78] LATIN_SMALL_LETTER_TESH_DIGRAPH + LATIN_LETTER_PHARYNGEAL_VOICED_FRICATIVE : 87 (cmuntt)
    0x2bc00294,   // [78:79] LATIN_SMALL_LETTER_TESH_DIGRAPH + LATIN_LETTER_GLOTTAL_STOP : 87 (cmuntt) ;;
    0xd4c002bb,   // [79:80] MODIFIER_LETTER_TURNED_COMMA + MODIFIER_LETTER_TURNED_COMMA : -87 (cmuntt) ;;
    0xd480f130,   // [80:81] GREEK_PSILI_AND_VARIA + GREEK_CAPITAL_LETTER_ALPHA_WITH_PROSGEGRAMMENI.alt : -87 (cmuntt)
    0xf180039f,   // [81:82] GREEK_PSILI_AND_VARIA + Omicron : -29 (cmuntt)
    0xd4c00391,   // [82:83] GREEK_PSILI_AND_VARIA + Alpha : -87 (cmuntt) ;;
    0xd480f130,   // [83:84] GREEK_DASIA_AND_VARIA + GREEK_CAPITAL_LETTER_ALPHA_WITH_PROSGEGRAMMENI.alt : -87 (cmuntt)
    0xf180039f,   // [84:85] GREEK_DASIA_AND_VARIA + Omicron : -29 (cmuntt)
    0xd4c00391,   // [85:86] GREEK_DASIA_AND_VARIA + Alpha : -87 (cmuntt) ;;
    0xd480f51a,   // [86:87] Alpha.sc + Upsilondieresis.sc : -87 (cmuntt)
    0xd480f517,   // [87:88] Alpha.sc + Psi.sc : -87 (cmuntt)
    0xe300f515,   // [88:89] Alpha.sc + Phi.sc : -58 (cmuntt)
    0xd480f514,   // [89:90] Alpha.sc + Upsilon.sc : -87 (cmuntt)
    0xe300f513,   // [90:91] Alpha.sc + Tau.sc : -58 (cmuntt)
    0xe300f50e,   // [91:92] Alpha.sc + Omicron.sc : -58 (cmuntt)
    0xe340f507,   // [92:93] Alpha.sc + Theta.sc : -58 (cmuntt) ;;
    0xd480f514,   // [93:94] GREEK_CAPITAL_LETTER_DELTA.sc + Upsilon.sc : -87 (cmuntt)
    0xf7c0f50e,   // [94:95] GREEK_CAPITAL_LETTER_DELTA.sc + Omicron.sc : -17 (cmuntt) ;;
    0xcb80f50a,   // [95:96] Gamma.sc + Lambda.sc : -105 (cmuntt)
    0xd480f502,   // [96:97] Gamma.sc + GREEK_CAPITAL_LETTER_DELTA.sc : -87 (cmuntt)
    0xc5c0f500,   // [97:98] Gamma.sc + Alpha.sc : -117 (cmuntt) ;;
    0xe340f500,   // [98:99] Theta.sc + Alpha.sc : -58 (cmuntt) ;;
    0xe340f50e,   // [99:100] Kappa.sc + Omicron.sc : -58 (cmuntt) ;;
    0xd480f514,   // [100:101] Lambda.sc + Upsilon.sc : -87 (cmuntt)
    0xf7c0f50e,   // [101:102] Lambda.sc + Omicron.sc : -17 (cmuntt) ;;
    0xd4c0f500,   // [102:103] Nu.sc + Alpha.sc : -87 (cmuntt) ;;
    0xf180f512,   // [103:104] Omicron.sc + Sigma.sc : -29 (cmuntt)
    0xe340f500,   // [104:105] Omicron.sc + Alpha.sc : -58 (cmuntt) ;;
    0xc5c0f500,   // [105:106] Rho.sc + Alpha.sc : -117 (cmuntt) ;;
    0xf640f500,   // [106:107] Sigma.sc + Alpha.sc : -20 (cmuntt) ;;
    0xe340f500,   // [107:108] Tau.sc + Alpha.sc : -58 (cmuntt) ;;
    0xd480f50a,   // [108:109] Upsilon.sc + Lambda.sc : -87 (cmuntt)
    0xd480f502,   // [109:110] Upsilon.sc + GREEK_CAPITAL_LETTER_DELTA.sc : -87 (cmuntt)
    0xd4c0f500,   // [110:111] Upsilon.sc + Alpha.sc : -87 (cmuntt) ;;
    0xe340f500,   // [111:112] Phi.sc + Alpha.sc : -58 (cmuntt) ;;
    0xd4c0f500,   // [112:113] Psi.sc + Alpha.sc : -87 (cmuntt) ;;
    0xd480f50a,   // [113:114] Upsilondieresis.sc + Lambda.sc : -87 (cmuntt)
    0xd480f502,   // [114:115] Upsilondieresis.sc + GREEK_CAPITAL_LETTER_DELTA.sc : -87 (cmuntt)
    0xd4c0f500,   // [115:116] Upsilondieresis.sc + Alpha.sc : -87 (cmuntt) ;;
    0x0fa0006c,   // [116:117] ff + l ligature #31 (cmuntt)
    0x10600069,   // [117:118] ff + i ligature #32 (cmuntt) ;;
    0xef00201d,   // [118:1] comma + quotedblright : -34 (cslSTIX-Regular)
    0xe3402019,   // [119:2] comma + quoteright : -58 (cslSTIX-Regular) ;;
    0xf30000c6,   // [120:3] hyphen + AE : -26 (cslSTIX-Regular)
    0xf10000c5,   // [121:4] hyphen + Aring : -30 (cslSTIX-Regular)
    0xf10000c4,   // [122:5] hyphen + Adieresis : -30 (cslSTIX-Regular)
    0xf10000c1,   // [123:6] hyphen + Aacute : -30 (cslSTIX-Regular)
    0xd0800059,   // [124:7] hyphen + Y : -95 (cslSTIX-Regular)
    0xe9800057,   // [125:8] hyphen + W : -45 (cslSTIX-Regular)
    0xd9000056,   // [126:9] hyphen + V : -78 (cslSTIX-Regular)
    0xd8800054,   // [127:10] hyphen + T : -79 (cslSTIX-Regular)
    0xf1400041,   // [128:11] hyphen + A : -30 (cslSTIX-Regular) ;;
    0xed80201d,   // [129:12] period + quotedblright : -37 (cslSTIX-Regular)
    0xedc02019,   // [130:13] period + quoteright : -37 (cslSTIX-Regular) ;;
    0xe7802039,   // [131:14] A + guilsinglleft : -49 (cslSTIX-Regular)
    0xe380201d,   // [132:15] A + quotedblright : -57 (cslSTIX-Regular)
    0xdb002019,   // [133:16] A + quoteright : -74 (cslSTIX-Regular)
    0xf90000fc,   // [134:17] A + udieresis : -14 (cslSTIX-Regular)
    0xf90000fb,   // [135:18] A + ucircumflex : -14 (cslSTIX-Regular)
    0xf90000fa,   // [136:19] A + uacute : -14 (cslSTIX-Regular)
    0xf90000f9,   // [137:20] A + ugrave : -14 (cslSTIX-Regular)
    0xf30000f6,   // [138:21] A + odieresis : -26 (cslSTIX-Regular)
    0xfb0000e7,   // [139:22] A + ccedilla : -10 (cslSTIX-Regular)
    0xee8000dc,   // [140:23] A + Udieresis : -35 (cslSTIX-Regular)
    0xee8000db,   // [141:24] A + Ucircumflex : -35 (cslSTIX-Regular)
    0xee8000da,   // [142:25] A + Uacute : -35 (cslSTIX-Regular)
    0xee8000d9,   // [143:26] A + Ugrave : -35 (cslSTIX-Regular)
    0xf38000d6,   // [144:27] A + Odieresis : -25 (cslSTIX-Regular)
    0xf38000c7,   // [145:28] A + Ccedilla : -25 (cslSTIX-Regular)
    0xeb0000ab,   // [146:29] A + guillemotleft : -42 (cslSTIX-Regular)
    0xdc000079,   // [147:30] A + y : -72 (cslSTIX-Regular)
    0xe3000077,   // [148:31] A + w : -58 (cslSTIX-Regular)
    0xe3000076,   // [149:32] A + v : -58 (cslSTIX-Regular)
    0xf9000075,   // [150:33] A + u : -14 (cslSTIX-Regular)
    0xf9800074,   // [151:34] A + t : -13 (cslSTIX-Regular)
    0xf9000071,   // [152:35] A + q : -14 (cslSTIX-Regular)
    0xf300006f,   // [153:36] A + o : -26 (cslSTIX-Regular)
    0xfa000067,   // [154:37] A + g : -12 (cslSTIX-Regular)
    0xf8000065,   // [155:38] A + e : -16 (cslSTIX-Regular)
    0xf7800064,   // [156:39] A + d : -17 (cslSTIX-Regular)
    0xfb000063,   // [157:40] A + c : -10 (cslSTIX-Regular)
    0xf9000062,   // [158:41] A + b : -14 (cslSTIX-Regular)
    0xfe000061,   // [159:42] A + a : -4 (cslSTIX-Regular)
    0xe6000059,   // [160:43] A + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [161:44] A + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [162:45] A + V : -83 (cslSTIX-Regular)
    0xee800055,   // [163:46] A + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [164:47] A + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [165:48] A + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [166:49] A + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [167:50] A + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [168:51] A + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [169:52] A + period : -5 (cslSTIX-Regular)
    0xf800002d,   // [170:53] A + hyphen : -16 (cslSTIX-Regular)
    0xff40002c,   // [171:54] A + comma : -2 (cslSTIX-Regular) ;;
    0xf7800152,   // [172:55] B + OE : -17 (cslSTIX-Regular)
    0xf78000d8,   // [173:56] B + Oslash : -17 (cslSTIX-Regular)
    0xf78000d6,   // [174:57] B + Odieresis : -17 (cslSTIX-Regular)
    0xf78000d4,   // [175:58] B + Ocircumflex : -17 (cslSTIX-Regular)
    0xf78000d3,   // [176:59] B + Oacute : -17 (cslSTIX-Regular)
    0xf78000d2,   // [177:60] B + Ograve : -17 (cslSTIX-Regular)
    0xf20000c6,   // [178:61] B + AE : -28 (cslSTIX-Regular)
    0xef8000c5,   // [179:62] B + Aring : -33 (cslSTIX-Regular)
    0xef8000c4,   // [180:63] B + Adieresis : -33 (cslSTIX-Regular)
    0xef8000c3,   // [181:64] B + Atilde : -33 (cslSTIX-Regular)
    0xef8000c2,   // [182:65] B + Acircumflex : -33 (cslSTIX-Regular)
    0xef8000c1,   // [183:66] B + Aacute : -33 (cslSTIX-Regular)
    0xe2000059,   // [184:67] B + Y : -60 (cslSTIX-Regular)
    0xe8800057,   // [185:68] B + W : -47 (cslSTIX-Regular)
    0xe7000056,   // [186:69] B + V : -50 (cslSTIX-Regular)
    0xf780004f,   // [187:70] B + O : -17 (cslSTIX-Regular)
    0xefc00041,   // [188:71] B + A : -33 (cslSTIX-Regular) ;;
    0xfc0000d6,   // [189:72] C + Odieresis : -8 (cslSTIX-Regular)
    0xfc0000d3,   // [190:73] C + Oacute : -8 (cslSTIX-Regular)
    0xfc0000d2,   // [191:74] C + Ograve : -8 (cslSTIX-Regular)
    0xfa8000c6,   // [192:75] C + AE : -11 (cslSTIX-Regular)
    0xf80000c5,   // [193:76] C + Aring : -16 (cslSTIX-Regular)
    0xf80000c4,   // [194:77] C + Adieresis : -16 (cslSTIX-Regular)
    0xf80000c1,   // [195:78] C + Aacute : -16 (cslSTIX-Regular)
    0xf80000c0,   // [196:79] C + Agrave : -16 (cslSTIX-Regular)
    0xfc00004f,   // [197:80] C + O : -8 (cslSTIX-Regular)
    0xfb80004b,   // [198:81] C + K : -9 (cslSTIX-Regular)
    0xf8400041,   // [199:82] C + A : -16 (cslSTIX-Regular) ;;
    0xec0000c5,   // [200:83] D + Aring : -40 (cslSTIX-Regular)
    0xec0000c4,   // [201:84] D + Adieresis : -40 (cslSTIX-Regular)
    0xec0000c3,   // [202:85] D + Atilde : -40 (cslSTIX-Regular)
    0xec0000c2,   // [203:86] D + Acircumflex : -40 (cslSTIX-Regular)
    0xec0000c1,   // [204:87] D + Aacute : -40 (cslSTIX-Regular)
    0xec0000c0,   // [205:88] D + Agrave : -40 (cslSTIX-Regular)
    0xe9000059,   // [206:89] D + Y : -46 (cslSTIX-Regular)
    0xeb000058,   // [207:90] D + X : -42 (cslSTIX-Regular)
    0xed000057,   // [208:91] D + W : -38 (cslSTIX-Regular)
    0xed000056,   // [209:92] D + V : -38 (cslSTIX-Regular)
    0xfd800054,   // [210:93] D + T : -5 (cslSTIX-Regular)
    0xf600004a,   // [211:94] D + J : -20 (cslSTIX-Regular)
    0xec400041,   // [212:95] D + A : -40 (cslSTIX-Regular) ;;
    0xfd800153,   // [213:96] F + oe : -5 (cslSTIX-Regular)
    0xfd8000f8,   // [214:97] F + oslash : -5 (cslSTIX-Regular)
    0xfd8000f6,   // [215:98] F + odieresis : -5 (cslSTIX-Regular)
    0xfd8000f3,   // [216:99] F + oacute : -5 (cslSTIX-Regular)
    0xfe0000e9,   // [217:100] F + eacute : -4 (cslSTIX-Regular)
    0xf88000e6,   // [218:101] F + ae : -15 (cslSTIX-Regular)
    0xfa8000e5,   // [219:102] F + aring : -11 (cslSTIX-Regular)
    0xff8000e4,   // [220:103] F + adieresis : -1 (cslSTIX-Regular)
    0xff8000e3,   // [221:104] F + atilde : -1 (cslSTIX-Regular)
    0xff0000e2,   // [222:105] F + acircumflex : -2 (cslSTIX-Regular)
    0xfa8000e1,   // [223:106] F + aacute : -11 (cslSTIX-Regular)
    0xff8000d6,   // [224:107] F + Odieresis : -1 (cslSTIX-Regular)
    0xff8000d3,   // [225:108] F + Oacute : -1 (cslSTIX-Regular)
    0xff8000d2,   // [226:109] F + Ograve : -1 (cslSTIX-Regular)
    0xee8000c5,   // [227:110] F + Aring : -35 (cslSTIX-Regular)
    0xee8000c4,   // [228:111] F + Adieresis : -35 (cslSTIX-Regular)
    0xee8000c3,   // [229:112] F + Atilde : -35 (cslSTIX-Regular)
    0xee8000c2,   // [230:113] F + Acircumflex : -35 (cslSTIX-Regular)
    0xee8000c1,   // [231:114] F + Aacute : -35 (cslSTIX-Regular)
    0xee8000c0,   // [232:115] F + Agrave : -35 (cslSTIX-Regular)
    0xff800075,   // [233:116] F + u : -1 (cslSTIX-Regular)
    0xff800072,   // [234:117] F + r : -1 (cslSTIX-Regular)
    0xfd80006f,   // [235:118] F + o : -5 (cslSTIX-Regular)
    0xfd80006a,   // [236:119] F + j : -5 (cslSTIX-Regular)
    0xfd800069,   // [237:120] F + i : -5 (cslSTIX-Regular)
    0xfe000065,   // [238:121] F + e : -4 (cslSTIX-Regular)
    0xfa800061,   // [239:122] F + a : -11 (cslSTIX-Regular)
    0xff80004f,   // [240:123] F + O : -1 (cslSTIX-Regular)
    0xff80004a,   // [241:124] F + J : -1 (cslSTIX-Regular)
    0xee800041,   // [242:125] F + A : -35 (cslSTIX-Regular)
    0xe880002e,   // [243:126] F + period : -47 (cslSTIX-Regular)
    0x0180002d,   // [244:127] F + hyphen : 3 (cslSTIX-Regular)
    0xed40002c,   // [245:128] F + comma : -38 (cslSTIX-Regular) ;;
    0xfa8000c6,   // [246:129] G + AE : -11 (cslSTIX-Regular)
    0xf88000c5,   // [247:130] G + Aring : -15 (cslSTIX-Regular)
    0xf88000c4,   // [248:131] G + Adieresis : -15 (cslSTIX-Regular)
    0xf88000c3,   // [249:132] G + Atilde : -15 (cslSTIX-Regular)
    0xf88000c2,   // [250:133] G + Acircumflex : -15 (cslSTIX-Regular)
    0xf88000c1,   // [251:134] G + Aacute : -15 (cslSTIX-Regular)
    0xf88000c0,   // [252:135] G + Agrave : -15 (cslSTIX-Regular)
    0xf5800059,   // [253:136] G + Y : -21 (cslSTIX-Regular)
    0xf8800057,   // [254:137] G + W : -15 (cslSTIX-Regular)
    0xf8800056,   // [255:138] G + V : -15 (cslSTIX-Regular)
    0xf9800054,   // [256:139] G + T : -13 (cslSTIX-Regular)
    0xf8c00041,   // [257:140] G + A : -15 (cslSTIX-Regular) ;;
    0x10e0004a,   // [258:141] I + J ligature #33 (cslSTIX-Regular) ;;
    0xf10000c6,   // [259:142] J + AE : -30 (cslSTIX-Regular)
    0xee8000c5,   // [260:143] J + Aring : -35 (cslSTIX-Regular)
    0xee8000c4,   // [261:144] J + Adieresis : -35 (cslSTIX-Regular)
    0xeec00041,   // [262:145] J + A : -35 (cslSTIX-Regular) ;;
    0xf6800152,   // [263:146] K + OE : -19 (cslSTIX-Regular)
    0xfb0000fc,   // [264:147] K + udieresis : -10 (cslSTIX-Regular)
    0xf88000f6,   // [265:148] K + odieresis : -15 (cslSTIX-Regular)
    0xf88000f3,   // [266:149] K + oacute : -15 (cslSTIX-Regular)
    0xff0000e6,   // [267:150] K + ae : -2 (cslSTIX-Regular)
    0x008000e5,   // [268:151] K + aring : 1 (cslSTIX-Regular)
    0x008000e4,   // [269:152] K + adieresis : 1 (cslSTIX-Regular)
    0xf68000d6,   // [270:153] K + Odieresis : -19 (cslSTIX-Regular)
    0xf68000d3,   // [271:154] K + Oacute : -19 (cslSTIX-Regular)
    0xe9800079,   // [272:155] K + y : -45 (cslSTIX-Regular)
    0xfb000075,   // [273:156] K + u : -10 (cslSTIX-Regular)
    0xf880006f,   // [274:157] K + o : -15 (cslSTIX-Regular)
    0xf8800065,   // [275:158] K + e : -15 (cslSTIX-Regular)
    0x00800061,   // [276:159] K + a : 1 (cslSTIX-Regular)
    0xff800054,   // [277:160] K + T : -1 (cslSTIX-Regular)
    0x00800053,   // [278:161] K + S : 1 (cslSTIX-Regular)
    0xf680004f,   // [279:162] K + O : -19 (cslSTIX-Regular)
    0xf6800047,   // [280:163] K + G : -19 (cslSTIX-Regular)
    0xf6800043,   // [281:164] K + C : -19 (cslSTIX-Regular)
    0xe740002d,   // [282:165] K + hyphen : -50 (cslSTIX-Regular) ;;
    0xd680201d,   // [283:166] L + quotedblright : -83 (cslSTIX-Regular)
    0xd6802019,   // [284:167] L + quoteright : -83 (cslSTIX-Regular)
    0xfc0000fc,   // [285:168] L + udieresis : -8 (cslSTIX-Regular)
    0xf70000dc,   // [286:169] L + Udieresis : -18 (cslSTIX-Regular)
    0xff8000d6,   // [287:170] L + Odieresis : -1 (cslSTIX-Regular)
    0xff8000d5,   // [288:171] L + Otilde : -1 (cslSTIX-Regular)
    0xff8000d4,   // [289:172] L + Ocircumflex : -1 (cslSTIX-Regular)
    0xff8000d3,   // [290:173] L + Oacute : -1 (cslSTIX-Regular)
    0xff8000d2,   // [291:174] L + Ograve : -1 (cslSTIX-Regular)
    0x030000c6,   // [292:175] L + AE : 6 (cslSTIX-Regular)
    0xff8000c5,   // [293:176] L + Aring : -1 (cslSTIX-Regular)
    0xff8000c4,   // [294:177] L + Adieresis : -1 (cslSTIX-Regular)
    0xff8000c1,   // [295:178] L + Aacute : -1 (cslSTIX-Regular)
    0xed800079,   // [296:179] L + y : -37 (cslSTIX-Regular)
    0xfc000075,   // [297:180] L + u : -8 (cslSTIX-Regular)
    0xda800059,   // [298:181] L + Y : -75 (cslSTIX-Regular)
    0xe3000057,   // [299:182] L + W : -58 (cslSTIX-Regular)
    0xdb000056,   // [300:183] L + V : -74 (cslSTIX-Regular)
    0xf7000055,   // [301:184] L + U : -18 (cslSTIX-Regular)
    0xe2000054,   // [302:185] L + T : -60 (cslSTIX-Regular)
    0x01800053,   // [303:186] L + S : 3 (cslSTIX-Regular)
    0xff80004f,   // [304:187] L + O : -1 (cslSTIX-Regular)
    0xfe800047,   // [305:188] L + G : -3 (cslSTIX-Regular)
    0xff800043,   // [306:189] L + C : -1 (cslSTIX-Regular)
    0xff800041,   // [307:190] L + A : -1 (cslSTIX-Regular)
    0x0bc0002d,   // [308:191] L + hyphen : 23 (cslSTIX-Regular) ;;
    0xf98000fc,   // [309:192] N + udieresis : -13 (cslSTIX-Regular)
    0xf98000f8,   // [310:193] N + oslash : -13 (cslSTIX-Regular)
    0xf98000f6,   // [311:194] N + odieresis : -13 (cslSTIX-Regular)
    0xf98000f3,   // [312:195] N + oacute : -13 (cslSTIX-Regular)
    0xfa0000e9,   // [313:196] N + eacute : -12 (cslSTIX-Regular)
    0xf70000e6,   // [314:197] N + ae : -18 (cslSTIX-Regular)
    0xf70000e5,   // [315:198] N + aring : -18 (cslSTIX-Regular)
    0xf70000e4,   // [316:199] N + adieresis : -18 (cslSTIX-Regular)
    0xf70000e1,   // [317:200] N + aacute : -18 (cslSTIX-Regular)
    0xfb0000d6,   // [318:201] N + Odieresis : -10 (cslSTIX-Regular)
    0xfb0000d3,   // [319:202] N + Oacute : -10 (cslSTIX-Regular)
    0xfb0000c7,   // [320:203] N + Ccedilla : -10 (cslSTIX-Regular)
    0xf98000c6,   // [321:204] N + AE : -13 (cslSTIX-Regular)
    0xf70000c5,   // [322:205] N + Aring : -18 (cslSTIX-Regular)
    0xf70000c4,   // [323:206] N + Adieresis : -18 (cslSTIX-Regular)
    0xf70000c1,   // [324:207] N + Aacute : -18 (cslSTIX-Regular)
    0xf9800075,   // [325:208] N + u : -13 (cslSTIX-Regular)
    0xf980006f,   // [326:209] N + o : -13 (cslSTIX-Regular)
    0xfa000065,   // [327:210] N + e : -12 (cslSTIX-Regular)
    0xf7000061,   // [328:211] N + a : -18 (cslSTIX-Regular)
    0xfb00004f,   // [329:212] N + O : -10 (cslSTIX-Regular)
    0xfb000047,   // [330:213] N + G : -10 (cslSTIX-Regular)
    0xfb000043,   // [331:214] N + C : -10 (cslSTIX-Regular)
    0xf7000041,   // [332:215] N + A : -18 (cslSTIX-Regular)
    0xf980002e,   // [333:216] N + period : -13 (cslSTIX-Regular)
    0xfa40002c,   // [334:217] N + comma : -12 (cslSTIX-Regular) ;;
    0xf40000c6,   // [335:218] O + AE : -24 (cslSTIX-Regular)
    0xf38000c5,   // [336:219] O + Aring : -25 (cslSTIX-Regular)
    0xf38000c4,   // [337:220] O + Adieresis : -25 (cslSTIX-Regular)
    0xf38000c1,   // [338:221] O + Aacute : -25 (cslSTIX-Regular)
    0xee000059,   // [339:222] O + Y : -36 (cslSTIX-Regular)
    0xf3800058,   // [340:223] O + X : -25 (cslSTIX-Regular)
    0xf3800057,   // [341:224] O + W : -25 (cslSTIX-Regular)
    0xf1000056,   // [342:225] O + V : -30 (cslSTIX-Regular)
    0xfe800054,   // [343:226] O + T : -3 (cslSTIX-Regular)
    0xf3c00041,   // [344:227] O + A : -25 (cslSTIX-Regular) ;;
    0xf8000153,   // [345:228] P + oe : -16 (cslSTIX-Regular)
    0xf80000f8,   // [346:229] P + oslash : -16 (cslSTIX-Regular)
    0xf80000f6,   // [347:230] P + odieresis : -16 (cslSTIX-Regular)
    0xf80000f3,   // [348:231] P + oacute : -16 (cslSTIX-Regular)
    0xf90000e9,   // [349:232] P + eacute : -14 (cslSTIX-Regular)
    0xfb8000e6,   // [350:233] P + ae : -9 (cslSTIX-Regular)
    0xfb0000e5,   // [351:234] P + aring : -10 (cslSTIX-Regular)
    0xfb0000e4,   // [352:235] P + adieresis : -10 (cslSTIX-Regular)
    0xfb0000e1,   // [353:236] P + aacute : -10 (cslSTIX-Regular)
    0xfb0000e0,   // [354:237] P + agrave : -10 (cslSTIX-Regular)
    0xe38000c6,   // [355:238] P + AE : -57 (cslSTIX-Regular)
    0xe28000c5,   // [356:239] P + Aring : -59 (cslSTIX-Regular)
    0xe28000c4,   // [357:240] P + Adieresis : -59 (cslSTIX-Regular)
    0xe28000c1,   // [358:241] P + Aacute : -59 (cslSTIX-Regular)
    0xf800006f,   // [359:242] P + o : -16 (cslSTIX-Regular)
    0xf9000065,   // [360:243] P + e : -14 (cslSTIX-Regular)
    0xfb000061,   // [361:244] P + a : -10 (cslSTIX-Regular)
    0xed00004a,   // [362:245] P + J : -38 (cslSTIX-Regular)
    0xe2800041,   // [363:246] P + A : -59 (cslSTIX-Regular)
    0xd300002e,   // [364:247] P + period : -90 (cslSTIX-Regular)
    0xf500002d,   // [365:248] P + hyphen : -22 (cslSTIX-Regular)
    0xe240002c,   // [366:249] P + comma : -60 (cslSTIX-Regular) ;;
    0xf5800153,   // [367:250] R + oe : -21 (cslSTIX-Regular)
    0xf4000152,   // [368:251] R + OE : -24 (cslSTIX-Regular)
    0xf90000fc,   // [369:252] R + udieresis : -14 (cslSTIX-Regular)
    0xf90000fa,   // [370:253] R + uacute : -14 (cslSTIX-Regular)
    0xf40000f6,   // [371:254] R + odieresis : -24 (cslSTIX-Regular)
    0xf40000f3,   // [372:255] R + oacute : -24 (cslSTIX-Regular)
    0xf90000e9,   // [373:256] R + eacute : -14 (cslSTIX-Regular)
    0xfd0000e6,   // [374:257] R + ae : -6 (cslSTIX-Regular)
    0xff0000e5,   // [375:258] R + aring : -2 (cslSTIX-Regular)
    0xff0000e4,   // [376:259] R + adieresis : -2 (cslSTIX-Regular)
    0xff0000e1,   // [377:260] R + aacute : -2 (cslSTIX-Regular)
    0xed8000dc,   // [378:261] R + Udieresis : -37 (cslSTIX-Regular)
    0xf30000d6,   // [379:262] R + Odieresis : -26 (cslSTIX-Regular)
    0xf30000d3,   // [380:263] R + Oacute : -26 (cslSTIX-Regular)
    0xf30000c7,   // [381:264] R + Ccedilla : -26 (cslSTIX-Regular)
    0xf0000079,   // [382:265] R + y : -32 (cslSTIX-Regular)
    0xf9000075,   // [383:266] R + u : -14 (cslSTIX-Regular)
    0xf400006f,   // [384:267] R + o : -24 (cslSTIX-Regular)
    0xf9000065,   // [385:268] R + e : -14 (cslSTIX-Regular)
    0xff000061,   // [386:269] R + a : -2 (cslSTIX-Regular)
    0xe2000059,   // [387:270] R + Y : -60 (cslSTIX-Regular)
    0xe2000057,   // [388:271] R + W : -60 (cslSTIX-Regular)
    0xdf800056,   // [389:272] R + V : -65 (cslSTIX-Regular)
    0xed800055,   // [390:273] R + U : -37 (cslSTIX-Regular)
    0xf4000054,   // [391:274] R + T : -24 (cslSTIX-Regular)
    0xf300004f,   // [392:275] R + O : -26 (cslSTIX-Regular)
    0xf3000047,   // [393:276] R + G : -26 (cslSTIX-Regular)
    0xf3000043,   // [394:277] R + C : -26 (cslSTIX-Regular)
    0xea40002d,   // [395:278] R + hyphen : -44 (cslSTIX-Regular) ;;
    0xf28000c6,   // [396:279] S + AE : -27 (cslSTIX-Regular)
    0xf10000c5,   // [397:280] S + Aring : -30 (cslSTIX-Regular)
    0xf10000c4,   // [398:281] S + Adieresis : -30 (cslSTIX-Regular)
    0xf10000c1,   // [399:282] S + Aacute : -30 (cslSTIX-Regular)
    0xfa800074,   // [400:283] S + t : -11 (cslSTIX-Regular)
    0xf2800059,   // [401:284] S + Y : -27 (cslSTIX-Regular)
    0xf6000057,   // [402:285] S + W : -20 (cslSTIX-Regular)
    0xf6000056,   // [403:286] S + V : -20 (cslSTIX-Regular)
    0xf7000054,   // [404:287] S + T : -18 (cslSTIX-Regular)
    0xf1400041,   // [405:288] S + A : -30 (cslSTIX-Regular) ;;
    0xe2002039,   // [406:289] T + guilsinglleft : -60 (cslSTIX-Regular)
    0xfd800152,   // [407:290] T + OE : -5 (cslSTIX-Regular)
    0xf00000f8,   // [408:291] T + oslash : -32 (cslSTIX-Regular)
    0xf80000f6,   // [409:292] T + odieresis : -16 (cslSTIX-Regular)
    0xf88000f4,   // [410:293] T + ocircumflex : -15 (cslSTIX-Regular)
    0xf00000f3,   // [411:294] T + oacute : -32 (cslSTIX-Regular)
    0xf70000f2,   // [412:295] T + ograve : -18 (cslSTIX-Regular)
    0xec0000e6,   // [413:296] T + ae : -40 (cslSTIX-Regular)
    0xfa0000e4,   // [414:297] T + adieresis : -12 (cslSTIX-Regular)
    0xfe8000e3,   // [415:298] T + atilde : -3 (cslSTIX-Regular)
    0xfc8000e2,   // [416:299] T + acircumflex : -7 (cslSTIX-Regular)
    0xf50000e1,   // [417:300] T + aacute : -22 (cslSTIX-Regular)
    0xf88000e0,   // [418:301] T + agrave : -15 (cslSTIX-Regular)
    0xfd8000d8,   // [419:302] T + Oslash : -5 (cslSTIX-Regular)
    0xfd8000d6,   // [420:303] T + Odieresis : -5 (cslSTIX-Regular)
    0xfd8000d5,   // [421:304] T + Otilde : -5 (cslSTIX-Regular)
    0xfd8000d4,   // [422:305] T + Ocircumflex : -5 (cslSTIX-Regular)
    0xfd8000d3,   // [423:306] T + Oacute : -5 (cslSTIX-Regular)
    0xfd8000d2,   // [424:307] T + Ograve : -5 (cslSTIX-Regular)
    0xe90000c6,   // [425:308] T + AE : -46 (cslSTIX-Regular)
    0xe58000c5,   // [426:309] T + Aring : -53 (cslSTIX-Regular)
    0xe58000c4,   // [427:310] T + Adieresis : -53 (cslSTIX-Regular)
    0xe58000c3,   // [428:311] T + Atilde : -53 (cslSTIX-Regular)
    0xe58000c2,   // [429:312] T + Acircumflex : -53 (cslSTIX-Regular)
    0xe58000c1,   // [430:313] T + Aacute : -53 (cslSTIX-Regular)
    0xe58000c0,   // [431:314] T + Agrave : -53 (cslSTIX-Regular)
    0xe50000ab,   // [432:315] T + guillemotleft : -54 (cslSTIX-Regular)
    0xe7800079,   // [433:316] T + y : -49 (cslSTIX-Regular)
    0xe6800077,   // [434:317] T + w : -51 (cslSTIX-Regular)
    0xe6800076,   // [435:318] T + v : -51 (cslSTIX-Regular)
    0xed000075,   // [436:319] T + u : -38 (cslSTIX-Regular)
    0xee000073,   // [437:320] T + s : -36 (cslSTIX-Regular)
    0xf4000072,   // [438:321] T + r : -24 (cslSTIX-Regular)
    0xf000006f,   // [439:322] T + o : -32 (cslSTIX-Regular)
    0xfb00006a,   // [440:323] T + j : -10 (cslSTIX-Regular)
    0xfb800069,   // [441:324] T + i : -9 (cslSTIX-Regular)
    0xea000067,   // [442:325] T + g : -44 (cslSTIX-Regular)
    0xeb800065,   // [443:326] T + e : -41 (cslSTIX-Regular)
    0xeb800063,   // [444:327] T + c : -41 (cslSTIX-Regular)
    0xed000061,   // [445:328] T + a : -38 (cslSTIX-Regular)
    0x05000059,   // [446:329] T + Y : 10 (cslSTIX-Regular)
    0x09800057,   // [447:330] T + W : 19 (cslSTIX-Regular)
    0x07000056,   // [448:331] T + V : 14 (cslSTIX-Regular)
    0xfb800053,   // [449:332] T + S : -9 (cslSTIX-Regular)
    0xfd80004f,   // [450:333] T + O : -5 (cslSTIX-Regular)
    0xfc80004a,   // [451:334] T + J : -7 (cslSTIX-Regular)
    0xff000047,   // [452:335] T + G : -2 (cslSTIX-Regular)
    0xff000043,   // [453:336] T + C : -2 (cslSTIX-Regular)
    0xe5800041,   // [454:337] T + A : -53 (cslSTIX-Regular)
    0xf300003b,   // [455:338] T + semicolon : -26 (cslSTIX-Regular)
    0xf300003a,   // [456:339] T + colon : -26 (cslSTIX-Regular)
    0xe980002e,   // [457:340] T + period : -45 (cslSTIX-Regular)
    0xee00002d,   // [458:341] T + hyphen : -36 (cslSTIX-Regular)
    0xf2c0002c,   // [459:342] T + comma : -27 (cslSTIX-Regular) ;;
    0xec8000c6,   // [460:343] U + AE : -39 (cslSTIX-Regular)
    0xea8000c5,   // [461:344] U + Aring : -43 (cslSTIX-Regular)
    0xea8000c4,   // [462:345] U + Adieresis : -43 (cslSTIX-Regular)
    0xea8000c3,   // [463:346] U + Atilde : -43 (cslSTIX-Regular)
    0xea8000c2,   // [464:347] U + Acircumflex : -43 (cslSTIX-Regular)
    0xea8000c1,   // [465:348] U + Aacute : -43 (cslSTIX-Regular)
    0xfd000072,   // [466:349] U + r : -6 (cslSTIX-Regular)
    0xf7000070,   // [467:350] U + p : -18 (cslSTIX-Regular)
    0xf600006e,   // [468:351] U + n : -20 (cslSTIX-Regular)
    0xf600006d,   // [469:352] U + m : -20 (cslSTIX-Regular)
    0xea800041,   // [470:353] U + A : -43 (cslSTIX-Regular)
    0xf200002e,   // [471:354] U + period : -28 (cslSTIX-Regular)
    0xf0c0002c,   // [472:355] U + comma : -31 (cslSTIX-Regular) ;;
    0xda802039,   // [473:356] V + guilsinglleft : -75 (cslSTIX-Regular)
    0xda8000f8,   // [474:357] V + oslash : -75 (cslSTIX-Regular)
    0xe78000f6,   // [475:358] V + odieresis : -49 (cslSTIX-Regular)
    0xed8000f5,   // [476:359] V + otilde : -37 (cslSTIX-Regular)
    0xe28000f4,   // [477:360] V + ocircumflex : -59 (cslSTIX-Regular)
    0xda8000f3,   // [478:361] V + oacute : -75 (cslSTIX-Regular)
    0xf28000f2,   // [479:362] V + ograve : -27 (cslSTIX-Regular)
    0xdd0000e9,   // [480:363] V + eacute : -70 (cslSTIX-Regular)
    0xf70000e8,   // [481:364] V + egrave : -18 (cslSTIX-Regular)
    0xd80000e6,   // [482:365] V + ae : -80 (cslSTIX-Regular)
    0xed0000e5,   // [483:366] V + aring : -38 (cslSTIX-Regular)
    0xed0000e4,   // [484:367] V + adieresis : -38 (cslSTIX-Regular)
    0xf10000e3,   // [485:368] V + atilde : -30 (cslSTIX-Regular)
    0xe70000e2,   // [486:369] V + acircumflex : -50 (cslSTIX-Regular)
    0xd80000e1,   // [487:370] V + aacute : -80 (cslSTIX-Regular)
    0xf70000e0,   // [488:371] V + agrave : -18 (cslSTIX-Regular)
    0xed0000d8,   // [489:372] V + Oslash : -38 (cslSTIX-Regular)
    0xed0000d6,   // [490:373] V + Odieresis : -38 (cslSTIX-Regular)
    0xed0000d5,   // [491:374] V + Otilde : -38 (cslSTIX-Regular)
    0xed0000d4,   // [492:375] V + Ocircumflex : -38 (cslSTIX-Regular)
    0xed0000d3,   // [493:376] V + Oacute : -38 (cslSTIX-Regular)
    0xed0000d2,   // [494:377] V + Ograve : -38 (cslSTIX-Regular)
    0xdb8000c6,   // [495:378] V + AE : -73 (cslSTIX-Regular)
    0xd60000c5,   // [496:379] V + Aring : -84 (cslSTIX-Regular)
    0xd60000c4,   // [497:380] V + Adieresis : -84 (cslSTIX-Regular)
    0xd60000c3,   // [498:381] V + Atilde : -84 (cslSTIX-Regular)
    0xd60000c2,   // [499:382] V + Acircumflex : -84 (cslSTIX-Regular)
    0xd60000c1,   // [500:383] V + Aacute : -84 (cslSTIX-Regular)
    0xd60000c0,   // [501:384] V + Agrave : -84 (cslSTIX-Regular)
    0xda8000ab,   // [502:385] V + guillemotleft : -75 (cslSTIX-Regular)
    0xe7000079,   // [503:386] V + y : -50 (cslSTIX-Regular)
    0xe7000075,   // [504:387] V + u : -50 (cslSTIX-Regular)
    0xec000072,   // [505:388] V + r : -40 (cslSTIX-Regular)
    0xda80006f,   // [506:389] V + o : -75 (cslSTIX-Regular)
    0xf6000069,   // [507:390] V + i : -20 (cslSTIX-Regular)
    0xdf800067,   // [508:391] V + g : -65 (cslSTIX-Regular)
    0xdd000065,   // [509:392] V + e : -70 (cslSTIX-Regular)
    0xd8000061,   // [510:393] V + a : -80 (cslSTIX-Regular)
    0x04800054,   // [511:394] V + T : 9 (cslSTIX-Regular)
    0xee800053,   // [512:395] V + S : -35 (cslSTIX-Regular)
    0xed00004f,   // [513:396] V + O : -38 (cslSTIX-Regular)
    0xed000047,   // [514:397] V + G : -38 (cslSTIX-Regular)
    0xed000043,   // [515:398] V + C : -38 (cslSTIX-Regular)
    0xd6000041,   // [516:399] V + A : -84 (cslSTIX-Regular)
    0xe280003b,   // [517:400] V + semicolon : -59 (cslSTIX-Regular)
    0xe200003a,   // [518:401] V + colon : -60 (cslSTIX-Regular)
    0xce00002e,   // [519:402] V + period : -100 (cslSTIX-Regular)
    0xe200002d,   // [520:403] V + hyphen : -60 (cslSTIX-Regular)
    0xce40002c,   // [521:404] V + comma : -100 (cslSTIX-Regular) ;;
    0xe1802039,   // [522:405] W + guilsinglleft : -61 (cslSTIX-Regular)
    0xdd0000f8,   // [523:406] W + oslash : -70 (cslSTIX-Regular)
    0xe48000f6,   // [524:407] W + odieresis : -55 (cslSTIX-Regular)
    0xec0000f5,   // [525:408] W + otilde : -40 (cslSTIX-Regular)
    0xe08000f4,   // [526:409] W + ocircumflex : -63 (cslSTIX-Regular)
    0xdd0000f3,   // [527:410] W + oacute : -70 (cslSTIX-Regular)
    0xf20000f2,   // [528:411] W + ograve : -28 (cslSTIX-Regular)
    0xe20000e9,   // [529:412] W + eacute : -60 (cslSTIX-Regular)
    0xfb8000e8,   // [530:413] W + egrave : -9 (cslSTIX-Regular)
    0xe20000e6,   // [531:414] W + ae : -60 (cslSTIX-Regular)
    0xf28000e5,   // [532:415] W + aring : -27 (cslSTIX-Regular)
    0xef0000e4,   // [533:416] W + adieresis : -34 (cslSTIX-Regular)
    0xf40000e3,   // [534:417] W + atilde : -24 (cslSTIX-Regular)
    0xef0000e2,   // [535:418] W + acircumflex : -34 (cslSTIX-Regular)
    0xe70000e1,   // [536:419] W + aacute : -50 (cslSTIX-Regular)
    0xf70000e0,   // [537:420] W + agrave : -18 (cslSTIX-Regular)
    0xef8000d8,   // [538:421] W + Oslash : -33 (cslSTIX-Regular)
    0xef8000d6,   // [539:422] W + Odieresis : -33 (cslSTIX-Regular)
    0xef8000d5,   // [540:423] W + Otilde : -33 (cslSTIX-Regular)
    0xef8000d4,   // [541:424] W + Ocircumflex : -33 (cslSTIX-Regular)
    0xef8000d3,   // [542:425] W + Oacute : -33 (cslSTIX-Regular)
    0xef8000d2,   // [543:426] W + Ograve : -33 (cslSTIX-Regular)
    0xd40000c6,   // [544:427] W + AE : -88 (cslSTIX-Regular)
    0xcd0000c5,   // [545:428] W + Aring : -102 (cslSTIX-Regular)
    0xcd0000c4,   // [546:429] W + Adieresis : -102 (cslSTIX-Regular)
    0xcd0000c3,   // [547:430] W + Atilde : -102 (cslSTIX-Regular)
    0xcd0000c2,   // [548:431] W + Acircumflex : -102 (cslSTIX-Regular)
    0xcd0000c1,   // [549:432] W + Aacute : -102 (cslSTIX-Regular)
    0xcd0000c0,   // [550:433] W + Agrave : -102 (cslSTIX-Regular)
    0xe18000ab,   // [551:434] W + guillemotleft : -61 (cslSTIX-Regular)
    0xe9800079,   // [552:435] W + y : -45 (cslSTIX-Regular)
    0xf1000075,   // [553:436] W + u : -30 (cslSTIX-Regular)
    0xf1000072,   // [554:437] W + r : -30 (cslSTIX-Regular)
    0xdd00006f,   // [555:438] W + o : -70 (cslSTIX-Regular)
    0xf9800069,   // [556:439] W + i : -13 (cslSTIX-Regular)
    0xe3800067,   // [557:440] W + g : -57 (cslSTIX-Regular)
    0xe2000065,   // [558:441] W + e : -60 (cslSTIX-Regular)
    0xe2000061,   // [559:442] W + a : -60 (cslSTIX-Regular)
    0x08000054,   // [560:443] W + T : 16 (cslSTIX-Regular)
    0xf4000053,   // [561:444] W + S : -24 (cslSTIX-Regular)
    0xef80004f,   // [562:445] W + O : -33 (cslSTIX-Regular)
    0xed000047,   // [563:446] W + G : -38 (cslSTIX-Regular)
    0xef800043,   // [564:447] W + C : -33 (cslSTIX-Regular)
    0xcd000041,   // [565:448] W + A : -102 (cslSTIX-Regular)
    0xe880003b,   // [566:449] W + semicolon : -47 (cslSTIX-Regular)
    0xef80003a,   // [567:450] W + colon : -33 (cslSTIX-Regular)
    0xd200002e,   // [568:451] W + period : -92 (cslSTIX-Regular)
    0xec00002d,   // [569:452] W + hyphen : -40 (cslSTIX-Regular)
    0xd840002c,   // [570:453] W + comma : -80 (cslSTIX-Regular) ;;
    0xed0000d6,   // [571:454] X + Odieresis : -38 (cslSTIX-Regular)
    0xe0800079,   // [572:455] X + y : -63 (cslSTIX-Regular)
    0xf7000075,   // [573:456] X + u : -18 (cslSTIX-Regular)
    0xf280006f,   // [574:457] X + o : -27 (cslSTIX-Regular)
    0xf1800065,   // [575:458] X + e : -29 (cslSTIX-Regular)
    0xfc800061,   // [576:459] X + a : -7 (cslSTIX-Regular)
    0xed000051,   // [577:460] X + Q : -38 (cslSTIX-Regular)
    0xed00004f,   // [578:461] X + O : -38 (cslSTIX-Regular)
    0xee800043,   // [579:462] X + C : -35 (cslSTIX-Regular)
    0xe9c0002d,   // [580:463] X + hyphen : -45 (cslSTIX-Regular) ;;
    0xbf002039,   // [581:464] Y + guilsinglleft : -130 (cslSTIX-Regular)
    0xde8000f8,   // [582:465] Y + oslash : -67 (cslSTIX-Regular)
    0xd58000e6,   // [583:466] Y + ae : -85 (cslSTIX-Regular)
    0xe68000d8,   // [584:467] Y + Oslash : -51 (cslSTIX-Regular)
    0xe68000d6,   // [585:468] Y + Odieresis : -51 (cslSTIX-Regular)
    0xe68000d5,   // [586:469] Y + Otilde : -51 (cslSTIX-Regular)
    0xe68000d4,   // [587:470] Y + Ocircumflex : -51 (cslSTIX-Regular)
    0xe68000d3,   // [588:471] Y + Oacute : -51 (cslSTIX-Regular)
    0xe68000d2,   // [589:472] Y + Ograve : -51 (cslSTIX-Regular)
    0xde8000c6,   // [590:473] Y + AE : -67 (cslSTIX-Regular)
    0xda8000c5,   // [591:474] Y + Aring : -75 (cslSTIX-Regular)
    0xda8000c4,   // [592:475] Y + Adieresis : -75 (cslSTIX-Regular)
    0xda8000c3,   // [593:476] Y + Atilde : -75 (cslSTIX-Regular)
    0xda8000c2,   // [594:477] Y + Acircumflex : -75 (cslSTIX-Regular)
    0xda8000c1,   // [595:478] Y + Aacute : -75 (cslSTIX-Regular)
    0xda8000c0,   // [596:479] Y + Agrave : -75 (cslSTIX-Regular)
    0xc58000ab,   // [597:480] Y + guillemotleft : -117 (cslSTIX-Regular)
    0xe1000076,   // [598:481] Y + v : -62 (cslSTIX-Regular)
    0xe2800075,   // [599:482] Y + u : -59 (cslSTIX-Regular)
    0xe3800070,   // [600:483] Y + p : -57 (cslSTIX-Regular)
    0xde80006f,   // [601:484] Y + o : -67 (cslSTIX-Regular)
    0xf7800069,   // [602:485] Y + i : -17 (cslSTIX-Regular)
    0xdb800067,   // [603:486] Y + g : -73 (cslSTIX-Regular)
    0xdf000065,   // [604:487] Y + e : -66 (cslSTIX-Regular)
    0xd8000061,   // [605:488] Y + a : -80 (cslSTIX-Regular)
    0x06000054,   // [606:489] Y + T : 12 (cslSTIX-Regular)
    0xea800053,   // [607:490] Y + S : -43 (cslSTIX-Regular)
    0xe680004f,   // [608:491] Y + O : -51 (cslSTIX-Regular)
    0xe6800047,   // [609:492] Y + G : -51 (cslSTIX-Regular)
    0xe7000043,   // [610:493] Y + C : -50 (cslSTIX-Regular)
    0xda800041,   // [611:494] Y + A : -75 (cslSTIX-Regular)
    0xe180003b,   // [612:495] Y + semicolon : -61 (cslSTIX-Regular)
    0xdf00003a,   // [613:496] Y + colon : -66 (cslSTIX-Regular)
    0xd180002e,   // [614:497] Y + period : -93 (cslSTIX-Regular)
    0xd800002d,   // [615:498] Y + hyphen : -80 (cslSTIX-Regular)
    0xdac0002c,   // [616:499] Y + comma : -75 (cslSTIX-Regular) ;;
    0xef800079,   // [617:500] Z + y : -33 (cslSTIX-Regular)
    0xef000077,   // [618:501] Z + w : -34 (cslSTIX-Regular)
    0xefc00076,   // [619:502] Z + v : -33 (cslSTIX-Regular) ;;
    0xeb802019,   // [620:503] a + quoteright : -41 (cslSTIX-Regular)
    0xf2800079,   // [621:504] a + y : -27 (cslSTIX-Regular)
    0xef800077,   // [622:505] a + w : -33 (cslSTIX-Regular)
    0xf0000076,   // [623:506] a + v : -32 (cslSTIX-Regular)
    0xfc40006a,   // [624:507] a + j : -8 (cslSTIX-Regular) ;;
    0xf6000079,   // [625:508] b + y : -20 (cslSTIX-Regular)
    0xf7000077,   // [626:509] b + w : -18 (cslSTIX-Regular)
    0xf7400076,   // [627:510] b + v : -18 (cslSTIX-Regular) ;;
    0xf500006b,   // [628:511] c + k : -22 (cslSTIX-Regular)
    0xf8400068,   // [629:512] c + h : -16 (cslSTIX-Regular) ;;
    0xf6002019,   // [630:513] e + quoteright : -20 (cslSTIX-Regular)
    0xf5800079,   // [631:514] e + y : -21 (cslSTIX-Regular)
    0xf5800078,   // [632:515] e + x : -21 (cslSTIX-Regular)
    0xf6800077,   // [633:516] e + w : -19 (cslSTIX-Regular)
    0xf6800076,   // [634:517] e + v : -19 (cslSTIX-Regular)
    0xfbc00074,   // [635:518] e + t : -9 (cslSTIX-Regular) ;;
    0x08002019,   // [636:519] f + quoteright : 16 (cslSTIX-Regular)
    0xee000153,   // [637:520] f + oe : -36 (cslSTIX-Regular)
    0xed8000f8,   // [638:521] f + oslash : -37 (cslSTIX-Regular)
    0x008000f6,   // [639:522] f + odieresis : 1 (cslSTIX-Regular)
    0xf40000f4,   // [640:523] f + ocircumflex : -24 (cslSTIX-Regular)
    0xf00000f3,   // [641:524] f + oacute : -32 (cslSTIX-Regular)
    0xff8000f2,   // [642:525] f + ograve : -1 (cslSTIX-Regular)
    0xf90000ea,   // [643:526] f + ecircumflex : -14 (cslSTIX-Regular)
    0xef0000e9,   // [644:527] f + eacute : -34 (cslSTIX-Regular)
    0x048000e8,   // [645:528] f + egrave : 9 (cslSTIX-Regular)
    0xf28000e6,   // [646:529] f + ae : -27 (cslSTIX-Regular)
    0xfc0000e5,   // [647:530] f + aring : -8 (cslSTIX-Regular)
    0x058000e4,   // [648:531] f + adieresis : 11 (cslSTIX-Regular)
    0xf30000e1,   // [649:532] f + aacute : -26 (cslSTIX-Regular)
    0x05000074,   // [650:533] f + t : 10 (cslSTIX-Regular)
    0xf8000073,   // [651:534] f + s : -16 (cslSTIX-Regular)
    0xf000006f,   // [652:535] f + o : -32 (cslSTIX-Regular)
    0x1600006c,   // [653:536] f + l : 44 (cslSTIX-Regular)
    0x0380006a,   // [654:537] f + j : 7 (cslSTIX-Regular)
    0x07000069,   // [655:538] f + i : 14 (cslSTIX-Regular)
    0x03000066,   // [656:539] f + f : 6 (cslSTIX-Regular)
    0xef000065,   // [657:540] f + e : -34 (cslSTIX-Regular)
    0xf3000061,   // [658:541] f + a : -26 (cslSTIX-Regular)
    0x1120006a,   // [659:542] f + j ligature #34 (cslSTIX-Regular)
    0x11a0006c,   // [660:543] f + l ligature #35 (cslSTIX-Regular)
    0x12200069,   // [661:544] f + i ligature #36 (cslSTIX-Regular)
    0x12e00066,   // [662:545] f + f ligature #37 (cslSTIX-Regular) ;;
    0xf80000f6,   // [663:546] g + odieresis : -16 (cslSTIX-Regular)
    0xf80000f3,   // [664:547] g + oacute : -16 (cslSTIX-Regular)
    0xf88000eb,   // [665:548] g + edieresis : -15 (cslSTIX-Regular)
    0xf88000ea,   // [666:549] g + ecircumflex : -15 (cslSTIX-Regular)
    0xf88000e9,   // [667:550] g + eacute : -15 (cslSTIX-Regular)
    0xf88000e8,   // [668:551] g + egrave : -15 (cslSTIX-Regular)
    0xfa8000e6,   // [669:552] g + ae : -11 (cslSTIX-Regular)
    0xfa8000e5,   // [670:553] g + aring : -11 (cslSTIX-Regular)
    0xfa8000e4,   // [671:554] g + adieresis : -11 (cslSTIX-Regular)
    0x05000072,   // [672:555] g + r : 10 (cslSTIX-Regular)
    0xff00006c,   // [673:556] g + l : -2 (cslSTIX-Regular)
    0xf8800065,   // [674:557] g + e : -15 (cslSTIX-Regular)
    0xfac00061,   // [675:558] g + a : -11 (cslSTIX-Regular) ;;
    0xec802019,   // [676:559] h + quoteright : -39 (cslSTIX-Regular)
    0xf6400079,   // [677:560] h + y : -20 (cslSTIX-Regular) ;;
    0xef00006a,   // [678:561] i + j : -34 (cslSTIX-Regular)
    0xf7000054,   // [679:562] i + T : -18 (cslSTIX-Regular)
    0x1360006a,   // [680:563] i + j ligature #38 (cslSTIX-Regular) ;;
    0x070000fc,   // [681:564] k + udieresis : 14 (cslSTIX-Regular)
    0xf58000f6,   // [682:565] k + odieresis : -21 (cslSTIX-Regular)
    0xf58000f3,   // [683:566] k + oacute : -21 (cslSTIX-Regular)
    0xf98000e9,   // [684:567] k + eacute : -13 (cslSTIX-Regular)
    0xf98000e8,   // [685:568] k + egrave : -13 (cslSTIX-Regular)
    0xff0000e6,   // [686:569] k + ae : -2 (cslSTIX-Regular)
    0x008000e5,   // [687:570] k + aring : 1 (cslSTIX-Regular)
    0x008000e4,   // [688:571] k + adieresis : 1 (cslSTIX-Regular)
    0x008000e1,   // [689:572] k + aacute : 1 (cslSTIX-Regular)
    0x07000075,   // [690:573] k + u : 14 (cslSTIX-Regular)
    0x02800073,   // [691:574] k + s : 5 (cslSTIX-Regular)
    0xf580006f,   // [692:575] k + o : -21 (cslSTIX-Regular)
    0xfc000067,   // [693:576] k + g : -8 (cslSTIX-Regular)
    0xf9800065,   // [694:577] k + e : -13 (cslSTIX-Regular)
    0x00800061,   // [695:578] k + a : 1 (cslSTIX-Regular)
    0xff80002e,   // [696:579] k + period : -1 (cslSTIX-Regular)
    0xe780002d,   // [697:580] k + hyphen : -49 (cslSTIX-Regular)
    0x02c0002c,   // [698:581] k + comma : 5 (cslSTIX-Regular) ;;
    0xf5800079,   // [699:582] l + y : -21 (cslSTIX-Regular)
    0xf5c00076,   // [700:583] l + v : -21 (cslSTIX-Regular) ;;
    0xf6000079,   // [701:584] m + y : -20 (cslSTIX-Regular)
    0xf6000077,   // [702:585] m + w : -20 (cslSTIX-Regular)
    0xf6000076,   // [703:586] m + v : -20 (cslSTIX-Regular)
    0xfec00070,   // [704:587] m + p : -3 (cslSTIX-Regular) ;;
    0xec002019,   // [705:588] n + quoteright : -40 (cslSTIX-Regular)
    0xf4000079,   // [706:589] n + y : -24 (cslSTIX-Regular)
    0xf0000077,   // [707:590] n + w : -32 (cslSTIX-Regular)
    0xf0800076,   // [708:591] n + v : -31 (cslSTIX-Regular)
    0xff800070,   // [709:592] n + p : -1 (cslSTIX-Regular)
    0xee400054,   // [710:593] n + T : -36 (cslSTIX-Regular) ;;
    0xee002019,   // [711:594] o + quoteright : -36 (cslSTIX-Regular)
    0xf5000079,   // [712:595] o + y : -22 (cslSTIX-Regular)
    0xf5000078,   // [713:596] o + x : -22 (cslSTIX-Regular)
    0xf5000077,   // [714:597] o + w : -22 (cslSTIX-Regular)
    0xf5000076,   // [715:598] o + v : -22 (cslSTIX-Regular)
    0xfc000074,   // [716:599] o + t : -8 (cslSTIX-Regular)
    0xd4c00054,   // [717:600] o + T : -87 (cslSTIX-Regular) ;;
    0xfc800079,   // [718:601] p + y : -7 (cslSTIX-Regular)
    0xffc00074,   // [719:602] p + t : -1 (cslSTIX-Regular) ;;
    0xf9800075,   // [720:603] q + u : -13 (cslSTIX-Regular)
    0xfec00063,   // [721:604] q + c : -3 (cslSTIX-Regular) ;;
    0xf7802019,   // [722:605] r + quoteright : -17 (cslSTIX-Regular)
    0xfd000153,   // [723:606] r + oe : -6 (cslSTIX-Regular)
    0xfd0000f8,   // [724:607] r + oslash : -6 (cslSTIX-Regular)
    0xfd0000f6,   // [725:608] r + odieresis : -6 (cslSTIX-Regular)
    0xfd0000f4,   // [726:609] r + ocircumflex : -6 (cslSTIX-Regular)
    0xfd0000f3,   // [727:610] r + oacute : -6 (cslSTIX-Regular)
    0xfd0000f2,   // [728:611] r + ograve : -6 (cslSTIX-Regular)
    0xff0000ea,   // [729:612] r + ecircumflex : -2 (cslSTIX-Regular)
    0xff0000e9,   // [730:613] r + eacute : -2 (cslSTIX-Regular)
    0xff0000e8,   // [731:614] r + egrave : -2 (cslSTIX-Regular)
    0xff0000e7,   // [732:615] r + ccedilla : -2 (cslSTIX-Regular)
    0xfe8000e6,   // [733:616] r + ae : -3 (cslSTIX-Regular)
    0xff0000e5,   // [734:617] r + aring : -2 (cslSTIX-Regular)
    0xff0000e4,   // [735:618] r + adieresis : -2 (cslSTIX-Regular)
    0xff0000e2,   // [736:619] r + acircumflex : -2 (cslSTIX-Regular)
    0xff0000e1,   // [737:620] r + aacute : -2 (cslSTIX-Regular)
    0xff0000e0,   // [738:621] r + agrave : -2 (cslSTIX-Regular)
    0x0100007a,   // [739:622] r + z : 2 (cslSTIX-Regular)
    0x0a800079,   // [740:623] r + y : 21 (cslSTIX-Regular)
    0x08000078,   // [741:624] r + x : 16 (cslSTIX-Regular)
    0x09800077,   // [742:625] r + w : 19 (cslSTIX-Regular)
    0x0a000076,   // [743:626] r + v : 20 (cslSTIX-Regular)
    0x0c000075,   // [744:627] r + u : 24 (cslSTIX-Regular)
    0x0b800074,   // [745:628] r + t : 23 (cslSTIX-Regular)
    0x0d000072,   // [746:629] r + r : 26 (cslSTIX-Regular)
    0xfb000071,   // [747:630] r + q : -10 (cslSTIX-Regular)
    0x0c800070,   // [748:631] r + p : 25 (cslSTIX-Regular)
    0xfd00006f,   // [749:632] r + o : -6 (cslSTIX-Regular)
    0x0a80006e,   // [750:633] r + n : 21 (cslSTIX-Regular)
    0x0a00006d,   // [751:634] r + m : 20 (cslSTIX-Regular)
    0xfc80006c,   // [752:635] r + l : -7 (cslSTIX-Regular)
    0xfc80006b,   // [753:636] r + k : -7 (cslSTIX-Regular)
    0x0680006a,   // [754:637] r + j : 13 (cslSTIX-Regular)
    0x09800069,   // [755:638] r + i : 19 (cslSTIX-Regular)
    0xff000068,   // [756:639] r + h : -2 (cslSTIX-Regular)
    0xf8000067,   // [757:640] r + g : -16 (cslSTIX-Regular)
    0x09800066,   // [758:641] r + f : 19 (cslSTIX-Regular)
    0xff000065,   // [759:642] r + e : -2 (cslSTIX-Regular)
    0xff000064,   // [760:643] r + d : -2 (cslSTIX-Regular)
    0xff000063,   // [761:644] r + c : -2 (cslSTIX-Regular)
    0xff000061,   // [762:645] r + a : -2 (cslSTIX-Regular)
    0xfc80003b,   // [763:646] r + semicolon : -7 (cslSTIX-Regular)
    0xfc00003a,   // [764:647] r + colon : -8 (cslSTIX-Regular)
    0xe800002e,   // [765:648] r + period : -48 (cslSTIX-Regular)
    0xf180002d,   // [766:649] r + hyphen : -29 (cslSTIX-Regular)
    0xf340002c,   // [767:650] r + comma : -26 (cslSTIX-Regular) ;;
    0xee802019,   // [768:651] s + quoteright : -35 (cslSTIX-Regular)
    0xfc400074,   // [769:652] s + t : -8 (cslSTIX-Regular) ;;
    0xf1002019,   // [770:653] t + quoteright : -30 (cslSTIX-Regular)
    0xff0000f6,   // [771:654] t + odieresis : -2 (cslSTIX-Regular)
    0xff0000f3,   // [772:655] t + oacute : -2 (cslSTIX-Regular)
    0x040000e6,   // [773:656] t + ae : 8 (cslSTIX-Regular)
    0x050000e5,   // [774:657] t + aring : 10 (cslSTIX-Regular)
    0x050000e4,   // [775:658] t + adieresis : 10 (cslSTIX-Regular)
    0x050000e1,   // [776:659] t + aacute : 10 (cslSTIX-Regular)
    0xff00006f,   // [777:660] t + o : -2 (cslSTIX-Regular)
    0x04000068,   // [778:661] t + h : 8 (cslSTIX-Regular)
    0x05000061,   // [779:662] t + a : 10 (cslSTIX-Regular)
    0x01000053,   // [780:663] t + S : 2 (cslSTIX-Regular)
    0x0300003b,   // [781:664] t + semicolon : 6 (cslSTIX-Regular)
    0x0140003a,   // [782:665] t + colon : 2 (cslSTIX-Regular) ;;
    0xedc02019,   // [783:666] u + quoteright : -37 (cslSTIX-Regular) ;;
    0xf28000f8,   // [784:667] v + oslash : -27 (cslSTIX-Regular)
    0xf28000f6,   // [785:668] v + odieresis : -27 (cslSTIX-Regular)
    0xf28000f3,   // [786:669] v + oacute : -27 (cslSTIX-Regular)
    0xf28000f2,   // [787:670] v + ograve : -27 (cslSTIX-Regular)
    0xf40000ea,   // [788:671] v + ecircumflex : -24 (cslSTIX-Regular)
    0xf40000e9,   // [789:672] v + eacute : -24 (cslSTIX-Regular)
    0xf40000e8,   // [790:673] v + egrave : -24 (cslSTIX-Regular)
    0xf60000e6,   // [791:674] v + ae : -20 (cslSTIX-Regular)
    0xf60000e5,   // [792:675] v + aring : -20 (cslSTIX-Regular)
    0xf60000e4,   // [793:676] v + adieresis : -20 (cslSTIX-Regular)
    0xf60000e3,   // [794:677] v + atilde : -20 (cslSTIX-Regular)
    0xf60000e2,   // [795:678] v + acircumflex : -20 (cslSTIX-Regular)
    0xf60000e1,   // [796:679] v + aacute : -20 (cslSTIX-Regular)
    0xf60000e0,   // [797:680] v + agrave : -20 (cslSTIX-Regular)
    0xf9800073,   // [798:681] v + s : -13 (cslSTIX-Regular)
    0xf280006f,   // [799:682] v + o : -27 (cslSTIX-Regular)
    0xf080006c,   // [800:683] v + l : -31 (cslSTIX-Regular)
    0xeb000067,   // [801:684] v + g : -42 (cslSTIX-Regular)
    0xf4000065,   // [802:685] v + e : -24 (cslSTIX-Regular)
    0xf2000063,   // [803:686] v + c : -28 (cslSTIX-Regular)
    0xf6000061,   // [804:687] v + a : -20 (cslSTIX-Regular)
    0xf780003b,   // [805:688] v + semicolon : -17 (cslSTIX-Regular)
    0xf580003a,   // [806:689] v + colon : -21 (cslSTIX-Regular)
    0xdb00002e,   // [807:690] v + period : -74 (cslSTIX-Regular)
    0xf200002d,   // [808:691] v + hyphen : -28 (cslSTIX-Regular)
    0xdf40002c,   // [809:692] v + comma : -66 (cslSTIX-Regular) ;;
    0xf30000f8,   // [810:693] w + oslash : -26 (cslSTIX-Regular)
    0xf30000f6,   // [811:694] w + odieresis : -26 (cslSTIX-Regular)
    0xf30000f3,   // [812:695] w + oacute : -26 (cslSTIX-Regular)
    0xf30000f2,   // [813:696] w + ograve : -26 (cslSTIX-Regular)
    0xf70000ea,   // [814:697] w + ecircumflex : -18 (cslSTIX-Regular)
    0xf70000e9,   // [815:698] w + eacute : -18 (cslSTIX-Regular)
    0xf70000e8,   // [816:699] w + egrave : -18 (cslSTIX-Regular)
    0xf18000e6,   // [817:700] w + ae : -29 (cslSTIX-Regular)
    0xf20000e5,   // [818:701] w + aring : -28 (cslSTIX-Regular)
    0xf20000e4,   // [819:702] w + adieresis : -28 (cslSTIX-Regular)
    0xf20000e3,   // [820:703] w + atilde : -28 (cslSTIX-Regular)
    0xf20000e2,   // [821:704] w + acircumflex : -28 (cslSTIX-Regular)
    0xf20000e1,   // [822:705] w + aacute : -28 (cslSTIX-Regular)
    0xf20000e0,   // [823:706] w + agrave : -28 (cslSTIX-Regular)
    0xf4800073,   // [824:707] w + s : -23 (cslSTIX-Regular)
    0xf300006f,   // [825:708] w + o : -26 (cslSTIX-Regular)
    0xef00006c,   // [826:709] w + l : -34 (cslSTIX-Regular)
    0xef800067,   // [827:710] w + g : -33 (cslSTIX-Regular)
    0xf7000065,   // [828:711] w + e : -18 (cslSTIX-Regular)
    0xf5800063,   // [829:712] w + c : -21 (cslSTIX-Regular)
    0xf2000061,   // [830:713] w + a : -28 (cslSTIX-Regular)
    0xfa00003b,   // [831:714] w + semicolon : -12 (cslSTIX-Regular)
    0xfa00003a,   // [832:715] w + colon : -12 (cslSTIX-Regular)
    0xde00002e,   // [833:716] w + period : -68 (cslSTIX-Regular)
    0xf400002d,   // [834:717] w + hyphen : -24 (cslSTIX-Regular)
    0xe740002c,   // [835:718] w + comma : -50 (cslSTIX-Regular) ;;
    0xf40000e9,   // [836:719] x + eacute : -24 (cslSTIX-Regular)
    0xf7800071,   // [837:720] x + q : -17 (cslSTIX-Regular)
    0xf180006f,   // [838:721] x + o : -29 (cslSTIX-Regular)
    0xf4000065,   // [839:722] x + e : -24 (cslSTIX-Regular)
    0xf4000063,   // [840:723] x + c : -24 (cslSTIX-Regular)
    0xfac00061,   // [841:724] x + a : -11 (cslSTIX-Regular) ;;
    0xf70000f8,   // [842:725] y + oslash : -18 (cslSTIX-Regular)
    0xf70000f6,   // [843:726] y + odieresis : -18 (cslSTIX-Regular)
    0xf70000f3,   // [844:727] y + oacute : -18 (cslSTIX-Regular)
    0xf70000f2,   // [845:728] y + ograve : -18 (cslSTIX-Regular)
    0xf78000ea,   // [846:729] y + ecircumflex : -17 (cslSTIX-Regular)
    0xf78000e9,   // [847:730] y + eacute : -17 (cslSTIX-Regular)
    0xf78000e8,   // [848:731] y + egrave : -17 (cslSTIX-Regular)
    0xf88000e6,   // [849:732] y + ae : -15 (cslSTIX-Regular)
    0xf88000e5,   // [850:733] y + aring : -15 (cslSTIX-Regular)
    0xf88000e4,   // [851:734] y + adieresis : -15 (cslSTIX-Regular)
    0xf88000e3,   // [852:735] y + atilde : -15 (cslSTIX-Regular)
    0xf88000e2,   // [853:736] y + acircumflex : -15 (cslSTIX-Regular)
    0xf88000e1,   // [854:737] y + aacute : -15 (cslSTIX-Regular)
    0xf88000e0,   // [855:738] y + agrave : -15 (cslSTIX-Regular)
    0xf9800073,   // [856:739] y + s : -13 (cslSTIX-Regular)
    0xf700006f,   // [857:740] y + o : -18 (cslSTIX-Regular)
    0xf700006c,   // [858:741] y + l : -18 (cslSTIX-Regular)
    0xf5000067,   // [859:742] y + g : -22 (cslSTIX-Regular)
    0xf4800065,   // [860:743] y + e : -23 (cslSTIX-Regular)
    0xf8800063,   // [861:744] y + c : -15 (cslSTIX-Regular)
    0xf8800061,   // [862:745] y + a : -15 (cslSTIX-Regular)
    0xf900003b,   // [863:746] y + semicolon : -14 (cslSTIX-Regular)
    0xfc00003a,   // [864:747] y + colon : -8 (cslSTIX-Regular)
    0xdc80002e,   // [865:748] y + period : -71 (cslSTIX-Regular)
    0xf700002d,   // [866:749] y + hyphen : -18 (cslSTIX-Regular)
    0xe5c0002c,   // [867:750] y + comma : -53 (cslSTIX-Regular) ;;
    0xe10000c6,   // [868:751] guillemotright + AE : -62 (cslSTIX-Regular)
    0xdf8000c5,   // [869:752] guillemotright + Aring : -65 (cslSTIX-Regular)
    0xdf8000c4,   // [870:753] guillemotright + Adieresis : -65 (cslSTIX-Regular)
    0xdf8000c1,   // [871:754] guillemotright + Aacute : -65 (cslSTIX-Regular)
    0xbb000059,   // [872:755] guillemotright + Y : -138 (cslSTIX-Regular)
    0xd1000057,   // [873:756] guillemotright + W : -94 (cslSTIX-Regular)
    0xc6000056,   // [874:757] guillemotright + V : -116 (cslSTIX-Regular)
    0xc7800054,   // [875:758] guillemotright + T : -113 (cslSTIX-Regular)
    0xdfc00041,   // [876:759] guillemotright + A : -65 (cslSTIX-Regular) ;;
    0xe6000059,   // [877:760] Agrave + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [878:761] Agrave + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [879:762] Agrave + V : -83 (cslSTIX-Regular)
    0xee800055,   // [880:763] Agrave + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [881:764] Agrave + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [882:765] Agrave + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [883:766] Agrave + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [884:767] Agrave + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [885:768] Agrave + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [886:769] Agrave + period : -5 (cslSTIX-Regular)
    0xff40002c,   // [887:770] Agrave + comma : -2 (cslSTIX-Regular) ;;
    0xe7802039,   // [888:771] Aacute + guilsinglleft : -49 (cslSTIX-Regular)
    0xe380201d,   // [889:772] Aacute + quotedblright : -57 (cslSTIX-Regular)
    0xdb002019,   // [890:773] Aacute + quoteright : -74 (cslSTIX-Regular)
    0xeb0000ab,   // [891:774] Aacute + guillemotleft : -42 (cslSTIX-Regular)
    0xdc000079,   // [892:775] Aacute + y : -72 (cslSTIX-Regular)
    0xe3000077,   // [893:776] Aacute + w : -58 (cslSTIX-Regular)
    0xe3000076,   // [894:777] Aacute + v : -58 (cslSTIX-Regular)
    0xf9000075,   // [895:778] Aacute + u : -14 (cslSTIX-Regular)
    0xf9800074,   // [896:779] Aacute + t : -13 (cslSTIX-Regular)
    0xf9000071,   // [897:780] Aacute + q : -14 (cslSTIX-Regular)
    0xf300006f,   // [898:781] Aacute + o : -26 (cslSTIX-Regular)
    0xfa000067,   // [899:782] Aacute + g : -12 (cslSTIX-Regular)
    0xf8000065,   // [900:783] Aacute + e : -16 (cslSTIX-Regular)
    0xf7800064,   // [901:784] Aacute + d : -17 (cslSTIX-Regular)
    0xfb000063,   // [902:785] Aacute + c : -10 (cslSTIX-Regular)
    0xf9000062,   // [903:786] Aacute + b : -14 (cslSTIX-Regular)
    0xfe000061,   // [904:787] Aacute + a : -4 (cslSTIX-Regular)
    0xe6000059,   // [905:788] Aacute + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [906:789] Aacute + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [907:790] Aacute + V : -83 (cslSTIX-Regular)
    0xee800055,   // [908:791] Aacute + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [909:792] Aacute + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [910:793] Aacute + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [911:794] Aacute + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [912:795] Aacute + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [913:796] Aacute + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [914:797] Aacute + period : -5 (cslSTIX-Regular)
    0xf800002d,   // [915:798] Aacute + hyphen : -16 (cslSTIX-Regular)
    0xff40002c,   // [916:799] Aacute + comma : -2 (cslSTIX-Regular) ;;
    0xe6000059,   // [917:800] Acircumflex + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [918:801] Acircumflex + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [919:802] Acircumflex + V : -83 (cslSTIX-Regular)
    0xee800055,   // [920:803] Acircumflex + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [921:804] Acircumflex + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [922:805] Acircumflex + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [923:806] Acircumflex + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [924:807] Acircumflex + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [925:808] Acircumflex + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [926:809] Acircumflex + period : -5 (cslSTIX-Regular)
    0xff40002c,   // [927:810] Acircumflex + comma : -2 (cslSTIX-Regular) ;;
    0xe6000059,   // [928:811] Atilde + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [929:812] Atilde + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [930:813] Atilde + V : -83 (cslSTIX-Regular)
    0xee800055,   // [931:814] Atilde + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [932:815] Atilde + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [933:816] Atilde + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [934:817] Atilde + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [935:818] Atilde + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [936:819] Atilde + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [937:820] Atilde + period : -5 (cslSTIX-Regular)
    0xff40002c,   // [938:821] Atilde + comma : -2 (cslSTIX-Regular) ;;
    0xe7802039,   // [939:822] Adieresis + guilsinglleft : -49 (cslSTIX-Regular)
    0xe380201d,   // [940:823] Adieresis + quotedblright : -57 (cslSTIX-Regular)
    0xdb002019,   // [941:824] Adieresis + quoteright : -74 (cslSTIX-Regular)
    0xeb0000ab,   // [942:825] Adieresis + guillemotleft : -42 (cslSTIX-Regular)
    0xdc000079,   // [943:826] Adieresis + y : -72 (cslSTIX-Regular)
    0xe3000077,   // [944:827] Adieresis + w : -58 (cslSTIX-Regular)
    0xe3000076,   // [945:828] Adieresis + v : -58 (cslSTIX-Regular)
    0xf9000075,   // [946:829] Adieresis + u : -14 (cslSTIX-Regular)
    0xf9800074,   // [947:830] Adieresis + t : -13 (cslSTIX-Regular)
    0xf9000071,   // [948:831] Adieresis + q : -14 (cslSTIX-Regular)
    0xf300006f,   // [949:832] Adieresis + o : -26 (cslSTIX-Regular)
    0xfa000067,   // [950:833] Adieresis + g : -12 (cslSTIX-Regular)
    0xf7800064,   // [951:834] Adieresis + d : -17 (cslSTIX-Regular)
    0xfb000063,   // [952:835] Adieresis + c : -10 (cslSTIX-Regular)
    0xf9000062,   // [953:836] Adieresis + b : -14 (cslSTIX-Regular)
    0xfe000061,   // [954:837] Adieresis + a : -4 (cslSTIX-Regular)
    0xe6000059,   // [955:838] Adieresis + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [956:839] Adieresis + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [957:840] Adieresis + V : -83 (cslSTIX-Regular)
    0xee800055,   // [958:841] Adieresis + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [959:842] Adieresis + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [960:843] Adieresis + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [961:844] Adieresis + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [962:845] Adieresis + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [963:846] Adieresis + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [964:847] Adieresis + period : -5 (cslSTIX-Regular)
    0xf800002d,   // [965:848] Adieresis + hyphen : -16 (cslSTIX-Regular)
    0xff40002c,   // [966:849] Adieresis + comma : -2 (cslSTIX-Regular) ;;
    0xe7802039,   // [967:850] Aring + guilsinglleft : -49 (cslSTIX-Regular)
    0xe380201d,   // [968:851] Aring + quotedblright : -57 (cslSTIX-Regular)
    0xdb002019,   // [969:852] Aring + quoteright : -74 (cslSTIX-Regular)
    0xeb0000ab,   // [970:853] Aring + guillemotleft : -42 (cslSTIX-Regular)
    0xdc000079,   // [971:854] Aring + y : -72 (cslSTIX-Regular)
    0xe3000077,   // [972:855] Aring + w : -58 (cslSTIX-Regular)
    0xe3000076,   // [973:856] Aring + v : -58 (cslSTIX-Regular)
    0xf9000075,   // [974:857] Aring + u : -14 (cslSTIX-Regular)
    0xf9800074,   // [975:858] Aring + t : -13 (cslSTIX-Regular)
    0xf9000071,   // [976:859] Aring + q : -14 (cslSTIX-Regular)
    0xf300006f,   // [977:860] Aring + o : -26 (cslSTIX-Regular)
    0xfa000067,   // [978:861] Aring + g : -12 (cslSTIX-Regular)
    0xf7800065,   // [979:862] Aring + e : -17 (cslSTIX-Regular)
    0xf7800064,   // [980:863] Aring + d : -17 (cslSTIX-Regular)
    0xfb000063,   // [981:864] Aring + c : -10 (cslSTIX-Regular)
    0xf9000062,   // [982:865] Aring + b : -14 (cslSTIX-Regular)
    0xfe000061,   // [983:866] Aring + a : -4 (cslSTIX-Regular)
    0xe6000059,   // [984:867] Aring + Y : -52 (cslSTIX-Regular)
    0xd6800057,   // [985:868] Aring + W : -83 (cslSTIX-Regular)
    0xd6800056,   // [986:869] Aring + V : -83 (cslSTIX-Regular)
    0xee800055,   // [987:870] Aring + U : -35 (cslSTIX-Regular)
    0xe5000054,   // [988:871] Aring + T : -54 (cslSTIX-Regular)
    0xf3800051,   // [989:872] Aring + Q : -25 (cslSTIX-Regular)
    0xf380004f,   // [990:873] Aring + O : -25 (cslSTIX-Regular)
    0xf3800047,   // [991:874] Aring + G : -25 (cslSTIX-Regular)
    0xf3800043,   // [992:875] Aring + C : -25 (cslSTIX-Regular)
    0xfd80002e,   // [993:876] Aring + period : -5 (cslSTIX-Regular)
    0xf800002d,   // [994:877] Aring + hyphen : -16 (cslSTIX-Regular)
    0xff40002c,   // [995:878] Aring + comma : -2 (cslSTIX-Regular) ;;
    0xf8400041,   // [996:879] Ccedilla + A : -16 (cslSTIX-Regular) ;;
    0xee000059,   // [997:880] Ograve + Y : -36 (cslSTIX-Regular)
    0xf1000056,   // [998:881] Ograve + V : -30 (cslSTIX-Regular)
    0xfec00054,   // [999:882] Ograve + T : -3 (cslSTIX-Regular) ;;
    0xee000059,   // [1000:883] Oacute + Y : -36 (cslSTIX-Regular)
    0xf3800057,   // [1001:884] Oacute + W : -25 (cslSTIX-Regular)
    0xf1000056,   // [1002:885] Oacute + V : -30 (cslSTIX-Regular)
    0xfe800054,   // [1003:886] Oacute + T : -3 (cslSTIX-Regular)
    0xf3c00041,   // [1004:887] Oacute + A : -25 (cslSTIX-Regular) ;;
    0xee000059,   // [1005:888] Ocircumflex + Y : -36 (cslSTIX-Regular)
    0xf1000056,   // [1006:889] Ocircumflex + V : -30 (cslSTIX-Regular)
    0xfec00054,   // [1007:890] Ocircumflex + T : -3 (cslSTIX-Regular) ;;
    0xee000059,   // [1008:891] Otilde + Y : -36 (cslSTIX-Regular)
    0xf1000056,   // [1009:892] Otilde + V : -30 (cslSTIX-Regular)
    0xfec00054,   // [1010:893] Otilde + T : -3 (cslSTIX-Regular) ;;
    0xee000059,   // [1011:894] Odieresis + Y : -36 (cslSTIX-Regular)
    0xf4000058,   // [1012:895] Odieresis + X : -24 (cslSTIX-Regular)
    0xf3800057,   // [1013:896] Odieresis + W : -25 (cslSTIX-Regular)
    0xf5000056,   // [1014:897] Odieresis + V : -22 (cslSTIX-Regular)
    0xfe800054,   // [1015:898] Odieresis + T : -3 (cslSTIX-Regular)
    0xf3c00041,   // [1016:899] Odieresis + A : -25 (cslSTIX-Regular) ;;
    0xf3c00041,   // [1017:900] Oslash + A : -25 (cslSTIX-Regular) ;;
    0xf8000072,   // [1018:901] Ugrave + r : -16 (cslSTIX-Regular)
    0xf7000070,   // [1019:902] Ugrave + p : -18 (cslSTIX-Regular)
    0xf600006e,   // [1020:903] Ugrave + n : -20 (cslSTIX-Regular)
    0xf600006d,   // [1021:904] Ugrave + m : -20 (cslSTIX-Regular)
    0xea800041,   // [1022:905] Ugrave + A : -43 (cslSTIX-Regular)
    0xf200002e,   // [1023:906] Ugrave + period : -28 (cslSTIX-Regular)
    0xf0c0002c,   // [1024:907] Ugrave + comma : -31 (cslSTIX-Regular) ;;
    0xf8000072,   // [1025:908] Uacute + r : -16 (cslSTIX-Regular)
    0xf7000070,   // [1026:909] Uacute + p : -18 (cslSTIX-Regular)
    0xf600006e,   // [1027:910] Uacute + n : -20 (cslSTIX-Regular)
    0xf600006d,   // [1028:911] Uacute + m : -20 (cslSTIX-Regular)
    0xea800041,   // [1029:912] Uacute + A : -43 (cslSTIX-Regular)
    0xf200002e,   // [1030:913] Uacute + period : -28 (cslSTIX-Regular)
    0xf0c0002c,   // [1031:914] Uacute + comma : -31 (cslSTIX-Regular) ;;
    0xeac00041,   // [1032:915] Ucircumflex + A : -43 (cslSTIX-Regular) ;;
    0xf8000072,   // [1033:916] Udieresis + r : -16 (cslSTIX-Regular)
    0xf7000070,   // [1034:917] Udieresis + p : -18 (cslSTIX-Regular)
    0xf600006e,   // [1035:918] Udieresis + n : -20 (cslSTIX-Regular)
    0xf600006d,   // [1036:919] Udieresis + m : -20 (cslSTIX-Regular)
    0x0a800062,   // [1037:920] Udieresis + b : 21 (cslSTIX-Regular)
    0xea800041,   // [1038:921] Udieresis + A : -43 (cslSTIX-Regular)
    0xf200002e,   // [1039:922] Udieresis + period : -28 (cslSTIX-Regular)
    0xf0c0002c,   // [1040:923] Udieresis + comma : -31 (cslSTIX-Regular) ;;
    0xf2800079,   // [1041:924] agrave + y : -27 (cslSTIX-Regular)
    0xef800077,   // [1042:925] agrave + w : -33 (cslSTIX-Regular)
    0xf0400076,   // [1043:926] agrave + v : -32 (cslSTIX-Regular) ;;
    0xf2800079,   // [1044:927] aacute + y : -27 (cslSTIX-Regular)
    0xef800077,   // [1045:928] aacute + w : -33 (cslSTIX-Regular)
    0xf0400076,   // [1046:929] aacute + v : -32 (cslSTIX-Regular) ;;
    0xf2800079,   // [1047:930] adieresis + y : -27 (cslSTIX-Regular)
    0xef800077,   // [1048:931] adieresis + w : -33 (cslSTIX-Regular)
    0xf0400076,   // [1049:932] adieresis + v : -32 (cslSTIX-Regular) ;;
    0xf2800079,   // [1050:933] aring + y : -27 (cslSTIX-Regular)
    0xef800077,   // [1051:934] aring + w : -33 (cslSTIX-Regular)
    0xf0400076,   // [1052:935] aring + v : -32 (cslSTIX-Regular) ;;
    0xf1000079,   // [1053:936] ae + y : -30 (cslSTIX-Regular)
    0xf1800077,   // [1054:937] ae + w : -29 (cslSTIX-Regular)
    0xf2400076,   // [1055:938] ae + v : -28 (cslSTIX-Regular) ;;
    0xf5800079,   // [1056:939] eacute + y : -21 (cslSTIX-Regular)
    0xf6800077,   // [1057:940] eacute + w : -19 (cslSTIX-Regular)
    0xf6c00076,   // [1058:941] eacute + v : -19 (cslSTIX-Regular) ;;
    0xf5800079,   // [1059:942] ecircumflex + y : -21 (cslSTIX-Regular)
    0xf6800077,   // [1060:943] ecircumflex + w : -19 (cslSTIX-Regular)
    0xf6c00076,   // [1061:944] ecircumflex + v : -19 (cslSTIX-Regular) ;;
    0xf5000079,   // [1062:945] ograve + y : -22 (cslSTIX-Regular)
    0xf5000077,   // [1063:946] ograve + w : -22 (cslSTIX-Regular)
    0xf5400076,   // [1064:947] ograve + v : -22 (cslSTIX-Regular) ;;
    0xf5000079,   // [1065:948] oacute + y : -22 (cslSTIX-Regular)
    0xf5000077,   // [1066:949] oacute + w : -22 (cslSTIX-Regular)
    0xf5400076,   // [1067:950] oacute + v : -22 (cslSTIX-Regular) ;;
    0xfc400074,   // [1068:951] ocircumflex + t : -8 (cslSTIX-Regular) ;;
    0xf5000079,   // [1069:952] odieresis + y : -22 (cslSTIX-Regular)
    0xf5000078,   // [1070:953] odieresis + x : -22 (cslSTIX-Regular)
    0xf5000077,   // [1071:954] odieresis + w : -22 (cslSTIX-Regular)
    0xf5000076,   // [1072:955] odieresis + v : -22 (cslSTIX-Regular)
    0xfc400074,   // [1073:956] odieresis + t : -8 (cslSTIX-Regular) ;;
    0xc80000c6,   // [1074:957] quoteleft + AE : -112 (cslSTIX-Regular)
    0xc70000c5,   // [1075:958] quoteleft + Aring : -114 (cslSTIX-Regular)
    0xc70000c4,   // [1076:959] quoteleft + Adieresis : -114 (cslSTIX-Regular)
    0xc70000c2,   // [1077:960] quoteleft + Acircumflex : -114 (cslSTIX-Regular)
    0xc70000c1,   // [1078:961] quoteleft + Aacute : -114 (cslSTIX-Regular)
    0xc70000c0,   // [1079:962] quoteleft + Agrave : -114 (cslSTIX-Regular)
    0xf2800059,   // [1080:963] quoteleft + Y : -27 (cslSTIX-Regular)
    0xf6000057,   // [1081:964] quoteleft + W : -20 (cslSTIX-Regular)
    0xf3800056,   // [1082:965] quoteleft + V : -25 (cslSTIX-Regular)
    0xfa800054,   // [1083:966] quoteleft + T : -11 (cslSTIX-Regular)
    0xc7400041,   // [1084:967] quoteleft + A : -114 (cslSTIX-Regular) ;;
    0xbc8000c6,   // [1085:968] quoteright + AE : -135 (cslSTIX-Regular)
    0xbe0000c5,   // [1086:969] quoteright + Aring : -132 (cslSTIX-Regular)
    0xbe0000c4,   // [1087:970] quoteright + Adieresis : -132 (cslSTIX-Regular)
    0xbe0000c2,   // [1088:971] quoteright + Acircumflex : -132 (cslSTIX-Regular)
    0xbe0000c1,   // [1089:972] quoteright + Aacute : -132 (cslSTIX-Regular)
    0xbe0000c0,   // [1090:973] quoteright + Agrave : -132 (cslSTIX-Regular)
    0xe9000079,   // [1091:974] quoteright + y : -46 (cslSTIX-Regular)
    0xea800077,   // [1092:975] quoteright + w : -43 (cslSTIX-Regular)
    0xe8000076,   // [1093:976] quoteright + v : -48 (cslSTIX-Regular)
    0xeb000074,   // [1094:977] quoteright + t : -42 (cslSTIX-Regular)
    0xe7800073,   // [1095:978] quoteright + s : -49 (cslSTIX-Regular)
    0xed000072,   // [1096:979] quoteright + r : -38 (cslSTIX-Regular)
    0xe400006f,   // [1097:980] quoteright + o : -56 (cslSTIX-Regular)
    0xe3000064,   // [1098:981] quoteright + d : -58 (cslSTIX-Regular)
    0xbe000041,   // [1099:982] quoteright + A : -132 (cslSTIX-Regular)
    0xd800002e,   // [1100:983] quoteright + period : -80 (cslSTIX-Regular)
    0xdbc0002c,   // [1101:984] quoteright + comma : -73 (cslSTIX-Regular) ;;
    0xd48000c6,   // [1102:985] quotedblleft + AE : -87 (cslSTIX-Regular)
    0xd38000c5,   // [1103:986] quotedblleft + Aring : -89 (cslSTIX-Regular)
    0xd38000c4,   // [1104:987] quotedblleft + Adieresis : -89 (cslSTIX-Regular)
    0xd38000c1,   // [1105:988] quotedblleft + Aacute : -89 (cslSTIX-Regular)
    0xfe800059,   // [1106:989] quotedblleft + Y : -3 (cslSTIX-Regular)
    0x02000057,   // [1107:990] quotedblleft + W : 4 (cslSTIX-Regular)
    0xff800056,   // [1108:991] quotedblleft + V : -1 (cslSTIX-Regular)
    0x06800054,   // [1109:992] quotedblleft + T : 13 (cslSTIX-Regular)
    0xd3c00041,   // [1110:993] quotedblleft + A : -89 (cslSTIX-Regular) ;;
    0xce8000c6,   // [1111:994] quotedblright + AE : -99 (cslSTIX-Regular)
    0xd00000c5,   // [1112:995] quotedblright + Aring : -96 (cslSTIX-Regular)
    0xd00000c4,   // [1113:996] quotedblright + Adieresis : -96 (cslSTIX-Regular)
    0xd00000c1,   // [1114:997] quotedblright + Aacute : -96 (cslSTIX-Regular)
    0xfe800059,   // [1115:998] quotedblright + Y : -3 (cslSTIX-Regular)
    0x03000057,   // [1116:999] quotedblright + W : 6 (cslSTIX-Regular)
    0x00800056,   // [1117:1000] quotedblright + V : 1 (cslSTIX-Regular)
    0x04800054,   // [1118:1001] quotedblright + T : 9 (cslSTIX-Regular)
    0xd0400041,   // [1119:1002] quotedblright + A : -96 (cslSTIX-Regular) ;;
    0x098000c6,   // [1120:1003] quotedblbase + AE : 19 (cslSTIX-Regular)
    0x060000c5,   // [1121:1004] quotedblbase + Aring : 12 (cslSTIX-Regular)
    0x060000c4,   // [1122:1005] quotedblbase + Adieresis : 12 (cslSTIX-Regular)
    0x060000c2,   // [1123:1006] quotedblbase + Acircumflex : 12 (cslSTIX-Regular)
    0x060000c1,   // [1124:1007] quotedblbase + Aacute : 12 (cslSTIX-Regular)
    0x060000c0,   // [1125:1008] quotedblbase + Agrave : 12 (cslSTIX-Regular)
    0xd4800059,   // [1126:1009] quotedblbase + Y : -87 (cslSTIX-Regular)
    0xdb800057,   // [1127:1010] quotedblbase + W : -73 (cslSTIX-Regular)
    0xce000056,   // [1128:1011] quotedblbase + V : -100 (cslSTIX-Regular)
    0xe2000054,   // [1129:1012] quotedblbase + T : -60 (cslSTIX-Regular)
    0x06400041,   // [1130:1013] quotedblbase + A : 12 (cslSTIX-Regular) ;;
    0xdb8000c6,   // [1131:1014] guilsinglright + AE : -73 (cslSTIX-Regular)
    0xda0000c5,   // [1132:1015] guilsinglright + Aring : -76 (cslSTIX-Regular)
    0xda0000c4,   // [1133:1016] guilsinglright + Adieresis : -76 (cslSTIX-Regular)
    0xda0000c1,   // [1134:1017] guilsinglright + Aacute : -76 (cslSTIX-Regular)
    0xb5800059,   // [1135:1018] guilsinglright + Y : -149 (cslSTIX-Regular)
    0xcb800057,   // [1136:1019] guilsinglright + W : -105 (cslSTIX-Regular)
    0xc1000056,   // [1137:1020] guilsinglright + V : -126 (cslSTIX-Regular)
    0xc2800054,   // [1138:1021] guilsinglright + T : -123 (cslSTIX-Regular)
    0xda400041,   // [1139:1022] guilsinglright + A : -76 (cslSTIX-Regular) ;;
    0x13a00069,   // [1140:1023] ff + i ligature #39 (cslSTIX-Regular)
    0x1460006c,   // [1141:1024] ff + l ligature #40 (cslSTIX-Regular) ;;
    0x0380201d,   // [1142:1] comma + quotedblright : 7 (cslSTIX-Bold)
    0xf5002019,   // [1143:2] comma + quoteright : -22 (cslSTIX-Bold)
    0xfac00031,   // [1144:3] comma + one : -11 (cslSTIX-Bold) ;;
    0xf58000c6,   // [1145:4] hyphen + AE : -21 (cslSTIX-Bold)
    0xf98000c5,   // [1146:5] hyphen + Aring : -13 (cslSTIX-Bold)
    0xf98000c4,   // [1147:6] hyphen + Adieresis : -13 (cslSTIX-Bold)
    0xf98000c1,   // [1148:7] hyphen + Aacute : -13 (cslSTIX-Bold)
    0xde800059,   // [1149:8] hyphen + Y : -67 (cslSTIX-Bold)
    0xe9800057,   // [1150:9] hyphen + W : -45 (cslSTIX-Bold)
    0xe2800056,   // [1151:10] hyphen + V : -59 (cslSTIX-Bold)
    0xe1800054,   // [1152:11] hyphen + T : -61 (cslSTIX-Bold)
    0xf9c00041,   // [1153:12] hyphen + A : -13 (cslSTIX-Bold) ;;
    0xf2c02019,   // [1154:13] period + quoteright : -27 (cslSTIX-Bold) ;;
    0xe3802039,   // [1155:14] A + guilsinglleft : -57 (cslSTIX-Bold)
    0xe180201d,   // [1156:15] A + quotedblright : -61 (cslSTIX-Bold)
    0xd3802019,   // [1157:16] A + quoteright : -89 (cslSTIX-Bold)
    0xf38000fc,   // [1158:17] A + udieresis : -25 (cslSTIX-Bold)
    0xf38000fb,   // [1159:18] A + ucircumflex : -25 (cslSTIX-Bold)
    0xf38000fa,   // [1160:19] A + uacute : -25 (cslSTIX-Bold)
    0xf38000f9,   // [1161:20] A + ugrave : -25 (cslSTIX-Bold)
    0xf00000f6,   // [1162:21] A + odieresis : -32 (cslSTIX-Bold)
    0xf00000e7,   // [1163:22] A + ccedilla : -32 (cslSTIX-Bold)
    0xe30000dc,   // [1164:23] A + Udieresis : -58 (cslSTIX-Bold)
    0xe30000db,   // [1165:24] A + Ucircumflex : -58 (cslSTIX-Bold)
    0xe30000da,   // [1166:25] A + Uacute : -58 (cslSTIX-Bold)
    0xe30000d9,   // [1167:26] A + Ugrave : -58 (cslSTIX-Bold)
    0xe90000d6,   // [1168:27] A + Odieresis : -46 (cslSTIX-Bold)
    0xe70000c7,   // [1169:28] A + Ccedilla : -50 (cslSTIX-Bold)
    0xe98000ab,   // [1170:29] A + guillemotleft : -45 (cslSTIX-Bold)
    0xdd800079,   // [1171:30] A + y : -69 (cslSTIX-Bold)
    0xde800077,   // [1172:31] A + w : -67 (cslSTIX-Bold)
    0xdc800076,   // [1173:32] A + v : -71 (cslSTIX-Bold)
    0xf3800075,   // [1174:33] A + u : -25 (cslSTIX-Bold)
    0xf4000074,   // [1175:34] A + t : -24 (cslSTIX-Bold)
    0xf2000071,   // [1176:35] A + q : -28 (cslSTIX-Bold)
    0xf000006f,   // [1177:36] A + o : -32 (cslSTIX-Bold)
    0xfc800067,   // [1178:37] A + g : -7 (cslSTIX-Bold)
    0xf2000065,   // [1179:38] A + e : -28 (cslSTIX-Bold)
    0xf4000064,   // [1180:39] A + d : -24 (cslSTIX-Bold)
    0xf2000063,   // [1181:40] A + c : -28 (cslSTIX-Bold)
    0xf6000062,   // [1182:41] A + b : -20 (cslSTIX-Bold)
    0xfe800061,   // [1183:42] A + a : -3 (cslSTIX-Bold)
    0xe0000059,   // [1184:43] A + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1185:44] A + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1186:45] A + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1187:46] A + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1188:47] A + T : -51 (cslSTIX-Bold)
    0xe9000051,   // [1189:48] A + Q : -46 (cslSTIX-Bold)
    0xe900004f,   // [1190:49] A + O : -46 (cslSTIX-Bold)
    0xe9000047,   // [1191:50] A + G : -46 (cslSTIX-Bold)
    0xe7000043,   // [1192:51] A + C : -50 (cslSTIX-Bold)
    0xf380002d,   // [1193:52] A + hyphen : -25 (cslSTIX-Bold)
    0x00c0002c,   // [1194:53] A + comma : 1 (cslSTIX-Bold) ;;
    0xfe800152,   // [1195:54] B + OE : -3 (cslSTIX-Bold)
    0xfc0000d8,   // [1196:55] B + Oslash : -8 (cslSTIX-Bold)
    0xfc0000d6,   // [1197:56] B + Odieresis : -8 (cslSTIX-Bold)
    0xfc0000d4,   // [1198:57] B + Ocircumflex : -8 (cslSTIX-Bold)
    0xfc0000d3,   // [1199:58] B + Oacute : -8 (cslSTIX-Bold)
    0xfc0000d2,   // [1200:59] B + Ograve : -8 (cslSTIX-Bold)
    0xf20000c6,   // [1201:60] B + AE : -28 (cslSTIX-Bold)
    0xf08000c5,   // [1202:61] B + Aring : -31 (cslSTIX-Bold)
    0xf08000c4,   // [1203:62] B + Adieresis : -31 (cslSTIX-Bold)
    0xf08000c3,   // [1204:63] B + Atilde : -31 (cslSTIX-Bold)
    0xf08000c2,   // [1205:64] B + Acircumflex : -31 (cslSTIX-Bold)
    0xf08000c1,   // [1206:65] B + Aacute : -31 (cslSTIX-Bold)
    0xec000059,   // [1207:66] B + Y : -40 (cslSTIX-Bold)
    0xed000057,   // [1208:67] B + W : -38 (cslSTIX-Bold)
    0xec000056,   // [1209:68] B + V : -40 (cslSTIX-Bold)
    0xfc00004f,   // [1210:69] B + O : -8 (cslSTIX-Bold)
    0xf0c00041,   // [1211:70] B + A : -31 (cslSTIX-Bold) ;;
    0xfc8000d6,   // [1212:71] C + Odieresis : -7 (cslSTIX-Bold)
    0xfc8000d3,   // [1213:72] C + Oacute : -7 (cslSTIX-Bold)
    0xfc8000d2,   // [1214:73] C + Ograve : -7 (cslSTIX-Bold)
    0xf50000c6,   // [1215:74] C + AE : -22 (cslSTIX-Bold)
    0xf48000c5,   // [1216:75] C + Aring : -23 (cslSTIX-Bold)
    0xf48000c4,   // [1217:76] C + Adieresis : -23 (cslSTIX-Bold)
    0xf48000c1,   // [1218:77] C + Aacute : -23 (cslSTIX-Bold)
    0xf48000c0,   // [1219:78] C + Agrave : -23 (cslSTIX-Bold)
    0xfc80004f,   // [1220:79] C + O : -7 (cslSTIX-Bold)
    0xfd80004b,   // [1221:80] C + K : -5 (cslSTIX-Bold)
    0xff000048,   // [1222:81] C + H : -2 (cslSTIX-Bold)
    0xf4c00041,   // [1223:82] C + A : -23 (cslSTIX-Bold) ;;
    0xef0000c5,   // [1224:83] D + Aring : -34 (cslSTIX-Bold)
    0xef0000c4,   // [1225:84] D + Adieresis : -34 (cslSTIX-Bold)
    0xef0000c3,   // [1226:85] D + Atilde : -34 (cslSTIX-Bold)
    0xef0000c2,   // [1227:86] D + Acircumflex : -34 (cslSTIX-Bold)
    0xef0000c1,   // [1228:87] D + Aacute : -34 (cslSTIX-Bold)
    0xef0000c0,   // [1229:88] D + Agrave : -34 (cslSTIX-Bold)
    0xe5800059,   // [1230:89] D + Y : -53 (cslSTIX-Bold)
    0xeb800058,   // [1231:90] D + X : -41 (cslSTIX-Bold)
    0xeb000057,   // [1232:91] D + W : -42 (cslSTIX-Bold)
    0xe5800056,   // [1233:92] D + V : -53 (cslSTIX-Bold)
    0xfd000054,   // [1234:93] D + T : -6 (cslSTIX-Bold)
    0xec00004a,   // [1235:94] D + J : -40 (cslSTIX-Bold)
    0xef400041,   // [1236:95] D + A : -34 (cslSTIX-Bold) ;;
    0xeb000153,   // [1237:96] F + oe : -42 (cslSTIX-Bold)
    0xea8000f8,   // [1238:97] F + oslash : -43 (cslSTIX-Bold)
    0xf50000f6,   // [1239:98] F + odieresis : -22 (cslSTIX-Bold)
    0xea8000f3,   // [1240:99] F + oacute : -43 (cslSTIX-Bold)
    0xea8000e9,   // [1241:100] F + eacute : -43 (cslSTIX-Bold)
    0xe90000e6,   // [1242:101] F + ae : -46 (cslSTIX-Bold)
    0xea8000e5,   // [1243:102] F + aring : -43 (cslSTIX-Bold)
    0xf48000e4,   // [1244:103] F + adieresis : -23 (cslSTIX-Bold)
    0xf70000e3,   // [1245:104] F + atilde : -18 (cslSTIX-Bold)
    0xf30000e2,   // [1246:105] F + acircumflex : -26 (cslSTIX-Bold)
    0xea8000e1,   // [1247:106] F + aacute : -43 (cslSTIX-Bold)
    0xfc0000d6,   // [1248:107] F + Odieresis : -8 (cslSTIX-Bold)
    0xfc0000d3,   // [1249:108] F + Oacute : -8 (cslSTIX-Bold)
    0xfc0000d2,   // [1250:109] F + Ograve : -8 (cslSTIX-Bold)
    0xe78000c5,   // [1251:110] F + Aring : -49 (cslSTIX-Bold)
    0xe78000c4,   // [1252:111] F + Adieresis : -49 (cslSTIX-Bold)
    0xe78000c3,   // [1253:112] F + Atilde : -49 (cslSTIX-Bold)
    0xe78000c2,   // [1254:113] F + Acircumflex : -49 (cslSTIX-Bold)
    0xe78000c1,   // [1255:114] F + Aacute : -49 (cslSTIX-Bold)
    0xe78000c0,   // [1256:115] F + Agrave : -49 (cslSTIX-Bold)
    0xfb800075,   // [1257:116] F + u : -9 (cslSTIX-Bold)
    0xfd000072,   // [1258:117] F + r : -6 (cslSTIX-Bold)
    0xea80006f,   // [1259:118] F + o : -43 (cslSTIX-Bold)
    0xf480006a,   // [1260:119] F + j : -23 (cslSTIX-Bold)
    0xff800069,   // [1261:120] F + i : -1 (cslSTIX-Bold)
    0xea800065,   // [1262:121] F + e : -43 (cslSTIX-Bold)
    0xea800061,   // [1263:122] F + a : -43 (cslSTIX-Bold)
    0xfc00004f,   // [1264:123] F + O : -8 (cslSTIX-Bold)
    0xed80004a,   // [1265:124] F + J : -37 (cslSTIX-Bold)
    0xdf000041,   // [1266:125] F + A : -66 (cslSTIX-Bold)
    0xe700002e,   // [1267:126] F + period : -50 (cslSTIX-Bold)
    0xf300002d,   // [1268:127] F + hyphen : -26 (cslSTIX-Bold)
    0xe7c0002c,   // [1269:128] F + comma : -49 (cslSTIX-Bold) ;;
    0xf50000c6,   // [1270:129] G + AE : -22 (cslSTIX-Bold)
    0xf40000c5,   // [1271:130] G + Aring : -24 (cslSTIX-Bold)
    0xf40000c4,   // [1272:131] G + Adieresis : -24 (cslSTIX-Bold)
    0xf40000c3,   // [1273:132] G + Atilde : -24 (cslSTIX-Bold)
    0xf40000c2,   // [1274:133] G + Acircumflex : -24 (cslSTIX-Bold)
    0xf40000c1,   // [1275:134] G + Aacute : -24 (cslSTIX-Bold)
    0xf40000c0,   // [1276:135] G + Agrave : -24 (cslSTIX-Bold)
    0xf1800059,   // [1277:136] G + Y : -29 (cslSTIX-Bold)
    0xf0800057,   // [1278:137] G + W : -31 (cslSTIX-Bold)
    0xf1800056,   // [1279:138] G + V : -29 (cslSTIX-Bold)
    0xee800054,   // [1280:139] G + T : -35 (cslSTIX-Bold)
    0xf4400041,   // [1281:140] G + A : -24 (cslSTIX-Bold) ;;
    0x14e0004a,   // [1282:141] I + J ligature #41 (cslSTIX-Bold) ;;
    0xf50000c6,   // [1283:142] J + AE : -22 (cslSTIX-Bold)
    0xf38000c5,   // [1284:143] J + Aring : -25 (cslSTIX-Bold)
    0xf38000c4,   // [1285:144] J + Adieresis : -25 (cslSTIX-Bold)
    0xf3c00041,   // [1286:145] J + A : -25 (cslSTIX-Bold) ;;
    0xec800152,   // [1287:146] K + OE : -39 (cslSTIX-Bold)
    0xf88000fc,   // [1288:147] K + udieresis : -15 (cslSTIX-Bold)
    0xf50000f6,   // [1289:148] K + odieresis : -22 (cslSTIX-Bold)
    0xf50000f3,   // [1290:149] K + oacute : -22 (cslSTIX-Bold)
    0x018000e6,   // [1291:150] K + ae : 3 (cslSTIX-Bold)
    0x040000e5,   // [1292:151] K + aring : 8 (cslSTIX-Bold)
    0x040000e4,   // [1293:152] K + adieresis : 8 (cslSTIX-Bold)
    0xec8000d6,   // [1294:153] K + Odieresis : -39 (cslSTIX-Bold)
    0xec8000d3,   // [1295:154] K + Oacute : -39 (cslSTIX-Bold)
    0xdd000079,   // [1296:155] K + y : -70 (cslSTIX-Bold)
    0xf8800075,   // [1297:156] K + u : -15 (cslSTIX-Bold)
    0xf500006f,   // [1298:157] K + o : -22 (cslSTIX-Bold)
    0xf7800065,   // [1299:158] K + e : -17 (cslSTIX-Bold)
    0x04000061,   // [1300:159] K + a : 8 (cslSTIX-Bold)
    0xfe800054,   // [1301:160] K + T : -3 (cslSTIX-Bold)
    0x06000053,   // [1302:161] K + S : 12 (cslSTIX-Bold)
    0xec80004f,   // [1303:162] K + O : -39 (cslSTIX-Bold)
    0xe7800047,   // [1304:163] K + G : -49 (cslSTIX-Bold)
    0xe6000043,   // [1305:164] K + C : -52 (cslSTIX-Bold)
    0xec40002d,   // [1306:165] K + hyphen : -40 (cslSTIX-Bold) ;;
    0xf280201d,   // [1307:166] L + quotedblright : -27 (cslSTIX-Bold)
    0xe4802019,   // [1308:167] L + quoteright : -55 (cslSTIX-Bold)
    0xfa8000fc,   // [1309:168] L + udieresis : -11 (cslSTIX-Bold)
    0xf40000dc,   // [1310:169] L + Udieresis : -24 (cslSTIX-Bold)
    0xfe8000d6,   // [1311:170] L + Odieresis : -3 (cslSTIX-Bold)
    0xfe8000d5,   // [1312:171] L + Otilde : -3 (cslSTIX-Bold)
    0xfe8000d4,   // [1313:172] L + Ocircumflex : -3 (cslSTIX-Bold)
    0xfe8000d3,   // [1314:173] L + Oacute : -3 (cslSTIX-Bold)
    0xfe8000d2,   // [1315:174] L + Ograve : -3 (cslSTIX-Bold)
    0xfb8000c7,   // [1316:175] L + Ccedilla : -9 (cslSTIX-Bold)
    0xff0000c5,   // [1317:176] L + Aring : -2 (cslSTIX-Bold)
    0xff0000c4,   // [1318:177] L + Adieresis : -2 (cslSTIX-Bold)
    0xff0000c1,   // [1319:178] L + Aacute : -2 (cslSTIX-Bold)
    0xee000079,   // [1320:179] L + y : -36 (cslSTIX-Bold)
    0xfc000075,   // [1321:180] L + u : -8 (cslSTIX-Bold)
    0xd9800059,   // [1322:181] L + Y : -77 (cslSTIX-Bold)
    0xdc000057,   // [1323:182] L + W : -72 (cslSTIX-Bold)
    0xd4800056,   // [1324:183] L + V : -87 (cslSTIX-Bold)
    0xf4000055,   // [1325:184] L + U : -24 (cslSTIX-Bold)
    0xdf800054,   // [1326:185] L + T : -65 (cslSTIX-Bold)
    0x01000053,   // [1327:186] L + S : 2 (cslSTIX-Bold)
    0xfe80004f,   // [1328:187] L + O : -3 (cslSTIX-Bold)
    0xfe000047,   // [1329:188] L + G : -4 (cslSTIX-Bold)
    0xfc000043,   // [1330:189] L + C : -8 (cslSTIX-Bold)
    0xff000041,   // [1331:190] L + A : -2 (cslSTIX-Bold)
    0x0ec0002d,   // [1332:191] L + hyphen : 29 (cslSTIX-Bold) ;;
    0xf98000fc,   // [1333:192] N + udieresis : -13 (cslSTIX-Bold)
    0xfa8000f8,   // [1334:193] N + oslash : -11 (cslSTIX-Bold)
    0xfa8000f6,   // [1335:194] N + odieresis : -11 (cslSTIX-Bold)
    0xfa8000f3,   // [1336:195] N + oacute : -11 (cslSTIX-Bold)
    0xfa8000e9,   // [1337:196] N + eacute : -11 (cslSTIX-Bold)
    0xf90000e6,   // [1338:197] N + ae : -14 (cslSTIX-Bold)
    0xf90000e5,   // [1339:198] N + aring : -14 (cslSTIX-Bold)
    0xf90000e4,   // [1340:199] N + adieresis : -14 (cslSTIX-Bold)
    0xf90000e1,   // [1341:200] N + aacute : -14 (cslSTIX-Bold)
    0xfb0000d6,   // [1342:201] N + Odieresis : -10 (cslSTIX-Bold)
    0xfb0000d3,   // [1343:202] N + Oacute : -10 (cslSTIX-Bold)
    0xf80000c7,   // [1344:203] N + Ccedilla : -16 (cslSTIX-Bold)
    0xf90000c6,   // [1345:204] N + AE : -14 (cslSTIX-Bold)
    0xf80000c5,   // [1346:205] N + Aring : -16 (cslSTIX-Bold)
    0xf80000c4,   // [1347:206] N + Adieresis : -16 (cslSTIX-Bold)
    0xf80000c1,   // [1348:207] N + Aacute : -16 (cslSTIX-Bold)
    0xf9800075,   // [1349:208] N + u : -13 (cslSTIX-Bold)
    0xfa80006f,   // [1350:209] N + o : -11 (cslSTIX-Bold)
    0xfa800065,   // [1351:210] N + e : -11 (cslSTIX-Bold)
    0xfa800061,   // [1352:211] N + a : -11 (cslSTIX-Bold)
    0xfb00004f,   // [1353:212] N + O : -10 (cslSTIX-Bold)
    0xfa800047,   // [1354:213] N + G : -11 (cslSTIX-Bold)
    0xf8000043,   // [1355:214] N + C : -16 (cslSTIX-Bold)
    0xf8000041,   // [1356:215] N + A : -16 (cslSTIX-Bold)
    0x00c0002c,   // [1357:216] N + comma : 1 (cslSTIX-Bold) ;;
    0xe80000c6,   // [1358:217] O + AE : -48 (cslSTIX-Bold)
    0xee0000c5,   // [1359:218] O + Aring : -36 (cslSTIX-Bold)
    0xee0000c4,   // [1360:219] O + Adieresis : -36 (cslSTIX-Bold)
    0xee0000c1,   // [1361:220] O + Aacute : -36 (cslSTIX-Bold)
    0xe5000059,   // [1362:221] O + Y : -54 (cslSTIX-Bold)
    0xeb000058,   // [1363:222] O + X : -42 (cslSTIX-Bold)
    0xec000057,   // [1364:223] O + W : -40 (cslSTIX-Bold)
    0xe5000056,   // [1365:224] O + V : -54 (cslSTIX-Bold)
    0xfb800054,   // [1366:225] O + T : -9 (cslSTIX-Bold)
    0xee400041,   // [1367:226] O + A : -36 (cslSTIX-Bold) ;;
    0xf3000153,   // [1368:227] P + oe : -26 (cslSTIX-Bold)
    0xf20000f8,   // [1369:228] P + oslash : -28 (cslSTIX-Bold)
    0xf50000f6,   // [1370:229] P + odieresis : -22 (cslSTIX-Bold)
    0xf20000f3,   // [1371:230] P + oacute : -28 (cslSTIX-Bold)
    0xf38000e9,   // [1372:231] P + eacute : -25 (cslSTIX-Bold)
    0xf68000e6,   // [1373:232] P + ae : -19 (cslSTIX-Bold)
    0xf88000e5,   // [1374:233] P + aring : -15 (cslSTIX-Bold)
    0xf88000e4,   // [1375:234] P + adieresis : -15 (cslSTIX-Bold)
    0xf88000e1,   // [1376:235] P + aacute : -15 (cslSTIX-Bold)
    0xf88000e0,   // [1377:236] P + agrave : -15 (cslSTIX-Bold)
    0xd88000c6,   // [1378:237] P + AE : -79 (cslSTIX-Bold)
    0xde8000c5,   // [1379:238] P + Aring : -67 (cslSTIX-Bold)
    0xde8000c4,   // [1380:239] P + Adieresis : -67 (cslSTIX-Bold)
    0xde8000c1,   // [1381:240] P + Aacute : -67 (cslSTIX-Bold)
    0xf200006f,   // [1382:241] P + o : -28 (cslSTIX-Bold)
    0xf3800065,   // [1383:242] P + e : -25 (cslSTIX-Bold)
    0xf8800061,   // [1384:243] P + a : -15 (cslSTIX-Bold)
    0xe300004a,   // [1385:244] P + J : -58 (cslSTIX-Bold)
    0xde800041,   // [1386:245] P + A : -67 (cslSTIX-Bold)
    0xdc00002e,   // [1387:246] P + period : -72 (cslSTIX-Bold)
    0xef80002d,   // [1388:247] P + hyphen : -33 (cslSTIX-Bold)
    0xdcc0002c,   // [1389:248] P + comma : -71 (cslSTIX-Bold) ;;
    0xf7800153,   // [1390:249] R + oe : -17 (cslSTIX-Bold)
    0xf7000152,   // [1391:250] R + OE : -18 (cslSTIX-Bold)
    0xf98000fc,   // [1392:251] R + udieresis : -13 (cslSTIX-Bold)
    0xf98000fa,   // [1393:252] R + uacute : -13 (cslSTIX-Bold)
    0xf58000f6,   // [1394:253] R + odieresis : -21 (cslSTIX-Bold)
    0xf58000f3,   // [1395:254] R + oacute : -21 (cslSTIX-Bold)
    0xf80000e9,   // [1396:255] R + eacute : -16 (cslSTIX-Bold)
    0x020000e6,   // [1397:256] R + ae : 4 (cslSTIX-Bold)
    0x050000e5,   // [1398:257] R + aring : 10 (cslSTIX-Bold)
    0x050000e4,   // [1399:258] R + adieresis : 10 (cslSTIX-Bold)
    0x050000e1,   // [1400:259] R + aacute : 10 (cslSTIX-Bold)
    0xf10000dc,   // [1401:260] R + Udieresis : -30 (cslSTIX-Bold)
    0xf48000d6,   // [1402:261] R + Odieresis : -23 (cslSTIX-Bold)
    0xf48000d3,   // [1403:262] R + Oacute : -23 (cslSTIX-Bold)
    0xf10000c7,   // [1404:263] R + Ccedilla : -30 (cslSTIX-Bold)
    0xf3800079,   // [1405:264] R + y : -25 (cslSTIX-Bold)
    0xf9800075,   // [1406:265] R + u : -13 (cslSTIX-Bold)
    0xf580006f,   // [1407:266] R + o : -21 (cslSTIX-Bold)
    0xf8000065,   // [1408:267] R + e : -16 (cslSTIX-Bold)
    0x05000061,   // [1409:268] R + a : 10 (cslSTIX-Bold)
    0xe8000059,   // [1410:269] R + Y : -48 (cslSTIX-Bold)
    0xe9800057,   // [1411:270] R + W : -45 (cslSTIX-Bold)
    0xe8000056,   // [1412:271] R + V : -48 (cslSTIX-Bold)
    0xf1000055,   // [1413:272] R + U : -30 (cslSTIX-Bold)
    0xf5000054,   // [1414:273] R + T : -22 (cslSTIX-Bold)
    0xf480004f,   // [1415:274] R + O : -23 (cslSTIX-Bold)
    0xf4000047,   // [1416:275] R + G : -24 (cslSTIX-Bold)
    0xf1000043,   // [1417:276] R + C : -30 (cslSTIX-Bold)
    0xf4c0002d,   // [1418:277] R + hyphen : -23 (cslSTIX-Bold) ;;
    0xf58000c6,   // [1419:278] S + AE : -21 (cslSTIX-Bold)
    0xf50000c5,   // [1420:279] S + Aring : -22 (cslSTIX-Bold)
    0xf50000c4,   // [1421:280] S + Adieresis : -22 (cslSTIX-Bold)
    0xf50000c1,   // [1422:281] S + Aacute : -22 (cslSTIX-Bold)
    0xfc800074,   // [1423:282] S + t : -7 (cslSTIX-Bold)
    0xfc800059,   // [1424:283] S + Y : -7 (cslSTIX-Bold)
    0xfb800057,   // [1425:284] S + W : -9 (cslSTIX-Bold)
    0xfc000056,   // [1426:285] S + V : -8 (cslSTIX-Bold)
    0xf9000054,   // [1427:286] S + T : -14 (cslSTIX-Bold)
    0xf5400041,   // [1428:287] S + A : -22 (cslSTIX-Bold) ;;
    0xd1002039,   // [1429:288] T + guilsinglleft : -94 (cslSTIX-Bold)
    0xfd800152,   // [1430:289] T + OE : -5 (cslSTIX-Bold)
    0xe30000f8,   // [1431:290] T + oslash : -58 (cslSTIX-Bold)
    0xf48000f6,   // [1432:291] T + odieresis : -23 (cslSTIX-Bold)
    0xf30000f4,   // [1433:292] T + ocircumflex : -26 (cslSTIX-Bold)
    0xe80000f3,   // [1434:293] T + oacute : -48 (cslSTIX-Bold)
    0xe90000f2,   // [1435:294] T + ograve : -46 (cslSTIX-Bold)
    0xe30000e6,   // [1436:295] T + ae : -58 (cslSTIX-Bold)
    0xf38000e4,   // [1437:296] T + adieresis : -25 (cslSTIX-Bold)
    0xf60000e3,   // [1438:297] T + atilde : -20 (cslSTIX-Bold)
    0xf28000e2,   // [1439:298] T + acircumflex : -27 (cslSTIX-Bold)
    0xea0000e1,   // [1440:299] T + aacute : -44 (cslSTIX-Bold)
    0xea8000e0,   // [1441:300] T + agrave : -43 (cslSTIX-Bold)
    0xfa0000d8,   // [1442:301] T + Oslash : -12 (cslSTIX-Bold)
    0xfb8000d6,   // [1443:302] T + Odieresis : -9 (cslSTIX-Bold)
    0xfb8000d5,   // [1444:303] T + Otilde : -9 (cslSTIX-Bold)
    0xfb8000d4,   // [1445:304] T + Ocircumflex : -9 (cslSTIX-Bold)
    0xfb8000d3,   // [1446:305] T + Oacute : -9 (cslSTIX-Bold)
    0xfb8000d2,   // [1447:306] T + Ograve : -9 (cslSTIX-Bold)
    0xed0000c6,   // [1448:307] T + AE : -38 (cslSTIX-Bold)
    0xec0000c5,   // [1449:308] T + Aring : -40 (cslSTIX-Bold)
    0xec0000c4,   // [1450:309] T + Adieresis : -40 (cslSTIX-Bold)
    0xec0000c3,   // [1451:310] T + Atilde : -40 (cslSTIX-Bold)
    0xec0000c2,   // [1452:311] T + Acircumflex : -40 (cslSTIX-Bold)
    0xec0000c1,   // [1453:312] T + Aacute : -40 (cslSTIX-Bold)
    0xec0000c0,   // [1454:313] T + Agrave : -40 (cslSTIX-Bold)
    0xd70000ab,   // [1455:314] T + guillemotleft : -82 (cslSTIX-Bold)
    0xd3000079,   // [1456:315] T + y : -90 (cslSTIX-Bold)
    0xd1800077,   // [1457:316] T + w : -93 (cslSTIX-Bold)
    0xd2000076,   // [1458:317] T + v : -92 (cslSTIX-Bold)
    0xda000075,   // [1459:318] T + u : -76 (cslSTIX-Bold)
    0xe6800073,   // [1460:319] T + s : -51 (cslSTIX-Bold)
    0xe5800072,   // [1461:320] T + r : -53 (cslSTIX-Bold)
    0xda00006f,   // [1462:321] T + o : -76 (cslSTIX-Bold)
    0xf400006a,   // [1463:322] T + j : -24 (cslSTIX-Bold)
    0xff000069,   // [1464:323] T + i : -2 (cslSTIX-Bold)
    0xe3000067,   // [1465:324] T + g : -58 (cslSTIX-Bold)
    0xdc000065,   // [1466:325] T + e : -72 (cslSTIX-Bold)
    0xdc000063,   // [1467:326] T + c : -72 (cslSTIX-Bold)
    0xe4800061,   // [1468:327] T + a : -55 (cslSTIX-Bold)
    0x05000059,   // [1469:328] T + Y : 10 (cslSTIX-Bold)
    0x04800057,   // [1470:329] T + W : 9 (cslSTIX-Bold)
    0x05000056,   // [1471:330] T + V : 10 (cslSTIX-Bold)
    0xfe800053,   // [1472:331] T + S : -3 (cslSTIX-Bold)
    0xfb80004f,   // [1473:332] T + O : -9 (cslSTIX-Bold)
    0xed00004a,   // [1474:333] T + J : -38 (cslSTIX-Bold)
    0xfa000047,   // [1475:334] T + G : -12 (cslSTIX-Bold)
    0xf8000043,   // [1476:335] T + C : -16 (cslSTIX-Bold)
    0xec000041,   // [1477:336] T + A : -40 (cslSTIX-Bold)
    0xeb00003b,   // [1478:337] T + semicolon : -42 (cslSTIX-Bold)
    0xeb00003a,   // [1479:338] T + colon : -42 (cslSTIX-Bold)
    0xe480002e,   // [1480:339] T + period : -55 (cslSTIX-Bold)
    0xe200002d,   // [1481:340] T + hyphen : -60 (cslSTIX-Bold)
    0xe4c0002c,   // [1482:341] T + comma : -55 (cslSTIX-Bold) ;;
    0xe88000c6,   // [1483:342] U + AE : -47 (cslSTIX-Bold)
    0xee0000c5,   // [1484:343] U + Aring : -36 (cslSTIX-Bold)
    0xee0000c4,   // [1485:344] U + Adieresis : -36 (cslSTIX-Bold)
    0xee0000c3,   // [1486:345] U + Atilde : -36 (cslSTIX-Bold)
    0xee0000c2,   // [1487:346] U + Acircumflex : -36 (cslSTIX-Bold)
    0xee0000c1,   // [1488:347] U + Aacute : -36 (cslSTIX-Bold)
    0xf9000072,   // [1489:348] U + r : -14 (cslSTIX-Bold)
    0xf9000070,   // [1490:349] U + p : -14 (cslSTIX-Bold)
    0xfa00006e,   // [1491:350] U + n : -12 (cslSTIX-Bold)
    0xfb80006d,   // [1492:351] U + m : -9 (cslSTIX-Bold)
    0xee000041,   // [1493:352] U + A : -36 (cslSTIX-Bold)
    0xf980002e,   // [1494:353] U + period : -13 (cslSTIX-Bold)
    0xfbc0002c,   // [1495:354] U + comma : -9 (cslSTIX-Bold) ;;
    0xd1002039,   // [1496:355] V + guilsinglleft : -94 (cslSTIX-Bold)
    0xdc0000f8,   // [1497:356] V + oslash : -72 (cslSTIX-Bold)
    0xed8000f6,   // [1498:357] V + odieresis : -37 (cslSTIX-Bold)
    0xee8000f5,   // [1499:358] V + otilde : -35 (cslSTIX-Bold)
    0xe30000f4,   // [1500:359] V + ocircumflex : -58 (cslSTIX-Bold)
    0xdc0000f3,   // [1501:360] V + oacute : -72 (cslSTIX-Bold)
    0xec0000f2,   // [1502:361] V + ograve : -40 (cslSTIX-Bold)
    0xdc0000e9,   // [1503:362] V + eacute : -72 (cslSTIX-Bold)
    0xee0000e8,   // [1504:363] V + egrave : -36 (cslSTIX-Bold)
    0xda0000e6,   // [1505:364] V + ae : -76 (cslSTIX-Bold)
    0xe80000e5,   // [1506:365] V + aring : -48 (cslSTIX-Bold)
    0xec8000e4,   // [1507:366] V + adieresis : -39 (cslSTIX-Bold)
    0xee0000e3,   // [1508:367] V + atilde : -36 (cslSTIX-Bold)
    0xe30000e2,   // [1509:368] V + acircumflex : -58 (cslSTIX-Bold)
    0xdc0000e1,   // [1510:369] V + aacute : -72 (cslSTIX-Bold)
    0xed0000e0,   // [1511:370] V + agrave : -38 (cslSTIX-Bold)
    0xe60000d8,   // [1512:371] V + Oslash : -52 (cslSTIX-Bold)
    0xe60000d6,   // [1513:372] V + Odieresis : -52 (cslSTIX-Bold)
    0xe60000d5,   // [1514:373] V + Otilde : -52 (cslSTIX-Bold)
    0xe60000d4,   // [1515:374] V + Ocircumflex : -52 (cslSTIX-Bold)
    0xe60000d3,   // [1516:375] V + Oacute : -52 (cslSTIX-Bold)
    0xe60000d2,   // [1517:376] V + Ograve : -52 (cslSTIX-Bold)
    0xcf8000c6,   // [1518:377] V + AE : -97 (cslSTIX-Bold)
    0xd00000c5,   // [1519:378] V + Aring : -96 (cslSTIX-Bold)
    0xd00000c4,   // [1520:379] V + Adieresis : -96 (cslSTIX-Bold)
    0xd00000c3,   // [1521:380] V + Atilde : -96 (cslSTIX-Bold)
    0xd00000c2,   // [1522:381] V + Acircumflex : -96 (cslSTIX-Bold)
    0xd00000c1,   // [1523:382] V + Aacute : -96 (cslSTIX-Bold)
    0xd00000c0,   // [1524:383] V + Agrave : -96 (cslSTIX-Bold)
    0xd68000ab,   // [1525:384] V + guillemotleft : -83 (cslSTIX-Bold)
    0xe7000079,   // [1526:385] V + y : -50 (cslSTIX-Bold)
    0xe7000075,   // [1527:386] V + u : -50 (cslSTIX-Bold)
    0xe6000072,   // [1528:387] V + r : -52 (cslSTIX-Bold)
    0xdc00006f,   // [1529:388] V + o : -72 (cslSTIX-Bold)
    0xfa000069,   // [1530:389] V + i : -12 (cslSTIX-Bold)
    0xdb800067,   // [1531:390] V + g : -73 (cslSTIX-Bold)
    0xdc000065,   // [1532:391] V + e : -72 (cslSTIX-Bold)
    0xdc000061,   // [1533:392] V + a : -72 (cslSTIX-Bold)
    0x03800054,   // [1534:393] V + T : 7 (cslSTIX-Bold)
    0xf3800053,   // [1535:394] V + S : -25 (cslSTIX-Bold)
    0xe600004f,   // [1536:395] V + O : -52 (cslSTIX-Bold)
    0xe5800047,   // [1537:396] V + G : -53 (cslSTIX-Bold)
    0xe3000043,   // [1538:397] V + C : -58 (cslSTIX-Bold)
    0xd0000041,   // [1539:398] V + A : -96 (cslSTIX-Bold)
    0xeb00003b,   // [1540:399] V + semicolon : -42 (cslSTIX-Bold)
    0xeb00003a,   // [1541:400] V + colon : -42 (cslSTIX-Bold)
    0xda00002e,   // [1542:401] V + period : -76 (cslSTIX-Bold)
    0xe380002d,   // [1543:402] V + hyphen : -57 (cslSTIX-Bold)
    0xda40002c,   // [1544:403] V + comma : -76 (cslSTIX-Bold) ;;
    0xda002039,   // [1545:404] W + guilsinglleft : -76 (cslSTIX-Bold)
    0xe48000f8,   // [1546:405] W + oslash : -55 (cslSTIX-Bold)
    0xf38000f6,   // [1547:406] W + odieresis : -25 (cslSTIX-Bold)
    0xf08000f5,   // [1548:407] W + otilde : -31 (cslSTIX-Bold)
    0xe50000f4,   // [1549:408] W + ocircumflex : -54 (cslSTIX-Bold)
    0xe48000f3,   // [1550:409] W + oacute : -55 (cslSTIX-Bold)
    0xec0000f2,   // [1551:410] W + ograve : -40 (cslSTIX-Bold)
    0xe48000e9,   // [1552:411] W + eacute : -55 (cslSTIX-Bold)
    0xee8000e8,   // [1553:412] W + egrave : -35 (cslSTIX-Bold)
    0xe10000e6,   // [1554:413] W + ae : -62 (cslSTIX-Bold)
    0xe88000e5,   // [1555:414] W + aring : -47 (cslSTIX-Bold)
    0xf28000e4,   // [1556:415] W + adieresis : -27 (cslSTIX-Bold)
    0xf08000e3,   // [1557:416] W + atilde : -31 (cslSTIX-Bold)
    0xe48000e2,   // [1558:417] W + acircumflex : -55 (cslSTIX-Bold)
    0xe48000e1,   // [1559:418] W + aacute : -55 (cslSTIX-Bold)
    0xed8000e0,   // [1560:419] W + agrave : -37 (cslSTIX-Bold)
    0xec0000d8,   // [1561:420] W + Oslash : -40 (cslSTIX-Bold)
    0xec0000d6,   // [1562:421] W + Odieresis : -40 (cslSTIX-Bold)
    0xec0000d5,   // [1563:422] W + Otilde : -40 (cslSTIX-Bold)
    0xec0000d4,   // [1564:423] W + Ocircumflex : -40 (cslSTIX-Bold)
    0xec0000d3,   // [1565:424] W + Oacute : -40 (cslSTIX-Bold)
    0xec0000d2,   // [1566:425] W + Ograve : -40 (cslSTIX-Bold)
    0xd48000c6,   // [1567:426] W + AE : -87 (cslSTIX-Bold)
    0xd80000c5,   // [1568:427] W + Aring : -80 (cslSTIX-Bold)
    0xd80000c4,   // [1569:428] W + Adieresis : -80 (cslSTIX-Bold)
    0xd80000c3,   // [1570:429] W + Atilde : -80 (cslSTIX-Bold)
    0xd80000c2,   // [1571:430] W + Acircumflex : -80 (cslSTIX-Bold)
    0xd80000c1,   // [1572:431] W + Aacute : -80 (cslSTIX-Bold)
    0xd80000c0,   // [1573:432] W + Agrave : -80 (cslSTIX-Bold)
    0xe08000ab,   // [1574:433] W + guillemotleft : -63 (cslSTIX-Bold)
    0xed000079,   // [1575:434] W + y : -38 (cslSTIX-Bold)
    0xed000075,   // [1576:435] W + u : -38 (cslSTIX-Bold)
    0xec000072,   // [1577:436] W + r : -40 (cslSTIX-Bold)
    0xe480006f,   // [1578:437] W + o : -55 (cslSTIX-Bold)
    0xfb000069,   // [1579:438] W + i : -10 (cslSTIX-Bold)
    0xe4000067,   // [1580:439] W + g : -56 (cslSTIX-Bold)
    0xe4800065,   // [1581:440] W + e : -55 (cslSTIX-Bold)
    0xe3000061,   // [1582:441] W + a : -58 (cslSTIX-Bold)
    0x04800054,   // [1583:442] W + T : 9 (cslSTIX-Bold)
    0xf5800053,   // [1584:443] W + S : -21 (cslSTIX-Bold)
    0xec00004f,   // [1585:444] W + O : -40 (cslSTIX-Bold)
    0xec000047,   // [1586:445] W + G : -40 (cslSTIX-Bold)
    0xe9000043,   // [1587:446] W + C : -46 (cslSTIX-Bold)
    0xd8000041,   // [1588:447] W + A : -80 (cslSTIX-Bold)
    0xf380003b,   // [1589:448] W + semicolon : -25 (cslSTIX-Bold)
    0xf380003a,   // [1590:449] W + colon : -25 (cslSTIX-Bold)
    0xe300002e,   // [1591:450] W + period : -58 (cslSTIX-Bold)
    0xec00002d,   // [1592:451] W + hyphen : -40 (cslSTIX-Bold)
    0xe340002c,   // [1593:452] W + comma : -58 (cslSTIX-Bold) ;;
    0xe78000d6,   // [1594:453] X + Odieresis : -49 (cslSTIX-Bold)
    0xe5800079,   // [1595:454] X + y : -53 (cslSTIX-Bold)
    0xf5800075,   // [1596:455] X + u : -21 (cslSTIX-Bold)
    0xf200006f,   // [1597:456] X + o : -28 (cslSTIX-Bold)
    0xf4800065,   // [1598:457] X + e : -23 (cslSTIX-Bold)
    0x00800061,   // [1599:458] X + a : 1 (cslSTIX-Bold)
    0xe9000051,   // [1600:459] X + Q : -46 (cslSTIX-Bold)
    0xe780004f,   // [1601:460] X + O : -49 (cslSTIX-Bold)
    0xe6800043,   // [1602:461] X + C : -51 (cslSTIX-Bold)
    0xef40002d,   // [1603:462] X + hyphen : -34 (cslSTIX-Bold) ;;
    0xcb802039,   // [1604:463] Y + guilsinglleft : -105 (cslSTIX-Bold)
    0xd70000f8,   // [1605:464] Y + oslash : -82 (cslSTIX-Bold)
    0xdb0000e6,   // [1606:465] Y + ae : -74 (cslSTIX-Bold)
    0xe18000d8,   // [1607:466] Y + Oslash : -61 (cslSTIX-Bold)
    0xe28000d6,   // [1608:467] Y + Odieresis : -59 (cslSTIX-Bold)
    0xe28000d5,   // [1609:468] Y + Otilde : -59 (cslSTIX-Bold)
    0xe28000d4,   // [1610:469] Y + Ocircumflex : -59 (cslSTIX-Bold)
    0xe28000d3,   // [1611:470] Y + Oacute : -59 (cslSTIX-Bold)
    0xe28000d2,   // [1612:471] Y + Ograve : -59 (cslSTIX-Bold)
    0xe58000c6,   // [1613:472] Y + AE : -53 (cslSTIX-Bold)
    0xe40000c5,   // [1614:473] Y + Aring : -56 (cslSTIX-Bold)
    0xe40000c4,   // [1615:474] Y + Adieresis : -56 (cslSTIX-Bold)
    0xe40000c3,   // [1616:475] Y + Atilde : -56 (cslSTIX-Bold)
    0xe40000c2,   // [1617:476] Y + Acircumflex : -56 (cslSTIX-Bold)
    0xe40000c1,   // [1618:477] Y + Aacute : -56 (cslSTIX-Bold)
    0xe40000c0,   // [1619:478] Y + Agrave : -56 (cslSTIX-Bold)
    0xd18000ab,   // [1620:479] Y + guillemotleft : -93 (cslSTIX-Bold)
    0xde800076,   // [1621:480] Y + v : -67 (cslSTIX-Bold)
    0xdf800075,   // [1622:481] Y + u : -65 (cslSTIX-Bold)
    0xe0000070,   // [1623:482] Y + p : -64 (cslSTIX-Bold)
    0xd700006f,   // [1624:483] Y + o : -82 (cslSTIX-Bold)
    0xf9800069,   // [1625:484] Y + i : -13 (cslSTIX-Bold)
    0xdb000067,   // [1626:485] Y + g : -74 (cslSTIX-Bold)
    0xd9800065,   // [1627:486] Y + e : -77 (cslSTIX-Bold)
    0xdd000061,   // [1628:487] Y + a : -70 (cslSTIX-Bold)
    0x03000054,   // [1629:488] Y + T : 6 (cslSTIX-Bold)
    0xf3000053,   // [1630:489] Y + S : -26 (cslSTIX-Bold)
    0xe280004f,   // [1631:490] Y + O : -59 (cslSTIX-Bold)
    0xe2000047,   // [1632:491] Y + G : -60 (cslSTIX-Bold)
    0xdf800043,   // [1633:492] Y + C : -65 (cslSTIX-Bold)
    0xe4000041,   // [1634:493] Y + A : -56 (cslSTIX-Bold)
    0xe800003b,   // [1635:494] Y + semicolon : -48 (cslSTIX-Bold)
    0xe800003a,   // [1636:495] Y + colon : -48 (cslSTIX-Bold)
    0xde00002e,   // [1637:496] Y + period : -68 (cslSTIX-Bold)
    0xdb00002d,   // [1638:497] Y + hyphen : -74 (cslSTIX-Bold)
    0xdec0002c,   // [1639:498] Y + comma : -67 (cslSTIX-Bold) ;;
    0xed000079,   // [1640:499] Z + y : -38 (cslSTIX-Bold)
    0xef000077,   // [1641:500] Z + w : -34 (cslSTIX-Bold)
    0xec400076,   // [1642:501] Z + v : -40 (cslSTIX-Bold) ;;
    0xf1802019,   // [1643:502] a + quoteright : -29 (cslSTIX-Bold)
    0xf5000079,   // [1644:503] a + y : -22 (cslSTIX-Bold)
    0xee000077,   // [1645:504] a + w : -36 (cslSTIX-Bold)
    0xef000076,   // [1646:505] a + v : -34 (cslSTIX-Bold)
    0xf440006a,   // [1647:506] a + j : -24 (cslSTIX-Bold) ;;
    0xf4000079,   // [1648:507] b + y : -24 (cslSTIX-Bold)
    0xee800077,   // [1649:508] b + w : -35 (cslSTIX-Bold)
    0xefc00076,   // [1650:509] b + v : -33 (cslSTIX-Bold) ;;
    0xfa00006b,   // [1651:510] c + k : -12 (cslSTIX-Bold)
    0xfbc00068,   // [1652:511] c + h : -9 (cslSTIX-Bold) ;;
    0xf6002019,   // [1653:512] e + quoteright : -20 (cslSTIX-Bold)
    0xf7800079,   // [1654:513] e + y : -17 (cslSTIX-Bold)
    0xfb000078,   // [1655:514] e + x : -10 (cslSTIX-Bold)
    0xf6800077,   // [1656:515] e + w : -19 (cslSTIX-Bold)
    0xf6800076,   // [1657:516] e + v : -19 (cslSTIX-Bold)
    0xfbc00074,   // [1658:517] e + t : -9 (cslSTIX-Bold) ;;
    0x09002019,   // [1659:518] f + quoteright : 18 (cslSTIX-Bold)
    0xf4000153,   // [1660:519] f + oe : -24 (cslSTIX-Bold)
    0xf80000f8,   // [1661:520] f + oslash : -16 (cslSTIX-Bold)
    0x0a8000f6,   // [1662:521] f + odieresis : 21 (cslSTIX-Bold)
    0xfe0000f4,   // [1663:522] f + ocircumflex : -4 (cslSTIX-Bold)
    0xf88000f3,   // [1664:523] f + oacute : -15 (cslSTIX-Bold)
    0xfc8000f2,   // [1665:524] f + ograve : -7 (cslSTIX-Bold)
    0x058000ea,   // [1666:525] f + ecircumflex : 11 (cslSTIX-Bold)
    0xfa0000e9,   // [1667:526] f + eacute : -12 (cslSTIX-Bold)
    0xfe8000e8,   // [1668:527] f + egrave : -3 (cslSTIX-Bold)
    0xf90000e6,   // [1669:528] f + ae : -14 (cslSTIX-Bold)
    0xfa8000e5,   // [1670:529] f + aring : -11 (cslSTIX-Bold)
    0x048000e4,   // [1671:530] f + adieresis : 9 (cslSTIX-Bold)
    0xfa8000e1,   // [1672:531] f + aacute : -11 (cslSTIX-Bold)
    0xfe800074,   // [1673:532] f + t : -3 (cslSTIX-Bold)
    0xfc800073,   // [1674:533] f + s : -7 (cslSTIX-Bold)
    0xf800006f,   // [1675:534] f + o : -16 (cslSTIX-Bold)
    0x1280006c,   // [1676:535] f + l : 37 (cslSTIX-Bold)
    0xff80006a,   // [1677:536] f + j : -1 (cslSTIX-Bold)
    0x0b800069,   // [1678:537] f + i : 23 (cslSTIX-Bold)
    0x06000066,   // [1679:538] f + f : 12 (cslSTIX-Bold)
    0xfd000065,   // [1680:539] f + e : -6 (cslSTIX-Bold)
    0xfa800061,   // [1681:540] f + a : -11 (cslSTIX-Bold)
    0x1520006a,   // [1682:541] f + j ligature #42 (cslSTIX-Bold)
    0x15a0006c,   // [1683:542] f + l ligature #43 (cslSTIX-Bold)
    0x16200069,   // [1684:543] f + i ligature #44 (cslSTIX-Bold)
    0x16e00066,   // [1685:544] f + f ligature #45 (cslSTIX-Bold) ;;
    0xf80000f6,   // [1686:545] g + odieresis : -16 (cslSTIX-Bold)
    0xf80000f3,   // [1687:546] g + oacute : -16 (cslSTIX-Bold)
    0xf78000eb,   // [1688:547] g + edieresis : -17 (cslSTIX-Bold)
    0xf78000ea,   // [1689:548] g + ecircumflex : -17 (cslSTIX-Bold)
    0xf78000e9,   // [1690:549] g + eacute : -17 (cslSTIX-Bold)
    0xf78000e8,   // [1691:550] g + egrave : -17 (cslSTIX-Bold)
    0xf90000e6,   // [1692:551] g + ae : -14 (cslSTIX-Bold)
    0xfa8000e5,   // [1693:552] g + aring : -11 (cslSTIX-Bold)
    0xfa8000e4,   // [1694:553] g + adieresis : -11 (cslSTIX-Bold)
    0xfe00006c,   // [1695:554] g + l : -4 (cslSTIX-Bold)
    0xf7800065,   // [1696:555] g + e : -17 (cslSTIX-Bold)
    0xfac00061,   // [1697:556] g + a : -11 (cslSTIX-Bold) ;;
    0xf1802019,   // [1698:557] h + quoteright : -29 (cslSTIX-Bold)
    0xf2c00079,   // [1699:558] h + y : -27 (cslSTIX-Bold) ;;
    0xf080006a,   // [1700:559] i + j : -31 (cslSTIX-Bold)
    0xf8000054,   // [1701:560] i + T : -16 (cslSTIX-Bold)
    0x1760006a,   // [1702:561] i + j ligature #46 (cslSTIX-Bold) ;;
    0xfe0000fc,   // [1703:562] k + udieresis : -4 (cslSTIX-Bold)
    0xf18000f6,   // [1704:563] k + odieresis : -29 (cslSTIX-Bold)
    0xf18000f3,   // [1705:564] k + oacute : -29 (cslSTIX-Bold)
    0xf48000e9,   // [1706:565] k + eacute : -23 (cslSTIX-Bold)
    0xf48000e8,   // [1707:566] k + egrave : -23 (cslSTIX-Bold)
    0xfe8000e6,   // [1708:567] k + ae : -3 (cslSTIX-Bold)
    0xfe000075,   // [1709:568] k + u : -4 (cslSTIX-Bold)
    0x02000073,   // [1710:569] k + s : 4 (cslSTIX-Bold)
    0xf180006f,   // [1711:570] k + o : -29 (cslSTIX-Bold)
    0xfe800067,   // [1712:571] k + g : -3 (cslSTIX-Bold)
    0xf4800065,   // [1713:572] k + e : -23 (cslSTIX-Bold)
    0xec40002d,   // [1714:573] k + hyphen : -40 (cslSTIX-Bold) ;;
    0xf7800079,   // [1715:574] l + y : -17 (cslSTIX-Bold)
    0xf6c00076,   // [1716:575] l + v : -19 (cslSTIX-Bold) ;;
    0xf3000079,   // [1717:576] m + y : -26 (cslSTIX-Bold)
    0xf1000077,   // [1718:577] m + w : -30 (cslSTIX-Bold)
    0xf1800076,   // [1719:578] m + v : -29 (cslSTIX-Bold)
    0xf8400070,   // [1720:579] m + p : -16 (cslSTIX-Bold) ;;
    0xf3002019,   // [1721:580] n + quoteright : -26 (cslSTIX-Bold)
    0xf3800079,   // [1722:581] n + y : -25 (cslSTIX-Bold)
    0xf1800077,   // [1723:582] n + w : -29 (cslSTIX-Bold)
    0xf2800076,   // [1724:583] n + v : -27 (cslSTIX-Bold)
    0xf9800070,   // [1725:584] n + p : -13 (cslSTIX-Bold)
    0xe7c00054,   // [1726:585] n + T : -49 (cslSTIX-Bold) ;;
    0xf4002019,   // [1727:586] o + quoteright : -24 (cslSTIX-Bold)
    0xf5000079,   // [1728:587] o + y : -22 (cslSTIX-Bold)
    0xf7800078,   // [1729:588] o + x : -17 (cslSTIX-Bold)
    0xf5800077,   // [1730:589] o + w : -21 (cslSTIX-Bold)
    0xf3000076,   // [1731:590] o + v : -26 (cslSTIX-Bold)
    0xfe800074,   // [1732:591] o + t : -3 (cslSTIX-Bold)
    0xda400054,   // [1733:592] o + T : -76 (cslSTIX-Bold) ;;
    0xf3000079,   // [1734:593] p + y : -26 (cslSTIX-Bold)
    0xfbc00074,   // [1735:594] p + t : -9 (cslSTIX-Bold) ;;
    0xf8800075,   // [1736:595] q + u : -15 (cslSTIX-Bold)
    0xfa400063,   // [1737:596] q + c : -12 (cslSTIX-Bold) ;;
    0xfc802019,   // [1738:597] r + quoteright : -7 (cslSTIX-Bold)
    0xfb000153,   // [1739:598] r + oe : -10 (cslSTIX-Bold)
    0xfa0000f8,   // [1740:599] r + oslash : -12 (cslSTIX-Bold)
    0xfa0000f6,   // [1741:600] r + odieresis : -12 (cslSTIX-Bold)
    0xfa0000f4,   // [1742:601] r + ocircumflex : -12 (cslSTIX-Bold)
    0xfa0000f3,   // [1743:602] r + oacute : -12 (cslSTIX-Bold)
    0xfa0000f2,   // [1744:603] r + ograve : -12 (cslSTIX-Bold)
    0xfb0000ea,   // [1745:604] r + ecircumflex : -10 (cslSTIX-Bold)
    0xfb0000e9,   // [1746:605] r + eacute : -10 (cslSTIX-Bold)
    0xfb0000e8,   // [1747:606] r + egrave : -10 (cslSTIX-Bold)
    0xfa8000e7,   // [1748:607] r + ccedilla : -11 (cslSTIX-Bold)
    0xfc8000e6,   // [1749:608] r + ae : -7 (cslSTIX-Bold)
    0xfe0000e5,   // [1750:609] r + aring : -4 (cslSTIX-Bold)
    0xfe0000e4,   // [1751:610] r + adieresis : -4 (cslSTIX-Bold)
    0xfe0000e2,   // [1752:611] r + acircumflex : -4 (cslSTIX-Bold)
    0xfe0000e1,   // [1753:612] r + aacute : -4 (cslSTIX-Bold)
    0xfe0000e0,   // [1754:613] r + agrave : -4 (cslSTIX-Bold)
    0xff80007a,   // [1755:614] r + z : -1 (cslSTIX-Bold)
    0x04000079,   // [1756:615] r + y : 8 (cslSTIX-Bold)
    0x04000078,   // [1757:616] r + x : 8 (cslSTIX-Bold)
    0x02000077,   // [1758:617] r + w : 4 (cslSTIX-Bold)
    0x03000076,   // [1759:618] r + v : 6 (cslSTIX-Bold)
    0x04000075,   // [1760:619] r + u : 8 (cslSTIX-Bold)
    0x05000074,   // [1761:620] r + t : 10 (cslSTIX-Bold)
    0xf8000071,   // [1762:621] r + q : -16 (cslSTIX-Bold)
    0x03000070,   // [1763:622] r + p : 6 (cslSTIX-Bold)
    0xfa00006f,   // [1764:623] r + o : -12 (cslSTIX-Bold)
    0x0180006e,   // [1765:624] r + n : 3 (cslSTIX-Bold)
    0x0380006d,   // [1766:625] r + m : 7 (cslSTIX-Bold)
    0xfa00006c,   // [1767:626] r + l : -12 (cslSTIX-Bold)
    0xf980006b,   // [1768:627] r + k : -13 (cslSTIX-Bold)
    0xfa80006a,   // [1769:628] r + j : -11 (cslSTIX-Bold)
    0x03800069,   // [1770:629] r + i : 7 (cslSTIX-Bold)
    0xf9800068,   // [1771:630] r + h : -13 (cslSTIX-Bold)
    0xfd800067,   // [1772:631] r + g : -5 (cslSTIX-Bold)
    0x05000066,   // [1773:632] r + f : 10 (cslSTIX-Bold)
    0xfb000065,   // [1774:633] r + e : -10 (cslSTIX-Bold)
    0xfb000064,   // [1775:634] r + d : -10 (cslSTIX-Bold)
    0xf9800063,   // [1776:635] r + c : -13 (cslSTIX-Bold)
    0xfe000061,   // [1777:636] r + a : -4 (cslSTIX-Bold)
    0xfb00003b,   // [1778:637] r + semicolon : -10 (cslSTIX-Bold)
    0xfb00003a,   // [1779:638] r + colon : -10 (cslSTIX-Bold)
    0xe300002e,   // [1780:639] r + period : -58 (cslSTIX-Bold)
    0xf880002d,   // [1781:640] r + hyphen : -15 (cslSTIX-Bold)
    0xe340002c,   // [1782:641] r + comma : -58 (cslSTIX-Bold) ;;
    0xf8002019,   // [1783:642] s + quoteright : -16 (cslSTIX-Bold)
    0xfcc00074,   // [1784:643] s + t : -7 (cslSTIX-Bold) ;;
    0xf2802019,   // [1785:644] t + quoteright : -27 (cslSTIX-Bold)
    0xfe0000f6,   // [1786:645] t + odieresis : -4 (cslSTIX-Bold)
    0xfe0000f3,   // [1787:646] t + oacute : -4 (cslSTIX-Bold)
    0xff0000e9,   // [1788:647] t + eacute : -2 (cslSTIX-Bold)
    0x030000e6,   // [1789:648] t + ae : 6 (cslSTIX-Bold)
    0x048000e5,   // [1790:649] t + aring : 9 (cslSTIX-Bold)
    0x048000e4,   // [1791:650] t + adieresis : 9 (cslSTIX-Bold)
    0x048000e1,   // [1792:651] t + aacute : 9 (cslSTIX-Bold)
    0xfe00006f,   // [1793:652] t + o : -4 (cslSTIX-Bold)
    0xff800068,   // [1794:653] t + h : -1 (cslSTIX-Bold)
    0xff000065,   // [1795:654] t + e : -2 (cslSTIX-Bold)
    0x04800061,   // [1796:655] t + a : 9 (cslSTIX-Bold)
    0x05000053,   // [1797:656] t + S : 10 (cslSTIX-Bold)
    0x0680003b,   // [1798:657] t + semicolon : 13 (cslSTIX-Bold)
    0x0540003a,   // [1799:658] t + colon : 10 (cslSTIX-Bold) ;;
    0xf4402019,   // [1800:659] u + quoteright : -24 (cslSTIX-Bold) ;;
    0xf38000f8,   // [1801:660] v + oslash : -25 (cslSTIX-Bold)
    0xf38000f6,   // [1802:661] v + odieresis : -25 (cslSTIX-Bold)
    0xf38000f3,   // [1803:662] v + oacute : -25 (cslSTIX-Bold)
    0xf38000f2,   // [1804:663] v + ograve : -25 (cslSTIX-Bold)
    0xf48000ea,   // [1805:664] v + ecircumflex : -23 (cslSTIX-Bold)
    0xf48000e9,   // [1806:665] v + eacute : -23 (cslSTIX-Bold)
    0xf48000e8,   // [1807:666] v + egrave : -23 (cslSTIX-Bold)
    0xf40000e6,   // [1808:667] v + ae : -24 (cslSTIX-Bold)
    0xf80000e5,   // [1809:668] v + aring : -16 (cslSTIX-Bold)
    0xf80000e4,   // [1810:669] v + adieresis : -16 (cslSTIX-Bold)
    0xf80000e3,   // [1811:670] v + atilde : -16 (cslSTIX-Bold)
    0xf80000e2,   // [1812:671] v + acircumflex : -16 (cslSTIX-Bold)
    0xf80000e1,   // [1813:672] v + aacute : -16 (cslSTIX-Bold)
    0xf80000e0,   // [1814:673] v + agrave : -16 (cslSTIX-Bold)
    0xfa800073,   // [1815:674] v + s : -11 (cslSTIX-Bold)
    0xf380006f,   // [1816:675] v + o : -25 (cslSTIX-Bold)
    0xfe80006c,   // [1817:676] v + l : -3 (cslSTIX-Bold)
    0xf8800067,   // [1818:677] v + g : -15 (cslSTIX-Bold)
    0xf4800065,   // [1819:678] v + e : -23 (cslSTIX-Bold)
    0xf3000063,   // [1820:679] v + c : -26 (cslSTIX-Bold)
    0xf8000061,   // [1821:680] v + a : -16 (cslSTIX-Bold)
    0xfb80003b,   // [1822:681] v + semicolon : -9 (cslSTIX-Bold)
    0xfb80003a,   // [1823:682] v + colon : -9 (cslSTIX-Bold)
    0xe680002e,   // [1824:683] v + period : -51 (cslSTIX-Bold)
    0xf300002d,   // [1825:684] v + hyphen : -26 (cslSTIX-Bold)
    0xe740002c,   // [1826:685] v + comma : -50 (cslSTIX-Bold) ;;
    0xf08000f8,   // [1827:686] w + oslash : -31 (cslSTIX-Bold)
    0xf08000f6,   // [1828:687] w + odieresis : -31 (cslSTIX-Bold)
    0xf08000f3,   // [1829:688] w + oacute : -31 (cslSTIX-Bold)
    0xf08000f2,   // [1830:689] w + ograve : -31 (cslSTIX-Bold)
    0xf18000ea,   // [1831:690] w + ecircumflex : -29 (cslSTIX-Bold)
    0xf18000e9,   // [1832:691] w + eacute : -29 (cslSTIX-Bold)
    0xf18000e8,   // [1833:692] w + egrave : -29 (cslSTIX-Bold)
    0xf40000e6,   // [1834:693] w + ae : -24 (cslSTIX-Bold)
    0xf58000e5,   // [1835:694] w + aring : -21 (cslSTIX-Bold)
    0xf58000e4,   // [1836:695] w + adieresis : -21 (cslSTIX-Bold)
    0xf58000e3,   // [1837:696] w + atilde : -21 (cslSTIX-Bold)
    0xf58000e2,   // [1838:697] w + acircumflex : -21 (cslSTIX-Bold)
    0xf58000e1,   // [1839:698] w + aacute : -21 (cslSTIX-Bold)
    0xf58000e0,   // [1840:699] w + agrave : -21 (cslSTIX-Bold)
    0xf8800073,   // [1841:700] w + s : -15 (cslSTIX-Bold)
    0xf080006f,   // [1842:701] w + o : -31 (cslSTIX-Bold)
    0xf900006c,   // [1843:702] w + l : -14 (cslSTIX-Bold)
    0xf9000067,   // [1844:703] w + g : -14 (cslSTIX-Bold)
    0xf1800065,   // [1845:704] w + e : -29 (cslSTIX-Bold)
    0xf0800063,   // [1846:705] w + c : -31 (cslSTIX-Bold)
    0xf5800061,   // [1847:706] w + a : -21 (cslSTIX-Bold)
    0xfd00003b,   // [1848:707] w + semicolon : -6 (cslSTIX-Bold)
    0xfd00003a,   // [1849:708] w + colon : -6 (cslSTIX-Bold)
    0xe800002e,   // [1850:709] w + period : -48 (cslSTIX-Bold)
    0xf480002d,   // [1851:710] w + hyphen : -23 (cslSTIX-Bold)
    0xeac0002c,   // [1852:711] w + comma : -43 (cslSTIX-Bold) ;;
    0xf38000e9,   // [1853:712] x + eacute : -25 (cslSTIX-Bold)
    0xf3800071,   // [1854:713] x + q : -25 (cslSTIX-Bold)
    0xf300006f,   // [1855:714] x + o : -26 (cslSTIX-Bold)
    0xf3800065,   // [1856:715] x + e : -25 (cslSTIX-Bold)
    0xf4400063,   // [1857:716] x + c : -24 (cslSTIX-Bold) ;;
    0xf40000f8,   // [1858:717] y + oslash : -24 (cslSTIX-Bold)
    0xf40000f6,   // [1859:718] y + odieresis : -24 (cslSTIX-Bold)
    0xf40000f3,   // [1860:719] y + oacute : -24 (cslSTIX-Bold)
    0xf40000f2,   // [1861:720] y + ograve : -24 (cslSTIX-Bold)
    0xf40000ea,   // [1862:721] y + ecircumflex : -24 (cslSTIX-Bold)
    0xf40000e9,   // [1863:722] y + eacute : -24 (cslSTIX-Bold)
    0xf40000e8,   // [1864:723] y + egrave : -24 (cslSTIX-Bold)
    0xf30000e6,   // [1865:724] y + ae : -26 (cslSTIX-Bold)
    0xf50000e5,   // [1866:725] y + aring : -22 (cslSTIX-Bold)
    0xf50000e4,   // [1867:726] y + adieresis : -22 (cslSTIX-Bold)
    0xf50000e3,   // [1868:727] y + atilde : -22 (cslSTIX-Bold)
    0xf50000e2,   // [1869:728] y + acircumflex : -22 (cslSTIX-Bold)
    0xf50000e1,   // [1870:729] y + aacute : -22 (cslSTIX-Bold)
    0xf50000e0,   // [1871:730] y + agrave : -22 (cslSTIX-Bold)
    0xf7800073,   // [1872:731] y + s : -17 (cslSTIX-Bold)
    0xf400006f,   // [1873:732] y + o : -24 (cslSTIX-Bold)
    0xfa80006c,   // [1874:733] y + l : -11 (cslSTIX-Bold)
    0xf6800067,   // [1875:734] y + g : -19 (cslSTIX-Bold)
    0xf4000065,   // [1876:735] y + e : -24 (cslSTIX-Bold)
    0xef800063,   // [1877:736] y + c : -33 (cslSTIX-Bold)
    0xf5000061,   // [1878:737] y + a : -22 (cslSTIX-Bold)
    0xfb00003b,   // [1879:738] y + semicolon : -10 (cslSTIX-Bold)
    0xfb00003a,   // [1880:739] y + colon : -10 (cslSTIX-Bold)
    0xe780002e,   // [1881:740] y + period : -49 (cslSTIX-Bold)
    0xf380002d,   // [1882:741] y + hyphen : -25 (cslSTIX-Bold)
    0xe840002c,   // [1883:742] y + comma : -48 (cslSTIX-Bold) ;;
    0xec0000c6,   // [1884:743] guillemotright + AE : -40 (cslSTIX-Bold)
    0xef0000c5,   // [1885:744] guillemotright + Aring : -34 (cslSTIX-Bold)
    0xef0000c4,   // [1886:745] guillemotright + Adieresis : -34 (cslSTIX-Bold)
    0xef0000c1,   // [1887:746] guillemotright + Aacute : -34 (cslSTIX-Bold)
    0xd3800059,   // [1888:747] guillemotright + Y : -89 (cslSTIX-Bold)
    0xde800057,   // [1889:748] guillemotright + W : -67 (cslSTIX-Bold)
    0xd5800056,   // [1890:749] guillemotright + V : -85 (cslSTIX-Bold)
    0xd7000054,   // [1891:750] guillemotright + T : -82 (cslSTIX-Bold)
    0xef400041,   // [1892:751] guillemotright + A : -34 (cslSTIX-Bold) ;;
    0xe0000059,   // [1893:752] Agrave + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1894:753] Agrave + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1895:754] Agrave + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1896:755] Agrave + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1897:756] Agrave + T : -51 (cslSTIX-Bold)
    0xe7000051,   // [1898:757] Agrave + Q : -50 (cslSTIX-Bold)
    0xe700004f,   // [1899:758] Agrave + O : -50 (cslSTIX-Bold)
    0xe7000047,   // [1900:759] Agrave + G : -50 (cslSTIX-Bold)
    0xe7000043,   // [1901:760] Agrave + C : -50 (cslSTIX-Bold)
    0x00c0002c,   // [1902:761] Agrave + comma : 1 (cslSTIX-Bold) ;;
    0xe3802039,   // [1903:762] Aacute + guilsinglleft : -57 (cslSTIX-Bold)
    0xe180201d,   // [1904:763] Aacute + quotedblright : -61 (cslSTIX-Bold)
    0xd3802019,   // [1905:764] Aacute + quoteright : -89 (cslSTIX-Bold)
    0xe98000ab,   // [1906:765] Aacute + guillemotleft : -45 (cslSTIX-Bold)
    0xdd800079,   // [1907:766] Aacute + y : -69 (cslSTIX-Bold)
    0xde800077,   // [1908:767] Aacute + w : -67 (cslSTIX-Bold)
    0xdc800076,   // [1909:768] Aacute + v : -71 (cslSTIX-Bold)
    0xf3800075,   // [1910:769] Aacute + u : -25 (cslSTIX-Bold)
    0xf4000074,   // [1911:770] Aacute + t : -24 (cslSTIX-Bold)
    0xf2000071,   // [1912:771] Aacute + q : -28 (cslSTIX-Bold)
    0xf000006f,   // [1913:772] Aacute + o : -32 (cslSTIX-Bold)
    0xfc800067,   // [1914:773] Aacute + g : -7 (cslSTIX-Bold)
    0xf2000065,   // [1915:774] Aacute + e : -28 (cslSTIX-Bold)
    0xf4000064,   // [1916:775] Aacute + d : -24 (cslSTIX-Bold)
    0xf2000063,   // [1917:776] Aacute + c : -28 (cslSTIX-Bold)
    0xf6000062,   // [1918:777] Aacute + b : -20 (cslSTIX-Bold)
    0xfe800061,   // [1919:778] Aacute + a : -3 (cslSTIX-Bold)
    0xe0000059,   // [1920:779] Aacute + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1921:780] Aacute + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1922:781] Aacute + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1923:782] Aacute + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1924:783] Aacute + T : -51 (cslSTIX-Bold)
    0xe9000051,   // [1925:784] Aacute + Q : -46 (cslSTIX-Bold)
    0xe900004f,   // [1926:785] Aacute + O : -46 (cslSTIX-Bold)
    0xe9000047,   // [1927:786] Aacute + G : -46 (cslSTIX-Bold)
    0xe7000043,   // [1928:787] Aacute + C : -50 (cslSTIX-Bold)
    0xf380002d,   // [1929:788] Aacute + hyphen : -25 (cslSTIX-Bold)
    0x00c0002c,   // [1930:789] Aacute + comma : 1 (cslSTIX-Bold) ;;
    0xe0000059,   // [1931:790] Acircumflex + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1932:791] Acircumflex + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1933:792] Acircumflex + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1934:793] Acircumflex + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1935:794] Acircumflex + T : -51 (cslSTIX-Bold)
    0xe9000051,   // [1936:795] Acircumflex + Q : -46 (cslSTIX-Bold)
    0xe900004f,   // [1937:796] Acircumflex + O : -46 (cslSTIX-Bold)
    0xe7000047,   // [1938:797] Acircumflex + G : -50 (cslSTIX-Bold)
    0xe7000043,   // [1939:798] Acircumflex + C : -50 (cslSTIX-Bold)
    0x00c0002c,   // [1940:799] Acircumflex + comma : 1 (cslSTIX-Bold) ;;
    0xe0000059,   // [1941:800] Atilde + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1942:801] Atilde + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1943:802] Atilde + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1944:803] Atilde + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1945:804] Atilde + T : -51 (cslSTIX-Bold)
    0xe9000051,   // [1946:805] Atilde + Q : -46 (cslSTIX-Bold)
    0xe900004f,   // [1947:806] Atilde + O : -46 (cslSTIX-Bold)
    0xe9000047,   // [1948:807] Atilde + G : -46 (cslSTIX-Bold)
    0xe7000043,   // [1949:808] Atilde + C : -50 (cslSTIX-Bold)
    0x00c0002c,   // [1950:809] Atilde + comma : 1 (cslSTIX-Bold) ;;
    0xe3802039,   // [1951:810] Adieresis + guilsinglleft : -57 (cslSTIX-Bold)
    0xe180201d,   // [1952:811] Adieresis + quotedblright : -61 (cslSTIX-Bold)
    0xd3802019,   // [1953:812] Adieresis + quoteright : -89 (cslSTIX-Bold)
    0xe98000ab,   // [1954:813] Adieresis + guillemotleft : -45 (cslSTIX-Bold)
    0xdd800079,   // [1955:814] Adieresis + y : -69 (cslSTIX-Bold)
    0xde800077,   // [1956:815] Adieresis + w : -67 (cslSTIX-Bold)
    0xdc800076,   // [1957:816] Adieresis + v : -71 (cslSTIX-Bold)
    0xf3800075,   // [1958:817] Adieresis + u : -25 (cslSTIX-Bold)
    0xf4000074,   // [1959:818] Adieresis + t : -24 (cslSTIX-Bold)
    0xf2000071,   // [1960:819] Adieresis + q : -28 (cslSTIX-Bold)
    0xf000006f,   // [1961:820] Adieresis + o : -32 (cslSTIX-Bold)
    0xfc800067,   // [1962:821] Adieresis + g : -7 (cslSTIX-Bold)
    0xf4000064,   // [1963:822] Adieresis + d : -24 (cslSTIX-Bold)
    0xf2000063,   // [1964:823] Adieresis + c : -28 (cslSTIX-Bold)
    0xf6000062,   // [1965:824] Adieresis + b : -20 (cslSTIX-Bold)
    0xfe800061,   // [1966:825] Adieresis + a : -3 (cslSTIX-Bold)
    0xe0000059,   // [1967:826] Adieresis + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1968:827] Adieresis + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1969:828] Adieresis + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1970:829] Adieresis + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1971:830] Adieresis + T : -51 (cslSTIX-Bold)
    0xe7000051,   // [1972:831] Adieresis + Q : -50 (cslSTIX-Bold)
    0xe700004f,   // [1973:832] Adieresis + O : -50 (cslSTIX-Bold)
    0xe7000047,   // [1974:833] Adieresis + G : -50 (cslSTIX-Bold)
    0xe7000043,   // [1975:834] Adieresis + C : -50 (cslSTIX-Bold)
    0xf380002d,   // [1976:835] Adieresis + hyphen : -25 (cslSTIX-Bold)
    0x00c0002c,   // [1977:836] Adieresis + comma : 1 (cslSTIX-Bold) ;;
    0xe3802039,   // [1978:837] Aring + guilsinglleft : -57 (cslSTIX-Bold)
    0xe180201d,   // [1979:838] Aring + quotedblright : -61 (cslSTIX-Bold)
    0xd3802019,   // [1980:839] Aring + quoteright : -89 (cslSTIX-Bold)
    0xe98000ab,   // [1981:840] Aring + guillemotleft : -45 (cslSTIX-Bold)
    0xdd800079,   // [1982:841] Aring + y : -69 (cslSTIX-Bold)
    0xde800077,   // [1983:842] Aring + w : -67 (cslSTIX-Bold)
    0xdc800076,   // [1984:843] Aring + v : -71 (cslSTIX-Bold)
    0xf3800075,   // [1985:844] Aring + u : -25 (cslSTIX-Bold)
    0xf4000074,   // [1986:845] Aring + t : -24 (cslSTIX-Bold)
    0xf2000071,   // [1987:846] Aring + q : -28 (cslSTIX-Bold)
    0xf000006f,   // [1988:847] Aring + o : -32 (cslSTIX-Bold)
    0xfc800067,   // [1989:848] Aring + g : -7 (cslSTIX-Bold)
    0xf2000065,   // [1990:849] Aring + e : -28 (cslSTIX-Bold)
    0xf4000064,   // [1991:850] Aring + d : -24 (cslSTIX-Bold)
    0xf2000063,   // [1992:851] Aring + c : -28 (cslSTIX-Bold)
    0xf6000062,   // [1993:852] Aring + b : -20 (cslSTIX-Bold)
    0xfe800061,   // [1994:853] Aring + a : -3 (cslSTIX-Bold)
    0xe0000059,   // [1995:854] Aring + Y : -64 (cslSTIX-Bold)
    0xd0800057,   // [1996:855] Aring + W : -95 (cslSTIX-Bold)
    0xc9800056,   // [1997:856] Aring + V : -109 (cslSTIX-Bold)
    0xe3000055,   // [1998:857] Aring + U : -58 (cslSTIX-Bold)
    0xe6800054,   // [1999:858] Aring + T : -51 (cslSTIX-Bold)
    0xe9000051,   // [2000:859] Aring + Q : -46 (cslSTIX-Bold)
    0xe900004f,   // [2001:860] Aring + O : -46 (cslSTIX-Bold)
    0xe9000047,   // [2002:861] Aring + G : -46 (cslSTIX-Bold)
    0xe7000043,   // [2003:862] Aring + C : -50 (cslSTIX-Bold)
    0xf380002d,   // [2004:863] Aring + hyphen : -25 (cslSTIX-Bold)
    0x00c0002c,   // [2005:864] Aring + comma : 1 (cslSTIX-Bold) ;;
    0xf4400041,   // [2006:865] Ccedilla + A : -24 (cslSTIX-Bold) ;;
    0xe5000059,   // [2007:866] Ograve + Y : -54 (cslSTIX-Bold)
    0xe5000056,   // [2008:867] Ograve + V : -54 (cslSTIX-Bold)
    0xfbc00054,   // [2009:868] Ograve + T : -9 (cslSTIX-Bold) ;;
    0xe5000059,   // [2010:869] Oacute + Y : -54 (cslSTIX-Bold)
    0xec000057,   // [2011:870] Oacute + W : -40 (cslSTIX-Bold)
    0xe5000056,   // [2012:871] Oacute + V : -54 (cslSTIX-Bold)
    0xfb800054,   // [2013:872] Oacute + T : -9 (cslSTIX-Bold)
    0xee400041,   // [2014:873] Oacute + A : -36 (cslSTIX-Bold) ;;
    0xe5000059,   // [2015:874] Ocircumflex + Y : -54 (cslSTIX-Bold)
    0xe5000056,   // [2016:875] Ocircumflex + V : -54 (cslSTIX-Bold)
    0xfbc00054,   // [2017:876] Ocircumflex + T : -9 (cslSTIX-Bold) ;;
    0xe5000059,   // [2018:877] Otilde + Y : -54 (cslSTIX-Bold)
    0xe5000056,   // [2019:878] Otilde + V : -54 (cslSTIX-Bold)
    0xfbc00054,   // [2020:879] Otilde + T : -9 (cslSTIX-Bold) ;;
    0xe5000059,   // [2021:880] Odieresis + Y : -54 (cslSTIX-Bold)
    0xeb000058,   // [2022:881] Odieresis + X : -42 (cslSTIX-Bold)
    0xec000057,   // [2023:882] Odieresis + W : -40 (cslSTIX-Bold)
    0xe5000056,   // [2024:883] Odieresis + V : -54 (cslSTIX-Bold)
    0xfb800054,   // [2025:884] Odieresis + T : -9 (cslSTIX-Bold)
    0xee400041,   // [2026:885] Odieresis + A : -36 (cslSTIX-Bold) ;;
    0xee400041,   // [2027:886] Oslash + A : -36 (cslSTIX-Bold) ;;
    0xf9000072,   // [2028:887] Ugrave + r : -14 (cslSTIX-Bold)
    0xf9000070,   // [2029:888] Ugrave + p : -14 (cslSTIX-Bold)
    0xfa00006e,   // [2030:889] Ugrave + n : -12 (cslSTIX-Bold)
    0xfb80006d,   // [2031:890] Ugrave + m : -9 (cslSTIX-Bold)
    0xee000041,   // [2032:891] Ugrave + A : -36 (cslSTIX-Bold)
    0xf980002e,   // [2033:892] Ugrave + period : -13 (cslSTIX-Bold)
    0xfbc0002c,   // [2034:893] Ugrave + comma : -9 (cslSTIX-Bold) ;;
    0xf9000072,   // [2035:894] Uacute + r : -14 (cslSTIX-Bold)
    0xf9000070,   // [2036:895] Uacute + p : -14 (cslSTIX-Bold)
    0xfa00006e,   // [2037:896] Uacute + n : -12 (cslSTIX-Bold)
    0xfb80006d,   // [2038:897] Uacute + m : -9 (cslSTIX-Bold)
    0xee800041,   // [2039:898] Uacute + A : -35 (cslSTIX-Bold)
    0xf980002e,   // [2040:899] Uacute + period : -13 (cslSTIX-Bold)
    0xfbc0002c,   // [2041:900] Uacute + comma : -9 (cslSTIX-Bold) ;;
    0xee400041,   // [2042:901] Ucircumflex + A : -36 (cslSTIX-Bold) ;;
    0xf9000072,   // [2043:902] Udieresis + r : -14 (cslSTIX-Bold)
    0xf9000070,   // [2044:903] Udieresis + p : -14 (cslSTIX-Bold)
    0xfa00006e,   // [2045:904] Udieresis + n : -12 (cslSTIX-Bold)
    0xfb80006d,   // [2046:905] Udieresis + m : -9 (cslSTIX-Bold)
    0x05000062,   // [2047:906] Udieresis + b : 10 (cslSTIX-Bold)
    0xee000041,   // [2048:907] Udieresis + A : -36 (cslSTIX-Bold)
    0xf980002e,   // [2049:908] Udieresis + period : -13 (cslSTIX-Bold)
    0xfbc0002c,   // [2050:909] Udieresis + comma : -9 (cslSTIX-Bold) ;;
    0xf5000079,   // [2051:910] agrave + y : -22 (cslSTIX-Bold)
    0xee000077,   // [2052:911] agrave + w : -36 (cslSTIX-Bold)
    0xef400076,   // [2053:912] agrave + v : -34 (cslSTIX-Bold) ;;
    0xf5000079,   // [2054:913] aacute + y : -22 (cslSTIX-Bold)
    0xee000077,   // [2055:914] aacute + w : -36 (cslSTIX-Bold)
    0xef400076,   // [2056:915] aacute + v : -34 (cslSTIX-Bold) ;;
    0xf5000079,   // [2057:916] adieresis + y : -22 (cslSTIX-Bold)
    0xee000077,   // [2058:917] adieresis + w : -36 (cslSTIX-Bold)
    0xef400076,   // [2059:918] adieresis + v : -34 (cslSTIX-Bold) ;;
    0xf5000079,   // [2060:919] aring + y : -22 (cslSTIX-Bold)
    0xee000077,   // [2061:920] aring + w : -36 (cslSTIX-Bold)
    0xef400076,   // [2062:921] aring + v : -34 (cslSTIX-Bold) ;;
    0xf2800079,   // [2063:922] ae + y : -27 (cslSTIX-Bold)
    0xf0800077,   // [2064:923] ae + w : -31 (cslSTIX-Bold)
    0xf1c00076,   // [2065:924] ae + v : -29 (cslSTIX-Bold) ;;
    0xf7800079,   // [2066:925] eacute + y : -17 (cslSTIX-Bold)
    0xf6800077,   // [2067:926] eacute + w : -19 (cslSTIX-Bold)
    0xf6c00076,   // [2068:927] eacute + v : -19 (cslSTIX-Bold) ;;
    0xf7800079,   // [2069:928] ecircumflex + y : -17 (cslSTIX-Bold)
    0xf6800077,   // [2070:929] ecircumflex + w : -19 (cslSTIX-Bold)
    0xf6c00076,   // [2071:930] ecircumflex + v : -19 (cslSTIX-Bold) ;;
    0xf5000079,   // [2072:931] ograve + y : -22 (cslSTIX-Bold)
    0xf5800077,   // [2073:932] ograve + w : -21 (cslSTIX-Bold)
    0xf3400076,   // [2074:933] ograve + v : -26 (cslSTIX-Bold) ;;
    0xf5000079,   // [2075:934] oacute + y : -22 (cslSTIX-Bold)
    0xf5800077,   // [2076:935] oacute + w : -21 (cslSTIX-Bold)
    0xf3400076,   // [2077:936] oacute + v : -26 (cslSTIX-Bold) ;;
    0xfec00074,   // [2078:937] ocircumflex + t : -3 (cslSTIX-Bold) ;;
    0xf5000079,   // [2079:938] odieresis + y : -22 (cslSTIX-Bold)
    0xf7800078,   // [2080:939] odieresis + x : -17 (cslSTIX-Bold)
    0xf5800077,   // [2081:940] odieresis + w : -21 (cslSTIX-Bold)
    0xf3000076,   // [2082:941] odieresis + v : -26 (cslSTIX-Bold)
    0xfec00074,   // [2083:942] odieresis + t : -3 (cslSTIX-Bold) ;;
    0xd10000c6,   // [2084:943] quoteleft + AE : -94 (cslSTIX-Bold)
    0xd90000c5,   // [2085:944] quoteleft + Aring : -78 (cslSTIX-Bold)
    0xd90000c4,   // [2086:945] quoteleft + Adieresis : -78 (cslSTIX-Bold)
    0xd90000c2,   // [2087:946] quoteleft + Acircumflex : -78 (cslSTIX-Bold)
    0xd90000c1,   // [2088:947] quoteleft + Aacute : -78 (cslSTIX-Bold)
    0xd90000c0,   // [2089:948] quoteleft + Agrave : -78 (cslSTIX-Bold)
    0xf9000059,   // [2090:949] quoteleft + Y : -14 (cslSTIX-Bold)
    0xf7800057,   // [2091:950] quoteleft + W : -17 (cslSTIX-Bold)
    0xf8800056,   // [2092:951] quoteleft + V : -15 (cslSTIX-Bold)
    0xf6000054,   // [2093:952] quoteleft + T : -20 (cslSTIX-Bold)
    0xd9400041,   // [2094:953] quoteleft + A : -78 (cslSTIX-Bold) ;;
    0xd28000c6,   // [2095:954] quoteright + AE : -91 (cslSTIX-Bold)
    0xda8000c5,   // [2096:955] quoteright + Aring : -75 (cslSTIX-Bold)
    0xda8000c4,   // [2097:956] quoteright + Adieresis : -75 (cslSTIX-Bold)
    0xda8000c2,   // [2098:957] quoteright + Acircumflex : -75 (cslSTIX-Bold)
    0xda8000c1,   // [2099:958] quoteright + Aacute : -75 (cslSTIX-Bold)
    0xda8000c0,   // [2100:959] quoteright + Agrave : -75 (cslSTIX-Bold)
    0xf6000079,   // [2101:960] quoteright + y : -20 (cslSTIX-Bold)
    0xf5800077,   // [2102:961] quoteright + w : -21 (cslSTIX-Bold)
    0xf5000076,   // [2103:962] quoteright + v : -22 (cslSTIX-Bold)
    0xf8000074,   // [2104:963] quoteright + t : -16 (cslSTIX-Bold)
    0xf8800073,   // [2105:964] quoteright + s : -15 (cslSTIX-Bold)
    0xf5000072,   // [2106:965] quoteright + r : -22 (cslSTIX-Bold)
    0xf200006f,   // [2107:966] quoteright + o : -28 (cslSTIX-Bold)
    0xf2000064,   // [2108:967] quoteright + d : -28 (cslSTIX-Bold)
    0xda800041,   // [2109:968] quoteright + A : -75 (cslSTIX-Bold)
    0xef80002e,   // [2110:969] quoteright + period : -33 (cslSTIX-Bold)
    0xf040002c,   // [2111:970] quoteright + comma : -32 (cslSTIX-Bold) ;;
    0xde8000c6,   // [2112:971] quotedblleft + AE : -67 (cslSTIX-Bold)
    0xe70000c5,   // [2113:972] quotedblleft + Aring : -50 (cslSTIX-Bold)
    0xe70000c4,   // [2114:973] quotedblleft + Adieresis : -50 (cslSTIX-Bold)
    0xe70000c1,   // [2115:974] quotedblleft + Aacute : -50 (cslSTIX-Bold)
    0x08000059,   // [2116:975] quotedblleft + Y : 16 (cslSTIX-Bold)
    0x06000057,   // [2117:976] quotedblleft + W : 12 (cslSTIX-Bold)
    0x07000056,   // [2118:977] quotedblleft + V : 14 (cslSTIX-Bold)
    0x04800054,   // [2119:978] quotedblleft + T : 9 (cslSTIX-Bold)
    0xe7400041,   // [2120:979] quotedblleft + A : -50 (cslSTIX-Bold) ;;
    0xda8000c6,   // [2121:980] quotedblright + AE : -75 (cslSTIX-Bold)
    0xe30000c5,   // [2122:981] quotedblright + Aring : -58 (cslSTIX-Bold)
    0xe30000c4,   // [2123:982] quotedblright + Adieresis : -58 (cslSTIX-Bold)
    0xe30000c1,   // [2124:983] quotedblright + Aacute : -58 (cslSTIX-Bold)
    0x02000059,   // [2125:984] quotedblright + Y : 4 (cslSTIX-Bold)
    0x01800057,   // [2126:985] quotedblright + W : 3 (cslSTIX-Bold)
    0x02000056,   // [2127:986] quotedblright + V : 4 (cslSTIX-Bold)
    0x03000054,   // [2128:987] quotedblright + T : 6 (cslSTIX-Bold)
    0xe3400041,   // [2129:988] quotedblright + A : -58 (cslSTIX-Bold) ;;
    0x090000c6,   // [2130:989] quotedblbase + AE : 18 (cslSTIX-Bold)
    0x090000c5,   // [2131:990] quotedblbase + Aring : 18 (cslSTIX-Bold)
    0x090000c4,   // [2132:991] quotedblbase + Adieresis : 18 (cslSTIX-Bold)
    0x090000c2,   // [2133:992] quotedblbase + Acircumflex : 18 (cslSTIX-Bold)
    0x090000c1,   // [2134:993] quotedblbase + Aacute : 18 (cslSTIX-Bold)
    0x090000c0,   // [2135:994] quotedblbase + Agrave : 18 (cslSTIX-Bold)
    0xe1800059,   // [2136:995] quotedblbase + Y : -61 (cslSTIX-Bold)
    0xe3000057,   // [2137:996] quotedblbase + W : -58 (cslSTIX-Bold)
    0xd8800056,   // [2138:997] quotedblbase + V : -79 (cslSTIX-Bold)
    0xe6800054,   // [2139:998] quotedblbase + T : -51 (cslSTIX-Bold)
    0x09400041,   // [2140:999] quotedblbase + A : 18 (cslSTIX-Bold) ;;
    0xe60000c6,   // [2141:1000] guilsinglright + AE : -52 (cslSTIX-Bold)
    0xe90000c5,   // [2142:1001] guilsinglright + Aring : -46 (cslSTIX-Bold)
    0xe90000c4,   // [2143:1002] guilsinglright + Adieresis : -46 (cslSTIX-Bold)
    0xe90000c1,   // [2144:1003] guilsinglright + Aacute : -46 (cslSTIX-Bold)
    0xcd000059,   // [2145:1004] guilsinglright + Y : -102 (cslSTIX-Bold)
    0xd8000057,   // [2146:1005] guilsinglright + W : -80 (cslSTIX-Bold)
    0xcf800056,   // [2147:1006] guilsinglright + V : -97 (cslSTIX-Bold)
    0xd1000054,   // [2148:1007] guilsinglright + T : -94 (cslSTIX-Bold)
    0xe9400041,   // [2149:1008] guilsinglright + A : -46 (cslSTIX-Bold) ;;
    0x17a00069,   // [2150:1009] ff + i ligature #47 (cslSTIX-Bold)
    0x1860006c,   // [2151:1010] ff + l ligature #48 (cslSTIX-Bold) ;;
    0xed80201d,   // [2152:1] comma + quotedblright : -37 (cslSTIX-Italic)
    0xed802019,   // [2153:2] comma + quoteright : -37 (cslSTIX-Italic)
    0xf1400031,   // [2154:3] comma + one : -30 (cslSTIX-Italic) ;;
    0xfb8000c6,   // [2155:4] hyphen + AE : -9 (cslSTIX-Italic)
    0x070000c5,   // [2156:5] hyphen + Aring : 14 (cslSTIX-Italic)
    0x070000c4,   // [2157:6] hyphen + Adieresis : 14 (cslSTIX-Italic)
    0x070000c1,   // [2158:7] hyphen + Aacute : 14 (cslSTIX-Italic)
    0xec000059,   // [2159:8] hyphen + Y : -40 (cslSTIX-Italic)
    0xf5000057,   // [2160:9] hyphen + W : -22 (cslSTIX-Italic)
    0xf2000056,   // [2161:10] hyphen + V : -28 (cslSTIX-Italic)
    0xef000054,   // [2162:11] hyphen + T : -34 (cslSTIX-Italic)
    0x07400041,   // [2163:12] hyphen + A : 14 (cslSTIX-Italic) ;;
    0xee80201d,   // [2164:13] period + quotedblright : -35 (cslSTIX-Italic)
    0xeec02019,   // [2165:14] period + quoteright : -35 (cslSTIX-Italic) ;;
    0xf2002039,   // [2166:15] A + guilsinglleft : -28 (cslSTIX-Italic)
    0xd980201d,   // [2167:16] A + quotedblright : -77 (cslSTIX-Italic)
    0xd9802019,   // [2168:17] A + quoteright : -77 (cslSTIX-Italic)
    0xfb8000fc,   // [2169:18] A + udieresis : -9 (cslSTIX-Italic)
    0xfb8000fb,   // [2170:19] A + ucircumflex : -9 (cslSTIX-Italic)
    0xfb8000fa,   // [2171:20] A + uacute : -9 (cslSTIX-Italic)
    0xfb8000f9,   // [2172:21] A + ugrave : -9 (cslSTIX-Italic)
    0xf80000f6,   // [2173:22] A + odieresis : -16 (cslSTIX-Italic)
    0xf68000e7,   // [2174:23] A + ccedilla : -19 (cslSTIX-Italic)
    0xe90000dc,   // [2175:24] A + Udieresis : -46 (cslSTIX-Italic)
    0xe90000db,   // [2176:25] A + Ucircumflex : -46 (cslSTIX-Italic)
    0xe90000da,   // [2177:26] A + Uacute : -46 (cslSTIX-Italic)
    0xe90000d9,   // [2178:27] A + Ugrave : -46 (cslSTIX-Italic)
    0xed8000d6,   // [2179:28] A + Odieresis : -37 (cslSTIX-Italic)
    0xee0000c7,   // [2180:29] A + Ccedilla : -36 (cslSTIX-Italic)
    0xf18000ab,   // [2181:30] A + guillemotleft : -29 (cslSTIX-Italic)
    0xe5000079,   // [2182:31] A + y : -54 (cslSTIX-Italic)
    0xf0000077,   // [2183:32] A + w : -32 (cslSTIX-Italic)
    0xee800076,   // [2184:33] A + v : -35 (cslSTIX-Italic)
    0xfb800075,   // [2185:34] A + u : -9 (cslSTIX-Italic)
    0xfc800074,   // [2186:35] A + t : -7 (cslSTIX-Italic)
    0xfa800071,   // [2187:36] A + q : -11 (cslSTIX-Italic)
    0xf800006f,   // [2188:37] A + o : -16 (cslSTIX-Italic)
    0xf0800067,   // [2189:38] A + g : -31 (cslSTIX-Italic)
    0xf7000065,   // [2190:39] A + e : -18 (cslSTIX-Italic)
    0xfe000064,   // [2191:40] A + d : -4 (cslSTIX-Italic)
    0xf7800063,   // [2192:41] A + c : -17 (cslSTIX-Italic)
    0xfd800061,   // [2193:42] A + a : -5 (cslSTIX-Italic)
    0xf6000059,   // [2194:43] A + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2195:44] A + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [2196:45] A + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [2197:46] A + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [2198:47] A + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2199:48] A + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2200:49] A + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2201:50] A + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2202:51] A + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [2203:52] A + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [2204:53] A + comma : 19 (cslSTIX-Italic) ;;
    0x04000152,   // [2205:54] B + OE : 8 (cslSTIX-Italic)
    0xf00000c6,   // [2206:55] B + AE : -32 (cslSTIX-Italic)
    0xf60000c5,   // [2207:56] B + Aring : -20 (cslSTIX-Italic)
    0xf60000c4,   // [2208:57] B + Adieresis : -20 (cslSTIX-Italic)
    0xf60000c3,   // [2209:58] B + Atilde : -20 (cslSTIX-Italic)
    0xf60000c2,   // [2210:59] B + Acircumflex : -20 (cslSTIX-Italic)
    0xf60000c1,   // [2211:60] B + Aacute : -20 (cslSTIX-Italic)
    0xef000059,   // [2212:61] B + Y : -34 (cslSTIX-Italic)
    0xf6800057,   // [2213:62] B + W : -19 (cslSTIX-Italic)
    0xf3800056,   // [2214:63] B + V : -25 (cslSTIX-Italic)
    0xf6400041,   // [2215:64] B + A : -20 (cslSTIX-Italic) ;;
    0xfc8000d6,   // [2216:65] C + Odieresis : -7 (cslSTIX-Italic)
    0xfc8000d3,   // [2217:66] C + Oacute : -7 (cslSTIX-Italic)
    0xfc8000d2,   // [2218:67] C + Ograve : -7 (cslSTIX-Italic)
    0xf08000c6,   // [2219:68] C + AE : -31 (cslSTIX-Italic)
    0xf68000c5,   // [2220:69] C + Aring : -19 (cslSTIX-Italic)
    0xf68000c4,   // [2221:70] C + Adieresis : -19 (cslSTIX-Italic)
    0xf68000c1,   // [2222:71] C + Aacute : -19 (cslSTIX-Italic)
    0xf68000c0,   // [2223:72] C + Agrave : -19 (cslSTIX-Italic)
    0xfc80004f,   // [2224:73] C + O : -7 (cslSTIX-Italic)
    0xf780004b,   // [2225:74] C + K : -17 (cslSTIX-Italic)
    0xfd800048,   // [2226:75] C + H : -5 (cslSTIX-Italic)
    0xf6c00041,   // [2227:76] C + A : -19 (cslSTIX-Italic) ;;
    0xf60000c5,   // [2228:77] D + Aring : -20 (cslSTIX-Italic)
    0xf60000c4,   // [2229:78] D + Adieresis : -20 (cslSTIX-Italic)
    0xf60000c3,   // [2230:79] D + Atilde : -20 (cslSTIX-Italic)
    0xf60000c2,   // [2231:80] D + Acircumflex : -20 (cslSTIX-Italic)
    0xf60000c1,   // [2232:81] D + Aacute : -20 (cslSTIX-Italic)
    0xf60000c0,   // [2233:82] D + Agrave : -20 (cslSTIX-Italic)
    0xed800059,   // [2234:83] D + Y : -37 (cslSTIX-Italic)
    0xf3000058,   // [2235:84] D + X : -26 (cslSTIX-Italic)
    0xf5000057,   // [2236:85] D + W : -22 (cslSTIX-Italic)
    0xf2000056,   // [2237:86] D + V : -28 (cslSTIX-Italic)
    0xfb800054,   // [2238:87] D + T : -9 (cslSTIX-Italic)
    0xf280004a,   // [2239:88] D + J : -27 (cslSTIX-Italic)
    0xf6400041,   // [2240:89] D + A : -20 (cslSTIX-Italic) ;;
    0xe1000153,   // [2241:90] F + oe : -62 (cslSTIX-Italic)
    0xdf0000f8,   // [2242:91] F + oslash : -66 (cslSTIX-Italic)
    0xea0000f6,   // [2243:92] F + odieresis : -44 (cslSTIX-Italic)
    0xdf8000f3,   // [2244:93] F + oacute : -65 (cslSTIX-Italic)
    0xde8000e9,   // [2245:94] F + eacute : -67 (cslSTIX-Italic)
    0xdd8000e6,   // [2246:95] F + ae : -69 (cslSTIX-Italic)
    0xe20000e5,   // [2247:96] F + aring : -60 (cslSTIX-Italic)
    0xe30000e4,   // [2248:97] F + adieresis : -58 (cslSTIX-Italic)
    0xe38000e3,   // [2249:98] F + atilde : -57 (cslSTIX-Italic)
    0xe20000e2,   // [2250:99] F + acircumflex : -60 (cslSTIX-Italic)
    0xe20000e1,   // [2251:100] F + aacute : -60 (cslSTIX-Italic)
    0xf00000d6,   // [2252:101] F + Odieresis : -32 (cslSTIX-Italic)
    0xf00000d3,   // [2253:102] F + Oacute : -32 (cslSTIX-Italic)
    0xf00000d2,   // [2254:103] F + Ograve : -32 (cslSTIX-Italic)
    0xe50000c5,   // [2255:104] F + Aring : -54 (cslSTIX-Italic)
    0xe50000c4,   // [2256:105] F + Adieresis : -54 (cslSTIX-Italic)
    0xe50000c3,   // [2257:106] F + Atilde : -54 (cslSTIX-Italic)
    0xe50000c2,   // [2258:107] F + Acircumflex : -54 (cslSTIX-Italic)
    0xe50000c1,   // [2259:108] F + Aacute : -54 (cslSTIX-Italic)
    0xe50000c0,   // [2260:109] F + Agrave : -54 (cslSTIX-Italic)
    0xea800075,   // [2261:110] F + u : -43 (cslSTIX-Italic)
    0xe9800072,   // [2262:111] F + r : -45 (cslSTIX-Italic)
    0xdf80006f,   // [2263:112] F + o : -65 (cslSTIX-Italic)
    0xed00006a,   // [2264:113] F + j : -38 (cslSTIX-Italic)
    0xee800069,   // [2265:114] F + i : -35 (cslSTIX-Italic)
    0xde800065,   // [2266:115] F + e : -67 (cslSTIX-Italic)
    0xe2000061,   // [2267:116] F + a : -60 (cslSTIX-Italic)
    0xf000004f,   // [2268:117] F + O : -32 (cslSTIX-Italic)
    0xe280004a,   // [2269:118] F + J : -59 (cslSTIX-Italic)
    0xe5000041,   // [2270:119] F + A : -54 (cslSTIX-Italic)
    0xd800002e,   // [2271:120] F + period : -80 (cslSTIX-Italic)
    0xf000002d,   // [2272:121] F + hyphen : -32 (cslSTIX-Italic)
    0xd9c0002c,   // [2273:122] F + comma : -77 (cslSTIX-Italic) ;;
    0xf18000c6,   // [2274:123] G + AE : -29 (cslSTIX-Italic)
    0xf88000c5,   // [2275:124] G + Aring : -15 (cslSTIX-Italic)
    0xf88000c4,   // [2276:125] G + Adieresis : -15 (cslSTIX-Italic)
    0xf88000c3,   // [2277:126] G + Atilde : -15 (cslSTIX-Italic)
    0xf88000c2,   // [2278:127] G + Acircumflex : -15 (cslSTIX-Italic)
    0xf88000c1,   // [2279:128] G + Aacute : -15 (cslSTIX-Italic)
    0xf88000c0,   // [2280:129] G + Agrave : -15 (cslSTIX-Italic)
    0xf9800059,   // [2281:130] G + Y : -13 (cslSTIX-Italic)
    0xfe800057,   // [2282:131] G + W : -3 (cslSTIX-Italic)
    0xfd800056,   // [2283:132] G + V : -5 (cslSTIX-Italic)
    0xf9000054,   // [2284:133] G + T : -14 (cslSTIX-Italic)
    0xf8c00041,   // [2285:134] G + A : -15 (cslSTIX-Italic) ;;
    0x18e0004a,   // [2286:135] I + J ligature #49 (cslSTIX-Italic) ;;
    0xe50000c6,   // [2287:136] J + AE : -54 (cslSTIX-Italic)
    0xf60000c5,   // [2288:137] J + Aring : -20 (cslSTIX-Italic)
    0xf60000c4,   // [2289:138] J + Adieresis : -20 (cslSTIX-Italic)
    0xf6400041,   // [2290:139] J + A : -20 (cslSTIX-Italic) ;;
    0xeb000152,   // [2291:140] K + OE : -42 (cslSTIX-Italic)
    0xfb8000fc,   // [2292:141] K + udieresis : -9 (cslSTIX-Italic)
    0xf80000f6,   // [2293:142] K + odieresis : -16 (cslSTIX-Italic)
    0xf80000f3,   // [2294:143] K + oacute : -16 (cslSTIX-Italic)
    0xfd8000e6,   // [2295:144] K + ae : -5 (cslSTIX-Italic)
    0xfd8000e5,   // [2296:145] K + aring : -5 (cslSTIX-Italic)
    0xfd8000e4,   // [2297:146] K + adieresis : -5 (cslSTIX-Italic)
    0xea0000d6,   // [2298:147] K + Odieresis : -44 (cslSTIX-Italic)
    0xea0000d3,   // [2299:148] K + Oacute : -44 (cslSTIX-Italic)
    0xe0800079,   // [2300:149] K + y : -63 (cslSTIX-Italic)
    0xfb800075,   // [2301:150] K + u : -9 (cslSTIX-Italic)
    0xf800006f,   // [2302:151] K + o : -16 (cslSTIX-Italic)
    0xf7000065,   // [2303:152] K + e : -18 (cslSTIX-Italic)
    0xfd800061,   // [2304:153] K + a : -5 (cslSTIX-Italic)
    0x09800054,   // [2305:154] K + T : 19 (cslSTIX-Italic)
    0x03000053,   // [2306:155] K + S : 6 (cslSTIX-Italic)
    0xea00004f,   // [2307:156] K + O : -44 (cslSTIX-Italic)
    0xed000047,   // [2308:157] K + G : -38 (cslSTIX-Italic)
    0xe9800043,   // [2309:158] K + C : -45 (cslSTIX-Italic)
    0xeb40002d,   // [2310:159] K + hyphen : -42 (cslSTIX-Italic) ;;
    0xd900201d,   // [2311:160] L + quotedblright : -78 (cslSTIX-Italic)
    0xd9002019,   // [2312:161] L + quoteright : -78 (cslSTIX-Italic)
    0x0a0000fc,   // [2313:162] L + udieresis : 20 (cslSTIX-Italic)
    0xfb8000dc,   // [2314:163] L + Udieresis : -9 (cslSTIX-Italic)
    0x050000d6,   // [2315:164] L + Odieresis : 10 (cslSTIX-Italic)
    0x050000d5,   // [2316:165] L + Otilde : 10 (cslSTIX-Italic)
    0x050000d4,   // [2317:166] L + Ocircumflex : 10 (cslSTIX-Italic)
    0x050000d3,   // [2318:167] L + Oacute : 10 (cslSTIX-Italic)
    0x050000d2,   // [2319:168] L + Ograve : 10 (cslSTIX-Italic)
    0x028000c7,   // [2320:169] L + Ccedilla : 5 (cslSTIX-Italic)
    0x0e8000c6,   // [2321:170] L + AE : 29 (cslSTIX-Italic)
    0x148000c5,   // [2322:171] L + Aring : 41 (cslSTIX-Italic)
    0x148000c4,   // [2323:172] L + Adieresis : 41 (cslSTIX-Italic)
    0x148000c1,   // [2324:173] L + Aacute : 41 (cslSTIX-Italic)
    0xf8800079,   // [2325:174] L + y : -15 (cslSTIX-Italic)
    0x0b000075,   // [2326:175] L + u : 22 (cslSTIX-Italic)
    0xf5800059,   // [2327:176] L + Y : -21 (cslSTIX-Italic)
    0xef000057,   // [2328:177] L + W : -34 (cslSTIX-Italic)
    0xeb800056,   // [2329:178] L + V : -41 (cslSTIX-Italic)
    0xfb800055,   // [2330:179] L + U : -9 (cslSTIX-Italic)
    0xf8800054,   // [2331:180] L + T : -15 (cslSTIX-Italic)
    0x09000053,   // [2332:181] L + S : 18 (cslSTIX-Italic)
    0x0500004f,   // [2333:182] L + O : 10 (cslSTIX-Italic)
    0x05800047,   // [2334:183] L + G : 11 (cslSTIX-Italic)
    0x02800043,   // [2335:184] L + C : 5 (cslSTIX-Italic)
    0x14800041,   // [2336:185] L + A : 41 (cslSTIX-Italic)
    0x1c40002d,   // [2337:186] L + hyphen : 56 (cslSTIX-Italic) ;;
    0xfa8000fc,   // [2338:187] N + udieresis : -11 (cslSTIX-Italic)
    0xf98000f8,   // [2339:188] N + oslash : -13 (cslSTIX-Italic)
    0xfa0000f6,   // [2340:189] N + odieresis : -12 (cslSTIX-Italic)
    0xfa0000f3,   // [2341:190] N + oacute : -12 (cslSTIX-Italic)
    0xf90000e9,   // [2342:191] N + eacute : -14 (cslSTIX-Italic)
    0xfa8000e6,   // [2343:192] N + ae : -11 (cslSTIX-Italic)
    0xfa0000e5,   // [2344:193] N + aring : -12 (cslSTIX-Italic)
    0xfa0000e4,   // [2345:194] N + adieresis : -12 (cslSTIX-Italic)
    0xfa0000e1,   // [2346:195] N + aacute : -12 (cslSTIX-Italic)
    0xff0000d6,   // [2347:196] N + Odieresis : -2 (cslSTIX-Italic)
    0xff0000d3,   // [2348:197] N + Oacute : -2 (cslSTIX-Italic)
    0xfd8000c7,   // [2349:198] N + Ccedilla : -5 (cslSTIX-Italic)
    0xf08000c6,   // [2350:199] N + AE : -31 (cslSTIX-Italic)
    0xf68000c5,   // [2351:200] N + Aring : -19 (cslSTIX-Italic)
    0xf68000c4,   // [2352:201] N + Adieresis : -19 (cslSTIX-Italic)
    0xf68000c1,   // [2353:202] N + Aacute : -19 (cslSTIX-Italic)
    0xfa800075,   // [2354:203] N + u : -11 (cslSTIX-Italic)
    0xfa00006f,   // [2355:204] N + o : -12 (cslSTIX-Italic)
    0xf9000065,   // [2356:205] N + e : -14 (cslSTIX-Italic)
    0xfc000061,   // [2357:206] N + a : -8 (cslSTIX-Italic)
    0xff00004f,   // [2358:207] N + O : -2 (cslSTIX-Italic)
    0x00800047,   // [2359:208] N + G : 1 (cslSTIX-Italic)
    0xfd800043,   // [2360:209] N + C : -5 (cslSTIX-Italic)
    0xf6800041,   // [2361:210] N + A : -19 (cslSTIX-Italic)
    0xfe00002e,   // [2362:211] N + period : -4 (cslSTIX-Italic)
    0x00c0002c,   // [2363:212] N + comma : 1 (cslSTIX-Italic) ;;
    0xe58000c6,   // [2364:213] O + AE : -53 (cslSTIX-Italic)
    0xf80000c5,   // [2365:214] O + Aring : -16 (cslSTIX-Italic)
    0xf80000c4,   // [2366:215] O + Adieresis : -16 (cslSTIX-Italic)
    0xf80000c1,   // [2367:216] O + Aacute : -16 (cslSTIX-Italic)
    0xef000059,   // [2368:217] O + Y : -34 (cslSTIX-Italic)
    0xf3000058,   // [2369:218] O + X : -26 (cslSTIX-Italic)
    0xf4800057,   // [2370:219] O + W : -23 (cslSTIX-Italic)
    0xf1800056,   // [2371:220] O + V : -29 (cslSTIX-Italic)
    0xfd800054,   // [2372:221] O + T : -5 (cslSTIX-Italic)
    0xf8400041,   // [2373:222] O + A : -16 (cslSTIX-Italic) ;;
    0xe2000153,   // [2374:223] P + oe : -60 (cslSTIX-Italic)
    0xea0000f8,   // [2375:224] P + oslash : -44 (cslSTIX-Italic)
    0xe98000f6,   // [2376:225] P + odieresis : -45 (cslSTIX-Italic)
    0xe50000f3,   // [2377:226] P + oacute : -54 (cslSTIX-Italic)
    0xdf0000e9,   // [2378:227] P + eacute : -66 (cslSTIX-Italic)
    0xde8000e6,   // [2379:228] P + ae : -67 (cslSTIX-Italic)
    0xe50000e5,   // [2380:229] P + aring : -54 (cslSTIX-Italic)
    0xe50000e4,   // [2381:230] P + adieresis : -54 (cslSTIX-Italic)
    0xe50000e1,   // [2382:231] P + aacute : -54 (cslSTIX-Italic)
    0xe50000e0,   // [2383:232] P + agrave : -54 (cslSTIX-Italic)
    0xcf0000c6,   // [2384:233] P + AE : -98 (cslSTIX-Italic)
    0xe10000c5,   // [2385:234] P + Aring : -62 (cslSTIX-Italic)
    0xe10000c4,   // [2386:235] P + Adieresis : -62 (cslSTIX-Italic)
    0xe10000c1,   // [2387:236] P + Aacute : -62 (cslSTIX-Italic)
    0xe500006f,   // [2388:237] P + o : -54 (cslSTIX-Italic)
    0xe1800065,   // [2389:238] P + e : -61 (cslSTIX-Italic)
    0xe5000061,   // [2390:239] P + a : -54 (cslSTIX-Italic)
    0xdc00004a,   // [2391:240] P + J : -72 (cslSTIX-Italic)
    0xe1000041,   // [2392:241] P + A : -62 (cslSTIX-Italic)
    0xce80002e,   // [2393:242] P + period : -99 (cslSTIX-Italic)
    0xe900002d,   // [2394:243] P + hyphen : -46 (cslSTIX-Italic)
    0xd040002c,   // [2395:244] P + comma : -96 (cslSTIX-Italic) ;;
    0xf8800153,   // [2396:245] R + oe : -15 (cslSTIX-Italic)
    0xfd800152,   // [2397:246] R + OE : -5 (cslSTIX-Italic)
    0xfc8000fc,   // [2398:247] R + udieresis : -7 (cslSTIX-Italic)
    0xfc8000fa,   // [2399:248] R + uacute : -7 (cslSTIX-Italic)
    0xf90000f6,   // [2400:249] R + odieresis : -14 (cslSTIX-Italic)
    0xf90000f3,   // [2401:250] R + oacute : -14 (cslSTIX-Italic)
    0xf80000e9,   // [2402:251] R + eacute : -16 (cslSTIX-Italic)
    0xfe8000e6,   // [2403:252] R + ae : -3 (cslSTIX-Italic)
    0xfe0000e5,   // [2404:253] R + aring : -4 (cslSTIX-Italic)
    0xfe0000e4,   // [2405:254] R + adieresis : -4 (cslSTIX-Italic)
    0xfe0000e1,   // [2406:255] R + aacute : -4 (cslSTIX-Italic)
    0xf48000dc,   // [2407:256] R + Udieresis : -23 (cslSTIX-Italic)
    0xf90000d6,   // [2408:257] R + Odieresis : -14 (cslSTIX-Italic)
    0xf90000d3,   // [2409:258] R + Oacute : -14 (cslSTIX-Italic)
    0xf98000c7,   // [2410:259] R + Ccedilla : -13 (cslSTIX-Italic)
    0xff800079,   // [2411:260] R + y : -1 (cslSTIX-Italic)
    0xfc800075,   // [2412:261] R + u : -7 (cslSTIX-Italic)
    0xf900006f,   // [2413:262] R + o : -14 (cslSTIX-Italic)
    0xf8000065,   // [2414:263] R + e : -16 (cslSTIX-Italic)
    0xfe000061,   // [2415:264] R + a : -4 (cslSTIX-Italic)
    0xf6800059,   // [2416:265] R + Y : -19 (cslSTIX-Italic)
    0xf1000057,   // [2417:266] R + W : -30 (cslSTIX-Italic)
    0xee000056,   // [2418:267] R + V : -36 (cslSTIX-Italic)
    0xf4800055,   // [2419:268] R + U : -23 (cslSTIX-Italic)
    0xff800054,   // [2420:269] R + T : -1 (cslSTIX-Italic)
    0xf900004f,   // [2421:270] R + O : -14 (cslSTIX-Italic)
    0xfd000047,   // [2422:271] R + G : -6 (cslSTIX-Italic)
    0xf9800043,   // [2423:272] R + C : -13 (cslSTIX-Italic)
    0xf840002d,   // [2424:273] R + hyphen : -16 (cslSTIX-Italic) ;;
    0xf88000c6,   // [2425:274] S + AE : -15 (cslSTIX-Italic)
    0xfe8000c5,   // [2426:275] S + Aring : -3 (cslSTIX-Italic)
    0xfe8000c4,   // [2427:276] S + Adieresis : -3 (cslSTIX-Italic)
    0xfe8000c1,   // [2428:277] S + Aacute : -3 (cslSTIX-Italic)
    0xff800074,   // [2429:278] S + t : -1 (cslSTIX-Italic)
    0xff000059,   // [2430:279] S + Y : -2 (cslSTIX-Italic)
    0x03800057,   // [2431:280] S + W : 7 (cslSTIX-Italic)
    0x02000056,   // [2432:281] S + V : 4 (cslSTIX-Italic)
    0xfec00041,   // [2433:282] S + A : -3 (cslSTIX-Italic) ;;
    0xd6802039,   // [2434:283] T + guilsinglleft : -83 (cslSTIX-Italic)
    0xfd000152,   // [2435:284] T + OE : -6 (cslSTIX-Italic)
    0xdb8000f8,   // [2436:285] T + oslash : -73 (cslSTIX-Italic)
    0xf38000f6,   // [2437:286] T + odieresis : -25 (cslSTIX-Italic)
    0xef0000f4,   // [2438:287] T + ocircumflex : -34 (cslSTIX-Italic)
    0xe38000f3,   // [2439:288] T + oacute : -57 (cslSTIX-Italic)
    0xec0000f2,   // [2440:289] T + ograve : -40 (cslSTIX-Italic)
    0xdc8000e6,   // [2441:290] T + ae : -71 (cslSTIX-Italic)
    0xec8000e4,   // [2442:291] T + adieresis : -39 (cslSTIX-Italic)
    0xec0000e3,   // [2443:292] T + atilde : -40 (cslSTIX-Italic)
    0xe80000e2,   // [2444:293] T + acircumflex : -48 (cslSTIX-Italic)
    0xdd0000e1,   // [2445:294] T + aacute : -70 (cslSTIX-Italic)
    0xe50000e0,   // [2446:295] T + agrave : -54 (cslSTIX-Italic)
    0xf48000d8,   // [2447:296] T + Oslash : -23 (cslSTIX-Italic)
    0xf60000d6,   // [2448:297] T + Odieresis : -20 (cslSTIX-Italic)
    0xf60000d5,   // [2449:298] T + Otilde : -20 (cslSTIX-Italic)
    0xf60000d4,   // [2450:299] T + Ocircumflex : -20 (cslSTIX-Italic)
    0xf60000d3,   // [2451:300] T + Oacute : -20 (cslSTIX-Italic)
    0xf60000d2,   // [2452:301] T + Ograve : -20 (cslSTIX-Italic)
    0xe98000c6,   // [2453:302] T + AE : -45 (cslSTIX-Italic)
    0xef8000c5,   // [2454:303] T + Aring : -33 (cslSTIX-Italic)
    0xef8000c4,   // [2455:304] T + Adieresis : -33 (cslSTIX-Italic)
    0xef8000c3,   // [2456:305] T + Atilde : -33 (cslSTIX-Italic)
    0xef8000c2,   // [2457:306] T + Acircumflex : -33 (cslSTIX-Italic)
    0xef8000c1,   // [2458:307] T + Aacute : -33 (cslSTIX-Italic)
    0xef8000c0,   // [2459:308] T + Agrave : -33 (cslSTIX-Italic)
    0xd60000ab,   // [2460:309] T + guillemotleft : -84 (cslSTIX-Italic)
    0xde000079,   // [2461:310] T + y : -68 (cslSTIX-Italic)
    0xde800077,   // [2462:311] T + w : -67 (cslSTIX-Italic)
    0xdd800076,   // [2463:312] T + v : -69 (cslSTIX-Italic)
    0xde000075,   // [2464:313] T + u : -68 (cslSTIX-Italic)
    0xdb800073,   // [2465:314] T + s : -73 (cslSTIX-Italic)
    0xd7800072,   // [2466:315] T + r : -81 (cslSTIX-Italic)
    0xdb80006f,   // [2467:316] T + o : -73 (cslSTIX-Italic)
    0xf780006a,   // [2468:317] T + j : -17 (cslSTIX-Italic)
    0xf8800069,   // [2469:318] T + i : -15 (cslSTIX-Italic)
    0xd8000067,   // [2470:319] T + g : -80 (cslSTIX-Italic)
    0xda800065,   // [2471:320] T + e : -75 (cslSTIX-Italic)
    0xde000063,   // [2472:321] T + c : -68 (cslSTIX-Italic)
    0xde000061,   // [2473:322] T + a : -68 (cslSTIX-Italic)
    0x0f800059,   // [2474:323] T + Y : 31 (cslSTIX-Italic)
    0x14800057,   // [2475:324] T + W : 41 (cslSTIX-Italic)
    0x13800056,   // [2476:325] T + V : 39 (cslSTIX-Italic)
    0xff800053,   // [2477:326] T + S : -1 (cslSTIX-Italic)
    0xf600004f,   // [2478:327] T + O : -20 (cslSTIX-Italic)
    0xed00004a,   // [2479:328] T + J : -38 (cslSTIX-Italic)
    0xfe000047,   // [2480:329] T + G : -4 (cslSTIX-Italic)
    0xf9800043,   // [2481:330] T + C : -13 (cslSTIX-Italic)
    0xef800041,   // [2482:331] T + A : -33 (cslSTIX-Italic)
    0xeb80003b,   // [2483:332] T + semicolon : -41 (cslSTIX-Italic)
    0xe580003a,   // [2484:333] T + colon : -53 (cslSTIX-Italic)
    0xe280002e,   // [2485:334] T + period : -59 (cslSTIX-Italic)
    0xe580002d,   // [2486:335] T + hyphen : -53 (cslSTIX-Italic)
    0xe440002c,   // [2487:336] T + comma : -56 (cslSTIX-Italic) ;;
    0xde0000c6,   // [2488:337] U + AE : -68 (cslSTIX-Italic)
    0xed0000c5,   // [2489:338] U + Aring : -38 (cslSTIX-Italic)
    0xed0000c4,   // [2490:339] U + Adieresis : -38 (cslSTIX-Italic)
    0xed0000c3,   // [2491:340] U + Atilde : -38 (cslSTIX-Italic)
    0xed0000c2,   // [2492:341] U + Acircumflex : -38 (cslSTIX-Italic)
    0xed0000c1,   // [2493:342] U + Aacute : -38 (cslSTIX-Italic)
    0xef000072,   // [2494:343] U + r : -34 (cslSTIX-Italic)
    0xf4800070,   // [2495:344] U + p : -23 (cslSTIX-Italic)
    0xf680006e,   // [2496:345] U + n : -19 (cslSTIX-Italic)
    0xf700006d,   // [2497:346] U + m : -18 (cslSTIX-Italic)
    0xed000041,   // [2498:347] U + A : -38 (cslSTIX-Italic)
    0xf500002e,   // [2499:348] U + period : -22 (cslSTIX-Italic)
    0xf840002c,   // [2500:349] U + comma : -16 (cslSTIX-Italic) ;;
    0xdf802039,   // [2501:350] V + guilsinglleft : -65 (cslSTIX-Italic)
    0xe40000f8,   // [2502:351] V + oslash : -56 (cslSTIX-Italic)
    0xe80000f6,   // [2503:352] V + odieresis : -48 (cslSTIX-Italic)
    0xe58000f5,   // [2504:353] V + otilde : -53 (cslSTIX-Italic)
    0xe48000f4,   // [2505:354] V + ocircumflex : -55 (cslSTIX-Italic)
    0xe48000f3,   // [2506:355] V + oacute : -55 (cslSTIX-Italic)
    0xec0000f2,   // [2507:356] V + ograve : -40 (cslSTIX-Italic)
    0xe38000e9,   // [2508:357] V + eacute : -57 (cslSTIX-Italic)
    0xed0000e8,   // [2509:358] V + egrave : -38 (cslSTIX-Italic)
    0xe58000e6,   // [2510:359] V + ae : -53 (cslSTIX-Italic)
    0xe68000e5,   // [2511:360] V + aring : -51 (cslSTIX-Italic)
    0xe68000e4,   // [2512:361] V + adieresis : -51 (cslSTIX-Italic)
    0xe68000e3,   // [2513:362] V + atilde : -51 (cslSTIX-Italic)
    0xe68000e2,   // [2514:363] V + acircumflex : -51 (cslSTIX-Italic)
    0xe68000e1,   // [2515:364] V + aacute : -51 (cslSTIX-Italic)
    0xe68000e0,   // [2516:365] V + agrave : -51 (cslSTIX-Italic)
    0xf08000d8,   // [2517:366] V + Oslash : -31 (cslSTIX-Italic)
    0xf08000d6,   // [2518:367] V + Odieresis : -31 (cslSTIX-Italic)
    0xf08000d5,   // [2519:368] V + Otilde : -31 (cslSTIX-Italic)
    0xf08000d4,   // [2520:369] V + Ocircumflex : -31 (cslSTIX-Italic)
    0xf08000d3,   // [2521:370] V + Oacute : -31 (cslSTIX-Italic)
    0xf08000d2,   // [2522:371] V + Ograve : -31 (cslSTIX-Italic)
    0xd68000c6,   // [2523:372] V + AE : -83 (cslSTIX-Italic)
    0xe60000c5,   // [2524:373] V + Aring : -52 (cslSTIX-Italic)
    0xe60000c4,   // [2525:374] V + Adieresis : -52 (cslSTIX-Italic)
    0xe60000c3,   // [2526:375] V + Atilde : -52 (cslSTIX-Italic)
    0xe60000c2,   // [2527:376] V + Acircumflex : -52 (cslSTIX-Italic)
    0xe60000c1,   // [2528:377] V + Aacute : -52 (cslSTIX-Italic)
    0xe60000c0,   // [2529:378] V + Agrave : -52 (cslSTIX-Italic)
    0xdf0000ab,   // [2530:379] V + guillemotleft : -66 (cslSTIX-Italic)
    0xf8000079,   // [2531:380] V + y : -16 (cslSTIX-Italic)
    0xed800075,   // [2532:381] V + u : -37 (cslSTIX-Italic)
    0xeb800072,   // [2533:382] V + r : -41 (cslSTIX-Italic)
    0xe480006f,   // [2534:383] V + o : -55 (cslSTIX-Italic)
    0xf7000069,   // [2535:384] V + i : -18 (cslSTIX-Italic)
    0xda800067,   // [2536:385] V + g : -75 (cslSTIX-Italic)
    0xe3800065,   // [2537:386] V + e : -57 (cslSTIX-Italic)
    0xe6800061,   // [2538:387] V + a : -51 (cslSTIX-Italic)
    0x0f000054,   // [2539:388] V + T : 30 (cslSTIX-Italic)
    0xf7800053,   // [2540:389] V + S : -17 (cslSTIX-Italic)
    0xf080004f,   // [2541:390] V + O : -31 (cslSTIX-Italic)
    0xf5000047,   // [2542:391] V + G : -22 (cslSTIX-Italic)
    0xf1800043,   // [2543:392] V + C : -29 (cslSTIX-Italic)
    0xe6000041,   // [2544:393] V + A : -52 (cslSTIX-Italic)
    0xf500003b,   // [2545:394] V + semicolon : -22 (cslSTIX-Italic)
    0xef00003a,   // [2546:395] V + colon : -34 (cslSTIX-Italic)
    0xdf80002e,   // [2547:396] V + period : -65 (cslSTIX-Italic)
    0xf080002d,   // [2548:397] V + hyphen : -31 (cslSTIX-Italic)
    0xe440002c,   // [2549:398] V + comma : -56 (cslSTIX-Italic) ;;
    0xe7002039,   // [2550:399] W + guilsinglleft : -50 (cslSTIX-Italic)
    0xeb8000f8,   // [2551:400] W + oslash : -41 (cslSTIX-Italic)
    0xeb8000f6,   // [2552:401] W + odieresis : -41 (cslSTIX-Italic)
    0xeb8000f5,   // [2553:402] W + otilde : -41 (cslSTIX-Italic)
    0xeb8000f4,   // [2554:403] W + ocircumflex : -41 (cslSTIX-Italic)
    0xeb8000f3,   // [2555:404] W + oacute : -41 (cslSTIX-Italic)
    0xeb8000f2,   // [2556:405] W + ograve : -41 (cslSTIX-Italic)
    0xea8000e9,   // [2557:406] W + eacute : -43 (cslSTIX-Italic)
    0xec0000e8,   // [2558:407] W + egrave : -40 (cslSTIX-Italic)
    0xea0000e6,   // [2559:408] W + ae : -44 (cslSTIX-Italic)
    0xeb0000e5,   // [2560:409] W + aring : -42 (cslSTIX-Italic)
    0xeb0000e4,   // [2561:410] W + adieresis : -42 (cslSTIX-Italic)
    0xeb0000e3,   // [2562:411] W + atilde : -42 (cslSTIX-Italic)
    0xeb0000e2,   // [2563:412] W + acircumflex : -42 (cslSTIX-Italic)
    0xeb0000e1,   // [2564:413] W + aacute : -42 (cslSTIX-Italic)
    0xeb0000e0,   // [2565:414] W + agrave : -42 (cslSTIX-Italic)
    0xf48000d8,   // [2566:415] W + Oslash : -23 (cslSTIX-Italic)
    0xf48000d6,   // [2567:416] W + Odieresis : -23 (cslSTIX-Italic)
    0xf48000d5,   // [2568:417] W + Otilde : -23 (cslSTIX-Italic)
    0xf48000d4,   // [2569:418] W + Ocircumflex : -23 (cslSTIX-Italic)
    0xf48000d3,   // [2570:419] W + Oacute : -23 (cslSTIX-Italic)
    0xf48000d2,   // [2571:420] W + Ograve : -23 (cslSTIX-Italic)
    0xd88000c6,   // [2572:421] W + AE : -79 (cslSTIX-Italic)
    0xeb0000c5,   // [2573:422] W + Aring : -42 (cslSTIX-Italic)
    0xeb0000c4,   // [2574:423] W + Adieresis : -42 (cslSTIX-Italic)
    0xeb0000c3,   // [2575:424] W + Atilde : -42 (cslSTIX-Italic)
    0xeb0000c2,   // [2576:425] W + Acircumflex : -42 (cslSTIX-Italic)
    0xeb0000c1,   // [2577:426] W + Aacute : -42 (cslSTIX-Italic)
    0xeb0000c0,   // [2578:427] W + Agrave : -42 (cslSTIX-Italic)
    0xe68000ab,   // [2579:428] W + guillemotleft : -51 (cslSTIX-Italic)
    0xfa000079,   // [2580:429] W + y : -12 (cslSTIX-Italic)
    0xf0000075,   // [2581:430] W + u : -32 (cslSTIX-Italic)
    0xed800072,   // [2582:431] W + r : -37 (cslSTIX-Italic)
    0xeb80006f,   // [2583:432] W + o : -41 (cslSTIX-Italic)
    0xf6800069,   // [2584:433] W + i : -19 (cslSTIX-Italic)
    0xe2000067,   // [2585:434] W + g : -60 (cslSTIX-Italic)
    0xea800065,   // [2586:435] W + e : -43 (cslSTIX-Italic)
    0xeb000061,   // [2587:436] W + a : -42 (cslSTIX-Italic)
    0x0e000054,   // [2588:437] W + T : 28 (cslSTIX-Italic)
    0xf7800053,   // [2589:438] W + S : -17 (cslSTIX-Italic)
    0xf480004f,   // [2590:439] W + O : -23 (cslSTIX-Italic)
    0xf9000047,   // [2591:440] W + G : -14 (cslSTIX-Italic)
    0xf5800043,   // [2592:441] W + C : -21 (cslSTIX-Italic)
    0xeb000041,   // [2593:442] W + A : -42 (cslSTIX-Italic)
    0xfa00003b,   // [2594:443] W + semicolon : -12 (cslSTIX-Italic)
    0xf400003a,   // [2595:444] W + colon : -24 (cslSTIX-Italic)
    0xea00002e,   // [2596:445] W + period : -44 (cslSTIX-Italic)
    0xf700002d,   // [2597:446] W + hyphen : -18 (cslSTIX-Italic)
    0xec40002c,   // [2598:447] W + comma : -40 (cslSTIX-Italic) ;;
    0xf00000d6,   // [2599:448] X + Odieresis : -32 (cslSTIX-Italic)
    0xdf800079,   // [2600:449] X + y : -65 (cslSTIX-Italic)
    0xfc800075,   // [2601:450] X + u : -7 (cslSTIX-Italic)
    0xf900006f,   // [2602:451] X + o : -14 (cslSTIX-Italic)
    0xf8000065,   // [2603:452] X + e : -16 (cslSTIX-Italic)
    0xfe800061,   // [2604:453] X + a : -3 (cslSTIX-Italic)
    0xed800051,   // [2605:454] X + Q : -37 (cslSTIX-Italic)
    0xf000004f,   // [2606:455] X + O : -32 (cslSTIX-Italic)
    0xee800043,   // [2607:456] X + C : -35 (cslSTIX-Italic)
    0xf2c0002d,   // [2608:457] X + hyphen : -27 (cslSTIX-Italic) ;;
    0xd9802039,   // [2609:458] Y + guilsinglleft : -77 (cslSTIX-Italic)
    0xe10000f8,   // [2610:459] Y + oslash : -62 (cslSTIX-Italic)
    0xe28000e6,   // [2611:460] Y + ae : -59 (cslSTIX-Italic)
    0xee0000d8,   // [2612:461] Y + Oslash : -36 (cslSTIX-Italic)
    0xee0000d6,   // [2613:462] Y + Odieresis : -36 (cslSTIX-Italic)
    0xee0000d5,   // [2614:463] Y + Otilde : -36 (cslSTIX-Italic)
    0xee0000d4,   // [2615:464] Y + Ocircumflex : -36 (cslSTIX-Italic)
    0xee0000d3,   // [2616:465] Y + Oacute : -36 (cslSTIX-Italic)
    0xee0000d2,   // [2617:466] Y + Ograve : -36 (cslSTIX-Italic)
    0xed0000c6,   // [2618:467] Y + AE : -38 (cslSTIX-Italic)
    0xf28000c5,   // [2619:468] Y + Aring : -27 (cslSTIX-Italic)
    0xf28000c4,   // [2620:469] Y + Adieresis : -27 (cslSTIX-Italic)
    0xf28000c3,   // [2621:470] Y + Atilde : -27 (cslSTIX-Italic)
    0xf28000c2,   // [2622:471] Y + Acircumflex : -27 (cslSTIX-Italic)
    0xf28000c1,   // [2623:472] Y + Aacute : -27 (cslSTIX-Italic)
    0xf28000c0,   // [2624:473] Y + Agrave : -27 (cslSTIX-Italic)
    0xd90000ab,   // [2625:474] Y + guillemotleft : -78 (cslSTIX-Italic)
    0xf0800076,   // [2626:475] Y + v : -31 (cslSTIX-Italic)
    0xe5800075,   // [2627:476] Y + u : -53 (cslSTIX-Italic)
    0xe5800070,   // [2628:477] Y + p : -53 (cslSTIX-Italic)
    0xe180006f,   // [2629:478] Y + o : -61 (cslSTIX-Italic)
    0xf8000069,   // [2630:479] Y + i : -16 (cslSTIX-Italic)
    0xd7000067,   // [2631:480] Y + g : -82 (cslSTIX-Italic)
    0xe0800065,   // [2632:481] Y + e : -63 (cslSTIX-Italic)
    0xe4000061,   // [2633:482] Y + a : -56 (cslSTIX-Italic)
    0x0f000054,   // [2634:483] Y + T : 30 (cslSTIX-Italic)
    0xf8800053,   // [2635:484] Y + S : -15 (cslSTIX-Italic)
    0xee00004f,   // [2636:485] Y + O : -36 (cslSTIX-Italic)
    0xf2800047,   // [2637:486] Y + G : -27 (cslSTIX-Italic)
    0xef000043,   // [2638:487] Y + C : -34 (cslSTIX-Italic)
    0xf2800041,   // [2639:488] Y + A : -27 (cslSTIX-Italic)
    0xf180003b,   // [2640:489] Y + semicolon : -29 (cslSTIX-Italic)
    0xec00003a,   // [2641:490] Y + colon : -40 (cslSTIX-Italic)
    0xe580002e,   // [2642:491] Y + period : -53 (cslSTIX-Italic)
    0xe880002d,   // [2643:492] Y + hyphen : -47 (cslSTIX-Italic)
    0xe740002c,   // [2644:493] Y + comma : -50 (cslSTIX-Italic) ;;
    0xfc800079,   // [2645:494] Z + y : -7 (cslSTIX-Italic)
    0x02c00077,   // [2646:495] Z + w : 5 (cslSTIX-Italic) ;;
    0xf5002019,   // [2647:496] a + quoteright : -22 (cslSTIX-Italic)
    0x01000079,   // [2648:497] a + y : 2 (cslSTIX-Italic)
    0x01800077,   // [2649:498] a + w : 3 (cslSTIX-Italic)
    0xfec0006a,   // [2650:499] a + j : -3 (cslSTIX-Italic) ;;
    0xff000079,   // [2651:500] b + y : -2 (cslSTIX-Italic)
    0xfd800077,   // [2652:501] b + w : -5 (cslSTIX-Italic)
    0xfcc00076,   // [2653:502] b + v : -7 (cslSTIX-Italic) ;;
    0xf200006b,   // [2654:503] c + k : -28 (cslSTIX-Italic)
    0xf3400068,   // [2655:504] c + h : -26 (cslSTIX-Italic) ;;
    0xf8002019,   // [2656:505] e + quoteright : -16 (cslSTIX-Italic)
    0x01800079,   // [2657:506] e + y : 3 (cslSTIX-Italic)
    0xfa000078,   // [2658:507] e + x : -12 (cslSTIX-Italic)
    0x01000077,   // [2659:508] e + w : 2 (cslSTIX-Italic)
    0xf8c00074,   // [2660:509] e + t : -15 (cslSTIX-Italic) ;;
    0x08802019,   // [2661:510] f + quoteright : 17 (cslSTIX-Italic)
    0xfb000153,   // [2662:511] f + oe : -10 (cslSTIX-Italic)
    0xf60000f8,   // [2663:512] f + oslash : -20 (cslSTIX-Italic)
    0x018000f6,   // [2664:513] f + odieresis : 3 (cslSTIX-Italic)
    0xf68000f4,   // [2665:514] f + ocircumflex : -19 (cslSTIX-Italic)
    0xf68000f3,   // [2666:515] f + oacute : -19 (cslSTIX-Italic)
    0xfa8000f2,   // [2667:516] f + ograve : -11 (cslSTIX-Italic)
    0xf58000ea,   // [2668:517] f + ecircumflex : -21 (cslSTIX-Italic)
    0xf58000e9,   // [2669:518] f + eacute : -21 (cslSTIX-Italic)
    0xfb8000e8,   // [2670:519] f + egrave : -9 (cslSTIX-Italic)
    0xf78000e6,   // [2671:520] f + ae : -17 (cslSTIX-Italic)
    0xf90000e5,   // [2672:521] f + aring : -14 (cslSTIX-Italic)
    0xfb0000e4,   // [2673:522] f + adieresis : -10 (cslSTIX-Italic)
    0xf90000e1,   // [2674:523] f + aacute : -14 (cslSTIX-Italic)
    0x09000074,   // [2675:524] f + t : 18 (cslSTIX-Italic)
    0xf7800073,   // [2676:525] f + s : -17 (cslSTIX-Italic)
    0xf900006f,   // [2677:526] f + o : -14 (cslSTIX-Italic)
    0x1480006c,   // [2678:527] f + l : 41 (cslSTIX-Italic)
    0x0700006a,   // [2679:528] f + j : 14 (cslSTIX-Italic)
    0x08000069,   // [2680:529] f + i : 16 (cslSTIX-Italic)
    0x0e800066,   // [2681:530] f + f : 29 (cslSTIX-Italic)
    0xf5800065,   // [2682:531] f + e : -21 (cslSTIX-Italic)
    0xf9000061,   // [2683:532] f + a : -14 (cslSTIX-Italic)
    0x1920006a,   // [2684:533] f + j ligature #50 (cslSTIX-Italic)
    0x19a0006c,   // [2685:534] f + l ligature #51 (cslSTIX-Italic)
    0x1a200069,   // [2686:535] f + i ligature #52 (cslSTIX-Italic)
    0x1ae00066,   // [2687:536] f + f ligature #53 (cslSTIX-Italic) ;;
    0xf08000f6,   // [2688:537] g + odieresis : -31 (cslSTIX-Italic)
    0xf08000f3,   // [2689:538] g + oacute : -31 (cslSTIX-Italic)
    0xef8000eb,   // [2690:539] g + edieresis : -33 (cslSTIX-Italic)
    0xef8000ea,   // [2691:540] g + ecircumflex : -33 (cslSTIX-Italic)
    0xef8000e9,   // [2692:541] g + eacute : -33 (cslSTIX-Italic)
    0xef8000e8,   // [2693:542] g + egrave : -33 (cslSTIX-Italic)
    0xef8000e6,   // [2694:543] g + ae : -33 (cslSTIX-Italic)
    0xf10000e5,   // [2695:544] g + aring : -30 (cslSTIX-Italic)
    0xf10000e4,   // [2696:545] g + adieresis : -30 (cslSTIX-Italic)
    0xf5800072,   // [2697:546] g + r : -21 (cslSTIX-Italic)
    0xef00006c,   // [2698:547] g + l : -34 (cslSTIX-Italic)
    0xef800065,   // [2699:548] g + e : -33 (cslSTIX-Italic)
    0xf1400061,   // [2700:549] g + a : -30 (cslSTIX-Italic) ;;
    0xf3002019,   // [2701:550] h + quoteright : -26 (cslSTIX-Italic)
    0xfe400079,   // [2702:551] h + y : -4 (cslSTIX-Italic) ;;
    0xf480006a,   // [2703:552] i + j : -23 (cslSTIX-Italic)
    0xfa800054,   // [2704:553] i + T : -11 (cslSTIX-Italic)
    0x1b60006a,   // [2705:554] i + j ligature #54 (cslSTIX-Italic) ;;
    0x040000fc,   // [2706:555] k + udieresis : 8 (cslSTIX-Italic)
    0x030000f6,   // [2707:556] k + odieresis : 6 (cslSTIX-Italic)
    0x030000f3,   // [2708:557] k + oacute : 6 (cslSTIX-Italic)
    0x020000e9,   // [2709:558] k + eacute : 4 (cslSTIX-Italic)
    0x020000e8,   // [2710:559] k + egrave : 4 (cslSTIX-Italic)
    0x048000e6,   // [2711:560] k + ae : 9 (cslSTIX-Italic)
    0x060000e5,   // [2712:561] k + aring : 12 (cslSTIX-Italic)
    0x060000e4,   // [2713:562] k + adieresis : 12 (cslSTIX-Italic)
    0x060000e1,   // [2714:563] k + aacute : 12 (cslSTIX-Italic)
    0x04000075,   // [2715:564] k + u : 8 (cslSTIX-Italic)
    0x03000073,   // [2716:565] k + s : 6 (cslSTIX-Italic)
    0x0300006f,   // [2717:566] k + o : 6 (cslSTIX-Italic)
    0xf2800067,   // [2718:567] k + g : -27 (cslSTIX-Italic)
    0x02000065,   // [2719:568] k + e : 4 (cslSTIX-Italic)
    0x06000061,   // [2720:569] k + a : 12 (cslSTIX-Italic)
    0x0d00002e,   // [2721:570] k + period : 26 (cslSTIX-Italic)
    0xef00002d,   // [2722:571] k + hyphen : -34 (cslSTIX-Italic)
    0x0ec0002c,   // [2723:572] k + comma : 29 (cslSTIX-Italic) ;;
    0xfb000079,   // [2724:573] l + y : -10 (cslSTIX-Italic)
    0xfac00076,   // [2725:574] l + v : -11 (cslSTIX-Italic) ;;
    0xfe800079,   // [2726:575] m + y : -3 (cslSTIX-Italic)
    0xfe000077,   // [2727:576] m + w : -4 (cslSTIX-Italic)
    0xfd000076,   // [2728:577] m + v : -6 (cslSTIX-Italic)
    0x01c00070,   // [2729:578] m + p : 3 (cslSTIX-Italic) ;;
    0xf2002019,   // [2730:579] n + quoteright : -28 (cslSTIX-Italic)
    0xfc800079,   // [2731:580] n + y : -7 (cslSTIX-Italic)
    0xfc800077,   // [2732:581] n + w : -7 (cslSTIX-Italic)
    0xfb000076,   // [2733:582] n + v : -10 (cslSTIX-Italic)
    0xebc00054,   // [2734:583] n + T : -41 (cslSTIX-Italic) ;;
    0xf6802019,   // [2735:584] o + quoteright : -19 (cslSTIX-Italic)
    0xfc000079,   // [2736:585] o + y : -8 (cslSTIX-Italic)
    0xf2800078,   // [2737:586] o + x : -27 (cslSTIX-Italic)
    0xfa000077,   // [2738:587] o + w : -12 (cslSTIX-Italic)
    0xf9000076,   // [2739:588] o + v : -14 (cslSTIX-Italic)
    0xfd800074,   // [2740:589] o + t : -5 (cslSTIX-Italic)
    0xe7400054,   // [2741:590] o + T : -50 (cslSTIX-Italic) ;;
    0xfe000079,   // [2742:591] p + y : -4 (cslSTIX-Italic)
    0xfdc00074,   // [2743:592] p + t : -5 (cslSTIX-Italic) ;;
    0xff000075,   // [2744:593] q + u : -2 (cslSTIX-Italic)
    0xff400063,   // [2745:594] q + c : -2 (cslSTIX-Italic) ;;
    0xf4800153,   // [2746:595] r + oe : -23 (cslSTIX-Italic)
    0xf18000f8,   // [2747:596] r + oslash : -29 (cslSTIX-Italic)
    0xf20000f6,   // [2748:597] r + odieresis : -28 (cslSTIX-Italic)
    0xf20000f4,   // [2749:598] r + ocircumflex : -28 (cslSTIX-Italic)
    0xf20000f3,   // [2750:599] r + oacute : -28 (cslSTIX-Italic)
    0xf20000f2,   // [2751:600] r + ograve : -28 (cslSTIX-Italic)
    0xf08000ea,   // [2752:601] r + ecircumflex : -31 (cslSTIX-Italic)
    0xf08000e9,   // [2753:602] r + eacute : -31 (cslSTIX-Italic)
    0xf08000e8,   // [2754:603] r + egrave : -31 (cslSTIX-Italic)
    0xf60000e7,   // [2755:604] r + ccedilla : -20 (cslSTIX-Italic)
    0xef0000e6,   // [2756:605] r + ae : -34 (cslSTIX-Italic)
    0xf20000e5,   // [2757:606] r + aring : -28 (cslSTIX-Italic)
    0xf20000e4,   // [2758:607] r + adieresis : -28 (cslSTIX-Italic)
    0xf20000e2,   // [2759:608] r + acircumflex : -28 (cslSTIX-Italic)
    0xf20000e1,   // [2760:609] r + aacute : -28 (cslSTIX-Italic)
    0xf20000e0,   // [2761:610] r + agrave : -28 (cslSTIX-Italic)
    0xff80007a,   // [2762:611] r + z : -1 (cslSTIX-Italic)
    0x0f800079,   // [2763:612] r + y : 31 (cslSTIX-Italic)
    0x03800078,   // [2764:613] r + x : 7 (cslSTIX-Italic)
    0x0f800077,   // [2765:614] r + w : 31 (cslSTIX-Italic)
    0x0e800076,   // [2766:615] r + v : 29 (cslSTIX-Italic)
    0x05800075,   // [2767:616] r + u : 11 (cslSTIX-Italic)
    0x06000074,   // [2768:617] r + t : 12 (cslSTIX-Italic)
    0xf7800073,   // [2769:618] r + s : -17 (cslSTIX-Italic)
    0x01800072,   // [2770:619] r + r : 3 (cslSTIX-Italic)
    0xf1800071,   // [2771:620] r + q : -29 (cslSTIX-Italic)
    0x05800070,   // [2772:621] r + p : 11 (cslSTIX-Italic)
    0xf200006f,   // [2773:622] r + o : -28 (cslSTIX-Italic)
    0x0800006e,   // [2774:623] r + n : 16 (cslSTIX-Italic)
    0x0880006d,   // [2775:624] r + m : 17 (cslSTIX-Italic)
    0xf580006c,   // [2776:625] r + l : -21 (cslSTIX-Italic)
    0xf800006b,   // [2777:626] r + k : -16 (cslSTIX-Italic)
    0x0180006a,   // [2778:627] r + j : 3 (cslSTIX-Italic)
    0x04000069,   // [2779:628] r + i : 8 (cslSTIX-Italic)
    0xf7800068,   // [2780:629] r + h : -17 (cslSTIX-Italic)
    0xf4000067,   // [2781:630] r + g : -24 (cslSTIX-Italic)
    0x0b000066,   // [2782:631] r + f : 22 (cslSTIX-Italic)
    0xf0800065,   // [2783:632] r + e : -31 (cslSTIX-Italic)
    0xf1800064,   // [2784:633] r + d : -29 (cslSTIX-Italic)
    0xf3800063,   // [2785:634] r + c : -25 (cslSTIX-Italic)
    0xf2000061,   // [2786:635] r + a : -28 (cslSTIX-Italic)
    0xf680003b,   // [2787:636] r + semicolon : -19 (cslSTIX-Italic)
    0xf380003a,   // [2788:637] r + colon : -25 (cslSTIX-Italic)
    0xe100002e,   // [2789:638] r + period : -62 (cslSTIX-Italic)
    0xea00002d,   // [2790:639] r + hyphen : -44 (cslSTIX-Italic)
    0xe3c0002c,   // [2791:640] r + comma : -57 (cslSTIX-Italic) ;;
    0xf9802019,   // [2792:641] s + quoteright : -13 (cslSTIX-Italic)
    0xfd400074,   // [2793:642] s + t : -6 (cslSTIX-Italic) ;;
    0xfb002019,   // [2794:643] t + quoteright : -10 (cslSTIX-Italic)
    0xfc0000f6,   // [2795:644] t + odieresis : -8 (cslSTIX-Italic)
    0xfc0000f3,   // [2796:645] t + oacute : -8 (cslSTIX-Italic)
    0xfb0000e9,   // [2797:646] t + eacute : -10 (cslSTIX-Italic)
    0xfb8000e6,   // [2798:647] t + ae : -9 (cslSTIX-Italic)
    0xfd0000e5,   // [2799:648] t + aring : -6 (cslSTIX-Italic)
    0xfd0000e4,   // [2800:649] t + adieresis : -6 (cslSTIX-Italic)
    0xfd0000e1,   // [2801:650] t + aacute : -6 (cslSTIX-Italic)
    0xfc00006f,   // [2802:651] t + o : -8 (cslSTIX-Italic)
    0xfe000068,   // [2803:652] t + h : -4 (cslSTIX-Italic)
    0xfb000065,   // [2804:653] t + e : -10 (cslSTIX-Italic)
    0xfe800061,   // [2805:654] t + a : -3 (cslSTIX-Italic)
    0xff000053,   // [2806:655] t + S : -2 (cslSTIX-Italic)
    0x0880003b,   // [2807:656] t + semicolon : 17 (cslSTIX-Italic)
    0x02c0003a,   // [2808:657] t + colon : 5 (cslSTIX-Italic) ;;
    0xf5c02019,   // [2809:658] u + quoteright : -21 (cslSTIX-Italic) ;;
    0xf68000f8,   // [2810:659] v + oslash : -19 (cslSTIX-Italic)
    0xf70000f6,   // [2811:660] v + odieresis : -18 (cslSTIX-Italic)
    0xf70000f3,   // [2812:661] v + oacute : -18 (cslSTIX-Italic)
    0xf70000f2,   // [2813:662] v + ograve : -18 (cslSTIX-Italic)
    0xf60000ea,   // [2814:663] v + ecircumflex : -20 (cslSTIX-Italic)
    0xf60000e9,   // [2815:664] v + eacute : -20 (cslSTIX-Italic)
    0xf60000e8,   // [2816:665] v + egrave : -20 (cslSTIX-Italic)
    0xf60000e6,   // [2817:666] v + ae : -20 (cslSTIX-Italic)
    0xf78000e5,   // [2818:667] v + aring : -17 (cslSTIX-Italic)
    0xf78000e4,   // [2819:668] v + adieresis : -17 (cslSTIX-Italic)
    0xf78000e3,   // [2820:669] v + atilde : -17 (cslSTIX-Italic)
    0xf78000e2,   // [2821:670] v + acircumflex : -17 (cslSTIX-Italic)
    0xf78000e1,   // [2822:671] v + aacute : -17 (cslSTIX-Italic)
    0xf78000e0,   // [2823:672] v + agrave : -17 (cslSTIX-Italic)
    0xf4000073,   // [2824:673] v + s : -24 (cslSTIX-Italic)
    0xf880006f,   // [2825:674] v + o : -15 (cslSTIX-Italic)
    0xf500006c,   // [2826:675] v + l : -22 (cslSTIX-Italic)
    0xed800067,   // [2827:676] v + g : -37 (cslSTIX-Italic)
    0xf6000065,   // [2828:677] v + e : -20 (cslSTIX-Italic)
    0xf8000063,   // [2829:678] v + c : -16 (cslSTIX-Italic)
    0xf9000061,   // [2830:679] v + a : -14 (cslSTIX-Italic)
    0x0200003b,   // [2831:680] v + semicolon : 4 (cslSTIX-Italic)
    0xfc80003a,   // [2832:681] v + colon : -7 (cslSTIX-Italic)
    0xec00002e,   // [2833:682] v + period : -40 (cslSTIX-Italic)
    0x0200002d,   // [2834:683] v + hyphen : 4 (cslSTIX-Italic)
    0xed40002c,   // [2835:684] v + comma : -38 (cslSTIX-Italic) ;;
    0xf68000f8,   // [2836:685] w + oslash : -19 (cslSTIX-Italic)
    0xf68000f6,   // [2837:686] w + odieresis : -19 (cslSTIX-Italic)
    0xf68000f3,   // [2838:687] w + oacute : -19 (cslSTIX-Italic)
    0xf68000f2,   // [2839:688] w + ograve : -19 (cslSTIX-Italic)
    0xf58000ea,   // [2840:689] w + ecircumflex : -21 (cslSTIX-Italic)
    0xf58000e9,   // [2841:690] w + eacute : -21 (cslSTIX-Italic)
    0xf58000e8,   // [2842:691] w + egrave : -21 (cslSTIX-Italic)
    0xf58000e6,   // [2843:692] w + ae : -21 (cslSTIX-Italic)
    0xf70000e5,   // [2844:693] w + aring : -18 (cslSTIX-Italic)
    0xf70000e4,   // [2845:694] w + adieresis : -18 (cslSTIX-Italic)
    0xf70000e3,   // [2846:695] w + atilde : -18 (cslSTIX-Italic)
    0xf70000e2,   // [2847:696] w + acircumflex : -18 (cslSTIX-Italic)
    0xf70000e1,   // [2848:697] w + aacute : -18 (cslSTIX-Italic)
    0xf70000e0,   // [2849:698] w + agrave : -18 (cslSTIX-Italic)
    0xf3800073,   // [2850:699] w + s : -25 (cslSTIX-Italic)
    0xf680006f,   // [2851:700] w + o : -19 (cslSTIX-Italic)
    0xf500006c,   // [2852:701] w + l : -22 (cslSTIX-Italic)
    0xed800067,   // [2853:702] w + g : -37 (cslSTIX-Italic)
    0xf5800065,   // [2854:703] w + e : -21 (cslSTIX-Italic)
    0xf7800063,   // [2855:704] w + c : -17 (cslSTIX-Italic)
    0xf7000061,   // [2856:705] w + a : -18 (cslSTIX-Italic)
    0xfc00003a,   // [2857:706] w + colon : -8 (cslSTIX-Italic)
    0xeb80002e,   // [2858:707] w + period : -41 (cslSTIX-Italic)
    0x0080002d,   // [2859:708] w + hyphen : 1 (cslSTIX-Italic)
    0xed40002c,   // [2860:709] w + comma : -38 (cslSTIX-Italic) ;;
    0xfe8000e9,   // [2861:710] x + eacute : -3 (cslSTIX-Italic)
    0xff80006f,   // [2862:711] x + o : -1 (cslSTIX-Italic)
    0xfe800065,   // [2863:712] x + e : -3 (cslSTIX-Italic)
    0xfe800063,   // [2864:713] x + c : -3 (cslSTIX-Italic)
    0x02400061,   // [2865:714] x + a : 4 (cslSTIX-Italic) ;;
    0xfd8000f8,   // [2866:715] y + oslash : -5 (cslSTIX-Italic)
    0xfd8000f6,   // [2867:716] y + odieresis : -5 (cslSTIX-Italic)
    0xfd8000f3,   // [2868:717] y + oacute : -5 (cslSTIX-Italic)
    0xfd8000f2,   // [2869:718] y + ograve : -5 (cslSTIX-Italic)
    0xfc8000ea,   // [2870:719] y + ecircumflex : -7 (cslSTIX-Italic)
    0xfc8000e9,   // [2871:720] y + eacute : -7 (cslSTIX-Italic)
    0xfc8000e8,   // [2872:721] y + egrave : -7 (cslSTIX-Italic)
    0xfd0000e6,   // [2873:722] y + ae : -6 (cslSTIX-Italic)
    0xfe8000e5,   // [2874:723] y + aring : -3 (cslSTIX-Italic)
    0xfe8000e4,   // [2875:724] y + adieresis : -3 (cslSTIX-Italic)
    0xfe8000e3,   // [2876:725] y + atilde : -3 (cslSTIX-Italic)
    0xfe8000e2,   // [2877:726] y + acircumflex : -3 (cslSTIX-Italic)
    0xfe8000e1,   // [2878:727] y + aacute : -3 (cslSTIX-Italic)
    0xfe8000e0,   // [2879:728] y + agrave : -3 (cslSTIX-Italic)
    0xf9800073,   // [2880:729] y + s : -13 (cslSTIX-Italic)
    0xfd80006f,   // [2881:730] y + o : -5 (cslSTIX-Italic)
    0xfb80006c,   // [2882:731] y + l : -9 (cslSTIX-Italic)
    0xf4000067,   // [2883:732] y + g : -24 (cslSTIX-Italic)
    0xfc800065,   // [2884:733] y + e : -7 (cslSTIX-Italic)
    0xfe800063,   // [2885:734] y + c : -3 (cslSTIX-Italic)
    0xfe800061,   // [2886:735] y + a : -3 (cslSTIX-Italic)
    0x0600003b,   // [2887:736] y + semicolon : 12 (cslSTIX-Italic)
    0x0280003a,   // [2888:737] y + colon : 5 (cslSTIX-Italic)
    0xf800002e,   // [2889:738] y + period : -16 (cslSTIX-Italic)
    0x0700002d,   // [2890:739] y + hyphen : 14 (cslSTIX-Italic)
    0xfa40002c,   // [2891:740] y + comma : -12 (cslSTIX-Italic) ;;
    0xed8000c6,   // [2892:741] guillemotright + AE : -37 (cslSTIX-Italic)
    0xfa8000c5,   // [2893:742] guillemotright + Aring : -11 (cslSTIX-Italic)
    0xfa8000c4,   // [2894:743] guillemotright + Adieresis : -11 (cslSTIX-Italic)
    0xfa8000c1,   // [2895:744] guillemotright + Aacute : -11 (cslSTIX-Italic)
    0xdf800059,   // [2896:745] guillemotright + Y : -65 (cslSTIX-Italic)
    0xe5800057,   // [2897:746] guillemotright + W : -53 (cslSTIX-Italic)
    0xe2000056,   // [2898:747] guillemotright + V : -60 (cslSTIX-Italic)
    0xe2000054,   // [2899:748] guillemotright + T : -60 (cslSTIX-Italic)
    0xfac00041,   // [2900:749] guillemotright + A : -11 (cslSTIX-Italic) ;;
    0xf6000059,   // [2901:750] Agrave + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2902:751] Agrave + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [2903:752] Agrave + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [2904:753] Agrave + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [2905:754] Agrave + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2906:755] Agrave + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2907:756] Agrave + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2908:757] Agrave + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2909:758] Agrave + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [2910:759] Agrave + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [2911:760] Agrave + comma : 19 (cslSTIX-Italic) ;;
    0xf2002039,   // [2912:761] Aacute + guilsinglleft : -28 (cslSTIX-Italic)
    0xd980201d,   // [2913:762] Aacute + quotedblright : -77 (cslSTIX-Italic)
    0xd9802019,   // [2914:763] Aacute + quoteright : -77 (cslSTIX-Italic)
    0xf18000ab,   // [2915:764] Aacute + guillemotleft : -29 (cslSTIX-Italic)
    0xe5000079,   // [2916:765] Aacute + y : -54 (cslSTIX-Italic)
    0xf0000077,   // [2917:766] Aacute + w : -32 (cslSTIX-Italic)
    0xee800076,   // [2918:767] Aacute + v : -35 (cslSTIX-Italic)
    0xfb800075,   // [2919:768] Aacute + u : -9 (cslSTIX-Italic)
    0xfc800074,   // [2920:769] Aacute + t : -7 (cslSTIX-Italic)
    0xfa800071,   // [2921:770] Aacute + q : -11 (cslSTIX-Italic)
    0xf800006f,   // [2922:771] Aacute + o : -16 (cslSTIX-Italic)
    0xf0800067,   // [2923:772] Aacute + g : -31 (cslSTIX-Italic)
    0xf7000065,   // [2924:773] Aacute + e : -18 (cslSTIX-Italic)
    0xfe000064,   // [2925:774] Aacute + d : -4 (cslSTIX-Italic)
    0xf7800063,   // [2926:775] Aacute + c : -17 (cslSTIX-Italic)
    0xfd800061,   // [2927:776] Aacute + a : -5 (cslSTIX-Italic)
    0xf6000059,   // [2928:777] Aacute + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2929:778] Aacute + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [2930:779] Aacute + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [2931:780] Aacute + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [2932:781] Aacute + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2933:782] Aacute + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2934:783] Aacute + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2935:784] Aacute + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2936:785] Aacute + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [2937:786] Aacute + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [2938:787] Aacute + comma : 19 (cslSTIX-Italic) ;;
    0xf6000059,   // [2939:788] Acircumflex + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2940:789] Acircumflex + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [2941:790] Acircumflex + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [2942:791] Acircumflex + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [2943:792] Acircumflex + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2944:793] Acircumflex + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2945:794] Acircumflex + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2946:795] Acircumflex + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2947:796] Acircumflex + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [2948:797] Acircumflex + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [2949:798] Acircumflex + comma : 19 (cslSTIX-Italic) ;;
    0xf6000059,   // [2950:799] Atilde + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2951:800] Atilde + W : -59 (cslSTIX-Italic)
    0xdf000056,   // [2952:801] Atilde + V : -66 (cslSTIX-Italic)
    0xe9800055,   // [2953:802] Atilde + U : -45 (cslSTIX-Italic)
    0xf9000054,   // [2954:803] Atilde + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2955:804] Atilde + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2956:805] Atilde + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2957:806] Atilde + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2958:807] Atilde + C : -36 (cslSTIX-Italic)
    0x0880002e,   // [2959:808] Atilde + period : 17 (cslSTIX-Italic)
    0x0a40002c,   // [2960:809] Atilde + comma : 20 (cslSTIX-Italic) ;;
    0xf2002039,   // [2961:810] Adieresis + guilsinglleft : -28 (cslSTIX-Italic)
    0xd980201d,   // [2962:811] Adieresis + quotedblright : -77 (cslSTIX-Italic)
    0xd9802019,   // [2963:812] Adieresis + quoteright : -77 (cslSTIX-Italic)
    0xf18000ab,   // [2964:813] Adieresis + guillemotleft : -29 (cslSTIX-Italic)
    0xe5000079,   // [2965:814] Adieresis + y : -54 (cslSTIX-Italic)
    0xf0000077,   // [2966:815] Adieresis + w : -32 (cslSTIX-Italic)
    0xee800076,   // [2967:816] Adieresis + v : -35 (cslSTIX-Italic)
    0xfb800075,   // [2968:817] Adieresis + u : -9 (cslSTIX-Italic)
    0xfc800074,   // [2969:818] Adieresis + t : -7 (cslSTIX-Italic)
    0xfa800071,   // [2970:819] Adieresis + q : -11 (cslSTIX-Italic)
    0xf800006f,   // [2971:820] Adieresis + o : -16 (cslSTIX-Italic)
    0xf0800067,   // [2972:821] Adieresis + g : -31 (cslSTIX-Italic)
    0xfe000064,   // [2973:822] Adieresis + d : -4 (cslSTIX-Italic)
    0xf7800063,   // [2974:823] Adieresis + c : -17 (cslSTIX-Italic)
    0xfd800061,   // [2975:824] Adieresis + a : -5 (cslSTIX-Italic)
    0xf6000059,   // [2976:825] Adieresis + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [2977:826] Adieresis + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [2978:827] Adieresis + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [2979:828] Adieresis + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [2980:829] Adieresis + T : -14 (cslSTIX-Italic)
    0xee000051,   // [2981:830] Adieresis + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [2982:831] Adieresis + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [2983:832] Adieresis + G : -29 (cslSTIX-Italic)
    0xee000043,   // [2984:833] Adieresis + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [2985:834] Adieresis + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [2986:835] Adieresis + comma : 19 (cslSTIX-Italic) ;;
    0xf2002039,   // [2987:836] Aring + guilsinglleft : -28 (cslSTIX-Italic)
    0xd980201d,   // [2988:837] Aring + quotedblright : -77 (cslSTIX-Italic)
    0xd9802019,   // [2989:838] Aring + quoteright : -77 (cslSTIX-Italic)
    0xf18000ab,   // [2990:839] Aring + guillemotleft : -29 (cslSTIX-Italic)
    0xe5000079,   // [2991:840] Aring + y : -54 (cslSTIX-Italic)
    0xf0000077,   // [2992:841] Aring + w : -32 (cslSTIX-Italic)
    0xee800076,   // [2993:842] Aring + v : -35 (cslSTIX-Italic)
    0xfb800075,   // [2994:843] Aring + u : -9 (cslSTIX-Italic)
    0xfc800074,   // [2995:844] Aring + t : -7 (cslSTIX-Italic)
    0xfa800071,   // [2996:845] Aring + q : -11 (cslSTIX-Italic)
    0xf800006f,   // [2997:846] Aring + o : -16 (cslSTIX-Italic)
    0xf0800067,   // [2998:847] Aring + g : -31 (cslSTIX-Italic)
    0xf7000065,   // [2999:848] Aring + e : -18 (cslSTIX-Italic)
    0xfe000064,   // [3000:849] Aring + d : -4 (cslSTIX-Italic)
    0xf7800063,   // [3001:850] Aring + c : -17 (cslSTIX-Italic)
    0xfd800061,   // [3002:851] Aring + a : -5 (cslSTIX-Italic)
    0xf6000059,   // [3003:852] Aring + Y : -20 (cslSTIX-Italic)
    0xe2800057,   // [3004:853] Aring + W : -59 (cslSTIX-Italic)
    0xdf800056,   // [3005:854] Aring + V : -65 (cslSTIX-Italic)
    0xe9000055,   // [3006:855] Aring + U : -46 (cslSTIX-Italic)
    0xf9000054,   // [3007:856] Aring + T : -14 (cslSTIX-Italic)
    0xee000051,   // [3008:857] Aring + Q : -36 (cslSTIX-Italic)
    0xed80004f,   // [3009:858] Aring + O : -37 (cslSTIX-Italic)
    0xf1800047,   // [3010:859] Aring + G : -29 (cslSTIX-Italic)
    0xee000043,   // [3011:860] Aring + C : -36 (cslSTIX-Italic)
    0x0980002e,   // [3012:861] Aring + period : 19 (cslSTIX-Italic)
    0x09c0002c,   // [3013:862] Aring + comma : 19 (cslSTIX-Italic) ;;
    0xf6c00041,   // [3014:863] Ccedilla + A : -19 (cslSTIX-Italic) ;;
    0xef000059,   // [3015:864] Ograve + Y : -34 (cslSTIX-Italic)
    0xf1800056,   // [3016:865] Ograve + V : -29 (cslSTIX-Italic)
    0xfdc00054,   // [3017:866] Ograve + T : -5 (cslSTIX-Italic) ;;
    0xef000059,   // [3018:867] Oacute + Y : -34 (cslSTIX-Italic)
    0xf4800057,   // [3019:868] Oacute + W : -23 (cslSTIX-Italic)
    0xf1800056,   // [3020:869] Oacute + V : -29 (cslSTIX-Italic)
    0xfd800054,   // [3021:870] Oacute + T : -5 (cslSTIX-Italic)
    0xf8400041,   // [3022:871] Oacute + A : -16 (cslSTIX-Italic) ;;
    0xef000059,   // [3023:872] Ocircumflex + Y : -34 (cslSTIX-Italic)
    0xf1800056,   // [3024:873] Ocircumflex + V : -29 (cslSTIX-Italic)
    0xfdc00054,   // [3025:874] Ocircumflex + T : -5 (cslSTIX-Italic) ;;
    0xef000059,   // [3026:875] Otilde + Y : -34 (cslSTIX-Italic)
    0xf1800056,   // [3027:876] Otilde + V : -29 (cslSTIX-Italic)
    0xfdc00054,   // [3028:877] Otilde + T : -5 (cslSTIX-Italic) ;;
    0xef000059,   // [3029:878] Odieresis + Y : -34 (cslSTIX-Italic)
    0xf3000058,   // [3030:879] Odieresis + X : -26 (cslSTIX-Italic)
    0xf4800057,   // [3031:880] Odieresis + W : -23 (cslSTIX-Italic)
    0xf1800056,   // [3032:881] Odieresis + V : -29 (cslSTIX-Italic)
    0xfd800054,   // [3033:882] Odieresis + T : -5 (cslSTIX-Italic)
    0xf8400041,   // [3034:883] Odieresis + A : -16 (cslSTIX-Italic) ;;
    0xf8400041,   // [3035:884] Oslash + A : -16 (cslSTIX-Italic) ;;
    0xef000072,   // [3036:885] Ugrave + r : -34 (cslSTIX-Italic)
    0xf4800070,   // [3037:886] Ugrave + p : -23 (cslSTIX-Italic)
    0xf680006e,   // [3038:887] Ugrave + n : -19 (cslSTIX-Italic)
    0xf700006d,   // [3039:888] Ugrave + m : -18 (cslSTIX-Italic)
    0xed000041,   // [3040:889] Ugrave + A : -38 (cslSTIX-Italic)
    0xf500002e,   // [3041:890] Ugrave + period : -22 (cslSTIX-Italic)
    0xf840002c,   // [3042:891] Ugrave + comma : -16 (cslSTIX-Italic) ;;
    0xef000072,   // [3043:892] Uacute + r : -34 (cslSTIX-Italic)
    0xf4800070,   // [3044:893] Uacute + p : -23 (cslSTIX-Italic)
    0xf680006e,   // [3045:894] Uacute + n : -19 (cslSTIX-Italic)
    0xf700006d,   // [3046:895] Uacute + m : -18 (cslSTIX-Italic)
    0xed000041,   // [3047:896] Uacute + A : -38 (cslSTIX-Italic)
    0xf500002e,   // [3048:897] Uacute + period : -22 (cslSTIX-Italic)
    0xf840002c,   // [3049:898] Uacute + comma : -16 (cslSTIX-Italic) ;;
    0xed400041,   // [3050:899] Ucircumflex + A : -38 (cslSTIX-Italic) ;;
    0xef000072,   // [3051:900] Udieresis + r : -34 (cslSTIX-Italic)
    0xf4800070,   // [3052:901] Udieresis + p : -23 (cslSTIX-Italic)
    0xf680006e,   // [3053:902] Udieresis + n : -19 (cslSTIX-Italic)
    0xf700006d,   // [3054:903] Udieresis + m : -18 (cslSTIX-Italic)
    0x00800062,   // [3055:904] Udieresis + b : 1 (cslSTIX-Italic)
    0xed000041,   // [3056:905] Udieresis + A : -38 (cslSTIX-Italic)
    0xf500002e,   // [3057:906] Udieresis + period : -22 (cslSTIX-Italic)
    0xf840002c,   // [3058:907] Udieresis + comma : -16 (cslSTIX-Italic) ;;
    0x01000079,   // [3059:908] agrave + y : 2 (cslSTIX-Italic)
    0x01c00077,   // [3060:909] agrave + w : 3 (cslSTIX-Italic) ;;
    0x01000079,   // [3061:910] aacute + y : 2 (cslSTIX-Italic)
    0x01c00077,   // [3062:911] aacute + w : 3 (cslSTIX-Italic) ;;
    0x01000079,   // [3063:912] adieresis + y : 2 (cslSTIX-Italic)
    0x01c00077,   // [3064:913] adieresis + w : 3 (cslSTIX-Italic) ;;
    0x01000079,   // [3065:914] aring + y : 2 (cslSTIX-Italic)
    0x01c00077,   // [3066:915] aring + w : 3 (cslSTIX-Italic) ;;
    0x02800079,   // [3067:916] ae + y : 5 (cslSTIX-Italic)
    0x01c00077,   // [3068:917] ae + w : 3 (cslSTIX-Italic) ;;
    0x01800079,   // [3069:918] eacute + y : 3 (cslSTIX-Italic)
    0x01400077,   // [3070:919] eacute + w : 2 (cslSTIX-Italic) ;;
    0x01800079,   // [3071:920] ecircumflex + y : 3 (cslSTIX-Italic)
    0x01400077,   // [3072:921] ecircumflex + w : 2 (cslSTIX-Italic) ;;
    0xfc000079,   // [3073:922] ograve + y : -8 (cslSTIX-Italic)
    0xfa000077,   // [3074:923] ograve + w : -12 (cslSTIX-Italic)
    0xf9400076,   // [3075:924] ograve + v : -14 (cslSTIX-Italic) ;;
    0xfc000079,   // [3076:925] oacute + y : -8 (cslSTIX-Italic)
    0xfa000077,   // [3077:926] oacute + w : -12 (cslSTIX-Italic)
    0xf9400076,   // [3078:927] oacute + v : -14 (cslSTIX-Italic) ;;
    0xfdc00074,   // [3079:928] ocircumflex + t : -5 (cslSTIX-Italic) ;;
    0xfc000079,   // [3080:929] odieresis + y : -8 (cslSTIX-Italic)
    0xf2000078,   // [3081:930] odieresis + x : -28 (cslSTIX-Italic)
    0xfa000077,   // [3082:931] odieresis + w : -12 (cslSTIX-Italic)
    0xf9000076,   // [3083:932] odieresis + v : -14 (cslSTIX-Italic)
    0xfdc00074,   // [3084:933] odieresis + t : -5 (cslSTIX-Italic) ;;
    0xc80000c6,   // [3085:934] quoteleft + AE : -112 (cslSTIX-Italic)
    0xe00000c5,   // [3086:935] quoteleft + Aring : -64 (cslSTIX-Italic)
    0xe00000c4,   // [3087:936] quoteleft + Adieresis : -64 (cslSTIX-Italic)
    0xe00000c2,   // [3088:937] quoteleft + Acircumflex : -64 (cslSTIX-Italic)
    0xe00000c1,   // [3089:938] quoteleft + Aacute : -64 (cslSTIX-Italic)
    0xe00000c0,   // [3090:939] quoteleft + Agrave : -64 (cslSTIX-Italic)
    0xfa000059,   // [3091:940] quoteleft + Y : -12 (cslSTIX-Italic)
    0x01800057,   // [3092:941] quoteleft + W : 3 (cslSTIX-Italic)
    0x00800056,   // [3093:942] quoteleft + V : 1 (cslSTIX-Italic)
    0x01800054,   // [3094:943] quoteleft + T : 3 (cslSTIX-Italic)
    0xe0400041,   // [3095:944] quoteleft + A : -64 (cslSTIX-Italic) ;;
    0xc30000c6,   // [3096:945] quoteright + AE : -122 (cslSTIX-Italic)
    0xde0000c5,   // [3097:946] quoteright + Aring : -68 (cslSTIX-Italic)
    0xde0000c4,   // [3098:947] quoteright + Adieresis : -68 (cslSTIX-Italic)
    0xde0000c2,   // [3099:948] quoteright + Acircumflex : -68 (cslSTIX-Italic)
    0xde0000c1,   // [3100:949] quoteright + Aacute : -68 (cslSTIX-Italic)
    0xde0000c0,   // [3101:950] quoteright + Agrave : -68 (cslSTIX-Italic)
    0xf4800079,   // [3102:951] quoteright + y : -23 (cslSTIX-Italic)
    0xf6800077,   // [3103:952] quoteright + w : -19 (cslSTIX-Italic)
    0xf3800076,   // [3104:953] quoteright + v : -25 (cslSTIX-Italic)
    0xef000074,   // [3105:954] quoteright + t : -34 (cslSTIX-Italic)
    0xe6800073,   // [3106:955] quoteright + s : -51 (cslSTIX-Italic)
    0xe9800072,   // [3107:956] quoteright + r : -45 (cslSTIX-Italic)
    0xdf80006f,   // [3108:957] quoteright + o : -65 (cslSTIX-Italic)
    0xe0000064,   // [3109:958] quoteright + d : -64 (cslSTIX-Italic)
    0xde000041,   // [3110:959] quoteright + A : -68 (cslSTIX-Italic)
    0xdd80002e,   // [3111:960] quoteright + period : -69 (cslSTIX-Italic)
    0xe0c0002c,   // [3112:961] quoteright + comma : -63 (cslSTIX-Italic) ;;
    0xc38000c6,   // [3113:962] quotedblleft + AE : -121 (cslSTIX-Italic)
    0xdb8000c5,   // [3114:963] quotedblleft + Aring : -73 (cslSTIX-Italic)
    0xdb8000c4,   // [3115:964] quotedblleft + Adieresis : -73 (cslSTIX-Italic)
    0xdb8000c1,   // [3116:965] quotedblleft + Aacute : -73 (cslSTIX-Italic)
    0xf5800059,   // [3117:966] quotedblleft + Y : -21 (cslSTIX-Italic)
    0xfd800057,   // [3118:967] quotedblleft + W : -5 (cslSTIX-Italic)
    0xfc800056,   // [3119:968] quotedblleft + V : -7 (cslSTIX-Italic)
    0xfd800054,   // [3120:969] quotedblleft + T : -5 (cslSTIX-Italic)
    0xdbc00041,   // [3121:970] quotedblleft + A : -73 (cslSTIX-Italic) ;;
    0xbf8000c6,   // [3122:971] quotedblright + AE : -129 (cslSTIX-Italic)
    0xda8000c5,   // [3123:972] quotedblright + Aring : -75 (cslSTIX-Italic)
    0xda8000c4,   // [3124:973] quotedblright + Adieresis : -75 (cslSTIX-Italic)
    0xda8000c1,   // [3125:974] quotedblright + Aacute : -75 (cslSTIX-Italic)
    0xf3800059,   // [3126:975] quotedblright + Y : -25 (cslSTIX-Italic)
    0xfb800057,   // [3127:976] quotedblright + W : -9 (cslSTIX-Italic)
    0xfa000056,   // [3128:977] quotedblright + V : -12 (cslSTIX-Italic)
    0xfb800054,   // [3129:978] quotedblright + T : -9 (cslSTIX-Italic)
    0xdac00041,   // [3130:979] quotedblright + A : -75 (cslSTIX-Italic) ;;
    0xff0000c6,   // [3131:980] quotedblbase + AE : -2 (cslSTIX-Italic)
    0x048000c5,   // [3132:981] quotedblbase + Aring : 9 (cslSTIX-Italic)
    0x048000c4,   // [3133:982] quotedblbase + Adieresis : 9 (cslSTIX-Italic)
    0x048000c2,   // [3134:983] quotedblbase + Acircumflex : 9 (cslSTIX-Italic)
    0x048000c1,   // [3135:984] quotedblbase + Aacute : 9 (cslSTIX-Italic)
    0x048000c0,   // [3136:985] quotedblbase + Agrave : 9 (cslSTIX-Italic)
    0xe4800059,   // [3137:986] quotedblbase + Y : -55 (cslSTIX-Italic)
    0xde800057,   // [3138:987] quotedblbase + W : -67 (cslSTIX-Italic)
    0xd8000056,   // [3139:988] quotedblbase + V : -80 (cslSTIX-Italic)
    0xe6000054,   // [3140:989] quotedblbase + T : -52 (cslSTIX-Italic)
    0x04c00041,   // [3141:990] quotedblbase + A : 9 (cslSTIX-Italic) ;;
    0xee0000c6,   // [3142:991] guilsinglright + AE : -36 (cslSTIX-Italic)
    0xfb0000c5,   // [3143:992] guilsinglright + Aring : -10 (cslSTIX-Italic)
    0xfb0000c4,   // [3144:993] guilsinglright + Adieresis : -10 (cslSTIX-Italic)
    0xfb0000c1,   // [3145:994] guilsinglright + Aacute : -10 (cslSTIX-Italic)
    0xe0000059,   // [3146:995] guilsinglright + Y : -64 (cslSTIX-Italic)
    0xe6000057,   // [3147:996] guilsinglright + W : -52 (cslSTIX-Italic)
    0xe2800056,   // [3148:997] guilsinglright + V : -59 (cslSTIX-Italic)
    0xe2800054,   // [3149:998] guilsinglright + T : -59 (cslSTIX-Italic)
    0xfb400041,   // [3150:999] guilsinglright + A : -10 (cslSTIX-Italic) ;;
    0x1ba00069,   // [3151:1000] ff + i ligature #55 (cslSTIX-Italic)
    0x1c60006c,   // [3152:1001] ff + l ligature #56 (cslSTIX-Italic) ;;
    0xff80201d,   // [3153:1] comma + quotedblright : -1 (cslSTIX-BoldItalic)
    0xf7002019,   // [3154:2] comma + quoteright : -18 (cslSTIX-BoldItalic)
    0xfb400031,   // [3155:3] comma + one : -10 (cslSTIX-BoldItalic) ;;
    0xfe8000c6,   // [3156:4] hyphen + AE : -3 (cslSTIX-BoldItalic)
    0x030000c5,   // [3157:5] hyphen + Aring : 6 (cslSTIX-BoldItalic)
    0x030000c4,   // [3158:6] hyphen + Adieresis : 6 (cslSTIX-BoldItalic)
    0x030000c1,   // [3159:7] hyphen + Aacute : 6 (cslSTIX-BoldItalic)
    0xec800059,   // [3160:8] hyphen + Y : -39 (cslSTIX-BoldItalic)
    0xef000057,   // [3161:9] hyphen + W : -34 (cslSTIX-BoldItalic)
    0xed800056,   // [3162:10] hyphen + V : -37 (cslSTIX-BoldItalic)
    0xef000054,   // [3163:11] hyphen + T : -34 (cslSTIX-BoldItalic)
    0x03400041,   // [3164:12] hyphen + A : 6 (cslSTIX-BoldItalic) ;;
    0xfe00201d,   // [3165:13] period + quotedblright : -4 (cslSTIX-BoldItalic)
    0xf5c02019,   // [3166:14] period + quoteright : -21 (cslSTIX-BoldItalic) ;;
    0xeb802039,   // [3167:15] A + guilsinglleft : -41 (cslSTIX-BoldItalic)
    0xe780201d,   // [3168:16] A + quotedblright : -49 (cslSTIX-BoldItalic)
    0xdf002019,   // [3169:17] A + quoteright : -66 (cslSTIX-BoldItalic)
    0xf90000fc,   // [3170:18] A + udieresis : -14 (cslSTIX-BoldItalic)
    0xf90000fb,   // [3171:19] A + ucircumflex : -14 (cslSTIX-BoldItalic)
    0xf90000fa,   // [3172:20] A + uacute : -14 (cslSTIX-BoldItalic)
    0xf90000f9,   // [3173:21] A + ugrave : -14 (cslSTIX-BoldItalic)
    0xf80000f6,   // [3174:22] A + odieresis : -16 (cslSTIX-BoldItalic)
    0xf70000e7,   // [3175:23] A + ccedilla : -18 (cslSTIX-BoldItalic)
    0xdf8000dc,   // [3176:24] A + Udieresis : -65 (cslSTIX-BoldItalic)
    0xdf8000db,   // [3177:25] A + Ucircumflex : -65 (cslSTIX-BoldItalic)
    0xdf8000da,   // [3178:26] A + Uacute : -65 (cslSTIX-BoldItalic)
    0xdf8000d9,   // [3179:27] A + Ugrave : -65 (cslSTIX-BoldItalic)
    0xee0000d6,   // [3180:28] A + Odieresis : -36 (cslSTIX-BoldItalic)
    0xed0000c7,   // [3181:29] A + Ccedilla : -38 (cslSTIX-BoldItalic)
    0xef0000ab,   // [3182:30] A + guillemotleft : -34 (cslSTIX-BoldItalic)
    0xeb800079,   // [3183:31] A + y : -41 (cslSTIX-BoldItalic)
    0xed800077,   // [3184:32] A + w : -37 (cslSTIX-BoldItalic)
    0xef000076,   // [3185:33] A + v : -34 (cslSTIX-BoldItalic)
    0xf9000075,   // [3186:34] A + u : -14 (cslSTIX-BoldItalic)
    0xfe800074,   // [3187:35] A + t : -3 (cslSTIX-BoldItalic)
    0xfa000071,   // [3188:36] A + q : -12 (cslSTIX-BoldItalic)
    0xf800006f,   // [3189:37] A + o : -16 (cslSTIX-BoldItalic)
    0xf8000067,   // [3190:38] A + g : -16 (cslSTIX-BoldItalic)
    0xf7800065,   // [3191:39] A + e : -17 (cslSTIX-BoldItalic)
    0xfe800064,   // [3192:40] A + d : -3 (cslSTIX-BoldItalic)
    0xf9000063,   // [3193:41] A + c : -14 (cslSTIX-BoldItalic)
    0xff000062,   // [3194:42] A + b : -2 (cslSTIX-BoldItalic)
    0xfe000061,   // [3195:43] A + a : -4 (cslSTIX-BoldItalic)
    0xce000059,   // [3196:44] A + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [3197:45] A + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [3198:46] A + V : -105 (cslSTIX-BoldItalic)
    0xdf800055,   // [3199:47] A + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [3200:48] A + T : -55 (cslSTIX-BoldItalic)
    0xee000051,   // [3201:49] A + Q : -36 (cslSTIX-BoldItalic)
    0xee00004f,   // [3202:50] A + O : -36 (cslSTIX-BoldItalic)
    0xef000047,   // [3203:51] A + G : -34 (cslSTIX-BoldItalic)
    0xed000043,   // [3204:52] A + C : -38 (cslSTIX-BoldItalic)
    0x0900002e,   // [3205:53] A + period : 18 (cslSTIX-BoldItalic)
    0xfd00002d,   // [3206:54] A + hyphen : -6 (cslSTIX-BoldItalic)
    0x06c0002c,   // [3207:55] A + comma : 13 (cslSTIX-BoldItalic) ;;
    0xff000152,   // [3208:56] B + OE : -2 (cslSTIX-BoldItalic)
    0xfc8000d8,   // [3209:57] B + Oslash : -7 (cslSTIX-BoldItalic)
    0xfc8000d6,   // [3210:58] B + Odieresis : -7 (cslSTIX-BoldItalic)
    0xfc8000d4,   // [3211:59] B + Ocircumflex : -7 (cslSTIX-BoldItalic)
    0xfc8000d3,   // [3212:60] B + Oacute : -7 (cslSTIX-BoldItalic)
    0xfc8000d2,   // [3213:61] B + Ograve : -7 (cslSTIX-BoldItalic)
    0xf28000c6,   // [3214:62] B + AE : -27 (cslSTIX-BoldItalic)
    0xf30000c5,   // [3215:63] B + Aring : -26 (cslSTIX-BoldItalic)
    0xf30000c4,   // [3216:64] B + Adieresis : -26 (cslSTIX-BoldItalic)
    0xf30000c3,   // [3217:65] B + Atilde : -26 (cslSTIX-BoldItalic)
    0xf30000c2,   // [3218:66] B + Acircumflex : -26 (cslSTIX-BoldItalic)
    0xf30000c1,   // [3219:67] B + Aacute : -26 (cslSTIX-BoldItalic)
    0xed000059,   // [3220:68] B + Y : -38 (cslSTIX-BoldItalic)
    0xf0000057,   // [3221:69] B + W : -32 (cslSTIX-BoldItalic)
    0xee000056,   // [3222:70] B + V : -36 (cslSTIX-BoldItalic)
    0xfc80004f,   // [3223:71] B + O : -7 (cslSTIX-BoldItalic)
    0xf3400041,   // [3224:72] B + A : -26 (cslSTIX-BoldItalic) ;;
    0xfc8000d6,   // [3225:73] C + Odieresis : -7 (cslSTIX-BoldItalic)
    0xfc8000d3,   // [3226:74] C + Oacute : -7 (cslSTIX-BoldItalic)
    0xfc8000d2,   // [3227:75] C + Ograve : -7 (cslSTIX-BoldItalic)
    0xf48000c6,   // [3228:76] C + AE : -23 (cslSTIX-BoldItalic)
    0xf50000c5,   // [3229:77] C + Aring : -22 (cslSTIX-BoldItalic)
    0xf50000c4,   // [3230:78] C + Adieresis : -22 (cslSTIX-BoldItalic)
    0xf50000c1,   // [3231:79] C + Aacute : -22 (cslSTIX-BoldItalic)
    0xf50000c0,   // [3232:80] C + Agrave : -22 (cslSTIX-BoldItalic)
    0xfc80004f,   // [3233:81] C + O : -7 (cslSTIX-BoldItalic)
    0xf980004b,   // [3234:82] C + K : -13 (cslSTIX-BoldItalic)
    0xfa000048,   // [3235:83] C + H : -12 (cslSTIX-BoldItalic)
    0xf5400041,   // [3236:84] C + A : -22 (cslSTIX-BoldItalic) ;;
    0xf18000c5,   // [3237:85] D + Aring : -29 (cslSTIX-BoldItalic)
    0xf18000c4,   // [3238:86] D + Adieresis : -29 (cslSTIX-BoldItalic)
    0xf18000c3,   // [3239:87] D + Atilde : -29 (cslSTIX-BoldItalic)
    0xf18000c2,   // [3240:88] D + Acircumflex : -29 (cslSTIX-BoldItalic)
    0xf18000c1,   // [3241:89] D + Aacute : -29 (cslSTIX-BoldItalic)
    0xf18000c0,   // [3242:90] D + Agrave : -29 (cslSTIX-BoldItalic)
    0xee800059,   // [3243:91] D + Y : -35 (cslSTIX-BoldItalic)
    0xed000058,   // [3244:92] D + X : -38 (cslSTIX-BoldItalic)
    0xef800057,   // [3245:93] D + W : -33 (cslSTIX-BoldItalic)
    0xef000056,   // [3246:94] D + V : -34 (cslSTIX-BoldItalic)
    0xf5800054,   // [3247:95] D + T : -21 (cslSTIX-BoldItalic)
    0xe700004a,   // [3248:96] D + J : -50 (cslSTIX-BoldItalic)
    0xf1c00041,   // [3249:97] D + A : -29 (cslSTIX-BoldItalic) ;;
    0xe3000153,   // [3250:98] F + oe : -58 (cslSTIX-BoldItalic)
    0xe70000f8,   // [3251:99] F + oslash : -50 (cslSTIX-BoldItalic)
    0xee0000f6,   // [3252:100] F + odieresis : -36 (cslSTIX-BoldItalic)
    0xe70000f3,   // [3253:101] F + oacute : -50 (cslSTIX-BoldItalic)
    0xe30000e9,   // [3254:102] F + eacute : -58 (cslSTIX-BoldItalic)
    0xe50000e6,   // [3255:103] F + ae : -54 (cslSTIX-BoldItalic)
    0xe88000e5,   // [3256:104] F + aring : -47 (cslSTIX-BoldItalic)
    0xeb0000e4,   // [3257:105] F + adieresis : -42 (cslSTIX-BoldItalic)
    0xe98000e3,   // [3258:106] F + atilde : -45 (cslSTIX-BoldItalic)
    0xe88000e2,   // [3259:107] F + acircumflex : -47 (cslSTIX-BoldItalic)
    0xe88000e1,   // [3260:108] F + aacute : -47 (cslSTIX-BoldItalic)
    0xef0000d6,   // [3261:109] F + Odieresis : -34 (cslSTIX-BoldItalic)
    0xef0000d3,   // [3262:110] F + Oacute : -34 (cslSTIX-BoldItalic)
    0xef0000d2,   // [3263:111] F + Ograve : -34 (cslSTIX-BoldItalic)
    0xe18000c5,   // [3264:112] F + Aring : -61 (cslSTIX-BoldItalic)
    0xe18000c4,   // [3265:113] F + Adieresis : -61 (cslSTIX-BoldItalic)
    0xe18000c3,   // [3266:114] F + Atilde : -61 (cslSTIX-BoldItalic)
    0xe18000c2,   // [3267:115] F + Acircumflex : -61 (cslSTIX-BoldItalic)
    0xe18000c1,   // [3268:116] F + Aacute : -61 (cslSTIX-BoldItalic)
    0xe18000c0,   // [3269:117] F + Agrave : -61 (cslSTIX-BoldItalic)
    0xf2800075,   // [3270:118] F + u : -27 (cslSTIX-BoldItalic)
    0xf4000072,   // [3271:119] F + r : -24 (cslSTIX-BoldItalic)
    0xe700006f,   // [3272:120] F + o : -50 (cslSTIX-BoldItalic)
    0xf300006a,   // [3273:121] F + j : -26 (cslSTIX-BoldItalic)
    0xf5000069,   // [3274:122] F + i : -22 (cslSTIX-BoldItalic)
    0xe3000065,   // [3275:123] F + e : -58 (cslSTIX-BoldItalic)
    0xe8800061,   // [3276:124] F + a : -47 (cslSTIX-BoldItalic)
    0xef00004f,   // [3277:125] F + O : -34 (cslSTIX-BoldItalic)
    0xdf00004a,   // [3278:126] F + J : -66 (cslSTIX-BoldItalic)
    0xe1800041,   // [3279:127] F + A : -61 (cslSTIX-BoldItalic)
    0xe180002e,   // [3280:128] F + period : -61 (cslSTIX-BoldItalic)
    0xf200002d,   // [3281:129] F + hyphen : -28 (cslSTIX-BoldItalic)
    0xe2c0002c,   // [3282:130] F + comma : -59 (cslSTIX-BoldItalic) ;;
    0xfa0000c6,   // [3283:131] G + AE : -12 (cslSTIX-BoldItalic)
    0xfb0000c5,   // [3284:132] G + Aring : -10 (cslSTIX-BoldItalic)
    0xfb0000c4,   // [3285:133] G + Adieresis : -10 (cslSTIX-BoldItalic)
    0xfb0000c3,   // [3286:134] G + Atilde : -10 (cslSTIX-BoldItalic)
    0xfb0000c2,   // [3287:135] G + Acircumflex : -10 (cslSTIX-BoldItalic)
    0xfb0000c1,   // [3288:136] G + Aacute : -10 (cslSTIX-BoldItalic)
    0xfb0000c0,   // [3289:137] G + Agrave : -10 (cslSTIX-BoldItalic)
    0xf5000059,   // [3290:138] G + Y : -22 (cslSTIX-BoldItalic)
    0xf6800057,   // [3291:139] G + W : -19 (cslSTIX-BoldItalic)
    0xf6000056,   // [3292:140] G + V : -20 (cslSTIX-BoldItalic)
    0xf5000054,   // [3293:141] G + T : -22 (cslSTIX-BoldItalic)
    0xfb400041,   // [3294:142] G + A : -10 (cslSTIX-BoldItalic) ;;
    0x1ce0004a,   // [3295:143] I + J ligature #57 (cslSTIX-BoldItalic) ;;
    0xf00000c6,   // [3296:144] J + AE : -32 (cslSTIX-BoldItalic)
    0xf10000c5,   // [3297:145] J + Aring : -30 (cslSTIX-BoldItalic)
    0xf10000c4,   // [3298:146] J + Adieresis : -30 (cslSTIX-BoldItalic)
    0xf1400041,   // [3299:147] J + A : -30 (cslSTIX-BoldItalic) ;;
    0xf4000152,   // [3300:148] K + OE : -24 (cslSTIX-BoldItalic)
    0xfc8000fc,   // [3301:149] K + udieresis : -7 (cslSTIX-BoldItalic)
    0xfc0000f6,   // [3302:150] K + odieresis : -8 (cslSTIX-BoldItalic)
    0xfc0000f3,   // [3303:151] K + oacute : -8 (cslSTIX-BoldItalic)
    0xff0000e6,   // [3304:152] K + ae : -2 (cslSTIX-BoldItalic)
    0x020000e5,   // [3305:153] K + aring : 4 (cslSTIX-BoldItalic)
    0x020000e4,   // [3306:154] K + adieresis : 4 (cslSTIX-BoldItalic)
    0xf30000d6,   // [3307:155] K + Odieresis : -26 (cslSTIX-BoldItalic)
    0xf30000d3,   // [3308:156] K + Oacute : -26 (cslSTIX-BoldItalic)
    0xe7800079,   // [3309:157] K + y : -49 (cslSTIX-BoldItalic)
    0xfc800075,   // [3310:158] K + u : -7 (cslSTIX-BoldItalic)
    0xfc00006f,   // [3311:159] K + o : -8 (cslSTIX-BoldItalic)
    0xfb000065,   // [3312:160] K + e : -10 (cslSTIX-BoldItalic)
    0x02000061,   // [3313:161] K + a : 4 (cslSTIX-BoldItalic)
    0xff800053,   // [3314:162] K + S : -1 (cslSTIX-BoldItalic)
    0xf300004f,   // [3315:163] K + O : -26 (cslSTIX-BoldItalic)
    0xf4800047,   // [3316:164] K + G : -23 (cslSTIX-BoldItalic)
    0xf2800043,   // [3317:165] K + C : -27 (cslSTIX-BoldItalic)
    0xfbc0002d,   // [3318:166] K + hyphen : -9 (cslSTIX-BoldItalic) ;;
    0xf000201d,   // [3319:167] L + quotedblright : -32 (cslSTIX-BoldItalic)
    0xe7802019,   // [3320:168] L + quoteright : -49 (cslSTIX-BoldItalic)
    0x090000fc,   // [3321:169] L + udieresis : 18 (cslSTIX-BoldItalic)
    0xf98000dc,   // [3322:170] L + Udieresis : -13 (cslSTIX-BoldItalic)
    0x020000d6,   // [3323:171] L + Odieresis : 4 (cslSTIX-BoldItalic)
    0x020000d5,   // [3324:172] L + Otilde : 4 (cslSTIX-BoldItalic)
    0x020000d4,   // [3325:173] L + Ocircumflex : 4 (cslSTIX-BoldItalic)
    0x020000d3,   // [3326:174] L + Oacute : 4 (cslSTIX-BoldItalic)
    0x020000d2,   // [3327:175] L + Ograve : 4 (cslSTIX-BoldItalic)
    0x0e8000c6,   // [3328:176] L + AE : 29 (cslSTIX-BoldItalic)
    0x0f8000c5,   // [3329:177] L + Aring : 31 (cslSTIX-BoldItalic)
    0x0f8000c4,   // [3330:178] L + Adieresis : 31 (cslSTIX-BoldItalic)
    0x0f8000c1,   // [3331:179] L + Aacute : 31 (cslSTIX-BoldItalic)
    0xfd000079,   // [3332:180] L + y : -6 (cslSTIX-BoldItalic)
    0x0a800075,   // [3333:181] L + u : 21 (cslSTIX-BoldItalic)
    0xf0000059,   // [3334:182] L + Y : -32 (cslSTIX-BoldItalic)
    0xeb000057,   // [3335:183] L + W : -42 (cslSTIX-BoldItalic)
    0xe8800056,   // [3336:184] L + V : -47 (cslSTIX-BoldItalic)
    0xf9800055,   // [3337:185] L + U : -13 (cslSTIX-BoldItalic)
    0xf4800054,   // [3338:186] L + T : -23 (cslSTIX-BoldItalic)
    0x0200004f,   // [3339:187] L + O : 4 (cslSTIX-BoldItalic)
    0x03000047,   // [3340:188] L + G : 6 (cslSTIX-BoldItalic)
    0x00800043,   // [3341:189] L + C : 1 (cslSTIX-BoldItalic)
    0x0f800041,   // [3342:190] L + A : 31 (cslSTIX-BoldItalic)
    0x2140002d,   // [3343:191] L + hyphen : 66 (cslSTIX-BoldItalic) ;;
    0xf88000fc,   // [3344:192] N + udieresis : -15 (cslSTIX-BoldItalic)
    0xf88000f8,   // [3345:193] N + oslash : -15 (cslSTIX-BoldItalic)
    0xf88000f6,   // [3346:194] N + odieresis : -15 (cslSTIX-BoldItalic)
    0xf88000f3,   // [3347:195] N + oacute : -15 (cslSTIX-BoldItalic)
    0xf70000e9,   // [3348:196] N + eacute : -18 (cslSTIX-BoldItalic)
    0xf90000e6,   // [3349:197] N + ae : -14 (cslSTIX-BoldItalic)
    0xfa0000e5,   // [3350:198] N + aring : -12 (cslSTIX-BoldItalic)
    0xfa0000e4,   // [3351:199] N + adieresis : -12 (cslSTIX-BoldItalic)
    0xfa0000e1,   // [3352:200] N + aacute : -12 (cslSTIX-BoldItalic)
    0xfa8000d6,   // [3353:201] N + Odieresis : -11 (cslSTIX-BoldItalic)
    0xfa8000d3,   // [3354:202] N + Oacute : -11 (cslSTIX-BoldItalic)
    0xfb0000c7,   // [3355:203] N + Ccedilla : -10 (cslSTIX-BoldItalic)
    0xf10000c6,   // [3356:204] N + AE : -30 (cslSTIX-BoldItalic)
    0xf18000c5,   // [3357:205] N + Aring : -29 (cslSTIX-BoldItalic)
    0xf18000c4,   // [3358:206] N + Adieresis : -29 (cslSTIX-BoldItalic)
    0xf18000c1,   // [3359:207] N + Aacute : -29 (cslSTIX-BoldItalic)
    0xf8800075,   // [3360:208] N + u : -15 (cslSTIX-BoldItalic)
    0xfa80006f,   // [3361:209] N + o : -11 (cslSTIX-BoldItalic)
    0xf7000065,   // [3362:210] N + e : -18 (cslSTIX-BoldItalic)
    0xfc000061,   // [3363:211] N + a : -8 (cslSTIX-BoldItalic)
    0xfa80004f,   // [3364:212] N + O : -11 (cslSTIX-BoldItalic)
    0xfd000047,   // [3365:213] N + G : -6 (cslSTIX-BoldItalic)
    0xfb000043,   // [3366:214] N + C : -10 (cslSTIX-BoldItalic)
    0xf1800041,   // [3367:215] N + A : -29 (cslSTIX-BoldItalic)
    0xfc00002e,   // [3368:216] N + period : -8 (cslSTIX-BoldItalic)
    0xfe40002c,   // [3369:217] N + comma : -4 (cslSTIX-BoldItalic) ;;
    0xec0000c6,   // [3370:218] O + AE : -40 (cslSTIX-BoldItalic)
    0xf10000c5,   // [3371:219] O + Aring : -30 (cslSTIX-BoldItalic)
    0xf10000c4,   // [3372:220] O + Adieresis : -30 (cslSTIX-BoldItalic)
    0xf10000c1,   // [3373:221] O + Aacute : -30 (cslSTIX-BoldItalic)
    0xee000059,   // [3374:222] O + Y : -36 (cslSTIX-BoldItalic)
    0xee800058,   // [3375:223] O + X : -35 (cslSTIX-BoldItalic)
    0xf1000057,   // [3376:224] O + W : -30 (cslSTIX-BoldItalic)
    0xf0800056,   // [3377:225] O + V : -31 (cslSTIX-BoldItalic)
    0xf8800054,   // [3378:226] O + T : -15 (cslSTIX-BoldItalic)
    0xf1400041,   // [3379:227] O + A : -30 (cslSTIX-BoldItalic) ;;
    0xeb000153,   // [3380:228] P + oe : -42 (cslSTIX-BoldItalic)
    0xee0000f8,   // [3381:229] P + oslash : -36 (cslSTIX-BoldItalic)
    0xf20000f6,   // [3382:230] P + odieresis : -28 (cslSTIX-BoldItalic)
    0xee0000f3,   // [3383:231] P + oacute : -36 (cslSTIX-BoldItalic)
    0xea0000e9,   // [3384:232] P + eacute : -44 (cslSTIX-BoldItalic)
    0xe98000e6,   // [3385:233] P + ae : -45 (cslSTIX-BoldItalic)
    0xee0000e5,   // [3386:234] P + aring : -36 (cslSTIX-BoldItalic)
    0xef0000e4,   // [3387:235] P + adieresis : -34 (cslSTIX-BoldItalic)
    0xee0000e1,   // [3388:236] P + aacute : -36 (cslSTIX-BoldItalic)
    0xee0000e0,   // [3389:237] P + agrave : -36 (cslSTIX-BoldItalic)
    0xdd8000c6,   // [3390:238] P + AE : -69 (cslSTIX-BoldItalic)
    0xe58000c5,   // [3391:239] P + Aring : -53 (cslSTIX-BoldItalic)
    0xe58000c4,   // [3392:240] P + Adieresis : -53 (cslSTIX-BoldItalic)
    0xe58000c1,   // [3393:241] P + Aacute : -53 (cslSTIX-BoldItalic)
    0xee00006f,   // [3394:242] P + o : -36 (cslSTIX-BoldItalic)
    0xea000065,   // [3395:243] P + e : -44 (cslSTIX-BoldItalic)
    0xed800061,   // [3396:244] P + a : -37 (cslSTIX-BoldItalic)
    0xdd00004a,   // [3397:245] P + J : -70 (cslSTIX-BoldItalic)
    0xe5800041,   // [3398:246] P + A : -53 (cslSTIX-BoldItalic)
    0xdd00002e,   // [3399:247] P + period : -70 (cslSTIX-BoldItalic)
    0xf000002d,   // [3400:248] P + hyphen : -32 (cslSTIX-BoldItalic)
    0xdd40002c,   // [3401:249] P + comma : -70 (cslSTIX-BoldItalic) ;;
    0xf7800153,   // [3402:250] R + oe : -17 (cslSTIX-BoldItalic)
    0xf9000152,   // [3403:251] R + OE : -14 (cslSTIX-BoldItalic)
    0xfb0000fc,   // [3404:252] R + udieresis : -10 (cslSTIX-BoldItalic)
    0xfb0000fa,   // [3405:253] R + uacute : -10 (cslSTIX-BoldItalic)
    0xfa0000f6,   // [3406:254] R + odieresis : -12 (cslSTIX-BoldItalic)
    0xfa0000f3,   // [3407:255] R + oacute : -12 (cslSTIX-BoldItalic)
    0xf90000e9,   // [3408:256] R + eacute : -14 (cslSTIX-BoldItalic)
    0xfd0000e6,   // [3409:257] R + ae : -6 (cslSTIX-BoldItalic)
    0xff8000e5,   // [3410:258] R + aring : -1 (cslSTIX-BoldItalic)
    0xff8000e4,   // [3411:259] R + adieresis : -1 (cslSTIX-BoldItalic)
    0xff8000e1,   // [3412:260] R + aacute : -1 (cslSTIX-BoldItalic)
    0xf50000dc,   // [3413:261] R + Udieresis : -22 (cslSTIX-BoldItalic)
    0xf88000d6,   // [3414:262] R + Odieresis : -15 (cslSTIX-BoldItalic)
    0xf88000d3,   // [3415:263] R + Oacute : -15 (cslSTIX-BoldItalic)
    0xf90000c7,   // [3416:264] R + Ccedilla : -14 (cslSTIX-BoldItalic)
    0xfb000079,   // [3417:265] R + y : -10 (cslSTIX-BoldItalic)
    0xfb000075,   // [3418:266] R + u : -10 (cslSTIX-BoldItalic)
    0xfa00006f,   // [3419:267] R + o : -12 (cslSTIX-BoldItalic)
    0xf9000065,   // [3420:268] R + e : -14 (cslSTIX-BoldItalic)
    0xff800061,   // [3421:269] R + a : -1 (cslSTIX-BoldItalic)
    0xf1000059,   // [3422:270] R + Y : -30 (cslSTIX-BoldItalic)
    0xee800057,   // [3423:271] R + W : -35 (cslSTIX-BoldItalic)
    0xee000056,   // [3424:272] R + V : -36 (cslSTIX-BoldItalic)
    0xf5000055,   // [3425:273] R + U : -22 (cslSTIX-BoldItalic)
    0xf7000054,   // [3426:274] R + T : -18 (cslSTIX-BoldItalic)
    0xf880004f,   // [3427:275] R + O : -15 (cslSTIX-BoldItalic)
    0xfb000047,   // [3428:276] R + G : -10 (cslSTIX-BoldItalic)
    0xf9000043,   // [3429:277] R + C : -14 (cslSTIX-BoldItalic)
    0xfa40002d,   // [3430:278] R + hyphen : -12 (cslSTIX-BoldItalic) ;;
    0xf98000c6,   // [3431:279] S + AE : -13 (cslSTIX-BoldItalic)
    0xfa8000c5,   // [3432:280] S + Aring : -11 (cslSTIX-BoldItalic)
    0xfa8000c4,   // [3433:281] S + Adieresis : -11 (cslSTIX-BoldItalic)
    0xfa8000c1,   // [3434:282] S + Aacute : -11 (cslSTIX-BoldItalic)
    0x02800074,   // [3435:283] S + t : 5 (cslSTIX-BoldItalic)
    0xf7000059,   // [3436:284] S + Y : -18 (cslSTIX-BoldItalic)
    0xf9000057,   // [3437:285] S + W : -14 (cslSTIX-BoldItalic)
    0xf8800056,   // [3438:286] S + V : -15 (cslSTIX-BoldItalic)
    0xf4000054,   // [3439:287] S + T : -24 (cslSTIX-BoldItalic)
    0xfac00041,   // [3440:288] S + A : -11 (cslSTIX-BoldItalic) ;;
    0xd9802039,   // [3441:289] T + guilsinglleft : -77 (cslSTIX-BoldItalic)
    0xf8800152,   // [3442:290] T + OE : -15 (cslSTIX-BoldItalic)
    0xe30000f8,   // [3443:291] T + oslash : -58 (cslSTIX-BoldItalic)
    0xf70000f6,   // [3444:292] T + odieresis : -18 (cslSTIX-BoldItalic)
    0xf20000f4,   // [3445:293] T + ocircumflex : -28 (cslSTIX-BoldItalic)
    0xe98000f3,   // [3446:294] T + oacute : -45 (cslSTIX-BoldItalic)
    0xed0000f2,   // [3447:295] T + ograve : -38 (cslSTIX-BoldItalic)
    0xe10000e6,   // [3448:296] T + ae : -62 (cslSTIX-BoldItalic)
    0xf48000e4,   // [3449:297] T + adieresis : -23 (cslSTIX-BoldItalic)
    0xf30000e3,   // [3450:298] T + atilde : -26 (cslSTIX-BoldItalic)
    0xef0000e2,   // [3451:299] T + acircumflex : -34 (cslSTIX-BoldItalic)
    0xe70000e1,   // [3452:300] T + aacute : -50 (cslSTIX-BoldItalic)
    0xeb0000e0,   // [3453:301] T + agrave : -42 (cslSTIX-BoldItalic)
    0xf70000d8,   // [3454:302] T + Oslash : -18 (cslSTIX-BoldItalic)
    0xf70000d6,   // [3455:303] T + Odieresis : -18 (cslSTIX-BoldItalic)
    0xf70000d5,   // [3456:304] T + Otilde : -18 (cslSTIX-BoldItalic)
    0xf70000d4,   // [3457:305] T + Ocircumflex : -18 (cslSTIX-BoldItalic)
    0xf70000d3,   // [3458:306] T + Oacute : -18 (cslSTIX-BoldItalic)
    0xf70000d2,   // [3459:307] T + Ograve : -18 (cslSTIX-BoldItalic)
    0xeb8000c6,   // [3460:308] T + AE : -41 (cslSTIX-BoldItalic)
    0xec0000c5,   // [3461:309] T + Aring : -40 (cslSTIX-BoldItalic)
    0xec0000c4,   // [3462:310] T + Adieresis : -40 (cslSTIX-BoldItalic)
    0xec0000c3,   // [3463:311] T + Atilde : -40 (cslSTIX-BoldItalic)
    0xec0000c2,   // [3464:312] T + Acircumflex : -40 (cslSTIX-BoldItalic)
    0xec0000c1,   // [3465:313] T + Aacute : -40 (cslSTIX-BoldItalic)
    0xec0000c0,   // [3466:314] T + Agrave : -40 (cslSTIX-BoldItalic)
    0xdd8000ab,   // [3467:315] T + guillemotleft : -69 (cslSTIX-BoldItalic)
    0xdf800079,   // [3468:316] T + y : -65 (cslSTIX-BoldItalic)
    0xdf000077,   // [3469:317] T + w : -66 (cslSTIX-BoldItalic)
    0xdf000076,   // [3470:318] T + v : -66 (cslSTIX-BoldItalic)
    0xe1800075,   // [3471:319] T + u : -61 (cslSTIX-BoldItalic)
    0xe3800073,   // [3472:320] T + s : -57 (cslSTIX-BoldItalic)
    0xe4000072,   // [3473:321] T + r : -56 (cslSTIX-BoldItalic)
    0xe300006f,   // [3474:322] T + o : -58 (cslSTIX-BoldItalic)
    0xfd00006a,   // [3475:323] T + j : -6 (cslSTIX-BoldItalic)
    0xff000069,   // [3476:324] T + i : -2 (cslSTIX-BoldItalic)
    0xe0800067,   // [3477:325] T + g : -63 (cslSTIX-BoldItalic)
    0xe1800065,   // [3478:326] T + e : -61 (cslSTIX-BoldItalic)
    0xe3000063,   // [3479:327] T + c : -58 (cslSTIX-BoldItalic)
    0xe4000061,   // [3480:328] T + a : -56 (cslSTIX-BoldItalic)
    0x07000059,   // [3481:329] T + Y : 14 (cslSTIX-BoldItalic)
    0x09000057,   // [3482:330] T + W : 18 (cslSTIX-BoldItalic)
    0x08000056,   // [3483:331] T + V : 16 (cslSTIX-BoldItalic)
    0xfa000053,   // [3484:332] T + S : -12 (cslSTIX-BoldItalic)
    0xf700004f,   // [3485:333] T + O : -18 (cslSTIX-BoldItalic)
    0xe800004a,   // [3486:334] T + J : -48 (cslSTIX-BoldItalic)
    0xfc000047,   // [3487:335] T + G : -8 (cslSTIX-BoldItalic)
    0xfa000043,   // [3488:336] T + C : -12 (cslSTIX-BoldItalic)
    0xec000041,   // [3489:337] T + A : -40 (cslSTIX-BoldItalic)
    0xed80003b,   // [3490:338] T + semicolon : -37 (cslSTIX-BoldItalic)
    0xee80003a,   // [3491:339] T + colon : -35 (cslSTIX-BoldItalic)
    0xe880002e,   // [3492:340] T + period : -47 (cslSTIX-BoldItalic)
    0xea80002d,   // [3493:341] T + hyphen : -43 (cslSTIX-BoldItalic)
    0xe940002c,   // [3494:342] T + comma : -46 (cslSTIX-BoldItalic) ;;
    0xe68000c6,   // [3495:343] U + AE : -51 (cslSTIX-BoldItalic)
    0xeb8000c5,   // [3496:344] U + Aring : -41 (cslSTIX-BoldItalic)
    0xeb8000c4,   // [3497:345] U + Adieresis : -41 (cslSTIX-BoldItalic)
    0xeb8000c3,   // [3498:346] U + Atilde : -41 (cslSTIX-BoldItalic)
    0xeb8000c2,   // [3499:347] U + Acircumflex : -41 (cslSTIX-BoldItalic)
    0xeb8000c1,   // [3500:348] U + Aacute : -41 (cslSTIX-BoldItalic)
    0xf7800072,   // [3501:349] U + r : -17 (cslSTIX-BoldItalic)
    0xf8000070,   // [3502:350] U + p : -16 (cslSTIX-BoldItalic)
    0xf500006e,   // [3503:351] U + n : -22 (cslSTIX-BoldItalic)
    0xf680006d,   // [3504:352] U + m : -19 (cslSTIX-BoldItalic)
    0xeb800041,   // [3505:353] U + A : -41 (cslSTIX-BoldItalic)
    0xf780002e,   // [3506:354] U + period : -17 (cslSTIX-BoldItalic)
    0xf740002c,   // [3507:355] U + comma : -18 (cslSTIX-BoldItalic) ;;
    0xdd802039,   // [3508:356] V + guilsinglleft : -69 (cslSTIX-BoldItalic)
    0xe70000f8,   // [3509:357] V + oslash : -50 (cslSTIX-BoldItalic)
    0xf30000f6,   // [3510:358] V + odieresis : -26 (cslSTIX-BoldItalic)
    0xed0000f5,   // [3511:359] V + otilde : -38 (cslSTIX-BoldItalic)
    0xe70000f4,   // [3512:360] V + ocircumflex : -50 (cslSTIX-BoldItalic)
    0xe70000f3,   // [3513:361] V + oacute : -50 (cslSTIX-BoldItalic)
    0xef0000f2,   // [3514:362] V + ograve : -34 (cslSTIX-BoldItalic)
    0xe30000e9,   // [3515:363] V + eacute : -58 (cslSTIX-BoldItalic)
    0xf28000e8,   // [3516:364] V + egrave : -27 (cslSTIX-BoldItalic)
    0xe50000e6,   // [3517:365] V + ae : -54 (cslSTIX-BoldItalic)
    0xeb8000e5,   // [3518:366] V + aring : -41 (cslSTIX-BoldItalic)
    0xf08000e4,   // [3519:367] V + adieresis : -31 (cslSTIX-BoldItalic)
    0xeb0000e3,   // [3520:368] V + atilde : -42 (cslSTIX-BoldItalic)
    0xe80000e2,   // [3521:369] V + acircumflex : -48 (cslSTIX-BoldItalic)
    0xe80000e1,   // [3522:370] V + aacute : -48 (cslSTIX-BoldItalic)
    0xed0000e0,   // [3523:371] V + agrave : -38 (cslSTIX-BoldItalic)
    0xee0000d8,   // [3524:372] V + Oslash : -36 (cslSTIX-BoldItalic)
    0xee0000d6,   // [3525:373] V + Odieresis : -36 (cslSTIX-BoldItalic)
    0xee0000d5,   // [3526:374] V + Otilde : -36 (cslSTIX-BoldItalic)
    0xee0000d4,   // [3527:375] V + Ocircumflex : -36 (cslSTIX-BoldItalic)
    0xee0000d3,   // [3528:376] V + Oacute : -36 (cslSTIX-BoldItalic)
    0xee0000d2,   // [3529:377] V + Ograve : -36 (cslSTIX-BoldItalic)
    0xdc8000c6,   // [3530:378] V + AE : -71 (cslSTIX-BoldItalic)
    0xdf8000c5,   // [3531:379] V + Aring : -65 (cslSTIX-BoldItalic)
    0xdf8000c4,   // [3532:380] V + Adieresis : -65 (cslSTIX-BoldItalic)
    0xdf8000c3,   // [3533:381] V + Atilde : -65 (cslSTIX-BoldItalic)
    0xdf8000c2,   // [3534:382] V + Acircumflex : -65 (cslSTIX-BoldItalic)
    0xdf8000c1,   // [3535:383] V + Aacute : -65 (cslSTIX-BoldItalic)
    0xdf8000c0,   // [3536:384] V + Agrave : -65 (cslSTIX-BoldItalic)
    0xe18000ab,   // [3537:385] V + guillemotleft : -61 (cslSTIX-BoldItalic)
    0xf2000079,   // [3538:386] V + y : -28 (cslSTIX-BoldItalic)
    0xef000075,   // [3539:387] V + u : -34 (cslSTIX-BoldItalic)
    0xf1000072,   // [3540:388] V + r : -30 (cslSTIX-BoldItalic)
    0xe700006f,   // [3541:389] V + o : -50 (cslSTIX-BoldItalic)
    0xfb000069,   // [3542:390] V + i : -10 (cslSTIX-BoldItalic)
    0xe5000067,   // [3543:391] V + g : -54 (cslSTIX-BoldItalic)
    0xe3000065,   // [3544:392] V + e : -58 (cslSTIX-BoldItalic)
    0xe8000061,   // [3545:393] V + a : -48 (cslSTIX-BoldItalic)
    0x04000054,   // [3546:394] V + T : 8 (cslSTIX-BoldItalic)
    0xf5000053,   // [3547:395] V + S : -22 (cslSTIX-BoldItalic)
    0xee00004f,   // [3548:396] V + O : -36 (cslSTIX-BoldItalic)
    0xf1000047,   // [3549:397] V + G : -30 (cslSTIX-BoldItalic)
    0xef000043,   // [3550:398] V + C : -34 (cslSTIX-BoldItalic)
    0xdf800041,   // [3551:399] V + A : -65 (cslSTIX-BoldItalic)
    0xf200003b,   // [3552:400] V + semicolon : -28 (cslSTIX-BoldItalic)
    0xf000003a,   // [3553:401] V + colon : -32 (cslSTIX-BoldItalic)
    0xe300002e,   // [3554:402] V + period : -58 (cslSTIX-BoldItalic)
    0xee80002d,   // [3555:403] V + hyphen : -35 (cslSTIX-BoldItalic)
    0xe340002c,   // [3556:404] V + comma : -58 (cslSTIX-BoldItalic) ;;
    0xe7002039,   // [3557:405] W + guilsinglleft : -50 (cslSTIX-BoldItalic)
    0xf00000f8,   // [3558:406] W + oslash : -32 (cslSTIX-BoldItalic)
    0xf18000f6,   // [3559:407] W + odieresis : -29 (cslSTIX-BoldItalic)
    0xf00000f5,   // [3560:408] W + otilde : -32 (cslSTIX-BoldItalic)
    0xf00000f4,   // [3561:409] W + ocircumflex : -32 (cslSTIX-BoldItalic)
    0xf00000f3,   // [3562:410] W + oacute : -32 (cslSTIX-BoldItalic)
    0xf00000f2,   // [3563:411] W + ograve : -32 (cslSTIX-BoldItalic)
    0xec8000e9,   // [3564:412] W + eacute : -39 (cslSTIX-BoldItalic)
    0xf30000e8,   // [3565:413] W + egrave : -26 (cslSTIX-BoldItalic)
    0xee0000e6,   // [3566:414] W + ae : -36 (cslSTIX-BoldItalic)
    0xf10000e5,   // [3567:415] W + aring : -30 (cslSTIX-BoldItalic)
    0xf10000e4,   // [3568:416] W + adieresis : -30 (cslSTIX-BoldItalic)
    0xf10000e3,   // [3569:417] W + atilde : -30 (cslSTIX-BoldItalic)
    0xf10000e2,   // [3570:418] W + acircumflex : -30 (cslSTIX-BoldItalic)
    0xf10000e1,   // [3571:419] W + aacute : -30 (cslSTIX-BoldItalic)
    0xf10000e0,   // [3572:420] W + agrave : -30 (cslSTIX-BoldItalic)
    0xf48000d8,   // [3573:421] W + Oslash : -23 (cslSTIX-BoldItalic)
    0xf48000d6,   // [3574:422] W + Odieresis : -23 (cslSTIX-BoldItalic)
    0xf48000d5,   // [3575:423] W + Otilde : -23 (cslSTIX-BoldItalic)
    0xf48000d4,   // [3576:424] W + Ocircumflex : -23 (cslSTIX-BoldItalic)
    0xf48000d3,   // [3577:425] W + Oacute : -23 (cslSTIX-BoldItalic)
    0xf48000d2,   // [3578:426] W + Ograve : -23 (cslSTIX-BoldItalic)
    0xe20000c6,   // [3579:427] W + AE : -60 (cslSTIX-BoldItalic)
    0xe78000c5,   // [3580:428] W + Aring : -49 (cslSTIX-BoldItalic)
    0xe78000c4,   // [3581:429] W + Adieresis : -49 (cslSTIX-BoldItalic)
    0xe78000c3,   // [3582:430] W + Atilde : -49 (cslSTIX-BoldItalic)
    0xe78000c2,   // [3583:431] W + Acircumflex : -49 (cslSTIX-BoldItalic)
    0xe78000c1,   // [3584:432] W + Aacute : -49 (cslSTIX-BoldItalic)
    0xe78000c0,   // [3585:433] W + Agrave : -49 (cslSTIX-BoldItalic)
    0xeb0000ab,   // [3586:434] W + guillemotleft : -42 (cslSTIX-BoldItalic)
    0xf5800079,   // [3587:435] W + y : -21 (cslSTIX-BoldItalic)
    0xf3000075,   // [3588:436] W + u : -26 (cslSTIX-BoldItalic)
    0xf4800072,   // [3589:437] W + r : -23 (cslSTIX-BoldItalic)
    0xf000006f,   // [3590:438] W + o : -32 (cslSTIX-BoldItalic)
    0xfc000069,   // [3591:439] W + i : -8 (cslSTIX-BoldItalic)
    0xec000067,   // [3592:440] W + g : -40 (cslSTIX-BoldItalic)
    0xec800065,   // [3593:441] W + e : -39 (cslSTIX-BoldItalic)
    0xf1000061,   // [3594:442] W + a : -30 (cslSTIX-BoldItalic)
    0x04800054,   // [3595:443] W + T : 9 (cslSTIX-BoldItalic)
    0xf5800053,   // [3596:444] W + S : -21 (cslSTIX-BoldItalic)
    0xf480004f,   // [3597:445] W + O : -23 (cslSTIX-BoldItalic)
    0xf7800047,   // [3598:446] W + G : -17 (cslSTIX-BoldItalic)
    0xf5000043,   // [3599:447] W + C : -22 (cslSTIX-BoldItalic)
    0xe7800041,   // [3600:448] W + A : -49 (cslSTIX-BoldItalic)
    0xf980003b,   // [3601:449] W + semicolon : -13 (cslSTIX-BoldItalic)
    0xf980003a,   // [3602:450] W + colon : -13 (cslSTIX-BoldItalic)
    0xef00002e,   // [3603:451] W + period : -34 (cslSTIX-BoldItalic)
    0xf700002d,   // [3604:452] W + hyphen : -18 (cslSTIX-BoldItalic)
    0xefc0002c,   // [3605:453] W + comma : -33 (cslSTIX-BoldItalic) ;;
    0xf00000d6,   // [3606:454] X + Odieresis : -32 (cslSTIX-BoldItalic)
    0xe3800079,   // [3607:455] X + y : -57 (cslSTIX-BoldItalic)
    0xf7800075,   // [3608:456] X + u : -17 (cslSTIX-BoldItalic)
    0xf700006f,   // [3609:457] X + o : -18 (cslSTIX-BoldItalic)
    0xf6000065,   // [3610:458] X + e : -20 (cslSTIX-BoldItalic)
    0xfc800061,   // [3611:459] X + a : -7 (cslSTIX-BoldItalic)
    0xf0000051,   // [3612:460] X + Q : -32 (cslSTIX-BoldItalic)
    0xf000004f,   // [3613:461] X + O : -32 (cslSTIX-BoldItalic)
    0xf1000043,   // [3614:462] X + C : -30 (cslSTIX-BoldItalic)
    0xf4c0002d,   // [3615:463] X + hyphen : -23 (cslSTIX-BoldItalic) ;;
    0xdd802039,   // [3616:464] Y + guilsinglleft : -69 (cslSTIX-BoldItalic)
    0xe70000f8,   // [3617:465] Y + oslash : -50 (cslSTIX-BoldItalic)
    0xe70000e6,   // [3618:466] Y + ae : -50 (cslSTIX-BoldItalic)
    0xee0000d8,   // [3619:467] Y + Oslash : -36 (cslSTIX-BoldItalic)
    0xf00000d6,   // [3620:468] Y + Odieresis : -32 (cslSTIX-BoldItalic)
    0xf00000d5,   // [3621:469] Y + Otilde : -32 (cslSTIX-BoldItalic)
    0xf00000d4,   // [3622:470] Y + Ocircumflex : -32 (cslSTIX-BoldItalic)
    0xf00000d3,   // [3623:471] Y + Oacute : -32 (cslSTIX-BoldItalic)
    0xf00000d2,   // [3624:472] Y + Ograve : -32 (cslSTIX-BoldItalic)
    0xee8000c6,   // [3625:473] Y + AE : -35 (cslSTIX-BoldItalic)
    0xef0000c5,   // [3626:474] Y + Aring : -34 (cslSTIX-BoldItalic)
    0xef0000c4,   // [3627:475] Y + Adieresis : -34 (cslSTIX-BoldItalic)
    0xef0000c3,   // [3628:476] Y + Atilde : -34 (cslSTIX-BoldItalic)
    0xef0000c2,   // [3629:477] Y + Acircumflex : -34 (cslSTIX-BoldItalic)
    0xef0000c1,   // [3630:478] Y + Aacute : -34 (cslSTIX-BoldItalic)
    0xef0000c0,   // [3631:479] Y + Agrave : -34 (cslSTIX-BoldItalic)
    0xe18000ab,   // [3632:480] Y + guillemotleft : -61 (cslSTIX-BoldItalic)
    0xed000076,   // [3633:481] Y + v : -38 (cslSTIX-BoldItalic)
    0xeb000075,   // [3634:482] Y + u : -42 (cslSTIX-BoldItalic)
    0xeb800070,   // [3635:483] Y + p : -41 (cslSTIX-BoldItalic)
    0xe900006f,   // [3636:484] Y + o : -46 (cslSTIX-BoldItalic)
    0xfb000069,   // [3637:485] Y + i : -10 (cslSTIX-BoldItalic)
    0xe7000067,   // [3638:486] Y + g : -50 (cslSTIX-BoldItalic)
    0xe5000065,   // [3639:487] Y + e : -54 (cslSTIX-BoldItalic)
    0xea800061,   // [3640:488] Y + a : -43 (cslSTIX-BoldItalic)
    0x04000054,   // [3641:489] Y + T : 8 (cslSTIX-BoldItalic)
    0xf5000053,   // [3642:490] Y + S : -22 (cslSTIX-BoldItalic)
    0xf000004f,   // [3643:491] Y + O : -32 (cslSTIX-BoldItalic)
    0xf1000047,   // [3644:492] Y + G : -30 (cslSTIX-BoldItalic)
    0xef000043,   // [3645:493] Y + C : -34 (cslSTIX-BoldItalic)
    0xef000041,   // [3646:494] Y + A : -34 (cslSTIX-BoldItalic)
    0xf480003b,   // [3647:495] Y + semicolon : -23 (cslSTIX-BoldItalic)
    0xf500003a,   // [3648:496] Y + colon : -22 (cslSTIX-BoldItalic)
    0xec80002e,   // [3649:497] Y + period : -39 (cslSTIX-BoldItalic)
    0xee80002d,   // [3650:498] Y + hyphen : -35 (cslSTIX-BoldItalic)
    0xed40002c,   // [3651:499] Y + comma : -38 (cslSTIX-BoldItalic) ;;
    0xf5000079,   // [3652:500] Z + y : -22 (cslSTIX-BoldItalic)
    0xf9000077,   // [3653:501] Z + w : -14 (cslSTIX-BoldItalic)
    0xf9400076,   // [3654:502] Z + v : -14 (cslSTIX-BoldItalic) ;;
    0xf9002019,   // [3655:503] a + quoteright : -14 (cslSTIX-BoldItalic)
    0xff000079,   // [3656:504] a + y : -2 (cslSTIX-BoldItalic)
    0xff000077,   // [3657:505] a + w : -2 (cslSTIX-BoldItalic)
    0xff000076,   // [3658:506] a + v : -2 (cslSTIX-BoldItalic)
    0x00c0006a,   // [3659:507] a + j : 1 (cslSTIX-BoldItalic) ;;
    0xfa800079,   // [3660:508] b + y : -11 (cslSTIX-BoldItalic)
    0xfa000077,   // [3661:509] b + w : -12 (cslSTIX-BoldItalic)
    0xfa400076,   // [3662:510] b + v : -12 (cslSTIX-BoldItalic) ;;
    0xfc00006b,   // [3663:511] c + k : -8 (cslSTIX-BoldItalic)
    0xfa400068,   // [3664:512] c + h : -12 (cslSTIX-BoldItalic) ;;
    0xfc002019,   // [3665:513] e + quoteright : -8 (cslSTIX-BoldItalic)
    0xff000079,   // [3666:514] e + y : -2 (cslSTIX-BoldItalic)
    0xfa800078,   // [3667:515] e + x : -11 (cslSTIX-BoldItalic)
    0xfe000077,   // [3668:516] e + w : -4 (cslSTIX-BoldItalic)
    0xfe000076,   // [3669:517] e + v : -4 (cslSTIX-BoldItalic)
    0x00c00074,   // [3670:518] e + t : 1 (cslSTIX-BoldItalic) ;;
    0x07002019,   // [3671:519] f + quoteright : 14 (cslSTIX-BoldItalic)
    0xf5800153,   // [3672:520] f + oe : -21 (cslSTIX-BoldItalic)
    0xf70000f8,   // [3673:521] f + oslash : -18 (cslSTIX-BoldItalic)
    0x0b8000f6,   // [3674:522] f + odieresis : 23 (cslSTIX-BoldItalic)
    0xfa8000f4,   // [3675:523] f + ocircumflex : -11 (cslSTIX-BoldItalic)
    0xf70000f3,   // [3676:524] f + oacute : -18 (cslSTIX-BoldItalic)
    0xfb0000f2,   // [3677:525] f + ograve : -10 (cslSTIX-BoldItalic)
    0xfa8000ea,   // [3678:526] f + ecircumflex : -11 (cslSTIX-BoldItalic)
    0xf60000e9,   // [3679:527] f + eacute : -20 (cslSTIX-BoldItalic)
    0xfe0000e8,   // [3680:528] f + egrave : -4 (cslSTIX-BoldItalic)
    0xf78000e6,   // [3681:529] f + ae : -17 (cslSTIX-BoldItalic)
    0xfb0000e5,   // [3682:530] f + aring : -10 (cslSTIX-BoldItalic)
    0x070000e4,   // [3683:531] f + adieresis : 14 (cslSTIX-BoldItalic)
    0xfb0000e1,   // [3684:532] f + aacute : -10 (cslSTIX-BoldItalic)
    0x05000074,   // [3685:533] f + t : 10 (cslSTIX-BoldItalic)
    0xfc000073,   // [3686:534] f + s : -8 (cslSTIX-BoldItalic)
    0xf900006f,   // [3687:535] f + o : -14 (cslSTIX-BoldItalic)
    0x1880006c,   // [3688:536] f + l : 49 (cslSTIX-BoldItalic)
    0x0700006a,   // [3689:537] f + j : 14 (cslSTIX-BoldItalic)
    0x0b800069,   // [3690:538] f + i : 23 (cslSTIX-BoldItalic)
    0x00800066,   // [3691:539] f + f : 1 (cslSTIX-BoldItalic)
    0xf6000065,   // [3692:540] f + e : -20 (cslSTIX-BoldItalic)
    0xfb000061,   // [3693:541] f + a : -10 (cslSTIX-BoldItalic)
    0x1d20006a,   // [3694:542] f + j ligature #58 (cslSTIX-BoldItalic)
    0x1da0006c,   // [3695:543] f + l ligature #59 (cslSTIX-BoldItalic)
    0x1e200069,   // [3696:544] f + i ligature #60 (cslSTIX-BoldItalic)
    0x1ee00066,   // [3697:545] f + f ligature #61 (cslSTIX-BoldItalic) ;;
    0xf78000f6,   // [3698:546] g + odieresis : -17 (cslSTIX-BoldItalic)
    0xf78000f3,   // [3699:547] g + oacute : -17 (cslSTIX-BoldItalic)
    0xf38000eb,   // [3700:548] g + edieresis : -25 (cslSTIX-BoldItalic)
    0xf38000ea,   // [3701:549] g + ecircumflex : -25 (cslSTIX-BoldItalic)
    0xf38000e9,   // [3702:550] g + eacute : -25 (cslSTIX-BoldItalic)
    0xf38000e8,   // [3703:551] g + egrave : -25 (cslSTIX-BoldItalic)
    0xf38000e6,   // [3704:552] g + ae : -25 (cslSTIX-BoldItalic)
    0xf70000e5,   // [3705:553] g + aring : -18 (cslSTIX-BoldItalic)
    0xf70000e4,   // [3706:554] g + adieresis : -18 (cslSTIX-BoldItalic)
    0x02000072,   // [3707:555] g + r : 4 (cslSTIX-BoldItalic)
    0xf780006c,   // [3708:556] g + l : -17 (cslSTIX-BoldItalic)
    0xf3800065,   // [3709:557] g + e : -25 (cslSTIX-BoldItalic)
    0xf7400061,   // [3710:558] g + a : -18 (cslSTIX-BoldItalic) ;;
    0xf3802019,   // [3711:559] h + quoteright : -25 (cslSTIX-BoldItalic)
    0xf9400079,   // [3712:560] h + y : -14 (cslSTIX-BoldItalic) ;;
    0xf8000054,   // [3713:561] i + T : -16 (cslSTIX-BoldItalic)
    0x1f60006a,   // [3714:562] i + j ligature #62 (cslSTIX-BoldItalic) ;;
    0x018000fc,   // [3715:563] k + udieresis : 3 (cslSTIX-BoldItalic)
    0x020000f6,   // [3716:564] k + odieresis : 4 (cslSTIX-BoldItalic)
    0x020000f3,   // [3717:565] k + oacute : 4 (cslSTIX-BoldItalic)
    0x020000e6,   // [3718:566] k + ae : 4 (cslSTIX-BoldItalic)
    0x070000e5,   // [3719:567] k + aring : 14 (cslSTIX-BoldItalic)
    0x070000e4,   // [3720:568] k + adieresis : 14 (cslSTIX-BoldItalic)
    0x070000e1,   // [3721:569] k + aacute : 14 (cslSTIX-BoldItalic)
    0x01800075,   // [3722:570] k + u : 3 (cslSTIX-BoldItalic)
    0x01800073,   // [3723:571] k + s : 3 (cslSTIX-BoldItalic)
    0x0200006f,   // [3724:572] k + o : 4 (cslSTIX-BoldItalic)
    0xfb000067,   // [3725:573] k + g : -10 (cslSTIX-BoldItalic)
    0x07000061,   // [3726:574] k + a : 14 (cslSTIX-BoldItalic)
    0x0c80002e,   // [3727:575] k + period : 25 (cslSTIX-BoldItalic)
    0x0200002d,   // [3728:576] k + hyphen : 4 (cslSTIX-BoldItalic)
    0x0ec0002c,   // [3729:577] k + comma : 29 (cslSTIX-BoldItalic) ;;
    0xfd000079,   // [3730:578] l + y : -6 (cslSTIX-BoldItalic)
    0xfc400076,   // [3731:579] l + v : -8 (cslSTIX-BoldItalic) ;;
    0xfa800079,   // [3732:580] m + y : -11 (cslSTIX-BoldItalic)
    0xf9800077,   // [3733:581] m + w : -13 (cslSTIX-BoldItalic)
    0xfa800076,   // [3734:582] m + v : -11 (cslSTIX-BoldItalic)
    0x02c00070,   // [3735:583] m + p : 5 (cslSTIX-BoldItalic) ;;
    0xf4802019,   // [3736:584] n + quoteright : -23 (cslSTIX-BoldItalic)
    0xf9000079,   // [3737:585] n + y : -14 (cslSTIX-BoldItalic)
    0xf8000077,   // [3738:586] n + w : -16 (cslSTIX-BoldItalic)
    0xf8000076,   // [3739:587] n + v : -16 (cslSTIX-BoldItalic)
    0x00800070,   // [3740:588] n + p : 1 (cslSTIX-BoldItalic)
    0xeac00054,   // [3741:589] n + T : -43 (cslSTIX-BoldItalic) ;;
    0xf9002019,   // [3742:590] o + quoteright : -14 (cslSTIX-BoldItalic)
    0xf8000079,   // [3743:591] o + y : -16 (cslSTIX-BoldItalic)
    0xf7000078,   // [3744:592] o + x : -18 (cslSTIX-BoldItalic)
    0xf8800077,   // [3745:593] o + w : -15 (cslSTIX-BoldItalic)
    0xf8800076,   // [3746:594] o + v : -15 (cslSTIX-BoldItalic)
    0x02000074,   // [3747:595] o + t : 4 (cslSTIX-BoldItalic)
    0xe8c00054,   // [3748:596] o + T : -47 (cslSTIX-BoldItalic) ;;
    0xfc000079,   // [3749:597] p + y : -8 (cslSTIX-BoldItalic)
    0x02400074,   // [3750:598] p + t : 4 (cslSTIX-BoldItalic) ;;
    0xff000075,   // [3751:599] q + u : -2 (cslSTIX-BoldItalic)
    0xff400063,   // [3752:600] q + c : -2 (cslSTIX-BoldItalic) ;;
    0x02802019,   // [3753:601] r + quoteright : 5 (cslSTIX-BoldItalic)
    0xf8800153,   // [3754:602] r + oe : -15 (cslSTIX-BoldItalic)
    0xfb0000f8,   // [3755:603] r + oslash : -10 (cslSTIX-BoldItalic)
    0xfb8000f6,   // [3756:604] r + odieresis : -9 (cslSTIX-BoldItalic)
    0xfb8000f4,   // [3757:605] r + ocircumflex : -9 (cslSTIX-BoldItalic)
    0xfb8000f3,   // [3758:606] r + oacute : -9 (cslSTIX-BoldItalic)
    0xfb8000f2,   // [3759:607] r + ograve : -9 (cslSTIX-BoldItalic)
    0xf78000ea,   // [3760:608] r + ecircumflex : -17 (cslSTIX-BoldItalic)
    0xf78000e9,   // [3761:609] r + eacute : -17 (cslSTIX-BoldItalic)
    0xf78000e8,   // [3762:610] r + egrave : -17 (cslSTIX-BoldItalic)
    0xfb0000e7,   // [3763:611] r + ccedilla : -10 (cslSTIX-BoldItalic)
    0xf70000e6,   // [3764:612] r + ae : -18 (cslSTIX-BoldItalic)
    0xfb0000e5,   // [3765:613] r + aring : -10 (cslSTIX-BoldItalic)
    0xfb0000e4,   // [3766:614] r + adieresis : -10 (cslSTIX-BoldItalic)
    0xfb0000e2,   // [3767:615] r + acircumflex : -10 (cslSTIX-BoldItalic)
    0xfb0000e1,   // [3768:616] r + aacute : -10 (cslSTIX-BoldItalic)
    0xfb0000e0,   // [3769:617] r + agrave : -10 (cslSTIX-BoldItalic)
    0x0580007a,   // [3770:618] r + z : 11 (cslSTIX-BoldItalic)
    0x0c800079,   // [3771:619] r + y : 25 (cslSTIX-BoldItalic)
    0x04000078,   // [3772:620] r + x : 8 (cslSTIX-BoldItalic)
    0x0b800077,   // [3773:621] r + w : 23 (cslSTIX-BoldItalic)
    0x0b800076,   // [3774:622] r + v : 23 (cslSTIX-BoldItalic)
    0x06800075,   // [3775:623] r + u : 13 (cslSTIX-BoldItalic)
    0x07800074,   // [3776:624] r + t : 15 (cslSTIX-BoldItalic)
    0x09000072,   // [3777:625] r + r : 18 (cslSTIX-BoldItalic)
    0xf7000071,   // [3778:626] r + q : -18 (cslSTIX-BoldItalic)
    0x09800070,   // [3779:627] r + p : 19 (cslSTIX-BoldItalic)
    0xfb80006f,   // [3780:628] r + o : -9 (cslSTIX-BoldItalic)
    0x0400006e,   // [3781:629] r + n : 8 (cslSTIX-BoldItalic)
    0x0680006d,   // [3782:630] r + m : 13 (cslSTIX-BoldItalic)
    0xfb00006c,   // [3783:631] r + l : -10 (cslSTIX-BoldItalic)
    0xfb00006b,   // [3784:632] r + k : -10 (cslSTIX-BoldItalic)
    0x0700006a,   // [3785:633] r + j : 14 (cslSTIX-BoldItalic)
    0x0a000069,   // [3786:634] r + i : 20 (cslSTIX-BoldItalic)
    0xfa000068,   // [3787:635] r + h : -12 (cslSTIX-BoldItalic)
    0xfe800067,   // [3788:636] r + g : -3 (cslSTIX-BoldItalic)
    0x04800066,   // [3789:637] r + f : 9 (cslSTIX-BoldItalic)
    0xf7800065,   // [3790:638] r + e : -17 (cslSTIX-BoldItalic)
    0xfb000064,   // [3791:639] r + d : -10 (cslSTIX-BoldItalic)
    0xfa000063,   // [3792:640] r + c : -12 (cslSTIX-BoldItalic)
    0xfb000061,   // [3793:641] r + a : -10 (cslSTIX-BoldItalic)
    0xf900003b,   // [3794:642] r + semicolon : -14 (cslSTIX-BoldItalic)
    0xfa00003a,   // [3795:643] r + colon : -12 (cslSTIX-BoldItalic)
    0xe700002e,   // [3796:644] r + period : -50 (cslSTIX-BoldItalic)
    0xef80002d,   // [3797:645] r + hyphen : -33 (cslSTIX-BoldItalic)
    0xe940002c,   // [3798:646] r + comma : -46 (cslSTIX-BoldItalic) ;;
    0xf9002019,   // [3799:647] s + quoteright : -14 (cslSTIX-BoldItalic)
    0xff400074,   // [3800:648] s + t : -2 (cslSTIX-BoldItalic) ;;
    0xfb002019,   // [3801:649] t + quoteright : -10 (cslSTIX-BoldItalic)
    0x018000f6,   // [3802:650] t + odieresis : 3 (cslSTIX-BoldItalic)
    0x018000f3,   // [3803:651] t + oacute : 3 (cslSTIX-BoldItalic)
    0xff8000e9,   // [3804:652] t + eacute : -1 (cslSTIX-BoldItalic)
    0x020000e6,   // [3805:653] t + ae : 4 (cslSTIX-BoldItalic)
    0x070000e5,   // [3806:654] t + aring : 14 (cslSTIX-BoldItalic)
    0x070000e4,   // [3807:655] t + adieresis : 14 (cslSTIX-BoldItalic)
    0x070000e1,   // [3808:656] t + aacute : 14 (cslSTIX-BoldItalic)
    0x0180006f,   // [3809:657] t + o : 3 (cslSTIX-BoldItalic)
    0xff800065,   // [3810:658] t + e : -1 (cslSTIX-BoldItalic)
    0x07000061,   // [3811:659] t + a : 14 (cslSTIX-BoldItalic)
    0x0a80003b,   // [3812:660] t + semicolon : 21 (cslSTIX-BoldItalic)
    0x0c40003a,   // [3813:661] t + colon : 24 (cslSTIX-BoldItalic) ;;
    0xf4402019,   // [3814:662] u + quoteright : -24 (cslSTIX-BoldItalic) ;;
    0xf88000f8,   // [3815:663] v + oslash : -15 (cslSTIX-BoldItalic)
    0xf98000f6,   // [3816:664] v + odieresis : -13 (cslSTIX-BoldItalic)
    0xf98000f3,   // [3817:665] v + oacute : -13 (cslSTIX-BoldItalic)
    0xf98000f2,   // [3818:666] v + ograve : -13 (cslSTIX-BoldItalic)
    0xf70000ea,   // [3819:667] v + ecircumflex : -18 (cslSTIX-BoldItalic)
    0xf70000e9,   // [3820:668] v + eacute : -18 (cslSTIX-BoldItalic)
    0xf70000e8,   // [3821:669] v + egrave : -18 (cslSTIX-BoldItalic)
    0xf78000e6,   // [3822:670] v + ae : -17 (cslSTIX-BoldItalic)
    0xfa8000e5,   // [3823:671] v + aring : -11 (cslSTIX-BoldItalic)
    0xfa8000e4,   // [3824:672] v + adieresis : -11 (cslSTIX-BoldItalic)
    0xfa8000e3,   // [3825:673] v + atilde : -11 (cslSTIX-BoldItalic)
    0xfa8000e2,   // [3826:674] v + acircumflex : -11 (cslSTIX-BoldItalic)
    0xfa8000e1,   // [3827:675] v + aacute : -11 (cslSTIX-BoldItalic)
    0xfa8000e0,   // [3828:676] v + agrave : -11 (cslSTIX-BoldItalic)
    0xfa000073,   // [3829:677] v + s : -12 (cslSTIX-BoldItalic)
    0xf980006f,   // [3830:678] v + o : -13 (cslSTIX-BoldItalic)
    0xf900006c,   // [3831:679] v + l : -14 (cslSTIX-BoldItalic)
    0xf8800067,   // [3832:680] v + g : -15 (cslSTIX-BoldItalic)
    0xf7000065,   // [3833:681] v + e : -18 (cslSTIX-BoldItalic)
    0xf9000063,   // [3834:682] v + c : -14 (cslSTIX-BoldItalic)
    0xfa800061,   // [3835:683] v + a : -11 (cslSTIX-BoldItalic)
    0xff00003a,   // [3836:684] v + colon : -2 (cslSTIX-BoldItalic)
    0xed80002e,   // [3837:685] v + period : -37 (cslSTIX-BoldItalic)
    0xeec0002c,   // [3838:686] v + comma : -35 (cslSTIX-BoldItalic) ;;
    0xf80000f8,   // [3839:687] w + oslash : -16 (cslSTIX-BoldItalic)
    0xf80000f6,   // [3840:688] w + odieresis : -16 (cslSTIX-BoldItalic)
    0xf80000f3,   // [3841:689] w + oacute : -16 (cslSTIX-BoldItalic)
    0xf80000f2,   // [3842:690] w + ograve : -16 (cslSTIX-BoldItalic)
    0xf50000ea,   // [3843:691] w + ecircumflex : -22 (cslSTIX-BoldItalic)
    0xf50000e9,   // [3844:692] w + eacute : -22 (cslSTIX-BoldItalic)
    0xf50000e8,   // [3845:693] w + egrave : -22 (cslSTIX-BoldItalic)
    0xf58000e6,   // [3846:694] w + ae : -21 (cslSTIX-BoldItalic)
    0xf88000e5,   // [3847:695] w + aring : -15 (cslSTIX-BoldItalic)
    0xf88000e4,   // [3848:696] w + adieresis : -15 (cslSTIX-BoldItalic)
    0xf88000e3,   // [3849:697] w + atilde : -15 (cslSTIX-BoldItalic)
    0xf88000e2,   // [3850:698] w + acircumflex : -15 (cslSTIX-BoldItalic)
    0xf88000e1,   // [3851:699] w + aacute : -15 (cslSTIX-BoldItalic)
    0xf88000e0,   // [3852:700] w + agrave : -15 (cslSTIX-BoldItalic)
    0xf8000073,   // [3853:701] w + s : -16 (cslSTIX-BoldItalic)
    0xf800006f,   // [3854:702] w + o : -16 (cslSTIX-BoldItalic)
    0xf700006c,   // [3855:703] w + l : -18 (cslSTIX-BoldItalic)
    0xf6000067,   // [3856:704] w + g : -20 (cslSTIX-BoldItalic)
    0xf5000065,   // [3857:705] w + e : -22 (cslSTIX-BoldItalic)
    0xf7000063,   // [3858:706] w + c : -18 (cslSTIX-BoldItalic)
    0xf8800061,   // [3859:707] w + a : -15 (cslSTIX-BoldItalic)
    0xfe80003b,   // [3860:708] w + semicolon : -3 (cslSTIX-BoldItalic)
    0xfd80003a,   // [3861:709] w + colon : -5 (cslSTIX-BoldItalic)
    0xed80002e,   // [3862:710] w + period : -37 (cslSTIX-BoldItalic)
    0xfd00002d,   // [3863:711] w + hyphen : -6 (cslSTIX-BoldItalic)
    0xeec0002c,   // [3864:712] w + comma : -35 (cslSTIX-BoldItalic) ;;
    0xfa0000e9,   // [3865:713] x + eacute : -12 (cslSTIX-BoldItalic)
    0xfb000071,   // [3866:714] x + q : -10 (cslSTIX-BoldItalic)
    0xfb00006f,   // [3867:715] x + o : -10 (cslSTIX-BoldItalic)
    0xfa000065,   // [3868:716] x + e : -12 (cslSTIX-BoldItalic)
    0xfb800063,   // [3869:717] x + c : -9 (cslSTIX-BoldItalic)
    0xff400061,   // [3870:718] x + a : -2 (cslSTIX-BoldItalic) ;;
    0xfc0000f8,   // [3871:719] y + oslash : -8 (cslSTIX-BoldItalic)
    0xfd0000f6,   // [3872:720] y + odieresis : -6 (cslSTIX-BoldItalic)
    0xfd0000f3,   // [3873:721] y + oacute : -6 (cslSTIX-BoldItalic)
    0xfd0000f2,   // [3874:722] y + ograve : -6 (cslSTIX-BoldItalic)
    0xfa8000ea,   // [3875:723] y + ecircumflex : -11 (cslSTIX-BoldItalic)
    0xfa8000e9,   // [3876:724] y + eacute : -11 (cslSTIX-BoldItalic)
    0xfa8000e8,   // [3877:725] y + egrave : -11 (cslSTIX-BoldItalic)
    0xfb0000e6,   // [3878:726] y + ae : -10 (cslSTIX-BoldItalic)
    0xfe0000e5,   // [3879:727] y + aring : -4 (cslSTIX-BoldItalic)
    0xfe0000e4,   // [3880:728] y + adieresis : -4 (cslSTIX-BoldItalic)
    0xfe0000e3,   // [3881:729] y + atilde : -4 (cslSTIX-BoldItalic)
    0xfe0000e2,   // [3882:730] y + acircumflex : -4 (cslSTIX-BoldItalic)
    0xfe0000e1,   // [3883:731] y + aacute : -4 (cslSTIX-BoldItalic)
    0xfe0000e0,   // [3884:732] y + agrave : -4 (cslSTIX-BoldItalic)
    0xfc800073,   // [3885:733] y + s : -7 (cslSTIX-BoldItalic)
    0xfd00006f,   // [3886:734] y + o : -6 (cslSTIX-BoldItalic)
    0xfc00006c,   // [3887:735] y + l : -8 (cslSTIX-BoldItalic)
    0xfa800067,   // [3888:736] y + g : -11 (cslSTIX-BoldItalic)
    0xfa800065,   // [3889:737] y + e : -11 (cslSTIX-BoldItalic)
    0xfc800063,   // [3890:738] y + c : -7 (cslSTIX-BoldItalic)
    0xfe000061,   // [3891:739] y + a : -4 (cslSTIX-BoldItalic)
    0x0500003b,   // [3892:740] y + semicolon : 10 (cslSTIX-BoldItalic)
    0x0280003a,   // [3893:741] y + colon : 5 (cslSTIX-BoldItalic)
    0xf800002e,   // [3894:742] y + period : -16 (cslSTIX-BoldItalic)
    0x0200002d,   // [3895:743] y + hyphen : 4 (cslSTIX-BoldItalic)
    0xf940002c,   // [3896:744] y + comma : -14 (cslSTIX-BoldItalic) ;;
    0xf60000c6,   // [3897:745] guillemotright + AE : -20 (cslSTIX-BoldItalic)
    0xfa8000c5,   // [3898:746] guillemotright + Aring : -11 (cslSTIX-BoldItalic)
    0xfa8000c4,   // [3899:747] guillemotright + Adieresis : -11 (cslSTIX-BoldItalic)
    0xfa8000c1,   // [3900:748] guillemotright + Aacute : -11 (cslSTIX-BoldItalic)
    0xe5800059,   // [3901:749] guillemotright + Y : -53 (cslSTIX-BoldItalic)
    0xe8800057,   // [3902:750] guillemotright + W : -47 (cslSTIX-BoldItalic)
    0xe8000056,   // [3903:751] guillemotright + V : -48 (cslSTIX-BoldItalic)
    0xe8000054,   // [3904:752] guillemotright + T : -48 (cslSTIX-BoldItalic)
    0xfac00041,   // [3905:753] guillemotright + A : -11 (cslSTIX-BoldItalic) ;;
    0xce000059,   // [3906:754] Agrave + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [3907:755] Agrave + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [3908:756] Agrave + V : -105 (cslSTIX-BoldItalic)
    0xdf800055,   // [3909:757] Agrave + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [3910:758] Agrave + T : -55 (cslSTIX-BoldItalic)
    0xee000051,   // [3911:759] Agrave + Q : -36 (cslSTIX-BoldItalic)
    0xee00004f,   // [3912:760] Agrave + O : -36 (cslSTIX-BoldItalic)
    0xef000047,   // [3913:761] Agrave + G : -34 (cslSTIX-BoldItalic)
    0xed000043,   // [3914:762] Agrave + C : -38 (cslSTIX-BoldItalic)
    0x0900002e,   // [3915:763] Agrave + period : 18 (cslSTIX-BoldItalic)
    0x06c0002c,   // [3916:764] Agrave + comma : 13 (cslSTIX-BoldItalic) ;;
    0xeb802039,   // [3917:765] Aacute + guilsinglleft : -41 (cslSTIX-BoldItalic)
    0xe780201d,   // [3918:766] Aacute + quotedblright : -49 (cslSTIX-BoldItalic)
    0xdf002019,   // [3919:767] Aacute + quoteright : -66 (cslSTIX-BoldItalic)
    0xef0000ab,   // [3920:768] Aacute + guillemotleft : -34 (cslSTIX-BoldItalic)
    0xeb800079,   // [3921:769] Aacute + y : -41 (cslSTIX-BoldItalic)
    0xed800077,   // [3922:770] Aacute + w : -37 (cslSTIX-BoldItalic)
    0xef000076,   // [3923:771] Aacute + v : -34 (cslSTIX-BoldItalic)
    0xf9000075,   // [3924:772] Aacute + u : -14 (cslSTIX-BoldItalic)
    0xfe800074,   // [3925:773] Aacute + t : -3 (cslSTIX-BoldItalic)
    0xfa000071,   // [3926:774] Aacute + q : -12 (cslSTIX-BoldItalic)
    0xf800006f,   // [3927:775] Aacute + o : -16 (cslSTIX-BoldItalic)
    0xf8000067,   // [3928:776] Aacute + g : -16 (cslSTIX-BoldItalic)
    0xf7800065,   // [3929:777] Aacute + e : -17 (cslSTIX-BoldItalic)
    0xfe800064,   // [3930:778] Aacute + d : -3 (cslSTIX-BoldItalic)
    0xf9000063,   // [3931:779] Aacute + c : -14 (cslSTIX-BoldItalic)
    0xff000062,   // [3932:780] Aacute + b : -2 (cslSTIX-BoldItalic)
    0xfe000061,   // [3933:781] Aacute + a : -4 (cslSTIX-BoldItalic)
    0xce000059,   // [3934:782] Aacute + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [3935:783] Aacute + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [3936:784] Aacute + V : -105 (cslSTIX-BoldItalic)
    0xdf800055,   // [3937:785] Aacute + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [3938:786] Aacute + T : -55 (cslSTIX-BoldItalic)
    0xee000051,   // [3939:787] Aacute + Q : -36 (cslSTIX-BoldItalic)
    0xee00004f,   // [3940:788] Aacute + O : -36 (cslSTIX-BoldItalic)
    0xef000047,   // [3941:789] Aacute + G : -34 (cslSTIX-BoldItalic)
    0xed000043,   // [3942:790] Aacute + C : -38 (cslSTIX-BoldItalic)
    0x0900002e,   // [3943:791] Aacute + period : 18 (cslSTIX-BoldItalic)
    0xfd00002d,   // [3944:792] Aacute + hyphen : -6 (cslSTIX-BoldItalic)
    0x06c0002c,   // [3945:793] Aacute + comma : 13 (cslSTIX-BoldItalic) ;;
    0xce000059,   // [3946:794] Acircumflex + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [3947:795] Acircumflex + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [3948:796] Acircumflex + V : -105 (cslSTIX-BoldItalic)
    0xdf800055,   // [3949:797] Acircumflex + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [3950:798] Acircumflex + T : -55 (cslSTIX-BoldItalic)
    0xee000051,   // [3951:799] Acircumflex + Q : -36 (cslSTIX-BoldItalic)
    0xee00004f,   // [3952:800] Acircumflex + O : -36 (cslSTIX-BoldItalic)
    0xef000047,   // [3953:801] Acircumflex + G : -34 (cslSTIX-BoldItalic)
    0xed000043,   // [3954:802] Acircumflex + C : -38 (cslSTIX-BoldItalic)
    0x0900002e,   // [3955:803] Acircumflex + period : 18 (cslSTIX-BoldItalic)
    0x06c0002c,   // [3956:804] Acircumflex + comma : 13 (cslSTIX-BoldItalic) ;;
    0xce000059,   // [3957:805] Atilde + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [3958:806] Atilde + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [3959:807] Atilde + V : -105 (cslSTIX-BoldItalic)
    0xde800055,   // [3960:808] Atilde + U : -67 (cslSTIX-BoldItalic)
    0xe4800054,   // [3961:809] Atilde + T : -55 (cslSTIX-BoldItalic)
    0xed000051,   // [3962:810] Atilde + Q : -38 (cslSTIX-BoldItalic)
    0xed00004f,   // [3963:811] Atilde + O : -38 (cslSTIX-BoldItalic)
    0xef800047,   // [3964:812] Atilde + G : -33 (cslSTIX-BoldItalic)
    0xed800043,   // [3965:813] Atilde + C : -37 (cslSTIX-BoldItalic)
    0x0500002e,   // [3966:814] Atilde + period : 10 (cslSTIX-BoldItalic)
    0x0540002c,   // [3967:815] Atilde + comma : 10 (cslSTIX-BoldItalic) ;;
    0xeb002039,   // [3968:816] Adieresis + guilsinglleft : -42 (cslSTIX-BoldItalic)
    0xe900201d,   // [3969:817] Adieresis + quotedblright : -46 (cslSTIX-BoldItalic)
    0xe0802019,   // [3970:818] Adieresis + quoteright : -63 (cslSTIX-BoldItalic)
    0xef0000ab,   // [3971:819] Adieresis + guillemotleft : -34 (cslSTIX-BoldItalic)
    0xea800079,   // [3972:820] Adieresis + y : -43 (cslSTIX-BoldItalic)
    0xef000077,   // [3973:821] Adieresis + w : -34 (cslSTIX-BoldItalic)
    0xef000076,   // [3974:822] Adieresis + v : -34 (cslSTIX-BoldItalic)
    0xf9000075,   // [3975:823] Adieresis + u : -14 (cslSTIX-BoldItalic)
    0xfe800074,   // [3976:824] Adieresis + t : -3 (cslSTIX-BoldItalic)
    0xfa000071,   // [3977:825] Adieresis + q : -12 (cslSTIX-BoldItalic)
    0xf880006f,   // [3978:826] Adieresis + o : -15 (cslSTIX-BoldItalic)
    0xf8000067,   // [3979:827] Adieresis + g : -16 (cslSTIX-BoldItalic)
    0xfe800064,   // [3980:828] Adieresis + d : -3 (cslSTIX-BoldItalic)
    0xf9000063,   // [3981:829] Adieresis + c : -14 (cslSTIX-BoldItalic)
    0xff000062,   // [3982:830] Adieresis + b : -2 (cslSTIX-BoldItalic)
    0xfe000061,   // [3983:831] Adieresis + a : -4 (cslSTIX-BoldItalic)
    0xd8000059,   // [3984:832] Adieresis + Y : -80 (cslSTIX-BoldItalic)
    0xda800057,   // [3985:833] Adieresis + W : -75 (cslSTIX-BoldItalic)
    0xda800056,   // [3986:834] Adieresis + V : -75 (cslSTIX-BoldItalic)
    0xdf800055,   // [3987:835] Adieresis + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [3988:836] Adieresis + T : -55 (cslSTIX-BoldItalic)
    0xed800051,   // [3989:837] Adieresis + Q : -37 (cslSTIX-BoldItalic)
    0xed80004f,   // [3990:838] Adieresis + O : -37 (cslSTIX-BoldItalic)
    0xef000047,   // [3991:839] Adieresis + G : -34 (cslSTIX-BoldItalic)
    0xee800043,   // [3992:840] Adieresis + C : -35 (cslSTIX-BoldItalic)
    0x0700002e,   // [3993:841] Adieresis + period : 14 (cslSTIX-BoldItalic)
    0xfc80002d,   // [3994:842] Adieresis + hyphen : -7 (cslSTIX-BoldItalic)
    0x05c0002c,   // [3995:843] Adieresis + comma : 11 (cslSTIX-BoldItalic) ;;
    0xeb802039,   // [3996:844] Aring + guilsinglleft : -41 (cslSTIX-BoldItalic)
    0xe780201d,   // [3997:845] Aring + quotedblright : -49 (cslSTIX-BoldItalic)
    0xdf002019,   // [3998:846] Aring + quoteright : -66 (cslSTIX-BoldItalic)
    0xef0000ab,   // [3999:847] Aring + guillemotleft : -34 (cslSTIX-BoldItalic)
    0xeb800079,   // [4000:848] Aring + y : -41 (cslSTIX-BoldItalic)
    0xed800077,   // [4001:849] Aring + w : -37 (cslSTIX-BoldItalic)
    0xef000076,   // [4002:850] Aring + v : -34 (cslSTIX-BoldItalic)
    0xf9000075,   // [4003:851] Aring + u : -14 (cslSTIX-BoldItalic)
    0xfe800074,   // [4004:852] Aring + t : -3 (cslSTIX-BoldItalic)
    0xfa000071,   // [4005:853] Aring + q : -12 (cslSTIX-BoldItalic)
    0xf800006f,   // [4006:854] Aring + o : -16 (cslSTIX-BoldItalic)
    0xf8000067,   // [4007:855] Aring + g : -16 (cslSTIX-BoldItalic)
    0xf7800065,   // [4008:856] Aring + e : -17 (cslSTIX-BoldItalic)
    0xfe800064,   // [4009:857] Aring + d : -3 (cslSTIX-BoldItalic)
    0xf9000063,   // [4010:858] Aring + c : -14 (cslSTIX-BoldItalic)
    0xff000062,   // [4011:859] Aring + b : -2 (cslSTIX-BoldItalic)
    0xfe000061,   // [4012:860] Aring + a : -4 (cslSTIX-BoldItalic)
    0xce000059,   // [4013:861] Aring + Y : -100 (cslSTIX-BoldItalic)
    0xcb800057,   // [4014:862] Aring + W : -105 (cslSTIX-BoldItalic)
    0xcb800056,   // [4015:863] Aring + V : -105 (cslSTIX-BoldItalic)
    0xdf800055,   // [4016:864] Aring + U : -65 (cslSTIX-BoldItalic)
    0xe4800054,   // [4017:865] Aring + T : -55 (cslSTIX-BoldItalic)
    0xee000051,   // [4018:866] Aring + Q : -36 (cslSTIX-BoldItalic)
    0xee00004f,   // [4019:867] Aring + O : -36 (cslSTIX-BoldItalic)
    0xef000047,   // [4020:868] Aring + G : -34 (cslSTIX-BoldItalic)
    0xed000043,   // [4021:869] Aring + C : -38 (cslSTIX-BoldItalic)
    0x0900002e,   // [4022:870] Aring + period : 18 (cslSTIX-BoldItalic)
    0xfd00002d,   // [4023:871] Aring + hyphen : -6 (cslSTIX-BoldItalic)
    0x06c0002c,   // [4024:872] Aring + comma : 13 (cslSTIX-BoldItalic) ;;
    0xf5400041,   // [4025:873] Ccedilla + A : -22 (cslSTIX-BoldItalic) ;;
    0xee000059,   // [4026:874] Ograve + Y : -36 (cslSTIX-BoldItalic)
    0xf0800056,   // [4027:875] Ograve + V : -31 (cslSTIX-BoldItalic)
    0xf8c00054,   // [4028:876] Ograve + T : -15 (cslSTIX-BoldItalic) ;;
    0xee000059,   // [4029:877] Oacute + Y : -36 (cslSTIX-BoldItalic)
    0xf1000057,   // [4030:878] Oacute + W : -30 (cslSTIX-BoldItalic)
    0xf0800056,   // [4031:879] Oacute + V : -31 (cslSTIX-BoldItalic)
    0xf8800054,   // [4032:880] Oacute + T : -15 (cslSTIX-BoldItalic)
    0xf1400041,   // [4033:881] Oacute + A : -30 (cslSTIX-BoldItalic) ;;
    0xee000059,   // [4034:882] Ocircumflex + Y : -36 (cslSTIX-BoldItalic)
    0xf0800056,   // [4035:883] Ocircumflex + V : -31 (cslSTIX-BoldItalic)
    0xf8c00054,   // [4036:884] Ocircumflex + T : -15 (cslSTIX-BoldItalic) ;;
    0xee000059,   // [4037:885] Otilde + Y : -36 (cslSTIX-BoldItalic)
    0xf0800056,   // [4038:886] Otilde + V : -31 (cslSTIX-BoldItalic)
    0xf8c00054,   // [4039:887] Otilde + T : -15 (cslSTIX-BoldItalic) ;;
    0xee000059,   // [4040:888] Odieresis + Y : -36 (cslSTIX-BoldItalic)
    0xee800058,   // [4041:889] Odieresis + X : -35 (cslSTIX-BoldItalic)
    0xf1000057,   // [4042:890] Odieresis + W : -30 (cslSTIX-BoldItalic)
    0xf0800056,   // [4043:891] Odieresis + V : -31 (cslSTIX-BoldItalic)
    0xf8800054,   // [4044:892] Odieresis + T : -15 (cslSTIX-BoldItalic)
    0xf1400041,   // [4045:893] Odieresis + A : -30 (cslSTIX-BoldItalic) ;;
    0xf1400041,   // [4046:894] Oslash + A : -30 (cslSTIX-BoldItalic) ;;
    0xf7800072,   // [4047:895] Ugrave + r : -17 (cslSTIX-BoldItalic)
    0xf8000070,   // [4048:896] Ugrave + p : -16 (cslSTIX-BoldItalic)
    0xf500006e,   // [4049:897] Ugrave + n : -22 (cslSTIX-BoldItalic)
    0xf680006d,   // [4050:898] Ugrave + m : -19 (cslSTIX-BoldItalic)
    0xeb800041,   // [4051:899] Ugrave + A : -41 (cslSTIX-BoldItalic)
    0xf780002e,   // [4052:900] Ugrave + period : -17 (cslSTIX-BoldItalic)
    0xf740002c,   // [4053:901] Ugrave + comma : -18 (cslSTIX-BoldItalic) ;;
    0xf7800072,   // [4054:902] Uacute + r : -17 (cslSTIX-BoldItalic)
    0xf8000070,   // [4055:903] Uacute + p : -16 (cslSTIX-BoldItalic)
    0xf500006e,   // [4056:904] Uacute + n : -22 (cslSTIX-BoldItalic)
    0xf680006d,   // [4057:905] Uacute + m : -19 (cslSTIX-BoldItalic)
    0xeb800041,   // [4058:906] Uacute + A : -41 (cslSTIX-BoldItalic)
    0xf780002e,   // [4059:907] Uacute + period : -17 (cslSTIX-BoldItalic)
    0xf740002c,   // [4060:908] Uacute + comma : -18 (cslSTIX-BoldItalic) ;;
    0xebc00041,   // [4061:909] Ucircumflex + A : -41 (cslSTIX-BoldItalic) ;;
    0xf7800072,   // [4062:910] Udieresis + r : -17 (cslSTIX-BoldItalic)
    0xf8000070,   // [4063:911] Udieresis + p : -16 (cslSTIX-BoldItalic)
    0xf500006e,   // [4064:912] Udieresis + n : -22 (cslSTIX-BoldItalic)
    0xf680006d,   // [4065:913] Udieresis + m : -19 (cslSTIX-BoldItalic)
    0x00800062,   // [4066:914] Udieresis + b : 1 (cslSTIX-BoldItalic)
    0xeb800041,   // [4067:915] Udieresis + A : -41 (cslSTIX-BoldItalic)
    0xf780002e,   // [4068:916] Udieresis + period : -17 (cslSTIX-BoldItalic)
    0xf740002c,   // [4069:917] Udieresis + comma : -18 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4070:918] agrave + y : -2 (cslSTIX-BoldItalic)
    0xff000077,   // [4071:919] agrave + w : -2 (cslSTIX-BoldItalic)
    0xff400076,   // [4072:920] agrave + v : -2 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4073:921] aacute + y : -2 (cslSTIX-BoldItalic)
    0xff000077,   // [4074:922] aacute + w : -2 (cslSTIX-BoldItalic)
    0xff400076,   // [4075:923] aacute + v : -2 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4076:924] adieresis + y : -2 (cslSTIX-BoldItalic)
    0xff000077,   // [4077:925] adieresis + w : -2 (cslSTIX-BoldItalic)
    0xff400076,   // [4078:926] adieresis + v : -2 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4079:927] aring + y : -2 (cslSTIX-BoldItalic)
    0xff000077,   // [4080:928] aring + w : -2 (cslSTIX-BoldItalic)
    0xff400076,   // [4081:929] aring + v : -2 (cslSTIX-BoldItalic) ;;
    0xfe000079,   // [4082:930] ae + y : -4 (cslSTIX-BoldItalic)
    0xfd800077,   // [4083:931] ae + w : -5 (cslSTIX-BoldItalic)
    0xfdc00076,   // [4084:932] ae + v : -5 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4085:933] eacute + y : -2 (cslSTIX-BoldItalic)
    0xfe000077,   // [4086:934] eacute + w : -4 (cslSTIX-BoldItalic)
    0xfe400076,   // [4087:935] eacute + v : -4 (cslSTIX-BoldItalic) ;;
    0xff000079,   // [4088:936] ecircumflex + y : -2 (cslSTIX-BoldItalic)
    0xfe000077,   // [4089:937] ecircumflex + w : -4 (cslSTIX-BoldItalic)
    0xfe400076,   // [4090:938] ecircumflex + v : -4 (cslSTIX-BoldItalic) ;;
    0xf8000079,   // [4091:939] ograve + y : -16 (cslSTIX-BoldItalic)
    0xf8800077,   // [4092:940] ograve + w : -15 (cslSTIX-BoldItalic)
    0xf8c00076,   // [4093:941] ograve + v : -15 (cslSTIX-BoldItalic) ;;
    0xf8000079,   // [4094:942] oacute + y : -16 (cslSTIX-BoldItalic)
    0xf8800077,   // [4095:943] oacute + w : -15 (cslSTIX-BoldItalic)
    0xf8c00076,   // [4096:944] oacute + v : -15 (cslSTIX-BoldItalic) ;;
    0x01c00074,   // [4097:945] ocircumflex + t : 3 (cslSTIX-BoldItalic) ;;
    0xf8000079,   // [4098:946] odieresis + y : -16 (cslSTIX-BoldItalic)
    0xf7000078,   // [4099:947] odieresis + x : -18 (cslSTIX-BoldItalic)
    0xf7800077,   // [4100:948] odieresis + w : -17 (cslSTIX-BoldItalic)
    0xf9400076,   // [4101:949] odieresis + v : -14 (cslSTIX-BoldItalic) ;;
    0xdf8000c6,   // [4102:950] quoteleft + AE : -65 (cslSTIX-BoldItalic)
    0xe70000c5,   // [4103:951] quoteleft + Aring : -50 (cslSTIX-BoldItalic)
    0xe70000c4,   // [4104:952] quoteleft + Adieresis : -50 (cslSTIX-BoldItalic)
    0xe70000c2,   // [4105:953] quoteleft + Acircumflex : -50 (cslSTIX-BoldItalic)
    0xe70000c1,   // [4106:954] quoteleft + Aacute : -50 (cslSTIX-BoldItalic)
    0xe70000c0,   // [4107:955] quoteleft + Agrave : -50 (cslSTIX-BoldItalic)
    0xfd000059,   // [4108:956] quoteleft + Y : -6 (cslSTIX-BoldItalic)
    0xff000057,   // [4109:957] quoteleft + W : -2 (cslSTIX-BoldItalic)
    0xfe800056,   // [4110:958] quoteleft + V : -3 (cslSTIX-BoldItalic)
    0xfd800054,   // [4111:959] quoteleft + T : -5 (cslSTIX-BoldItalic)
    0xe7400041,   // [4112:960] quoteleft + A : -50 (cslSTIX-BoldItalic) ;;
    0xda0000c6,   // [4113:961] quoteright + AE : -76 (cslSTIX-BoldItalic)
    0xe40000c5,   // [4114:962] quoteright + Aring : -56 (cslSTIX-BoldItalic)
    0xe40000c4,   // [4115:963] quoteright + Adieresis : -56 (cslSTIX-BoldItalic)
    0xe40000c2,   // [4116:964] quoteright + Acircumflex : -56 (cslSTIX-BoldItalic)
    0xe40000c1,   // [4117:965] quoteright + Aacute : -56 (cslSTIX-BoldItalic)
    0xe40000c0,   // [4118:966] quoteright + Agrave : -56 (cslSTIX-BoldItalic)
    0xfb000079,   // [4119:967] quoteright + y : -10 (cslSTIX-BoldItalic)
    0xfb000077,   // [4120:968] quoteright + w : -10 (cslSTIX-BoldItalic)
    0xfb000076,   // [4121:969] quoteright + v : -10 (cslSTIX-BoldItalic)
    0xfb000074,   // [4122:970] quoteright + t : -10 (cslSTIX-BoldItalic)
    0xf8000073,   // [4123:971] quoteright + s : -16 (cslSTIX-BoldItalic)
    0xfa000072,   // [4124:972] quoteright + r : -12 (cslSTIX-BoldItalic)
    0xf080006f,   // [4125:973] quoteright + o : -31 (cslSTIX-BoldItalic)
    0xef000064,   // [4126:974] quoteright + d : -34 (cslSTIX-BoldItalic)
    0xe4000041,   // [4127:975] quoteright + A : -56 (cslSTIX-BoldItalic)
    0xeb00002e,   // [4128:976] quoteright + period : -42 (cslSTIX-BoldItalic)
    0xeb40002c,   // [4129:977] quoteright + comma : -42 (cslSTIX-BoldItalic) ;;
    0xe20000c6,   // [4130:978] quotedblleft + AE : -60 (cslSTIX-BoldItalic)
    0xea0000c5,   // [4131:979] quotedblleft + Aring : -44 (cslSTIX-BoldItalic)
    0xea0000c4,   // [4132:980] quotedblleft + Adieresis : -44 (cslSTIX-BoldItalic)
    0xea0000c1,   // [4133:981] quotedblleft + Aacute : -44 (cslSTIX-BoldItalic)
    0x02000057,   // [4134:982] quotedblleft + W : 4 (cslSTIX-BoldItalic)
    0x01800056,   // [4135:983] quotedblleft + V : 3 (cslSTIX-BoldItalic)
    0xea400041,   // [4136:984] quotedblleft + A : -44 (cslSTIX-BoldItalic) ;;
    0xe20000c6,   // [4137:985] quotedblright + AE : -60 (cslSTIX-BoldItalic)
    0xec8000c5,   // [4138:986] quotedblright + Aring : -39 (cslSTIX-BoldItalic)
    0xec8000c4,   // [4139:987] quotedblright + Adieresis : -39 (cslSTIX-BoldItalic)
    0xec8000c1,   // [4140:988] quotedblright + Aacute : -39 (cslSTIX-BoldItalic)
    0x02000059,   // [4141:989] quotedblright + Y : 4 (cslSTIX-BoldItalic)
    0x07800057,   // [4142:990] quotedblright + W : 15 (cslSTIX-BoldItalic)
    0x07000056,   // [4143:991] quotedblright + V : 14 (cslSTIX-BoldItalic)
    0x06800054,   // [4144:992] quotedblright + T : 13 (cslSTIX-BoldItalic)
    0xecc00041,   // [4145:993] quotedblright + A : -39 (cslSTIX-BoldItalic) ;;
    0x138000c6,   // [4146:994] quotedblbase + AE : 39 (cslSTIX-BoldItalic)
    0x148000c5,   // [4147:995] quotedblbase + Aring : 41 (cslSTIX-BoldItalic)
    0x148000c4,   // [4148:996] quotedblbase + Adieresis : 41 (cslSTIX-BoldItalic)
    0x148000c2,   // [4149:997] quotedblbase + Acircumflex : 41 (cslSTIX-BoldItalic)
    0x148000c1,   // [4150:998] quotedblbase + Aacute : 41 (cslSTIX-BoldItalic)
    0x148000c0,   // [4151:999] quotedblbase + Agrave : 41 (cslSTIX-BoldItalic)
    0xf2800059,   // [4152:1000] quotedblbase + Y : -27 (cslSTIX-BoldItalic)
    0xea000057,   // [4153:1001] quotedblbase + W : -44 (cslSTIX-BoldItalic)
    0xe8800056,   // [4154:1002] quotedblbase + V : -47 (cslSTIX-BoldItalic)
    0xf6800054,   // [4155:1003] quotedblbase + T : -19 (cslSTIX-BoldItalic)
    0x14c00041,   // [4156:1004] quotedblbase + A : 41 (cslSTIX-BoldItalic) ;;
    0xf20000c6,   // [4157:1005] guilsinglright + AE : -28 (cslSTIX-BoldItalic)
    0xf70000c5,   // [4158:1006] guilsinglright + Aring : -18 (cslSTIX-BoldItalic)
    0xf70000c4,   // [4159:1007] guilsinglright + Adieresis : -18 (cslSTIX-BoldItalic)
    0xf70000c1,   // [4160:1008] guilsinglright + Aacute : -18 (cslSTIX-BoldItalic)
    0xe1800059,   // [4161:1009] guilsinglright + Y : -61 (cslSTIX-BoldItalic)
    0xe5000057,   // [4162:1010] guilsinglright + W : -54 (cslSTIX-BoldItalic)
    0xe4800056,   // [4163:1011] guilsinglright + V : -55 (cslSTIX-BoldItalic)
    0xe4000054,   // [4164:1012] guilsinglright + T : -56 (cslSTIX-BoldItalic)
    0xf7400041,   // [4165:1013] guilsinglright + A : -18 (cslSTIX-BoldItalic) ;;
    0x1fa00069,   // [4166:1014] ff + i ligature #63 (cslSTIX-BoldItalic)
    0x2060006c,   // [4167:1015] ff + l ligature #64 (cslSTIX-BoldItalic) ;;
    0xef00201d,   // [4168:1] comma + quotedblright : -34 (cslSTIXMath-Regular)
    0xe3402019,   // [4169:2] comma + quoteright : -58 (cslSTIXMath-Regular) ;;
    0xf30000c6,   // [4170:3] hyphen + AE : -26 (cslSTIXMath-Regular)
    0xf10000c5,   // [4171:4] hyphen + Aring : -30 (cslSTIXMath-Regular)
    0xf10000c4,   // [4172:5] hyphen + Adieresis : -30 (cslSTIXMath-Regular)
    0xf10000c1,   // [4173:6] hyphen + Aacute : -30 (cslSTIXMath-Regular)
    0xd0800059,   // [4174:7] hyphen + Y : -95 (cslSTIXMath-Regular)
    0xe9800057,   // [4175:8] hyphen + W : -45 (cslSTIXMath-Regular)
    0xd9000056,   // [4176:9] hyphen + V : -78 (cslSTIXMath-Regular)
    0xd8800054,   // [4177:10] hyphen + T : -79 (cslSTIXMath-Regular)
    0xf1400041,   // [4178:11] hyphen + A : -30 (cslSTIXMath-Regular) ;;
    0xed80201d,   // [4179:12] period + quotedblright : -37 (cslSTIXMath-Regular)
    0xedc02019,   // [4180:13] period + quoteright : -37 (cslSTIXMath-Regular) ;;
    0xe7802039,   // [4181:14] A + guilsinglleft : -49 (cslSTIXMath-Regular)
    0xe380201d,   // [4182:15] A + quotedblright : -57 (cslSTIXMath-Regular)
    0xdb002019,   // [4183:16] A + quoteright : -74 (cslSTIXMath-Regular)
    0xf90000fc,   // [4184:17] A + udieresis : -14 (cslSTIXMath-Regular)
    0xf90000fb,   // [4185:18] A + ucircumflex : -14 (cslSTIXMath-Regular)
    0xf90000fa,   // [4186:19] A + uacute : -14 (cslSTIXMath-Regular)
    0xf90000f9,   // [4187:20] A + ugrave : -14 (cslSTIXMath-Regular)
    0xf30000f6,   // [4188:21] A + odieresis : -26 (cslSTIXMath-Regular)
    0xfb0000e7,   // [4189:22] A + ccedilla : -10 (cslSTIXMath-Regular)
    0xee8000dc,   // [4190:23] A + Udieresis : -35 (cslSTIXMath-Regular)
    0xee8000db,   // [4191:24] A + Ucircumflex : -35 (cslSTIXMath-Regular)
    0xee8000da,   // [4192:25] A + Uacute : -35 (cslSTIXMath-Regular)
    0xee8000d9,   // [4193:26] A + Ugrave : -35 (cslSTIXMath-Regular)
    0xf38000d6,   // [4194:27] A + Odieresis : -25 (cslSTIXMath-Regular)
    0xf38000c7,   // [4195:28] A + Ccedilla : -25 (cslSTIXMath-Regular)
    0xeb0000ab,   // [4196:29] A + guillemotleft : -42 (cslSTIXMath-Regular)
    0xdc000079,   // [4197:30] A + y : -72 (cslSTIXMath-Regular)
    0xe3000077,   // [4198:31] A + w : -58 (cslSTIXMath-Regular)
    0xe3000076,   // [4199:32] A + v : -58 (cslSTIXMath-Regular)
    0xf9000075,   // [4200:33] A + u : -14 (cslSTIXMath-Regular)
    0xf9800074,   // [4201:34] A + t : -13 (cslSTIXMath-Regular)
    0xf9000071,   // [4202:35] A + q : -14 (cslSTIXMath-Regular)
    0xf300006f,   // [4203:36] A + o : -26 (cslSTIXMath-Regular)
    0xfa000067,   // [4204:37] A + g : -12 (cslSTIXMath-Regular)
    0xf8000065,   // [4205:38] A + e : -16 (cslSTIXMath-Regular)
    0xf7800064,   // [4206:39] A + d : -17 (cslSTIXMath-Regular)
    0xfb000063,   // [4207:40] A + c : -10 (cslSTIXMath-Regular)
    0xf9000062,   // [4208:41] A + b : -14 (cslSTIXMath-Regular)
    0xfe000061,   // [4209:42] A + a : -4 (cslSTIXMath-Regular)
    0xe6000059,   // [4210:43] A + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [4211:44] A + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [4212:45] A + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [4213:46] A + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [4214:47] A + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [4215:48] A + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [4216:49] A + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [4217:50] A + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [4218:51] A + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [4219:52] A + period : -5 (cslSTIXMath-Regular)
    0xf800002d,   // [4220:53] A + hyphen : -16 (cslSTIXMath-Regular)
    0xff40002c,   // [4221:54] A + comma : -2 (cslSTIXMath-Regular) ;;
    0xf7800152,   // [4222:55] B + OE : -17 (cslSTIXMath-Regular)
    0xf78000d8,   // [4223:56] B + Oslash : -17 (cslSTIXMath-Regular)
    0xf78000d6,   // [4224:57] B + Odieresis : -17 (cslSTIXMath-Regular)
    0xf78000d4,   // [4225:58] B + Ocircumflex : -17 (cslSTIXMath-Regular)
    0xf78000d3,   // [4226:59] B + Oacute : -17 (cslSTIXMath-Regular)
    0xf78000d2,   // [4227:60] B + Ograve : -17 (cslSTIXMath-Regular)
    0xf20000c6,   // [4228:61] B + AE : -28 (cslSTIXMath-Regular)
    0xef8000c5,   // [4229:62] B + Aring : -33 (cslSTIXMath-Regular)
    0xef8000c4,   // [4230:63] B + Adieresis : -33 (cslSTIXMath-Regular)
    0xef8000c3,   // [4231:64] B + Atilde : -33 (cslSTIXMath-Regular)
    0xef8000c2,   // [4232:65] B + Acircumflex : -33 (cslSTIXMath-Regular)
    0xef8000c1,   // [4233:66] B + Aacute : -33 (cslSTIXMath-Regular)
    0xe2000059,   // [4234:67] B + Y : -60 (cslSTIXMath-Regular)
    0xe8800057,   // [4235:68] B + W : -47 (cslSTIXMath-Regular)
    0xe7000056,   // [4236:69] B + V : -50 (cslSTIXMath-Regular)
    0xf780004f,   // [4237:70] B + O : -17 (cslSTIXMath-Regular)
    0xefc00041,   // [4238:71] B + A : -33 (cslSTIXMath-Regular) ;;
    0xfc0000d6,   // [4239:72] C + Odieresis : -8 (cslSTIXMath-Regular)
    0xfc0000d3,   // [4240:73] C + Oacute : -8 (cslSTIXMath-Regular)
    0xfc0000d2,   // [4241:74] C + Ograve : -8 (cslSTIXMath-Regular)
    0xfa8000c6,   // [4242:75] C + AE : -11 (cslSTIXMath-Regular)
    0xf80000c5,   // [4243:76] C + Aring : -16 (cslSTIXMath-Regular)
    0xf80000c4,   // [4244:77] C + Adieresis : -16 (cslSTIXMath-Regular)
    0xf80000c1,   // [4245:78] C + Aacute : -16 (cslSTIXMath-Regular)
    0xf80000c0,   // [4246:79] C + Agrave : -16 (cslSTIXMath-Regular)
    0xfc00004f,   // [4247:80] C + O : -8 (cslSTIXMath-Regular)
    0xfb80004b,   // [4248:81] C + K : -9 (cslSTIXMath-Regular)
    0xf8400041,   // [4249:82] C + A : -16 (cslSTIXMath-Regular) ;;
    0xec0000c5,   // [4250:83] D + Aring : -40 (cslSTIXMath-Regular)
    0xec0000c4,   // [4251:84] D + Adieresis : -40 (cslSTIXMath-Regular)
    0xec0000c3,   // [4252:85] D + Atilde : -40 (cslSTIXMath-Regular)
    0xec0000c2,   // [4253:86] D + Acircumflex : -40 (cslSTIXMath-Regular)
    0xec0000c1,   // [4254:87] D + Aacute : -40 (cslSTIXMath-Regular)
    0xec0000c0,   // [4255:88] D + Agrave : -40 (cslSTIXMath-Regular)
    0xe9000059,   // [4256:89] D + Y : -46 (cslSTIXMath-Regular)
    0xeb000058,   // [4257:90] D + X : -42 (cslSTIXMath-Regular)
    0xed000057,   // [4258:91] D + W : -38 (cslSTIXMath-Regular)
    0xed000056,   // [4259:92] D + V : -38 (cslSTIXMath-Regular)
    0xfd800054,   // [4260:93] D + T : -5 (cslSTIXMath-Regular)
    0xf600004a,   // [4261:94] D + J : -20 (cslSTIXMath-Regular)
    0xec400041,   // [4262:95] D + A : -40 (cslSTIXMath-Regular) ;;
    0xfd800153,   // [4263:96] F + oe : -5 (cslSTIXMath-Regular)
    0xfd8000f8,   // [4264:97] F + oslash : -5 (cslSTIXMath-Regular)
    0xfd8000f6,   // [4265:98] F + odieresis : -5 (cslSTIXMath-Regular)
    0xfd8000f3,   // [4266:99] F + oacute : -5 (cslSTIXMath-Regular)
    0xfe0000e9,   // [4267:100] F + eacute : -4 (cslSTIXMath-Regular)
    0xf88000e6,   // [4268:101] F + ae : -15 (cslSTIXMath-Regular)
    0xfa8000e5,   // [4269:102] F + aring : -11 (cslSTIXMath-Regular)
    0xff8000e4,   // [4270:103] F + adieresis : -1 (cslSTIXMath-Regular)
    0xff8000e3,   // [4271:104] F + atilde : -1 (cslSTIXMath-Regular)
    0xff0000e2,   // [4272:105] F + acircumflex : -2 (cslSTIXMath-Regular)
    0xfa8000e1,   // [4273:106] F + aacute : -11 (cslSTIXMath-Regular)
    0xff8000d6,   // [4274:107] F + Odieresis : -1 (cslSTIXMath-Regular)
    0xff8000d3,   // [4275:108] F + Oacute : -1 (cslSTIXMath-Regular)
    0xff8000d2,   // [4276:109] F + Ograve : -1 (cslSTIXMath-Regular)
    0xee8000c5,   // [4277:110] F + Aring : -35 (cslSTIXMath-Regular)
    0xee8000c4,   // [4278:111] F + Adieresis : -35 (cslSTIXMath-Regular)
    0xee8000c3,   // [4279:112] F + Atilde : -35 (cslSTIXMath-Regular)
    0xee8000c2,   // [4280:113] F + Acircumflex : -35 (cslSTIXMath-Regular)
    0xee8000c1,   // [4281:114] F + Aacute : -35 (cslSTIXMath-Regular)
    0xee8000c0,   // [4282:115] F + Agrave : -35 (cslSTIXMath-Regular)
    0xff800075,   // [4283:116] F + u : -1 (cslSTIXMath-Regular)
    0xff800072,   // [4284:117] F + r : -1 (cslSTIXMath-Regular)
    0xfd80006f,   // [4285:118] F + o : -5 (cslSTIXMath-Regular)
    0xfd80006a,   // [4286:119] F + j : -5 (cslSTIXMath-Regular)
    0xfd800069,   // [4287:120] F + i : -5 (cslSTIXMath-Regular)
    0xfe000065,   // [4288:121] F + e : -4 (cslSTIXMath-Regular)
    0xfa800061,   // [4289:122] F + a : -11 (cslSTIXMath-Regular)
    0xff80004f,   // [4290:123] F + O : -1 (cslSTIXMath-Regular)
    0xff80004a,   // [4291:124] F + J : -1 (cslSTIXMath-Regular)
    0xee800041,   // [4292:125] F + A : -35 (cslSTIXMath-Regular)
    0xe880002e,   // [4293:126] F + period : -47 (cslSTIXMath-Regular)
    0x0180002d,   // [4294:127] F + hyphen : 3 (cslSTIXMath-Regular)
    0xed40002c,   // [4295:128] F + comma : -38 (cslSTIXMath-Regular) ;;
    0xfa8000c6,   // [4296:129] G + AE : -11 (cslSTIXMath-Regular)
    0xf88000c5,   // [4297:130] G + Aring : -15 (cslSTIXMath-Regular)
    0xf88000c4,   // [4298:131] G + Adieresis : -15 (cslSTIXMath-Regular)
    0xf88000c3,   // [4299:132] G + Atilde : -15 (cslSTIXMath-Regular)
    0xf88000c2,   // [4300:133] G + Acircumflex : -15 (cslSTIXMath-Regular)
    0xf88000c1,   // [4301:134] G + Aacute : -15 (cslSTIXMath-Regular)
    0xf88000c0,   // [4302:135] G + Agrave : -15 (cslSTIXMath-Regular)
    0xf5800059,   // [4303:136] G + Y : -21 (cslSTIXMath-Regular)
    0xf8800057,   // [4304:137] G + W : -15 (cslSTIXMath-Regular)
    0xf8800056,   // [4305:138] G + V : -15 (cslSTIXMath-Regular)
    0xf9800054,   // [4306:139] G + T : -13 (cslSTIXMath-Regular)
    0xf8c00041,   // [4307:140] G + A : -15 (cslSTIXMath-Regular) ;;
    0x20e0004a,   // [4308:141] I + J ligature #65 (cslSTIXMath-Regular) ;;
    0xf10000c6,   // [4309:142] J + AE : -30 (cslSTIXMath-Regular)
    0xee8000c5,   // [4310:143] J + Aring : -35 (cslSTIXMath-Regular)
    0xee8000c4,   // [4311:144] J + Adieresis : -35 (cslSTIXMath-Regular)
    0xeec00041,   // [4312:145] J + A : -35 (cslSTIXMath-Regular) ;;
    0xf6800152,   // [4313:146] K + OE : -19 (cslSTIXMath-Regular)
    0xfb0000fc,   // [4314:147] K + udieresis : -10 (cslSTIXMath-Regular)
    0xf88000f6,   // [4315:148] K + odieresis : -15 (cslSTIXMath-Regular)
    0xf88000f3,   // [4316:149] K + oacute : -15 (cslSTIXMath-Regular)
    0xff0000e6,   // [4317:150] K + ae : -2 (cslSTIXMath-Regular)
    0x008000e5,   // [4318:151] K + aring : 1 (cslSTIXMath-Regular)
    0x008000e4,   // [4319:152] K + adieresis : 1 (cslSTIXMath-Regular)
    0xf68000d6,   // [4320:153] K + Odieresis : -19 (cslSTIXMath-Regular)
    0xf68000d3,   // [4321:154] K + Oacute : -19 (cslSTIXMath-Regular)
    0xe9800079,   // [4322:155] K + y : -45 (cslSTIXMath-Regular)
    0xfb000075,   // [4323:156] K + u : -10 (cslSTIXMath-Regular)
    0xf880006f,   // [4324:157] K + o : -15 (cslSTIXMath-Regular)
    0xf8800065,   // [4325:158] K + e : -15 (cslSTIXMath-Regular)
    0x00800061,   // [4326:159] K + a : 1 (cslSTIXMath-Regular)
    0xff800054,   // [4327:160] K + T : -1 (cslSTIXMath-Regular)
    0x00800053,   // [4328:161] K + S : 1 (cslSTIXMath-Regular)
    0xf680004f,   // [4329:162] K + O : -19 (cslSTIXMath-Regular)
    0xf6800047,   // [4330:163] K + G : -19 (cslSTIXMath-Regular)
    0xf6800043,   // [4331:164] K + C : -19 (cslSTIXMath-Regular)
    0xe740002d,   // [4332:165] K + hyphen : -50 (cslSTIXMath-Regular) ;;
    0xd680201d,   // [4333:166] L + quotedblright : -83 (cslSTIXMath-Regular)
    0xd6802019,   // [4334:167] L + quoteright : -83 (cslSTIXMath-Regular)
    0xfc0000fc,   // [4335:168] L + udieresis : -8 (cslSTIXMath-Regular)
    0xf70000dc,   // [4336:169] L + Udieresis : -18 (cslSTIXMath-Regular)
    0xff8000d6,   // [4337:170] L + Odieresis : -1 (cslSTIXMath-Regular)
    0xff8000d5,   // [4338:171] L + Otilde : -1 (cslSTIXMath-Regular)
    0xff8000d4,   // [4339:172] L + Ocircumflex : -1 (cslSTIXMath-Regular)
    0xff8000d3,   // [4340:173] L + Oacute : -1 (cslSTIXMath-Regular)
    0xff8000d2,   // [4341:174] L + Ograve : -1 (cslSTIXMath-Regular)
    0x030000c6,   // [4342:175] L + AE : 6 (cslSTIXMath-Regular)
    0xff8000c5,   // [4343:176] L + Aring : -1 (cslSTIXMath-Regular)
    0xff8000c4,   // [4344:177] L + Adieresis : -1 (cslSTIXMath-Regular)
    0xff8000c1,   // [4345:178] L + Aacute : -1 (cslSTIXMath-Regular)
    0xed800079,   // [4346:179] L + y : -37 (cslSTIXMath-Regular)
    0xfc000075,   // [4347:180] L + u : -8 (cslSTIXMath-Regular)
    0xda800059,   // [4348:181] L + Y : -75 (cslSTIXMath-Regular)
    0xe3000057,   // [4349:182] L + W : -58 (cslSTIXMath-Regular)
    0xdb000056,   // [4350:183] L + V : -74 (cslSTIXMath-Regular)
    0xf7000055,   // [4351:184] L + U : -18 (cslSTIXMath-Regular)
    0xe2000054,   // [4352:185] L + T : -60 (cslSTIXMath-Regular)
    0x01800053,   // [4353:186] L + S : 3 (cslSTIXMath-Regular)
    0xff80004f,   // [4354:187] L + O : -1 (cslSTIXMath-Regular)
    0xfe800047,   // [4355:188] L + G : -3 (cslSTIXMath-Regular)
    0xff800043,   // [4356:189] L + C : -1 (cslSTIXMath-Regular)
    0xff800041,   // [4357:190] L + A : -1 (cslSTIXMath-Regular)
    0x0bc0002d,   // [4358:191] L + hyphen : 23 (cslSTIXMath-Regular) ;;
    0xf98000fc,   // [4359:192] N + udieresis : -13 (cslSTIXMath-Regular)
    0xf98000f8,   // [4360:193] N + oslash : -13 (cslSTIXMath-Regular)
    0xf98000f6,   // [4361:194] N + odieresis : -13 (cslSTIXMath-Regular)
    0xf98000f3,   // [4362:195] N + oacute : -13 (cslSTIXMath-Regular)
    0xfa0000e9,   // [4363:196] N + eacute : -12 (cslSTIXMath-Regular)
    0xf70000e6,   // [4364:197] N + ae : -18 (cslSTIXMath-Regular)
    0xf70000e5,   // [4365:198] N + aring : -18 (cslSTIXMath-Regular)
    0xf70000e4,   // [4366:199] N + adieresis : -18 (cslSTIXMath-Regular)
    0xf70000e1,   // [4367:200] N + aacute : -18 (cslSTIXMath-Regular)
    0xfb0000d6,   // [4368:201] N + Odieresis : -10 (cslSTIXMath-Regular)
    0xfb0000d3,   // [4369:202] N + Oacute : -10 (cslSTIXMath-Regular)
    0xfb0000c7,   // [4370:203] N + Ccedilla : -10 (cslSTIXMath-Regular)
    0xf98000c6,   // [4371:204] N + AE : -13 (cslSTIXMath-Regular)
    0xf70000c5,   // [4372:205] N + Aring : -18 (cslSTIXMath-Regular)
    0xf70000c4,   // [4373:206] N + Adieresis : -18 (cslSTIXMath-Regular)
    0xf70000c1,   // [4374:207] N + Aacute : -18 (cslSTIXMath-Regular)
    0xf9800075,   // [4375:208] N + u : -13 (cslSTIXMath-Regular)
    0xf980006f,   // [4376:209] N + o : -13 (cslSTIXMath-Regular)
    0xfa000065,   // [4377:210] N + e : -12 (cslSTIXMath-Regular)
    0xf7000061,   // [4378:211] N + a : -18 (cslSTIXMath-Regular)
    0xfb00004f,   // [4379:212] N + O : -10 (cslSTIXMath-Regular)
    0xfb000047,   // [4380:213] N + G : -10 (cslSTIXMath-Regular)
    0xfb000043,   // [4381:214] N + C : -10 (cslSTIXMath-Regular)
    0xf7000041,   // [4382:215] N + A : -18 (cslSTIXMath-Regular)
    0xf980002e,   // [4383:216] N + period : -13 (cslSTIXMath-Regular)
    0xfa40002c,   // [4384:217] N + comma : -12 (cslSTIXMath-Regular) ;;
    0xf40000c6,   // [4385:218] O + AE : -24 (cslSTIXMath-Regular)
    0xf38000c5,   // [4386:219] O + Aring : -25 (cslSTIXMath-Regular)
    0xf38000c4,   // [4387:220] O + Adieresis : -25 (cslSTIXMath-Regular)
    0xf38000c1,   // [4388:221] O + Aacute : -25 (cslSTIXMath-Regular)
    0xee000059,   // [4389:222] O + Y : -36 (cslSTIXMath-Regular)
    0xf3800058,   // [4390:223] O + X : -25 (cslSTIXMath-Regular)
    0xf3800057,   // [4391:224] O + W : -25 (cslSTIXMath-Regular)
    0xf1000056,   // [4392:225] O + V : -30 (cslSTIXMath-Regular)
    0xfe800054,   // [4393:226] O + T : -3 (cslSTIXMath-Regular)
    0xf3c00041,   // [4394:227] O + A : -25 (cslSTIXMath-Regular) ;;
    0xf8000153,   // [4395:228] P + oe : -16 (cslSTIXMath-Regular)
    0xf80000f8,   // [4396:229] P + oslash : -16 (cslSTIXMath-Regular)
    0xf80000f6,   // [4397:230] P + odieresis : -16 (cslSTIXMath-Regular)
    0xf80000f3,   // [4398:231] P + oacute : -16 (cslSTIXMath-Regular)
    0xf90000e9,   // [4399:232] P + eacute : -14 (cslSTIXMath-Regular)
    0xfb8000e6,   // [4400:233] P + ae : -9 (cslSTIXMath-Regular)
    0xfb0000e5,   // [4401:234] P + aring : -10 (cslSTIXMath-Regular)
    0xfb0000e4,   // [4402:235] P + adieresis : -10 (cslSTIXMath-Regular)
    0xfb0000e1,   // [4403:236] P + aacute : -10 (cslSTIXMath-Regular)
    0xfb0000e0,   // [4404:237] P + agrave : -10 (cslSTIXMath-Regular)
    0xe38000c6,   // [4405:238] P + AE : -57 (cslSTIXMath-Regular)
    0xe28000c5,   // [4406:239] P + Aring : -59 (cslSTIXMath-Regular)
    0xe28000c4,   // [4407:240] P + Adieresis : -59 (cslSTIXMath-Regular)
    0xe28000c1,   // [4408:241] P + Aacute : -59 (cslSTIXMath-Regular)
    0xf800006f,   // [4409:242] P + o : -16 (cslSTIXMath-Regular)
    0xf9000065,   // [4410:243] P + e : -14 (cslSTIXMath-Regular)
    0xfb000061,   // [4411:244] P + a : -10 (cslSTIXMath-Regular)
    0xed00004a,   // [4412:245] P + J : -38 (cslSTIXMath-Regular)
    0xe2800041,   // [4413:246] P + A : -59 (cslSTIXMath-Regular)
    0xd300002e,   // [4414:247] P + period : -90 (cslSTIXMath-Regular)
    0xf500002d,   // [4415:248] P + hyphen : -22 (cslSTIXMath-Regular)
    0xe240002c,   // [4416:249] P + comma : -60 (cslSTIXMath-Regular) ;;
    0xf5800153,   // [4417:250] R + oe : -21 (cslSTIXMath-Regular)
    0xf4000152,   // [4418:251] R + OE : -24 (cslSTIXMath-Regular)
    0xf90000fc,   // [4419:252] R + udieresis : -14 (cslSTIXMath-Regular)
    0xf90000fa,   // [4420:253] R + uacute : -14 (cslSTIXMath-Regular)
    0xf40000f6,   // [4421:254] R + odieresis : -24 (cslSTIXMath-Regular)
    0xf40000f3,   // [4422:255] R + oacute : -24 (cslSTIXMath-Regular)
    0xf90000e9,   // [4423:256] R + eacute : -14 (cslSTIXMath-Regular)
    0xfd0000e6,   // [4424:257] R + ae : -6 (cslSTIXMath-Regular)
    0xff0000e5,   // [4425:258] R + aring : -2 (cslSTIXMath-Regular)
    0xff0000e4,   // [4426:259] R + adieresis : -2 (cslSTIXMath-Regular)
    0xff0000e1,   // [4427:260] R + aacute : -2 (cslSTIXMath-Regular)
    0xed8000dc,   // [4428:261] R + Udieresis : -37 (cslSTIXMath-Regular)
    0xf30000d6,   // [4429:262] R + Odieresis : -26 (cslSTIXMath-Regular)
    0xf30000d3,   // [4430:263] R + Oacute : -26 (cslSTIXMath-Regular)
    0xf30000c7,   // [4431:264] R + Ccedilla : -26 (cslSTIXMath-Regular)
    0xf0000079,   // [4432:265] R + y : -32 (cslSTIXMath-Regular)
    0xf9000075,   // [4433:266] R + u : -14 (cslSTIXMath-Regular)
    0xf400006f,   // [4434:267] R + o : -24 (cslSTIXMath-Regular)
    0xf9000065,   // [4435:268] R + e : -14 (cslSTIXMath-Regular)
    0xff000061,   // [4436:269] R + a : -2 (cslSTIXMath-Regular)
    0xe2000059,   // [4437:270] R + Y : -60 (cslSTIXMath-Regular)
    0xe2000057,   // [4438:271] R + W : -60 (cslSTIXMath-Regular)
    0xdf800056,   // [4439:272] R + V : -65 (cslSTIXMath-Regular)
    0xed800055,   // [4440:273] R + U : -37 (cslSTIXMath-Regular)
    0xf4000054,   // [4441:274] R + T : -24 (cslSTIXMath-Regular)
    0xf300004f,   // [4442:275] R + O : -26 (cslSTIXMath-Regular)
    0xf3000047,   // [4443:276] R + G : -26 (cslSTIXMath-Regular)
    0xf3000043,   // [4444:277] R + C : -26 (cslSTIXMath-Regular)
    0xea40002d,   // [4445:278] R + hyphen : -44 (cslSTIXMath-Regular) ;;
    0xf28000c6,   // [4446:279] S + AE : -27 (cslSTIXMath-Regular)
    0xf10000c5,   // [4447:280] S + Aring : -30 (cslSTIXMath-Regular)
    0xf10000c4,   // [4448:281] S + Adieresis : -30 (cslSTIXMath-Regular)
    0xf10000c1,   // [4449:282] S + Aacute : -30 (cslSTIXMath-Regular)
    0xfa800074,   // [4450:283] S + t : -11 (cslSTIXMath-Regular)
    0xf2800059,   // [4451:284] S + Y : -27 (cslSTIXMath-Regular)
    0xf6000057,   // [4452:285] S + W : -20 (cslSTIXMath-Regular)
    0xf6000056,   // [4453:286] S + V : -20 (cslSTIXMath-Regular)
    0xf7000054,   // [4454:287] S + T : -18 (cslSTIXMath-Regular)
    0xf1400041,   // [4455:288] S + A : -30 (cslSTIXMath-Regular) ;;
    0xe2002039,   // [4456:289] T + guilsinglleft : -60 (cslSTIXMath-Regular)
    0xfd800152,   // [4457:290] T + OE : -5 (cslSTIXMath-Regular)
    0xf00000f8,   // [4458:291] T + oslash : -32 (cslSTIXMath-Regular)
    0xf80000f6,   // [4459:292] T + odieresis : -16 (cslSTIXMath-Regular)
    0xf88000f4,   // [4460:293] T + ocircumflex : -15 (cslSTIXMath-Regular)
    0xf00000f3,   // [4461:294] T + oacute : -32 (cslSTIXMath-Regular)
    0xf70000f2,   // [4462:295] T + ograve : -18 (cslSTIXMath-Regular)
    0xec0000e6,   // [4463:296] T + ae : -40 (cslSTIXMath-Regular)
    0xfa0000e4,   // [4464:297] T + adieresis : -12 (cslSTIXMath-Regular)
    0xfe8000e3,   // [4465:298] T + atilde : -3 (cslSTIXMath-Regular)
    0xfc8000e2,   // [4466:299] T + acircumflex : -7 (cslSTIXMath-Regular)
    0xf50000e1,   // [4467:300] T + aacute : -22 (cslSTIXMath-Regular)
    0xf88000e0,   // [4468:301] T + agrave : -15 (cslSTIXMath-Regular)
    0xfd8000d8,   // [4469:302] T + Oslash : -5 (cslSTIXMath-Regular)
    0xfd8000d6,   // [4470:303] T + Odieresis : -5 (cslSTIXMath-Regular)
    0xfd8000d5,   // [4471:304] T + Otilde : -5 (cslSTIXMath-Regular)
    0xfd8000d4,   // [4472:305] T + Ocircumflex : -5 (cslSTIXMath-Regular)
    0xfd8000d3,   // [4473:306] T + Oacute : -5 (cslSTIXMath-Regular)
    0xfd8000d2,   // [4474:307] T + Ograve : -5 (cslSTIXMath-Regular)
    0xe90000c6,   // [4475:308] T + AE : -46 (cslSTIXMath-Regular)
    0xe58000c5,   // [4476:309] T + Aring : -53 (cslSTIXMath-Regular)
    0xe58000c4,   // [4477:310] T + Adieresis : -53 (cslSTIXMath-Regular)
    0xe58000c3,   // [4478:311] T + Atilde : -53 (cslSTIXMath-Regular)
    0xe58000c2,   // [4479:312] T + Acircumflex : -53 (cslSTIXMath-Regular)
    0xe58000c1,   // [4480:313] T + Aacute : -53 (cslSTIXMath-Regular)
    0xe58000c0,   // [4481:314] T + Agrave : -53 (cslSTIXMath-Regular)
    0xe50000ab,   // [4482:315] T + guillemotleft : -54 (cslSTIXMath-Regular)
    0xe7800079,   // [4483:316] T + y : -49 (cslSTIXMath-Regular)
    0xe6800077,   // [4484:317] T + w : -51 (cslSTIXMath-Regular)
    0xe6800076,   // [4485:318] T + v : -51 (cslSTIXMath-Regular)
    0xed000075,   // [4486:319] T + u : -38 (cslSTIXMath-Regular)
    0xee000073,   // [4487:320] T + s : -36 (cslSTIXMath-Regular)
    0xf4000072,   // [4488:321] T + r : -24 (cslSTIXMath-Regular)
    0xf000006f,   // [4489:322] T + o : -32 (cslSTIXMath-Regular)
    0xfb00006a,   // [4490:323] T + j : -10 (cslSTIXMath-Regular)
    0xfb800069,   // [4491:324] T + i : -9 (cslSTIXMath-Regular)
    0xea000067,   // [4492:325] T + g : -44 (cslSTIXMath-Regular)
    0xeb800065,   // [4493:326] T + e : -41 (cslSTIXMath-Regular)
    0xeb800063,   // [4494:327] T + c : -41 (cslSTIXMath-Regular)
    0xed000061,   // [4495:328] T + a : -38 (cslSTIXMath-Regular)
    0x05000059,   // [4496:329] T + Y : 10 (cslSTIXMath-Regular)
    0x09800057,   // [4497:330] T + W : 19 (cslSTIXMath-Regular)
    0x07000056,   // [4498:331] T + V : 14 (cslSTIXMath-Regular)
    0xfb800053,   // [4499:332] T + S : -9 (cslSTIXMath-Regular)
    0xfd80004f,   // [4500:333] T + O : -5 (cslSTIXMath-Regular)
    0xfc80004a,   // [4501:334] T + J : -7 (cslSTIXMath-Regular)
    0xff000047,   // [4502:335] T + G : -2 (cslSTIXMath-Regular)
    0xff000043,   // [4503:336] T + C : -2 (cslSTIXMath-Regular)
    0xe5800041,   // [4504:337] T + A : -53 (cslSTIXMath-Regular)
    0xf300003b,   // [4505:338] T + semicolon : -26 (cslSTIXMath-Regular)
    0xf300003a,   // [4506:339] T + colon : -26 (cslSTIXMath-Regular)
    0xe980002e,   // [4507:340] T + period : -45 (cslSTIXMath-Regular)
    0xee00002d,   // [4508:341] T + hyphen : -36 (cslSTIXMath-Regular)
    0xf2c0002c,   // [4509:342] T + comma : -27 (cslSTIXMath-Regular) ;;
    0xec8000c6,   // [4510:343] U + AE : -39 (cslSTIXMath-Regular)
    0xea8000c5,   // [4511:344] U + Aring : -43 (cslSTIXMath-Regular)
    0xea8000c4,   // [4512:345] U + Adieresis : -43 (cslSTIXMath-Regular)
    0xea8000c3,   // [4513:346] U + Atilde : -43 (cslSTIXMath-Regular)
    0xea8000c2,   // [4514:347] U + Acircumflex : -43 (cslSTIXMath-Regular)
    0xea8000c1,   // [4515:348] U + Aacute : -43 (cslSTIXMath-Regular)
    0xfd000072,   // [4516:349] U + r : -6 (cslSTIXMath-Regular)
    0xf7000070,   // [4517:350] U + p : -18 (cslSTIXMath-Regular)
    0xf600006e,   // [4518:351] U + n : -20 (cslSTIXMath-Regular)
    0xf600006d,   // [4519:352] U + m : -20 (cslSTIXMath-Regular)
    0xea800041,   // [4520:353] U + A : -43 (cslSTIXMath-Regular)
    0xf200002e,   // [4521:354] U + period : -28 (cslSTIXMath-Regular)
    0xf0c0002c,   // [4522:355] U + comma : -31 (cslSTIXMath-Regular) ;;
    0xda802039,   // [4523:356] V + guilsinglleft : -75 (cslSTIXMath-Regular)
    0xda8000f8,   // [4524:357] V + oslash : -75 (cslSTIXMath-Regular)
    0xe78000f6,   // [4525:358] V + odieresis : -49 (cslSTIXMath-Regular)
    0xed8000f5,   // [4526:359] V + otilde : -37 (cslSTIXMath-Regular)
    0xe28000f4,   // [4527:360] V + ocircumflex : -59 (cslSTIXMath-Regular)
    0xda8000f3,   // [4528:361] V + oacute : -75 (cslSTIXMath-Regular)
    0xf28000f2,   // [4529:362] V + ograve : -27 (cslSTIXMath-Regular)
    0xdd0000e9,   // [4530:363] V + eacute : -70 (cslSTIXMath-Regular)
    0xf70000e8,   // [4531:364] V + egrave : -18 (cslSTIXMath-Regular)
    0xd80000e6,   // [4532:365] V + ae : -80 (cslSTIXMath-Regular)
    0xed0000e5,   // [4533:366] V + aring : -38 (cslSTIXMath-Regular)
    0xed0000e4,   // [4534:367] V + adieresis : -38 (cslSTIXMath-Regular)
    0xf10000e3,   // [4535:368] V + atilde : -30 (cslSTIXMath-Regular)
    0xe70000e2,   // [4536:369] V + acircumflex : -50 (cslSTIXMath-Regular)
    0xd80000e1,   // [4537:370] V + aacute : -80 (cslSTIXMath-Regular)
    0xf70000e0,   // [4538:371] V + agrave : -18 (cslSTIXMath-Regular)
    0xed0000d8,   // [4539:372] V + Oslash : -38 (cslSTIXMath-Regular)
    0xed0000d6,   // [4540:373] V + Odieresis : -38 (cslSTIXMath-Regular)
    0xed0000d5,   // [4541:374] V + Otilde : -38 (cslSTIXMath-Regular)
    0xed0000d4,   // [4542:375] V + Ocircumflex : -38 (cslSTIXMath-Regular)
    0xed0000d3,   // [4543:376] V + Oacute : -38 (cslSTIXMath-Regular)
    0xed0000d2,   // [4544:377] V + Ograve : -38 (cslSTIXMath-Regular)
    0xdb8000c6,   // [4545:378] V + AE : -73 (cslSTIXMath-Regular)
    0xd60000c5,   // [4546:379] V + Aring : -84 (cslSTIXMath-Regular)
    0xd60000c4,   // [4547:380] V + Adieresis : -84 (cslSTIXMath-Regular)
    0xd60000c3,   // [4548:381] V + Atilde : -84 (cslSTIXMath-Regular)
    0xd60000c2,   // [4549:382] V + Acircumflex : -84 (cslSTIXMath-Regular)
    0xd60000c1,   // [4550:383] V + Aacute : -84 (cslSTIXMath-Regular)
    0xd60000c0,   // [4551:384] V + Agrave : -84 (cslSTIXMath-Regular)
    0xda8000ab,   // [4552:385] V + guillemotleft : -75 (cslSTIXMath-Regular)
    0xe7000079,   // [4553:386] V + y : -50 (cslSTIXMath-Regular)
    0xe7000075,   // [4554:387] V + u : -50 (cslSTIXMath-Regular)
    0xec000072,   // [4555:388] V + r : -40 (cslSTIXMath-Regular)
    0xda80006f,   // [4556:389] V + o : -75 (cslSTIXMath-Regular)
    0xf6000069,   // [4557:390] V + i : -20 (cslSTIXMath-Regular)
    0xdf800067,   // [4558:391] V + g : -65 (cslSTIXMath-Regular)
    0xdd000065,   // [4559:392] V + e : -70 (cslSTIXMath-Regular)
    0xd8000061,   // [4560:393] V + a : -80 (cslSTIXMath-Regular)
    0x04800054,   // [4561:394] V + T : 9 (cslSTIXMath-Regular)
    0xee800053,   // [4562:395] V + S : -35 (cslSTIXMath-Regular)
    0xed00004f,   // [4563:396] V + O : -38 (cslSTIXMath-Regular)
    0xed000047,   // [4564:397] V + G : -38 (cslSTIXMath-Regular)
    0xed000043,   // [4565:398] V + C : -38 (cslSTIXMath-Regular)
    0xd6000041,   // [4566:399] V + A : -84 (cslSTIXMath-Regular)
    0xe280003b,   // [4567:400] V + semicolon : -59 (cslSTIXMath-Regular)
    0xe200003a,   // [4568:401] V + colon : -60 (cslSTIXMath-Regular)
    0xce00002e,   // [4569:402] V + period : -100 (cslSTIXMath-Regular)
    0xe200002d,   // [4570:403] V + hyphen : -60 (cslSTIXMath-Regular)
    0xce40002c,   // [4571:404] V + comma : -100 (cslSTIXMath-Regular) ;;
    0xe1802039,   // [4572:405] W + guilsinglleft : -61 (cslSTIXMath-Regular)
    0xdd0000f8,   // [4573:406] W + oslash : -70 (cslSTIXMath-Regular)
    0xe48000f6,   // [4574:407] W + odieresis : -55 (cslSTIXMath-Regular)
    0xec0000f5,   // [4575:408] W + otilde : -40 (cslSTIXMath-Regular)
    0xe08000f4,   // [4576:409] W + ocircumflex : -63 (cslSTIXMath-Regular)
    0xdd0000f3,   // [4577:410] W + oacute : -70 (cslSTIXMath-Regular)
    0xf20000f2,   // [4578:411] W + ograve : -28 (cslSTIXMath-Regular)
    0xe20000e9,   // [4579:412] W + eacute : -60 (cslSTIXMath-Regular)
    0xfb8000e8,   // [4580:413] W + egrave : -9 (cslSTIXMath-Regular)
    0xe20000e6,   // [4581:414] W + ae : -60 (cslSTIXMath-Regular)
    0xf28000e5,   // [4582:415] W + aring : -27 (cslSTIXMath-Regular)
    0xef0000e4,   // [4583:416] W + adieresis : -34 (cslSTIXMath-Regular)
    0xf40000e3,   // [4584:417] W + atilde : -24 (cslSTIXMath-Regular)
    0xef0000e2,   // [4585:418] W + acircumflex : -34 (cslSTIXMath-Regular)
    0xe70000e1,   // [4586:419] W + aacute : -50 (cslSTIXMath-Regular)
    0xf70000e0,   // [4587:420] W + agrave : -18 (cslSTIXMath-Regular)
    0xef8000d8,   // [4588:421] W + Oslash : -33 (cslSTIXMath-Regular)
    0xef8000d6,   // [4589:422] W + Odieresis : -33 (cslSTIXMath-Regular)
    0xef8000d5,   // [4590:423] W + Otilde : -33 (cslSTIXMath-Regular)
    0xef8000d4,   // [4591:424] W + Ocircumflex : -33 (cslSTIXMath-Regular)
    0xef8000d3,   // [4592:425] W + Oacute : -33 (cslSTIXMath-Regular)
    0xef8000d2,   // [4593:426] W + Ograve : -33 (cslSTIXMath-Regular)
    0xd40000c6,   // [4594:427] W + AE : -88 (cslSTIXMath-Regular)
    0xcd0000c5,   // [4595:428] W + Aring : -102 (cslSTIXMath-Regular)
    0xcd0000c4,   // [4596:429] W + Adieresis : -102 (cslSTIXMath-Regular)
    0xcd0000c3,   // [4597:430] W + Atilde : -102 (cslSTIXMath-Regular)
    0xcd0000c2,   // [4598:431] W + Acircumflex : -102 (cslSTIXMath-Regular)
    0xcd0000c1,   // [4599:432] W + Aacute : -102 (cslSTIXMath-Regular)
    0xcd0000c0,   // [4600:433] W + Agrave : -102 (cslSTIXMath-Regular)
    0xe18000ab,   // [4601:434] W + guillemotleft : -61 (cslSTIXMath-Regular)
    0xe9800079,   // [4602:435] W + y : -45 (cslSTIXMath-Regular)
    0xf1000075,   // [4603:436] W + u : -30 (cslSTIXMath-Regular)
    0xf1000072,   // [4604:437] W + r : -30 (cslSTIXMath-Regular)
    0xdd00006f,   // [4605:438] W + o : -70 (cslSTIXMath-Regular)
    0xf9800069,   // [4606:439] W + i : -13 (cslSTIXMath-Regular)
    0xe3800067,   // [4607:440] W + g : -57 (cslSTIXMath-Regular)
    0xe2000065,   // [4608:441] W + e : -60 (cslSTIXMath-Regular)
    0xe2000061,   // [4609:442] W + a : -60 (cslSTIXMath-Regular)
    0x08000054,   // [4610:443] W + T : 16 (cslSTIXMath-Regular)
    0xf4000053,   // [4611:444] W + S : -24 (cslSTIXMath-Regular)
    0xef80004f,   // [4612:445] W + O : -33 (cslSTIXMath-Regular)
    0xed000047,   // [4613:446] W + G : -38 (cslSTIXMath-Regular)
    0xef800043,   // [4614:447] W + C : -33 (cslSTIXMath-Regular)
    0xcd000041,   // [4615:448] W + A : -102 (cslSTIXMath-Regular)
    0xe880003b,   // [4616:449] W + semicolon : -47 (cslSTIXMath-Regular)
    0xef80003a,   // [4617:450] W + colon : -33 (cslSTIXMath-Regular)
    0xd200002e,   // [4618:451] W + period : -92 (cslSTIXMath-Regular)
    0xec00002d,   // [4619:452] W + hyphen : -40 (cslSTIXMath-Regular)
    0xd840002c,   // [4620:453] W + comma : -80 (cslSTIXMath-Regular) ;;
    0xed0000d6,   // [4621:454] X + Odieresis : -38 (cslSTIXMath-Regular)
    0xe0800079,   // [4622:455] X + y : -63 (cslSTIXMath-Regular)
    0xf7000075,   // [4623:456] X + u : -18 (cslSTIXMath-Regular)
    0xf280006f,   // [4624:457] X + o : -27 (cslSTIXMath-Regular)
    0xf1800065,   // [4625:458] X + e : -29 (cslSTIXMath-Regular)
    0xfc800061,   // [4626:459] X + a : -7 (cslSTIXMath-Regular)
    0xed000051,   // [4627:460] X + Q : -38 (cslSTIXMath-Regular)
    0xed00004f,   // [4628:461] X + O : -38 (cslSTIXMath-Regular)
    0xee800043,   // [4629:462] X + C : -35 (cslSTIXMath-Regular)
    0xe9c0002d,   // [4630:463] X + hyphen : -45 (cslSTIXMath-Regular) ;;
    0xbf002039,   // [4631:464] Y + guilsinglleft : -130 (cslSTIXMath-Regular)
    0xde8000f8,   // [4632:465] Y + oslash : -67 (cslSTIXMath-Regular)
    0xd58000e6,   // [4633:466] Y + ae : -85 (cslSTIXMath-Regular)
    0xe68000d8,   // [4634:467] Y + Oslash : -51 (cslSTIXMath-Regular)
    0xe68000d6,   // [4635:468] Y + Odieresis : -51 (cslSTIXMath-Regular)
    0xe68000d5,   // [4636:469] Y + Otilde : -51 (cslSTIXMath-Regular)
    0xe68000d4,   // [4637:470] Y + Ocircumflex : -51 (cslSTIXMath-Regular)
    0xe68000d3,   // [4638:471] Y + Oacute : -51 (cslSTIXMath-Regular)
    0xe68000d2,   // [4639:472] Y + Ograve : -51 (cslSTIXMath-Regular)
    0xde8000c6,   // [4640:473] Y + AE : -67 (cslSTIXMath-Regular)
    0xda8000c5,   // [4641:474] Y + Aring : -75 (cslSTIXMath-Regular)
    0xda8000c4,   // [4642:475] Y + Adieresis : -75 (cslSTIXMath-Regular)
    0xda8000c3,   // [4643:476] Y + Atilde : -75 (cslSTIXMath-Regular)
    0xda8000c2,   // [4644:477] Y + Acircumflex : -75 (cslSTIXMath-Regular)
    0xda8000c1,   // [4645:478] Y + Aacute : -75 (cslSTIXMath-Regular)
    0xda8000c0,   // [4646:479] Y + Agrave : -75 (cslSTIXMath-Regular)
    0xc58000ab,   // [4647:480] Y + guillemotleft : -117 (cslSTIXMath-Regular)
    0xe1000076,   // [4648:481] Y + v : -62 (cslSTIXMath-Regular)
    0xe2800075,   // [4649:482] Y + u : -59 (cslSTIXMath-Regular)
    0xe3800070,   // [4650:483] Y + p : -57 (cslSTIXMath-Regular)
    0xde80006f,   // [4651:484] Y + o : -67 (cslSTIXMath-Regular)
    0xf7800069,   // [4652:485] Y + i : -17 (cslSTIXMath-Regular)
    0xdb800067,   // [4653:486] Y + g : -73 (cslSTIXMath-Regular)
    0xdf000065,   // [4654:487] Y + e : -66 (cslSTIXMath-Regular)
    0xd8000061,   // [4655:488] Y + a : -80 (cslSTIXMath-Regular)
    0x06000054,   // [4656:489] Y + T : 12 (cslSTIXMath-Regular)
    0xea800053,   // [4657:490] Y + S : -43 (cslSTIXMath-Regular)
    0xe680004f,   // [4658:491] Y + O : -51 (cslSTIXMath-Regular)
    0xe6800047,   // [4659:492] Y + G : -51 (cslSTIXMath-Regular)
    0xe7000043,   // [4660:493] Y + C : -50 (cslSTIXMath-Regular)
    0xda800041,   // [4661:494] Y + A : -75 (cslSTIXMath-Regular)
    0xe180003b,   // [4662:495] Y + semicolon : -61 (cslSTIXMath-Regular)
    0xdf00003a,   // [4663:496] Y + colon : -66 (cslSTIXMath-Regular)
    0xd180002e,   // [4664:497] Y + period : -93 (cslSTIXMath-Regular)
    0xd800002d,   // [4665:498] Y + hyphen : -80 (cslSTIXMath-Regular)
    0xdac0002c,   // [4666:499] Y + comma : -75 (cslSTIXMath-Regular) ;;
    0xef800079,   // [4667:500] Z + y : -33 (cslSTIXMath-Regular)
    0xef000077,   // [4668:501] Z + w : -34 (cslSTIXMath-Regular)
    0xefc00076,   // [4669:502] Z + v : -33 (cslSTIXMath-Regular) ;;
    0xeb802019,   // [4670:503] a + quoteright : -41 (cslSTIXMath-Regular)
    0xf2800079,   // [4671:504] a + y : -27 (cslSTIXMath-Regular)
    0xef800077,   // [4672:505] a + w : -33 (cslSTIXMath-Regular)
    0xf0000076,   // [4673:506] a + v : -32 (cslSTIXMath-Regular)
    0xfc40006a,   // [4674:507] a + j : -8 (cslSTIXMath-Regular) ;;
    0xf6000079,   // [4675:508] b + y : -20 (cslSTIXMath-Regular)
    0xf7000077,   // [4676:509] b + w : -18 (cslSTIXMath-Regular)
    0xf7400076,   // [4677:510] b + v : -18 (cslSTIXMath-Regular) ;;
    0xf500006b,   // [4678:511] c + k : -22 (cslSTIXMath-Regular)
    0xf8400068,   // [4679:512] c + h : -16 (cslSTIXMath-Regular) ;;
    0xf6002019,   // [4680:513] e + quoteright : -20 (cslSTIXMath-Regular)
    0xf5800079,   // [4681:514] e + y : -21 (cslSTIXMath-Regular)
    0xf5800078,   // [4682:515] e + x : -21 (cslSTIXMath-Regular)
    0xf6800077,   // [4683:516] e + w : -19 (cslSTIXMath-Regular)
    0xf6800076,   // [4684:517] e + v : -19 (cslSTIXMath-Regular)
    0xfbc00074,   // [4685:518] e + t : -9 (cslSTIXMath-Regular) ;;
    0x08002019,   // [4686:519] f + quoteright : 16 (cslSTIXMath-Regular)
    0xee000153,   // [4687:520] f + oe : -36 (cslSTIXMath-Regular)
    0xed8000f8,   // [4688:521] f + oslash : -37 (cslSTIXMath-Regular)
    0x008000f6,   // [4689:522] f + odieresis : 1 (cslSTIXMath-Regular)
    0xf40000f4,   // [4690:523] f + ocircumflex : -24 (cslSTIXMath-Regular)
    0xf00000f3,   // [4691:524] f + oacute : -32 (cslSTIXMath-Regular)
    0xff8000f2,   // [4692:525] f + ograve : -1 (cslSTIXMath-Regular)
    0xf90000ea,   // [4693:526] f + ecircumflex : -14 (cslSTIXMath-Regular)
    0xef0000e9,   // [4694:527] f + eacute : -34 (cslSTIXMath-Regular)
    0x048000e8,   // [4695:528] f + egrave : 9 (cslSTIXMath-Regular)
    0xf28000e6,   // [4696:529] f + ae : -27 (cslSTIXMath-Regular)
    0xfc0000e5,   // [4697:530] f + aring : -8 (cslSTIXMath-Regular)
    0x058000e4,   // [4698:531] f + adieresis : 11 (cslSTIXMath-Regular)
    0xf30000e1,   // [4699:532] f + aacute : -26 (cslSTIXMath-Regular)
    0x05000074,   // [4700:533] f + t : 10 (cslSTIXMath-Regular)
    0xf8000073,   // [4701:534] f + s : -16 (cslSTIXMath-Regular)
    0xf000006f,   // [4702:535] f + o : -32 (cslSTIXMath-Regular)
    0x1600006c,   // [4703:536] f + l : 44 (cslSTIXMath-Regular)
    0x0380006a,   // [4704:537] f + j : 7 (cslSTIXMath-Regular)
    0x07000069,   // [4705:538] f + i : 14 (cslSTIXMath-Regular)
    0x03000066,   // [4706:539] f + f : 6 (cslSTIXMath-Regular)
    0xef000065,   // [4707:540] f + e : -34 (cslSTIXMath-Regular)
    0xf3000061,   // [4708:541] f + a : -26 (cslSTIXMath-Regular)
    0x2120006a,   // [4709:542] f + j ligature #66 (cslSTIXMath-Regular)
    0x21a0006c,   // [4710:543] f + l ligature #67 (cslSTIXMath-Regular)
    0x22200069,   // [4711:544] f + i ligature #68 (cslSTIXMath-Regular)
    0x22e00066,   // [4712:545] f + f ligature #69 (cslSTIXMath-Regular) ;;
    0xf80000f6,   // [4713:546] g + odieresis : -16 (cslSTIXMath-Regular)
    0xf80000f3,   // [4714:547] g + oacute : -16 (cslSTIXMath-Regular)
    0xf88000eb,   // [4715:548] g + edieresis : -15 (cslSTIXMath-Regular)
    0xf88000ea,   // [4716:549] g + ecircumflex : -15 (cslSTIXMath-Regular)
    0xf88000e9,   // [4717:550] g + eacute : -15 (cslSTIXMath-Regular)
    0xf88000e8,   // [4718:551] g + egrave : -15 (cslSTIXMath-Regular)
    0xfa8000e6,   // [4719:552] g + ae : -11 (cslSTIXMath-Regular)
    0xfa8000e5,   // [4720:553] g + aring : -11 (cslSTIXMath-Regular)
    0xfa8000e4,   // [4721:554] g + adieresis : -11 (cslSTIXMath-Regular)
    0x05000072,   // [4722:555] g + r : 10 (cslSTIXMath-Regular)
    0xff00006c,   // [4723:556] g + l : -2 (cslSTIXMath-Regular)
    0xf8800065,   // [4724:557] g + e : -15 (cslSTIXMath-Regular)
    0xfac00061,   // [4725:558] g + a : -11 (cslSTIXMath-Regular) ;;
    0xec802019,   // [4726:559] h + quoteright : -39 (cslSTIXMath-Regular)
    0xf6400079,   // [4727:560] h + y : -20 (cslSTIXMath-Regular) ;;
    0xef00006a,   // [4728:561] i + j : -34 (cslSTIXMath-Regular)
    0xf7000054,   // [4729:562] i + T : -18 (cslSTIXMath-Regular)
    0x2360006a,   // [4730:563] i + j ligature #70 (cslSTIXMath-Regular) ;;
    0x070000fc,   // [4731:564] k + udieresis : 14 (cslSTIXMath-Regular)
    0xf58000f6,   // [4732:565] k + odieresis : -21 (cslSTIXMath-Regular)
    0xf58000f3,   // [4733:566] k + oacute : -21 (cslSTIXMath-Regular)
    0xf98000e9,   // [4734:567] k + eacute : -13 (cslSTIXMath-Regular)
    0xf98000e8,   // [4735:568] k + egrave : -13 (cslSTIXMath-Regular)
    0xff0000e6,   // [4736:569] k + ae : -2 (cslSTIXMath-Regular)
    0x008000e5,   // [4737:570] k + aring : 1 (cslSTIXMath-Regular)
    0x008000e4,   // [4738:571] k + adieresis : 1 (cslSTIXMath-Regular)
    0x008000e1,   // [4739:572] k + aacute : 1 (cslSTIXMath-Regular)
    0x07000075,   // [4740:573] k + u : 14 (cslSTIXMath-Regular)
    0x02800073,   // [4741:574] k + s : 5 (cslSTIXMath-Regular)
    0xf580006f,   // [4742:575] k + o : -21 (cslSTIXMath-Regular)
    0xfc000067,   // [4743:576] k + g : -8 (cslSTIXMath-Regular)
    0xf9800065,   // [4744:577] k + e : -13 (cslSTIXMath-Regular)
    0x00800061,   // [4745:578] k + a : 1 (cslSTIXMath-Regular)
    0xff80002e,   // [4746:579] k + period : -1 (cslSTIXMath-Regular)
    0xe780002d,   // [4747:580] k + hyphen : -49 (cslSTIXMath-Regular)
    0x02c0002c,   // [4748:581] k + comma : 5 (cslSTIXMath-Regular) ;;
    0xf5800079,   // [4749:582] l + y : -21 (cslSTIXMath-Regular)
    0xf5c00076,   // [4750:583] l + v : -21 (cslSTIXMath-Regular) ;;
    0xf6000079,   // [4751:584] m + y : -20 (cslSTIXMath-Regular)
    0xf6000077,   // [4752:585] m + w : -20 (cslSTIXMath-Regular)
    0xf6000076,   // [4753:586] m + v : -20 (cslSTIXMath-Regular)
    0xfec00070,   // [4754:587] m + p : -3 (cslSTIXMath-Regular) ;;
    0xec002019,   // [4755:588] n + quoteright : -40 (cslSTIXMath-Regular)
    0xf4000079,   // [4756:589] n + y : -24 (cslSTIXMath-Regular)
    0xf0000077,   // [4757:590] n + w : -32 (cslSTIXMath-Regular)
    0xf0800076,   // [4758:591] n + v : -31 (cslSTIXMath-Regular)
    0xff800070,   // [4759:592] n + p : -1 (cslSTIXMath-Regular)
    0xee400054,   // [4760:593] n + T : -36 (cslSTIXMath-Regular) ;;
    0xee002019,   // [4761:594] o + quoteright : -36 (cslSTIXMath-Regular)
    0xf5000079,   // [4762:595] o + y : -22 (cslSTIXMath-Regular)
    0xf5000078,   // [4763:596] o + x : -22 (cslSTIXMath-Regular)
    0xf5000077,   // [4764:597] o + w : -22 (cslSTIXMath-Regular)
    0xf5000076,   // [4765:598] o + v : -22 (cslSTIXMath-Regular)
    0xfc000074,   // [4766:599] o + t : -8 (cslSTIXMath-Regular)
    0xd4c00054,   // [4767:600] o + T : -87 (cslSTIXMath-Regular) ;;
    0xfc800079,   // [4768:601] p + y : -7 (cslSTIXMath-Regular)
    0xffc00074,   // [4769:602] p + t : -1 (cslSTIXMath-Regular) ;;
    0xf9800075,   // [4770:603] q + u : -13 (cslSTIXMath-Regular)
    0xfec00063,   // [4771:604] q + c : -3 (cslSTIXMath-Regular) ;;
    0xf7802019,   // [4772:605] r + quoteright : -17 (cslSTIXMath-Regular)
    0xfd000153,   // [4773:606] r + oe : -6 (cslSTIXMath-Regular)
    0xfd0000f8,   // [4774:607] r + oslash : -6 (cslSTIXMath-Regular)
    0xfd0000f6,   // [4775:608] r + odieresis : -6 (cslSTIXMath-Regular)
    0xfd0000f4,   // [4776:609] r + ocircumflex : -6 (cslSTIXMath-Regular)
    0xfd0000f3,   // [4777:610] r + oacute : -6 (cslSTIXMath-Regular)
    0xfd0000f2,   // [4778:611] r + ograve : -6 (cslSTIXMath-Regular)
    0xff0000ea,   // [4779:612] r + ecircumflex : -2 (cslSTIXMath-Regular)
    0xff0000e9,   // [4780:613] r + eacute : -2 (cslSTIXMath-Regular)
    0xff0000e8,   // [4781:614] r + egrave : -2 (cslSTIXMath-Regular)
    0xff0000e7,   // [4782:615] r + ccedilla : -2 (cslSTIXMath-Regular)
    0xfe8000e6,   // [4783:616] r + ae : -3 (cslSTIXMath-Regular)
    0xff0000e5,   // [4784:617] r + aring : -2 (cslSTIXMath-Regular)
    0xff0000e4,   // [4785:618] r + adieresis : -2 (cslSTIXMath-Regular)
    0xff0000e2,   // [4786:619] r + acircumflex : -2 (cslSTIXMath-Regular)
    0xff0000e1,   // [4787:620] r + aacute : -2 (cslSTIXMath-Regular)
    0xff0000e0,   // [4788:621] r + agrave : -2 (cslSTIXMath-Regular)
    0x0100007a,   // [4789:622] r + z : 2 (cslSTIXMath-Regular)
    0x0a800079,   // [4790:623] r + y : 21 (cslSTIXMath-Regular)
    0x08000078,   // [4791:624] r + x : 16 (cslSTIXMath-Regular)
    0x09800077,   // [4792:625] r + w : 19 (cslSTIXMath-Regular)
    0x0a000076,   // [4793:626] r + v : 20 (cslSTIXMath-Regular)
    0x0c000075,   // [4794:627] r + u : 24 (cslSTIXMath-Regular)
    0x0b800074,   // [4795:628] r + t : 23 (cslSTIXMath-Regular)
    0x0d000072,   // [4796:629] r + r : 26 (cslSTIXMath-Regular)
    0xfb000071,   // [4797:630] r + q : -10 (cslSTIXMath-Regular)
    0x0c800070,   // [4798:631] r + p : 25 (cslSTIXMath-Regular)
    0xfd00006f,   // [4799:632] r + o : -6 (cslSTIXMath-Regular)
    0x0a80006e,   // [4800:633] r + n : 21 (cslSTIXMath-Regular)
    0x0a00006d,   // [4801:634] r + m : 20 (cslSTIXMath-Regular)
    0xfc80006c,   // [4802:635] r + l : -7 (cslSTIXMath-Regular)
    0xfc80006b,   // [4803:636] r + k : -7 (cslSTIXMath-Regular)
    0x0680006a,   // [4804:637] r + j : 13 (cslSTIXMath-Regular)
    0x09800069,   // [4805:638] r + i : 19 (cslSTIXMath-Regular)
    0xff000068,   // [4806:639] r + h : -2 (cslSTIXMath-Regular)
    0xf8000067,   // [4807:640] r + g : -16 (cslSTIXMath-Regular)
    0x09800066,   // [4808:641] r + f : 19 (cslSTIXMath-Regular)
    0xff000065,   // [4809:642] r + e : -2 (cslSTIXMath-Regular)
    0xff000064,   // [4810:643] r + d : -2 (cslSTIXMath-Regular)
    0xff000063,   // [4811:644] r + c : -2 (cslSTIXMath-Regular)
    0xff000061,   // [4812:645] r + a : -2 (cslSTIXMath-Regular)
    0xfc80003b,   // [4813:646] r + semicolon : -7 (cslSTIXMath-Regular)
    0xfc00003a,   // [4814:647] r + colon : -8 (cslSTIXMath-Regular)
    0xe800002e,   // [4815:648] r + period : -48 (cslSTIXMath-Regular)
    0xf180002d,   // [4816:649] r + hyphen : -29 (cslSTIXMath-Regular)
    0xf340002c,   // [4817:650] r + comma : -26 (cslSTIXMath-Regular) ;;
    0xee802019,   // [4818:651] s + quoteright : -35 (cslSTIXMath-Regular)
    0xfc400074,   // [4819:652] s + t : -8 (cslSTIXMath-Regular) ;;
    0xf1002019,   // [4820:653] t + quoteright : -30 (cslSTIXMath-Regular)
    0xff0000f6,   // [4821:654] t + odieresis : -2 (cslSTIXMath-Regular)
    0xff0000f3,   // [4822:655] t + oacute : -2 (cslSTIXMath-Regular)
    0x040000e6,   // [4823:656] t + ae : 8 (cslSTIXMath-Regular)
    0x050000e5,   // [4824:657] t + aring : 10 (cslSTIXMath-Regular)
    0x050000e4,   // [4825:658] t + adieresis : 10 (cslSTIXMath-Regular)
    0x050000e1,   // [4826:659] t + aacute : 10 (cslSTIXMath-Regular)
    0xff00006f,   // [4827:660] t + o : -2 (cslSTIXMath-Regular)
    0x04000068,   // [4828:661] t + h : 8 (cslSTIXMath-Regular)
    0x05000061,   // [4829:662] t + a : 10 (cslSTIXMath-Regular)
    0x01000053,   // [4830:663] t + S : 2 (cslSTIXMath-Regular)
    0x0300003b,   // [4831:664] t + semicolon : 6 (cslSTIXMath-Regular)
    0x0140003a,   // [4832:665] t + colon : 2 (cslSTIXMath-Regular) ;;
    0xedc02019,   // [4833:666] u + quoteright : -37 (cslSTIXMath-Regular) ;;
    0xf28000f8,   // [4834:667] v + oslash : -27 (cslSTIXMath-Regular)
    0xf28000f6,   // [4835:668] v + odieresis : -27 (cslSTIXMath-Regular)
    0xf28000f3,   // [4836:669] v + oacute : -27 (cslSTIXMath-Regular)
    0xf28000f2,   // [4837:670] v + ograve : -27 (cslSTIXMath-Regular)
    0xf40000ea,   // [4838:671] v + ecircumflex : -24 (cslSTIXMath-Regular)
    0xf40000e9,   // [4839:672] v + eacute : -24 (cslSTIXMath-Regular)
    0xf40000e8,   // [4840:673] v + egrave : -24 (cslSTIXMath-Regular)
    0xf60000e6,   // [4841:674] v + ae : -20 (cslSTIXMath-Regular)
    0xf60000e5,   // [4842:675] v + aring : -20 (cslSTIXMath-Regular)
    0xf60000e4,   // [4843:676] v + adieresis : -20 (cslSTIXMath-Regular)
    0xf60000e3,   // [4844:677] v + atilde : -20 (cslSTIXMath-Regular)
    0xf60000e2,   // [4845:678] v + acircumflex : -20 (cslSTIXMath-Regular)
    0xf60000e1,   // [4846:679] v + aacute : -20 (cslSTIXMath-Regular)
    0xf60000e0,   // [4847:680] v + agrave : -20 (cslSTIXMath-Regular)
    0xf9800073,   // [4848:681] v + s : -13 (cslSTIXMath-Regular)
    0xf280006f,   // [4849:682] v + o : -27 (cslSTIXMath-Regular)
    0xf080006c,   // [4850:683] v + l : -31 (cslSTIXMath-Regular)
    0xeb000067,   // [4851:684] v + g : -42 (cslSTIXMath-Regular)
    0xf4000065,   // [4852:685] v + e : -24 (cslSTIXMath-Regular)
    0xf2000063,   // [4853:686] v + c : -28 (cslSTIXMath-Regular)
    0xf6000061,   // [4854:687] v + a : -20 (cslSTIXMath-Regular)
    0xf780003b,   // [4855:688] v + semicolon : -17 (cslSTIXMath-Regular)
    0xf580003a,   // [4856:689] v + colon : -21 (cslSTIXMath-Regular)
    0xdb00002e,   // [4857:690] v + period : -74 (cslSTIXMath-Regular)
    0xf200002d,   // [4858:691] v + hyphen : -28 (cslSTIXMath-Regular)
    0xdf40002c,   // [4859:692] v + comma : -66 (cslSTIXMath-Regular) ;;
    0xf30000f8,   // [4860:693] w + oslash : -26 (cslSTIXMath-Regular)
    0xf30000f6,   // [4861:694] w + odieresis : -26 (cslSTIXMath-Regular)
    0xf30000f3,   // [4862:695] w + oacute : -26 (cslSTIXMath-Regular)
    0xf30000f2,   // [4863:696] w + ograve : -26 (cslSTIXMath-Regular)
    0xf70000ea,   // [4864:697] w + ecircumflex : -18 (cslSTIXMath-Regular)
    0xf70000e9,   // [4865:698] w + eacute : -18 (cslSTIXMath-Regular)
    0xf70000e8,   // [4866:699] w + egrave : -18 (cslSTIXMath-Regular)
    0xf18000e6,   // [4867:700] w + ae : -29 (cslSTIXMath-Regular)
    0xf20000e5,   // [4868:701] w + aring : -28 (cslSTIXMath-Regular)
    0xf20000e4,   // [4869:702] w + adieresis : -28 (cslSTIXMath-Regular)
    0xf20000e3,   // [4870:703] w + atilde : -28 (cslSTIXMath-Regular)
    0xf20000e2,   // [4871:704] w + acircumflex : -28 (cslSTIXMath-Regular)
    0xf20000e1,   // [4872:705] w + aacute : -28 (cslSTIXMath-Regular)
    0xf20000e0,   // [4873:706] w + agrave : -28 (cslSTIXMath-Regular)
    0xf4800073,   // [4874:707] w + s : -23 (cslSTIXMath-Regular)
    0xf300006f,   // [4875:708] w + o : -26 (cslSTIXMath-Regular)
    0xef00006c,   // [4876:709] w + l : -34 (cslSTIXMath-Regular)
    0xef800067,   // [4877:710] w + g : -33 (cslSTIXMath-Regular)
    0xf7000065,   // [4878:711] w + e : -18 (cslSTIXMath-Regular)
    0xf5800063,   // [4879:712] w + c : -21 (cslSTIXMath-Regular)
    0xf2000061,   // [4880:713] w + a : -28 (cslSTIXMath-Regular)
    0xfa00003b,   // [4881:714] w + semicolon : -12 (cslSTIXMath-Regular)
    0xfa00003a,   // [4882:715] w + colon : -12 (cslSTIXMath-Regular)
    0xde00002e,   // [4883:716] w + period : -68 (cslSTIXMath-Regular)
    0xf400002d,   // [4884:717] w + hyphen : -24 (cslSTIXMath-Regular)
    0xe740002c,   // [4885:718] w + comma : -50 (cslSTIXMath-Regular) ;;
    0xf40000e9,   // [4886:719] x + eacute : -24 (cslSTIXMath-Regular)
    0xf7800071,   // [4887:720] x + q : -17 (cslSTIXMath-Regular)
    0xf180006f,   // [4888:721] x + o : -29 (cslSTIXMath-Regular)
    0xf4000065,   // [4889:722] x + e : -24 (cslSTIXMath-Regular)
    0xf4000063,   // [4890:723] x + c : -24 (cslSTIXMath-Regular)
    0xfac00061,   // [4891:724] x + a : -11 (cslSTIXMath-Regular) ;;
    0xf70000f8,   // [4892:725] y + oslash : -18 (cslSTIXMath-Regular)
    0xf70000f6,   // [4893:726] y + odieresis : -18 (cslSTIXMath-Regular)
    0xf70000f3,   // [4894:727] y + oacute : -18 (cslSTIXMath-Regular)
    0xf70000f2,   // [4895:728] y + ograve : -18 (cslSTIXMath-Regular)
    0xf78000ea,   // [4896:729] y + ecircumflex : -17 (cslSTIXMath-Regular)
    0xf78000e9,   // [4897:730] y + eacute : -17 (cslSTIXMath-Regular)
    0xf78000e8,   // [4898:731] y + egrave : -17 (cslSTIXMath-Regular)
    0xf88000e6,   // [4899:732] y + ae : -15 (cslSTIXMath-Regular)
    0xf88000e5,   // [4900:733] y + aring : -15 (cslSTIXMath-Regular)
    0xf88000e4,   // [4901:734] y + adieresis : -15 (cslSTIXMath-Regular)
    0xf88000e3,   // [4902:735] y + atilde : -15 (cslSTIXMath-Regular)
    0xf88000e2,   // [4903:736] y + acircumflex : -15 (cslSTIXMath-Regular)
    0xf88000e1,   // [4904:737] y + aacute : -15 (cslSTIXMath-Regular)
    0xf88000e0,   // [4905:738] y + agrave : -15 (cslSTIXMath-Regular)
    0xf9800073,   // [4906:739] y + s : -13 (cslSTIXMath-Regular)
    0xf700006f,   // [4907:740] y + o : -18 (cslSTIXMath-Regular)
    0xf700006c,   // [4908:741] y + l : -18 (cslSTIXMath-Regular)
    0xf5000067,   // [4909:742] y + g : -22 (cslSTIXMath-Regular)
    0xf4800065,   // [4910:743] y + e : -23 (cslSTIXMath-Regular)
    0xf8800063,   // [4911:744] y + c : -15 (cslSTIXMath-Regular)
    0xf8800061,   // [4912:745] y + a : -15 (cslSTIXMath-Regular)
    0xf900003b,   // [4913:746] y + semicolon : -14 (cslSTIXMath-Regular)
    0xfc00003a,   // [4914:747] y + colon : -8 (cslSTIXMath-Regular)
    0xdc80002e,   // [4915:748] y + period : -71 (cslSTIXMath-Regular)
    0xf700002d,   // [4916:749] y + hyphen : -18 (cslSTIXMath-Regular)
    0xe5c0002c,   // [4917:750] y + comma : -53 (cslSTIXMath-Regular) ;;
    0xe10000c6,   // [4918:751] guillemotright + AE : -62 (cslSTIXMath-Regular)
    0xdf8000c5,   // [4919:752] guillemotright + Aring : -65 (cslSTIXMath-Regular)
    0xdf8000c4,   // [4920:753] guillemotright + Adieresis : -65 (cslSTIXMath-Regular)
    0xdf8000c1,   // [4921:754] guillemotright + Aacute : -65 (cslSTIXMath-Regular)
    0xbb000059,   // [4922:755] guillemotright + Y : -138 (cslSTIXMath-Regular)
    0xd1000057,   // [4923:756] guillemotright + W : -94 (cslSTIXMath-Regular)
    0xc6000056,   // [4924:757] guillemotright + V : -116 (cslSTIXMath-Regular)
    0xc7800054,   // [4925:758] guillemotright + T : -113 (cslSTIXMath-Regular)
    0xdfc00041,   // [4926:759] guillemotright + A : -65 (cslSTIXMath-Regular) ;;
    0xe6000059,   // [4927:760] Agrave + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [4928:761] Agrave + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [4929:762] Agrave + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [4930:763] Agrave + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [4931:764] Agrave + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [4932:765] Agrave + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [4933:766] Agrave + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [4934:767] Agrave + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [4935:768] Agrave + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [4936:769] Agrave + period : -5 (cslSTIXMath-Regular)
    0xff40002c,   // [4937:770] Agrave + comma : -2 (cslSTIXMath-Regular) ;;
    0xe7802039,   // [4938:771] Aacute + guilsinglleft : -49 (cslSTIXMath-Regular)
    0xe380201d,   // [4939:772] Aacute + quotedblright : -57 (cslSTIXMath-Regular)
    0xdb002019,   // [4940:773] Aacute + quoteright : -74 (cslSTIXMath-Regular)
    0xeb0000ab,   // [4941:774] Aacute + guillemotleft : -42 (cslSTIXMath-Regular)
    0xdc000079,   // [4942:775] Aacute + y : -72 (cslSTIXMath-Regular)
    0xe3000077,   // [4943:776] Aacute + w : -58 (cslSTIXMath-Regular)
    0xe3000076,   // [4944:777] Aacute + v : -58 (cslSTIXMath-Regular)
    0xf9000075,   // [4945:778] Aacute + u : -14 (cslSTIXMath-Regular)
    0xf9800074,   // [4946:779] Aacute + t : -13 (cslSTIXMath-Regular)
    0xf9000071,   // [4947:780] Aacute + q : -14 (cslSTIXMath-Regular)
    0xf300006f,   // [4948:781] Aacute + o : -26 (cslSTIXMath-Regular)
    0xfa000067,   // [4949:782] Aacute + g : -12 (cslSTIXMath-Regular)
    0xf8000065,   // [4950:783] Aacute + e : -16 (cslSTIXMath-Regular)
    0xf7800064,   // [4951:784] Aacute + d : -17 (cslSTIXMath-Regular)
    0xfb000063,   // [4952:785] Aacute + c : -10 (cslSTIXMath-Regular)
    0xf9000062,   // [4953:786] Aacute + b : -14 (cslSTIXMath-Regular)
    0xfe000061,   // [4954:787] Aacute + a : -4 (cslSTIXMath-Regular)
    0xe6000059,   // [4955:788] Aacute + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [4956:789] Aacute + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [4957:790] Aacute + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [4958:791] Aacute + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [4959:792] Aacute + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [4960:793] Aacute + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [4961:794] Aacute + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [4962:795] Aacute + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [4963:796] Aacute + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [4964:797] Aacute + period : -5 (cslSTIXMath-Regular)
    0xf800002d,   // [4965:798] Aacute + hyphen : -16 (cslSTIXMath-Regular)
    0xff40002c,   // [4966:799] Aacute + comma : -2 (cslSTIXMath-Regular) ;;
    0xe6000059,   // [4967:800] Acircumflex + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [4968:801] Acircumflex + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [4969:802] Acircumflex + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [4970:803] Acircumflex + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [4971:804] Acircumflex + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [4972:805] Acircumflex + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [4973:806] Acircumflex + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [4974:807] Acircumflex + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [4975:808] Acircumflex + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [4976:809] Acircumflex + period : -5 (cslSTIXMath-Regular)
    0xff40002c,   // [4977:810] Acircumflex + comma : -2 (cslSTIXMath-Regular) ;;
    0xe6000059,   // [4978:811] Atilde + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [4979:812] Atilde + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [4980:813] Atilde + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [4981:814] Atilde + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [4982:815] Atilde + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [4983:816] Atilde + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [4984:817] Atilde + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [4985:818] Atilde + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [4986:819] Atilde + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [4987:820] Atilde + period : -5 (cslSTIXMath-Regular)
    0xff40002c,   // [4988:821] Atilde + comma : -2 (cslSTIXMath-Regular) ;;
    0xe7802039,   // [4989:822] Adieresis + guilsinglleft : -49 (cslSTIXMath-Regular)
    0xe380201d,   // [4990:823] Adieresis + quotedblright : -57 (cslSTIXMath-Regular)
    0xdb002019,   // [4991:824] Adieresis + quoteright : -74 (cslSTIXMath-Regular)
    0xeb0000ab,   // [4992:825] Adieresis + guillemotleft : -42 (cslSTIXMath-Regular)
    0xdc000079,   // [4993:826] Adieresis + y : -72 (cslSTIXMath-Regular)
    0xe3000077,   // [4994:827] Adieresis + w : -58 (cslSTIXMath-Regular)
    0xe3000076,   // [4995:828] Adieresis + v : -58 (cslSTIXMath-Regular)
    0xf9000075,   // [4996:829] Adieresis + u : -14 (cslSTIXMath-Regular)
    0xf9800074,   // [4997:830] Adieresis + t : -13 (cslSTIXMath-Regular)
    0xf9000071,   // [4998:831] Adieresis + q : -14 (cslSTIXMath-Regular)
    0xf300006f,   // [4999:832] Adieresis + o : -26 (cslSTIXMath-Regular)
    0xfa000067,   // [5000:833] Adieresis + g : -12 (cslSTIXMath-Regular)
    0xf7800064,   // [5001:834] Adieresis + d : -17 (cslSTIXMath-Regular)
    0xfb000063,   // [5002:835] Adieresis + c : -10 (cslSTIXMath-Regular)
    0xf9000062,   // [5003:836] Adieresis + b : -14 (cslSTIXMath-Regular)
    0xfe000061,   // [5004:837] Adieresis + a : -4 (cslSTIXMath-Regular)
    0xe6000059,   // [5005:838] Adieresis + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [5006:839] Adieresis + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [5007:840] Adieresis + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [5008:841] Adieresis + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [5009:842] Adieresis + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [5010:843] Adieresis + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [5011:844] Adieresis + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [5012:845] Adieresis + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [5013:846] Adieresis + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [5014:847] Adieresis + period : -5 (cslSTIXMath-Regular)
    0xf800002d,   // [5015:848] Adieresis + hyphen : -16 (cslSTIXMath-Regular)
    0xff40002c,   // [5016:849] Adieresis + comma : -2 (cslSTIXMath-Regular) ;;
    0xe7802039,   // [5017:850] Aring + guilsinglleft : -49 (cslSTIXMath-Regular)
    0xe380201d,   // [5018:851] Aring + quotedblright : -57 (cslSTIXMath-Regular)
    0xdb002019,   // [5019:852] Aring + quoteright : -74 (cslSTIXMath-Regular)
    0xeb0000ab,   // [5020:853] Aring + guillemotleft : -42 (cslSTIXMath-Regular)
    0xdc000079,   // [5021:854] Aring + y : -72 (cslSTIXMath-Regular)
    0xe3000077,   // [5022:855] Aring + w : -58 (cslSTIXMath-Regular)
    0xe3000076,   // [5023:856] Aring + v : -58 (cslSTIXMath-Regular)
    0xf9000075,   // [5024:857] Aring + u : -14 (cslSTIXMath-Regular)
    0xf9800074,   // [5025:858] Aring + t : -13 (cslSTIXMath-Regular)
    0xf9000071,   // [5026:859] Aring + q : -14 (cslSTIXMath-Regular)
    0xf300006f,   // [5027:860] Aring + o : -26 (cslSTIXMath-Regular)
    0xfa000067,   // [5028:861] Aring + g : -12 (cslSTIXMath-Regular)
    0xf7800065,   // [5029:862] Aring + e : -17 (cslSTIXMath-Regular)
    0xf7800064,   // [5030:863] Aring + d : -17 (cslSTIXMath-Regular)
    0xfb000063,   // [5031:864] Aring + c : -10 (cslSTIXMath-Regular)
    0xf9000062,   // [5032:865] Aring + b : -14 (cslSTIXMath-Regular)
    0xfe000061,   // [5033:866] Aring + a : -4 (cslSTIXMath-Regular)
    0xe6000059,   // [5034:867] Aring + Y : -52 (cslSTIXMath-Regular)
    0xd6800057,   // [5035:868] Aring + W : -83 (cslSTIXMath-Regular)
    0xd6800056,   // [5036:869] Aring + V : -83 (cslSTIXMath-Regular)
    0xee800055,   // [5037:870] Aring + U : -35 (cslSTIXMath-Regular)
    0xe5000054,   // [5038:871] Aring + T : -54 (cslSTIXMath-Regular)
    0xf3800051,   // [5039:872] Aring + Q : -25 (cslSTIXMath-Regular)
    0xf380004f,   // [5040:873] Aring + O : -25 (cslSTIXMath-Regular)
    0xf3800047,   // [5041:874] Aring + G : -25 (cslSTIXMath-Regular)
    0xf3800043,   // [5042:875] Aring + C : -25 (cslSTIXMath-Regular)
    0xfd80002e,   // [5043:876] Aring + period : -5 (cslSTIXMath-Regular)
    0xf800002d,   // [5044:877] Aring + hyphen : -16 (cslSTIXMath-Regular)
    0xff40002c,   // [5045:878] Aring + comma : -2 (cslSTIXMath-Regular) ;;
    0xf8400041,   // [5046:879] Ccedilla + A : -16 (cslSTIXMath-Regular) ;;
    0xee000059,   // [5047:880] Ograve + Y : -36 (cslSTIXMath-Regular)
    0xf1000056,   // [5048:881] Ograve + V : -30 (cslSTIXMath-Regular)
    0xfec00054,   // [5049:882] Ograve + T : -3 (cslSTIXMath-Regular) ;;
    0xee000059,   // [5050:883] Oacute + Y : -36 (cslSTIXMath-Regular)
    0xf3800057,   // [5051:884] Oacute + W : -25 (cslSTIXMath-Regular)
    0xf1000056,   // [5052:885] Oacute + V : -30 (cslSTIXMath-Regular)
    0xfe800054,   // [5053:886] Oacute + T : -3 (cslSTIXMath-Regular)
    0xf3c00041,   // [5054:887] Oacute + A : -25 (cslSTIXMath-Regular) ;;
    0xee000059,   // [5055:888] Ocircumflex + Y : -36 (cslSTIXMath-Regular)
    0xf1000056,   // [5056:889] Ocircumflex + V : -30 (cslSTIXMath-Regular)
    0xfec00054,   // [5057:890] Ocircumflex + T : -3 (cslSTIXMath-Regular) ;;
    0xee000059,   // [5058:891] Otilde + Y : -36 (cslSTIXMath-Regular)
    0xf1000056,   // [5059:892] Otilde + V : -30 (cslSTIXMath-Regular)
    0xfec00054,   // [5060:893] Otilde + T : -3 (cslSTIXMath-Regular) ;;
    0xee000059,   // [5061:894] Odieresis + Y : -36 (cslSTIXMath-Regular)
    0xf4000058,   // [5062:895] Odieresis + X : -24 (cslSTIXMath-Regular)
    0xf3800057,   // [5063:896] Odieresis + W : -25 (cslSTIXMath-Regular)
    0xf5000056,   // [5064:897] Odieresis + V : -22 (cslSTIXMath-Regular)
    0xfe800054,   // [5065:898] Odieresis + T : -3 (cslSTIXMath-Regular)
    0xf3c00041,   // [5066:899] Odieresis + A : -25 (cslSTIXMath-Regular) ;;
    0xf3c00041,   // [5067:900] Oslash + A : -25 (cslSTIXMath-Regular) ;;
    0xf8000072,   // [5068:901] Ugrave + r : -16 (cslSTIXMath-Regular)
    0xf7000070,   // [5069:902] Ugrave + p : -18 (cslSTIXMath-Regular)
    0xf600006e,   // [5070:903] Ugrave + n : -20 (cslSTIXMath-Regular)
    0xf600006d,   // [5071:904] Ugrave + m : -20 (cslSTIXMath-Regular)
    0xea800041,   // [5072:905] Ugrave + A : -43 (cslSTIXMath-Regular)
    0xf200002e,   // [5073:906] Ugrave + period : -28 (cslSTIXMath-Regular)
    0xf0c0002c,   // [5074:907] Ugrave + comma : -31 (cslSTIXMath-Regular) ;;
    0xf8000072,   // [5075:908] Uacute + r : -16 (cslSTIXMath-Regular)
    0xf7000070,   // [5076:909] Uacute + p : -18 (cslSTIXMath-Regular)
    0xf600006e,   // [5077:910] Uacute + n : -20 (cslSTIXMath-Regular)
    0xf600006d,   // [5078:911] Uacute + m : -20 (cslSTIXMath-Regular)
    0xea800041,   // [5079:912] Uacute + A : -43 (cslSTIXMath-Regular)
    0xf200002e,   // [5080:913] Uacute + period : -28 (cslSTIXMath-Regular)
    0xf0c0002c,   // [5081:914] Uacute + comma : -31 (cslSTIXMath-Regular) ;;
    0xeac00041,   // [5082:915] Ucircumflex + A : -43 (cslSTIXMath-Regular) ;;
    0xf8000072,   // [5083:916] Udieresis + r : -16 (cslSTIXMath-Regular)
    0xf7000070,   // [5084:917] Udieresis + p : -18 (cslSTIXMath-Regular)
    0xf600006e,   // [5085:918] Udieresis + n : -20 (cslSTIXMath-Regular)
    0xf600006d,   // [5086:919] Udieresis + m : -20 (cslSTIXMath-Regular)
    0x0a800062,   // [5087:920] Udieresis + b : 21 (cslSTIXMath-Regular)
    0xea800041,   // [5088:921] Udieresis + A : -43 (cslSTIXMath-Regular)
    0xf200002e,   // [5089:922] Udieresis + period : -28 (cslSTIXMath-Regular)
    0xf0c0002c,   // [5090:923] Udieresis + comma : -31 (cslSTIXMath-Regular) ;;
    0xf2800079,   // [5091:924] agrave + y : -27 (cslSTIXMath-Regular)
    0xef800077,   // [5092:925] agrave + w : -33 (cslSTIXMath-Regular)
    0xf0400076,   // [5093:926] agrave + v : -32 (cslSTIXMath-Regular) ;;
    0xf2800079,   // [5094:927] aacute + y : -27 (cslSTIXMath-Regular)
    0xef800077,   // [5095:928] aacute + w : -33 (cslSTIXMath-Regular)
    0xf0400076,   // [5096:929] aacute + v : -32 (cslSTIXMath-Regular) ;;
    0xf2800079,   // [5097:930] adieresis + y : -27 (cslSTIXMath-Regular)
    0xef800077,   // [5098:931] adieresis + w : -33 (cslSTIXMath-Regular)
    0xf0400076,   // [5099:932] adieresis + v : -32 (cslSTIXMath-Regular) ;;
    0xf2800079,   // [5100:933] aring + y : -27 (cslSTIXMath-Regular)
    0xef800077,   // [5101:934] aring + w : -33 (cslSTIXMath-Regular)
    0xf0400076,   // [5102:935] aring + v : -32 (cslSTIXMath-Regular) ;;
    0xf1000079,   // [5103:936] ae + y : -30 (cslSTIXMath-Regular)
    0xf1800077,   // [5104:937] ae + w : -29 (cslSTIXMath-Regular)
    0xf2400076,   // [5105:938] ae + v : -28 (cslSTIXMath-Regular) ;;
    0xf5800079,   // [5106:939] eacute + y : -21 (cslSTIXMath-Regular)
    0xf6800077,   // [5107:940] eacute + w : -19 (cslSTIXMath-Regular)
    0xf6c00076,   // [5108:941] eacute + v : -19 (cslSTIXMath-Regular) ;;
    0xf5800079,   // [5109:942] ecircumflex + y : -21 (cslSTIXMath-Regular)
    0xf6800077,   // [5110:943] ecircumflex + w : -19 (cslSTIXMath-Regular)
    0xf6c00076,   // [5111:944] ecircumflex + v : -19 (cslSTIXMath-Regular) ;;
    0xf5000079,   // [5112:945] ograve + y : -22 (cslSTIXMath-Regular)
    0xf5000077,   // [5113:946] ograve + w : -22 (cslSTIXMath-Regular)
    0xf5400076,   // [5114:947] ograve + v : -22 (cslSTIXMath-Regular) ;;
    0xf5000079,   // [5115:948] oacute + y : -22 (cslSTIXMath-Regular)
    0xf5000077,   // [5116:949] oacute + w : -22 (cslSTIXMath-Regular)
    0xf5400076,   // [5117:950] oacute + v : -22 (cslSTIXMath-Regular) ;;
    0xfc400074,   // [5118:951] ocircumflex + t : -8 (cslSTIXMath-Regular) ;;
    0xf5000079,   // [5119:952] odieresis + y : -22 (cslSTIXMath-Regular)
    0xf5000078,   // [5120:953] odieresis + x : -22 (cslSTIXMath-Regular)
    0xf5000077,   // [5121:954] odieresis + w : -22 (cslSTIXMath-Regular)
    0xf5000076,   // [5122:955] odieresis + v : -22 (cslSTIXMath-Regular)
    0xfc400074,   // [5123:956] odieresis + t : -8 (cslSTIXMath-Regular) ;;
    0xc80000c6,   // [5124:957] quoteleft + AE : -112 (cslSTIXMath-Regular)
    0xc70000c5,   // [5125:958] quoteleft + Aring : -114 (cslSTIXMath-Regular)
    0xc70000c4,   // [5126:959] quoteleft + Adieresis : -114 (cslSTIXMath-Regular)
    0xc70000c2,   // [5127:960] quoteleft + Acircumflex : -114 (cslSTIXMath-Regular)
    0xc70000c1,   // [5128:961] quoteleft + Aacute : -114 (cslSTIXMath-Regular)
    0xc70000c0,   // [5129:962] quoteleft + Agrave : -114 (cslSTIXMath-Regular)
    0xf2800059,   // [5130:963] quoteleft + Y : -27 (cslSTIXMath-Regular)
    0xf6000057,   // [5131:964] quoteleft + W : -20 (cslSTIXMath-Regular)
    0xf3800056,   // [5132:965] quoteleft + V : -25 (cslSTIXMath-Regular)
    0xfa800054,   // [5133:966] quoteleft + T : -11 (cslSTIXMath-Regular)
    0xc7400041,   // [5134:967] quoteleft + A : -114 (cslSTIXMath-Regular) ;;
    0xbc8000c6,   // [5135:968] quoteright + AE : -135 (cslSTIXMath-Regular)
    0xbe0000c5,   // [5136:969] quoteright + Aring : -132 (cslSTIXMath-Regular)
    0xbe0000c4,   // [5137:970] quoteright + Adieresis : -132 (cslSTIXMath-Regular)
    0xbe0000c2,   // [5138:971] quoteright + Acircumflex : -132 (cslSTIXMath-Regular)
    0xbe0000c1,   // [5139:972] quoteright + Aacute : -132 (cslSTIXMath-Regular)
    0xbe0000c0,   // [5140:973] quoteright + Agrave : -132 (cslSTIXMath-Regular)
    0xe9000079,   // [5141:974] quoteright + y : -46 (cslSTIXMath-Regular)
    0xea800077,   // [5142:975] quoteright + w : -43 (cslSTIXMath-Regular)
    0xe8000076,   // [5143:976] quoteright + v : -48 (cslSTIXMath-Regular)
    0xeb000074,   // [5144:977] quoteright + t : -42 (cslSTIXMath-Regular)
    0xe7800073,   // [5145:978] quoteright + s : -49 (cslSTIXMath-Regular)
    0xed000072,   // [5146:979] quoteright + r : -38 (cslSTIXMath-Regular)
    0xe400006f,   // [5147:980] quoteright + o : -56 (cslSTIXMath-Regular)
    0xe3000064,   // [5148:981] quoteright + d : -58 (cslSTIXMath-Regular)
    0xbe000041,   // [5149:982] quoteright + A : -132 (cslSTIXMath-Regular)
    0xd800002e,   // [5150:983] quoteright + period : -80 (cslSTIXMath-Regular)
    0xdbc0002c,   // [5151:984] quoteright + comma : -73 (cslSTIXMath-Regular) ;;
    0xd48000c6,   // [5152:985] quotedblleft + AE : -87 (cslSTIXMath-Regular)
    0xd38000c5,   // [5153:986] quotedblleft + Aring : -89 (cslSTIXMath-Regular)
    0xd38000c4,   // [5154:987] quotedblleft + Adieresis : -89 (cslSTIXMath-Regular)
    0xd38000c1,   // [5155:988] quotedblleft + Aacute : -89 (cslSTIXMath-Regular)
    0xfe800059,   // [5156:989] quotedblleft + Y : -3 (cslSTIXMath-Regular)
    0x02000057,   // [5157:990] quotedblleft + W : 4 (cslSTIXMath-Regular)
    0xff800056,   // [5158:991] quotedblleft + V : -1 (cslSTIXMath-Regular)
    0x06800054,   // [5159:992] quotedblleft + T : 13 (cslSTIXMath-Regular)
    0xd3c00041,   // [5160:993] quotedblleft + A : -89 (cslSTIXMath-Regular) ;;
    0xce8000c6,   // [5161:994] quotedblright + AE : -99 (cslSTIXMath-Regular)
    0xd00000c5,   // [5162:995] quotedblright + Aring : -96 (cslSTIXMath-Regular)
    0xd00000c4,   // [5163:996] quotedblright + Adieresis : -96 (cslSTIXMath-Regular)
    0xd00000c1,   // [5164:997] quotedblright + Aacute : -96 (cslSTIXMath-Regular)
    0xfe800059,   // [5165:998] quotedblright + Y : -3 (cslSTIXMath-Regular)
    0x03000057,   // [5166:999] quotedblright + W : 6 (cslSTIXMath-Regular)
    0x00800056,   // [5167:1000] quotedblright + V : 1 (cslSTIXMath-Regular)
    0x04800054,   // [5168:1001] quotedblright + T : 9 (cslSTIXMath-Regular)
    0xd0400041,   // [5169:1002] quotedblright + A : -96 (cslSTIXMath-Regular) ;;
    0x098000c6,   // [5170:1003] quotedblbase + AE : 19 (cslSTIXMath-Regular)
    0x060000c5,   // [5171:1004] quotedblbase + Aring : 12 (cslSTIXMath-Regular)
    0x060000c4,   // [5172:1005] quotedblbase + Adieresis : 12 (cslSTIXMath-Regular)
    0x060000c2,   // [5173:1006] quotedblbase + Acircumflex : 12 (cslSTIXMath-Regular)
    0x060000c1,   // [5174:1007] quotedblbase + Aacute : 12 (cslSTIXMath-Regular)
    0x060000c0,   // [5175:1008] quotedblbase + Agrave : 12 (cslSTIXMath-Regular)
    0xd4800059,   // [5176:1009] quotedblbase + Y : -87 (cslSTIXMath-Regular)
    0xdb800057,   // [5177:1010] quotedblbase + W : -73 (cslSTIXMath-Regular)
    0xce000056,   // [5178:1011] quotedblbase + V : -100 (cslSTIXMath-Regular)
    0xe2000054,   // [5179:1012] quotedblbase + T : -60 (cslSTIXMath-Regular)
    0x06400041,   // [5180:1013] quotedblbase + A : 12 (cslSTIXMath-Regular) ;;
    0xdb8000c6,   // [5181:1014] guilsinglright + AE : -73 (cslSTIXMath-Regular)
    0xda0000c5,   // [5182:1015] guilsinglright + Aring : -76 (cslSTIXMath-Regular)
    0xda0000c4,   // [5183:1016] guilsinglright + Adieresis : -76 (cslSTIXMath-Regular)
    0xda0000c1,   // [5184:1017] guilsinglright + Aacute : -76 (cslSTIXMath-Regular)
    0xb5800059,   // [5185:1018] guilsinglright + Y : -149 (cslSTIXMath-Regular)
    0xcb800057,   // [5186:1019] guilsinglright + W : -105 (cslSTIXMath-Regular)
    0xc1000056,   // [5187:1020] guilsinglright + V : -126 (cslSTIXMath-Regular)
    0xc2800054,   // [5188:1021] guilsinglright + T : -123 (cslSTIXMath-Regular)
    0xda400041,   // [5189:1022] guilsinglright + A : -76 (cslSTIXMath-Regular) ;;
    0x23a00069,   // [5190:1023] ff + i ligature #71 (cslSTIXMath-Regular)
    0x2460006c    // [5191:1024] ff + l ligature #72 (cslSTIXMath-Regular) ;;
};

const uint32_t ligaturetable[] =
{   62083,    // [0] space + hookabovecomb => space_hookabovecomb (cmuntt)
    61877,    // [1] space + COMBINING_CYRILLIC_MILLIONS_SIGN => space_COMBINING_CYRILLIC_MILLIONS_SIGN (cmuntt)
    61876,    // [2] space + COMBINING_CYRILLIC_HUNDRED_THOUSANDS_SIGN => space_COMBINING_CYRILLIC_HUNDRED_THOUSANDS_SIGN (cmuntt)
    61875,    // [3] space + COMBINING_CYRILLIC_PSILI_PNEUMATA => space_COMBINING_CYRILLIC_PSILI_PNEUMATA (cmuntt)
    61874,    // [4] space + COMBINING_CYRILLIC_DASIA_PNEUMATA => space_COMBINING_CYRILLIC_DASIA_PNEUMATA (cmuntt)
    61873,    // [5] space + COMBINING_CYRILLIC_PALATALIZATION => space_COMBINING_CYRILLIC_PALATALIZATION (cmuntt)
    61872,    // [6] space + COMBINING_LEFT_RIGHT_ARROW_BELOW => space_COMBINING_LEFT_RIGHT_ARROW_BELOW (cmuntt)
    61871,    // [7] space + COMBINING_SEAGULL_BELOW => space_COMBINING_SEAGULL_BELOW (cmuntt)
    61870,    // [8] space + COMBINING_SQUARE_BELOW => space_COMBINING_SQUARE_BELOW (cmuntt)
    61869,    // [9] space + COMBINING_INVERTED_BRIDGE_BELOW => space_COMBINING_INVERTED_BRIDGE_BELOW (cmuntt)
    61868,    // [10] space + COMBINING_SHORT_SOLIDUS_OVERLAY => space_COMBINING_SHORT_SOLIDUS_OVERLAY (cmuntt)
    61867,    // [11] space + COMBINING_SHORT_STROKE_OVERLAY => space_COMBINING_SHORT_STROKE_OVERLAY (cmuntt)
    61866,    // [12] space + COMBINING_INVERTED_DOUBLE_ARCH_BELOW => space_COMBINING_INVERTED_DOUBLE_ARCH_BELOW (cmuntt)
    61865,    // [13] space + COMBINING_BRIDGE_BELOW => space_COMBINING_BRIDGE_BELOW (cmuntt)
    61864,    // [14] space + COMBINING_RETROFLEX_HOOK_BELOW => space_COMBINING_RETROFLEX_HOOK_BELOW (cmuntt)
    61863,    // [15] space + COMBINING_PALATALIZED_HOOK_BELOW => space_COMBINING_PALATALIZED_HOOK_BELOW (cmuntt)
    61862,    // [16] space + COMBINING_INVERTED_BREVE => space_COMBINING_INVERTED_BREVE (cmuntt)
    61861,    // [17] space + COMBINING_DOUBLE_GRAVE_ACCENT => space_COMBINING_DOUBLE_GRAVE_ACCENT (cmuntt)
    61860,    // [18] space + COMBINING_DOUBLE_VERTICAL_LINE_ABOVE => space_COMBINING_DOUBLE_VERTICAL_LINE_ABOVE (cmuntt)
    61859,    // [19] space + COMBINING_VERTICAL_LINE_ABOVE => space_COMBINING_VERTICAL_LINE_ABOVE (cmuntt)
    61858,    // [20] space + COMBINING_TILDE => space_COMBINING_TILDE (cmuntt)
    61794,    // [21] space + COMBINING_ACUTE_GRAVE_ACUTE => space_COMBINING_ACUTE_GRAVE_ACUTE (cmuntt)
    61793,    // [22] space + COMBINING_GRAVE_ACUTE_GRAVE => space_COMBINING_GRAVE_ACUTE_GRAVE (cmuntt)
    61792,    // [23] space + COMBINING_GRAVE_MACRON => space_COMBINING_GRAVE_MACRON (cmuntt)
    61791,    // [24] space + COMBINING_MACRON_ACUTE => space_COMBINING_MACRON_ACUTE (cmuntt)
    306,      // [25] I + J => IJ (cmuntt)
    64258,    // [26] f + l => fl (cmuntt)
    64257,    // [27] f + i => fi (cmuntt)
    64256,    // [28] f + f => ff (cmuntt)
    307,      // [29] i + j => ij (cmuntt)
    64261,    // [30] longs + t => LATIN_SMALL_LIGATURE_LONG_S_T (cmuntt)
    64260,    // [31] ff + l => ffl (cmuntt)
    64259,    // [32] ff + i => ffi (cmuntt)
    306,      // [33] I + J => LATIN_CAPITAL_LIGATURE_IJ (cslSTIX-Regular)
    1081345,   // [34] f + j => f_j (cslSTIX-Regular)
    64258,    // [35] f + l => fl (cslSTIX-Regular)
    64257,    // [36] f + i => fi (cslSTIX-Regular)
    64256,    // [37] f + f => ff (cslSTIX-Regular)
    307,      // [38] i + j => LATIN_SMALL_LIGATURE_IJ (cslSTIX-Regular)
    64259,    // [39] ff + i => ffi (cslSTIX-Regular)
    64260,    // [40] ff + l => ffl (cslSTIX-Regular)
    306,      // [41] I + J => LATIN_CAPITAL_LIGATURE_IJ (cslSTIX-Bold)
    1081345,   // [42] f + j => f_j (cslSTIX-Bold)
    64258,    // [43] f + l => fl (cslSTIX-Bold)
    64257,    // [44] f + i => fi (cslSTIX-Bold)
    64256,    // [45] f + f => ff (cslSTIX-Bold)
    307,      // [46] i + j => LATIN_SMALL_LIGATURE_IJ (cslSTIX-Bold)
    64259,    // [47] ff + i => ffi (cslSTIX-Bold)
    64260,    // [48] ff + l => ffl (cslSTIX-Bold)
    306,      // [49] I + J => LATIN_CAPITAL_LIGATURE_IJ (cslSTIX-Italic)
    1081345,   // [50] f + j => f_j (cslSTIX-Italic)
    64258,    // [51] f + l => fl (cslSTIX-Italic)
    64257,    // [52] f + i => fi (cslSTIX-Italic)
    64256,    // [53] f + f => ff (cslSTIX-Italic)
    307,      // [54] i + j => LATIN_SMALL_LIGATURE_IJ (cslSTIX-Italic)
    64259,    // [55] ff + i => ffi (cslSTIX-Italic)
    64260,    // [56] ff + l => ffl (cslSTIX-Italic)
    306,      // [57] I + J => LATIN_CAPITAL_LIGATURE_IJ (cslSTIX-BoldItalic)
    1081345,   // [58] f + j => f_j (cslSTIX-BoldItalic)
    64258,    // [59] f + l => fl (cslSTIX-BoldItalic)
    64257,    // [60] f + i => fi (cslSTIX-BoldItalic)
    64256,    // [61] f + f => ff (cslSTIX-BoldItalic)
    307,      // [62] i + j => LATIN_SMALL_LIGATURE_IJ (cslSTIX-BoldItalic)
    64259,    // [63] ff + i => ffi (cslSTIX-BoldItalic)
    64260,    // [64] ff + l => ffl (cslSTIX-BoldItalic)
    306,      // [65] I + J => LATIN_CAPITAL_LIGATURE_IJ (cslSTIXMath-Regular)
    1081345,   // [66] f + j => uniE0B3 (cslSTIXMath-Regular)
    64258,    // [67] f + l => fl (cslSTIXMath-Regular)
    64257,    // [68] f + i => fi (cslSTIXMath-Regular)
    64256,    // [69] f + f => ff (cslSTIXMath-Regular)
    307,      // [70] i + j => LATIN_SMALL_LIGATURE_IJ (cslSTIXMath-Regular)
    64259,    // [71] ff + i => ffi (cslSTIXMath-Regular)
    64260     // [72] ff + l => ffl (cslSTIXMath-Regular)
};

// end of charmetrics.h
